/*
 * Copyright 2016-2018 Leo McCormack
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

/**
 * @file saf_utility_loudspeaker_presets.c
 * @ingroup Utilities
 * @brief A collection of loudspeaker array directions and (nearly) uniform
 *        spherical grids
 *
 * @author Leo McCormack
 * @date 11.07.2016
 */

#include "saf_utility_loudspeaker_presets.h"
 
/* mono */
const float __mono_dirs_deg[1][2] =
{ { 0.0f,    0.0f}};

/* stereo pair */
const float __stereo_dirs_deg[2][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f}};

/* 5.X surround */
const float __5pX_dirs_deg[5][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f}};

/* 7.X surround */
const float __7pX_dirs_deg[7][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 135.0f,    0.0f},
  { -135.0f,    0.0f}};

/* 8.X surround */
const float __8pX_dirs_deg[8][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 110.0f,    45.0f},
  { -110.0f,    45.0f}};

/* 9.X surround */
const float __9pX_dirs_deg[9][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 110.0f,    45.0f},
  { -110.0f,    45.0f}};

/* 10.X surround */
const float __10pX_dirs_deg[10][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 0.0f,    90.0f},
  { 110.0f,    45.0f},
  { -110.0f,    45.0f}};

/* 11.X surround */
const float __11pX_dirs_deg[11][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 0.0f,    45.0f},
  { 0.0f,    90.0f},
  { 110.0f,    45.0f},
  { -110.0f,    45.0f}};

/* 11.X (7+4) surround */
const float __11pX_7_4_dirs_deg[11][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 110.0f,    0.0f},
  { -110.0f,    0.0f},
  { 135.0f,    0.0f},
  { -135.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 110.0f,    45.0f},
  { -110.0f,    45.0f}};

/* 13.X surround */
const float __13pX_dirs_deg[13][2] =
{ { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { 0.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 135.0f,    0.0f},
  { -135.0f,    0.0f},
  { 30.0f,    45.0f},
  { -30.0f,    45.0f},
  { 0.0f,    45.0f},
  { 0.0f,    90.0f},
  { 90.0f,    45.0f},
  { -90.0f,    45.0f}};

/* 22.X surround */
const float __22pX_dirs_deg[22][2] =
{ { 45.0f,    0.0f},
  { -45.0f,    0.0f},
  { 0.0f,    0.0f},
  { 135.0f,    0.0f},
  { -135.0f,    0.0f},
  { 15.0f,    0.0f},
  { -15.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 180.0f,    0.0f},
  { 45.0f,    45.0f},
  { -45.0f,    45.0f},
  { 0.0f,    45.0f},
  { 135.0f,    45.0f},
  { -135.0f,    45.0f},
  { 90.0f,    45.0f},
  { -90.0f,    45.0f},
  { 180.0f,    45.0f},
  { 0.0f,    90.0f},
  { 45.0f,    -30.0f},
  { -45.0f,    -30.0f},
  { 0.0f,    -30.0f}};

/* 9+10+3 */
const float __9_10_3p2_dirs_deg[24][2] =
{ { 45.0f,     0.0f},
  { -45.0f,     0.0f},
  { 0.0f,     0.0f},
  { 30.0f,     -15.0f},
  { 110.0f,     0.0f},
  { -110.0f,     0.0f},
  { 22.5f,     0.0f},
  { -22.5f,     0.0f},
  { 180.0f,     0.0f},
  { -30.0f,     -15.0f},
  { 90.0f,     0.0f},
  { -90.0f,     0.0f},
  { 45.0f,     30.0f},
  { -45.0f,     30.0f},
  { 0.0f,     30.0f},
  { 0.0f,     90.0f},
  { 110.0f,     30.0f},
  { -110.0f,     30.0f},
  { 90.0f,     30.0f},
  { -90.0f,     30.0f},
  { 180.0f,     30.0f},
  { 0.0f,     -15.0f},
  { 45.0f,     -15.0f},
  { -45.0f,     -15.0f}};

/* Aalto MCC */
const float __Aalto_MCC_dirs_deg[45][2] =
{ { 120.0f,     0.0f},
  { 90.0f,     0.0f},
  { 75.0f,     0.0f},
  { 60.0f,     0.0f},
  { 45.0f,     0.0f},
  { 30.0f,     0.0f},
  { 15.0f,     0.0f},
  { 0.0f,     0.0f},
  { -15.0f,    0.0f},
  { -30.0f,    0.0f},
  { -45.0f,    0.0f},
  { -60.0f,    0.0f},
  { 135.0f,    -30.0f},
  { 45.0f,    -30.0f},
  { 0.0f,    -30.0f},
  { 0.0f,    -15.0f},
  { 0.0f,     -60.0f},
  { 90.0f,     -60.0f},
  { 135.0f,     30.0f},
  { 90.0f,     30.0f},
  { 45.0f,     30.0f},
  { 90.0f,     -30.0f},
  { 0.0f,    30.0f},
  { 0.0f,    15.0f},
  { -45.0f,    30.0f},
  { -90.0f,    30.0f},
  { 0.0f,    60.0f},
  { 90.0f,    60.0f},
  { 0.0f,    90.0f},
  { -90.0f,    60.0f},
  { -75.0f,     0.0f},
  { -90.0f,     0.0f},
  { -120.0f,     0.0f},
  { -150.0f,    0.0f},
  { 180.0f,    0.0f},
  { 150.0f,    0.0f},
  { -45.0f,    -30.0f},
  { -135.0f,    -30.0f},
  { 180.0f,    -30.0f},
  { -90.0f,     -60.0f},
  { -90.0f,     30.0f},
  { -135.0f,     30.0f},
  { 180.0f,    30.0f},
  { 180.0f,    60.0f},
  { 180.0f,    -60.0f}};

const float __Aalto_MCCsubset_dirs_deg[37][2] = 
{ { 0.0f,     90.0f},
  { 0.0f,     60.0f},
  { 90.0f,     60.0f},
  { 180.0f,     60.0f},
  { -90.0f,     60.0f},
  { 0.0f,     30.0f},
  { 45.0f,     30.0f},
  { 90.0f,     0.0f},
  { 135.0f,     30.0f},
  { 180.0f,     30.0f},
  { -135.0f,     30.0f},
  { -90.0f,     30.0f},
  { -45.0f,     30.0f},
  { 0.0f,     0.0f},
  { 30.0f,     0.0f},
  { 60.0f,     0.0f},
  { 90.0f,     0.0f},
  { 120.0f,     0.0f},
  { 150.0f,     0.0f},
  { 180.0f,     0.0f},
  { -150.0f,     0.0f},
  { -120.0f,     0.0f},
  { -90.0f,     0.0f},
  { -60.0f,     0.0f},
  { -30.0f,     0.0f},
  { 0.0f,     -30.0f},
  { 45.0f,     -30.0f},
  { 90.0f,     -30.0f},
  { 135.0f,     -30.0f},
  { 180.0f,     -30.0f},
  { -135.0f,     -30.0f},
  { -90.0f,     -30.0f},
  { -45.0f,     -30.0f},
  { 0.0f,     -60.0f},
  { 90.0f,     -60.0f},
  { 180.0f,     -60.0f},
  { -90.0f,     -60.0}};

/* Aalto Apaja */
const float __Aalto_Apaja_dirs_deg[29][2] =
{ { 18.0f,     0.0f},
  { 54.0f,     0.0f},
  { 90.0f,     0.0f},
  { 126.0f,     0.0f},
  { 166.0f,     0.0f},
  { -166.0f,     0.0f},
  { -126.0f,     0.0f},
  { -90.0f,     0.0f},
  { -54.0f,     0.0f},
  { -18.0f,     0.0f},
  { 18.0f,    -25.0f},
  { 54.0f,    -25.0f},
  { 90.0f,    -25.0f},
  { 144.0f,    -25.0f},
  { -144.0f,    -25.0f},
  { -90.0f,    -25.0f},
  { -54.0f,    -25.0f},
  { -18.0f,    -25.0f},
  { 18.0f,    25.0f},
  { 54.0f,    25.0f},
  { 90.0f,    25.0f},
  { -90.0f,    25.0f},
  { -54.0f,    25.0f},
  { -18.0f,    25.0f},
  { 40.0f,    45.0f},
  { 137.0f,    45.0f},
  { -137.0f,    45.0f},
  { -40.0f,    45.0f},
  { 0.0f,    90.0f}};

/* Aalto listening room */
const float __Aalto_LR_dirs_deg[13][2] =
{ { 30.0f,     0.0f},
  { -30.0f,     0.0f},
  { 0.0f,     0.0f},
  { 70.0f,     0.0f},
  { 110.0f,     0.0f},
  { -70.0f,     0.0f},
  { -110.0f,     0.0f},
  { 160.0f,     0.0f},
  { -160.0f,     0.0f},
  { 45.0f,    45.0f},
  { 135.0f,    45.0f},
  { -45.0f,    45.0f},
  { -135.0f,    45.0f}};

/* DTU AVIL */
const float __DTU_AVIL_dirs_deg[64][2] =
{ { 0.0f,    80.0f},
  { -180.0f,    80.0f},
  { 0.0f,    56.0f},
  { 60.0f,    56.0f},
  { 120.0f,    56.0f},
  { -180.0f,    56.0f},
  { -120.0f,    56.0f},
  { -60.0f,    56.0f},
  { 0.0f,    28.0f},
  { 30.0f,    28.0f},
  { 60.0f,    28.0f},
  { 90.0f,    28.0f},
  { 120.0f,    28.0f},
  { 150.0f,    28.0f},
  { -180.0f,    28.0f},
  { -150.0f,    28.0f},
  { -120.0f,    28.0f},
  { -90.0f,    28.0f},
  { -60.0f,    28.0f},
  { -30.0f,    28.0f},
  { 0.0f,     0.0f},
  { 15.0f,    0.0f},
  { 30.0f,    0.0f},
  { 45.0f,    0.0f},
  { 60.0f,    0.0f},
  { 75.0f,    0.0f},
  { 90.0f,    0.0f},
  { 105.0f,    0.0f},
  { 120.0f,    0.0f},
  { 135.0f,    0.0f},
  { 150.0f,    0.0f},
  { 165.0f,    0.0f},
  { -180.0f,    0.0f},
  { -165.0f,    0.0f},
  { -150.0f,    0.0f},
  { -135.0f,    0.0f},
  { -120.0f,    0.0f},
  { -105.0f,    0.0f},
  { -90.0f,    0.0f},
  { -75.0f,    0.0f},
  { -60.0f,    0.0f},
  { -45.0f,    0.0f},
  { -30.0f,    0.0f},
  { -15.0f,    0.0f},
  { 0.0f,    -28.0f},
  { 30.0f,    -28.0f},
  { 60.0f,    -28.0f},
  { 90.0f,    -28.0f},
  { 120.0f,    -28.0f},
  { 150.0f,    -28.0f},
  { -180.0f,    -28.0f},
  { -150.0f,    -28.0f},
  { -120.0f,    -28.0f},
  { -90.0f,    -28.0f},
  { -60.0f,    -28.0f},
  { -30.0f,    -28.0f},
  { 30.0f,    -56.0f},
  { 90.0f,    -56.0f},
  { 150.0f,    -56.0f},
  { -150.0f,    -56.0f},
  { -90.0f,    -56.0f},
  { -30.0f,    -56.0f},
  { 90.0f,    -80.0f},
  { -90.0f,    -80.0f}};

/* Zylia Lab */
const float __Zylia_Lab_dirs_deg[22][2] =
{ { 45.0f,    0.0f},
  { -45.0f,    0.0f},
  { 0.0f,    0.0f},
  { 135.0f,    0.0f},
  { -135.0f,    0.0f},
  { 30.0f,    0.0f},
  { -30.0f,    0.0f},
  { -180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 45.0f,    30.0f},
  { -45.0f,    30.0f},
  { 0.0f,    30.0f},
  { 0.0f,    90.0f},
  { 135.0f,    30.0f},
  { -135.0f,    30.0f},
  { 90.0f,    30.0f},
  { -90.0f,    30.0f},
  { 180.0f,    30.0f},
  { 0.0f,    -25.0f},
  { 45.0f,    -25.0f},
  { -45.0f,    -25.0f}};

/* 64 point sphere-covering (IN RADIANS!) */
const float __default_LScoords64_rad[64][2] =
{ { 0.0f,     1.57079632679490f},
  { 2.39996322972865f,     1.31814860051033f},
  { -1.48325884772228f,     1.21253510828228f},
  { 0.916704382006374f,     1.13082037200398f},
  { -2.96651769544456f,     1.06135247798300f},
  { -0.566554465715905f,     0.999627828229359f},
  { 1.83340876401275f,     0.943340553871706f},
  { -2.04981331343819f,     0.891122507886653f},
  { 0.350149916290469f,     0.842082876051491f},
  { 2.75011314601912f,     0.795602953484535f},
  { -1.13310893143181f,     0.751231999126636f},
  { 1.26685429829684f,     0.708629271562384f},
  { -2.61636777915409f,     0.667529444635833f},
  { -0.216404549425435f,     0.627720817657913f},
  { 2.18355868030321f,     0.589030970216274f},
  { -1.69966339714772f,     0.551316969131883f},
  { 0.700299832580937f,     0.514458475561187f},
  { 3.10026306230959f,     0.478352764649258f},
  { -0.782959015141339f,     0.442911044073639f},
  { 1.61700421458731f,     0.408055677301279f},
  { -2.26621786286362f,     0.373718050942914f},
  { 0.133745366865034f,     0.339836909454122f},
  { 2.53370859659368f,     0.306357034556919f},
  { -1.34951348085725f,     0.273228182540638f},
  { 1.05044974887141f,     0.240404216769259f},
  { -2.83277232857953f,     0.207842389353347f},
  { -0.432809098850870f,     0.175502737575723f},
  { 1.96715413087779f,     0.143347568905365f},
  { -1.91606794657316f,     0.111341014340964f},
  { 0.483895283155495f,     0.0794486340851373f},
  { 2.88385851288415f,     0.0476370626244032f},
  { -0.999363564566781f,     0.0158736824899558f},
  { 1.40059966516187f,     -0.0158736824899556f},
  { -2.48262241228906f,     -0.0476370626244031f},
  { -0.0826591825604016f,     -0.0794486340851372f},
  { 2.31730404716825f,     -0.111341014340964f},
  { -1.56591803028268f,     -0.143347568905365f},
  { 0.834045199445964f,     -0.175502737575723f},
  { -3.04917687800497f,     -0.207842389353347f},
  { -0.649213648276312f,     -0.240404216769259f},
  { 1.75074958145234f,     -0.273228182540638f},
  { -2.13247249599859f,     -0.306357034556919f},
  { 0.267490733730067f,     -0.339836909454122f},
  { 2.66745396345872f,     -0.373718050942914f},
  { -1.21576811399222f,     -0.408055677301278f},
  { 1.18419511573643f,     -0.442911044073639f},
  { -2.69902696171450f,     -0.478352764649258f},
  { -0.299063731985844f,     -0.514458475561187f},
  { 2.10089949774281f,     -0.551316969131883f},
  { -1.78232257970812f,     -0.589030970216274f},
  { 0.617640650020536f,     -0.627720817657913f},
  { 3.01760387974919f,     -0.667529444635833f},
  { -0.865618197701740f,     -0.708629271562384f},
  { 1.53434503202690f,     -0.751231999126636f},
  { -2.34887704542402f,     -0.795602953484536f},
  { 0.0510861843046390f,     -0.842082876051491f},
  { 2.45104941403327f,     -0.891122507886652f},
  { -1.43217266341767f,     -0.943340553871706f},
  { 0.967790566310990f,     -0.999627828229359f},
  { -2.91543151113994f,     -1.06135247798300f},
  { -0.515468281411286f,     -1.13082037200398f},
  { 1.88449494831737f,     -1.21253510828228f},
  { -1.99872712913356f,     -1.31814860051033f},
  { 0.401236100595093f,     -1.57079632679490f}};

/* Minimum T-design of degree 1 (2 points) */
const float __Tdesign_degree_1_dirs_deg[2][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f}};

/* Minimum T-design of degree 2 (4 points) */
const float __Tdesign_degree_2_dirs_deg[4][2] =
{ { 45.0f,    35.2643896827547f},
  { -45.0f,    -35.2643896827547f},
  { 135.0f,    -35.2643896827547f},
  { -135.0f,    35.2643896827547f}};

/* Minimum T-design of degree 3 (6 points) */
const float __Tdesign_degree_3_dirs_deg[6][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f}};

/* Minimum T-design of degree 4 (12 points) */
const float __Tdesign_degree_4_dirs_deg[12][2] =
{ { 0.0f,   -31.7174744114557f},
  { -58.2825255885443f,    0.0f},
  { -90.0f,    58.2825255885443f},
  { 0.0f,    31.7174744114557f},
  { -121.717474411456f,    0.0f},
  { 90.0f,    -58.2825255885443f},
  { 180.0f,    -31.7174744114557f},
  { 121.717474411456f,    0.0f},
  { 90.0f,    58.2825255885443f},
  { 180.0f,    31.7174744114557f},
  { 58.2825255885443f,    0.0f},
  { -90.0f,    -58.2825255885443f}};

/* Minimum T-design of degree 5 (12 points) */
const float __Tdesign_degree_5_dirs_deg[12][2] =
{ { 0.0f,    -31.7174744114557f},
  { -58.2825255885443f,    0.0f},
  { -90.0f,    58.2825255885443f},
  { 0.0f,    31.7174744114557f},
  { -121.717474411456f,    0.0f},
  { 90.0f,    -58.2825255885443f},
  { 180.0f,    -31.7174744114557f},
  { 121.717474411456f,    0.0f},
  { 90.0f,    58.2825255885443f},
  { 180.0f,    31.7174744114557f},
  { 58.2825255885443f,    0.0f},
  { -90.0f,    -58.2825255885443f}};

/* Minimum T-design of degree 6 (24 points) */
const float __Tdesign_degree_6_dirs_deg[24][2] =
{ { 26.0011675216559f,    15.4641512961471f},
  { -26.0011675216559f,    -15.4641512961471f},
  { 17.1086452559122f,    -24.9937030546433f},
  { -17.1086452559122f,    24.9937030546433f},
  { 153.998832478344f,    -15.4641512961471f},
  { -153.998832478344f,    15.4641512961471f},
  { 162.891354744088f,    24.9937030546433f},
  { -162.891354744088f,    -24.9937030546433f},
  { 72.8913547440879f,    24.9937030546433f},
  { 107.108645255912f,    -24.9937030546433f},
  { 116.001167521656f,    15.4641512961471f},
  { 63.9988324783441f,    -15.4641512961471f},
  { -107.108645255912f,    24.9937030546433f},
  { -72.8913547440879f,    -24.9937030546433f},
  { -63.9988324783441f,    15.4641512961471f},
  { -116.001167521656f,    -15.4641512961471f},
  { 32.2544599366034f,    60.0253819510733f},
  { -147.745540063397f,    60.0253819510733f},
  { -57.7455400633966f,    60.0253819510733f},
  { 122.254459936603f,    60.0253819510733f},
  { -32.2544599366034f,    -60.0253819510733f},
  { 147.745540063397f,    -60.0253819510733f},
  { 57.7455400633966f,    -60.0253819510733f},
  { -122.254459936603f,    -60.0253819510733f}};

/* Minimum T-design of degree 7 (24 points) */
const float __Tdesign_degree_7_dirs_deg[24][2] =
{ { 26.0011675216559f,    15.4641512961471f},
  { -26.0011675216559f,    -15.4641512961471f},
  { 17.1086452559122f,    -24.9937030546433f},
  { -17.1086452559122f,    24.9937030546433f},
  { 153.998832478344f,    -15.4641512961471f},
  { -153.998832478344f,    15.4641512961471f},
  { 162.891354744088f,    24.9937030546433f},
  { -162.891354744088f,    -24.9937030546433f},
  { 72.8913547440879f,    24.9937030546433f},
  { 107.108645255912f,    -24.9937030546433f},
  { 116.001167521656f,    15.4641512961471f},
  { 63.9988324783441f,    -15.4641512961471f},
  { -107.108645255912f,    24.9937030546433f},
  { -72.8913547440879f,    -24.9937030546433f},
  { -63.9988324783441f,    15.4641512961471f},
  { -116.001167521656f,    -15.4641512961471f},
  { 32.2544599366034f,    60.0253819510733f},
  { -147.745540063397f,    60.0253819510733f},
  { -57.7455400633966f,    60.0253819510733f},
  { 122.254459936603f,    60.0253819510733f},
  { -32.2544599366034f,    -60.0253819510733f},
  { 147.745540063397f,    -60.0253819510733f},
  { 57.7455400633966f,    -60.0253819510733f},
  { -122.254459936603f,    -60.0253819510733f}};

/* Minimum T-design of degree 8 (36 points) */
const float __Tdesign_degree_8_dirs_deg[36][2] =
{ { -31.1058979072627f,    53.6513674205628f},
  { 110.815389050019f,    30.4958163263089f},
  { 148.894102092737f,    53.6513674205628f},
  { 32.2138500404991f,    -17.8303741454541f},
  { 69.1846109499806f,    -30.4958163263089f},
  { -32.2138500404991f,    17.8303741454541f},
  { -69.1846109499805f,    30.4958163263088f},
  { -147.786149959501f,    -17.8303741454541f},
  { -110.815389050020f,    -30.4958163263088f},
  { 147.786149959501f,    17.8303741454541f},
  { 31.1058979072626f,    -53.6513674205628f},
  { -148.894102092737f,    -53.6513674205628f},
  { -21.2458891990563f,    -47.7753220046865f},
  { -108.204102998351f,    38.7823473661649f},
  { 158.754110800944f,    -47.7753220046865f},
  { 139.773839008605f,    -14.0948465161603f},
  { -71.7958970016493f,    -38.7823473661649f},
  { -139.773839008605f,    14.0948465161603f},
  { 71.7958970016493f,    38.7823473661649f},
  { -40.2261609913953f,    -14.0948465161603f},
  { 108.204102998351f,    -38.7823473661649f},
  { 40.2261609913953f,    14.0948465161603f},
  { 21.2458891990563f,    47.7753220046865f},
  { -158.754110800944f,    47.7753220046865f},
  { 106.650497293011f,    -2.55185592464980f},
  { -2.66337948016217f,    -16.6335052481040f},
  { -73.3495027069895f,    -2.55185592464980f},
  { -98.8410299750320f,    73.1605627016104f},
  { -177.336620519838f,    16.6335052481040f},
  { 98.8410299750322f,    -73.1605627016104f},
  { 177.336620519838f,    -16.6335052481040f},
  { 81.1589700249678f,    73.1605627016104f},
  { 2.66337948016217f,    16.6335052481040f},
  { -81.1589700249678f,    -73.1605627016104f},
  { -106.650497293011f,    2.55185592464980f},
  { 73.3495027069895f,    2.55185592464980f}};

/* Minimum T-design of degree 9 (48 points) */
const float __Tdesign_degree_9_dirs_deg[48][2] =
{ { 20.7458095293620f,    -3.55196449756749f},
  { -20.7458095293620f,    3.55196449756749f},
  { -3.79754064134731f,    -20.7041249340221f},
  { 3.79754064134731f,    20.7041249340221f},
  { 159.254190470638f,    3.55196449756749f},
  { -159.254190470638f,    -3.55196449756749f},
  { -176.202459358653f,    20.7041249340221f},
  { 176.202459358653f,    -20.7041249340221f},
  { 93.7975406413473f,    20.7041249340221f},
  { 86.2024593586527f,    -20.7041249340221f},
  { 110.745809529362f,    -3.55196449756749f},
  { 69.2541904706380f,    3.55196449756749f},
  { -86.2024593586527f,    20.7041249340221f},
  { -93.7975406413473f,    -20.7041249340221f},
  { -69.2541904706380f,    -3.55196449756749f},
  { -110.745809529362f,    3.55196449756749f},
  { -9.93943655954546f,    68.9655356143243f},
  { 170.060563440455f,    68.9655356143243f},
  { -99.9394365595455f,    68.9655356143243f},
  { 80.0605634404545f,    68.9655356143243f},
  { 9.93943655954546f,    -68.9655356143243f},
  { -170.060563440455f,    -68.9655356143243f},
  { 99.9394365595455f,    -68.9655356143243f},
  { -80.0605634404545f,    -68.9655356143243f},
  { 42.1472502314589f,    17.5681548413340f},
  { -42.1472502314589f,    -17.5681548413340f},
  { 23.1237609836156f,    -39.7724420378712f},
  { -23.1237609836156f,    39.7724420378712f},
  { 137.852749768541f,    -17.5681548413340f},
  { -137.852749768541f,    17.5681548413340f},
  { 156.876239016384f,    39.7724420378712f},
  { -156.876239016384f,    -39.7724420378712f},
  { 66.8762390163844f,    39.7724420378712f},
  { 113.123760983616f,    -39.7724420378712f},
  { 132.147250231459f,    17.5681548413340f},
  { 47.8527497685411f,    -17.5681548413340f},
  { -113.123760983616f,    39.7724420378712f},
  { -66.8762390163844f,    -39.7724420378712f},
  { -47.8527497685411f,    17.5681548413340f},
  { -132.147250231459f,    -17.5681548413340f},
  { 25.2587207267662f,    44.9785247162095f},
  { -154.741279273234f,    44.9785247162095f},
  { -64.7412792732338f,    44.9785247162095f},
  { 115.258720726766f,    44.9785247162095f},
  { -25.2587207267662f,    -44.9785247162095f},
  { 154.741279273234f,    -44.9785247162095f},
  { 64.7412792732338f,    -44.9785247162095f},
  { -115.258720726766f,    -44.9785247162095f}};

/* Minimum T-design of degree 10 (60 points) */
const float __Tdesign_degree_10_dirs_deg[60][2] =
{ { 144.086488857244f,    -21.4458167943600f},
  { -33.8100172036462f,    -48.9231262881892f},
  { -35.9135111427562f,    -21.4458167943601f},
  { -115.874264298536f,    33.0897377259147f},
  { -146.189982796354f,    48.9231262881893f},
  { 115.874264298536f,    -33.0897377259147f},
  { 146.189982796354f,    -48.9231262881893f},
  { 64.1257357014642f,    33.0897377259148f},
  { 33.8100172036461f,    48.9231262881892f},
  { -64.1257357014642f,    -33.0897377259147f},
  { -144.086488857244f,    21.4458167943600f},
  { 35.9135111427562f,    21.4458167943600f},
  { -45.5257844715611f,    1.95356582377390f},
  { 177.263270291625f,    44.4415290143179f},
  { 134.474215528439f,    1.95356582377390f},
  { 87.2126547494995f,    -45.4918765044616f},
  { 2.73672970837549f,    -44.4415290143179f},
  { -87.2126547494994f,    45.4918765044617f},
  { -2.73672970837557f,    44.4415290143178f},
  { -92.7873452505006f,    -45.4918765044617f},
  { -177.263270291625f,    -44.4415290143179f},
  { 92.7873452505006f,    45.4918765044617f},
  { 45.5257844715611f,    -1.95356582377396f},
  { -134.474215528439f,    -1.95356582377396f},
  { 15.5857325914359f,    -73.3350170374203f},
  { -85.4016616760217f,    16.0353405554795f},
  { -164.414267408564f,    -73.3350170374203f},
  { 163.915552255786f,    4.41906100711915f},
  { -94.5983383239783f,    -16.0353405554795f},
  { -163.915552255786f,    -4.41906100711904f},
  { 94.5983383239784f,    16.0353405554796f},
  { -16.0844477442143f,    4.41906100711910f},
  { 85.4016616760218f,    -16.0353405554795f},
  { 16.0844477442145f,    -4.41906100711915f},
  { -15.5857325914359f,    73.3350170374202f},
  { 164.414267408564f,    73.3350170374204f},
  { -60.0247824185970f,    25.2687130564713f},
  { 151.413274180519f,    26.8604557126932f},
  { 119.975217581403f,    25.2687130564713f},
  { 46.6267878204903f,    -51.5688772343955f},
  { 28.5867258194805f,    -26.8604557126932f},
  { -46.6267878204902f,    51.5688772343955f},
  { -28.5867258194805f,    26.8604557126931f},
  { -133.373212179510f,    -51.5688772343955f},
  { -151.413274180520f,    -26.8604557126932f},
  { 133.373212179510f,    51.5688772343955f},
  { 60.0247824185970f,    -25.2687130564714f},
  { -119.975217581403f,    -25.2687130564713f},
  { -109.941626532878f,    6.91123181401059f},
  { 172.652618018678f,    -19.7906490831313f},
  { 70.0583734671221f,    6.91123181401071f},
  { -70.4350022017499f,    -68.9408617534649f},
  { 7.34738198132191f,    19.7906490831313f},
  { 70.4350022017499f,    68.9408617534649f},
  { -7.34738198132179f,    -19.7906490831314f},
  { 109.564997798250f,    -68.9408617534649f},
  { -172.652618018678f,    19.7906490831314f},
  { -109.564997798250f,    68.9408617534649f},
  { 109.941626532878f,    -6.91123181401059f},
  { -70.0583734671221f,    -6.91123181401053f}};

/* Minimum T-design of degree 11 (70 points) */
const float __Tdesign_degree_11_dirs_deg[70][2] =
{ { 132.927291884332f,    7.69254738757899f},
  { -83.9349499672527f,    -23.7300652200871f},
  { 8.47410038634525f,    23.5127556185301f},
  { -113.340833834572f,    70.4225940747938f},
  { -103.265909909537f,    -9.89694439538752f},
  { -33.2370360923825f,    -70.7513316063095f},
  { 21.8564347471830f,    -26.4618527647561f},
  { -156.539486489880f,    47.7764936689044f},
  { -64.2647531387317f,    -7.72047049524459f},
  { 165.779530068738f,    44.5343602375216f},
  { -25.2028339893249f,    26.3897904767450f},
  { -97.0037973959711f,    -44.6578850137166f},
  { 27.8546391256925f,    9.76703456924600f},
  { 153.214218975132f,    -47.7053318175498f},
  { -155.061608694663f,    7.45302934155972f},
  { -11.8421354925543f,    -23.5901209534773f},
  { 80.5387312016125f,    23.7194484034707f},
  { -42.0561606270165f,    70.4382693912270f},
  { -31.2233262205060f,    -9.83541588740259f},
  { 38.8379041944063f,    -70.4980825105727f},
  { 93.7606877469492f,    -26.2949218109204f},
  { -84.7560200078398f,    47.6148028805222f},
  { 7.75536818082863f,    -7.51718499746626f},
  { -122.276883381108f,    44.2862347125773f},
  { 46.8012705252113f,    26.6442619674660f},
  { -24.7686335284573f,    -44.5693707254340f},
  { 99.8904719062334f,    9.91271928508000f},
  { -134.783996960185f,    -47.9599550372574f},
  { -83.0880230164493f,    7.29679922953795f},
  { 60.1281736000420f,    -23.3445981426306f},
  { 152.644656278084f,    23.6415261666079f},
  { 29.7576658909417f,    70.6843143997832f},
  { 40.7793187974476f,    -9.58140351749889f},
  { 110.183927562412f,    -70.3934534122902f},
  { 165.652065916454f,    -26.4258159091605f},
  { -12.9926632105736f,    47.7510668062369f},
  { 79.7359893585681f,    -7.30853603036844f},
  { -50.5245271190884f,    44.2632768570349f},
  { 118.923930267733f,    26.7140614474957f},
  { 47.2202861862577f,    -44.3149733480527f},
  { 171.925276523721f,    9.75899721561506f},
  { -62.5145800558502f,    -48.0361913333593f},
  { -11.1156697680531f,    7.43965099805872f},
  { 132.018041099963f,    -23.3326075548841f},
  { -135.355486412425f,    23.3868959687598f},
  { 102.370921576708f,    70.8219078016791f},
  { 112.739282398012f,    -9.48596399169388f},
  { -178.304963670831f,    -70.5801867828491f},
  { -122.319932198534f,    -26.6740262349265f},
  { 59.0763464570905f,    47.9978414043199f},
  { 151.704200334501f,    -7.38276167631068f},
  { 21.3763364190503f,    44.4970603752708f},
  { -169.005476417779f,    26.5024990214418f},
  { 118.980811786769f,    -44.2461913308458f},
  { -116.089295979010f,    9.51845076548334f},
  { 9.64767870353308f,    -47.8281351088411f},
  { 60.8933243657771f,    7.68427447425834f},
  { -156.021526862757f,    -23.5706842106942f},
  { -63.4602993325163f,    23.3074499244045f},
  { 174.929787427393f,    70.6586472132300f},
  { -175.288768596346f,    -9.68088860263008f},
  { -105.951907934032f,    -70.8026785673948f},
  { -50.1928304519800f,    -26.6963451935976f},
  { 131.358266702971f,    48.0136296461397f},
  { -136.296815007542f,    -7.63734823159200f},
  { 93.5644603506407f,    44.6651234222196f},
  { -97.0840116473627f,    26.3023490002159f},
  { -169.158278888619f,    -44.4576351865647f},
  { -44.1323835471345f,    9.52341455917443f},
  { 81.4795403841382f,    -47.6242211091394f}};

/* Minimum T-design of degree 12 (84 points) */
const float __Tdesign_degree_12_dirs_deg[84][2] =
{ { -154.471854278240f,    7.90217308341494f},
  { 162.147490498404f,    -63.3553431389084f},
  { 25.5281457217257f,    7.90217307936515f},
  { -81.2554874853320f,    -25.2685910920330f},
  { 17.8525094920942f,    63.3553431383131f},
  { 81.2554874704586f,    25.2685910914935f},
  { -17.8525095073670f,    -63.3553431416872f},
  { 98.7445125165515f,    -25.2685910933493f},
  { -162.147490509394f,    63.3553431422790f},
  { -98.7445125279264f,    25.2685910939373f},
  { 154.471854265406f,    -7.90217308016803f},
  { -25.5281457350890f,    -7.90217308261847f},
  { 1.29947714901319f,    -10.4733705479448f},
  { -83.0059666491382f,    79.4472145506146f},
  { -178.700522850663f,    -10.4733705444250f},
  { 100.476005172511f,    1.27782359434127f},
  { -96.9940333623604f,    -79.4472145487100f},
  { -100.476005185653f,    -1.27782359370633f},
  { 96.9940333304727f,    79.4472145491341f},
  { -79.5239948274546f,    1.27782359558816f},
  { 83.0059666551207f,    -79.4472145510240f},
  { 79.5239948144318f,    -1.27782359617381f},
  { -1.29947716210305f,    10.4733705444298f},
  { 178.700522837562f,    10.4733705478752f},
  { 157.241693353358f,    13.1533445898284f},
  { 31.1357511222634f,    -63.8901573764588f},
  { -22.7583066459503f,    13.1533445871984f},
  { -75.7795388656139f,    22.1291502833803f},
  { 148.864248861286f,    63.8901573754847f},
  { 75.7795388539796f,    -22.1291502841989f},
  { -148.864248878128f,    -63.8901573724759f},
  { 104.220461132715f,    22.1291502824035f},
  { -31.1357511316254f,    63.8901573733956f},
  { -104.220461147326f,    -22.1291502815231f},
  { -157.241693366750f,    -13.1533445864770f},
  { 22.7583066331656f,    -13.1533445905536f},
  { 110.436905260280f,    -60.6153505191573f},
  { -62.1805578014362f,    -9.86516586433484f},
  { -69.5630947468225f,    -60.6153505185931f},
  { -168.876004352720f,    27.3733119168848f},
  { -117.819442211826f,    9.86516586603182f},
  { 168.876004339933f,    -27.3733119134699f},
  { 117.819442199355f,    -9.86516586429413f},
  { 11.1239956478932f,    27.3733119130858f},
  { 62.1805577877021f,    9.86516586259459f},
  { -11.1239956613407f,    -27.3733119165322f},
  { -110.436905267536f,    60.6153505204452f},
  { 69.5630947277636f,    60.6153505173552f},
  { -125.929940749245f,    -47.3994855979503f},
  { -126.671481040740f,    -23.4027352002794f},
  { 54.0700592527834f,    -47.3994856016111f},
  { -151.648882073130f,    -33.2364603810367f},
  { -53.3285189719047f,    23.4027352017874f},
  { 151.648882058869f,    33.2364603842090f},
  { 53.3285189601170f,    -23.4027352039908f},
  { 28.3511179269492f,    -33.2364603851095f},
  { 126.671481026299f,    23.4027352024511f},
  { -28.3511179389262f,    33.2364603819160f},
  { 125.929940732929f,    47.3994856000570f},
  { -54.0700592627114f,    47.3994855994707f},
  { 61.4093869712020f,    37.5359641921079f},
  { 41.1867022349720f,    22.3011497399151f},
  { -118.590613027003f,    37.5359641955588f},
  { 31.9165501791293f,    44.1290873505536f},
  { 138.813297752503f,    -22.3011497411599f},
  { -31.9165501940578f,    -44.1290873535754f},
  { -138.813297764599f,    22.3011497439225f},
  { -148.083449820620f,    44.1290873545683f},
  { -41.1867022490880f,    -22.3011497426817f},
  { 148.083449809271f,    -44.1290873515606f},
  { -61.4093869868180f,    -37.5359641938559f},
  { 118.590613016358f,    -37.5359641937859f},
  { 132.923697873404f,    4.73307797901005f},
  { 6.45099119915111f,    -42.7422526473549f},
  { -47.0763021263360f,    4.73307797785435f},
  { -83.0682214348878f,    46.8666308746944f},
  { 173.549008786020f,    42.7422526471375f},
  { 83.0682214256211f,    -46.8666308750510f},
  { -173.549008799468f,    -42.7422526436592f},
  { 96.9317785610360f,    46.8666308731931f},
  { -6.45099121192998f,    42.7422526438352f},
  { -96.9317785780300f,    -46.8666308727651f},
  { -132.923697886652f,    -4.73307797647834f},
  { 47.0763021134026f,    -4.73307798038859f}};

/* Minimum T-design of degree 13 (94 points) */
const float __Tdesign_degree_13_dirs_deg[94][2] =
{ { -40.3627167222348f,    68.6994138693883f},
  { 61.1178757094869f,    65.6841788540926f},
  { 141.729207778738f,    70.7473887065821f},
  { -131.254071010305f,    72.3178990997495f},
  { -154.879003534434f,    -12.6182154153575f},
  { -66.1952941496760f,    -9.78390290250878f},
  { 26.3619203195797f,    -11.9654063218404f},
  { 114.952572653282f,    -12.5785685168723f},
  { 37.0184004816904f,    51.1256389892300f},
  { 129.769829567165f,    51.9548524146586f},
  { -140.632268790346f,    50.1516259787950f},
  { -56.5007238296603f,    47.8775543682241f},
  { -65.0474273466871f,    12.5785685169632f},
  { 25.1209964655500f,    12.6182154151376f},
  { 113.804705850467f,    9.78390290268633f},
  { -153.638079680491f,    11.9654063218214f},
  { -134.514914083258f,    -9.72778866158662f},
  { -46.2302615549395f,    -8.36621679129740f},
  { 47.9052586063038f,    -9.73252662507003f},
  { 141.506162727637f,    -8.72815719945242f},
  { -17.8362968684834f,    -44.0982842994686f},
  { 69.3686740106101f,    -43.2665797312820f},
  { 151.216430500609f,    -42.6650502889612f},
  { -106.776151619671f,    -40.1759720348795f},
  { -50.2301704328105f,    -51.9548524144938f},
  { 39.3677312097665f,    -50.1516259788305f},
  { 123.499276170276f,    -47.8775543681000f},
  { -142.981599518156f,    -51.1256389893727f},
  { -179.192692804150f,    -60.7498597598460f},
  { -84.5683722244082f,    -54.0721250135385f},
  { 5.39023497898549f,    -58.0518216444931f},
  { 89.5021760546650f,    -60.7465965609431f},
  { -145.976954290766f,    31.0152557263501f},
  { -54.3906597983015f,    26.4284741481704f},
  { 28.9208513184387f,    32.5135882095920f},
  { 125.336198321601f,    30.9431238792627f},
  { 168.711773251522f,    -7.05940715762768f},
  { -112.485952534025f,    -10.3751837493173f},
  { -21.9560854554006f,    -9.60167976599116f},
  { 73.1112518340806f,    -8.31254840811588f},
  { 95.6758711125988f,    0.0405182785108394f},
  { -170.708278926686f,    2.32277503975178f},
  { -84.3241288875143f,    -0.0405182784656330f},
  { 9.29172107320429f,    -2.32277503993912f},
  { 9.18585986004928f,    -34.3279751514861f},
  { 98.2142505588550f,    -37.3058168381220f},
  { -179.203725583107f,    -40.4848300096384f},
  { -77.8092661173471f,    -31.6042234205206f},
  { -177.084812005157f,    -21.7372544942682f},
  { -93.7724854158139f,    -18.8290843901069f},
  { -2.71509126119301f,    -19.7973423473147f},
  { 90.5055865842376f,    -20.9130131157502f},
  { -106.888748165935f,    8.31254840848543f},
  { -11.2882267487850f,    7.05940715728591f},
  { 67.5140474661053f,    10.3751837496826f},
  { 158.043914544759f,    9.60167976557666f},
  { -118.882124290049f,    -65.6841788541697f},
  { -38.2707922214634f,    -70.7473887063923f},
  { 48.7459289902140f,    -72.3178990998148f},
  { 139.637283277007f,    -68.6994138693407f},
  { -54.6638016787848f,    -30.9431238791174f},
  { 34.0230457094863f,    -31.0152557263048f},
  { 125.609340201678f,    -26.4284741480840f},
  { -151.079148681228f,    -32.5135882098331f},
  { -170.814140140253f,    34.3279751514361f},
  { -81.7857494411369f,    37.3058168382986f},
  { 0.796274416838389f,    40.4848300092311f},
  { 102.190733882995f,    31.6042234207107f},
  { -28.7835694993777f,    42.6650502888879f},
  { 73.2238483805505f,    40.1759720348183f},
  { 162.163703131160f,    44.0982842994090f},
  { -110.631325989256f,    43.2665797315176f},
  { -89.4944134156443f,    20.9130131158586f},
  { 2.91518799464472f,    21.7372544938146f},
  { 86.2275145844840f,    18.8290843902342f},
  { 177.284908738826f,    19.7973423471420f},
  { 133.769738445099f,    8.36621679123653f},
  { -132.094741393724f,    9.73252662529703f},
  { -38.4938372724335f,    8.72815719936124f},
  { 45.4850859168882f,    9.72778866179206f},
  { -25.6021660105829f,    24.0412140934934f},
  { 55.1230998001090f,    30.2272431389321f},
  { 149.298950110694f,    28.0455421626317f},
  { -118.706558467820f,    26.0579506724807f},
  { 0.807307195922662f,    60.7498597594458f},
  { 95.4316277754395f,    54.0721250137300f},
  { -174.609765021233f,    58.0518216443168f},
  { -90.4978239451188f,    60.7465965612768f},
  { -124.876900199724f,    -30.2272431386341f},
  { -30.7010498895602f,    -28.0455421629911f},
  { 61.2934415322725f,    -26.0579506721723f},
  { 154.397833989464f,    -24.0412140935974f},
  { -132.915308725410f,    -85.6000936509801f},
  { 47.0846912731536f,    85.6000936508629f}};

/* Minimum T-design of degree 14 (108 points) */
const float __Tdesign_degree_14_dirs_deg[108][2] =
{ { -129.185496888476f,    8.10563028578741f},
  { 169.578599576646f,    -38.7292166126126f},
  { 50.8148695271600f,    8.12050690100986f},
  { -77.2740193615098f,    -50.1149231885907f},
  { 10.4239654092118f,    38.7310086728624f},
  { 77.2976960009371f,    50.1174305751531f},
  { -10.4119358669821f,    -38.7198037761506f},
  { 102.710075564305f,    -50.1077672707642f},
  { -169.569875076608f,    38.7180119969233f},
  { -102.712992621563f,    50.1052603901109f},
  { 129.194616327794f,    -8.11282927824916f},
  { -50.8032326138493f,    -8.11330777101311f},
  { -4.59215033966074f,    -56.0145426200334f},
  { -93.0961080949996f,    33.8503331657228f},
  { 175.391842883684f,    -56.0251031820136f},
  { 146.110504990858f,    -2.57401246703588f},
  { -86.8931538003163f,    -33.8602245481812f},
  { -146.100412500120f,    2.56455577491969f},
  { 86.9111644059348f,    33.8608402235944f},
  { -33.8888402494298f,    -2.57007990733392f},
  { 93.0988554959334f,    -33.8509487698007f},
  { 33.8995035311197f,    2.57953664040591f},
  { 4.60388295935663f,    56.0258986613491f},
  { -175.382824609272f,    56.0137473745907f},
  { 106.566908554445f,    26.0969762431768f},
  { 27.0704791250355f,    -14.8221953013083f},
  { -73.4398224140619f,    26.0907279761036f},
  { -30.2019611726677f,    59.4125144873598f},
  { 152.942234123029f,    14.8267023571315f},
  { 30.2026449688438f,    -59.4026682881680f},
  { -152.930483939354f,    -14.8368475821330f},
  { 149.822214569457f,    59.4076496872466f},
  { -27.0614738204129f,    14.8323403149657f},
  { -149.802145490903f,    -59.4174973348394f},
  { -106.551181547953f,    -26.1002232021554f},
  { 73.4448527743907f,    -26.0874812806078f},
  { -171.424549759738f,    77.4508785133046f},
  { 91.8987958716823f,    -12.3962958134667f},
  { 8.53903505084858f,    77.4636176315132f},
  { -12.4022480507938f,    -1.84616868079121f},
  { 88.1115107175677f,    12.4061962956428f},
  { 12.4127047713575f,    1.85729530242821f},
  { -88.0986284057768f,    -12.4058195758491f},
  { 167.597360255578f,    -1.85516686202351f},
  { -91.8909209594892f,    12.3959191080322f},
  { -167.587061725650f,    1.84404025367733f},
  { 171.427293256838f,    -77.4621438946096f},
  { -8.52106137787343f,    -77.4523509482603f},
  { -122.725444898284f,    -10.4436681618452f},
  { -167.646619541773f,    -32.1304560946658f},
  { 57.2737763995243f,    -10.4291751560142f},
  { -108.800959121800f,    -55.8290478813134f},
  { -12.3490789438205f,    32.1283371856927f},
  { 108.827221090328f,    55.8253737505810f},
  { 12.3579262049535f,    -32.1172082630069f},
  { 71.1878632374232f,    -55.8159974423682f},
  { 167.658526826702f,    32.1193269135384f},
  { -71.1933648961921f,    55.8196706876724f},
  { 122.737588435530f,    10.4375080894726f},
  { -57.2651633537552f,    10.4353350634210f},
  { -135.835854784211f,    -23.0547585640994f},
  { -148.583253700629f,    -41.3187537286755f},
  { 44.1637920456130f,    -23.0396843683594f},
  { -120.662099788593f,    -39.8769052826722f},
  { -31.4137988125835f,    41.3135904099716f},
  { 120.680663009779f,    39.8710937105608f},
  { 31.4189576996095f,    -41.3038678530177f},
  { 59.3339359050566f,    -39.8625737566942f},
  { 148.598849815269f,    41.3090304016975f},
  { -59.3317418485565f,    39.8683846070324f},
  { 135.849609497709f,    23.0465850901611f},
  { -44.1567906522068f,    23.0478574234931f},
  { -161.547774160409f,    20.6214832030571f},
  { 130.038592068425f,    -62.5981252668102f},
  { 18.4500454157052f,    20.6354249039959f},
  { -68.3479254388390f,    -17.2274007359150f},
  { 49.9594914575609f,    62.6057092231770f},
  { 68.3615869487824f,    17.2316030343816f},
  { -49.9322890507680f,    -62.5983778933698f},
  { 111.647658776834f,    -17.2223955106639f},
  { -130.045036899004f,    62.5907958085244f},
  { -111.640563285857f,    17.2181934215587f},
  { 161.556794742584f,    -20.6322904708357f},
  { -18.4383108625927f,    -20.6246174136712f},
  { -105.229923721163f,    -3.38418721629137f},
  { -176.495407017449f,    -15.2142387436252f},
  { 74.7695811147565f,    -3.37163564281617f},
  { -102.637896944791f,    -74.4094065275369f},
  { -3.49690334883675f,    15.2136338742552f},
  { 102.688108006896f,    74.4069090611730f},
  { 3.50709175282307f,    -15.2022624413437f},
  { 77.3300588827299f,    -74.3972447248446f},
  { 176.505973637731f,    15.2028672780281f},
  { -77.3594900583547f,    74.3997406824127f},
  { 105.240951032319f,    3.38119337243154f},
  { -74.7598514248982f,    3.37462945722995f},
  { -142.390512576284f,    25.4175908027638f},
  { 142.077529927039f,    -45.6868543254021f},
  { 37.6090614633499f,    25.4326612225385f},
  { -59.0248292186823f,    -33.4439538525117f},
  { 37.9248440888004f,    45.6929425950841f},
  { 59.0416596504450f,    33.4498158632456f},
  { -37.9072953200755f,    -45.6839547544076f},
  { 120.965352734802f,    -33.4413214611586f},
  { -142.074323295910f,    45.6778674627608f},
  { -120.961426152688f,    33.4354600243172f},
  { 142.397585749735f,    -25.4266164045797f},
  { -37.5953788557179f,    -25.4236351680798f}};

/* Minimum T-design of degree 15 (120 points) */
const float __Tdesign_degree_15_dirs_deg[120][2] =
{ { -30.6030525550461f,    6.94295470706933f},
  { 166.556601881679f,    58.6896167148965f},
  { 149.397921574412f,    6.95676040033414f},
  { 81.9503184046850f,    -30.3562374013217f},
  { 13.4838352132044f,    -58.6862798372820f},
  { -81.9279696395992f,    30.3552068726552f},
  { -13.4632635060741f,    58.6787728253845f},
  { -98.0553212926679f,    -30.3693638204857f},
  { -166.541916444220f,    -58.6821134278307f},
  { 98.0687585715236f,    30.3704560408842f},
  { 30.6215511098796f,    -6.94955577140730f},
  { -149.380390687206f,    -6.95008925738623f},
  { 106.689953124681f,    -22.6783689069045f},
  { -23.5657187755504f,    -15.3628427250753f},
  { -73.3114441250527f,    -22.6942854521603f},
  { -145.501414237039f,    62.1042310518677f},
  { -156.412773408714f,    15.3570707316218f},
  { 145.527116313848f,    -62.0978793943380f},
  { 156.431582868341f,    -15.3501087206496f},
  { 34.4680326551376f,    62.1059340346835f},
  { 23.5828302580784f,    15.3556650727373f},
  { -34.4587547006376f,    -62.1123240120557f},
  { -106.668780509784f,    22.6806221234814f},
  { 73.3264081825364f,    22.6921040187581f},
  { 166.815146476476f,    1.39137137645932f},
  { 6.09164877259154f,    -76.7377960648265f},
  { -13.1851422017595f,    1.38060121346851f},
  { -88.5665379871222f,    13.1827681324909f},
  { 173.986274618312f,    76.7393112171196f},
  { 88.5863712660644f,    -13.1829911005176f},
  { -173.972192649359f,    -76.7316416441619f},
  { 91.4317067978856f,    13.1972659475335f},
  { -6.07044089187156f,    76.7301145219168f},
  { -91.4155231870143f,    -13.1971051505712f},
  { -166.797273031735f,    -1.38387705718332f},
  { 13.2031163001391f,    -1.38813486593518f},
  { -74.6719653628180f,    48.1128159205733f},
  { 130.860845670697f,    10.1648476117330f},
  { 105.324001632841f,    48.1286252881803f},
  { 13.3402738550233f,    -40.0816483636324f},
  { 49.1587806791108f,    -10.1540507848293f},
  { -13.3209329958570f,    40.0741920019256f},
  { -49.1397371265836f,    10.1490822313170f},
  { -166.672794690516f,    -40.0774058487576f},
  { -130.843884042363f,    -10.1597669438345f},
  { 166.689200908705f,    40.0849666149935f},
  { 74.6979186340827f,    -48.1147886327798f},
  { -105.314588513838f,    -48.1265844469817f},
  { -126.994710754480f,    26.5534844401442f},
  { 147.963786150663f,    -32.5685324439693f},
  { 52.9979184448058f,    26.5602526628589f},
  { -50.2839814792401f,    -45.5940649038881f},
  { 32.0464832809211f,    32.5759524833013f},
  { 50.2955756473413f,    45.5891069117364f},
  { -32.0312406436033f,    -32.5825905716800f},
  { 129.712795354186f,    -45.5781144439024f},
  { -147.943226072877f,    32.5749436398314f},
  { -129.689094831424f,    45.5830214583665f},
  { 127.015829484310f,    -26.5488550940039f},
  { -52.9830053081905f,    -26.5648704461645f},
  { -171.925165097520f,    30.3656373764858f},
  { 103.474315132854f,    -58.6762932898078f},
  { 8.06598910342026f,    30.3599497256541f},
  { -59.3793154805561f,    -6.95793281328386f},
  { 76.5377173373163f,    58.6902848585247f},
  { 59.3965972400983f,    6.95406151527783f},
  { -76.5325167840642f,    -58.6920609528879f},
  { 120.620572530444f,    -6.94173212922671f},
  { -103.444514431496f,    58.6781755803937f},
  { -120.601682190330f,    6.94569013736595f},
  { 171.943791968762f,    -30.3580618441019f},
  { -8.04868833080616f,    -30.3676509880131f},
  { 40.8591506674273f,    10.1586784460910f},
  { 15.3172714874563f,    48.1188014520829f},
  { -139.138012788032f,    10.1552322152725f},
  { 76.6795691484389f,    40.0856595700319f},
  { 164.685199993217f,    -48.1151686422521f},
  { -76.6680975158839f,    -40.0873731256771f},
  { -164.665133980975f,    48.1225223368022f},
  { -103.311377388557f,    40.0735260689792f},
  { -15.3018010383944f,    -48.1263067557738f},
  { 103.335438313858f,    -40.0716468871635f},
  { -40.8419970065716f,    -10.1644693751345f},
  { 139.156972194678f,    -10.1493915261064f},
  { 103.203017496907f,    -1.37818298743888f},
  { -1.41295359216869f,    -13.1940817732791f},
  { -76.7970625008060f,    -1.39388969574653f},
  { -96.0226016809032f,    76.7280399662938f},
  { -178.565790556190f,    13.1937156944364f},
  { 96.0727952593646f,    -76.7271870367777f},
  { 178.583741131798f,    -13.1860167806370f},
  { 83.9384305400478f,    76.7414148352344f},
  { 1.43082080108730f,    13.1863482688284f},
  { -83.9533413278650f,    -76.7422528505934f},
  { -103.184809097502f,    1.37992019271170f},
  { 76.8149044656686f,    1.39206982161959f},
  { 37.0203773932360f,    -26.5556799548844f},
  { -39.6998537331091f,    45.5785590048425f},
  { -142.987611559503f,    -26.5580137886591f},
  { 122.051959505631f,    32.5835914894780f},
  { -140.293652413608f,    -45.5876530266288f},
  { -122.038265069292f,    -32.5795114954413f},
  { 140.306374919687f,    45.5935766807050f},
  { -57.9487870815201f,    32.5674048936121f},
  { 39.7225744490892f,    -45.5845144211182f},
  { 57.9707590481942f,    -32.5714872904771f},
  { -37.0001343655174f,    26.5495567479843f},
  { 143.003247815211f,    26.5642004349055f},
  { 163.331127304408f,    22.6920948057362f},
  { 55.5479236516533f,    -62.1013914093322f},
  { -16.6736304241223f,    22.6806589773217f},
  { -66.4147692500420f,    15.3482860362968f},
  { 124.485029630888f,    62.1131464175266f},
  { 66.4346475986745f,    -15.3513839575877f},
  { -124.479439274632f,    -62.1088084825996f},
  { 113.584840357941f,    15.3644980586459f},
  { -55.5182605036017f,    62.0970279081496f},
  { -113.568822266193f,    -15.3614254483317f},
  { -163.314069620612f,    -22.6846279659434f},
  { 16.6925187091527f,    -22.6879825807004f}};

/* Minimum T-design of degree 16 (144 points) */
const float __Tdesign_degree_16_dirs_deg[144][2] =
{ { -10.5692580668100f,    -17.3490128634352f},
  { -120.415474946769f,    69.7642225220975f},
  { 169.426697678052f,    -17.3466306179260f},
  { 107.629281568279f,    -10.0793856720614f},
  { -59.5674172539630f,    -69.7763953095265f},
  { -107.629493132161f,    10.0801907586439f},
  { 59.5651193014662f,    69.7750019669784f},
  { -72.3715136529922f,    -10.0909592824967f},
  { 120.418832045969f,    -69.7630732527928f},
  { 72.3705387467529f,    10.0908849630425f},
  { 10.5699823270029f,    17.3489915451677f},
  { -169.426851150349f,    17.3476277272366f},
  { -30.7728814706980f,    68.2546206344998f},
  { 101.529181734204f,    18.5653877408229f},
  { 149.250234431564f,    68.2619792676765f},
  { 18.9183978612362f,    -10.9197242075287f},
  { 78.4729799557400f,    -18.5558892226653f},
  { -18.9161109968480f,    10.9195275229078f},
  { -78.4723066816396f,    18.5565169424192f},
  { -161.085243560336f,    -10.9220975953767f},
  { -101.530063994217f,    -18.5648176173994f},
  { 161.085855454581f,    10.9237834974412f},
  { 30.7790837424989f,    -68.2562996173874f},
  { -149.255648697558f,    -68.2594892022576f},
  { 56.4638592172755f,    41.2645742763346f},
  { 46.4626904606488f,    24.5386896277818f},
  { -123.528839291756f,    41.2565889648213f},
  { 32.1932295461248f,    38.8027347956192f},
  { 133.534545076742f,    -24.5310453346374f},
  { -32.1935458377634f,    -38.8041545028931f},
  { -133.533477606995f,    24.5319420428588f},
  { -147.797881221742f,    38.7980990097138f},
  { -46.4626630742447f,    -24.5400749997299f},
  { 147.799544204027f,    -38.7965984357137f},
  { -56.4649576930965f,    -41.2658215291547f},
  { 123.530547454259f,    -41.2555316199835f},
  { 84.7385596473317f,    27.3132538556364f},
  { 27.4072860474693f,    4.67593317316088f},
  { -95.2612516264878f,    27.3036967398252f},
  { 10.0607591758466f,    62.2287426217158f},
  { 152.592710471291f,    -4.67007594116539f},
  { -10.0604693351594f,    -62.2317767049329f},
  { -152.592615409799f,    4.67188744845125f},
  { -169.918387889403f,    62.2272712110016f},
  { -27.4047652820400f,    -4.67684895484045f},
  { 169.918493477140f,    -62.2246416140826f},
  { -84.7390238025025f,    -27.3120311727342f},
  { 95.2622793653641f,    -27.3027426350066f},
  { 136.272846725200f,    -0.726139068263599f},
  { -1.05456977390436f,    -46.2698051615206f},
  { -43.7263256019408f,    -0.737666959763443f},
  { -91.0084089447853f,    43.7170295842697f},
  { -178.936314291048f,    46.2682278552731f},
  { 91.0100541861312f,    -43.7163112498585f},
  { 178.938490558618f,    -46.2666218304286f},
  { 88.9860902890987f,    43.7285680951606f},
  { 1.05095788619312f,    46.2682407577363f},
  { -88.9851906686018f,    -43.7278600822776f},
  { -136.273314433943f,    0.726484363994244f},
  { 43.7273191822901f,    0.733967099281741f},
  { 55.2321077221357f,    10.8187456245361f},
  { 13.0867814081219f,    34.0654299770448f},
  { -124.767898552651f,    10.8099467408727f},
  { 71.4751302534919f,    53.7984109774923f},
  { 166.905549455530f,    -34.0625862919868f},
  { -71.4765158530160f,    -53.8002805142714f},
  { -166.904587570338f,    34.0633858486074f},
  { -108.515564585463f,    53.7864186881779f},
  { -13.0869589355977f,    -34.0644403811701f},
  { 108.517274377512f,    -53.7853093207457f},
  { -55.2337689585088f,    -10.8200079671294f},
  { 124.767519417537f,    -10.8096092876561f},
  { -105.493359369733f,    -68.1345228940484f},
  { -111.151973933085f,    -5.71277564515985f},
  { 74.5244447329726f,    -68.1205741555505f},
  { -173.885132403874f,    -21.0401043953034f},
  { -68.8521018079939f,    5.70313885775370f},
  { 173.885050918764f,    21.0419786777516f},
  { 68.8513569996117f,    -5.70297394874866f},
  { 6.12041448982713f,    -21.0387333986971f},
  { 111.151515131908f,    5.71357243375474f},
  { -6.11913621739966f,    21.0397756114589f},
  { 105.487055344890f,    68.1331981897040f},
  { -74.5188619841755f,    68.1214716326340f},
  { 35.2822222163987f,    -15.1801841162885f},
  { -25.1732516880600f,    51.9826514887145f},
  { -144.722913967824f,    -15.1859965048827f},
  { 108.386507250934f,    33.8824333892584f},
  { -154.839566662519f,    -51.9856067440744f},
  { -108.385758587361f,    -33.8826426573830f},
  { 154.840828582605f,    51.9884126722963f},
  { -71.6110721306039f,    33.8714084774643f},
  { 25.1699762263473f,    -51.9843011897216f},
  { 71.6126173250777f,    -33.8705490198003f},
  { -35.2810594358470f,    15.1763090469469f},
  { 144.722474074749f,    15.1863112389402f},
  { -125.277803641269f,    -28.5561977838671f},
  { -146.317489805034f,    -30.4888082061806f},
  { 54.7220842889970f,    -28.5463137051834f},
  { -133.292600372594f,    -45.8199818171681f},
  { -33.6854220654234f,    30.4794807422719f},
  { 133.295169860873f,    45.8200619710773f},
  { 33.6830420572493f,    -30.4828689919959f},
  { 46.7098267598019f,    -45.8106991831118f},
  { 146.318736830907f,    30.4889588527258f},
  { -46.7107531920589f,    45.8077510168776f},
  { 125.278543193551f,    28.5567151663926f},
  { -54.7226679373581f,    28.5424675755949f},
  { -144.402199049652f,    54.7141577397485f},
  { 112.382216591782f,    -28.0100385212576f},
  { 35.5834085793474f,    54.7193853715013f},
  { -29.9155874729000f,    -19.6471282266303f},
  { 67.6163420587218f,    28.0211878299760f},
  { 29.9151078604395f,    19.6461577548916f},
  { -67.6164804090156f,    -28.0221553885221f},
  { 150.081679085309f,    -19.6396879622134f},
  { -112.381365317985f,    28.0105570315277f},
  { -150.081127534239f,    19.6412999777121f},
  { 144.402936805264f,    -54.7127597303097f},
  { -35.5837994140428f,    -54.7203576209200f},
  { 68.5348504928164f,    -52.8527273534571f},
  { -54.8218918027732f,    12.7570858971645f},
  { -111.464947340552f,    -52.8652964226689f},
  { 164.510735700738f,    34.1892226850382f},
  { -125.181243085238f,    -12.7710900404128f},
  { -164.510172672569f,    -34.1865270720665f},
  { 125.180742749445f,    12.7714932994183f},
  { -15.4967238792034f,    34.1855682977571f},
  { 54.8232484475423f,    -12.7607435822084f},
  { 15.4931170933516f,    -34.1868032875460f},
  { -68.5328366887261f,    52.8538785361128f},
  { 111.467075873460f,    52.8645433650926f},
  { 91.4783759173555f,    -7.37343377883849f},
  { -7.38018069449028f,    -1.46605904801040f},
  { -88.5233821655574f,    -7.38351008527854f},
  { -168.689339775081f,    82.4742907515114f},
  { -172.618562111102f,    1.46559142196641f},
  { 168.689017629111f,    -82.4717071901232f},
  { 172.618489604721f,    -1.46441483340862f},
  { 11.2157017776617f,    82.4763593030642f},
  { 7.38236788128392f,    1.46620895892741f},
  { -11.2117036062431f,    -82.4789181367532f},
  { -91.4785749918475f,    7.37448337501027f},
  { 88.5224297767631f,    7.38438652912129f}};

/* Minimum T-design of degree 17 (156 points) */
const float __Tdesign_degree_17_dirs_deg[156][2] =
{ { -110.972536098452f,    -81.3395149878408f},
  { -98.0945259291915f,    -3.08934606926242f},
  { 69.0283101966918f,    -81.3395963121933f},
  { -176.879500974752f,    -8.08271861041067f},
  { -81.9053305069526f,    3.08941705867335f},
  { 176.879644538453f,    8.08278710024119f},
  { 81.9054851307679f,    -3.08940328913739f},
  { 3.12051971992014f,    -8.08274990022289f},
  { 98.0946906822435f,    3.08939807146030f},
  { -3.12033418366235f,    8.08275384646156f},
  { 110.971757607529f,    81.3396073060199f},
  { -69.0272255211550f,    81.3394832453476f},
  { 145.758325050956f,    30.5199949962486f},
  { 46.3338654430728f,    -45.4090808372840f},
  { -34.2417244572974f,    30.5199299435608f},
  { -54.5065763702847f,    28.9944543220334f},
  { 133.666175283428f,    45.4090645714201f},
  { 54.5065169257390f,    -28.9946358709077f},
  { -133.666075964290f,    -45.4090639295985f},
  { 125.493507490318f,    28.9945948799985f},
  { -46.3338672664529f,    45.4089150225299f},
  { -125.493394782705f,    -28.9945644971856f},
  { -145.758189836803f,    -30.5199956551973f},
  { 34.2417015167707f,    -30.5199717945651f},
  { 159.575321832626f,    41.3951427412058f},
  { 68.4011541800515f,    -44.6686954718411f},
  { -20.4246846261942f,    41.3950488549860f},
  { -46.7530721727418f,    15.1762870269464f},
  { 111.598907199965f,    44.6686624702884f},
  { 46.7532214376113f,    -15.1763613881014f},
  { -111.598821312324f,    -44.6685911089863f},
  { 133.246909041379f,    15.1763548809948f},
  { -68.4011026670182f,    44.6685187574964f},
  { -133.246756962063f,    -15.1763306571922f},
  { -159.575235433158f,    -41.3951042577738f},
  { 20.4247512806759f,    -41.3951386279467f},
  { 85.4331069665815f,    -37.9336689610989f},
  { -38.0219422832657f,    3.60044246504075f},
  { -94.5668221766801f,    -37.9336369935624f},
  { 174.167181422254f,    51.8336299061990f},
  { -141.977923622517f,    -3.60052335289898f},
  { -174.167161198305f,    -51.8335093466070f},
  { 141.978017527634f,    3.60052739735483f},
  { -5.83282102213420f,    51.8335060526221f},
  { 38.0221451086255f,    -3.60052175089752f},
  { 5.83286258546759f,    -51.8336195514750f},
  { -85.4331017546707f,    37.9336600665386f},
  { 94.5669635429985f,    37.9336717569877f},
  { 21.1823835996204f,    27.1703161950420f},
  { 54.8552196599246f,    56.0519373127499f},
  { -158.817667169175f,    27.1703764189990f},
  { 61.1687858728062f,    18.7515034281209f},
  { 125.144826414049f,    -56.0519088097055f},
  { -61.1686424166986f,    -18.7515067531668f},
  { -125.144783207922f,    56.0520170026195f},
  { -118.831333169078f,    18.7515075100248f},
  { -54.8552325260821f,    -56.0520018474777f},
  { 118.831357033983f,    -18.7514876111844f},
  { -21.1821737960870f,    -27.1703273487209f},
  { 158.817752395942f,    -27.1702501653088f},
  { 104.655973590479f,    -9.56052038782342f},
  { -9.87572204741232f,    -14.4479351672395f},
  { -75.3439962252486f,    -9.56049660768317f},
  { -123.650852748573f,    72.5589038610187f},
  { -170.124133559486f,    14.4479810651928f},
  { 123.651377037836f,    -72.5587964003922f},
  { 170.124261965706f,    -14.4478957329142f},
  { 56.3486516053099f,    72.5588326417427f},
  { 9.87590710801076f,    14.4479358622388f},
  { -56.3489612874038f,    -72.5588747160005f},
  { -104.655934361393f,    9.56053438641502f},
  { 75.3441507750264f,    9.56049031651235f},
  { 25.9352433504815f,    -16.8263454567228f},
  { -34.6629602764395f,    59.4049520144158f},
  { -154.064725964219f,    -16.8263938405563f},
  { 108.587307515400f,    24.7481093505154f},
  { -145.337042726548f,    -59.4050775637726f},
  { -108.587166936804f,    -24.7480813625633f},
  { 145.337024797510f,    59.4051213680066f},
  { -71.4127913520217f,    24.7480131711128f},
  { 34.6630103111907f,    -59.4051325021881f},
  { 71.4127455463276f,    -24.7481551074827f},
  { -25.9351333497229f,    16.8263470460926f},
  { 154.064872752799f,    16.8263949148262f},
  { -100.890653808628f,    26.4867629664574f},
  { 153.095389375993f,    -9.73572407837462f},
  { 79.1094689486786f,    26.4867038147623f},
  { -20.7650174418830f,    -61.5121429737782f},
  { 26.9048023491372f,    9.73577822208466f},
  { 20.7650094793833f,    61.5120919702165f},
  { -26.9045763350532f,    -9.73578359740380f},
  { 159.234989909912f,    -61.5120095244715f},
  { -153.095320693647f,    9.73577672831727f},
  { -159.234918273987f,    61.5121876738641f},
  { 100.890617375289f,    -26.4866867526547f},
  { -79.1093472045108f,    -26.4866920777842f},
  { 44.3136042236380f,    12.2784148864384f},
  { 17.3042148418974f,    44.3597818131229f},
  { -135.686522732100f,    12.2784100472036f},
  { 73.0820593381593f,    43.0474367769055f},
  { 162.695847122876f,    -44.3597017539863f},
  { -73.0819188464220f,    -43.0474262674942f},
  { -162.695765371418f,    44.3598645801098f},
  { -106.917995967658f,    43.0475197742591f},
  { -17.3041690092237f,    -44.3598140007833f},
  { 106.918015628643f,    -43.0474025708186f},
  { -44.3133852955836f,    -12.2785203404408f},
  { 135.686584721680f,    -12.2784215377938f},
  { -169.079669371546f,    -24.5321909536177f},
  { -112.542166671839f,    -63.2850601072096f},
  { 10.9202953872870f,    -24.5321908793467f},
  { -114.929999529610f,    -9.92395631971344f},
  { -67.4576447966383f,    63.2849658247549f},
  { 114.930166180530f,    9.92397510754817f},
  { 67.4579214268390f,    -63.2851359676343f},
  { 65.0699795536169f,    -9.92401372287082f},
  { 112.542146133756f,    63.2851183198918f},
  { -65.0698516207557f,    9.92390739777342f},
  { 169.079802219111f,    24.5322412571671f},
  { -10.9202113755098f,    24.5321946491817f},
  { 93.2040875742355f,    -57.3930220085365f},
  { -57.4335812294807f,    -1.72597809895388f},
  { -86.7960241525694f,    -57.3929860941125f},
  { -177.952273324117f,    32.5497631173123f},
  { -122.566253987475f,    1.72593957294755f},
  { 177.952374311331f,    -32.5496484135080f},
  { 122.566350583694f,    -1.72594309051837f},
  { 2.04775449117076f,    32.5496635362235f},
  { 57.4338233352979f,    1.72591921625950f},
  { -2.04759683730983f,    -32.5497546915263f},
  { -93.2038717181709f,    57.3930478036049f},
  { 86.7959914092532f,    57.3930448748746f},
  { -17.5897066057983f,    3.03732751711178f},
  { 170.041283651154f,    72.1581357589733f},
  { 162.410264047604f,    3.03735885797477f},
  { 86.8139300357902f,    -17.5643027243221f},
  { 9.95871784192333f,    -72.1581260767696f},
  { -86.8139808546443f,    17.5643089786731f},
  { -9.95884326138181f,    72.1579520781074f},
  { -93.1860064822312f,    -17.5642466145111f},
  { -170.041415817683f,    -72.1579882194655f},
  { 93.1861408789812f,    17.5642898521383f},
  { 17.5899070081613f,    -3.03728636624082f},
  { -162.410130590415f,    -3.03733315883086f},
  { 39.3792257392670f,    44.2573534137099f},
  { 56.9313776923427f,    33.6148193811549f},
  { -140.620759157715f,    44.2573759844125f},
  { 38.4239055017828f,    27.0264132991360f},
  { 123.068707795201f,    -33.6147936065186f},
  { -38.4237030347613f,    -27.0264785477125f},
  { -123.068677102487f,    33.6148394522083f},
  { -141.576163554034f,    27.0264135594767f},
  { -56.9312444104479f,    -33.6148138613919f},
  { 141.576226557237f,    -27.0264010423640f},
  { -39.3791791176139f,    -44.2574050937807f},
  { 140.620835015981f,    -44.2573180792070f}};

/* Minimum T-design of degree 18 (180 points) */
const float __Tdesign_degree_18_dirs_deg[180][2] =
{ { 165.522525351678f,    26.5185969118445f},
  { 63.3884960659874f,    -60.0402391549481f},
  { -14.4774747587378f,    26.5185969244391f},
  { -62.7356215863733f,    12.9262574799323f},
  { 116.611503753114f,    60.0402392311778f},
  { 62.7356216534755f,    -12.9262573768089f},
  { -116.611503960788f,    -60.0402392777927f},
  { 117.264378479103f,    12.9262574013180f},
  { -63.3884961827724f,    60.0402393764858f},
  { -117.264378506232f,    -12.9262575163645f},
  { -165.522525403520f,    -26.5185967690928f},
  { 14.4774746957471f,    -26.5185968169478f},
  { -150.217450616133f,    -21.6221948527669f},
  { -141.410000251229f,    -53.7887936217025f},
  { 29.7825495902173f,    -21.6221950203635f},
  { -114.546102790075f,    -27.5006115805657f},
  { -38.5899997172093f,    53.7887938971789f},
  { 114.546102736916f,    27.5006114363632f},
  { 38.5899997865273f,    -53.7887936503052f},
  { 65.4538974225092f,    -27.5006114292486f},
  { 141.410000115890f,    53.7887937976550f},
  { -65.4538971677670f,    27.5006116108567f},
  { 150.217450576809f,    21.6221950037115f},
  { -29.7825495511979f,    21.6221950095317f},
  { -163.468939687510f,    81.9141711880659f},
  { 92.3148380088438f,    -7.74950785743483f},
  { 16.5310604080828f,    81.9141711765953f},
  { -7.75575993532564f,    -2.29368526450881f},
  { 87.6851620086179f,    7.74950781982179f},
  { 7.75575978589441f,    2.29368524973419f},
  { -87.6851621029077f,    -7.74950792835357f},
  { 172.244240168478f,    -2.29368534133714f},
  { -92.3148378847329f,    7.74950778602442f},
  { -172.244240142217f,    2.29368535781371f},
  { 163.468940319517f,    -81.9141712591068f},
  { -16.5310594957412f,    -81.9141711307485f},
  { -79.9069240872412f,    -73.4928298922544f},
  { -106.265313752353f,    2.85417063577255f},
  { 100.093075450962f,    -73.4928298832334f},
  { 177.027037325915f,    -16.2445778332985f},
  { -73.7346862724408f,    -2.85417076248328f},
  { -177.027037359582f,    16.2445778708826f},
  { 73.7346861330546f,    2.85417077799392f},
  { -2.97296266678957f,    -16.2445777170525f},
  { 106.265313900319f,    -2.85417075916001f},
  { 2.97296252214977f,    16.2445777463947f},
  { 79.9069242475240f,    73.4928297213284f},
  { -100.093075924320f,    73.4928297504114f},
  { -43.1928640166220f,    73.6304207741543f},
  { 101.367856910908f,    11.8570587394614f},
  { 136.807135671159f,    73.6304207358966f},
  { 12.0874142750063f,    -11.1221753611675f},
  { 78.6321431755501f,    -11.8570587892693f},
  { -12.0874143568441f,    11.1221753851378f},
  { -78.6321430525525f,    11.8570588882244f},
  { -167.912585805617f,    -11.1221752564724f},
  { -101.367856964503f,    -11.8570589676836f},
  { 167.912585849640f,    11.1221753505886f},
  { 43.1928639454143f,    -73.6304205435030f},
  { -136.807135980905f,    -73.6304206111304f},
  { 109.863477411331f,    -34.8309444444304f},
  { -36.4953684573441f,    -16.1948859619229f},
  { -70.1365225856530f,    -34.8309444309736f},
  { -153.972978190969f,    50.5342660057627f},
  { -143.504631539358f,    16.1948859540727f},
  { 153.972978187359f,    -50.5342660958429f},
  { 143.504631621503f,    -16.1948859557381f},
  { 26.0270217639140f,    50.5342659511289f},
  { 36.4953682958062f,    16.1948859321165f},
  { -26.0270217764730f,    -50.5342659584325f},
  { -109.863477345633f,    34.8309442863438f},
  { 70.1365226011247f,    34.8309443621136f},
  { -23.3144176246880f,    -6.54377493040509f},
  { -163.836601816190f,    65.8340912397072f},
  { 156.685582511874f,    -6.54377498490282f},
  { 97.1198721371224f,    -23.1536444499380f},
  { -16.1633979077694f,    -65.8340911421026f},
  { -97.1198719381115f,    23.1536443807155f},
  { 16.1633982812712f,    65.8340911823371f},
  { -82.8801280056832f,    -23.1536445259357f},
  { 163.836602088907f,    -65.8340912805086f},
  { 82.8801279292869f,    23.1536443812616f},
  { 23.3144174717292f,    6.54377488581314f},
  { -156.685582444247f,    6.54377505059661f},
  { -0.871525155053854f,    -31.9228878345329f},
  { -91.3985860869862f,    58.0664743343015f},
  { 179.128474793439f,    -31.9228879881299f},
  { 121.925863139225f,    -0.739708138349097f},
  { -88.6014139907392f,    -58.0664745214493f},
  { -121.925862927300f,    0.739708140763914f},
  { 88.6014140770374f,    58.0664742415090f},
  { -58.0741371324967f,    -0.739708171109085f},
  { 91.3985859230815f,    -58.0664744459172f},
  { 58.0741369468673f,    0.739708221512544f},
  { 0.871525106485463f,    31.9228879048823f},
  { -179.128474897751f,    31.9228880865407f},
  { 163.118919203476f,    43.3490042807202f},
  { 72.9009079952637f,    -44.0950803474695f},
  { -16.8810807237829f,    43.3490042469511f},
  { -45.3900610991516f,    12.1906201180173f},
  { 107.099092058332f,    44.0950802946323f},
  { 45.3900611170782f,    -12.1906201460423f},
  { -107.099092029623f,    -44.0950804951930f},
  { 134.609939068526f,    12.1906201756408f},
  { -72.9009079168893f,    44.0950804987426f},
  { -134.609939017185f,    -12.1906201217856f},
  { -163.118919307274f,    -43.3490040821782f},
  { 16.8810806660299f,    -43.3490041124859f},
  { -114.226485782122f,    50.3719680420707f},
  { 127.058874767029f,    -15.1720365932007f},
  { 65.7735143230453f,    50.3719681032687f},
  { -18.7680122129425f,    -35.5652767976860f},
  { 52.9411252032868f,    15.1720366801078f},
  { 18.7680121845148f,    35.5652767553954f},
  { -52.9411253805515f,    -15.1720366270455f},
  { 161.231987695175f,    -35.5652769283117f},
  { -127.058874580761f,    15.1720366187415f},
  { -161.231987809323f,    35.5652769144754f},
  { 114.226485786593f,    -50.3719682654292f},
  { -65.7735141972913f,    -50.3719682192744f},
  { 54.1696638774737f,    30.1608310237706f},
  { 35.6306478157225f,    30.4068206160989f},
  { -125.830335996810f,    30.1608309118647f},
  { 45.2106162344126f,    44.5067175274920f},
  { 144.369352021074f,    -30.4068206860435f},
  { -45.2106162718128f,    -44.5067174987149f},
  { -144.369352124760f,    30.4068205977378f},
  { -134.789383740334f,    44.5067174067404f},
  { -35.6306478988259f,    -30.4068206396040f},
  { 134.789383697533f,    -44.5067175741724f},
  { -54.1696639660126f,    -30.1608309659934f},
  { 125.830336002098f,    -30.1608309772511f},
  { 126.198365586012f,    41.7296556110488f},
  { 47.8615829548800f,    -26.1516257329903f},
  { -53.8016343365832f,    41.7296557416691f},
  { -33.5111352365998f,    37.0306401796118f},
  { 132.138417183687f,    26.1516257455572f},
  { 33.5111352223607f,    -37.0306400834847f},
  { -132.138417282523f,    -26.1516257480742f},
  { 146.488864794235f,    37.0306401121596f},
  { -47.8615827989520f,    26.1516257755240f},
  { -146.488864880324f,    -37.0306399818889f},
  { -126.198365627788f,    -41.7296556060802f},
  { 53.8016344598902f,    -41.7296555261562f},
  { -161.752580861636f,    20.3811731565148f},
  { 130.124344291216f,    -62.9056646127018f},
  { 18.2474190537101f,    20.3811729886130f},
  { -68.6348318327801f,    -17.0687243103703f},
  { 49.8756558579031f,    62.9056644214068f},
  { 68.6348316582127f,    17.0687243084354f},
  { -49.8756555688743f,    -62.9056644946146f},
  { 111.365168318519f,    -17.0687242634348f},
  { -130.124344209978f,    62.9056643590092f},
  { -111.365168110473f,    17.0687242002251f},
  { 161.752580799423f,    -20.3811731737686f},
  { -18.2474191916171f,    -20.3811730380741f},
  { 2.71484106575056f,    48.4926781319369f},
  { 87.5997969787878f,    41.4504384176454f},
  { -177.285159050149f,    48.4926783149728f},
  { 41.4753914417279f,    1.79879023185710f},
  { 92.4002031412684f,    -41.4504386189539f},
  { -41.4753915917106f,    -1.79879025270390f},
  { -92.4002030805454f,    41.4504385125983f},
  { -138.524608439003f,    1.79879027718797f},
  { -87.5997968745335f,    -41.4504386683033f},
  { 138.524608649476f,    -1.79879018100771f},
  { -2.71484103403969f,    -48.4926780911917f},
  { 177.285158982351f,    -48.4926782083146f},
  { -98.1523160065715f,    -27.5438574820543f},
  { -152.216845561549f,    -7.22306125879088f},
  { 81.8476841627651f,    -27.5438573643616f},
  { -164.789178447502f,    -61.3658005156890f},
  { -27.7831545331737f,    7.22306140132871f},
  { 164.789178443534f,    61.3658007136397f},
  { 27.7831544823965f,    -7.22306144684801f},
  { 15.2108214755339f,    -61.3658004925845f},
  { 152.216845683231f,    7.22306141603186f},
  { -15.2108212260461f,    61.3658006465818f},
  { 98.1523159907921f,    27.5438572420303f},
  { -81.8476839310339f,    27.5438574201819f}};

/* Minimum T-design of degree 19 (204 points) */
const float __Tdesign_degree_19_dirs_deg[204][2] =
{ { -40.4827971398993f,    43.3558448910510f},
  { 124.512074059868f,    33.5755419701843f},
  { 139.517202845142f,    43.3558448775343f},
  { 38.8533744745300f,    -28.1674635453882f},
  { 55.4879258399038f,    -33.5755419872292f},
  { -38.8533744890089f,    28.1674635757744f},
  { -55.4879258707753f,    33.5755419610037f},
  { -141.146625425535f,    -28.1674634939008f},
  { -124.512074072873f,    -33.5755419765917f},
  { 141.146625395667f,    28.1674635597046f},
  { 40.4827970519826f,    -43.3558448545089f},
  { -139.517202930702f,    -43.3558447765375f},
  { 56.0057011667352f,    17.1841974993980f},
  { 20.4553778525436f,    32.2863965179037f},
  { -123.994298855438f,    17.1841975427728f},
  { 61.0525838868499f,    52.3805429894876f},
  { 159.544622091585f,    -32.2863964721300f},
  { -61.0525838977133f,    -52.3805429922499f},
  { -159.544622184482f,    32.2863964966156f},
  { -118.947416102450f,    52.3805429726830f},
  { -20.4553779060528f,    -32.2863965275596f},
  { 118.947416101692f,    -52.3805430422566f},
  { -56.0057011638433f,    -17.1841974992667f},
  { 123.994298849785f,    -17.1841975659790f},
  { -179.514728897842f,    -8.95189558912681f},
  { -93.0776196403050f,    -81.0350681242251f},
  { 0.485271044590434f,    -8.95189557283304f},
  { -98.9522115463824f,    -0.479359901942935f},
  { -86.9223806320116f,    81.0350680967773f},
  { 98.9522115996105f,    0.479359924611257f},
  { 86.9223801723002f,    -81.0350680963880f},
  { 81.0477884925503f,    -0.479359910390806f},
  { 93.0776194117918f,    81.0350681591717f},
  { -81.0477884388582f,    0.479359910640051f},
  { 179.514728907578f,    8.95189554407248f},
  { -0.485271033430437f,    8.95189552791142f},
  { 12.0366745311780f,    -13.5567796222345f},
  { -49.1452517185299f,    71.9460236947636f},
  { -167.963325421518f,    -13.5567796807998f},
  { 103.849922026142f,    11.6965010678641f},
  { -130.854748248662f,    -71.9460236355101f},
  { -103.849922019208f,    -11.6965010499568f},
  { 130.854748306321f,    71.9460237069809f},
  { -76.1500779540575f,    11.6965010382476f},
  { 49.1452517246089f,    -71.9460236210567f},
  { 76.1500779677235f,    -11.6965010395382f},
  { -12.0366745072388f,    13.5567796452543f},
  { 167.963325445597f,    13.5567797059840f},
  { -13.6177687853505f,    -58.2047889412688f},
  { -98.3039108388766f,    30.8020093497657f},
  { 166.382231262410f,    -58.2047889609688f},
  { 148.931723642678f,    -7.12599043766624f},
  { -81.6960891547253f,    -30.8020093817807f},
  { -148.931723587907f,    7.12599046490582f},
  { 81.6960891391292f,    30.8020094073706f},
  { -31.0682763825774f,    -7.12599044471178f},
  { 98.3039108451470f,    -30.8020093261083f},
  { 31.0682764509820f,    7.12599044985374f},
  { 13.6177687363442f,    58.2047889110994f},
  { -166.382231242976f,    58.2047889260604f},
  { 65.2592668684524f,    -20.5490633099000f},
  { -22.4282240836465f,    23.0717596473609f},
  { -114.740733101132f,    -20.5490633088132f},
  { 131.850731121270f,    58.2577280834148f},
  { -157.571775811254f,    -23.0717596621340f},
  { -131.850731298332f,    -58.2577279909000f},
  { 157.571775790124f,    23.0717596613445f},
  { -48.1492689459914f,    58.2577280717716f},
  { 22.4282240971360f,    -23.0717596293833f},
  { 48.1492688583973f,    -58.2577279963523f},
  { -65.2592668455931f,    20.5490632830229f},
  { 114.740733145086f,    20.5490632899652f},
  { -135.391308878658f,    26.4960083481501f},
  { 144.631254301060f,    -39.5791364301515f},
  { 44.6086911773665f,    26.4960083352508f},
  { -54.9998265959725f,    -38.9394958301661f},
  { 35.3687456809888f,    39.5791363676049f},
  { 54.9998265626444f,    38.9394958381227f},
  { -35.3687457312500f,    -39.5791363951356f},
  { 125.000173427157f,    -38.9394958686094f},
  { -144.631254355751f,    39.5791363679164f},
  { -125.000173460926f,    38.9394958286948f},
  { 135.391308874058f,    -26.4960083937547f},
  { -44.6086911769571f,    -26.4960083422148f},
  { 114.949924840136f,    -4.74760500909852f},
  { -5.23368224859043f,    -24.8584988627303f},
  { -65.0500751920101f,    -4.74760499288687f},
  { -101.138255685242f,    64.6308294162604f},
  { -174.766317811030f,    24.8584988453073f},
  { 101.138255567901f,    -64.6308294291222f},
  { 174.766317773328f,    -24.8584988721886f},
  { 78.8617442781326f,    64.6308294673680f},
  { 5.23368223316436f,    24.8584988162044f},
  { -78.8617443485117f,    -64.6308294202555f},
  { -114.949924797503f,    4.74760502019486f},
  { 65.0500752381225f,    4.74760499991936f},
  { 35.8486173395199f,    52.6418633116691f},
  { 65.9114275005164f,    29.4621373645687f},
  { -144.151382639220f,    52.6418633114291f},
  { 31.7487031863914f,    20.8159390557746f},
  { 114.088572496823f,    -29.4621373762114f},
  { -31.7487031850602f,    -20.8159390525807f},
  { -114.088572500623f,    29.4621373751269f},
  { -148.251296867652f,    20.8159390403083f},
  { -65.9114275150986f,    -29.4621373406821f},
  { 148.251296825615f,    -20.8159390193568f},
  { -35.8486173588977f,    -52.6418633449108f},
  { 144.151382653085f,    -52.6418633878030f},
  { 86.4503159839449f,    11.5178574130264f},
  { 11.5394039344984f,    3.47811299112874f},
  { -93.5496840137305f,    11.5178573996117f},
  { 16.9004866502571f,    77.9546248709343f},
  { 168.460596138258f,    -3.47811298337902f},
  { -16.9004867221619f,    -77.9546248991060f},
  { -168.460596111272f,    3.47811298260875f},
  { -163.099513269263f,    77.9546248607774f},
  { -11.5394039093281f,    -3.47811299154152f},
  { 163.099513332859f,    -77.9546249203649f},
  { -86.4503159901669f,    -11.5178574116444f},
  { 93.5496840314420f,    -11.5178573868150f},
  { 135.239557049900f,    4.01664853175916f},
  { 5.69483356922942f,    -45.0978192046165f},
  { -44.7604429511609f,    4.01664859407937f},
  { -84.3522817714199f,    44.6210488255627f},
  { 174.305166377085f,    45.0978191925169f},
  { 84.3522818209917f,    -44.6210488088963f},
  { -174.305166498240f,    -45.0978191964870f},
  { 95.6477182624038f,    44.6210488702794f},
  { -5.69483363835931f,    45.0978192003566f},
  { -95.6477182517814f,    -44.6210488918365f},
  { -135.239557005999f,    -4.01664858322133f},
  { 44.7604430095175f,    -4.01664863014437f},
  { -129.844345684641f,    -18.1635402775425f},
  { -156.862821803731f,    -37.5012149238997f},
  { 50.1556542868047f,    -18.1635403296931f},
  { -117.115121676957f,    -46.8469614266862f},
  { -23.1371782327970f,    37.5012149248207f},
  { 117.115121614901f,    46.8469614192763f},
  { 23.1371781765387f,    -37.5012149223183f},
  { 62.8848783969846f,    -46.8469613590856f},
  { 156.862821718985f,    37.5012149202724f},
  { -62.8848784049570f,    46.8469613748204f},
  { 129.844345741672f,    18.1635402578975f},
  { -50.1556542410030f,    18.1635403103006f},
  { -74.0995232303200f,    32.8713230086832f},
  { 146.101745919219f,    13.3031224588686f},
  { 105.900476735914f,    32.8713230206360f},
  { 22.9747088726656f,    -53.8773163872051f},
  { 33.8982540672711f,    -13.3031224827402f},
  { -22.9747089674201f,    53.8773163832442f},
  { -33.8982540495291f,    13.3031225041922f},
  { -157.025291161850f,    -53.8773163302415f},
  { -146.101745928058f,    -13.3031224379725f},
  { 157.025291066549f,    53.8773163620209f},
  { 74.0995232337151f,    -32.8713230260657f},
  { -105.900476736100f,    -32.8713230683644f},
  { -119.920654767573f,    -5.64202479284406f},
  { -173.497291648383f,    -29.7610425696254f},
  { 60.0793452237798f,    -5.64202480225216f},
  { -101.202909567639f,    -59.6005156643046f},
  { -6.50270830937896f,    29.7610425765434f},
  { 101.202909449688f,    59.6005156740098f},
  { 6.50270832065502f,    -29.7610425545166f},
  { 78.7970904807001f,    -59.6005155822608f},
  { 173.497291620421f,    29.7610425883022f},
  { -78.7970905474922f,    59.6005156241448f},
  { 119.920654807640f,    5.64202480200863f},
  { -60.0793451699613f,    5.64202480527789f},
  { 73.1351626703152f,    16.1311872967927f},
  { 16.8160239465430f,    16.1821871118731f},
  { -106.864837344194f,    16.1311872926822f},
  { 45.0880197945698f,    66.8258807656198f},
  { 163.183976128393f,    -16.1821870964153f},
  { -45.0880199162156f,    -66.8258808054124f},
  { -163.183976133830f,    16.1821870986028f},
  { -134.911980145466f,    66.8258807597722f},
  { -16.8160239202286f,    -16.1821871244424f},
  { 134.911980086750f,    -66.8258808490522f},
  { -73.1351626743642f,    -16.1311872709203f},
  { 106.864837351405f,    -16.1311872661606f},
  { -11.6972412007216f,    -43.3797087092533f},
  { -102.108896696603f,    45.3753435662694f},
  { 168.302758743556f,    -43.3797086959446f},
  { 133.980267409617f,    -8.47368576575139f},
  { -77.8911033135351f,    -45.3753435838886f},
  { -133.980267393688f,    8.47368572892681f},
  { 77.8911033110979f,    45.3753436128045f},
  { -46.0197325747014f,    -8.47368572030878f},
  { 102.108896727441f,    -45.3753435680196f},
  { 46.0197326174467f,    8.47368569981079f},
  { 11.6972411587316f,    43.3797087026179f},
  { -168.302758831810f,    43.3797087184118f},
  { -24.1050869937083f,    3.72760403870489f},
  { 170.936389463879f,    65.6254155023735f},
  { 155.894913041978f,    3.72760403395435f},
  { 85.9174400505778f,    -24.0508636473787f},
  { 9.06361051045597f,    -65.6254155365736f},
  { -85.9174400449213f,    24.0508636535097f},
  { -9.06361054264494f,    65.6254155057437f},
  { -94.0825599392362f,    -24.0508636829675f},
  { -170.936389442212f,    -65.6254155366095f},
  { 94.0825599175670f,    24.0508636802220f},
  { 24.1050870259372f,    -3.72760401034628f},
  { -155.894913008243f,    -3.72760399794214f}};

/* Minimum T-design of degree 20 (216 points) */
const float __Tdesign_degree_20_dirs_deg[216][2] =
{ { 104.601925128729f,    -3.68269227044959f},
  { -3.80525550883653f,    -14.5711050801146f},
  { -75.3980748712553f,    -3.68269227045763f},
  { -104.322150154107f,    74.9506325254330f},
  { -176.194744491161f,    14.5711050801105f},
  { 104.322150154173f,    -74.9506325254441f},
  { 176.194744491153f,    -14.5711050800763f},
  { 75.6778498458485f,    74.9506325254389f},
  { 3.80525550882118f,    14.5711050800919f},
  { -75.6778498458492f,    -74.9506325254403f},
  { -104.601925128768f,    3.68269227044241f},
  { 75.3980748712369f,    3.68269227047015f},
  { 153.768474770232f,    -30.3270175342204f},
  { -52.9262789947920f,    -50.7384907165667f},
  { -26.2315252297731f,    -30.3270175342403f},
  { -123.110247235077f,    22.4275681615901f},
  { -127.073721005197f,    50.7384907165599f},
  { 123.110247235061f,    -22.4275681615786f},
  { 127.073721005201f,    -50.7384907165500f},
  { 56.8897527649198f,    22.4275681615975f},
  { 52.9262789947688f,    50.7384907165558f},
  { -56.8897527649381f,    -22.4275681615922f},
  { -153.768474770235f,    30.3270175342297f},
  { 26.2315252297398f,    30.3270175342289f},
  { 35.9931099488083f,    -39.7686413526156f},
  { -54.7721532822064f,    38.4544369321159f},
  { -144.006890051185f,    -39.7686413526327f},
  { 135.808213283634f,    26.8538854974352f},
  { -125.227846717788f,    -38.4544369321345f},
  { -135.808213283626f,    -26.8538854974234f},
  { 125.227846717813f,    38.4544369321259f},
  { -44.1917867163936f,    26.8538854974017f},
  { 54.7721532821843f,    -38.4544369320921f},
  { 44.1917867163770f,    -26.8538854973852f},
  { -35.9931099488081f,    39.7686413526122f},
  { 144.006890051227f,    39.7686413526415f},
  { 71.8181243527431f,    -25.4259475349729f},
  { -26.5821567459587f,    16.3683231733698f},
  { -108.181875647256f,    -25.4259475349581f},
  { 146.720082499332f,    59.0980968893182f},
  { -153.417843254058f,    -16.3683231734038f},
  { -146.720082499387f,    -59.0980968892868f},
  { 153.417843254019f,    16.3683231733929f},
  { -33.2799175006570f,    59.0980968893018f},
  { 26.5821567459460f,    -16.3683231733863f},
  { 33.2799175007043f,    -59.0980968892965f},
  { -71.8181243527681f,    25.4259475349455f},
  { 108.181875647262f,    25.4259475349649f},
  { -136.553309024926f,    26.5218051743481f},
  { 144.031174024577f,    -40.5123550309482f},
  { 43.4466909750663f,    26.5218051743684f},
  { -55.4956627592942f,    -37.9745276892037f},
  { 35.9688259753990f,    40.5123550309626f},
  { 55.4956627592931f,    37.9745276892040f},
  { -35.9688259754170f,    -40.5123550309794f},
  { 124.504337240699f,    -37.9745276891974f},
  { -144.031174024568f,    40.5123550309630f},
  { -124.504337240703f,    37.9745276891987f},
  { 136.553309024903f,    -26.5218051743490f},
  { -43.4466909750886f,    -26.5218051743638f},
  { -6.51647188097905f,    -1.07894020321228f},
  { -170.577727686183f,    83.3951916100318f},
  { 173.483528119024f,    -1.07894020316580f},
  { 91.0859545987182f,    -6.51531150975361f},
  { -9.42227231365960f,    -83.3951916100331f},
  { -91.0859545987667f,    6.51531150975609f},
  { 9.42227231372673f,    83.3951916100228f},
  { -88.9140454012564f,    -6.51531150977155f},
  { 170.577727686387f,    -83.3951916100274f},
  { 88.9140454012482f,    6.51531150977345f},
  { 6.51647188096395f,    1.07894020318976f},
  { -173.483528119033f,    1.07894020319893f},
  { -71.1275142926455f,    40.3810353657942f},
  { 138.050297322915f,    14.2645186119443f},
  { 108.872485707367f,    40.3810353658053f},
  { 20.8229498967013f,    -46.1206992229688f},
  { 41.9497026770528f,    -14.2645186119106f},
  { -20.8229498966960f,    46.1206992229519f},
  { -41.9497026770679f,    14.2645186119222f},
  { -159.177050103330f,    -46.1206992229406f},
  { -138.050297322956f,    -14.2645186119422f},
  { 159.177050103338f,    46.1206992229801f},
  { 71.1275142926223f,    -40.3810353658125f},
  { -108.872485707350f,    -40.3810353658089f},
  { -153.965515116410f,    -27.9742815167833f},
  { -129.569241451541f,    -52.5179675742976f},
  { 26.0344848836063f,    -27.9742815167699f},
  { -120.587933155882f,    -22.8070349941593f},
  { -50.4307585484472f,    52.5179675742717f},
  { 120.587933155880f,    22.8070349941496f},
  { 50.4307585484843f,    -52.5179675742662f},
  { 59.4120668441292f,    -22.8070349941230f},
  { 129.569241451543f,    52.5179675742836f},
  { -59.4120668441431f,    22.8070349941496f},
  { 153.965515116415f,    27.9742815167763f},
  { -26.0344848836226f,    27.9742815167530f},
  { -80.2345666344704f,    13.4760126044716f},
  { 166.333234244309f,    9.49404277183603f},
  { 99.7654333655448f,    13.4760126044987f},
  { 35.2909555217966f,    -73.4109005990371f},
  { 13.6667657556541f,    -9.49404277183004f},
  { -35.2909555217526f,    73.4109005990409f},
  { -13.6667657556718f,    9.49404277180524f},
  { -144.709044478302f,    -73.4109005990245f},
  { -166.333234244339f,    -9.49404277182330f},
  { 144.709044478216f,    73.4109005990569f},
  { 80.2345666344400f,    -13.4760126044968f},
  { -99.7654333655573f,    -13.4760126044823f},
  { 55.4258411798564f,    -9.00107370832261f},
  { -10.8894834060658f,    34.0892935061751f},
  { -124.574158820165f,    -9.00107370834749f},
  { 105.596487285914f,    54.4149524298600f},
  { -169.110516593985f,    -34.0892935061760f},
  { -105.596487285927f,    -54.4149524298770f},
  { 169.110516593966f,    34.0892935061904f},
  { -74.4035127140624f,    54.4149524298548f},
  { 10.8894834060485f,    -34.0892935062018f},
  { 74.4035127140993f,    -54.4149524298663f},
  { -55.4258411798645f,    9.00107370833931f},
  { 124.574158820128f,    9.00107370833328f},
  { 68.3153335856534f,    -7.14339480007468f},
  { -7.68115300267237f,    21.5079337786373f},
  { -111.684666414376f,    -7.14339480009472f},
  { 108.735829009062f,    67.2230290834693f},
  { -172.318846997362f,    -21.5079337786408f},
  { -108.735829009106f,    -67.2230290834936f},
  { 172.318846997345f,    21.5079337786427f},
  { -71.2641709909006f,    67.2230290834666f},
  { 7.68115300265523f,    -21.5079337786644f},
  { 71.2641709909482f,    -67.2230290834717f},
  { -68.3153335856545f,    7.14339480008368f},
  { 111.684666414342f,    7.14339480008680f},
  { -174.969151608304f,    36.1200320060754f},
  { 96.8522393339851f,    -53.5785859742491f},
  { 5.03084839167049f,    36.1200320060603f},
  { -53.7746004797745f,    -4.06202009701441f},
  { 83.1477606660237f,    53.5785859742662f},
  { 53.7746004797533f,    4.06202009702199f},
  { -83.1477606660565f,    -53.5785859742666f},
  { 126.225399520230f,    -4.06202009702366f},
  { -96.8522393339372f,    53.5785859742576f},
  { -126.225399520243f,    4.06202009701189f},
  { 174.969151608272f,    -36.1200320060616f},
  { -5.03084839169859f,    -36.1200320060754f},
  { -149.906407826552f,    16.2606931032710f},
  { 149.813145407035f,    -56.1601274346795f},
  { 30.0935921734429f,    16.2606931032798f},
  { -71.3701782091179f,    -28.7740165038951f},
  { 30.1868545929113f,    56.1601274346756f},
  { 71.3701782091094f,    28.7740165038888f},
  { -30.1868545929296f,    -56.1601274347022f},
  { 108.629821790888f,    -28.7740165038695f},
  { -149.813145407052f,    56.1601274346882f},
  { -108.629821790894f,    28.7740165038910f},
  { 149.906407826543f,    -16.2606931032760f},
  { -30.0935921734718f,    -16.2606931032870f},
  { 2.52384960625872f,    -51.4512271200113f},
  { -87.9904040024997f,    38.5045003504209f},
  { -177.476150393760f,    -51.4512271199885f},
  { 141.478324772432f,    1.57250294138216f},
  { -92.0095959974952f,    -38.5045003504335f},
  { -141.478324772458f,    -1.57250294137597f},
  { 92.0095959974981f,    38.5045003504403f},
  { -38.5216752275558f,    1.57250294136416f},
  { 87.9904040024888f,    -38.5045003504353f},
  { 38.5216752275388f,    -1.57250294135528f},
  { -2.52384960627135f,    51.4512271200002f},
  { 177.476150393775f,    51.4512271200177f},
  { 161.118570987041f,    -9.98113371764410f},
  { -28.5384349241665f,    -68.7281514711054f},
  { -18.8814290129620f,    -9.98113371768145f},
  { -100.536379000037f,    18.5851021633060f},
  { -151.461565075782f,    68.7281514710960f},
  { 100.536379000000f,    -18.5851021633009f},
  { 151.461565075802f,    -68.7281514710950f},
  { 79.4636209999824f,    18.5851021633233f},
  { 28.5384349241945f,    68.7281514710894f},
  { -79.4636209999861f,    -18.5851021633212f},
  { -161.118570987047f,    9.98113371765684f},
  { 18.8814290129454f,    9.98113371766005f},
  { 12.8381161367250f,    24.9726634007955f},
  { 64.4943057166319f,    62.1101091429154f},
  { -167.161883863252f,    24.9726634008075f},
  { 64.4676966669117f,    11.6202315990140f},
  { 115.505694283338f,    -62.1101091429170f},
  { -64.4676966669310f,    -11.6202315990073f},
  { -115.505694283325f,    62.1101091429139f},
  { -115.532303333088f,    11.6202315990016f},
  { -64.4943057166577f,    -62.1101091429204f},
  { 115.532303333060f,    -11.6202315989894f},
  { -12.8381161367529f,    -24.9726634008133f},
  { 167.161883863254f,    -24.9726634007839f},
  { 74.1348174889496f,    41.6486122381530f},
  { 42.7558348462444f,    11.7870638762109f},
  { -105.865182511039f,    41.6486122381531f},
  { 17.0865743937080f,    45.9530694773398f},
  { 137.244165153751f,    -11.7870638761928f},
  { -17.0865743937328f,    -45.9530694773589f},
  { -137.244165153753f,    11.7870638761900f},
  { -162.913425606246f,    45.9530694773475f},
  { -42.7558348462681f,    -11.7870638761989f},
  { 162.913425606243f,    -45.9530694773507f},
  { -74.1348174889504f,    -41.6486122381588f},
  { 105.865182511041f,    -41.6486122381280f},
  { 154.035881840976f,    1.26890773623963f},
  { 2.89632328150881f,    -64.0070432393520f},
  { -25.9641181590101f,    1.26890773621974f},
  { -88.5886963134671f,    25.9572763928128f},
  { 177.103676718535f,    64.0070432393631f},
  { 88.5886963134513f,    -25.9572763928294f},
  { -177.103676718576f,    -64.0070432393329f},
  { 91.4113036865553f,    25.9572763928366f},
  { -2.89632328146162f,    64.0070432393464f},
  { -91.4113036865597f,    -25.9572763928243f},
  { -154.035881841000f,    -1.26890773624989f},
  { 25.9641181589920f,    -1.26890773624060f}};

/* Minimum T-design of degree 21 (240 points) */
const float __Tdesign_degree_21_dirs_deg[240][2] =
{ { 24.8037137130410f,    -10.4640534970921f},
  { -23.7615514381126f,    63.2086065432396f},
  { -155.196286289587f,    -10.4640534884876f},
  { 101.500381898951f,    24.3641198026634f},
  { -156.238448590064f,    -63.2086065458284f},
  { -101.500381888658f,    -24.3641198041221f},
  { 156.238448559844f,    63.2086065503554f},
  { -78.4996181151217f,    24.3641197988060f},
  { 23.7615514173273f,    -63.2086065546655f},
  { 78.4996181039697f,    -24.3641197939936f},
  { -24.8037137007022f,    10.4640534915037f},
  { 155.196286295679f,    10.4640534868210f},
  { -134.641988338408f,    65.4373209186038f},
  { 108.014259287393f,    -16.9833470063578f},
  { 45.3580116803753f,    65.4373209229614f},
  { -17.8047381745055f,    -17.2035374034867f},
  { 71.9857407057664f,    16.9833470000679f},
  { 17.8047381601855f,    17.2035374099490f},
  { -71.9857407205158f,    -16.9833470025761f},
  { 162.195261811364f,    -17.2035373982681f},
  { -108.014259277257f,    16.9833470258064f},
  { -162.195261838500f,    17.2035374025352f},
  { 134.641988352629f,    -65.4373209298950f},
  { -45.3580116114767f,    -65.4373209177246f},
  { 177.598859321494f,    54.8491444701208f},
  { 88.3102490381675f,    -35.1154409081831f},
  { -2.40114069350551f,    54.8491444641872f},
  { -35.1271676217506f,    1.38214105769103f},
  { 91.6897509798567f,    35.1154409144503f},
  { 35.1271676308592f,    -1.38214107563309f},
  { -91.6897509639350f,    -35.1154409011172f},
  { 144.872832376883f,    1.38214105394520f},
  { -88.3102490433253f,    35.1154409067993f},
  { -144.872832382919f,    -1.38214106461202f},
  { -177.598859332362f,    -54.8491444783476f},
  { 2.40114065608711f,    -54.8491444798977f},
  { 157.925233039179f,    82.0086003047453f},
  { 86.9798751292437f,    -7.40217041390136f},
  { -22.0747667900841f,    82.0086003072532f},
  { -7.41235125348781f,    2.99493295875481f},
  { 93.0201248530804f,    7.40217041721125f},
  { 7.41235125061808f,    -2.99493293491206f},
  { -93.0201248532064f,    -7.40217041956111f},
  { 172.587648748194f,    2.99493295087326f},
  { -86.9798751388389f,    7.40217042031169f},
  { -172.587648746020f,    -2.99493293413946f},
  { -157.925233195472f,    -82.0086003095430f},
  { 22.0747668758532f,    -82.0086003013030f},
  { 42.4789657272739f,    -13.2111181753926f},
  { -19.1683264518727f,    45.8901283160366f},
  { -137.521034279365f,    -13.2111181758982f},
  { 107.656153639328f,    41.1054465172179f},
  { -160.831673556076f,    -45.8901283225774f},
  { -107.656153633174f,    -41.1054465117865f},
  { 160.831673547914f,    45.8901283096751f},
  { -72.3438463610700f,    41.1054465255977f},
  { 19.1683264546669f,    -45.8901283135983f},
  { 72.3438463541316f,    -41.1054465270478f},
  { -42.4789657380600f,    13.2111181856663f},
  { 137.521034261997f,    13.2111181824029f},
  { 28.4751425057663f,    10.8961540414938f},
  { 21.9865110033477f,    59.6751300986148f},
  { -151.524857500301f,    10.8961540321600f},
  { 77.6476311575040f,    27.9163389078453f},
  { 158.013489026279f,    -59.6751301088912f},
  { -77.6476311797186f,    -27.9163389070926f},
  { -158.013489002791f,    59.6751301020170f},
  { -102.352368827724f,    27.9163389107776f},
  { -21.9865109722689f,    -59.6751301009165f},
  { 102.352368834074f,    -27.9163389060307f},
  { -28.4751425050509f,    -10.8961540375005f},
  { 151.524857492048f,    -10.8961540334268f},
  { -33.7911316991742f,    21.6144180011898f},
  { 144.533534418273f,    50.5909094016008f},
  { 146.208868301887f,    21.6144180106661f},
  { 64.5101777390638f,    -31.1352111476882f},
  { 35.4664656025411f,    -50.5909093960748f},
  { -64.5101777345804f,    31.1352111445890f},
  { -35.4664655736399f,    50.5909094048493f},
  { -115.489822254523f,    -31.1352111461926f},
  { -144.533534408969f,    -50.5909094020511f},
  { 115.489822248740f,    31.1352111512252f},
  { 33.7911316959830f,    -21.6144179901815f},
  { -146.208868306746f,    -21.6144179961501f},
  { -175.526802493543f,    -31.7437477517178f},
  { -97.1851235504065f,    -57.9752633407046f},
  { 4.47319750122846f,    -31.7437477558340f},
  { -121.822006774074f,    -3.80297880236091f},
  { -82.8148764588498f,    57.9752633312572f},
  { 121.822006781217f,    3.80297879452026f},
  { 82.8148764473182f,    -57.9752633346200f},
  { 58.1779932190027f,    -3.80297880510012f},
  { 97.1851235458442f,    57.9752633377470f},
  { -58.1779932152383f,    3.80297880602732f},
  { 175.526802490312f,    31.7437477614596f},
  { -4.47319751327627f,    31.7437477578885f},
  { 89.4042236531957f,    19.4323787867568f},
  { 19.4333506525231f,    0.561836705127642f},
  { -90.5957763559802f,    19.4323787960110f},
  { 1.68823277243644f,    70.5588431716103f},
  { 160.566649341630f,    -0.561836710906428f},
  { -1.68823278785115f,    -70.5588431610286f},
  { -160.566649341969f,    0.561836710268294f},
  { -178.311767230745f,    70.5588431660223f},
  { -19.4333506583259f,    -0.561836706033704f},
  { 178.311767196809f,    -70.5588431573574f},
  { -89.4042236487700f,    -19.4323788021388f},
  { 90.5957763503897f,    -19.4323787954717f},
  { -27.2815067583220f,    -27.8808314491943f},
  { -130.905568835078f,    51.7761354312400f},
  { 152.718493229630f,    -27.8808314607976f},
  { 120.763555336382f,    -23.9009625080681f},
  { -49.0944311638630f,    -51.7761354094479f},
  { -120.763555353242f,    23.9009625165290f},
  { 49.0944311840534f,    51.7761354238881f},
  { -59.2364446514503f,    -23.9009625056751f},
  { 130.905568826389f,    -51.7761354158204f},
  { 59.2364446451883f,    23.9009624998341f},
  { 27.2815067487963f,    27.8808314664501f},
  { -152.718493245622f,    27.8808314654532f},
  { 139.901804856188f,    -15.4407806825701f},
  { -23.2113006173074f,    -47.5046360327723f},
  { -40.0981951499553f,    -15.4407806839368f},
  { -109.854078352440f,    38.3783422891448f},
  { -156.788699380889f,    47.5046360210690f},
  { 109.854078359427f,    -38.3783422964609f},
  { 156.788699381528f,    -47.5046360219201f},
  { 70.1459216294922f,    38.3783423074265f},
  { 23.2113006228677f,    47.5046360104477f},
  { -70.1459216446086f,    -38.3783423039165f},
  { -139.901804841453f,    15.4407806892863f},
  { 40.0981951684407f,    15.4407806908990f},
  { -2.16573695217950f,    -43.3318799197451f},
  { -92.2938906844325f,    46.6247554661090f},
  { 177.834263046153f,    -43.3318799190707f},
  { 133.352316326215f,    -1.57516097031429f},
  { -87.7061093016348f,    -46.6247554622056f},
  { -133.352316333975f,    1.57516097799139f},
  { 87.7061093086320f,    46.6247554641101f},
  { -46.6476836570233f,    -1.57516097385903f},
  { 92.2938906898742f,    -46.6247554600048f},
  { 46.6476836611639f,    1.57516097246983f},
  { 2.16573695129284f,    43.3318799277644f},
  { -177.834263050771f,    43.3318799270053f},
  { 41.1208632162695f,    27.9646947927357f},
  { 38.9138728981725f,    41.7102093980315f},
  { -138.879136782853f,    27.9646947982887f},
  { 54.8250100121816f,    35.5110720533891f},
  { 141.086127088125f,    -41.7102093910023f},
  { -54.8250100053603f,    -35.5110720615024f},
  { -141.086127101073f,    41.7102093902888f},
  { -125.174989994100f,    35.5110720440380f},
  { -38.9138729172550f,    -41.7102094018118f},
  { 125.174989996741f,    -35.5110720651527f},
  { -41.1208632217385f,    -27.9646947948851f},
  { 138.879136794409f,    -27.9646948052303f},
  { -126.671738977382f,    13.7455063912272f},
  { 163.039348194203f,    -35.4593607782491f},
  { 53.3282610301012f,    13.7455063979840f},
  { -67.7268090440833f,    -51.1782092144301f},
  { 16.9606518215385f,    35.4593607703504f},
  { 67.7268090257288f,    51.1782092265546f},
  { -16.9606517993452f,    -35.4593607689611f},
  { 112.273190969834f,    -51.1782092192888f},
  { -163.039348185547f,    35.4593607723283f},
  { -112.273190957376f,    51.1782092341149f},
  { 126.671738959185f,    -13.7455063782139f},
  { -53.3282610282180f,    -13.7455063955631f},
  { 6.02089778878271f,    -15.4236613833786f},
  { -69.1836095324131f,    73.4694224515592f},
  { -173.979102199487f,    -15.4236613862140f},
  { 105.505109197649f,    5.80330045208592f},
  { -110.816390476091f,    -73.4694224391469f},
  { -105.505109202905f,    -5.80330046619823f},
  { 110.816390459930f,    73.4694224564217f},
  { -74.4948908231566f,    5.80330045064407f},
  { 69.1836095333777f,    -73.4694224655132f},
  { 74.4948908093487f,    -5.80330045470907f},
  { -6.02089778873206f,    15.4236613640530f},
  { 173.979102218519f,    15.4236613605212f},
  { 160.076840795498f,    33.4498741022970f},
  { 62.7147965588111f,    -51.6681585392984f},
  { -19.9231591961972f,    33.4498741039417f},
  { -54.9049223659198f,    16.5181477389673f},
  { 117.285203425055f,    51.6681585436760f},
  { 54.9049223709557f,    -16.5181477240261f},
  { -117.285203439130f,    -51.6681585533349f},
  { 125.095077628614f,    16.5181477187047f},
  { -62.7147965903038f,    51.6681585461130f},
  { -125.095077616411f,    -16.5181477269097f},
  { -160.076840817170f,    -33.4498740878939f},
  { 19.9231591810885f,    -33.4498741013166f},
  { 80.3431888573962f,    6.51249291423965f},
  { 6.60527867036010f,    9.59390620161123f},
  { -99.6568111304047f,    6.51249292700582f},
  { 55.7635185484911f,    78.3697009259062f},
  { 173.394721316931f,    -9.59390618747535f},
  { -55.7635184680459f,    -78.3697009324641f},
  { -173.394721325160f,    9.59390619886874f},
  { -124.236481485585f,    78.3697009268863f},
  { -6.60527868432510f,    -9.59390619362584f},
  { 124.236481466870f,    -78.3697009269409f},
  { -80.3431888585603f,    -6.51249291997702f},
  { 99.6568111422229f,    -6.51249292210797f},
  { 6.25305021559283f,    24.6392950256081f},
  { 76.6413523791277f,    64.6277102501776f},
  { -173.746949786342f,    24.6392950270846f},
  { 65.2308926368024f,    5.68174387058010f},
  { 103.358647610018f,    -64.6277102549169f},
  { -65.2308926438328f,    -5.68174387530126f},
  { -103.358647609088f,    64.6277102395408f},
  { -114.769107378169f,    5.68174385444224f},
  { -76.6413524385702f,    -64.6277102366225f},
  { 114.769107372832f,    -5.68174386565954f},
  { -6.25305020304394f,    -24.6392950440344f},
  { 173.746949810513f,    -24.6392950534774f},
  { 51.4090024464058f,    -63.4544715386100f},
  { -68.6705283534937f,    16.1863550443443f},
  { -128.590997556725f,    -63.4544715401145f},
  { 162.692298810171f,    20.4452510722808f},
  { -111.329471638371f,    -16.1863550400168f},
  { -162.692298815208f,    -20.4452510670904f},
  { 111.329471642948f,    16.1863550327398f},
  { -17.3077011861304f,    20.4452510665979f},
  { 68.6705283570530f,    -16.1863550335289f},
  { 17.3077011789208f,    -20.4452510732108f},
  { -51.4090024537963f,    63.4544715440609f},
  { 128.590997552950f,    63.4544715477391f},
  { -50.6030753394785f,    28.8535490286561f},
  { 144.511670270974f,    33.7724670889798f},
  { 129.396924676242f,    28.8535490380526f},
  { 49.0388136434489f,    -42.5966719232101f},
  { 35.4883297409466f,    -33.7724671046099f},
  { -49.0388136542479f,    42.5966719106232f},
  { -35.4883297326542f,    33.7724670909314f},
  { -130.961186367338f,    -42.5966719056932f},
  { -144.511670245097f,    -33.7724670921865f},
  { 130.961186354940f,    42.5966719131738f},
  { 50.6030753326950f,    -28.8535490411630f},
  { -129.396924672948f,    -28.8535490533258f}};

/* Minimum T-design of degree 30 (480 points) */
const float __Tdesign_degree_30_dirs_deg[480][2] =
{ { 13.6960601261042f,    2.12849455078750f},
  { 22.3672529329788f,    3.26018326470744f},
  { 19.7822526888058f,    12.7245912054259f},
  { 31.4961835085632f,    0.749608973508380f},
  { 30.0922529861350f,    9.95347112317270f},
  { 28.9776077078097f,    18.9944686965084f},
  { 39.5201303606249f,    11.1641896255397f},
  { 41.7164394113011f,    3.08397032332731f},
  { 38.7165113655867f,    20.5789647432244f},
  { 40.2284760177905f,    31.2584018565766f},
  { 49.0148578232563f,    14.0296855277638f},
  { 47.6760670431202f,    23.5720687312074f},
  { 52.2116961020212f,    6.15820050294267f},
  { 58.0891457617985f,    17.1240137367092f},
  { 58.0975802885148f,    26.2928953884844f},
  { 62.2534167045455f,    8.20383864923332f},
  { 67.5870707270298f,    17.3445388644534f},
  { 70.5764372977999f,    6.57616823288140f},
  { 78.4367051268425f,    10.3495864631808f},
  { 83.9434226672775f,    2.68591230435371f},
  { 2.19072891002142f,    -13.6864264797006f},
  { 3.52477947518199f,    -22.3291001286183f},
  { 13.4942488344899f,    -19.2769243983019f},
  { 0.879107453588522f,    -31.4931791059370f},
  { 11.4655896703476f,    -29.5937488441603f},
  { 21.4786307841915f,    -27.2638302586611f},
  { 14.3507782613724f,    -38.6313588224717f},
  { 4.12836656345967f,    -41.6425085039365f},
  { 25.6968567837124f,    -35.8422505012108f},
  { 38.4871278890970f,    -33.5101544329916f},
  { 20.8564191950947f,    -47.0847603563625f},
  { 32.9430318528116f,    -42.6608163612897f},
  { 9.98667458383702f,    -51.7872993130739f},
  { 30.2361749043852f,    -54.2173413119525f},
  { 43.0733481971825f,    -49.5628776276434f},
  { 17.2061941935799f,    -61.1586471998043f},
  { 39.3220435694026f,    -61.9361924300299f},
  { 19.1195422289134f,    -69.5341786055489f},
  { 42.3358183079906f,    -74.5286602462756f},
  { 23.9710329455048f,    -83.3766051243163f},
  { 98.9210048264399f,    76.1426581989395f},
  { 98.5132622022133f,    67.4084710025918f},
  { 123.711134389156f,    66.6173911749998f},
  { 91.4346002282839f,    58.4958143382882f},
  { 109.290245299578f,    58.4513637344324f},
  { 125.394209196642f,    55.8097037544560f},
  { 107.230633468620f,    49.1831921823224f},
  { 94.6288352137224f,    48.1905665325417f},
  { 120.975597856623f,    46.9250864921666f},
  { 133.225165739314f,    40.7413878157755f},
  { 108.315447712762f,    39.5164001582652f},
  { 120.545915807373f,    38.1074343439095f},
  { 97.7745045532820f,    37.5323944976275f},
  { 109.948430323503f,    30.3423852744922f},
  { 120.198798808278f,    28.2811798041250f},
  { 99.2523132528176f,    27.4385806787638f},
  { 108.666953765664f,    21.3424856507812f},
  { 96.9692379986232f,    19.2906876046471f},
  { 100.559220052228f,    11.3726271758806f},
  { 92.7009664390342f,    6.04989888883805f},
  { -76.3039398738958f,    2.12849455078750f},
  { -67.6327470670212f,    3.26018326470744f},
  { -70.2177473111942f,    12.7245912054259f},
  { -58.5038164914368f,    0.749608973508380f},
  { -59.9077470138650f,    9.95347112317270f},
  { -61.0223922921903f,    18.9944686965084f},
  { -50.4798696393751f,    11.1641896255397f},
  { -48.2835605886989f,    3.08397032332731f},
  { -51.2834886344133f,    20.5789647432244f},
  { -49.7715239822095f,    31.2584018565766f},
  { -40.9851421767437f,    14.0296855277638f},
  { -42.3239329568798f,    23.5720687312074f},
  { -37.7883038979788f,    6.15820050294267f},
  { -31.9108542382015f,    17.1240137367092f},
  { -31.9024197114852f,    26.2928953884844f},
  { -27.7465832954545f,    8.20383864923332f},
  { -22.4129292729702f,    17.3445388644534f},
  { -19.4235627022001f,    6.57616823288140f},
  { -11.5632948731575f,    10.3495864631808f},
  { -6.05657733272249f,    2.68591230435371f},
  { -2.19072891002142f,    13.6864264797006f},
  { -3.52477947518199f,    22.3291001286183f},
  { -13.4942488344899f,    19.2769243983019f},
  { -0.879107453588522f,    31.4931791059370f},
  { -11.4655896703476f,    29.5937488441603f},
  { -21.4786307841915f,    27.2638302586611f},
  { -14.3507782613724f,    38.6313588224717f},
  { -4.12836656345967f,    41.6425085039365f},
  { -25.6968567837124f,    35.8422505012108f},
  { -38.4871278890970f,    33.5101544329916f},
  { -20.8564191950947f,    47.0847603563625f},
  { -32.9430318528116f,    42.6608163612897f},
  { -9.98667458383702f,    51.7872993130739f},
  { -30.2361749043852f,    54.2173413119525f},
  { -43.0733481971825f,    49.5628776276434f},
  { -17.2061941935799f,    61.1586471998043f},
  { -39.3220435694026f,    61.9361924300299f},
  { -19.1195422289134f,    69.5341786055489f},
  { -42.3358183079906f,    74.5286602462756f},
  { -23.9710329455048f,    83.3766051243163f},
  { 81.0789951735601f,    -76.1426581989395f},
  { 81.4867377977868f,    -67.4084710025918f},
  { 56.2888656108444f,    -66.6173911749998f},
  { 88.5653997717161f,    -58.4958143382882f},
  { 70.7097547004225f,    -58.4513637344324f},
  { 54.6057908033581f,    -55.8097037544560f},
  { 72.7693665313804f,    -49.1831921823224f},
  { 85.3711647862776f,    -48.1905665325417f},
  { 59.0244021433773f,    -46.9250864921666f},
  { 46.7748342606863f,    -40.7413878157755f},
  { 71.6845522872380f,    -39.5164001582652f},
  { 59.4540841926275f,    -38.1074343439095f},
  { 82.2254954467180f,    -37.5323944976275f},
  { 70.0515696764966f,    -30.3423852744922f},
  { 59.8012011917224f,    -28.2811798041250f},
  { 80.7476867471824f,    -27.4385806787638f},
  { 71.3330462343361f,    -21.3424856507812f},
  { 83.0307620013768f,    -19.2906876046471f},
  { 79.4407799477720f,    -11.3726271758806f},
  { 87.2990335609658f,    -6.04989888883805f},
  { 103.696060126104f,    2.12849455078750f},
  { 112.367252932979f,    3.26018326470744f},
  { 109.782252688806f,    12.7245912054259f},
  { 121.496183508563f,    0.749608973508380f},
  { 120.092252986135f,    9.95347112317270f},
  { 118.977607707810f,    18.9944686965084f},
  { 129.520130360625f,    11.1641896255397f},
  { 131.716439411301f,    3.08397032332731f},
  { 128.716511365587f,    20.5789647432244f},
  { 130.228476017790f,    31.2584018565766f},
  { 139.014857823256f,    14.0296855277638f},
  { 137.676067043120f,    23.5720687312074f},
  { 142.211696102021f,    6.15820050294267f},
  { 148.089145761799f,    17.1240137367092f},
  { 148.097580288515f,    26.2928953884844f},
  { 152.253416704546f,    8.20383864923332f},
  { 157.587070727030f,    17.3445388644534f},
  { 160.576437297800f,    6.57616823288140f},
  { 168.436705126843f,    10.3495864631808f},
  { 173.943422667278f,    2.68591230435371f},
  { 8.92100482643985f,    76.1426581989395f},
  { 8.51326220221325f,    67.4084710025918f},
  { 33.7111343891556f,    66.6173911749998f},
  { 1.43460022828393f,    58.4958143382882f},
  { 19.2902452995775f,    58.4513637344324f},
  { 35.3942091966419f,    55.8097037544560f},
  { 17.2306334686196f,    49.1831921823224f},
  { 4.62883521372239f,    48.1905665325417f},
  { 30.9755978566227f,    46.9250864921666f},
  { 43.2251657393137f,    40.7413878157755f},
  { 18.3154477127620f,    39.5164001582652f},
  { 30.5459158073725f,    38.1074343439095f},
  { 7.77450455328202f,    37.5323944976275f},
  { 19.9484303235034f,    30.3423852744922f},
  { 30.1987988082776f,    28.2811798041250f},
  { 9.25231325281759f,    27.4385806787638f},
  { 18.6669537656639f,    21.3424856507812f},
  { 6.96923799862323f,    19.2906876046471f},
  { 10.5592200522280f,    11.3726271758806f},
  { 2.70096643903422f,    6.04989888883805f},
  { -8.92100482643985f,    -76.1426581989395f},
  { -8.51326220221325f,    -67.4084710025918f},
  { -33.7111343891556f,    -66.6173911749998f},
  { -1.43460022828393f,    -58.4958143382882f},
  { -19.2902452995775f,    -58.4513637344324f},
  { -35.3942091966419f,    -55.8097037544560f},
  { -17.2306334686196f,    -49.1831921823224f},
  { -4.62883521372239f,    -48.1905665325417f},
  { -30.9755978566227f,    -46.9250864921666f},
  { -43.2251657393137f,    -40.7413878157755f},
  { -18.3154477127620f,    -39.5164001582652f},
  { -30.5459158073725f,    -38.1074343439095f},
  { -7.77450455328202f,    -37.5323944976275f},
  { -19.9484303235034f,    -30.3423852744922f},
  { -30.1987988082776f,    -28.2811798041250f},
  { -9.25231325281759f,    -27.4385806787638f},
  { -18.6669537656639f,    -21.3424856507812f},
  { -6.96923799862323f,    -19.2906876046471f},
  { -10.5592200522280f,    -11.3726271758806f},
  { -2.70096643903422f,    -6.04989888883805f},
  { -171.078995173560f,    76.1426581989395f},
  { -171.486737797787f,    67.4084710025918f},
  { -146.288865610844f,    66.6173911749998f},
  { -178.565399771716f,    58.4958143382882f},
  { -160.709754700422f,    58.4513637344324f},
  { -144.605790803358f,    55.8097037544560f},
  { -162.769366531380f,    49.1831921823224f},
  { -175.371164786278f,    48.1905665325417f},
  { -149.024402143377f,    46.9250864921666f},
  { -136.774834260686f,    40.7413878157755f},
  { -161.684552287238f,    39.5164001582652f},
  { -149.454084192628f,    38.1074343439095f},
  { -172.225495446718f,    37.5323944976275f},
  { -160.051569676497f,    30.3423852744922f},
  { -149.801201191722f,    28.2811798041250f},
  { -170.747686747182f,    27.4385806787638f},
  { -161.333046234336f,    21.3424856507812f},
  { -173.030762001377f,    19.2906876046471f},
  { -169.440779947772f,    11.3726271758806f},
  { -177.299033560966f,    6.04989888883805f},
  { 171.078995173560f,    -76.1426581989395f},
  { 171.486737797787f,    -67.4084710025918f},
  { 146.288865610844f,    -66.6173911749998f},
  { 178.565399771716f,    -58.4958143382882f},
  { 160.709754700422f,    -58.4513637344324f},
  { 144.605790803358f,    -55.8097037544560f},
  { 162.769366531380f,    -49.1831921823224f},
  { 175.371164786278f,    -48.1905665325417f},
  { 149.024402143377f,    -46.9250864921666f},
  { 136.774834260686f,    -40.7413878157755f},
  { 161.684552287238f,    -39.5164001582652f},
  { 149.454084192628f,    -38.1074343439095f},
  { 172.225495446718f,    -37.5323944976275f},
  { 160.051569676497f,    -30.3423852744922f},
  { 149.801201191722f,    -28.2811798041250f},
  { 170.747686747182f,    -27.4385806787638f},
  { 161.333046234336f,    -21.3424856507812f},
  { 173.030762001377f,    -19.2906876046471f},
  { 169.440779947772f,    -11.3726271758806f},
  { 177.299033560966f,    -6.04989888883805f},
  { 87.8092710899786f,    13.6864264797006f},
  { 86.4752205248180f,    22.3291001286183f},
  { 76.5057511655102f,    19.2769243983019f},
  { 89.1208925464115f,    31.4931791059370f},
  { 78.5344103296524f,    29.5937488441603f},
  { 68.5213692158085f,    27.2638302586611f},
  { 75.6492217386276f,    38.6313588224717f},
  { 85.8716334365403f,    41.6425085039365f},
  { 64.3031432162877f,    35.8422505012108f},
  { 51.5128721109030f,    33.5101544329916f},
  { 69.1435808049053f,    47.0847603563625f},
  { 57.0569681471884f,    42.6608163612897f},
  { 80.0133254161630f,    51.7872993130739f},
  { 59.7638250956148f,    54.2173413119525f},
  { 46.9266518028175f,    49.5628776276434f},
  { 72.7938058064201f,    61.1586471998043f},
  { 50.6779564305974f,    61.9361924300299f},
  { 70.8804577710866f,    69.5341786055489f},
  { 47.6641816920094f,    74.5286602462756f},
  { 66.0289670544953f,    83.3766051243163f},
  { 92.1907289100214f,    -13.6864264797006f},
  { 93.5247794751820f,    -22.3291001286183f},
  { 103.494248834490f,    -19.2769243983019f},
  { 90.8791074535885f,    -31.4931791059370f},
  { 101.465589670348f,    -29.5937488441603f},
  { 111.478630784192f,    -27.2638302586611f},
  { 104.350778261372f,    -38.6313588224717f},
  { 94.1283665634597f,    -41.6425085039365f},
  { 115.696856783712f,    -35.8422505012108f},
  { 128.487127889097f,    -33.5101544329916f},
  { 110.856419195095f,    -47.0847603563625f},
  { 122.943031852812f,    -42.6608163612897f},
  { 99.9866745838370f,    -51.7872993130739f},
  { 120.236174904385f,    -54.2173413119525f},
  { 133.073348197183f,    -49.5628776276434f},
  { 107.206194193580f,    -61.1586471998043f},
  { 129.322043569403f,    -61.9361924300299f},
  { 109.119542228913f,    -69.5341786055489f},
  { 132.335818307991f,    -74.5286602462756f},
  { 113.971032945505f,    -83.3766051243163f},
  { -87.8092710899786f,    -13.6864264797006f},
  { -86.4752205248180f,    -22.3291001286183f},
  { -76.5057511655102f,    -19.2769243983019f},
  { -89.1208925464115f,    -31.4931791059370f},
  { -78.5344103296524f,    -29.5937488441603f},
  { -68.5213692158085f,    -27.2638302586611f},
  { -75.6492217386276f,    -38.6313588224717f},
  { -85.8716334365403f,    -41.6425085039365f},
  { -64.3031432162877f,    -35.8422505012108f},
  { -51.5128721109030f,    -33.5101544329916f},
  { -69.1435808049053f,    -47.0847603563625f},
  { -57.0569681471884f,    -42.6608163612897f},
  { -80.0133254161630f,    -51.7872993130739f},
  { -59.7638250956148f,    -54.2173413119525f},
  { -46.9266518028175f,    -49.5628776276434f},
  { -72.7938058064201f,    -61.1586471998043f},
  { -50.6779564305974f,    -61.9361924300299f},
  { -70.8804577710866f,    -69.5341786055489f},
  { -47.6641816920094f,    -74.5286602462756f},
  { -66.0289670544953f,    -83.3766051243163f},
  { -92.1907289100214f,    13.6864264797006f},
  { -93.5247794751820f,    22.3291001286183f},
  { -103.494248834490f,    19.2769243983019f},
  { -90.8791074535885f,    31.4931791059370f},
  { -101.465589670348f,    29.5937488441603f},
  { -111.478630784192f,    27.2638302586611f},
  { -104.350778261372f,    38.6313588224717f},
  { -94.1283665634597f,    41.6425085039365f},
  { -115.696856783712f,    35.8422505012108f},
  { -128.487127889097f,    33.5101544329916f},
  { -110.856419195095f,    47.0847603563625f},
  { -122.943031852812f,    42.6608163612897f},
  { -99.9866745838370f,    51.7872993130739f},
  { -120.236174904385f,    54.2173413119525f},
  { -133.073348197183f,    49.5628776276434f},
  { -107.206194193580f,    61.1586471998043f},
  { -129.322043569403f,    61.9361924300299f},
  { -109.119542228913f,    69.5341786055489f},
  { -132.335818307991f,    74.5286602462756f},
  { -113.971032945505f,    83.3766051243163f},
  { -13.6960601261042f,    -2.12849455078750f},
  { -22.3672529329788f,    -3.26018326470744f},
  { -19.7822526888058f,    -12.7245912054259f},
  { -31.4961835085632f,    -0.749608973508380f},
  { -30.0922529861350f,    -9.95347112317270f},
  { -28.9776077078097f,    -18.9944686965084f},
  { -39.5201303606249f,    -11.1641896255397f},
  { -41.7164394113011f,    -3.08397032332731f},
  { -38.7165113655867f,    -20.5789647432244f},
  { -40.2284760177905f,    -31.2584018565766f},
  { -49.0148578232563f,    -14.0296855277638f},
  { -47.6760670431202f,    -23.5720687312074f},
  { -52.2116961020212f,    -6.15820050294267f},
  { -58.0891457617985f,    -17.1240137367092f},
  { -58.0975802885148f,    -26.2928953884844f},
  { -62.2534167045455f,    -8.20383864923332f},
  { -67.5870707270298f,    -17.3445388644534f},
  { -70.5764372977999f,    -6.57616823288140f},
  { -78.4367051268425f,    -10.3495864631808f},
  { -83.9434226672775f,    -2.68591230435371f},
  { 166.303939873896f,    -2.12849455078750f},
  { 157.632747067021f,    -3.26018326470744f},
  { 160.217747311194f,    -12.7245912054259f},
  { 148.503816491437f,    -0.749608973508380f},
  { 149.907747013865f,    -9.95347112317270f},
  { 151.022392292190f,    -18.9944686965084f},
  { 140.479869639375f,    -11.1641896255397f},
  { 138.283560588699f,    -3.08397032332731f},
  { 141.283488634413f,    -20.5789647432244f},
  { 139.771523982210f,    -31.2584018565766f},
  { 130.985142176744f,    -14.0296855277638f},
  { 132.323932956880f,    -23.5720687312074f},
  { 127.788303897979f,    -6.15820050294267f},
  { 121.910854238202f,    -17.1240137367092f},
  { 121.902419711485f,    -26.2928953884844f},
  { 117.746583295455f,    -8.20383864923332f},
  { 112.412929272970f,    -17.3445388644534f},
  { 109.423562702200f,    -6.57616823288140f},
  { 101.563294873158f,    -10.3495864631808f},
  { 96.0565773327225f,    -2.68591230435371f},
  { -166.303939873896f,    2.12849455078750f},
  { -157.632747067021f,    3.26018326470744f},
  { -160.217747311194f,    12.7245912054259f},
  { -148.503816491437f,    0.749608973508380f},
  { -149.907747013865f,    9.95347112317270f},
  { -151.022392292190f,    18.9944686965084f},
  { -140.479869639375f,    11.1641896255397f},
  { -138.283560588699f,    3.08397032332731f},
  { -141.283488634413f,    20.5789647432244f},
  { -139.771523982210f,    31.2584018565766f},
  { -130.985142176744f,    14.0296855277638f},
  { -132.323932956880f,    23.5720687312074f},
  { -127.788303897979f,    6.15820050294267f},
  { -121.910854238202f,    17.1240137367092f},
  { -121.902419711485f,    26.2928953884844f},
  { -117.746583295455f,    8.20383864923332f},
  { -112.412929272970f,    17.3445388644534f},
  { -109.423562702200f,    6.57616823288140f},
  { -101.563294873158f,    10.3495864631808f},
  { -96.0565773327225f,    2.68591230435371f},
  { 76.3039398738958f,    -2.12849455078750f},
  { 67.6327470670212f,    -3.26018326470744f},
  { 70.2177473111942f,    -12.7245912054259f},
  { 58.5038164914368f,    -0.749608973508380f},
  { 59.9077470138650f,    -9.95347112317270f},
  { 61.0223922921903f,    -18.9944686965084f},
  { 50.4798696393751f,    -11.1641896255397f},
  { 48.2835605886989f,    -3.08397032332731f},
  { 51.2834886344133f,    -20.5789647432244f},
  { 49.7715239822095f,    -31.2584018565766f},
  { 40.9851421767437f,    -14.0296855277638f},
  { 42.3239329568798f,    -23.5720687312074f},
  { 37.7883038979788f,    -6.15820050294267f},
  { 31.9108542382015f,    -17.1240137367092f},
  { 31.9024197114852f,    -26.2928953884844f},
  { 27.7465832954545f,    -8.20383864923332f},
  { 22.4129292729702f,    -17.3445388644534f},
  { 19.4235627022001f,    -6.57616823288140f},
  { 11.5632948731575f,    -10.3495864631808f},
  { 6.05657733272249f,    -2.68591230435371f},
  { -103.696060126104f,    -2.12849455078750f},
  { -112.367252932979f,    -3.26018326470744f},
  { -109.782252688806f,    -12.7245912054259f},
  { -121.496183508563f,    -0.749608973508380f},
  { -120.092252986135f,    -9.95347112317270f},
  { -118.977607707810f,    -18.9944686965084f},
  { -129.520130360625f,    -11.1641896255397f},
  { -131.716439411301f,    -3.08397032332731f},
  { -128.716511365587f,    -20.5789647432244f},
  { -130.228476017790f,    -31.2584018565766f},
  { -139.014857823256f,    -14.0296855277638f},
  { -137.676067043120f,    -23.5720687312074f},
  { -142.211696102021f,    -6.15820050294267f},
  { -148.089145761799f,    -17.1240137367092f},
  { -148.097580288515f,    -26.2928953884844f},
  { -152.253416704546f,    -8.20383864923332f},
  { -157.587070727030f,    -17.3445388644534f},
  { -160.576437297800f,    -6.57616823288140f},
  { -168.436705126843f,    -10.3495864631808f},
  { -173.943422667278f,    -2.68591230435371f},
  { 177.809271089979f,    13.6864264797006f},
  { 176.475220524818f,    22.3291001286183f},
  { 166.505751165510f,    19.2769243983019f},
  { 179.120892546411f,    31.4931791059370f},
  { 168.534410329652f,    29.5937488441603f},
  { 158.521369215808f,    27.2638302586611f},
  { 165.649221738628f,    38.6313588224717f},
  { 175.871633436540f,    41.6425085039365f},
  { 154.303143216288f,    35.8422505012108f},
  { 141.512872110903f,    33.5101544329916f},
  { 159.143580804905f,    47.0847603563625f},
  { 147.056968147188f,    42.6608163612897f},
  { 170.013325416163f,    51.7872993130739f},
  { 149.763825095615f,    54.2173413119525f},
  { 136.926651802818f,    49.5628776276434f},
  { 162.793805806420f,    61.1586471998043f},
  { 140.677956430597f,    61.9361924300299f},
  { 160.880457771087f,    69.5341786055489f},
  { 137.664181692009f,    74.5286602462756f},
  { 156.028967054495f,    83.3766051243163f},
  { -177.809271089979f,    -13.6864264797006f},
  { -176.475220524818f,    -22.3291001286183f},
  { -166.505751165510f,    -19.2769243983019f},
  { -179.120892546411f,    -31.4931791059370f},
  { -168.534410329652f,    -29.5937488441603f},
  { -158.521369215808f,    -27.2638302586611f},
  { -165.649221738628f,    -38.6313588224717f},
  { -175.871633436540f,    -41.6425085039365f},
  { -154.303143216288f,    -35.8422505012108f},
  { -141.512872110903f,    -33.5101544329916f},
  { -159.143580804905f,    -47.0847603563625f},
  { -147.056968147188f,    -42.6608163612897f},
  { -170.013325416163f,    -51.7872993130739f},
  { -149.763825095615f,    -54.2173413119525f},
  { -136.926651802818f,    -49.5628776276434f},
  { -162.793805806420f,    -61.1586471998043f},
  { -140.677956430597f,    -61.9361924300299f},
  { -160.880457771087f,    -69.5341786055489f},
  { -137.664181692009f,    -74.5286602462756f},
  { -156.028967054495f,    -83.3766051243163f},
  { -81.0789951735601f,    76.1426581989395f},
  { -81.4867377977868f,    67.4084710025918f},
  { -56.2888656108444f,    66.6173911749998f},
  { -88.5653997717161f,    58.4958143382882f},
  { -70.7097547004225f,    58.4513637344324f},
  { -54.6057908033581f,    55.8097037544560f},
  { -72.7693665313804f,    49.1831921823224f},
  { -85.3711647862776f,    48.1905665325417f},
  { -59.0244021433773f,    46.9250864921666f},
  { -46.7748342606863f,    40.7413878157755f},
  { -71.6845522872380f,    39.5164001582652f},
  { -59.4540841926275f,    38.1074343439095f},
  { -82.2254954467180f,    37.5323944976275f},
  { -70.0515696764966f,    30.3423852744922f},
  { -59.8012011917224f,    28.2811798041250f},
  { -80.7476867471824f,    27.4385806787638f},
  { -71.3330462343361f,    21.3424856507812f},
  { -83.0307620013768f,    19.2906876046471f},
  { -79.4407799477720f,    11.3726271758806f},
  { -87.2990335609658f,    6.04989888883805f},
  { -98.9210048264399f,    -76.1426581989395f},
  { -98.5132622022133f,    -67.4084710025918f},
  { -123.711134389156f,    -66.6173911749998f},
  { -91.4346002282839f,    -58.4958143382882f},
  { -109.290245299578f,    -58.4513637344324f},
  { -125.394209196642f,    -55.8097037544560f},
  { -107.230633468620f,    -49.1831921823224f},
  { -94.6288352137224f,    -48.1905665325417f},
  { -120.975597856623f,    -46.9250864921666f},
  { -133.225165739314f,    -40.7413878157755f},
  { -108.315447712762f,    -39.5164001582652f},
  { -120.545915807373f,    -38.1074343439095f},
  { -97.7745045532820f,    -37.5323944976275f},
  { -109.948430323503f,    -30.3423852744922f},
  { -120.198798808278f,    -28.2811798041250f},
  { -99.2523132528176f,    -27.4385806787638f},
  { -108.666953765664f,    -21.3424856507812f},
  { -96.9692379986232f,    -19.2906876046471f},
  { -100.559220052228f,    -11.3726271758806f},
  { -92.7009664390342f,    -6.04989888883805f}};

/* Minimum T-design of degree 40 (840 points) */
const float __Tdesign_degree_40_dirs_deg[840][2] =
{ { 149.263603834783f,    -53.0060664811199f},
  { -164.365030657005f,    60.2080167597588f},
  { -114.133156395014f,    -22.4117838350371f},
  { 99.7994227322605f,    -5.21863546706097f},
  { -106.287016605210f,    -70.2090939724247f},
  { 154.363310133555f,    56.6615370152244f},
  { -24.1886191948301f,    -37.8476655192962f},
  { 145.913308751917f,    32.9908659694550f},
  { 30.1584949552393f,    31.9524129967228f},
  { -178.686993681972f,    -11.1793164384719f},
  { 95.0873822760472f,    25.3965414160564f},
  { -70.4240468864204f,    53.7720444422919f},
  { -94.0330048989651f,    -2.83751338514659f},
  { 114.353915132664f,    -63.4833127536594f},
  { 131.581019315399f,    8.21952951813733f},
  { -132.940913655637f,    39.2157096320986f},
  { 124.364778287929f,    -8.00246008797146f},
  { 67.3130907697372f,    -29.7820566125311f},
  { -163.043979850952f,    -7.50723653872003f},
  { 167.879987492863f,    -7.95180234835099f},
  { 44.0921698625726f,    -25.2905871681089f},
  { 31.5781658177062f,    60.0282132459340f},
  { -35.5498000646238f,    -3.60507288940253f},
  { -30.4944240768537f,    -45.7861265646663f},
  { 138.476918887805f,    -51.0190550805302f},
  { -39.6759162188722f,    12.9913367417341f},
  { 41.9794954535117f,    -6.08388687053162f},
  { -42.2620746238942f,    -14.7511416956518f},
  { -154.146027087023f,    2.47755771672502f},
  { 0.0625705952352197f,    48.7809279839986f},
  { 105.091553924230f,    -13.8676921795039f},
  { -1.36871600594072f,    -18.3598920706058f},
  { 71.9893843573992f,    -20.4620187149798f},
  { -177.335015058207f,    32.3794914148608f},
  { -158.856612739765f,    11.6979132966246f},
  { -122.925397131742f,    -17.9109112965780f},
  { 118.869502795247f,    7.69533758561870f},
  { -134.752179532797f,    57.5294033214029f},
  { -151.780426631651f,    -78.9096156385351f},
  { -95.7626451719474f,    18.9656135432167f},
  { 120.671252161945f,    -13.7558255086613f},
  { -40.4243419504315f,    18.8777830371654f},
  { 141.908713656985f,    -28.0394087488386f},
  { 35.8051353818043f,    -25.2313946559004f},
  { -168.817821457456f,    1.28808805920167f},
  { 100.579692918687f,    -64.1321186392502f},
  { 76.2080180841815f,    33.8374380889844f},
  { -144.826476401732f,    85.0701615892237f},
  { -101.626412581563f,    -23.9996224086939f},
  { 167.721029521791f,    -47.7598767470431f},
  { 129.855803342282f,    34.5535588711217f},
  { -166.014429537489f,    -54.8275910830078f},
  { -56.0219525477008f,    -53.2033822144056f},
  { -172.155740988996f,    16.8063417598105f},
  { -171.869308336958f,    -12.0017323058903f},
  { -33.3397486273608f,    -38.9854284838043f},
  { 63.8350743552927f,    -15.1654552210621f},
  { -4.42797677109172f,    35.4688197817794f},
  { -50.0251524443123f,    20.7240614349903f},
  { -121.210164186962f,    -24.6465135229038f},
  { 16.6861673909000f,    38.4698531326131f},
  { -8.15933074488397f,    -41.6898001127167f},
  { -19.5841943769194f,    40.5685249452216f},
  { 177.247280514400f,    25.8280223666138f},
  { 48.7809449181953f,    -0.0412302558525259f},
  { -136.522915872743f,    -69.6141590079244f},
  { -18.3647804513672f,    1.29903163608235f},
  { -50.3531920170481f,    -62.9985615621742f},
  { 147.592461816819f,    2.25039932569429f},
  { 167.483504268373f,    20.6838680219578f},
  { 21.0591189997961f,    -31.1448908269880f},
  { -171.228527088859f,    -28.5854008636956f},
  { -65.6813738256384f,    -22.2084036076950f},
  { 84.7045313345268f,    -9.75840459955865f},
  { -19.0550132238698f,    -5.44883907181121f},
  { 164.112452742443f,    -29.7014307878198f},
  { -27.8040071892610f,    46.0805128313995f},
  { 139.195516345974f,    -43.9994072574599f},
  { 141.149045353151f,    47.1920617550663f},
  { -6.61364075589592f,    -78.7448159660119f},
  { 115.484271121278f,    -4.59463012694014f},
  { -145.383029625808f,    11.4210789348729f},
  { -87.1554541976338f,    -4.02805202442754f},
  { 24.4440271458979f,    -10.6090655964160f},
  { 100.929879165434f,    -41.0609005548679f},
  { -138.105107262278f,    -31.8574725173498f},
  { 80.3346646541484f,    -33.9841135805965f},
  { 58.1893665188901f,    19.5575204637846f},
  { -65.6834250290337f,    -71.5011245649553f},
  { 56.3648061751115f,    -75.5369883084871f},
  { 55.8213137724635f,    40.4950119904576f},
  { 163.196089446938f,    25.1886426922989f},
  { -83.8153715099452f,    54.2918464978552f},
  { -26.2763399522643f,    36.9330763410585f},
  { 28.2117101755392f,    -28.0973806306467f},
  { -109.867882950281f,    48.5875728654897f},
  { 80.9459421723854f,    47.6630864203711f},
  { -68.6191645350763f,    45.6992947586867f},
  { -95.6664393081424f,    -64.0357211750186f},
  { 179.945186811529f,    41.2190420862713f},
  { 75.6570477065003f,    -14.6416703213570f},
  { -4.11665805433526f,    71.5909138286826f},
  { 68.5771923006971f,    16.8390616567175f},
  { -175.806351393131f,    -57.5229183609744f},
  { -119.856831742108f,    -65.9667217175294f},
  { 59.2636038347830f,    -53.0060664811199f},
  { 105.634969342995f,    60.2080167597588f},
  { 155.866843604986f,    -22.4117838350371f},
  { 9.79942273226048f,    -5.21863546706097f},
  { 163.712983394790f,    -70.2090939724247f},
  { 64.3633101335554f,    56.6615370152244f},
  { -114.188619194830f,    -37.8476655192962f},
  { 55.9133087519166f,    32.9908659694550f},
  { -59.8415050447607f,    31.9524129967228f},
  { 91.3130063180282f,    -11.1793164384719f},
  { 5.08738227604718f,    25.3965414160564f},
  { -160.424046886420f,    53.7720444422919f},
  { 175.966995101035f,    -2.83751338514659f},
  { 24.3539151326635f,    -63.4833127536594f},
  { 41.5810193153988f,    8.21952951813733f},
  { 137.059086344363f,    39.2157096320986f},
  { 34.3647782879289f,    -8.00246008797146f},
  { -22.6869092302628f,    -29.7820566125311f},
  { 106.956020149048f,    -7.50723653872003f},
  { 77.8799874928630f,    -7.95180234835099f},
  { -45.9078301374274f,    -25.2905871681089f},
  { -58.4218341822938f,    60.0282132459340f},
  { -125.549800064624f,    -3.60507288940253f},
  { -120.494424076854f,    -45.7861265646663f},
  { 48.4769188878050f,    -51.0190550805302f},
  { -129.675916218872f,    12.9913367417341f},
  { -48.0205045464884f,    -6.08388687053162f},
  { -132.262074623894f,    -14.7511416956518f},
  { 115.853972912977f,    2.47755771672502f},
  { -89.9374294047648f,    48.7809279839986f},
  { 15.0915539242296f,    -13.8676921795039f},
  { -91.3687160059407f,    -18.3598920706058f},
  { -18.0106156426008f,    -20.4620187149798f},
  { 92.6649849417932f,    32.3794914148608f},
  { 111.143387260235f,    11.6979132966246f},
  { 122.925397131742f,    17.9109112965780f},
  { -118.869502795247f,    -7.69533758561870f},
  { 134.752179532797f,    -57.5294033214029f},
  { 151.780426631651f,    78.9096156385351f},
  { 95.7626451719474f,    -18.9656135432167f},
  { -120.671252161945f,    13.7558255086613f},
  { 40.4243419504315f,    -18.8777830371654f},
  { -141.908713656985f,    28.0394087488386f},
  { -35.8051353818043f,    25.2313946559004f},
  { 168.817821457456f,    -1.28808805920167f},
  { -100.579692918687f,    64.1321186392502f},
  { -76.2080180841815f,    -33.8374380889844f},
  { 144.826476401732f,    -85.0701615892237f},
  { 101.626412581563f,    23.9996224086939f},
  { -167.721029521791f,    47.7598767470431f},
  { -129.855803342282f,    -34.5535588711217f},
  { 166.014429537489f,    54.8275910830078f},
  { 56.0219525477008f,    53.2033822144056f},
  { 172.155740988996f,    -16.8063417598105f},
  { 171.869308336958f,    12.0017323058903f},
  { 33.3397486273608f,    38.9854284838043f},
  { -63.8350743552927f,    15.1654552210621f},
  { 4.42797677109172f,    -35.4688197817794f},
  { 50.0251524443123f,    -20.7240614349903f},
  { 121.210164186962f,    24.6465135229038f},
  { -16.6861673909000f,    -38.4698531326131f},
  { 8.15933074488397f,    41.6898001127167f},
  { 19.5841943769194f,    -40.5685249452216f},
  { -177.247280514400f,    -25.8280223666138f},
  { -48.7809449181953f,    0.0412302558525259f},
  { 136.522915872743f,    69.6141590079244f},
  { 18.3647804513672f,    -1.29903163608235f},
  { 50.3531920170481f,    62.9985615621742f},
  { -147.592461816819f,    -2.25039932569429f},
  { -167.483504268373f,    -20.6838680219578f},
  { 158.940881000204f,    31.1448908269880f},
  { -8.77147291114150f,    28.5854008636956f},
  { -114.318626174362f,    22.2084036076950f},
  { 95.2954686654733f,    9.75840459955865f},
  { -160.944986776130f,    5.44883907181121f},
  { 15.8875472575573f,    29.7014307878198f},
  { -152.195992810739f,    -46.0805128313995f},
  { 40.8044836540265f,    43.9994072574599f},
  { 38.8509546468490f,    -47.1920617550663f},
  { -173.386359244104f,    78.7448159660119f},
  { 64.5157288787217f,    4.59463012694014f},
  { -34.6169703741920f,    -11.4210789348729f},
  { -92.8445458023662f,    4.02805202442754f},
  { 155.555972854102f,    10.6090655964160f},
  { 79.0701208345658f,    41.0609005548679f},
  { -41.8948927377216f,    31.8574725173498f},
  { 99.6653353458516f,    33.9841135805965f},
  { 121.810633481110f,    -19.5575204637846f},
  { -114.316574970966f,    71.5011245649553f},
  { 123.635193824889f,    75.5369883084871f},
  { 124.178686227537f,    -40.4950119904576f},
  { 16.8039105530620f,    -25.1886426922989f},
  { -96.1846284900549f,    -54.2918464978552f},
  { -153.723660047736f,    -36.9330763410585f},
  { 151.788289824461f,    28.0973806306467f},
  { -70.1321170497187f,    -48.5875728654897f},
  { 99.0540578276146f,    -47.6630864203711f},
  { -111.380835464924f,    -45.6992947586867f},
  { -84.3335606918577f,    64.0357211750186f},
  { 0.0548131884707396f,    -41.2190420862713f},
  { 104.342952293500f,    14.6416703213570f},
  { -175.883341945665f,    -71.5909138286826f},
  { 111.422807699303f,    -16.8390616567175f},
  { -4.19364860686896f,    57.5229183609744f},
  { -60.1431682578916f,    65.9667217175294f},
  { -120.736396165217f,    -53.0060664811199f},
  { -74.3650306570054f,    60.2080167597588f},
  { -24.1331563950140f,    -22.4117838350371f},
  { -170.200577267740f,    -5.21863546706097f},
  { -16.2870166052097f,    -70.2090939724247f},
  { -115.636689866445f,    56.6615370152244f},
  { 65.8113808051699f,    -37.8476655192962f},
  { -124.086691248083f,    32.9908659694550f},
  { 120.158494955239f,    31.9524129967228f},
  { -88.6869936819718f,    -11.1793164384719f},
  { -174.912617723953f,    25.3965414160564f},
  { 19.5759531135796f,    53.7720444422919f},
  { -4.03300489896510f,    -2.83751338514659f},
  { -155.646084867337f,    -63.4833127536594f},
  { -138.418980684601f,    8.21952951813733f},
  { -42.9409136556371f,    39.2157096320986f},
  { -145.635221712071f,    -8.00246008797146f},
  { 157.313090769737f,    -29.7820566125311f},
  { -73.0439798509519f,    -7.50723653872003f},
  { -102.120012507137f,    -7.95180234835099f},
  { 134.092169862573f,    -25.2905871681089f},
  { 121.578165817706f,    60.0282132459340f},
  { 54.4501999353762f,    -3.60507288940253f},
  { 59.5055759231463f,    -45.7861265646663f},
  { -131.523081112195f,    -51.0190550805302f},
  { 50.3240837811279f,    12.9913367417341f},
  { 131.979495453512f,    -6.08388687053162f},
  { 47.7379253761058f,    -14.7511416956518f},
  { -64.1460270870231f,    2.47755771672502f},
  { 90.0625705952352f,    48.7809279839986f},
  { -164.908446075770f,    -13.8676921795039f},
  { 88.6312839940593f,    -18.3598920706058f},
  { 161.989384357399f,    -20.4620187149798f},
  { -87.3350150582068f,    32.3794914148608f},
  { -68.8566127397652f,    11.6979132966246f},
  { -68.9408810002039f,    -31.1448908269880f},
  { 98.7714729111415f,    -28.5854008636956f},
  { -155.681373825638f,    -22.2084036076950f},
  { -5.29546866547326f,    -9.75840459955865f},
  { -109.055013223870f,    -5.44883907181121f},
  { 74.1124527424428f,    -29.7014307878198f},
  { -117.804007189261f,    46.0805128313995f},
  { 49.1955163459735f,    -43.9994072574599f},
  { 51.1490453531510f,    47.1920617550663f},
  { -96.6136407558959f,    -78.7448159660119f},
  { 25.4842711212784f,    -4.59463012694014f},
  { 124.616970374192f,    11.4210789348729f},
  { -177.155454197634f,    -4.02805202442754f},
  { -65.5559728541021f,    -10.6090655964160f},
  { 10.9298791654342f,    -41.0609005548679f},
  { 131.894892737722f,    -31.8574725173498f},
  { -9.66533534585163f,    -33.9841135805965f},
  { -31.8106334811099f,    19.5575204637846f},
  { -155.683425029034f,    -71.5011245649553f},
  { -33.6351938248885f,    -75.5369883084871f},
  { -34.1786862275365f,    40.4950119904576f},
  { 73.1960894469380f,    25.1886426922989f},
  { -173.815371509945f,    54.2918464978552f},
  { -116.276339952264f,    36.9330763410585f},
  { -61.7882898244608f,    -28.0973806306467f},
  { 160.132117049719f,    48.5875728654897f},
  { -9.05405782761462f,    47.6630864203711f},
  { -158.619164535076f,    45.6992947586867f},
  { 174.333560691858f,    -64.0357211750186f},
  { 89.9451868115293f,    41.2190420862713f},
  { -14.3429522934998f,    -14.6416703213570f},
  { -94.1166580543353f,    71.5909138286826f},
  { -21.4228076993029f,    16.8390616567175f},
  { 94.1936486068690f,    -57.5229183609744f},
  { 150.143168257892f,    -65.9667217175294f},
  { 68.9408810002039f,    31.1448908269880f},
  { -98.7714729111415f,    28.5854008636956f},
  { 155.681373825638f,    22.2084036076950f},
  { 5.29546866547326f,    9.75840459955865f},
  { 109.055013223870f,    5.44883907181121f},
  { -74.1124527424428f,    29.7014307878198f},
  { 117.804007189261f,    -46.0805128313995f},
  { -49.1955163459735f,    43.9994072574599f},
  { -51.1490453531510f,    -47.1920617550663f},
  { 96.6136407558959f,    78.7448159660119f},
  { -25.4842711212784f,    4.59463012694014f},
  { -124.616970374192f,    -11.4210789348729f},
  { 177.155454197634f,    4.02805202442754f},
  { 65.5559728541021f,    10.6090655964160f},
  { -10.9298791654342f,    41.0609005548679f},
  { -131.894892737722f,    31.8574725173498f},
  { 9.66533534585163f,    33.9841135805965f},
  { 31.8106334811099f,    -19.5575204637846f},
  { 155.683425029034f,    71.5011245649553f},
  { 33.6351938248885f,    75.5369883084871f},
  { 34.1786862275365f,    -40.4950119904576f},
  { -73.1960894469380f,    -25.1886426922989f},
  { 173.815371509945f,    -54.2918464978552f},
  { 116.276339952264f,    -36.9330763410585f},
  { 61.7882898244608f,    28.0973806306467f},
  { -160.132117049719f,    -48.5875728654897f},
  { 9.05405782761462f,    -47.6630864203711f},
  { 158.619164535076f,    -45.6992947586867f},
  { -174.333560691858f,    64.0357211750186f},
  { -89.9451868115293f,    -41.2190420862713f},
  { 14.3429522934998f,    14.6416703213570f},
  { 94.1166580543353f,    -71.5909138286826f},
  { 21.4228076993029f,    -16.8390616567175f},
  { -94.1936486068690f,    57.5229183609744f},
  { -150.143168257892f,    65.9667217175294f},
  { 111.059118999796f,    -31.1448908269880f},
  { -81.2285270888585f,    -28.5854008636956f},
  { 24.3186261743616f,    -22.2084036076950f},
  { 174.704531334527f,    -9.75840459955865f},
  { 70.9449867761303f,    -5.44883907181121f},
  { -105.887547257557f,    -29.7014307878198f},
  { 62.1959928107390f,    46.0805128313995f},
  { -130.804483654027f,    -43.9994072574599f},
  { -128.850954646849f,    47.1920617550663f},
  { 83.3863592441041f,    -78.7448159660119f},
  { -154.515728878722f,    -4.59463012694014f},
  { -55.3830296258080f,    11.4210789348729f},
  { 2.84454580236616f,    -4.02805202442754f},
  { 114.444027145898f,    -10.6090655964160f},
  { -169.070120834566f,    -41.0609005548679f},
  { -48.1051072622784f,    -31.8574725173498f},
  { 170.334664654148f,    -33.9841135805965f},
  { 148.189366518890f,    19.5575204637846f},
  { 24.3165749709663f,    -71.5011245649553f},
  { 146.364806175111f,    -75.5369883084871f},
  { 145.821313772464f,    40.4950119904576f},
  { -106.803910553062f,    25.1886426922989f},
  { 6.18462849005484f,    54.2918464978552f},
  { 63.7236600477357f,    36.9330763410585f},
  { 118.211710175539f,    -28.0973806306467f},
  { -19.8678829502813f,    48.5875728654897f},
  { 170.945942172385f,    47.6630864203711f},
  { 21.3808354649238f,    45.6992947586867f},
  { -5.66643930814235f,    -64.0357211750186f},
  { -90.0548131884708f,    41.2190420862713f},
  { 165.657047706500f,    -14.6416703213570f},
  { 85.8833419456647f,    71.5909138286826f},
  { 158.577192300697f,    16.8390616567175f},
  { -85.8063513931311f,    -57.5229183609744f},
  { -29.8568317421084f,    -65.9667217175294f},
  { -111.059118999796f,    31.1448908269880f},
  { 81.2285270888585f,    28.5854008636956f},
  { -24.3186261743616f,    22.2084036076950f},
  { -174.704531334527f,    9.75840459955865f},
  { -70.9449867761303f,    5.44883907181121f},
  { 105.887547257557f,    29.7014307878198f},
  { -62.1959928107390f,    -46.0805128313995f},
  { 130.804483654027f,    43.9994072574599f},
  { 128.850954646849f,    -47.1920617550663f},
  { -83.3863592441041f,    78.7448159660119f},
  { 154.515728878722f,    4.59463012694014f},
  { 55.3830296258080f,    -11.4210789348729f},
  { -2.84454580236616f,    4.02805202442754f},
  { -114.444027145898f,    10.6090655964160f},
  { 169.070120834566f,    41.0609005548679f},
  { 48.1051072622784f,    31.8574725173498f},
  { -170.334664654148f,    33.9841135805965f},
  { -148.189366518890f,    -19.5575204637846f},
  { -24.3165749709663f,    71.5011245649553f},
  { -146.364806175111f,    75.5369883084871f},
  { -145.821313772464f,    -40.4950119904576f},
  { 106.803910553062f,    -25.1886426922989f},
  { -6.18462849005484f,    -54.2918464978552f},
  { -63.7236600477357f,    -36.9330763410585f},
  { -118.211710175539f,    28.0973806306467f},
  { 19.8678829502813f,    -48.5875728654897f},
  { -170.945942172385f,    -47.6630864203711f},
  { -21.3808354649238f,    -45.6992947586867f},
  { 5.66643930814235f,    64.0357211750186f},
  { 90.0548131884708f,    -41.2190420862713f},
  { -165.657047706500f,    14.6416703213570f},
  { -85.8833419456647f,    -71.5909138286826f},
  { -158.577192300697f,    -16.8390616567175f},
  { 85.8063513931311f,    57.5229183609744f},
  { 29.8568317421084f,    65.9667217175294f},
  { -147.074602868258f,    17.9109112965780f},
  { -28.8695027952470f,    -7.69533758561870f},
  { -135.247820467203f,    -57.5294033214029f},
  { -118.219573368349f,    78.9096156385351f},
  { -174.237354828053f,    -18.9656135432167f},
  { -30.6712521619453f,    13.7558255086613f},
  { 130.424341950431f,    -18.8777830371654f},
  { -51.9087136569847f,    28.0394087488386f},
  { 54.1948646181957f,    25.2313946559004f},
  { -101.182178542544f,    -1.28808805920167f},
  { -10.5796929186869f,    64.1321186392502f},
  { 13.7919819158185f,    -33.8374380889844f},
  { -125.173523598268f,    -85.0701615892237f},
  { -168.373587418437f,    23.9996224086939f},
  { -77.7210295217906f,    47.7598767470431f},
  { -39.8558033422815f,    -34.5535588711217f},
  { -103.985570462511f,    54.8275910830078f},
  { 146.021952547701f,    53.2033822144056f},
  { -97.8442590110040f,    -16.8063417598105f},
  { -98.1306916630421f,    12.0017323058903f},
  { 123.339748627361f,    38.9854284838043f},
  { 26.1649256447073f,    15.1654552210621f},
  { 94.4279767710917f,    -35.4688197817794f},
  { 140.025152444312f,    -20.7240614349903f},
  { -148.789835813038f,    24.6465135229038f},
  { 73.3138326091001f,    -38.4698531326131f},
  { 98.1593307448840f,    41.6898001127167f},
  { 109.584194376919f,    -40.5685249452216f},
  { -87.2472805144002f,    -25.8280223666138f},
  { 41.2190550818047f,    0.0412302558525259f},
  { -133.477084127257f,    69.6141590079244f},
  { 108.364780451367f,    -1.29903163608235f},
  { 140.353192017048f,    62.9985615621742f},
  { -57.5924618168189f,    -2.25039932569429f},
  { -77.4835042683727f,    -20.6838680219578f},
  { -32.9253971317421f,    -17.9109112965780f},
  { -151.130497204753f,    7.69533758561870f},
  { -44.7521795327973f,    57.5294033214029f},
  { -61.7804266316514f,    -78.9096156385351f},
  { -5.76264517194740f,    18.9656135432167f},
  { -149.328747838055f,    -13.7558255086613f},
  { 49.5756580495686f,    18.8777830371654f},
  { -128.091286343015f,    -28.0394087488386f},
  { 125.805135381804f,    -25.2313946559004f},
  { -78.8178214574562f,    1.28808805920167f},
  { -169.420307081313f,    -64.1321186392502f},
  { 166.208018084181f,    33.8374380889844f},
  { -54.8264764017325f,    85.0701615892237f},
  { -11.6264125815634f,    -23.9996224086939f},
  { -102.278970478209f,    -47.7598767470431f},
  { -140.144196657718f,    34.5535588711217f},
  { -76.0144295374893f,    -54.8275910830078f},
  { 33.9780474522993f,    -53.2033822144056f},
  { -82.1557409889960f,    16.8063417598105f},
  { -81.8693083369579f,    -12.0017323058903f},
  { 56.6602513726392f,    -38.9854284838043f},
  { 153.835074355293f,    -15.1654552210621f},
  { 85.5720232289083f,    35.4688197817794f},
  { 39.9748475556877f,    20.7240614349903f},
  { -31.2101641869618f,    -24.6465135229038f},
  { 106.686167390900f,    38.4698531326131f},
  { 81.8406692551160f,    -41.6898001127167f},
  { 70.4158056230806f,    40.5685249452216f},
  { -92.7527194855998f,    25.8280223666138f},
  { 138.780944918195f,    -0.0412302558525259f},
  { -46.5229158727434f,    -69.6141590079244f},
  { 71.6352195486328f,    1.29903163608235f},
  { 39.6468079829519f,    -62.9985615621742f},
  { -122.407538183181f,    2.25039932569429f},
  { -102.516495731627f,    20.6838680219578f},
  { 147.074602868258f,    -17.9109112965780f},
  { 28.8695027952470f,    7.69533758561870f},
  { 135.247820467203f,    57.5294033214029f},
  { 118.219573368349f,    -78.9096156385351f},
  { 174.237354828053f,    18.9656135432167f},
  { 30.6712521619453f,    -13.7558255086613f},
  { -130.424341950431f,    18.8777830371654f},
  { 51.9087136569847f,    -28.0394087488386f},
  { -54.1948646181957f,    -25.2313946559004f},
  { 101.182178542544f,    1.28808805920167f},
  { 10.5796929186869f,    -64.1321186392502f},
  { -13.7919819158185f,    33.8374380889844f},
  { 125.173523598268f,    85.0701615892237f},
  { 168.373587418437f,    -23.9996224086939f},
  { 77.7210295217906f,    -47.7598767470431f},
  { 39.8558033422815f,    34.5535588711217f},
  { 103.985570462511f,    -54.8275910830078f},
  { -146.021952547701f,    -53.2033822144056f},
  { 97.8442590110040f,    16.8063417598105f},
  { 98.1306916630421f,    -12.0017323058903f},
  { -123.339748627361f,    -38.9854284838043f},
  { -26.1649256447073f,    -15.1654552210621f},
  { -94.4279767710917f,    35.4688197817794f},
  { -140.025152444312f,    20.7240614349903f},
  { 148.789835813038f,    -24.6465135229038f},
  { -73.3138326091001f,    38.4698531326131f},
  { -98.1593307448840f,    -41.6898001127167f},
  { -109.584194376919f,    40.5685249452216f},
  { 87.2472805144002f,    25.8280223666138f},
  { -41.2190550818047f,    -0.0412302558525259f},
  { 133.477084127257f,    -69.6141590079244f},
  { -108.364780451367f,    1.29903163608235f},
  { -140.353192017048f,    -62.9985615621742f},
  { 57.5924618168189f,    2.25039932569429f},
  { 77.4835042683727f,    20.6838680219578f},
  { 32.9253971317421f,    17.9109112965780f},
  { 151.130497204753f,    -7.69533758561870f},
  { 44.7521795327973f,    -57.5294033214029f},
  { 61.7804266316514f,    78.9096156385351f},
  { 5.76264517194740f,    -18.9656135432167f},
  { 149.328747838055f,    13.7558255086613f},
  { -49.5756580495686f,    -18.8777830371654f},
  { 128.091286343015f,    28.0394087488386f},
  { -125.805135381804f,    25.2313946559004f},
  { 78.8178214574562f,    -1.28808805920167f},
  { 169.420307081313f,    64.1321186392502f},
  { -166.208018084181f,    -33.8374380889844f},
  { 54.8264764017325f,    -85.0701615892237f},
  { 11.6264125815634f,    23.9996224086939f},
  { 102.278970478209f,    47.7598767470431f},
  { 140.144196657718f,    -34.5535588711217f},
  { 76.0144295374893f,    54.8275910830078f},
  { -33.9780474522993f,    53.2033822144056f},
  { 82.1557409889960f,    -16.8063417598105f},
  { 81.8693083369579f,    12.0017323058903f},
  { -56.6602513726392f,    38.9854284838043f},
  { -153.835074355293f,    15.1654552210621f},
  { -85.5720232289083f,    -35.4688197817794f},
  { -39.9748475556877f,    -20.7240614349903f},
  { 31.2101641869618f,    24.6465135229038f},
  { -106.686167390900f,    -38.4698531326131f},
  { -81.8406692551160f,    41.6898001127167f},
  { -70.4158056230806f,    -40.5685249452216f},
  { 92.7527194855998f,    -25.8280223666138f},
  { -138.780944918195f,    0.0412302558525259f},
  { 46.5229158727434f,    69.6141590079244f},
  { -71.6352195486328f,    -1.29903163608235f},
  { -39.6468079829519f,    62.9985615621742f},
  { 122.407538183181f,    -2.25039932569429f},
  { 102.516495731627f,    -20.6838680219578f},
  { -149.263603834783f,    53.0060664811199f},
  { 164.365030657005f,    -60.2080167597588f},
  { 114.133156395014f,    22.4117838350371f},
  { -99.7994227322605f,    5.21863546706097f},
  { 106.287016605210f,    70.2090939724247f},
  { -154.363310133555f,    -56.6615370152244f},
  { 24.1886191948301f,    37.8476655192962f},
  { -145.913308751917f,    -32.9908659694550f},
  { -30.1584949552393f,    -31.9524129967228f},
  { 178.686993681972f,    11.1793164384719f},
  { -95.0873822760472f,    -25.3965414160564f},
  { 70.4240468864204f,    -53.7720444422919f},
  { 94.0330048989651f,    2.83751338514659f},
  { -114.353915132664f,    63.4833127536594f},
  { -131.581019315399f,    -8.21952951813733f},
  { 132.940913655637f,    -39.2157096320986f},
  { -124.364778287929f,    8.00246008797146f},
  { -67.3130907697372f,    29.7820566125311f},
  { 163.043979850952f,    7.50723653872003f},
  { -167.879987492863f,    7.95180234835099f},
  { -44.0921698625726f,    25.2905871681089f},
  { -31.5781658177062f,    -60.0282132459340f},
  { 35.5498000646238f,    3.60507288940253f},
  { 30.4944240768537f,    45.7861265646663f},
  { -138.476918887805f,    51.0190550805302f},
  { 39.6759162188722f,    -12.9913367417341f},
  { -41.9794954535117f,    6.08388687053162f},
  { 42.2620746238942f,    14.7511416956518f},
  { 154.146027087023f,    -2.47755771672502f},
  { -0.0625705952352197f,    -48.7809279839986f},
  { -105.091553924230f,    13.8676921795039f},
  { 1.36871600594072f,    18.3598920706058f},
  { -71.9893843573992f,    20.4620187149798f},
  { 177.335015058207f,    -32.3794914148608f},
  { 158.856612739765f,    -11.6979132966246f},
  { 30.7363961652170f,    53.0060664811199f},
  { -15.6349693429946f,    -60.2080167597588f},
  { -65.8668436049860f,    22.4117838350371f},
  { 80.2005772677395f,    5.21863546706097f},
  { -73.7129833947903f,    70.2090939724247f},
  { 25.6366898664446f,    -56.6615370152244f},
  { -155.811380805170f,    37.8476655192962f},
  { 34.0866912480834f,    -32.9908659694550f},
  { 149.841505044761f,    -31.9524129967228f},
  { -1.31300631802821f,    11.1793164384719f},
  { 84.9126177239528f,    -25.3965414160564f},
  { -109.575953113580f,    -53.7720444422919f},
  { -85.9669951010349f,    2.83751338514659f},
  { 65.6460848673365f,    63.4833127536594f},
  { 48.4189806846012f,    -8.21952951813733f},
  { -47.0590863443629f,    -39.2157096320986f},
  { 55.6352217120711f,    8.00246008797146f},
  { 112.686909230263f,    29.7820566125311f},
  { -16.9560201490481f,    7.50723653872003f},
  { 12.1200125071370f,    7.95180234835099f},
  { 135.907830137427f,    25.2905871681089f},
  { 148.421834182294f,    -60.0282132459340f},
  { -144.450199935376f,    3.60507288940253f},
  { -149.505575923146f,    45.7861265646663f},
  { 41.5230811121950f,    51.0190550805302f},
  { -140.324083781128f,    -12.9913367417341f},
  { 138.020504546488f,    6.08388687053162f},
  { -137.737925376106f,    14.7511416956518f},
  { -25.8539729129769f,    -2.47755771672502f},
  { 179.937429404765f,    -48.7809279839986f},
  { 74.9084460757705f,    13.8676921795039f},
  { -178.631283994059f,    18.3598920706058f},
  { 108.010615642601f,    20.4620187149798f},
  { -2.66498494179317f,    -32.3794914148608f},
  { -21.1433872602348f,    -11.6979132966246f},
  { -30.7363961652170f,    -53.0060664811199f},
  { 15.6349693429946f,    60.2080167597588f},
  { 65.8668436049860f,    -22.4117838350371f},
  { -80.2005772677395f,    -5.21863546706097f},
  { 73.7129833947903f,    -70.2090939724247f},
  { -25.6366898664446f,    56.6615370152244f},
  { 155.811380805170f,    -37.8476655192962f},
  { -34.0866912480834f,    32.9908659694550f},
  { -149.841505044761f,    31.9524129967228f},
  { 1.31300631802821f,    -11.1793164384719f},
  { -84.9126177239528f,    25.3965414160564f},
  { 109.575953113580f,    53.7720444422919f},
  { 85.9669951010349f,    -2.83751338514659f},
  { -65.6460848673365f,    -63.4833127536594f},
  { -48.4189806846012f,    8.21952951813733f},
  { 47.0590863443629f,    39.2157096320986f},
  { -55.6352217120711f,    -8.00246008797146f},
  { -112.686909230263f,    -29.7820566125311f},
  { 16.9560201490481f,    -7.50723653872003f},
  { -12.1200125071370f,    -7.95180234835099f},
  { -135.907830137427f,    -25.2905871681089f},
  { -148.421834182294f,    60.0282132459340f},
  { 144.450199935376f,    -3.60507288940253f},
  { 149.505575923146f,    -45.7861265646663f},
  { -41.5230811121950f,    -51.0190550805302f},
  { 140.324083781128f,    12.9913367417341f},
  { -138.020504546488f,    -6.08388687053162f},
  { 137.737925376106f,    -14.7511416956518f},
  { 25.8539729129769f,    2.47755771672502f},
  { -179.937429404765f,    48.7809279839986f},
  { -74.9084460757705f,    -13.8676921795039f},
  { 178.631283994059f,    -18.3598920706058f},
  { -108.010615642601f,    -20.4620187149798f},
  { 2.66498494179317f,    32.3794914148608f},
  { 21.1433872602348f,    11.6979132966246f},
  { -59.2636038347830f,    53.0060664811199f},
  { -105.634969342995f,    -60.2080167597588f},
  { -155.866843604986f,    22.4117838350371f},
  { -9.79942273226048f,    5.21863546706097f},
  { -163.712983394790f,    70.2090939724247f},
  { -64.3633101335554f,    -56.6615370152244f},
  { 114.188619194830f,    37.8476655192962f},
  { -55.9133087519166f,    -32.9908659694550f},
  { 59.8415050447607f,    -31.9524129967228f},
  { -91.3130063180282f,    11.1793164384719f},
  { -5.08738227604718f,    -25.3965414160564f},
  { 160.424046886420f,    -53.7720444422919f},
  { -175.966995101035f,    2.83751338514659f},
  { -24.3539151326635f,    63.4833127536594f},
  { -41.5810193153988f,    -8.21952951813733f},
  { -137.059086344363f,    -39.2157096320986f},
  { -34.3647782879289f,    8.00246008797146f},
  { 22.6869092302628f,    29.7820566125311f},
  { -106.956020149048f,    7.50723653872003f},
  { -77.8799874928630f,    7.95180234835099f},
  { 45.9078301374274f,    25.2905871681089f},
  { 58.4218341822938f,    -60.0282132459340f},
  { 125.549800064624f,    3.60507288940253f},
  { 120.494424076854f,    45.7861265646663f},
  { -48.4769188878050f,    51.0190550805302f},
  { 129.675916218872f,    -12.9913367417341f},
  { 48.0205045464884f,    6.08388687053162f},
  { 132.262074623894f,    14.7511416956518f},
  { -115.853972912977f,    -2.47755771672502f},
  { 89.9374294047648f,    -48.7809279839986f},
  { -15.0915539242296f,    13.8676921795039f},
  { 91.3687160059407f,    18.3598920706058f},
  { 18.0106156426008f,    20.4620187149798f},
  { -92.6649849417932f,    -32.3794914148608f},
  { -111.143387260235f,    -11.6979132966246f},
  { 120.736396165217f,    53.0060664811199f},
  { 74.3650306570054f,    -60.2080167597588f},
  { 24.1331563950140f,    22.4117838350371f},
  { 170.200577267740f,    5.21863546706097f},
  { 16.2870166052097f,    70.2090939724247f},
  { 115.636689866445f,    -56.6615370152244f},
  { -65.8113808051699f,    37.8476655192962f},
  { 124.086691248083f,    -32.9908659694550f},
  { -120.158494955239f,    -31.9524129967228f},
  { 88.6869936819718f,    11.1793164384719f},
  { 174.912617723953f,    -25.3965414160564f},
  { -19.5759531135796f,    -53.7720444422919f},
  { 4.03300489896510f,    2.83751338514659f},
  { 155.646084867337f,    63.4833127536594f},
  { 138.418980684601f,    -8.21952951813733f},
  { 42.9409136556371f,    -39.2157096320986f},
  { 145.635221712071f,    8.00246008797146f},
  { -157.313090769737f,    29.7820566125311f},
  { 73.0439798509519f,    7.50723653872003f},
  { 102.120012507137f,    7.95180234835099f},
  { -134.092169862573f,    25.2905871681089f},
  { -121.578165817706f,    -60.0282132459340f},
  { -54.4501999353762f,    3.60507288940253f},
  { -59.5055759231463f,    45.7861265646663f},
  { 131.523081112195f,    51.0190550805302f},
  { -50.3240837811279f,    -12.9913367417341f},
  { -131.979495453512f,    6.08388687053162f},
  { -47.7379253761058f,    14.7511416956518f},
  { 64.1460270870231f,    -2.47755771672502f},
  { -90.0625705952352f,    -48.7809279839986f},
  { 164.908446075770f,    13.8676921795039f},
  { -88.6312839940593f,    18.3598920706058f},
  { -161.989384357399f,    20.4620187149798f},
  { 87.3350150582068f,    -32.3794914148608f},
  { 68.8566127397652f,    -11.6979132966246f},
  { -57.0746028682579f,    17.9109112965780f},
  { 61.1304972047530f,    -7.69533758561870f},
  { -45.2478204672027f,    -57.5294033214029f},
  { -28.2195733683486f,    78.9096156385351f},
  { -84.2373548280526f,    -18.9656135432167f},
  { 59.3287478380547f,    13.7558255086613f},
  { -139.575658049569f,    -18.8777830371654f},
  { 38.0912863430153f,    28.0394087488386f},
  { 144.194864618196f,    25.2313946559004f},
  { -11.1821785425438f,    -1.28808805920167f},
  { 79.4203070813131f,    64.1321186392502f},
  { 103.791981915819f,    -33.8374380889844f},
  { -35.1735235982676f,    -85.0701615892237f},
  { -78.3735874184366f,    23.9996224086939f},
  { 12.2789704782094f,    47.7598767470431f},
  { 50.1441966577185f,    -34.5535588711217f},
  { -13.9855704625107f,    54.8275910830078f},
  { -123.978047452299f,    53.2033822144056f},
  { -7.84425901100403f,    -16.8063417598105f},
  { -8.13069166304214f,    12.0017323058903f},
  { -146.660251372639f,    38.9854284838043f},
  { 116.164925644707f,    15.1654552210621f},
  { -175.572023228908f,    -35.4688197817794f},
  { -129.974847555688f,    -20.7240614349903f},
  { -58.7898358130382f,    24.6465135229038f},
  { 163.313832609100f,    -38.4698531326131f},
  { -171.840669255116f,    41.6898001127167f},
  { -160.415805623081f,    -40.5685249452216f},
  { 2.75271948559983f,    -25.8280223666138f},
  { 131.219055081805f,    0.0412302558525259f},
  { -43.4770841272566f,    69.6141590079244f},
  { -161.635219548633f,    -1.29903163608235f},
  { -129.646807982952f,    62.9985615621742f},
  { 32.4075381831811f,    -2.25039932569429f},
  { 12.5164957316273f,    -20.6838680219578f},
  { 57.0746028682579f,    -17.9109112965780f},
  { -61.1304972047530f,    7.69533758561870f},
  { 45.2478204672027f,    57.5294033214029f},
  { 28.2195733683486f,    -78.9096156385351f},
  { 84.2373548280526f,    18.9656135432167f},
  { -59.3287478380547f,    -13.7558255086613f},
  { 139.575658049569f,    18.8777830371654f},
  { -38.0912863430153f,    -28.0394087488386f},
  { -144.194864618196f,    -25.2313946559004f},
  { 11.1821785425438f,    1.28808805920167f},
  { -79.4203070813131f,    -64.1321186392502f},
  { -103.791981915819f,    33.8374380889844f},
  { 35.1735235982676f,    85.0701615892237f},
  { 78.3735874184366f,    -23.9996224086939f},
  { -12.2789704782094f,    -47.7598767470431f},
  { -50.1441966577185f,    34.5535588711217f},
  { 13.9855704625107f,    -54.8275910830078f},
  { 123.978047452299f,    -53.2033822144056f},
  { 7.84425901100403f,    16.8063417598105f},
  { 8.13069166304214f,    -12.0017323058903f},
  { 146.660251372639f,    -38.9854284838043f},
  { -116.164925644707f,    -15.1654552210621f},
  { 175.572023228908f,    35.4688197817794f},
  { 129.974847555688f,    20.7240614349903f},
  { 58.7898358130382f,    -24.6465135229038f},
  { -163.313832609100f,    38.4698531326131f},
  { 171.840669255116f,    -41.6898001127167f},
  { 160.415805623081f,    40.5685249452216f},
  { -2.75271948559983f,    25.8280223666138f},
  { -131.219055081805f,    -0.0412302558525259f},
  { 43.4770841272566f,    -69.6141590079244f},
  { 161.635219548633f,    1.29903163608235f},
  { 129.646807982952f,    -62.9985615621742f},
  { -32.4075381831811f,    2.25039932569429f},
  { -12.5164957316273f,    20.6838680219578f},
  { -158.940881000204f,    -31.1448908269880f},
  { 8.77147291114150f,    -28.5854008636956f},
  { 114.318626174362f,    -22.2084036076950f},
  { -95.2954686654733f,    -9.75840459955865f},
  { 160.944986776130f,    -5.44883907181121f},
  { -15.8875472575573f,    -29.7014307878198f},
  { 152.195992810739f,    46.0805128313995f},
  { -40.8044836540265f,    -43.9994072574599f},
  { -38.8509546468490f,    47.1920617550663f},
  { 173.386359244104f,    -78.7448159660119f},
  { -64.5157288787217f,    -4.59463012694014f},
  { 34.6169703741920f,    11.4210789348729f},
  { 92.8445458023662f,    -4.02805202442754f},
  { -155.555972854102f,    -10.6090655964160f},
  { -79.0701208345658f,    -41.0609005548679f},
  { 41.8948927377216f,    -31.8574725173498f},
  { -99.6653353458516f,    -33.9841135805965f},
  { -121.810633481110f,    19.5575204637846f},
  { 114.316574970966f,    -71.5011245649553f},
  { -123.635193824889f,    -75.5369883084871f},
  { -124.178686227537f,    40.4950119904576f},
  { -16.8039105530620f,    25.1886426922989f},
  { 96.1846284900549f,    54.2918464978552f},
  { 153.723660047736f,    36.9330763410585f},
  { -151.788289824461f,    -28.0973806306467f},
  { 70.1321170497187f,    48.5875728654897f},
  { -99.0540578276146f,    47.6630864203711f},
  { 111.380835464924f,    45.6992947586867f},
  { 84.3335606918577f,    -64.0357211750186f},
  { -0.0548131884707396f,    41.2190420862713f},
  { -104.342952293500f,    -14.6416703213570f},
  { 175.883341945665f,    71.5909138286826f},
  { -111.422807699303f,    16.8390616567175f},
  { 4.19364860686896f,    -57.5229183609744f},
  { 60.1431682578916f,    -65.9667217175294f},
  { -21.0591189997961f,    31.1448908269880f},
  { 171.228527088859f,    28.5854008636956f},
  { 65.6813738256384f,    22.2084036076950f},
  { -84.7045313345268f,    9.75840459955865f},
  { 19.0550132238698f,    5.44883907181121f},
  { -164.112452742443f,    29.7014307878198f},
  { 27.8040071892610f,    -46.0805128313995f},
  { -139.195516345974f,    43.9994072574599f},
  { -141.149045353151f,    -47.1920617550663f},
  { 6.61364075589592f,    78.7448159660119f},
  { -115.484271121278f,    4.59463012694014f},
  { 145.383029625808f,    -11.4210789348729f},
  { 87.1554541976338f,    4.02805202442754f},
  { -24.4440271458979f,    10.6090655964160f},
  { -100.929879165434f,    41.0609005548679f},
  { 138.105107262278f,    31.8574725173498f},
  { -80.3346646541484f,    33.9841135805965f},
  { -58.1893665188901f,    -19.5575204637846f},
  { 65.6834250290337f,    71.5011245649553f},
  { -56.3648061751115f,    75.5369883084871f},
  { -55.8213137724635f,    -40.4950119904576f},
  { -163.196089446938f,    -25.1886426922989f},
  { 83.8153715099452f,    -54.2918464978552f},
  { 26.2763399522643f,    -36.9330763410585f},
  { -28.2117101755392f,    28.0973806306467f},
  { 109.867882950281f,    -48.5875728654897f},
  { -80.9459421723854f,    -47.6630864203711f},
  { 68.6191645350763f,    -45.6992947586867f},
  { 95.6664393081424f,    64.0357211750186f},
  { -179.945186811529f,    -41.2190420862713f},
  { -75.6570477065003f,    14.6416703213570f},
  { 4.11665805433526f,    -71.5909138286826f},
  { -68.5771923006971f,    -16.8390616567175f},
  { 175.806351393131f,    57.5229183609744f},
  { 119.856831742108f,    65.9667217175294f}};

/* Minimum T-design of degree 50 (1296 points) */
const float __Tdesign_degree_50_dirs_deg[1296][2] =
{ { 6.31750018785189f,    5.71545247841618f},
  { 9.33666137417181f,    0.764875955810381f},
  { 12.7366434704163f,    5.68658142485854f},
  { 15.2949620394255f,    0.0333725968431899f},
  { 17.9037567992250f,    7.39288805432145f},
  { 20.0145383651178f,    1.54090664969374f},
  { 17.5251570761378f,    12.7500898369804f},
  { 23.3660740997915f,    7.58337618937538f},
  { 25.2427126430236f,    1.44949797785652f},
  { 21.7325149417603f,    15.4969703568972f},
  { 26.3232402377679f,    11.3420388968073f},
  { 29.5127172272453f,    5.15038681388645f},
  { 26.4638989064965f,    18.2428054047352f},
  { 32.1994237734007f,    0.645828887361602f},
  { 32.8255254258234f,    9.45207104671435f},
  { 31.2765374113426f,    14.7605851234668f},
  { 27.2393828124950f,    24.4919598722384f},
  { 37.0067464683316f,    5.67319469384795f},
  { 37.8039482456071f,    0.211578413422824f},
  { 32.1675256474504f,    21.2033937416403f},
  { 36.7283958680624f,    16.6027707299358f},
  { 38.8165575377462f,    11.3206889735115f},
  { 34.2338114580427f,    27.3451993103093f},
  { 42.7221806617192f,    4.42815492425229f},
  { 38.0643720059691f,    22.5745832466480f},
  { 44.0399479534900f,    10.0771456172288f},
  { 42.6132955686271f,    16.7525752947323f},
  { 47.1140649940803f,    2.56161948787102f},
  { 44.3058607688132f,    22.1806218085542f},
  { 41.6941737998675f,    27.6814347266868f},
  { 49.0989023407907f,    10.2508584891005f},
  { 48.0164081682955f,    16.1958835812124f},
  { 51.8828613038781f,    5.84246705766425f},
  { 45.1152426096349f,    31.7947821823200f},
  { 49.0981004711244f,    25.6147724979923f},
  { 51.6557722489001f,    20.7251460716084f},
  { 54.0049846553049f,    14.9454498617205f},
  { 56.2717490234111f,    10.5495845833380f},
  { 51.6956193505213f,    30.4282286732420f},
  { 57.6601397157023f,    4.86945471142963f},
  { 56.5569775382357f,    24.5800546431214f},
  { 58.3971963902080f,    19.1974452067879f},
  { 62.2426248572200f,    9.90888883052971f},
  { 61.7491860990915f,    15.2099635536057f},
  { 63.1423170276076f,    3.90963003576066f},
  { 65.7329780396423f,    19.8351059344430f},
  { 67.6554935549511f,    10.7242893874799f},
  { 68.4409624396340f,    4.84946097246874f},
  { 69.1997547135081f,    15.9632840431089f},
  { 73.3731586388839f,    5.97265382890480f},
  { 73.7397506527837f,    11.8702529919968f},
  { 78.9600906047694f,    4.79287632116766f},
  { 79.0167878592892f,    10.7308534022379f},
  { 86.1003993367101f,    0.941320326013424f},
  { 5.75013893576308f,    -6.28596736411227f},
  { 0.775143746480118f,    -9.33582199237545f},
  { 5.82906101631532f,    -12.6729186634914f},
  { 0.0345980377159959f,    -15.2949593814911f},
  { 7.76463283854887f,    -17.7499507937437f},
  { 1.63989766580014f,    -20.0069914045751f},
  { 13.3489649253100f,    -17.0795632730803f},
  { 8.25189697987607f,    -23.1497498450146f},
  { 1.60244798458380f,    -25.2340688810457f},
  { 16.6194698858780f,    -20.9045698239642f},
  { 12.6142805124942f,    -25.7709637762776f},
  { 5.91321703533615f,    -29.3818529057923f},
  { 20.2133150251387f,    -25.0390330618349f},
  { 0.763199204949105f,    -32.1971317480184f},
  { 11.2061618761426f,    -32.3251244997213f},
  { 17.1333283897928f,    -30.1347928043513f},
  { 27.1296291213104f,    -24.6153423375053f},
  { 7.09111921936131f,    -36.7955110794301f},
  { 0.267781817068857f,    -37.8036451826794f},
  { 24.6216451421964f,    -29.7591139153385f},
  { 20.4061606876503f,    -34.9659927158395f},
  { 14.4097803056218f,    -37.9252811999644f},
  { 32.0263078244225f,    -29.9806104984563f},
  { 6.01736274754162f,    -42.5644347873680f},
  { 27.8360599546156f,    -34.7019151535163f},
  { 13.8862489772669f,    -43.1912324580756f},
  { 22.2455886418650f,    -40.4144409417626f},
  { 3.76119065271173f,    -47.0524578843549f},
  { 29.6706791479963f,    -40.3007054606959f},
  { 35.0899743588341f,    -36.0877537269908f},
  { 15.4402760103867f,    -48.0540926095389f},
  { 23.4708357716892f,    -45.5478998079337f},
  { 9.41234055365334f,    -51.5051093349784f},
  { 41.2973624112110f,    -37.0280853543408f},
  { 36.2125682608781f,    -42.9648408872558f},
  { 31.3788505457600f,    -47.1844026346915f},
  { 24.4266536670622f,    -51.4163813785196f},
  { 18.5416333565258f,    -54.8474909165491f},
  { 43.4588103116940f,    -42.5819270586039f},
  { 9.04879015157560f,    -57.3349762636390f},
  { 39.6923986886031f,    -49.3601633240716f},
  { 33.6018018419065f,    -53.5464733375333f},
  { 20.5603211773556f,    -60.6599421898286f},
  { 29.8729434756305f,    -58.2146527221248f},
  { 8.60214919610703f,    -62.8801970376911f},
  { 41.2725405778403f,    -59.0431744233908f},
  { 26.4810381695465f,    -65.3345680955193f},
  { 13.0010346392792f,    -67.9276564617882f},
  { 38.8524348607004f,    -63.9975877642549f},
  { 20.0841637024558f,    -72.3614991600447f},
  { 36.8948129890217f,    -69.9629169158151f},
  { 23.6467244931021f,    -77.9764340554490f},
  { 44.8474810043001f,    -74.6910173266745f},
  { 13.5822254675175f,    -85.9885662606724f},
  { 132.288284291487f,    81.4885588960931f},
  { 94.7042909675691f,    80.6323379034743f},
  { 114.306625502019f,    76.0707311615365f},
  { 90.1265126407422f,    74.7050024211839f},
  { 112.882953426825f,    70.6762638448625f},
  { 94.4940182128308f,    69.9286581905608f},
  { 126.922851637536f,    68.4459332864143f},
  { 108.556005638253f,    65.4999305467253f},
  { 93.3957013841290f,    64.7184286440470f},
  { 126.826508566610f,    63.5278607546529f},
  { 114.339094021825f,    61.4982250812636f},
  { 100.368719324282f,    60.0811568843395f},
  { 126.488336417225f,    58.2356503760772f},
  { 91.2118577418347f,    57.7947966537854f},
  { 107.072611674294f,    55.9875826638032f},
  { 116.907550985591f,    55.7374435239716f},
  { 134.864966365994f,    54.0081009249607f},
  { 99.3717754616855f,    52.6225130948174f},
  { 90.3451714584523f,    52.1955482039109f},
  { 126.079782661543f,    52.1098199033771f},
  { 116.500200830029f,    50.1804165363630f},
  { 107.712326587357f,    49.8178833648037f},
  { 132.590454172503f,    47.2534861883163f},
  { 96.5117703987492f,    47.0929405560488f},
  { 123.991928381362f,    46.6350359944150f},
  { 104.340362488158f,    45.0534039865584f},
  { 113.969921713854f,    44.8057592388772f},
  { 93.4940826230670f,    42.8327805761788f},
  { 120.271511275129f,    41.5033711197403f},
  { 128.262375692931f,    41.3945881367105f},
  { 103.455811148776f,    40.1135201030065f},
  { 111.342420447763f,    39.9688401152677f},
  { 97.4103235661173f,    37.8840017273009f},
  { 131.183110621447f,    36.8547713099170f},
  { 122.387642121263f,    36.1865484007360f},
  { 115.754138460121f,    35.4673404514309f},
  { 108.258836785273f,    34.5992892066947f},
  { 102.621763796510f,    33.0840688505261f},
  { 126.814355003156f,    32.3076388572271f},
  { 95.7578743135873f,    32.2090209525075f},
  { 118.730384219651f,    30.0772138863420f},
  { 112.235398627297f,    29.6622959610919f},
  { 101.166813666368f,    27.3084752395310f},
  { 107.152596737804f,    27.1777031823352f},
  { 94.3806062016010f,    26.7901812288672f},
  { 111.587537008777f,    22.7435373722770f},
  { 101.572258060176f,    21.9337711352895f},
  { 95.2123134646669f,    21.4780231505450f},
  { 107.013876299185f,    19.9632424259738f},
  { 96.2312677327183f,    16.5339859327585f},
  { 102.349961185568f,    15.9032151905555f},
  { 94.8828125811786f,    11.0008233036095f},
  { 100.926300780106f,    10.7888210685814f},
  { 90.9435043783131f,    3.89907357767865f},
  { -83.6824998121481f,    5.71545247841618f},
  { -80.6633386258282f,    0.764875955810381f},
  { -77.2633565295837f,    5.68658142485854f},
  { -74.7050379605745f,    0.0333725968431899f},
  { -72.0962432007750f,    7.39288805432145f},
  { -69.9854616348822f,    1.54090664969374f},
  { -72.4748429238622f,    12.7500898369804f},
  { -66.6339259002085f,    7.58337618937538f},
  { -64.7572873569764f,    1.44949797785652f},
  { -68.2674850582397f,    15.4969703568972f},
  { -63.6767597622321f,    11.3420388968073f},
  { -60.4872827727547f,    5.15038681388645f},
  { -63.5361010935035f,    18.2428054047352f},
  { -57.8005762265993f,    0.645828887361602f},
  { -57.1744745741766f,    9.45207104671435f},
  { -58.7234625886574f,    14.7605851234668f},
  { -62.7606171875050f,    24.4919598722384f},
  { -52.9932535316684f,    5.67319469384795f},
  { -52.1960517543929f,    0.211578413422824f},
  { -57.8324743525497f,    21.2033937416403f},
  { -53.2716041319376f,    16.6027707299358f},
  { -51.1834424622538f,    11.3206889735115f},
  { -55.7661885419573f,    27.3451993103093f},
  { -47.2778193382808f,    4.42815492425229f},
  { -51.9356279940309f,    22.5745832466480f},
  { -45.9600520465100f,    10.0771456172288f},
  { -47.3867044313730f,    16.7525752947323f},
  { -42.8859350059197f,    2.56161948787102f},
  { -45.6941392311868f,    22.1806218085542f},
  { -48.3058262001325f,    27.6814347266868f},
  { -40.9010976592093f,    10.2508584891005f},
  { -41.9835918317045f,    16.1958835812124f},
  { -38.1171386961219f,    5.84246705766425f},
  { -44.8847573903651f,    31.7947821823200f},
  { -40.9018995288756f,    25.6147724979923f},
  { -38.3442277510999f,    20.7251460716084f},
  { -35.9950153446952f,    14.9454498617205f},
  { -33.7282509765889f,    10.5495845833380f},
  { -38.3043806494787f,    30.4282286732420f},
  { -32.3398602842977f,    4.86945471142963f},
  { -33.4430224617643f,    24.5800546431214f},
  { -31.6028036097920f,    19.1974452067879f},
  { -27.7573751427801f,    9.90888883052971f},
  { -28.2508139009085f,    15.2099635536057f},
  { -26.8576829723924f,    3.90963003576066f},
  { -24.2670219603577f,    19.8351059344430f},
  { -22.3445064450489f,    10.7242893874799f},
  { -21.5590375603660f,    4.84946097246874f},
  { -20.8002452864919f,    15.9632840431089f},
  { -16.6268413611161f,    5.97265382890480f},
  { -16.2602493472163f,    11.8702529919968f},
  { -11.0399093952307f,    4.79287632116766f},
  { -10.9832121407108f,    10.7308534022379f},
  { -3.89960066328990f,    0.941320326013424f},
  { -5.75013893576308f,    6.28596736411227f},
  { -0.775143746480118f,    9.33582199237545f},
  { -5.82906101631532f,    12.6729186634914f},
  { -0.0345980377159959f,    15.2949593814911f},
  { -7.76463283854887f,    17.7499507937437f},
  { -1.63989766580014f,    20.0069914045751f},
  { -13.3489649253100f,    17.0795632730803f},
  { -8.25189697987607f,    23.1497498450146f},
  { -1.60244798458380f,    25.2340688810457f},
  { -16.6194698858780f,    20.9045698239642f},
  { -12.6142805124942f,    25.7709637762776f},
  { -5.91321703533615f,    29.3818529057923f},
  { -20.2133150251387f,    25.0390330618349f},
  { -0.763199204949105f,    32.1971317480184f},
  { -11.2061618761426f,    32.3251244997213f},
  { -17.1333283897928f,    30.1347928043513f},
  { -27.1296291213104f,    24.6153423375053f},
  { -7.09111921936131f,    36.7955110794301f},
  { -0.267781817068857f,    37.8036451826794f},
  { -24.6216451421964f,    29.7591139153385f},
  { -20.4061606876503f,    34.9659927158395f},
  { -14.4097803056218f,    37.9252811999644f},
  { -32.0263078244225f,    29.9806104984563f},
  { -6.01736274754162f,    42.5644347873680f},
  { -27.8360599546156f,    34.7019151535163f},
  { -13.8862489772669f,    43.1912324580756f},
  { -22.2455886418650f,    40.4144409417626f},
  { -3.76119065271173f,    47.0524578843549f},
  { -29.6706791479963f,    40.3007054606959f},
  { -35.0899743588341f,    36.0877537269908f},
  { -15.4402760103867f,    48.0540926095389f},
  { -23.4708357716892f,    45.5478998079337f},
  { -9.41234055365334f,    51.5051093349784f},
  { -41.2973624112110f,    37.0280853543408f},
  { -36.2125682608781f,    42.9648408872558f},
  { -31.3788505457600f,    47.1844026346915f},
  { -24.4266536670622f,    51.4163813785196f},
  { -18.5416333565258f,    54.8474909165491f},
  { -43.4588103116940f,    42.5819270586039f},
  { -9.04879015157560f,    57.3349762636390f},
  { -39.6923986886031f,    49.3601633240716f},
  { -33.6018018419065f,    53.5464733375333f},
  { -20.5603211773556f,    60.6599421898286f},
  { -29.8729434756305f,    58.2146527221248f},
  { -8.60214919610703f,    62.8801970376911f},
  { -41.2725405778403f,    59.0431744233908f},
  { -26.4810381695465f,    65.3345680955193f},
  { -13.0010346392792f,    67.9276564617882f},
  { -38.8524348607004f,    63.9975877642549f},
  { -20.0841637024558f,    72.3614991600447f},
  { -36.8948129890217f,    69.9629169158151f},
  { -23.6467244931021f,    77.9764340554490f},
  { -44.8474810043001f,    74.6910173266745f},
  { -13.5822254675175f,    85.9885662606724f},
  { 47.7117157085134f,    -81.4885588960931f},
  { 85.2957090324310f,    -80.6323379034743f},
  { 65.6933744979809f,    -76.0707311615365f},
  { 89.8734873592578f,    -74.7050024211839f},
  { 67.1170465731750f,    -70.6762638448625f},
  { 85.5059817871692f,    -69.9286581905608f},
  { 53.0771483624640f,    -68.4459332864143f},
  { 71.4439943617467f,    -65.4999305467253f},
  { 86.6042986158710f,    -64.7184286440470f},
  { 53.1734914333898f,    -63.5278607546529f},
  { 65.6609059781750f,    -61.4982250812636f},
  { 79.6312806757181f,    -60.0811568843395f},
  { 53.5116635827746f,    -58.2356503760772f},
  { 88.7881422581654f,    -57.7947966537854f},
  { 72.9273883257056f,    -55.9875826638032f},
  { 63.0924490144092f,    -55.7374435239716f},
  { 45.1350336340063f,    -54.0081009249607f},
  { 80.6282245383145f,    -52.6225130948174f},
  { 89.6548285415478f,    -52.1955482039109f},
  { 53.9202173384570f,    -52.1098199033771f},
  { 63.4997991699715f,    -50.1804165363630f},
  { 72.2876734126434f,    -49.8178833648037f},
  { 47.4095458274966f,    -47.2534861883163f},
  { 83.4882296012508f,    -47.0929405560488f},
  { 56.0080716186379f,    -46.6350359944150f},
  { 75.6596375118418f,    -45.0534039865584f},
  { 66.0300782861463f,    -44.8057592388772f},
  { 86.5059173769330f,    -42.8327805761788f},
  { 59.7284887248714f,    -41.5033711197403f},
  { 51.7376243070689f,    -41.3945881367105f},
  { 76.5441888512239f,    -40.1135201030065f},
  { 68.6575795522371f,    -39.9688401152677f},
  { 82.5896764338827f,    -37.8840017273009f},
  { 48.8168893785527f,    -36.8547713099170f},
  { 57.6123578787368f,    -36.1865484007360f},
  { 64.2458615398789f,    -35.4673404514309f},
  { 71.7411632147266f,    -34.5992892066947f},
  { 77.3782362034898f,    -33.0840688505261f},
  { 53.1856449968436f,    -32.3076388572271f},
  { 84.2421256864128f,    -32.2090209525075f},
  { 61.2696157803493f,    -30.0772138863420f},
  { 67.7646013727032f,    -29.6622959610919f},
  { 78.8331863336316f,    -27.3084752395310f},
  { 72.8474032621962f,    -27.1777031823352f},
  { 85.6193937983990f,    -26.7901812288672f},
  { 68.4124629912234f,    -22.7435373722770f},
  { 78.4277419398238f,    -21.9337711352895f},
  { 84.7876865353331f,    -21.4780231505450f},
  { 72.9861237008146f,    -19.9632424259738f},
  { 83.7687322672817f,    -16.5339859327585f},
  { 77.6500388144325f,    -15.9032151905555f},
  { 85.1171874188214f,    -11.0008233036095f},
  { 79.0736992198945f,    -10.7888210685814f},
  { 89.0564956216869f,    -3.89907357767865f},
  { 96.3175001878519f,    5.71545247841618f},
  { 99.3366613741718f,    0.764875955810381f},
  { 102.736643470416f,    5.68658142485854f},
  { 105.294962039426f,    0.0333725968431899f},
  { 107.903756799225f,    7.39288805432145f},
  { 110.014538365118f,    1.54090664969374f},
  { 107.525157076138f,    12.7500898369804f},
  { 113.366074099792f,    7.58337618937538f},
  { 115.242712643024f,    1.44949797785652f},
  { 111.732514941760f,    15.4969703568972f},
  { 116.323240237768f,    11.3420388968073f},
  { 119.512717227245f,    5.15038681388645f},
  { 116.463898906497f,    18.2428054047352f},
  { 122.199423773401f,    0.645828887361602f},
  { 122.825525425823f,    9.45207104671435f},
  { 121.276537411343f,    14.7605851234668f},
  { 117.239382812495f,    24.4919598722384f},
  { 127.006746468332f,    5.67319469384795f},
  { 127.803948245607f,    0.211578413422824f},
  { 122.167525647450f,    21.2033937416403f},
  { 126.728395868062f,    16.6027707299358f},
  { 128.816557537746f,    11.3206889735115f},
  { 124.233811458043f,    27.3451993103093f},
  { 132.722180661719f,    4.42815492425229f},
  { 128.064372005969f,    22.5745832466480f},
  { 134.039947953490f,    10.0771456172288f},
  { 132.613295568627f,    16.7525752947323f},
  { 137.114064994080f,    2.56161948787102f},
  { 134.305860768813f,    22.1806218085542f},
  { 131.694173799868f,    27.6814347266868f},
  { 139.098902340791f,    10.2508584891005f},
  { 138.016408168296f,    16.1958835812124f},
  { 141.882861303878f,    5.84246705766425f},
  { 135.115242609635f,    31.7947821823200f},
  { 139.098100471124f,    25.6147724979923f},
  { 141.655772248900f,    20.7251460716084f},
  { 144.004984655305f,    14.9454498617205f},
  { 146.271749023411f,    10.5495845833380f},
  { 141.695619350521f,    30.4282286732420f},
  { 147.660139715702f,    4.86945471142963f},
  { 146.556977538236f,    24.5800546431214f},
  { 148.397196390208f,    19.1974452067879f},
  { 152.242624857220f,    9.90888883052971f},
  { 151.749186099092f,    15.2099635536057f},
  { 153.142317027608f,    3.90963003576066f},
  { 155.732978039642f,    19.8351059344430f},
  { 157.655493554951f,    10.7242893874799f},
  { 158.440962439634f,    4.84946097246874f},
  { 159.199754713508f,    15.9632840431089f},
  { 163.373158638884f,    5.97265382890480f},
  { 163.739750652784f,    11.8702529919968f},
  { 168.960090604769f,    4.79287632116766f},
  { 169.016787859289f,    10.7308534022379f},
  { 176.100399336710f,    0.941320326013424f},
  { 42.2882842914866f,    81.4885588960931f},
  { 4.70429096756905f,    80.6323379034743f},
  { 24.3066255020191f,    76.0707311615365f},
  { 0.126512640742192f,    74.7050024211839f},
  { 22.8829534268250f,    70.6762638448625f},
  { 4.49401821283082f,    69.9286581905608f},
  { 36.9228516375360f,    68.4459332864143f},
  { 18.5560056382534f,    65.4999305467253f},
  { 3.39570138412897f,    64.7184286440470f},
  { 36.8265085666102f,    63.5278607546529f},
  { 24.3390940218250f,    61.4982250812636f},
  { 10.3687193242819f,    60.0811568843395f},
  { 36.4883364172254f,    58.2356503760772f},
  { 1.21185774183464f,    57.7947966537854f},
  { 17.0726116742944f,    55.9875826638032f},
  { 26.9075509855909f,    55.7374435239716f},
  { 44.8649663659937f,    54.0081009249607f},
  { 9.37177546168550f,    52.6225130948174f},
  { 0.345171458452253f,    52.1955482039109f},
  { 36.0797826615430f,    52.1098199033771f},
  { 26.5002008300285f,    50.1804165363630f},
  { 17.7123265873566f,    49.8178833648037f},
  { 42.5904541725034f,    47.2534861883163f},
  { 6.51177039874922f,    47.0929405560488f},
  { 33.9919283813621f,    46.6350359944150f},
  { 14.3403624881582f,    45.0534039865584f},
  { 23.9699217138537f,    44.8057592388772f},
  { 3.49408262306700f,    42.8327805761788f},
  { 30.2715112751286f,    41.5033711197403f},
  { 38.2623756929311f,    41.3945881367105f},
  { 13.4558111487761f,    40.1135201030065f},
  { 21.3424204477629f,    39.9688401152677f},
  { 7.41032356611733f,    37.8840017273009f},
  { 41.1831106214473f,    36.8547713099170f},
  { 32.3876421212632f,    36.1865484007360f},
  { 25.7541384601211f,    35.4673404514309f},
  { 18.2588367852734f,    34.5992892066947f},
  { 12.6217637965102f,    33.0840688505261f},
  { 36.8143550031564f,    32.3076388572271f},
  { 5.75787431358725f,    32.2090209525075f},
  { 28.7303842196507f,    30.0772138863420f},
  { 22.2353986272968f,    29.6622959610919f},
  { 11.1668136663684f,    27.3084752395310f},
  { 17.1525967378038f,    27.1777031823352f},
  { 4.38060620160102f,    26.7901812288672f},
  { 21.5875370087766f,    22.7435373722770f},
  { 11.5722580601762f,    21.9337711352895f},
  { 5.21231346466690f,    21.4780231505450f},
  { 17.0138762991854f,    19.9632424259738f},
  { 6.23126773271827f,    16.5339859327585f},
  { 12.3499611855675f,    15.9032151905555f},
  { 4.88281258117857f,    11.0008233036095f},
  { 10.9263007801055f,    10.7888210685814f},
  { 0.943504378313093f,    3.89907357767865f},
  { -42.2882842914866f,    -81.4885588960931f},
  { -4.70429096756905f,    -80.6323379034743f},
  { -24.3066255020191f,    -76.0707311615365f},
  { -0.126512640742192f,    -74.7050024211839f},
  { -22.8829534268250f,    -70.6762638448625f},
  { -4.49401821283082f,    -69.9286581905608f},
  { -36.9228516375360f,    -68.4459332864143f},
  { -18.5560056382534f,    -65.4999305467253f},
  { -3.39570138412897f,    -64.7184286440470f},
  { -36.8265085666102f,    -63.5278607546529f},
  { -24.3390940218250f,    -61.4982250812636f},
  { -10.3687193242819f,    -60.0811568843395f},
  { -36.4883364172254f,    -58.2356503760772f},
  { -1.21185774183464f,    -57.7947966537854f},
  { -17.0726116742944f,    -55.9875826638032f},
  { -26.9075509855909f,    -55.7374435239716f},
  { -44.8649663659937f,    -54.0081009249607f},
  { -9.37177546168550f,    -52.6225130948174f},
  { -0.345171458452253f,    -52.1955482039109f},
  { -36.0797826615430f,    -52.1098199033771f},
  { -26.5002008300285f,    -50.1804165363630f},
  { -17.7123265873566f,    -49.8178833648037f},
  { -42.5904541725034f,    -47.2534861883163f},
  { -6.51177039874922f,    -47.0929405560488f},
  { -33.9919283813621f,    -46.6350359944150f},
  { -14.3403624881582f,    -45.0534039865584f},
  { -23.9699217138537f,    -44.8057592388772f},
  { -3.49408262306700f,    -42.8327805761788f},
  { -30.2715112751286f,    -41.5033711197403f},
  { -38.2623756929311f,    -41.3945881367105f},
  { -13.4558111487761f,    -40.1135201030065f},
  { -21.3424204477629f,    -39.9688401152677f},
  { -7.41032356611733f,    -37.8840017273009f},
  { -41.1831106214473f,    -36.8547713099170f},
  { -32.3876421212632f,    -36.1865484007360f},
  { -25.7541384601211f,    -35.4673404514309f},
  { -18.2588367852734f,    -34.5992892066947f},
  { -12.6217637965102f,    -33.0840688505261f},
  { -36.8143550031564f,    -32.3076388572271f},
  { -5.75787431358725f,    -32.2090209525075f},
  { -28.7303842196507f,    -30.0772138863420f},
  { -22.2353986272968f,    -29.6622959610919f},
  { -11.1668136663684f,    -27.3084752395310f},
  { -17.1525967378038f,    -27.1777031823352f},
  { -4.38060620160102f,    -26.7901812288672f},
  { -21.5875370087766f,    -22.7435373722770f},
  { -11.5722580601762f,    -21.9337711352895f},
  { -5.21231346466690f,    -21.4780231505450f},
  { -17.0138762991854f,    -19.9632424259738f},
  { -6.23126773271827f,    -16.5339859327585f},
  { -12.3499611855675f,    -15.9032151905555f},
  { -4.88281258117857f,    -11.0008233036095f},
  { -10.9263007801055f,    -10.7888210685814f},
  { -0.943504378313093f,    -3.89907357767865f},
  { -137.711715708513f,    81.4885588960931f},
  { -175.295709032431f,    80.6323379034743f},
  { -155.693374497981f,    76.0707311615365f},
  { -179.873487359258f,    74.7050024211839f},
  { -157.117046573175f,    70.6762638448625f},
  { -175.505981787169f,    69.9286581905608f},
  { -143.077148362464f,    68.4459332864143f},
  { -161.443994361747f,    65.4999305467253f},
  { -176.604298615871f,    64.7184286440470f},
  { -143.173491433390f,    63.5278607546529f},
  { -155.660905978175f,    61.4982250812636f},
  { -169.631280675718f,    60.0811568843395f},
  { -143.511663582775f,    58.2356503760772f},
  { -178.788142258165f,    57.7947966537854f},
  { -162.927388325706f,    55.9875826638032f},
  { -153.092449014409f,    55.7374435239716f},
  { -135.135033634006f,    54.0081009249607f},
  { -170.628224538315f,    52.6225130948174f},
  { -179.654828541548f,    52.1955482039109f},
  { -143.920217338457f,    52.1098199033771f},
  { -153.499799169971f,    50.1804165363630f},
  { -162.287673412643f,    49.8178833648037f},
  { -137.409545827497f,    47.2534861883163f},
  { -173.488229601251f,    47.0929405560488f},
  { -146.008071618638f,    46.6350359944150f},
  { -165.659637511842f,    45.0534039865584f},
  { -156.030078286146f,    44.8057592388772f},
  { -176.505917376933f,    42.8327805761788f},
  { -149.728488724871f,    41.5033711197403f},
  { -141.737624307069f,    41.3945881367105f},
  { -166.544188851224f,    40.1135201030065f},
  { -158.657579552237f,    39.9688401152677f},
  { -172.589676433883f,    37.8840017273009f},
  { -138.816889378553f,    36.8547713099170f},
  { -147.612357878737f,    36.1865484007360f},
  { -154.245861539879f,    35.4673404514309f},
  { -161.741163214727f,    34.5992892066947f},
  { -167.378236203490f,    33.0840688505261f},
  { -143.185644996844f,    32.3076388572271f},
  { -174.242125686413f,    32.2090209525075f},
  { -151.269615780349f,    30.0772138863420f},
  { -157.764601372703f,    29.6622959610919f},
  { -168.833186333632f,    27.3084752395310f},
  { -162.847403262196f,    27.1777031823352f},
  { -175.619393798399f,    26.7901812288672f},
  { -158.412462991223f,    22.7435373722770f},
  { -168.427741939824f,    21.9337711352895f},
  { -174.787686535333f,    21.4780231505450f},
  { -162.986123700815f,    19.9632424259738f},
  { -173.768732267282f,    16.5339859327585f},
  { -167.650038814433f,    15.9032151905555f},
  { -175.117187418821f,    11.0008233036095f},
  { -169.073699219894f,    10.7888210685814f},
  { -179.056495621687f,    3.89907357767865f},
  { 137.711715708513f,    -81.4885588960931f},
  { 175.295709032431f,    -80.6323379034743f},
  { 155.693374497981f,    -76.0707311615365f},
  { 179.873487359258f,    -74.7050024211839f},
  { 157.117046573175f,    -70.6762638448625f},
  { 175.505981787169f,    -69.9286581905608f},
  { 143.077148362464f,    -68.4459332864143f},
  { 161.443994361747f,    -65.4999305467253f},
  { 176.604298615871f,    -64.7184286440470f},
  { 143.173491433390f,    -63.5278607546529f},
  { 155.660905978175f,    -61.4982250812636f},
  { 169.631280675718f,    -60.0811568843395f},
  { 143.511663582775f,    -58.2356503760772f},
  { 178.788142258165f,    -57.7947966537854f},
  { 162.927388325706f,    -55.9875826638032f},
  { 153.092449014409f,    -55.7374435239716f},
  { 135.135033634006f,    -54.0081009249607f},
  { 170.628224538315f,    -52.6225130948174f},
  { 179.654828541548f,    -52.1955482039109f},
  { 143.920217338457f,    -52.1098199033771f},
  { 153.499799169971f,    -50.1804165363630f},
  { 162.287673412643f,    -49.8178833648037f},
  { 137.409545827497f,    -47.2534861883163f},
  { 173.488229601251f,    -47.0929405560488f},
  { 146.008071618638f,    -46.6350359944150f},
  { 165.659637511842f,    -45.0534039865584f},
  { 156.030078286146f,    -44.8057592388772f},
  { 176.505917376933f,    -42.8327805761788f},
  { 149.728488724871f,    -41.5033711197403f},
  { 141.737624307069f,    -41.3945881367105f},
  { 166.544188851224f,    -40.1135201030065f},
  { 158.657579552237f,    -39.9688401152677f},
  { 172.589676433883f,    -37.8840017273009f},
  { 138.816889378553f,    -36.8547713099170f},
  { 147.612357878737f,    -36.1865484007360f},
  { 154.245861539879f,    -35.4673404514309f},
  { 161.741163214727f,    -34.5992892066947f},
  { 167.378236203490f,    -33.0840688505261f},
  { 143.185644996844f,    -32.3076388572271f},
  { 174.242125686413f,    -32.2090209525075f},
  { 151.269615780349f,    -30.0772138863420f},
  { 157.764601372703f,    -29.6622959610919f},
  { 168.833186333632f,    -27.3084752395310f},
  { 162.847403262196f,    -27.1777031823352f},
  { 175.619393798399f,    -26.7901812288672f},
  { 158.412462991223f,    -22.7435373722770f},
  { 168.427741939824f,    -21.9337711352895f},
  { 174.787686535333f,    -21.4780231505450f},
  { 162.986123700815f,    -19.9632424259738f},
  { 173.768732267282f,    -16.5339859327585f},
  { 167.650038814433f,    -15.9032151905555f},
  { 175.117187418821f,    -11.0008233036095f},
  { 169.073699219894f,    -10.7888210685814f},
  { 179.056495621687f,    -3.89907357767865f},
  { 84.2498610642369f,    6.28596736411227f},
  { 89.2248562535199f,    9.33582199237545f},
  { 84.1709389836847f,    12.6729186634914f},
  { 89.9654019622840f,    15.2949593814911f},
  { 82.2353671614511f,    17.7499507937437f},
  { 88.3601023341999f,    20.0069914045751f},
  { 76.6510350746900f,    17.0795632730803f},
  { 81.7481030201239f,    23.1497498450146f},
  { 88.3975520154162f,    25.2340688810457f},
  { 73.3805301141220f,    20.9045698239642f},
  { 77.3857194875059f,    25.7709637762776f},
  { 84.0867829646638f,    29.3818529057923f},
  { 69.7866849748613f,    25.0390330618349f},
  { 89.2368007950509f,    32.1971317480184f},
  { 78.7938381238574f,    32.3251244997213f},
  { 72.8666716102072f,    30.1347928043513f},
  { 62.8703708786896f,    24.6153423375053f},
  { 82.9088807806387f,    36.7955110794301f},
  { 89.7322181829312f,    37.8036451826794f},
  { 65.3783548578036f,    29.7591139153385f},
  { 69.5938393123497f,    34.9659927158395f},
  { 75.5902196943783f,    37.9252811999644f},
  { 57.9736921755776f,    29.9806104984563f},
  { 83.9826372524584f,    42.5644347873680f},
  { 62.1639400453844f,    34.7019151535163f},
  { 76.1137510227331f,    43.1912324580756f},
  { 67.7544113581351f,    40.4144409417626f},
  { 86.2388093472883f,    47.0524578843549f},
  { 60.3293208520037f,    40.3007054606959f},
  { 54.9100256411659f,    36.0877537269908f},
  { 74.5597239896133f,    48.0540926095389f},
  { 66.5291642283108f,    45.5478998079337f},
  { 80.5876594463467f,    51.5051093349784f},
  { 48.7026375887890f,    37.0280853543408f},
  { 53.7874317391219f,    42.9648408872558f},
  { 58.6211494542400f,    47.1844026346915f},
  { 65.5733463329378f,    51.4163813785196f},
  { 71.4583666434742f,    54.8474909165491f},
  { 46.5411896883060f,    42.5819270586039f},
  { 80.9512098484244f,    57.3349762636390f},
  { 50.3076013113969f,    49.3601633240716f},
  { 56.3981981580935f,    53.5464733375333f},
  { 69.4396788226444f,    60.6599421898286f},
  { 60.1270565243695f,    58.2146527221248f},
  { 81.3978508038930f,    62.8801970376911f},
  { 48.7274594221597f,    59.0431744233908f},
  { 63.5189618304535f,    65.3345680955193f},
  { 76.9989653607208f,    67.9276564617882f},
  { 51.1475651392996f,    63.9975877642549f},
  { 69.9158362975442f,    72.3614991600447f},
  { 53.1051870109783f,    69.9629169158151f},
  { 66.3532755068979f,    77.9764340554490f},
  { 45.1525189956999f,    74.6910173266745f},
  { 76.4177745324825f,    85.9885662606724f},
  { 95.7501389357631f,    -6.28596736411227f},
  { 90.7751437464801f,    -9.33582199237545f},
  { 95.8290610163153f,    -12.6729186634914f},
  { 90.0345980377160f,    -15.2949593814911f},
  { 97.7646328385489f,    -17.7499507937437f},
  { 91.6398976658002f,    -20.0069914045751f},
  { 103.348964925310f,    -17.0795632730803f},
  { 98.2518969798761f,    -23.1497498450146f},
  { 91.6024479845838f,    -25.2340688810457f},
  { 106.619469885878f,    -20.9045698239642f},
  { 102.614280512494f,    -25.7709637762776f},
  { 95.9132170353362f,    -29.3818529057923f},
  { 110.213315025139f,    -25.0390330618349f},
  { 90.7631992049491f,    -32.1971317480184f},
  { 101.206161876143f,    -32.3251244997213f},
  { 107.133328389793f,    -30.1347928043513f},
  { 117.129629121310f,    -24.6153423375053f},
  { 97.0911192193613f,    -36.7955110794301f},
  { 90.2677818170689f,    -37.8036451826794f},
  { 114.621645142196f,    -29.7591139153385f},
  { 110.406160687650f,    -34.9659927158395f},
  { 104.409780305622f,    -37.9252811999644f},
  { 122.026307824422f,    -29.9806104984563f},
  { 96.0173627475416f,    -42.5644347873680f},
  { 117.836059954616f,    -34.7019151535163f},
  { 103.886248977267f,    -43.1912324580756f},
  { 112.245588641865f,    -40.4144409417626f},
  { 93.7611906527118f,    -47.0524578843549f},
  { 119.670679147996f,    -40.3007054606959f},
  { 125.089974358834f,    -36.0877537269908f},
  { 105.440276010387f,    -48.0540926095389f},
  { 113.470835771689f,    -45.5478998079337f},
  { 99.4123405536534f,    -51.5051093349784f},
  { 131.297362411211f,    -37.0280853543408f},
  { 126.212568260878f,    -42.9648408872558f},
  { 121.378850545760f,    -47.1844026346915f},
  { 114.426653667062f,    -51.4163813785196f},
  { 108.541633356526f,    -54.8474909165491f},
  { 133.458810311694f,    -42.5819270586039f},
  { 99.0487901515756f,    -57.3349762636390f},
  { 129.692398688603f,    -49.3601633240716f},
  { 123.601801841906f,    -53.5464733375333f},
  { 110.560321177356f,    -60.6599421898286f},
  { 119.872943475631f,    -58.2146527221248f},
  { 98.6021491961070f,    -62.8801970376911f},
  { 131.272540577840f,    -59.0431744233908f},
  { 116.481038169546f,    -65.3345680955193f},
  { 103.001034639279f,    -67.9276564617882f},
  { 128.852434860700f,    -63.9975877642549f},
  { 110.084163702456f,    -72.3614991600447f},
  { 126.894812989022f,    -69.9629169158151f},
  { 113.646724493102f,    -77.9764340554490f},
  { 134.847481004300f,    -74.6910173266745f},
  { 103.582225467517f,    -85.9885662606724f},
  { -84.2498610642369f,    -6.28596736411227f},
  { -89.2248562535199f,    -9.33582199237545f},
  { -84.1709389836847f,    -12.6729186634914f},
  { -89.9654019622840f,    -15.2949593814911f},
  { -82.2353671614511f,    -17.7499507937437f},
  { -88.3601023341999f,    -20.0069914045751f},
  { -76.6510350746900f,    -17.0795632730803f},
  { -81.7481030201239f,    -23.1497498450146f},
  { -88.3975520154162f,    -25.2340688810457f},
  { -73.3805301141220f,    -20.9045698239642f},
  { -77.3857194875059f,    -25.7709637762776f},
  { -84.0867829646638f,    -29.3818529057923f},
  { -69.7866849748613f,    -25.0390330618349f},
  { -89.2368007950509f,    -32.1971317480184f},
  { -78.7938381238574f,    -32.3251244997213f},
  { -72.8666716102072f,    -30.1347928043513f},
  { -62.8703708786896f,    -24.6153423375053f},
  { -82.9088807806387f,    -36.7955110794301f},
  { -89.7322181829312f,    -37.8036451826794f},
  { -65.3783548578036f,    -29.7591139153385f},
  { -69.5938393123497f,    -34.9659927158395f},
  { -75.5902196943783f,    -37.9252811999644f},
  { -57.9736921755776f,    -29.9806104984563f},
  { -83.9826372524584f,    -42.5644347873680f},
  { -62.1639400453844f,    -34.7019151535163f},
  { -76.1137510227331f,    -43.1912324580756f},
  { -67.7544113581351f,    -40.4144409417626f},
  { -86.2388093472883f,    -47.0524578843549f},
  { -60.3293208520037f,    -40.3007054606959f},
  { -54.9100256411659f,    -36.0877537269908f},
  { -74.5597239896133f,    -48.0540926095389f},
  { -66.5291642283108f,    -45.5478998079337f},
  { -80.5876594463467f,    -51.5051093349784f},
  { -48.7026375887890f,    -37.0280853543408f},
  { -53.7874317391219f,    -42.9648408872558f},
  { -58.6211494542400f,    -47.1844026346915f},
  { -65.5733463329378f,    -51.4163813785196f},
  { -71.4583666434742f,    -54.8474909165491f},
  { -46.5411896883060f,    -42.5819270586039f},
  { -80.9512098484244f,    -57.3349762636390f},
  { -50.3076013113969f,    -49.3601633240716f},
  { -56.3981981580935f,    -53.5464733375333f},
  { -69.4396788226444f,    -60.6599421898286f},
  { -60.1270565243695f,    -58.2146527221248f},
  { -81.3978508038930f,    -62.8801970376911f},
  { -48.7274594221597f,    -59.0431744233908f},
  { -63.5189618304535f,    -65.3345680955193f},
  { -76.9989653607208f,    -67.9276564617882f},
  { -51.1475651392996f,    -63.9975877642549f},
  { -69.9158362975442f,    -72.3614991600447f},
  { -53.1051870109783f,    -69.9629169158151f},
  { -66.3532755068979f,    -77.9764340554490f},
  { -45.1525189956999f,    -74.6910173266745f},
  { -76.4177745324825f,    -85.9885662606724f},
  { -95.7501389357631f,    6.28596736411227f},
  { -90.7751437464801f,    9.33582199237545f},
  { -95.8290610163153f,    12.6729186634914f},
  { -90.0345980377160f,    15.2949593814911f},
  { -97.7646328385489f,    17.7499507937437f},
  { -91.6398976658002f,    20.0069914045751f},
  { -103.348964925310f,    17.0795632730803f},
  { -98.2518969798761f,    23.1497498450146f},
  { -91.6024479845838f,    25.2340688810457f},
  { -106.619469885878f,    20.9045698239642f},
  { -102.614280512494f,    25.7709637762776f},
  { -95.9132170353362f,    29.3818529057923f},
  { -110.213315025139f,    25.0390330618349f},
  { -90.7631992049491f,    32.1971317480184f},
  { -101.206161876143f,    32.3251244997213f},
  { -107.133328389793f,    30.1347928043513f},
  { -117.129629121310f,    24.6153423375053f},
  { -97.0911192193613f,    36.7955110794301f},
  { -90.2677818170689f,    37.8036451826794f},
  { -114.621645142196f,    29.7591139153385f},
  { -110.406160687650f,    34.9659927158395f},
  { -104.409780305622f,    37.9252811999644f},
  { -122.026307824422f,    29.9806104984563f},
  { -96.0173627475416f,    42.5644347873680f},
  { -117.836059954616f,    34.7019151535163f},
  { -103.886248977267f,    43.1912324580756f},
  { -112.245588641865f,    40.4144409417626f},
  { -93.7611906527118f,    47.0524578843549f},
  { -119.670679147996f,    40.3007054606959f},
  { -125.089974358834f,    36.0877537269908f},
  { -105.440276010387f,    48.0540926095389f},
  { -113.470835771689f,    45.5478998079337f},
  { -99.4123405536534f,    51.5051093349784f},
  { -131.297362411211f,    37.0280853543408f},
  { -126.212568260878f,    42.9648408872558f},
  { -121.378850545760f,    47.1844026346915f},
  { -114.426653667062f,    51.4163813785196f},
  { -108.541633356526f,    54.8474909165491f},
  { -133.458810311694f,    42.5819270586039f},
  { -99.0487901515756f,    57.3349762636390f},
  { -129.692398688603f,    49.3601633240716f},
  { -123.601801841906f,    53.5464733375333f},
  { -110.560321177356f,    60.6599421898286f},
  { -119.872943475631f,    58.2146527221248f},
  { -98.6021491961070f,    62.8801970376911f},
  { -131.272540577840f,    59.0431744233908f},
  { -116.481038169546f,    65.3345680955193f},
  { -103.001034639279f,    67.9276564617882f},
  { -128.852434860700f,    63.9975877642549f},
  { -110.084163702456f,    72.3614991600447f},
  { -126.894812989022f,    69.9629169158151f},
  { -113.646724493102f,    77.9764340554490f},
  { -134.847481004300f,    74.6910173266745f},
  { -103.582225467517f,    85.9885662606724f},
  { -6.31750018785189f,    -5.71545247841618f},
  { -9.33666137417181f,    -0.764875955810381f},
  { -12.7366434704163f,    -5.68658142485854f},
  { -15.2949620394255f,    -0.0333725968431899f},
  { -17.9037567992250f,    -7.39288805432145f},
  { -20.0145383651178f,    -1.54090664969374f},
  { -17.5251570761378f,    -12.7500898369804f},
  { -23.3660740997915f,    -7.58337618937538f},
  { -25.2427126430236f,    -1.44949797785652f},
  { -21.7325149417603f,    -15.4969703568972f},
  { -26.3232402377679f,    -11.3420388968073f},
  { -29.5127172272453f,    -5.15038681388645f},
  { -26.4638989064965f,    -18.2428054047352f},
  { -32.1994237734007f,    -0.645828887361602f},
  { -32.8255254258234f,    -9.45207104671435f},
  { -31.2765374113426f,    -14.7605851234668f},
  { -27.2393828124950f,    -24.4919598722384f},
  { -37.0067464683316f,    -5.67319469384795f},
  { -37.8039482456071f,    -0.211578413422824f},
  { -32.1675256474504f,    -21.2033937416403f},
  { -36.7283958680624f,    -16.6027707299358f},
  { -38.8165575377462f,    -11.3206889735115f},
  { -34.2338114580427f,    -27.3451993103093f},
  { -42.7221806617192f,    -4.42815492425229f},
  { -38.0643720059691f,    -22.5745832466480f},
  { -44.0399479534900f,    -10.0771456172288f},
  { -42.6132955686271f,    -16.7525752947323f},
  { -47.1140649940803f,    -2.56161948787102f},
  { -44.3058607688132f,    -22.1806218085542f},
  { -41.6941737998675f,    -27.6814347266868f},
  { -49.0989023407907f,    -10.2508584891005f},
  { -48.0164081682955f,    -16.1958835812124f},
  { -51.8828613038781f,    -5.84246705766425f},
  { -45.1152426096349f,    -31.7947821823200f},
  { -49.0981004711244f,    -25.6147724979923f},
  { -51.6557722489001f,    -20.7251460716084f},
  { -54.0049846553049f,    -14.9454498617205f},
  { -56.2717490234111f,    -10.5495845833380f},
  { -51.6956193505213f,    -30.4282286732420f},
  { -57.6601397157023f,    -4.86945471142963f},
  { -56.5569775382357f,    -24.5800546431214f},
  { -58.3971963902080f,    -19.1974452067879f},
  { -62.2426248572200f,    -9.90888883052971f},
  { -61.7491860990915f,    -15.2099635536057f},
  { -63.1423170276076f,    -3.90963003576066f},
  { -65.7329780396423f,    -19.8351059344430f},
  { -67.6554935549511f,    -10.7242893874799f},
  { -68.4409624396340f,    -4.84946097246874f},
  { -69.1997547135081f,    -15.9632840431089f},
  { -73.3731586388839f,    -5.97265382890480f},
  { -73.7397506527837f,    -11.8702529919968f},
  { -78.9600906047694f,    -4.79287632116766f},
  { -79.0167878592892f,    -10.7308534022379f},
  { -86.1003993367101f,    -0.941320326013424f},
  { 173.682499812148f,    -5.71545247841618f},
  { 170.663338625828f,    -0.764875955810381f},
  { 167.263356529584f,    -5.68658142485854f},
  { 164.705037960575f,    -0.0333725968431899f},
  { 162.096243200775f,    -7.39288805432145f},
  { 159.985461634882f,    -1.54090664969374f},
  { 162.474842923862f,    -12.7500898369804f},
  { 156.633925900209f,    -7.58337618937538f},
  { 154.757287356976f,    -1.44949797785652f},
  { 158.267485058240f,    -15.4969703568972f},
  { 153.676759762232f,    -11.3420388968073f},
  { 150.487282772755f,    -5.15038681388645f},
  { 153.536101093503f,    -18.2428054047352f},
  { 147.800576226599f,    -0.645828887361602f},
  { 147.174474574177f,    -9.45207104671435f},
  { 148.723462588657f,    -14.7605851234668f},
  { 152.760617187505f,    -24.4919598722384f},
  { 142.993253531668f,    -5.67319469384795f},
  { 142.196051754393f,    -0.211578413422824f},
  { 147.832474352550f,    -21.2033937416403f},
  { 143.271604131938f,    -16.6027707299358f},
  { 141.183442462254f,    -11.3206889735115f},
  { 145.766188541957f,    -27.3451993103093f},
  { 137.277819338281f,    -4.42815492425229f},
  { 141.935627994031f,    -22.5745832466480f},
  { 135.960052046510f,    -10.0771456172288f},
  { 137.386704431373f,    -16.7525752947323f},
  { 132.885935005920f,    -2.56161948787102f},
  { 135.694139231187f,    -22.1806218085542f},
  { 138.305826200133f,    -27.6814347266868f},
  { 130.901097659209f,    -10.2508584891005f},
  { 131.983591831704f,    -16.1958835812124f},
  { 128.117138696122f,    -5.84246705766425f},
  { 134.884757390365f,    -31.7947821823200f},
  { 130.901899528876f,    -25.6147724979923f},
  { 128.344227751100f,    -20.7251460716084f},
  { 125.995015344695f,    -14.9454498617205f},
  { 123.728250976589f,    -10.5495845833380f},
  { 128.304380649479f,    -30.4282286732420f},
  { 122.339860284298f,    -4.86945471142963f},
  { 123.443022461764f,    -24.5800546431214f},
  { 121.602803609792f,    -19.1974452067879f},
  { 117.757375142780f,    -9.90888883052971f},
  { 118.250813900909f,    -15.2099635536057f},
  { 116.857682972392f,    -3.90963003576066f},
  { 114.267021960358f,    -19.8351059344430f},
  { 112.344506445049f,    -10.7242893874799f},
  { 111.559037560366f,    -4.84946097246874f},
  { 110.800245286492f,    -15.9632840431089f},
  { 106.626841361116f,    -5.97265382890480f},
  { 106.260249347216f,    -11.8702529919968f},
  { 101.039909395231f,    -4.79287632116766f},
  { 100.983212140711f,    -10.7308534022379f},
  { 93.8996006632899f,    -0.941320326013424f},
  { -173.682499812148f,    5.71545247841618f},
  { -170.663338625828f,    0.764875955810381f},
  { -167.263356529584f,    5.68658142485854f},
  { -164.705037960575f,    0.0333725968431899f},
  { -162.096243200775f,    7.39288805432145f},
  { -159.985461634882f,    1.54090664969374f},
  { -162.474842923862f,    12.7500898369804f},
  { -156.633925900209f,    7.58337618937538f},
  { -154.757287356976f,    1.44949797785652f},
  { -158.267485058240f,    15.4969703568972f},
  { -153.676759762232f,    11.3420388968073f},
  { -150.487282772755f,    5.15038681388645f},
  { -153.536101093503f,    18.2428054047352f},
  { -147.800576226599f,    0.645828887361602f},
  { -147.174474574177f,    9.45207104671435f},
  { -148.723462588657f,    14.7605851234668f},
  { -152.760617187505f,    24.4919598722384f},
  { -142.993253531668f,    5.67319469384795f},
  { -142.196051754393f,    0.211578413422824f},
  { -147.832474352550f,    21.2033937416403f},
  { -143.271604131938f,    16.6027707299358f},
  { -141.183442462254f,    11.3206889735115f},
  { -145.766188541957f,    27.3451993103093f},
  { -137.277819338281f,    4.42815492425229f},
  { -141.935627994031f,    22.5745832466480f},
  { -135.960052046510f,    10.0771456172288f},
  { -137.386704431373f,    16.7525752947323f},
  { -132.885935005920f,    2.56161948787102f},
  { -135.694139231187f,    22.1806218085542f},
  { -138.305826200133f,    27.6814347266868f},
  { -130.901097659209f,    10.2508584891005f},
  { -131.983591831704f,    16.1958835812124f},
  { -128.117138696122f,    5.84246705766425f},
  { -134.884757390365f,    31.7947821823200f},
  { -130.901899528876f,    25.6147724979923f},
  { -128.344227751100f,    20.7251460716084f},
  { -125.995015344695f,    14.9454498617205f},
  { -123.728250976589f,    10.5495845833380f},
  { -128.304380649479f,    30.4282286732420f},
  { -122.339860284298f,    4.86945471142963f},
  { -123.443022461764f,    24.5800546431214f},
  { -121.602803609792f,    19.1974452067879f},
  { -117.757375142780f,    9.90888883052971f},
  { -118.250813900909f,    15.2099635536057f},
  { -116.857682972392f,    3.90963003576066f},
  { -114.267021960358f,    19.8351059344430f},
  { -112.344506445049f,    10.7242893874799f},
  { -111.559037560366f,    4.84946097246874f},
  { -110.800245286492f,    15.9632840431089f},
  { -106.626841361116f,    5.97265382890480f},
  { -106.260249347216f,    11.8702529919968f},
  { -101.039909395231f,    4.79287632116766f},
  { -100.983212140711f,    10.7308534022379f},
  { -93.8996006632899f,    0.941320326013424f},
  { 83.6824998121481f,    -5.71545247841618f},
  { 80.6633386258282f,    -0.764875955810381f},
  { 77.2633565295837f,    -5.68658142485854f},
  { 74.7050379605745f,    -0.0333725968431899f},
  { 72.0962432007750f,    -7.39288805432145f},
  { 69.9854616348822f,    -1.54090664969374f},
  { 72.4748429238622f,    -12.7500898369804f},
  { 66.6339259002085f,    -7.58337618937538f},
  { 64.7572873569764f,    -1.44949797785652f},
  { 68.2674850582397f,    -15.4969703568972f},
  { 63.6767597622321f,    -11.3420388968073f},
  { 60.4872827727547f,    -5.15038681388645f},
  { 63.5361010935035f,    -18.2428054047352f},
  { 57.8005762265993f,    -0.645828887361602f},
  { 57.1744745741766f,    -9.45207104671435f},
  { 58.7234625886574f,    -14.7605851234668f},
  { 62.7606171875050f,    -24.4919598722384f},
  { 52.9932535316684f,    -5.67319469384795f},
  { 52.1960517543929f,    -0.211578413422824f},
  { 57.8324743525497f,    -21.2033937416403f},
  { 53.2716041319376f,    -16.6027707299358f},
  { 51.1834424622538f,    -11.3206889735115f},
  { 55.7661885419573f,    -27.3451993103093f},
  { 47.2778193382808f,    -4.42815492425229f},
  { 51.9356279940309f,    -22.5745832466480f},
  { 45.9600520465100f,    -10.0771456172288f},
  { 47.3867044313730f,    -16.7525752947323f},
  { 42.8859350059197f,    -2.56161948787102f},
  { 45.6941392311868f,    -22.1806218085542f},
  { 48.3058262001325f,    -27.6814347266868f},
  { 40.9010976592093f,    -10.2508584891005f},
  { 41.9835918317045f,    -16.1958835812124f},
  { 38.1171386961219f,    -5.84246705766425f},
  { 44.8847573903651f,    -31.7947821823200f},
  { 40.9018995288756f,    -25.6147724979923f},
  { 38.3442277510999f,    -20.7251460716084f},
  { 35.9950153446952f,    -14.9454498617205f},
  { 33.7282509765889f,    -10.5495845833380f},
  { 38.3043806494787f,    -30.4282286732420f},
  { 32.3398602842977f,    -4.86945471142963f},
  { 33.4430224617643f,    -24.5800546431214f},
  { 31.6028036097920f,    -19.1974452067879f},
  { 27.7573751427801f,    -9.90888883052971f},
  { 28.2508139009085f,    -15.2099635536057f},
  { 26.8576829723924f,    -3.90963003576066f},
  { 24.2670219603577f,    -19.8351059344430f},
  { 22.3445064450489f,    -10.7242893874799f},
  { 21.5590375603660f,    -4.84946097246874f},
  { 20.8002452864919f,    -15.9632840431089f},
  { 16.6268413611161f,    -5.97265382890480f},
  { 16.2602493472163f,    -11.8702529919968f},
  { 11.0399093952307f,    -4.79287632116766f},
  { 10.9832121407108f,    -10.7308534022379f},
  { 3.89960066328990f,    -0.941320326013424f},
  { -96.3175001878519f,    -5.71545247841618f},
  { -99.3366613741718f,    -0.764875955810381f},
  { -102.736643470416f,    -5.68658142485854f},
  { -105.294962039426f,    -0.0333725968431899f},
  { -107.903756799225f,    -7.39288805432145f},
  { -110.014538365118f,    -1.54090664969374f},
  { -107.525157076138f,    -12.7500898369804f},
  { -113.366074099792f,    -7.58337618937538f},
  { -115.242712643024f,    -1.44949797785652f},
  { -111.732514941760f,    -15.4969703568972f},
  { -116.323240237768f,    -11.3420388968073f},
  { -119.512717227245f,    -5.15038681388645f},
  { -116.463898906497f,    -18.2428054047352f},
  { -122.199423773401f,    -0.645828887361602f},
  { -122.825525425823f,    -9.45207104671435f},
  { -121.276537411343f,    -14.7605851234668f},
  { -117.239382812495f,    -24.4919598722384f},
  { -127.006746468332f,    -5.67319469384795f},
  { -127.803948245607f,    -0.211578413422824f},
  { -122.167525647450f,    -21.2033937416403f},
  { -126.728395868062f,    -16.6027707299358f},
  { -128.816557537746f,    -11.3206889735115f},
  { -124.233811458043f,    -27.3451993103093f},
  { -132.722180661719f,    -4.42815492425229f},
  { -128.064372005969f,    -22.5745832466480f},
  { -134.039947953490f,    -10.0771456172288f},
  { -132.613295568627f,    -16.7525752947323f},
  { -137.114064994080f,    -2.56161948787102f},
  { -134.305860768813f,    -22.1806218085542f},
  { -131.694173799868f,    -27.6814347266868f},
  { -139.098902340791f,    -10.2508584891005f},
  { -138.016408168296f,    -16.1958835812124f},
  { -141.882861303878f,    -5.84246705766425f},
  { -135.115242609635f,    -31.7947821823200f},
  { -139.098100471124f,    -25.6147724979923f},
  { -141.655772248900f,    -20.7251460716084f},
  { -144.004984655305f,    -14.9454498617205f},
  { -146.271749023411f,    -10.5495845833380f},
  { -141.695619350521f,    -30.4282286732420f},
  { -147.660139715702f,    -4.86945471142963f},
  { -146.556977538236f,    -24.5800546431214f},
  { -148.397196390208f,    -19.1974452067879f},
  { -152.242624857220f,    -9.90888883052971f},
  { -151.749186099092f,    -15.2099635536057f},
  { -153.142317027608f,    -3.90963003576066f},
  { -155.732978039642f,    -19.8351059344430f},
  { -157.655493554951f,    -10.7242893874799f},
  { -158.440962439634f,    -4.84946097246874f},
  { -159.199754713508f,    -15.9632840431089f},
  { -163.373158638884f,    -5.97265382890480f},
  { -163.739750652784f,    -11.8702529919968f},
  { -168.960090604769f,    -4.79287632116766f},
  { -169.016787859289f,    -10.7308534022379f},
  { -176.100399336710f,    -0.941320326013424f},
  { 174.249861064237f,    6.28596736411227f},
  { 179.224856253520f,    9.33582199237545f},
  { 174.170938983685f,    12.6729186634914f},
  { 179.965401962284f,    15.2949593814911f},
  { 172.235367161451f,    17.7499507937437f},
  { 178.360102334200f,    20.0069914045751f},
  { 166.651035074690f,    17.0795632730803f},
  { 171.748103020124f,    23.1497498450146f},
  { 178.397552015416f,    25.2340688810457f},
  { 163.380530114122f,    20.9045698239642f},
  { 167.385719487506f,    25.7709637762776f},
  { 174.086782964664f,    29.3818529057923f},
  { 159.786684974861f,    25.0390330618349f},
  { 179.236800795051f,    32.1971317480184f},
  { 168.793838123857f,    32.3251244997213f},
  { 162.866671610207f,    30.1347928043513f},
  { 152.870370878690f,    24.6153423375053f},
  { 172.908880780639f,    36.7955110794301f},
  { 179.732218182931f,    37.8036451826794f},
  { 155.378354857804f,    29.7591139153385f},
  { 159.593839312350f,    34.9659927158395f},
  { 165.590219694378f,    37.9252811999644f},
  { 147.973692175578f,    29.9806104984563f},
  { 173.982637252458f,    42.5644347873680f},
  { 152.163940045384f,    34.7019151535163f},
  { 166.113751022733f,    43.1912324580756f},
  { 157.754411358135f,    40.4144409417626f},
  { 176.238809347288f,    47.0524578843549f},
  { 150.329320852004f,    40.3007054606959f},
  { 144.910025641166f,    36.0877537269908f},
  { 164.559723989613f,    48.0540926095389f},
  { 156.529164228311f,    45.5478998079337f},
  { 170.587659446347f,    51.5051093349784f},
  { 138.702637588789f,    37.0280853543408f},
  { 143.787431739122f,    42.9648408872558f},
  { 148.621149454240f,    47.1844026346915f},
  { 155.573346332938f,    51.4163813785196f},
  { 161.458366643474f,    54.8474909165491f},
  { 136.541189688306f,    42.5819270586039f},
  { 170.951209848424f,    57.3349762636390f},
  { 140.307601311397f,    49.3601633240716f},
  { 146.398198158094f,    53.5464733375333f},
  { 159.439678822644f,    60.6599421898286f},
  { 150.127056524370f,    58.2146527221248f},
  { 171.397850803893f,    62.8801970376911f},
  { 138.727459422160f,    59.0431744233908f},
  { 153.518961830454f,    65.3345680955193f},
  { 166.998965360721f,    67.9276564617882f},
  { 141.147565139300f,    63.9975877642549f},
  { 159.915836297544f,    72.3614991600447f},
  { 143.105187010978f,    69.9629169158151f},
  { 156.353275506898f,    77.9764340554490f},
  { 135.152518995700f,    74.6910173266745f},
  { 166.417774532483f,    85.9885662606724f},
  { -174.249861064237f,    -6.28596736411227f},
  { -179.224856253520f,    -9.33582199237545f},
  { -174.170938983685f,    -12.6729186634914f},
  { -179.965401962284f,    -15.2949593814911f},
  { -172.235367161451f,    -17.7499507937437f},
  { -178.360102334200f,    -20.0069914045751f},
  { -166.651035074690f,    -17.0795632730803f},
  { -171.748103020124f,    -23.1497498450146f},
  { -178.397552015416f,    -25.2340688810457f},
  { -163.380530114122f,    -20.9045698239642f},
  { -167.385719487506f,    -25.7709637762776f},
  { -174.086782964664f,    -29.3818529057923f},
  { -159.786684974861f,    -25.0390330618349f},
  { -179.236800795051f,    -32.1971317480184f},
  { -168.793838123857f,    -32.3251244997213f},
  { -162.866671610207f,    -30.1347928043513f},
  { -152.870370878690f,    -24.6153423375053f},
  { -172.908880780639f,    -36.7955110794301f},
  { -179.732218182931f,    -37.8036451826794f},
  { -155.378354857804f,    -29.7591139153385f},
  { -159.593839312350f,    -34.9659927158395f},
  { -165.590219694378f,    -37.9252811999644f},
  { -147.973692175578f,    -29.9806104984563f},
  { -173.982637252458f,    -42.5644347873680f},
  { -152.163940045384f,    -34.7019151535163f},
  { -166.113751022733f,    -43.1912324580756f},
  { -157.754411358135f,    -40.4144409417626f},
  { -176.238809347288f,    -47.0524578843549f},
  { -150.329320852004f,    -40.3007054606959f},
  { -144.910025641166f,    -36.0877537269908f},
  { -164.559723989613f,    -48.0540926095389f},
  { -156.529164228311f,    -45.5478998079337f},
  { -170.587659446347f,    -51.5051093349784f},
  { -138.702637588789f,    -37.0280853543408f},
  { -143.787431739122f,    -42.9648408872558f},
  { -148.621149454240f,    -47.1844026346915f},
  { -155.573346332938f,    -51.4163813785196f},
  { -161.458366643474f,    -54.8474909165491f},
  { -136.541189688306f,    -42.5819270586039f},
  { -170.951209848424f,    -57.3349762636390f},
  { -140.307601311397f,    -49.3601633240716f},
  { -146.398198158094f,    -53.5464733375333f},
  { -159.439678822644f,    -60.6599421898286f},
  { -150.127056524370f,    -58.2146527221248f},
  { -171.397850803893f,    -62.8801970376911f},
  { -138.727459422160f,    -59.0431744233908f},
  { -153.518961830454f,    -65.3345680955193f},
  { -166.998965360721f,    -67.9276564617882f},
  { -141.147565139300f,    -63.9975877642549f},
  { -159.915836297544f,    -72.3614991600447f},
  { -143.105187010978f,    -69.9629169158151f},
  { -156.353275506898f,    -77.9764340554490f},
  { -135.152518995700f,    -74.6910173266745f},
  { -166.417774532483f,    -85.9885662606724f},
  { -47.7117157085134f,    81.4885588960931f},
  { -85.2957090324310f,    80.6323379034743f},
  { -65.6933744979809f,    76.0707311615365f},
  { -89.8734873592578f,    74.7050024211839f},
  { -67.1170465731750f,    70.6762638448625f},
  { -85.5059817871692f,    69.9286581905608f},
  { -53.0771483624640f,    68.4459332864143f},
  { -71.4439943617467f,    65.4999305467253f},
  { -86.6042986158710f,    64.7184286440470f},
  { -53.1734914333898f,    63.5278607546529f},
  { -65.6609059781750f,    61.4982250812636f},
  { -79.6312806757181f,    60.0811568843395f},
  { -53.5116635827746f,    58.2356503760772f},
  { -88.7881422581654f,    57.7947966537854f},
  { -72.9273883257056f,    55.9875826638032f},
  { -63.0924490144092f,    55.7374435239716f},
  { -45.1350336340063f,    54.0081009249607f},
  { -80.6282245383145f,    52.6225130948174f},
  { -89.6548285415478f,    52.1955482039109f},
  { -53.9202173384570f,    52.1098199033771f},
  { -63.4997991699715f,    50.1804165363630f},
  { -72.2876734126434f,    49.8178833648037f},
  { -47.4095458274966f,    47.2534861883163f},
  { -83.4882296012508f,    47.0929405560488f},
  { -56.0080716186379f,    46.6350359944150f},
  { -75.6596375118418f,    45.0534039865584f},
  { -66.0300782861463f,    44.8057592388772f},
  { -86.5059173769330f,    42.8327805761788f},
  { -59.7284887248714f,    41.5033711197403f},
  { -51.7376243070689f,    41.3945881367105f},
  { -76.5441888512239f,    40.1135201030065f},
  { -68.6575795522371f,    39.9688401152677f},
  { -82.5896764338827f,    37.8840017273009f},
  { -48.8168893785527f,    36.8547713099170f},
  { -57.6123578787368f,    36.1865484007360f},
  { -64.2458615398789f,    35.4673404514309f},
  { -71.7411632147266f,    34.5992892066947f},
  { -77.3782362034898f,    33.0840688505261f},
  { -53.1856449968436f,    32.3076388572271f},
  { -84.2421256864128f,    32.2090209525075f},
  { -61.2696157803493f,    30.0772138863420f},
  { -67.7646013727032f,    29.6622959610919f},
  { -78.8331863336316f,    27.3084752395310f},
  { -72.8474032621962f,    27.1777031823352f},
  { -85.6193937983990f,    26.7901812288672f},
  { -68.4124629912234f,    22.7435373722770f},
  { -78.4277419398238f,    21.9337711352895f},
  { -84.7876865353331f,    21.4780231505450f},
  { -72.9861237008146f,    19.9632424259738f},
  { -83.7687322672817f,    16.5339859327585f},
  { -77.6500388144325f,    15.9032151905555f},
  { -85.1171874188214f,    11.0008233036095f},
  { -79.0736992198945f,    10.7888210685814f},
  { -89.0564956216869f,    3.89907357767865f},
  { -132.288284291487f,    -81.4885588960931f},
  { -94.7042909675691f,    -80.6323379034743f},
  { -114.306625502019f,    -76.0707311615365f},
  { -90.1265126407422f,    -74.7050024211839f},
  { -112.882953426825f,    -70.6762638448625f},
  { -94.4940182128308f,    -69.9286581905608f},
  { -126.922851637536f,    -68.4459332864143f},
  { -108.556005638253f,    -65.4999305467253f},
  { -93.3957013841290f,    -64.7184286440470f},
  { -126.826508566610f,    -63.5278607546529f},
  { -114.339094021825f,    -61.4982250812636f},
  { -100.368719324282f,    -60.0811568843395f},
  { -126.488336417225f,    -58.2356503760772f},
  { -91.2118577418347f,    -57.7947966537854f},
  { -107.072611674294f,    -55.9875826638032f},
  { -116.907550985591f,    -55.7374435239716f},
  { -134.864966365994f,    -54.0081009249607f},
  { -99.3717754616855f,    -52.6225130948174f},
  { -90.3451714584523f,    -52.1955482039109f},
  { -126.079782661543f,    -52.1098199033771f},
  { -116.500200830029f,    -50.1804165363630f},
  { -107.712326587357f,    -49.8178833648037f},
  { -132.590454172503f,    -47.2534861883163f},
  { -96.5117703987492f,    -47.0929405560488f},
  { -123.991928381362f,    -46.6350359944150f},
  { -104.340362488158f,    -45.0534039865584f},
  { -113.969921713854f,    -44.8057592388772f},
  { -93.4940826230670f,    -42.8327805761788f},
  { -120.271511275129f,    -41.5033711197403f},
  { -128.262375692931f,    -41.3945881367105f},
  { -103.455811148776f,    -40.1135201030065f},
  { -111.342420447763f,    -39.9688401152677f},
  { -97.4103235661173f,    -37.8840017273009f},
  { -131.183110621447f,    -36.8547713099170f},
  { -122.387642121263f,    -36.1865484007360f},
  { -115.754138460121f,    -35.4673404514309f},
  { -108.258836785273f,    -34.5992892066947f},
  { -102.621763796510f,    -33.0840688505261f},
  { -126.814355003156f,    -32.3076388572271f},
  { -95.7578743135873f,    -32.2090209525075f},
  { -118.730384219651f,    -30.0772138863420f},
  { -112.235398627297f,    -29.6622959610919f},
  { -101.166813666368f,    -27.3084752395310f},
  { -107.152596737804f,    -27.1777031823352f},
  { -94.3806062016010f,    -26.7901812288672f},
  { -111.587537008777f,    -22.7435373722770f},
  { -101.572258060176f,    -21.9337711352895f},
  { -95.2123134646669f,    -21.4780231505450f},
  { -107.013876299185f,    -19.9632424259738f},
  { -96.2312677327183f,    -16.5339859327585f},
  { -102.349961185568f,    -15.9032151905555f},
  { -94.8828125811786f,    -11.0008233036095f},
  { -100.926300780106f,    -10.7888210685814f},
  { -90.9435043783131f,    -3.89907357767865f}};

/* Minimum T-design of degree 100 (5100 points) */
const float __Tdesign_degree_100_dirs_deg[5100][2] =
{ { 135.112963634543f,    -10.9070121585005f},
  { -149.773699959762f,    -5.04659267603735f},
  { 76.2994903762475f,    -14.0988534742932f},
  { 59.5000934898624f,    10.5432336322085f},
  { -162.921783362287f,    28.3536664014027f},
  { 103.675994855384f,    -0.837283967857134f},
  { -169.599206311779f,    61.9087522222797f},
  { 141.523979356684f,    -4.29946141054661f},
  { -53.9122841415104f,    -15.9665329498015f},
  { -44.0665165059163f,    29.1604953548073f},
  { -118.650072877756f,    17.7268104061847f},
  { 140.624682189535f,    -62.2336942496416f},
  { -169.826934955866f,    74.0579617493018f},
  { -174.924574979239f,    -11.0455351926701f},
  { 45.2805865752663f,    61.2546950762206f},
  { -52.8398089485056f,    -34.1298689414046f},
  { 8.37903274878709f,    56.3151333517369f},
  { 15.0708229219679f,    -17.9772003446941f},
  { 39.6307406711233f,    10.0794925730507f},
  { -77.7927877665195f,    -20.2960272241656f},
  { 25.0032356032190f,    -1.95766547893131f},
  { 67.3904436399691f,    38.5957126032353f},
  { -74.3709844064322f,    1.48902498866737f},
  { 153.972693036858f,    68.0968933422460f},
  { 11.5374976072462f,    -50.9756729786897f},
  { 160.863428572808f,    14.3384241532246f},
  { -42.4124051947123f,    18.6753700967494f},
  { -50.9896057568714f,    -41.0940650650316f},
  { 133.085681837097f,    -3.31314533193323f},
  { 75.9449020542073f,    -43.5615361963055f},
  { -89.5550411564168f,    -3.71469119237678f},
  { -100.068996746674f,    73.6712496717014f},
  { -69.5653596993217f,    10.9683268071274f},
  { -170.624755540029f,    28.4569306408453f},
  { 66.3585862216109f,    72.2456930119684f},
  { -175.847799424660f,    -16.9563286710119f},
  { 130.988991916770f,    -38.8614598365319f},
  { -101.419721936179f,    -29.3199254220964f},
  { 63.4955514785026f,    23.0836641602584f},
  { 133.731958269579f,    -36.6270675845425f},
  { -57.1932154799566f,    61.8343721338794f},
  { 105.427284784398f,    -71.1024301432785f},
  { -138.449889902351f,    -27.6247112732190f},
  { -97.8248798600467f,    -14.6232335898561f},
  { 91.7674701043864f,    -72.5720085009507f},
  { 72.0700065226631f,    26.1891998542538f},
  { 29.2392869531603f,    -50.7415662161816f},
  { -2.27224820848086f,    -37.7145121186939f},
  { -10.5942415758875f,    39.8919382717409f},
  { -26.6671098109331f,    20.2402883700549f},
  { -129.146515556787f,    17.3340836582898f},
  { -5.65262753529062f,    28.0146205608293f},
  { -131.919786468572f,    -38.1445272797611f},
  { -121.501149831346f,    54.1895126313824f},
  { -13.8150315427633f,    -55.9576047613440f},
  { 176.930100537907f,    2.50879170313885f},
  { 159.004251392201f,    -2.11751059077362f},
  { 67.7979604218379f,    -8.72202083169227f},
  { -8.36084141559409f,    -9.76270698851306f},
  { -106.538998121625f,    13.8998248954034f},
  { 91.0132415244906f,    -39.5248038831165f},
  { 74.4112131154684f,    32.4214717402202f},
  { -125.941168669629f,    -19.4834760662258f},
  { -132.631558556291f,    -49.1418843662286f},
  { -107.416562328147f,    -27.8070106755734f},
  { 44.4386802972763f,    18.7809816688234f},
  { -107.771147931925f,    -49.6919987103964f},
  { 56.1106510588800f,    -52.0574257295187f},
  { -30.5666672946455f,    -12.0147049226057f},
  { -135.224599589030f,    -1.91022092330936f},
  { 118.364137654501f,    -39.4207409089961f},
  { 129.983477858289f,    -47.9922579006854f},
  { 35.9419894257653f,    -25.7874367133622f},
  { 25.1973899479258f,    -6.50716971976569f},
  { -111.980366719905f,    -62.8657993900750f},
  { 32.2783995368209f,    63.8049887243174f},
  { 94.8486812733326f,    9.70999246611946f},
  { 51.5266866907638f,    -58.1049566303861f},
  { 82.0282456283157f,    -28.2279321494500f},
  { 120.302105369087f,    42.2679848551773f},
  { -173.774373524572f,    -45.9864707634367f},
  { 9.04728606281999f,    -32.2763487639339f},
  { -55.1295322093835f,    9.75292004018489f},
  { -0.340334749165017f,    36.6980637731376f},
  { 137.706488232958f,    38.5434744401805f},
  { -121.794802754939f,    -45.9199736303281f},
  { 39.9904752703630f,    69.7314968813966f},
  { -143.548179590381f,    8.80462147655749f},
  { -80.7269128777346f,    -38.6283548682931f},
  { -33.4570118803930f,    40.9619390884210f},
  { -2.39685567397453f,    5.94712855224664f},
  { -131.080497338389f,    -26.6594457214366f},
  { 161.766142643688f,    -12.9481593901667f},
  { -163.558526948424f,    65.2958241542219f},
  { 22.4899864576418f,    -81.7243049279946f},
  { -9.63381088166517f,    -71.2947558564855f},
  { -162.971860301779f,    0.214626123421731f},
  { 68.3169441067463f,    25.5903546841355f},
  { 83.1486928677538f,    -7.80081914661730f},
  { -144.909926078873f,    39.0284928576167f},
  { -129.759893296427f,    -40.2435138095444f},
  { 79.8313379068002f,    -64.6242451504367f},
  { 143.254993570606f,    9.79807916300374f},
  { 142.823513086557f,    -32.9059072799982f},
  { 26.7859831743539f,    -4.26064584765240f},
  { 55.0732687852541f,    -6.31232097458445f},
  { -1.61961624577598f,    40.2373240229352f},
  { -151.789062848573f,    -12.2196640459080f},
  { -38.3530379793283f,    -72.7994566938388f},
  { 135.887964065363f,    44.3646900205413f},
  { 49.1336755390157f,    44.1748238509645f},
  { 77.6701218310649f,    46.2469582005938f},
  { 162.233874502056f,    28.3364425867084f},
  { -17.7307065066041f,    45.7291417300263f},
  { 174.728120089547f,    7.08633293927769f},
  { -64.1687257415648f,    -7.37354745945388f},
  { 129.739029533640f,    -20.7499865955426f},
  { 22.5699130960712f,    -0.575112206850396f},
  { 79.1334605367030f,    22.7580059982164f},
  { -121.844406401116f,    37.9862943157380f},
  { 135.738490844058f,    65.1320157280903f},
  { 69.3546131046436f,    6.50620672998477f},
  { 20.1901338686206f,    -43.0210634299778f},
  { -36.8272843749689f,    15.6957516072835f},
  { -105.742485506789f,    37.3404883108800f},
  { 10.2031639231906f,    -8.85366090244810f},
  { -27.6011253032353f,    -33.7935054042650f},
  { -87.2028797860356f,    28.9244097124329f},
  { -82.5565479685054f,    -65.0540856267293f},
  { 149.405575254485f,    42.5227835363354f},
  { 0.609760380237820f,    -3.01587678922753f},
  { 155.358627260151f,    -3.96822060608715f},
  { 20.4911758394502f,    72.6478050529700f},
  { -80.6466152971891f,    31.3830291585629f},
  { -87.8076831500616f,    -44.2158989742383f},
  { -74.0592158625642f,    8.69895120881935f},
  { 63.0664054461084f,    44.7289945617809f},
  { 124.992822723311f,    68.9264035354923f},
  { -79.1989252297070f,    -5.40273744949736f},
  { -152.280943003893f,    12.6797305403720f},
  { -19.9064021639194f,    61.8453691358413f},
  { 143.916241534221f,    3.26623893742798f},
  { 41.3209448290236f,    -2.63505381463942f},
  { 61.7727165999950f,    -11.3476574165480f},
  { -89.2116905637764f,    6.15341486095787f},
  { -174.755316964369f,    8.05090054598179f},
  { 167.563251083586f,    -33.3027001932534f},
  { 37.8032894253526f,    -12.9898206255174f},
  { 165.195321715403f,    25.9451245830986f},
  { 52.4505235594418f,    -6.26707108261434f},
  { 160.649298203185f,    -22.6722312605179f},
  { -36.3166637337512f,    -24.4157708029608f},
  { -33.3208549505161f,    -50.2572267096481f},
  { -159.365180279031f,    -38.8453775301901f},
  { 147.181809018358f,    63.7585346400210f},
  { 109.973941191429f,    -45.3450855123758f},
  { -27.7262282099788f,    -69.2348366197789f},
  { -21.3856843646275f,    6.74158116584303f},
  { -37.7982101741625f,    -18.9854526889250f},
  { -10.1215775716803f,    -48.0249442255521f},
  { 176.478994128782f,    -30.6474464336699f},
  { -110.827244855698f,    -37.8542425171815f},
  { 127.744368616112f,    27.0747794767833f},
  { -47.0247371879881f,    51.7551268128034f},
  { -48.9037331567324f,    -25.8059325740881f},
  { -121.791678182581f,    20.8876829672787f},
  { 9.30781036514151f,    -0.288033396543367f},
  { -6.17499173131088f,    -46.4511025777881f},
  { -5.11483505423029f,    -48.9650667088159f},
  { 177.128683590075f,    -6.60665390614924f},
  { -77.3821059344785f,    -54.4669065634202f},
  { 113.897544175288f,    44.1832198826274f},
  { 34.6781867784941f,    -60.3276979548377f},
  { 95.0910711205903f,    15.5094168762453f},
  { -2.09010887984729f,    -76.5749203879789f},
  { 29.9103343793705f,    -34.9007473264585f},
  { -45.4370441650275f,    -48.7607997191196f},
  { 132.845490233390f,    -16.3149624937994f},
  { 125.708789519767f,    55.7707122804385f},
  { 29.6210324408826f,    -60.8603198206624f},
  { 162.149272286382f,    76.5230371680387f},
  { -9.15187313289931f,    -40.3847574282767f},
  { -138.608363798244f,    13.4947797130046f},
  { -77.6392720799493f,    24.3602805461700f},
  { 115.470562788177f,    -69.5094146444459f},
  { -79.1129016841344f,    58.8666362641537f},
  { 102.999255401046f,    -7.29867179809849f},
  { -39.2113717754909f,    32.8521869950436f},
  { -113.514023551486f,    6.56800917648408f},
  { -113.623223030243f,    -74.0686307595768f},
  { 81.9588289618524f,    26.3232599158052f},
  { -143.912820987295f,    -5.89853630152955f},
  { 52.6309026198533f,    -9.25578332463549f},
  { -149.288302984237f,    -0.408454613737791f},
  { 152.459018799162f,    -74.3264487024600f},
  { -0.127595951816178f,    6.52755572233228f},
  { -145.821905838254f,    -68.1567091571640f},
  { -46.0322848671772f,    -5.67311817942373f},
  { -146.533739684724f,    -29.8442306997777f},
  { -117.312277649794f,    -7.71928345316452f},
  { -78.0603952371364f,    40.9215263427125f},
  { 123.247402118852f,    -59.6000972305949f},
  { -127.513573606240f,    5.06986479549042f},
  { 109.560502086974f,    -14.3958195729735f},
  { 168.879484614382f,    11.8915324246868f},
  { -90.6636835796201f,    1.15745289147311f},
  { 71.2690562757332f,    74.8689470968796f},
  { -9.21450957640742f,    18.5327196098818f},
  { 70.7028442158494f,    49.1746857529709f},
  { 131.545906778382f,    19.5399190296941f},
  { -59.3306070981310f,    13.2042610769474f},
  { -170.869477105216f,    -68.9043100959912f},
  { 174.178996949888f,    -40.0578380037105f},
  { 129.603028890642f,    -9.61175375194759f},
  { 119.127961616512f,    34.7649084815199f},
  { -76.5549118829935f,    38.7654439809148f},
  { 132.438511705005f,    2.95587972779214f},
  { -96.3069208259613f,    54.5696984075897f},
  { 54.4752344862251f,    -39.6485132228187f},
  { -6.59435142703094f,    21.1510632994500f},
  { 73.4212365574985f,    46.6190516350398f},
  { -123.484578612872f,    -74.9709623224838f},
  { -59.0007875168060f,    -66.7690478126826f},
  { 119.048791984731f,    -20.8012312574888f},
  { -69.1368956664917f,    -0.154261526442591f},
  { -149.391294702592f,    -19.4195696175534f},
  { 168.340268999175f,    -16.9400516162021f},
  { 90.1954200265675f,    27.0800666743258f},
  { 120.308668283937f,    -0.263934784114302f},
  { -65.3007533120038f,    26.8631466055043f},
  { 68.0083925934770f,    15.9884627635624f},
  { -64.5853190167752f,    -13.6356613674728f},
  { 85.7713234114753f,    -59.3341695762894f},
  { 165.365602186580f,    -82.5298575355086f},
  { -46.2199488323651f,    59.3578904921679f},
  { 149.353924679067f,    -4.98252277557722f},
  { -85.7034919243867f,    -62.1095277002024f},
  { 148.918708092873f,    -52.5090811030748f},
  { -116.401458752743f,    85.5775933596753f},
  { -111.492991794629f,    -13.7327190023880f},
  { -63.7576199958557f,    -19.7299889968305f},
  { -63.0759107646597f,    16.3720771026781f},
  { 89.9984649967640f,    37.1774274542115f},
  { 117.189169672889f,    37.2406858930086f},
  { -99.0202274086698f,    -42.4112112972201f},
  { -95.6862524209551f,    -54.2319226181052f},
  { -17.7700596064288f,    31.8413016321649f},
  { -9.85225828909682f,    56.3438477986082f},
  { 4.74502306235263f,    62.1570805224970f},
  { -40.6891956194858f,    13.5361475798077f},
  { 58.1086810282021f,    -10.7571443198955f},
  { -10.0692665510980f,    -21.4516408592173f},
  { 42.0916257731336f,    46.0075503880130f},
  { 77.0136631537621f,    51.8893074254159f},
  { -27.8138229860156f,    24.2637249102616f},
  { -168.824195388972f,    -1.89551322590027f},
  { 89.9615481615776f,    -42.1170394996486f},
  { 3.79215810892616f,    -38.2830589232788f},
  { 161.567093227159f,    -70.2683119004341f},
  { -165.072680439919f,    59.0310601298339f},
  { 6.05247747397108f,    32.6776339940352f},
  { -140.655633113947f,    -83.3956608313680f},
  { 25.6655514801939f,    10.0706577785609f},
  { 159.332381640469f,    29.7617036593562f},
  { -88.6007237815586f,    56.2281936402504f},
  { 134.282987159009f,    7.76547379335372f},
  { 133.407090344503f,    12.3612310283071f},
  { -127.767369822060f,    25.2322167823102f},
  { -149.051954662763f,    -37.8335365805038f},
  { 112.224552702438f,    -41.4824350497544f},
  { 58.3773367296224f,    -36.1652844612277f},
  { -135.697572696746f,    66.5168645780371f},
  { -166.798709484637f,    -10.1513786149524f},
  { 23.2110747853852f,    -36.4083262614618f},
  { 158.234311902221f,    11.4030199226861f},
  { 29.4559689986814f,    45.2804459416641f},
  { -139.225944902692f,    81.8242264840177f},
  { -32.3480695671567f,    48.4152460628433f},
  { -168.609735448037f,    8.81031539170456f},
  { -177.879871983754f,    -19.1764505638229f},
  { 81.5602406143841f,    -17.3333975924685f},
  { 46.3476396086641f,    46.0284975695521f},
  { 91.2826281842281f,    -66.4839737154617f},
  { -150.189650940582f,    -54.4217535403336f},
  { -98.1304207756545f,    -46.5737144954322f},
  { -71.9382313090495f,    -23.7138578729902f},
  { -141.537535317571f,    26.5066897550129f},
  { 122.339328298632f,    -24.1151924049031f},
  { 113.673215702073f,    11.6581605770965f},
  { -141.576331033818f,    -63.0375977550849f},
  { -68.5222052035487f,    53.5950034767582f},
  { -52.3837913007349f,    15.8288698932269f},
  { 40.4987514485807f,    33.2145194328305f},
  { -137.814584880280f,    10.3907520484615f},
  { -179.734081887537f,    43.2706646930475f},
  { -132.082495848615f,    52.8094539594383f},
  { 66.3928728824137f,    28.7484450397573f},
  { -34.2054498310080f,    -44.6327009854747f},
  { 126.927259184511f,    77.6452584179362f},
  { 75.2116963353449f,    57.0908516652165f},
  { 44.7239838201039f,    -68.1618521450301f},
  { -175.418679176699f,    3.24912865780537f},
  { -126.219719925823f,    -22.1490438705785f},
  { 10.7973586187035f,    -66.3767367536038f},
  { 40.0431367064759f,    -5.41063849273577f},
  { -3.86686164607098f,    -65.6479368281171f},
  { -33.5035255261223f,    19.8286430265157f},
  { -2.01290488072056f,    -40.5394798045575f},
  { 155.888147500342f,    18.8900312638985f},
  { -137.727839930450f,    -41.5660665295109f},
  { -38.6463750557554f,    2.75223296053804f},
  { 88.9113418835574f,    -34.2944117546512f},
  { -72.3733934461288f,    -34.8032913279942f},
  { -68.6049528955827f,    40.8672007033017f},
  { 177.174942376009f,    -19.7198244574299f},
  { -168.716332134351f,    23.7428658517588f},
  { 6.35694555728808f,    -18.1617679198804f},
  { -170.060889612043f,    14.6670767237319f},
  { -110.692503598551f,    34.4462371509779f},
  { -125.780192014100f,    13.6296588420586f},
  { 126.966130515393f,    13.3721060678769f},
  { -10.1774377602060f,    -53.6903989043287f},
  { 82.9491465629807f,    -11.8504348204251f},
  { 53.5396272549544f,    10.9994261757589f},
  { -78.1326756474701f,    -32.0075379439159f},
  { 159.888773800623f,    55.6301997689592f},
  { -98.1647367509331f,    -49.5083501966845f},
  { 63.0259803003666f,    4.77251182994046f},
  { -20.9140478318519f,    19.0720411281375f},
  { 79.9512956665237f,    -68.4817238967080f},
  { -151.499532171498f,    -47.3738213427314f},
  { -18.6469793924421f,    -52.8628694165051f},
  { -103.626943665293f,    -3.83933840447496f},
  { -93.8696237552820f,    9.05463468442693f},
  { -148.796642737677f,    -25.1449390379157f},
  { -44.0861000806496f,    -28.9070533940600f},
  { -80.3507220595006f,    0.529958226635486f},
  { 4.50814068749867f,    -32.7732969889126f},
  { -151.963619765890f,    -17.7243797336922f},
  { 39.3559418741735f,    15.4330450095360f},
  { -63.2491990233159f,    -30.4623973105923f},
  { 132.275275827336f,    42.7252754175174f},
  { 169.464292058744f,    -11.7459707840831f},
  { 25.9794969326115f,    -53.4397331430159f},
  { -29.9714303682253f,    -15.0054189310047f},
  { 93.1120403612788f,    17.5716279469446f},
  { 134.029755525491f,    -58.4387426510059f},
  { 16.1856697731825f,    9.44481407742668f},
  { 151.868283862515f,    -40.0695284547822f},
  { 8.47213864632326f,    7.04791768450433f},
  { -96.6226586691290f,    -77.7289008657874f},
  { -113.454910323281f,    -17.5923988007467f},
  { -62.0056269962903f,    51.7861346813342f},
  { -29.8172220834591f,    -18.1376195203615f},
  { 3.29177687659080f,    -9.09511828098826f},
  { -154.912281826063f,    13.2202611838159f},
  { 137.320288723167f,    48.1924930229303f},
  { 80.6421203474740f,    -23.0239538217940f},
  { -142.577177657855f,    30.0586950166840f},
  { 150.027191812917f,    -23.0914937270495f},
  { 56.7564169461136f,    -23.0891527918315f},
  { 108.778824938687f,    -42.4515517411109f},
  { -177.374545710002f,    -27.5889640319185f},
  { -1.94102485770550f,    -33.4569630074206f},
  { 18.7716865054043f,    18.4649545481562f},
  { 25.1034813127402f,    -28.0109149983383f},
  { -33.2994515967163f,    -47.2734490782859f},
  { 168.322519344254f,    -26.3249094626074f},
  { 166.604703206083f,    -4.01300593441310f},
  { -57.6590038076802f,    -41.0003192831975f},
  { 5.21915316661720f,    -57.4099403458448f},
  { 118.448536721084f,    -12.7177049986953f},
  { 44.6708406456625f,    -56.8168831376384f},
  { -169.855315543457f,    -29.3576141818601f},
  { -44.1507282876054f,    -51.4861448568760f},
  { 24.7520523930262f,    -56.4907252814227f},
  { -49.6217198349201f,    -35.6743184391846f},
  { -100.950455608386f,    28.2131443600712f},
  { 10.2697516648761f,    78.6220142607089f},
  { -165.004348286808f,    -54.5754470511006f},
  { -35.0745507928465f,    23.0464859965473f},
  { -168.230818158075f,    -14.6126284138008f},
  { -175.896889995860f,    22.2067900914894f},
  { 152.242280162677f,    -77.4169475249471f},
  { 63.3836366827181f,    54.3013683072767f},
  { 163.155648963993f,    70.4016544068807f},
  { 112.628806748259f,    -48.2129122837310f},
  { -149.936093100267f,    -31.6760999125778f},
  { -134.965247690182f,    -37.0292406739374f},
  { 67.1217288835171f,    13.1804473656160f},
  { -111.418520920663f,    23.0323278706626f},
  { -14.7395043062176f,    1.67754359765327f},
  { 144.964519424894f,    12.5293771941401f},
  { 67.0596034834402f,    55.5579521467858f},
  { 110.792157741237f,    16.6897874437738f},
  { 81.6013379653769f,    20.8480543580473f},
  { 116.388065999780f,    0.199520664095598f},
  { 130.812784247270f,    -23.1405240470593f},
  { 33.2557773901847f,    25.5676733854070f},
  { 95.5900667909122f,    66.2685780710084f},
  { 117.478011829817f,    22.0106922892138f},
  { 141.691782573907f,    -64.9125112955009f},
  { -76.0647168641906f,    -12.5626645297809f},
  { 63.7058550804950f,    -41.9867299279931f},
  { -163.997001848024f,    -10.7731207705523f},
  { -38.0157128215652f,    -41.9398944224156f},
  { 123.484683960126f,    33.1521948037556f},
  { 74.0093816116859f,    -64.7174271624996f},
  { 158.401532654571f,    -4.70073714019514f},
  { -17.4190388529285f,    39.4811668817950f},
  { 80.5893238799655f,    2.05959953438535f},
  { 60.3246938447940f,    5.65497446251534f},
  { 146.313918637624f,    -1.71143871307232f},
  { 156.286198137837f,    -34.6440375056665f},
  { -125.018377352044f,    49.9741188328188f},
  { 30.7577940368970f,    2.10454476083660f},
  { 77.9638490361842f,    2.82362063438452f},
  { -152.872442785706f,    22.2701295093799f},
  { -104.578763741866f,    -47.0720331345554f},
  { 11.1762469985188f,    45.5656544352614f},
  { 4.68087281729577f,    53.6220571354719f},
  { -55.6421190201258f,    -18.3503649952846f},
  { -38.0439573333798f,    51.2854250417478f},
  { 160.391959955208f,    0.590889931370865f},
  { -32.6959400355056f,    -15.7984645909217f},
  { -15.2724873060483f,    -44.0830949127842f},
  { -170.050539563649f,    -59.3946332432259f},
  { -14.4945951196825f,    13.2801298732065f},
  { 12.1891138784757f,    29.9317647781702f},
  { 118.554833204948f,    -57.2699820156663f},
  { -0.861711456092793f,    -13.6745062509686f},
  { 95.5040327019893f,    -27.5899207321931f},
  { -6.88983651974276f,    -51.3215487171266f},
  { -160.503502694453f,    34.4926208802630f},
  { 141.260801472763f,    38.8633228180360f},
  { 159.985640529669f,    -27.1739718092969f},
  { -71.5305124616388f,    -21.1074384523449f},
  { 92.8882518554905f,    -15.6848988992053f},
  { -114.380051711184f,    -77.8573598792618f},
  { 68.7065796129315f,    19.7788279590554f},
  { -139.618600537084f,    30.0006313598990f},
  { 84.4523641641444f,    33.2781782022872f},
  { -51.2940346465277f,    66.7025157444241f},
  { 15.5725385430083f,    49.3131717333017f},
  { -159.273957283885f,    11.4387148090826f},
  { -4.62341217878234f,    64.9251559757401f},
  { 40.8472218224914f,    17.4860115092893f},
  { 178.453833687025f,    15.6236031573140f},
  { 79.9946150612550f,    -19.5850919034989f},
  { -80.7921691057875f,    38.0921168176831f},
  { 37.9445320226825f,    -66.2499444805101f},
  { 153.382565639199f,    44.3817617537126f},
  { -131.698308070262f,    28.3191133457150f},
  { -4.53207320431034f,    -42.9961766589098f},
  { -44.4317108389353f,    10.1357540200865f},
  { -176.285197100644f,    0.444023983564376f},
  { 106.090363023090f,    -2.81746779460127f},
  { 168.314740631877f,    20.0451644263499f},
  { 106.728805503965f,    -60.1615879529241f},
  { 73.6528887127457f,    7.02386391992281f},
  { -103.358976269811f,    -72.5571967965817f},
  { -46.8699802494892f,    -30.7129689203005f},
  { -150.188398678269f,    -9.94085139126176f},
  { 25.4662926725902f,    24.2385196867934f},
  { -45.8135525463208f,    -33.6956561480977f},
  { 114.228911688849f,    14.8177439793580f},
  { -71.7373878982309f,    -4.94251887542312f},
  { -141.725877298316f,    -41.5339575353950f},
  { -165.245297206856f,    -7.56989697482882f},
  { -72.5797984341638f,    -0.529293144238883f},
  { 27.3359144056700f,    16.0365699754986f},
  { -68.2377740608023f,    33.5173949541811f},
  { -92.9350669218194f,    52.2272677709710f},
  { 102.404580909068f,    48.9534777937647f},
  { 140.594402149581f,    56.9765468818325f},
  { 158.077406530693f,    -37.0587839998051f},
  { 100.488526829726f,    61.4661289807893f},
  { -133.454130675467f,    -31.6966929725523f},
  { 121.598715156472f,    -17.8019145549491f},
  { -99.1641101526173f,    32.9298584551120f},
  { 39.2878629033924f,    -86.0361264373848f},
  { -5.89174087868414f,    -68.9025410790662f},
  { -9.40844491776371f,    21.9318743602209f},
  { -130.200992184776f,    77.1728671427286f},
  { 165.524980359325f,    -16.0414099942838f},
  { -39.5292022547424f,    -0.781546484841279f},
  { 33.4009002698567f,    13.1112992066295f},
  { -156.395262932044f,    -33.5967227211758f},
  { -122.471185039887f,    -26.2998836066371f},
  { -151.068927171870f,    -15.3522589503249f},
  { 25.9056292446808f,    42.5306161497219f},
  { -128.932094046807f,    -11.3874706095592f},
  { -57.0882944767236f,    20.0504007601108f},
  { -157.290633451718f,    57.3706505700685f},
  { -177.288907194553f,    -45.1925179357727f},
  { -43.0505510551844f,    -21.5304768822415f},
  { -55.3902923594091f,    -25.4691819102283f},
  { -39.4587748625094f,    46.8014419947198f},
  { -14.9982668156435f,    64.0291737990266f},
  { -115.418389562979f,    -9.82876769247620f},
  { 75.2799375599416f,    21.9137791868593f},
  { 9.74419802716439f,    -4.77905668014385f},
  { -64.0238627228055f,    19.1912392224307f},
  { -28.4604076960682f,    7.01859833587053f},
  { 46.4716711427444f,    -21.9240660220047f},
  { -95.9812083145331f,    -43.6879621991246f},
  { -76.0192230026089f,    56.6120488567637f},
  { 168.167640181396f,    34.2954326000185f},
  { 90.4566139455999f,    53.3005800809702f},
  { 49.8131879156086f,    -35.3487204808272f},
  { -129.456952040756f,    -21.5017538617150f},
  { 76.6492352948576f,    15.3913250208690f},
  { 165.388516233946f,    -52.6437574909619f},
  { -141.003551679550f,    7.23184571800371f},
  { 122.418098889030f,    39.0514087346796f},
  { 111.873577988527f,    83.5896485002506f},
  { -146.334339001816f,    -35.9628016251027f},
  { -36.3083542342691f,    -67.7646464173432f},
  { 97.4185695302239f,    -23.6309343558319f},
  { -86.8154487378793f,    7.64238112779267f},
  { -93.2430322518040f,    18.4318830036241f},
  { 179.267127304330f,    -72.9705478164780f},
  { 27.2656766599496f,    19.4648625788881f},
  { -7.85622666422132f,    6.78760475917541f},
  { 125.343347945195f,    -39.4676908478457f},
  { -132.247948329354f,    -29.2215996906535f},
  { -64.9737269957331f,    4.33911453177047f},
  { 16.1016003679269f,    -52.1498077435459f},
  { -46.9590726843577f,    -41.9857673596353f},
  { -9.38682417061308f,    62.9020417788055f},
  { -7.68393451420386f,    34.6845217202594f},
  { 91.9130603737003f,    49.7356328172613f},
  { -155.386046047939f,    -59.4569820928223f},
  { -100.873216760438f,    13.4090787188007f},
  { 54.5615452196666f,    -30.8830006753207f},
  { 52.1049236141293f,    27.9870804214778f},
  { 46.9144476824353f,    8.49214325422109f},
  { 60.4975265324781f,    -56.9505318303055f},
  { 106.535594697110f,    41.6740891733659f},
  { 53.5315516821509f,    -81.1757779120381f},
  { -171.818446299325f,    25.6021293919061f},
  { -26.2292971292566f,    -36.7142333927591f},
  { -1.49813639786544f,    67.4231436311476f},
  { 23.1301627597884f,    10.0114476020334f},
  { 137.408777044242f,    -24.5724445130511f},
  { 72.0737908622944f,    -17.5276928029669f},
  { 6.94847890057156f,    20.5064153779646f},
  { -69.7035400301462f,    43.3283570410209f},
  { 154.882320506543f,    50.4076945751962f},
  { 141.598085157580f,    -12.4570104343930f},
  { -41.3267938193239f,    76.5218626422000f},
  { -124.693270924213f,    47.4315523343345f},
  { 151.046666694894f,    2.44797575526281f},
  { -114.761000328018f,    3.13210574865185f},
  { 60.9703126136592f,    -39.3762987946713f},
  { -78.5803178985455f,    86.9231546870455f},
  { -9.44626694234193f,    -65.0608604062709f},
  { 83.7576635988502f,    16.2228122151928f},
  { 148.275001910022f,    7.97534228909585f},
  { -135.763132319245f,    1.57108452063893f},
  { 170.958800845519f,    15.7526171208103f},
  { 48.0125968225912f,    18.7718584095748f},
  { 72.4802117736062f,    -11.8998818678077f},
  { -174.500412074797f,    10.7525179486425f},
  { 154.186796387022f,    -59.7289071010347f},
  { 100.327090289926f,    26.3376124240628f},
  { 5.53444289956598f,    -53.7887245961459f},
  { -3.98720260338553f,    48.6101930163952f},
  { -12.8314567997931f,    27.6277009146372f},
  { 173.846007144778f,    0.783767264503392f},
  { 122.872326586518f,    -80.4009205719345f},
  { -71.8496558701181f,    -54.7031471375908f},
  { -20.6235353753141f,    50.3449698428696f},
  { 62.2924746137856f,    -60.3845694113033f},
  { -7.88620574653739f,    37.2867298418667f},
  { -51.5786917069760f,    -60.5282670494810f},
  { -142.530182173305f,    47.1972487303289f},
  { -114.548704258040f,    32.2923148985652f},
  { -113.634188526224f,    -46.7921897392182f},
  { 75.0415462603963f,    -21.8133853873883f},
  { -47.1203853432278f,    -13.8914614742006f},
  { -100.004460409058f,    18.2902363364927f},
  { 162.038560434909f,    67.6244762744038f},
  { -150.692173504310f,    48.3475185135689f},
  { -98.9838914295035f,    41.1778404983141f},
  { -84.0824115889269f,    -59.1705074113635f},
  { -140.255393507376f,    -16.3040362311791f},
  { 32.8801551463088f,    -33.0285706666262f},
  { 119.971132810800f,    24.9595067046181f},
  { -147.313998283850f,    36.2829582706797f},
  { 155.821019982721f,    -29.4859112072366f},
  { -1.78030213360078f,    80.6877733100478f},
  { -95.8382252510577f,    43.2337163318954f},
  { -94.4369352644656f,    40.8366585812746f},
  { -66.6114784040319f,    -82.7989059125011f},
  { -124.873288219046f,    7.29373268220492f},
  { 46.7496707114961f,    -16.8881865235000f},
  { -72.0389485261920f,    24.0228145991868f},
  { 15.5678717026861f,    -4.90522458407042f},
  { -90.4987778395565f,    13.4159810169254f},
  { -54.4439653311543f,    45.3087408610681f},
  { -121.988509494770f,    27.5523328282382f},
  { -21.7628574301895f,    -40.7409754806145f},
  { 61.0820920606410f,    -19.1665682121696f},
  { -74.5944202832474f,    25.0433519362724f},
  { 85.7984299255685f,    -12.8168202594319f},
  { -100.591213223654f,    48.7651641611959f},
  { 160.051710505998f,    -46.8440961812710f},
  { 155.130887454222f,    11.2451628345406f},
  { -71.3568195610080f,    -8.65819569512514f},
  { 120.674774559137f,    5.60409567495968f},
  { -7.48848316534340f,    -12.8921053093270f},
  { 134.391936053632f,    40.6098268184209f},
  { 172.843125683288f,    -23.3505065986695f},
  { -104.656241005574f,    -6.31485645641428f},
  { 26.5489442453971f,    7.20267264318446f},
  { -170.051442970705f,    -53.4986707453079f},
  { -11.5880202028527f,    36.8015250259757f},
  { -179.200273635581f,    -59.2858521883024f},
  { -82.6077543945760f,    -13.8595123680490f},
  { 91.1149851338603f,    83.4712027190460f},
  { -102.690380569431f,    -17.9273723726398f},
  { -172.141516366321f,    43.6976310463691f},
  { -133.865351629765f,    -46.3507622731438f},
  { -171.326117219647f,    -27.0444739071031f},
  { 138.457333386491f,    8.99338621861224f},
  { -63.8647133327651f,    -16.1071707647558f},
  { 173.618202354631f,    -37.3416834334259f},
  { -15.2376192499825f,    -18.9225482580957f},
  { 47.5127985742410f,    -73.7738115647869f},
  { 178.842469473749f,    -0.663548155160963f},
  { 75.6363129159631f,    4.80831175072516f},
  { 115.532634996254f,    69.3752118427466f},
  { 50.8058790086430f,    12.4768309397115f},
  { 25.3703029806357f,    -38.6839648047261f},
  { 164.741977630714f,    29.7751720913189f},
  { -93.5031252674226f,    -20.8158861212099f},
  { -83.1221348028995f,    -49.5920610875628f},
  { -12.3962718134035f,    -38.9407106035933f},
  { 38.4706315629602f,    -23.5703073598212f},
  { 140.454986841371f,    10.4451589954151f},
  { 4.00228133116523f,    -42.3688490538350f},
  { 125.266166505277f,    -3.65129060305555f},
  { -45.5174486940331f,    26.5767779026310f},
  { 106.532198542432f,    67.8898727553161f},
  { 47.8316932919170f,    11.3019720644799f},
  { -102.622123075860f,    -8.22517253582149f},
  { -110.200267425440f,    11.7209972970157f},
  { -23.4874942234784f,    -26.9944942821933f},
  { -179.834914534455f,    20.8630251873377f},
  { -145.302128945304f,    -54.2624319034759f},
  { -56.4349974180523f,    -69.5324822064906f},
  { 27.0802017552490f,    -0.173996300641060f},
  { -0.305720228296931f,    -30.3083129268004f},
  { 150.859174657463f,    21.8861628329693f},
  { 17.1717534381920f,    21.0992865661484f},
  { -164.966512747584f,    24.6497602994111f},
  { -59.4026458144142f,    2.15530287839289f},
  { -88.1025931036298f,    -7.22647820590291f},
  { 113.156726861200f,    20.6488253537474f},
  { -9.70556655851865f,    -58.9904455688244f},
  { -117.823858884209f,    2.00836092813275f},
  { -68.3961656689622f,    -31.4166279696797f},
  { 93.9627061532816f,    -1.96489155465916f},
  { -165.284067920567f,    -20.8494572120680f},
  { -158.205758950096f,    24.5956273405416f},
  { 161.762836110003f,    25.7502281669099f},
  { 37.1774274641116f,    0.00122304153082434f},
  { 40.5169054140825f,    -21.3314132697898f},
  { -137.233557042027f,    -6.64727001332420f},
  { -125.634143251148f,    -3.32005131727536f},
  { 116.171448122646f,    53.9965427199589f},
  { 96.4994502982522f,    33.0953698492831f},
  { 87.4981441031278f,    27.7392465879266f},
  { 159.732437653478f,    47.4929971322362f},
  { -12.6128260025385f,    31.2669255587668f},
  { -113.986894848109f,    66.4040017990331f},
  { 57.0911540713827f,    31.0253615230631f},
  { 52.6078934939306f,    7.97219361674645f},
  { 135.989522896127f,    53.7396428003661f},
  { -170.310125589486f,    -78.6666027424834f},
  { -42.1170459183740f,    0.0285226663302499f},
  { -85.2100721239038f,    51.5582786382794f},
  { -83.5296653886967f,    -18.6807279790964f},
  { 98.7876696191524f,    -29.8154678381831f},
  { 80.6652164591927f,    56.8277725472984f},
  { -94.1981321624214f,    -5.10290338239701f},
  { 22.2959780701398f,    62.5546128123649f},
  { 58.3156666309924f,    -54.3143700428141f},
  { 123.763910583304f,    0.777785530040157f},
  { 10.7848529603739f,    -43.7727508342241f},
  { 16.7866171707877f,    -42.1633687134087f},
  { 149.199126347575f,    -33.6433037634648f},
  { -123.511732700472f,    -42.6373748511893f},
  { -43.6855210063305f,    -16.4608409303940f},
  { -40.6433681386932f,    25.0428035063905f},
  { 106.880335290436f,    -16.5694183555613f},
  { -141.902282534656f,    -73.4020862737075f},
  { -61.8795891388157f,    47.7029518033688f},
  { 28.5424508194845f,    -65.5573721294340f},
  { 64.0355859606815f,    37.7835938413853f},
  { 95.3602598284480f,    -6.18243896994417f},
  { 115.398139393962f,    34.1059873448295f},
  { 141.874858055063f,    -75.6357065027583f},
  { -96.0720195569122f,    -70.7110892573056f},
  { -17.5116188358654f,    8.05388067181887f},
  { 55.0841650338319f,    28.6372443815363f},
  { -66.4892262874780f,    -0.511739843878896f},
  { -109.576668668188f,    -30.3198053825930f},
  { -133.137401865358f,    -5.57909417980254f},
  { -155.201877526628f,    16.4909455786307f},
  { 141.277005185901f,    -44.4837505748274f},
  { -27.9154147122787f,    -29.2254045365174f},
  { 12.6959994711851f,    -23.1560397945952f},
  { -107.543934017772f,    -20.8066048257218f},
  { 124.457638478003f,    12.5505249476043f},
  { 160.306614991756f,    35.9602955722954f},
  { 45.2333761587111f,    39.5080689884303f},
  { 164.727219767744f,    -46.7878964947383f},
  { 90.2824717671546f,    -46.7286798080069f},
  { 119.386156726352f,    -23.8982826269323f},
  { 30.9083556181930f,    20.5547251786388f},
  { -119.657887812102f,    36.0529968149371f},
  { 80.0683404818377f,    -7.38578556873742f},
  { 57.9650093065246f,    7.97139081626441f},
  { -74.2515262712616f,    15.3252359046220f},
  { 144.597430350823f,    -84.3854808880661f},
  { -153.226425957804f,    -33.1806002088145f},
  { -85.3158939347040f,    23.1803492839881f},
  { -8.37484652860451f,    49.6540445940998f},
  { -91.7483156179850f,    24.2930397489084f},
  { 146.845377929554f,    51.6665013475434f},
  { -92.3517173890092f,    49.4192000933288f},
  { 39.9500689916536f,    -59.7221513665905f},
  { -127.181471298865f,    -33.6164084237279f},
  { 175.598273393841f,    51.2710474047081f},
  { -34.2992266112344f,    0.899381262066922f},
  { -143.895008856298f,    14.3973040145159f},
  { 137.099804853137f,    16.0138213980345f},
  { -82.1708023391497f,    -70.0868202682595f},
  { 113.981241579076f,    -63.8532240503450f},
  { -71.3493192438275f,    70.7933120290820f},
  { 123.403309975359f,    -72.3439115067856f},
  { 143.751177641595f,    -16.9414719303444f},
  { 163.359498223056f,    -34.6256533554036f},
  { 16.5690152612362f,    -35.8057850835203f},
  { -97.3980414079029f,    35.6499021668553f},
  { -11.9381539204376f,    6.89983983937935f},
  { 13.5863367477490f,    35.6864722526444f},
  { -147.433479225236f,    10.0427809432249f},
  { 76.7662957702636f,    -32.0129382030327f},
  { -130.203629588725f,    -5.29126034881621f},
  { 5.35187832724210f,    26.8729608135584f},
  { 135.915770521913f,    61.9871004665682f},
  { -68.7822606173081f,    3.66948601795074f},
  { -113.710077721735f,    -36.5228307195076f},
  { -103.611642388383f,    34.8921254694351f},
  { -176.049802133937f,    -13.5957731930607f},
  { 170.925019288417f,    -3.82133122089724f},
  { -137.822624079758f,    -50.7396987495080f},
  { -141.561848224384f,    38.9615810738397f},
  { 179.462436896961f,    9.64886123162515f},
  { -83.0392417036479f,    56.9523818491975f},
  { -145.785315645868f,    -57.2195876291135f},
  { 23.5255459985051f,    48.1883686959911f},
  { -146.628881757282f,    22.8287767957295f},
  { 51.3005332505884f,    -29.6150882158520f},
  { -48.6720177318396f,    -74.2708525640276f},
  { -72.0034472862407f,    32.3762158485532f},
  { -151.783567825128f,    56.7970178496411f},
  { 17.5959792077941f,    -2.96670033324950f},
  { -66.1714017854531f,    -21.3329224215482f},
  { 30.8278540748646f,    71.3240651302868f},
  { -60.7280986683753f,    -42.4432931015365f},
  { 40.0023549685228f,    78.9960299569436f},
  { -102.191674115987f,    -1.40457196709822f},
  { -160.933268004215f,    -22.2972545805126f},
  { 124.806779004816f,    16.8795569835615f},
  { -146.623188638472f,    55.5383915765304f},
  { -70.2678324107743f,    80.3322296380082f},
  { 151.011266599935f,    -61.8430180901115f},
  { 58.7727460758906f,    -29.3455948068319f},
  { -23.3015484293758f,    8.60653478555094f},
  { 136.399036511516f,    -43.4220061963886f},
  { -40.4782037242702f,    -52.8310463838783f},
  { -27.0093557946620f,    30.2839970411682f},
  { -44.0154427442054f,    -13.7405127277513f},
  { -95.0098230960211f,    -62.2961606045130f},
  { -92.9342304060284f,    56.4933195105540f},
  { 46.0586955990174f,    63.9047949331664f},
  { -51.4262387759961f,    53.0791421814110f},
  { -116.888998440072f,    34.5481763503295f},
  { -67.7513984577392f,    -61.3712450577394f},
  { -16.8476345237913f,    -76.0270400005355f},
  { -134.183916613758f,    23.8118942104723f},
  { -86.6718485155173f,    32.4383291745101f},
  { -14.3958810177562f,    -27.6896589080522f},
  { -65.3091092178413f,    22.9064874556324f},
  { -107.386890240533f,    -59.0872490700160f},
  { -119.001120982270f,    26.5383147207002f},
  { -74.5053577427669f,    30.0894949274588f},
  { -136.699021147207f,    31.7532672601516f},
  { 151.346025506720f,    -9.63621267583399f},
  { 87.9452902348601f,    11.1933274190420f},
  { -100.428466167001f,    -34.0482863848922f},
  { 143.485821453504f,    48.8584186964107f},
  { -128.037638420252f,    -71.3179796855643f},
  { 99.9413900012479f,    -67.4361974459431f},
  { -84.0648968133253f,    -11.1153542246964f},
  { 57.2836876181037f,    15.1547044671039f},
  { 84.1094075043498f,    -18.7254166893063f},
  { -50.4808422824117f,    -14.8559655348626f},
  { -129.072793840213f,    -47.4373967714934f},
  { -133.165653961370f,    -34.3440385315725f},
  { 14.2616324853737f,    22.2428425196845f},
  { 155.454758922378f,    -19.6374536913591f},
  { 173.433541180285f,    75.1674418498854f},
  { 21.1618454572044f,    -53.0630208901224f},
  { 57.7253780903864f,    12.7351733201854f},
  { 17.7813749505843f,    -19.8784005134237f},
  { 21.0543469473868f,    7.84519427537937f},
  { 0.222727725030534f,    -26.3878936420562f},
  { -29.4522455101627f,    -36.9418485720427f},
  { 41.1027265314498f,    48.9680771626971f},
  { 66.3690428160757f,    -2.24672998227578f},
  { 24.4961747123605f,    -25.3266254889109f},
  { -73.8165639058137f,    -19.4332712625845f},
  { -167.069009764370f,    13.5951648647855f},
  { -45.1101270600719f,    19.2242684708714f},
  { -145.387347115007f,    -70.7876279655545f},
  { -124.428532568212f,    35.8760228674956f},
  { 38.0670152239321f,    -27.5102009860739f},
  { -65.5795773372308f,    6.75665632492799f},
  { -12.5922748462720f,    -67.9198383343094f},
  { 109.970811792012f,    47.4288679222888f},
  { 2.08767165987887f,    9.40454145810874f},
  { 6.50177580112888f,    29.5165426276179f},
  { -3.08359942372755f,    -56.2755941203827f},
  { -59.7997974465192f,    -48.8714421590451f},
  { 124.521220769929f,    -21.6573621515482f},
  { 4.10996900842198f,    59.1773281453812f},
  { 2.88698357507791f,    12.0213196301989f},
  { 138.072180611046f,    -55.4474832389765f},
  { -131.994123095643f,    -9.87124521482449f},
  { 79.2399418103833f,    43.3784892040924f},
  { 86.5597185436124f,    36.2372927389970f},
  { -158.109950328580f,    32.3888224004639f},
  { 116.288427236037f,    29.5089032004728f},
  { 1.76029062999027f,    -70.3834088846708f},
  { -152.354795047863f,    54.0724688056757f},
  { 15.2724873060483f,    44.0830949127842f},
  { 170.050539563649f,    59.3946332432259f},
  { 14.4945951196825f,    -13.2801298732065f},
  { -12.1891138784757f,    -29.9317647781702f},
  { -118.554833204948f,    57.2699820156663f},
  { 0.861711456092793f,    13.6745062509686f},
  { -95.5040327019893f,    27.5899207321931f},
  { 6.88983651974276f,    51.3215487171266f},
  { 160.503502694453f,    -34.4926208802630f},
  { -141.260801472763f,    -38.8633228180360f},
  { -159.985640529669f,    27.1739718092969f},
  { 71.5305124616388f,    21.1074384523449f},
  { -92.8882518554905f,    15.6848988992053f},
  { 114.380051711184f,    77.8573598792618f},
  { -68.7065796129315f,    -19.7788279590554f},
  { 139.618600537084f,    -30.0006313598990f},
  { -84.4523641641444f,    -33.2781782022872f},
  { 51.2940346465277f,    -66.7025157444241f},
  { -15.5725385430083f,    -49.3131717333017f},
  { 159.273957283885f,    -11.4387148090826f},
  { 4.62341217878234f,    -64.9251559757401f},
  { -40.8472218224914f,    -17.4860115092893f},
  { -178.453833687025f,    -15.6236031573140f},
  { -79.9946150612550f,    19.5850919034989f},
  { 80.7921691057875f,    -38.0921168176831f},
  { -37.9445320226825f,    66.2499444805101f},
  { -153.382565639199f,    -44.3817617537126f},
  { 131.698308070262f,    -28.3191133457150f},
  { 4.53207320431034f,    42.9961766589098f},
  { 44.4317108389353f,    -10.1357540200865f},
  { 176.285197100644f,    -0.444023983564376f},
  { -106.090363023090f,    2.81746779460127f},
  { -168.314740631877f,    -20.0451644263499f},
  { -106.728805503965f,    60.1615879529241f},
  { -73.6528887127457f,    -7.02386391992281f},
  { 103.358976269811f,    72.5571967965817f},
  { 46.8699802494892f,    30.7129689203005f},
  { 150.188398678269f,    9.94085139126176f},
  { -25.4662926725902f,    -24.2385196867934f},
  { 45.8135525463208f,    33.6956561480977f},
  { -114.228911688849f,    -14.8177439793580f},
  { 71.7373878982309f,    4.94251887542312f},
  { 141.725877298316f,    41.5339575353950f},
  { 165.245297206856f,    7.56989697482882f},
  { 72.5797984341638f,    0.529293144238883f},
  { -27.3359144056700f,    -16.0365699754986f},
  { 68.2377740608023f,    -33.5173949541811f},
  { 92.9350669218194f,    -52.2272677709710f},
  { -102.404580909068f,    -48.9534777937647f},
  { -140.594402149581f,    -56.9765468818325f},
  { -158.077406530693f,    37.0587839998051f},
  { -100.488526829726f,    -61.4661289807893f},
  { 133.454130675467f,    31.6966929725523f},
  { -121.598715156472f,    17.8019145549491f},
  { 99.1641101526173f,    -32.9298584551120f},
  { -39.2878629033924f,    86.0361264373848f},
  { 5.89174087868414f,    68.9025410790662f},
  { 9.40844491776371f,    -21.9318743602209f},
  { 130.200992184776f,    -77.1728671427286f},
  { -165.524980359325f,    16.0414099942838f},
  { 39.5292022547424f,    0.781546484841279f},
  { -33.4009002698567f,    -13.1112992066295f},
  { 156.395262932044f,    33.5967227211758f},
  { 122.471185039887f,    26.2998836066371f},
  { 151.068927171870f,    15.3522589503249f},
  { -25.9056292446808f,    -42.5306161497219f},
  { 128.932094046807f,    11.3874706095592f},
  { 57.0882944767236f,    -20.0504007601108f},
  { 157.290633451718f,    -57.3706505700685f},
  { 177.288907194553f,    45.1925179357727f},
  { 43.0505510551844f,    21.5304768822415f},
  { 55.3902923594091f,    25.4691819102283f},
  { 39.4587748625094f,    -46.8014419947198f},
  { 14.9982668156435f,    -64.0291737990266f},
  { 115.418389562979f,    9.82876769247620f},
  { -75.2799375599416f,    -21.9137791868593f},
  { -9.74419802716439f,    4.77905668014385f},
  { 64.0238627228055f,    -19.1912392224307f},
  { 28.4604076960682f,    -7.01859833587053f},
  { -46.4716711427444f,    21.9240660220047f},
  { 95.9812083145331f,    43.6879621991246f},
  { 76.0192230026089f,    -56.6120488567637f},
  { -168.167640181396f,    -34.2954326000185f},
  { -90.4566139455999f,    -53.3005800809702f},
  { -49.8131879156086f,    35.3487204808272f},
  { 129.456952040756f,    21.5017538617150f},
  { -76.6492352948576f,    -15.3913250208690f},
  { -165.388516233946f,    52.6437574909619f},
  { 141.003551679550f,    -7.23184571800371f},
  { -122.418098889030f,    -39.0514087346796f},
  { -111.873577988527f,    -83.5896485002506f},
  { 146.334339001816f,    35.9628016251027f},
  { 36.3083542342691f,    67.7646464173432f},
  { -97.4185695302239f,    23.6309343558319f},
  { 86.8154487378793f,    -7.64238112779267f},
  { 93.2430322518040f,    -18.4318830036241f},
  { -179.267127304330f,    72.9705478164780f},
  { -27.2656766599496f,    -19.4648625788881f},
  { 7.85622666422132f,    -6.78760475917541f},
  { -125.343347945195f,    39.4676908478457f},
  { 132.247948329354f,    29.2215996906535f},
  { 64.9737269957331f,    -4.33911453177047f},
  { -16.1016003679269f,    52.1498077435459f},
  { 46.9590726843577f,    41.9857673596353f},
  { 9.38682417061308f,    -62.9020417788055f},
  { 7.68393451420386f,    -34.6845217202594f},
  { -91.9130603737003f,    -49.7356328172613f},
  { 155.386046047939f,    59.4569820928223f},
  { 100.873216760438f,    -13.4090787188007f},
  { -54.5615452196666f,    30.8830006753207f},
  { -52.1049236141293f,    -27.9870804214778f},
  { -46.9144476824353f,    -8.49214325422109f},
  { -60.4975265324781f,    56.9505318303055f},
  { -106.535594697110f,    -41.6740891733659f},
  { -53.5315516821509f,    81.1757779120381f},
  { 171.818446299325f,    -25.6021293919061f},
  { 26.2292971292566f,    36.7142333927591f},
  { 1.49813639786544f,    -67.4231436311476f},
  { -23.1301627597884f,    -10.0114476020334f},
  { -137.408777044242f,    24.5724445130511f},
  { -72.0737908622944f,    17.5276928029669f},
  { -6.94847890057156f,    -20.5064153779646f},
  { 69.7035400301462f,    -43.3283570410209f},
  { -154.882320506543f,    -50.4076945751962f},
  { -141.598085157580f,    12.4570104343930f},
  { 41.3267938193239f,    -76.5218626422000f},
  { 124.693270924213f,    -47.4315523343345f},
  { -151.046666694894f,    -2.44797575526281f},
  { 114.761000328018f,    -3.13210574865185f},
  { -60.9703126136592f,    39.3762987946713f},
  { 78.5803178985455f,    -86.9231546870455f},
  { 9.44626694234193f,    65.0608604062709f},
  { -83.7576635988502f,    -16.2228122151928f},
  { -148.275001910022f,    -7.97534228909585f},
  { 135.763132319245f,    -1.57108452063893f},
  { -170.958800845519f,    -15.7526171208103f},
  { -48.0125968225912f,    -18.7718584095748f},
  { -72.4802117736062f,    11.8998818678077f},
  { 174.500412074797f,    -10.7525179486425f},
  { -154.186796387022f,    59.7289071010347f},
  { -100.327090289926f,    -26.3376124240628f},
  { -5.53444289956598f,    53.7887245961459f},
  { 3.98720260338553f,    -48.6101930163952f},
  { 12.8314567997931f,    -27.6277009146372f},
  { -173.846007144778f,    -0.783767264503392f},
  { -122.872326586518f,    80.4009205719345f},
  { 71.8496558701181f,    54.7031471375908f},
  { 20.6235353753141f,    -50.3449698428696f},
  { -62.2924746137856f,    60.3845694113033f},
  { 7.88620574653739f,    -37.2867298418667f},
  { 51.5786917069760f,    60.5282670494810f},
  { 142.530182173305f,    -47.1972487303289f},
  { 114.548704258040f,    -32.2923148985652f},
  { 113.634188526224f,    46.7921897392182f},
  { -75.0415462603963f,    21.8133853873883f},
  { 47.1203853432278f,    13.8914614742006f},
  { 100.004460409058f,    -18.2902363364927f},
  { -162.038560434909f,    -67.6244762744038f},
  { 150.692173504310f,    -48.3475185135689f},
  { 98.9838914295035f,    -41.1778404983141f},
  { 84.0824115889269f,    59.1705074113635f},
  { 140.255393507376f,    16.3040362311791f},
  { -32.8801551463088f,    33.0285706666262f},
  { -119.971132810800f,    -24.9595067046181f},
  { 147.313998283850f,    -36.2829582706797f},
  { -155.821019982721f,    29.4859112072366f},
  { 1.78030213360078f,    -80.6877733100478f},
  { 95.8382252510577f,    -43.2337163318954f},
  { 94.4369352644656f,    -40.8366585812746f},
  { 66.6114784040319f,    82.7989059125011f},
  { 124.873288219046f,    -7.29373268220492f},
  { -46.7496707114961f,    16.8881865235000f},
  { 72.0389485261920f,    -24.0228145991868f},
  { -15.5678717026861f,    4.90522458407042f},
  { 90.4987778395565f,    -13.4159810169254f},
  { 54.4439653311543f,    -45.3087408610681f},
  { 121.988509494770f,    -27.5523328282382f},
  { 21.7628574301895f,    40.7409754806145f},
  { -61.0820920606410f,    19.1665682121696f},
  { 74.5944202832474f,    -25.0433519362724f},
  { -85.7984299255685f,    12.8168202594319f},
  { 100.591213223654f,    -48.7651641611959f},
  { -160.051710505998f,    46.8440961812710f},
  { -155.130887454222f,    -11.2451628345406f},
  { 71.3568195610080f,    8.65819569512514f},
  { -120.674774559137f,    -5.60409567495968f},
  { 7.48848316534340f,    12.8921053093270f},
  { -134.391936053632f,    -40.6098268184209f},
  { -172.843125683288f,    23.3505065986695f},
  { 104.656241005574f,    6.31485645641428f},
  { -26.5489442453971f,    -7.20267264318446f},
  { 170.051442970705f,    53.4986707453079f},
  { 11.5880202028527f,    -36.8015250259757f},
  { 179.200273635581f,    59.2858521883024f},
  { 82.6077543945760f,    13.8595123680490f},
  { -91.1149851338603f,    -83.4712027190460f},
  { 102.690380569431f,    17.9273723726398f},
  { 172.141516366321f,    -43.6976310463691f},
  { 133.865351629765f,    46.3507622731438f},
  { 171.326117219647f,    27.0444739071031f},
  { -138.457333386491f,    -8.99338621861224f},
  { 63.8647133327651f,    16.1071707647558f},
  { -173.618202354631f,    37.3416834334259f},
  { 15.2376192499825f,    18.9225482580957f},
  { -47.5127985742410f,    73.7738115647869f},
  { -178.842469473749f,    0.663548155160963f},
  { -75.6363129159631f,    -4.80831175072516f},
  { -115.532634996254f,    -69.3752118427466f},
  { -50.8058790086430f,    -12.4768309397115f},
  { -25.3703029806357f,    38.6839648047261f},
  { -164.741977630714f,    -29.7751720913189f},
  { 93.5031252674226f,    20.8158861212099f},
  { 83.1221348028995f,    49.5920610875628f},
  { 12.3962718134035f,    38.9407106035933f},
  { -38.4706315629602f,    23.5703073598212f},
  { -140.454986841371f,    -10.4451589954151f},
  { -4.00228133116523f,    42.3688490538350f},
  { -125.266166505277f,    3.65129060305555f},
  { 45.5174486940331f,    -26.5767779026310f},
  { -106.532198542432f,    -67.8898727553161f},
  { -47.8316932919170f,    -11.3019720644799f},
  { 102.622123075860f,    8.22517253582149f},
  { 110.200267425440f,    -11.7209972970157f},
  { 23.4874942234784f,    26.9944942821933f},
  { 179.834914534455f,    -20.8630251873377f},
  { 145.302128945304f,    54.2624319034759f},
  { 56.4349974180523f,    69.5324822064906f},
  { -27.0802017552490f,    0.173996300641060f},
  { 0.305720228296931f,    30.3083129268004f},
  { -150.859174657463f,    -21.8861628329693f},
  { -17.1717534381920f,    -21.0992865661484f},
  { 164.966512747584f,    -24.6497602994111f},
  { 59.4026458144142f,    -2.15530287839289f},
  { 88.1025931036298f,    7.22647820590291f},
  { -113.156726861200f,    -20.6488253537474f},
  { 9.70556655851865f,    58.9904455688244f},
  { 117.823858884209f,    -2.00836092813275f},
  { 68.3961656689622f,    31.4166279696797f},
  { -93.9627061532816f,    1.96489155465916f},
  { 165.284067920567f,    20.8494572120680f},
  { 158.205758950096f,    -24.5956273405416f},
  { -161.762836110003f,    -25.7502281669099f},
  { -37.1774274641116f,    -0.00122304153082434f},
  { -40.5169054140825f,    21.3314132697898f},
  { 137.233557042027f,    6.64727001332420f},
  { 125.634143251148f,    3.32005131727536f},
  { -116.171448122646f,    -53.9965427199589f},
  { -96.4994502982522f,    -33.0953698492831f},
  { -87.4981441031278f,    -27.7392465879266f},
  { -159.732437653478f,    -47.4929971322362f},
  { 12.6128260025385f,    -31.2669255587668f},
  { 113.986894848109f,    -66.4040017990331f},
  { -57.0911540713827f,    -31.0253615230631f},
  { -52.6078934939306f,    -7.97219361674645f},
  { -135.989522896127f,    -53.7396428003661f},
  { 170.310125589486f,    78.6666027424834f},
  { 42.1170459183740f,    -0.0285226663302499f},
  { 85.2100721239038f,    -51.5582786382794f},
  { 83.5296653886967f,    18.6807279790964f},
  { -98.7876696191524f,    29.8154678381831f},
  { -80.6652164591927f,    -56.8277725472984f},
  { 94.1981321624214f,    5.10290338239701f},
  { -22.2959780701398f,    -62.5546128123649f},
  { -58.3156666309924f,    54.3143700428141f},
  { -123.763910583304f,    -0.777785530040157f},
  { -10.7848529603739f,    43.7727508342241f},
  { -16.7866171707877f,    42.1633687134087f},
  { -149.199126347575f,    33.6433037634648f},
  { 123.511732700472f,    42.6373748511893f},
  { 43.6855210063305f,    16.4608409303940f},
  { 40.6433681386932f,    -25.0428035063905f},
  { -106.880335290436f,    16.5694183555613f},
  { 141.902282534656f,    73.4020862737075f},
  { 61.8795891388157f,    -47.7029518033688f},
  { -28.5424508194845f,    65.5573721294340f},
  { -64.0355859606815f,    -37.7835938413853f},
  { -95.3602598284480f,    6.18243896994417f},
  { -115.398139393962f,    -34.1059873448295f},
  { -141.874858055063f,    75.6357065027583f},
  { 96.0720195569122f,    70.7110892573056f},
  { 17.5116188358654f,    -8.05388067181887f},
  { -55.0841650338319f,    -28.6372443815363f},
  { 66.4892262874780f,    0.511739843878896f},
  { 109.576668668188f,    30.3198053825930f},
  { 133.137401865358f,    5.57909417980254f},
  { 155.201877526628f,    -16.4909455786307f},
  { -141.277005185901f,    44.4837505748274f},
  { 27.9154147122787f,    29.2254045365174f},
  { -12.6959994711851f,    23.1560397945952f},
  { 107.543934017772f,    20.8066048257218f},
  { -124.457638478003f,    -12.5505249476043f},
  { -160.306614991756f,    -35.9602955722954f},
  { -45.2333761587111f,    -39.5080689884303f},
  { -164.727219767744f,    46.7878964947383f},
  { -90.2824717671546f,    46.7286798080069f},
  { -119.386156726352f,    23.8982826269323f},
  { -30.9083556181930f,    -20.5547251786388f},
  { 119.657887812102f,    -36.0529968149371f},
  { -80.0683404818377f,    7.38578556873742f},
  { -57.9650093065246f,    -7.97139081626441f},
  { 74.2515262712616f,    -15.3252359046220f},
  { -144.597430350823f,    84.3854808880661f},
  { 153.226425957804f,    33.1806002088145f},
  { 85.3158939347040f,    -23.1803492839881f},
  { 8.37484652860451f,    -49.6540445940998f},
  { 91.7483156179850f,    -24.2930397489084f},
  { -146.845377929554f,    -51.6665013475434f},
  { 92.3517173890092f,    -49.4192000933288f},
  { -39.9500689916536f,    59.7221513665905f},
  { 127.181471298865f,    33.6164084237279f},
  { -175.598273393841f,    -51.2710474047081f},
  { 34.2992266112344f,    -0.899381262066922f},
  { 143.895008856298f,    -14.3973040145159f},
  { -137.099804853137f,    -16.0138213980345f},
  { 82.1708023391497f,    70.0868202682595f},
  { -113.981241579076f,    63.8532240503450f},
  { 71.3493192438275f,    -70.7933120290820f},
  { -123.403309975359f,    72.3439115067856f},
  { -143.751177641595f,    16.9414719303444f},
  { -163.359498223056f,    34.6256533554036f},
  { -16.5690152612362f,    35.8057850835203f},
  { 97.3980414079029f,    -35.6499021668553f},
  { 11.9381539204376f,    -6.89983983937935f},
  { -13.5863367477490f,    -35.6864722526444f},
  { 147.433479225236f,    -10.0427809432249f},
  { -76.7662957702636f,    32.0129382030327f},
  { 130.203629588725f,    5.29126034881621f},
  { -5.35187832724210f,    -26.8729608135584f},
  { -135.915770521913f,    -61.9871004665682f},
  { 68.7822606173081f,    -3.66948601795074f},
  { 113.710077721735f,    36.5228307195076f},
  { 103.611642388383f,    -34.8921254694351f},
  { 176.049802133937f,    13.5957731930607f},
  { -170.925019288417f,    3.82133122089724f},
  { 137.822624079758f,    50.7396987495080f},
  { 141.561848224384f,    -38.9615810738397f},
  { -179.462436896961f,    -9.64886123162515f},
  { 83.0392417036479f,    -56.9523818491975f},
  { 145.785315645868f,    57.2195876291135f},
  { -23.5255459985051f,    -48.1883686959911f},
  { 146.628881757282f,    -22.8287767957295f},
  { -51.3005332505884f,    29.6150882158520f},
  { 48.6720177318396f,    74.2708525640276f},
  { 72.0034472862407f,    -32.3762158485532f},
  { 151.783567825128f,    -56.7970178496411f},
  { -17.5959792077941f,    2.96670033324950f},
  { 66.1714017854531f,    21.3329224215482f},
  { -30.8278540748646f,    -71.3240651302868f},
  { 60.7280986683753f,    42.4432931015365f},
  { -40.0023549685228f,    -78.9960299569436f},
  { 102.191674115987f,    1.40457196709822f},
  { 160.933268004215f,    22.2972545805126f},
  { -124.806779004816f,    -16.8795569835615f},
  { 146.623188638472f,    -55.5383915765304f},
  { 70.2678324107743f,    -80.3322296380082f},
  { -151.011266599935f,    61.8430180901115f},
  { -58.7727460758906f,    29.3455948068319f},
  { 23.3015484293758f,    -8.60653478555094f},
  { -136.399036511516f,    43.4220061963886f},
  { 40.4782037242702f,    52.8310463838783f},
  { 27.0093557946620f,    -30.2839970411682f},
  { 44.0154427442054f,    13.7405127277513f},
  { 95.0098230960211f,    62.2961606045130f},
  { 92.9342304060284f,    -56.4933195105540f},
  { -46.0586955990174f,    -63.9047949331664f},
  { 51.4262387759961f,    -53.0791421814110f},
  { 116.888998440072f,    -34.5481763503295f},
  { 67.7513984577392f,    61.3712450577394f},
  { 16.8476345237913f,    76.0270400005355f},
  { 134.183916613758f,    -23.8118942104723f},
  { 86.6718485155173f,    -32.4383291745101f},
  { 14.3958810177562f,    27.6896589080522f},
  { 65.3091092178413f,    -22.9064874556324f},
  { 107.386890240533f,    59.0872490700160f},
  { 119.001120982270f,    -26.5383147207002f},
  { 74.5053577427669f,    -30.0894949274588f},
  { 136.699021147207f,    -31.7532672601516f},
  { -151.346025506720f,    9.63621267583399f},
  { -87.9452902348601f,    -11.1933274190420f},
  { 100.428466167001f,    34.0482863848922f},
  { -143.485821453504f,    -48.8584186964107f},
  { 128.037638420252f,    71.3179796855643f},
  { -99.9413900012479f,    67.4361974459431f},
  { 84.0648968133253f,    11.1153542246964f},
  { -57.2836876181037f,    -15.1547044671039f},
  { -84.1094075043498f,    18.7254166893063f},
  { 50.4808422824117f,    14.8559655348626f},
  { 129.072793840213f,    47.4373967714934f},
  { 133.165653961370f,    34.3440385315725f},
  { -14.2616324853737f,    -22.2428425196845f},
  { -155.454758922378f,    19.6374536913591f},
  { -173.433541180285f,    -75.1674418498854f},
  { -21.1618454572044f,    53.0630208901224f},
  { -57.7253780903864f,    -12.7351733201854f},
  { -17.7813749505843f,    19.8784005134237f},
  { -21.0543469473868f,    -7.84519427537937f},
  { -0.222727725030534f,    26.3878936420562f},
  { 29.4522455101627f,    36.9418485720427f},
  { -41.1027265314498f,    -48.9680771626971f},
  { -66.3690428160757f,    2.24672998227578f},
  { -24.4961747123605f,    25.3266254889109f},
  { 73.8165639058137f,    19.4332712625845f},
  { 167.069009764370f,    -13.5951648647855f},
  { 45.1101270600719f,    -19.2242684708714f},
  { 145.387347115007f,    70.7876279655545f},
  { 124.428532568212f,    -35.8760228674956f},
  { -38.0670152239321f,    27.5102009860739f},
  { 65.5795773372308f,    -6.75665632492799f},
  { 12.5922748462720f,    67.9198383343094f},
  { -109.970811792012f,    -47.4288679222888f},
  { -2.08767165987887f,    -9.40454145810874f},
  { -6.50177580112888f,    -29.5165426276179f},
  { 3.08359942372755f,    56.2755941203827f},
  { 59.7997974465192f,    48.8714421590451f},
  { -124.521220769929f,    21.6573621515482f},
  { -4.10996900842198f,    -59.1773281453812f},
  { -2.88698357507791f,    -12.0213196301989f},
  { -138.072180611046f,    55.4474832389765f},
  { 131.994123095643f,    9.87124521482449f},
  { -79.2399418103833f,    -43.3784892040924f},
  { -86.5597185436124f,    -36.2372927389970f},
  { 158.109950328580f,    -32.3888224004639f},
  { -116.288427236037f,    -29.5089032004728f},
  { -1.76029062999027f,    70.3834088846708f},
  { 152.354795047863f,    -54.0724688056757f},
  { 164.727512693952f,    -44.0830949127842f},
  { 9.94946043635073f,    -59.3946332432259f},
  { 165.505404880318f,    13.2801298732065f},
  { -167.810886121524f,    29.9317647781702f},
  { -61.4451667950518f,    -57.2699820156663f},
  { 179.138288543907f,    -13.6745062509686f},
  { -84.4959672980107f,    -27.5899207321931f},
  { 173.110163480257f,    -51.3215487171266f},
  { 19.4964973055475f,    34.4926208802630f},
  { -38.7391985272367f,    38.8633228180360f},
  { -20.0143594703306f,    -27.1739718092969f},
  { 108.469487538361f,    -21.1074384523449f},
  { -87.1117481445096f,    -15.6848988992053f},
  { 65.6199482888161f,    -77.8573598792618f},
  { -111.293420387068f,    19.7788279590554f},
  { 40.3813994629159f,    30.0006313598990f},
  { -95.5476358358556f,    33.2781782022872f},
  { 128.705965353472f,    66.7025157444241f},
  { -164.427461456992f,    49.3131717333017f},
  { 20.7260427161146f,    11.4387148090826f},
  { 175.376587821218f,    64.9251559757401f},
  { -139.152778177509f,    17.4860115092893f},
  { -1.54616631297520f,    15.6236031573140f},
  { -100.005384938745f,    -19.5850919034989f},
  { 99.2078308942125f,    38.0921168176831f},
  { -142.055467977318f,    -66.2499444805101f},
  { -26.6174343608009f,    44.3817617537126f},
  { 48.3016919297376f,    28.3191133457150f},
  { 175.467926795690f,    -42.9961766589098f},
  { 135.568289161065f,    10.1357540200865f},
  { 3.71480289935651f,    0.444023983564376f},
  { -73.9096369769102f,    -2.81746779460127f},
  { -11.6852593681234f,    20.0451644263499f},
  { -73.2711944960354f,    -60.1615879529241f},
  { -106.347111287254f,    7.02386391992281f},
  { 76.6410237301893f,    -72.5571967965817f},
  { 133.130019750511f,    -30.7129689203005f},
  { 29.8116013217315f,    -9.94085139126176f},
  { -154.533707327410f,    24.2385196867934f},
  { 134.186447453679f,    -33.6956561480977f},
  { -65.7710883111514f,    14.8177439793580f},
  { 108.262612101769f,    -4.94251887542312f},
  { 38.2741227016836f,    -41.5339575353950f},
  { 14.7547027931441f,    -7.56989697482882f},
  { 107.420201565836f,    -0.529293144238883f},
  { -152.664085594330f,    16.0365699754986f},
  { 111.762225939198f,    33.5173949541811f},
  { 87.0649330781807f,    52.2272677709710f},
  { -77.5954190909323f,    48.9534777937647f},
  { -39.4055978504186f,    56.9765468818325f},
  { -21.9225934693067f,    -37.0587839998051f},
  { -79.5114731702736f,    61.4661289807893f},
  { 46.5458693245325f,    -31.6966929725523f},
  { -58.4012848435277f,    -17.8019145549491f},
  { 80.8358898473828f,    32.9298584551120f},
  { -140.712137096608f,    -86.0361264373848f},
  { 174.108259121316f,    -68.9025410790662f},
  { 170.591555082236f,    21.9318743602209f},
  { 49.7990078152236f,    77.1728671427286f},
  { -14.4750196406747f,    -16.0414099942838f},
  { 140.470797745258f,    -0.781546484841279f},
  { -146.599099730143f,    13.1112992066295f},
  { 23.6047370679558f,    -33.5967227211758f},
  { 57.5288149601129f,    -26.2998836066371f},
  { 28.9310728281296f,    -15.3522589503249f},
  { -154.094370755319f,    42.5306161497219f},
  { 51.0679059531933f,    -11.3874706095592f},
  { 122.911705523276f,    20.0504007601108f},
  { 22.7093665482818f,    57.3706505700685f},
  { 2.71109280544668f,    -45.1925179357727f},
  { 136.949448944816f,    -21.5304768822415f},
  { 124.609707640591f,    -25.4691819102283f},
  { 140.541225137491f,    46.8014419947198f},
  { 165.001733184357f,    64.0291737990266f},
  { 64.5816104370214f,    -9.82876769247620f},
  { -104.720062440058f,    21.9137791868593f},
  { -170.255801972836f,    -4.77905668014385f},
  { 115.976137277195f,    19.1912392224307f},
  { 151.539592303932f,    7.01859833587053f},
  { -133.528328857256f,    -21.9240660220047f},
  { 84.0187916854670f,    -43.6879621991246f},
  { 103.980776997391f,    56.6120488567637f},
  { -11.8323598186041f,    34.2954326000185f},
  { -89.5433860544001f,    53.3005800809702f},
  { -130.186812084391f,    -35.3487204808272f},
  { 50.5430479592438f,    -21.5017538617150f},
  { -103.350764705142f,    15.3913250208690f},
  { -14.6114837660536f,    -52.6437574909619f},
  { 38.9964483204503f,    7.23184571800371f},
  { -57.5819011109703f,    39.0514087346796f},
  { -68.1264220114729f,    83.5896485002506f},
  { 33.6656609981838f,    -35.9628016251027f},
  { 143.691645765731f,    -67.7646464173432f},
  { -82.5814304697762f,    -23.6309343558319f},
  { 93.1845512621207f,    7.64238112779267f},
  { 86.7569677481960f,    18.4318830036241f},
  { -0.732872695669594f,    -72.9705478164780f},
  { -152.734323340050f,    19.4648625788881f},
  { 172.143773335779f,    6.78760475917541f},
  { -54.6566520548053f,    -39.4676908478457f},
  { 47.7520516706459f,    -29.2215996906535f},
  { 115.026273004267f,    4.33911453177047f},
  { -163.898399632073f,    -52.1498077435459f},
  { 133.040927315642f,    -41.9857673596353f},
  { 170.613175829387f,    62.9020417788055f},
  { 172.316065485796f,    34.6845217202594f},
  { -88.0869396262997f,    49.7356328172613f},
  { 24.6139539520614f,    -59.4569820928223f},
  { 79.1267832395616f,    13.4090787188007f},
  { -125.438454780333f,    -30.8830006753207f},
  { -127.895076385871f,    27.9870804214778f},
  { -133.085552317565f,    8.49214325422109f},
  { -119.502473467522f,    -56.9505318303055f},
  { -73.4644053028897f,    41.6740891733659f},
  { -126.468448317849f,    -81.1757779120381f},
  { 8.18155370067543f,    25.6021293919061f},
  { 153.770702870743f,    -36.7142333927591f},
  { 178.501863602135f,    67.4231436311476f},
  { -156.869837240212f,    10.0114476020334f},
  { -42.5912229557585f,    -24.5724445130511f},
  { -107.926209137706f,    -17.5276928029669f},
  { -173.051521099428f,    20.5064153779646f},
  { 110.296459969854f,    43.3283570410209f},
  { -25.1176794934572f,    50.4076945751962f},
  { -38.4019148424203f,    -12.4570104343930f},
  { 138.673206180676f,    76.5218626422000f},
  { 55.3067290757868f,    47.4315523343345f},
  { -28.9533333051057f,    2.44797575526281f},
  { 65.2389996719823f,    3.13210574865185f},
  { -119.029687386341f,    -39.3762987946713f},
  { 101.419682101455f,    86.9231546870455f},
  { 170.553733057658f,    -65.0608604062709f},
  { -96.2423364011498f,    16.2228122151928f},
  { -31.7249980899784f,    7.97534228909585f},
  { 44.2368676807553f,    1.57108452063893f},
  { -9.04119915448089f,    15.7526171208103f},
  { -131.987403177409f,    18.7718584095748f},
  { -107.519788226394f,    -11.8998818678077f},
  { 5.49958792520265f,    10.7525179486425f},
  { -25.8132036129778f,    -59.7289071010347f},
  { -79.6729097100742f,    26.3376124240628f},
  { -174.465557100434f,    -53.7887245961459f},
  { 176.012797396614f,    48.6101930163952f},
  { 167.168543200207f,    27.6277009146372f},
  { -6.15399285522190f,    0.783767264503392f},
  { -57.1276734134818f,    -80.4009205719345f},
  { 108.150344129882f,    -54.7031471375908f},
  { 159.376464624686f,    50.3449698428696f},
  { -117.707525386214f,    -60.3845694113033f},
  { 172.113794253463f,    37.2867298418667f},
  { 128.421308293024f,    -60.5282670494810f},
  { 37.4698178266945f,    47.1972487303289f},
  { 65.4512957419604f,    32.2923148985652f},
  { 66.3658114737764f,    -46.7921897392182f},
  { -104.958453739604f,    -21.8133853873883f},
  { 132.879614656772f,    -13.8914614742006f},
  { 79.9955395909424f,    18.2902363364927f},
  { -17.9614395650914f,    67.6244762744038f},
  { 29.3078264956900f,    48.3475185135689f},
  { 81.0161085704965f,    41.1778404983141f},
  { 95.9175884110731f,    -59.1705074113635f},
  { 39.7446064926241f,    -16.3040362311791f},
  { -147.119844853691f,    -33.0285706666262f},
  { -60.0288671891997f,    24.9595067046181f},
  { 32.6860017161502f,    36.2829582706797f},
  { -24.1789800172791f,    -29.4859112072366f},
  { 178.219697866399f,    80.6877733100478f},
  { 84.1617747489423f,    43.2337163318954f},
  { 85.5630647355345f,    40.8366585812746f},
  { 113.388521595968f,    -82.7989059125011f},
  { 55.1267117809535f,    7.29373268220492f},
  { -133.250329288504f,    -16.8881865235000f},
  { 107.961051473808f,    24.0228145991868f},
  { -164.432128297314f,    -4.90522458407042f},
  { 89.5012221604435f,    13.4159810169254f},
  { 125.556034668846f,    45.3087408610681f},
  { 58.0114905052303f,    27.5523328282382f},
  { 158.237142569811f,    -40.7409754806145f},
  { -118.917907939359f,    -19.1665682121696f},
  { 105.405579716753f,    25.0433519362724f},
  { -94.2015700744315f,    -12.8168202594319f},
  { 79.4087867763458f,    48.7651641611959f},
  { -19.9482894940020f,    -46.8440961812710f},
  { -24.8691125457782f,    11.2451628345406f},
  { 108.643180438992f,    -8.65819569512514f},
  { -59.3252254408626f,    5.60409567495968f},
  { 172.511516834657f,    -12.8921053093270f},
  { -45.6080639463682f,    40.6098268184209f},
  { -7.15687431671250f,    -23.3505065986695f},
  { 75.3437589944260f,    -6.31485645641428f},
  { -153.451055754603f,    7.20267264318446f},
  { 9.94855702929540f,    -53.4986707453079f},
  { 168.411979797147f,    36.8015250259757f},
  { 0.799726364418894f,    -59.2858521883024f},
  { 97.3922456054241f,    -13.8595123680490f},
  { -88.8850148661397f,    83.4712027190460f},
  { 77.3096194305693f,    -17.9273723726398f},
  { 7.85848363367946f,    43.6976310463691f},
  { 46.1346483702354f,    -46.3507622731438f},
  { 8.67388278035287f,    -27.0444739071031f},
  { -41.5426666135085f,    8.99338621861224f},
  { 116.135286667235f,    -16.1071707647558f},
  { -6.38179764536891f,    -37.3416834334259f},
  { 164.762380750018f,    -18.9225482580957f},
  { -132.487201425759f,    -73.7738115647869f},
  { -1.15753052625121f,    -0.663548155160963f},
  { -104.363687084037f,    4.80831175072516f},
  { -64.4673650037460f,    69.3752118427466f},
  { -129.194120991357f,    12.4768309397115f},
  { -154.629697019364f,    -38.6839648047261f},
  { -15.2580223692863f,    29.7751720913189f},
  { 86.4968747325774f,    -20.8158861212099f},
  { 96.8778651971005f,    -49.5920610875628f},
  { 167.603728186596f,    -38.9407106035933f},
  { -141.529368437040f,    -23.5703073598212f},
  { -39.5450131586291f,    10.4451589954151f},
  { -175.997718668835f,    -42.3688490538350f},
  { -54.7338334947230f,    -3.65129060305555f},
  { 134.482551305967f,    26.5767779026310f},
  { -73.4678014575678f,    67.8898727553161f},
  { -132.168306708083f,    11.3019720644799f},
  { 77.3778769241396f,    -8.22517253582149f},
  { 69.7997325745604f,    11.7209972970157f},
  { 156.512505776522f,    -26.9944942821933f},
  { 0.165085465545444f,    20.8630251873377f},
  { 34.6978710546962f,    -54.2624319034759f},
  { 123.565002581948f,    -69.5324822064906f},
  { -152.919798244751f,    -0.173996300641060f},
  { 179.694279771703f,    -30.3083129268004f},
  { -29.1408253425372f,    21.8861628329693f},
  { -162.828246561808f,    21.0992865661484f},
  { 15.0334872524158f,    24.6497602994111f},
  { 120.597354185586f,    2.15530287839289f},
  { 91.8974068963703f,    -7.22647820590291f},
  { -66.8432731388005f,    20.6488253537474f},
  { 170.294433441481f,    -58.9904455688244f},
  { 62.1761411157911f,    2.00836092813275f},
  { 111.603834331038f,    -31.4166279696797f},
  { -86.0372938467184f,    -1.96489155465916f},
  { 14.7159320794333f,    -20.8494572120680f},
  { 21.7942410499041f,    24.5956273405416f},
  { -18.2371638899974f,    25.7502281669099f},
  { -142.822572535888f,    0.00122304153082434f},
  { -139.483094585917f,    -21.3314132697898f},
  { 42.7664429579734f,    -6.64727001332420f},
  { 54.3658567488516f,    -3.32005131727536f},
  { -63.8285518773537f,    53.9965427199589f},
  { -83.5005497017478f,    33.0953698492831f},
  { -92.5018558968723f,    27.7392465879266f},
  { -20.2675623465217f,    47.4929971322362f},
  { 167.387173997461f,    31.2669255587668f},
  { 66.0131051518908f,    66.4040017990331f},
  { -122.908845928617f,    31.0253615230631f},
  { -127.392106506069f,    7.97219361674645f},
  { -44.0104771038730f,    53.7396428003661f},
  { 9.68987441051391f,    -78.6666027424834f},
  { 137.882954081626f,    0.0285226663302499f},
  { 94.7899278760962f,    51.5582786382794f},
  { 96.4703346113033f,    -18.6807279790964f},
  { -81.2123303808476f,    -29.8154678381831f},
  { -99.3347835408073f,    56.8277725472984f},
  { 85.8018678375787f,    -5.10290338239701f},
  { -157.704021929860f,    62.5546128123649f},
  { -121.684333369008f,    -54.3143700428141f},
  { -56.2360894166961f,    0.777785530040157f},
  { -169.215147039626f,    -43.7727508342241f},
  { -163.213382829212f,    -42.1633687134087f},
  { -30.8008736524250f,    -33.6433037634648f},
  { 56.4882672995278f,    -42.6373748511893f},
  { 136.314478993669f,    -16.4608409303940f},
  { 139.356631861307f,    25.0428035063905f},
  { -73.1196647095645f,    -16.5694183555613f},
  { 38.0977174653441f,    -73.4020862737075f},
  { 118.120410861184f,    47.7029518033688f},
  { -151.457549180516f,    -65.5573721294340f},
  { -115.964414039319f,    37.7835938413853f},
  { -84.6397401715521f,    -6.18243896994417f},
  { -64.6018606060384f,    34.1059873448295f},
  { -38.1251419449371f,    -75.6357065027583f},
  { 83.9279804430878f,    -70.7110892573056f},
  { 162.488381164135f,    8.05388067181887f},
  { -124.915834966168f,    28.6372443815363f},
  { 113.510773712522f,    -0.511739843878896f},
  { 70.4233313318117f,    -30.3198053825930f},
  { 46.8625981346420f,    -5.57909417980254f},
  { 24.7981224733719f,    16.4909455786307f},
  { -38.7229948140994f,    -44.4837505748274f},
  { 152.084585287721f,    -29.2254045365174f},
  { -167.304000528815f,    -23.1560397945952f},
  { 72.4560659822285f,    -20.8066048257218f},
  { -55.5423615219973f,    12.5505249476043f},
  { -19.6933850082444f,    35.9602955722954f},
  { -134.766623841289f,    39.5080689884303f},
  { -15.2727802322559f,    -46.7878964947383f},
  { -89.7175282328454f,    -46.7286798080069f},
  { -60.6138432736476f,    -23.8982826269323f},
  { -149.091644381807f,    20.5547251786388f},
  { 60.3421121878982f,    36.0529968149371f},
  { -99.9316595181624f,    -7.38578556873742f},
  { -122.034990693475f,    7.97139081626441f},
  { 105.748473728738f,    15.3252359046220f},
  { -35.4025696491775f,    -84.3854808880661f},
  { 26.7735740421956f,    -33.1806002088145f},
  { 94.6841060652961f,    23.1803492839881f},
  { 171.625153471396f,    49.6540445940998f},
  { 88.2516843820150f,    24.2930397489084f},
  { -33.1546220704464f,    51.6665013475434f},
  { 87.6482826109908f,    49.4192000933288f},
  { -140.049931008346f,    -59.7221513665905f},
  { 52.8185287011349f,    -33.6164084237279f},
  { -4.40172660615861f,    51.2710474047081f},
  { 145.700773388766f,    0.899381262066922f},
  { 36.1049911437016f,    14.3973040145159f},
  { -42.9001951468629f,    16.0138213980345f},
  { 97.8291976608503f,    -70.0868202682595f},
  { -66.0187584209245f,    -63.8532240503450f},
  { 108.650680756173f,    70.7933120290820f},
  { -56.5966900246415f,    -72.3439115067856f},
  { -36.2488223584049f,    -16.9414719303444f},
  { -16.6405017769441f,    -34.6256533554036f},
  { -163.430984738764f,    -35.8057850835203f},
  { 82.6019585920972f,    35.6499021668553f},
  { 168.061846079562f,    6.89983983937935f},
  { -166.413663252251f,    35.6864722526444f},
  { 32.5665207747639f,    10.0427809432249f},
  { -103.233704229736f,    -32.0129382030327f},
  { 49.7963704112751f,    -5.29126034881621f},
  { -174.648121672758f,    26.8729608135584f},
  { -44.0842294780867f,    61.9871004665682f},
  { 111.217739382692f,    3.66948601795074f},
  { 66.2899222782653f,    -36.5228307195076f},
  { 76.3883576116169f,    34.8921254694351f},
  { 3.95019786606329f,    -13.5957731930607f},
  { -9.07498071158293f,    -3.82133122089724f},
  { 42.1773759202422f,    -50.7396987495080f},
  { 38.4381517756165f,    38.9615810738397f},
  { -0.537563103039008f,    9.64886123162515f},
  { 96.9607582963521f,    56.9523818491975f},
  { 34.2146843541318f,    -57.2195876291135f},
  { -156.474454001495f,    48.1883686959911f},
  { 33.3711182427180f,    22.8287767957295f},
  { -128.699466749412f,    -29.6150882158520f},
  { 131.327982268160f,    -74.2708525640276f},
  { 107.996552713759f,    32.3762158485532f},
  { 28.2164321748721f,    56.7970178496411f},
  { -162.404020792206f,    -2.96670033324950f},
  { 113.828598214547f,    -21.3329224215482f},
  { -149.172145925135f,    71.3240651302868f},
  { 119.271901331625f,    -42.4432931015365f},
  { -139.997645031477f,    78.9960299569436f},
  { 77.8083258840131f,    -1.40457196709822f},
  { 19.0667319957850f,    -22.2972545805126f},
  { -55.1932209951842f,    16.8795569835615f},
  { 33.3768113615278f,    55.5383915765304f},
  { 109.732167589226f,    80.3322296380082f},
  { -28.9887334000648f,    -61.8430180901115f},
  { -121.227253924109f,    -29.3455948068319f},
  { 156.698451570624f,    8.60653478555094f},
  { -43.6009634884839f,    -43.4220061963886f},
  { 139.521796275730f,    -52.8310463838783f},
  { 152.990644205338f,    30.2839970411682f},
  { 135.984557255795f,    -13.7405127277513f},
  { 84.9901769039789f,    -62.2961606045130f},
  { 87.0657695939716f,    56.4933195105540f},
  { -133.941304400983f,    63.9047949331664f},
  { 128.573761224004f,    53.0791421814110f},
  { 63.1110015599277f,    34.5481763503295f},
  { 112.248601542261f,    -61.3712450577394f},
  { 163.152365476209f,    -76.0270400005355f},
  { 45.8160833862418f,    23.8118942104723f},
  { 93.3281514844828f,    32.4383291745101f},
  { 165.604118982244f,    -27.6896589080522f},
  { 114.690890782159f,    22.9064874556324f},
  { 72.6131097594671f,    -59.0872490700160f},
  { 60.9988790177304f,    26.5383147207002f},
  { 105.494642257233f,    30.0894949274588f},
  { 43.3009788527933f,    31.7532672601516f},
  { -28.6539744932802f,    -9.63621267583399f},
  { -92.0547097651399f,    11.1933274190420f},
  { 79.5715338329986f,    -34.0482863848922f},
  { -36.5141785464958f,    48.8584186964107f},
  { 51.9623615797484f,    -71.3179796855643f},
  { -80.0586099987521f,    -67.4361974459431f},
  { 95.9351031866747f,    -11.1153542246964f},
  { -122.716312381896f,    15.1547044671039f},
  { -95.8905924956502f,    -18.7254166893063f},
  { 129.519157717588f,    -14.8559655348626f},
  { 50.9272061597869f,    -47.4373967714934f},
  { 46.8343460386301f,    -34.3440385315725f},
  { -165.738367514626f,    22.2428425196845f},
  { -24.5452410776222f,    -19.6374536913591f},
  { -6.56645881971502f,    75.1674418498854f},
  { -158.838154542796f,    -53.0630208901224f},
  { -122.274621909614f,    12.7351733201854f},
  { -162.218625049416f,    -19.8784005134237f},
  { -158.945653052613f,    7.84519427537937f},
  { -179.777272274969f,    -26.3878936420562f},
  { 150.547754489837f,    -36.9418485720427f},
  { -138.897273468550f,    48.9680771626971f},
  { -113.630957183924f,    -2.24672998227578f},
  { -155.503825287640f,    -25.3266254889109f},
  { 106.183436094186f,    -19.4332712625845f},
  { 12.9309902356300f,    13.5951648647855f},
  { 134.889872939928f,    19.2242684708714f},
  { 34.6126528849932f,    -70.7876279655545f},
  { 55.5714674317882f,    35.8760228674956f},
  { -141.932984776068f,    -27.5102009860739f},
  { 114.420422662769f,    6.75665632492799f},
  { 167.407725153728f,    -67.9198383343094f},
  { -70.0291882079876f,    47.4288679222888f},
  { -177.912328340121f,    9.40454145810874f},
  { -173.498224198871f,    29.5165426276179f},
  { 176.916400576272f,    -56.2755941203827f},
  { 120.200202553481f,    -48.8714421590451f},
  { -55.4787792300715f,    -21.6573621515482f},
  { -175.890030991578f,    59.1773281453812f},
  { -177.113016424922f,    12.0213196301989f},
  { -41.9278193889537f,    -55.4474832389765f},
  { 48.0058769043571f,    -9.87124521482449f},
  { -100.760058189617f,    43.3784892040924f},
  { -93.4402814563876f,    36.2372927389970f},
  { 21.8900496714200f,    32.3888224004639f},
  { -63.7115727639628f,    29.5089032004728f},
  { -178.239709370010f,    -70.3834088846708f},
  { 27.6452049521372f,    54.0724688056757f},
  { -164.727512693952f,    44.0830949127842f},
  { -9.94946043635073f,    59.3946332432259f},
  { -165.505404880318f,    -13.2801298732065f},
  { 167.810886121524f,    -29.9317647781702f},
  { 61.4451667950518f,    57.2699820156663f},
  { -179.138288543907f,    13.6745062509686f},
  { 84.4959672980107f,    27.5899207321931f},
  { -173.110163480257f,    51.3215487171266f},
  { -19.4964973055475f,    -34.4926208802630f},
  { 38.7391985272367f,    -38.8633228180360f},
  { 20.0143594703306f,    27.1739718092969f},
  { -108.469487538361f,    21.1074384523449f},
  { 87.1117481445096f,    15.6848988992053f},
  { -65.6199482888161f,    77.8573598792618f},
  { 111.293420387068f,    -19.7788279590554f},
  { -40.3813994629159f,    -30.0006313598990f},
  { 95.5476358358556f,    -33.2781782022872f},
  { -128.705965353472f,    -66.7025157444241f},
  { 164.427461456992f,    -49.3131717333017f},
  { -20.7260427161146f,    -11.4387148090826f},
  { -175.376587821218f,    -64.9251559757401f},
  { 139.152778177509f,    -17.4860115092893f},
  { 1.54616631297520f,    -15.6236031573140f},
  { 100.005384938745f,    19.5850919034989f},
  { -99.2078308942125f,    -38.0921168176831f},
  { 142.055467977318f,    66.2499444805101f},
  { 26.6174343608009f,    -44.3817617537126f},
  { -48.3016919297376f,    -28.3191133457150f},
  { -175.467926795690f,    42.9961766589098f},
  { -135.568289161065f,    -10.1357540200865f},
  { -3.71480289935651f,    -0.444023983564376f},
  { 73.9096369769102f,    2.81746779460127f},
  { 11.6852593681234f,    -20.0451644263499f},
  { 73.2711944960354f,    60.1615879529241f},
  { 106.347111287254f,    -7.02386391992281f},
  { -76.6410237301893f,    72.5571967965817f},
  { -133.130019750511f,    30.7129689203005f},
  { -29.8116013217315f,    9.94085139126176f},
  { 154.533707327410f,    -24.2385196867934f},
  { -134.186447453679f,    33.6956561480977f},
  { 65.7710883111514f,    -14.8177439793580f},
  { -108.262612101769f,    4.94251887542312f},
  { -38.2741227016836f,    41.5339575353950f},
  { -14.7547027931441f,    7.56989697482882f},
  { -107.420201565836f,    0.529293144238883f},
  { 152.664085594330f,    -16.0365699754986f},
  { -111.762225939198f,    -33.5173949541811f},
  { -87.0649330781807f,    -52.2272677709710f},
  { 77.5954190909323f,    -48.9534777937647f},
  { 39.4055978504186f,    -56.9765468818325f},
  { 21.9225934693067f,    37.0587839998051f},
  { 79.5114731702736f,    -61.4661289807893f},
  { -46.5458693245325f,    31.6966929725523f},
  { 58.4012848435277f,    17.8019145549491f},
  { -80.8358898473828f,    -32.9298584551120f},
  { 140.712137096608f,    86.0361264373848f},
  { -174.108259121316f,    68.9025410790662f},
  { -170.591555082236f,    -21.9318743602209f},
  { -49.7990078152236f,    -77.1728671427286f},
  { 14.4750196406747f,    16.0414099942838f},
  { -140.470797745258f,    0.781546484841279f},
  { 146.599099730143f,    -13.1112992066295f},
  { -23.6047370679558f,    33.5967227211758f},
  { -57.5288149601129f,    26.2998836066371f},
  { -28.9310728281296f,    15.3522589503249f},
  { 154.094370755319f,    -42.5306161497219f},
  { -51.0679059531933f,    11.3874706095592f},
  { -122.911705523276f,    -20.0504007601108f},
  { -22.7093665482818f,    -57.3706505700685f},
  { -2.71109280544668f,    45.1925179357727f},
  { -136.949448944816f,    21.5304768822415f},
  { -124.609707640591f,    25.4691819102283f},
  { -140.541225137491f,    -46.8014419947198f},
  { -165.001733184357f,    -64.0291737990266f},
  { -64.5816104370214f,    9.82876769247620f},
  { 104.720062440058f,    -21.9137791868593f},
  { 170.255801972836f,    4.77905668014385f},
  { -115.976137277195f,    -19.1912392224307f},
  { -151.539592303932f,    -7.01859833587053f},
  { 133.528328857256f,    21.9240660220047f},
  { -84.0187916854670f,    43.6879621991246f},
  { -103.980776997391f,    -56.6120488567637f},
  { 11.8323598186041f,    -34.2954326000185f},
  { 89.5433860544001f,    -53.3005800809702f},
  { 130.186812084391f,    35.3487204808272f},
  { -50.5430479592438f,    21.5017538617150f},
  { 103.350764705142f,    -15.3913250208690f},
  { 14.6114837660536f,    52.6437574909619f},
  { -38.9964483204503f,    -7.23184571800371f},
  { 57.5819011109703f,    -39.0514087346796f},
  { 68.1264220114729f,    -83.5896485002506f},
  { -33.6656609981838f,    35.9628016251027f},
  { -143.691645765731f,    67.7646464173432f},
  { 82.5814304697762f,    23.6309343558319f},
  { -93.1845512621207f,    -7.64238112779267f},
  { -86.7569677481960f,    -18.4318830036241f},
  { 0.732872695669594f,    72.9705478164780f},
  { 152.734323340050f,    -19.4648625788881f},
  { -172.143773335779f,    -6.78760475917541f},
  { 54.6566520548053f,    39.4676908478457f},
  { -47.7520516706459f,    29.2215996906535f},
  { -115.026273004267f,    -4.33911453177047f},
  { 163.898399632073f,    52.1498077435459f},
  { -133.040927315642f,    41.9857673596353f},
  { -170.613175829387f,    -62.9020417788055f},
  { -172.316065485796f,    -34.6845217202594f},
  { 88.0869396262997f,    -49.7356328172613f},
  { -24.6139539520614f,    59.4569820928223f},
  { -79.1267832395616f,    -13.4090787188007f},
  { 125.438454780333f,    30.8830006753207f},
  { 127.895076385871f,    -27.9870804214778f},
  { 133.085552317565f,    -8.49214325422109f},
  { 119.502473467522f,    56.9505318303055f},
  { 73.4644053028897f,    -41.6740891733659f},
  { 126.468448317849f,    81.1757779120381f},
  { -8.18155370067543f,    -25.6021293919061f},
  { -153.770702870743f,    36.7142333927591f},
  { -178.501863602135f,    -67.4231436311476f},
  { 156.869837240212f,    -10.0114476020334f},
  { 42.5912229557585f,    24.5724445130511f},
  { 107.926209137706f,    17.5276928029669f},
  { 173.051521099428f,    -20.5064153779646f},
  { -110.296459969854f,    -43.3283570410209f},
  { 25.1176794934572f,    -50.4076945751962f},
  { 38.4019148424203f,    12.4570104343930f},
  { -138.673206180676f,    -76.5218626422000f},
  { -55.3067290757868f,    -47.4315523343345f},
  { 28.9533333051057f,    -2.44797575526281f},
  { -65.2389996719823f,    -3.13210574865185f},
  { 119.029687386341f,    39.3762987946713f},
  { -101.419682101455f,    -86.9231546870455f},
  { -170.553733057658f,    65.0608604062709f},
  { 96.2423364011498f,    -16.2228122151928f},
  { 31.7249980899784f,    -7.97534228909585f},
  { -44.2368676807553f,    -1.57108452063893f},
  { 9.04119915448089f,    -15.7526171208103f},
  { 131.987403177409f,    -18.7718584095748f},
  { 107.519788226394f,    11.8998818678077f},
  { -5.49958792520265f,    -10.7525179486425f},
  { 25.8132036129778f,    59.7289071010347f},
  { 79.6729097100742f,    -26.3376124240628f},
  { 174.465557100434f,    53.7887245961459f},
  { -176.012797396614f,    -48.6101930163952f},
  { -167.168543200207f,    -27.6277009146372f},
  { 6.15399285522190f,    -0.783767264503392f},
  { 57.1276734134818f,    80.4009205719345f},
  { -108.150344129882f,    54.7031471375908f},
  { -159.376464624686f,    -50.3449698428696f},
  { 117.707525386214f,    60.3845694113033f},
  { -172.113794253463f,    -37.2867298418667f},
  { -128.421308293024f,    60.5282670494810f},
  { -37.4698178266945f,    -47.1972487303289f},
  { -65.4512957419604f,    -32.2923148985652f},
  { -66.3658114737764f,    46.7921897392182f},
  { 104.958453739604f,    21.8133853873883f},
  { -132.879614656772f,    13.8914614742006f},
  { -79.9955395909424f,    -18.2902363364927f},
  { 17.9614395650914f,    -67.6244762744038f},
  { -29.3078264956900f,    -48.3475185135689f},
  { -81.0161085704965f,    -41.1778404983141f},
  { -95.9175884110731f,    59.1705074113635f},
  { -39.7446064926241f,    16.3040362311791f},
  { 147.119844853691f,    33.0285706666262f},
  { 60.0288671891997f,    -24.9595067046181f},
  { -32.6860017161502f,    -36.2829582706797f},
  { 24.1789800172791f,    29.4859112072366f},
  { -178.219697866399f,    -80.6877733100478f},
  { -84.1617747489423f,    -43.2337163318954f},
  { -85.5630647355345f,    -40.8366585812746f},
  { -113.388521595968f,    82.7989059125011f},
  { -55.1267117809535f,    -7.29373268220492f},
  { 133.250329288504f,    16.8881865235000f},
  { -107.961051473808f,    -24.0228145991868f},
  { 164.432128297314f,    4.90522458407042f},
  { -89.5012221604435f,    -13.4159810169254f},
  { -125.556034668846f,    -45.3087408610681f},
  { -58.0114905052303f,    -27.5523328282382f},
  { -158.237142569811f,    40.7409754806145f},
  { 118.917907939359f,    19.1665682121696f},
  { -105.405579716753f,    -25.0433519362724f},
  { 94.2015700744315f,    12.8168202594319f},
  { -79.4087867763458f,    -48.7651641611959f},
  { 19.9482894940020f,    46.8440961812710f},
  { 24.8691125457782f,    -11.2451628345406f},
  { -108.643180438992f,    8.65819569512514f},
  { 59.3252254408626f,    -5.60409567495968f},
  { -172.511516834657f,    12.8921053093270f},
  { 45.6080639463682f,    -40.6098268184209f},
  { 7.15687431671250f,    23.3505065986695f},
  { -75.3437589944260f,    6.31485645641428f},
  { 153.451055754603f,    -7.20267264318446f},
  { -9.94855702929540f,    53.4986707453079f},
  { -168.411979797147f,    -36.8015250259757f},
  { -0.799726364418894f,    59.2858521883024f},
  { -97.3922456054241f,    13.8595123680490f},
  { 88.8850148661397f,    -83.4712027190460f},
  { -77.3096194305693f,    17.9273723726398f},
  { -7.85848363367946f,    -43.6976310463691f},
  { -46.1346483702354f,    46.3507622731438f},
  { -8.67388278035287f,    27.0444739071031f},
  { 41.5426666135085f,    -8.99338621861224f},
  { -116.135286667235f,    16.1071707647558f},
  { 6.38179764536891f,    37.3416834334259f},
  { -164.762380750018f,    18.9225482580957f},
  { 132.487201425759f,    73.7738115647869f},
  { 1.15753052625121f,    0.663548155160963f},
  { 104.363687084037f,    -4.80831175072516f},
  { 64.4673650037460f,    -69.3752118427466f},
  { 129.194120991357f,    -12.4768309397115f},
  { 154.629697019364f,    38.6839648047261f},
  { 15.2580223692863f,    -29.7751720913189f},
  { -86.4968747325774f,    20.8158861212099f},
  { -96.8778651971005f,    49.5920610875628f},
  { -167.603728186596f,    38.9407106035933f},
  { 141.529368437040f,    23.5703073598212f},
  { 39.5450131586291f,    -10.4451589954151f},
  { 175.997718668835f,    42.3688490538350f},
  { 54.7338334947230f,    3.65129060305555f},
  { -134.482551305967f,    -26.5767779026310f},
  { 73.4678014575678f,    -67.8898727553161f},
  { 132.168306708083f,    -11.3019720644799f},
  { -77.3778769241396f,    8.22517253582149f},
  { -69.7997325745604f,    -11.7209972970157f},
  { -156.512505776522f,    26.9944942821933f},
  { -0.165085465545444f,    -20.8630251873377f},
  { -34.6978710546962f,    54.2624319034759f},
  { -123.565002581948f,    69.5324822064906f},
  { 152.919798244751f,    0.173996300641060f},
  { -179.694279771703f,    30.3083129268004f},
  { 29.1408253425372f,    -21.8861628329693f},
  { 162.828246561808f,    -21.0992865661484f},
  { -15.0334872524158f,    -24.6497602994111f},
  { -120.597354185586f,    -2.15530287839289f},
  { -91.8974068963703f,    7.22647820590291f},
  { 66.8432731388005f,    -20.6488253537474f},
  { -170.294433441481f,    58.9904455688244f},
  { -62.1761411157911f,    -2.00836092813275f},
  { -111.603834331038f,    31.4166279696797f},
  { 86.0372938467184f,    1.96489155465916f},
  { -14.7159320794333f,    20.8494572120680f},
  { -21.7942410499041f,    -24.5956273405416f},
  { 18.2371638899974f,    -25.7502281669099f},
  { 142.822572535888f,    -0.00122304153082434f},
  { 139.483094585917f,    21.3314132697898f},
  { -42.7664429579734f,    6.64727001332420f},
  { -54.3658567488516f,    3.32005131727536f},
  { 63.8285518773537f,    -53.9965427199589f},
  { 83.5005497017478f,    -33.0953698492831f},
  { 92.5018558968723f,    -27.7392465879266f},
  { 20.2675623465217f,    -47.4929971322362f},
  { -167.387173997461f,    -31.2669255587668f},
  { -66.0131051518908f,    -66.4040017990331f},
  { 122.908845928617f,    -31.0253615230631f},
  { 127.392106506069f,    -7.97219361674645f},
  { 44.0104771038730f,    -53.7396428003661f},
  { -9.68987441051391f,    78.6666027424834f},
  { -137.882954081626f,    -0.0285226663302499f},
  { -94.7899278760962f,    -51.5582786382794f},
  { -96.4703346113033f,    18.6807279790964f},
  { 81.2123303808476f,    29.8154678381831f},
  { 99.3347835408073f,    -56.8277725472984f},
  { -85.8018678375787f,    5.10290338239701f},
  { 157.704021929860f,    -62.5546128123649f},
  { 121.684333369008f,    54.3143700428141f},
  { 56.2360894166961f,    -0.777785530040157f},
  { 169.215147039626f,    43.7727508342241f},
  { 163.213382829212f,    42.1633687134087f},
  { 30.8008736524250f,    33.6433037634648f},
  { -56.4882672995278f,    42.6373748511893f},
  { -136.314478993669f,    16.4608409303940f},
  { -139.356631861307f,    -25.0428035063905f},
  { 73.1196647095645f,    16.5694183555613f},
  { -38.0977174653441f,    73.4020862737075f},
  { -118.120410861184f,    -47.7029518033688f},
  { 151.457549180516f,    65.5573721294340f},
  { 115.964414039319f,    -37.7835938413853f},
  { 84.6397401715521f,    6.18243896994417f},
  { 64.6018606060384f,    -34.1059873448295f},
  { 38.1251419449371f,    75.6357065027583f},
  { -83.9279804430878f,    70.7110892573056f},
  { -162.488381164135f,    -8.05388067181887f},
  { 124.915834966168f,    -28.6372443815363f},
  { -113.510773712522f,    0.511739843878896f},
  { -70.4233313318117f,    30.3198053825930f},
  { -46.8625981346420f,    5.57909417980254f},
  { -24.7981224733719f,    -16.4909455786307f},
  { 38.7229948140994f,    44.4837505748274f},
  { -152.084585287721f,    29.2254045365174f},
  { 167.304000528815f,    23.1560397945952f},
  { -72.4560659822285f,    20.8066048257218f},
  { 55.5423615219973f,    -12.5505249476043f},
  { 19.6933850082444f,    -35.9602955722954f},
  { 134.766623841289f,    -39.5080689884303f},
  { 15.2727802322559f,    46.7878964947383f},
  { 89.7175282328454f,    46.7286798080069f},
  { 60.6138432736476f,    23.8982826269323f},
  { 149.091644381807f,    -20.5547251786388f},
  { -60.3421121878982f,    -36.0529968149371f},
  { 99.9316595181624f,    7.38578556873742f},
  { 122.034990693475f,    -7.97139081626441f},
  { -105.748473728738f,    -15.3252359046220f},
  { 35.4025696491775f,    84.3854808880661f},
  { -26.7735740421956f,    33.1806002088145f},
  { -94.6841060652961f,    -23.1803492839881f},
  { -171.625153471396f,    -49.6540445940998f},
  { -88.2516843820150f,    -24.2930397489084f},
  { 33.1546220704464f,    -51.6665013475434f},
  { -87.6482826109908f,    -49.4192000933288f},
  { 140.049931008346f,    59.7221513665905f},
  { -52.8185287011349f,    33.6164084237279f},
  { 4.40172660615861f,    -51.2710474047081f},
  { -145.700773388766f,    -0.899381262066922f},
  { -36.1049911437016f,    -14.3973040145159f},
  { 42.9001951468629f,    -16.0138213980345f},
  { -97.8291976608503f,    70.0868202682595f},
  { 66.0187584209245f,    63.8532240503450f},
  { -108.650680756173f,    -70.7933120290820f},
  { 56.5966900246415f,    72.3439115067856f},
  { 36.2488223584049f,    16.9414719303444f},
  { 16.6405017769441f,    34.6256533554036f},
  { 163.430984738764f,    35.8057850835203f},
  { -82.6019585920972f,    -35.6499021668553f},
  { -168.061846079562f,    -6.89983983937935f},
  { 166.413663252251f,    -35.6864722526444f},
  { -32.5665207747639f,    -10.0427809432249f},
  { 103.233704229736f,    32.0129382030327f},
  { -49.7963704112751f,    5.29126034881621f},
  { 174.648121672758f,    -26.8729608135584f},
  { 44.0842294780867f,    -61.9871004665682f},
  { -111.217739382692f,    -3.66948601795074f},
  { -66.2899222782653f,    36.5228307195076f},
  { -76.3883576116169f,    -34.8921254694351f},
  { -3.95019786606329f,    13.5957731930607f},
  { 9.07498071158293f,    3.82133122089724f},
  { -42.1773759202422f,    50.7396987495080f},
  { -38.4381517756165f,    -38.9615810738397f},
  { 0.537563103039008f,    -9.64886123162515f},
  { -96.9607582963521f,    -56.9523818491975f},
  { -34.2146843541318f,    57.2195876291135f},
  { 156.474454001495f,    -48.1883686959911f},
  { -33.3711182427180f,    -22.8287767957295f},
  { 128.699466749412f,    29.6150882158520f},
  { -131.327982268160f,    74.2708525640276f},
  { -107.996552713759f,    -32.3762158485532f},
  { -28.2164321748721f,    -56.7970178496411f},
  { 162.404020792206f,    2.96670033324950f},
  { -113.828598214547f,    21.3329224215482f},
  { 149.172145925135f,    -71.3240651302868f},
  { -119.271901331625f,    42.4432931015365f},
  { 139.997645031477f,    -78.9960299569436f},
  { -77.8083258840131f,    1.40457196709822f},
  { -19.0667319957850f,    22.2972545805126f},
  { 55.1932209951842f,    -16.8795569835615f},
  { -33.3768113615278f,    -55.5383915765304f},
  { -109.732167589226f,    -80.3322296380082f},
  { 28.9887334000648f,    61.8430180901115f},
  { 121.227253924109f,    29.3455948068319f},
  { -156.698451570624f,    -8.60653478555094f},
  { 43.6009634884839f,    43.4220061963886f},
  { -139.521796275730f,    52.8310463838783f},
  { -152.990644205338f,    -30.2839970411682f},
  { -135.984557255795f,    13.7405127277513f},
  { -84.9901769039789f,    62.2961606045130f},
  { -87.0657695939716f,    -56.4933195105540f},
  { 133.941304400983f,    -63.9047949331664f},
  { -128.573761224004f,    -53.0791421814110f},
  { -63.1110015599277f,    -34.5481763503295f},
  { -112.248601542261f,    61.3712450577394f},
  { -163.152365476209f,    76.0270400005355f},
  { -45.8160833862418f,    -23.8118942104723f},
  { -93.3281514844828f,    -32.4383291745101f},
  { -165.604118982244f,    27.6896589080522f},
  { -114.690890782159f,    -22.9064874556324f},
  { -72.6131097594671f,    59.0872490700160f},
  { -60.9988790177304f,    -26.5383147207002f},
  { -105.494642257233f,    -30.0894949274588f},
  { -43.3009788527933f,    -31.7532672601516f},
  { 28.6539744932802f,    9.63621267583399f},
  { 92.0547097651399f,    -11.1933274190420f},
  { -79.5715338329986f,    34.0482863848922f},
  { 36.5141785464958f,    -48.8584186964107f},
  { -51.9623615797484f,    71.3179796855643f},
  { 80.0586099987521f,    67.4361974459431f},
  { -95.9351031866747f,    11.1153542246964f},
  { 122.716312381896f,    -15.1547044671039f},
  { 95.8905924956502f,    18.7254166893063f},
  { -129.519157717588f,    14.8559655348626f},
  { -50.9272061597869f,    47.4373967714934f},
  { -46.8343460386301f,    34.3440385315725f},
  { 165.738367514626f,    -22.2428425196845f},
  { 24.5452410776222f,    19.6374536913591f},
  { 6.56645881971502f,    -75.1674418498854f},
  { 158.838154542796f,    53.0630208901224f},
  { 122.274621909614f,    -12.7351733201854f},
  { 162.218625049416f,    19.8784005134237f},
  { 158.945653052613f,    -7.84519427537937f},
  { 179.777272274969f,    26.3878936420562f},
  { -150.547754489837f,    36.9418485720427f},
  { 138.897273468550f,    -48.9680771626971f},
  { 113.630957183924f,    2.24672998227578f},
  { 155.503825287640f,    25.3266254889109f},
  { -106.183436094186f,    19.4332712625845f},
  { -12.9309902356300f,    -13.5951648647855f},
  { -134.889872939928f,    -19.2242684708714f},
  { -34.6126528849932f,    70.7876279655545f},
  { -55.5714674317882f,    -35.8760228674956f},
  { 141.932984776068f,    27.5102009860739f},
  { -114.420422662769f,    -6.75665632492799f},
  { -167.407725153728f,    67.9198383343094f},
  { 70.0291882079876f,    -47.4288679222888f},
  { 177.912328340121f,    -9.40454145810874f},
  { 173.498224198871f,    -29.5165426276179f},
  { -176.916400576272f,    56.2755941203827f},
  { -120.200202553481f,    48.8714421590451f},
  { 55.4787792300715f,    21.6573621515482f},
  { 175.890030991578f,    -59.1773281453812f},
  { 177.113016424922f,    -12.0213196301989f},
  { 41.9278193889537f,    55.4474832389765f},
  { -48.0058769043571f,    9.87124521482449f},
  { 100.760058189617f,    -43.3784892040924f},
  { 93.4402814563876f,    -36.2372927389970f},
  { -21.8900496714200f,    -32.3888224004639f},
  { 63.7115727639628f,    -29.5089032004728f},
  { 178.239709370010f,    70.3834088846708f},
  { -27.6452049521372f,    -54.0724688056757f},
  { -105.215174804733f,    43.8651167036432f},
  { -95.8355564202385f,    -30.0969788262413f},
  { 136.680155926050f,    70.4382173792202f},
  { 69.8614318196901f,    57.8958243541804f},
  { -60.5532211164125f,    -14.9778349457056f},
  { -93.5370966736206f,    76.2988863571906f},
  { -27.6987476661327f,    -4.87652814447257f},
  { -95.4854423222915f,    38.3479886073407f},
  { 115.907799848353f,    -50.9813536297129f},
  { 52.1688778766232f,    -37.3977758125295f},
  { -56.3085212932640f,    -56.7093308501247f},
  { -157.854682114479f,    17.1901900680136f},
  { -15.7038649827398f,    -2.78061596389676f},
  { -101.087938684479f,    -4.98116408424739f},
  { 21.1042268417709f,    19.9817529028294f},
  { 138.293517865579f,    -41.2761970145407f},
  { 33.4016877717366f,    4.63572731314592f},
  { 108.574293421831f,    14.3189115703482f},
  { 77.0035415584883f,    38.9021929491112f},
  { 150.241686975044f,    -66.4491904302668f},
  { 92.1599189843686f,    24.9876403976636f},
  { 25.7185001909227f,    46.1785184702480f},
  { 84.4887805200761f,    -74.3019729101659f},
  { -19.8640508953462f,    9.42107919115538f},
  { 141.546538673753f,    7.23480366290173f},
  { -74.8603930340358f,    18.5184080709352f},
  { 65.4014198085818f,    -39.7136037305857f},
  { 144.181412959797f,    -35.8450784784296f},
  { -94.8440620319955f,    46.8120276275259f},
  { 165.674771778856f,    44.6634398502118f},
  { 173.179004188486f,    -86.2587914331641f},
  { -2.93214337045507f,    -16.0703859913471f},
  { 60.9653682086765f,    -66.9192326102841f},
  { -61.2191158869755f,    -8.23406739348625f},
  { 7.31662889480752f,    16.2210309180213f},
  { -106.998382602301f,    -3.97139706100809f},
  { -140.854309724849f,    35.9985200114242f},
  { -160.580653440674f,    -58.7197782563616f},
  { 46.3176602256581f,    55.4112720585649f},
  { -137.080287258273f,    35.4433409335273f},
  { 16.1772543754189f,    -23.3742650202122f},
  { -174.796725466008f,    18.1923220619770f},
  { -124.964959170800f,    -35.9914084272573f},
  { -152.444259614377f,    -73.4551881302050f},
  { -179.445265339798f,    17.4194344783215f},
  { 32.0441464330633f,    58.6230936983008f},
  { 144.505510034418f,    18.0053449990008f},
  { 127.736323733658f,    -1.79732786476893f},
  { 49.6226072894642f,    -8.10929674720791f},
  { 67.5781056918769f,    -24.9035688605843f},
  { -63.6921710210550f,    -47.7580229896147f},
  { 61.8694115053249f,    -4.98850536087643f},
  { -139.612724141299f,    -35.8161533257883f},
  { -20.6562221727002f,    -29.9258526178061f},
  { 146.733440153129f,    -7.68201583442896f},
  { -87.4876074803720f,    3.06695419975881f},
  { -92.2679442394988f,    20.9807341938284f},
  { 112.096907873455f,    66.2270699770113f},
  { 99.8655302178104f,    -8.23891588156307f},
  { -48.9984281546412f,    -68.5214881547318f},
  { -178.772175316906f,    50.4643387466079f},
  { 22.9333309793704f,    54.3975629091681f},
  { -121.079999361715f,    -49.7523893131743f},
  { -149.637655874158f,    -28.7704710229944f},
  { -150.423522899402f,    -57.5619483596157f},
  { 64.5329051442387f,    41.5189022360922f},
  { -165.484022557630f,    -38.0266884111826f},
  { 156.503564672780f,    30.6914454570282f},
  { 103.883661876949f,    -29.8281732827046f},
  { -92.6899519309944f,    -44.7438176051669f},
  { -149.974780316198f,    42.8250215164701f},
  { -149.940879848802f,    30.8494977671562f},
  { 120.827298873383f,    31.9049965614365f},
  { 97.1846472883089f,    25.0238403215857f},
  { -169.141681612328f,    -25.0193940575390f},
  { 22.5837960397480f,    13.6350856791287f},
  { -26.2879833456298f,    79.1571196520320f},
  { 158.834611601758f,    24.4346959066494f},
  { 165.514706716188f,    60.7573612948984f},
  { -29.0358071492329f,    39.7102213182724f},
  { -136.155801446463f,    -4.32132968225788f},
  { 122.601065723553f,    7.64019416749633f},
  { 73.2670213616701f,    -53.9583113376126f},
  { 53.3014519022048f,    -0.272878412898698f},
  { -42.8764771111489f,    31.7572502403651f},
  { -152.968191350858f,    -36.2473003814680f},
  { 15.7977605231503f,    12.8635748671880f},
  { -79.1002695251783f,    -35.9546848676500f},
  { 168.599075596622f,    -50.4439868425900f},
  { 43.8618728087655f,    -24.6024709995092f},
  { 84.0477012791974f,    -2.38394815649296f},
  { -127.381059583027f,    -42.3495106751389f},
  { -103.607843757590f,    17.7545706678341f},
  { -23.8083048920801f,    -6.79342799438821f},
  { 172.345939111182f,    3.15623551924696f},
  { 161.540554431835f,    -3.07650098814593f},
  { -89.7755335989624f,    -17.0280165825829f},
  { 37.6494948028893f,    56.9387780024591f},
  { 138.951685461449f,    79.6316581886646f},
  { -45.2687933597244f,    -26.5240156911605f},
  { -142.922894328093f,    -35.9286260956624f},
  { 175.213284046906f,    24.9496253561397f},
  { -77.8377815428151f,    36.1235480969487f},
  { -129.080566342209f,    30.4858187200167f},
  { 94.7705465544761f,    26.7060752292626f},
  { 100.935419068100f,    54.5788757123567f},
  { 49.7513862274307f,    -1.23630648043532f},
  { -103.807370129368f,    -27.5168039633764f},
  { 166.354873916908f,    -10.5733425837884f},
  { -36.2817064968225f,    29.8425651750705f},
  { 33.9570969152368f,    32.8462651097259f},
  { 11.5544231220092f,    42.5007861370517f},
  { -60.4783924250340f,    15.5791938547136f},
  { 42.8785187313801f,    -12.2740004696889f},
  { -82.8838718915494f,    5.23149737166266f},
  { 106.541215120446f,    -63.2066150700692f},
  { -120.652410308458f,    45.9790382725339f},
  { 90.6228086044930f,    22.5687134079145f},
  { 24.1990424968209f,    64.9058206791233f},
  { -34.0448734859848f,    -42.0306178595419f},
  { -18.3633251743701f,    17.0677062863682f},
  { 72.0758519762432f,    68.3964564966348f},
  { 134.836238032345f,    14.6154545430050f},
  { 70.6556916814712f,    -35.2436106179536f},
  { -19.5768720984680f,    -49.9268864595524f},
  { 98.9936165823516f,    10.0803118111226f},
  { 127.061067683758f,    -22.6459433000054f},
  { 5.04677337544235f,    -60.9522970306370f},
  { 176.551513024144f,    -24.7215301359737f},
  { -43.1871188262524f,    22.0308759745299f},
  { 93.0160472697171f,    0.608915826636266f},
  { -94.3643235497202f,    24.5783795369254f},
  { 16.3144408285961f,    5.99285284765303f},
  { 14.9189299439464f,    -57.3896256077255f},
  { 177.748551692504f,    -45.7410164786008f},
  { 60.8780201150254f,    -71.8943445614376f},
  { 24.5728573381381f,    39.3001664803112f},
  { -12.4610505171885f,    17.1314992696109f},
  { 116.778953914737f,    -77.9374137858883f},
  { -75.7399091752730f,    -26.9872928089807f},
  { 26.7115174951351f,    -9.24512617358828f},
  { -85.9607310896728f,    36.0159577866263f},
  { 93.5067108445391f,    41.2677039707420f},
  { 112.991873948084f,    59.7521519675659f},
  { 7.27234036880005f,    -83.7964884854227f},
  { -81.9156984992198f,    -5.19284892989513f},
  { -123.929934660503f,    10.3694878170491f},
  { 106.275537717527f,    36.6744087727640f},
  { -63.2859759351422f,    13.2836050193709f},
  { 100.214810751667f,    52.0073116800525f},
  { -113.881443643278f,    17.8029556568936f},
  { 119.396321119065f,    -32.6348293383468f},
  { 145.208847654029f,    -20.5612097287484f},
  { -130.713062709805f,    -15.9305432103548f},
  { -22.5033040405404f,    13.8651185512754f},
  { -161.350329224962f,    41.3424641546771f},
  { 161.446553153532f,    -9.49421865641912f},
  { 82.7650203433960f,    -21.2306274005472f},
  { 113.528419281680f,    -35.4183575038600f},
  { 138.471405273428f,    -6.74983031251173f},
  { -120.694944077280f,    3.02869649970139f},
  { -155.416890572207f,    -47.5597127200273f},
  { -50.1362438714596f,    44.7563993680868f},
  { 28.2489730405064f,    -26.9303788527913f},
  { 126.339190849170f,    -42.7227843218745f},
  { -54.0819105267844f,    -52.5712937196515f},
  { 90.2918762683113f,    9.30769170602622f},
  { 136.617562187771f,    -4.25007017636107f},
  { 139.078245365887f,    -3.35544603345923f},
  { -96.6148850358957f,    2.85223335876547f},
  { 171.132881782885f,    -34.5508156260187f},
  { -22.6273557262694f,    40.9680026987570f},
  { 154.895052632642f,    16.3593291014409f},
  { -17.7331672616115f,    73.6958742266972f},
  { 166.583528998528f,    -0.485166461422033f},
  { 128.828030564581f,    24.1395308367399f},
  { 148.402917859974f,    -28.0128544675960f},
  { -113.288592747368f,    44.7218775208384f},
  { -21.6577871060380f,    27.1776895454452f},
  { 154.143196295691f,    13.9264213239911f},
  { -12.8502135206102f,    4.09670383933257f},
  { 130.747390284827f,    -6.95856837512662f},
  { -72.2611502604325f,    -40.0119484996736f},
  { 25.3036807194395f,    -62.8547609689638f},
  { -170.870198902503f,    18.4230348800415f},
  { 6.50852515338894f,    -30.5124736995394f},
  { -119.657084773905f,    75.1227323266721f},
  { 50.1916630381715f,    -32.0784453558476f},
  { -73.9026013866870f,    -65.6361648135907f},
  { -173.474557759099f,    -14.5653408704797f},
  { 15.7880397163586f,    62.5599925017521f},
  { -97.2854451002615f,    -35.8663816720691f},
  { 105.029318766907f,    51.6647264221959f},
  { -90.4750835825850f,    -30.7108321642385f},
  { -166.028862258135f,    7.17587804486720f},
  { 83.4724282309747f,    -0.126768783806846f},
  { -161.653454200476f,    -12.0647098901493f},
  { 98.1430789310052f,    -45.7421152009923f},
  { -124.518370570031f,    -28.5753409049472f},
  { -106.457910835656f,    -61.6987748035781f},
  { 13.4224747430656f,    -47.6684832418540f},
  { -162.169185623680f,    25.0368068353602f},
  { -81.7109272530489f,    -52.1954010733911f},
  { -127.475856895268f,    65.8814387634490f},
  { -77.8875747223732f,    10.8789198056757f},
  { -29.8260463751248f,    -88.6657911823910f},
  { 4.96270401145955f,    14.3120567584373f},
  { 71.2418505855464f,    -8.73282633052416f},
  { 15.9344054315540f,    38.0992307947422f},
  { -61.8478325288853f,    44.8555495611701f},
  { 65.2986890280926f,    -56.8649853557123f},
  { -159.148493882485f,    -3.27420844087041f},
  { -130.203805966514f,    4.45218628135222f},
  { -104.877511468810f,    49.4345332060033f},
  { -35.0409984574206f,    45.8573104295138f},
  { 16.1481461484859f,    -49.3186212669315f},
  { -85.6242487896659f,    47.4781848612267f},
  { -4.46897634342862f,    -35.1839636966442f},
  { 144.963207555960f,    38.8036786690712f},
  { 68.7206361250173f,    -6.14832949603732f},
  { 15.0906944546390f,    41.1697859129614f},
  { -171.574012017929f,    -12.4901771833085f},
  { 167.534120376514f,    -19.7613088814185f},
  { -128.038990026004f,    54.8078676798206f},
  { 90.4331457971108f,    -69.1363508010318f},
  { -112.274650257390f,    -28.6987884800029f},
  { -107.275928916695f,    11.1471817637867f},
  { -0.382205643918457f,    62.9192815238565f},
  { -90.5229862494441f,    59.6902917787608f},
  { 39.5209993315536f,    -54.1423142064665f},
  { 52.5784779057485f,    63.0459726568391f},
  { 119.477101207120f,    -61.3742151086062f},
  { 177.496475506008f,    30.5733883034465f},
  { -172.769599397166f,    1.88232973009511f},
  { 22.2872692586157f,    -21.5914090608793f},
  { -95.7862838874942f,    30.5178830923288f},
  { 177.729350808644f,    -27.8052852867651f},
  { -146.697063713224f,    18.3131547706309f},
  { -1.96959668534852f,    -3.96037245960534f},
  { -123.703113187684f,    -64.6707050432905f},
  { 129.045418668238f,    -57.5944770922965f},
  { 57.0241684537285f,    -58.8099112975414f},
  { 0.00202394730670824f,    52.8225725186769f},
  { -31.0099065322919f,    45.0832534906829f},
  { -170.261099344183f,    -46.8187475208563f},
  { -175.917403936639f,    -35.5652390941893f},
  { 56.8900664620558f,    -15.0267102275093f},
  { 33.2645845339635f,    -5.44150430705723f},
  { 27.7617619515453f,    2.21417759870943f},
  { 72.3853959463809f,    -39.3344249117212f},
  { 109.779060633845f,    56.5255088065839f},
  { 111.756093987444f,    -9.36521337404056f},
  { 35.6186976488097f,    27.7476731067727f},
  { 9.99681660713551f,    36.9700490588779f},
  { 62.9950546882047f,    -25.1748632066066f},
  { -91.9321249498500f,    -11.1696106264579f},
  { 179.957469951699f,    47.8829462291547f},
  { 128.344148815471f,    2.97585586937439f},
  { -161.207900672177f,    6.12812228232499f},
  { -30.1081982193371f,    -7.61701647566548f},
  { 57.1766461401543f,    5.09173079720226f},
  { -174.883440773261f,    -4.18146345267463f},
  { 78.8532783544636f,    25.2421073284771f},
  { -58.5688346294240f,    17.8419060666222f},
  { 0.935560822729587f,    -33.7603814155747f},
  { -78.9484335324145f,    45.1808250438887f},
  { -72.3114482510242f,    45.2062573804445f},
  { -52.4237374538326f,    -45.6497541245122f},
  { -132.161936921178f,    -23.9639038357658f},
  { -156.839539652569f,    43.9080257979142f},
  { 144.347833604310f,    43.4283641158210f},
  { -17.2719713165641f,    -16.1597234904786f},
  { -100.420963381379f,    -12.9909850230568f},
  { 128.744878762878f,    18.4930970740153f},
  { -77.7471758439369f,    21.3148193094379f},
  { 40.7695991125187f,    20.2438787827193f},
  { -6.20938187012891f,    -5.32899418522077f},
  { 36.8573685230626f,    -20.8017030701953f},
  { -81.0155366641506f,    -11.2541023571857f},
  { -109.188626473334f,    -2.00243587629306f},
  { 154.814419200873f,    70.7779622363148f},
  { 33.6609746822056f,    30.1559360103775f},
  { -179.441935010162f,    23.5097795291665f},
  { -148.172265260398f,    -16.8123240576211f},
  { -172.375502388897f,    -42.8836108425672f},
  { 144.784341581725f,    -60.5108434187774f},
  { -57.5056573135436f,    -33.8222572559338f},
  { -129.923123496210f,    50.4579355241053f},
  { -62.8027635978540f,    63.7609145194176f},
  { -158.271054982606f,    -16.3663929954881f},
  { 15.1091173914599f,    -33.5233598265761f},
  { 65.0851896484675f,    -49.6479518177324f},
  { 49.2706001950190f,    32.9106804222341f},
  { -76.1003939969667f,    -41.3394871207568f},
  { -46.7290273264954f,    -0.193620979858986f},
  { -26.9549362401683f,    -26.6554481563509f},
  { 36.1292450045000f,    53.4529386443691f},
  { 140.047121617448f,    -23.5812210110604f},
  { -7.49688284500747f,    9.84843258567302f},
  { 9.37972611180582f,    31.6892284109843f},
  { 164.106725148625f,    15.1747336888820f},
  { -86.7404794287501f,    -4.57394080280570f},
  { -124.562680104667f,    -48.3504266050628f},
  { 156.750210035600f,    4.30521520726145f},
  { 97.0527985136191f,    39.8289372152604f},
  { 155.696955095202f,    -1.59346610389211f},
  { 66.6146412083409f,    -31.2827363025915f},
  { 130.556949619167f,    -1.52959076015471f},
  { -69.4494021017328f,    22.7379580031879f},
  { -140.157378408396f,    -30.2167819538330f},
  { 86.4778135872908f,    -38.5935481119565f},
  { 178.404261237726f,    55.6904267576167f},
  { 156.460292367202f,    -51.4960402802874f},
  { 22.8612527020079f,    -44.7580479297380f},
  { -109.741965952875f,    2.65925609217010f},
  { -65.8422475319093f,    -10.3176741326118f},
  { 108.266688342554f,    6.03903482507053f},
  { -75.1192875814015f,    -9.61210206641801f},
  { -27.2559806808950f,    -50.4850769636584f},
  { -67.4755611620996f,    -52.0381077357589f},
  { -68.4304225467123f,    51.0166990228247f},
  { 144.122962436238f,    -6.00590222646529f},
  { 149.672396304396f,    76.2364358504411f},
  { 71.8883608538326f,    52.1381502921373f},
  { 161.788471518592f,    -56.0837474260228f},
  { -32.7101346566735f,    11.1927820553189f},
  { -173.086039724955f,    -39.9975928769987f},
  { 79.5706092013376f,    62.6382435502732f},
  { 69.6895180577746f,    -19.7168775786590f},
  { 176.064487967049f,    21.1721428100298f},
  { -141.032282628358f,    -18.8530831842397f},
  { 144.338317300475f,    -11.1298237475361f},
  { -105.899412124273f,    -75.8523838767584f},
  { -22.9515934865897f,    -80.1594971878931f},
  { -118.757144836062f,    -27.9682341132440f},
  { 127.551328582089f,    -37.5210624219500f},
  { 86.8413866977111f,    -80.3363174622849f},
  { 122.854154912882f,    3.78938229915202f},
  { -109.905453489077f,    -26.5975356488901f},
  { 70.3516976082159f,    37.6814251147732f},
  { 142.573720170356f,    -50.3277243445087f},
  { -36.0675653656258f,    32.9261540352792f},
  { -101.941654121029f,    10.3126475069436f},
  { 146.310471122485f,    15.1253745998665f},
  { 107.193588438003f,    -28.8510004710438f},
  { -9.72788062498358f,    72.1634843106536f},
  { -156.880480080257f,    22.1059617727149f},
  { 80.1728291325780f,    15.9603576451761f},
  { -133.646983510901f,    21.1506860271258f},
  { 82.8751190954865f,    8.40765688220728f},
  { -178.563032654892f,    -12.1879546893370f},
  { -128.541194058955f,    -60.9821631804947f},
  { 20.2819641400888f,    -33.1079792854500f},
  { 110.684539622321f,    -28.1985163590085f},
  { 99.1098975612658f,    3.25034554474043f},
  { -75.4584122449589f,    -24.3788570913706f},
  { -33.3240195219665f,    26.8661758393090f},
  { 159.062234659561f,    65.2432000848629f},
  { -53.9190171853593f,    -31.7332538112601f},
  { -116.205678230079f,    27.3588310748960f},
  { 127.870748025818f,    50.2957857947368f},
  { -160.612702854140f,    44.3125005730389f},
  { -117.613670705265f,    -2.32674646373715f},
  { 123.472090260528f,    -1.61930318676770f},
  { 70.5735454721590f,    17.7720569817710f},
  { 120.431756415596f,    21.9969038952949f},
  { 142.332465726129f,    -21.8702915838201f},
  { -116.804222458172f,    10.4520160948530f},
  { -94.1248511951517f,    13.3618446372592f},
  { 148.392244016761f,    -39.6157220912829f},
  { 147.517872055114f,    2.80840274370882f},
  { -115.349701097361f,    59.0564626318734f},
  { 155.058316955685f,    22.6301812716343f},
  { -119.744963612390f,    -8.83066457957962f},
  { 150.272921649919f,    -25.7053577014398f},
  { 148.981996938551f,    13.3646841591136f},
  { 137.936724416296f,    -38.2308458853338f},
  { -19.4979859918782f,    -59.8998439634515f},
  { 11.2003468237118f,    2.01560831317327f},
  { -145.507630465237f,    -8.62558802700491f},
  { 62.5331712913180f,    -31.9229411081417f},
  { -104.912579120957f,    -11.3833126679549f},
  { -67.7417098929497f,    -3.79830049256292f},
  { -168.826245966527f,    5.82337811499330f},
  { 17.8441357421638f,    31.4452697518181f},
  { -18.8178386455198f,    5.57777472509964f},
  { -161.024025273861f,    37.9566900924630f},
  { -125.487043112079f,    -25.2353144519553f},
  { -136.869025785700f,    -34.3915719978409f},
  { 58.9363132237020f,    63.7745316280337f},
  { -40.6611614764462f,    -58.9510987406156f},
  { 88.2654086136712f,    -14.7330440620767f},
  { -74.8149556261145f,    34.0842765919950f},
  { 14.9654898678653f,    31.3887298041247f},
  { -49.8151084645483f,    63.5716285102151f},
  { 20.9836247970084f,    67.5938920364209f},
  { -89.5510900784065f,    63.6112338213990f},
  { -123.180070030349f,    44.1034477845373f},
  { 60.2251711875326f,    29.6485407584113f},
  { -2.45212983593625f,    23.6116876061397f},
  { -48.7781442897902f,    55.3382928476971f},
  { -159.828407158107f,    15.2383515725468f},
  { 132.778896060509f,    -71.3217107535002f},
  { 153.793225968158f,    41.7891906876392f},
  { -101.196670859493f,    -15.7135837566658f},
  { 138.754578154247f,    -27.2657500235597f},
  { -40.1859318726247f,    44.2883802386034f},
  { 172.586536326741f,    24.2398710100095f},
  { -95.0539432045797f,    21.5221885790897f},
  { 49.1933891010712f,    -13.3591602507128f},
  { 77.5958803046065f,    80.3685640760221f},
  { 78.6897550365339f,    59.8389518167176f},
  { -92.0565262758586f,    33.6690465234106f},
  { -127.042305539228f,    19.3212107434163f},
  { -25.7317465974836f,    -31.7831505250909f},
  { 87.5513563376567f,    30.7347971636243f},
  { 76.6928991133291f,    77.6418399883612f},
  { -65.2909850439137f,    -24.9584436688353f},
  { -166.822530962768f,    -41.2350298833982f},
  { 43.8860817397015f,    7.79893164572718f},
  { 36.2865809336855f,    2.77426437061402f},
  { 120.444209930878f,    -51.5867433398137f},
  { 32.2628267069358f,    -22.6709366139652f},
  { -89.3727380387949f,    19.6069546181195f},
  { 108.583485016369f,    -31.3171950839166f},
  { -74.7848251952670f,    -43.8651167036432f},
  { -84.1644435797616f,    30.0969788262413f},
  { 43.3198440739499f,    -70.4382173792202f},
  { 110.138568180310f,    -57.8958243541804f},
  { -119.446778883588f,    14.9778349457056f},
  { -86.4629033263795f,    -76.2988863571906f},
  { -152.301252333867f,    4.87652814447257f},
  { -84.5145576777085f,    -38.3479886073407f},
  { 64.0922001516470f,    50.9813536297129f},
  { 127.831122123377f,    37.3977758125295f},
  { -123.691478706736f,    56.7093308501247f},
  { -22.1453178855212f,    -17.1901900680136f},
  { -164.296135017260f,    2.78061596389676f},
  { -78.9120613155213f,    4.98116408424739f},
  { 158.895773158229f,    -19.9817529028294f},
  { 41.7064821344211f,    41.2761970145407f},
  { 146.598312228263f,    -4.63572731314592f},
  { 71.4257065781688f,    -14.3189115703482f},
  { 102.996458441512f,    -38.9021929491112f},
  { 29.7583130249564f,    66.4491904302668f},
  { 87.8400810156315f,    -24.9876403976636f},
  { 154.281499809077f,    -46.1785184702480f},
  { 95.5112194799239f,    74.3019729101659f},
  { -160.135949104654f,    -9.42107919115538f},
  { 38.4534613262471f,    -7.23480366290173f},
  { -105.139606965964f,    -18.5184080709352f},
  { 114.598580191418f,    39.7136037305857f},
  { 35.8185870402032f,    35.8450784784296f},
  { -85.1559379680045f,    -46.8120276275259f},
  { 14.3252282211436f,    -44.6634398502118f},
  { 6.82099581151372f,    86.2587914331641f},
  { -177.067856629545f,    16.0703859913471f},
  { 119.034631791324f,    66.9192326102841f},
  { -118.780884113024f,    8.23406739348625f},
  { 172.683371105192f,    -16.2210309180213f},
  { -73.0016173976989f,    3.97139706100809f},
  { -39.1456902751512f,    -35.9985200114242f},
  { -19.4193465593261f,    58.7197782563616f},
  { 133.682339774342f,    -55.4112720585649f},
  { -42.9197127417268f,    -35.4433409335273f},
  { 163.822745624581f,    23.3742650202122f},
  { -5.20327453399252f,    -18.1923220619770f},
  { -55.0350408291998f,    35.9914084272573f},
  { -27.5557403856226f,    73.4551881302050f},
  { -0.554734660201537f,    -17.4194344783215f},
  { 147.955853566937f,    -58.6230936983008f},
  { 35.4944899655817f,    -18.0053449990008f},
  { 52.2636762663416f,    1.79732786476893f},
  { 130.377392710536f,    8.10929674720791f},
  { 112.421894308123f,    24.9035688605843f},
  { -116.307828978945f,    47.7580229896147f},
  { 118.130588494675f,    4.98850536087643f},
  { -40.3872758587007f,    35.8161533257883f},
  { -159.343777827300f,    29.9258526178061f},
  { 33.2665598468711f,    7.68201583442896f},
  { -92.5123925196280f,    -3.06695419975881f},
  { -87.7320557605012f,    -20.9807341938284f},
  { 67.9030921265448f,    -66.2270699770113f},
  { 80.1344697821897f,    8.23891588156307f},
  { -131.001571845359f,    68.5214881547318f},
  { -1.22782468309418f,    -50.4643387466079f},
  { 157.066669020630f,    -54.3975629091681f},
  { -58.9200006382852f,    49.7523893131743f},
  { -30.3623441258417f,    28.7704710229944f},
  { -29.5764771005985f,    57.5619483596157f},
  { 115.467094855761f,    -41.5189022360922f},
  { -14.5159774423702f,    38.0266884111826f},
  { 23.4964353272202f,    -30.6914454570282f},
  { 76.1163381230512f,    29.8281732827046f},
  { -87.3100480690056f,    44.7438176051669f},
  { -30.0252196838023f,    -42.8250215164701f},
  { -30.0591201511977f,    -30.8494977671562f},
  { 59.1727011266166f,    -31.9049965614365f},
  { 82.8153527116912f,    -25.0238403215857f},
  { -10.8583183876716f,    25.0193940575390f},
  { 157.416203960252f,    -13.6350856791287f},
  { -153.712016654370f,    -79.1571196520320f},
  { 21.1653883982416f,    -24.4346959066494f},
  { 14.4852932838116f,    -60.7573612948984f},
  { -150.964192850767f,    -39.7102213182724f},
  { -43.8441985535369f,    4.32132968225788f},
  { 57.3989342764473f,    -7.64019416749633f},
  { 106.732978638330f,    53.9583113376126f},
  { 126.698548097795f,    0.272878412898698f},
  { -137.123522888851f,    -31.7572502403651f},
  { -27.0318086491423f,    36.2473003814680f},
  { 164.202239476850f,    -12.8635748671880f},
  { -100.899730474822f,    35.9546848676500f},
  { 11.4009244033777f,    50.4439868425900f},
  { 136.138127191234f,    24.6024709995092f},
  { 95.9522987208026f,    2.38394815649296f},
  { -52.6189404169727f,    42.3495106751389f},
  { -76.3921562424101f,    -17.7545706678341f},
  { -156.191695107920f,    6.79342799438821f},
  { 7.65406088881811f,    -3.15623551924696f},
  { 18.4594455681646f,    3.07650098814593f},
  { -90.2244664010376f,    17.0280165825829f},
  { 142.350505197111f,    -56.9387780024591f},
  { 41.0483145385507f,    -79.6316581886646f},
  { -134.731206640276f,    26.5240156911605f},
  { -37.0771056719068f,    35.9286260956624f},
  { 4.78671595309442f,    -24.9496253561397f},
  { -102.162218457185f,    -36.1235480969487f},
  { -50.9194336577915f,    -30.4858187200167f},
  { 85.2294534455239f,    -26.7060752292626f},
  { 79.0645809318999f,    -54.5788757123567f},
  { 130.248613772569f,    1.23630648043532f},
  { -76.1926298706323f,    27.5168039633764f},
  { 13.6451260830917f,    10.5733425837884f},
  { -143.718293503178f,    -29.8425651750705f},
  { 146.042903084763f,    -32.8462651097259f},
  { 168.445576877991f,    -42.5007861370517f},
  { -119.521607574966f,    -15.5791938547136f},
  { 137.121481268620f,    12.2740004696889f},
  { -97.1161281084506f,    -5.23149737166266f},
  { 73.4587848795541f,    63.2066150700692f},
  { -59.3475896915418f,    -45.9790382725339f},
  { 89.3771913955071f,    -22.5687134079145f},
  { 155.800957503179f,    -64.9058206791233f},
  { -145.955126514015f,    42.0306178595419f},
  { -161.636674825630f,    -17.0677062863682f},
  { 107.924148023757f,    -68.3964564966348f},
  { 45.1637619676550f,    -14.6154545430050f},
  { 109.344308318529f,    35.2436106179536f},
  { -160.423127901532f,    49.9268864595524f},
  { 81.0063834176484f,    -10.0803118111226f},
  { 52.9389323162424f,    22.6459433000054f},
  { 174.953226624558f,    60.9522970306370f},
  { 3.44848697585623f,    24.7215301359737f},
  { -136.812881173748f,    -22.0308759745299f},
  { 86.9839527302829f,    -0.608915826636266f},
  { -85.6356764502799f,    -24.5783795369254f},
  { 163.685559171404f,    -5.99285284765303f},
  { 165.081070056054f,    57.3896256077255f},
  { 2.25144830749611f,    45.7410164786008f},
  { 119.121979884975f,    71.8943445614376f},
  { 155.427142661862f,    -39.3001664803112f},
  { -167.538949482812f,    -17.1314992696109f},
  { 63.2210460852630f,    77.9374137858883f},
  { -104.260090824727f,    26.9872928089807f},
  { 153.288482504865f,    9.24512617358828f},
  { -94.0392689103273f,    -36.0159577866263f},
  { 86.4932891554609f,    -41.2677039707420f},
  { 67.0081260519160f,    -59.7521519675659f},
  { 172.727659631200f,    83.7964884854227f},
  { -98.0843015007802f,    5.19284892989513f},
  { -56.0700653394972f,    -10.3694878170491f},
  { 73.7244622824734f,    -36.6744087727640f},
  { -116.714024064858f,    -13.2836050193709f},
  { 79.7851892483328f,    -52.0073116800525f},
  { -66.1185563567218f,    -17.8029556568936f},
  { 60.6036788809347f,    32.6348293383468f},
  { 34.7911523459707f,    20.5612097287484f},
  { -49.2869372901949f,    15.9305432103548f},
  { -157.496695959460f,    -13.8651185512754f},
  { -18.6496707750376f,    -41.3424641546771f},
  { 18.5534468464676f,    9.49421865641912f},
  { 97.2349796566041f,    21.2306274005472f},
  { 66.4715807183197f,    35.4183575038600f},
  { 41.5285947265724f,    6.74983031251173f},
  { -59.3050559227199f,    -3.02869649970139f},
  { -24.5831094277934f,    47.5597127200273f},
  { -129.863756128540f,    -44.7563993680868f},
  { 151.751026959494f,    26.9303788527913f},
  { 53.6608091508298f,    42.7227843218745f},
  { -125.918089473216f,    52.5712937196515f},
  { 89.7081237316887f,    -9.30769170602622f},
  { 43.3824378122288f,    4.25007017636107f},
  { 40.9217546341132f,    3.35544603345923f},
  { -83.3851149641043f,    -2.85223335876547f},
  { 8.86711821711497f,    34.5508156260187f},
  { -157.372644273731f,    -40.9680026987570f},
  { 25.1049473673579f,    -16.3593291014409f},
  { -162.266832738389f,    -73.6958742266972f},
  { 13.4164710014717f,    0.485166461422033f},
  { 51.1719694354188f,    -24.1395308367399f},
  { 31.5970821400262f,    28.0128544675960f},
  { -66.7114072526321f,    -44.7218775208384f},
  { -158.342212893962f,    -27.1776895454452f},
  { 25.8568037043090f,    -13.9264213239911f},
  { -167.149786479390f,    -4.09670383933257f},
  { 49.2526097151730f,    6.95856837512662f},
  { -107.738849739568f,    40.0119484996736f},
  { 154.696319280561f,    62.8547609689638f},
  { -9.12980109749704f,    -18.4230348800415f},
  { 173.491474846611f,    30.5124736995394f},
  { -60.3429152260950f,    -75.1227323266721f},
  { 129.808336961828f,    32.0784453558476f},
  { -106.097398613313f,    65.6361648135907f},
  { -6.52544224090071f,    14.5653408704797f},
  { 164.211960283641f,    -62.5599925017521f},
  { -82.7145548997385f,    35.8663816720691f},
  { 74.9706812330930f,    -51.6647264221959f},
  { -89.5249164174150f,    30.7108321642385f},
  { -13.9711377418648f,    -7.17587804486720f},
  { 96.5275717690253f,    0.126768783806846f},
  { -18.3465457995242f,    12.0647098901493f},
  { 81.8569210689948f,    45.7421152009923f},
  { -55.4816294299688f,    28.5753409049472f},
  { -73.5420891643444f,    61.6987748035781f},
  { 166.577525256934f,    47.6684832418540f},
  { -17.8308143763198f,    -25.0368068353602f},
  { -98.2890727469511f,    52.1954010733911f},
  { -52.5241431047322f,    -65.8814387634490f},
  { -102.112425277627f,    -10.8789198056757f},
  { -150.173953624875f,    88.6657911823910f},
  { 175.037295988540f,    -14.3120567584373f},
  { 108.758149414454f,    8.73282633052416f},
  { 164.065594568446f,    -38.0992307947422f},
  { -118.152167471115f,    -44.8555495611701f},
  { 114.701310971907f,    56.8649853557123f},
  { -20.8515061175153f,    3.27420844087041f},
  { -49.7961940334861f,    -4.45218628135222f},
  { -75.1224885311900f,    -49.4345332060033f},
  { -144.959001542579f,    -45.8573104295138f},
  { 163.851853851514f,    49.3186212669315f},
  { -94.3757512103341f,    -47.4781848612267f},
  { -175.531023656571f,    35.1839636966442f},
  { 35.0367924440402f,    -38.8036786690712f},
  { 111.279363874983f,    6.14832949603732f},
  { 164.909305545361f,    -41.1697859129614f},
  { -8.42598798207133f,    12.4901771833085f},
  { 12.4658796234856f,    19.7613088814185f},
  { -51.9610099739962f,    -54.8078676798206f},
  { 89.5668542028892f,    69.1363508010318f},
  { -67.7253497426101f,    28.6987884800029f},
  { -72.7240710833047f,    -11.1471817637867f},
  { -179.617794356082f,    -62.9192815238565f},
  { -89.4770137505559f,    -59.6902917787608f},
  { 140.479000668446f,    54.1423142064665f},
  { 127.421522094251f,    -63.0459726568391f},
  { 60.5228987928801f,    61.3742151086062f},
  { 2.50352449399168f,    -30.5733883034465f},
  { -7.23040060283409f,    -1.88232973009511f},
  { 157.712730741384f,    21.5914090608793f},
  { -84.2137161125059f,    -30.5178830923288f},
  { 2.27064919135642f,    27.8052852867651f},
  { -33.3029362867759f,    -18.3131547706309f},
  { -178.030403314652f,    3.96037245960534f},
  { -56.2968868123161f,    64.6707050432905f},
  { 50.9545813317625f,    57.5944770922965f},
  { 122.975831546272f,    58.8099112975414f},
  { 179.997976052693f,    -52.8225725186769f},
  { -148.990093467708f,    -45.0832534906829f},
  { -9.73890065581691f,    46.8187475208563f},
  { -4.08259606336060f,    35.5652390941893f},
  { 123.109933537944f,    15.0267102275093f},
  { 146.735415466037f,    5.44150430705723f},
  { 152.238238048455f,    -2.21417759870943f},
  { 107.614604053619f,    39.3344249117212f},
  { 70.2209393661553f,    -56.5255088065839f},
  { 68.2439060125565f,    9.36521337404056f},
  { 144.381302351190f,    -27.7476731067727f},
  { 170.003183392865f,    -36.9700490588779f},
  { 117.004945311795f,    25.1748632066066f},
  { -88.0678750501500f,    11.1696106264579f},
  { 0.0425300483008077f,    -47.8829462291547f},
  { 51.6558511845294f,    -2.97585586937439f},
  { -18.7920993278230f,    -6.12812228232499f},
  { -149.891801780663f,    7.61701647566548f},
  { 122.823353859846f,    -5.09173079720226f},
  { -5.11655922673943f,    4.18146345267463f},
  { 101.146721645536f,    -25.2421073284771f},
  { -121.431165370576f,    -17.8419060666222f},
  { 179.064439177270f,    33.7603814155747f},
  { -101.051566467586f,    -45.1808250438887f},
  { -107.688551748976f,    -45.2062573804445f},
  { -127.576262546167f,    45.6497541245122f},
  { -47.8380630788215f,    23.9639038357658f},
  { -23.1604603474314f,    -43.9080257979142f},
  { 35.6521663956901f,    -43.4283641158210f},
  { -162.728028683436f,    16.1597234904786f},
  { -79.5790366186215f,    12.9909850230568f},
  { 51.2551212371218f,    -18.4930970740153f},
  { -102.252824156063f,    -21.3148193094379f},
  { 139.230400887481f,    -20.2438787827193f},
  { -173.790618129871f,    5.32899418522077f},
  { 143.142631476937f,    20.8017030701953f},
  { -98.9844633358494f,    11.2541023571857f},
  { -70.8113735266661f,    2.00243587629306f},
  { 25.1855807991271f,    -70.7779622363148f},
  { 146.339025317794f,    -30.1559360103775f},
  { -0.558064989837754f,    -23.5097795291665f},
  { -31.8277347396020f,    16.8123240576211f},
  { -7.62449761110261f,    42.8836108425672f},
  { 35.2156584182750f,    60.5108434187774f},
  { -122.494342686456f,    33.8222572559338f},
  { -50.0768765037896f,    -50.4579355241053f},
  { -117.197236402146f,    -63.7609145194176f},
  { -21.7289450173940f,    16.3663929954881f},
  { 164.890882608540f,    33.5233598265761f},
  { 114.914810351533f,    49.6479518177324f},
  { 130.729399804981f,    -32.9106804222341f},
  { -103.899606003033f,    41.3394871207568f},
  { -133.270972673505f,    0.193620979858986f},
  { -153.045063759832f,    26.6554481563509f},
  { 143.870754995500f,    -53.4529386443691f},
  { 39.9528783825515f,    23.5812210110604f},
  { -172.503117154993f,    -9.84843258567302f},
  { 170.620273888194f,    -31.6892284109843f},
  { 15.8932748513753f,    -15.1747336888820f},
  { -93.2595205712499f,    4.57394080280570f},
  { -55.4373198953333f,    48.3504266050628f},
  { 23.2497899644004f,    -4.30521520726145f},
  { 82.9472014863809f,    -39.8289372152604f},
  { 24.3030449047981f,    1.59346610389211f},
  { 113.385358791659f,    31.2827363025915f},
  { 49.4430503808335f,    1.52959076015471f},
  { -110.550597898267f,    -22.7379580031879f},
  { -39.8426215916044f,    30.2167819538330f},
  { 93.5221864127092f,    38.5935481119565f},
  { 1.59573876227419f,    -55.6904267576167f},
  { 23.5397076327978f,    51.4960402802874f},
  { 157.138747297992f,    44.7580479297380f},
  { -70.2580340471250f,    -2.65925609217010f},
  { -114.157752468091f,    10.3176741326118f},
  { 71.7333116574457f,    -6.03903482507053f},
  { -104.880712418599f,    9.61210206641801f},
  { -152.744019319105f,    50.4850769636584f},
  { -112.524438837900f,    52.0381077357589f},
  { -111.569577453288f,    -51.0166990228247f},
  { 35.8770375637623f,    6.00590222646529f},
  { 30.3276036956036f,    -76.2364358504411f},
  { 108.111639146167f,    -52.1381502921373f},
  { 18.2115284814082f,    56.0837474260228f},
  { -147.289865343326f,    -11.1927820553189f},
  { -6.91396027504523f,    39.9975928769987f},
  { 100.429390798662f,    -62.6382435502732f},
  { 110.310481942225f,    19.7168775786590f},
  { 3.93551203295096f,    -21.1721428100298f},
  { -38.9677173716424f,    18.8530831842397f},
  { 35.6616826995249f,    11.1298237475361f},
  { -74.1005878757272f,    75.8523838767584f},
  { -157.048406513410f,    80.1594971878931f},
  { -61.2428551639378f,    27.9682341132440f},
  { 52.4486714179108f,    37.5210624219500f},
  { 93.1586133022889f,    80.3363174622849f},
  { 57.1458450871182f,    -3.78938229915202f},
  { -70.0945465109233f,    26.5975356488901f},
  { 109.648302391784f,    -37.6814251147732f},
  { 37.4262798296439f,    50.3277243445087f},
  { -143.932434634374f,    -32.9261540352792f},
  { -78.0583458789707f,    -10.3126475069436f},
  { 33.6895288775155f,    -15.1253745998665f},
  { 72.8064115619973f,    28.8510004710438f},
  { -170.272119375016f,    -72.1634843106536f},
  { -23.1195199197431f,    -22.1059617727149f},
  { 99.8271708674220f,    -15.9603576451761f},
  { -46.3530164890991f,    -21.1506860271258f},
  { 97.1248809045135f,    -8.40765688220728f},
  { -1.43696734510787f,    12.1879546893370f},
  { -51.4588059410446f,    60.9821631804947f},
  { 159.718035859911f,    33.1079792854500f},
  { 69.3154603776786f,    28.1985163590085f},
  { 80.8901024387342f,    -3.25034554474043f},
  { -104.541587755041f,    24.3788570913706f},
  { -146.675980478034f,    -26.8661758393090f},
  { 20.9377653404392f,    -65.2432000848629f},
  { -126.080982814641f,    31.7332538112601f},
  { -63.7943217699208f,    -27.3588310748960f},
  { 52.1292519741822f,    -50.2957857947368f},
  { -19.3872971458597f,    -44.3125005730389f},
  { -62.3863292947352f,    2.32674646373715f},
  { 56.5279097394721f,    1.61930318676770f},
  { 109.426454527841f,    -17.7720569817710f},
  { 59.5682435844045f,    -21.9969038952949f},
  { 37.6675342738708f,    21.8702915838201f},
  { -63.1957775418282f,    -10.4520160948530f},
  { -85.8751488048483f,    -13.3618446372592f},
  { 31.6077559832393f,    39.6157220912829f},
  { 32.4821279448863f,    -2.80840274370882f},
  { -64.6502989026393f,    -59.0564626318734f},
  { 24.9416830443150f,    -22.6301812716343f},
  { -60.2550363876100f,    8.83066457957962f},
  { 29.7270783500814f,    25.7053577014398f},
  { 31.0180030614487f,    -13.3646841591136f},
  { 42.0632755837043f,    38.2308458853338f},
  { -160.502014008122f,    59.8998439634515f},
  { 168.799653176288f,    -2.01560831317327f},
  { -34.4923695347633f,    8.62558802700491f},
  { 117.466828708682f,    31.9229411081417f},
  { -75.0874208790426f,    11.3833126679549f},
  { -112.258290107050f,    3.79830049256292f},
  { -11.1737540334727f,    -5.82337811499330f},
  { 162.155864257836f,    -31.4452697518181f},
  { -161.182161354480f,    -5.57777472509964f},
  { -18.9759747261390f,    -37.9566900924630f},
  { -54.5129568879215f,    25.2353144519553f},
  { -43.1309742142998f,    34.3915719978409f},
  { 121.063686776298f,    -63.7745316280337f},
  { -139.338838523554f,    58.9510987406156f},
  { 91.7345913863288f,    14.7330440620767f},
  { -105.185044373885f,    -34.0842765919950f},
  { 165.034510132135f,    -31.3887298041247f},
  { -130.184891535452f,    -63.5716285102151f},
  { 159.016375202992f,    -67.5938920364209f},
  { -90.4489099215935f,    -63.6112338213990f},
  { -56.8199299696515f,    -44.1034477845373f},
  { 119.774828812467f,    -29.6485407584113f},
  { -177.547870164064f,    -23.6116876061397f},
  { -131.221855710210f,    -55.3382928476971f},
  { -20.1715928418928f,    -15.2383515725468f},
  { 47.2211039394912f,    71.3217107535002f},
  { 26.2067740318424f,    -41.7891906876392f},
  { -78.8033291405068f,    15.7135837566658f},
  { 41.2454218457530f,    27.2657500235597f},
  { -139.814068127375f,    -44.2883802386034f},
  { 7.41346367325931f,    -24.2398710100095f},
  { -84.9460567954203f,    -21.5221885790897f},
  { 130.806610898929f,    13.3591602507128f},
  { 102.404119695394f,    -80.3685640760221f},
  { 101.310244963466f,    -59.8389518167176f},
  { -87.9434737241414f,    -33.6690465234106f},
  { -52.9576944607724f,    -19.3212107434163f},
  { -154.268253402516f,    31.7831505250909f},
  { 92.4486436623433f,    -30.7347971636243f},
  { 103.307100886671f,    -77.6418399883612f},
  { -114.709014956086f,    24.9584436688353f},
  { -13.1774690372325f,    41.2350298833982f},
  { 136.113918260298f,    -7.79893164572718f},
  { 143.713419066315f,    -2.77426437061402f},
  { 59.5557900691219f,    51.5867433398137f},
  { 147.737173293064f,    22.6709366139652f},
  { -90.6272619612051f,    -19.6069546181195f},
  { 71.4165149836313f,    31.3171950839166f},
  { 105.215174804733f,    -43.8651167036432f},
  { 95.8355564202385f,    30.0969788262413f},
  { -136.680155926050f,    -70.4382173792202f},
  { -69.8614318196901f,    -57.8958243541804f},
  { 60.5532211164125f,    14.9778349457056f},
  { 93.5370966736206f,    -76.2988863571906f},
  { 27.6987476661327f,    4.87652814447257f},
  { 95.4854423222915f,    -38.3479886073407f},
  { -115.907799848353f,    50.9813536297129f},
  { -52.1688778766232f,    37.3977758125295f},
  { 56.3085212932640f,    56.7093308501247f},
  { 157.854682114479f,    -17.1901900680136f},
  { 15.7038649827398f,    2.78061596389676f},
  { 101.087938684479f,    4.98116408424739f},
  { -21.1042268417709f,    -19.9817529028294f},
  { -138.293517865579f,    41.2761970145407f},
  { -33.4016877717366f,    -4.63572731314592f},
  { -108.574293421831f,    -14.3189115703482f},
  { -77.0035415584883f,    -38.9021929491112f},
  { -150.241686975044f,    66.4491904302668f},
  { -92.1599189843686f,    -24.9876403976636f},
  { -25.7185001909227f,    -46.1785184702480f},
  { -84.4887805200761f,    74.3019729101659f},
  { 19.8640508953462f,    -9.42107919115538f},
  { -141.546538673753f,    -7.23480366290173f},
  { 74.8603930340358f,    -18.5184080709352f},
  { -65.4014198085818f,    39.7136037305857f},
  { -144.181412959797f,    35.8450784784296f},
  { 94.8440620319955f,    -46.8120276275259f},
  { -165.674771778856f,    -44.6634398502118f},
  { -173.179004188486f,    86.2587914331641f},
  { 2.93214337045507f,    16.0703859913471f},
  { -60.9653682086765f,    66.9192326102841f},
  { 61.2191158869755f,    8.23406739348625f},
  { -7.31662889480752f,    -16.2210309180213f},
  { 106.998382602301f,    3.97139706100809f},
  { 140.854309724849f,    -35.9985200114242f},
  { 160.580653440674f,    58.7197782563616f},
  { -46.3176602256581f,    -55.4112720585649f},
  { 137.080287258273f,    -35.4433409335273f},
  { -16.1772543754189f,    23.3742650202122f},
  { 174.796725466008f,    -18.1923220619770f},
  { 124.964959170800f,    35.9914084272573f},
  { 152.444259614377f,    73.4551881302050f},
  { 179.445265339798f,    -17.4194344783215f},
  { -32.0441464330633f,    -58.6230936983008f},
  { -144.505510034418f,    -18.0053449990008f},
  { -127.736323733658f,    1.79732786476893f},
  { -49.6226072894642f,    8.10929674720791f},
  { -67.5781056918769f,    24.9035688605843f},
  { 63.6921710210550f,    47.7580229896147f},
  { -61.8694115053249f,    4.98850536087643f},
  { 139.612724141299f,    35.8161533257883f},
  { 20.6562221727002f,    29.9258526178061f},
  { -146.733440153129f,    7.68201583442896f},
  { 87.4876074803720f,    -3.06695419975881f},
  { 92.2679442394988f,    -20.9807341938284f},
  { -112.096907873455f,    -66.2270699770113f},
  { -99.8655302178104f,    8.23891588156307f},
  { 48.9984281546412f,    68.5214881547318f},
  { 178.772175316906f,    -50.4643387466079f},
  { -22.9333309793704f,    -54.3975629091681f},
  { 121.079999361715f,    49.7523893131743f},
  { 149.637655874158f,    28.7704710229944f},
  { 150.423522899402f,    57.5619483596157f},
  { -64.5329051442387f,    -41.5189022360922f},
  { 165.484022557630f,    38.0266884111826f},
  { -156.503564672780f,    -30.6914454570282f},
  { -103.883661876949f,    29.8281732827046f},
  { 92.6899519309944f,    44.7438176051669f},
  { 149.974780316198f,    -42.8250215164701f},
  { 149.940879848802f,    -30.8494977671562f},
  { -120.827298873383f,    -31.9049965614365f},
  { -97.1846472883089f,    -25.0238403215857f},
  { 169.141681612328f,    25.0193940575390f},
  { -22.5837960397480f,    -13.6350856791287f},
  { 26.2879833456298f,    -79.1571196520320f},
  { -158.834611601758f,    -24.4346959066494f},
  { -165.514706716188f,    -60.7573612948984f},
  { 29.0358071492329f,    -39.7102213182724f},
  { 136.155801446463f,    4.32132968225788f},
  { -122.601065723553f,    -7.64019416749633f},
  { -73.2670213616701f,    53.9583113376126f},
  { -53.3014519022048f,    0.272878412898698f},
  { 42.8764771111489f,    -31.7572502403651f},
  { 152.968191350858f,    36.2473003814680f},
  { -15.7977605231503f,    -12.8635748671880f},
  { 79.1002695251783f,    35.9546848676500f},
  { -168.599075596622f,    50.4439868425900f},
  { -43.8618728087655f,    24.6024709995092f},
  { -84.0477012791974f,    2.38394815649296f},
  { 127.381059583027f,    42.3495106751389f},
  { 103.607843757590f,    -17.7545706678341f},
  { 23.8083048920801f,    6.79342799438821f},
  { -172.345939111182f,    -3.15623551924696f},
  { -161.540554431835f,    3.07650098814593f},
  { 89.7755335989624f,    17.0280165825829f},
  { -37.6494948028893f,    -56.9387780024591f},
  { -138.951685461449f,    -79.6316581886646f},
  { 45.2687933597244f,    26.5240156911605f},
  { 142.922894328093f,    35.9286260956624f},
  { -175.213284046906f,    -24.9496253561397f},
  { 77.8377815428151f,    -36.1235480969487f},
  { 129.080566342209f,    -30.4858187200167f},
  { -94.7705465544761f,    -26.7060752292626f},
  { -100.935419068100f,    -54.5788757123567f},
  { -49.7513862274307f,    1.23630648043532f},
  { 103.807370129368f,    27.5168039633764f},
  { -166.354873916908f,    10.5733425837884f},
  { 36.2817064968225f,    -29.8425651750705f},
  { -33.9570969152368f,    -32.8462651097259f},
  { -11.5544231220092f,    -42.5007861370517f},
  { 60.4783924250340f,    -15.5791938547136f},
  { -42.8785187313801f,    12.2740004696889f},
  { 82.8838718915494f,    -5.23149737166266f},
  { -106.541215120446f,    63.2066150700692f},
  { 120.652410308458f,    -45.9790382725339f},
  { -90.6228086044930f,    -22.5687134079145f},
  { -24.1990424968209f,    -64.9058206791233f},
  { 34.0448734859848f,    42.0306178595419f},
  { 18.3633251743701f,    -17.0677062863682f},
  { -72.0758519762432f,    -68.3964564966348f},
  { -134.836238032345f,    -14.6154545430050f},
  { -70.6556916814712f,    35.2436106179536f},
  { 19.5768720984680f,    49.9268864595524f},
  { -98.9936165823516f,    -10.0803118111226f},
  { -127.061067683758f,    22.6459433000054f},
  { -5.04677337544235f,    60.9522970306370f},
  { -176.551513024144f,    24.7215301359737f},
  { 43.1871188262524f,    -22.0308759745299f},
  { -93.0160472697171f,    -0.608915826636266f},
  { 94.3643235497202f,    -24.5783795369254f},
  { -16.3144408285961f,    -5.99285284765303f},
  { -14.9189299439464f,    57.3896256077255f},
  { -177.748551692504f,    45.7410164786008f},
  { -60.8780201150254f,    71.8943445614376f},
  { -24.5728573381381f,    -39.3001664803112f},
  { 12.4610505171885f,    -17.1314992696109f},
  { -116.778953914737f,    77.9374137858883f},
  { 75.7399091752730f,    26.9872928089807f},
  { -26.7115174951351f,    9.24512617358828f},
  { 85.9607310896728f,    -36.0159577866263f},
  { -93.5067108445391f,    -41.2677039707420f},
  { -112.991873948084f,    -59.7521519675659f},
  { -7.27234036880005f,    83.7964884854227f},
  { 81.9156984992198f,    5.19284892989513f},
  { 123.929934660503f,    -10.3694878170491f},
  { -106.275537717527f,    -36.6744087727640f},
  { 63.2859759351422f,    -13.2836050193709f},
  { -100.214810751667f,    -52.0073116800525f},
  { 113.881443643278f,    -17.8029556568936f},
  { -119.396321119065f,    32.6348293383468f},
  { -145.208847654029f,    20.5612097287484f},
  { 130.713062709805f,    15.9305432103548f},
  { 22.5033040405404f,    -13.8651185512754f},
  { 161.350329224962f,    -41.3424641546771f},
  { -161.446553153532f,    9.49421865641912f},
  { -82.7650203433960f,    21.2306274005472f},
  { -113.528419281680f,    35.4183575038600f},
  { -138.471405273428f,    6.74983031251173f},
  { 120.694944077280f,    -3.02869649970139f},
  { 155.416890572207f,    47.5597127200273f},
  { 50.1362438714596f,    -44.7563993680868f},
  { -28.2489730405064f,    26.9303788527913f},
  { -126.339190849170f,    42.7227843218745f},
  { 54.0819105267844f,    52.5712937196515f},
  { -90.2918762683113f,    -9.30769170602622f},
  { -136.617562187771f,    4.25007017636107f},
  { -139.078245365887f,    3.35544603345923f},
  { 96.6148850358957f,    -2.85223335876547f},
  { -171.132881782885f,    34.5508156260187f},
  { 22.6273557262694f,    -40.9680026987570f},
  { -154.895052632642f,    -16.3593291014409f},
  { 17.7331672616115f,    -73.6958742266972f},
  { -166.583528998528f,    0.485166461422033f},
  { -128.828030564581f,    -24.1395308367399f},
  { -148.402917859974f,    28.0128544675960f},
  { 113.288592747368f,    -44.7218775208384f},
  { 21.6577871060380f,    -27.1776895454452f},
  { -154.143196295691f,    -13.9264213239911f},
  { 12.8502135206102f,    -4.09670383933257f},
  { -130.747390284827f,    6.95856837512662f},
  { 72.2611502604325f,    40.0119484996736f},
  { -25.3036807194395f,    62.8547609689638f},
  { 170.870198902503f,    -18.4230348800415f},
  { -6.50852515338894f,    30.5124736995394f},
  { 119.657084773905f,    -75.1227323266721f},
  { -50.1916630381715f,    32.0784453558476f},
  { 73.9026013866870f,    65.6361648135907f},
  { 173.474557759099f,    14.5653408704797f},
  { -15.7880397163586f,    -62.5599925017521f},
  { 97.2854451002615f,    35.8663816720691f},
  { -105.029318766907f,    -51.6647264221959f},
  { 90.4750835825850f,    30.7108321642385f},
  { 166.028862258135f,    -7.17587804486720f},
  { -83.4724282309747f,    0.126768783806846f},
  { 161.653454200476f,    12.0647098901493f},
  { -98.1430789310052f,    45.7421152009923f},
  { 124.518370570031f,    28.5753409049472f},
  { 106.457910835656f,    61.6987748035781f},
  { -13.4224747430656f,    47.6684832418540f},
  { 162.169185623680f,    -25.0368068353602f},
  { 81.7109272530489f,    52.1954010733911f},
  { 127.475856895268f,    -65.8814387634490f},
  { 77.8875747223732f,    -10.8789198056757f},
  { 29.8260463751248f,    88.6657911823910f},
  { -4.96270401145955f,    -14.3120567584373f},
  { -71.2418505855464f,    8.73282633052416f},
  { -15.9344054315540f,    -38.0992307947422f},
  { 61.8478325288853f,    -44.8555495611701f},
  { -65.2986890280926f,    56.8649853557123f},
  { 159.148493882485f,    3.27420844087041f},
  { 130.203805966514f,    -4.45218628135222f},
  { 104.877511468810f,    -49.4345332060033f},
  { 35.0409984574206f,    -45.8573104295138f},
  { -16.1481461484859f,    49.3186212669315f},
  { 85.6242487896659f,    -47.4781848612267f},
  { 4.46897634342862f,    35.1839636966442f},
  { -144.963207555960f,    -38.8036786690712f},
  { -68.7206361250173f,    6.14832949603732f},
  { -15.0906944546390f,    -41.1697859129614f},
  { 171.574012017929f,    12.4901771833085f},
  { -167.534120376514f,    19.7613088814185f},
  { 128.038990026004f,    -54.8078676798206f},
  { -90.4331457971108f,    69.1363508010318f},
  { 112.274650257390f,    28.6987884800029f},
  { 107.275928916695f,    -11.1471817637867f},
  { 0.382205643918457f,    -62.9192815238565f},
  { 90.5229862494441f,    -59.6902917787608f},
  { -39.5209993315536f,    54.1423142064665f},
  { -52.5784779057485f,    -63.0459726568391f},
  { -119.477101207120f,    61.3742151086062f},
  { -177.496475506008f,    -30.5733883034465f},
  { 172.769599397166f,    -1.88232973009511f},
  { -22.2872692586157f,    21.5914090608793f},
  { 95.7862838874942f,    -30.5178830923288f},
  { -177.729350808644f,    27.8052852867651f},
  { 146.697063713224f,    -18.3131547706309f},
  { 1.96959668534852f,    3.96037245960534f},
  { 123.703113187684f,    64.6707050432905f},
  { -129.045418668238f,    57.5944770922965f},
  { -57.0241684537285f,    58.8099112975414f},
  { -0.00202394730670824f,    -52.8225725186769f},
  { 31.0099065322919f,    -45.0832534906829f},
  { 170.261099344183f,    46.8187475208563f},
  { 175.917403936639f,    35.5652390941893f},
  { -56.8900664620558f,    15.0267102275093f},
  { -33.2645845339635f,    5.44150430705723f},
  { -27.7617619515453f,    -2.21417759870943f},
  { -72.3853959463809f,    39.3344249117212f},
  { -109.779060633845f,    -56.5255088065839f},
  { -111.756093987444f,    9.36521337404056f},
  { -35.6186976488097f,    -27.7476731067727f},
  { -9.99681660713551f,    -36.9700490588779f},
  { -62.9950546882047f,    25.1748632066066f},
  { 91.9321249498500f,    11.1696106264579f},
  { -179.957469951699f,    -47.8829462291547f},
  { -128.344148815471f,    -2.97585586937439f},
  { 161.207900672177f,    -6.12812228232499f},
  { 30.1081982193371f,    7.61701647566548f},
  { -57.1766461401543f,    -5.09173079720226f},
  { 174.883440773261f,    4.18146345267463f},
  { -78.8532783544636f,    -25.2421073284771f},
  { 58.5688346294240f,    -17.8419060666222f},
  { -0.935560822729587f,    33.7603814155747f},
  { 78.9484335324145f,    -45.1808250438887f},
  { 72.3114482510242f,    -45.2062573804445f},
  { 52.4237374538326f,    45.6497541245122f},
  { 132.161936921178f,    23.9639038357658f},
  { 156.839539652569f,    -43.9080257979142f},
  { -144.347833604310f,    -43.4283641158210f},
  { 17.2719713165641f,    16.1597234904786f},
  { 100.420963381379f,    12.9909850230568f},
  { -128.744878762878f,    -18.4930970740153f},
  { 77.7471758439369f,    -21.3148193094379f},
  { -40.7695991125187f,    -20.2438787827193f},
  { 6.20938187012891f,    5.32899418522077f},
  { -36.8573685230626f,    20.8017030701953f},
  { 81.0155366641506f,    11.2541023571857f},
  { 109.188626473334f,    2.00243587629306f},
  { -154.814419200873f,    -70.7779622363148f},
  { -33.6609746822056f,    -30.1559360103775f},
  { 179.441935010162f,    -23.5097795291665f},
  { 148.172265260398f,    16.8123240576211f},
  { 172.375502388897f,    42.8836108425672f},
  { -144.784341581725f,    60.5108434187774f},
  { 57.5056573135436f,    33.8222572559338f},
  { 129.923123496210f,    -50.4579355241053f},
  { 62.8027635978540f,    -63.7609145194176f},
  { 158.271054982606f,    16.3663929954881f},
  { -15.1091173914599f,    33.5233598265761f},
  { -65.0851896484675f,    49.6479518177324f},
  { -49.2706001950190f,    -32.9106804222341f},
  { 76.1003939969667f,    41.3394871207568f},
  { 46.7290273264954f,    0.193620979858986f},
  { 26.9549362401683f,    26.6554481563509f},
  { -36.1292450045000f,    -53.4529386443691f},
  { -140.047121617448f,    23.5812210110604f},
  { 7.49688284500747f,    -9.84843258567302f},
  { -9.37972611180582f,    -31.6892284109843f},
  { -164.106725148625f,    -15.1747336888820f},
  { 86.7404794287501f,    4.57394080280570f},
  { 124.562680104667f,    48.3504266050628f},
  { -156.750210035600f,    -4.30521520726145f},
  { -97.0527985136191f,    -39.8289372152604f},
  { -155.696955095202f,    1.59346610389211f},
  { -66.6146412083409f,    31.2827363025915f},
  { -130.556949619167f,    1.52959076015471f},
  { 69.4494021017328f,    -22.7379580031879f},
  { 140.157378408396f,    30.2167819538330f},
  { -86.4778135872908f,    38.5935481119565f},
  { -178.404261237726f,    -55.6904267576167f},
  { -156.460292367202f,    51.4960402802874f},
  { -22.8612527020079f,    44.7580479297380f},
  { 109.741965952875f,    -2.65925609217010f},
  { 65.8422475319093f,    10.3176741326118f},
  { -108.266688342554f,    -6.03903482507053f},
  { 75.1192875814015f,    9.61210206641801f},
  { 27.2559806808950f,    50.4850769636584f},
  { 67.4755611620996f,    52.0381077357589f},
  { 68.4304225467123f,    -51.0166990228247f},
  { -144.122962436238f,    6.00590222646529f},
  { -149.672396304396f,    -76.2364358504411f},
  { -71.8883608538326f,    -52.1381502921373f},
  { -161.788471518592f,    56.0837474260228f},
  { 32.7101346566735f,    -11.1927820553189f},
  { 173.086039724955f,    39.9975928769987f},
  { -79.5706092013376f,    -62.6382435502732f},
  { -69.6895180577746f,    19.7168775786590f},
  { -176.064487967049f,    -21.1721428100298f},
  { 141.032282628358f,    18.8530831842397f},
  { -144.338317300475f,    11.1298237475361f},
  { 105.899412124273f,    75.8523838767584f},
  { 22.9515934865897f,    80.1594971878931f},
  { 118.757144836062f,    27.9682341132440f},
  { -127.551328582089f,    37.5210624219500f},
  { -86.8413866977111f,    80.3363174622849f},
  { -122.854154912882f,    -3.78938229915202f},
  { 109.905453489077f,    26.5975356488901f},
  { -70.3516976082159f,    -37.6814251147732f},
  { -142.573720170356f,    50.3277243445087f},
  { 36.0675653656258f,    -32.9261540352792f},
  { 101.941654121029f,    -10.3126475069436f},
  { -146.310471122485f,    -15.1253745998665f},
  { -107.193588438003f,    28.8510004710438f},
  { 9.72788062498358f,    -72.1634843106536f},
  { 156.880480080257f,    -22.1059617727149f},
  { -80.1728291325780f,    -15.9603576451761f},
  { 133.646983510901f,    -21.1506860271258f},
  { -82.8751190954865f,    -8.40765688220728f},
  { 178.563032654892f,    12.1879546893370f},
  { 128.541194058955f,    60.9821631804947f},
  { -20.2819641400888f,    33.1079792854500f},
  { -110.684539622321f,    28.1985163590085f},
  { -99.1098975612658f,    -3.25034554474043f},
  { 75.4584122449589f,    24.3788570913706f},
  { 33.3240195219665f,    -26.8661758393090f},
  { -159.062234659561f,    -65.2432000848629f},
  { 53.9190171853593f,    31.7332538112601f},
  { 116.205678230079f,    -27.3588310748960f},
  { -127.870748025818f,    -50.2957857947368f},
  { 160.612702854140f,    -44.3125005730389f},
  { 117.613670705265f,    2.32674646373715f},
  { -123.472090260528f,    1.61930318676770f},
  { -70.5735454721590f,    -17.7720569817710f},
  { -120.431756415596f,    -21.9969038952949f},
  { -142.332465726129f,    21.8702915838201f},
  { 116.804222458172f,    -10.4520160948530f},
  { 94.1248511951517f,    -13.3618446372592f},
  { -148.392244016761f,    39.6157220912829f},
  { -147.517872055114f,    -2.80840274370882f},
  { 115.349701097361f,    -59.0564626318734f},
  { -155.058316955685f,    -22.6301812716343f},
  { 119.744963612390f,    8.83066457957962f},
  { -150.272921649919f,    25.7053577014398f},
  { -148.981996938551f,    -13.3646841591136f},
  { -137.936724416296f,    38.2308458853338f},
  { 19.4979859918782f,    59.8998439634515f},
  { -11.2003468237118f,    -2.01560831317327f},
  { 145.507630465237f,    8.62558802700491f},
  { -62.5331712913180f,    31.9229411081417f},
  { 104.912579120957f,    11.3833126679549f},
  { 67.7417098929497f,    3.79830049256292f},
  { 168.826245966527f,    -5.82337811499330f},
  { -17.8441357421638f,    -31.4452697518181f},
  { 18.8178386455198f,    -5.57777472509964f},
  { 161.024025273861f,    -37.9566900924630f},
  { 125.487043112079f,    25.2353144519553f},
  { 136.869025785700f,    34.3915719978409f},
  { -58.9363132237020f,    -63.7745316280337f},
  { 40.6611614764462f,    58.9510987406156f},
  { -88.2654086136712f,    14.7330440620767f},
  { 74.8149556261145f,    -34.0842765919950f},
  { -14.9654898678653f,    -31.3887298041247f},
  { 49.8151084645483f,    -63.5716285102151f},
  { -20.9836247970084f,    -67.5938920364209f},
  { 89.5510900784065f,    -63.6112338213990f},
  { 123.180070030349f,    -44.1034477845373f},
  { -60.2251711875326f,    -29.6485407584113f},
  { 2.45212983593625f,    -23.6116876061397f},
  { 48.7781442897902f,    -55.3382928476971f},
  { 159.828407158107f,    -15.2383515725468f},
  { -132.778896060509f,    71.3217107535002f},
  { -153.793225968158f,    -41.7891906876392f},
  { 101.196670859493f,    15.7135837566658f},
  { -138.754578154247f,    27.2657500235597f},
  { 40.1859318726247f,    -44.2883802386034f},
  { -172.586536326741f,    -24.2398710100095f},
  { 95.0539432045797f,    -21.5221885790897f},
  { -49.1933891010712f,    13.3591602507128f},
  { -77.5958803046065f,    -80.3685640760221f},
  { -78.6897550365339f,    -59.8389518167176f},
  { 92.0565262758586f,    -33.6690465234106f},
  { 127.042305539228f,    -19.3212107434163f},
  { 25.7317465974836f,    31.7831505250909f},
  { -87.5513563376567f,    -30.7347971636243f},
  { -76.6928991133291f,    -77.6418399883612f},
  { 65.2909850439137f,    24.9584436688353f},
  { 166.822530962768f,    41.2350298833982f},
  { -43.8860817397015f,    -7.79893164572718f},
  { -36.2865809336855f,    -2.77426437061402f},
  { -120.444209930878f,    51.5867433398137f},
  { -32.2628267069358f,    22.6709366139652f},
  { 89.3727380387949f,    -19.6069546181195f},
  { -108.583485016369f,    31.3171950839166f},
  { 74.7848251952670f,    43.8651167036432f},
  { 84.1644435797616f,    -30.0969788262413f},
  { -43.3198440739499f,    70.4382173792202f},
  { -110.138568180310f,    57.8958243541804f},
  { 119.446778883588f,    -14.9778349457056f},
  { 86.4629033263795f,    76.2988863571906f},
  { 152.301252333867f,    -4.87652814447257f},
  { 84.5145576777085f,    38.3479886073407f},
  { -64.0922001516470f,    -50.9813536297129f},
  { -127.831122123377f,    -37.3977758125295f},
  { 123.691478706736f,    -56.7093308501247f},
  { 22.1453178855212f,    17.1901900680136f},
  { 164.296135017260f,    -2.78061596389676f},
  { 78.9120613155213f,    -4.98116408424739f},
  { -158.895773158229f,    19.9817529028294f},
  { -41.7064821344211f,    -41.2761970145407f},
  { -146.598312228263f,    4.63572731314592f},
  { -71.4257065781688f,    14.3189115703482f},
  { -102.996458441512f,    38.9021929491112f},
  { -29.7583130249564f,    -66.4491904302668f},
  { -87.8400810156315f,    24.9876403976636f},
  { -154.281499809077f,    46.1785184702480f},
  { -95.5112194799239f,    -74.3019729101659f},
  { 160.135949104654f,    9.42107919115538f},
  { -38.4534613262471f,    7.23480366290173f},
  { 105.139606965964f,    18.5184080709352f},
  { -114.598580191418f,    -39.7136037305857f},
  { -35.8185870402032f,    -35.8450784784296f},
  { 85.1559379680045f,    46.8120276275259f},
  { -14.3252282211436f,    44.6634398502118f},
  { -6.82099581151372f,    -86.2587914331641f},
  { 177.067856629545f,    -16.0703859913471f},
  { -119.034631791324f,    -66.9192326102841f},
  { 118.780884113024f,    -8.23406739348625f},
  { -172.683371105192f,    16.2210309180213f},
  { 73.0016173976989f,    -3.97139706100809f},
  { 39.1456902751512f,    35.9985200114242f},
  { 19.4193465593261f,    -58.7197782563616f},
  { -133.682339774342f,    55.4112720585649f},
  { 42.9197127417268f,    35.4433409335273f},
  { -163.822745624581f,    -23.3742650202122f},
  { 5.20327453399252f,    18.1923220619770f},
  { 55.0350408291998f,    -35.9914084272573f},
  { 27.5557403856226f,    -73.4551881302050f},
  { 0.554734660201537f,    17.4194344783215f},
  { -147.955853566937f,    58.6230936983008f},
  { -35.4944899655817f,    18.0053449990008f},
  { -52.2636762663416f,    -1.79732786476893f},
  { -130.377392710536f,    -8.10929674720791f},
  { -112.421894308123f,    -24.9035688605843f},
  { 116.307828978945f,    -47.7580229896147f},
  { -118.130588494675f,    -4.98850536087643f},
  { 40.3872758587007f,    -35.8161533257883f},
  { 159.343777827300f,    -29.9258526178061f},
  { -33.2665598468711f,    -7.68201583442896f},
  { 92.5123925196280f,    3.06695419975881f},
  { 87.7320557605012f,    20.9807341938284f},
  { -67.9030921265448f,    66.2270699770113f},
  { -80.1344697821897f,    -8.23891588156307f},
  { 131.001571845359f,    -68.5214881547318f},
  { 1.22782468309418f,    50.4643387466079f},
  { -157.066669020630f,    54.3975629091681f},
  { 58.9200006382852f,    -49.7523893131743f},
  { 30.3623441258417f,    -28.7704710229944f},
  { 29.5764771005985f,    -57.5619483596157f},
  { -115.467094855761f,    41.5189022360922f},
  { 14.5159774423702f,    -38.0266884111826f},
  { -23.4964353272202f,    30.6914454570282f},
  { -76.1163381230512f,    -29.8281732827046f},
  { 87.3100480690056f,    -44.7438176051669f},
  { 30.0252196838023f,    42.8250215164701f},
  { 30.0591201511977f,    30.8494977671562f},
  { -59.1727011266166f,    31.9049965614365f},
  { -82.8153527116912f,    25.0238403215857f},
  { 10.8583183876716f,    -25.0193940575390f},
  { -157.416203960252f,    13.6350856791287f},
  { 153.712016654370f,    79.1571196520320f},
  { -21.1653883982416f,    24.4346959066494f},
  { -14.4852932838116f,    60.7573612948984f},
  { 150.964192850767f,    39.7102213182724f},
  { 43.8441985535369f,    -4.32132968225788f},
  { -57.3989342764473f,    7.64019416749633f},
  { -106.732978638330f,    -53.9583113376126f},
  { -126.698548097795f,    -0.272878412898698f},
  { 137.123522888851f,    31.7572502403651f},
  { 27.0318086491423f,    -36.2473003814680f},
  { -164.202239476850f,    12.8635748671880f},
  { 100.899730474822f,    -35.9546848676500f},
  { -11.4009244033777f,    -50.4439868425900f},
  { -136.138127191234f,    -24.6024709995092f},
  { -95.9522987208026f,    -2.38394815649296f},
  { 52.6189404169727f,    -42.3495106751389f},
  { 76.3921562424101f,    17.7545706678341f},
  { 156.191695107920f,    -6.79342799438821f},
  { -7.65406088881811f,    3.15623551924696f},
  { -18.4594455681646f,    -3.07650098814593f},
  { 90.2244664010376f,    -17.0280165825829f},
  { -142.350505197111f,    56.9387780024591f},
  { -41.0483145385507f,    79.6316581886646f},
  { 134.731206640276f,    -26.5240156911605f},
  { 37.0771056719068f,    -35.9286260956624f},
  { -4.78671595309442f,    24.9496253561397f},
  { 102.162218457185f,    36.1235480969487f},
  { 50.9194336577915f,    30.4858187200167f},
  { -85.2294534455239f,    26.7060752292626f},
  { -79.0645809318999f,    54.5788757123567f},
  { -130.248613772569f,    -1.23630648043532f},
  { 76.1926298706323f,    -27.5168039633764f},
  { -13.6451260830917f,    -10.5733425837884f},
  { 143.718293503178f,    29.8425651750705f},
  { -146.042903084763f,    32.8462651097259f},
  { -168.445576877991f,    42.5007861370517f},
  { 119.521607574966f,    15.5791938547136f},
  { -137.121481268620f,    -12.2740004696889f},
  { 97.1161281084506f,    5.23149737166266f},
  { -73.4587848795541f,    -63.2066150700692f},
  { 59.3475896915418f,    45.9790382725339f},
  { -89.3771913955071f,    22.5687134079145f},
  { -155.800957503179f,    64.9058206791233f},
  { 145.955126514015f,    -42.0306178595419f},
  { 161.636674825630f,    17.0677062863682f},
  { -107.924148023757f,    68.3964564966348f},
  { -45.1637619676550f,    14.6154545430050f},
  { -109.344308318529f,    -35.2436106179536f},
  { 160.423127901532f,    -49.9268864595524f},
  { -81.0063834176484f,    10.0803118111226f},
  { -52.9389323162424f,    -22.6459433000054f},
  { -174.953226624558f,    -60.9522970306370f},
  { -3.44848697585623f,    -24.7215301359737f},
  { 136.812881173748f,    22.0308759745299f},
  { -86.9839527302829f,    0.608915826636266f},
  { 85.6356764502799f,    24.5783795369254f},
  { -163.685559171404f,    5.99285284765303f},
  { -165.081070056054f,    -57.3896256077255f},
  { -2.25144830749611f,    -45.7410164786008f},
  { -119.121979884975f,    -71.8943445614376f},
  { -155.427142661862f,    39.3001664803112f},
  { 167.538949482812f,    17.1314992696109f},
  { -63.2210460852630f,    -77.9374137858883f},
  { 104.260090824727f,    -26.9872928089807f},
  { -153.288482504865f,    -9.24512617358828f},
  { 94.0392689103273f,    36.0159577866263f},
  { -86.4932891554609f,    41.2677039707420f},
  { -67.0081260519160f,    59.7521519675659f},
  { -172.727659631200f,    -83.7964884854227f},
  { 98.0843015007802f,    -5.19284892989513f},
  { 56.0700653394972f,    10.3694878170491f},
  { -73.7244622824734f,    36.6744087727640f},
  { 116.714024064858f,    13.2836050193709f},
  { -79.7851892483328f,    52.0073116800525f},
  { 66.1185563567218f,    17.8029556568936f},
  { -60.6036788809347f,    -32.6348293383468f},
  { -34.7911523459707f,    -20.5612097287484f},
  { 49.2869372901949f,    -15.9305432103548f},
  { 157.496695959460f,    13.8651185512754f},
  { 18.6496707750376f,    41.3424641546771f},
  { -18.5534468464676f,    -9.49421865641912f},
  { -97.2349796566041f,    -21.2306274005472f},
  { -66.4715807183197f,    -35.4183575038600f},
  { -41.5285947265724f,    -6.74983031251173f},
  { 59.3050559227199f,    3.02869649970139f},
  { 24.5831094277934f,    -47.5597127200273f},
  { 129.863756128540f,    44.7563993680868f},
  { -151.751026959494f,    -26.9303788527913f},
  { -53.6608091508298f,    -42.7227843218745f},
  { 125.918089473216f,    -52.5712937196515f},
  { -89.7081237316887f,    9.30769170602622f},
  { -43.3824378122288f,    -4.25007017636107f},
  { -40.9217546341132f,    -3.35544603345923f},
  { 83.3851149641043f,    2.85223335876547f},
  { -8.86711821711497f,    -34.5508156260187f},
  { 157.372644273731f,    40.9680026987570f},
  { -25.1049473673579f,    16.3593291014409f},
  { 162.266832738389f,    73.6958742266972f},
  { -13.4164710014717f,    -0.485166461422033f},
  { -51.1719694354188f,    24.1395308367399f},
  { -31.5970821400262f,    -28.0128544675960f},
  { 66.7114072526321f,    44.7218775208384f},
  { 158.342212893962f,    27.1776895454452f},
  { -25.8568037043090f,    13.9264213239911f},
  { 167.149786479390f,    4.09670383933257f},
  { -49.2526097151730f,    -6.95856837512662f},
  { 107.738849739568f,    -40.0119484996736f},
  { -154.696319280561f,    -62.8547609689638f},
  { 9.12980109749704f,    18.4230348800415f},
  { -173.491474846611f,    -30.5124736995394f},
  { 60.3429152260950f,    75.1227323266721f},
  { -129.808336961828f,    -32.0784453558476f},
  { 106.097398613313f,    -65.6361648135907f},
  { 6.52544224090071f,    -14.5653408704797f},
  { -164.211960283641f,    62.5599925017521f},
  { 82.7145548997385f,    -35.8663816720691f},
  { -74.9706812330930f,    51.6647264221959f},
  { 89.5249164174150f,    -30.7108321642385f},
  { 13.9711377418648f,    7.17587804486720f},
  { -96.5275717690253f,    -0.126768783806846f},
  { 18.3465457995242f,    -12.0647098901493f},
  { -81.8569210689948f,    -45.7421152009923f},
  { 55.4816294299688f,    -28.5753409049472f},
  { 73.5420891643444f,    -61.6987748035781f},
  { -166.577525256934f,    -47.6684832418540f},
  { 17.8308143763198f,    25.0368068353602f},
  { 98.2890727469511f,    -52.1954010733911f},
  { 52.5241431047322f,    65.8814387634490f},
  { 102.112425277627f,    10.8789198056757f},
  { 150.173953624875f,    -88.6657911823910f},
  { -175.037295988540f,    14.3120567584373f},
  { -108.758149414454f,    -8.73282633052416f},
  { -164.065594568446f,    38.0992307947422f},
  { 118.152167471115f,    44.8555495611701f},
  { -114.701310971907f,    -56.8649853557123f},
  { 20.8515061175153f,    -3.27420844087041f},
  { 49.7961940334861f,    4.45218628135222f},
  { 75.1224885311900f,    49.4345332060033f},
  { 144.959001542579f,    45.8573104295138f},
  { -163.851853851514f,    -49.3186212669315f},
  { 94.3757512103341f,    47.4781848612267f},
  { 175.531023656571f,    -35.1839636966442f},
  { -35.0367924440402f,    38.8036786690712f},
  { -111.279363874983f,    -6.14832949603732f},
  { -164.909305545361f,    41.1697859129614f},
  { 8.42598798207133f,    -12.4901771833085f},
  { -12.4658796234856f,    -19.7613088814185f},
  { 51.9610099739962f,    54.8078676798206f},
  { -89.5668542028892f,    -69.1363508010318f},
  { 67.7253497426101f,    -28.6987884800029f},
  { 72.7240710833047f,    11.1471817637867f},
  { 179.617794356082f,    62.9192815238565f},
  { 89.4770137505559f,    59.6902917787608f},
  { -140.479000668446f,    -54.1423142064665f},
  { -127.421522094251f,    63.0459726568391f},
  { -60.5228987928801f,    -61.3742151086062f},
  { -2.50352449399168f,    30.5733883034465f},
  { 7.23040060283409f,    1.88232973009511f},
  { -157.712730741384f,    -21.5914090608793f},
  { 84.2137161125059f,    30.5178830923288f},
  { -2.27064919135642f,    -27.8052852867651f},
  { 33.3029362867759f,    18.3131547706309f},
  { 178.030403314652f,    -3.96037245960534f},
  { 56.2968868123161f,    -64.6707050432905f},
  { -50.9545813317625f,    -57.5944770922965f},
  { -122.975831546272f,    -58.8099112975414f},
  { -179.997976052693f,    52.8225725186769f},
  { 148.990093467708f,    45.0832534906829f},
  { 9.73890065581691f,    -46.8187475208563f},
  { 4.08259606336060f,    -35.5652390941893f},
  { -123.109933537944f,    -15.0267102275093f},
  { -146.735415466037f,    -5.44150430705723f},
  { -152.238238048455f,    2.21417759870943f},
  { -107.614604053619f,    -39.3344249117212f},
  { -70.2209393661553f,    56.5255088065839f},
  { -68.2439060125565f,    -9.36521337404056f},
  { -144.381302351190f,    27.7476731067727f},
  { -170.003183392865f,    36.9700490588779f},
  { -117.004945311795f,    -25.1748632066066f},
  { 88.0678750501500f,    -11.1696106264579f},
  { -0.0425300483008077f,    47.8829462291547f},
  { -51.6558511845294f,    2.97585586937439f},
  { 18.7920993278230f,    6.12812228232499f},
  { 149.891801780663f,    -7.61701647566548f},
  { -122.823353859846f,    5.09173079720226f},
  { 5.11655922673943f,    -4.18146345267463f},
  { -101.146721645536f,    25.2421073284771f},
  { 121.431165370576f,    17.8419060666222f},
  { -179.064439177270f,    -33.7603814155747f},
  { 101.051566467586f,    45.1808250438887f},
  { 107.688551748976f,    45.2062573804445f},
  { 127.576262546167f,    -45.6497541245122f},
  { 47.8380630788215f,    -23.9639038357658f},
  { 23.1604603474314f,    43.9080257979142f},
  { -35.6521663956901f,    43.4283641158210f},
  { 162.728028683436f,    -16.1597234904786f},
  { 79.5790366186215f,    -12.9909850230568f},
  { -51.2551212371218f,    18.4930970740153f},
  { 102.252824156063f,    21.3148193094379f},
  { -139.230400887481f,    20.2438787827193f},
  { 173.790618129871f,    -5.32899418522077f},
  { -143.142631476937f,    -20.8017030701953f},
  { 98.9844633358494f,    -11.2541023571857f},
  { 70.8113735266661f,    -2.00243587629306f},
  { -25.1855807991271f,    70.7779622363148f},
  { -146.339025317794f,    30.1559360103775f},
  { 0.558064989837754f,    23.5097795291665f},
  { 31.8277347396020f,    -16.8123240576211f},
  { 7.62449761110261f,    -42.8836108425672f},
  { -35.2156584182750f,    -60.5108434187774f},
  { 122.494342686456f,    -33.8222572559338f},
  { 50.0768765037896f,    50.4579355241053f},
  { 117.197236402146f,    63.7609145194176f},
  { 21.7289450173940f,    -16.3663929954881f},
  { -164.890882608540f,    -33.5233598265761f},
  { -114.914810351533f,    -49.6479518177324f},
  { -130.729399804981f,    32.9106804222341f},
  { 103.899606003033f,    -41.3394871207568f},
  { 133.270972673505f,    -0.193620979858986f},
  { 153.045063759832f,    -26.6554481563509f},
  { -143.870754995500f,    53.4529386443691f},
  { -39.9528783825515f,    -23.5812210110604f},
  { 172.503117154993f,    9.84843258567302f},
  { -170.620273888194f,    31.6892284109843f},
  { -15.8932748513753f,    15.1747336888820f},
  { 93.2595205712499f,    -4.57394080280570f},
  { 55.4373198953333f,    -48.3504266050628f},
  { -23.2497899644004f,    4.30521520726145f},
  { -82.9472014863809f,    39.8289372152604f},
  { -24.3030449047981f,    -1.59346610389211f},
  { -113.385358791659f,    -31.2827363025915f},
  { -49.4430503808335f,    -1.52959076015471f},
  { 110.550597898267f,    22.7379580031879f},
  { 39.8426215916044f,    -30.2167819538330f},
  { -93.5221864127092f,    -38.5935481119565f},
  { -1.59573876227419f,    55.6904267576167f},
  { -23.5397076327978f,    -51.4960402802874f},
  { -157.138747297992f,    -44.7580479297380f},
  { 70.2580340471250f,    2.65925609217010f},
  { 114.157752468091f,    -10.3176741326118f},
  { -71.7333116574457f,    6.03903482507053f},
  { 104.880712418599f,    -9.61210206641801f},
  { 152.744019319105f,    -50.4850769636584f},
  { 112.524438837900f,    -52.0381077357589f},
  { 111.569577453288f,    51.0166990228247f},
  { -35.8770375637623f,    -6.00590222646529f},
  { -30.3276036956036f,    76.2364358504411f},
  { -108.111639146167f,    52.1381502921373f},
  { -18.2115284814082f,    -56.0837474260228f},
  { 147.289865343326f,    11.1927820553189f},
  { 6.91396027504523f,    -39.9975928769987f},
  { -100.429390798662f,    62.6382435502732f},
  { -110.310481942225f,    -19.7168775786590f},
  { -3.93551203295096f,    21.1721428100298f},
  { 38.9677173716424f,    -18.8530831842397f},
  { -35.6616826995249f,    -11.1298237475361f},
  { 74.1005878757272f,    -75.8523838767584f},
  { 157.048406513410f,    -80.1594971878931f},
  { 61.2428551639378f,    -27.9682341132440f},
  { -52.4486714179108f,    -37.5210624219500f},
  { -93.1586133022889f,    -80.3363174622849f},
  { -57.1458450871182f,    3.78938229915202f},
  { 70.0945465109233f,    -26.5975356488901f},
  { -109.648302391784f,    37.6814251147732f},
  { -37.4262798296439f,    -50.3277243445087f},
  { 143.932434634374f,    32.9261540352792f},
  { 78.0583458789707f,    10.3126475069436f},
  { -33.6895288775155f,    15.1253745998665f},
  { -72.8064115619973f,    -28.8510004710438f},
  { 170.272119375016f,    72.1634843106536f},
  { 23.1195199197431f,    22.1059617727149f},
  { -99.8271708674220f,    15.9603576451761f},
  { 46.3530164890991f,    21.1506860271258f},
  { -97.1248809045135f,    8.40765688220728f},
  { 1.43696734510787f,    -12.1879546893370f},
  { 51.4588059410446f,    -60.9821631804947f},
  { -159.718035859911f,    -33.1079792854500f},
  { -69.3154603776786f,    -28.1985163590085f},
  { -80.8901024387342f,    3.25034554474043f},
  { 104.541587755041f,    -24.3788570913706f},
  { 146.675980478034f,    26.8661758393090f},
  { -20.9377653404392f,    65.2432000848629f},
  { 126.080982814641f,    -31.7332538112601f},
  { 63.7943217699208f,    27.3588310748960f},
  { -52.1292519741822f,    50.2957857947368f},
  { 19.3872971458597f,    44.3125005730389f},
  { 62.3863292947352f,    -2.32674646373715f},
  { -56.5279097394721f,    -1.61930318676770f},
  { -109.426454527841f,    17.7720569817710f},
  { -59.5682435844045f,    21.9969038952949f},
  { -37.6675342738708f,    -21.8702915838201f},
  { 63.1957775418282f,    10.4520160948530f},
  { 85.8751488048483f,    13.3618446372592f},
  { -31.6077559832393f,    -39.6157220912829f},
  { -32.4821279448863f,    2.80840274370882f},
  { 64.6502989026393f,    59.0564626318734f},
  { -24.9416830443150f,    22.6301812716343f},
  { 60.2550363876100f,    -8.83066457957962f},
  { -29.7270783500814f,    -25.7053577014398f},
  { -31.0180030614487f,    13.3646841591136f},
  { -42.0632755837043f,    -38.2308458853338f},
  { 160.502014008122f,    -59.8998439634515f},
  { -168.799653176288f,    2.01560831317327f},
  { 34.4923695347633f,    -8.62558802700491f},
  { -117.466828708682f,    -31.9229411081417f},
  { 75.0874208790426f,    -11.3833126679549f},
  { 112.258290107050f,    -3.79830049256292f},
  { 11.1737540334727f,    5.82337811499330f},
  { -162.155864257836f,    31.4452697518181f},
  { 161.182161354480f,    5.57777472509964f},
  { 18.9759747261390f,    37.9566900924630f},
  { 54.5129568879215f,    -25.2353144519553f},
  { 43.1309742142998f,    -34.3915719978409f},
  { -121.063686776298f,    63.7745316280337f},
  { 139.338838523554f,    -58.9510987406156f},
  { -91.7345913863288f,    -14.7330440620767f},
  { 105.185044373885f,    34.0842765919950f},
  { -165.034510132135f,    31.3887298041247f},
  { 130.184891535452f,    63.5716285102151f},
  { -159.016375202992f,    67.5938920364209f},
  { 90.4489099215935f,    63.6112338213990f},
  { 56.8199299696515f,    44.1034477845373f},
  { -119.774828812467f,    29.6485407584113f},
  { 177.547870164064f,    23.6116876061397f},
  { 131.221855710210f,    55.3382928476971f},
  { 20.1715928418928f,    15.2383515725468f},
  { -47.2211039394912f,    -71.3217107535002f},
  { -26.2067740318424f,    41.7891906876392f},
  { 78.8033291405068f,    -15.7135837566658f},
  { -41.2454218457530f,    -27.2657500235597f},
  { 139.814068127375f,    44.2883802386034f},
  { -7.41346367325931f,    24.2398710100095f},
  { 84.9460567954203f,    21.5221885790897f},
  { -130.806610898929f,    -13.3591602507128f},
  { -102.404119695394f,    80.3685640760221f},
  { -101.310244963466f,    59.8389518167176f},
  { 87.9434737241414f,    33.6690465234106f},
  { 52.9576944607724f,    19.3212107434163f},
  { 154.268253402516f,    -31.7831505250909f},
  { -92.4486436623433f,    30.7347971636243f},
  { -103.307100886671f,    77.6418399883612f},
  { 114.709014956086f,    -24.9584436688353f},
  { 13.1774690372325f,    -41.2350298833982f},
  { -136.113918260298f,    7.79893164572718f},
  { -143.713419066315f,    2.77426437061402f},
  { -59.5557900691219f,    -51.5867433398137f},
  { -147.737173293064f,    -22.6709366139652f},
  { 90.6272619612051f,    19.6069546181195f},
  { -71.4165149836313f,    -31.3171950839166f},
  { -135.112963634543f,    10.9070121585005f},
  { 149.773699959762f,    5.04659267603735f},
  { -76.2994903762475f,    14.0988534742932f},
  { -59.5000934898624f,    -10.5432336322085f},
  { 162.921783362287f,    -28.3536664014027f},
  { -103.675994855384f,    0.837283967857134f},
  { 169.599206311779f,    -61.9087522222797f},
  { -141.523979356684f,    4.29946141054661f},
  { 53.9122841415104f,    15.9665329498015f},
  { 44.0665165059163f,    -29.1604953548073f},
  { 118.650072877756f,    -17.7268104061847f},
  { -140.624682189535f,    62.2336942496416f},
  { 169.826934955866f,    -74.0579617493018f},
  { 174.924574979239f,    11.0455351926701f},
  { -45.2805865752663f,    -61.2546950762206f},
  { 52.8398089485056f,    34.1298689414046f},
  { -8.37903274878709f,    -56.3151333517369f},
  { -15.0708229219679f,    17.9772003446941f},
  { -39.6307406711233f,    -10.0794925730507f},
  { 77.7927877665195f,    20.2960272241656f},
  { -25.0032356032190f,    1.95766547893131f},
  { -67.3904436399691f,    -38.5957126032353f},
  { 74.3709844064322f,    -1.48902498866737f},
  { -153.972693036858f,    -68.0968933422460f},
  { -11.5374976072462f,    50.9756729786897f},
  { -160.863428572808f,    -14.3384241532246f},
  { 42.4124051947123f,    -18.6753700967494f},
  { 50.9896057568714f,    41.0940650650316f},
  { -133.085681837097f,    3.31314533193323f},
  { -75.9449020542073f,    43.5615361963055f},
  { 89.5550411564168f,    3.71469119237678f},
  { 100.068996746674f,    -73.6712496717014f},
  { 69.5653596993217f,    -10.9683268071274f},
  { 170.624755540029f,    -28.4569306408453f},
  { -66.3585862216109f,    -72.2456930119684f},
  { 175.847799424660f,    16.9563286710119f},
  { -130.988991916770f,    38.8614598365319f},
  { 101.419721936179f,    29.3199254220964f},
  { -63.4955514785026f,    -23.0836641602584f},
  { -133.731958269579f,    36.6270675845425f},
  { 57.1932154799566f,    -61.8343721338794f},
  { -105.427284784398f,    71.1024301432785f},
  { 138.449889902351f,    27.6247112732190f},
  { 97.8248798600467f,    14.6232335898561f},
  { -91.7674701043864f,    72.5720085009507f},
  { -72.0700065226631f,    -26.1891998542538f},
  { -29.2392869531603f,    50.7415662161816f},
  { 2.27224820848086f,    37.7145121186939f},
  { 10.5942415758875f,    -39.8919382717409f},
  { 26.6671098109331f,    -20.2402883700549f},
  { 129.146515556787f,    -17.3340836582898f},
  { 5.65262753529062f,    -28.0146205608293f},
  { 131.919786468572f,    38.1445272797611f},
  { 121.501149831346f,    -54.1895126313824f},
  { 13.8150315427633f,    55.9576047613440f},
  { -176.930100537907f,    -2.50879170313885f},
  { -159.004251392201f,    2.11751059077362f},
  { -67.7979604218379f,    8.72202083169227f},
  { 8.36084141559409f,    9.76270698851306f},
  { 106.538998121625f,    -13.8998248954034f},
  { -91.0132415244906f,    39.5248038831165f},
  { -74.4112131154684f,    -32.4214717402202f},
  { 125.941168669629f,    19.4834760662258f},
  { 132.631558556291f,    49.1418843662286f},
  { 107.416562328147f,    27.8070106755734f},
  { -44.4386802972763f,    -18.7809816688234f},
  { 107.771147931925f,    49.6919987103964f},
  { -56.1106510588800f,    52.0574257295187f},
  { 30.5666672946455f,    12.0147049226057f},
  { 135.224599589030f,    1.91022092330936f},
  { -118.364137654501f,    39.4207409089961f},
  { -129.983477858289f,    47.9922579006854f},
  { -35.9419894257653f,    25.7874367133622f},
  { -25.1973899479258f,    6.50716971976569f},
  { 111.980366719905f,    62.8657993900750f},
  { -32.2783995368209f,    -63.8049887243174f},
  { -94.8486812733326f,    -9.70999246611946f},
  { -51.5266866907638f,    58.1049566303861f},
  { -82.0282456283157f,    28.2279321494500f},
  { -120.302105369087f,    -42.2679848551773f},
  { 173.774373524572f,    45.9864707634367f},
  { -9.04728606281999f,    32.2763487639339f},
  { 55.1295322093835f,    -9.75292004018489f},
  { 0.340334749165017f,    -36.6980637731376f},
  { -137.706488232958f,    -38.5434744401805f},
  { 121.794802754939f,    45.9199736303281f},
  { -39.9904752703630f,    -69.7314968813966f},
  { 143.548179590381f,    -8.80462147655749f},
  { 80.7269128777346f,    38.6283548682931f},
  { 33.4570118803930f,    -40.9619390884210f},
  { 2.39685567397453f,    -5.94712855224664f},
  { 131.080497338389f,    26.6594457214366f},
  { -161.766142643688f,    12.9481593901667f},
  { 163.558526948424f,    -65.2958241542219f},
  { -22.4899864576418f,    81.7243049279946f},
  { 9.63381088166517f,    71.2947558564855f},
  { 162.971860301779f,    -0.214626123421731f},
  { -68.3169441067463f,    -25.5903546841355f},
  { -83.1486928677538f,    7.80081914661730f},
  { 144.909926078873f,    -39.0284928576167f},
  { 129.759893296427f,    40.2435138095444f},
  { -79.8313379068002f,    64.6242451504367f},
  { -143.254993570606f,    -9.79807916300374f},
  { -142.823513086557f,    32.9059072799982f},
  { -26.7859831743539f,    4.26064584765240f},
  { -55.0732687852541f,    6.31232097458445f},
  { 1.61961624577598f,    -40.2373240229352f},
  { 151.789062848573f,    12.2196640459080f},
  { 38.3530379793283f,    72.7994566938388f},
  { -135.887964065363f,    -44.3646900205413f},
  { -49.1336755390157f,    -44.1748238509645f},
  { -77.6701218310649f,    -46.2469582005938f},
  { -162.233874502056f,    -28.3364425867084f},
  { 17.7307065066041f,    -45.7291417300263f},
  { -174.728120089547f,    -7.08633293927769f},
  { 64.1687257415648f,    7.37354745945388f},
  { -129.739029533640f,    20.7499865955426f},
  { -22.5699130960712f,    0.575112206850396f},
  { -79.1334605367030f,    -22.7580059982164f},
  { 121.844406401116f,    -37.9862943157380f},
  { -135.738490844058f,    -65.1320157280903f},
  { -69.3546131046436f,    -6.50620672998477f},
  { -20.1901338686206f,    43.0210634299778f},
  { 36.8272843749689f,    -15.6957516072835f},
  { 105.742485506789f,    -37.3404883108800f},
  { -10.2031639231906f,    8.85366090244810f},
  { 27.6011253032353f,    33.7935054042650f},
  { 87.2028797860356f,    -28.9244097124329f},
  { 82.5565479685054f,    65.0540856267293f},
  { -149.405575254485f,    -42.5227835363354f},
  { -0.609760380237820f,    3.01587678922753f},
  { -155.358627260151f,    3.96822060608715f},
  { -20.4911758394502f,    -72.6478050529700f},
  { 80.6466152971891f,    -31.3830291585629f},
  { 87.8076831500616f,    44.2158989742383f},
  { 74.0592158625642f,    -8.69895120881935f},
  { -63.0664054461084f,    -44.7289945617809f},
  { -124.992822723311f,    -68.9264035354923f},
  { 79.1989252297070f,    5.40273744949736f},
  { 152.280943003893f,    -12.6797305403720f},
  { 19.9064021639194f,    -61.8453691358413f},
  { -143.916241534221f,    -3.26623893742798f},
  { -41.3209448290236f,    2.63505381463942f},
  { -61.7727165999950f,    11.3476574165480f},
  { 89.2116905637764f,    -6.15341486095787f},
  { 174.755316964369f,    -8.05090054598179f},
  { -167.563251083586f,    33.3027001932534f},
  { -37.8032894253526f,    12.9898206255174f},
  { -165.195321715403f,    -25.9451245830986f},
  { -52.4505235594418f,    6.26707108261434f},
  { -160.649298203185f,    22.6722312605179f},
  { 36.3166637337512f,    24.4157708029608f},
  { 33.3208549505161f,    50.2572267096481f},
  { 159.365180279031f,    38.8453775301901f},
  { -147.181809018358f,    -63.7585346400210f},
  { -109.973941191429f,    45.3450855123758f},
  { 27.7262282099788f,    69.2348366197789f},
  { 21.3856843646275f,    -6.74158116584303f},
  { 37.7982101741625f,    18.9854526889250f},
  { 10.1215775716803f,    48.0249442255521f},
  { -176.478994128782f,    30.6474464336699f},
  { 110.827244855698f,    37.8542425171815f},
  { -127.744368616112f,    -27.0747794767833f},
  { 47.0247371879881f,    -51.7551268128034f},
  { 48.9037331567324f,    25.8059325740881f},
  { 121.791678182581f,    -20.8876829672787f},
  { -9.30781036514151f,    0.288033396543367f},
  { 6.17499173131088f,    46.4511025777881f},
  { 5.11483505423029f,    48.9650667088159f},
  { -177.128683590075f,    6.60665390614924f},
  { 77.3821059344785f,    54.4669065634202f},
  { -113.897544175288f,    -44.1832198826274f},
  { -34.6781867784941f,    60.3276979548377f},
  { -95.0910711205903f,    -15.5094168762453f},
  { 2.09010887984729f,    76.5749203879789f},
  { -29.9103343793705f,    34.9007473264585f},
  { -132.845490233390f,    16.3149624937994f},
  { -125.708789519767f,    -55.7707122804385f},
  { -29.6210324408826f,    60.8603198206624f},
  { -162.149272286382f,    -76.5230371680387f},
  { 9.15187313289931f,    40.3847574282767f},
  { 138.608363798244f,    -13.4947797130046f},
  { 77.6392720799493f,    -24.3602805461700f},
  { -115.470562788177f,    69.5094146444459f},
  { 79.1129016841344f,    -58.8666362641537f},
  { -102.999255401046f,    7.29867179809849f},
  { 39.2113717754909f,    -32.8521869950436f},
  { 113.514023551486f,    -6.56800917648408f},
  { 113.623223030243f,    74.0686307595768f},
  { -81.9588289618524f,    -26.3232599158052f},
  { 143.912820987295f,    5.89853630152955f},
  { -52.6309026198533f,    9.25578332463549f},
  { 149.288302984237f,    0.408454613737791f},
  { -152.459018799162f,    74.3264487024600f},
  { 0.127595951816178f,    -6.52755572233228f},
  { 145.821905838254f,    68.1567091571640f},
  { 46.0322848671772f,    5.67311817942373f},
  { 146.533739684724f,    29.8442306997777f},
  { 117.312277649794f,    7.71928345316452f},
  { 78.0603952371364f,    -40.9215263427125f},
  { -123.247402118852f,    59.6000972305949f},
  { 127.513573606240f,    -5.06986479549042f},
  { -109.560502086974f,    14.3958195729735f},
  { -168.879484614382f,    -11.8915324246868f},
  { 90.6636835796201f,    -1.15745289147311f},
  { -71.2690562757332f,    -74.8689470968796f},
  { 9.21450957640742f,    -18.5327196098818f},
  { -70.7028442158494f,    -49.1746857529709f},
  { -131.545906778382f,    -19.5399190296941f},
  { 59.3306070981310f,    -13.2042610769474f},
  { 170.869477105216f,    68.9043100959912f},
  { -174.178996949888f,    40.0578380037105f},
  { -129.603028890642f,    9.61175375194759f},
  { -119.127961616512f,    -34.7649084815199f},
  { 76.5549118829935f,    -38.7654439809148f},
  { -132.438511705005f,    -2.95587972779214f},
  { 96.3069208259613f,    -54.5696984075897f},
  { -54.4752344862251f,    39.6485132228187f},
  { 6.59435142703094f,    -21.1510632994500f},
  { -73.4212365574985f,    -46.6190516350398f},
  { 123.484578612872f,    74.9709623224838f},
  { 59.0007875168060f,    66.7690478126826f},
  { -119.048791984731f,    20.8012312574888f},
  { 69.1368956664917f,    0.154261526442591f},
  { 149.391294702592f,    19.4195696175534f},
  { -168.340268999175f,    16.9400516162021f},
  { -90.1954200265675f,    -27.0800666743258f},
  { -120.308668283937f,    0.263934784114302f},
  { 65.3007533120038f,    -26.8631466055043f},
  { -68.0083925934770f,    -15.9884627635624f},
  { 64.5853190167752f,    13.6356613674728f},
  { -85.7713234114753f,    59.3341695762894f},
  { -165.365602186580f,    82.5298575355086f},
  { 46.2199488323651f,    -59.3578904921679f},
  { -149.353924679067f,    4.98252277557722f},
  { 85.7034919243867f,    62.1095277002024f},
  { -148.918708092873f,    52.5090811030748f},
  { 116.401458752743f,    -85.5775933596753f},
  { 111.492991794629f,    13.7327190023880f},
  { 63.7576199958557f,    19.7299889968305f},
  { 63.0759107646597f,    -16.3720771026781f},
  { -89.9984649967640f,    -37.1774274542115f},
  { -117.189169672889f,    -37.2406858930086f},
  { 99.0202274086698f,    42.4112112972201f},
  { 95.6862524209551f,    54.2319226181052f},
  { 17.7700596064288f,    -31.8413016321649f},
  { 9.85225828909682f,    -56.3438477986082f},
  { -4.74502306235263f,    -62.1570805224970f},
  { 40.6891956194858f,    -13.5361475798077f},
  { -58.1086810282021f,    10.7571443198955f},
  { 10.0692665510980f,    21.4516408592173f},
  { -42.0916257731336f,    -46.0075503880130f},
  { -77.0136631537621f,    -51.8893074254159f},
  { 27.8138229860156f,    -24.2637249102616f},
  { 168.824195388972f,    1.89551322590027f},
  { -89.9615481615776f,    42.1170394996486f},
  { -3.79215810892616f,    38.2830589232788f},
  { -161.567093227159f,    70.2683119004341f},
  { 165.072680439919f,    -59.0310601298339f},
  { -6.05247747397108f,    -32.6776339940352f},
  { 140.655633113947f,    83.3956608313680f},
  { -25.6655514801939f,    -10.0706577785609f},
  { -159.332381640469f,    -29.7617036593562f},
  { 88.6007237815586f,    -56.2281936402504f},
  { -134.282987159009f,    -7.76547379335372f},
  { -133.407090344503f,    -12.3612310283071f},
  { 127.767369822060f,    -25.2322167823102f},
  { 149.051954662763f,    37.8335365805038f},
  { -112.224552702438f,    41.4824350497544f},
  { -58.3773367296224f,    36.1652844612277f},
  { 135.697572696746f,    -66.5168645780371f},
  { 166.798709484637f,    10.1513786149524f},
  { -23.2110747853852f,    36.4083262614618f},
  { -158.234311902221f,    -11.4030199226861f},
  { -29.4559689986814f,    -45.2804459416641f},
  { 139.225944902692f,    -81.8242264840177f},
  { 32.3480695671567f,    -48.4152460628433f},
  { 168.609735448037f,    -8.81031539170456f},
  { 177.879871983754f,    19.1764505638229f},
  { -81.5602406143841f,    17.3333975924685f},
  { -46.3476396086641f,    -46.0284975695521f},
  { -91.2826281842281f,    66.4839737154617f},
  { 150.189650940582f,    54.4217535403336f},
  { 98.1304207756545f,    46.5737144954322f},
  { 71.9382313090495f,    23.7138578729902f},
  { 141.537535317571f,    -26.5066897550129f},
  { -122.339328298632f,    24.1151924049031f},
  { -113.673215702073f,    -11.6581605770965f},
  { 141.576331033818f,    63.0375977550849f},
  { 68.5222052035487f,    -53.5950034767582f},
  { 52.3837913007349f,    -15.8288698932269f},
  { -40.4987514485807f,    -33.2145194328305f},
  { 137.814584880280f,    -10.3907520484615f},
  { 179.734081887537f,    -43.2706646930475f},
  { 132.082495848615f,    -52.8094539594383f},
  { -66.3928728824137f,    -28.7484450397573f},
  { 34.2054498310080f,    44.6327009854747f},
  { -126.927259184511f,    -77.6452584179362f},
  { -75.2116963353449f,    -57.0908516652165f},
  { -44.7239838201039f,    68.1618521450301f},
  { 175.418679176699f,    -3.24912865780537f},
  { 126.219719925823f,    22.1490438705785f},
  { -10.7973586187035f,    66.3767367536038f},
  { -40.0431367064759f,    5.41063849273577f},
  { 3.86686164607098f,    65.6479368281171f},
  { 33.5035255261223f,    -19.8286430265157f},
  { 2.01290488072056f,    40.5394798045575f},
  { -155.888147500342f,    -18.8900312638985f},
  { 137.727839930450f,    41.5660665295109f},
  { 38.6463750557554f,    -2.75223296053804f},
  { -88.9113418835574f,    34.2944117546512f},
  { 72.3733934461288f,    34.8032913279942f},
  { 68.6049528955827f,    -40.8672007033017f},
  { -177.174942376009f,    19.7198244574299f},
  { 168.716332134351f,    -23.7428658517588f},
  { -6.35694555728808f,    18.1617679198804f},
  { 170.060889612043f,    -14.6670767237319f},
  { 110.692503598551f,    -34.4462371509779f},
  { 125.780192014100f,    -13.6296588420586f},
  { -126.966130515393f,    -13.3721060678769f},
  { 10.1774377602060f,    53.6903989043287f},
  { -82.9491465629807f,    11.8504348204251f},
  { -53.5396272549544f,    -10.9994261757589f},
  { 78.1326756474701f,    32.0075379439159f},
  { -159.888773800623f,    -55.6301997689592f},
  { 98.1647367509331f,    49.5083501966845f},
  { -63.0259803003666f,    -4.77251182994046f},
  { 20.9140478318519f,    -19.0720411281375f},
  { -79.9512956665237f,    68.4817238967080f},
  { 151.499532171498f,    47.3738213427314f},
  { 18.6469793924421f,    52.8628694165051f},
  { 103.626943665293f,    3.83933840447496f},
  { 93.8696237552820f,    -9.05463468442693f},
  { 148.796642737677f,    25.1449390379157f},
  { 44.0861000806496f,    28.9070533940600f},
  { 80.3507220595006f,    -0.529958226635486f},
  { -4.50814068749867f,    32.7732969889126f},
  { 151.963619765890f,    17.7243797336922f},
  { -39.3559418741735f,    -15.4330450095360f},
  { 63.2491990233159f,    30.4623973105923f},
  { -132.275275827336f,    -42.7252754175174f},
  { -169.464292058744f,    11.7459707840831f},
  { -25.9794969326115f,    53.4397331430159f},
  { 29.9714303682253f,    15.0054189310047f},
  { -93.1120403612788f,    -17.5716279469446f},
  { -134.029755525491f,    58.4387426510059f},
  { -16.1856697731825f,    -9.44481407742668f},
  { -151.868283862515f,    40.0695284547822f},
  { -8.47213864632326f,    -7.04791768450433f},
  { 96.6226586691290f,    77.7289008657874f},
  { 113.454910323281f,    17.5923988007467f},
  { 62.0056269962903f,    -51.7861346813342f},
  { 29.8172220834591f,    18.1376195203615f},
  { -3.29177687659080f,    9.09511828098826f},
  { 154.912281826063f,    -13.2202611838159f},
  { -137.320288723167f,    -48.1924930229303f},
  { -80.6421203474740f,    23.0239538217940f},
  { 142.577177657855f,    -30.0586950166840f},
  { -150.027191812917f,    23.0914937270495f},
  { -56.7564169461136f,    23.0891527918315f},
  { -108.778824938687f,    42.4515517411109f},
  { 177.374545710002f,    27.5889640319185f},
  { 1.94102485770550f,    33.4569630074206f},
  { -18.7716865054043f,    -18.4649545481562f},
  { -25.1034813127402f,    28.0109149983383f},
  { 33.2994515967163f,    47.2734490782859f},
  { -168.322519344254f,    26.3249094626074f},
  { -166.604703206083f,    4.01300593441310f},
  { 57.6590038076802f,    41.0003192831975f},
  { -5.21915316661720f,    57.4099403458448f},
  { -118.448536721084f,    12.7177049986953f},
  { -44.6708406456625f,    56.8168831376384f},
  { 169.855315543457f,    29.3576141818601f},
  { 44.1507282876054f,    51.4861448568760f},
  { -24.7520523930262f,    56.4907252814227f},
  { 49.6217198349201f,    35.6743184391846f},
  { 100.950455608386f,    -28.2131443600712f},
  { -10.2697516648761f,    -78.6220142607089f},
  { 165.004348286808f,    54.5754470511006f},
  { 35.0745507928465f,    -23.0464859965473f},
  { 168.230818158075f,    14.6126284138008f},
  { 175.896889995860f,    -22.2067900914894f},
  { -152.242280162677f,    77.4169475249471f},
  { -63.3836366827181f,    -54.3013683072767f},
  { -163.155648963993f,    -70.4016544068807f},
  { -112.628806748259f,    48.2129122837310f},
  { 149.936093100267f,    31.6760999125778f},
  { 134.965247690182f,    37.0292406739374f},
  { -67.1217288835171f,    -13.1804473656160f},
  { 111.418520920663f,    -23.0323278706626f},
  { 14.7395043062176f,    -1.67754359765327f},
  { -144.964519424894f,    -12.5293771941401f},
  { -67.0596034834402f,    -55.5579521467858f},
  { -110.792157741237f,    -16.6897874437738f},
  { -81.6013379653769f,    -20.8480543580473f},
  { -116.388065999780f,    -0.199520664095598f},
  { -130.812784247270f,    23.1405240470593f},
  { -33.2557773901847f,    -25.5676733854070f},
  { -95.5900667909122f,    -66.2685780710084f},
  { -117.478011829817f,    -22.0106922892138f},
  { -141.691782573907f,    64.9125112955009f},
  { 76.0647168641906f,    12.5626645297809f},
  { -63.7058550804950f,    41.9867299279931f},
  { 163.997001848024f,    10.7731207705523f},
  { 38.0157128215652f,    41.9398944224156f},
  { -123.484683960126f,    -33.1521948037556f},
  { -74.0093816116859f,    64.7174271624996f},
  { -158.401532654571f,    4.70073714019514f},
  { 17.4190388529285f,    -39.4811668817950f},
  { -80.5893238799655f,    -2.05959953438535f},
  { -60.3246938447940f,    -5.65497446251534f},
  { -146.313918637624f,    1.71143871307232f},
  { -156.286198137837f,    34.6440375056665f},
  { 125.018377352044f,    -49.9741188328188f},
  { -30.7577940368970f,    -2.10454476083660f},
  { -77.9638490361842f,    -2.82362063438452f},
  { 152.872442785706f,    -22.2701295093799f},
  { 104.578763741866f,    47.0720331345554f},
  { -11.1762469985188f,    -45.5656544352614f},
  { -4.68087281729577f,    -53.6220571354719f},
  { 55.6421190201258f,    18.3503649952846f},
  { 38.0439573333798f,    -51.2854250417478f},
  { -160.391959955208f,    -0.590889931370865f},
  { 32.6959400355056f,    15.7984645909217f},
  { 44.8870363654566f,    10.9070121585005f},
  { -30.2263000402381f,    5.04659267603735f},
  { 103.700509623753f,    14.0988534742932f},
  { 120.499906510138f,    -10.5432336322085f},
  { -17.0782166377125f,    -28.3536664014027f},
  { 76.3240051446162f,    0.837283967857134f},
  { -10.4007936882209f,    -61.9087522222797f},
  { 38.4760206433161f,    4.29946141054661f},
  { -126.087715858490f,    15.9665329498015f},
  { -135.933483494084f,    -29.1604953548073f},
  { -61.3499271222442f,    -17.7268104061847f},
  { 39.3753178104653f,    62.2336942496416f},
  { -10.1730650441343f,    -74.0579617493018f},
  { -5.07542502076061f,    11.0455351926701f},
  { 134.719413424734f,    -61.2546950762206f},
  { -127.160191051494f,    34.1298689414046f},
  { 171.620967251213f,    -56.3151333517369f},
  { 164.929177078032f,    17.9772003446941f},
  { 140.369259328877f,    -10.0794925730507f},
  { -102.207212233480f,    20.2960272241656f},
  { 154.996764396781f,    1.95766547893131f},
  { 112.609556360031f,    -38.5957126032353f},
  { -105.629015593568f,    -1.48902498866737f},
  { 26.0273069631424f,    -68.0968933422460f},
  { 168.462502392754f,    50.9756729786897f},
  { 19.1365714271925f,    -14.3384241532246f},
  { -137.587594805288f,    -18.6753700967494f},
  { -129.010394243129f,    41.0940650650316f},
  { 46.9143181629027f,    3.31314533193323f},
  { 104.055097945793f,    43.5615361963055f},
  { -90.4449588435833f,    3.71469119237678f},
  { -79.9310032533264f,    -73.6712496717014f},
  { -110.434640300678f,    -10.9683268071274f},
  { -9.37524445997150f,    -28.4569306408453f},
  { 113.641413778389f,    -72.2456930119684f},
  { -4.15220057533968f,    16.9563286710119f},
  { 49.0110080832298f,    38.8614598365319f},
  { -78.5802780638212f,    29.3199254220964f},
  { 116.504448521497f,    -23.0836641602584f},
  { 46.2680417304210f,    36.6270675845425f},
  { -122.806784520043f,    -61.8343721338794f},
  { 74.5727152156022f,    71.1024301432785f},
  { -41.5501100976493f,    27.6247112732190f},
  { -82.1751201399533f,    14.6232335898561f},
  { 88.2325298956136f,    72.5720085009507f},
  { 107.929993477337f,    -26.1891998542538f},
  { 150.760713046840f,    50.7415662161816f},
  { -177.727751791519f,    37.7145121186939f},
  { -169.405758424113f,    -39.8919382717409f},
  { -153.332890189067f,    -20.2402883700549f},
  { -50.8534844432131f,    -17.3340836582898f},
  { -174.347372464709f,    -28.0146205608293f},
  { -48.0802135314279f,    38.1445272797611f},
  { -58.4988501686537f,    -54.1895126313824f},
  { -166.184968457237f,    55.9576047613440f},
  { 3.06989946209256f,    -2.50879170313885f},
  { 20.9957486077986f,    2.11751059077362f},
  { 112.202039578162f,    8.72202083169227f},
  { -171.639158584406f,    9.76270698851306f},
  { -73.4610018783747f,    -13.8998248954034f},
  { 88.9867584755095f,    39.5248038831165f},
  { 105.588786884532f,    -32.4214717402202f},
  { -54.0588313303714f,    19.4834760662258f},
  { -47.3684414437086f,    49.1418843662286f},
  { -72.5834376718532f,    27.8070106755734f},
  { 135.561319702724f,    -18.7809816688234f},
  { -72.2288520680752f,    49.6919987103964f},
  { 123.889348941120f,    52.0574257295187f},
  { -149.433332705354f,    12.0147049226057f},
  { -44.7754004109703f,    1.91022092330936f},
  { 61.6358623454992f,    39.4207409089961f},
  { 50.0165221417108f,    47.9922579006854f},
  { 144.058010574235f,    25.7874367133622f},
  { 154.802610052074f,    6.50716971976569f},
  { -68.0196332800945f,    62.8657993900750f},
  { 147.721600463179f,    -63.8049887243174f},
  { 85.1513187266674f,    -9.70999246611946f},
  { 128.473313309236f,    58.1049566303861f},
  { 97.9717543716843f,    28.2279321494500f},
  { 59.6978946309126f,    -42.2679848551773f},
  { -6.22562647542839f,    45.9864707634367f},
  { 170.952713937180f,    32.2763487639339f},
  { -124.870467790617f,    -9.75292004018489f},
  { -179.659665250835f,    -36.6980637731376f},
  { 42.2935117670423f,    -38.5434744401805f},
  { -58.2051972450611f,    45.9199736303281f},
  { 140.009524729637f,    -69.7314968813966f},
  { -36.4518204096194f,    -8.80462147655749f},
  { -99.2730871222654f,    38.6283548682931f},
  { -146.542988119607f,    -40.9619390884210f},
  { -177.603144326025f,    -5.94712855224664f},
  { -48.9195026616110f,    26.6594457214366f},
  { 18.2338573563122f,    12.9481593901667f},
  { -16.4414730515758f,    -65.2958241542219f},
  { 157.510013542358f,    81.7243049279946f},
  { -170.366189118335f,    71.2947558564855f},
  { -17.0281396982214f,    -0.214626123421731f},
  { 111.683055893254f,    -25.5903546841355f},
  { 96.8513071322462f,    7.80081914661730f},
  { -35.0900739211275f,    -39.0284928576167f},
  { -50.2401067035727f,    40.2435138095444f},
  { 100.168662093200f,    64.6242451504367f},
  { 36.7450064293936f,    -9.79807916300374f},
  { 37.1764869134434f,    32.9059072799982f},
  { 153.214016825646f,    4.26064584765240f},
  { 124.926731214746f,    6.31232097458445f},
  { -178.380383754224f,    -40.2373240229352f},
  { -28.2109371514272f,    12.2196640459080f},
  { -141.646962020672f,    72.7994566938388f},
  { 44.1120359346370f,    -44.3646900205413f},
  { 130.866324460984f,    -44.1748238509645f},
  { 102.329878168935f,    -46.2469582005938f},
  { 17.7661254979442f,    -28.3364425867084f},
  { -162.269293493396f,    -45.7291417300263f},
  { 5.27187991045267f,    -7.08633293927769f},
  { -115.831274258435f,    7.37354745945388f},
  { 50.2609704663601f,    20.7499865955426f},
  { 157.430086903929f,    0.575112206850396f},
  { 100.866539463297f,    -22.7580059982164f},
  { -58.1555935988842f,    -37.9862943157380f},
  { 44.2615091559423f,    -65.1320157280903f},
  { 110.645386895356f,    -6.50620672998477f},
  { 159.809866131379f,    43.0210634299778f},
  { -143.172715625031f,    -15.6957516072835f},
  { -74.2575144932108f,    -37.3404883108800f},
  { 169.796836076809f,    8.85366090244810f},
  { -152.398874696765f,    33.7935054042650f},
  { -92.7971202139645f,    -28.9244097124329f},
  { -97.4434520314946f,    65.0540856267293f},
  { 30.5944247455151f,    -42.5227835363354f},
  { 179.390239619762f,    3.01587678922753f},
  { 24.6413727398486f,    3.96822060608715f},
  { 159.508824160550f,    -72.6478050529700f},
  { -99.3533847028109f,    -31.3830291585629f},
  { -92.1923168499385f,    44.2158989742383f},
  { -105.940784137436f,    -8.69895120881935f},
  { 116.933594553892f,    -44.7289945617809f},
  { 55.0071772766895f,    -68.9264035354923f},
  { -100.801074770293f,    5.40273744949736f},
  { -27.7190569961073f,    -12.6797305403720f},
  { -160.093597836081f,    -61.8453691358413f},
  { 36.0837584657790f,    -3.26623893742798f},
  { 138.679055170976f,    2.63505381463942f},
  { 118.227283400005f,    11.3476574165480f},
  { -90.7883094362236f,    -6.15341486095787f},
  { -5.24468303563075f,    -8.05090054598179f},
  { 12.4367489164145f,    33.3027001932534f},
  { 142.196710574647f,    12.9898206255174f},
  { 14.8046782845970f,    -25.9451245830986f},
  { 127.549476440558f,    6.26707108261434f},
  { 19.3507017968153f,    22.6722312605179f},
  { -143.683336266249f,    24.4157708029608f},
  { -146.679145049484f,    50.2572267096481f},
  { -20.6348197209687f,    38.8453775301901f},
  { 32.8181909816420f,    -63.7585346400210f},
  { 70.0260588085708f,    45.3450855123758f},
  { -152.273771790021f,    69.2348366197789f},
  { -158.614315635373f,    -6.74158116584303f},
  { -142.201789825837f,    18.9854526889250f},
  { -169.878422428320f,    48.0249442255521f},
  { 3.52100587121826f,    30.6474464336699f},
  { -69.1727551443023f,    37.8542425171815f},
  { 52.2556313838882f,    -27.0747794767833f},
  { -132.975262812012f,    -51.7551268128034f},
  { -131.096266843268f,    25.8059325740881f},
  { -58.2083218174186f,    -20.8876829672787f},
  { 170.692189634859f,    0.288033396543367f},
  { -173.825008268689f,    46.4511025777881f},
  { -174.885164945770f,    48.9650667088159f},
  { 2.87131640992491f,    6.60665390614924f},
  { -102.617894065522f,    54.4669065634202f},
  { 66.1024558247117f,    -44.1832198826274f},
  { 145.321813221506f,    60.3276979548377f},
  { 84.9089288794097f,    -15.5094168762453f},
  { -177.909891120153f,    76.5749203879789f},
  { 150.089665620630f,    34.9007473264585f},
  { -134.562955834973f,    48.7607997191196f},
  { 47.1545097666097f,    16.3149624937994f},
  { 54.2912104802332f,    -55.7707122804385f},
  { 150.378967559117f,    60.8603198206624f},
  { 17.8507277136182f,    -76.5230371680387f},
  { -170.848126867101f,    40.3847574282767f},
  { -41.3916362017557f,    -13.4947797130046f},
  { -102.360727920051f,    -24.3602805461700f},
  { 64.5294372118233f,    69.5094146444459f},
  { -100.887098315866f,    -58.8666362641537f},
  { 77.0007445989545f,    7.29867179809849f},
  { -140.788628224509f,    -32.8521869950436f},
  { -66.4859764485145f,    -6.56800917648408f},
  { -66.3767769697574f,    74.0686307595768f},
  { 98.0411710381476f,    -26.3232599158052f},
  { -36.0871790127050f,    5.89853630152955f},
  { 127.369097380147f,    9.25578332463549f},
  { -30.7116970157635f,    0.408454613737791f},
  { 27.5409812008385f,    74.3264487024600f},
  { -179.872404048184f,    -6.52755572233228f},
  { -34.1780941617459f,    68.1567091571640f},
  { -133.967715132823f,    5.67311817942373f},
  { -33.4662603152764f,    29.8442306997777f},
  { -62.6877223502062f,    7.71928345316452f},
  { -101.939604762864f,    -40.9215263427125f},
  { 56.7525978811481f,    59.6000972305949f},
  { -52.4864263937599f,    -5.06986479549042f},
  { 70.4394979130264f,    14.3958195729735f},
  { 11.1205153856184f,    -11.8915324246868f},
  { -89.3363164203799f,    -1.15745289147311f},
  { 108.730943724267f,    -74.8689470968796f},
  { -170.785490423593f,    -18.5327196098818f},
  { 109.297155784151f,    -49.1746857529709f},
  { 48.4540932216185f,    -19.5399190296941f},
  { -120.669392901869f,    -13.2042610769474f},
  { -9.13052289478372f,    68.9043100959912f},
  { 5.82100305011206f,    40.0578380037105f},
  { 50.3969711093577f,    9.61175375194759f},
  { 60.8720383834880f,    -34.7649084815199f},
  { -103.445088117007f,    -38.7654439809148f},
  { 47.5614882949955f,    -2.95587972779214f},
  { -83.6930791740387f,    -54.5696984075897f},
  { 125.524765513775f,    39.6485132228187f},
  { -173.405648572969f,    -21.1510632994500f},
  { 106.578763442502f,    -46.6190516350398f},
  { -56.5154213871281f,    74.9709623224838f},
  { -120.999212483194f,    66.7690478126826f},
  { 60.9512080152688f,    20.8012312574888f},
  { -110.863104333508f,    0.154261526442591f},
  { -30.6087052974076f,    19.4195696175534f},
  { 11.6597310008251f,    16.9400516162021f},
  { 89.8045799734326f,    -27.0800666743258f},
  { 59.6913317160634f,    0.263934784114302f},
  { -114.699246687996f,    -26.8631466055043f},
  { 111.991607406523f,    -15.9884627635624f},
  { -115.414680983225f,    13.6356613674728f},
  { 94.2286765885247f,    59.3341695762894f},
  { 14.6343978134203f,    82.5298575355086f},
  { -133.780051167635f,    -59.3578904921679f},
  { 30.6460753209330f,    4.98252277557722f},
  { -94.2965080756133f,    62.1095277002024f},
  { 31.0812919071270f,    52.5090811030748f},
  { -63.5985412472567f,    -85.5775933596753f},
  { -68.5070082053708f,    13.7327190023880f},
  { -116.242380004144f,    19.7299889968305f},
  { -116.924089235340f,    -16.3720771026781f},
  { 90.0015350032360f,    -37.1774274542115f},
  { 62.8108303271110f,    -37.2406858930086f},
  { -80.9797725913302f,    42.4112112972201f},
  { -84.3137475790449f,    54.2319226181052f},
  { -162.229940393571f,    -31.8413016321649f},
  { -170.147741710903f,    -56.3438477986082f},
  { 175.254976937647f,    -62.1570805224970f},
  { -139.310804380514f,    -13.5361475798077f},
  { 121.891318971798f,    10.7571443198955f},
  { -169.930733448902f,    21.4516408592173f},
  { 137.908374226866f,    -46.0075503880130f},
  { 102.986336846238f,    -51.8893074254159f},
  { -152.186177013984f,    -24.2637249102616f},
  { -11.1758046110281f,    1.89551322590027f},
  { 90.0384518384224f,    42.1170394996486f},
  { 176.207841891074f,    38.2830589232788f},
  { 18.4329067728413f,    70.2683119004341f},
  { -14.9273195600812f,    -59.0310601298339f},
  { 173.947522526029f,    -32.6776339940352f},
  { -39.3443668860528f,    83.3956608313680f},
  { 154.334448519806f,    -10.0706577785609f},
  { 20.6676183595315f,    -29.7617036593562f},
  { -91.3992762184414f,    -56.2281936402504f},
  { 45.7170128409913f,    -7.76547379335372f},
  { 46.5929096554972f,    -12.3612310283071f},
  { -52.2326301779398f,    -25.2322167823102f},
  { -30.9480453372373f,    37.8335365805038f},
  { 67.7754472975621f,    41.4824350497544f},
  { 121.622663270378f,    36.1652844612277f},
  { -44.3024273032537f,    -66.5168645780371f},
  { -13.2012905153634f,    10.1513786149524f},
  { 156.788925214615f,    36.4083262614618f},
  { 21.7656880977792f,    -11.4030199226861f},
  { 150.544031001319f,    -45.2804459416641f},
  { -40.7740550973078f,    -81.8242264840177f},
  { -147.651930432843f,    -48.4152460628433f},
  { -11.3902645519631f,    -8.81031539170456f},
  { -2.12012801624649f,    19.1764505638229f},
  { 98.4397593856159f,    17.3333975924685f},
  { 133.652360391336f,    -46.0284975695521f},
  { 88.7173718157719f,    66.4839737154617f},
  { -29.8103490594183f,    54.4217535403336f},
  { -81.8695792243455f,    46.5737144954322f},
  { -108.061768690950f,    23.7138578729902f},
  { -38.4624646824295f,    -26.5066897550129f},
  { 57.6606717013683f,    24.1151924049031f},
  { 66.3267842979266f,    -11.6581605770965f},
  { -38.4236689661825f,    63.0375977550849f},
  { -111.477794796451f,    -53.5950034767582f},
  { -127.616208699265f,    -15.8288698932269f},
  { 139.501248551419f,    -33.2145194328305f},
  { -42.1854151197198f,    -10.3907520484615f},
  { -0.265918112463301f,    -43.2706646930475f},
  { -47.9175041513846f,    -52.8094539594383f},
  { 113.607127117586f,    -28.7484450397573f},
  { -145.794550168992f,    44.6327009854747f},
  { 53.0727408154886f,    -77.6452584179362f},
  { 104.788303664655f,    -57.0908516652165f},
  { 135.276016179896f,    68.1618521450301f},
  { -4.58132082330074f,    -3.24912865780537f},
  { -53.7802800741766f,    22.1490438705785f},
  { 169.202641381296f,    66.3767367536038f},
  { 139.956863293524f,    5.41063849273577f},
  { -176.133138353929f,    65.6479368281171f},
  { -146.496474473878f,    -19.8286430265157f},
  { -177.987095119279f,    40.5394798045575f},
  { 24.1118524996585f,    -18.8900312638985f},
  { -42.2721600695499f,    41.5660665295109f},
  { -141.353624944245f,    -2.75223296053804f},
  { 91.0886581164427f,    34.2944117546512f},
  { -107.626606553871f,    34.8032913279942f},
  { -111.395047104417f,    -40.8672007033017f},
  { 2.82505762399135f,    19.7198244574299f},
  { -11.2836678656487f,    -23.7428658517588f},
  { 173.643054442712f,    18.1617679198804f},
  { -9.93911038795652f,    -14.6670767237319f},
  { -69.3074964014491f,    -34.4462371509779f},
  { -54.2198079859001f,    -13.6296588420586f},
  { 53.0338694846072f,    -13.3721060678769f},
  { -169.822562239794f,    53.6903989043287f},
  { 97.0508534370193f,    11.8504348204251f},
  { 126.460372745046f,    -10.9994261757589f},
  { -101.867324352530f,    32.0075379439159f},
  { 20.1112261993772f,    -55.6301997689592f},
  { -81.8352632490669f,    49.5083501966845f},
  { 116.974019699633f,    -4.77251182994046f},
  { -159.085952168148f,    -19.0720411281375f},
  { 100.048704333476f,    68.4817238967080f},
  { -28.5004678285019f,    47.3738213427314f},
  { -161.353020607558f,    52.8628694165051f},
  { -76.3730563347074f,    3.83933840447496f},
  { -86.1303762447180f,    -9.05463468442693f},
  { -31.2033572623228f,    25.1449390379157f},
  { -135.913899919350f,    28.9070533940600f},
  { -99.6492779404994f,    -0.529958226635486f},
  { 175.491859312501f,    32.7732969889126f},
  { -28.0363802341099f,    17.7243797336922f},
  { 140.644058125827f,    -15.4330450095360f},
  { -116.750800976684f,    30.4623973105923f},
  { 47.7247241726644f,    -42.7252754175174f},
  { 10.5357079412558f,    11.7459707840831f},
  { 154.020503067389f,    53.4397331430159f},
  { -150.028569631775f,    15.0054189310047f},
  { 86.8879596387212f,    -17.5716279469446f},
  { 45.9702444745094f,    58.4387426510059f},
  { 163.814330226817f,    -9.44481407742668f},
  { 28.1317161374850f,    40.0695284547822f},
  { 171.527861353677f,    -7.04791768450433f},
  { -83.3773413308710f,    77.7289008657874f},
  { -66.5450896767189f,    17.5923988007467f},
  { -117.994373003710f,    -51.7861346813342f},
  { -150.182777916541f,    18.1376195203615f},
  { 176.708223123409f,    9.09511828098826f},
  { -25.0877181739368f,    -13.2202611838159f},
  { 42.6797112768326f,    -48.1924930229303f},
  { 99.3578796525260f,    23.0239538217940f},
  { -37.4228223421450f,    -30.0586950166840f},
  { 29.9728081870829f,    23.0914937270495f},
  { 123.243583053886f,    23.0891527918315f},
  { 71.2211750613129f,    42.4515517411109f},
  { -2.62545428999846f,    27.5889640319185f},
  { -178.058975142295f,    33.4569630074206f},
  { 161.228313494596f,    -18.4649545481562f},
  { 154.896518687260f,    28.0109149983383f},
  { -146.700548403284f,    47.2734490782859f},
  { 11.6774806557457f,    26.3249094626074f},
  { 13.3952967939166f,    4.01300593441310f},
  { -122.340996192320f,    41.0003192831975f},
  { 174.780846833383f,    57.4099403458448f},
  { 61.5514632789159f,    12.7177049986953f},
  { 135.329159354338f,    56.8168831376384f},
  { -10.1446844565427f,    29.3576141818601f},
  { -135.849271712395f,    51.4861448568760f},
  { 155.247947606974f,    56.4907252814227f},
  { -130.378280165080f,    35.6743184391846f},
  { -79.0495443916138f,    -28.2131443600712f},
  { 169.730248335124f,    -78.6220142607089f},
  { -14.9956517131916f,    54.5754470511006f},
  { -144.925449207154f,    -23.0464859965473f},
  { -11.7691818419248f,    14.6126284138008f},
  { -4.10311000413954f,    -22.2067900914894f},
  { 27.7577198373228f,    77.4169475249471f},
  { 116.616363317282f,    -54.3013683072767f},
  { 16.8443510360069f,    -70.4016544068807f},
  { 67.3711932517415f,    48.2129122837310f},
  { -30.0639068997333f,    31.6760999125778f},
  { -45.0347523098179f,    37.0292406739374f},
  { 112.878271116483f,    -13.1804473656160f},
  { -68.5814790793370f,    -23.0323278706626f},
  { -165.260495693782f,    -1.67754359765327f},
  { 35.0354805751063f,    -12.5293771941401f},
  { 112.940396516560f,    -55.5579521467858f},
  { 69.2078422587628f,    -16.6897874437738f},
  { 98.3986620346231f,    -20.8480543580473f},
  { 63.6119340002199f,    -0.199520664095598f},
  { 49.1872157527301f,    23.1405240470593f},
  { 146.744222609815f,    -25.5676733854070f},
  { 84.4099332090879f,    -66.2685780710084f},
  { 62.5219881701826f,    -22.0106922892138f},
  { 38.3082174260928f,    64.9125112955009f},
  { -103.935283135809f,    12.5626645297809f},
  { 116.294144919505f,    41.9867299279931f},
  { -16.0029981519764f,    10.7731207705523f},
  { -141.984287178435f,    41.9398944224156f},
  { 56.5153160398744f,    -33.1521948037556f},
  { 105.990618388314f,    64.7174271624996f},
  { 21.5984673454286f,    4.70073714019514f},
  { -162.580961147072f,    -39.4811668817950f},
  { 99.4106761200345f,    -2.05959953438535f},
  { 119.675306155206f,    -5.65497446251534f},
  { 33.6860813623761f,    1.71143871307232f},
  { 23.7138018621635f,    34.6440375056665f},
  { -54.9816226479558f,    -49.9741188328188f},
  { 149.242205963103f,    -2.10454476083660f},
  { 102.036150963816f,    -2.82362063438452f},
  { -27.1275572142944f,    -22.2701295093799f},
  { -75.4212362581339f,    47.0720331345554f},
  { 168.823753001481f,    -45.5656544352614f},
  { 175.319127182704f,    -53.6220571354719f},
  { -124.357880979874f,    18.3503649952846f},
  { -141.956042666620f,    -51.2854250417478f},
  { 19.6080400447921f,    -0.590889931370865f},
  { -147.304059964494f,    15.7984645909217f},
  { -44.8870363654566f,    -10.9070121585005f},
  { 30.2263000402381f,    -5.04659267603735f},
  { -103.700509623753f,    -14.0988534742932f},
  { -120.499906510138f,    10.5432336322085f},
  { 17.0782166377125f,    28.3536664014027f},
  { -76.3240051446162f,    -0.837283967857134f},
  { 10.4007936882209f,    61.9087522222797f},
  { -38.4760206433161f,    -4.29946141054661f},
  { 126.087715858490f,    -15.9665329498015f},
  { 135.933483494084f,    29.1604953548073f},
  { 61.3499271222442f,    17.7268104061847f},
  { -39.3753178104653f,    -62.2336942496416f},
  { 10.1730650441343f,    74.0579617493018f},
  { 5.07542502076061f,    -11.0455351926701f},
  { -134.719413424734f,    61.2546950762206f},
  { 127.160191051494f,    -34.1298689414046f},
  { -171.620967251213f,    56.3151333517369f},
  { -164.929177078032f,    -17.9772003446941f},
  { -140.369259328877f,    10.0794925730507f},
  { 102.207212233480f,    -20.2960272241656f},
  { -154.996764396781f,    -1.95766547893131f},
  { -112.609556360031f,    38.5957126032353f},
  { 105.629015593568f,    1.48902498866737f},
  { -26.0273069631424f,    68.0968933422460f},
  { -168.462502392754f,    -50.9756729786897f},
  { -19.1365714271925f,    14.3384241532246f},
  { 137.587594805288f,    18.6753700967494f},
  { 129.010394243129f,    -41.0940650650316f},
  { -46.9143181629027f,    -3.31314533193323f},
  { -104.055097945793f,    -43.5615361963055f},
  { 90.4449588435833f,    -3.71469119237678f},
  { 79.9310032533264f,    73.6712496717014f},
  { 110.434640300678f,    10.9683268071274f},
  { 9.37524445997150f,    28.4569306408453f},
  { -113.641413778389f,    72.2456930119684f},
  { 4.15220057533968f,    -16.9563286710119f},
  { -49.0110080832298f,    -38.8614598365319f},
  { 78.5802780638212f,    -29.3199254220964f},
  { -116.504448521497f,    23.0836641602584f},
  { -46.2680417304210f,    -36.6270675845425f},
  { 122.806784520043f,    61.8343721338794f},
  { -74.5727152156022f,    -71.1024301432785f},
  { 41.5501100976493f,    -27.6247112732190f},
  { 82.1751201399533f,    -14.6232335898561f},
  { -88.2325298956136f,    -72.5720085009507f},
  { -107.929993477337f,    26.1891998542538f},
  { -150.760713046840f,    -50.7415662161816f},
  { 177.727751791519f,    -37.7145121186939f},
  { 169.405758424113f,    39.8919382717409f},
  { 153.332890189067f,    20.2402883700549f},
  { 50.8534844432131f,    17.3340836582898f},
  { 174.347372464709f,    28.0146205608293f},
  { 48.0802135314279f,    -38.1445272797611f},
  { 58.4988501686537f,    54.1895126313824f},
  { 166.184968457237f,    -55.9576047613440f},
  { -3.06989946209256f,    2.50879170313885f},
  { -20.9957486077986f,    -2.11751059077362f},
  { -112.202039578162f,    -8.72202083169227f},
  { 171.639158584406f,    -9.76270698851306f},
  { 73.4610018783747f,    13.8998248954034f},
  { -88.9867584755095f,    -39.5248038831165f},
  { -105.588786884532f,    32.4214717402202f},
  { 54.0588313303714f,    -19.4834760662258f},
  { 47.3684414437086f,    -49.1418843662286f},
  { 72.5834376718532f,    -27.8070106755734f},
  { -135.561319702724f,    18.7809816688234f},
  { 72.2288520680752f,    -49.6919987103964f},
  { -123.889348941120f,    -52.0574257295187f},
  { 149.433332705354f,    -12.0147049226057f},
  { 44.7754004109703f,    -1.91022092330936f},
  { -61.6358623454992f,    -39.4207409089961f},
  { -50.0165221417108f,    -47.9922579006854f},
  { -144.058010574235f,    -25.7874367133622f},
  { -154.802610052074f,    -6.50716971976569f},
  { 68.0196332800945f,    -62.8657993900750f},
  { -147.721600463179f,    63.8049887243174f},
  { -85.1513187266674f,    9.70999246611946f},
  { -128.473313309236f,    -58.1049566303861f},
  { -97.9717543716843f,    -28.2279321494500f},
  { -59.6978946309126f,    42.2679848551773f},
  { 6.22562647542839f,    -45.9864707634367f},
  { -170.952713937180f,    -32.2763487639339f},
  { 124.870467790617f,    9.75292004018489f},
  { 179.659665250835f,    36.6980637731376f},
  { -42.2935117670423f,    38.5434744401805f},
  { 58.2051972450611f,    -45.9199736303281f},
  { -140.009524729637f,    69.7314968813966f},
  { 36.4518204096194f,    8.80462147655749f},
  { 99.2730871222654f,    -38.6283548682931f},
  { 146.542988119607f,    40.9619390884210f},
  { 177.603144326025f,    5.94712855224664f},
  { 48.9195026616110f,    -26.6594457214366f},
  { -18.2338573563122f,    -12.9481593901667f},
  { 16.4414730515758f,    65.2958241542219f},
  { -157.510013542358f,    -81.7243049279946f},
  { 170.366189118335f,    -71.2947558564855f},
  { 17.0281396982214f,    0.214626123421731f},
  { -111.683055893254f,    25.5903546841355f},
  { -96.8513071322462f,    -7.80081914661730f},
  { 35.0900739211275f,    39.0284928576167f},
  { 50.2401067035727f,    -40.2435138095444f},
  { -100.168662093200f,    -64.6242451504367f},
  { -36.7450064293936f,    9.79807916300374f},
  { -37.1764869134434f,    -32.9059072799982f},
  { -153.214016825646f,    -4.26064584765240f},
  { -124.926731214746f,    -6.31232097458445f},
  { 178.380383754224f,    40.2373240229352f},
  { 28.2109371514272f,    -12.2196640459080f},
  { 141.646962020672f,    -72.7994566938388f},
  { -44.1120359346370f,    44.3646900205413f},
  { -130.866324460984f,    44.1748238509645f},
  { -102.329878168935f,    46.2469582005938f},
  { -17.7661254979442f,    28.3364425867084f},
  { 162.269293493396f,    45.7291417300263f},
  { -5.27187991045267f,    7.08633293927769f},
  { 115.831274258435f,    -7.37354745945388f},
  { -50.2609704663601f,    -20.7499865955426f},
  { -157.430086903929f,    -0.575112206850396f},
  { -100.866539463297f,    22.7580059982164f},
  { 58.1555935988842f,    37.9862943157380f},
  { -44.2615091559423f,    65.1320157280903f},
  { -110.645386895356f,    6.50620672998477f},
  { -159.809866131379f,    -43.0210634299778f},
  { 143.172715625031f,    15.6957516072835f},
  { 74.2575144932108f,    37.3404883108800f},
  { -169.796836076809f,    -8.85366090244810f},
  { 152.398874696765f,    -33.7935054042650f},
  { 92.7971202139645f,    28.9244097124329f},
  { 97.4434520314946f,    -65.0540856267293f},
  { -30.5944247455151f,    42.5227835363354f},
  { -179.390239619762f,    -3.01587678922753f},
  { -24.6413727398486f,    -3.96822060608715f},
  { -159.508824160550f,    72.6478050529700f},
  { 99.3533847028109f,    31.3830291585629f},
  { 92.1923168499385f,    -44.2158989742383f},
  { 105.940784137436f,    8.69895120881935f},
  { -116.933594553892f,    44.7289945617809f},
  { -55.0071772766895f,    68.9264035354923f},
  { 100.801074770293f,    -5.40273744949736f},
  { 27.7190569961073f,    12.6797305403720f},
  { 160.093597836081f,    61.8453691358413f},
  { -36.0837584657790f,    3.26623893742798f},
  { -138.679055170976f,    -2.63505381463942f},
  { -118.227283400005f,    -11.3476574165480f},
  { 90.7883094362236f,    6.15341486095787f},
  { 5.24468303563075f,    8.05090054598179f},
  { -12.4367489164145f,    -33.3027001932534f},
  { -142.196710574647f,    -12.9898206255174f},
  { -14.8046782845970f,    25.9451245830986f},
  { -127.549476440558f,    -6.26707108261434f},
  { -19.3507017968153f,    -22.6722312605179f},
  { 143.683336266249f,    -24.4157708029608f},
  { 146.679145049484f,    -50.2572267096481f},
  { 20.6348197209687f,    -38.8453775301901f},
  { -32.8181909816420f,    63.7585346400210f},
  { -70.0260588085708f,    -45.3450855123758f},
  { 152.273771790021f,    -69.2348366197789f},
  { 158.614315635373f,    6.74158116584303f},
  { 142.201789825837f,    -18.9854526889250f},
  { 169.878422428320f,    -48.0249442255521f},
  { -3.52100587121826f,    -30.6474464336699f},
  { 69.1727551443023f,    -37.8542425171815f},
  { -52.2556313838882f,    27.0747794767833f},
  { 132.975262812012f,    51.7551268128034f},
  { 131.096266843268f,    -25.8059325740881f},
  { 58.2083218174186f,    20.8876829672787f},
  { -170.692189634859f,    -0.288033396543367f},
  { 173.825008268689f,    -46.4511025777881f},
  { 174.885164945770f,    -48.9650667088159f},
  { -2.87131640992491f,    -6.60665390614924f},
  { 102.617894065522f,    -54.4669065634202f},
  { -66.1024558247117f,    44.1832198826274f},
  { -145.321813221506f,    -60.3276979548377f},
  { -84.9089288794097f,    15.5094168762453f},
  { 177.909891120153f,    -76.5749203879789f},
  { -150.089665620630f,    -34.9007473264585f},
  { 134.562955834973f,    -48.7607997191196f},
  { -47.1545097666097f,    -16.3149624937994f},
  { -54.2912104802332f,    55.7707122804385f},
  { -150.378967559117f,    -60.8603198206624f},
  { -17.8507277136182f,    76.5230371680387f},
  { 170.848126867101f,    -40.3847574282767f},
  { 41.3916362017557f,    13.4947797130046f},
  { 102.360727920051f,    24.3602805461700f},
  { -64.5294372118233f,    -69.5094146444459f},
  { 100.887098315866f,    58.8666362641537f},
  { -77.0007445989545f,    -7.29867179809849f},
  { 140.788628224509f,    32.8521869950436f},
  { 66.4859764485145f,    6.56800917648408f},
  { 66.3767769697574f,    -74.0686307595768f},
  { -98.0411710381476f,    26.3232599158052f},
  { 36.0871790127050f,    -5.89853630152955f},
  { -127.369097380147f,    -9.25578332463549f},
  { 30.7116970157635f,    -0.408454613737791f},
  { -27.5409812008385f,    -74.3264487024600f},
  { 179.872404048184f,    6.52755572233228f},
  { 34.1780941617459f,    -68.1567091571640f},
  { 133.967715132823f,    -5.67311817942373f},
  { 33.4662603152764f,    -29.8442306997777f},
  { 62.6877223502062f,    -7.71928345316452f},
  { 101.939604762864f,    40.9215263427125f},
  { -56.7525978811481f,    -59.6000972305949f},
  { 52.4864263937599f,    5.06986479549042f},
  { -70.4394979130264f,    -14.3958195729735f},
  { -11.1205153856184f,    11.8915324246868f},
  { 89.3363164203799f,    1.15745289147311f},
  { -108.730943724267f,    74.8689470968796f},
  { 170.785490423593f,    18.5327196098818f},
  { -109.297155784151f,    49.1746857529709f},
  { -48.4540932216185f,    19.5399190296941f},
  { 120.669392901869f,    13.2042610769474f},
  { 9.13052289478372f,    -68.9043100959912f},
  { -5.82100305011206f,    -40.0578380037105f},
  { -50.3969711093577f,    -9.61175375194759f},
  { -60.8720383834880f,    34.7649084815199f},
  { 103.445088117007f,    38.7654439809148f},
  { -47.5614882949955f,    2.95587972779214f},
  { 83.6930791740387f,    54.5696984075897f},
  { -125.524765513775f,    -39.6485132228187f},
  { 173.405648572969f,    21.1510632994500f},
  { -106.578763442502f,    46.6190516350398f},
  { 56.5154213871281f,    -74.9709623224838f},
  { 120.999212483194f,    -66.7690478126826f},
  { -60.9512080152688f,    -20.8012312574888f},
  { 110.863104333508f,    -0.154261526442591f},
  { 30.6087052974076f,    -19.4195696175534f},
  { -11.6597310008251f,    -16.9400516162021f},
  { -89.8045799734326f,    27.0800666743258f},
  { -59.6913317160634f,    -0.263934784114302f},
  { 114.699246687996f,    26.8631466055043f},
  { -111.991607406523f,    15.9884627635624f},
  { 115.414680983225f,    -13.6356613674728f},
  { -94.2286765885247f,    -59.3341695762894f},
  { -14.6343978134203f,    -82.5298575355086f},
  { 133.780051167635f,    59.3578904921679f},
  { -30.6460753209330f,    -4.98252277557722f},
  { 94.2965080756133f,    -62.1095277002024f},
  { -31.0812919071270f,    -52.5090811030748f},
  { 63.5985412472567f,    85.5775933596753f},
  { 68.5070082053708f,    -13.7327190023880f},
  { 116.242380004144f,    -19.7299889968305f},
  { 116.924089235340f,    16.3720771026781f},
  { -90.0015350032360f,    37.1774274542115f},
  { -62.8108303271110f,    37.2406858930086f},
  { 80.9797725913302f,    -42.4112112972201f},
  { 84.3137475790449f,    -54.2319226181052f},
  { 162.229940393571f,    31.8413016321649f},
  { 170.147741710903f,    56.3438477986082f},
  { -175.254976937647f,    62.1570805224970f},
  { 139.310804380514f,    13.5361475798077f},
  { -121.891318971798f,    -10.7571443198955f},
  { 169.930733448902f,    -21.4516408592173f},
  { -137.908374226866f,    46.0075503880130f},
  { -102.986336846238f,    51.8893074254159f},
  { 152.186177013984f,    24.2637249102616f},
  { 11.1758046110281f,    -1.89551322590027f},
  { -90.0384518384224f,    -42.1170394996486f},
  { -176.207841891074f,    -38.2830589232788f},
  { -18.4329067728413f,    -70.2683119004341f},
  { 14.9273195600812f,    59.0310601298339f},
  { -173.947522526029f,    32.6776339940352f},
  { 39.3443668860528f,    -83.3956608313680f},
  { -154.334448519806f,    10.0706577785609f},
  { -20.6676183595315f,    29.7617036593562f},
  { 91.3992762184414f,    56.2281936402504f},
  { -45.7170128409913f,    7.76547379335372f},
  { -46.5929096554972f,    12.3612310283071f},
  { 52.2326301779398f,    25.2322167823102f},
  { 30.9480453372373f,    -37.8335365805038f},
  { -67.7754472975621f,    -41.4824350497544f},
  { -121.622663270378f,    -36.1652844612277f},
  { 44.3024273032537f,    66.5168645780371f},
  { 13.2012905153634f,    -10.1513786149524f},
  { -156.788925214615f,    -36.4083262614618f},
  { -21.7656880977792f,    11.4030199226861f},
  { -150.544031001319f,    45.2804459416641f},
  { 40.7740550973078f,    81.8242264840177f},
  { 147.651930432843f,    48.4152460628433f},
  { 11.3902645519631f,    8.81031539170456f},
  { 2.12012801624649f,    -19.1764505638229f},
  { -98.4397593856159f,    -17.3333975924685f},
  { -133.652360391336f,    46.0284975695521f},
  { -88.7173718157719f,    -66.4839737154617f},
  { 29.8103490594183f,    -54.4217535403336f},
  { 81.8695792243455f,    -46.5737144954322f},
  { 108.061768690950f,    -23.7138578729902f},
  { 38.4624646824295f,    26.5066897550129f},
  { -57.6606717013683f,    -24.1151924049031f},
  { -66.3267842979266f,    11.6581605770965f},
  { 38.4236689661825f,    -63.0375977550849f},
  { 111.477794796451f,    53.5950034767582f},
  { 127.616208699265f,    15.8288698932269f},
  { -139.501248551419f,    33.2145194328305f},
  { 42.1854151197198f,    10.3907520484615f},
  { 0.265918112463301f,    43.2706646930475f},
  { 47.9175041513846f,    52.8094539594383f},
  { -113.607127117586f,    28.7484450397573f},
  { 145.794550168992f,    -44.6327009854747f},
  { -53.0727408154886f,    77.6452584179362f},
  { -104.788303664655f,    57.0908516652165f},
  { -135.276016179896f,    -68.1618521450301f},
  { 4.58132082330074f,    3.24912865780537f},
  { 53.7802800741766f,    -22.1490438705785f},
  { -169.202641381296f,    -66.3767367536038f},
  { -139.956863293524f,    -5.41063849273577f},
  { 176.133138353929f,    -65.6479368281171f},
  { 146.496474473878f,    19.8286430265157f},
  { 177.987095119279f,    -40.5394798045575f},
  { -24.1118524996585f,    18.8900312638985f},
  { 42.2721600695499f,    -41.5660665295109f},
  { 141.353624944245f,    2.75223296053804f},
  { -91.0886581164427f,    -34.2944117546512f},
  { 107.626606553871f,    -34.8032913279942f},
  { 111.395047104417f,    40.8672007033017f},
  { -2.82505762399135f,    -19.7198244574299f},
  { 11.2836678656487f,    23.7428658517588f},
  { -173.643054442712f,    -18.1617679198804f},
  { 9.93911038795652f,    14.6670767237319f},
  { 69.3074964014491f,    34.4462371509779f},
  { 54.2198079859001f,    13.6296588420586f},
  { -53.0338694846072f,    13.3721060678769f},
  { 169.822562239794f,    -53.6903989043287f},
  { -97.0508534370193f,    -11.8504348204251f},
  { -126.460372745046f,    10.9994261757589f},
  { 101.867324352530f,    -32.0075379439159f},
  { -20.1112261993772f,    55.6301997689592f},
  { 81.8352632490669f,    -49.5083501966845f},
  { -116.974019699633f,    4.77251182994046f},
  { 159.085952168148f,    19.0720411281375f},
  { -100.048704333476f,    -68.4817238967080f},
  { 28.5004678285019f,    -47.3738213427314f},
  { 161.353020607558f,    -52.8628694165051f},
  { 76.3730563347074f,    -3.83933840447496f},
  { 86.1303762447180f,    9.05463468442693f},
  { 31.2033572623228f,    -25.1449390379157f},
  { 135.913899919350f,    -28.9070533940600f},
  { 99.6492779404994f,    0.529958226635486f},
  { -175.491859312501f,    -32.7732969889126f},
  { 28.0363802341099f,    -17.7243797336922f},
  { -140.644058125827f,    15.4330450095360f},
  { 116.750800976684f,    -30.4623973105923f},
  { -47.7247241726644f,    42.7252754175174f},
  { -10.5357079412558f,    -11.7459707840831f},
  { -154.020503067389f,    -53.4397331430159f},
  { 150.028569631775f,    -15.0054189310047f},
  { -86.8879596387212f,    17.5716279469446f},
  { -45.9702444745094f,    -58.4387426510059f},
  { -163.814330226817f,    9.44481407742668f},
  { -28.1317161374850f,    -40.0695284547822f},
  { -171.527861353677f,    7.04791768450433f},
  { 83.3773413308710f,    -77.7289008657874f},
  { 66.5450896767189f,    -17.5923988007467f},
  { 117.994373003710f,    51.7861346813342f},
  { 150.182777916541f,    -18.1376195203615f},
  { -176.708223123409f,    -9.09511828098826f},
  { 25.0877181739368f,    13.2202611838159f},
  { -42.6797112768326f,    48.1924930229303f},
  { -99.3578796525260f,    -23.0239538217940f},
  { 37.4228223421450f,    30.0586950166840f},
  { -29.9728081870829f,    -23.0914937270495f},
  { -123.243583053886f,    -23.0891527918315f},
  { -71.2211750613129f,    -42.4515517411109f},
  { 2.62545428999846f,    -27.5889640319185f},
  { 178.058975142295f,    -33.4569630074206f},
  { -161.228313494596f,    18.4649545481562f},
  { -154.896518687260f,    -28.0109149983383f},
  { 146.700548403284f,    -47.2734490782859f},
  { -11.6774806557457f,    -26.3249094626074f},
  { -13.3952967939166f,    -4.01300593441310f},
  { 122.340996192320f,    -41.0003192831975f},
  { -174.780846833383f,    -57.4099403458448f},
  { -61.5514632789159f,    -12.7177049986953f},
  { -135.329159354338f,    -56.8168831376384f},
  { 10.1446844565427f,    -29.3576141818601f},
  { 135.849271712395f,    -51.4861448568760f},
  { -155.247947606974f,    -56.4907252814227f},
  { 130.378280165080f,    -35.6743184391846f},
  { 79.0495443916138f,    28.2131443600712f},
  { -169.730248335124f,    78.6220142607089f},
  { 14.9956517131916f,    -54.5754470511006f},
  { 144.925449207154f,    23.0464859965473f},
  { 11.7691818419248f,    -14.6126284138008f},
  { 4.10311000413954f,    22.2067900914894f},
  { -27.7577198373228f,    -77.4169475249471f},
  { -116.616363317282f,    54.3013683072767f},
  { -16.8443510360069f,    70.4016544068807f},
  { -67.3711932517415f,    -48.2129122837310f},
  { 30.0639068997333f,    -31.6760999125778f},
  { 45.0347523098179f,    -37.0292406739374f},
  { -112.878271116483f,    13.1804473656160f},
  { 68.5814790793370f,    23.0323278706626f},
  { 165.260495693782f,    1.67754359765327f},
  { -35.0354805751063f,    12.5293771941401f},
  { -112.940396516560f,    55.5579521467858f},
  { -69.2078422587628f,    16.6897874437738f},
  { -98.3986620346231f,    20.8480543580473f},
  { -63.6119340002199f,    0.199520664095598f},
  { -49.1872157527301f,    -23.1405240470593f},
  { -146.744222609815f,    25.5676733854070f},
  { -84.4099332090879f,    66.2685780710084f},
  { -62.5219881701826f,    22.0106922892138f},
  { -38.3082174260928f,    -64.9125112955009f},
  { 103.935283135809f,    -12.5626645297809f},
  { -116.294144919505f,    -41.9867299279931f},
  { 16.0029981519764f,    -10.7731207705523f},
  { 141.984287178435f,    -41.9398944224156f},
  { -56.5153160398744f,    33.1521948037556f},
  { -105.990618388314f,    -64.7174271624996f},
  { -21.5984673454286f,    -4.70073714019514f},
  { 162.580961147072f,    39.4811668817950f},
  { -99.4106761200345f,    2.05959953438535f},
  { -119.675306155206f,    5.65497446251534f},
  { -33.6860813623761f,    -1.71143871307232f},
  { -23.7138018621635f,    -34.6440375056665f},
  { 54.9816226479558f,    49.9741188328188f},
  { -149.242205963103f,    2.10454476083660f},
  { -102.036150963816f,    2.82362063438452f},
  { 27.1275572142944f,    22.2701295093799f},
  { 75.4212362581339f,    -47.0720331345554f},
  { -168.823753001481f,    45.5656544352614f},
  { -175.319127182704f,    53.6220571354719f},
  { 124.357880979874f,    -18.3503649952846f},
  { 141.956042666620f,    51.2854250417478f},
  { -19.6080400447921f,    0.590889931370865f},
  { 147.304059964494f,    -15.7984645909217f}};

/* Number of points in each minimum T-Design */
const int __Tdesign_nPoints_per_degree[21] =
{   2, 4, 6, 12, 12, 24, 24, 36, 48, 60, 70, 84, 94, 108, 120, 144, 156, 180, 204, 216, 240   };

/* Minimum T-design HANDLES */
const float* __HANDLES_Tdesign_dirs_deg[21] =
{ &__Tdesign_degree_1_dirs_deg[0][0],
  &__Tdesign_degree_2_dirs_deg[0][0],
  &__Tdesign_degree_3_dirs_deg[0][0],
  &__Tdesign_degree_4_dirs_deg[0][0],
  &__Tdesign_degree_5_dirs_deg[0][0],
  &__Tdesign_degree_6_dirs_deg[0][0],
  &__Tdesign_degree_7_dirs_deg[0][0],
  &__Tdesign_degree_8_dirs_deg[0][0],
  &__Tdesign_degree_9_dirs_deg[0][0],
  &__Tdesign_degree_10_dirs_deg[0][0],
  &__Tdesign_degree_11_dirs_deg[0][0],
  &__Tdesign_degree_12_dirs_deg[0][0],
  &__Tdesign_degree_13_dirs_deg[0][0],
  &__Tdesign_degree_14_dirs_deg[0][0],
  &__Tdesign_degree_15_dirs_deg[0][0],
  &__Tdesign_degree_16_dirs_deg[0][0],
  &__Tdesign_degree_17_dirs_deg[0][0],
  &__Tdesign_degree_18_dirs_deg[0][0],
  &__Tdesign_degree_19_dirs_deg[0][0],
  &__Tdesign_degree_20_dirs_deg[0][0],
  &__Tdesign_degree_21_dirs_deg[0][0]};

/* Sphere Covering (4 points) */
const float __SphCovering_4_dirs_deg[4][2] =
{ { -161.035517899469f,    32.3554996488327f},
  { -66.6980169311791f,    -31.3940764686032f},
  { 24.3822189717922f,    37.9922584373298f},
  { 109.291699421205f,    -39.0356145822630f}};

/* Sphere Covering (5 points) */
const float __SphCovering_5_dirs_deg[5][2] =
{ { 0.0f,    -90.0f},
  { -105.888330118127f,    0.0f},
  { 14.1136693674576f,    0.0f},
  { 134.112231106272f,    0.0f},
  { 0.0f,    90.0f}};

/* Sphere Covering (6 points) */
const float __SphCovering_6_dirs_deg[6][2] =
{ { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { -77.7274544874475f,    0.0f},
  { 12.2716100561120f,    0.0f},
  { 102.272966430852f,    0.0f},
  { -167.727664946597f,    0.0f}};

/* Sphere Covering (7 points) */
const float __SphCovering_7_dirs_deg[7][2] =
{ { 148.510660497909f,    30.6406369680062f},
  { -65.2942703331086f,    -58.3442922781717f},
  { -14.5084372883027f,    1.52429691816604f},
  { -105.229428653727f,    25.3018162329772f},
  { 40.3665955403519f,    60.1892163784930f},
  { 74.7709922645724f,    -25.3018162329772f},
  { -176.958015026155f,    -33.5518355250659f}};

/* Sphere Covering (8 points) */
const float __SphCovering_8_dirs_deg[8][2] =
{ { -94.9906728547392f,    43.0342870344810f},
  { 109.916223417897f,    -54.2636868612500f},
  { 0.930999121308075f,    49.8685276148015f},
  { 128.147740458960f,    36.0636824989194f},
  { -105.372668102510f,    -38.6694945511744f},
  { -30.9385750214742f,    -27.3489944349796f},
  { -176.075660021653f,    1.82630297197950f},
  { 55.3849652663210f,    -9.88123013482618f}};

/* Sphere Covering (9 points) */
const float __SphCovering_9_dirs_deg[9][2] =
{ { -116.522426795756f,    85.0212072194629f},
  { 16.9658532716188f,    24.5094156023112f},
  { -91.1518556273627f,    -50.4202859715125f},
  { -132.834535223130f,    8.34455732828531f},
  { 149.163832384359f,    -47.7944203964279f},
  { 86.7057031371475f,    6.33576729855664f},
  { 156.577906253351f,    20.8482152914253f},
  { 22.0130384889262f,    -44.3005237617201f},
  { -54.6979888699592f,    10.8036921849868f}};

/* Sphere Covering (10 points) */
const float __SphCovering_10_dirs_deg[10][2] =
{ { 132.152715446924f,    -39.0184258484091f},
  { 62.9623321069262f,    9.21831796585981f},
  { 130.130174430113f,    26.4844011221272f},
  { -146.740220910955f,    51.0642905332395f},
  { 33.2573352183686f,    -51.0642905332395f},
  { 8.69635341449563f,    61.7190136914923f},
  { -158.434289509575f,    -13.6656163716653f},
  { -16.8890769270713f,    -0.849180748163393f},
  { -91.2893654981941f,    -53.5709808869368f},
  { -81.6980520077041f,    11.4454049155333f}};

/* Sphere Covering (11 points) */
const float __SphCovering_11_dirs_deg[11][2] =
{ { 111.113705209721f,    -75.3725979494598f},
  { 43.5510949656890f,    21.3770553363310f},
  { -124.165683782801f,    -29.8889163407945f},
  { 91.2721767643401f,    67.9871719702235f},
  { 16.3757067426341f,    -30.0705439618510f},
  { -58.3672105899770f,    -25.2989514440015f},
  { -112.895603952577f,    36.2183811036047f},
  { 102.267236852901f,    -12.7070579804114f},
  { 178.768561658768f,    -13.2055312621752f},
  { -34.5997753323602f,    32.6041633319195f},
  { 158.050407786838f,    33.4286495991128f}};

/* Sphere Covering (12 points) */
const float __SphCovering_12_dirs_deg[12][2] =
{ { 0.0f,    -31.7172246650570f},
  { -58.2812669207074f,    0.0f},
  { -90.0002104591497f,    58.2812669207074f},
  { 0.0f,    31.7172246650570f},
  { -121.719153997592f,    0.0f},
  { 90.0002104591497f,    -58.2812669207074f},
  { 180.0f,    -31.7172246650570f},
  { 121.719153997592f,    0.0f},
  { 90.0002104591497f,    58.2812669207074f},
  { 180.0f,    31.7172246650570f},
  { 58.2812669207074f,    0.0f},
  { -90.0002104591497f,    -58.2812669207074f}};

/* Sphere Covering (13 points) */
const float __SphCovering_13_dirs_deg[13][2] =
{ { 5.50772869303358f,    23.7794673713146f},
  { 142.546169850598f,    -67.9012283009539f},
  { -42.2682424623911f,    68.0788452174444f},
  { -93.0999121308075f,    -41.3142677334983f},
  { -45.8286022013340f,    4.83301359348752f},
  { 44.9027024044026f,    -2.11455803870982f},
  { -2.49288207083470f,    -45.6252021840626f},
  { -108.564043021388f,    19.9635684557433f},
  { -161.018329165615f,    -16.1694419363870f},
  { -179.788426534101f,    44.3910510933508f},
  { 135.991532674301f,    -4.76592023567770f},
  { 89.3470385727006f,    42.6813450326804f},
  { 85.5884354366424f,    -27.0178248293940f}};

/* Sphere Covering (14 points) */
const float __SphCovering_14_dirs_deg[14][2] =
{ { -45.9265779843014f,    -54.4951618104829f},
  { 126.881503731721f,    -63.0253574643906f},
  { -177.490865775626f,    -14.0007966818168f},
  { 79.0681757280536f,    -10.6495665380966f},
  { 175.880854371309f,    48.0665753491150f},
  { 131.052636480273f,    -0.752580063904336f},
  { 27.8898665935831f,    -35.8218943093742f},
  { 89.9371851016853f,    50.9783468639699f},
  { -22.7137658723712f,    4.77726480002129f},
  { -53.1200631021689f,    63.0253574643906f},
  { -125.798613498924f,    22.3717100686781f},
  { -75.3840571053624f,    2.88272255464171f},
  { -124.950635962130f,    -39.9712546617116f},
  { 27.5065578286406f,    26.5250811255815f}};

/* Sphere Covering (15 points) */
const float __SphCovering_15_dirs_deg[15][2] =
{ { 38.2804562082806f,    -30.8503395210240f},
  { -10.9944871307654f,    52.3288083870932f},
  { 107.773361264108f,    -72.5078089738057f},
  { 59.1464331913549f,    28.4645432620993f},
  { -104.931490600259f,    -11.1594989757630f},
  { -142.534710694695f,    -43.2348222627768f},
  { 156.658120344670f,    -21.2555882837633f},
  { 95.6495743191396f,    -14.9478959171680f},
  { 1.54996542738790f,    -1.22343677994285f},
  { -102.771439712616f,    43.3889479096670f},
  { -52.6920636292062f,    5.49340474815531f},
  { 131.132850571592f,    27.4269166951174f},
  { 147.645494227262f,    72.1869526085324f},
  { -161.528261603282f,    16.8449591768462f},
  { -44.6563305524964f,    -50.0312476286186f}};

/* Sphere Covering (16 points) */
const float __SphCovering_16_dirs_deg[16][2] =
{ { 64.8015266292961f,    -34.0268175372293f},
  { -127.517486884316f,    -50.0524470670385f},
  { 5.17552776341673f,    51.2487829432716f},
  { 170.202842621562f,    27.6291707967986f},
  { -73.0864963468878f,    32.0959497676385f},
  { -136.776484853630f,    66.7725014445461f},
  { 120.011739768102f,    -5.26416433432346f},
  { 178.069553148709f,    -22.2737342857108f},
  { -18.2194849273651f,    -56.5056707135969f},
  { 19.3224286829919f,    -5.99199262147815f},
  { -78.1800911456008f,    -18.1461463295883f},
  { -131.035447746419f,    8.38466437394447f},
  { 65.0421689032511f,    16.4519101293865f},
  { -30.7328831730222f,    1.12740905347892f},
  { 110.076651600534f,    52.6427892588249f},
  { 133.321549348991f,    -62.9623321069262f}};

/* Sphere Covering (17 points) */
const float __SphCovering_17_dirs_deg[17][2] =
{ { -128.010230588129f,    59.8855487470736f},
  { 80.3516011891467f,    -14.4746327783900f},
  { 153.363613022667f,    38.9754540137743f},
  { -161.763174299285f,    6.07449854397699f},
  { 141.457550039849f,    -11.9203869276968f},
  { -117.020900077519f,    -25.7091892253152f},
  { 29.5955619496875f,    17.4442730305530f},
  { -34.6633736476197f,    37.7447406698332f},
  { -62.7446081447765f,    -9.22462050160626f},
  { 29.6035833588194f,    -36.3616205523874f},
  { 108.844792341002f,    -60.0402473517590f},
  { -54.3095234848605f,    -64.1025181192365f},
  { -14.6734491333004f,    -10.8667175424512f},
  { -102.433394613489f,    22.3470728834875f},
  { -171.343028633873f,    -45.3072106077650f},
  { 93.5640079448634f,    25.1133131183791f},
  { 46.4164568991383f,    68.4398086283768f}};

/* Sphere Covering (18 points) */
const float __SphCovering_18_dirs_deg[18][2] =
{ { 123.294787934202f,    12.2653075203655f},
  { 123.638562611280f,    -42.2808475338840f},
  { 41.9342080678298f,    -42.1473483676185f},
  { -80.5865138851503f,    22.0485618722243f},
  { 76.5357022735754f,    -8.28095901302579f},
  { 127.815424937784f,    64.7614195836370f},
  { 19.5132236287704f,    6.30711940880010f},
  { -35.4890058304032f,    -84.8149424132158f},
  { -20.6104378064460f,    -38.0696077396724f},
  { -118.785610086522f,    58.5104500387597f},
  { -176.975203760009f,    31.7538939639454f},
  { -131.356304111693f,    1.40747082373887f},
  { -34.2703246001599f,    5.64907738109235f},
  { -13.8844862494052f,    53.6775510368312f},
  { 63.5753969477161f,    37.7773992641557f},
  { 171.148222983528f,    -12.3260410466494f},
  { -83.7893479599316f,    -28.6713810261415f},
  { -149.668035244074f,    -46.7957549595149f}};

/* Sphere Covering (19 points) */
const float __SphCovering_19_dirs_deg[19][2] =
{ { 179.049310978382f,    67.6090198254371f},
  { -76.3981924027440f,    -13.5000315688725f},
  { 37.0921417411792f,    -5.35864507474054f},
  { -113.766499801176f,    32.0397999037156f},
  { 90.4299288054978f,    -31.9716179260951f},
  { 143.514468524369f,    -9.92305605387073f},
  { -170.483591941176f,    17.3079090753119f},
  { 88.4589539902478f,    8.78859961951170f},
  { -36.4464183060668f,    63.9191716247946f},
  { 131.442247780962f,    36.6280459271233f},
  { -165.733771819542f,    -33.2378546533342f},
  { -125.775695187118f,    -14.4740598205949f},
  { 26.0208782658663f,    -52.9424461856783f},
  { 138.821944182247f,    -65.8901464400447f},
  { -21.0796902406581f,    -25.1178967807402f},
  { -0.924123627766505f,    19.1803351497994f},
  { -54.3416091213878f,    21.0779713672727f},
  { -84.5341930936017f,    -59.6563656290213f},
  { 57.0717530151862f,    46.9315459569609f}};

/* Sphere Covering (20 points) */
const float __SphCovering_20_dirs_deg[20][2] =
{ { -82.8611563318196f,    -56.0461585619020f},
  { 134.811239616331f,    23.6568544031566f},
  { -100.943704346148f,    -3.51280424194708f},
  { -143.606141771590f,    -43.0818425314769f},
  { 124.108388003288f,    73.1266033925470f},
  { 36.5730419787907f,    -6.54375097818913f},
  { -61.6846362237844f,    23.3130797260781f},
  { -116.436483126486f,    49.4559980023073f},
  { 179.387356077509f,    30.8812792419611f},
  { 7.71888741600245f,    -50.1349529895373f},
  { -48.4796779194044f,    -24.7540685808321f},
  { 83.6174606213923f,    23.1131174555774f},
  { 125.592348692676f,    -71.6770201708660f},
  { -146.642818085783f,    6.58443098164342f},
  { 115.055654840221f,    -22.2278976621003f},
  { -28.3384925471705f,    54.7570035228576f},
  { -9.34837938535451f,    2.50164832510020f},
  { 32.0180275075007f,    43.0268385831443f},
  { 71.0696849080273f,    -30.0315828317821f},
  { 165.750960553396f,    -22.0193410246727f}};

/* Sphere Covering (21 points) */
const float __SphCovering_21_dirs_deg[21][2] =
{ { 159.104650129878f,    -5.29229656206439f},
  { -154.377748320049f,    -15.7276914763411f},
  { -114.969711170951f,    60.4756952760584f},
  { 19.0531385192804f,    -14.8636711212838f},
  { 109.772983969114f,    -22.8787777173689f},
  { -175.399569823399f,    34.2617302332330f},
  { -124.240168296168f,    17.3038983707460f},
  { 83.7435113363211f,    23.5915372145116f},
  { -19.0794945778564f,    52.6107036222976f},
  { 32.0237570854520f,    33.8532113253047f},
  { 60.6361234586950f,    -62.9164954833157f},
  { -17.9278494096435f,    -50.9920978510530f},
  { 107.125918955610f,    69.2877861651705f},
  { -68.3022987575454f,    21.5139922493673f},
  { 128.308168641597f,    26.6368078956320f},
  { 59.8912783250249f,    -14.5067184149173f},
  { -118.413187519687f,    -62.8649292817539f},
  { 160.124515005211f,    -53.9371009180254f},
  { -104.358532805128f,    -17.8911801107551f},
  { -17.6740291064005f,    5.12218539269005f},
  { -56.7377186206249f,    -24.9190804258298f}};

/* Sphere Covering (22 points) */
const float __SphCovering_22_dirs_deg[22][2] =
{ { 71.9978765361392f,    43.7808510415365f},
  { -71.9978765361392f,    -9.65892251031542f},
  { 59.2151881267706f,    90.0f},
  { -120.785232791529f,    -90.0f},
  { 108.002544382160f,    -43.7814239993316f},
  { -107.996814804209f,    9.65949546811055f},
  { 36.0000841836599f,    -43.7819969571267f},
  { -180.0f,    9.65892251031542f},
  { 144.001482650230f,    43.7814239993316f},
  { 0.000325308247341427f,    -9.65892251031542f},
  { 0.000637759321760119f,    43.7814239993316f},
  { -144.001482650230f,    -9.65892251031542f},
  { -36.0000841836599f,    -43.7814239993316f},
  { 108.002544382160f,    9.65892251031542f},
  { -180.0f,    -43.7814239993316f},
  { -35.9995112258648f,    9.65892251031542f},
  { -144.001482650230f,    43.7819969571267f},
  { 71.9978765361392f,    -9.65949546811055f},
  { -71.9978765361392f,    43.7814239993316f},
  { 144.001482650230f,    -9.65892251031542f},
  { -108.002544382160f,    -43.7808510415365f},
  { 36.0006571414550f,    9.65892251031542f}};

/* Sphere Covering (23 points) */
const float __SphCovering_23_dirs_deg[23][2] =
{ { 68.5028339858412f,    -56.9485670892330f},
  { -53.8041747095551f,    -19.1768974030287f},
  { 126.193954377564f,    19.1768974030287f},
  { -18.5856049584536f,    7.72347107836350f},
  { -116.820364849224f,    37.7751074329752f},
  { 111.950223590612f,    61.9424672315933f},
  { -53.5176958119897f,    31.0778037656910f},
  { 71.6999384826712f,    31.2932358966602f},
  { -94.5838728201963f,    75.3955162612650f},
  { 93.2947177811520f,    -10.4243941246102f},
  { 50.8941220680856f,    -16.3430481483116f},
  { -179.862911047468f,    38.6328252522860f},
  { 166.742177538972f,    0.484664998901163f},
  { 179.817074423858f,    -71.8431779314539f},
  { -108.168702142748f,    -34.7246801316987f},
  { -46.9510265219953f,    -62.8248222360948f},
  { -167.630262121425f,    -33.4200552321858f},
  { -142.964429041043f,    2.40986048632024f},
  { 23.4603298794267f,    11.6602640887074f},
  { -92.6014388490437f,    2.59332157232113f},
  { 130.479678685142f,    -30.9093541739225f},
  { 8.96564357820712f,    52.7625374380073f},
  { 0.526966472915672f,    -35.2139860887404f}};

/* Sphere Covering (24 points) */
const float __SphCovering_24_dirs_deg[24][2] =
{ { 23.1314521050216f,    -10.3550662313994f},
  { -26.5777932427335f,    66.1651661817075f},
  { -26.5686259180114f,    -5.69256487774278f},
  { 137.005667971682f,    32.8448056058744f},
  { -0.620513292126682f,    28.7229472277033f},
  { 97.0876983849180f,    17.0197113043611f},
  { -11.8550697390519f,    -45.3564849781462f},
  { 91.6102218634673f,    65.7182591015054f},
  { -97.4429322178991f,    -1.18373080474028f},
  { 179.014933510674f,    15.5638255469337f},
  { 138.409414569753f,    -13.1705808366722f},
  { 103.269912994380f,    -30.4120268077490f},
  { 50.2501175063586f,    -56.5543721261830f},
  { -128.445678512428f,    -36.5094436635312f},
  { -175.262059952568f,    -26.5955549343826f},
  { -85.5024917673728f,    -70.8061243222671f},
  { -102.800087602372f,    43.6559462421979f},
  { 60.3668332949835f,    -9.40739403825299f},
  { 154.240238449218f,    -62.2060278173535f},
  { 47.4179871250269f,    34.3917916527277f},
  { -69.7346932453725f,    -26.3921549171111f},
  { -54.6848108406712f,    22.2319083666662f},
  { -141.262744389504f,    9.40166446030168f},
  { -158.422830353673f,    58.4416951033440f}};

/* Sphere Covering (25 points) */
const float __SphCovering_25_dirs_deg[25][2] =
{ { -101.585417076695f,    -9.81533998838613f},
  { 177.284600969379f,    46.2101920928912f},
  { -117.639694496261f,    36.2911467435863f},
  { -145.749003925379f,    9.78726505642472f},
  { -177.788803829094f,    -26.9886039818423f},
  { -74.1063612222207f,    -54.4092181412132f},
  { 136.060287609717f,    -23.4442870611630f},
  { -118.636641059788f,    81.6865928518015f},
  { -21.8067736826791f,    13.6822321477241f},
  { 14.9324260566995f,    -65.3859435803296f},
  { -70.8175834781698f,    17.2488944224134f},
  { -48.6550030047144f,    50.3102780748473f},
  { 166.335377504429f,    8.57259453074738f},
  { 46.0377317965568f,    -22.8455461652513f},
  { -8.56228129043502f,    -30.4893761100916f},
  { -54.5352688561420f,    -20.2964569347143f},
  { -133.166850744306f,    -35.9811765764206f},
  { 66.7381239768383f,    21.7345810004926f},
  { 100.852031098928f,    55.9143782690219f},
  { 94.0395629148220f,    -49.1053478316872f},
  { 122.836421698097f,    22.4335895105523f},
  { 92.0227514759615f,    -6.76605860269989f},
  { 173.451513319954f,    -68.3309466473020f},
  { 20.9725471329687f,    6.61250591360483f},
  { 19.8518416856928f,    49.2405658713381f}};

/* Sphere Covering (26 points) */
const float __SphCovering_26_dirs_deg[26][2] =
{ { 100.072808497550f,    3.17304026943450f},
  { -165.418645032220f,    -74.0777133324641f},
  { 128.543081337600f,    38.3560866372378f},
  { 171.543563862169f,    -38.9215959810320f},
  { 70.0555496106458f,    37.4731586749412f},
  { 26.1257295423753f,    -15.6623742876962f},
  { 67.8324733655382f,    -11.1142353099477f},
  { -56.7130814354343f,    31.1946871558977f},
  { -1.47737167474483f,    58.2698077648047f},
  { -94.6927348012712f,    3.87531463892635f},
  { 106.392532977843f,    -44.7720680271128f},
  { -9.65319293236411f,    -27.9368491327838f},
  { 165.172273180314f,    10.8019733116014f},
  { -51.7077221371714f,    -11.2007519370125f},
  { -166.959901501122f,    47.0123330060743f},
  { -37.6565051693831f,    -59.5990698495082f},
  { 49.4244853235751f,    -56.2592988616907f},
  { -130.903667453539f,    -30.3071755312400f},
  { -96.1079405552443f,    53.3452355156553f},
  { 134.977397376919f,    -10.8025462693965f},
  { -89.1694216562100f,    -39.7913459140406f},
  { -128.388382732915f,    16.2032464462997f},
  { -12.6199683955515f,    15.1495770610541f},
  { 106.759225966726f,    80.2140913183152f},
  { -161.774633455188f,    -1.90806404934467f},
  { 33.6990856784145f,    21.2154812381041f}};

/* Sphere Covering (27 points) */
const float __SphCovering_27_dirs_deg[27][2] =
{ { 101.642712856208f,    -10.3711090496630f},
  { 13.2857453534935f,    -25.0302342380851f},
  { -112.838308173064f,    -42.1169816044766f},
  { 148.527849231763f,    -18.0372843485134f},
  { 37.3230437326170f,    9.55292531821621f},
  { 100.502526843898f,    55.1414582033904f},
  { -140.821566887254f,    32.1807475213178f},
  { 130.846371674026f,    20.0603983231204f},
  { 79.1827672870798f,    20.6264806247096f},
  { -38.6752241291257f,    -55.1787004600739f},
  { -55.3379827271203f,    15.4228779293315f},
  { -27.7678265832202f,    50.9611581301159f},
  { -25.7584635956964f,    -14.5347933468787f},
  { 178.419057403738f,    6.01147318651260f},
  { 115.479643608617f,    -50.6517609207453f},
  { -73.7110203435804f,    -22.3694182374976f},
  { -94.1484248958969f,    40.3236237057171f},
  { -128.760805299750f,    80.1625251167535f},
  { 172.448837178475f,    43.7052206125792f},
  { -151.919759378938f,    -76.3810036688901f},
  { -142.677950143478f,    -9.55292531821621f},
  { 41.3469263278207f,    -57.3817231823520f},
  { 64.0165744499669f,    -23.0397788578007f},
  { -4.94898025122200f,    17.5136009237639f},
  { -175.181845861249f,    -39.7111318227222f},
  { 34.8157804211245f,    51.2728471706671f},
  { -102.708414355151f,    3.06016758379373f}};

/* Sphere Covering (28 points) */
const float __SphCovering_28_dirs_deg[28][2] =
{ { 130.771887160659f,    1.46413634967731f},
  { -43.4376493222531f,    -25.4828708962385f},
  { -65.9302534857038f,    62.4867771369676f},
  { -76.2549529539613f,    -7.86441869596568f},
  { 167.366701535665f,    -14.8476283030202f},
  { -138.294823010727f,    39.6463875918725f},
  { 86.5395453765595f,    8.84073877886860f},
  { -15.2773466493683f,    -1.99612766245628f},
  { -2.53917706068127f,    41.0959708135534f},
  { -98.3023689105954f,    -42.5123224831168f},
  { -153.180266528226f,    -35.7342317667192f},
  { 54.2218609422055f,    65.5922083865766f},
  { 11.2660691256574f,    -34.6226936441654f},
  { -35.7141782438896f,    -60.9627094019196f},
  { 143.915538980960f,    -44.7250854879121f},
  { 102.616741107930f,    -29.3772650297427f},
  { 55.1764086288934f,    -17.9186820849214f},
  { -158.428559931624f,    8.61556636538219f},
  { 164.049275901857f,    29.6620250539227f},
  { 24.8119373181403f,    8.12167674597942f},
  { -178.854505328038f,    68.0673860615418f},
  { -121.123277890656f,    -5.29424461856783f},
  { 113.222189895802f,    40.7367262760064f},
  { -47.6930068666897f,    21.9866824303502f},
  { -96.8241377991578f,    28.2748942319110f},
  { 64.5150477317307f,    -59.7365797203396f},
  { -170.386189116004f,    -74.5704570362767f},
  { 55.3660576590817f,    29.7296340737482f}};

/* Sphere Covering (29 points) */
const float __SphCovering_29_dirs_deg[29][2] =
{ { -174.671913423583f,    -23.7983749785539f},
  { -102.507879126856f,    17.4540133130703f},
  { -18.5202877698087f,    1.43600412193638f},
  { 7.18832849771131f,    35.8356452964573f},
  { -62.9852504187314f,    16.5269676005486f},
  { 48.9151258437038f,    -69.6773974658594f},
  { -60.0631656635642f,    -58.8484951378869f},
  { 97.9585942335169f,    -32.0965227254336f},
  { 106.679011875408f,    4.72208896435019f},
  { 68.8122311952119f,    4.51175615775767f},
  { -50.7210888139561f,    -20.8505071226058f},
  { -36.0293050312116f,    41.4145353476462f},
  { 48.2373167720640f,    32.5514512147675f},
  { -137.234851089735f,    -4.65946467734239f},
  { 51.5031762043097f,    -31.1121812333988f},
  { 94.9448362311287f,    44.1893699494647f},
  { 35.1102807278217f,    77.9509080275485f},
  { -179.238387050775f,    14.0099640065389f},
  { -92.5556022254332f,    -18.9987075287430f},
  { 170.237220089270f,    61.3408615467059f},
  { 143.646248817249f,    -13.0588540666217f},
  { 144.242124924185f,    28.7538869486404f},
  { 136.908265146510f,    -50.5211265434555f},
  { -146.344880032315f,    36.2791146298886f},
  { -161.957979949630f,    -66.8355268020105f},
  { -2.71427296287325f,    -37.7802640531314f},
  { -123.409379493228f,    -40.4221724464796f},
  { 23.1486408388755f,    -3.25995796695584f},
  { -90.9570499770182f,    57.3244274028389f}};
  
/* Sphere Covering (30 points) */
const float __SphCovering_30_dirs_deg[30][2] =
{ { 134.003369125197f,    37.1362594914043f},
  { -46.2205053332035f,    -58.1380274719246f},
  { -130.880749141734f,    -56.1893980106847f},
  { 75.3554092156059f,    -9.36213037243765f},
  { 14.2036237412931f,    41.4368807016563f},
  { -111.176730567185f,    51.0471017993856f},
  { -52.1150951295094f,    -14.3548845992076f},
  { 157.105027424872f,    4.74489268459640f},
  { 38.4689593228786f,    -25.6713740108365f},
  { 90.9742387108721f,    26.8104141075566f},
  { 143.015995242605f,    -36.1811388469212f},
  { 70.2102482153311f,    61.2950249230955f},
  { 30.0923163580660f,    -56.2558611149199f},
  { 49.3883889824818f,    18.4068421263728f},
  { 118.252759337051f,    -4.35356251052205f},
  { -26.0071272787832f,    16.0336509389410f},
  { -165.504588701490f,    0.431339816908338f},
  { -92.7045712521672f,    7.23473807911691f},
  { -66.7152056650331f,    26.2311537766794f},
  { 141.755488093317f,    -77.9165305598407f},
  { -89.3527681506519f,    -29.8350583080522f},
  { 91.8852416051301f,    -41.3177054802691f},
  { -12.6320005092493f,    -26.0082731943735f},
  { -130.451030795386f,    19.3757137579391f},
  { 162.553856056566f,    78.4837587770202f},
  { -178.487812339154f,    -34.7390040765770f},
  { -130.611458978022f,    -18.4068421263728f},
  { 13.0319250502506f,    2.23144142891650f},
  { -174.282302122894f,    40.4708738590657f},
  { -35.2380503161359f,    59.3641571535046f}};

/* Sphere Covering (31 points) */
const float __SphCovering_31_dirs_deg[31][2] =
{ { 106.959761195022f,    15.4400666631854f},
  { -12.7706562956709f,    41.9685855355377f},
  { -93.8504868424288f,    -28.3568271966147f},
  { 39.0917644461858f,    -24.7053671682460f},
  { -176.098578333459f,    53.8706378137903f},
  { -173.365569650685f,    9.57527067222632f},
  { -133.241335257673f,    -12.5580889536774f},
  { 144.832271453170f,    -8.08042378473000f},
  { 70.6285074057766f,    26.5657611290358f},
  { 106.100324502326f,    -21.3730446317651f},
  { -56.7657935525863f,    80.7412124898356f},
  { -43.2245090224644f,    -27.4114468346488f},
  { 38.3984855140775f,    53.6242659618840f},
  { 119.839852429563f,    -81.2797928172586f},
  { -174.815152872365f,    -22.8655996880809f},
  { -59.4329120889203f,    -0.266746231101106f},
  { 67.8496620993921f,    -3.25153548736742f},
  { -18.9694866811913f,    11.4494156200992f},
  { 146.178722271727f,    -48.3788373474613f},
  { 149.244046475677f,    26.9444862316172f},
  { -68.6804509023318f,    -60.5043431658149f},
  { 6.14382643718782f,    -54.5902728044746f},
  { -2.30649890007864f,    -18.1610432322617f},
  { -107.630121815325f,    47.0518670939383f},
  { -97.1679124762363f,    10.5217969497824f},
  { 114.333728018356f,    56.9537237093892f},
  { -140.678327438471f,    21.5472238014849f},
  { -57.0287811805514f,    36.5856470502836f},
  { 26.0495261556229f,    13.5756619978297f},
  { 78.2259277692113f,    -46.9028980672043f},
  { -143.571764303882f,    -49.9074887448704f}};

/* Sphere Covering (32 points) */
const float __SphCovering_32_dirs_deg[32][2] =
{ { 148.281477379857f,    -0.000106083135768472f},
  { 180.0f,    -20.9055110709384f},
  { 45.0001052295749f,    35.2644063747119f},
  { -45.0001052295749f,    -35.2644063747119f},
  { -135.000315688725f,    35.2644063747119f},
  { 135.000315688725f,    -35.2644063747119f},
  { -135.000315688725f,    -35.2644063747119f},
  { 135.000315688725f,    35.2644063747119f},
  { 44.9995322717797f,    -35.2638334169168f},
  { -45.0001052295749f,    35.2644063747119f},
  { -148.281477379857f,    -0.000273312327433305f},
  { 180.0f,    20.9049381131432f},
  { 6.47556900056856e-05f,    -58.2812669207074f},
  { -90.0002104591497f,    -69.0929805148260f},
  { -0.000445480415292166f,    58.2812669207074f},
  { 90.0002104591497f,    69.0929805148260f},
  { 180.0f,    -58.2812669207074f},
  { 90.0002104591497f,    -69.0929805148260f},
  { -90.0002104591497f,    31.7172246650570f},
  { -110.907440403473f,    -0.000228919557466569f},
  { 90.0002104591497f,    31.7177976228521f},
  { 69.0929805148260f,    0.000226461568525458f},
  { -90.0002104591497f,    -31.7177976228521f},
  { -69.0929805148260f,    -6.85142931417439e-05f},
  { 31.7172246650570f,    0.000273472755615942f},
  { -0.000250875300175982f,    20.9055110709384f},
  { -180.0f,    58.2812669207074f},
  { -90.0002104591497f,    69.0929805148260f},
  { 90.0002104591497f,    -31.7172246650570f},
  { 110.907440403473f,    6.78210142096355e-05f},
  { -31.7177976228521f,    0.000107681688016887f},
  { -0.000131035447746419f,    -20.9049381131432f}};

/* Sphere Covering (33 points) */
const float __SphCovering_33_dirs_deg[33][2] =
{ { -82.0762041524904f,    -28.1517083059579f},
  { 143.520198102320f,    -38.9645678156668f},
  { -179.748319488442f,    55.9923005291597f},
  { 96.2053433804165f,    -35.9531016444592f},
  { -132.118337979217f,    34.0308282417953f},
  { 140.827296465205f,    31.1901034935366f},
  { 155.449179396944f,    -3.15648178915522f},
  { 38.7789294900444f,    -1.34862805817893f},
  { 54.6555899931195f,    32.1744449855714f},
  { -125.580889536774f,    -34.4250232048453f},
  { 76.1289022390325f,    -4.32056014152251f},
  { 5.62524233681491f,    -19.6197937786648f},
  { -35.3457663816205f,    80.9474772960827f},
  { -95.7355179884093f,    60.0001403060998f},
  { -36.1261348985887f,    38.1945125390109f},
  { 48.9363252821236f,    -37.4622724768337f},
  { -78.9650433249301f,    -65.4088618921348f},
  { 0.839440465646169f,    -56.5939062140471f},
  { -166.604667668141f,    -59.6219881613135f},
  { 8.76453539211620f,    21.7998981891376f},
  { -171.646696265292f,    -22.3911906337126f},
  { -103.831411633608f,    2.06196051311681f},
  { -61.9481968095446f,    3.77246871470037f},
  { 118.138167778024f,    -4.47015942183117f},
  { 100.107185965257f,    61.8909010300315f},
  { 93.0769938190022f,    -72.9547160540077f},
  { -141.640896534291f,    -1.53707387699746f},
  { -174.505755662995f,    19.2278906467953f},
  { 19.2657058612739f,    54.1955048836294f},
  { -84.1675001047179f,    29.6608791383325f},
  { 97.5174167312661f,    25.2582714405472f},
  { -24.4733192612180f,    2.86994559581029f},
  { -39.0585328940682f,    -31.5906009923331f}};

/* Sphere Covering (34 points) */
const float __SphCovering_34_dirs_deg[34][2] =
{ { -145.754733503330f,    -23.4958532627248f},
  { -35.8786171310922f,    -38.2180038086113f},
  { 137.280687713345f,    43.6358927193684f},
  { -77.8936122480354f,    -46.4628664805438f},
  { -51.8005412999826f,    65.6036675424793f},
  { 63.0138983084879f,    -46.3385346390005f},
  { 144.637465802825f,    -22.2817556948426f},
  { -3.39133718937934f,    -75.6819951588305f},
  { -177.725778471630f,    -0.704967271128965f},
  { 76.2549529539613f,    -8.28668859097710f},
  { -24.6801570252602f,    29.1807405060128f},
  { -72.0609018936036f,    -9.43031235005822f},
  { -116.986522609811f,    46.3385346390005f},
  { 12.0074765125567f,    -38.6081880670954f},
  { -112.299727845641f,    -21.9746503166525f},
  { 128.021689744031f,    -64.4806702640229f},
  { 73.3099498869888f,    64.9275773442249f},
  { 151.117618465755f,    14.2013319101126f},
  { 174.626076799972f,    -39.7117047805174f},
  { 39.3937132042198f,    -12.9431165920053f},
  { 108.346319059239f,    -31.3522505495587f},
  { -4.84579055231894f,    -8.76682722329673f},
  { 114.236325193184f,    5.30381301374652f},
  { -132.674107040493f,    -62.1086249921812f},
  { 17.6499648790050f,    51.6647503025366f},
  { -169.727287651604f,    37.0239597635587f},
  { 55.0474931249890f,    24.1157935970564f},
  { -176.333491029462f,    74.6220232378384f},
  { -69.3164340549270f,    30.7242888060953f},
  { -41.2621285741414f,    -1.18029305796950f},
  { 15.3455286269888f,    14.6677195553491f},
  { -103.573580625799f,    10.3751197542289f},
  { 93.1171008646614f,    30.0132481823379f},
  { -139.566789315917f,    12.4572483817344f}};
  
/* Sphere Covering (35 points) */
const float __SphCovering_35_dirs_deg[35][2] =
{ { -167.991225532357f,    64.3030533475323f},
  { -134.713836791159f,    29.1005264146945f},
  { -1.52802114383439f,    -26.6591532496421f},
  { -25.6180889358894f,    1.56004948458221f},
  { -177.783074251143f,    -33.8434710427875f},
  { -10.8798955717392f,    36.0333157357775f},
  { -98.0273491689326f,    51.6578748089950f},
  { 13.4266929710957f,    8.24486267193255f},
  { -157.271185185460f,    -3.87073097656530f},
  { 136.747836963874f,    -0.991503464473890f},
  { 96.0334560418773f,    66.9501183610367f},
  { 139.159989281374f,    38.1738860583862f},
  { 105.223699075776f,    -25.1992567876487f},
  { 84.8321311470697f,    36.3501613964848f},
  { -42.2447511927907f,    -36.3799552018316f},
  { 147.966350592535f,    -36.4607422509451f},
  { -60.5730981012306f,    -3.48467201420615f},
  { 72.0265244258958f,    -4.37447547004432f},
  { 104.536149721619f,    13.6673352450507f},
  { 106.759225966726f,    -64.0566814956260f},
  { -176.751750219908f,    28.9779134465365f},
  { 49.3419794010762f,    12.1793638510959f},
  { -133.567921200898f,    -39.6641492835215f},
  { 33.1776940848455f,    -24.5747327909561f},
  { 35.0632981886210f,    45.5770737292716f},
  { -82.4715450311307f,    -62.1716503496456f},
  { 62.0169517449603f,    -40.2147617246422f},
  { -169.773124275214f,    -73.6365358302134f},
  { -89.0204526294760f,    -25.8879520573960f},
  { -53.7331279429589f,    32.9129875834950f},
  { -29.5222233519108f,    71.8374483535026f},
  { 7.38198823246553f,    -63.7071772405962f},
  { 165.241028115729f,    2.81459787280066f},
  { -92.9509431040735f,    14.9117995760748f},
  { -121.656128640128f,    -5.78687373082131f}};

/* Sphere Covering (36 points) */
const float __SphCovering_36_dirs_deg[36][2] =
{ { 42.0688531496856f,    39.0092585236870f},
  { -8.21277703540522f,    -31.9395322895677f},
  { -84.5399226715530f,    -29.0993804991043f},
  { -115.823418285696f,    -44.9628629728914f},
  { 99.9524873605721f,    -60.2235938462008f},
  { 132.582433793273f,    34.9103184573211f},
  { -43.9332578150413f,    -16.3040870182427f},
  { 22.2548266784714f,    71.6426427031581f},
  { -109.709958611650f,    -82.0819337304417f},
  { 80.6953758662252f,    41.7560181935441f},
  { 99.6660084630067f,    12.8159199614863f},
  { -149.186750696164f,    -25.2044134078049f},
  { -45.6223373950869f,    20.2448907331525f},
  { -90.9226725093103f,    27.9924260389115f},
  { 165.808256332909f,    14.3531657258223f},
  { -179.450381434974f,    -54.8360716985857f},
  { -52.0612370967671f,    -53.8202175278188f},
  { -117.954821283583f,    65.8672281282394f},
  { -117.347486020744f,    -4.31878397235761f},
  { -8.47404578998488f,    37.0067710297047f},
  { 22.6203737517649f,    -58.0062471790445f},
  { -11.2712257458136f,    0.512006544884806f},
  { 172.351434353303f,    -17.0586724344300f},
  { 132.771509865666f,    -3.38979020333249f},
  { -78.3405193282375f,    1.84572624123443f},
  { 59.7308501423883f,    -32.1423593490441f},
  { 130.726050537049f,    68.0501973276879f},
  { 24.0733947202167f,    11.8510590344859f},
  { 27.3885285228436f,    -16.7406808581324f},
  { -55.4634604842539f,    51.9071114498769f},
  { 135.790997446005f,    -35.3371720146935f},
  { -128.852478546971f,    30.4870842789111f},
  { -155.672632937045f,    8.25231112326925f},
  { 63.2201631147350f,    4.21083872375496f},
  { -174.883907807781f,    43.9298200682705f},
  { 95.2485038625481f,    -20.9536395257293f}};

/* Sphere Covering (37 points) */
const float __SphCovering_37_dirs_deg[37][2] =
{ { -41.8230542555744f,    31.6845660707345f},
  { 47.3142817641083f,    -30.2086267904775f},
  { -18.3249091616691f,    -22.4020768318201f},
  { -167.790690304062f,    36.1931709606190f},
  { -73.8771781041684f,    62.6758532093608f},
  { 78.1686319896982f,    -55.9441720743687f},
  { 20.0615442387106f,    19.6530253307824f},
  { -152.469798862263f,    -54.2132665752785f},
  { -14.9501877483486f,    10.2484960815050f},
  { 117.502184625429f,    -28.8535816049931f},
  { -110.523558680736f,    8.76625426550160f},
  { -51.7174624196886f,    -4.18923821487853f},
  { 80.1224180710943f,    19.1333526105987f},
  { -143.766569954226f,    11.5462454874764f},
  { 113.588882884686f,    9.77752477390750f},
  { -79.0968236178101f,    24.0218285186549f},
  { 163.092436383989f,    -30.6016758379373f},
  { -99.0242957324602f,    -48.9334604931480f},
  { -50.4936245692892f,    -42.9890233686657f},
  { -4.34949451017662f,    48.6733376541586f},
  { -119.559103109949f,    45.0258883303557f},
  { -42.0092655389920f,    -81.6980520077041f},
  { 178.390409513982f,    68.9783889557998f},
  { 145.027077103514f,    -2.58346669824488f},
  { 108.432262728508f,    48.3496164999097f},
  { -122.458269553311f,    -21.8932903097439f},
  { -81.5720012927753f,    -14.7181398413206f},
  { 15.5454908974895f,    -15.0940001549264f},
  { 146.362068766169f,    34.4857567311291f},
  { 176.568403725466f,    6.66120732619095f},
  { 44.3268798202961f,    74.4386767433966f},
  { 51.2636798459450f,    2.39164042843508f},
  { -159.941168510769f,    -19.6530253307824f},
  { 52.1529103439881f,    41.8196165088037f},
  { 144.929674278342f,    -61.1632446302154f},
  { 7.54241641510216f,    -53.9502789473135f},
  { 85.6915678397659f,    -17.5514161382425f}};
  
/* Sphere Covering (38 points) */
const float __SphCovering_38_dirs_deg[38][2] =
{ { 52.3058900752880f,    68.7377466818449f},
  { 18.7374387741633f,    -20.8860305059039f},
  { -3.43872079903666f,    -51.2304482938274f},
  { 154.853303290008f,    -30.2498797517269f},
  { 52.4680371313100f,    31.6140922619334f},
  { -109.618285364429f,    11.5622883057400f},
  { 4.39206527435484f,    49.0635219126427f},
  { 175.588645895792f,    69.1044396707286f},
  { 151.902570645084f,    9.40109150250655f},
  { -6.50880055268615f,    10.3648065139166f},
  { -34.7069184400496f,    37.4955040289513f},
  { -174.259383811089f,    34.6249854753459f},
  { 52.6026822131658f,    -45.1920460909437f},
  { -127.511757306365f,    -20.8785820545672f},
  { 173.560375301029f,    -57.6682020799174f},
  { -68.6575325905265f,    -57.8114415287001f},
  { -127.397165747339f,    45.1920460909437f},
  { 53.1166253553981f,    -82.0933928863444f},
  { 52.5310624887744f,    -8.28955337995275f},
  { 24.1610572628717f,    15.6302886511689f},
  { 139.738676654456f,    37.6610888317441f},
  { -75.1033077857483f,    -2.03210941199049f},
  { -80.6266209308094f,    34.5040913805733f},
  { 80.8386153150079f,    15.7093568268969f},
  { -127.614889709488f,    -58.0005176010932f},
  { -160.187540362676f,    -25.5670956921227f},
  { -145.313556001079f,    11.6132815495067f},
  { -179.862911047468f,    -1.90061559800797f},
  { -70.3191101964059f,    68.9669297998972f},
  { -20.1079538201162f,    -16.8037062155968f},
  { 108.592690911145f,    -51.0923654652009f},
  { 125.162630346328f,    -16.6449969063455f},
  { 86.3504693041664f,    -20.7937843008878f},
  { 100.519715577752f,    49.1867078385958f},
  { -46.8599262325695f,    9.23722557309913f},
  { -94.8417038280052f,    -25.6570500659583f},
  { 111.531964400166f,    10.5069000471090f},
  { -49.8358690204790f,    -30.4120268077490f}};

/* Sphere Covering (39 points) */
const float __SphCovering_39_dirs_deg[39][2] =
{ { 162.542396900663f,    38.2042528215282f},
  { -146.058401134749f,    43.9779485230615f},
  { -92.6759233624107f,    -42.7271816562909f},
  { -35.4764007589103f,    47.2558400690049f},
  { 93.9765375573576f,    25.3533824345389f},
  { -126.761182594743f,    -24.9328314129129f},
  { -61.7820390489567f,    -76.1117135051786f},
  { 163.877388563318f,    0.604470473863019f},
  { -134.163797307834f,    4.93763568687841f},
  { -145.577116586840f,    -54.9856136831148f},
  { 26.2420399747868f,    75.2465472345310f},
  { 3.67495129796910f,    -48.3948801657250f},
  { 78.6327278037542f,    -28.1803561957144f},
  { -96.9616476699892f,    -5.29985960496011f},
  { 13.8839132916101f,    42.6057146037231f},
  { -39.4000157399662f,    11.4803553410363f},
  { -26.0581205225498f,    -18.3037097232493f},
  { 33.3438518454334f,    12.3277599200348f},
  { -86.2072298553837f,    60.3267262493244f},
  { -65.7984731928238f,    -13.1115661837738f},
  { 111.961682746514f,    55.7430638882778f},
  { 59.3526979976020f,    42.4349731807742f},
  { 165.189461914168f,    -36.9958848315973f},
  { 40.8513178350326f,    -24.5712950441854f},
  { -6.25669912282859f,    17.1125304671723f},
  { 114.688961851337f,    -43.1947152171176f},
  { 64.6067209789516f,    5.10625716598541f},
  { -46.7040817122939f,    -43.6496437064515f},
  { 131.528191450232f,    21.0688040425506f},
  { -110.237079783170f,    28.2737483163207f},
  { -175.898043105163f,    71.0238482844169f},
  { -164.014898434149f,    -20.0455014204470f},
  { 144.797893985462f,    -69.8836622721065f},
  { -72.6453188446371f,    23.9083828752190f},
  { 133.338738082845f,    -14.8149697086977f},
  { 7.25364568635622f,    -12.5901745902047f},
  { 102.003676267140f,    -4.36851670897496f},
  { 55.9802684154620f,    -60.7220671279647f},
  { -165.418645032220f,    15.8554610646553f}};

/* Sphere Covering (40 points) */
const float __SphCovering_40_dirs_deg[40][2] =
{ { 34.0296823262050f,    -14.9238316897726f},
  { -133.035070451426f,    29.7044239307624f},
  { -106.896735837558f,    -59.7193909864857f},
  { 107.830657043621f,    -54.6005860447869f},
  { 68.4971044078899f,    38.4408843909172f},
  { -127.964393964518f,    -4.12896305483076f},
  { -176.843423467129f,    -25.7974247257653f},
  { 31.5241378880979f,    57.3358865587415f},
  { -101.906273441968f,    49.1580599488392f},
  { -33.7689865294205f,    11.8923119957354f},
  { -60.3038079375191f,    80.4318152804650f},
  { 44.4323040546002f,    -51.0986680009473f},
  { -38.3744212866820f,    51.1697147675436f},
  { -165.917118313984f,    59.4902078684334f},
  { 132.152715446924f,    4.39538842956660f},
  { -138.655786421659f,    -36.8692611588733f},
  { 162.668447615592f,    2.05055865299370f},
  { -31.3339159001145f,    -18.5718539713705f},
  { 92.5556022254332f,    5.56290452870468f},
  { -171.480538504704f,    25.4834438540336f},
  { 109.526612117208f,    31.1488505322872f},
  { -62.8190926581435f,    -39.2957374212524f},
  { 108.162972564797f,    -20.3755251104423f},
  { -98.4570675152807f,    -23.0661349163767f},
  { 144.511415087896f,    -30.8096595175698f},
  { -97.7007632257080f,    12.1891041336131f},
  { 62.3091602204770f,    4.72781854230150f},
  { 150.584767716283f,    37.4811800840731f},
  { 2.78159550380112f,    -34.4657032082995f},
  { 175.027147256564f,    -59.5532332258978f},
  { 30.9156567096690f,    20.2059296030836f},
  { 35.3302965211520f,    -85.2102832918560f},
  { -66.8813634256210f,    -5.26731560219668f},
  { -25.7349723260961f,    -57.8114415287001f},
  { -65.6036675424793f,    28.9360875274920f},
  { 1.57534745771220f,    -0.258535745896881f},
  { -158.044678208886f,    -1.69933552457851f},
  { 113.256567363510f,    65.2999999110599f},
  { -4.71257786495102f,    33.4527138265082f},
  { 71.5165919882294f,    -28.2817697254526f}};

/* Sphere Covering (41 points) */
const float __SphCovering_41_dirs_deg[41][2] =
{ { 42.8778695564103f,    26.0013977008319f},
  { -87.8859961951170f,    42.3765314856708f},
  { 1.92679976924545f,    34.5356040593055f},
  { -47.9875071733870f,    -18.7242607448753f},
  { -10.1442177627912f,    -18.4698674838372f},
  { -77.7675615331066f,    -79.5380011200609f},
  { 129.465543387761f,    38.5594866545093f},
  { -124.274545763876f,    71.3676229614953f},
  { 38.3612432573940f,    -7.37797752789961f},
  { -114.746257630850f,    -23.1990611248470f},
  { 119.811204539806f,    68.2793804457402f},
  { 134.003369125197f,    -1.13777958957079f},
  { 64.6983942261726f,    -30.2791005992786f},
  { -27.6045336116079f,    10.4559068033424f},
  { 20.2454636909476f,    -33.4355250926543f},
  { 170.351811648296f,    -11.8361621318125f},
  { 72.2327892321429f,    4.41372307901078f},
  { -131.075554792078f,    -50.5898814788712f},
  { 36.2647906850103f,    60.4470473863019f},
  { 157.632148596392f,    19.4118100990323f},
  { -76.8450994829460f,    -44.7967052123034f},
  { -130.605729400071f,    3.64338132345739f},
  { 149.702412711781f,    -70.4165130215782f},
  { -165.315512629096f,    14.1262744389504f},
  { 177.072606585181f,    -46.1076326475627f},
  { -155.139782187573f,    -19.8644467571856f},
  { 10.0250425414040f,    5.28977554776581f},
  { -101.144239574444f,    14.4333798171406f},
  { 99.8780028472051f,    -18.2395384501946f},
  { -179.564972994000f,    47.5188276969700f},
  { -134.960208643065f,    37.0720882183497f},
  { -30.2951434175423f,    70.4165130215782f},
  { -62.3263489543310f,    13.7698946903791f},
  { -40.8776738936086f,    43.2250819802596f},
  { 137.257769401540f,    -31.1717688440924f},
  { 80.6151617749068f,    39.0224365529750f},
  { 107.080082332000f,    14.6700113865296f},
  { -26.2133920850303f,    -49.5648599833821f},
  { 100.989540969759f,    -52.9716670332300f},
  { -84.4539790022834f,    -11.7674071963968f},
  { 37.1735017480878f,    -61.6044221324661f}};

/* Sphere Covering (42 points) */
const float __SphCovering_42_dirs_deg[42][2] =
{ { 108.856251496905f,    -64.2342984121166f},
  { -132.937667626254f,    -45.9706957345265f},
  { 17.1503456816509f,    55.2549038468263f},
  { -6.91044396707286f,    -7.18259891976000f},
  { -18.8932832944389f,    27.5294761404458f},
  { -59.4157233550664f,    82.8898042215762f},
  { -70.5253750026530f,    -15.1770790352204f},
  { 152.532824219728f,    -18.7603570859685f},
  { -120.653452498649f,    17.9977502606494f},
  { 94.6067911320015f,    -9.01663682197377f},
  { -82.9929366246997f,    -44.7296691502731f},
  { 5.02014160937725f,    -33.2773887411982f},
  { 150.498824047013f,    65.9474422195578f},
  { -108.839062763051f,    -17.7823181296802f},
  { -154.366289164146f,    21.9345432709933f},
  { 44.9422364922666f,    -31.6404483205095f},
  { -137.847915930525f,    52.9808343579521f},
  { -169.005360829739f,    -27.2813854151542f},
  { 172.185276592715f,    38.9078449939488f},
  { 123.277599200348f,    37.5997823476652f},
  { 122.223356857307f,    3.89427954194518f},
  { 13.6988479237829f,    22.9549811041213f},
  { 75.6476176911226f,    57.7140387035278f},
  { -126.102281130343f,    -77.2289812056837f},
  { -90.1148020181759f,    10.7693147172790f},
  { 168.575642483391f,    -52.7046687006990f},
  { 152.641686200803f,    14.7588198447749f},
  { -37.9005851901088f,    -30.9506071351719f},
  { -94.1197770061403f,    46.8685205994965f},
  { -61.2090812538259f,    21.6681178962575f},
  { -140.111099221292f,    -9.71220758526258f},
  { 124.732911999980f,    -32.2002280863523f},
  { 36.5037140855799f,    -64.4061857506559f},
  { 89.6048695805094f,    24.6486443465280f},
  { -42.2516266863323f,    51.3977519700056f},
  { 62.9279546392183f,    -1.49415933814216f},
  { 51.1101271568500f,    27.7557944695225f},
  { -176.511107945953f,    1.57420154212194f},
  { -31.2708905426501f,    -61.0429234932379f},
  { 29.2311607919843f,    -3.40445792288784f},
  { 80.4260857025137f,    -35.8104351534716f},
  { -39.5237746237144f,    0.881266384690719f}};
  
/* Sphere Covering (43 points) */
const float __SphCovering_43_dirs_deg[43][2] =
{ { 150.212345149448f,    -9.06533823455989f},
  { 123.466675272741f,    -60.1491093328338f},
  { 55.3471500518424f,    -57.5077738972807f},
  { 6.78210142096356f,    -51.9982117393027f},
  { -166.226515523354f,    -54.7518469027015f},
  { -53.0249521081772f,    27.2722180904321f},
  { 28.5963235549794f,    40.5522338659743f},
  { 155.139782187573f,    -35.9645608003618f},
  { -21.9637641185450f,    5.97881459219014f},
  { 90.3153372464717f,    1.82435491547605f},
  { 6.58844168620934f,    -16.7441186049032f},
  { 146.018294089090f,    28.5814266523060f},
  { 34.3883539059569f,    6.97862594469343f},
  { -77.6415108181779f,    57.5249626311347f},
  { -69.2992453210731f,    -32.8448056058744f},
  { 114.832201300120f,    -25.7859655698627f},
  { -22.6685022065559f,    42.0952092082616f},
  { 122.996849880734f,    6.17247432694436f},
  { 65.0765463709589f,    28.9412441476481f},
  { -112.735175769941f,    -52.3815205042453f},
  { -150.521742358819f,    10.3986110238293f},
  { 74.4386767433966f,    63.6785293508397f},
  { -171.314380744116f,    40.1700710166220f},
  { -49.2348362933868f,    -58.6021232859806f},
  { 6.02637008918600f,    18.6984776440944f},
  { 39.3100613661307f,    -26.6551425450762f},
  { 102.633929841784f,    33.5037070702749f},
  { -100.244695836089f,    -88.5964638610792f},
  { -171.629507531438f,    -18.9167745640393f},
  { 0.250182021243874f,    69.9753355193274f},
  { -26.3967385794722f,    -27.4607212050301f},
  { -93.6041149905226f,    28.0422733670879f},
  { 61.3064840789981f,    -6.31972448029298f},
  { -120.934201818263f,    6.29050363274131f},
  { -149.473229593729f,    75.0230936944300f},
  { -140.311634449587f,    -24.3014319226787f},
  { -82.7465647727935f,    2.81688970398118f},
  { -129.878073000255f,    41.4724040849544f},
  { 174.517214818897f,    9.88982450175314f},
  { 82.7637535066474f,    -33.5896507395445f},
  { 140.517899255834f,    57.3874527603033f},
  { -52.7665481425732f,    -5.65354645189437f},
  { -103.241265104623f,    -20.4288101853895f}};

/* Sphere Covering (44 points) */
const float __SphCovering_44_dirs_deg[44][2] =
{ { 120.819610259237f,    60.6991488161594f},
  { 29.3950267213918f,    -27.9557567400231f},
  { -0.0728114766052250f,    -17.3073361175168f},
  { 168.919417160469f,    16.7779231148159f},
  { 105.011704691577f,    -18.8084855407595f},
  { -21.8990198876952f,    64.8416336749553f},
  { 49.3156233425002f,    29.5084723648277f},
  { 43.5327603162448f,    -52.1070737203776f},
  { 158.623365581968f,    -64.6124505569029f},
  { 7.35849696286516f,    34.6513415339219f},
  { -53.7915696380622f,    4.27655698285647f},
  { -82.4371675634229f,    53.2948152296838f},
  { 71.6254539693042f,    -24.6675519537673f},
  { 168.031332578017f,    -16.2874712421839f},
  { 45.7862033244943f,    -3.39511871082721f},
  { 136.621786248945f,    -32.6121847410513f},
  { -176.682995284492f,    -42.2871500696304f},
  { -162.055382774802f,    -6.03496445611296f},
  { -76.1231726610812f,    26.2913143451681f},
  { 46.2382670248526f,    59.2209177047219f},
  { 89.1465033444048f,    37.3299192261585f},
  { 17.1085197626064f,    8.66827848253423f},
  { -153.569877828915f,    20.1474879079803f},
  { -69.4424847698558f,    -22.2553996362666f},
  { -170.506510252982f,    82.5746774342542f},
  { 135.246687540631f,    31.5270026770736f},
  { -11.9902877787027f,    -48.5707782088302f},
  { -103.682442606874f,    -30.8543502255900f},
  { 138.145853983993f,    -0.787072123171212f},
  { -20.0162805728953f,    9.25556022254332f},
  { -62.4810475590163f,    -53.3555487559677f},
  { 10.3527744002188f,    -82.5918661681082f},
  { -136.214986214402f,    52.3843852932209f},
  { -115.594235167644f,    25.7498692287695f},
  { 77.5670263048108f,    6.65948845280556f},
  { 97.5231463092174f,    -53.4042501685538f},
  { 109.165648706276f,    12.4738641577932f},
  { -120.825339837188f,    -60.6189347248411f},
  { -36.0384723559337f,    35.2036728484280f},
  { -142.735245922991f,    -31.2473992730497f},
  { -35.1160103057730f,    -22.5292734623391f},
  { 176.877800934836f,    45.8177160032265f},
  { -91.6102218634673f,    -0.511960708261196f},
  { -125.454838821845f,    -2.46772922362846f}};

/* Sphere Covering (45 points) */
const float __SphCovering_45_dirs_deg[45][2] =
{ { 77.2175220497810f,    -86.8088355402710f},
  { -140.472062632224f,    -55.6399314851542f},
  { -17.8258629221102f,    14.8424716828640f},
  { -76.2950599996204f,    -16.6656233869703f},
  { 10.2645388997687f,    -1.48757032349816f},
  { -109.933412151751f,    -2.28690374348517f},
  { -44.4305851812148f,    -3.72176194983129f},
  { -163.539343464191f,    27.5873448777540f},
  { 16.8896498848664f,    30.3667631419336f},
  { -4.68564884857987f,    -54.6613195710708f},
  { -12.0298218665668f,    49.5906430841630f},
  { 39.7867622516795f,    0.436387575083440f},
  { 155.684092092947f,    -26.4672123882733f},
  { -74.7308852189133f,    -63.3519434076151f},
  { -167.126059261710f,    -25.1585767841944f},
  { -16.2616881414030f,    -20.7055488004377f},
  { 59.2381064385758f,    -56.8471535594949f},
  { 87.8458891494578f,    35.4953083661496f},
  { 179.221198316922f,    60.8538474208447f},
  { 105.676335733929f,    3.63301078736552f},
  { 143.617600927492f,    3.57141782438896f},
  { 125.775695187118f,    -19.9990918390414f},
  { 125.930393791804f,    27.6102631895592f},
  { -47.2151600655506f,    -37.2926769694750f},
  { 118.682477683399f,    -51.9002359563354f},
  { -71.1040623757352f,    56.0295427858432f},
  { 169.526752423308f,    -56.0524610976484f},
  { 117.702719853725f,    59.4902078684334f},
  { -127.076309382065f,    23.6150284841120f},
  { 176.528296679807f,    2.27853855967626f},
  { -75.7106430485870f,    9.46984643792225f},
  { 90.5502499424753f,    -29.1755838858567f},
  { -129.837965954596f,    54.0952372694815f},
  { -33.7271606103759f,    84.1331226370101f},
  { 55.4863787960592f,    -23.8516600535010f},
  { 76.5700797412832f,    2.61721391237809f},
  { 52.1254083698218f,    26.5353943658938f},
  { -91.5873035516621f,    29.7703140772024f},
  { -48.0207387255046f,    29.7697411194073f},
  { 158.090514832497f,    32.8127199693471f},
  { 21.3902333656190f,    -32.2242923137478f},
  { 47.6826936263774f,    59.5016670243360f},
  { -130.428112483581f,    -25.9773334734364f},
  { -147.221505458865f,    0.0291457900805098f},
  { -94.5036587288780f,    -35.9187241767513f}};

/* Sphere Covering (46 points) */
const float __SphCovering_46_dirs_deg[46][2] =
{ { -135.309712898095f,    -6.19367376536420f},
  { 61.9424672315933f,    18.7660866639199f},
  { -15.3758953901308f,    -8.16579449620449f},
  { 114.442589999431f,    41.5102192994330f},
  { -168.186031182702f,    52.7132630676260f},
  { -53.1550135276719f,    -61.1002192727510f},
  { -110.242809361122f,    12.0195086262544f},
  { -178.006527791244f,    20.7794603560096f},
  { 70.3534876641138f,    -61.7476615812488f},
  { 33.0413301296043f,    34.1041668395720f},
  { 21.5323268988115f,    69.6831270438107f},
  { 37.0165113122220f,    -1.84400736784904f},
  { -168.329270631485f,    -8.59150213798669f},
  { 9.66064138370081f,    10.0473878954141f},
  { -146.831894158176f,    20.6310642870707f},
  { -26.5485723951818f,    18.7838483555689f},
  { -57.4046414941572f,    19.6169289896891f},
  { 50.0369772065699f,    -32.5147819158791f},
  { -86.1155566081627f,    35.7325128933338f},
  { 157.019083755602f,    0.0523889689555820f},
  { -95.9761602623642f,    71.5051328323267f},
  { 138.059910314723f,    71.2874088701770f},
  { -110.093840334388f,    -53.0862585922562f},
  { 170.179924309757f,    -30.5100025907163f},
  { 12.9820777220742f,    -25.3917706068127f},
  { -70.9493637710499f,    -31.2817767407576f},
  { -107.257699248490f,    -23.4471518501387f},
  { 10.5235158231678f,    -55.0537956607354f},
  { 129.368140562589f,    11.4081626588498f},
  { -124.022444334018f,    42.5793585451471f},
  { -146.957944873105f,    -34.1041668395720f},
  { 152.446880550458f,    35.9439343197371f},
  { 133.613757824508f,    -23.9353118915901f},
  { -170.454944051420f,    -59.7766867659988f},
  { 71.1900060450048f,    50.7560392394591f},
  { -152.836491851147f,    -88.8313765570828f},
  { 132.525138013759f,    -53.6414546957379f},
  { 93.8848643101367f,    -35.7325128933338f},
  { -46.3883819671768f,    51.2711282972817f},
  { 92.7561374537290f,    18.9924049929965f},
  { -3.99804219864337f,    40.5596823173110f},
  { -48.5759348289863f,    -10.0044160607793f},
  { -27.5541133256364f,    -35.9439343197371f},
  { 104.879924398697f,    -7.55502148659504f},
  { -82.9986662026511f,    -0.293073641787367f},
  { 71.7686934180869f,    -10.5464341349731f}};

/* Sphere Covering (47 points) */
const float __SphCovering_47_dirs_deg[47][2] =
{ { 174.528673974800f,    35.0650170620064f},
  { -140.025155552022f,    -38.1836263409035f},
  { 65.1453013063746f,    -19.2874782574889f},
  { 9.07565147487224f,    51.6819390363905f},
  { 160.284943187848f,    -17.6213169892485f},
  { -23.2804211317556f,    -5.61676256144697f},
  { 121.249328605585f,    17.0214301777465f},
  { -45.5014433003143f,    -26.8413538284937f},
  { 7.96124856334279f,    -6.22346757071100f},
  { 5.69978414596143f,    -69.8550143823500f},
  { 91.8909711830814f,    -2.15810283113976f},
  { 93.7244361275001f,    31.6335728269679f},
  { -149.341449300849f,    29.3629410848644f},
  { -169.343405928866f,    63.1972448029298f},
  { -57.6051767224530f,    3.12548477243864f},
  { -133.184039478160f,    -74.2438710930521f},
  { -114.975440748902f,    -15.1266587492489f},
  { 139.205825904985f,    -41.7829472099153f},
  { -88.7511624657645f,    3.81183091522585f},
  { 30.2493067939318f,    81.3542773306256f},
  { -115.153057665393f,    20.0099780371489f},
  { -39.7334771767323f,    23.1744239396564f},
  { -77.6300516622752f,    33.1501921106792f},
  { 27.7741291189667f,    -37.1643344233657f},
  { 29.7101535087137f,    21.4314863268684f},
  { 152.658874934657f,    9.38447572644775f},
  { -35.4064999079044f,    49.8880081798359f},
  { -7.33958935562585f,    -36.1880143404628f},
  { -140.884592244718f,    -1.99670062025141f},
  { -166.679152181508f,    -24.8142291493208f},
  { -120.407080646743f,    45.0109914276824f},
  { 100.330639505358f,    -33.5403763691633f},
  { 38.9066990783586f,    -8.14001139542361f},
  { 113.205001161948f,    -68.5028339858412f},
  { 109.188567018081f,    62.5039658708215f},
  { -50.2982459611496f,    -55.6731630372718f},
  { -4.79915178779529f,    22.0147573623116f},
  { -79.1312010855180f,    -25.3098376421090f},
  { 56.2060137867435f,    46.3923926717428f},
  { 64.0051152940643f,    14.9627928198414f},
  { 138.678704733464f,    43.0743940801402f},
  { 63.9478195145512f,    -49.2096261504010f},
  { -100.198859212478f,    -47.2707369716783f},
  { 124.973554273935f,    -11.3084680024971f},
  { -179.673834975075f,    -50.7491637459175f},
  { -82.5402999665464f,    66.4401859233703f},
  { -173.182223156243f,    5.68740825758661f}};

/* Sphere Covering (48 points) */
const float __SphCovering_48_dirs_deg[48][2] =
{ { -104.771062417622f,    -1.27345599545777f},
  { -130.938044921247f,    -15.2882328474758f},
  { -107.538448568104f,    -66.2110028053179f},
  { 172.838448479164f,    56.3968087325221f},
  { 157.935816227811f,    -29.5743625112677f},
  { -13.7234851089735f,    -3.35661594699441f},
  { 85.4280072540058f,    -81.6063787604831f},
  { 122.217627279356f,    -49.0910238868089f},
  { -166.066087340718f,    25.4616714578187f},
  { -138.438062459510f,    49.2399929135430f},
  { -14.7278801238378f,    -35.3446204660302f},
  { -74.0662541765615f,    15.3346424288814f},
  { 86.8260242741250f,    -25.6599148549339f},
  { -87.1640693732521f,    61.7190136914923f},
  { 50.6804088105018f,    -21.2807984267490f},
  { 11.1274133392357f,    12.4119847159190f},
  { 72.8974202744947f,    -51.2344589983933f},
  { -166.209326789501f,    84.8264015691184f},
  { -19.4507712291012f,    29.8545388730867f},
  { -168.535535437732f,    -27.7323031999221f},
  { -98.2966393326440f,    -32.9972123793792f},
  { -19.1677300783066f,    -63.5352899020570f},
  { 16.0496937572046f,    -20.4780845557708f},
  { -55.0268666443643f,    41.7084626965483f},
  { -156.148187907003f,    -2.21476835707820f},
  { -130.216118099382f,    19.1534061334283f},
  { 63.1113011336602f,    62.6815827873121f},
  { -9.75403350430713f,    61.4898305734399f},
  { 38.8161717467279f,    5.69479941314379f},
  { -61.4325347939269f,    -45.4172185044301f},
  { 170.701315903326f,    0.342972536165311f},
  { -44.2695840407831f,    -20.8533719115814f},
  { 73.1896287500114f,    2.30867613970014f},
  { 28.7218013121130f,    -49.7155478835015f},
  { -73.0750371909852f,    -12.3770342904160f},
  { 126.554917788496f,    24.7271395644609f},
  { 21.1994384198405f,    39.0350416244679f},
  { -139.045397722348f,    -42.5461269930295f},
  { 122.085846986476f,    54.1863375589073f},
  { 139.807431589872f,    -1.07309265450052f},
  { 53.7571921703544f,    29.9920487439181f},
  { 103.968921504439f,    4.55850951384034f},
  { -43.4479625625655f,    8.76453539211620f},
  { 173.835395042692f,    -60.0173290399537f},
  { 158.038948630935f,    28.6725269417318f},
  { -98.6862506333330f,    31.5630990181668f},
  { 120.493024316012f,    -21.4303404112782f},
  { 88.0808018454615f,    32.9347599797100f}};

/* Sphere Covering (49 points) */
const float __SphCovering_49_dirs_deg[49][2] =
{ { -93.1285600205640f,    -56.4999411356456f},
  { 91.5815739737108f,    -34.3619978473809f},
  { -111.434561574994f,    -29.1360497979926f},
  { -1.53902193350090f,    -19.4358743264278f},
  { 7.26338596887345f,    -43.9550302112562f},
  { -139.962130194558f,    53.5726997603222f},
  { -152.670334090559f,    -77.7675615331066f},
  { 74.8454767779394f,    69.7232340894699f},
  { -173.600482346688f,    35.8511151569259f},
  { 24.1232420483931f,    34.9687601524244f},
  { 161.281889751375f,    -53.1338140892520f},
  { -112.569018009353f,    30.4320803305785f},
  { 130.657295601633f,    -30.4836465321403f},
  { -82.5460295444977f,    48.5919776472500f},
  { -48.9403359866895f,    26.8023926984248f},
  { 104.702307482207f,    43.9132042922117f},
  { 44.5302798375676f,    9.90815915119733f},
  { 144.414012262724f,    32.9427813888418f},
  { 112.947170154139f,    19.2863323418986f},
  { 14.6808975846371f,    60.1662980666877f},
  { 11.6866201472834f,    5.10786144781178f},
  { 63.0081687305366f,    39.2814134763741f},
  { 102.330262210365f,    -8.59207509578183f},
  { -35.8035596599300f,    -61.2606474553876f},
  { -85.1816354020995f,    -11.0156865691852f},
  { -69.2247608077061f,    -34.6513415339219f},
  { 52.7751425095001f,    -42.5570131911370f},
  { -145.250530643615f,    -46.2491532229601f},
  { 105.361208946607f,    -59.3928050432611f},
  { 155.168430077330f,    61.0601122270918f},
  { -169.331946772964f,    2.42149152956140f},
  { 162.124137710218f,    -19.4020698165151f},
  { -138.776107558637f,    -12.5896016324096f},
  { -11.2981547621847f,    28.2021285919294f},
  { -32.6477081243494f,    55.7596796643366f},
  { 162.645529303787f,    10.8649986690658f},
  { -143.577493881833f,    18.9316714667127f},
  { -32.7869368685662f,    -30.3679090575239f},
  { -168.982442517934f,    -27.5231736046994f},
  { -103.201158058964f,    78.9020179674657f},
  { 34.9487066295948f,    -18.6234201729323f},
  { 133.098095808890f,    -0.836862155568080f},
  { -52.3820934620404f,    -4.85879669426841f},
  { 78.9478545910761f,    15.5999218880269f},
  { -112.861226484870f,    4.61116433521287f},
  { -79.1884968650311f,    16.6215056367452f},
  { 40.2376800364475f,    -72.6796963123449f},
  { -23.5714836916821f,    2.85533517203446f},
  { 68.8637973967736f,    -11.7983469173339f}};

/* Sphere Covering (50 points) */
const float __SphCovering_50_dirs_deg[50][2] =
{ { -79.0051503705892f,    0.825402999665464f},
  { -111.990330636271f,    -45.5088917516510f},
  { -141.211178187943f,    13.7257769401540f},
  { -124.904799338519f,    41.8408159472235f},
  { -130.817723784270f,    73.1838991720601f},
  { -69.2419495415600f,    -59.0604895220853f},
  { -36.9019197531958f,    -37.5218600875274f},
  { -75.8424233414671f,    55.4903895006251f},
  { 172.517592113891f,    -30.9546178397379f},
  { -16.0451100948436f,    -13.0943774499198f},
  { -57.5478809429399f,    23.3846994504694f},
  { 34.9000052170087f,    15.9179134643245f},
  { 132.015205576093f,    7.18259891976000f},
  { 19.8484039389220f,    -14.9776897225148f},
  { -90.4070104936926f,    28.6690891949610f},
  { 157.844142980591f,    27.7775668657374f},
  { 139.647003407236f,    -23.9009344238823f},
  { 64.1311660089930f,    18.1736483037546f},
  { -4.67126760792209f,    -67.2996226160665f},
  { -163.705501224779f,    -12.3214573842884f},
  { 123.134359751565f,    38.1200280256439f},
  { -162.983574402914f,    42.3937202195247f},
  { -135.143555137507f,    -17.1715451200708f},
  { -47.9852153422064f,    -7.18259891976000f},
  { 1.34977397376919f,    -38.7159041325800f},
  { 5.02277721523485f,    37.9406922357680f},
  { -112.981547621847f,    9.20284810539129f},
  { -147.439229421015f,    -76.5987276310398f},
  { 100.370746551018f,    14.9129454916651f},
  { -25.8054461348971f,    16.0468289682290f},
  { 42.1857365398923f,    -43.4748915789366f},
  { 157.975923273471f,    57.6739316578687f},
  { 79.2572518004468f,    -6.43718082829480f},
  { 77.0685530230470f,    -35.1492418578906f},
  { -173.027524551557f,    14.9404474658313f},
  { 41.3841685845042f,    46.9252434212144f},
  { -102.794358024421f,    -17.7634105224409f},
  { 162.508019432955f,    -2.59057137490450f},
  { -72.5536455974162f,    -29.2472036102480f},
  { 154.767359620738f,    -56.9669017386772f},
  { 4.04697279434754f,    9.17133542665909f},
  { 48.5902587738646f,    -12.4916258494422f},
  { 77.2232516277324f,    -69.4367551919045f},
  { -152.274993211919f,    -45.1937649643291f},
  { 0.197985566107456f,    72.1411159849220f},
  { 108.523935975729f,    -14.7072536432131f},
  { 112.901333530529f,    -44.3727164439066f},
  { 84.1617705267667f,    41.3045274509811f},
  { 94.9333770752261f,    69.5341580170767f},
  { -32.5892664292461f,    45.3965920238054f}};

/* Sphere Covering (51 points) */
const float __SphCovering_51_dirs_deg[51][2] =
{ { -101.568228342841f,    -23.6368008803270f},
  { -94.4578221052675f,    5.01928217268455f},
  { -34.5293015235591f,    -12.7139334739530f},
  { 153.231832729787f,    61.0658418050432f},
  { 65.5463717629662f,    -66.8584451138158f},
  { -85.2102832918560f,    60.4241290744966f},
  { 135.556084750001f,    -22.5779748749252f},
  { 0.178275817954956f,    67.7866367419277f},
  { -36.3702149193144f,    20.7874817651414f},
  { -40.9704930564198f,    47.1509887924960f},
  { 103.722549652533f,    -21.4165894241950f},
  { -110.494910790979f,    32.4431621914877f},
  { -74.1350091119772f,    30.6698578155578f},
  { -4.34771834101171f,    -55.3935596332480f},
  { 99.1961830709994f,    11.8739773462912f},
  { -5.23494348677179f,    -26.6597262074372f},
  { 15.8961410681096f,    -12.5993419149268f},
  { -69.4367551919045f,    -17.8671158833596f},
  { 53.2833560737812f,    16.2296025048757f},
  { -43.5659918683624f,    -39.6567008321848f},
  { -82.1277703540522f,    -49.5362120936256f},
  { 117.702719853725f,    39.3054777037696f},
  { 172.162358280910f,    -73.6881020317752f},
  { -163.419022327213f,    -41.7468508688220f},
  { 156.669579500572f,    -0.651453013063746f},
  { 71.3561638055927f,    -34.4897674356950f},
  { 22.9177388474378f,    14.8023646372048f},
  { -177.932043277877f,    11.8075142420560f},
  { -60.7220671279647f,    5.83442922781717f},
  { 82.4543562972768f,    60.7621741736238f},
  { 123.965148554505f,    5.76567429240147f},
  { 76.1289022390325f,    -3.92579222067737f},
  { -52.0549345610207f,    -74.7308852189133f},
  { -124.286004919778f,    2.29853478672632f},
  { -141.927375431856f,    52.3626128970059f},
  { -156.681038656475f,    -8.35487056859767f},
  { 33.7861752632744f,    -40.8644958643206f},
  { 110.930358715279f,    -50.7697902265422f},
  { -144.889567232683f,    21.1346941889907f},
  { 37.7040606663790f,    45.1714196103190f},
  { 173.812476730887f,    -20.6769009106811f},
  { -1.48264288646003f,    34.9601657854974f},
  { -169.669991872091f,    83.8695620512499f},
  { -124.515188037831f,    -54.7999753574925f},
  { 148.075212573610f,    26.9427673582318f},
  { 155.197077967086f,    -44.9949486094187f},
  { 46.1190918034654f,    -11.4625936493872f},
  { -10.0365016973066f,    5.70923794958109f},
  { -131.912073172969f,    -26.5565938043137f},
  { -178.407598247836f,    38.3394708611790f},
  { 77.4123277001255f,    31.2961006856358f}};

/* Sphere Covering (52 points) */
const float __SphCovering_52_dirs_deg[52][2] =
{ { -129.832236376645f,    -2.67645774839461f},
  { 174.139062674111f,    -34.9996998733615f},
  { 39.7764490113671f,    17.6035552975994f},
  { -104.547608877521f,    21.1026085524634f},
  { -63.7816617539632f,    70.1013862342562f},
  { -18.4916398800522f,    -5.37159392091049f},
  { -155.139782187573f,    -18.3850697301579f},
  { 12.0561779251428f,    37.5063902270588f},
  { 106.724848499018f,    -36.7833174896037f},
  { 170.775800416693f,    25.8449802227612f},
  { 173.228059779853f,    -6.85887776551108f},
  { 54.4309905374282f,    44.3589654568235f},
  { 61.9539263874959f,    -2.50881029753934f},
  { 31.5052302808586f,    -47.5342975574385f},
  { -12.9133227866585f,    18.8892725898730f},
  { -125.334517684868f,    -34.8851083143353f},
  { 132.737132397958f,    26.0907791168723f},
  { -98.0617266366404f,    -63.2602701603942f},
  { -71.3160567599336f,    38.0982556294290f},
  { -53.9193392263764f,    -47.2632885203416f},
  { 38.0443975966867f,    -16.4851416815040f},
  { 142.826919170212f,    -55.8353100932939f},
  { 7.92801701122520f,    -20.8986355773968f},
  { -156.400289336861f,    -51.9621153982095f},
  { -32.9805966033205f,    42.5587320645224f},
  { -9.22462050160626f,    -68.9382819101407f},
  { 172.001930098273f,    -80.2885758316823f},
  { -111.572071445825f,    49.4943861745810f},
  { 75.7163726265383f,    21.1679257411083f},
  { 88.4646835681991f,    74.2438710930521f},
  { 142.454496603377f,    -25.6490286568264f},
  { -161.597016538697f,    43.4009800233647f},
  { 10.6644634407700f,    62.2977010645744f},
  { 148.871623908842f,    51.2493559010668f},
  { -102.994893252717f,    -9.25957092710923f},
  { 100.038431029842f,    47.0885363928267f},
  { -135.917048160934f,    25.6381424587190f},
  { 120.435728536499f,    -6.84340790504255f},
  { -67.3798367073848f,    -14.0065262597681f},
  { 15.2074457983623f,    4.39813862698323f},
  { -77.0570938671444f,    8.25116520767899f},
  { 79.7901025499184f,    -63.4035096091769f},
  { 148.040835105902f,    3.49286531067652f},
  { -158.674931783530f,    70.8806088356342f},
  { -13.6530113001724f,    -38.9261796433930f},
  { 104.123620109125f,    16.0353698123264f},
  { -161.849117968555f,    9.71048871187719f},
  { 89.8455118544644f,    -10.4914301866405f},
  { -41.8230542555744f,    -17.9731130754588f},
  { -46.7327296020505f,    13.7418197584177f},
  { 67.5517240459241f,    -34.0554654269859f},
  { -88.1151793131693f,    -34.2628761488232f}};

/* Sphere Covering (53 points) */
const float __SphCovering_53_dirs_deg[53][2] =
{ { -71.9921469581879f,    24.6308826548790f},
  { 151.398367785369f,    35.7955382507982f},
  { 134.347143802275f,    62.2232165512074f},
  { -39.2143774143438f,    38.1160173210780f},
  { 76.0143106800063f,    31.7876984738581f},
  { -139.629814673382f,    85.4967621894215f},
  { -179.261305362581f,    -10.0313450771505f},
  { 1.07504071100396f,    -8.71468806393982f},
  { 146.448012435438f,    -16.1402210888353f},
  { -66.1594366037562f,    56.7256865069272f},
  { 39.4469982791669f,    41.3509370323866f},
  { -125.460568399796f,    -50.9204781266617f},
  { -98.6117661199660f,    3.57456909226218f},
  { 32.3446134507252f,    -12.9752022285326f},
  { -124.486540148074f,    -18.9471413271812f},
  { 109.033868413396f,    37.6381705199389f},
  { -81.9272351257564f,    -76.5471614294780f},
  { -141.383065526482f,    30.0441879032750f},
  { -125.225655703793f,    11.0231350205219f},
  { -154.349100430292f,    -30.6119890782496f},
  { 167.710476212743f,    -37.0600561046519f},
  { -126.440326229470f,    57.4218302280111f},
  { 128.955610950094f,    -46.3889549249720f},
  { 18.1472922451786f,    15.5930463944854f},
  { 99.6774676189093f,    7.38829076821197f},
  { -57.2877581039505f,    -24.1902781104234f},
  { -1.98879380267860f,    64.4004561727045f},
  { -101.258831133470f,    34.8089049275829f},
  { 70.8347722120237f,    61.8794418741289f},
  { -175.439676869058f,    51.9283108882968f},
  { -29.5892594139411f,    -19.1281959904425f},
  { 77.9108009818893f,    1.19381486193458f},
  { -45.0264612881509f,    7.69768797758261f},
  { -72.5651047533188f,    -6.26414757416529f},
  { 61.2606474553876f,    -18.9746433013475f},
  { 130.628647711876f,    14.7359015329696f},
  { 49.2892672839242f,    12.1547266659053f},
  { 158.915574057485f,    8.91063962987456f},
  { 3.13912116796275f,    -34.5333122281250f},
  { -75.3668683715085f,    -47.7531674351785f},
  { 79.3374658917651f,    -48.6687539917975f},
  { -175.714696610721f,    -60.8653065767474f},
  { -172.918662570482f,    22.4192655656740f},
  { 95.7985433458736f,    -26.6860822660132f},
  { 38.4918776346838f,    -41.2500964604436f},
  { -16.9727287651604f,    11.5995305624235f},
  { -1.35899859427080f,    37.1528752674631f},
  { 111.256944658503f,    -72.5421864415135f},
  { -96.3600419851018f,    -26.6757690257009f},
  { 20.7250293654721f,    -66.6865577752765f},
  { -152.326559413481f,    -0.706227778278253f},
  { -29.1475089538952f,    -50.1842273599185f},
  { 120.819610259237f,    -14.6167263115824f}};

/* Sphere Covering (54 points) */
const float __SphCovering_54_dirs_deg[54][2] =
{ { 43.8031963955466f,    0.662912168966363f},
  { -111.703851738705f,    24.7563604120126f},
  { 120.034658079907f,    16.0032841757990f},
  { -14.6877730781787f,    30.6074054158886f},
  { 42.1158356888863f,    -32.4993120554106f},
  { 43.6479248330661f,    30.8497665632289f},
  { 67.5689127797780f,    50.6689496545992f},
  { 99.5514169039805f,    -19.4862946123993f},
  { 108.426533150557f,    -68.5601297653543f},
  { -49.9011862091239f,    28.8925427350620f},
  { 18.2624567619999f,    -16.1253241861619f},
  { -35.2231534134625f,    57.6166358783556f},
  { 67.4256733309953f,    -12.9568675790884f},
  { -172.637913250868f,    -28.2565595824668f},
  { 176.132955801166f,    28.4983477720120f},
  { 99.7863295999842f,    38.2082635260941f},
  { 79.3431954697164f,    -41.5211054975405f},
  { -173.629130236445f,    54.5558953367667f},
  { -76.6732121444068f,    -28.5516328469592f},
  { -149.209669007969f,    -76.3294374673283f},
  { -67.0131437185011f,    0.823225760043967f},
  { 151.553066390054f,    -22.7578836225963f},
  { -117.771474789141f,    79.9505307325551f},
  { -43.5144256668006f,    -48.8257444276634f},
  { -124.154224626898f,    49.5385039248061f},
  { 166.965631079073f,    -50.7972922007085f},
  { 123.523971052254f,    -42.4567455769891f},
  { -145.680248989963f,    -18.2126094338235f},
  { 127.013284024601f,    65.1854083520338f},
  { 71.7056680606225f,    18.0361384329232f},
  { -114.201947725476f,    -26.5313836613279f},
  { -36.1307185609497f,    4.92520250272407f},
  { -44.6299744939203f,    -20.3818276461888f},
  { 150.670711385553f,    9.70533209172102f},
  { -8.81209088911206f,    0.517850714395141f},
  { -81.8584801903407f,    51.2321671672128f},
  { 95.0021320106418f,    5.69783608945799f},
  { 3.11912494091269f,    -44.6792488643016f},
  { 41.8780582039070f,    -62.6873123652634f},
  { -16.1052706633323f,    -24.0630814799043f},
  { 15.8193647235620f,    14.1744028937414f},
  { -83.6346493552463f,    23.6322172179659f},
  { -93.5754671007661f,    -52.5889312260826f},
  { 141.795595138976f,    39.4693436331770f},
  { 175.651671253257f,    -2.97674492882268f},
  { -159.849495263548f,    4.44357418013710f},
  { 34.0354119041563f,    71.3160567599336f},
  { 126.394489605860f,    -9.89039745954827f},
  { 14.8504930919958f,    45.2699683510815f},
  { -96.8642448448170f,    -5.07995840318891f},
  { -148.258559068052f,    27.8480406745385f},
  { -141.938834587759f,    -48.5547353905665f},
  { -126.978906556893f,    3.47309826674451f},
  { -29.7101535087137f,    -74.0032288190971f}};

/* Sphere Covering (55 points) */
const float __SphCovering_55_dirs_deg[55][2] =
{ { -40.5826006291162f,    -58.9401683851078f},
  { -100.124374699111f,    42.8681292738931f},
  { -177.582539022847f,    -4.38467411879765f},
  { 18.7271255338510f,    12.8869667280825f},
  { 58.7682810465685f,    -36.7225839633199f},
  { 6.08194699531369f,    -11.3353970188682f},
  { -138.059910314723f,    46.6112625494827f},
  { -49.0663867016183f,    -27.4802017700645f},
  { 154.773089198689f,    -6.88752565526763f},
  { 147.164209679352f,    -29.8642791556039f},
  { -121.489970879540f,    74.8683950897447f},
  { 114.751987208801f,    -40.3625848357860f},
  { 53.7749538620034f,    51.8320539787148f},
  { -123.403649915277f,    -7.56189698013661f},
  { -11.2150758818907f,    16.7343783223860f},
  { 139.561059737966f,    42.1662559748578f},
  { -154.922058225423f,    -56.3458154887554f},
  { -126.325734670444f,    21.7511967765514f},
  { -69.2133016518035f,    -4.49903649470576f},
  { -14.7043888542374f,    46.8925848268920f},
  { -42.4636210705307f,    28.5837184834865f},
  { 23.8625462516085f,    -61.9940334331551f},
  { -150.567578982429f,    3.81354978861125f},
  { 84.0299902338865f,    -59.9600332604407f},
  { 47.4787206513108f,    20.9948924869788f},
  { -151.295235382245f,    -25.2640010184985f},
  { 89.6564357820712f,    -26.8127059387371f},
  { -163.166920897356f,    26.1303132047363f},
  { 65.1510308843259f,    -6.04871544319610f},
  { 7.96067560554766f,    72.1010089392628f},
  { -21.9110520013929f,    -10.8844792341003f},
  { 97.9012984540038f,    46.4812011299880f},
  { 138.334930056386f,    13.4943019909211f},
  { 23.1658295727294f,    -34.4078344709913f},
  { -63.0826532439036f,    56.3309185860820f},
  { -70.8462313679263f,    24.2974212181128f},
  { 146.407905389779f,    -89.5017371773859f},
  { -43.0852802782476f,    4.52653846887204f},
  { 121.025875065484f,    -10.5790927292955f},
  { 17.7393462950454f,    39.0940562773663f},
  { -97.8096252067828f,    11.7399052222306f},
  { 39.1215582515326f,    -9.54547686687952f},
  { 177.164279832402f,    -32.8046985602153f},
  { 115.857795753404f,    72.8172061831763f},
  { 110.552206570492f,    20.6350749916366f},
  { -13.7504141253446f,    -38.6351170834665f},
  { 175.760533234331f,    54.1078423409744f},
  { 93.3520135606650f,    -0.832794155222652f},
  { 168.965253784080f,    22.5183872642316f},
  { -77.2404403615863f,    -37.2445485146840f},
  { 75.4929190864373f,    23.9221338623021f},
  { -102.376098833975f,    -61.0371939152866f},
  { -94.9276474972748f,    -17.2070685033689f},
  { -120.017469346054f,    -36.8520724250194f},
  { 152.263534056016f,    -61.0486530711892f}};

/* Sphere Covering (56 points) */
const float __SphCovering_56_dirs_deg[56][2] =
{ { -108.896358542564f,    38.6947046941601f},
  { 95.0307799003983f,    31.1803632110194f},
  { -24.8337097143553f,    -67.2537859924560f},
  { -14.5565657430937f,    34.3270474218779f},
  { 64.6582871805134f,    -14.3136316379582f},
  { -90.7851626384789f,    60.8824953106013f},
  { 127.053391070260f,    -40.7653741657629f},
  { -141.812783872830f,    -29.1566762786173f},
  { 117.823040990702f,    12.4841773981055f},
  { -156.171106218809f,    -2.67078546622282f},
  { -71.0066595505629f,    -30.6435017569818f},
  { 18.1753671771400f,    46.6530884685273f},
  { 3.76828612279591f,    -42.2768368293181f},
  { 155.305939948161f,    -22.3528024614388f},
  { -41.4792795784960f,    -42.6716047501632f},
  { 146.138615226068f,    3.65518425403709f},
  { 69.5971833745411f,    -41.5027708480963f},
  { 122.423892085603f,    -14.2168017705811f},
  { -165.682205617980f,    26.3251188550808f},
  { -95.4375799349412f,    -13.7469763785738f},
  { 53.4861831332575f,    70.6800736073384f},
  { -134.857076239942f,    18.7357199007779f},
  { -109.394831824328f,    -36.7592532622082f},
  { -24.3776353094311f,    65.9875492652169f},
  { -87.2500130425218f,    -58.8771430276434f},
  { -173.073361175168f,    -30.7265806372758f},
  { 164.244081552202f,    25.2250398884296f},
  { -146.453742013390f,    -57.9947880231419f},
  { -44.2581248848804f,    16.8781907289638f},
  { 178.333113734469f,    -83.4971394844149f},
  { -101.992217111238f,    12.5489216289553f},
  { 11.2082003883492f,    15.3197455262080f},
  { 104.232482090199f,    -61.4611826836834f},
  { 133.487707109579f,    34.4571088413726f},
  { 170.265867979027f,    53.8929831678004f},
  { 34.7911432359338f,    -30.3186346871426f},
  { -70.7946651663645f,    1.57362858432681f},
  { 66.3198647863928f,    45.2722601822620f},
  { -76.3695445129874f,    28.4702728400506f},
  { 40.8707984000670f,    -62.2690531748179f},
  { -51.7317863645669f,    43.6651135669200f},
  { -161.900684170117f,    79.1483898193719f},
  { 41.1160243363830f,    25.7487233131792f},
  { 39.2424523463052f,    -1.23672940078988f},
  { -17.9593620883757f,    5.10958032119717f},
  { 68.9554706439946f,    13.1637053431307f},
  { -46.7602315762167f,    -10.8162972564797f},
  { -18.7431683521146f,    -21.9253759462712f},
  { 177.582539022847f,    -1.62473641965248f},
  { -124.452162680366f,    -6.79069578789052f},
  { 163.608098399607f,    -53.4907667956185f},
  { 10.2009405845092f,    -10.9096893770860f},
  { -144.631736224874f,    49.5213151909522f},
  { 115.691637992816f,    57.9947880231419f},
  { 95.4032024672334f,    -28.1534271793433f},
  { 92.1029655672798f,    -0.917534613122500f}};

/* Sphere Covering (57 points) */
const float __SphCovering_57_dirs_deg[57][2] =
{ { 157.425883790145f,    -74.1407386899285f},
  { 128.835289813117f,    5.11049705366938f},
  { -177.238764345769f,    -20.7112783783890f},
  { 131.728726678528f,    -24.2618978348147f},
  { 95.4089320451847f,    -30.2275343977168f},
  { -81.8928576580486f,    70.1987890594285f},
  { -52.7287329280945f,    -56.3022706963255f},
  { 31.4124111180474f,    -48.2229928271857f},
  { -110.168324847755f,    -65.3057294890112f},
  { -38.2752995881244f,    2.48892866204830f},
  { 153.741765167454f,    17.2546240003647f},
  { 77.5612967268595f,    -2.51551390374237f},
  { 73.9745809293406f,    -59.4902078684334f},
  { 139.853268213483f,    60.5444502114741f},
  { 52.3018793707221f,    -0.612319995656311f},
  { 104.868465242795f,    -7.94463278728400f},
  { -20.4500096238093f,    -18.9832376682744f},
  { 156.377371025056f,    -11.4998359060708f},
  { 8.49295339722419f,    -21.9185004527296f},
  { -118.699666417253f,    -38.5193796088501f},
  { 4.17382565018951f,    -77.8535052023763f},
  { -64.3603491270454f,    -12.4475080992171f},
  { -23.1830183065834f,    28.3923505799128f},
  { -87.1869876850574f,    39.9769842396629f},
  { -147.719978740629f,    -23.7645704686412f},
  { 125.626726160384f,    34.4685679972752f},
  { 120.338325711327f,    -50.4655496373278f},
  { -122.985390724831f,    45.9420478447699f},
  { -159.150486753489f,    64.1197068530904f},
  { 66.0047379990708f,    -30.3501473658748f},
  { -49.9183749429779f,    46.0841413779624f},
  { -7.33443273546967f,    58.8656838717408f},
  { 90.2580414669586f,    50.9663147502721f},
  { 161.081354523080f,    -42.6160278440355f},
  { -179.095147601993f,    8.86480300626410f},
  { -82.4199788295689f,    -37.2869473915237f},
  { -80.5521364174424f,    10.1298938179130f},
  { 99.9009211590103f,    20.5107431500932f},
  { 65.4145914700861f,    81.5891900266292f},
  { -57.6166358783556f,    19.3264393875578f},
  { -44.1773378357670f,    -29.4775326438906f},
  { 45.5879599273791f,    52.5728884078189f},
  { -107.796279575913f,    16.5114977400801f},
  { 166.323918348527f,    38.5657891902557f},
  { -162.393427873929f,    34.5545116665448f},
  { -123.621373877426f,    -8.49868297517550f},
  { 9.58844870151433f,    34.0440062710833f},
  { -6.64401859233703f,    6.48244449411013f},
  { 20.1050890311406f,    5.30879774656416f},
  { -8.91006667207943f,    -47.9244818159226f},
  { 40.8043352958318f,    24.5266043361652f},
  { 71.2014652009074f,    26.4723690084294f},
  { 37.5923338963284f,    -19.7859515392527f},
  { -154.326182118487f,    2.97193208334358f},
  { -136.300929883672f,    19.2399227604930f},
  { -96.9845659817945f,    -13.6982749659877f},
  { -160.462560104338f,    -50.2495445485635f}};

/* Sphere Covering (58 points) */
const float __SphCovering_58_dirs_deg[58][2] =
{ { -96.0219968859747f,    47.0398349802406f},
  { 90.9112133534077f,    -40.3161752543804f},
  { 82.3684126280071f,    15.3461015847840f},
  { -25.9091514958158f,    -0.125214196547890f},
  { 128.342546109304f,    50.6872843040434f},
  { 56.5182757850898f,    -1.28737886987945f},
  { 55.1752627133032f,    26.3577774494033f},
  { 141.297121857212f,    -5.37560462547641f},
  { 29.7892216844418f,    5.28129577239788f},
  { -9.87721943026026f,    -72.7599104036632f},
  { -81.8298323005842f,    -10.5235158231678f},
  { -37.0480239909542f,    -50.1641738370890f},
  { 45.8555312177052f,    -52.6003903819852f},
  { 86.4650608631926f,    -10.1906273441968f},
  { -163.590909665753f,    0.573015090910336f},
  { 169.836149632679f,    -18.2171930961845f},
  { -112.116381351200f,    23.5078853764225f},
  { -49.6926295716963f,    54.4401578621503f},
  { 115.531209810179f,    -24.6457795575524f},
  { 40.2222101759789f,    56.7004763639414f},
  { -7.31036850807417f,    43.0044932291342f},
  { 130.307791346603f,    -54.4401578621503f},
  { -165.114977400801f,    -75.1262260975535f},
  { 108.415073994654f,    74.8798542456473f},
  { -0.689669297998972f,    13.1654242165161f},
  { -162.347591250319f,    -27.2258085090265f},
  { 8.26949985712317f,    -43.4565569294924f},
  { -109.967789619459f,    -4.23702289499244f},
  { -106.999868240681f,    -31.7109221293105f},
  { -51.0224646141949f,    -20.4110484937404f},
  { 108.598420489096f,    28.1706159131972f},
  { -55.2932920191001f,    4.78316626531114f},
  { 81.9788013273182f,    -69.6258312642976f},
  { 112.328375735398f,    1.58892655745680f},
  { -84.4711677361373f,    15.7374317588583f},
  { -176.173062846826f,    59.8970079029763f},
  { 163.430481483116f,    36.4922549296773f},
  { -114.379564641966f,    71.4134595851058f},
  { 23.8533789268864f,    32.0529779330036f},
  { -165.481670389684f,    28.1282170363575f},
  { 138.890699117663f,    20.0563876185545f},
  { -20.3228129932903f,    -26.3996033684478f},
  { 4.62451425183941f,    -12.1495700457491f},
  { -64.0624110735774f,    30.3535851126456f},
  { 168.741800243979f,    8.15147055132622f},
  { 148.063753417707f,    -31.4897604203900f},
  { -138.036992002918f,    11.6407835236729f},
  { 80.5234885276859f,    46.7734096055048f},
  { -74.3126260284678f,    -38.4758348164202f},
  { -92.4295515105044f,    -62.9566025289749f},
  { 31.5671097227327f,    -22.3390514743557f},
  { -139.005290676689f,    42.6962419353538f},
  { -31.0233727751536f,    25.8272185311121f},
  { 179.249846206678f,    -50.0747924210486f},
  { -134.473194517204f,    -46.7980467906954f},
  { 61.7362024253462f,    -28.1700429554021f},
  { -135.579003061807f,    -15.7448802101950f},
  { -3.80472623856623f,    72.6453188446371f}};

/* Sphere Covering (59 points) */
const float __SphCovering_59_dirs_deg[59][2] =
{ { 87.7828637919934f,    -7.58366937635158f},
  { 108.054110583722f,    4.74827313558767f},
  { -99.3737999874900f,    13.4232552243249f},
  { -103.401693287260f,    -13.3384574706456f},
  { 179.605080039659f,    36.7770149538573f},
  { 92.0284810539128f,    -34.0451521866735f},
  { 32.5938500916071f,    -42.6933771463782f},
  { 13.3923155033879f,    5.88943317614973f},
  { 165.882740846276f,    -46.5694366304382f},
  { -5.10534043351320f,    24.0097964049571f},
  { 118.384539629931f,    -22.1327866681086f},
  { -179.192550427165f,    10.5922707585835f},
  { -87.6224356093568f,    -68.3080283354967f},
  { 178.883153217794f,    -19.7704816787842f},
  { 133.012152139621f,    80.0364744018247f},
  { 146.889189937689f,    55.3551714609742f},
  { -68.2965691795941f,    17.2569158315453f},
  { 61.6617179119792f,    -27.0166789138037f},
  { 24.2223637469507f,    36.8102465059749f},
  { 8.31590943852877f,    -19.4192585503690f},
  { -9.59016757489972f,    49.9057698714850f},
  { 87.2041764189113f,    23.3084960637170f},
  { 9.66751687724238f,    -72.9833639437643f},
  { 119.645046779219f,    32.6563024912764f},
  { -16.3791444894048f,    -0.596792839408265f},
  { -32.8940799762557f,    72.3932174147795f},
  { -149.541984529145f,    26.4580450635512f},
  { 44.8093102837963f,    61.2434587215337f},
  { -156.153917484955f,    -1.24515188037831f},
  { -47.0667639966117f,    -51.5404184609932f},
  { -54.6194936520262f,    -24.8382933767163f},
  { 61.0601122270918f,    33.5512625672708f},
  { -126.784100906549f,    8.93183906829440f},
  { 122.641616047753f,    -52.7293058858897f},
  { -45.1909001753534f,    0.552457365221042f},
  { 94.0109150250655f,    52.8410326559402f},
  { 144.975510901952f,    -26.5159138008594f},
  { -76.4898656499649f,    -7.36938316097265f},
  { -131.241712552666f,    -17.4333868324456f},
  { -4.95362120936256f,    -46.6851741050546f},
  { -156.451855538423f,    -32.0208922964763f},
  { 132.851723956984f,    3.49395393048727f},
  { -35.9376317839906f,    24.9557497247181f},
  { 68.8466086629197f,    -58.0234359128985f},
  { 143.279555828365f,    -77.5326488371030f},
  { -24.4887891216865f,    -24.6005158917370f},
  { -85.2274720257100f,    -37.3608589470956f},
  { 39.0401982446240f,    12.7740940424417f},
  { 37.8180792676100f,    -15.2744818603926f},
  { 64.3259716593375f,    2.54227103277498f},
  { 158.474396555234f,    -1.68352188943290f},
  { -118.080871998511f,    -42.6515512273336f},
  { -101.270290289373f,    65.9302534857038f},
  { -53.0811019721000f,    45.2762708868279f},
  { -153.558418673012f,    -59.2667543283324f},
  { -122.635886469801f,    37.0010414517534f},
  { -88.1438272029258f,    40.2371070786523f},
  { 150.258181773058f,    25.3562472235146f},
  { -154.790277932543f,    57.9489513995315f}};

/* Sphere Covering (60 points) */
const float __SphCovering_60_dirs_deg[60][2] =
{ { 165.665016884126f,    9.90071069986063f},
  { 107.733254218449f,    -51.8400753878466f},
  { 155.649714625239f,    67.2480564145047f},
  { -172.357163931254f,    -38.5434438362456f},
  { -123.827638683674f,    65.1682196181798f},
  { -50.2827761006811f,    -44.4758488470302f},
  { 166.100464808426f,    -17.9318601142094f},
  { 20.8866034636990f,    -52.2680748608094f},
  { 65.1109238386668f,    -17.2145169547056f},
  { -145.038536259417f,    -24.1834026168818f},
  { -14.2534710694695f,    17.7106984052889f},
  { 161.900684170117f,    -63.8103096437198f},
  { 58.4646134151492f,    8.54165480981031f},
  { -88.3672807430269f,    -54.3301499654852f},
  { 11.3755040645274f,    5.55345072508502f},
  { 110.844415046009f,    1.88938562522340f},
  { -87.1984468409600f,    16.2307484204660f},
  { -95.3688249995255f,    -8.32049310088982f},
  { -145.141668662540f,    7.09722820828551f},
  { -76.1747388626429f,    -26.0346292529495f},
  { 8.43508465991598f,    -22.2536807628812f},
  { -141.973212055467f,    -51.0631446176492f},
  { 45.4894111866166f,    53.0301087283333f},
  { 121.060252533192f,    -24.3604465755772f},
  { 35.1102807278217f,    -3.63043247728744f},
  { 88.8428357129855f,    57.4332893839137f},
  { -167.126059261710f,    19.4742624987016f},
  { -66.2625690068797f,    60.9970868696274f},
  { -14.8178344976734f,    -36.5260594395900f},
  { -115.387970361397f,    17.3394217540441f},
  { -110.890251669620f,    -34.3614248895857f},
  { -168.220408650410f,    47.5234113593310f},
  { 73.6594541420187f,    -72.6968850461988f},
  { 68.5830480771596f,    -47.3475133162258f},
  { 7.93890320933269f,    35.0753303023187f},
  { -43.4439518579995f,    -17.9788426534101f},
  { 35.6740711982305f,    26.0564016491644f},
  { 70.9321750371959f,    33.0825830908537f},
  { 134.656541011646f,    17.1789935714075f},
  { -5.03841896304192f,    59.4959374463847f},
  { 102.628200263833f,    27.7025093945753f},
  { 147.788733676045f,    -39.2424523463052f},
  { 86.3561988821177f,    5.29464568902442f},
  { -63.0081687305366f,    30.9786820671334f},
  { -98.3367463783032f,    43.9756566918809f},
  { -64.3660787049967f,    -0.501498498922107f},
  { 91.9769148523511f,    -23.7256093385723f},
  { -119.020522782526f,    -77.8191277346684f},
  { -16.2582503946322f,    -9.32717994693467f},
  { -121.931148381790f,    -8.44253311125268f},
  { 38.7090286390384f,    -29.2535061459944f},
  { 141.400254260336f,    -7.35391330050412f},
  { -32.6826585498524f,    38.9130016141050f},
  { 35.6780819027964f,    84.4425198463807f},
  { -136.278011571866f,    35.4901517459935f},
  { -168.197490338604f,    -7.48798542456473f},
  { -41.5812660660292f,    9.18909711830814f},
  { 129.196253224049f,    44.6334122406911f},
  { 161.327726374986f,    35.0128779026495f},
  { -23.4689242463536f,    -66.8985521594749f}};

/* Sphere Covering (61 points) */
const float __SphCovering_61_dirs_deg[61][2] =
{ { -154.755900464835f,    -2.67044169154574f},
  { -127.918557340908f,    -13.9337606197865f},
  { -89.3986047742623f,    47.3916310664509f},
  { 30.6847547182312f,    30.1839896052869f},
  { 56.8448617283144f,    -40.7304237402600f},
  { -27.3959769741803f,    -14.3818136155788f},
  { 137.028586283488f,    -58.6365007536885f},
  { 28.2181714101930f,    -1.63344537813846f},
  { 75.8710712312236f,    3.08423181118922f},
  { 68.2220846662271f,    34.2273527655251f},
  { -101.413529738156f,    -6.26586644755068f},
  { 177.542431977188f,    -51.2934736512918f},
  { 159.511450164421f,    -7.08405017899750f},
  { -127.557593929975f,    12.0601886297087f},
  { 7.35219442711872f,    17.3692155593909f},
  { 150.705088853260f,    18.7832753977738f},
  { -48.9890373992757f,    54.4241150438866f},
  { -6.69386592051341f,    42.1290137181743f},
  { 94.9964024326905f,    20.7754496514437f},
  { 21.0940141855364f,    -54.3015020757286f},
  { 0.778592347803276f,    -11.1572071445825f},
  { 176.625699504979f,    32.0993875144092f},
  { -128.835289813117f,    42.4407027587255f},
  { 22.9463867371943f,    -27.0487645503310f},
  { -166.862498675950f,    54.9747274850074f},
  { 85.3879002083466f,    -21.0900034809705f},
  { 73.3672456665019f,    63.3748617194204f},
  { 144.064508007694f,    48.1112660571352f},
  { 32.1102737125167f,    50.6540527519258f},
  { -46.7894524237684f,    -40.3568552578347f},
  { -111.394454529335f,    -34.7246801316987f},
  { -70.2388961050876f,    26.1635447568539f},
  { -57.1634262624071f,    -16.3023681448573f},
  { -11.4803553410363f,    -37.6387434777340f},
  { 106.209186483401f,    46.5310484581644f},
  { -133.367385972602f,    -56.2650284396420f},
  { -47.6649319347283f,    6.80330085938340f},
  { -177.187198144207f,    -23.3314143755223f},
  { -18.7626489171491f,    12.1787908933008f},
  { -178.928989841405f,    6.49218477662736f},
  { 124.687075376370f,    21.4968035155134f},
  { 92.5727909592871f,    -48.5048880623901f},
  { -7.71430375364140f,    72.4562427722439f},
  { 65.4432393598426f,    -72.3015441675586f},
  { -103.201158058964f,    21.5976440874564f},
  { 140.855944354962f,    75.0918486298457f},
  { -113.846713892495f,    72.1010089392628f},
  { -37.7327085561355f,    33.4464112907618f},
  { 149.839922582613f,    -33.2412924001050f},
  { -151.083240998047f,    -80.8787223606670f},
  { 53.8311037259262f,    -17.2380082243059f},
  { -80.9875843417419f,    -28.3975072000690f},
  { -152.492717174069f,    25.2095700279611f},
  { -79.8588574853341f,    -56.8482994750852f},
  { -149.880029628272f,    -32.2913283757781f},
  { -78.9134771233683f,    2.85436114378274f},
  { -30.5953733021908f,    -66.3886197218085f},
  { 51.2831604109795f,    12.1879582180229f},
  { 117.765745211189f,    -30.2470149627513f},
  { 106.982679506827f,    -3.89800376761353f},
  { 132.410546454733f,    -6.75173465782162f}};

/* Sphere Covering (62 points) */
const float __SphCovering_62_dirs_deg[62][2] =
{ { 64.7499604277343f,    15.2338018569383f},
  { -68.5028339858412f,    -53.4053960841440f},
  { 5.27688399737537f,    10.4198104622492f},
  { 165.338430940902f,    -5.52680818761143f},
  { 55.9871439090035f,    40.6015082363555f},
  { -11.5989576046284f,    86.3905763498255f},
  { 148.310125269614f,    17.3841124620643f},
  { -34.7756733754653f,    -70.7430989648027f},
  { 157.609230284587f,    -30.0888786112952f},
  { -91.0945598478496f,    40.1494445359973f},
  { 177.817451718851f,    20.6453882319490f},
  { 0.660219267329248f,    -48.4200903087107f},
  { 22.0393945475022f,    -7.63867332468413f},
  { 32.5434298056356f,    61.1174080066049f},
  { -17.9267034940532f,    55.0572334075062f},
  { 28.8936886506523f,    -33.5730349634857f},
  { 125.873098012291f,    42.6824909482707f},
  { 77.0800121789496f,    -11.4064437854644f},
  { -17.0924769443427f,    30.4968245614283f},
  { -87.8344299935552f,    -10.5785197715004f},
  { -173.067631597216f,    -53.8608975312730f},
  { 90.7164077030632f,    35.1675765073348f},
  { 136.106124233327f,    -7.67706149695790f},
  { -151.982784736402f,    19.6856839251048f},
  { -159.305185358174f,    44.7720680271128f},
  { 59.4672895566281f,    -36.2132244834486f},
  { 93.8275685306236f,    -56.6660988962335f},
  { -36.1553557461403f,    -19.2141396597122f},
  { -122.893717477610f,    35.7124593705042f},
  { 93.4780642755938f,    -31.2892251920943f},
  { 49.4628734958488f,    -9.16102218634673f},
  { 160.267754453994f,    66.6579098855200f},
  { -67.9298761907104f,    -30.0075186043866f},
  { -101.253101555519f,    -37.7900043356486f},
  { 151.060322686242f,    -83.3768183474374f},
  { -2.72263814668216f,    -20.4992839941906f},
  { -52.2262489417648f,    35.8860655824289f},
  { 108.861981074856f,    -7.74123277001255f},
  { 44.3973536290972f,    -63.0196278864393f},
  { -31.1356725029992f,    -39.1759892420700f},
  { -167.091681794002f,    -4.19164463761808f},
  { 36.5025681699896f,    17.2718127342187f},
  { -116.293243677703f,    -14.2173747283762f},
  { -146.195911005581f,    -12.1197762404023f},
  { 143.858243201447f,    -55.7705658624441f},
  { -129.860884266401f,    9.25957092710923f},
  { -41.9622829997912f,    11.9708072136683f},
  { 92.5154951797740f,    10.5412775148169f},
  { 119.885689053173f,    17.9330060297996f},
  { 94.1369657399943f,    61.1345967404588f},
  { -18.4979424157986f,    0.890204526294760f},
  { 161.058436211274f,    41.4271404191390f},
  { -60.1949459564443f,    -4.55621768265982f},
  { -128.697779942286f,    62.5498024944320f},
  { -74.1579274237825f,    17.1881608961296f},
  { 125.718399407605f,    -33.7340361039175f},
  { -68.8924452865302f,    61.9195489197881f},
  { -137.372360960566f,    -37.1683451279316f},
  { 17.1354487789775f,    35.6345371103664f},
  { -101.751574837283f,    12.3735965436453f},
  { -171.686803310951f,    -28.5539246781397f},
  { -122.687452671363f,    -63.6842589287910f}};

/* Sphere Covering (63 points) */
const float __SphCovering_63_dirs_deg[63][2] =
{ { 155.288751214307f,    26.4070518197845f},
  { 63.2774588942481f,    -25.4444827239647f},
  { -34.9779274771465f,    1.93149802316552f},
  { 16.8277704429923f,    33.6228822916621f},
  { -18.8457277974430f,    49.6187180161244f},
  { -11.6098438027359f,    23.5852346787652f},
  { 53.7325549851637f,    44.2936482681786f},
  { -52.9424461856783f,    51.1748713876997f},
  { -43.8633569640353f,    -48.7999613268825f},
  { -145.084372883027f,    35.1790356632374f},
  { -24.0246933076306f,    -26.8522400266012f},
  { 74.9657979149169f,    -48.6996937127346f},
  { 36.8549372139951f,    -63.9993857161130f},
  { 34.4330446139771f,    -37.2835096447529f},
  { -127.196630519043f,    10.0044160607793f},
  { 166.576019778384f,    71.1613581552483f},
  { 19.1247582436717f,    61.1804333640693f},
  { 125.237114859695f,    24.4148775661146f},
  { 115.508291498374f,    -74.9486091810630f},
  { -95.9303236387537f,    11.3256567363510f},
  { -10.2725603089005f,    -3.35243335508996f},
  { 13.7727594793547f,    6.12721066112902f},
  { -151.896841067133f,    6.51338421504720f},
  { -41.9943686363186f,    25.4313046946767f},
  { -135.401386145316f,    59.9313853706841f},
  { -95.8329208135815f,    -60.0860839753694f},
  { 168.197490338604f,    2.37210256762112f},
  { 98.4341492034754f,    16.4587856229280f},
  { -42.6744695391388f,    -76.7133191900659f},
  { 115.227542178760f,    -4.62462884339844f},
  { -170.426296161663f,    -13.8180231451701f},
  { 83.8695620512499f,    -4.62955628043657f},
  { -135.796727023956f,    -43.9223716169338f},
  { 71.0410370182708f,    24.0653733110848f},
  { 96.6121434149594f,    43.9487276755098f},
  { 159.746362860425f,    -22.6610537552192f},
  { -49.8100859196981f,    79.6984293026975f},
  { 158.153540189961f,    -49.0898779712187f},
  { -140.918969712426f,    -18.3999666328313f},
  { 97.1621828982850f,    -25.9481126258847f},
  { 58.8599542937895f,    -0.0703191101964059f},
  { -75.4871895084860f,    29.3761191141524f},
  { 34.0050451410144f,    -11.5216083022857f},
  { -172.809800589408f,    17.9421733545217f},
  { -51.8910686316133f,    -18.9127638594733f},
  { -169.182977746229f,    -37.3035631675825f},
  { -64.5780730891951f,    6.15127488852452f},
  { -108.501017663924f,    -31.0205079861779f},
  { -75.7564796721975f,    -36.9941659582119f},
  { -5.04827383711817f,    -49.5362120936256f},
  { 140.254338670074f,    0.632488110044916f},
  { 8.92782836372849f,    -24.0458927460504f},
  { -179.542054682195f,    43.9831051432177f},
  { 38.8746134418312f,    18.3335035285961f},
  { -111.904386967001f,    33.1255549254886f},
  { 136.346766507282f,    49.2898402417193f},
  { 84.8665086147775f,    67.9986311261261f},
  { 116.608370465025f,    -49.3557303881594f},
  { 131.150039305445f,    -26.0615582693206f},
  { -161.843388390604f,    -67.6777747608528f},
  { -95.8902165930946f,    53.7468789300420f},
  { -82.7694830845987f,    -11.8184004401635f},
  { -113.611801196491f,    -9.61079405552443f}};

/* Sphere Covering (64 points) */
const float __SphCovering_64_dirs_deg[64][2] =
{ { 102.009405845092f,    -77.6930770197396f},
  { 113.130516648581f,    11.4815012566266f},
  { 73.1323329704983f,    55.0515038295549f},
  { 5.30450056310067f,    4.57730252952063f},
  { 82.8382380200144f,    14.7708519584726f},
  { -27.6830288295409f,    34.1849538886854f},
  { 94.6297094438068f,    35.8728875531408f},
  { -124.893340182617f,    -28.4502193172210f},
  { 29.0151557032200f,    49.8874352220408f},
  { 160.703202378293f,    43.2846695909532f},
  { -163.000763136768f,    14.9301342255190f},
  { -13.0491137841045f,    -16.8678774886514f},
  { -67.4199437530440f,    -9.13065542320480f},
  { 176.786127687616f,    -33.7454952598201f},
  { 127.082038960017f,    36.3507343542800f},
  { -104.266859557907f,    -49.6061129446315f},
  { 72.9375273201538f,    -12.3798990793917f},
  { -148.951838000160f,    -51.5879739579891f},
  { -55.7344695213508f,    46.7865876347928f},
  { 24.0321417589673f,    -46.7384591800018f},
  { -83.9726944543735f,    71.2931384481283f},
  { 144.350986905260f,    -34.1964130445881f},
  { 12.9706185661716f,    -24.3243502344840f},
  { -73.3958935562585f,    -34.7596305572017f},
  { 30.6985057053144f,    -5.05761304917880f},
  { 143.812406577837f,    15.9133298019635f},
  { 130.794805472464f,    -7.91483898193719f},
  { -119.215328432870f,    -75.3038430140441f},
  { 123.816179527771f,    61.1747037861180f},
  { 4.48104561993866f,    -71.1384398434430f},
  { -73.9344738836814f,    20.6946626023302f},
  { 63.5295603241057f,    -53.3423707266797f},
  { -135.252417118582f,    24.9408528220447f},
  { -107.870764089280f,    24.8291260519942f},
  { -166.449969063455f,    40.0245397366588f},
  { 57.9203035097749f,    30.1920110144187f},
  { 115.892173221112f,    -53.6236930040889f},
  { 48.1273088753989f,    79.5838377436714f},
  { 158.216565547426f,    -9.81533998838613f},
  { 47.1343730164372f,    -26.0266078438176f},
  { -54.8996700138452f,    -59.0031937425722f},
  { 112.311187001544f,    -27.9597674445890f},
  { -175.428217713155f,    -6.72022197908943f},
  { 169.452267909941f,    17.1830042759734f},
  { 165.561884481003f,    -59.0604895220853f},
  { -130.536974464656f,    50.8563068536070f},
  { 55.0686925634088f,    3.78684995535815f},
  { 81.0219618094497f,    -35.8614283972382f},
  { -14.4425471418627f,    60.3267262493244f},
  { -43.2560217011966f,    -30.2240966509461f},
  { -16.0262024876043f,    -43.7258470932039f},
  { -92.3951740427966f,    43.5820346866261f},
  { -173.205141468048f,    69.2190312297548f},
  { 99.1446168694376f,    -9.04700358511570f},
  { 29.9754329678593f,    21.9081872124173f},
  { -144.184829144672f,    -3.33461436766139f},
  { -20.4345397633408f,    11.4052978698742f},
  { -155.134052609622f,    -25.1104483294035f},
  { -92.2003683924521f,    3.70720882183497f},
  { -39.6687329458825f,    -6.36326927272292f},
  { -120.676370810454f,    -1.14614477337970f},
  { -97.3226110809217f,    -19.7750653411452f},
  { -51.8326269365099f,    16.5172273180314f},
  { 2.49465823999960f,    32.1039711767703f}};

/* sphere covering handles */
const float* __HANDLES_SphCovering_dirs_deg[64] =
{ NULL, /* sphere covering not defined for 1 point */
  NULL, /* sphere covering not defined for 2 points */
  NULL, /* sphere covering not defined for 3 points */
  &__SphCovering_4_dirs_deg[0][0],
  &__SphCovering_5_dirs_deg[0][0],
  &__SphCovering_6_dirs_deg[0][0],
  &__SphCovering_7_dirs_deg[0][0],
  &__SphCovering_8_dirs_deg[0][0],
  &__SphCovering_9_dirs_deg[0][0],
  &__SphCovering_10_dirs_deg[0][0],
  &__SphCovering_11_dirs_deg[0][0],
  &__SphCovering_12_dirs_deg[0][0],
  &__SphCovering_13_dirs_deg[0][0],
  &__SphCovering_14_dirs_deg[0][0],
  &__SphCovering_15_dirs_deg[0][0],
  &__SphCovering_16_dirs_deg[0][0],
  &__SphCovering_17_dirs_deg[0][0],
  &__SphCovering_18_dirs_deg[0][0],
  &__SphCovering_19_dirs_deg[0][0],
  &__SphCovering_20_dirs_deg[0][0],
  &__SphCovering_21_dirs_deg[0][0],
  &__SphCovering_22_dirs_deg[0][0],
  &__SphCovering_23_dirs_deg[0][0],
  &__SphCovering_24_dirs_deg[0][0],
  &__SphCovering_25_dirs_deg[0][0],
  &__SphCovering_26_dirs_deg[0][0],
  &__SphCovering_27_dirs_deg[0][0],
  &__SphCovering_28_dirs_deg[0][0],
  &__SphCovering_29_dirs_deg[0][0],
  &__SphCovering_30_dirs_deg[0][0],
  &__SphCovering_31_dirs_deg[0][0],
  &__SphCovering_32_dirs_deg[0][0],
  &__SphCovering_33_dirs_deg[0][0],
  &__SphCovering_34_dirs_deg[0][0],
  &__SphCovering_35_dirs_deg[0][0],
  &__SphCovering_36_dirs_deg[0][0],
  &__SphCovering_37_dirs_deg[0][0],
  &__SphCovering_38_dirs_deg[0][0],
  &__SphCovering_39_dirs_deg[0][0],
  &__SphCovering_40_dirs_deg[0][0],
  &__SphCovering_41_dirs_deg[0][0],
  &__SphCovering_42_dirs_deg[0][0],
  &__SphCovering_43_dirs_deg[0][0],
  &__SphCovering_44_dirs_deg[0][0],
  &__SphCovering_45_dirs_deg[0][0],
  &__SphCovering_46_dirs_deg[0][0],
  &__SphCovering_47_dirs_deg[0][0],
  &__SphCovering_48_dirs_deg[0][0],
  &__SphCovering_49_dirs_deg[0][0],
  &__SphCovering_50_dirs_deg[0][0],
  &__SphCovering_51_dirs_deg[0][0],
  &__SphCovering_52_dirs_deg[0][0],
  &__SphCovering_53_dirs_deg[0][0],
  &__SphCovering_54_dirs_deg[0][0],
  &__SphCovering_55_dirs_deg[0][0],
  &__SphCovering_56_dirs_deg[0][0],
  &__SphCovering_57_dirs_deg[0][0],
  &__SphCovering_58_dirs_deg[0][0],
  &__SphCovering_59_dirs_deg[0][0],
  &__SphCovering_60_dirs_deg[0][0],
  &__SphCovering_61_dirs_deg[0][0],
  &__SphCovering_62_dirs_deg[0][0],
  &__SphCovering_63_dirs_deg[0][0],
  &__SphCovering_64_dirs_deg[0][0]};

/* 3LD geosphere 'ico' [0 0] (12 points) */
const float __geosphere_ico_0_0_dirs_deg[12][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f}};

/* 3LD geosphere 'ico' [1 0] (32 points) */
const float __geosphere_ico_1_0_dirs_deg[32][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 90.0f,    69.0948425521107f},
  { -90.0f,    69.0948425521107f},
  { 90.0f,    -69.0948425521107f},
  { -90.0f,    -69.0948425521107f},
  { 0.0f,    20.9051574478893f},
  { 0.0f,    -20.9051574478893f},
  { 180.0f,    20.9051574478893f},
  { 180.0f,    -20.9051574478893f},
  { 69.0948425521107f,    0.0f},
  { 110.905157447889f,    0.0f},
  { -69.0948425521107f,    0.0f},
  { -110.905157447889f,    0.0f},
  { 45.0000000000000f,    35.2643896827547f},
  { 45.0000000000000f,    -35.2643896827547f},
  { -45.0000000000000f,    35.2643896827547f},
  { -45.0000000000000f,    -35.2643896827547f},
  { 135.0f,    35.2643896827547f},
  { 135.0f,    -35.2643896827547f},
  { -135.0f,    35.2643896827547f},
  { -135.0f,    -35.2643896827547f}};

/* 3LD geosphere 'ico' [2 0] (42 points) */
const float __geosphere_ico_2_0_dirs_deg[42][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    90.0f},
  { 58.2825255885390f,    54.0f},
  { 121.717474411461f,    54.0f},
  { -58.2825255885390f,    54.0f},
  { -121.717474411461f,    54.0f},
  { 58.2825255885390f,    -54.0f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -54.0f},
  { -58.2825255885390f,    -54.0f},
  { -121.717474411461f,    -54.0f},
  { 0.0f,    0.0f},
  { 20.9051574478893f,    30.0000000000000f},
  { -20.9051574478893f,    30.0000000000000f},
  { 20.9051574478893f,    -30.0000000000000f},
  { -20.9051574478893f,    -30.0000000000000f},
  { 180.0f,    0.0f},
  { -159.094842552111f,    30.0000000000000f},
  { 159.094842552111f,    30.0000000000000f},
  { -159.094842552111f,    -30.0000000000000f},
  { 159.094842552111f,    -30.0000000000000f},
  { 90.0f,    0.0f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    -18.0f},
  { 121.717474411461f,    18.0f},
  { 121.717474411461f,    -18.0f},
  { -58.2825255885390f,    18.0f},
  { -90.0f,    0.0f},
  { -58.2825255885390f,    -18.0f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    -18.0f}};

/* 3LD geosphere 'ico' [3 0] (92 points) */
const float __geosphere_ico_3_0_dirs_deg[92][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    78.3592768632294f},
  { 180.0f,    78.3592768632294f},
  { 38.9734473368657f,    58.7297899739513f},
  { 90.0f,    69.0948425521107f},
  { 141.026552663134f,    58.7297899739513f},
  { 72.8279621494862f,    46.8885380691238f},
  { 107.172037850514f,    46.8885380691238f},
  { -38.9734473368657f,    58.7297899739513f},
  { -72.8279621494862f,    46.8885380691238f},
  { -90.0f,    69.0948425521107f},
  { -107.172037850514f,    46.8885380691238f},
  { -141.026552663134f,    58.7297899739513f},
  { 38.9734473368657f,    -58.7297899739513f},
  { 72.8279621494862f,    -46.8885380691238f},
  { 0.0f,    -78.3592768632294f},
  { 90.0f,    -69.0948425521107f},
  { 107.172037850514f,    -46.8885380691238f},
  { 180.0f,    -78.3592768632294f},
  { 141.026552663134f,    -58.7297899739513f},
  { -38.9734473368657f,    -58.7297899739513f},
  { -90.0f,    -69.0948425521107f},
  { -141.026552663134f,    -58.7297899739513f},
  { -72.8279621494862f,    -46.8885380691238f},
  { -107.172037850514f,    -46.8885380691238f},
  { 11.6407231367706f,    0.0f},
  { -11.6407231367706f,    0.0f},
  { 25.2736805860404f,    19.0551045548580f},
  { 0.0f,    20.9051574478893f},
  { -25.2736805860404f,    19.0551045548580f},
  { 15.4504370863258f,    40.7647476003615f},
  { -15.4504370863258f,    40.7647476003615f},
  { 25.2736805860404f,    -19.0551045548580f},
  { 15.4504370863258f,    -40.7647476003615f},
  { 0.0f,    -20.9051574478893f},
  { -15.4504370863258f,    -40.7647476003615f},
  { -25.2736805860404f,    -19.0551045548580f},
  { -168.359276863229f,    0.0f},
  { 168.359276863229f,    0.0f},
  { -154.726319413960f,    19.0551045548580f},
  { 180.0f,    20.9051574478893f},
  { 154.726319413960f,    19.0551045548580f},
  { -164.549562913674f,    40.7647476003615f},
  { 164.549562913674f,    40.7647476003615f},
  { -154.726319413960f,    -19.0551045548580f},
  { -164.549562913674f,    -40.7647476003615f},
  { 180.0f,    -20.9051574478893f},
  { 164.549562913674f,    -40.7647476003615f},
  { 154.726319413960f,    -19.0551045548580f},
  { 90.0f,    11.6407231367706f},
  { 90.0f,    -11.6407231367706f},
  { 69.0948425521107f,    23.8001826021751f},
  { 69.0948425521107f,    0.0f},
  { 69.0948425521107f,    -23.8001826021751f},
  { 48.1896851042214f,    11.6407231367706f},
  { 48.1896851042214f,    -11.6407231367706f},
  { 110.905157447889f,    23.8001826021751f},
  { 131.810314895779f,    11.6407231367706f},
  { 110.905157447889f,    0.0f},
  { 131.810314895779f,    -11.6407231367706f},
  { 110.905157447889f,    -23.8001826021751f},
  { -69.0948425521107f,    23.8001826021751f},
  { -48.1896851042214f,    11.6407231367706f},
  { -90.0f,    11.6407231367706f},
  { -69.0948425521107f,    0.0f},
  { -48.1896851042214f,    -11.6407231367706f},
  { -90.0f,    -11.6407231367706f},
  { -69.0948425521107f,    -23.8001826021751f},
  { -110.905157447889f,    23.8001826021751f},
  { -110.905157447889f,    0.0f},
  { -110.905157447889f,    -23.8001826021751f},
  { -131.810314895779f,    11.6407231367706f},
  { -131.810314895779f,    -11.6407231367706f},
  { 45.0000000000000f,    35.2643896827547f},
  { 45.0000000000000f,    -35.2643896827547f},
  { -45.0000000000000f,    35.2643896827547f},
  { -45.0000000000000f,    -35.2643896827547f},
  { 135.0f,    35.2643896827547f},
  { 135.0f,    -35.2643896827547f},
  { -135.0f,    35.2643896827547f},
  { -135.0f,    -35.2643896827547f}};

/* 3LD geosphere 'ico' [4 0] (162 points) */
const float __geosphere_ico_4_0_dirs_deg[162][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    72.8279621494862f},
  { 0.0f,    90.0f},
  { 180.0f,    72.8279621494862f},
  { 28.3399673707642f,    59.7900946927798f},
  { 58.2825255885390f,    72.0f},
  { 121.717474411461f,    72.0f},
  { 151.660032629236f,    59.7900946927798f},
  { 58.2825255885390f,    54.0f},
  { 90.0f,    58.2825255885390f},
  { 121.717474411461f,    54.0f},
  { 78.3592768632294f,    42.9780676586279f},
  { 101.640723136771f,    42.9780676586279f},
  { -28.3399673707642f,    59.7900946927798f},
  { -58.2825255885390f,    54.0f},
  { -78.3592768632294f,    42.9780676586279f},
  { -58.2825255885390f,    72.0f},
  { -90.0f,    58.2825255885390f},
  { -101.640723136771f,    42.9780676586279f},
  { -121.717474411461f,    72.0f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    59.7900946927798f},
  { 28.3399673707642f,    -59.7900946927798f},
  { 58.2825255885390f,    -54.0f},
  { 78.3592768632294f,    -42.9780676586279f},
  { 0.0f,    -72.8279621494862f},
  { 58.2825255885390f,    -72.0f},
  { 90.0f,    -58.2825255885390f},
  { 101.640723136771f,    -42.9780676586279f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -72.0f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -72.8279621494862f},
  { 151.660032629236f,    -59.7900946927798f},
  { -28.3399673707642f,    -59.7900946927798f},
  { -58.2825255885390f,    -72.0f},
  { -121.717474411461f,    -72.0f},
  { -151.660032629236f,    -59.7900946927798f},
  { -58.2825255885390f,    -54.0f},
  { -90.0f,    -58.2825255885390f},
  { -121.717474411461f,    -54.0f},
  { -78.3592768632294f,    -42.9780676586279f},
  { -101.640723136771f,    -42.9780676586279f},
  { 17.1720378505138f,    0.0f},
  { 0.0f,    0.0f},
  { -17.1720378505138f,    0.0f},
  { 27.1333780408531f,    13.8190092060518f},
  { 9.69372389531482f,    15.2401622826778f},
  { -9.69372389531481f,    15.2401622826778f},
  { -27.1333780408531f,    13.8190092060518f},
  { 20.9051574478893f,    30.0000000000000f},
  { 0.0f,    31.7174744114610f},
  { -20.9051574478893f,    30.0000000000000f},
  { 12.2183198662487f,    45.7717617455715f},
  { -12.2183198662487f,    45.7717617455715f},
  { 27.1333780408531f,    -13.8190092060518f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 12.2183198662487f,    -45.7717617455715f},
  { 9.69372389531481f,    -15.2401622826778f},
  { 0.0f,    -31.7174744114610f},
  { -12.2183198662487f,    -45.7717617455715f},
  { -9.69372389531481f,    -15.2401622826778f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -27.1333780408531f,    -13.8190092060518f},
  { -162.827962149486f,    0.0f},
  { 180.0f,    0.0f},
  { 162.827962149486f,    0.0f},
  { -152.866621959147f,    13.8190092060518f},
  { -170.306276104685f,    15.2401622826778f},
  { 170.306276104685f,    15.2401622826778f},
  { 152.866621959147f,    13.8190092060518f},
  { -159.094842552111f,    30.0000000000000f},
  { 180.0f,    31.7174744114610f},
  { 159.094842552111f,    30.0000000000000f},
  { -167.781680133751f,    45.7717617455715f},
  { 167.781680133751f,    45.7717617455715f},
  { -152.866621959147f,    -13.8190092060518f},
  { -159.094842552111f,    -30.0000000000000f},
  { -167.781680133751f,    -45.7717617455715f},
  { -170.306276104685f,    -15.2401622826778f},
  { 180.0f,    -31.7174744114610f},
  { 167.781680133751f,    -45.7717617455715f},
  { 170.306276104685f,    -15.2401622826778f},
  { 159.094842552111f,    -30.0000000000000f},
  { 152.866621959147f,    -13.8190092060518f},
  { 90.0f,    17.1720378505138f},
  { 90.0f,    0.0f},
  { 90.0f,    -17.1720378505138f},
  { 74.5495629136742f,    26.2866766550738f},
  { 74.5495629136742f,    9.34970354257458f},
  { 74.5495629136742f,    -9.34970354257458f},
  { 74.5495629136742f,    -26.2866766550738f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    0.0f},
  { 58.2825255885390f,    -18.0f},
  { 43.5723495188822f,    8.48907956344660f},
  { 43.5723495188822f,    -8.48907956344660f},
  { 105.450437086326f,    26.2866766550738f},
  { 121.717474411461f,    18.0f},
  { 136.427650481118f,    8.48907956344661f},
  { 105.450437086326f,    9.34970354257458f},
  { 121.717474411461f,    0.0f},
  { 136.427650481118f,    -8.48907956344661f},
  { 105.450437086326f,    -9.34970354257458f},
  { 121.717474411461f,    -18.0f},
  { 105.450437086326f,    -26.2866766550738f},
  { -74.5495629136742f,    26.2866766550738f},
  { -58.2825255885390f,    18.0f},
  { -43.5723495188822f,    8.48907956344660f},
  { -90.0f,    17.1720378505138f},
  { -74.5495629136742f,    9.34970354257458f},
  { -58.2825255885390f,    0.0f},
  { -43.5723495188822f,    -8.48907956344661f},
  { -90.0f,    0.0f},
  { -74.5495629136742f,    -9.34970354257458f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -17.1720378505138f},
  { -74.5495629136742f,    -26.2866766550738f},
  { -105.450437086326f,    26.2866766550738f},
  { -105.450437086326f,    9.34970354257458f},
  { -105.450437086326f,    -9.34970354257458f},
  { -105.450437086326f,    -26.2866766550738f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    0.0f},
  { -121.717474411461f,    -18.0f},
  { -136.427650481118f,    8.48907956344661f},
  { -136.427650481118f,    -8.48907956344661f},
  { 58.2825255885390f,    36.0f},
  { 40.5007050124200f,    25.1712621719368f},
  { 35.8897546595841f,    43.4870777455218f},
  { 58.2825255885390f,    -36.0f},
  { 35.8897546595841f,    -43.4870777455218f},
  { 40.5007050124200f,    -25.1712621719368f},
  { -35.8897546595841f,    43.4870777455218f},
  { -40.5007050124200f,    25.1712621719368f},
  { -58.2825255885390f,    36.0f},
  { -35.8897546595841f,    -43.4870777455218f},
  { -58.2825255885390f,    -36.0f},
  { -40.5007050124200f,    -25.1712621719368f},
  { 144.110245340416f,    43.4870777455218f},
  { 139.499294987580f,    25.1712621719368f},
  { 121.717474411461f,    36.0f},
  { 144.110245340416f,    -43.4870777455218f},
  { 121.717474411461f,    -36.0f},
  { 139.499294987580f,    -25.1712621719368f},
  { -144.110245340416f,    43.4870777455218f},
  { -121.717474411461f,    36.0f},
  { -139.499294987580f,    25.1712621719368f},
  { -144.110245340416f,    -43.4870777455218f},
  { -139.499294987580f,    -25.1712621719368f},
  { -121.717474411461f,    -36.0f}};

/* 3LD geosphere 'ico' [5 0] (252 points) */
const float __geosphere_ico_5_0_dirs_deg[252][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    69.6541923856616f},
  { 0.0f,    82.9535936191428f},
  { 180.0f,    82.9535936191428f},
  { 180.0f,    69.6541923856616f},
  { 22.0237505161462f,    59.9952713133382f},
  { 38.9734473368657f,    71.0021166134344f},
  { 90.0f,    77.7816801337513f},
  { 141.026552663134f,    71.0021166134344f},
  { 157.976249483854f,    59.9952713133382f},
  { 47.1679114977868f,    57.2262493118836f},
  { 72.8279621494862f,    63.7030502822517f},
  { 107.172037850514f,    63.7030502822517f},
  { 132.832088502213f,    57.2262493118836f},
  { 67.6072290710451f,    49.9068566415137f},
  { 90.0f,    52.1031151288303f},
  { 112.392770928955f,    49.9068566415137f},
  { 81.2167693990410f,    40.6247965576687f},
  { 98.7832306009590f,    40.6247965576687f},
  { -22.0237505161462f,    59.9952713133382f},
  { -47.1679114977868f,    57.2262493118836f},
  { -67.6072290710451f,    49.9068566415137f},
  { -81.2167693990410f,    40.6247965576687f},
  { -38.9734473368657f,    71.0021166134344f},
  { -72.8279621494862f,    63.7030502822517f},
  { -90.0f,    52.1031151288303f},
  { -98.7832306009590f,    40.6247965576687f},
  { -90.0000000000000f,    77.7816801337513f},
  { -107.172037850514f,    63.7030502822517f},
  { -112.392770928955f,    49.9068566415137f},
  { -141.026552663134f,    71.0021166134344f},
  { -132.832088502213f,    57.2262493118836f},
  { -157.976249483854f,    59.9952713133382f},
  { 22.0237505161462f,    -59.9952713133382f},
  { 47.1679114977868f,    -57.2262493118836f},
  { 67.6072290710451f,    -49.9068566415137f},
  { 81.2167693990410f,    -40.6247965576687f},
  { 0.0f,    -69.6541923856616f},
  { 38.9734473368657f,    -71.0021166134344f},
  { 72.8279621494862f,    -63.7030502822517f},
  { 90.0f,    -52.1031151288303f},
  { 98.7832306009590f,    -40.6247965576687f},
  { 0.0f,    -82.9535936191428f},
  { 90.0000000000000f,    -77.7816801337513f},
  { 107.172037850514f,    -63.7030502822517f},
  { 112.392770928955f,    -49.9068566415137f},
  { 180.0f,    -82.9535936191428f},
  { 141.026552663134f,    -71.0021166134344f},
  { 132.832088502213f,    -57.2262493118836f},
  { 180.0f,    -69.6541923856616f},
  { 157.976249483854f,    -59.9952713133382f},
  { -22.0237505161462f,    -59.9952713133382f},
  { -38.9734473368657f,    -71.0021166134344f},
  { -90.0f,    -77.7816801337513f},
  { -141.026552663134f,    -71.0021166134344f},
  { -157.976249483854f,    -59.9952713133382f},
  { -47.1679114977868f,    -57.2262493118836f},
  { -72.8279621494862f,    -63.7030502822517f},
  { -107.172037850514f,    -63.7030502822517f},
  { -132.832088502213f,    -57.2262493118836f},
  { -67.6072290710451f,    -49.9068566415137f},
  { -90.0f,    -52.1031151288303f},
  { -112.392770928955f,    -49.9068566415137f},
  { -81.2167693990410f,    -40.6247965576687f},
  { -98.7832306009590f,    -40.6247965576687f},
  { 20.3458076143384f,    0.0f},
  { 7.04640638085718f,    0.0f},
  { -7.04640638085718f,    0.0f},
  { -20.3458076143384f,    0.0f},
  { 28.1611303732939f,    10.8082208669889f},
  { 14.9845972116948f,    11.8147143864651f},
  { 0.0f,    12.2183198662487f},
  { -14.9845972116948f,    11.8147143864651f},
  { -28.1611303732939f,    10.8082208669889f},
  { 23.6387115659933f,    23.3894469911546f},
  { 8.30077479951012f,    25.0415008212998f},
  { -8.30077479951012f,    25.0415008212998f},
  { -23.6387115659933f,    23.3894469911546f},
  { 17.7818205761190f,    36.5460212963266f},
  { 0.0f,    37.8968848711697f},
  { -17.7818205761190f,    36.5460212963266f},
  { 10.0928404843176f,    48.5980887666628f},
  { -10.0928404843176f,    48.5980887666628f},
  { 28.1611303732939f,    -10.8082208669889f},
  { 23.6387115659933f,    -23.3894469911546f},
  { 17.7818205761190f,    -36.5460212963266f},
  { 10.0928404843176f,    -48.5980887666628f},
  { 14.9845972116948f,    -11.8147143864651f},
  { 8.30077479951012f,    -25.0415008212998f},
  { 0.0f,    -37.8968848711697f},
  { -10.0928404843176f,    -48.5980887666628f},
  { -1.01205574733119e-15f,    -12.2183198662487f},
  { -8.30077479951012f,    -25.0415008212998f},
  { -17.7818205761190f,    -36.5460212963266f},
  { -14.9845972116948f,    -11.8147143864651f},
  { -23.6387115659933f,    -23.3894469911546f},
  { -28.1611303732939f,    -10.8082208669889f},
  { -159.654192385662f,    0.0f},
  { -172.953593619143f,    0.0f},
  { 172.953593619143f,    0.0f},
  { 159.654192385662f,    0.0f},
  { -151.838869626706f,    10.8082208669889f},
  { -165.015402788305f,    11.8147143864651f},
  { 180.0f,    12.2183198662487f},
  { 165.015402788305f,    11.8147143864651f},
  { 151.838869626706f,    10.8082208669889f},
  { -156.361288434007f,    23.3894469911546f},
  { -171.699225200490f,    25.0415008212998f},
  { 171.699225200490f,    25.0415008212998f},
  { 156.361288434007f,    23.3894469911546f},
  { -162.218179423881f,    36.5460212963266f},
  { 180.0f,    37.8968848711697f},
  { 162.218179423881f,    36.5460212963266f},
  { -169.907159515682f,    48.5980887666628f},
  { 169.907159515682f,    48.5980887666628f},
  { -151.838869626706f,    -10.8082208669889f},
  { -156.361288434007f,    -23.3894469911546f},
  { -162.218179423881f,    -36.5460212963266f},
  { -169.907159515682f,    -48.5980887666628f},
  { -165.015402788305f,    -11.8147143864651f},
  { -171.699225200490f,    -25.0415008212998f},
  { 180.0f,    -37.8968848711697f},
  { 169.907159515682f,    -48.5980887666628f},
  { 180.0f,    -12.2183198662487f},
  { 171.699225200490f,    -25.0415008212998f},
  { 162.218179423881f,    -36.5460212963266f},
  { 165.015402788305f,    -11.8147143864651f},
  { 156.361288434007f,    -23.3894469911546f},
  { 151.838869626706f,    -10.8082208669889f},
  { 90.0f,    20.3458076143384f},
  { 90.0f,    7.04640638085718f},
  { 90.0f,    -7.04640638085718f},
  { 90.0f,    -20.3458076143384f},
  { 77.7816801337513f,    27.6183932347186f},
  { 77.7816801337513f,    14.6599491964913f},
  { 77.7816801337513f,    0.0f},
  { 77.7816801337513f,    -14.6599491964913f},
  { 77.7816801337513f,    -27.6183932347186f},
  { 64.7263194139596f,    21.5935345276613f},
  { 64.7263194139596f,    7.51576871353677f},
  { 64.7263194139596f,    -7.51576871353677f},
  { 64.7263194139596f,    -21.5935345276613f},
  { 52.1031151288303f,    14.2023070245089f},
  { 52.1031151288303f,    0.0f},
  { 52.1031151288303f,    -14.2023070245089f},
  { 40.9590912848605f,    6.65525720600019f},
  { 40.9590912848605f,    -6.65525720600019f},
  { 102.218319866249f,    27.6183932347186f},
  { 115.273680586040f,    21.5935345276613f},
  { 127.896884871170f,    14.2023070245089f},
  { 139.040908715140f,    6.65525720600019f},
  { 102.218319866249f,    14.6599491964913f},
  { 115.273680586040f,    7.51576871353677f},
  { 127.896884871170f,    0.0f},
  { 139.040908715140f,    -6.65525720600019f},
  { 102.218319866249f,    -9.89130938771355e-16f},
  { 115.273680586040f,    -7.51576871353677f},
  { 127.896884871170f,    -14.2023070245090f},
  { 102.218319866249f,    -14.6599491964913f},
  { 115.273680586040f,    -21.5935345276613f},
  { 102.218319866249f,    -27.6183932347186f},
  { -77.7816801337513f,    27.6183932347186f},
  { -64.7263194139596f,    21.5935345276613f},
  { -52.1031151288303f,    14.2023070245089f},
  { -40.9590912848605f,    6.65525720600019f},
  { -90.0f,    20.3458076143384f},
  { -77.7816801337513f,    14.6599491964913f},
  { -64.7263194139596f,    7.51576871353677f},
  { -52.1031151288303f,    0.0f},
  { -40.9590912848605f,    -6.65525720600019f},
  { -90.0f,    7.04640638085718f},
  { -77.7816801337513f,    -9.89130938771355e-16f},
  { -64.7263194139596f,    -7.51576871353677f},
  { -52.1031151288303f,    -14.2023070245090f},
  { -90.0f,    -7.04640638085718f},
  { -77.7816801337513f,    -14.6599491964913f},
  { -64.7263194139596f,    -21.5935345276613f},
  { -90.0f,    -20.3458076143384f},
  { -77.7816801337513f,    -27.6183932347186f},
  { -102.218319866249f,    27.6183932347186f},
  { -102.218319866249f,    14.6599491964913f},
  { -102.218319866249f,    0.0f},
  { -102.218319866249f,    -14.6599491964913f},
  { -102.218319866249f,    -27.6183932347186f},
  { -115.273680586040f,    21.5935345276613f},
  { -115.273680586040f,    7.51576871353677f},
  { -115.273680586040f,    -7.51576871353677f},
  { -115.273680586040f,    -21.5935345276613f},
  { -127.896884871170f,    14.2023070245089f},
  { -127.896884871170f,    0.0f},
  { -127.896884871170f,    -14.2023070245089f},
  { -139.040908715140f,    6.65525720600019f},
  { -139.040908715140f,    -6.65525720600019f},
  { 65.9051574478893f,    35.7584993532105f},
  { 51.0265526631344f,    28.2445194108839f},
  { 38.2682973779569f,    19.3469890055802f},
  { 49.4992949875800f,    43.2255973783311f},
  { 34.6438140225457f,    33.6469345870906f},
  { 29.5495629136742f,    47.7981724013467f},
  { 65.9051574478893f,    -35.7584993532105f},
  { 49.4992949875800f,    -43.2255973783311f},
  { 29.5495629136742f,    -47.7981724013467f},
  { 51.0265526631344f,    -28.2445194108839f},
  { 34.6438140225457f,    -33.6469345870906f},
  { 38.2682973779569f,    -19.3469890055802f},
  { -29.5495629136742f,    47.7981724013467f},
  { -34.6438140225457f,    33.6469345870906f},
  { -38.2682973779569f,    19.3469890055802f},
  { -49.4992949875800f,    43.2255973783311f},
  { -51.0265526631344f,    28.2445194108839f},
  { -65.9051574478893f,    35.7584993532105f},
  { -29.5495629136742f,    -47.7981724013467f},
  { -49.4992949875800f,    -43.2255973783311f},
  { -65.9051574478893f,    -35.7584993532105f},
  { -34.6438140225457f,    -33.6469345870906f},
  { -51.0265526631344f,    -28.2445194108839f},
  { -38.2682973779569f,    -19.3469890055802f},
  { 150.450437086326f,    47.7981724013467f},
  { 145.356185977454f,    33.6469345870906f},
  { 141.731702622043f,    19.3469890055802f},
  { 130.500705012420f,    43.2255973783311f},
  { 128.973447336866f,    28.2445194108839f},
  { 114.094842552111f,    35.7584993532105f},
  { 150.450437086326f,    -47.7981724013467f},
  { 130.500705012420f,    -43.2255973783311f},
  { 114.094842552111f,    -35.7584993532105f},
  { 145.356185977454f,    -33.6469345870906f},
  { 128.973447336866f,    -28.2445194108839f},
  { 141.731702622043f,    -19.3469890055802f},
  { -150.450437086326f,    47.7981724013467f},
  { -130.500705012420f,    43.2255973783311f},
  { -114.094842552111f,    35.7584993532105f},
  { -145.356185977454f,    33.6469345870906f},
  { -128.973447336866f,    28.2445194108839f},
  { -141.731702622043f,    19.3469890055802f},
  { -150.450437086326f,    -47.7981724013467f},
  { -145.356185977454f,    -33.6469345870906f},
  { -141.731702622043f,    -19.3469890055802f},
  { -130.500705012420f,    -43.2255973783311f},
  { -128.973447336866f,    -28.2445194108839f},
  { -114.094842552111f,    -35.7584993532105f}};

/* 3LD geosphere 'ico' [6 0] (362 points) */
const float __geosphere_ico_6_0_dirs_deg[362][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    67.6072290710451f},
  { 0.0f,    78.3592768632294f},
  { 0.0f,    90.0f},
  { 180.0f,    78.3592768632294f},
  { 180.0f,    67.6072290710451f},
  { 17.9319334355159f,    59.9665693791348f},
  { 28.3399673707642f,    69.4455573432933f},
  { 58.2825255885390f,    78.1814142633759f},
  { 121.717474411461f,    78.1814142633759f},
  { 151.660032629236f,    69.4455573432933f},
  { 162.068066564484f,    59.9665693791348f},
  { 38.9734473368657f,    58.7297899739513f},
  { 58.2825255885390f,    65.8185857366241f},
  { 90.0f,    69.0948425521107f},
  { 121.717474411461f,    65.8185857366241f},
  { 141.026552663134f,    58.7297899739513f},
  { 58.2825255885390f,    54.0f},
  { 78.3592768632294f,    57.7475579750497f},
  { 101.640723136771f,    57.7475579750497f},
  { 121.717474411461f,    54.0f},
  { 72.8279621494862f,    46.8885380691238f},
  { 90.0f,    48.1896851042214f},
  { 107.172037850514f,    46.8885380691238f},
  { 82.9535936191428f,    39.0716078459179f},
  { 97.0464063808572f,    39.0716078459179f},
  { -17.9319334355159f,    59.9665693791348f},
  { -38.9734473368657f,    58.7297899739513f},
  { -58.2825255885390f,    54.0f},
  { -72.8279621494862f,    46.8885380691238f},
  { -82.9535936191428f,    39.0716078459179f},
  { -28.3399673707642f,    69.4455573432933f},
  { -58.2825255885390f,    65.8185857366241f},
  { -78.3592768632294f,    57.7475579750497f},
  { -90.0f,    48.1896851042214f},
  { -97.0464063808572f,    39.0716078459179f},
  { -58.2825255885390f,    78.1814142633759f},
  { -90.0f,    69.0948425521107f},
  { -101.640723136771f,    57.7475579750497f},
  { -107.172037850514f,    46.8885380691238f},
  { -121.717474411461f,    78.1814142633759f},
  { -121.717474411461f,    65.8185857366241f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    69.4455573432933f},
  { -141.026552663134f,    58.7297899739513f},
  { -162.068066564484f,    59.9665693791348f},
  { 17.9319334355159f,    -59.9665693791348f},
  { 38.9734473368657f,    -58.7297899739513f},
  { 58.2825255885390f,    -54.0f},
  { 72.8279621494862f,    -46.8885380691238f},
  { 82.9535936191428f,    -39.0716078459179f},
  { 0.0f,    -67.6072290710451f},
  { 28.3399673707642f,    -69.4455573432933f},
  { 58.2825255885390f,    -65.8185857366241f},
  { 78.3592768632294f,    -57.7475579750497f},
  { 90.0f,    -48.1896851042214f},
  { 97.0464063808572f,    -39.0716078459179f},
  { 0.0f,    -78.3592768632294f},
  { 58.2825255885390f,    -78.1814142633759f},
  { 90.0f,    -69.0948425521107f},
  { 101.640723136771f,    -57.7475579750497f},
  { 107.172037850514f,    -46.8885380691238f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -78.1814142633759f},
  { 121.717474411461f,    -65.8185857366241f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -78.3592768632294f},
  { 151.660032629236f,    -69.4455573432933f},
  { 141.026552663134f,    -58.7297899739513f},
  { 180.0f,    -67.6072290710451f},
  { 162.068066564484f,    -59.9665693791348f},
  { -17.9319334355159f,    -59.9665693791348f},
  { -28.3399673707642f,    -69.4455573432933f},
  { -58.2825255885390f,    -78.1814142633759f},
  { -121.717474411461f,    -78.1814142633759f},
  { -151.660032629236f,    -69.4455573432933f},
  { -162.068066564484f,    -59.9665693791348f},
  { -38.9734473368657f,    -58.7297899739513f},
  { -58.2825255885390f,    -65.8185857366241f},
  { -90.0f,    -69.0948425521107f},
  { -121.717474411461f,    -65.8185857366241f},
  { -141.026552663134f,    -58.7297899739513f},
  { -58.2825255885390f,    -54.0f},
  { -78.3592768632294f,    -57.7475579750497f},
  { -101.640723136771f,    -57.7475579750497f},
  { -121.717474411461f,    -54.0f},
  { -72.8279621494862f,    -46.8885380691238f},
  { -90.0f,    -48.1896851042214f},
  { -107.172037850514f,    -46.8885380691238f},
  { -82.9535936191428f,    -39.0716078459179f},
  { -97.0464063808572f,    -39.0716078459179f},
  { 22.3927709289549f,    0.0f},
  { 11.6407231367706f,    0.0f},
  { 0.0f,    0.0f},
  { -11.6407231367706f,    0.0f},
  { -22.3927709289549f,    0.0f},
  { 28.8127609872844f,    8.86454598437525f},
  { 18.2642763775679f,    9.59406822686046f},
  { 6.27780483909150f,    10.0335510226076f},
  { -6.27780483909150f,    10.0335510226076f},
  { -18.2642763775679f,    9.59406822686046f},
  { -28.8127609872844f,    8.86454598437525f},
  { 25.2736805860404f,    19.0551045548580f},
  { 13.2825255885390f,    20.3925183693779f},
  { 0.0f,    20.9051574478893f},
  { -13.2825255885390f,    20.3925183693779f},
  { -25.2736805860404f,    19.0551045548580f},
  { 20.9051574478893f,    30.0000000000000f},
  { 7.25597292540465f,    31.5118220765682f},
  { -7.25597292540465f,    31.5118220765682f},
  { -20.9051574478893f,    30.0000000000000f},
  { 15.4504370863258f,    40.7647476003615f},
  { 0.0f,    41.8103148957786f},
  { -15.4504370863258f,    40.7647476003615f},
  { 8.59252164094535f,    50.3983682135674f},
  { -8.59252164094535f,    50.3983682135674f},
  { 28.8127609872844f,    -8.86454598437525f},
  { 25.2736805860404f,    -19.0551045548580f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 15.4504370863258f,    -40.7647476003615f},
  { 8.59252164094535f,    -50.3983682135674f},
  { 18.2642763775679f,    -9.59406822686046f},
  { 13.2825255885390f,    -20.3925183693779f},
  { 7.25597292540464f,    -31.5118220765682f},
  { 0.0f,    -41.8103148957786f},
  { -8.59252164094535f,    -50.3983682135674f},
  { 6.27780483909150f,    -10.0335510226076f},
  { 0.0f,    -20.9051574478893f},
  { -7.25597292540464f,    -31.5118220765682f},
  { -15.4504370863258f,    -40.7647476003615f},
  { -6.27780483909150f,    -10.0335510226076f},
  { -13.2825255885390f,    -20.3925183693779f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -18.2642763775679f,    -9.59406822686046f},
  { -25.2736805860404f,    -19.0551045548580f},
  { -28.8127609872844f,    -8.86454598437524f},
  { -157.607229071045f,    0.0f},
  { -168.359276863229f,    0.0f},
  { 180.0f,    0.0f},
  { 168.359276863229f,    0.0f},
  { 157.607229071045f,    0.0f},
  { -151.187239012716f,    8.86454598437525f},
  { -161.735723622432f,    9.59406822686046f},
  { -173.722195160908f,    10.0335510226076f},
  { 173.722195160908f,    10.0335510226076f},
  { 161.735723622432f,    9.59406822686046f},
  { 151.187239012716f,    8.86454598437525f},
  { -154.726319413960f,    19.0551045548580f},
  { -166.717474411461f,    20.3925183693779f},
  { 180.0f,    20.9051574478893f},
  { 166.717474411461f,    20.3925183693779f},
  { 154.726319413960f,    19.0551045548580f},
  { -159.094842552111f,    30.0000000000000f},
  { -172.744027074595f,    31.5118220765682f},
  { 172.744027074595f,    31.5118220765682f},
  { 159.094842552111f,    30.0000000000000f},
  { -164.549562913674f,    40.7647476003615f},
  { 180.0f,    41.8103148957786f},
  { 164.549562913674f,    40.7647476003615f},
  { -171.407478359055f,    50.3983682135674f},
  { 171.407478359055f,    50.3983682135674f},
  { -151.187239012716f,    -8.86454598437525f},
  { -154.726319413960f,    -19.0551045548580f},
  { -159.094842552111f,    -30.0000000000000f},
  { -164.549562913674f,    -40.7647476003615f},
  { -171.407478359055f,    -50.3983682135674f},
  { -161.735723622432f,    -9.59406822686046f},
  { -166.717474411461f,    -20.3925183693779f},
  { -172.744027074595f,    -31.5118220765682f},
  { 180.0f,    -41.8103148957786f},
  { 171.407478359055f,    -50.3983682135674f},
  { -173.722195160908f,    -10.0335510226076f},
  { 180.0f,    -20.9051574478893f},
  { 172.744027074595f,    -31.5118220765682f},
  { 164.549562913674f,    -40.7647476003615f},
  { 173.722195160908f,    -10.0335510226076f},
  { 166.717474411461f,    -20.3925183693779f},
  { 159.094842552111f,    -30.0000000000000f},
  { 161.735723622432f,    -9.59406822686046f},
  { 154.726319413960f,    -19.0551045548580f},
  { 151.187239012716f,    -8.86454598437524f},
  { 90.0f,    22.3927709289549f},
  { 90.0f,    11.6407231367706f},
  { 90.0f,    0.0f},
  { 90.0f,    -11.6407231367706f},
  { 90.0f,    -22.3927709289549f},
  { 79.9071595156824f,    28.4369994004269f},
  { 79.9071595156824f,    18.0f},
  { 79.9071595156824f,    6.18141426337592f},
  { 79.9071595156824f,    -6.18141426337592f},
  { 79.9071595156824f,    -18.0000000000000f},
  { 79.9071595156824f,    -28.4369994004269f},
  { 69.0948425521107f,    23.8001826021751f},
  { 69.0948425521107f,    12.4362806936207f},
  { 69.0948425521107f,    0.0f},
  { 69.0948425521107f,    -12.4362806936207f},
  { 69.0948425521107f,    -23.8001826021751f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    6.18141426337591f},
  { 58.2825255885390f,    -6.18141426337592f},
  { 58.2825255885390f,    -18.0f},
  { 48.1896851042214f,    11.6407231367706f},
  { 48.1896851042214f,    0.0f},
  { 48.1896851042214f,    -11.6407231367706f},
  { 39.2843328795171f,    5.46504310153393f},
  { 39.2843328795171f,    -5.46504310153393f},
  { 100.092840484318f,    28.4369994004269f},
  { 110.905157447889f,    23.8001826021751f},
  { 121.717474411461f,    18.0f},
  { 131.810314895779f,    11.6407231367706f},
  { 140.715667120483f,    5.46504310153393f},
  { 100.092840484318f,    18.0000000000000f},
  { 110.905157447889f,    12.4362806936207f},
  { 121.717474411461f,    6.18141426337591f},
  { 131.810314895779f,    0.0f},
  { 140.715667120483f,    -5.46504310153393f},
  { 100.092840484318f,    6.18141426337591f},
  { 110.905157447889f,    0.0f},
  { 121.717474411461f,    -6.18141426337591f},
  { 131.810314895779f,    -11.6407231367706f},
  { 100.092840484318f,    -6.18141426337591f},
  { 110.905157447889f,    -12.4362806936207f},
  { 121.717474411461f,    -18.0f},
  { 100.092840484318f,    -18.0f},
  { 110.905157447889f,    -23.8001826021751f},
  { 100.092840484318f,    -28.4369994004269f},
  { -79.9071595156824f,    28.4369994004269f},
  { -69.0948425521107f,    23.8001826021751f},
  { -58.2825255885390f,    18.0f},
  { -48.1896851042214f,    11.6407231367706f},
  { -39.2843328795171f,    5.46504310153393f},
  { -90.0f,    22.3927709289549f},
  { -79.9071595156824f,    18.0000000000000f},
  { -69.0948425521107f,    12.4362806936207f},
  { -58.2825255885390f,    6.18141426337591f},
  { -48.1896851042214f,    0.0f},
  { -39.2843328795171f,    -5.46504310153393f},
  { -90.0f,    11.6407231367706f},
  { -79.9071595156824f,    6.18141426337591f},
  { -69.0948425521107f,    0.0f},
  { -58.2825255885390f,    -6.18141426337591f},
  { -48.1896851042214f,    -11.6407231367706f},
  { -90.0f,    0.0f},
  { -79.9071595156824f,    -6.18141426337591f},
  { -69.0948425521107f,    -12.4362806936207f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -11.6407231367706f},
  { -79.9071595156824f,    -18.0f},
  { -69.0948425521107f,    -23.8001826021751f},
  { -90.0f,    -22.3927709289549f},
  { -79.9071595156824f,    -28.4369994004269f},
  { -100.092840484318f,    28.4369994004269f},
  { -100.092840484318f,    18.0f},
  { -100.092840484318f,    6.18141426337592f},
  { -100.092840484318f,    -6.18141426337592f},
  { -100.092840484318f,    -18.0000000000000f},
  { -100.092840484318f,    -28.4369994004269f},
  { -110.905157447889f,    23.8001826021751f},
  { -110.905157447889f,    12.4362806936207f},
  { -110.905157447889f,    0.0f},
  { -110.905157447889f,    -12.4362806936207f},
  { -110.905157447889f,    -23.8001826021751f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    6.18141426337591f},
  { -121.717474411461f,    -6.18141426337592f},
  { -121.717474411461f,    -18.0f},
  { -131.810314895779f,    11.6407231367706f},
  { -131.810314895779f,    0.0f},
  { -131.810314895779f,    -11.6407231367706f},
  { -140.715667120483f,    5.46504310153393f},
  { -140.715667120483f,    -5.46504310153393f},
  { 70.6909217483267f,    35.3584095083404f},
  { 58.2825255885390f,    29.8185857366241f},
  { 46.8095496995643f,    22.9282785675942f},
  { 36.9382074899021f,    15.6447696005703f},
  { 58.2825255885390f,    42.1814142633759f},
  { 45.0000000000000f,    35.2643896827547f},
  { 33.9705954101743f,    27.1374399585075f},
  { 42.5297913750327f,    47.5637193063793f},
  { 30.1213952152451f,    39.0760490721854f},
  { 24.9857717894179f,    50.3244736173956f},
  { 70.6909217483267f,    -35.3584095083404f},
  { 58.2825255885390f,    -42.1814142633759f},
  { 42.5297913750327f,    -47.5637193063793f},
  { 24.9857717894179f,    -50.3244736173956f},
  { 58.2825255885390f,    -29.8185857366241f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 30.1213952152451f,    -39.0760490721854f},
  { 46.8095496995643f,    -22.9282785675942f},
  { 33.9705954101743f,    -27.1374399585075f},
  { 36.9382074899021f,    -15.6447696005703f},
  { -24.9857717894179f,    50.3244736173956f},
  { -30.1213952152451f,    39.0760490721854f},
  { -33.9705954101743f,    27.1374399585075f},
  { -36.9382074899021f,    15.6447696005703f},
  { -42.5297913750327f,    47.5637193063793f},
  { -45.0000000000000f,    35.2643896827547f},
  { -46.8095496995643f,    22.9282785675942f},
  { -58.2825255885390f,    42.1814142633759f},
  { -58.2825255885390f,    29.8185857366241f},
  { -70.6909217483267f,    35.3584095083404f},
  { -24.9857717894179f,    -50.3244736173956f},
  { -42.5297913750327f,    -47.5637193063793f},
  { -58.2825255885390f,    -42.1814142633759f},
  { -70.6909217483267f,    -35.3584095083404f},
  { -30.1213952152451f,    -39.0760490721854f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -58.2825255885390f,    -29.8185857366241f},
  { -33.9705954101743f,    -27.1374399585075f},
  { -46.8095496995643f,    -22.9282785675942f},
  { -36.9382074899021f,    -15.6447696005703f},
  { 155.014228210582f,    50.3244736173956f},
  { 149.878604784755f,    39.0760490721854f},
  { 146.029404589826f,    27.1374399585075f},
  { 143.061792510098f,    15.6447696005703f},
  { 137.470208624967f,    47.5637193063793f},
  { 135.0f,    35.2643896827547f},
  { 133.190450300436f,    22.9282785675942f},
  { 121.717474411461f,    42.1814142633759f},
  { 121.717474411461f,    29.8185857366241f},
  { 109.309078251673f,    35.3584095083404f},
  { 155.014228210582f,    -50.3244736173956f},
  { 137.470208624967f,    -47.5637193063793f},
  { 121.717474411461f,    -42.1814142633759f},
  { 109.309078251673f,    -35.3584095083403f},
  { 149.878604784755f,    -39.0760490721854f},
  { 135.0f,    -35.2643896827547f},
  { 121.717474411461f,    -29.8185857366241f},
  { 146.029404589826f,    -27.1374399585075f},
  { 133.190450300436f,    -22.9282785675942f},
  { 143.061792510098f,    -15.6447696005703f},
  { -155.014228210582f,    50.3244736173956f},
  { -137.470208624967f,    47.5637193063793f},
  { -121.717474411461f,    42.1814142633759f},
  { -109.309078251673f,    35.3584095083403f},
  { -149.878604784755f,    39.0760490721854f},
  { -135.0f,    35.2643896827547f},
  { -121.717474411461f,    29.8185857366241f},
  { -146.029404589826f,    27.1374399585075f},
  { -133.190450300436f,    22.9282785675942f},
  { -143.061792510098f,    15.6447696005703f},
  { -155.014228210582f,    -50.3244736173956f},
  { -149.878604784755f,    -39.0760490721854f},
  { -146.029404589826f,    -27.1374399585075f},
  { -143.061792510098f,    -15.6447696005703f},
  { -137.470208624967f,    -47.5637193063793f},
  { -135.0f,    -35.2643896827547f},
  { -133.190450300436f,    -22.9282785675942f},
  { -121.717474411461f,    -42.1814142633759f},
  { -121.717474411461f,    -29.8185857366241f},
  { -109.309078251673f,    -35.3584095083404f}};

/* 3LD geosphere 'ico' [7 0] (492 points) */
const float __geosphere_ico_7_0_dirs_deg[492][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    66.1808024884231f},
  { 0.0f,    75.1646413480039f},
  { 0.0f,    84.9544063480986f},
  { 180.0f,    84.9544063480986f},
  { 180.0f,    75.1646413480039f},
  { 180.0f,    66.1808024884231f},
  { 15.0920752881032f,    59.8719245592249f},
  { 22.0237505161462f,    68.0530702426802f},
  { 38.9734473368657f,    76.4911891305542f},
  { 90.0000000000000f,    81.4074783590547f},
  { 141.026552663134f,    76.4911891305542f},
  { 157.976249483854f,    68.0530702426802f},
  { 164.907924711897f,    59.8719245592249f},
  { 32.9114931857064f,    59.4482400957790f},
  { 47.1679114977869f,    66.3788444575833f},
  { 72.8279621494862f,    71.4441996352265f},
  { 107.172037850514f,    71.4441996352265f},
  { 132.832088502213f,    66.3788444575833f},
  { 147.088506814294f,    59.4482400957790f},
  { 50.5099309427673f,    56.4143320128269f},
  { 67.6072290710451f,    61.0023561147386f},
  { 90.0f,    62.8666219591469f},
  { 112.392770928955f,    61.0023561147386f},
  { 129.490069057233f,    56.4143320128269f},
  { 65.1310748815380f,    51.1421142488325f},
  { 81.2167693990410f,    53.5115270803982f},
  { 98.7832306009590f,    53.5115270803982f},
  { 114.868925118462f,    51.1421142488325f},
  { 76.1141128518623f,    44.6623463365959f},
  { 90.0f,    45.5120052212781f},
  { 103.885887148138f,    44.6623463365959f},
  { 84.1189512527134f,    37.9745512724670f},
  { 95.8810487472866f,    37.9745512724670f},
  { -15.0920752881032f,    59.8719245592249f},
  { -32.9114931857064f,    59.4482400957790f},
  { -50.5099309427673f,    56.4143320128269f},
  { -65.1310748815380f,    51.1421142488325f},
  { -76.1141128518623f,    44.6623463365959f},
  { -84.1189512527134f,    37.9745512724670f},
  { -22.0237505161462f,    68.0530702426802f},
  { -47.1679114977869f,    66.3788444575833f},
  { -67.6072290710451f,    61.0023561147386f},
  { -81.2167693990410f,    53.5115270803982f},
  { -90.0000000000000f,    45.5120052212781f},
  { -95.8810487472866f,    37.9745512724671f},
  { -38.9734473368657f,    76.4911891305542f},
  { -72.8279621494862f,    71.4441996352265f},
  { -90.0f,    62.8666219591469f},
  { -98.7832306009590f,    53.5115270803982f},
  { -103.885887148138f,    44.6623463365959f},
  { -90.0000000000000f,    81.4074783590547f},
  { -107.172037850514f,    71.4441996352265f},
  { -112.392770928955f,    61.0023561147386f},
  { -114.868925118462f,    51.1421142488325f},
  { -141.026552663134f,    76.4911891305542f},
  { -132.832088502213f,    66.3788444575833f},
  { -129.490069057233f,    56.4143320128269f},
  { -157.976249483854f,    68.0530702426802f},
  { -147.088506814294f,    59.4482400957790f},
  { -164.907924711897f,    59.8719245592249f},
  { 15.0920752881032f,    -59.8719245592249f},
  { 32.9114931857064f,    -59.4482400957790f},
  { 50.5099309427673f,    -56.4143320128269f},
  { 65.1310748815380f,    -51.1421142488325f},
  { 76.1141128518623f,    -44.6623463365959f},
  { 84.1189512527134f,    -37.9745512724670f},
  { 0.0f,    -66.1808024884231f},
  { 22.0237505161462f,    -68.0530702426802f},
  { 47.1679114977869f,    -66.3788444575833f},
  { 67.6072290710451f,    -61.0023561147386f},
  { 81.2167693990410f,    -53.5115270803982f},
  { 90.0000000000000f,    -45.5120052212781f},
  { 95.8810487472866f,    -37.9745512724671f},
  { 0.0f,    -75.1646413480039f},
  { 38.9734473368657f,    -76.4911891305542f},
  { 72.8279621494862f,    -71.4441996352265f},
  { 90.0f,    -62.8666219591469f},
  { 98.7832306009590f,    -53.5115270803982f},
  { 103.885887148138f,    -44.6623463365959f},
  { 0.0f,    -84.9544063480986f},
  { 90.0000000000000f,    -81.4074783590547f},
  { 107.172037850514f,    -71.4441996352265f},
  { 112.392770928955f,    -61.0023561147386f},
  { 114.868925118462f,    -51.1421142488325f},
  { 180.0f,    -84.9544063480986f},
  { 141.026552663134f,    -76.4911891305542f},
  { 132.832088502213f,    -66.3788444575833f},
  { 129.490069057233f,    -56.4143320128269f},
  { 180.0f,    -75.1646413480039f},
  { 157.976249483854f,    -68.0530702426802f},
  { 147.088506814294f,    -59.4482400957790f},
  { 180.0f,    -66.1808024884231f},
  { 164.907924711897f,    -59.8719245592249f},
  { -15.0920752881032f,    -59.8719245592249f},
  { -22.0237505161462f,    -68.0530702426802f},
  { -38.9734473368657f,    -76.4911891305542f},
  { -90.0000000000000f,    -81.4074783590547f},
  { -141.026552663134f,    -76.4911891305542f},
  { -157.976249483854f,    -68.0530702426802f},
  { -164.907924711897f,    -59.8719245592249f},
  { -32.9114931857064f,    -59.4482400957790f},
  { -47.1679114977869f,    -66.3788444575833f},
  { -72.8279621494862f,    -71.4441996352265f},
  { -107.172037850514f,    -71.4441996352265f},
  { -132.832088502213f,    -66.3788444575833f},
  { -147.088506814294f,    -59.4482400957790f},
  { -50.5099309427673f,    -56.4143320128269f},
  { -67.6072290710451f,    -61.0023561147386f},
  { -90.0f,    -62.8666219591469f},
  { -112.392770928955f,    -61.0023561147386f},
  { -129.490069057233f,    -56.4143320128269f},
  { -65.1310748815380f,    -51.1421142488325f},
  { -81.2167693990410f,    -53.5115270803982f},
  { -98.7832306009590f,    -53.5115270803982f},
  { -114.868925118462f,    -51.1421142488325f},
  { -76.1141128518623f,    -44.6623463365959f},
  { -90.0f,    -45.5120052212781f},
  { -103.885887148138f,    -44.6623463365959f},
  { -84.1189512527134f,    -37.9745512724670f},
  { -95.8810487472866f,    -37.9745512724670f},
  { 23.8191975115769f,    0.0f},
  { 14.8353586519961f,    0.0f},
  { 5.04559365190141f,    0.0f},
  { -5.04559365190141f,    0.0f},
  { -14.8353586519961f,    0.0f},
  { -23.8191975115769f,    0.0f},
  { 29.2626971786928f,    7.50942039010128f},
  { 20.4829414460949f,    8.05663962251492f},
  { 10.5793931769549f,    8.44858701261369f},
  { 3.95476420138187e-15f,    8.59252164094535f},
  { -10.5793931769549f,    8.44858701261369f},
  { -20.4829414460949f,    8.05663962251492f},
  { -29.2626971786928f,    7.50942039010128f},
  { 26.3598778094528f,    16.0329525274240f},
  { 16.5582178401620f,    17.0882173737518f},
  { 5.65989372918869f,    17.7005548371051f},
  { -5.65989372918869f,    17.7005548371051f},
  { -16.5582178401620f,    17.0882173737518f},
  { -26.3598778094528f,    16.0329525274240f},
  { 22.8939075042604f,    25.2716662118425f},
  { 11.9226312702234f,    26.6294653632014f},
  { 0.0f,    27.1333780408531f},
  { -11.9226312702234f,    26.6294653632014f},
  { -22.8939075042604f,    25.2716662118425f},
  { 18.7177168090702f,    34.6954547665617f},
  { 6.44379382542064f,    35.9930793767210f},
  { -6.44379382542064f,    35.9930793767210f},
  { -18.7177168090702f,    34.6954547665617f},
  { 13.6491845224847f,    43.6677528316015f},
  { 0.0f,    44.4879947787219f},
  { -13.6491845224847f,    43.6677528316015f},
  { 7.47831395519902f,    51.6407658043736f},
  { -7.47831395519902f,    51.6407658043736f},
  { 29.2626971786928f,    -7.50942039010128f},
  { 26.3598778094528f,    -16.0329525274240f},
  { 22.8939075042604f,    -25.2716662118425f},
  { 18.7177168090702f,    -34.6954547665617f},
  { 13.6491845224847f,    -43.6677528316015f},
  { 7.47831395519902f,    -51.6407658043736f},
  { 20.4829414460949f,    -8.05663962251492f},
  { 16.5582178401620f,    -17.0882173737518f},
  { 11.9226312702234f,    -26.6294653632014f},
  { 6.44379382542064f,    -35.9930793767210f},
  { -5.14182516329846e-15f,    -44.4879947787219f},
  { -7.47831395519902f,    -51.6407658043736f},
  { 10.5793931769549f,    -8.44858701261369f},
  { 5.65989372918869f,    -17.7005548371051f},
  { 0.0f,    -27.1333780408531f},
  { -6.44379382542064f,    -35.9930793767210f},
  { -13.6491845224847f,    -43.6677528316015f},
  { -1.97738210069094e-15f,    -8.59252164094535f},
  { -5.65989372918869f,    -17.7005548371051f},
  { -11.9226312702234f,    -26.6294653632014f},
  { -18.7177168090702f,    -34.6954547665617f},
  { -10.5793931769549f,    -8.44858701261369f},
  { -16.5582178401620f,    -17.0882173737518f},
  { -22.8939075042604f,    -25.2716662118425f},
  { -20.4829414460949f,    -8.05663962251491f},
  { -26.3598778094528f,    -16.0329525274240f},
  { -29.2626971786928f,    -7.50942039010127f},
  { -156.180802488423f,    0.0f},
  { -165.164641348004f,    0.0f},
  { -174.954406348099f,    0.0f},
  { 174.954406348099f,    0.0f},
  { 165.164641348004f,    0.0f},
  { 156.180802488423f,    0.0f},
  { -150.737302821307f,    7.50942039010128f},
  { -159.517058553905f,    8.05663962251492f},
  { -169.420606823045f,    8.44858701261369f},
  { -180.0f,    8.59252164094535f},
  { 169.420606823045f,    8.44858701261369f},
  { 159.517058553905f,    8.05663962251492f},
  { 150.737302821307f,    7.50942039010128f},
  { -153.640122190547f,    16.0329525274240f},
  { -163.441782159838f,    17.0882173737518f},
  { -174.340106270811f,    17.7005548371051f},
  { 174.340106270811f,    17.7005548371051f},
  { 163.441782159838f,    17.0882173737518f},
  { 153.640122190547f,    16.0329525274240f},
  { -157.106092495740f,    25.2716662118425f},
  { -168.077368729777f,    26.6294653632014f},
  { 180.0f,    27.1333780408531f},
  { 168.077368729777f,    26.6294653632014f},
  { 157.106092495740f,    25.2716662118425f},
  { -161.282283190930f,    34.6954547665617f},
  { -173.556206174579f,    35.9930793767210f},
  { 173.556206174579f,    35.9930793767210f},
  { 161.282283190930f,    34.6954547665617f},
  { -166.350815477515f,    43.6677528316015f},
  { 180.0f,    44.4879947787219f},
  { 166.350815477515f,    43.6677528316015f},
  { -172.521686044801f,    51.6407658043736f},
  { 172.521686044801f,    51.6407658043736f},
  { -150.737302821307f,    -7.50942039010128f},
  { -153.640122190547f,    -16.0329525274240f},
  { -157.106092495740f,    -25.2716662118425f},
  { -161.282283190930f,    -34.6954547665617f},
  { -166.350815477515f,    -43.6677528316015f},
  { -172.521686044801f,    -51.6407658043736f},
  { -159.517058553905f,    -8.05663962251492f},
  { -163.441782159838f,    -17.0882173737518f},
  { -168.077368729777f,    -26.6294653632014f},
  { -173.556206174579f,    -35.9930793767210f},
  { 180.0f,    -44.4879947787219f},
  { 172.521686044801f,    -51.6407658043736f},
  { -169.420606823045f,    -8.44858701261369f},
  { -174.340106270811f,    -17.7005548371051f},
  { 180.0f,    -27.1333780408531f},
  { 173.556206174579f,    -35.9930793767210f},
  { 166.350815477515f,    -43.6677528316015f},
  { 180.0f,    -8.59252164094535f},
  { 174.340106270811f,    -17.7005548371051f},
  { 168.077368729777f,    -26.6294653632014f},
  { 161.282283190930f,    -34.6954547665617f},
  { 169.420606823045f,    -8.44858701261369f},
  { 163.441782159838f,    -17.0882173737518f},
  { 157.106092495740f,    -25.2716662118425f},
  { 159.517058553905f,    -8.05663962251492f},
  { 153.640122190547f,    -16.0329525274240f},
  { 150.737302821307f,    -7.50942039010127f},
  { 90.0f,    23.8191975115769f},
  { 90.0f,    14.8353586519961f},
  { 90.0f,    5.04559365190141f},
  { 90.0f,    -5.04559365190141f},
  { 90.0f,    -14.8353586519961f},
  { 90.0f,    -23.8191975115769f},
  { 81.4074783590547f,    28.9877234630627f},
  { 81.4074783590547f,    20.2718429567436f},
  { 81.4074783590547f,    10.4632857609175f},
  { 81.4074783590547f,    3.91037549404927e-15f},
  { 81.4074783590547f,    -10.4632857609175f},
  { 81.4074783590547f,    -20.2718429567436f},
  { 81.4074783590547f,    -28.9877234630627f},
  { 72.2181794238810f,    25.2606763729821f},
  { 72.2181794238810f,    15.8076162308333f},
  { 72.2181794238810f,    5.39113258470827f},
  { 72.2181794238810f,    -5.39113258470827f},
  { 72.2181794238810f,    -15.8076162308333f},
  { 72.2181794238810f,    -25.2606763729821f},
  { 62.8666219591469f,    20.5970703451119f},
  { 62.8666219591469f,    10.6422621988362f},
  { 62.8666219591469f,    0.0f},
  { 62.8666219591469f,    -10.6422621988362f},
  { 62.8666219591469f,    -20.5970703451119f},
  { 53.8340928444730f,    15.2983539399373f},
  { 53.8340928444730f,    5.20978847514459f},
  { 53.8340928444730f,    -5.20978847514459f},
  { 53.8340928444730f,    -15.2983539399373f},
  { 45.5120052212781f,    9.82822484687696f},
  { 45.5120052212781f,    0.0f},
  { 45.5120052212781f,    -9.82822484687696f},
  { 38.1213002554066f,    4.63284787319209f},
  { 38.1213002554066f,    -4.63284787319209f},
  { 98.5925216409454f,    28.9877234630627f},
  { 107.781820576119f,    25.2606763729821f},
  { 117.133378040853f,    20.5970703451119f},
  { 126.165907155527f,    15.2983539399373f},
  { 134.487994778722f,    9.82822484687696f},
  { 141.878699744593f,    4.63284787319209f},
  { 98.5925216409454f,    20.2718429567436f},
  { 107.781820576119f,    15.8076162308333f},
  { 117.133378040853f,    10.6422621988362f},
  { 126.165907155527f,    5.20978847514459f},
  { 134.487994778722f,    -3.66816416523558e-15f},
  { 141.878699744593f,    -4.63284787319209f},
  { 98.5925216409454f,    10.4632857609175f},
  { 107.781820576119f,    5.39113258470827f},
  { 117.133378040853f,    0.0f},
  { 126.165907155527f,    -5.20978847514458f},
  { 134.487994778722f,    -9.82822484687696f},
  { 98.5925216409454f,    -1.95518774702464e-15f},
  { 107.781820576119f,    -5.39113258470827f},
  { 117.133378040853f,    -10.6422621988362f},
  { 126.165907155527f,    -15.2983539399373f},
  { 98.5925216409454f,    -10.4632857609175f},
  { 107.781820576119f,    -15.8076162308333f},
  { 117.133378040853f,    -20.5970703451119f},
  { 98.5925216409454f,    -20.2718429567436f},
  { 107.781820576119f,    -25.2606763729821f},
  { 98.5925216409454f,    -28.9877234630627f},
  { -81.4074783590547f,    28.9877234630627f},
  { -72.2181794238810f,    25.2606763729821f},
  { -62.8666219591469f,    20.5970703451119f},
  { -53.8340928444730f,    15.2983539399373f},
  { -45.5120052212781f,    9.82822484687696f},
  { -38.1213002554066f,    4.63284787319209f},
  { -90.0f,    23.8191975115769f},
  { -81.4074783590547f,    20.2718429567436f},
  { -72.2181794238810f,    15.8076162308333f},
  { -62.8666219591469f,    10.6422621988362f},
  { -53.8340928444730f,    5.20978847514459f},
  { -45.5120052212781f,    -3.66816416523558e-15f},
  { -38.1213002554066f,    -4.63284787319209f},
  { -90.0f,    14.8353586519961f},
  { -81.4074783590547f,    10.4632857609175f},
  { -72.2181794238810f,    5.39113258470827f},
  { -62.8666219591469f,    0.0f},
  { -53.8340928444730f,    -5.20978847514458f},
  { -45.5120052212781f,    -9.82822484687696f},
  { -90.0f,    5.04559365190141f},
  { -81.4074783590547f,    -1.95518774702464e-15f},
  { -72.2181794238810f,    -5.39113258470827f},
  { -62.8666219591469f,    -10.6422621988362f},
  { -53.8340928444730f,    -15.2983539399373f},
  { -90.0f,    -5.04559365190141f},
  { -81.4074783590547f,    -10.4632857609175f},
  { -72.2181794238810f,    -15.8076162308333f},
  { -62.8666219591469f,    -20.5970703451119f},
  { -90.0f,    -14.8353586519961f},
  { -81.4074783590547f,    -20.2718429567436f},
  { -72.2181794238810f,    -25.2606763729821f},
  { -90.0f,    -23.8191975115769f},
  { -81.4074783590547f,    -28.9877234630627f},
  { -98.5925216409454f,    28.9877234630627f},
  { -98.5925216409454f,    20.2718429567436f},
  { -98.5925216409454f,    10.4632857609175f},
  { -98.5925216409454f,    3.91037549404927e-15f},
  { -98.5925216409454f,    -10.4632857609175f},
  { -98.5925216409454f,    -20.2718429567436f},
  { -98.5925216409454f,    -28.9877234630627f},
  { -107.781820576119f,    25.2606763729821f},
  { -107.781820576119f,    15.8076162308333f},
  { -107.781820576119f,    5.39113258470827f},
  { -107.781820576119f,    -5.39113258470827f},
  { -107.781820576119f,    -15.8076162308333f},
  { -107.781820576119f,    -25.2606763729821f},
  { -117.133378040853f,    20.5970703451119f},
  { -117.133378040853f,    10.6422621988362f},
  { -117.133378040853f,    0.0f},
  { -117.133378040853f,    -10.6422621988362f},
  { -117.133378040853f,    -20.5970703451119f},
  { -126.165907155527f,    15.2983539399373f},
  { -126.165907155527f,    5.20978847514459f},
  { -126.165907155527f,    -5.20978847514459f},
  { -126.165907155527f,    -15.2983539399373f},
  { -134.487994778722f,    9.82822484687696f},
  { -134.487994778722f,    0.0f},
  { -134.487994778722f,    -9.82822484687696f},
  { -141.878699744593f,    4.63284787319209f},
  { -141.878699744593f,    -4.63284787319209f},
  { 73.9332208850461f,    34.9768443320308f},
  { 63.4349488229220f,    30.6726725373555f},
  { 53.3007747995101f,    25.2406273115173f},
  { 44.0804766102169f,    19.1794139209880f},
  { 36.0562259208489f,    13.1070197584612f},
  { 64.1635743358256f,    41.0756272443228f},
  { 52.6226318593503f,    35.8669489234169f},
  { 42.2968675884159f,    29.4688797542767f},
  { 33.5491126061725f,    22.6217951428160f},
  { 51.7317026220431f,    46.4882636786595f},
  { 40.0182492109711f,    40.0886354813392f},
  { 30.4545634390528f,    32.7215661485233f},
  { 37.0167508575842f,    50.2904398912844f},
  { 26.5650511770780f,    42.7257778636242f},
  { 21.5833003880491f,    51.9411964822148f},
  { 73.9332208850461f,    -34.9768443320308f},
  { 64.1635743358256f,    -41.0756272443228f},
  { 51.7317026220431f,    -46.4882636786595f},
  { 37.0167508575842f,    -50.2904398912844f},
  { 21.5833003880491f,    -51.9411964822148f},
  { 63.4349488229220f,    -30.6726725373555f},
  { 52.6226318593503f,    -35.8669489234169f},
  { 40.0182492109711f,    -40.0886354813392f},
  { 26.5650511770780f,    -42.7257778636242f},
  { 53.3007747995101f,    -25.2406273115173f},
  { 42.2968675884159f,    -29.4688797542767f},
  { 30.4545634390528f,    -32.7215661485233f},
  { 44.0804766102169f,    -19.1794139209880f},
  { 33.5491126061725f,    -22.6217951428160f},
  { 36.0562259208489f,    -13.1070197584612f},
  { -21.5833003880491f,    51.9411964822148f},
  { -26.5650511770780f,    42.7257778636242f},
  { -30.4545634390528f,    32.7215661485233f},
  { -33.5491126061725f,    22.6217951428160f},
  { -36.0562259208489f,    13.1070197584612f},
  { -37.0167508575842f,    50.2904398912844f},
  { -40.0182492109711f,    40.0886354813392f},
  { -42.2968675884159f,    29.4688797542767f},
  { -44.0804766102169f,    19.1794139209880f},
  { -51.7317026220431f,    46.4882636786595f},
  { -52.6226318593503f,    35.8669489234169f},
  { -53.3007747995101f,    25.2406273115173f},
  { -64.1635743358256f,    41.0756272443228f},
  { -63.4349488229220f,    30.6726725373555f},
  { -73.9332208850461f,    34.9768443320308f},
  { -21.5833003880491f,    -51.9411964822148f},
  { -37.0167508575842f,    -50.2904398912844f},
  { -51.7317026220431f,    -46.4882636786595f},
  { -64.1635743358256f,    -41.0756272443228f},
  { -73.9332208850461f,    -34.9768443320308f},
  { -26.5650511770780f,    -42.7257778636242f},
  { -40.0182492109711f,    -40.0886354813392f},
  { -52.6226318593503f,    -35.8669489234169f},
  { -63.4349488229220f,    -30.6726725373555f},
  { -30.4545634390528f,    -32.7215661485233f},
  { -42.2968675884159f,    -29.4688797542767f},
  { -53.3007747995101f,    -25.2406273115173f},
  { -33.5491126061725f,    -22.6217951428160f},
  { -44.0804766102169f,    -19.1794139209880f},
  { -36.0562259208489f,    -13.1070197584612f},
  { 158.416699611951f,    51.9411964822148f},
  { 153.434948822922f,    42.7257778636242f},
  { 149.545436560947f,    32.7215661485233f},
  { 146.450887393828f,    22.6217951428160f},
  { 143.943774079151f,    13.1070197584612f},
  { 142.983249142416f,    50.2904398912844f},
  { 139.981750789029f,    40.0886354813392f},
  { 137.703132411584f,    29.4688797542767f},
  { 135.919523389783f,    19.1794139209880f},
  { 128.268297377957f,    46.4882636786595f},
  { 127.377368140650f,    35.8669489234169f},
  { 126.699225200490f,    25.2406273115173f},
  { 115.836425664174f,    41.0756272443228f},
  { 116.565051177078f,    30.6726725373555f},
  { 106.066779114954f,    34.9768443320308f},
  { 158.416699611951f,    -51.9411964822148f},
  { 142.983249142416f,    -50.2904398912844f},
  { 128.268297377957f,    -46.4882636786595f},
  { 115.836425664174f,    -41.0756272443228f},
  { 106.066779114954f,    -34.9768443320308f},
  { 153.434948822922f,    -42.7257778636242f},
  { 139.981750789029f,    -40.0886354813392f},
  { 127.377368140650f,    -35.8669489234169f},
  { 116.565051177078f,    -30.6726725373555f},
  { 149.545436560947f,    -32.7215661485233f},
  { 137.703132411584f,    -29.4688797542767f},
  { 126.699225200490f,    -25.2406273115173f},
  { 146.450887393828f,    -22.6217951428160f},
  { 135.919523389783f,    -19.1794139209880f},
  { 143.943774079151f,    -13.1070197584612f},
  { -158.416699611951f,    51.9411964822148f},
  { -142.983249142416f,    50.2904398912844f},
  { -128.268297377957f,    46.4882636786595f},
  { -115.836425664174f,    41.0756272443228f},
  { -106.066779114954f,    34.9768443320308f},
  { -153.434948822922f,    42.7257778636242f},
  { -139.981750789029f,    40.0886354813392f},
  { -127.377368140650f,    35.8669489234169f},
  { -116.565051177078f,    30.6726725373555f},
  { -149.545436560947f,    32.7215661485233f},
  { -137.703132411584f,    29.4688797542767f},
  { -126.699225200490f,    25.2406273115173f},
  { -146.450887393828f,    22.6217951428160f},
  { -135.919523389783f,    19.1794139209880f},
  { -143.943774079151f,    13.1070197584612f},
  { -158.416699611951f,    -51.9411964822148f},
  { -153.434948822922f,    -42.7257778636242f},
  { -149.545436560947f,    -32.7215661485233f},
  { -146.450887393828f,    -22.6217951428160f},
  { -143.943774079151f,    -13.1070197584612f},
  { -142.983249142416f,    -50.2904398912844f},
  { -139.981750789029f,    -40.0886354813392f},
  { -137.703132411584f,    -29.4688797542767f},
  { -135.919523389783f,    -19.1794139209880f},
  { -128.268297377957f,    -46.4882636786595f},
  { -127.377368140650f,    -35.8669489234169f},
  { -126.699225200490f,    -25.2406273115173f},
  { -115.836425664174f,    -41.0756272443228f},
  { -116.565051177078f,    -30.6726725373555f},
  { -106.066779114954f,    -34.9768443320308f}};

/* 3LD geosphere 'ico' [8 0] (642 points) */
const float __geosphere_ico_8_0_dirs_deg[642][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    65.1310748815380f},
  { 0.0f,    72.8279621494862f},
  { 0.0f,    81.2167693990410f},
  { 0.0f,    90.0f},
  { 180.0f,    81.2167693990410f},
  { 180.0f,    72.8279621494862f},
  { 180.0f,    65.1310748815380f},
  { 13.0152040462228f,    59.7634611728920f},
  { 17.9319334355159f,    66.9090449293358f},
  { 28.3399673707642f,    74.5425010655038f},
  { 58.2825255885390f,    81.2276448816065f},
  { 121.717474411461f,    81.2276448816065f},
  { 151.660032629236f,    74.5425010655038f},
  { 162.068066564484f,    66.9090449293358f},
  { 166.984795953777f,    59.7634611728920f},
  { 28.3399673707642f,    59.7900946927798f},
  { 38.9734473368657f,    66.2771675744593f},
  { 58.2825255885390f,    72.0f},
  { 90.0f,    74.5495629136742f},
  { 121.717474411461f,    72.0f},
  { 141.026552663134f,    66.2771675744593f},
  { 151.660032629236f,    59.7900946927798f},
  { 44.1517511327961f,    57.8562154800787f},
  { 58.2825255885390f,    62.7723551183935f},
  { 78.3592768632294f,    65.9208196976405f},
  { 101.640723136771f,    65.9208196976405f},
  { 121.717474411461f,    62.7723551183935f},
  { 135.848248867204f,    57.8562154800787f},
  { 58.2825255885390f,    54.0f},
  { 72.8279621494862f,    57.1023923174897f},
  { 90.0f,    58.2825255885390f},
  { 107.172037850514f,    57.1023923174897f},
  { 121.717474411461f,    54.0f},
  { 69.6541923856616f,    48.7938239499718f},
  { 82.9535936191428f,    50.4009770857117f},
  { 97.0464063808572f,    50.4009770857117f},
  { 110.345807614338f,    48.7938239499718f},
  { 78.3592768632294f,    42.9780676586279f},
  { 90.0f,    43.5723495188822f},
  { 101.640723136771f,    42.9780676586279f},
  { 84.9544063480986f,    37.1600422047896f},
  { 95.0455936519014f,    37.1600422047896f},
  { -13.0152040462228f,    59.7634611728920f},
  { -28.3399673707642f,    59.7900946927798f},
  { -44.1517511327961f,    57.8562154800787f},
  { -58.2825255885390f,    54.0f},
  { -69.6541923856616f,    48.7938239499718f},
  { -78.3592768632294f,    42.9780676586279f},
  { -84.9544063480986f,    37.1600422047896f},
  { -17.9319334355159f,    66.9090449293358f},
  { -38.9734473368657f,    66.2771675744593f},
  { -58.2825255885390f,    62.7723551183935f},
  { -72.8279621494862f,    57.1023923174897f},
  { -82.9535936191428f,    50.4009770857117f},
  { -90.0f,    43.5723495188822f},
  { -95.0455936519014f,    37.1600422047896f},
  { -28.3399673707642f,    74.5425010655038f},
  { -58.2825255885390f,    72.0f},
  { -78.3592768632294f,    65.9208196976405f},
  { -90.0f,    58.2825255885390f},
  { -97.0464063808572f,    50.4009770857117f},
  { -101.640723136771f,    42.9780676586279f},
  { -58.2825255885390f,    81.2276448816065f},
  { -90.0000000000000f,    74.5495629136742f},
  { -101.640723136771f,    65.9208196976405f},
  { -107.172037850514f,    57.1023923174897f},
  { -110.345807614338f,    48.7938239499718f},
  { -121.717474411461f,    81.2276448816065f},
  { -121.717474411461f,    72.0f},
  { -121.717474411461f,    62.7723551183935f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    74.5425010655038f},
  { -141.026552663134f,    66.2771675744593f},
  { -135.848248867204f,    57.8562154800787f},
  { -162.068066564484f,    66.9090449293358f},
  { -151.660032629236f,    59.7900946927798f},
  { -166.984795953777f,    59.7634611728920f},
  { 13.0152040462228f,    -59.7634611728920f},
  { 28.3399673707642f,    -59.7900946927798f},
  { 44.1517511327961f,    -57.8562154800787f},
  { 58.2825255885390f,    -54.0f},
  { 69.6541923856616f,    -48.7938239499718f},
  { 78.3592768632294f,    -42.9780676586279f},
  { 84.9544063480986f,    -37.1600422047896f},
  { 0.0f,    -65.1310748815380f},
  { 17.9319334355159f,    -66.9090449293358f},
  { 38.9734473368657f,    -66.2771675744593f},
  { 58.2825255885390f,    -62.7723551183935f},
  { 72.8279621494862f,    -57.1023923174897f},
  { 82.9535936191428f,    -50.4009770857117f},
  { 90.0f,    -43.5723495188822f},
  { 95.0455936519014f,    -37.1600422047896f},
  { 0.0f,    -72.8279621494862f},
  { 28.3399673707642f,    -74.5425010655038f},
  { 58.2825255885390f,    -72.0f},
  { 78.3592768632294f,    -65.9208196976405f},
  { 90.0f,    -58.2825255885390f},
  { 97.0464063808572f,    -50.4009770857117f},
  { 101.640723136771f,    -42.9780676586279f},
  { 0.0f,    -81.2167693990410f},
  { 58.2825255885390f,    -81.2276448816065f},
  { 90.0000000000000f,    -74.5495629136742f},
  { 101.640723136771f,    -65.9208196976405f},
  { 107.172037850514f,    -57.1023923174897f},
  { 110.345807614338f,    -48.7938239499718f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -81.2276448816065f},
  { 121.717474411461f,    -72.0f},
  { 121.717474411461f,    -62.7723551183935f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -81.2167693990410f},
  { 151.660032629236f,    -74.5425010655038f},
  { 141.026552663134f,    -66.2771675744593f},
  { 135.848248867204f,    -57.8562154800787f},
  { 180.0f,    -72.8279621494862f},
  { 162.068066564484f,    -66.9090449293358f},
  { 151.660032629236f,    -59.7900946927798f},
  { 180.0f,    -65.1310748815380f},
  { 166.984795953777f,    -59.7634611728920f},
  { -13.0152040462228f,    -59.7634611728920f},
  { -17.9319334355159f,    -66.9090449293358f},
  { -28.3399673707642f,    -74.5425010655038f},
  { -58.2825255885390f,    -81.2276448816065f},
  { -121.717474411461f,    -81.2276448816065f},
  { -151.660032629236f,    -74.5425010655038f},
  { -162.068066564484f,    -66.9090449293358f},
  { -166.984795953777f,    -59.7634611728920f},
  { -28.3399673707642f,    -59.7900946927798f},
  { -38.9734473368657f,    -66.2771675744593f},
  { -58.2825255885390f,    -72.0f},
  { -90.0f,    -74.5495629136742f},
  { -121.717474411461f,    -72.0f},
  { -141.026552663134f,    -66.2771675744593f},
  { -151.660032629236f,    -59.7900946927798f},
  { -44.1517511327961f,    -57.8562154800787f},
  { -58.2825255885390f,    -62.7723551183935f},
  { -78.3592768632294f,    -65.9208196976405f},
  { -101.640723136771f,    -65.9208196976405f},
  { -121.717474411461f,    -62.7723551183935f},
  { -135.848248867204f,    -57.8562154800787f},
  { -58.2825255885390f,    -54.0f},
  { -72.8279621494862f,    -57.1023923174897f},
  { -90.0f,    -58.2825255885390f},
  { -107.172037850514f,    -57.1023923174897f},
  { -121.717474411461f,    -54.0f},
  { -69.6541923856616f,    -48.7938239499718f},
  { -82.9535936191428f,    -50.4009770857117f},
  { -97.0464063808572f,    -50.4009770857117f},
  { -110.345807614338f,    -48.7938239499718f},
  { -78.3592768632294f,    -42.9780676586279f},
  { -90.0f,    -43.5723495188822f},
  { -101.640723136771f,    -42.9780676586279f},
  { -84.9544063480986f,    -37.1600422047896f},
  { -95.0455936519014f,    -37.1600422047896f},
  { 24.8689251184620f,    0.0f},
  { 17.1720378505138f,    0.0f},
  { 8.78323060095898f,    0.0f},
  { 0.0f,    0.0f},
  { -8.78323060095898f,    0.0f},
  { -17.1720378505138f,    0.0f},
  { -24.8689251184620f,    0.0f},
  { 29.5919950295299f,    6.51186878797759f},
  { 22.0793821319241f,    6.93543325054564f},
  { 13.6788812485618f,    7.26850261623438f},
  { 4.63812012275413f,    7.45410055983883f},
  { -4.63812012275412f,    7.45410055983883f},
  { -13.6788812485618f,    7.26850261623438f},
  { -22.0793821319241f,    6.93543325054564f},
  { -29.5919950295299f,    6.51186878797759f},
  { 27.1333780408531f,    13.8190092060518f},
  { 18.8622597253010f,    14.6573954710189f},
  { 9.69372389531482f,    15.2401622826778f},
  { 0.0f,    15.4504370863258f},
  { -9.69372389531481f,    15.2401622826778f},
  { -18.8622597253010f,    14.6573954710189f},
  { -27.1333780408531f,    13.8190092060518f},
  { 24.2682191755604f,    21.7527747654215f},
  { 15.1367980035503f,    22.9044641727044f},
  { 5.15242323438301f,    23.5536156829348f},
  { -5.15242323438301f,    23.5536156829348f},
  { -15.1367980035503f,    22.9044641727044f},
  { -24.2682191755604f,    21.7527747654215f},
  { 20.9051574478893f,    30.0000000000000f},
  { 10.8123169635717f,    31.2603465080791f},
  { 0.0f,    31.7174744114610f},
  { -10.8123169635717f,    31.2603465080791f},
  { -20.9051574478893f,    30.0000000000000f},
  { 16.9324379151258f,    38.1462352379972f},
  { 5.79457193955913f,    39.2419529905605f},
  { -5.79457193955913f,    39.2419529905605f},
  { -16.9324379151258f,    38.1462352379972f},
  { 12.2183198662487f,    45.7717617455715f},
  { 0.0f,    46.4276504811178f},
  { -12.2183198662487f,    45.7717617455715f},
  { 6.61875877204995f,    52.5480139114809f},
  { -6.61875877204995f,    52.5480139114809f},
  { 29.5919950295299f,    -6.51186878797759f},
  { 27.1333780408531f,    -13.8190092060518f},
  { 24.2682191755604f,    -21.7527747654215f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 16.9324379151258f,    -38.1462352379972f},
  { 12.2183198662487f,    -45.7717617455715f},
  { 6.61875877204995f,    -52.5480139114809f},
  { 22.0793821319241f,    -6.93543325054564f},
  { 18.8622597253010f,    -14.6573954710189f},
  { 15.1367980035503f,    -22.9044641727044f},
  { 10.8123169635717f,    -31.2603465080791f},
  { 5.79457193955914f,    -39.2419529905605f},
  { 5.24012991261292e-15f,    -46.4276504811178f},
  { -6.61875877204995f,    -52.5480139114809f},
  { 13.6788812485618f,    -7.26850261623438f},
  { 9.69372389531481f,    -15.2401622826778f},
  { 5.15242323438301f,    -23.5536156829348f},
  { 0.0f,    -31.7174744114610f},
  { -5.79457193955914f,    -39.2419529905605f},
  { -12.2183198662487f,    -45.7717617455715f},
  { 4.63812012275412f,    -7.45410055983883f},
  { -2.06684972190655e-15f,    -15.4504370863258f},
  { -5.15242323438302f,    -23.5536156829348f},
  { -10.8123169635717f,    -31.2603465080791f},
  { -16.9324379151258f,    -38.1462352379972f},
  { -4.63812012275413f,    -7.45410055983883f},
  { -9.69372389531481f,    -15.2401622826778f},
  { -15.1367980035503f,    -22.9044641727044f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -13.6788812485618f,    -7.26850261623438f},
  { -18.8622597253010f,    -14.6573954710189f},
  { -24.2682191755604f,    -21.7527747654215f},
  { -22.0793821319241f,    -6.93543325054564f},
  { -27.1333780408531f,    -13.8190092060518f},
  { -29.5919950295299f,    -6.51186878797759f},
  { -155.131074881538f,    0.0f},
  { -162.827962149486f,    0.0f},
  { -171.216769399041f,    0.0f},
  { 180.0f,    0.0f},
  { 171.216769399041f,    0.0f},
  { 162.827962149486f,    0.0f},
  { 155.131074881538f,    0.0f},
  { -150.408004970470f,    6.51186878797759f},
  { -157.920617868076f,    6.93543325054564f},
  { -166.321118751438f,    7.26850261623438f},
  { -175.361879877246f,    7.45410055983883f},
  { 175.361879877246f,    7.45410055983883f},
  { 166.321118751438f,    7.26850261623438f},
  { 157.920617868076f,    6.93543325054564f},
  { 150.408004970470f,    6.51186878797759f},
  { -152.866621959147f,    13.8190092060518f},
  { -161.137740274699f,    14.6573954710189f},
  { -170.306276104685f,    15.2401622826778f},
  { 180.0f,    15.4504370863258f},
  { 170.306276104685f,    15.2401622826778f},
  { 161.137740274699f,    14.6573954710189f},
  { 152.866621959147f,    13.8190092060518f},
  { -155.731780824440f,    21.7527747654215f},
  { -164.863201996450f,    22.9044641727044f},
  { -174.847576765617f,    23.5536156829348f},
  { 174.847576765617f,    23.5536156829348f},
  { 164.863201996450f,    22.9044641727044f},
  { 155.731780824440f,    21.7527747654215f},
  { -159.094842552111f,    30.0000000000000f},
  { -169.187683036428f,    31.2603465080791f},
  { 180.0f,    31.7174744114610f},
  { 169.187683036428f,    31.2603465080791f},
  { 159.094842552111f,    30.0000000000000f},
  { -163.067562084874f,    38.1462352379972f},
  { -174.205428060441f,    39.2419529905605f},
  { 174.205428060441f,    39.2419529905605f},
  { 163.067562084874f,    38.1462352379972f},
  { -167.781680133751f,    45.7717617455715f},
  { 180.0f,    46.4276504811178f},
  { 167.781680133751f,    45.7717617455715f},
  { -173.381241227950f,    52.5480139114809f},
  { 173.381241227950f,    52.5480139114809f},
  { -150.408004970470f,    -6.51186878797759f},
  { -152.866621959147f,    -13.8190092060518f},
  { -155.731780824440f,    -21.7527747654215f},
  { -159.094842552111f,    -30.0000000000000f},
  { -163.067562084874f,    -38.1462352379972f},
  { -167.781680133751f,    -45.7717617455715f},
  { -173.381241227950f,    -52.5480139114809f},
  { -157.920617868076f,    -6.93543325054564f},
  { -161.137740274699f,    -14.6573954710189f},
  { -164.863201996450f,    -22.9044641727044f},
  { -169.187683036428f,    -31.2603465080791f},
  { -174.205428060441f,    -39.2419529905605f},
  { -180.0f,    -46.4276504811178f},
  { 173.381241227950f,    -52.5480139114809f},
  { -166.321118751438f,    -7.26850261623438f},
  { -170.306276104685f,    -15.2401622826778f},
  { -174.847576765617f,    -23.5536156829348f},
  { 180.0f,    -31.7174744114610f},
  { 174.205428060441f,    -39.2419529905605f},
  { 167.781680133751f,    -45.7717617455715f},
  { -175.361879877246f,    -7.45410055983883f},
  { 180.0f,    -15.4504370863258f},
  { 174.847576765617f,    -23.5536156829348f},
  { 169.187683036428f,    -31.2603465080791f},
  { 163.067562084874f,    -38.1462352379972f},
  { 175.361879877246f,    -7.45410055983883f},
  { 170.306276104685f,    -15.2401622826778f},
  { 164.863201996450f,    -22.9044641727044f},
  { 159.094842552111f,    -30.0000000000000f},
  { 166.321118751438f,    -7.26850261623438f},
  { 161.137740274699f,    -14.6573954710189f},
  { 155.731780824440f,    -21.7527747654215f},
  { 157.920617868076f,    -6.93543325054564f},
  { 152.866621959147f,    -13.8190092060518f},
  { 150.408004970470f,    -6.51186878797759f},
  { 90.0f,    24.8689251184620f},
  { 90.0f,    17.1720378505138f},
  { 90.0f,    8.78323060095898f},
  { 90.0f,    0.0f},
  { 90.0f,    -8.78323060095898f},
  { 90.0f,    -17.1720378505138f},
  { 90.0f,    -24.8689251184620f},
  { 82.5216860448010f,    29.3822901816024f},
  { 82.5216860448010f,    21.9094235462107f},
  { 82.5216860448010f,    13.5668491634509f},
  { 82.5216860448010f,    4.59883911411213f},
  { 82.5216860448010f,    -4.59883911411213f},
  { 82.5216860448010f,    -13.5668491634508f},
  { 82.5216860448010f,    -21.9094235462107f},
  { 82.5216860448010f,    -29.3822901816024f},
  { 74.5495629136742f,    26.2866766550738f},
  { 74.5495629136742f,    18.2264204988720f},
  { 74.5495629136742f,    9.34970354257458f},
  { 74.5495629136742f,    0.0f},
  { 74.5495629136742f,    -9.34970354257458f},
  { 74.5495629136742f,    -18.2264204988720f},
  { 74.5495629136742f,    -26.2866766550738f},
  { 66.3612884340067f,    22.4415960116381f},
  { 66.3612884340067f,    13.9181885789507f},
  { 66.3612884340067f,    4.72213775783758f},
  { 66.3612884340067f,    -4.72213775783758f},
  { 66.3612884340067f,    -13.9181885789507f},
  { 66.3612884340067f,    -22.4415960116381f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    9.22764488160650f},
  { 58.2825255885390f,    0.0f},
  { 58.2825255885390f,    -9.22764488160650f},
  { 58.2825255885390f,    -18.0f},
  { 50.6141704736363f,    13.2408470050412f},
  { 50.6141704736363f,    4.48472037239199f},
  { 50.6141704736363f,    -4.48472037239199f},
  { 50.6141704736363f,    -13.2408470050412f},
  { 43.5723495188822f,    8.48907956344660f},
  { 43.5723495188822f,    0.0f},
  { 43.5723495188822f,    -8.48907956344660f},
  { 37.2671844918060f,    4.01919298814923f},
  { 37.2671844918060f,    -4.01919298814923f},
  { 97.4783139551990f,    29.3822901816024f},
  { 105.450437086326f,    26.2866766550738f},
  { 113.638711565993f,    22.4415960116381f},
  { 121.717474411461f,    18.0f},
  { 129.385829526364f,    13.2408470050412f},
  { 136.427650481118f,    8.48907956344661f},
  { 142.732815508194f,    4.01919298814923f},
  { 97.4783139551990f,    21.9094235462107f},
  { 105.450437086326f,    18.2264204988720f},
  { 113.638711565993f,    13.9181885789507f},
  { 121.717474411461f,    9.22764488160650f},
  { 129.385829526364f,    4.48472037239199f},
  { 136.427650481118f,    3.61186426073483e-15f},
  { 142.732815508194f,    -4.01919298814923f},
  { 97.4783139551990f,    13.5668491634509f},
  { 105.450437086326f,    9.34970354257458f},
  { 113.638711565993f,    4.72213775783758f},
  { 121.717474411461f,    0.0f},
  { 129.385829526364f,    -4.48472037239199f},
  { 136.427650481118f,    -8.48907956344661f},
  { 97.4783139551990f,    4.59883911411213f},
  { 105.450437086326f,    -1.99215638435455e-15f},
  { 113.638711565993f,    -4.72213775783758f},
  { 121.717474411461f,    -9.22764488160650f},
  { 129.385829526364f,    -13.2408470050412f},
  { 97.4783139551990f,    -4.59883911411213f},
  { 105.450437086326f,    -9.34970354257458f},
  { 113.638711565993f,    -13.9181885789507f},
  { 121.717474411461f,    -18.0f},
  { 97.4783139551990f,    -13.5668491634508f},
  { 105.450437086326f,    -18.2264204988720f},
  { 113.638711565993f,    -22.4415960116381f},
  { 97.4783139551990f,    -21.9094235462107f},
  { 105.450437086326f,    -26.2866766550738f},
  { 97.4783139551990f,    -29.3822901816024f},
  { -82.5216860448010f,    29.3822901816024f},
  { -74.5495629136742f,    26.2866766550738f},
  { -66.3612884340067f,    22.4415960116381f},
  { -58.2825255885390f,    18.0f},
  { -50.6141704736363f,    13.2408470050412f},
  { -43.5723495188822f,    8.48907956344660f},
  { -37.2671844918060f,    4.01919298814923f},
  { -90.0f,    24.8689251184620f},
  { -82.5216860448010f,    21.9094235462107f},
  { -74.5495629136742f,    18.2264204988720f},
  { -66.3612884340067f,    13.9181885789507f},
  { -58.2825255885390f,    9.22764488160650f},
  { -50.6141704736363f,    4.48472037239199f},
  { -43.5723495188822f,    3.61186426073483e-15f},
  { -37.2671844918060f,    -4.01919298814923f},
  { -90.0f,    17.1720378505138f},
  { -82.5216860448010f,    13.5668491634509f},
  { -74.5495629136742f,    9.34970354257458f},
  { -66.3612884340067f,    4.72213775783758f},
  { -58.2825255885390f,    0.0f},
  { -50.6141704736363f,    -4.48472037239199f},
  { -43.5723495188822f,    -8.48907956344661f},
  { -90.0f,    8.78323060095898f},
  { -82.5216860448010f,    4.59883911411213f},
  { -74.5495629136742f,    -1.99215638435455e-15f},
  { -66.3612884340067f,    -4.72213775783758f},
  { -58.2825255885390f,    -9.22764488160650f},
  { -50.6141704736363f,    -13.2408470050412f},
  { -90.0f,    0.0f},
  { -82.5216860448010f,    -4.59883911411213f},
  { -74.5495629136742f,    -9.34970354257458f},
  { -66.3612884340067f,    -13.9181885789507f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -8.78323060095898f},
  { -82.5216860448010f,    -13.5668491634508f},
  { -74.5495629136742f,    -18.2264204988720f},
  { -66.3612884340067f,    -22.4415960116381f},
  { -90.0f,    -17.1720378505138f},
  { -82.5216860448010f,    -21.9094235462107f},
  { -74.5495629136742f,    -26.2866766550738f},
  { -90.0f,    -24.8689251184620f},
  { -82.5216860448010f,    -29.3822901816024f},
  { -97.4783139551990f,    29.3822901816024f},
  { -97.4783139551990f,    21.9094235462107f},
  { -97.4783139551990f,    13.5668491634509f},
  { -97.4783139551990f,    4.59883911411213f},
  { -97.4783139551990f,    -4.59883911411213f},
  { -97.4783139551990f,    -13.5668491634508f},
  { -97.4783139551990f,    -21.9094235462107f},
  { -97.4783139551990f,    -29.3822901816024f},
  { -105.450437086326f,    26.2866766550738f},
  { -105.450437086326f,    18.2264204988720f},
  { -105.450437086326f,    9.34970354257458f},
  { -105.450437086326f,    0.0f},
  { -105.450437086326f,    -9.34970354257458f},
  { -105.450437086326f,    -18.2264204988720f},
  { -105.450437086326f,    -26.2866766550738f},
  { -113.638711565993f,    22.4415960116381f},
  { -113.638711565993f,    13.9181885789507f},
  { -113.638711565993f,    4.72213775783758f},
  { -113.638711565993f,    -4.72213775783758f},
  { -113.638711565993f,    -13.9181885789507f},
  { -113.638711565993f,    -22.4415960116381f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    9.22764488160650f},
  { -121.717474411461f,    0.0f},
  { -121.717474411461f,    -9.22764488160650f},
  { -121.717474411461f,    -18.0f},
  { -129.385829526364f,    13.2408470050412f},
  { -129.385829526364f,    4.48472037239199f},
  { -129.385829526364f,    -4.48472037239199f},
  { -129.385829526364f,    -13.2408470050412f},
  { -136.427650481118f,    8.48907956344661f},
  { -136.427650481118f,    0.0f},
  { -136.427650481118f,    -8.48907956344661f},
  { -142.732815508194f,    4.01919298814923f},
  { -142.732815508194f,    -4.01919298814923f},
  { 76.2613434253436f,    34.6470309014331f},
  { 67.2262996676901f,    31.1637336664207f},
  { 58.2825255885390f,    26.7723551183935f},
  { 49.8383783329354f,    21.7321521847258f},
  { 42.1778722582693f,    16.4337077246310f},
  { 35.4288203482238f,    11.2668552009329f},
  { 68.2966481036082f,    40.0910997872291f},
  { 58.2825255885390f,    36.0f},
  { 48.8895122619748f,    30.8889289128016f},
  { 40.5007050124200f,    25.1712621719368f},
  { 33.2604381086868f,    19.3435614209637f},
  { 58.2825255885390f,    45.2276448816065f},
  { 47.7031324115841f,    40.2840762101672f},
  { 38.4491770335041f,    34.3505184589462f},
  { 30.6726725373555f,    27.9936984541351f},
  { 46.1788380339297f,    49.4169517893457f},
  { 35.8897546595841f,    43.4870777455218f},
  { 27.5451941633379f,    36.8059220640418f},
  { 32.6224749368397f,    52.0905755468793f},
  { 23.7154719527922f,    45.2992117296112f},
  { 18.9659334090314f,    53.0473569347008f},
  { 76.2613434253436f,    -34.6470309014331f},
  { 68.2966481036082f,    -40.0910997872291f},
  { 58.2825255885390f,    -45.2276448816065f},
  { 46.1788380339297f,    -49.4169517893457f},
  { 32.6224749368397f,    -52.0905755468793f},
  { 18.9659334090314f,    -53.0473569347008f},
  { 67.2262996676901f,    -31.1637336664207f},
  { 58.2825255885390f,    -36.0f},
  { 47.7031324115841f,    -40.2840762101672f},
  { 35.8897546595841f,    -43.4870777455218f},
  { 23.7154719527922f,    -45.2992117296112f},
  { 58.2825255885390f,    -26.7723551183935f},
  { 48.8895122619748f,    -30.8889289128016f},
  { 38.4491770335041f,    -34.3505184589462f},
  { 27.5451941633379f,    -36.8059220640418f},
  { 49.8383783329354f,    -21.7321521847258f},
  { 40.5007050124200f,    -25.1712621719368f},
  { 30.6726725373555f,    -27.9936984541351f},
  { 42.1778722582693f,    -16.4337077246310f},
  { 33.2604381086868f,    -19.3435614209637f},
  { 35.4288203482238f,    -11.2668552009329f},
  { -18.9659334090314f,    53.0473569347008f},
  { -23.7154719527922f,    45.2992117296112f},
  { -27.5451941633379f,    36.8059220640418f},
  { -30.6726725373555f,    27.9936984541352f},
  { -33.2604381086868f,    19.3435614209637f},
  { -35.4288203482238f,    11.2668552009329f},
  { -32.6224749368397f,    52.0905755468793f},
  { -35.8897546595841f,    43.4870777455218f},
  { -38.4491770335041f,    34.3505184589462f},
  { -40.5007050124200f,    25.1712621719368f},
  { -42.1778722582693f,    16.4337077246310f},
  { -46.1788380339297f,    49.4169517893457f},
  { -47.7031324115841f,    40.2840762101672f},
  { -48.8895122619748f,    30.8889289128016f},
  { -49.8383783329354f,    21.7321521847258f},
  { -58.2825255885390f,    45.2276448816065f},
  { -58.2825255885390f,    36.0f},
  { -58.2825255885390f,    26.7723551183935f},
  { -68.2966481036082f,    40.0910997872291f},
  { -67.2262996676901f,    31.1637336664207f},
  { -76.2613434253436f,    34.6470309014331f},
  { -18.9659334090314f,    -53.0473569347008f},
  { -32.6224749368397f,    -52.0905755468793f},
  { -46.1788380339297f,    -49.4169517893457f},
  { -58.2825255885390f,    -45.2276448816065f},
  { -68.2966481036082f,    -40.0910997872291f},
  { -76.2613434253436f,    -34.6470309014331f},
  { -23.7154719527922f,    -45.2992117296112f},
  { -35.8897546595841f,    -43.4870777455218f},
  { -47.7031324115841f,    -40.2840762101672f},
  { -58.2825255885390f,    -36.0f},
  { -67.2262996676901f,    -31.1637336664207f},
  { -27.5451941633379f,    -36.8059220640418f},
  { -38.4491770335041f,    -34.3505184589462f},
  { -48.8895122619748f,    -30.8889289128016f},
  { -58.2825255885390f,    -26.7723551183935f},
  { -30.6726725373555f,    -27.9936984541352f},
  { -40.5007050124200f,    -25.1712621719368f},
  { -49.8383783329354f,    -21.7321521847258f},
  { -33.2604381086868f,    -19.3435614209637f},
  { -42.1778722582693f,    -16.4337077246310f},
  { -35.4288203482238f,    -11.2668552009329f},
  { 161.034066590969f,    53.0473569347008f},
  { 156.284528047208f,    45.2992117296112f},
  { 152.454805836662f,    36.8059220640418f},
  { 149.327327462644f,    27.9936984541352f},
  { 146.739561891313f,    19.3435614209637f},
  { 144.571179651776f,    11.2668552009329f},
  { 147.377525063160f,    52.0905755468793f},
  { 144.110245340416f,    43.4870777455218f},
  { 141.550822966496f,    34.3505184589462f},
  { 139.499294987580f,    25.1712621719368f},
  { 137.822127741731f,    16.4337077246310f},
  { 133.821161966070f,    49.4169517893457f},
  { 132.296867588416f,    40.2840762101672f},
  { 131.110487738025f,    30.8889289128016f},
  { 130.161621667065f,    21.7321521847258f},
  { 121.717474411461f,    45.2276448816065f},
  { 121.717474411461f,    36.0f},
  { 121.717474411461f,    26.7723551183935f},
  { 111.703351896392f,    40.0910997872291f},
  { 112.773700332310f,    31.1637336664207f},
  { 103.738656574656f,    34.6470309014331f},
  { 161.034066590969f,    -53.0473569347008f},
  { 147.377525063160f,    -52.0905755468793f},
  { 133.821161966070f,    -49.4169517893457f},
  { 121.717474411461f,    -45.2276448816065f},
  { 111.703351896392f,    -40.0910997872291f},
  { 103.738656574656f,    -34.6470309014331f},
  { 156.284528047208f,    -45.2992117296112f},
  { 144.110245340416f,    -43.4870777455218f},
  { 132.296867588416f,    -40.2840762101672f},
  { 121.717474411461f,    -36.0f},
  { 112.773700332310f,    -31.1637336664207f},
  { 152.454805836662f,    -36.8059220640418f},
  { 141.550822966496f,    -34.3505184589462f},
  { 131.110487738025f,    -30.8889289128016f},
  { 121.717474411461f,    -26.7723551183935f},
  { 149.327327462644f,    -27.9936984541352f},
  { 139.499294987580f,    -25.1712621719368f},
  { 130.161621667065f,    -21.7321521847258f},
  { 146.739561891313f,    -19.3435614209637f},
  { 137.822127741731f,    -16.4337077246310f},
  { 144.571179651776f,    -11.2668552009329f},
  { -161.034066590969f,    53.0473569347008f},
  { -147.377525063160f,    52.0905755468793f},
  { -133.821161966070f,    49.4169517893457f},
  { -121.717474411461f,    45.2276448816065f},
  { -111.703351896392f,    40.0910997872291f},
  { -103.738656574656f,    34.6470309014331f},
  { -156.284528047208f,    45.2992117296112f},
  { -144.110245340416f,    43.4870777455218f},
  { -132.296867588416f,    40.2840762101672f},
  { -121.717474411461f,    36.0f},
  { -112.773700332310f,    31.1637336664207f},
  { -152.454805836662f,    36.8059220640418f},
  { -141.550822966496f,    34.3505184589462f},
  { -131.110487738025f,    30.8889289128016f},
  { -121.717474411461f,    26.7723551183935f},
  { -149.327327462644f,    27.9936984541352f},
  { -139.499294987580f,    25.1712621719368f},
  { -130.161621667065f,    21.7321521847258f},
  { -146.739561891313f,    19.3435614209637f},
  { -137.822127741731f,    16.4337077246310f},
  { -144.571179651776f,    11.2668552009329f},
  { -161.034066590969f,    -53.0473569347008f},
  { -156.284528047208f,    -45.2992117296112f},
  { -152.454805836662f,    -36.8059220640418f},
  { -149.327327462644f,    -27.9936984541352f},
  { -146.739561891313f,    -19.3435614209637f},
  { -144.571179651776f,    -11.2668552009329f},
  { -147.377525063160f,    -52.0905755468793f},
  { -144.110245340416f,    -43.4870777455218f},
  { -141.550822966496f,    -34.3505184589462f},
  { -139.499294987580f,    -25.1712621719368f},
  { -137.822127741731f,    -16.4337077246310f},
  { -133.821161966070f,    -49.4169517893457f},
  { -132.296867588416f,    -40.2840762101672f},
  { -131.110487738025f,    -30.8889289128016f},
  { -130.161621667065f,    -21.7321521847258f},
  { -121.717474411461f,    -45.2276448816065f},
  { -121.717474411461f,    -36.0f},
  { -121.717474411461f,    -26.7723551183935f},
  { -111.703351896392f,    -40.0910997872291f},
  { -112.773700332310f,    -31.1637336664207f},
  { -103.738656574656f,    -34.6470309014331f}};

/* 3LD geosphere 'ico' [9 0] (812 points) */
const float __geosphere_ico_9_0_dirs_deg[812][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    64.3267273510545f},
  { 0.0f,    71.0499775117081f},
  { 0.0f,    78.3592768632294f},
  { 0.0f,    86.0716405780585f},
  { 180.0f,    86.0716405780585f},
  { 180.0f,    78.3592768632294f},
  { 180.0f,    71.0499775117081f},
  { 180.0f,    64.3267273510545f},
  { 11.4340697442537f,    59.6583225872452f},
  { 15.0920752881032f,    65.9795958419257f},
  { 22.0237505161462f,    72.8060416440631f},
  { 38.9734473368657f,    79.5471366108243f},
  { 90.0f,    83.3812412279501f},
  { 141.026552663134f,    79.5471366108243f},
  { 157.976249483854f,    72.8060416440631f},
  { 164.907924711897f,    65.9795958419257f},
  { 168.565930255746f,    59.6583225872452f},
  { 24.8108847601265f,    59.9422780929273f},
  { 32.9114931857064f,    65.9188666651293f},
  { 47.1679114977869f,    71.6787159647749f},
  { 72.8279621494862f,    75.7394148284850f},
  { 107.172037850514f,    75.7394148284850f},
  { 132.832088502213f,    71.6787159647749f},
  { 147.088506814294f,    65.9188666651293f},
  { 155.189115239873f,    59.9422780929273f},
  { 38.9734473368657f,    58.7297899739513f},
  { 50.5099309427673f,    63.6671378427701f},
  { 67.6072290710451f,    67.5536234094196f},
  { 90.0000000000000f,    69.0948425521107f},
  { 112.392770928955f,    67.5536234094196f},
  { 129.490069057233f,    63.6671378427701f},
  { 141.026552663134f,    58.7297899739513f},
  { 52.3123893000598f,    55.9231386404846f},
  { 65.1310748815380f,    59.4579173207107f},
  { 81.2167693990410f,    61.5567324441311f},
  { 98.7832306009590f,    61.5567324441311f},
  { 114.868925118462f,    59.4579173207107f},
  { 127.687610699940f,    55.9231386404846f},
  { 63.6909565444517f,    51.8082310918389f},
  { 76.1141128518623f,    54.0041745863830f},
  { 90.0f,    54.8084438762714f},
  { 103.885887148138f,    54.0041745863830f},
  { 116.309043455548f,    51.8082310918389f},
  { 72.8279621494862f,    46.8885380691238f},
  { 84.1189512527134f,    48.0393989970589f},
  { 95.8810487472866f,    48.0393989970589f},
  { 107.172037850514f,    46.8885380691238f},
  { 79.9858774849308f,    41.6684303531219f},
  { 90.0f,    42.1056437002381f},
  { 100.014122515069f,    41.6684303531219f},
  { 85.5824320004692f,    36.5320305814123f},
  { 94.4175679995308f,    36.5320305814123f},
  { -11.4340697442537f,    59.6583225872452f},
  { -24.8108847601265f,    59.9422780929273f},
  { -38.9734473368657f,    58.7297899739513f},
  { -52.3123893000598f,    55.9231386404846f},
  { -63.6909565444517f,    51.8082310918389f},
  { -72.8279621494862f,    46.8885380691238f},
  { -79.9858774849308f,    41.6684303531219f},
  { -85.5824320004692f,    36.5320305814123f},
  { -15.0920752881032f,    65.9795958419257f},
  { -32.9114931857064f,    65.9188666651293f},
  { -50.5099309427673f,    63.6671378427701f},
  { -65.1310748815380f,    59.4579173207107f},
  { -76.1141128518623f,    54.0041745863830f},
  { -84.1189512527134f,    48.0393989970589f},
  { -90.0f,    42.1056437002381f},
  { -94.4175679995308f,    36.5320305814124f},
  { -22.0237505161462f,    72.8060416440631f},
  { -47.1679114977869f,    71.6787159647749f},
  { -67.6072290710451f,    67.5536234094196f},
  { -81.2167693990410f,    61.5567324441311f},
  { -90.0f,    54.8084438762714f},
  { -95.8810487472866f,    48.0393989970589f},
  { -100.014122515069f,    41.6684303531219f},
  { -38.9734473368657f,    79.5471366108243f},
  { -72.8279621494862f,    75.7394148284850f},
  { -90.0f,    69.0948425521107f},
  { -98.7832306009590f,    61.5567324441311f},
  { -103.885887148138f,    54.0041745863831f},
  { -107.172037850514f,    46.8885380691238f},
  { -90.0f,    83.3812412279501f},
  { -107.172037850514f,    75.7394148284850f},
  { -112.392770928955f,    67.5536234094196f},
  { -114.868925118462f,    59.4579173207107f},
  { -116.309043455548f,    51.8082310918389f},
  { -141.026552663134f,    79.5471366108243f},
  { -132.832088502213f,    71.6787159647749f},
  { -129.490069057233f,    63.6671378427701f},
  { -127.687610699940f,    55.9231386404846f},
  { -157.976249483854f,    72.8060416440631f},
  { -147.088506814294f,    65.9188666651293f},
  { -141.026552663134f,    58.7297899739513f},
  { -164.907924711897f,    65.9795958419257f},
  { -155.189115239873f,    59.9422780929273f},
  { -168.565930255746f,    59.6583225872452f},
  { 11.4340697442537f,    -59.6583225872452f},
  { 24.8108847601265f,    -59.9422780929273f},
  { 38.9734473368657f,    -58.7297899739513f},
  { 52.3123893000598f,    -55.9231386404846f},
  { 63.6909565444517f,    -51.8082310918389f},
  { 72.8279621494862f,    -46.8885380691238f},
  { 79.9858774849308f,    -41.6684303531219f},
  { 85.5824320004692f,    -36.5320305814123f},
  { 0.0f,    -64.3267273510545f},
  { 15.0920752881032f,    -65.9795958419257f},
  { 32.9114931857064f,    -65.9188666651293f},
  { 50.5099309427673f,    -63.6671378427701f},
  { 65.1310748815380f,    -59.4579173207107f},
  { 76.1141128518623f,    -54.0041745863830f},
  { 84.1189512527134f,    -48.0393989970589f},
  { 90.0f,    -42.1056437002381f},
  { 94.4175679995308f,    -36.5320305814124f},
  { 0.0f,    -71.0499775117081f},
  { 22.0237505161462f,    -72.8060416440631f},
  { 47.1679114977869f,    -71.6787159647749f},
  { 67.6072290710451f,    -67.5536234094196f},
  { 81.2167693990410f,    -61.5567324441311f},
  { 90.0f,    -54.8084438762714f},
  { 95.8810487472866f,    -48.0393989970589f},
  { 100.014122515069f,    -41.6684303531219f},
  { 0.0f,    -78.3592768632294f},
  { 38.9734473368657f,    -79.5471366108243f},
  { 72.8279621494862f,    -75.7394148284850f},
  { 90.0f,    -69.0948425521107f},
  { 98.7832306009590f,    -61.5567324441311f},
  { 103.885887148138f,    -54.0041745863831f},
  { 107.172037850514f,    -46.8885380691238f},
  { 0.0f,    -86.0716405780585f},
  { 90.0f,    -83.3812412279501f},
  { 107.172037850514f,    -75.7394148284850f},
  { 112.392770928955f,    -67.5536234094196f},
  { 114.868925118462f,    -59.4579173207107f},
  { 116.309043455548f,    -51.8082310918389f},
  { 180.0f,    -86.0716405780585f},
  { 141.026552663134f,    -79.5471366108243f},
  { 132.832088502213f,    -71.6787159647749f},
  { 129.490069057233f,    -63.6671378427701f},
  { 127.687610699940f,    -55.9231386404846f},
  { 180.0f,    -78.3592768632294f},
  { 157.976249483854f,    -72.8060416440631f},
  { 147.088506814294f,    -65.9188666651293f},
  { 141.026552663134f,    -58.7297899739513f},
  { 180.0f,    -71.0499775117081f},
  { 164.907924711897f,    -65.9795958419257f},
  { 155.189115239873f,    -59.9422780929273f},
  { 180.0f,    -64.3267273510545f},
  { 168.565930255746f,    -59.6583225872452f},
  { -11.4340697442537f,    -59.6583225872452f},
  { -15.0920752881032f,    -65.9795958419257f},
  { -22.0237505161462f,    -72.8060416440631f},
  { -38.9734473368657f,    -79.5471366108243f},
  { -90.0f,    -83.3812412279501f},
  { -141.026552663134f,    -79.5471366108243f},
  { -157.976249483854f,    -72.8060416440631f},
  { -164.907924711897f,    -65.9795958419257f},
  { -168.565930255746f,    -59.6583225872452f},
  { -24.8108847601265f,    -59.9422780929273f},
  { -32.9114931857064f,    -65.9188666651293f},
  { -47.1679114977869f,    -71.6787159647749f},
  { -72.8279621494862f,    -75.7394148284850f},
  { -107.172037850514f,    -75.7394148284850f},
  { -132.832088502213f,    -71.6787159647749f},
  { -147.088506814294f,    -65.9188666651293f},
  { -155.189115239873f,    -59.9422780929273f},
  { -38.9734473368657f,    -58.7297899739513f},
  { -50.5099309427673f,    -63.6671378427701f},
  { -67.6072290710451f,    -67.5536234094196f},
  { -90.0000000000000f,    -69.0948425521107f},
  { -112.392770928955f,    -67.5536234094196f},
  { -129.490069057233f,    -63.6671378427701f},
  { -141.026552663134f,    -58.7297899739513f},
  { -52.3123893000598f,    -55.9231386404846f},
  { -65.1310748815380f,    -59.4579173207107f},
  { -81.2167693990410f,    -61.5567324441311f},
  { -98.7832306009590f,    -61.5567324441311f},
  { -114.868925118462f,    -59.4579173207107f},
  { -127.687610699940f,    -55.9231386404846f},
  { -63.6909565444517f,    -51.8082310918389f},
  { -76.1141128518623f,    -54.0041745863830f},
  { -90.0f,    -54.8084438762714f},
  { -103.885887148138f,    -54.0041745863830f},
  { -116.309043455548f,    -51.8082310918389f},
  { -72.8279621494862f,    -46.8885380691238f},
  { -84.1189512527134f,    -48.0393989970589f},
  { -95.8810487472866f,    -48.0393989970589f},
  { -107.172037850514f,    -46.8885380691238f},
  { -79.9858774849308f,    -41.6684303531219f},
  { -90.0f,    -42.1056437002381f},
  { -100.014122515069f,    -41.6684303531219f},
  { -85.5824320004692f,    -36.5320305814123f},
  { -94.4175679995308f,    -36.5320305814123f},
  { 25.6732726489455f,    0.0f},
  { 18.9500224882919f,    0.0f},
  { 11.6407231367706f,    0.0f},
  { 3.92835942194152f,    0.0f},
  { -3.92835942194152f,    0.0f},
  { -11.6407231367706f,    0.0f},
  { -18.9500224882919f,    0.0f},
  { -25.6732726489455f,    0.0f},
  { 29.8434225696728f,    5.74735530909063f},
  { 23.2814462735412f,    6.08404140398331f},
  { 16.0058633963009f,    6.36431979943284f},
  { 8.16215723942556f,    6.55229816511686f},
  { 0.0f,    6.61875877204995f},
  { -8.16215723942556f,    6.55229816511686f},
  { -16.0058633963009f,    6.36431979943284f},
  { -23.2814462735412f,    6.08404140398331f},
  { -29.8434225696728f,    5.74735530909063f},
  { 27.7120267179463f,    12.1327993265071f},
  { 20.5660741025339f,    12.8089082327083f},
  { 12.6869165471497f,    13.3279681548719f},
  { 4.29142851321895f,    13.6123425501156f},
  { -4.29142851321895f,    13.6123425501156f},
  { -12.6869165471497f,    13.3279681548719f},
  { -20.5660741025339f,    12.8089082327083f},
  { -27.7120267179463f,    12.1327993265071f},
  { 25.2736805860404f,    19.0551045548580f},
  { 17.4717761720319f,    20.0189075682215f},
  { 8.94377407915109f,    20.6725861219905f},
  { 4.28447409905487e-15f,    20.9051574478893f},
  { -8.94377407915109f,    20.6725861219905f},
  { -17.4717761720319f,    20.0189075682215f},
  { -25.2736805860404f,    19.0551045548580f},
  { 22.4679869150732f,    26.3210305496768f},
  { 13.9356538353420f,    27.4548510647047f},
  { 4.72827555958840f,    28.0799376047285f},
  { -4.72827555958839f,    28.0799376047285f},
  { -13.9356538353420f,    27.4548510647047f},
  { -22.4679869150732f,    26.3210305496768f},
  { 19.2221769999903f,    33.6590909637086f},
  { 9.88929693375803f,    34.7886137215967f},
  { 0.0f,    35.1915561237286f},
  { -9.88929693375803f,    34.7886137215967f},
  { -19.2221769999903f,    33.6590909637086f},
  { 15.4504370863258f,    40.7647476003615f},
  { 5.26386143430317f,    41.6900227136846f},
  { -5.26386143430316f,    41.6900227136846f},
  { -15.4504370863258f,    40.7647476003615f},
  { 11.0555976958256f,    47.3599639711825f},
  { 0.0f,    47.8943562997619f},
  { -11.0555976958256f,    47.3599639711825f},
  { 5.93579226409057f,    53.2388232386656f},
  { -5.93579226409057f,    53.2388232386656f},
  { 29.8434225696728f,    -5.74735530909063f},
  { 27.7120267179463f,    -12.1327993265071f},
  { 25.2736805860404f,    -19.0551045548580f},
  { 22.4679869150732f,    -26.3210305496768f},
  { 19.2221769999903f,    -33.6590909637086f},
  { 15.4504370863258f,    -40.7647476003615f},
  { 11.0555976958256f,    -47.3599639711825f},
  { 5.93579226409057f,    -53.2388232386656f},
  { 23.2814462735412f,    -6.08404140398331f},
  { 20.5660741025339f,    -12.8089082327083f},
  { 17.4717761720319f,    -20.0189075682215f},
  { 13.9356538353420f,    -27.4548510647047f},
  { 9.88929693375803f,    -34.7886137215967f},
  { 5.26386143430317f,    -41.6900227136846f},
  { 0.0f,    -47.8943562997619f},
  { -5.93579226409056f,    -53.2388232386656f},
  { 16.0058633963009f,    -6.36431979943284f},
  { 12.6869165471497f,    -13.3279681548719f},
  { 8.94377407915109f,    -20.6725861219905f},
  { 4.72827555958840f,    -28.0799376047285f},
  { 4.74610576512321e-15f,    -35.1915561237286f},
  { -5.26386143430317f,    -41.6900227136846f},
  { -11.0555976958256f,    -47.3599639711825f},
  { 8.16215723942556f,    -6.55229816511685f},
  { 4.29142851321895f,    -13.6123425501156f},
  { 2.14223704952744e-15f,    -20.9051574478893f},
  { -4.72827555958840f,    -28.0799376047285f},
  { -9.88929693375803f,    -34.7886137215967f},
  { -15.4504370863258f,    -40.7647476003615f},
  { 9.76170913507095e-16f,    -6.61875877204995f},
  { -4.29142851321895f,    -13.6123425501156f},
  { -8.94377407915109f,    -20.6725861219905f},
  { -13.9356538353420f,    -27.4548510647047f},
  { -19.2221769999903f,    -33.6590909637086f},
  { -8.16215723942556f,    -6.55229816511686f},
  { -12.6869165471497f,    -13.3279681548719f},
  { -17.4717761720319f,    -20.0189075682215f},
  { -22.4679869150732f,    -26.3210305496768f},
  { -16.0058633963009f,    -6.36431979943284f},
  { -20.5660741025339f,    -12.8089082327083f},
  { -25.2736805860404f,    -19.0551045548580f},
  { -23.2814462735412f,    -6.08404140398331f},
  { -27.7120267179463f,    -12.1327993265071f},
  { -29.8434225696728f,    -5.74735530909063f},
  { -154.326727351054f,    0.0f},
  { -161.049977511708f,    0.0f},
  { -168.359276863229f,    0.0f},
  { -176.071640578058f,    0.0f},
  { 176.071640578058f,    0.0f},
  { 168.359276863229f,    0.0f},
  { 161.049977511708f,    0.0f},
  { 154.326727351054f,    0.0f},
  { -150.156577430327f,    5.74735530909063f},
  { -156.718553726459f,    6.08404140398331f},
  { -163.994136603699f,    6.36431979943284f},
  { -171.837842760574f,    6.55229816511686f},
  { 180.0f,    6.61875877204995f},
  { 171.837842760574f,    6.55229816511686f},
  { 163.994136603699f,    6.36431979943284f},
  { 156.718553726459f,    6.08404140398331f},
  { 150.156577430327f,    5.74735530909063f},
  { -152.287973282054f,    12.1327993265071f},
  { -159.433925897466f,    12.8089082327083f},
  { -167.313083452850f,    13.3279681548719f},
  { -175.708571486781f,    13.6123425501156f},
  { 175.708571486781f,    13.6123425501156f},
  { 167.313083452850f,    13.3279681548719f},
  { 159.433925897466f,    12.8089082327083f},
  { 152.287973282054f,    12.1327993265071f},
  { -154.726319413960f,    19.0551045548580f},
  { -162.528223827968f,    20.0189075682215f},
  { -171.056225920849f,    20.6725861219905f},
  { -180.0f,    20.9051574478893f},
  { 171.056225920849f,    20.6725861219905f},
  { 162.528223827968f,    20.0189075682215f},
  { 154.726319413960f,    19.0551045548580f},
  { -157.532013084927f,    26.3210305496768f},
  { -166.064346164658f,    27.4548510647047f},
  { -175.271724440412f,    28.0799376047285f},
  { 175.271724440412f,    28.0799376047285f},
  { 166.064346164658f,    27.4548510647047f},
  { 157.532013084927f,    26.3210305496768f},
  { -160.777823000010f,    33.6590909637086f},
  { -170.110703066242f,    34.7886137215967f},
  { 180.0f,    35.1915561237286f},
  { 170.110703066242f,    34.7886137215967f},
  { 160.777823000010f,    33.6590909637086f},
  { -164.549562913674f,    40.7647476003615f},
  { -174.736138565697f,    41.6900227136846f},
  { 174.736138565697f,    41.6900227136846f},
  { 164.549562913674f,    40.7647476003615f},
  { -168.944402304174f,    47.3599639711825f},
  { 180.0f,    47.8943562997619f},
  { 168.944402304174f,    47.3599639711825f},
  { -174.064207735909f,    53.2388232386656f},
  { 174.064207735909f,    53.2388232386656f},
  { -150.156577430327f,    -5.74735530909063f},
  { -152.287973282054f,    -12.1327993265071f},
  { -154.726319413960f,    -19.0551045548580f},
  { -157.532013084927f,    -26.3210305496768f},
  { -160.777823000010f,    -33.6590909637086f},
  { -164.549562913674f,    -40.7647476003615f},
  { -168.944402304174f,    -47.3599639711825f},
  { -174.064207735909f,    -53.2388232386656f},
  { -156.718553726459f,    -6.08404140398331f},
  { -159.433925897466f,    -12.8089082327083f},
  { -162.528223827968f,    -20.0189075682215f},
  { -166.064346164658f,    -27.4548510647047f},
  { -170.110703066242f,    -34.7886137215967f},
  { -174.736138565697f,    -41.6900227136846f},
  { 180.0f,    -47.8943562997619f},
  { 174.064207735909f,    -53.2388232386656f},
  { -163.994136603699f,    -6.36431979943284f},
  { -167.313083452850f,    -13.3279681548719f},
  { -171.056225920849f,    -20.6725861219905f},
  { -175.271724440412f,    -28.0799376047285f},
  { -180.0f,    -35.1915561237286f},
  { 174.736138565697f,    -41.6900227136846f},
  { 168.944402304174f,    -47.3599639711825f},
  { -171.837842760574f,    -6.55229816511685f},
  { -175.708571486781f,    -13.6123425501156f},
  { -180.0f,    -20.9051574478893f},
  { 175.271724440412f,    -28.0799376047285f},
  { 170.110703066242f,    -34.7886137215967f},
  { 164.549562913674f,    -40.7647476003615f},
  { -180.0f,    -6.61875877204995f},
  { 175.708571486781f,    -13.6123425501156f},
  { 171.056225920849f,    -20.6725861219905f},
  { 166.064346164658f,    -27.4548510647047f},
  { 160.777823000010f,    -33.6590909637086f},
  { 171.837842760574f,    -6.55229816511685f},
  { 167.313083452850f,    -13.3279681548719f},
  { 162.528223827968f,    -20.0189075682215f},
  { 157.532013084927f,    -26.3210305496768f},
  { 163.994136603699f,    -6.36431979943284f},
  { 159.433925897466f,    -12.8089082327083f},
  { 154.726319413960f,    -19.0551045548580f},
  { 156.718553726459f,    -6.08404140398331f},
  { 152.287973282054f,    -12.1327993265071f},
  { 150.156577430327f,    -5.74735530909063f},
  { 90.0f,    25.6732726489455f},
  { 90.0f,    18.9500224882919f},
  { 90.0f,    11.6407231367706f},
  { 90.0f,    3.92835942194152f},
  { 90.0f,    -3.92835942194152f},
  { 90.0f,    -11.6407231367706f},
  { 90.0f,    -18.9500224882919f},
  { 90.0f,    -25.6732726489455f},
  { 83.3812412279501f,    29.6783194887260f},
  { 83.3812412279501f,    23.1426578826804f},
  { 83.3812412279501f,    15.9045984655888f},
  { 83.3812412279501f,    8.10848294356066f},
  { 83.3812412279501f,    0.0f},
  { 83.3812412279501f,    -8.10848294356066f},
  { 83.3812412279501f,    -15.9045984655888f},
  { 83.3812412279501f,    -23.1426578826804f},
  { 83.3812412279501f,    -29.6783194887260f},
  { 76.3508154775154f,    27.0418117799956f},
  { 76.3508154775154f,    20.0320362052904f},
  { 76.3508154775154f,    12.3397512522743f},
  { 76.3508154775154f,    4.17066721420311f},
  { 76.3508154775154f,    -4.17066721420311f},
  { 76.3508154775154f,    -12.3397512522743f},
  { 76.3508154775154f,    -20.0320362052904f},
  { 76.3508154775154f,    -27.0418117799956f},
  { 69.0948425521107f,    23.8001826021751f},
  { 69.0948425521107f,    16.3853175925535f},
  { 69.0948425521107f,    8.36364070535862f},
  { 69.0948425521107f,    4.00243727602875e-15f},
  { 69.0948425521107f,    -8.36364070535862f},
  { 69.0948425521107f,    -16.3853175925535f},
  { 69.0948425521107f,    -23.8001826021750f},
  { 61.8388696267061f,    20.0320362052904f},
  { 61.8388696267061f,    12.3397512522743f},
  { 61.8388696267061f,    4.17066721420311f},
  { 61.8388696267061f,    -4.17066721420311f},
  { 61.8388696267061f,    -12.3397512522743f},
  { 61.8388696267061f,    -20.0320362052904f},
  { 54.8084438762714f,    15.9045984655888f},
  { 54.8084438762714f,    8.10848294356065f},
  { 54.8084438762714f,    0.0f},
  { 54.8084438762714f,    -8.10848294356065f},
  { 54.8084438762714f,    -15.9045984655888f},
  { 48.1896851042214f,    11.6407231367706f},
  { 48.1896851042214f,    3.92835942194152f},
  { 48.1896851042214f,    -3.92835942194152f},
  { 48.1896851042214f,    -11.6407231367706f},
  { 42.1056437002381f,    7.46365844000401f},
  { 42.1056437002381f,    0.0f},
  { 42.1056437002381f,    -7.46365844000401f},
  { 36.6136036344282f,    3.54837484763455f},
  { 36.6136036344282f,    -3.54837484763455f},
  { 96.6187587720500f,    29.6783194887260f},
  { 103.649184522485f,    27.0418117799956f},
  { 110.905157447889f,    23.8001826021751f},
  { 118.161130373294f,    20.0320362052904f},
  { 125.191556123729f,    15.9045984655888f},
  { 131.810314895779f,    11.6407231367706f},
  { 137.894356299762f,    7.46365844000401f},
  { 143.386396365572f,    3.54837484763455f},
  { 96.6187587720500f,    23.1426578826804f},
  { 103.649184522485f,    20.0320362052904f},
  { 110.905157447889f,    16.3853175925535f},
  { 118.161130373294f,    12.3397512522743f},
  { 125.191556123729f,    8.10848294356065f},
  { 131.810314895779f,    3.92835942194152f},
  { 137.894356299762f,    0.0f},
  { 143.386396365572f,    -3.54837484763454f},
  { 96.6187587720500f,    15.9045984655888f},
  { 103.649184522485f,    12.3397512522743f},
  { 110.905157447889f,    8.36364070535862f},
  { 118.161130373294f,    4.17066721420311f},
  { 125.191556123729f,    3.87865925636419e-15f},
  { 131.810314895779f,    -3.92835942194152f},
  { 137.894356299762f,    -7.46365844000401f},
  { 96.6187587720500f,    8.10848294356065f},
  { 103.649184522485f,    4.17066721420311f},
  { 110.905157447889f,    2.00121863801437e-15f},
  { 118.161130373294f,    -4.17066721420311f},
  { 125.191556123729f,    -8.10848294356065f},
  { 131.810314895779f,    -11.6407231367706f},
  { 96.6187587720500f,    9.69664814091048e-16f},
  { 103.649184522485f,    -4.17066721420311f},
  { 110.905157447889f,    -8.36364070535862f},
  { 118.161130373294f,    -12.3397512522743f},
  { 125.191556123729f,    -15.9045984655888f},
  { 96.6187587720500f,    -8.10848294356065f},
  { 103.649184522485f,    -12.3397512522743f},
  { 110.905157447889f,    -16.3853175925535f},
  { 118.161130373294f,    -20.0320362052904f},
  { 96.6187587720500f,    -15.9045984655888f},
  { 103.649184522485f,    -20.0320362052904f},
  { 110.905157447889f,    -23.8001826021751f},
  { 96.6187587720500f,    -23.1426578826804f},
  { 103.649184522485f,    -27.0418117799956f},
  { 96.6187587720500f,    -29.6783194887260f},
  { -83.3812412279501f,    29.6783194887260f},
  { -76.3508154775154f,    27.0418117799956f},
  { -69.0948425521107f,    23.8001826021751f},
  { -61.8388696267061f,    20.0320362052904f},
  { -54.8084438762714f,    15.9045984655888f},
  { -48.1896851042214f,    11.6407231367706f},
  { -42.1056437002381f,    7.46365844000401f},
  { -36.6136036344282f,    3.54837484763455f},
  { -90.0f,    25.6732726489455f},
  { -83.3812412279501f,    23.1426578826804f},
  { -76.3508154775154f,    20.0320362052904f},
  { -69.0948425521107f,    16.3853175925535f},
  { -61.8388696267061f,    12.3397512522743f},
  { -54.8084438762714f,    8.10848294356065f},
  { -48.1896851042214f,    3.92835942194152f},
  { -42.1056437002381f,    0.0f},
  { -36.6136036344282f,    -3.54837484763455f},
  { -90.0f,    18.9500224882919f},
  { -83.3812412279501f,    15.9045984655888f},
  { -76.3508154775154f,    12.3397512522743f},
  { -69.0948425521107f,    8.36364070535862f},
  { -61.8388696267061f,    4.17066721420311f},
  { -54.8084438762714f,    3.87865925636419e-15f},
  { -48.1896851042214f,    -3.92835942194152f},
  { -42.1056437002381f,    -7.46365844000401f},
  { -90.0f,    11.6407231367706f},
  { -83.3812412279501f,    8.10848294356065f},
  { -76.3508154775154f,    4.17066721420311f},
  { -69.0948425521107f,    2.00121863801437e-15f},
  { -61.8388696267061f,    -4.17066721420311f},
  { -54.8084438762714f,    -8.10848294356065f},
  { -48.1896851042214f,    -11.6407231367706f},
  { -90.0f,    3.92835942194152f},
  { -83.3812412279501f,    9.69664814091048e-16f},
  { -76.3508154775154f,    -4.17066721420311f},
  { -69.0948425521107f,    -8.36364070535862f},
  { -61.8388696267061f,    -12.3397512522743f},
  { -54.8084438762714f,    -15.9045984655888f},
  { -90.0f,    -3.92835942194152f},
  { -83.3812412279501f,    -8.10848294356065f},
  { -76.3508154775154f,    -12.3397512522743f},
  { -69.0948425521107f,    -16.3853175925535f},
  { -61.8388696267061f,    -20.0320362052904f},
  { -90.0f,    -11.6407231367706f},
  { -83.3812412279501f,    -15.9045984655888f},
  { -76.3508154775154f,    -20.0320362052904f},
  { -69.0948425521107f,    -23.8001826021751f},
  { -90.0f,    -18.9500224882919f},
  { -83.3812412279501f,    -23.1426578826804f},
  { -76.3508154775154f,    -27.0418117799956f},
  { -90.0f,    -25.6732726489455f},
  { -83.3812412279501f,    -29.6783194887260f},
  { -96.6187587720500f,    29.6783194887260f},
  { -96.6187587720500f,    23.1426578826804f},
  { -96.6187587720500f,    15.9045984655888f},
  { -96.6187587720500f,    8.10848294356066f},
  { -96.6187587720500f,    0.0f},
  { -96.6187587720500f,    -8.10848294356066f},
  { -96.6187587720500f,    -15.9045984655888f},
  { -96.6187587720500f,    -23.1426578826804f},
  { -96.6187587720500f,    -29.6783194887260f},
  { -103.649184522485f,    27.0418117799956f},
  { -103.649184522485f,    20.0320362052904f},
  { -103.649184522485f,    12.3397512522743f},
  { -103.649184522485f,    4.17066721420311f},
  { -103.649184522485f,    -4.17066721420311f},
  { -103.649184522485f,    -12.3397512522743f},
  { -103.649184522485f,    -20.0320362052904f},
  { -103.649184522485f,    -27.0418117799956f},
  { -110.905157447889f,    23.8001826021751f},
  { -110.905157447889f,    16.3853175925535f},
  { -110.905157447889f,    8.36364070535862f},
  { -110.905157447889f,    4.00243727602875e-15f},
  { -110.905157447889f,    -8.36364070535862f},
  { -110.905157447889f,    -16.3853175925536f},
  { -110.905157447889f,    -23.8001826021750f},
  { -118.161130373294f,    20.0320362052904f},
  { -118.161130373294f,    12.3397512522743f},
  { -118.161130373294f,    4.17066721420311f},
  { -118.161130373294f,    -4.17066721420311f},
  { -118.161130373294f,    -12.3397512522743f},
  { -118.161130373294f,    -20.0320362052904f},
  { -125.191556123729f,    15.9045984655888f},
  { -125.191556123729f,    8.10848294356065f},
  { -125.191556123729f,    0.0f},
  { -125.191556123729f,    -8.10848294356065f},
  { -125.191556123729f,    -15.9045984655888f},
  { -131.810314895779f,    11.6407231367706f},
  { -131.810314895779f,    3.92835942194152f},
  { -131.810314895779f,    -3.92835942194152f},
  { -131.810314895779f,    -11.6407231367706f},
  { -137.894356299762f,    7.46365844000401f},
  { -137.894356299762f,    0.0f},
  { -137.894356299762f,    -7.46365844000401f},
  { -143.386396365572f,    3.54837484763455f},
  { -143.386396365572f,    -3.54837484763455f},
  { 78.0088450024987f,    34.3684513220687f},
  { 70.1097039351377f,    31.4600994569254f},
  { 62.1720378505138f,    27.8222160665198f},
  { 54.4911747437709f,    23.5910973678578f},
  { 47.3092140391515f,    19.0068859665187f},
  { 40.7785125515860f,    14.3514024196091f},
  { 34.9597735481804f,    9.87450591183940f},
  { 71.3311766285625f,    39.2536542921232f},
  { 62.5739541017580f,    35.9235381066138f},
  { 54.1102453404159f,    31.7532340351390f},
  { 46.2629109724082f,    26.9706688639076f},
  { 39.2229293576909f,    21.9005622096226f},
  { 33.0503538411220f,    16.8704815689481f},
  { 63.0682898889764f,    44.0232126088419f},
  { 53.6444054657849f,    40.0860717757720f},
  { 45.0000000000000f,    35.2643896827547f},
  { 37.3773681406497f,    29.8996501180392f},
  { 30.8265451741538f,    24.3842531433103f},
  { 53.0617925100979f,    48.2445277172896f},
  { 43.4471669365429f,    43.5419746771421f},
  { 35.1575728180233f,    38.0361246322092f},
  { 28.2124239207466f,    32.1600123259279f},
  { 41.4949495092856f,    51.4552841510614f},
  { 32.4460999243646f,    45.9541777182743f},
  { 25.1097039351377f,    39.8719722274367f},
  { 29.0765933411396f,    53.3328859959901f},
  { 21.3917101405266f,    47.1898449136092f},
  { 16.8980222630011f,    53.8439817231271f},
  { 78.0088450024986f,    -34.3684513220687f},
  { 71.3311766285625f,    -39.2536542921232f},
  { 63.0682898889764f,    -44.0232126088419f},
  { 53.0617925100979f,    -48.2445277172896f},
  { 41.4949495092856f,    -51.4552841510614f},
  { 29.0765933411396f,    -53.3328859959901f},
  { 16.8980222630011f,    -53.8439817231271f},
  { 70.1097039351377f,    -31.4600994569254f},
  { 62.5739541017579f,    -35.9235381066138f},
  { 53.6444054657849f,    -40.0860717757720f},
  { 43.4471669365429f,    -43.5419746771421f},
  { 32.4460999243646f,    -45.9541777182743f},
  { 21.3917101405266f,    -47.1898449136092f},
  { 62.1720378505138f,    -27.8222160665198f},
  { 54.1102453404159f,    -31.7532340351390f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 35.1575728180233f,    -38.0361246322092f},
  { 25.1097039351377f,    -39.8719722274367f},
  { 54.4911747437709f,    -23.5910973678578f},
  { 46.2629109724082f,    -26.9706688639076f},
  { 37.3773681406497f,    -29.8996501180392f},
  { 28.2124239207466f,    -32.1600123259279f},
  { 47.3092140391515f,    -19.0068859665187f},
  { 39.2229293576909f,    -21.9005622096226f},
  { 30.8265451741538f,    -24.3842531433103f},
  { 40.7785125515860f,    -14.3514024196091f},
  { 33.0503538411220f,    -16.8704815689481f},
  { 34.9597735481804f,    -9.87450591183940f},
  { -16.8980222630011f,    53.8439817231271f},
  { -21.3917101405266f,    47.1898449136092f},
  { -25.1097039351377f,    39.8719722274367f},
  { -28.2124239207466f,    32.1600123259279f},
  { -30.8265451741538f,    24.3842531433103f},
  { -33.0503538411220f,    16.8704815689481f},
  { -34.9597735481804f,    9.87450591183939f},
  { -29.0765933411396f,    53.3328859959901f},
  { -32.4460999243646f,    45.9541777182743f},
  { -35.1575728180233f,    38.0361246322092f},
  { -37.3773681406497f,    29.8996501180392f},
  { -39.2229293576909f,    21.9005622096226f},
  { -40.7785125515860f,    14.3514024196091f},
  { -41.4949495092856f,    51.4552841510614f},
  { -43.4471669365429f,    43.5419746771421f},
  { -45.0000000000000f,    35.2643896827547f},
  { -46.2629109724082f,    26.9706688639076f},
  { -47.3092140391515f,    19.0068859665187f},
  { -53.0617925100979f,    48.2445277172896f},
  { -53.6444054657849f,    40.0860717757720f},
  { -54.1102453404159f,    31.7532340351390f},
  { -54.4911747437709f,    23.5910973678578f},
  { -63.0682898889764f,    44.0232126088419f},
  { -62.5739541017579f,    35.9235381066138f},
  { -62.1720378505138f,    27.8222160665198f},
  { -71.3311766285625f,    39.2536542921232f},
  { -70.1097039351377f,    31.4600994569254f},
  { -78.0088450024987f,    34.3684513220687f},
  { -16.8980222630011f,    -53.8439817231271f},
  { -29.0765933411396f,    -53.3328859959901f},
  { -41.4949495092856f,    -51.4552841510614f},
  { -53.0617925100979f,    -48.2445277172896f},
  { -63.0682898889764f,    -44.0232126088419f},
  { -71.3311766285625f,    -39.2536542921232f},
  { -78.0088450024986f,    -34.3684513220687f},
  { -21.3917101405266f,    -47.1898449136092f},
  { -32.4460999243646f,    -45.9541777182743f},
  { -43.4471669365429f,    -43.5419746771421f},
  { -53.6444054657849f,    -40.0860717757720f},
  { -62.5739541017579f,    -35.9235381066138f},
  { -70.1097039351377f,    -31.4600994569254f},
  { -25.1097039351377f,    -39.8719722274367f},
  { -35.1575728180233f,    -38.0361246322092f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -54.1102453404159f,    -31.7532340351390f},
  { -62.1720378505138f,    -27.8222160665198f},
  { -28.2124239207466f,    -32.1600123259279f},
  { -37.3773681406497f,    -29.8996501180392f},
  { -46.2629109724082f,    -26.9706688639076f},
  { -54.4911747437709f,    -23.5910973678578f},
  { -30.8265451741538f,    -24.3842531433103f},
  { -39.2229293576909f,    -21.9005622096226f},
  { -47.3092140391515f,    -19.0068859665187f},
  { -33.0503538411220f,    -16.8704815689481f},
  { -40.7785125515860f,    -14.3514024196091f},
  { -34.9597735481804f,    -9.87450591183940f},
  { 163.101977736999f,    53.8439817231271f},
  { 158.608289859473f,    47.1898449136092f},
  { 154.890296064862f,    39.8719722274367f},
  { 151.787576079253f,    32.1600123259279f},
  { 149.173454825846f,    24.3842531433103f},
  { 146.949646158878f,    16.8704815689481f},
  { 145.040226451820f,    9.87450591183939f},
  { 150.923406658860f,    53.3328859959901f},
  { 147.553900075635f,    45.9541777182743f},
  { 144.842427181977f,    38.0361246322092f},
  { 142.622631859350f,    29.8996501180392f},
  { 140.777070642309f,    21.9005622096226f},
  { 139.221487448414f,    14.3514024196091f},
  { 138.505050490714f,    51.4552841510614f},
  { 136.552833063457f,    43.5419746771421f},
  { 135.0f,    35.2643896827547f},
  { 133.737089027592f,    26.9706688639076f},
  { 132.690785960849f,    19.0068859665187f},
  { 126.938207489902f,    48.2445277172896f},
  { 126.355594534215f,    40.0860717757720f},
  { 125.889754659584f,    31.7532340351390f},
  { 125.508825256229f,    23.5910973678578f},
  { 116.931710111024f,    44.0232126088419f},
  { 117.426045898242f,    35.9235381066138f},
  { 117.827962149486f,    27.8222160665198f},
  { 108.668823371438f,    39.2536542921232f},
  { 109.890296064862f,    31.4600994569254f},
  { 101.991154997501f,    34.3684513220687f},
  { 163.101977736999f,    -53.8439817231271f},
  { 150.923406658860f,    -53.3328859959901f},
  { 138.505050490714f,    -51.4552841510614f},
  { 126.938207489902f,    -48.2445277172896f},
  { 116.931710111024f,    -44.0232126088419f},
  { 108.668823371438f,    -39.2536542921232f},
  { 101.991154997501f,    -34.3684513220687f},
  { 158.608289859473f,    -47.1898449136092f},
  { 147.553900075635f,    -45.9541777182743f},
  { 136.552833063457f,    -43.5419746771421f},
  { 126.355594534215f,    -40.0860717757720f},
  { 117.426045898242f,    -35.9235381066138f},
  { 109.890296064862f,    -31.4600994569254f},
  { 154.890296064862f,    -39.8719722274367f},
  { 144.842427181977f,    -38.0361246322092f},
  { 135.0f,    -35.2643896827547f},
  { 125.889754659584f,    -31.7532340351390f},
  { 117.827962149486f,    -27.8222160665198f},
  { 151.787576079253f,    -32.1600123259279f},
  { 142.622631859350f,    -29.8996501180392f},
  { 133.737089027592f,    -26.9706688639076f},
  { 125.508825256229f,    -23.5910973678578f},
  { 149.173454825846f,    -24.3842531433103f},
  { 140.777070642309f,    -21.9005622096226f},
  { 132.690785960849f,    -19.0068859665187f},
  { 146.949646158878f,    -16.8704815689481f},
  { 139.221487448414f,    -14.3514024196091f},
  { 145.040226451820f,    -9.87450591183940f},
  { -163.101977736999f,    53.8439817231271f},
  { -150.923406658860f,    53.3328859959901f},
  { -138.505050490714f,    51.4552841510614f},
  { -126.938207489902f,    48.2445277172896f},
  { -116.931710111024f,    44.0232126088419f},
  { -108.668823371438f,    39.2536542921232f},
  { -101.991154997501f,    34.3684513220687f},
  { -158.608289859473f,    47.1898449136092f},
  { -147.553900075635f,    45.9541777182743f},
  { -136.552833063457f,    43.5419746771421f},
  { -126.355594534215f,    40.0860717757720f},
  { -117.426045898242f,    35.9235381066138f},
  { -109.890296064862f,    31.4600994569254f},
  { -154.890296064862f,    39.8719722274367f},
  { -144.842427181977f,    38.0361246322092f},
  { -135.0f,    35.2643896827547f},
  { -125.889754659584f,    31.7532340351390f},
  { -117.827962149486f,    27.8222160665198f},
  { -151.787576079253f,    32.1600123259279f},
  { -142.622631859350f,    29.8996501180392f},
  { -133.737089027592f,    26.9706688639076f},
  { -125.508825256229f,    23.5910973678578f},
  { -149.173454825846f,    24.3842531433103f},
  { -140.777070642309f,    21.9005622096226f},
  { -132.690785960849f,    19.0068859665187f},
  { -146.949646158878f,    16.8704815689481f},
  { -139.221487448414f,    14.3514024196091f},
  { -145.040226451820f,    9.87450591183940f},
  { -163.101977736999f,    -53.8439817231271f},
  { -158.608289859473f,    -47.1898449136092f},
  { -154.890296064862f,    -39.8719722274367f},
  { -151.787576079253f,    -32.1600123259279f},
  { -149.173454825846f,    -24.3842531433103f},
  { -146.949646158878f,    -16.8704815689481f},
  { -145.040226451820f,    -9.87450591183939f},
  { -150.923406658860f,    -53.3328859959901f},
  { -147.553900075635f,    -45.9541777182743f},
  { -144.842427181977f,    -38.0361246322092f},
  { -142.622631859350f,    -29.8996501180392f},
  { -140.777070642309f,    -21.9005622096226f},
  { -139.221487448414f,    -14.3514024196091f},
  { -138.505050490714f,    -51.4552841510614f},
  { -136.552833063457f,    -43.5419746771421f},
  { -135.0f,    -35.2643896827547f},
  { -133.737089027592f,    -26.9706688639076f},
  { -132.690785960849f,    -19.0068859665187f},
  { -126.938207489902f,    -48.2445277172896f},
  { -126.355594534215f,    -40.0860717757720f},
  { -125.889754659584f,    -31.7532340351390f},
  { -125.508825256229f,    -23.5910973678578f},
  { -116.931710111024f,    -44.0232126088419f},
  { -117.426045898242f,    -35.9235381066138f},
  { -117.827962149486f,    -27.8222160665198f},
  { -108.668823371438f,    -39.2536542921232f},
  { -109.890296064862f,    -31.4600994569254f},
  { -101.991154997501f,    -34.3684513220687f}};

/* 3LD geosphere 'ico' [10 0] (1002 points) */
const float __geosphere_ico_10_0_dirs_deg[1002][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    63.6909565444517f},
  { 0.0f,    69.6541923856616f},
  { 0.0f,    76.1141128518623f},
  { 0.0f,    82.9535936191428f},
  { 0.0f,    90.0f},
  { 180.0f,    82.9535936191428f},
  { 180.0f,    76.1141128518623f},
  { 180.0f,    69.6541923856616f},
  { 180.0f,    63.6909565444517f},
  { 10.1918500362761f,    59.5618368610333f},
  { 13.0152040462228f,    65.2188912720402f},
  { 17.9319334355159f,    71.3405195718550f},
  { 28.3399673707642f,    77.6459130495745f},
  { 58.2825255885390f,    83.0315494669714f},
  { 121.717474411461f,    83.0315494669714f},
  { 151.660032629236f,    77.6459130495745f},
  { 162.068066564484f,    71.3405195718550f},
  { 166.984795953777f,    65.2188912720402f},
  { 169.808149963724f,    59.5618368610333f},
  { 22.0237505161462f,    59.9952713133382f},
  { 28.3399673707642f,    65.4792084042997f},
  { 38.9734473368657f,    71.0021166134344f},
  { 58.2825255885390f,    75.7180776303631f},
  { 90.0f,    77.7816801337513f},
  { 121.717474411461f,    75.7180776303631f},
  { 141.026552663134f,    71.0021166134344f},
  { 151.660032629236f,    65.4792084042997f},
  { 157.976249483854f,    59.9952713133382f},
  { 34.7391081893039f,    59.2643719580340f},
  { 44.1517511327961f,    64.0604700951918f},
  { 58.2825255885390f,    68.2819223696369f},
  { 78.3592768632294f,    70.9173838370166f},
  { 101.640723136771f,    70.9173838370166f},
  { 121.717474411461f,    68.2819223696369f},
  { 135.848248867204f,    64.0604700951918f},
  { 145.260891810696f,    59.2643719580340f},
  { 47.1679114977868f,    57.2262493118836f},
  { 58.2825255885390f,    60.9684505330286f},
  { 72.8279621494862f,    63.7030502822517f},
  { 90.0f,    64.7263194139596f},
  { 107.172037850514f,    63.7030502822517f},
  { 121.717474411461f,    60.9684505330286f},
  { 132.832088502213f,    57.2262493118836f},
  { 58.2825255885390f,    54.0f},
  { 69.6541923856616f,    56.6088055804906f},
  { 82.9535936191428f,    58.0879319294352f},
  { 97.0464063808572f,    58.0879319294352f},
  { 110.345807614338f,    56.6088055804906f},
  { 121.717474411461f,    54.0f},
  { 67.6072290710451f,    49.9068566415137f},
  { 78.3592768632294f,    51.5245159887128f},
  { 90.0f,    52.1031151288303f},
  { 101.640723136771f,    51.5245159887128f},
  { 112.392770928955f,    49.9068566415137f},
  { 75.1646413480039f,    45.3337202344714f},
  { 84.9544063480986f,    46.1933976926144f},
  { 95.0455936519014f,    46.1933976926144f},
  { 104.835358651996f,    45.3337202344714f},
  { 81.2167693990410f,    40.6247965576687f},
  { 90.0f,    40.9590912848605f},
  { 98.7832306009590f,    40.6247965576687f},
  { 86.0716405780585f,    36.0333403752908f},
  { 93.9283594219415f,    36.0333403752908f},
  { -10.1918500362761f,    59.5618368610333f},
  { -22.0237505161462f,    59.9952713133382f},
  { -34.7391081893039f,    59.2643719580340f},
  { -47.1679114977868f,    57.2262493118836f},
  { -58.2825255885390f,    54.0f},
  { -67.6072290710451f,    49.9068566415137f},
  { -75.1646413480039f,    45.3337202344714f},
  { -81.2167693990410f,    40.6247965576687f},
  { -86.0716405780585f,    36.0333403752908f},
  { -13.0152040462228f,    65.2188912720402f},
  { -28.3399673707642f,    65.4792084042997f},
  { -44.1517511327961f,    64.0604700951918f},
  { -58.2825255885390f,    60.9684505330286f},
  { -69.6541923856616f,    56.6088055804906f},
  { -78.3592768632294f,    51.5245159887128f},
  { -84.9544063480986f,    46.1933976926144f},
  { -90.0f,    40.9590912848605f},
  { -93.9283594219415f,    36.0333403752908f},
  { -17.9319334355159f,    71.3405195718550f},
  { -38.9734473368657f,    71.0021166134344f},
  { -58.2825255885390f,    68.2819223696369f},
  { -72.8279621494862f,    63.7030502822517f},
  { -82.9535936191428f,    58.0879319294352f},
  { -90.0f,    52.1031151288303f},
  { -95.0455936519014f,    46.1933976926144f},
  { -98.7832306009590f,    40.6247965576687f},
  { -28.3399673707642f,    77.6459130495745f},
  { -58.2825255885390f,    75.7180776303631f},
  { -78.3592768632294f,    70.9173838370166f},
  { -90.0000000000000f,    64.7263194139596f},
  { -97.0464063808572f,    58.0879319294352f},
  { -101.640723136771f,    51.5245159887128f},
  { -104.835358651996f,    45.3337202344714f},
  { -58.2825255885390f,    83.0315494669714f},
  { -90.0000000000000f,    77.7816801337513f},
  { -101.640723136771f,    70.9173838370166f},
  { -107.172037850514f,    63.7030502822517f},
  { -110.345807614338f,    56.6088055804906f},
  { -112.392770928955f,    49.9068566415137f},
  { -121.717474411461f,    83.0315494669714f},
  { -121.717474411461f,    75.7180776303631f},
  { -121.717474411461f,    68.2819223696369f},
  { -121.717474411461f,    60.9684505330286f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    77.6459130495745f},
  { -141.026552663134f,    71.0021166134344f},
  { -135.848248867204f,    64.0604700951918f},
  { -132.832088502213f,    57.2262493118836f},
  { -162.068066564484f,    71.3405195718550f},
  { -151.660032629236f,    65.4792084042997f},
  { -145.260891810696f,    59.2643719580340f},
  { -166.984795953777f,    65.2188912720402f},
  { -157.976249483854f,    59.9952713133382f},
  { -169.808149963724f,    59.5618368610333f},
  { 10.1918500362761f,    -59.5618368610333f},
  { 22.0237505161462f,    -59.9952713133382f},
  { 34.7391081893039f,    -59.2643719580340f},
  { 47.1679114977868f,    -57.2262493118836f},
  { 58.2825255885390f,    -54.0f},
  { 67.6072290710451f,    -49.9068566415137f},
  { 75.1646413480039f,    -45.3337202344714f},
  { 81.2167693990410f,    -40.6247965576687f},
  { 86.0716405780585f,    -36.0333403752908f},
  { 0.0f,    -63.6909565444517f},
  { 13.0152040462228f,    -65.2188912720402f},
  { 28.3399673707642f,    -65.4792084042997f},
  { 44.1517511327961f,    -64.0604700951918f},
  { 58.2825255885390f,    -60.9684505330286f},
  { 69.6541923856616f,    -56.6088055804906f},
  { 78.3592768632294f,    -51.5245159887128f},
  { 84.9544063480986f,    -46.1933976926144f},
  { 90.0f,    -40.9590912848605f},
  { 93.9283594219415f,    -36.0333403752908f},
  { 0.0f,    -69.6541923856616f},
  { 17.9319334355159f,    -71.3405195718550f},
  { 38.9734473368657f,    -71.0021166134344f},
  { 58.2825255885390f,    -68.2819223696369f},
  { 72.8279621494862f,    -63.7030502822517f},
  { 82.9535936191428f,    -58.0879319294352f},
  { 90.0f,    -52.1031151288303f},
  { 95.0455936519014f,    -46.1933976926144f},
  { 98.7832306009590f,    -40.6247965576687f},
  { 0.0f,    -76.1141128518623f},
  { 28.3399673707642f,    -77.6459130495745f},
  { 58.2825255885390f,    -75.7180776303631f},
  { 78.3592768632294f,    -70.9173838370166f},
  { 90.0000000000000f,    -64.7263194139596f},
  { 97.0464063808572f,    -58.0879319294352f},
  { 101.640723136771f,    -51.5245159887128f},
  { 104.835358651996f,    -45.3337202344714f},
  { 0.0f,    -82.9535936191428f},
  { 58.2825255885390f,    -83.0315494669714f},
  { 90.0000000000000f,    -77.7816801337513f},
  { 101.640723136771f,    -70.9173838370166f},
  { 107.172037850514f,    -63.7030502822517f},
  { 110.345807614338f,    -56.6088055804906f},
  { 112.392770928955f,    -49.9068566415137f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -83.0315494669714f},
  { 121.717474411461f,    -75.7180776303631f},
  { 121.717474411461f,    -68.2819223696369f},
  { 121.717474411461f,    -60.9684505330286f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -82.9535936191428f},
  { 151.660032629236f,    -77.6459130495745f},
  { 141.026552663134f,    -71.0021166134344f},
  { 135.848248867204f,    -64.0604700951918f},
  { 132.832088502213f,    -57.2262493118836f},
  { 180.0f,    -76.1141128518623f},
  { 162.068066564484f,    -71.3405195718550f},
  { 151.660032629236f,    -65.4792084042997f},
  { 145.260891810696f,    -59.2643719580340f},
  { 180.0f,    -69.6541923856616f},
  { 166.984795953777f,    -65.2188912720402f},
  { 157.976249483854f,    -59.9952713133382f},
  { 180.0f,    -63.6909565444517f},
  { 169.808149963724f,    -59.5618368610333f},
  { -10.1918500362761f,    -59.5618368610333f},
  { -13.0152040462228f,    -65.2188912720402f},
  { -17.9319334355159f,    -71.3405195718550f},
  { -28.3399673707642f,    -77.6459130495745f},
  { -58.2825255885390f,    -83.0315494669714f},
  { -121.717474411461f,    -83.0315494669714f},
  { -151.660032629236f,    -77.6459130495745f},
  { -162.068066564484f,    -71.3405195718550f},
  { -166.984795953777f,    -65.2188912720402f},
  { -169.808149963724f,    -59.5618368610333f},
  { -22.0237505161462f,    -59.9952713133382f},
  { -28.3399673707642f,    -65.4792084042997f},
  { -38.9734473368657f,    -71.0021166134344f},
  { -58.2825255885390f,    -75.7180776303631f},
  { -90.0f,    -77.7816801337513f},
  { -121.717474411461f,    -75.7180776303631f},
  { -141.026552663134f,    -71.0021166134344f},
  { -151.660032629236f,    -65.4792084042997f},
  { -157.976249483854f,    -59.9952713133382f},
  { -34.7391081893039f,    -59.2643719580340f},
  { -44.1517511327961f,    -64.0604700951918f},
  { -58.2825255885390f,    -68.2819223696369f},
  { -78.3592768632294f,    -70.9173838370166f},
  { -101.640723136771f,    -70.9173838370166f},
  { -121.717474411461f,    -68.2819223696369f},
  { -135.848248867204f,    -64.0604700951918f},
  { -145.260891810696f,    -59.2643719580340f},
  { -47.1679114977868f,    -57.2262493118836f},
  { -58.2825255885390f,    -60.9684505330286f},
  { -72.8279621494862f,    -63.7030502822517f},
  { -90.0f,    -64.7263194139596f},
  { -107.172037850514f,    -63.7030502822517f},
  { -121.717474411461f,    -60.9684505330286f},
  { -132.832088502213f,    -57.2262493118836f},
  { -58.2825255885390f,    -54.0f},
  { -69.6541923856616f,    -56.6088055804906f},
  { -82.9535936191428f,    -58.0879319294352f},
  { -97.0464063808572f,    -58.0879319294352f},
  { -110.345807614338f,    -56.6088055804906f},
  { -121.717474411461f,    -54.0f},
  { -67.6072290710451f,    -49.9068566415137f},
  { -78.3592768632294f,    -51.5245159887128f},
  { -90.0f,    -52.1031151288303f},
  { -101.640723136771f,    -51.5245159887128f},
  { -112.392770928955f,    -49.9068566415137f},
  { -75.1646413480039f,    -45.3337202344714f},
  { -84.9544063480986f,    -46.1933976926144f},
  { -95.0455936519014f,    -46.1933976926144f},
  { -104.835358651996f,    -45.3337202344714f},
  { -81.2167693990410f,    -40.6247965576687f},
  { -90.0f,    -40.9590912848605f},
  { -98.7832306009590f,    -40.6247965576687f},
  { -86.0716405780585f,    -36.0333403752908f},
  { -93.9283594219415f,    -36.0333403752908f},
  { 26.3090434555483f,    0.0f},
  { 20.3458076143384f,    0.0f},
  { 13.8858871481377f,    0.0f},
  { 7.04640638085718f,    0.0f},
  { 0.0f,    0.0f},
  { -7.04640638085718f,    0.0f},
  { -13.8858871481377f,    0.0f},
  { -20.3458076143384f,    0.0f},
  { -26.3090434555483f,    0.0f},
  { 30.0416736238016f,    5.14299173780424f},
  { 24.2184442313710f,    5.41662750205155f},
  { 17.8116482802838f,    5.65316210041390f},
  { 10.9112612070943f,    5.82922790608419f},
  { 3.67664761751924f,    5.92366239668334f},
  { -3.67664761751923f,    5.92366239668334f},
  { -10.9112612070943f,    5.82922790608419f},
  { -17.8116482802838f,    5.65316210041390f},
  { -24.2184442313710f,    5.41662750205155f},
  { -30.0416736238016f,    5.14299173780424f},
  { 28.1611303732939f,    10.8082208669889f},
  { 21.8750472294843f,    11.3623161420624f},
  { 14.9845972116948f,    11.8147143864651f},
  { 7.62263185935030f,    12.1135516334358f},
  { 0.0f,    12.2183198662487f},
  { -7.62263185935030f,    12.1135516334358f},
  { -14.9845972116948f,    11.8147143864651f},
  { -21.8750472294843f,    11.3623161420624f},
  { -28.1611303732939f,    10.8082208669889f},
  { 26.0407808737165f,    16.9317696239740f},
  { 19.2394969897313f,    17.7395979912771f},
  { 11.8271783465987f,    18.3472152555362f},
  { 3.99288330178752f,    18.6754371077394f},
  { -3.99288330178752f,    18.6754371077394f},
  { -11.8271783465987f,    18.3472152555362f},
  { -19.2394969897313f,    17.7395979912771f},
  { -26.0407808737165f,    16.9317696239740f},
  { 23.6387115659933f,    23.3894469911546f},
  { 16.2670373251352f,    24.3816853792088f},
  { 8.30077479951012f,    25.0415008212998f},
  { 0.0f,    25.2736805860404f},
  { -8.30077479951012f,    25.0415008212998f},
  { -16.2670373251352f,    24.3816853792088f},
  { -23.6387115659933f,    23.3894469911546f},
  { 20.9051574478893f,    30.0000000000000f},
  { 12.9081130872723f,    31.0654325470156f},
  { 4.36852313815106f,    31.6429720773099f},
  { -4.36852313815107f,    31.6429720773099f},
  { -12.9081130872723f,    31.0654325470156f},
  { -20.9051574478893f,    30.0000000000000f},
  { 17.7818205761190f,    36.5460212963266f},
  { 9.11024534041592f,    37.5448869067115f},
  { 0.0f,    37.8968848711697f},
  { -9.11024534041592f,    37.5448869067115f},
  { -17.7818205761190f,    36.5460212963266f},
  { 14.2020489783221f,    42.8079368339490f},
  { 4.82199690182553f,    43.5940218663723f},
  { -4.82199690182553f,    43.5940218663723f},
  { -14.2020489783221f,    42.8079368339490f},
  { 10.0928404843176f,    48.5980887666628f},
  { 0.0f,    49.0409087151395f},
  { -10.0928404843176f,    48.5980887666628f},
  { 5.38021214855672f,    53.7820105755072f},
  { -5.38021214855672f,    53.7820105755072f},
  { 30.0416736238016f,    -5.14299173780424f},
  { 28.1611303732939f,    -10.8082208669889f},
  { 26.0407808737165f,    -16.9317696239740f},
  { 23.6387115659933f,    -23.3894469911546f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 17.7818205761190f,    -36.5460212963266f},
  { 14.2020489783221f,    -42.8079368339490f},
  { 10.0928404843176f,    -48.5980887666628f},
  { 5.38021214855672f,    -53.7820105755072f},
  { 24.2184442313710f,    -5.41662750205154f},
  { 21.8750472294843f,    -11.3623161420624f},
  { 19.2394969897313f,    -17.7395979912771f},
  { 16.2670373251352f,    -24.3816853792088f},
  { 12.9081130872723f,    -31.0654325470156f},
  { 9.11024534041592f,    -37.5448869067115f},
  { 4.82199690182553f,    -43.5940218663723f},
  { 0.0f,    -49.0409087151395f},
  { -5.38021214855674f,    -53.7820105755072f},
  { 17.8116482802838f,    -5.65316210041390f},
  { 14.9845972116948f,    -11.8147143864651f},
  { 11.8271783465987f,    -18.3472152555362f},
  { 8.30077479951012f,    -25.0415008212998f},
  { 4.36852313815107f,    -31.6429720773099f},
  { 0.0f,    -37.8968848711697f},
  { -4.82199690182553f,    -43.5940218663723f},
  { -10.0928404843176f,    -48.5980887666628f},
  { 10.9112612070943f,    -5.82922790608419f},
  { 7.62263185935030f,    -12.1135516334358f},
  { 3.99288330178752f,    -18.6754371077394f},
  { -2.20662551427875e-15f,    -25.2736805860404f},
  { -4.36852313815107f,    -31.6429720773099f},
  { -9.11024534041592f,    -37.5448869067115f},
  { -14.2020489783221f,    -42.8079368339490f},
  { 3.67664761751923f,    -5.92366239668334f},
  { -1.01205574733119e-15f,    -12.2183198662487f},
  { -3.99288330178752f,    -18.6754371077394f},
  { -8.30077479951012f,    -25.0415008212998f},
  { -12.9081130872723f,    -31.0654325470156f},
  { -17.7818205761190f,    -36.5460212963266f},
  { -3.67664761751924f,    -5.92366239668334f},
  { -7.62263185935030f,    -12.1135516334358f},
  { -11.8271783465987f,    -18.3472152555362f},
  { -16.2670373251352f,    -24.3816853792088f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -10.9112612070943f,    -5.82922790608419f},
  { -14.9845972116948f,    -11.8147143864651f},
  { -19.2394969897313f,    -17.7395979912771f},
  { -23.6387115659933f,    -23.3894469911546f},
  { -17.8116482802838f,    -5.65316210041390f},
  { -21.8750472294843f,    -11.3623161420624f},
  { -26.0407808737165f,    -16.9317696239740f},
  { -24.2184442313710f,    -5.41662750205154f},
  { -28.1611303732939f,    -10.8082208669889f},
  { -30.0416736238016f,    -5.14299173780424f},
  { -153.690956544452f,    0.0f},
  { -159.654192385662f,    0.0f},
  { -166.114112851862f,    0.0f},
  { -172.953593619143f,    0.0f},
  { 180.0f,    0.0f},
  { 172.953593619143f,    0.0f},
  { 166.114112851862f,    0.0f},
  { 159.654192385662f,    0.0f},
  { 153.690956544452f,    0.0f},
  { -149.958326376198f,    5.14299173780424f},
  { -155.781555768629f,    5.41662750205155f},
  { -162.188351719716f,    5.65316210041390f},
  { -169.088738792906f,    5.82922790608419f},
  { -176.323352382481f,    5.92366239668334f},
  { 176.323352382481f,    5.92366239668334f},
  { 169.088738792906f,    5.82922790608419f},
  { 162.188351719716f,    5.65316210041390f},
  { 155.781555768629f,    5.41662750205155f},
  { 149.958326376198f,    5.14299173780424f},
  { -151.838869626706f,    10.8082208669889f},
  { -158.124952770516f,    11.3623161420624f},
  { -165.015402788305f,    11.8147143864651f},
  { -172.377368140650f,    12.1135516334358f},
  { 180.0f,    12.2183198662487f},
  { 172.377368140650f,    12.1135516334358f},
  { 165.015402788305f,    11.8147143864651f},
  { 158.124952770516f,    11.3623161420624f},
  { 151.838869626706f,    10.8082208669889f},
  { -153.959219126284f,    16.9317696239740f},
  { -160.760503010269f,    17.7395979912771f},
  { -168.172821653401f,    18.3472152555362f},
  { -176.007116698213f,    18.6754371077394f},
  { 176.007116698212f,    18.6754371077394f},
  { 168.172821653401f,    18.3472152555362f},
  { 160.760503010269f,    17.7395979912771f},
  { 153.959219126284f,    16.9317696239740f},
  { -156.361288434007f,    23.3894469911546f},
  { -163.732962674865f,    24.3816853792088f},
  { -171.699225200490f,    25.0415008212998f},
  { 180.0f,    25.2736805860404f},
  { 171.699225200490f,    25.0415008212998f},
  { 163.732962674865f,    24.3816853792088f},
  { 156.361288434007f,    23.3894469911546f},
  { -159.094842552111f,    30.0000000000000f},
  { -167.091886912728f,    31.0654325470156f},
  { -175.631476861849f,    31.6429720773099f},
  { 175.631476861849f,    31.6429720773099f},
  { 167.091886912728f,    31.0654325470156f},
  { 159.094842552111f,    30.0000000000000f},
  { -162.218179423881f,    36.5460212963266f},
  { -170.889754659584f,    37.5448869067115f},
  { 180.0f,    37.8968848711697f},
  { 170.889754659584f,    37.5448869067115f},
  { 162.218179423881f,    36.5460212963266f},
  { -165.797951021678f,    42.8079368339490f},
  { -175.178003098174f,    43.5940218663723f},
  { 175.178003098174f,    43.5940218663723f},
  { 165.797951021678f,    42.8079368339490f},
  { -169.907159515682f,    48.5980887666628f},
  { 180.0f,    49.0409087151395f},
  { 169.907159515682f,    48.5980887666628f},
  { -174.619787851443f,    53.7820105755072f},
  { 174.619787851443f,    53.7820105755072f},
  { -149.958326376198f,    -5.14299173780424f},
  { -151.838869626706f,    -10.8082208669889f},
  { -153.959219126284f,    -16.9317696239740f},
  { -156.361288434007f,    -23.3894469911546f},
  { -159.094842552111f,    -30.0000000000000f},
  { -162.218179423881f,    -36.5460212963266f},
  { -165.797951021678f,    -42.8079368339490f},
  { -169.907159515682f,    -48.5980887666628f},
  { -174.619787851443f,    -53.7820105755072f},
  { -155.781555768629f,    -5.41662750205154f},
  { -158.124952770516f,    -11.3623161420624f},
  { -160.760503010269f,    -17.7395979912771f},
  { -163.732962674865f,    -24.3816853792088f},
  { -167.091886912728f,    -31.0654325470156f},
  { -170.889754659584f,    -37.5448869067115f},
  { -175.178003098174f,    -43.5940218663723f},
  { 180.0f,    -49.0409087151395f},
  { 174.619787851443f,    -53.7820105755072f},
  { -162.188351719716f,    -5.65316210041390f},
  { -165.015402788305f,    -11.8147143864651f},
  { -168.172821653401f,    -18.3472152555362f},
  { -171.699225200490f,    -25.0415008212998f},
  { -175.631476861849f,    -31.6429720773099f},
  { 180.0f,    -37.8968848711697f},
  { 175.178003098174f,    -43.5940218663723f},
  { 169.907159515682f,    -48.5980887666628f},
  { -169.088738792906f,    -5.82922790608419f},
  { -172.377368140650f,    -12.1135516334358f},
  { -176.007116698213f,    -18.6754371077394f},
  { 180.0f,    -25.2736805860404f},
  { 175.631476861849f,    -31.6429720773099f},
  { 170.889754659584f,    -37.5448869067115f},
  { 165.797951021678f,    -42.8079368339490f},
  { -176.323352382481f,    -5.92366239668334f},
  { 180.0f,    -12.2183198662487f},
  { 176.007116698212f,    -18.6754371077394f},
  { 171.699225200490f,    -25.0415008212998f},
  { 167.091886912728f,    -31.0654325470156f},
  { 162.218179423881f,    -36.5460212963266f},
  { 176.323352382481f,    -5.92366239668334f},
  { 172.377368140650f,    -12.1135516334358f},
  { 168.172821653401f,    -18.3472152555362f},
  { 163.732962674865f,    -24.3816853792088f},
  { 159.094842552111f,    -30.0000000000000f},
  { 169.088738792906f,    -5.82922790608419f},
  { 165.015402788305f,    -11.8147143864651f},
  { 160.760503010269f,    -17.7395979912771f},
  { 156.361288434007f,    -23.3894469911546f},
  { 162.188351719716f,    -5.65316210041390f},
  { 158.124952770516f,    -11.3623161420624f},
  { 153.959219126284f,    -16.9317696239740f},
  { 155.781555768629f,    -5.41662750205154f},
  { 151.838869626706f,    -10.8082208669889f},
  { 149.958326376198f,    -5.14299173780424f},
  { 90.0f,    26.3090434555483f},
  { 90.0f,    20.3458076143384f},
  { 90.0f,    13.8858871481377f},
  { 90.0f,    7.04640638085718f},
  { 90.0f,    0.0f},
  { 90.0f,    -7.04640638085718f},
  { 90.0f,    -13.8858871481377f},
  { 90.0f,    -20.3458076143384f},
  { 90.0f,    -26.3090434555483f},
  { 84.0642077359095f,    29.9083631516625f},
  { 84.0642077359095f,    24.1034144840352f},
  { 84.0642077359095f,    17.7221396001114f},
  { 84.0642077359095f,    10.8541527460023f},
  { 84.0642077359095f,    3.65698857497185f},
  { 84.0642077359095f,    -3.65698857497184f},
  { 84.0642077359095f,    -10.8541527460023f},
  { 84.0642077359095f,    -17.7221396001114f},
  { 84.0642077359095f,    -24.1034144840352f},
  { 84.0642077359095f,    -29.9083631516625f},
  { 77.7816801337513f,    27.6183932347186f},
  { 77.7816801337513f,    21.4249007093178f},
  { 77.7816801337513f,    14.6599491964913f},
  { 77.7816801337513f,    7.45192878147619f},
  { 77.7816801337513f,    0.0f},
  { 77.7816801337513f,    -7.45192878147619f},
  { 77.7816801337513f,    -14.6599491964913f},
  { 77.7816801337513f,    -21.4249007093178f},
  { 77.7816801337513f,    -27.6183932347186f},
  { 71.2822831909298f,    24.8333646999319f},
  { 71.2822831909298f,    18.2913664299137f},
  { 71.2822831909298f,    11.2179396537745f},
  { 71.2822831909298f,    3.78233409570435f},
  { 71.2822831909298f,    -3.78233409570436f},
  { 71.2822831909298f,    -11.2179396537745f},
  { 71.2822831909298f,    -18.2913664299137f},
  { 71.2822831909298f,    -24.8333646999319f},
  { 64.7263194139596f,    21.5935345276613f},
  { 64.7263194139596f,    14.7814457540342f},
  { 64.7263194139596f,    7.51576871353677f},
  { 64.7263194139596f,    0.0f},
  { 64.7263194139596f,    -7.51576871353677f},
  { 64.7263194139596f,    -14.7814457540342f},
  { 64.7263194139596f,    -21.5935345276613f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    11.0315494669714f},
  { 58.2825255885390f,    3.71807763036312f},
  { 58.2825255885390f,    -3.71807763036312f},
  { 58.2825255885390f,    -11.0315494669714f},
  { 58.2825255885390f,    -18.0f},
  { 52.1031151288303f,    14.2023070245089f},
  { 52.1031151288303f,    7.21192704412376f},
  { 52.1031151288303f,    0.0f},
  { 52.1031151288303f,    -7.21192704412376f},
  { 52.1031151288303f,    -14.2023070245089f},
  { 46.3045173538851f,    10.3692763159630f},
  { 46.3045173538851f,    3.49033866189271f},
  { 46.3045173538851f,    -3.49033866189271f},
  { 46.3045173538851f,    -10.3692763159630f},
  { 40.9590912848605f,    6.65525720600019f},
  { 40.9590912848605f,    0.0f},
  { 40.9590912848605f,    -6.65525720600019f},
  { 36.0974772082685f,    3.17590310185081f},
  { 36.0974772082685f,    -3.17590310185081f},
  { 95.9357922640906f,    29.9083631516625f},
  { 102.218319866249f,    27.6183932347186f},
  { 108.717716809070f,    24.8333646999319f},
  { 115.273680586040f,    21.5935345276613f},
  { 121.717474411461f,    18.0f},
  { 127.896884871170f,    14.2023070245089f},
  { 133.695482646115f,    10.3692763159630f},
  { 139.040908715140f,    6.65525720600019f},
  { 143.902522791732f,    3.17590310185081f},
  { 95.9357922640906f,    24.1034144840352f},
  { 102.218319866249f,    21.4249007093178f},
  { 108.717716809070f,    18.2913664299137f},
  { 115.273680586040f,    14.7814457540342f},
  { 121.717474411461f,    11.0315494669714f},
  { 127.896884871170f,    7.21192704412376f},
  { 133.695482646115f,    3.49033866189271f},
  { 139.040908715140f,    0.0f},
  { 143.902522791732f,    -3.17590310185082f},
  { 95.9357922640906f,    17.7221396001114f},
  { 102.218319866249f,    14.6599491964913f},
  { 108.717716809070f,    11.2179396537745f},
  { 115.273680586040f,    7.51576871353677f},
  { 121.717474411461f,    3.71807763036312f},
  { 127.896884871170f,    0.0f},
  { 133.695482646115f,    -3.49033866189271f},
  { 139.040908715140f,    -6.65525720600019f},
  { 95.9357922640906f,    10.8541527460023f},
  { 102.218319866249f,    7.45192878147619f},
  { 108.717716809070f,    3.78233409570435f},
  { 115.273680586040f,    -1.99540459610511e-15f},
  { 121.717474411461f,    -3.71807763036312f},
  { 127.896884871170f,    -7.21192704412377f},
  { 133.695482646115f,    -10.3692763159630f},
  { 95.9357922640906f,    3.65698857497185f},
  { 102.218319866249f,    -9.89130938771355e-16f},
  { 108.717716809070f,    -3.78233409570436f},
  { 115.273680586040f,    -7.51576871353677f},
  { 121.717474411461f,    -11.0315494669714f},
  { 127.896884871170f,    -14.2023070245090f},
  { 95.9357922640906f,    -3.65698857497185f},
  { 102.218319866249f,    -7.45192878147619f},
  { 108.717716809070f,    -11.2179396537745f},
  { 115.273680586040f,    -14.7814457540342f},
  { 121.717474411461f,    -18.0f},
  { 95.9357922640906f,    -10.8541527460023f},
  { 102.218319866249f,    -14.6599491964913f},
  { 108.717716809070f,    -18.2913664299137f},
  { 115.273680586040f,    -21.5935345276613f},
  { 95.9357922640906f,    -17.7221396001114f},
  { 102.218319866249f,    -21.4249007093178f},
  { 108.717716809070f,    -24.8333646999319f},
  { 95.9357922640906f,    -24.1034144840352f},
  { 102.218319866249f,    -27.6183932347186f},
  { 95.9357922640906f,    -29.9083631516625f},
  { -84.0642077359095f,    29.9083631516625f},
  { -77.7816801337513f,    27.6183932347186f},
  { -71.2822831909298f,    24.8333646999319f},
  { -64.7263194139596f,    21.5935345276613f},
  { -58.2825255885390f,    18.0f},
  { -52.1031151288303f,    14.2023070245089f},
  { -46.3045173538851f,    10.3692763159630f},
  { -40.9590912848605f,    6.65525720600019f},
  { -36.0974772082685f,    3.17590310185081f},
  { -90.0f,    26.3090434555483f},
  { -84.0642077359095f,    24.1034144840352f},
  { -77.7816801337513f,    21.4249007093178f},
  { -71.2822831909298f,    18.2913664299137f},
  { -64.7263194139596f,    14.7814457540342f},
  { -58.2825255885390f,    11.0315494669714f},
  { -52.1031151288303f,    7.21192704412376f},
  { -46.3045173538851f,    3.49033866189271f},
  { -40.9590912848605f,    0.0f},
  { -36.0974772082685f,    -3.17590310185082f},
  { -90.0f,    20.3458076143384f},
  { -84.0642077359095f,    17.7221396001114f},
  { -77.7816801337513f,    14.6599491964913f},
  { -71.2822831909298f,    11.2179396537745f},
  { -64.7263194139596f,    7.51576871353677f},
  { -58.2825255885390f,    3.71807763036312f},
  { -52.1031151288303f,    0.0f},
  { -46.3045173538851f,    -3.49033866189271f},
  { -40.9590912848605f,    -6.65525720600019f},
  { -90.0f,    13.8858871481377f},
  { -84.0642077359095f,    10.8541527460023f},
  { -77.7816801337513f,    7.45192878147619f},
  { -71.2822831909298f,    3.78233409570435f},
  { -64.7263194139596f,    -1.99540459610511e-15f},
  { -58.2825255885390f,    -3.71807763036312f},
  { -52.1031151288303f,    -7.21192704412377f},
  { -46.3045173538851f,    -10.3692763159630f},
  { -90.0f,    7.04640638085718f},
  { -84.0642077359095f,    3.65698857497185f},
  { -77.7816801337513f,    -9.89130938771355e-16f},
  { -71.2822831909298f,    -3.78233409570436f},
  { -64.7263194139596f,    -7.51576871353677f},
  { -58.2825255885390f,    -11.0315494669714f},
  { -52.1031151288303f,    -14.2023070245090f},
  { -90.0f,    0.0f},
  { -84.0642077359095f,    -3.65698857497185f},
  { -77.7816801337513f,    -7.45192878147619f},
  { -71.2822831909298f,    -11.2179396537745f},
  { -64.7263194139596f,    -14.7814457540342f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -7.04640638085718f},
  { -84.0642077359095f,    -10.8541527460023f},
  { -77.7816801337513f,    -14.6599491964913f},
  { -71.2822831909298f,    -18.2913664299137f},
  { -64.7263194139596f,    -21.5935345276613f},
  { -90.0f,    -13.8858871481377f},
  { -84.0642077359095f,    -17.7221396001114f},
  { -77.7816801337513f,    -21.4249007093178f},
  { -71.2822831909298f,    -24.8333646999319f},
  { -90.0f,    -20.3458076143384f},
  { -84.0642077359095f,    -24.1034144840352f},
  { -77.7816801337513f,    -27.6183932347186f},
  { -90.0f,    -26.3090434555483f},
  { -84.0642077359095f,    -29.9083631516625f},
  { -95.9357922640906f,    29.9083631516625f},
  { -95.9357922640906f,    24.1034144840352f},
  { -95.9357922640906f,    17.7221396001114f},
  { -95.9357922640906f,    10.8541527460023f},
  { -95.9357922640906f,    3.65698857497185f},
  { -95.9357922640906f,    -3.65698857497184f},
  { -95.9357922640906f,    -10.8541527460023f},
  { -95.9357922640906f,    -17.7221396001114f},
  { -95.9357922640906f,    -24.1034144840352f},
  { -95.9357922640906f,    -29.9083631516625f},
  { -102.218319866249f,    27.6183932347186f},
  { -102.218319866249f,    21.4249007093178f},
  { -102.218319866249f,    14.6599491964913f},
  { -102.218319866249f,    7.45192878147619f},
  { -102.218319866249f,    0.0f},
  { -102.218319866249f,    -7.45192878147619f},
  { -102.218319866249f,    -14.6599491964913f},
  { -102.218319866249f,    -21.4249007093178f},
  { -102.218319866249f,    -27.6183932347186f},
  { -108.717716809070f,    24.8333646999319f},
  { -108.717716809070f,    18.2913664299137f},
  { -108.717716809070f,    11.2179396537745f},
  { -108.717716809070f,    3.78233409570435f},
  { -108.717716809070f,    -3.78233409570436f},
  { -108.717716809070f,    -11.2179396537745f},
  { -108.717716809070f,    -18.2913664299137f},
  { -108.717716809070f,    -24.8333646999319f},
  { -115.273680586040f,    21.5935345276613f},
  { -115.273680586040f,    14.7814457540342f},
  { -115.273680586040f,    7.51576871353677f},
  { -115.273680586040f,    0.0f},
  { -115.273680586040f,    -7.51576871353677f},
  { -115.273680586040f,    -14.7814457540342f},
  { -115.273680586040f,    -21.5935345276613f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    11.0315494669714f},
  { -121.717474411461f,    3.71807763036312f},
  { -121.717474411461f,    -3.71807763036312f},
  { -121.717474411461f,    -11.0315494669714f},
  { -121.717474411461f,    -18.0f},
  { -127.896884871170f,    14.2023070245089f},
  { -127.896884871170f,    7.21192704412376f},
  { -127.896884871170f,    0.0f},
  { -127.896884871170f,    -7.21192704412376f},
  { -127.896884871170f,    -14.2023070245089f},
  { -133.695482646115f,    10.3692763159630f},
  { -133.695482646115f,    3.49033866189271f},
  { -133.695482646115f,    -3.49033866189271f},
  { -133.695482646115f,    -10.3692763159630f},
  { -139.040908715140f,    6.65525720600019f},
  { -139.040908715140f,    0.0f},
  { -139.040908715140f,    -6.65525720600019f},
  { -143.902522791732f,    3.17590310185081f},
  { -143.902522791732f,    -3.17590310185081f},
  { 79.3665200338986f,    34.1334810267585f},
  { 72.3658791308137f,    31.6458891485011f},
  { 65.2665870176335f,    28.5668020560704f},
  { 58.2825255885390f,    24.9684505330286f},
  { 51.6077704763233f,    20.9980381353659f},
  { 45.3876771827074f,    16.8503785613218f},
  { 39.7072308752386f,    12.7247006938203f},
  { 34.5958937673059f,    8.78562653286162f},
  { 73.6418243122865f,    38.5483024802721f},
  { 65.9051574478893f,    35.7584993532105f},
  { 58.2825255885390f,    32.2819223696369f},
  { 51.0265526631344f,    28.2445194108839f},
  { 44.3218570598402f,    23.8529889684413f},
  { 38.2682973779569f,    19.3469890055802f},
  { 32.8906152938680f,    14.9450648361650f},
  { 66.6713328380939f,    42.9461472577416f},
  { 58.2825255885390f,    39.7180776303631f},
  { 50.3351698394960f,    35.7374493804081f},
  { 43.0684893347418f,    31.2052664566532f},
  { 36.5992704691010f,    26.3893742240811f},
  { 30.9409152799674f,    21.5591318742637f},
  { 58.2825255885390f,    47.0315494669714f},
  { 49.4992949875800f,    43.2255973783311f},
  { 41.5748854333382f,    38.6705372603619f},
  { 34.6438140225457f,    33.6469345870906f},
  { 28.6958406336181f,    28.4633471612544f},
  { 48.4688461888651f,    50.4572066397107f},
  { 39.7674171731903f,    45.9856701791279f},
  { 32.3272571370662f,    40.8680240213278f},
  { 26.0915113001869f,    35.4364628178122f},
  { 37.5405889662787f,    52.9103051134963f},
  { 29.5495629136742f,    47.7981724013467f},
  { 23.0480256277032f,    42.2334257255740f},
  { 26.1757460354111f,    54.2235755376742f},
  { 19.4664313178702f,    48.6271378369227f},
  { 15.2270244263796f,    54.4411370888702f},
  { 79.3665200338986f,    -34.1334810267585f},
  { 73.6418243122865f,    -38.5483024802721f},
  { 66.6713328380939f,    -42.9461472577416f},
  { 58.2825255885390f,    -47.0315494669714f},
  { 48.4688461888651f,    -50.4572066397107f},
  { 37.5405889662787f,    -52.9103051134963f},
  { 26.1757460354111f,    -54.2235755376742f},
  { 15.2270244263796f,    -54.4411370888702f},
  { 72.3658791308137f,    -31.6458891485011f},
  { 65.9051574478893f,    -35.7584993532105f},
  { 58.2825255885390f,    -39.7180776303631f},
  { 49.4992949875800f,    -43.2255973783311f},
  { 39.7674171731903f,    -45.9856701791279f},
  { 29.5495629136742f,    -47.7981724013467f},
  { 19.4664313178702f,    -48.6271378369227f},
  { 65.2665870176335f,    -28.5668020560704f},
  { 58.2825255885390f,    -32.2819223696369f},
  { 50.3351698394960f,    -35.7374493804081f},
  { 41.5748854333382f,    -38.6705372603619f},
  { 32.3272571370662f,    -40.8680240213278f},
  { 23.0480256277032f,    -42.2334257255739f},
  { 58.2825255885390f,    -24.9684505330286f},
  { 51.0265526631344f,    -28.2445194108839f},
  { 43.0684893347418f,    -31.2052664566532f},
  { 34.6438140225457f,    -33.6469345870906f},
  { 26.0915113001869f,    -35.4364628178122f},
  { 51.6077704763233f,    -20.9980381353659f},
  { 44.3218570598402f,    -23.8529889684413f},
  { 36.5992704691010f,    -26.3893742240811f},
  { 28.6958406336181f,    -28.4633471612544f},
  { 45.3876771827074f,    -16.8503785613218f},
  { 38.2682973779569f,    -19.3469890055802f},
  { 30.9409152799674f,    -21.5591318742637f},
  { 39.7072308752386f,    -12.7247006938203f},
  { 32.8906152938680f,    -14.9450648361650f},
  { 34.5958937673059f,    -8.78562653286162f},
  { -15.2270244263796f,    54.4411370888702f},
  { -19.4664313178702f,    48.6271378369227f},
  { -23.0480256277032f,    42.2334257255739f},
  { -26.0915113001869f,    35.4364628178122f},
  { -28.6958406336181f,    28.4633471612544f},
  { -30.9409152799674f,    21.5591318742637f},
  { -32.8906152938680f,    14.9450648361650f},
  { -34.5958937673059f,    8.78562653286162f},
  { -26.1757460354111f,    54.2235755376742f},
  { -29.5495629136742f,    47.7981724013467f},
  { -32.3272571370662f,    40.8680240213278f},
  { -34.6438140225457f,    33.6469345870906f},
  { -36.5992704691010f,    26.3893742240811f},
  { -38.2682973779569f,    19.3469890055802f},
  { -39.7072308752386f,    12.7247006938203f},
  { -37.5405889662787f,    52.9103051134963f},
  { -39.7674171731903f,    45.9856701791279f},
  { -41.5748854333382f,    38.6705372603619f},
  { -43.0684893347418f,    31.2052664566532f},
  { -44.3218570598402f,    23.8529889684413f},
  { -45.3876771827074f,    16.8503785613218f},
  { -48.4688461888651f,    50.4572066397107f},
  { -49.4992949875800f,    43.2255973783311f},
  { -50.3351698394960f,    35.7374493804082f},
  { -51.0265526631344f,    28.2445194108839f},
  { -51.6077704763233f,    20.9980381353659f},
  { -58.2825255885390f,    47.0315494669714f},
  { -58.2825255885390f,    39.7180776303631f},
  { -58.2825255885390f,    32.2819223696369f},
  { -58.2825255885390f,    24.9684505330286f},
  { -66.6713328380939f,    42.9461472577416f},
  { -65.9051574478893f,    35.7584993532105f},
  { -65.2665870176335f,    28.5668020560704f},
  { -73.6418243122865f,    38.5483024802721f},
  { -72.3658791308137f,    31.6458891485011f},
  { -79.3665200338986f,    34.1334810267585f},
  { -15.2270244263796f,    -54.4411370888702f},
  { -26.1757460354111f,    -54.2235755376742f},
  { -37.5405889662787f,    -52.9103051134963f},
  { -48.4688461888651f,    -50.4572066397107f},
  { -58.2825255885390f,    -47.0315494669714f},
  { -66.6713328380939f,    -42.9461472577416f},
  { -73.6418243122865f,    -38.5483024802721f},
  { -79.3665200338986f,    -34.1334810267585f},
  { -19.4664313178702f,    -48.6271378369227f},
  { -29.5495629136742f,    -47.7981724013467f},
  { -39.7674171731903f,    -45.9856701791279f},
  { -49.4992949875800f,    -43.2255973783311f},
  { -58.2825255885390f,    -39.7180776303631f},
  { -65.9051574478893f,    -35.7584993532105f},
  { -72.3658791308137f,    -31.6458891485011f},
  { -23.0480256277032f,    -42.2334257255739f},
  { -32.3272571370662f,    -40.8680240213278f},
  { -41.5748854333382f,    -38.6705372603619f},
  { -50.3351698394960f,    -35.7374493804081f},
  { -58.2825255885390f,    -32.2819223696369f},
  { -65.2665870176335f,    -28.5668020560704f},
  { -26.0915113001869f,    -35.4364628178122f},
  { -34.6438140225457f,    -33.6469345870906f},
  { -43.0684893347418f,    -31.2052664566532f},
  { -51.0265526631344f,    -28.2445194108839f},
  { -58.2825255885390f,    -24.9684505330286f},
  { -28.6958406336181f,    -28.4633471612544f},
  { -36.5992704691010f,    -26.3893742240811f},
  { -44.3218570598402f,    -23.8529889684413f},
  { -51.6077704763233f,    -20.9980381353659f},
  { -30.9409152799674f,    -21.5591318742637f},
  { -38.2682973779569f,    -19.3469890055802f},
  { -45.3876771827074f,    -16.8503785613218f},
  { -32.8906152938680f,    -14.9450648361650f},
  { -39.7072308752386f,    -12.7247006938203f},
  { -34.5958937673059f,    -8.78562653286162f},
  { 164.772975573620f,    54.4411370888702f},
  { 160.533568682130f,    48.6271378369227f},
  { 156.951974372297f,    42.2334257255739f},
  { 153.908488699813f,    35.4364628178122f},
  { 151.304159366382f,    28.4633471612544f},
  { 149.059084720033f,    21.5591318742637f},
  { 147.109384706132f,    14.9450648361650f},
  { 145.404106232694f,    8.78562653286162f},
  { 153.824253964589f,    54.2235755376742f},
  { 150.450437086326f,    47.7981724013467f},
  { 147.672742862934f,    40.8680240213278f},
  { 145.356185977454f,    33.6469345870906f},
  { 143.400729530899f,    26.3893742240811f},
  { 141.731702622043f,    19.3469890055802f},
  { 140.292769124761f,    12.7247006938203f},
  { 142.459411033721f,    52.9103051134963f},
  { 140.232582826810f,    45.9856701791279f},
  { 138.425114566662f,    38.6705372603619f},
  { 136.931510665258f,    31.2052664566532f},
  { 135.678142940160f,    23.8529889684413f},
  { 134.612322817293f,    16.8503785613218f},
  { 131.531153811135f,    50.4572066397107f},
  { 130.500705012420f,    43.2255973783311f},
  { 129.664830160504f,    35.7374493804082f},
  { 128.973447336866f,    28.2445194108839f},
  { 128.392229523677f,    20.9980381353659f},
  { 121.717474411461f,    47.0315494669714f},
  { 121.717474411461f,    39.7180776303631f},
  { 121.717474411461f,    32.2819223696369f},
  { 121.717474411461f,    24.9684505330286f},
  { 113.328667161906f,    42.9461472577416f},
  { 114.094842552111f,    35.7584993532105f},
  { 114.733412982366f,    28.5668020560704f},
  { 106.358175687714f,    38.5483024802721f},
  { 107.634120869186f,    31.6458891485011f},
  { 100.633479966101f,    34.1334810267585f},
  { 164.772975573620f,    -54.4411370888702f},
  { 153.824253964589f,    -54.2235755376742f},
  { 142.459411033721f,    -52.9103051134963f},
  { 131.531153811135f,    -50.4572066397107f},
  { 121.717474411461f,    -47.0315494669714f},
  { 113.328667161906f,    -42.9461472577416f},
  { 106.358175687714f,    -38.5483024802721f},
  { 100.633479966101f,    -34.1334810267585f},
  { 160.533568682130f,    -48.6271378369227f},
  { 150.450437086326f,    -47.7981724013467f},
  { 140.232582826810f,    -45.9856701791279f},
  { 130.500705012420f,    -43.2255973783311f},
  { 121.717474411461f,    -39.7180776303631f},
  { 114.094842552111f,    -35.7584993532105f},
  { 107.634120869186f,    -31.6458891485011f},
  { 156.951974372297f,    -42.2334257255739f},
  { 147.672742862934f,    -40.8680240213278f},
  { 138.425114566662f,    -38.6705372603619f},
  { 129.664830160504f,    -35.7374493804081f},
  { 121.717474411461f,    -32.2819223696369f},
  { 114.733412982366f,    -28.5668020560704f},
  { 153.908488699813f,    -35.4364628178122f},
  { 145.356185977454f,    -33.6469345870906f},
  { 136.931510665258f,    -31.2052664566532f},
  { 128.973447336866f,    -28.2445194108839f},
  { 121.717474411461f,    -24.9684505330286f},
  { 151.304159366382f,    -28.4633471612544f},
  { 143.400729530899f,    -26.3893742240811f},
  { 135.678142940160f,    -23.8529889684413f},
  { 128.392229523677f,    -20.9980381353659f},
  { 149.059084720033f,    -21.5591318742637f},
  { 141.731702622043f,    -19.3469890055802f},
  { 134.612322817293f,    -16.8503785613218f},
  { 147.109384706132f,    -14.9450648361650f},
  { 140.292769124761f,    -12.7247006938203f},
  { 145.404106232694f,    -8.78562653286162f},
  { -164.772975573620f,    54.4411370888702f},
  { -153.824253964589f,    54.2235755376742f},
  { -142.459411033721f,    52.9103051134963f},
  { -131.531153811135f,    50.4572066397107f},
  { -121.717474411461f,    47.0315494669714f},
  { -113.328667161906f,    42.9461472577416f},
  { -106.358175687714f,    38.5483024802721f},
  { -100.633479966101f,    34.1334810267585f},
  { -160.533568682130f,    48.6271378369227f},
  { -150.450437086326f,    47.7981724013467f},
  { -140.232582826810f,    45.9856701791279f},
  { -130.500705012420f,    43.2255973783311f},
  { -121.717474411461f,    39.7180776303631f},
  { -114.094842552111f,    35.7584993532105f},
  { -107.634120869186f,    31.6458891485011f},
  { -156.951974372297f,    42.2334257255739f},
  { -147.672742862934f,    40.8680240213278f},
  { -138.425114566662f,    38.6705372603619f},
  { -129.664830160504f,    35.7374493804081f},
  { -121.717474411461f,    32.2819223696369f},
  { -114.733412982366f,    28.5668020560704f},
  { -153.908488699813f,    35.4364628178122f},
  { -145.356185977454f,    33.6469345870906f},
  { -136.931510665258f,    31.2052664566532f},
  { -128.973447336866f,    28.2445194108839f},
  { -121.717474411461f,    24.9684505330286f},
  { -151.304159366382f,    28.4633471612544f},
  { -143.400729530899f,    26.3893742240811f},
  { -135.678142940160f,    23.8529889684413f},
  { -128.392229523677f,    20.9980381353659f},
  { -149.059084720033f,    21.5591318742637f},
  { -141.731702622043f,    19.3469890055802f},
  { -134.612322817293f,    16.8503785613218f},
  { -147.109384706132f,    14.9450648361650f},
  { -140.292769124761f,    12.7247006938203f},
  { -145.404106232694f,    8.78562653286162f},
  { -164.772975573620f,    -54.4411370888702f},
  { -160.533568682130f,    -48.6271378369227f},
  { -156.951974372297f,    -42.2334257255739f},
  { -153.908488699813f,    -35.4364628178122f},
  { -151.304159366382f,    -28.4633471612544f},
  { -149.059084720033f,    -21.5591318742637f},
  { -147.109384706132f,    -14.9450648361650f},
  { -145.404106232694f,    -8.78562653286162f},
  { -153.824253964589f,    -54.2235755376742f},
  { -150.450437086326f,    -47.7981724013467f},
  { -147.672742862934f,    -40.8680240213278f},
  { -145.356185977454f,    -33.6469345870906f},
  { -143.400729530899f,    -26.3893742240811f},
  { -141.731702622043f,    -19.3469890055802f},
  { -140.292769124761f,    -12.7247006938203f},
  { -142.459411033721f,    -52.9103051134963f},
  { -140.232582826810f,    -45.9856701791279f},
  { -138.425114566662f,    -38.6705372603619f},
  { -136.931510665258f,    -31.2052664566532f},
  { -135.678142940160f,    -23.8529889684413f},
  { -134.612322817293f,    -16.8503785613218f},
  { -131.531153811135f,    -50.4572066397107f},
  { -130.500705012420f,    -43.2255973783311f},
  { -129.664830160504f,    -35.7374493804082f},
  { -128.973447336866f,    -28.2445194108839f},
  { -128.392229523677f,    -20.9980381353659f},
  { -121.717474411461f,    -47.0315494669714f},
  { -121.717474411461f,    -39.7180776303631f},
  { -121.717474411461f,    -32.2819223696369f},
  { -121.717474411461f,    -24.9684505330286f},
  { -113.328667161906f,    -42.9461472577416f},
  { -114.094842552111f,    -35.7584993532105f},
  { -114.733412982366f,    -28.5668020560704f},
  { -106.358175687714f,    -38.5483024802721f},
  { -107.634120869186f,    -31.6458891485011f},
  { -100.633479966101f,    -34.1334810267585f}};

/* 3LD geosphere 'ico' [11 0] (1212 points) */
const float __geosphere_ico_11_0_dirs_deg[1212][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    63.1759109378142f},
  { 0.0f,    68.5305657000874f},
  { 0.0f,    74.3086441830498f},
  { 0.0f,    80.4324561201698f},
  { 0.0f,    86.7842228465557f},
  { 180.0f,    86.7842228465557f},
  { 180.0f,    80.4324561201698f},
  { 180.0f,    74.3086441830498f},
  { 180.0f,    68.5305657000874f},
  { 180.0f,    63.1759109378142f},
  { 9.19099647608508f,    59.4750930157754f},
  { 11.4340697442537f,    64.5886747619424f},
  { 15.0920752881032f,    70.1143173072657f},
  { 22.0237505161462f,    75.9016975056681f},
  { 38.9734473368657f,    81.4839022669666f},
  { 90.0f,    84.6197878514433f},
  { 141.026552663134f,    81.4839022669666f},
  { 157.976249483854f,    75.9016975056681f},
  { 164.907924711897f,    70.1143173072657f},
  { 168.565930255746f,    64.5886747619424f},
  { 170.809003523915f,    59.4750930157754f},
  { 19.7767131841238f,    59.9951876481773f},
  { 24.8108847601265f,    65.0321522571359f},
  { 32.9114931857064f,    70.2212812855334f},
  { 47.1679114977869f,    75.0811491202448f},
  { 72.8279621494862f,    78.4421610808621f},
  { 107.172037850514f,    78.4421610808621f},
  { 132.832088502213f,    75.0811491202448f},
  { 147.088506814294f,    70.2212812855334f},
  { 155.189115239873f,    65.0321522571359f},
  { 160.223286815876f,    59.9951876481773f},
  { 31.2478160930689f,    59.5919152427842f},
  { 38.9734473368657f,    64.1711710398197f},
  { 50.5099309427673f,    68.4712566441962f},
  { 67.6072290710451f,    71.7748006410625f},
  { 90.0f,    73.0675620848742f},
  { 112.392770928955f,    71.7748006410625f},
  { 129.490069057233f,    68.4712566441962f},
  { 141.026552663134f,    64.1711710398197f},
  { 148.752183906931f,    59.5919152427842f},
  { 42.7561895905164f,    58.1166941548564f},
  { 52.3123893000598f,    61.9142772940601f},
  { 65.1310748815380f,    65.0403053382520f},
  { 81.2167693990410f,    66.8628767649577f},
  { 98.7832306009590f,    66.8628767649577f},
  { 114.868925118462f,    65.0403053382520f},
  { 127.687610699940f,    61.9142772940601f},
  { 137.243810409484f,    58.1166941548564f},
  { 53.4378607612508f,    55.5963831147141f},
  { 63.6909565444517f,    58.4665560721094f},
  { 76.1141128518623f,    60.4639637602226f},
  { 90.0f,    61.1872390127156f},
  { 103.885887148138f,    60.4639637602226f},
  { 116.309043455548f,    58.4665560721094f},
  { 126.562139238749f,    55.5963831147141f},
  { 62.7502499362301f,    52.2236867301566f},
  { 72.8279621494862f,    54.2026444179102f},
  { 84.1189512527134f,    55.2915500640743f},
  { 95.8810487472866f,    55.2915500640743f},
  { 107.172037850514f,    54.2026444179102f},
  { 117.249750063770f,    52.2236867301566f},
  { 70.5487009486867f,    48.2796684813088f},
  { 79.9858774849308f,    49.5118734071249f},
  { 90.0f,    49.9457026117864f},
  { 100.014122515069f,    49.5118734071249f},
  { 109.451299051313f,    48.2796684813088f},
  { 76.9513489599765f,    44.0504703995995f},
  { 85.5824320004692f,    44.7145105417809f},
  { 94.4175679995308f,    44.7145105417809f},
  { 103.048651040024f,    44.0504703995995f},
  { 82.1798710590254f,    39.7754154552892f},
  { 90.0f,    40.0388650148264f},
  { 97.8201289409746f,    39.7754154552892f},
  { 86.4634243465556f,    35.6279040820967f},
  { 93.5365756534444f,    35.6279040820967f},
  { -9.19099647608508f,    59.4750930157754f},
  { -19.7767131841238f,    59.9951876481773f},
  { -31.2478160930689f,    59.5919152427842f},
  { -42.7561895905164f,    58.1166941548564f},
  { -53.4378607612508f,    55.5963831147141f},
  { -62.7502499362301f,    52.2236867301566f},
  { -70.5487009486867f,    48.2796684813088f},
  { -76.9513489599765f,    44.0504703995995f},
  { -82.1798710590254f,    39.7754154552892f},
  { -86.4634243465556f,    35.6279040820967f},
  { -11.4340697442537f,    64.5886747619424f},
  { -24.8108847601265f,    65.0321522571359f},
  { -38.9734473368657f,    64.1711710398197f},
  { -52.3123893000598f,    61.9142772940601f},
  { -63.6909565444517f,    58.4665560721094f},
  { -72.8279621494862f,    54.2026444179102f},
  { -79.9858774849308f,    49.5118734071249f},
  { -85.5824320004692f,    44.7145105417809f},
  { -90.0000000000000f,    40.0388650148264f},
  { -93.5365756534444f,    35.6279040820967f},
  { -15.0920752881032f,    70.1143173072657f},
  { -32.9114931857063f,    70.2212812855334f},
  { -50.5099309427673f,    68.4712566441962f},
  { -65.1310748815380f,    65.0403053382520f},
  { -76.1141128518623f,    60.4639637602226f},
  { -84.1189512527134f,    55.2915500640744f},
  { -90.0f,    49.9457026117864f},
  { -94.4175679995308f,    44.7145105417809f},
  { -97.8201289409746f,    39.7754154552892f},
  { -22.0237505161462f,    75.9016975056681f},
  { -47.1679114977869f,    75.0811491202448f},
  { -67.6072290710451f,    71.7748006410625f},
  { -81.2167693990410f,    66.8628767649577f},
  { -90.0000000000000f,    61.1872390127156f},
  { -95.8810487472866f,    55.2915500640744f},
  { -100.014122515069f,    49.5118734071249f},
  { -103.048651040024f,    44.0504703995995f},
  { -38.9734473368657f,    81.4839022669666f},
  { -72.8279621494862f,    78.4421610808621f},
  { -90.0f,    73.0675620848742f},
  { -98.7832306009590f,    66.8628767649577f},
  { -103.885887148138f,    60.4639637602226f},
  { -107.172037850514f,    54.2026444179102f},
  { -109.451299051313f,    48.2796684813088f},
  { -90.0000000000000f,    84.6197878514433f},
  { -107.172037850514f,    78.4421610808621f},
  { -112.392770928955f,    71.7748006410625f},
  { -114.868925118462f,    65.0403053382520f},
  { -116.309043455548f,    58.4665560721094f},
  { -117.249750063770f,    52.2236867301566f},
  { -141.026552663134f,    81.4839022669666f},
  { -132.832088502213f,    75.0811491202448f},
  { -129.490069057233f,    68.4712566441962f},
  { -127.687610699940f,    61.9142772940601f},
  { -126.562139238749f,    55.5963831147141f},
  { -157.976249483854f,    75.9016975056681f},
  { -147.088506814294f,    70.2212812855334f},
  { -141.026552663134f,    64.1711710398197f},
  { -137.243810409484f,    58.1166941548564f},
  { -164.907924711897f,    70.1143173072657f},
  { -155.189115239873f,    65.0321522571359f},
  { -148.752183906931f,    59.5919152427842f},
  { -168.565930255746f,    64.5886747619424f},
  { -160.223286815876f,    59.9951876481773f},
  { -170.809003523915f,    59.4750930157754f},
  { 9.19099647608508f,    -59.4750930157754f},
  { 19.7767131841238f,    -59.9951876481773f},
  { 31.2478160930689f,    -59.5919152427842f},
  { 42.7561895905164f,    -58.1166941548564f},
  { 53.4378607612508f,    -55.5963831147141f},
  { 62.7502499362301f,    -52.2236867301566f},
  { 70.5487009486867f,    -48.2796684813088f},
  { 76.9513489599765f,    -44.0504703995995f},
  { 82.1798710590254f,    -39.7754154552892f},
  { 86.4634243465556f,    -35.6279040820967f},
  { 0.0f,    -63.1759109378142f},
  { 11.4340697442537f,    -64.5886747619424f},
  { 24.8108847601265f,    -65.0321522571359f},
  { 38.9734473368657f,    -64.1711710398197f},
  { 52.3123893000598f,    -61.9142772940601f},
  { 63.6909565444517f,    -58.4665560721094f},
  { 72.8279621494862f,    -54.2026444179102f},
  { 79.9858774849308f,    -49.5118734071249f},
  { 85.5824320004692f,    -44.7145105417809f},
  { 90.0000000000000f,    -40.0388650148264f},
  { 93.5365756534444f,    -35.6279040820967f},
  { 0.0f,    -68.5305657000874f},
  { 15.0920752881032f,    -70.1143173072657f},
  { 32.9114931857063f,    -70.2212812855334f},
  { 50.5099309427673f,    -68.4712566441962f},
  { 65.1310748815380f,    -65.0403053382520f},
  { 76.1141128518623f,    -60.4639637602226f},
  { 84.1189512527134f,    -55.2915500640744f},
  { 90.0f,    -49.9457026117864f},
  { 94.4175679995308f,    -44.7145105417809f},
  { 97.8201289409746f,    -39.7754154552892f},
  { 0.0f,    -74.3086441830498f},
  { 22.0237505161462f,    -75.9016975056681f},
  { 47.1679114977869f,    -75.0811491202448f},
  { 67.6072290710451f,    -71.7748006410625f},
  { 81.2167693990410f,    -66.8628767649577f},
  { 90.0000000000000f,    -61.1872390127156f},
  { 95.8810487472866f,    -55.2915500640744f},
  { 100.014122515069f,    -49.5118734071249f},
  { 103.048651040024f,    -44.0504703995995f},
  { 0.0f,    -80.4324561201698f},
  { 38.9734473368657f,    -81.4839022669666f},
  { 72.8279621494862f,    -78.4421610808621f},
  { 90.0f,    -73.0675620848742f},
  { 98.7832306009590f,    -66.8628767649577f},
  { 103.885887148138f,    -60.4639637602226f},
  { 107.172037850514f,    -54.2026444179102f},
  { 109.451299051313f,    -48.2796684813088f},
  { 0.0f,    -86.7842228465557f},
  { 90.0000000000000f,    -84.6197878514433f},
  { 107.172037850514f,    -78.4421610808621f},
  { 112.392770928955f,    -71.7748006410625f},
  { 114.868925118462f,    -65.0403053382520f},
  { 116.309043455548f,    -58.4665560721094f},
  { 117.249750063770f,    -52.2236867301566f},
  { 180.0f,    -86.7842228465557f},
  { 141.026552663134f,    -81.4839022669666f},
  { 132.832088502213f,    -75.0811491202448f},
  { 129.490069057233f,    -68.4712566441962f},
  { 127.687610699940f,    -61.9142772940601f},
  { 126.562139238749f,    -55.5963831147141f},
  { 180.0f,    -80.4324561201698f},
  { 157.976249483854f,    -75.9016975056681f},
  { 147.088506814294f,    -70.2212812855334f},
  { 141.026552663134f,    -64.1711710398197f},
  { 137.243810409484f,    -58.1166941548564f},
  { 180.0f,    -74.3086441830498f},
  { 164.907924711897f,    -70.1143173072657f},
  { 155.189115239873f,    -65.0321522571359f},
  { 148.752183906931f,    -59.5919152427842f},
  { 180.0f,    -68.5305657000874f},
  { 168.565930255746f,    -64.5886747619424f},
  { 160.223286815876f,    -59.9951876481773f},
  { 180.0f,    -63.1759109378142f},
  { 170.809003523915f,    -59.4750930157754f},
  { -9.19099647608508f,    -59.4750930157754f},
  { -11.4340697442537f,    -64.5886747619424f},
  { -15.0920752881032f,    -70.1143173072657f},
  { -22.0237505161462f,    -75.9016975056681f},
  { -38.9734473368657f,    -81.4839022669666f},
  { -90.0f,    -84.6197878514433f},
  { -141.026552663134f,    -81.4839022669666f},
  { -157.976249483854f,    -75.9016975056681f},
  { -164.907924711897f,    -70.1143173072657f},
  { -168.565930255746f,    -64.5886747619424f},
  { -170.809003523915f,    -59.4750930157754f},
  { -19.7767131841238f,    -59.9951876481773f},
  { -24.8108847601265f,    -65.0321522571359f},
  { -32.9114931857064f,    -70.2212812855334f},
  { -47.1679114977869f,    -75.0811491202448f},
  { -72.8279621494862f,    -78.4421610808621f},
  { -107.172037850514f,    -78.4421610808621f},
  { -132.832088502213f,    -75.0811491202448f},
  { -147.088506814294f,    -70.2212812855334f},
  { -155.189115239873f,    -65.0321522571359f},
  { -160.223286815876f,    -59.9951876481773f},
  { -31.2478160930689f,    -59.5919152427842f},
  { -38.9734473368657f,    -64.1711710398197f},
  { -50.5099309427673f,    -68.4712566441962f},
  { -67.6072290710451f,    -71.7748006410625f},
  { -90.0f,    -73.0675620848742f},
  { -112.392770928955f,    -71.7748006410625f},
  { -129.490069057233f,    -68.4712566441962f},
  { -141.026552663134f,    -64.1711710398197f},
  { -148.752183906931f,    -59.5919152427842f},
  { -42.7561895905164f,    -58.1166941548564f},
  { -52.3123893000598f,    -61.9142772940601f},
  { -65.1310748815380f,    -65.0403053382520f},
  { -81.2167693990410f,    -66.8628767649577f},
  { -98.7832306009590f,    -66.8628767649577f},
  { -114.868925118462f,    -65.0403053382520f},
  { -127.687610699940f,    -61.9142772940601f},
  { -137.243810409484f,    -58.1166941548564f},
  { -53.4378607612508f,    -55.5963831147141f},
  { -63.6909565444517f,    -58.4665560721094f},
  { -76.1141128518623f,    -60.4639637602226f},
  { -90.0f,    -61.1872390127156f},
  { -103.885887148138f,    -60.4639637602226f},
  { -116.309043455548f,    -58.4665560721094f},
  { -126.562139238749f,    -55.5963831147141f},
  { -62.7502499362301f,    -52.2236867301566f},
  { -72.8279621494862f,    -54.2026444179102f},
  { -84.1189512527134f,    -55.2915500640743f},
  { -95.8810487472866f,    -55.2915500640743f},
  { -107.172037850514f,    -54.2026444179102f},
  { -117.249750063770f,    -52.2236867301566f},
  { -70.5487009486867f,    -48.2796684813088f},
  { -79.9858774849308f,    -49.5118734071249f},
  { -90.0f,    -49.9457026117864f},
  { -100.014122515069f,    -49.5118734071249f},
  { -109.451299051313f,    -48.2796684813088f},
  { -76.9513489599765f,    -44.0504703995995f},
  { -85.5824320004692f,    -44.7145105417809f},
  { -94.4175679995308f,    -44.7145105417809f},
  { -103.048651040024f,    -44.0504703995995f},
  { -82.1798710590254f,    -39.7754154552892f},
  { -90.0f,    -40.0388650148264f},
  { -97.8201289409746f,    -39.7754154552892f},
  { -86.4634243465556f,    -35.6279040820967f},
  { -93.5365756534444f,    -35.6279040820967f},
  { 26.8240890621858f,    0.0f},
  { 21.4694342999126f,    0.0f},
  { 15.6913558169502f,    0.0f},
  { 9.56754387983022f,    0.0f},
  { 3.21577715344426f,    0.0f},
  { -3.21577715344426f,    0.0f},
  { -9.56754387983022f,    0.0f},
  { -15.6913558169503f,    0.0f},
  { -21.4694342999126f,    0.0f},
  { -26.8240890621858f,    0.0f},
  { 30.2020013474811f,    4.65334435269144f},
  { 24.9689719808620f,    4.87991118301845f},
  { 19.2510603421880f,    5.08098739442645f},
  { 13.1063577163078f,    5.24085275917997f},
  { 6.64003671544863f,    5.34433246947857f},
  { 0.0f,    5.38021214855673f},
  { -6.64003671544863f,    5.34433246947857f},
  { -13.1063577163078f,    5.24085275917997f},
  { -19.2510603421880f,    5.08098739442645f},
  { -24.9689719808620f,    4.87991118301845f},
  { -30.2020013474811f,    4.65334435269144f},
  { 28.5197716915180f,    9.74144555329516f},
  { 22.9111780425796f,    10.2024919920318f},
  { 16.7985575596605f,    10.5947297486039f},
  { 10.2669065947755f,    10.8828372167504f},
  { 3.45521376237835f,    11.0359947952031f},
  { -3.45521376237835f,    11.0359947952031f},
  { -10.2669065947755f,    10.8828372167504f},
  { -16.7985575596605f,    10.5947297486039f},
  { -22.9111780425796f,    10.2024919920318f},
  { -28.5197716915180f,    9.74144555329516f},
  { 26.6450987790749f,    15.2222212843682f},
  { 20.6218554159671f,    15.9040925262813f},
  { 14.0833535422747f,    16.4513996729311f},
  { 7.14966228681572f,    16.8082297996342f},
  { 0.0f,    16.9324379151258f},
  { -7.14966228681572f,    16.8082297996342f},
  { -14.0833535422747f,    16.4513996729311f},
  { -20.6218554159671f,    15.9040925262813f},
  { -26.6450987790749f,    15.2222212843682f},
  { 24.5477486770727f,    21.0129107120547f},
  { 18.0682898889764f,    21.8738327838630f},
  { 11.0752009382906f,    22.5104100950456f},
  { 3.73311959737546f,    22.8503240035940f},
  { -3.73311959737546f,    22.8503240035940f},
  { -11.0752009382906f,    22.5104100950456f},
  { -18.0682898889764f,    21.8738327838630f},
  { -24.5477486770727f,    21.0129107120547f},
  { 22.1922961938272f,    26.9896607617352f},
  { 15.2140362537972f,    27.9578822105749f},
  { 7.74350293451688f,    28.5916635489123f},
  { 0.0f,    28.8127609872845f},
  { -7.74350293451688f,    28.5916635489123f},
  { -15.2140362537972f,    27.9578822105749f},
  { -22.1922961938272f,    26.9896607617352f},
  { 19.5376021821740f,    32.9970359799122f},
  { 12.0196146161308f,    33.9760281659424f},
  { 4.05955859949662f,    34.4998357882260f},
  { -4.05955859949663f,    34.4998357882260f},
  { -12.0196146161308f,    33.9760281659424f},
  { -19.5376021821740f,    32.9970359799122f},
  { 16.5366343097382f,    38.8668029188042f},
  { 8.44414725560361f,    39.7469791157059f},
  { 0.0f,    40.0542973882136f},
  { -8.44414725560361f,    39.7469791157059f},
  { -16.5366343097382f,    38.8668029188042f},
  { 13.1370216726912f,    44.4405753118447f},
  { 4.44843274406601f,    45.1138288011647f},
  { -4.44843274406601f,    45.1138288011647f},
  { -13.1370216726912f,    44.4405753118447f},
  { 9.28297589604064f,    49.5887114401663f},
  { 0.0f,    49.9611349851736f},
  { -9.28297589604064f,    49.5887114401663f},
  { 4.91949944776620f,    54.2201266303858f},
  { -4.91949944776620f,    54.2201266303858f},
  { 30.2020013474811f,    -4.65334435269144f},
  { 28.5197716915180f,    -9.74144555329516f},
  { 26.6450987790749f,    -15.2222212843682f},
  { 24.5477486770727f,    -21.0129107120547f},
  { 22.1922961938272f,    -26.9896607617352f},
  { 19.5376021821740f,    -32.9970359799122f},
  { 16.5366343097382f,    -38.8668029188042f},
  { 13.1370216726912f,    -44.4405753118447f},
  { 9.28297589604064f,    -49.5887114401663f},
  { 4.91949944776620f,    -54.2201266303858f},
  { 24.9689719808620f,    -4.87991118301845f},
  { 22.9111780425796f,    -10.2024919920318f},
  { 20.6218554159671f,    -15.9040925262813f},
  { 18.0682898889764f,    -21.8738327838630f},
  { 15.2140362537972f,    -27.9578822105749f},
  { 12.0196146161308f,    -33.9760281659424f},
  { 8.44414725560361f,    -39.7469791157059f},
  { 4.44843274406601f,    -45.1138288011647f},
  { -5.43863598483001e-15f,    -49.9611349851736f},
  { -4.91949944776620f,    -54.2201266303858f},
  { 19.2510603421880f,    -5.08098739442645f},
  { 16.7985575596605f,    -10.5947297486039f},
  { 14.0833535422747f,    -16.4513996729311f},
  { 11.0752009382906f,    -22.5104100950456f},
  { 7.74350293451688f,    -28.5916635489123f},
  { 4.05955859949663f,    -34.4998357882260f},
  { 4.93964133360460e-15f,    -40.0542973882136f},
  { -4.44843274406601f,    -45.1138288011647f},
  { -9.28297589604064f,    -49.5887114401663f},
  { 13.1063577163078f,    -5.24085275917997f},
  { 10.2669065947755f,    -10.8828372167504f},
  { 7.14966228681571f,    -16.8082297996342f},
  { 3.73311959737546f,    -22.8503240035940f},
  { -4.52451706004787e-15f,    -28.8127609872845f},
  { -4.05955859949663f,    -34.4998357882260f},
  { -8.44414725560362f,    -39.7469791157059f},
  { -13.1370216726912f,    -44.4405753118447f},
  { 6.64003671544863f,    -5.34433246947857f},
  { 3.45521376237834f,    -11.0359947952031f},
  { 0.0f,    -16.9324379151258f},
  { -3.73311959737546f,    -22.8503240035940f},
  { -7.74350293451688f,    -28.5916635489123f},
  { -12.0196146161308f,    -33.9760281659424f},
  { -16.5366343097382f,    -38.8668029188042f},
  { 1.45255099835828e-15f,    -5.38021214855673f},
  { -3.45521376237834f,    -11.0359947952031f},
  { -7.14966228681571f,    -16.8082297996342f},
  { -11.0752009382906f,    -22.5104100950456f},
  { -15.2140362537972f,    -27.9578822105749f},
  { -19.5376021821740f,    -32.9970359799122f},
  { -6.64003671544864f,    -5.34433246947857f},
  { -10.2669065947755f,    -10.8828372167504f},
  { -14.0833535422747f,    -16.4513996729311f},
  { -18.0682898889764f,    -21.8738327838630f},
  { -22.1922961938272f,    -26.9896607617352f},
  { -13.1063577163078f,    -5.24085275917997f},
  { -16.7985575596605f,    -10.5947297486039f},
  { -20.6218554159671f,    -15.9040925262813f},
  { -24.5477486770727f,    -21.0129107120547f},
  { -19.2510603421880f,    -5.08098739442645f},
  { -22.9111780425796f,    -10.2024919920318f},
  { -26.6450987790749f,    -15.2222212843682f},
  { -24.9689719808620f,    -4.87991118301845f},
  { -28.5197716915180f,    -9.74144555329515f},
  { -30.2020013474811f,    -4.65334435269144f},
  { -153.175910937814f,    0.0f},
  { -158.530565700087f,    0.0f},
  { -164.308644183050f,    0.0f},
  { -170.432456120170f,    0.0f},
  { -176.784222846556f,    0.0f},
  { 176.784222846556f,    0.0f},
  { 170.432456120170f,    0.0f},
  { 164.308644183050f,    0.0f},
  { 158.530565700087f,    0.0f},
  { 153.175910937814f,    0.0f},
  { -149.797998652519f,    4.65334435269144f},
  { -155.031028019138f,    4.87991118301845f},
  { -160.748939657812f,    5.08098739442645f},
  { -166.893642283692f,    5.24085275917997f},
  { -173.359963284551f,    5.34433246947857f},
  { 180.0f,    5.38021214855673f},
  { 173.359963284551f,    5.34433246947857f},
  { 166.893642283692f,    5.24085275917997f},
  { 160.748939657812f,    5.08098739442645f},
  { 155.031028019138f,    4.87991118301845f},
  { 149.797998652519f,    4.65334435269144f},
  { -151.480228308482f,    9.74144555329516f},
  { -157.088821957420f,    10.2024919920318f},
  { -163.201442440340f,    10.5947297486039f},
  { -169.733093405224f,    10.8828372167504f},
  { -176.544786237622f,    11.0359947952031f},
  { 176.544786237622f,    11.0359947952031f},
  { 169.733093405224f,    10.8828372167504f},
  { 163.201442440340f,    10.5947297486039f},
  { 157.088821957420f,    10.2024919920318f},
  { 151.480228308482f,    9.74144555329516f},
  { -153.354901220925f,    15.2222212843682f},
  { -159.378144584033f,    15.9040925262813f},
  { -165.916646457725f,    16.4513996729311f},
  { -172.850337713184f,    16.8082297996342f},
  { 180.0f,    16.9324379151258f},
  { 172.850337713184f,    16.8082297996342f},
  { 165.916646457725f,    16.4513996729311f},
  { 159.378144584033f,    15.9040925262813f},
  { 153.354901220925f,    15.2222212843682f},
  { -155.452251322927f,    21.0129107120547f},
  { -161.931710111024f,    21.8738327838630f},
  { -168.924799061709f,    22.5104100950456f},
  { -176.266880402625f,    22.8503240035940f},
  { 176.266880402625f,    22.8503240035940f},
  { 168.924799061709f,    22.5104100950456f},
  { 161.931710111024f,    21.8738327838630f},
  { 155.452251322927f,    21.0129107120547f},
  { -157.807703806173f,    26.9896607617352f},
  { -164.785963746203f,    27.9578822105749f},
  { -172.256497065483f,    28.5916635489123f},
  { 180.0f,    28.8127609872845f},
  { 172.256497065483f,    28.5916635489123f},
  { 164.785963746203f,    27.9578822105749f},
  { 157.807703806173f,    26.9896607617352f},
  { -160.462397817826f,    32.9970359799122f},
  { -167.980385383869f,    33.9760281659424f},
  { -175.940441400503f,    34.4998357882260f},
  { 175.940441400503f,    34.4998357882260f},
  { 167.980385383869f,    33.9760281659424f},
  { 160.462397817826f,    32.9970359799122f},
  { -163.463365690262f,    38.8668029188042f},
  { -171.555852744396f,    39.7469791157059f},
  { 180.0f,    40.0542973882136f},
  { 171.555852744396f,    39.7469791157059f},
  { 163.463365690262f,    38.8668029188042f},
  { -166.862978327309f,    44.4405753118447f},
  { -175.551567255934f,    45.1138288011647f},
  { 175.551567255934f,    45.1138288011647f},
  { 166.862978327309f,    44.4405753118447f},
  { -170.717024103959f,    49.5887114401663f},
  { 180.0f,    49.9611349851736f},
  { 170.717024103959f,    49.5887114401663f},
  { -175.080500552234f,    54.2201266303858f},
  { 175.080500552234f,    54.2201266303858f},
  { -149.797998652519f,    -4.65334435269144f},
  { -151.480228308482f,    -9.74144555329516f},
  { -153.354901220925f,    -15.2222212843682f},
  { -155.452251322927f,    -21.0129107120547f},
  { -157.807703806173f,    -26.9896607617352f},
  { -160.462397817826f,    -32.9970359799122f},
  { -163.463365690262f,    -38.8668029188042f},
  { -166.862978327309f,    -44.4405753118447f},
  { -170.717024103959f,    -49.5887114401663f},
  { -175.080500552234f,    -54.2201266303858f},
  { -155.031028019138f,    -4.87991118301845f},
  { -157.088821957420f,    -10.2024919920318f},
  { -159.378144584033f,    -15.9040925262813f},
  { -161.931710111024f,    -21.8738327838630f},
  { -164.785963746203f,    -27.9578822105749f},
  { -167.980385383869f,    -33.9760281659424f},
  { -171.555852744396f,    -39.7469791157059f},
  { -175.551567255934f,    -45.1138288011647f},
  { 180.0f,    -49.9611349851736f},
  { 175.080500552234f,    -54.2201266303858f},
  { -160.748939657812f,    -5.08098739442645f},
  { -163.201442440340f,    -10.5947297486039f},
  { -165.916646457725f,    -16.4513996729311f},
  { -168.924799061709f,    -22.5104100950456f},
  { -172.256497065483f,    -28.5916635489123f},
  { -175.940441400503f,    -34.4998357882260f},
  { -180.0f,    -40.0542973882136f},
  { 175.551567255934f,    -45.1138288011647f},
  { 170.717024103959f,    -49.5887114401663f},
  { -166.893642283692f,    -5.24085275917997f},
  { -169.733093405225f,    -10.8828372167504f},
  { -172.850337713184f,    -16.8082297996342f},
  { -176.266880402625f,    -22.8503240035940f},
  { 180.0f,    -28.8127609872845f},
  { 175.940441400503f,    -34.4998357882260f},
  { 171.555852744396f,    -39.7469791157059f},
  { 166.862978327309f,    -44.4405753118447f},
  { -173.359963284551f,    -5.34433246947857f},
  { -176.544786237622f,    -11.0359947952031f},
  { 180.0f,    -16.9324379151258f},
  { 176.266880402625f,    -22.8503240035940f},
  { 172.256497065483f,    -28.5916635489123f},
  { 167.980385383869f,    -33.9760281659424f},
  { 163.463365690262f,    -38.8668029188042f},
  { -180.0f,    -5.38021214855673f},
  { 176.544786237622f,    -11.0359947952031f},
  { 172.850337713184f,    -16.8082297996342f},
  { 168.924799061709f,    -22.5104100950456f},
  { 164.785963746203f,    -27.9578822105749f},
  { 160.462397817826f,    -32.9970359799122f},
  { 173.359963284551f,    -5.34433246947857f},
  { 169.733093405224f,    -10.8828372167504f},
  { 165.916646457725f,    -16.4513996729311f},
  { 161.931710111024f,    -21.8738327838630f},
  { 157.807703806173f,    -26.9896607617352f},
  { 166.893642283692f,    -5.24085275917997f},
  { 163.201442440340f,    -10.5947297486039f},
  { 159.378144584033f,    -15.9040925262813f},
  { 155.452251322927f,    -21.0129107120547f},
  { 160.748939657812f,    -5.08098739442645f},
  { 157.088821957420f,    -10.2024919920318f},
  { 153.354901220925f,    -15.2222212843682f},
  { 155.031028019138f,    -4.87991118301845f},
  { 151.480228308482f,    -9.74144555329515f},
  { 149.797998652519f,    -4.65334435269144f},
  { 90.0f,    26.8240890621858f},
  { 90.0f,    21.4694342999126f},
  { 90.0f,    15.6913558169502f},
  { 90.0f,    9.56754387983022f},
  { 90.0f,    3.21577715344426f},
  { 90.0f,    -3.21577715344426f},
  { 90.0f,    -9.56754387983022f},
  { 90.0f,    -15.6913558169503f},
  { 90.0f,    -21.4694342999126f},
  { 90.0f,    -26.8240890621858f},
  { 84.6197878514433f,    30.0921348592392f},
  { 84.6197878514433f,    24.8723009094510f},
  { 84.6197878514433f,    19.1724509248615f},
  { 84.6197878514433f,    13.0505970990951f},
  { 84.6197878514433f,    6.61104293301485f},
  { 84.6197878514433f,    0.0f},
  { 84.6197878514433f,    -6.61104293301485f},
  { 84.6197878514433f,    -13.0505970990951f},
  { 84.6197878514433f,    -19.1724509248615f},
  { 84.6197878514433f,    -24.8723009094510f},
  { 84.6197878514433f,    -30.0921348592392f},
  { 78.9444023041744f,    28.0717982346117f},
  { 78.9444023041744f,    22.5288109912611f},
  { 78.9444023041744f,    16.5039096025293f},
  { 78.9444023041744f,    10.0803121566769f},
  { 78.9444023041744f,    3.39124147564268f},
  { 78.9444023041744f,    -3.39124147564268f},
  { 78.9444023041744f,    -10.0803121566769f},
  { 78.9444023041744f,    -16.5039096025293f},
  { 78.9444023041744f,    -22.5288109912611f},
  { 78.9444023041744f,    -28.0717982346117f},
  { 73.0675620848742f,    25.6408314185646f},
  { 73.0675620848742f,    19.7987338779611f},
  { 73.0675620848742f,    13.4956318300578f},
  { 73.0675620848742f,    6.84272052704990f},
  { 73.0675620848742f,    0.0f},
  { 73.0675620848742f,    -6.84272052704990f},
  { 73.0675620848742f,    -13.4956318300578f},
  { 73.0675620848742f,    -19.7987338779611f},
  { 73.0675620848742f,    -25.6408314185646f},
  { 67.1060924957396f,    22.8191555387934f},
  { 67.1060924957396f,    16.7275782516185f},
  { 67.1060924957396f,    10.2219242765593f},
  { 67.1060924957396f,    3.43978589553867f},
  { 67.1060924957396f,    -3.43978589553867f},
  { 67.1060924957396f,    -10.2219242765593f},
  { 67.1060924957396f,    -16.7275782516185f},
  { 67.1060924957396f,    -22.8191555387934f},
  { 61.1872390127156f,    19.6685616799211f},
  { 61.1872390127156f,    13.4029822122779f},
  { 61.1872390127156f,    6.79443674899073f},
  { 61.1872390127156f,    0.0f},
  { 61.1872390127156f,    -6.79443674899073f},
  { 61.1872390127156f,    -13.4029822122779f},
  { 61.1872390127156f,    -19.6685616799211f},
  { 55.4329463642532f,    16.2889850528425f},
  { 55.4329463642532f,    9.94442758961885f},
  { 55.4329463642532f,    3.34469597060064f},
  { 55.4329463642532f,    -3.34469597060064f},
  { 55.4329463642532f,    -9.94442758961885f},
  { 55.4329463642532f,    -16.2889850528425f},
  { 49.9457026117864f,    12.8038241283666f},
  { 49.9457026117864f,    6.48284427362099f},
  { 49.9457026117864f,    0.0f},
  { 49.9457026117864f,    -6.48284427362099f},
  { 49.9457026117864f,    -12.8038241283666f},
  { 44.7997417898187f,    9.33886100758483f},
  { 44.7997417898187f,    3.13767672900606f},
  { 44.7997417898187f,    -3.13767672900606f},
  { 44.7997417898187f,    -9.33886100758483f},
  { 40.0388650148264f,    6.00255468128483f},
  { 40.0388650148264f,    0.0f},
  { 40.0388650148264f,    -6.00255468128483f},
  { 35.6796319893852f,    2.87396940394908f},
  { 35.6796319893852f,    -2.87396940394908f},
  { 95.3802121485567f,    30.0921348592392f},
  { 101.055597695826f,    28.0717982346117f},
  { 106.932437915126f,    25.6408314185646f},
  { 112.893907504260f,    22.8191555387934f},
  { 118.812760987284f,    19.6685616799211f},
  { 124.567053635747f,    16.2889850528425f},
  { 130.054297388214f,    12.8038241283666f},
  { 135.200258210181f,    9.33886100758483f},
  { 139.961134985174f,    6.00255468128483f},
  { 144.320368010615f,    2.87396940394908f},
  { 95.3802121485567f,    24.8723009094510f},
  { 101.055597695826f,    22.5288109912611f},
  { 106.932437915126f,    19.7987338779611f},
  { 112.893907504260f,    16.7275782516185f},
  { 118.812760987284f,    13.4029822122779f},
  { 124.567053635747f,    9.94442758961885f},
  { 130.054297388214f,    6.48284427362099f},
  { 135.200258210181f,    3.13767672900606f},
  { 139.961134985174f,    -3.49871307225888e-15f},
  { 144.320368010615f,    -2.87396940394908f},
  { 95.3802121485567f,    19.1724509248615f},
  { 101.055597695826f,    16.5039096025293f},
  { 106.932437915126f,    13.4956318300578f},
  { 112.893907504260f,    10.2219242765593f},
  { 118.812760987284f,    6.79443674899073f},
  { 124.567053635747f,    3.34469597060064f},
  { 130.054297388214f,    3.78097411646819e-15f},
  { 135.200258210181f,    -3.13767672900606f},
  { 139.961134985174f,    -6.00255468128483f},
  { 95.3802121485567f,    13.0505970990951f},
  { 101.055597695826f,    10.0803121566769f},
  { 106.932437915126f,    6.84272052704990f},
  { 112.893907504260f,    3.43978589553867f},
  { 118.812760987284f,    -3.96437895925777e-15f},
  { 124.567053635747f,    -3.34469597060064f},
  { 130.054297388214f,    -6.48284427362099f},
  { 135.200258210181f,    -9.33886100758483f},
  { 95.3802121485567f,    6.61104293301485f},
  { 101.055597695826f,    3.39124147564267f},
  { 106.932437915126f,    0.0f},
  { 112.893907504260f,    -3.43978589553867f},
  { 118.812760987284f,    -6.79443674899073f},
  { 124.567053635747f,    -9.94442758961885f},
  { 130.054297388214f,    -12.8038241283666f},
  { 95.3802121485567f,    1.44615164951904e-15f},
  { 101.055597695826f,    -3.39124147564267f},
  { 106.932437915126f,    -6.84272052704990f},
  { 112.893907504260f,    -10.2219242765593f},
  { 118.812760987284f,    -13.4029822122779f},
  { 124.567053635747f,    -16.2889850528425f},
  { 95.3802121485567f,    -6.61104293301485f},
  { 101.055597695826f,    -10.0803121566769f},
  { 106.932437915126f,    -13.4956318300578f},
  { 112.893907504260f,    -16.7275782516185f},
  { 118.812760987284f,    -19.6685616799211f},
  { 95.3802121485567f,    -13.0505970990951f},
  { 101.055597695826f,    -16.5039096025293f},
  { 106.932437915126f,    -19.7987338779611f},
  { 112.893907504260f,    -22.8191555387935f},
  { 95.3802121485567f,    -19.1724509248615f},
  { 101.055597695826f,    -22.5288109912611f},
  { 106.932437915126f,    -25.6408314185646f},
  { 95.3802121485567f,    -24.8723009094510f},
  { 101.055597695826f,    -28.0717982346117f},
  { 95.3802121485567f,    -30.0921348592392f},
  { -84.6197878514433f,    30.0921348592392f},
  { -78.9444023041744f,    28.0717982346117f},
  { -73.0675620848742f,    25.6408314185646f},
  { -67.1060924957396f,    22.8191555387934f},
  { -61.1872390127156f,    19.6685616799211f},
  { -55.4329463642532f,    16.2889850528425f},
  { -49.9457026117864f,    12.8038241283666f},
  { -44.7997417898187f,    9.33886100758483f},
  { -40.0388650148264f,    6.00255468128483f},
  { -35.6796319893852f,    2.87396940394908f},
  { -90.0f,    26.8240890621858f},
  { -84.6197878514433f,    24.8723009094510f},
  { -78.9444023041744f,    22.5288109912611f},
  { -73.0675620848742f,    19.7987338779611f},
  { -67.1060924957396f,    16.7275782516185f},
  { -61.1872390127156f,    13.4029822122779f},
  { -55.4329463642532f,    9.94442758961885f},
  { -49.9457026117864f,    6.48284427362099f},
  { -44.7997417898187f,    3.13767672900606f},
  { -40.0388650148264f,    -3.49871307225888e-15f},
  { -35.6796319893852f,    -2.87396940394908f},
  { -90.0f,    21.4694342999126f},
  { -84.6197878514433f,    19.1724509248615f},
  { -78.9444023041744f,    16.5039096025293f},
  { -73.0675620848742f,    13.4956318300578f},
  { -67.1060924957396f,    10.2219242765593f},
  { -61.1872390127156f,    6.79443674899073f},
  { -55.4329463642532f,    3.34469597060064f},
  { -49.9457026117864f,    3.78097411646819e-15f},
  { -44.7997417898187f,    -3.13767672900606f},
  { -40.0388650148264f,    -6.00255468128483f},
  { -90.0f,    15.6913558169502f},
  { -84.6197878514433f,    13.0505970990951f},
  { -78.9444023041744f,    10.0803121566769f},
  { -73.0675620848742f,    6.84272052704990f},
  { -67.1060924957396f,    3.43978589553867f},
  { -61.1872390127156f,    -3.96437895925777e-15f},
  { -55.4329463642532f,    -3.34469597060064f},
  { -49.9457026117864f,    -6.48284427362099f},
  { -44.7997417898187f,    -9.33886100758483f},
  { -90.0f,    9.56754387983022f},
  { -84.6197878514433f,    6.61104293301485f},
  { -78.9444023041744f,    3.39124147564267f},
  { -73.0675620848742f,    0.0f},
  { -67.1060924957396f,    -3.43978589553867f},
  { -61.1872390127156f,    -6.79443674899073f},
  { -55.4329463642532f,    -9.94442758961885f},
  { -49.9457026117864f,    -12.8038241283666f},
  { -90.0f,    3.21577715344426f},
  { -84.6197878514433f,    1.44615164951904e-15f},
  { -78.9444023041744f,    -3.39124147564267f},
  { -73.0675620848742f,    -6.84272052704990f},
  { -67.1060924957396f,    -10.2219242765593f},
  { -61.1872390127156f,    -13.4029822122779f},
  { -55.4329463642532f,    -16.2889850528425f},
  { -90.0f,    -3.21577715344426f},
  { -84.6197878514433f,    -6.61104293301485f},
  { -78.9444023041744f,    -10.0803121566769f},
  { -73.0675620848742f,    -13.4956318300578f},
  { -67.1060924957396f,    -16.7275782516185f},
  { -61.1872390127156f,    -19.6685616799211f},
  { -90.0f,    -9.56754387983022f},
  { -84.6197878514433f,    -13.0505970990951f},
  { -78.9444023041744f,    -16.5039096025293f},
  { -73.0675620848742f,    -19.7987338779611f},
  { -67.1060924957396f,    -22.8191555387935f},
  { -90.0f,    -15.6913558169503f},
  { -84.6197878514433f,    -19.1724509248615f},
  { -78.9444023041744f,    -22.5288109912611f},
  { -73.0675620848742f,    -25.6408314185646f},
  { -90.0f,    -21.4694342999126f},
  { -84.6197878514433f,    -24.8723009094510f},
  { -78.9444023041744f,    -28.0717982346117f},
  { -90.0f,    -26.8240890621858f},
  { -84.6197878514433f,    -30.0921348592392f},
  { -95.3802121485567f,    30.0921348592392f},
  { -95.3802121485567f,    24.8723009094510f},
  { -95.3802121485567f,    19.1724509248615f},
  { -95.3802121485567f,    13.0505970990951f},
  { -95.3802121485567f,    6.61104293301485f},
  { -95.3802121485567f,    0.0f},
  { -95.3802121485567f,    -6.61104293301485f},
  { -95.3802121485567f,    -13.0505970990951f},
  { -95.3802121485567f,    -19.1724509248615f},
  { -95.3802121485567f,    -24.8723009094510f},
  { -95.3802121485567f,    -30.0921348592392f},
  { -101.055597695826f,    28.0717982346117f},
  { -101.055597695826f,    22.5288109912611f},
  { -101.055597695826f,    16.5039096025293f},
  { -101.055597695826f,    10.0803121566769f},
  { -101.055597695826f,    3.39124147564268f},
  { -101.055597695826f,    -3.39124147564268f},
  { -101.055597695826f,    -10.0803121566769f},
  { -101.055597695826f,    -16.5039096025293f},
  { -101.055597695826f,    -22.5288109912611f},
  { -101.055597695826f,    -28.0717982346117f},
  { -106.932437915126f,    25.6408314185646f},
  { -106.932437915126f,    19.7987338779611f},
  { -106.932437915126f,    13.4956318300578f},
  { -106.932437915126f,    6.84272052704990f},
  { -106.932437915126f,    0.0f},
  { -106.932437915126f,    -6.84272052704990f},
  { -106.932437915126f,    -13.4956318300578f},
  { -106.932437915126f,    -19.7987338779611f},
  { -106.932437915126f,    -25.6408314185646f},
  { -112.893907504260f,    22.8191555387934f},
  { -112.893907504260f,    16.7275782516185f},
  { -112.893907504260f,    10.2219242765593f},
  { -112.893907504260f,    3.43978589553867f},
  { -112.893907504260f,    -3.43978589553867f},
  { -112.893907504260f,    -10.2219242765593f},
  { -112.893907504260f,    -16.7275782516185f},
  { -112.893907504260f,    -22.8191555387934f},
  { -118.812760987284f,    19.6685616799211f},
  { -118.812760987284f,    13.4029822122779f},
  { -118.812760987284f,    6.79443674899073f},
  { -118.812760987284f,    0.0f},
  { -118.812760987284f,    -6.79443674899073f},
  { -118.812760987284f,    -13.4029822122779f},
  { -118.812760987284f,    -19.6685616799211f},
  { -124.567053635747f,    16.2889850528425f},
  { -124.567053635747f,    9.94442758961885f},
  { -124.567053635747f,    3.34469597060064f},
  { -124.567053635747f,    -3.34469597060064f},
  { -124.567053635747f,    -9.94442758961885f},
  { -124.567053635747f,    -16.2889850528425f},
  { -130.054297388214f,    12.8038241283666f},
  { -130.054297388214f,    6.48284427362099f},
  { -130.054297388214f,    0.0f},
  { -130.054297388214f,    -6.48284427362099f},
  { -130.054297388214f,    -12.8038241283666f},
  { -135.200258210181f,    9.33886100758483f},
  { -135.200258210181f,    3.13767672900606f},
  { -135.200258210181f,    -3.13767672900606f},
  { -135.200258210181f,    -9.33886100758483f},
  { -139.961134985174f,    6.00255468128483f},
  { -139.961134985174f,    0.0f},
  { -139.961134985174f,    -6.00255468128483f},
  { -144.320368010615f,    2.87396940394908f},
  { -144.320368010615f,    -2.87396940394908f},
  { 80.4505940088365f,    33.9341349165501f},
  { 74.1742135368039f,    31.7657157810479f},
  { 67.7737003323099f,    29.1116228424807f},
  { 61.4058624603093f,    26.0099241206311f},
  { 55.2228355626197f,    22.5514345501505f},
  { 49.3515952806473f,    18.8685593858082f},
  { 43.8814069317431f,    15.1116730522860f},
  { 38.8613032994452f,    11.4220522643814f},
  { 34.3053982645257f,    7.91143578186176f},
  { 75.4545634390528f,    37.9527138972067f},
  { 68.5494321833145f,    35.5613266886593f},
  { 61.6600326292358f,    32.6059411168087f},
  { 54.9793107331665f,    29.1563521920881f},
  { 48.6671712321805f,    25.3432853634985f},
  { 42.8320885022132f,    21.3355790823425f},
  { 37.5283656040889f,    17.3071035926576f},
  { 32.7650630693237f,    13.4067773207989f},
  { 69.4615014860564f,    42.0051882309740f},
  { 61.9591732060582f,    39.2909610056656f},
  { 54.6937238953148f,    35.9465343507995f},
  { 47.8703899738212f,    32.0894083943225f},
  { 41.6224929593032f,    27.8971437305300f},
  { 36.0089029246800f,    23.5711490518741f},
  { 31.0292635006555f,    19.2979587453841f},
  { 62.3163557546651f,    45.8854719044003f},
  { 54.3541661665975f,    42.7359121642884f},
  { 46.9315106652582f,    38.9265233622418f},
  { 40.2142356995627f,    34.6334163777661f},
  { 34.2634815389756f,    30.0766978885962f},
  { 29.0621575565141f,    25.4757026816237f},
  { 53.9437740791511f,    49.3383428000302f},
  { 45.8094705409058f,    45.6578884146671f},
  { 38.5562061745794f,    41.3506313820636f},
  { 32.2417447148225f,    36.6461739187852f},
  { 26.8199201064981f,    31.7876115709362f},
  { 44.4460578544387f,    52.1017837736988f},
  { 36.5791736921472f,    47.8535687236878f},
  { 29.8789010549864f,    43.0855410044008f},
  { 24.2490315228135f,    38.0544667408175f},
  { 34.1876830364283f,    53.9746669264628f},
  { 27.0911602230964f,    49.2132010392859f},
  { 21.2845280472078f,    44.0950952193463f},
  { 23.7701186624180f,    54.8831947134468f},
  { 17.8486514163197f,    49.7511649608675f},
  { 13.8508524523141f,    54.9033073202126f},
  { 80.4505940088365f,    -33.9341349165501f},
  { 75.4545634390528f,    -37.9527138972067f},
  { 69.4615014860564f,    -42.0051882309740f},
  { 62.3163557546651f,    -45.8854719044003f},
  { 53.9437740791511f,    -49.3383428000302f},
  { 44.4460578544387f,    -52.1017837736988f},
  { 34.1876830364283f,    -53.9746669264628f},
  { 23.7701186624180f,    -54.8831947134468f},
  { 13.8508524523141f,    -54.9033073202126f},
  { 74.1742135368039f,    -31.7657157810479f},
  { 68.5494321833145f,    -35.5613266886593f},
  { 61.9591732060582f,    -39.2909610056656f},
  { 54.3541661665975f,    -42.7359121642884f},
  { 45.8094705409058f,    -45.6578884146671f},
  { 36.5791736921472f,    -47.8535687236878f},
  { 27.0911602230964f,    -49.2132010392859f},
  { 17.8486514163197f,    -49.7511649608675f},
  { 67.7737003323099f,    -29.1116228424807f},
  { 61.6600326292358f,    -32.6059411168087f},
  { 54.6937238953148f,    -35.9465343507995f},
  { 46.9315106652582f,    -38.9265233622418f},
  { 38.5562061745794f,    -41.3506313820636f},
  { 29.8789010549864f,    -43.0855410044008f},
  { 21.2845280472078f,    -44.0950952193463f},
  { 61.4058624603093f,    -26.0099241206311f},
  { 54.9793107331665f,    -29.1563521920881f},
  { 47.8703899738212f,    -32.0894083943225f},
  { 40.2142356995626f,    -34.6334163777661f},
  { 32.2417447148225f,    -36.6461739187852f},
  { 24.2490315228135f,    -38.0544667408175f},
  { 55.2228355626197f,    -22.5514345501505f},
  { 48.6671712321805f,    -25.3432853634985f},
  { 41.6224929593032f,    -27.8971437305300f},
  { 34.2634815389756f,    -30.0766978885962f},
  { 26.8199201064981f,    -31.7876115709362f},
  { 49.3515952806473f,    -18.8685593858082f},
  { 42.8320885022132f,    -21.3355790823425f},
  { 36.0089029246800f,    -23.5711490518741f},
  { 29.0621575565141f,    -25.4757026816236f},
  { 43.8814069317431f,    -15.1116730522860f},
  { 37.5283656040889f,    -17.3071035926577f},
  { 31.0292635006555f,    -19.2979587453841f},
  { 38.8613032994452f,    -11.4220522643814f},
  { 32.7650630693237f,    -13.4067773207989f},
  { 34.3053982645257f,    -7.91143578186175f},
  { -13.8508524523141f,    54.9033073202126f},
  { -17.8486514163197f,    49.7511649608675f},
  { -21.2845280472078f,    44.0950952193463f},
  { -24.2490315228135f,    38.0544667408175f},
  { -26.8199201064981f,    31.7876115709362f},
  { -29.0621575565141f,    25.4757026816236f},
  { -31.0292635006555f,    19.2979587453841f},
  { -32.7650630693237f,    13.4067773207989f},
  { -34.3053982645256f,    7.91143578186176f},
  { -23.7701186624180f,    54.8831947134468f},
  { -27.0911602230964f,    49.2132010392859f},
  { -29.8789010549864f,    43.0855410044008f},
  { -32.2417447148225f,    36.6461739187852f},
  { -34.2634815389756f,    30.0766978885962f},
  { -36.0089029246800f,    23.5711490518741f},
  { -37.5283656040889f,    17.3071035926576f},
  { -38.8613032994452f,    11.4220522643814f},
  { -34.1876830364283f,    53.9746669264628f},
  { -36.5791736921472f,    47.8535687236878f},
  { -38.5562061745794f,    41.3506313820636f},
  { -40.2142356995627f,    34.6334163777661f},
  { -41.6224929593032f,    27.8971437305300f},
  { -42.8320885022132f,    21.3355790823425f},
  { -43.8814069317431f,    15.1116730522860f},
  { -44.4460578544387f,    52.1017837736988f},
  { -45.8094705409058f,    45.6578884146671f},
  { -46.9315106652582f,    38.9265233622418f},
  { -47.8703899738212f,    32.0894083943225f},
  { -48.6671712321805f,    25.3432853634985f},
  { -49.3515952806473f,    18.8685593858082f},
  { -53.9437740791511f,    49.3383428000302f},
  { -54.3541661665975f,    42.7359121642884f},
  { -54.6937238953148f,    35.9465343507995f},
  { -54.9793107331665f,    29.1563521920881f},
  { -55.2228355626197f,    22.5514345501505f},
  { -62.3163557546651f,    45.8854719044003f},
  { -61.9591732060582f,    39.2909610056656f},
  { -61.6600326292358f,    32.6059411168087f},
  { -61.4058624603093f,    26.0099241206311f},
  { -69.4615014860564f,    42.0051882309740f},
  { -68.5494321833145f,    35.5613266886593f},
  { -67.7737003323099f,    29.1116228424807f},
  { -75.4545634390528f,    37.9527138972067f},
  { -74.1742135368040f,    31.7657157810479f},
  { -80.4505940088365f,    33.9341349165501f},
  { -13.8508524523141f,    -54.9033073202126f},
  { -23.7701186624180f,    -54.8831947134468f},
  { -34.1876830364283f,    -53.9746669264628f},
  { -44.4460578544387f,    -52.1017837736988f},
  { -53.9437740791511f,    -49.3383428000302f},
  { -62.3163557546651f,    -45.8854719044003f},
  { -69.4615014860564f,    -42.0051882309740f},
  { -75.4545634390528f,    -37.9527138972067f},
  { -80.4505940088365f,    -33.9341349165501f},
  { -17.8486514163197f,    -49.7511649608675f},
  { -27.0911602230964f,    -49.2132010392859f},
  { -36.5791736921472f,    -47.8535687236878f},
  { -45.8094705409058f,    -45.6578884146671f},
  { -54.3541661665975f,    -42.7359121642884f},
  { -61.9591732060582f,    -39.2909610056656f},
  { -68.5494321833145f,    -35.5613266886593f},
  { -74.1742135368039f,    -31.7657157810479f},
  { -21.2845280472078f,    -44.0950952193463f},
  { -29.8789010549864f,    -43.0855410044008f},
  { -38.5562061745794f,    -41.3506313820636f},
  { -46.9315106652582f,    -38.9265233622418f},
  { -54.6937238953148f,    -35.9465343507995f},
  { -61.6600326292358f,    -32.6059411168087f},
  { -67.7737003323099f,    -29.1116228424807f},
  { -24.2490315228135f,    -38.0544667408175f},
  { -32.2417447148225f,    -36.6461739187852f},
  { -40.2142356995627f,    -34.6334163777661f},
  { -47.8703899738212f,    -32.0894083943225f},
  { -54.9793107331665f,    -29.1563521920881f},
  { -61.4058624603093f,    -26.0099241206311f},
  { -26.8199201064981f,    -31.7876115709362f},
  { -34.2634815389756f,    -30.0766978885962f},
  { -41.6224929593032f,    -27.8971437305300f},
  { -48.6671712321805f,    -25.3432853634985f},
  { -55.2228355626197f,    -22.5514345501505f},
  { -29.0621575565141f,    -25.4757026816236f},
  { -36.0089029246800f,    -23.5711490518741f},
  { -42.8320885022132f,    -21.3355790823425f},
  { -49.3515952806473f,    -18.8685593858082f},
  { -31.0292635006555f,    -19.2979587453841f},
  { -37.5283656040889f,    -17.3071035926577f},
  { -43.8814069317431f,    -15.1116730522860f},
  { -32.7650630693237f,    -13.4067773207989f},
  { -38.8613032994452f,    -11.4220522643814f},
  { -34.3053982645257f,    -7.91143578186175f},
  { 166.149147547686f,    54.9033073202126f},
  { 162.151348583680f,    49.7511649608675f},
  { 158.715471952792f,    44.0950952193463f},
  { 155.750968477187f,    38.0544667408175f},
  { 153.180079893502f,    31.7876115709362f},
  { 150.937842443486f,    25.4757026816236f},
  { 148.970736499345f,    19.2979587453841f},
  { 147.234936930676f,    13.4067773207989f},
  { 145.694601735474f,    7.91143578186176f},
  { 156.229881337582f,    54.8831947134468f},
  { 152.908839776904f,    49.2132010392859f},
  { 150.121098945014f,    43.0855410044008f},
  { 147.758255285177f,    36.6461739187852f},
  { 145.736518461024f,    30.0766978885962f},
  { 143.991097075320f,    23.5711490518741f},
  { 142.471634395911f,    17.3071035926576f},
  { 141.138696700555f,    11.4220522643814f},
  { 145.812316963572f,    53.9746669264628f},
  { 143.420826307853f,    47.8535687236878f},
  { 141.443793825421f,    41.3506313820636f},
  { 139.785764300437f,    34.6334163777661f},
  { 138.377507040697f,    27.8971437305300f},
  { 137.167911497787f,    21.3355790823425f},
  { 136.118593068257f,    15.1116730522860f},
  { 135.553942145561f,    52.1017837736988f},
  { 134.190529459094f,    45.6578884146671f},
  { 133.068489334742f,    38.9265233622418f},
  { 132.129610026179f,    32.0894083943225f},
  { 131.332828767820f,    25.3432853634985f},
  { 130.648404719353f,    18.8685593858082f},
  { 126.056225920849f,    49.3383428000302f},
  { 125.645833833403f,    42.7359121642884f},
  { 125.306276104685f,    35.9465343507995f},
  { 125.020689266834f,    29.1563521920881f},
  { 124.777164437380f,    22.5514345501505f},
  { 117.683644245335f,    45.8854719044003f},
  { 118.040826793942f,    39.2909610056656f},
  { 118.339967370764f,    32.6059411168087f},
  { 118.594137539691f,    26.0099241206311f},
  { 110.538498513944f,    42.0051882309740f},
  { 111.450567816686f,    35.5613266886593f},
  { 112.226299667690f,    29.1116228424807f},
  { 104.545436560947f,    37.9527138972067f},
  { 105.825786463196f,    31.7657157810479f},
  { 99.5494059911635f,    33.9341349165501f},
  { 166.149147547686f,    -54.9033073202126f},
  { 156.229881337582f,    -54.8831947134468f},
  { 145.812316963572f,    -53.9746669264628f},
  { 135.553942145561f,    -52.1017837736988f},
  { 126.056225920849f,    -49.3383428000302f},
  { 117.683644245335f,    -45.8854719044003f},
  { 110.538498513944f,    -42.0051882309740f},
  { 104.545436560947f,    -37.9527138972067f},
  { 99.5494059911635f,    -33.9341349165501f},
  { 162.151348583680f,    -49.7511649608675f},
  { 152.908839776904f,    -49.2132010392859f},
  { 143.420826307853f,    -47.8535687236878f},
  { 134.190529459094f,    -45.6578884146671f},
  { 125.645833833403f,    -42.7359121642884f},
  { 118.040826793942f,    -39.2909610056656f},
  { 111.450567816686f,    -35.5613266886593f},
  { 105.825786463196f,    -31.7657157810479f},
  { 158.715471952792f,    -44.0950952193463f},
  { 150.121098945014f,    -43.0855410044008f},
  { 141.443793825421f,    -41.3506313820636f},
  { 133.068489334742f,    -38.9265233622418f},
  { 125.306276104685f,    -35.9465343507995f},
  { 118.339967370764f,    -32.6059411168087f},
  { 112.226299667690f,    -29.1116228424807f},
  { 155.750968477187f,    -38.0544667408175f},
  { 147.758255285177f,    -36.6461739187852f},
  { 139.785764300437f,    -34.6334163777661f},
  { 132.129610026179f,    -32.0894083943225f},
  { 125.020689266833f,    -29.1563521920881f},
  { 118.594137539691f,    -26.0099241206311f},
  { 153.180079893502f,    -31.7876115709362f},
  { 145.736518461024f,    -30.0766978885962f},
  { 138.377507040697f,    -27.8971437305300f},
  { 131.332828767820f,    -25.3432853634985f},
  { 124.777164437380f,    -22.5514345501505f},
  { 150.937842443486f,    -25.4757026816236f},
  { 143.991097075320f,    -23.5711490518741f},
  { 137.167911497787f,    -21.3355790823425f},
  { 130.648404719353f,    -18.8685593858082f},
  { 148.970736499345f,    -19.2979587453841f},
  { 142.471634395911f,    -17.3071035926577f},
  { 136.118593068257f,    -15.1116730522860f},
  { 147.234936930676f,    -13.4067773207989f},
  { 141.138696700555f,    -11.4220522643814f},
  { 145.694601735474f,    -7.91143578186175f},
  { -166.149147547686f,    54.9033073202126f},
  { -156.229881337582f,    54.8831947134468f},
  { -145.812316963572f,    53.9746669264628f},
  { -135.553942145561f,    52.1017837736988f},
  { -126.056225920849f,    49.3383428000302f},
  { -117.683644245335f,    45.8854719044003f},
  { -110.538498513944f,    42.0051882309740f},
  { -104.545436560947f,    37.9527138972067f},
  { -99.5494059911635f,    33.9341349165501f},
  { -162.151348583680f,    49.7511649608675f},
  { -152.908839776904f,    49.2132010392859f},
  { -143.420826307853f,    47.8535687236878f},
  { -134.190529459094f,    45.6578884146671f},
  { -125.645833833403f,    42.7359121642884f},
  { -118.040826793942f,    39.2909610056656f},
  { -111.450567816686f,    35.5613266886593f},
  { -105.825786463196f,    31.7657157810479f},
  { -158.715471952792f,    44.0950952193463f},
  { -150.121098945014f,    43.0855410044008f},
  { -141.443793825421f,    41.3506313820636f},
  { -133.068489334742f,    38.9265233622418f},
  { -125.306276104685f,    35.9465343507995f},
  { -118.339967370764f,    32.6059411168087f},
  { -112.226299667690f,    29.1116228424807f},
  { -155.750968477187f,    38.0544667408175f},
  { -147.758255285177f,    36.6461739187852f},
  { -139.785764300437f,    34.6334163777661f},
  { -132.129610026179f,    32.0894083943225f},
  { -125.020689266833f,    29.1563521920881f},
  { -118.594137539691f,    26.0099241206311f},
  { -153.180079893502f,    31.7876115709362f},
  { -145.736518461024f,    30.0766978885962f},
  { -138.377507040697f,    27.8971437305300f},
  { -131.332828767820f,    25.3432853634985f},
  { -124.777164437380f,    22.5514345501505f},
  { -150.937842443486f,    25.4757026816236f},
  { -143.991097075320f,    23.5711490518741f},
  { -137.167911497787f,    21.3355790823425f},
  { -130.648404719353f,    18.8685593858082f},
  { -148.970736499345f,    19.2979587453841f},
  { -142.471634395911f,    17.3071035926577f},
  { -136.118593068257f,    15.1116730522860f},
  { -147.234936930676f,    13.4067773207989f},
  { -141.138696700555f,    11.4220522643814f},
  { -145.694601735474f,    7.91143578186175f},
  { -166.149147547686f,    -54.9033073202126f},
  { -162.151348583680f,    -49.7511649608675f},
  { -158.715471952792f,    -44.0950952193463f},
  { -155.750968477187f,    -38.0544667408175f},
  { -153.180079893502f,    -31.7876115709362f},
  { -150.937842443486f,    -25.4757026816236f},
  { -148.970736499345f,    -19.2979587453841f},
  { -147.234936930676f,    -13.4067773207989f},
  { -145.694601735474f,    -7.91143578186176f},
  { -156.229881337582f,    -54.8831947134468f},
  { -152.908839776904f,    -49.2132010392859f},
  { -150.121098945014f,    -43.0855410044008f},
  { -147.758255285177f,    -36.6461739187852f},
  { -145.736518461024f,    -30.0766978885962f},
  { -143.991097075320f,    -23.5711490518741f},
  { -142.471634395911f,    -17.3071035926576f},
  { -141.138696700555f,    -11.4220522643814f},
  { -145.812316963572f,    -53.9746669264628f},
  { -143.420826307853f,    -47.8535687236878f},
  { -141.443793825421f,    -41.3506313820636f},
  { -139.785764300437f,    -34.6334163777661f},
  { -138.377507040697f,    -27.8971437305300f},
  { -137.167911497787f,    -21.3355790823425f},
  { -136.118593068257f,    -15.1116730522860f},
  { -135.553942145561f,    -52.1017837736988f},
  { -134.190529459094f,    -45.6578884146671f},
  { -133.068489334742f,    -38.9265233622418f},
  { -132.129610026179f,    -32.0894083943225f},
  { -131.332828767820f,    -25.3432853634985f},
  { -130.648404719353f,    -18.8685593858082f},
  { -126.056225920849f,    -49.3383428000302f},
  { -125.645833833403f,    -42.7359121642884f},
  { -125.306276104685f,    -35.9465343507995f},
  { -125.020689266834f,    -29.1563521920881f},
  { -124.777164437380f,    -22.5514345501505f},
  { -117.683644245335f,    -45.8854719044003f},
  { -118.040826793942f,    -39.2909610056656f},
  { -118.339967370764f,    -32.6059411168087f},
  { -118.594137539691f,    -26.0099241206311f},
  { -110.538498513944f,    -42.0051882309740f},
  { -111.450567816686f,    -35.5613266886593f},
  { -112.226299667690f,    -29.1116228424807f},
  { -104.545436560947f,    -37.9527138972067f},
  { -105.825786463196f,    -31.7657157810479f},
  { -99.5494059911635f,    -33.9341349165501f}};

/* 3LD geosphere 'ico' [12 0] (1442 points) */
const float __geosphere_ico_12_0_dirs_deg[1442][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    62.7502499362301f},
  { 0.0f,    67.6072290710451f},
  { 0.0f,    72.8279621494862f},
  { 0.0f,    78.3592768632294f},
  { 0.0f,    84.1189512527134f},
  { 0.0f,    90.0f},
  { 180.0f,    84.1189512527134f},
  { 180.0f,    78.3592768632294f},
  { 180.0f,    72.8279621494862f},
  { 180.0f,    67.6072290710451f},
  { 180.0f,    62.7502499362301f},
  { 8.36785885565049f,    59.3976574802411f},
  { 10.1918500362761f,    64.0598368480186f},
  { 13.0152040462228f,    69.0835941901292f},
  { 17.9319334355159f,    74.3810971336279f},
  { 28.3399673707642f,    79.7227786716467f},
  { 58.2825255885390f,    84.2221928049541f},
  { 121.717474411461f,    84.2221928049541f},
  { 151.660032629236f,    79.7227786716467f},
  { 162.068066564484f,    74.3810971336279f},
  { 166.984795953777f,    69.0835941901292f},
  { 169.808149963724f,    64.0598368480186f},
  { 171.632141144350f,    59.3976574802411f},
  { 17.9319334355159f,    59.9665693791348f},
  { 22.0237505161462f,    64.6075511946472f},
  { 28.3399673707642f,    69.4455573432933f},
  { 38.9734473368657f,    74.1982775048736f},
  { 58.2825255885390f,    78.1814142633759f},
  { 90.0f,    79.9071595156824f},
  { 121.717474411461f,    78.1814142633759f},
  { 141.026552663134f,    74.1982775048736f},
  { 151.660032629236f,    69.4455573432933f},
  { 157.976249483854f,    64.6075511946472f},
  { 162.068066564484f,    59.9665693791348f},
  { 28.3399673707642f,    59.7900946927798f},
  { 34.7391081893039f,    64.1249435633682f},
  { 44.1517511327961f,    68.3569826266350f},
  { 58.2825255885390f,    72.0f},
  { 78.3592768632294f,    74.2410708523759f},
  { 101.640723136771f,    74.2410708523759f},
  { 121.717474411461f,    72.0f},
  { 135.848248867204f,    68.3569826266350f},
  { 145.260891810696f,    64.1249435633682f},
  { 151.660032629236f,    59.7900946927798f},
  { 38.9734473368657f,    58.7297899739513f},
  { 47.1679114977869f,    62.4871807253393f},
  { 58.2825255885390f,    65.8185857366241f},
  { 72.8279621494862f,    68.2090865913442f},
  { 90.0f,    69.0948425521107f},
  { 107.172037850514f,    68.2090865913442f},
  { 121.717474411461f,    65.8185857366241f},
  { 132.832088502213f,    62.4871807253393f},
  { 141.026552663134f,    58.7297899739513f},
  { 49.1320813134511f,    56.7641893271062f},
  { 58.2825255885390f,    59.7778071950459f},
  { 69.6541923856616f,    62.1433456787522f},
  { 82.9535936191428f,    63.4669072935504f},
  { 97.0464063808572f,    63.4669072935504f},
  { 110.345807614338f,    62.1433456787522f},
  { 121.717474411461f,    59.7778071950459f},
  { 130.867918686549f,    56.7641893271062f},
  { 58.2825255885390f,    54.0f},
  { 67.6072290710451f,    56.2397210741351f},
  { 78.3592768632294f,    57.7475579750497f},
  { 90.0f,    58.2825255885390f},
  { 101.640723136771f,    57.7475579750497f},
  { 112.392770928955f,    56.2397210741351f},
  { 121.717474411461f,    54.0f},
  { 66.1808024884231f,    50.6327247195825f},
  { 75.1646413480039f,    52.1723325493831f},
  { 84.9544063480986f,    53.0023277621793f},
  { 95.0455936519014f,    53.0023277621793f},
  { 104.835358651996f,    52.1723325493831f},
  { 113.819197511577f,    50.6327247195825f},
  { 72.8279621494862f,    46.8885380691238f},
  { 81.2167693990410f,    47.8536292564083f},
  { 90.0f,    48.1896851042214f},
  { 98.7832306009590f,    47.8536292564083f},
  { 107.172037850514f,    46.8885380691238f},
  { 78.3592768632294f,    42.9780676586279f},
  { 86.0716405780585f,    43.5050495262159f},
  { 93.9283594219415f,    43.5050495262159f},
  { 101.640723136771f,    42.9780676586279f},
  { 82.9535936191428f,    39.0716078459179f},
  { 90.0f,    39.2843328795171f},
  { 97.0464063808572f,    39.0716078459179f},
  { 86.7842228465557f,    35.2918809834389f},
  { 93.2157771534443f,    35.2918809834389f},
  { -8.36785885565049f,    59.3976574802411f},
  { -17.9319334355159f,    59.9665693791348f},
  { -28.3399673707642f,    59.7900946927798f},
  { -38.9734473368657f,    58.7297899739513f},
  { -49.1320813134511f,    56.7641893271062f},
  { -58.2825255885390f,    54.0f},
  { -66.1808024884231f,    50.6327247195825f},
  { -72.8279621494862f,    46.8885380691238f},
  { -78.3592768632294f,    42.9780676586279f},
  { -82.9535936191428f,    39.0716078459179f},
  { -86.7842228465557f,    35.2918809834389f},
  { -10.1918500362761f,    64.0598368480186f},
  { -22.0237505161462f,    64.6075511946472f},
  { -34.7391081893039f,    64.1249435633682f},
  { -47.1679114977869f,    62.4871807253393f},
  { -58.2825255885390f,    59.7778071950459f},
  { -67.6072290710451f,    56.2397210741351f},
  { -75.1646413480039f,    52.1723325493831f},
  { -81.2167693990410f,    47.8536292564083f},
  { -86.0716405780585f,    43.5050495262159f},
  { -90.0f,    39.2843328795171f},
  { -93.2157771534443f,    35.2918809834389f},
  { -13.0152040462228f,    69.0835941901292f},
  { -28.3399673707642f,    69.4455573432933f},
  { -44.1517511327961f,    68.3569826266350f},
  { -58.2825255885390f,    65.8185857366241f},
  { -69.6541923856617f,    62.1433456787522f},
  { -78.3592768632294f,    57.7475579750497f},
  { -84.9544063480986f,    53.0023277621793f},
  { -90.0f,    48.1896851042214f},
  { -93.9283594219415f,    43.5050495262159f},
  { -97.0464063808572f,    39.0716078459179f},
  { -17.9319334355159f,    74.3810971336279f},
  { -38.9734473368657f,    74.1982775048736f},
  { -58.2825255885390f,    72.0f},
  { -72.8279621494862f,    68.2090865913442f},
  { -82.9535936191428f,    63.4669072935504f},
  { -90.0f,    58.2825255885390f},
  { -95.0455936519014f,    53.0023277621793f},
  { -98.7832306009590f,    47.8536292564083f},
  { -101.640723136771f,    42.9780676586279f},
  { -28.3399673707642f,    79.7227786716467f},
  { -58.2825255885390f,    78.1814142633759f},
  { -78.3592768632294f,    74.2410708523759f},
  { -90.0f,    69.0948425521107f},
  { -97.0464063808572f,    63.4669072935504f},
  { -101.640723136771f,    57.7475579750497f},
  { -104.835358651996f,    52.1723325493831f},
  { -107.172037850514f,    46.8885380691238f},
  { -58.2825255885390f,    84.2221928049541f},
  { -90.0f,    79.9071595156824f},
  { -101.640723136771f,    74.2410708523759f},
  { -107.172037850514f,    68.2090865913442f},
  { -110.345807614338f,    62.1433456787522f},
  { -112.392770928955f,    56.2397210741351f},
  { -113.819197511577f,    50.6327247195825f},
  { -121.717474411461f,    84.2221928049541f},
  { -121.717474411461f,    78.1814142633759f},
  { -121.717474411461f,    72.0f},
  { -121.717474411461f,    65.8185857366241f},
  { -121.717474411461f,    59.7778071950459f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    79.7227786716467f},
  { -141.026552663134f,    74.1982775048736f},
  { -135.848248867204f,    68.3569826266350f},
  { -132.832088502213f,    62.4871807253393f},
  { -130.867918686549f,    56.7641893271062f},
  { -162.068066564484f,    74.3810971336279f},
  { -151.660032629236f,    69.4455573432933f},
  { -145.260891810696f,    64.1249435633682f},
  { -141.026552663134f,    58.7297899739513f},
  { -166.984795953777f,    69.0835941901292f},
  { -157.976249483854f,    64.6075511946472f},
  { -151.660032629236f,    59.7900946927798f},
  { -169.808149963724f,    64.0598368480186f},
  { -162.068066564484f,    59.9665693791348f},
  { -171.632141144350f,    59.3976574802411f},
  { 8.36785885565049f,    -59.3976574802411f},
  { 17.9319334355159f,    -59.9665693791348f},
  { 28.3399673707642f,    -59.7900946927798f},
  { 38.9734473368657f,    -58.7297899739513f},
  { 49.1320813134511f,    -56.7641893271062f},
  { 58.2825255885390f,    -54.0f},
  { 66.1808024884231f,    -50.6327247195825f},
  { 72.8279621494862f,    -46.8885380691238f},
  { 78.3592768632294f,    -42.9780676586279f},
  { 82.9535936191428f,    -39.0716078459179f},
  { 86.7842228465557f,    -35.2918809834389f},
  { 0.0f,    -62.7502499362301f},
  { 10.1918500362761f,    -64.0598368480186f},
  { 22.0237505161462f,    -64.6075511946472f},
  { 34.7391081893039f,    -64.1249435633682f},
  { 47.1679114977869f,    -62.4871807253393f},
  { 58.2825255885390f,    -59.7778071950459f},
  { 67.6072290710451f,    -56.2397210741351f},
  { 75.1646413480039f,    -52.1723325493831f},
  { 81.2167693990410f,    -47.8536292564083f},
  { 86.0716405780585f,    -43.5050495262159f},
  { 90.0f,    -39.2843328795171f},
  { 93.2157771534443f,    -35.2918809834389f},
  { 0.0f,    -67.6072290710451f},
  { 13.0152040462228f,    -69.0835941901292f},
  { 28.3399673707642f,    -69.4455573432933f},
  { 44.1517511327961f,    -68.3569826266350f},
  { 58.2825255885390f,    -65.8185857366241f},
  { 69.6541923856617f,    -62.1433456787522f},
  { 78.3592768632294f,    -57.7475579750497f},
  { 84.9544063480986f,    -53.0023277621793f},
  { 90.0f,    -48.1896851042214f},
  { 93.9283594219415f,    -43.5050495262159f},
  { 97.0464063808572f,    -39.0716078459179f},
  { 0.0f,    -72.8279621494862f},
  { 17.9319334355159f,    -74.3810971336279f},
  { 38.9734473368657f,    -74.1982775048736f},
  { 58.2825255885390f,    -72.0f},
  { 72.8279621494862f,    -68.2090865913442f},
  { 82.9535936191428f,    -63.4669072935504f},
  { 90.0f,    -58.2825255885390f},
  { 95.0455936519014f,    -53.0023277621793f},
  { 98.7832306009590f,    -47.8536292564083f},
  { 101.640723136771f,    -42.9780676586279f},
  { 0.0f,    -78.3592768632294f},
  { 28.3399673707642f,    -79.7227786716467f},
  { 58.2825255885390f,    -78.1814142633759f},
  { 78.3592768632294f,    -74.2410708523759f},
  { 90.0f,    -69.0948425521107f},
  { 97.0464063808572f,    -63.4669072935504f},
  { 101.640723136771f,    -57.7475579750497f},
  { 104.835358651996f,    -52.1723325493831f},
  { 107.172037850514f,    -46.8885380691238f},
  { 0.0f,    -84.1189512527134f},
  { 58.2825255885390f,    -84.2221928049541f},
  { 90.0f,    -79.9071595156824f},
  { 101.640723136771f,    -74.2410708523759f},
  { 107.172037850514f,    -68.2090865913442f},
  { 110.345807614338f,    -62.1433456787522f},
  { 112.392770928955f,    -56.2397210741351f},
  { 113.819197511577f,    -50.6327247195825f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -84.2221928049541f},
  { 121.717474411461f,    -78.1814142633759f},
  { 121.717474411461f,    -72.0f},
  { 121.717474411461f,    -65.8185857366241f},
  { 121.717474411461f,    -59.7778071950459f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -84.1189512527134f},
  { 151.660032629236f,    -79.7227786716467f},
  { 141.026552663134f,    -74.1982775048736f},
  { 135.848248867204f,    -68.3569826266350f},
  { 132.832088502213f,    -62.4871807253393f},
  { 130.867918686549f,    -56.7641893271062f},
  { 180.0f,    -78.3592768632294f},
  { 162.068066564484f,    -74.3810971336279f},
  { 151.660032629236f,    -69.4455573432933f},
  { 145.260891810696f,    -64.1249435633682f},
  { 141.026552663134f,    -58.7297899739513f},
  { 180.0f,    -72.8279621494862f},
  { 166.984795953777f,    -69.0835941901292f},
  { 157.976249483854f,    -64.6075511946472f},
  { 151.660032629236f,    -59.7900946927798f},
  { 180.0f,    -67.6072290710451f},
  { 169.808149963724f,    -64.0598368480186f},
  { 162.068066564484f,    -59.9665693791348f},
  { 180.0f,    -62.7502499362301f},
  { 171.632141144350f,    -59.3976574802411f},
  { -8.36785885565049f,    -59.3976574802411f},
  { -10.1918500362761f,    -64.0598368480186f},
  { -13.0152040462228f,    -69.0835941901292f},
  { -17.9319334355159f,    -74.3810971336279f},
  { -28.3399673707642f,    -79.7227786716467f},
  { -58.2825255885390f,    -84.2221928049541f},
  { -121.717474411461f,    -84.2221928049541f},
  { -151.660032629236f,    -79.7227786716467f},
  { -162.068066564484f,    -74.3810971336279f},
  { -166.984795953777f,    -69.0835941901292f},
  { -169.808149963724f,    -64.0598368480186f},
  { -171.632141144350f,    -59.3976574802411f},
  { -17.9319334355159f,    -59.9665693791348f},
  { -22.0237505161462f,    -64.6075511946472f},
  { -28.3399673707642f,    -69.4455573432933f},
  { -38.9734473368657f,    -74.1982775048736f},
  { -58.2825255885390f,    -78.1814142633759f},
  { -90.0f,    -79.9071595156824f},
  { -121.717474411461f,    -78.1814142633759f},
  { -141.026552663134f,    -74.1982775048736f},
  { -151.660032629236f,    -69.4455573432933f},
  { -157.976249483854f,    -64.6075511946472f},
  { -162.068066564484f,    -59.9665693791348f},
  { -28.3399673707642f,    -59.7900946927798f},
  { -34.7391081893039f,    -64.1249435633682f},
  { -44.1517511327961f,    -68.3569826266350f},
  { -58.2825255885390f,    -72.0f},
  { -78.3592768632294f,    -74.2410708523759f},
  { -101.640723136771f,    -74.2410708523759f},
  { -121.717474411461f,    -72.0f},
  { -135.848248867204f,    -68.3569826266350f},
  { -145.260891810696f,    -64.1249435633682f},
  { -151.660032629236f,    -59.7900946927798f},
  { -38.9734473368657f,    -58.7297899739513f},
  { -47.1679114977869f,    -62.4871807253393f},
  { -58.2825255885390f,    -65.8185857366241f},
  { -72.8279621494862f,    -68.2090865913442f},
  { -90.0f,    -69.0948425521107f},
  { -107.172037850514f,    -68.2090865913442f},
  { -121.717474411461f,    -65.8185857366241f},
  { -132.832088502213f,    -62.4871807253393f},
  { -141.026552663134f,    -58.7297899739513f},
  { -49.1320813134511f,    -56.7641893271062f},
  { -58.2825255885390f,    -59.7778071950459f},
  { -69.6541923856616f,    -62.1433456787522f},
  { -82.9535936191428f,    -63.4669072935504f},
  { -97.0464063808572f,    -63.4669072935504f},
  { -110.345807614338f,    -62.1433456787522f},
  { -121.717474411461f,    -59.7778071950459f},
  { -130.867918686549f,    -56.7641893271062f},
  { -58.2825255885390f,    -54.0f},
  { -67.6072290710451f,    -56.2397210741351f},
  { -78.3592768632294f,    -57.7475579750497f},
  { -90.0f,    -58.2825255885390f},
  { -101.640723136771f,    -57.7475579750497f},
  { -112.392770928955f,    -56.2397210741351f},
  { -121.717474411461f,    -54.0f},
  { -66.1808024884231f,    -50.6327247195825f},
  { -75.1646413480039f,    -52.1723325493831f},
  { -84.9544063480986f,    -53.0023277621793f},
  { -95.0455936519014f,    -53.0023277621793f},
  { -104.835358651996f,    -52.1723325493831f},
  { -113.819197511577f,    -50.6327247195825f},
  { -72.8279621494862f,    -46.8885380691238f},
  { -81.2167693990410f,    -47.8536292564083f},
  { -90.0f,    -48.1896851042214f},
  { -98.7832306009590f,    -47.8536292564083f},
  { -107.172037850514f,    -46.8885380691238f},
  { -78.3592768632294f,    -42.9780676586279f},
  { -86.0716405780585f,    -43.5050495262159f},
  { -93.9283594219415f,    -43.5050495262159f},
  { -101.640723136771f,    -42.9780676586279f},
  { -82.9535936191428f,    -39.0716078459179f},
  { -90.0f,    -39.2843328795171f},
  { -97.0464063808572f,    -39.0716078459179f},
  { -86.7842228465557f,    -35.2918809834389f},
  { -93.2157771534443f,    -35.2918809834389f},
  { 27.2497500637699f,    0.0f},
  { 22.3927709289549f,    0.0f},
  { 17.1720378505138f,    0.0f},
  { 11.6407231367706f,    0.0f},
  { 5.88104874728658f,    0.0f},
  { 0.0f,    0.0f},
  { -5.88104874728658f,    0.0f},
  { -11.6407231367706f,    0.0f},
  { -17.1720378505138f,    0.0f},
  { -22.3927709289549f,    0.0f},
  { -27.2497500637699f,    0.0f},
  { 30.3343354039248f,    4.24864627780931f},
  { 25.5834595342983f,    4.43921003719892f},
  { 20.4239537377296f,    4.61161924706584f},
  { 14.8947040331857f,    4.75497448828983f},
  { 9.06727399481518f,    4.85832138221458f},
  { 3.04504133231240f,    4.91258754288848f},
  { -3.04504133231240f,    4.91258754288848f},
  { -9.06727399481518f,    4.85832138221458f},
  { -14.8947040331857f,    4.75497448828983f},
  { -20.4239537377296f,    4.61161924706584f},
  { -25.5834595342983f,    4.43921003719892f},
  { -30.3343354039248f,    4.24864627780931f},
  { 28.8127609872844f,    8.86454598437525f},
  { 23.7512118858359f,    9.25345760302041f},
  { 18.2642763775679f,    9.59406822686046f},
  { 12.4083961597877f,    9.86176221445378f},
  { 6.27780483909150f,    10.0335510226076f},
  { 0.0f,    10.0928404843176f},
  { -6.27780483909150f,    10.0335510226076f},
  { -12.4083961597877f,    9.86176221445378f},
  { -18.2642763775679f,    9.59406822686046f},
  { -23.7512118858359f,    9.25345760302041f},
  { -28.8127609872844f,    8.86454598437525f},
  { 27.1333780408531f,    13.8190092060518f},
  { 21.7312825906606f,    14.3996630265555f},
  { 15.8916879482649f,    14.8866391823119f},
  { 9.69372389531482f,    15.2401622826778f},
  { 3.25891024627191f,    15.4266423269812f},
  { -3.25891024627191f,    15.4266423269812f},
  { -9.69372389531481f,    15.2401622826778f},
  { -15.8916879482650f,    14.8866391823119f},
  { -21.7312825906606f,    14.3996630265555f},
  { -27.1333780408531f,    13.8190092060518f},
  { 25.2736805860404f,    19.0551045548580f},
  { 19.4991545452123f,    19.8018634182196f},
  { 13.2825255885390f,    20.3925183693779f},
  { 6.73170262204313f,    20.7733751937705f},
  { 0.0f,    20.9051574478893f},
  { -6.73170262204313f,    20.7733751937705f},
  { -13.2825255885390f,    20.3925183693779f},
  { -19.4991545452123f,    19.8018634182196f},
  { -25.2736805860404f,    19.0551045548580f},
  { 23.2076057298227f,    24.4861741588079f},
  { 17.0274949342831f,    25.3520541808113f},
  { 10.4121356147178f,    25.9833065914758f},
  { 3.50505049071442f,    26.3172227713073f},
  { -3.50505049071442f,    26.3172227713073f},
  { -10.4121356147178f,    25.9833065914758f},
  { -17.0274949342831f,    25.3520541808113f},
  { -23.2076057298227f,    24.4861741588079f},
  { 20.9051574478893f,    30.0000000000000f},
  { 14.2863986758393f,    30.9182723263292f},
  { 7.25597292540465f,    31.5118220765682f},
  { 0.0f,    31.7174744114610f},
  { -7.25597292540465f,    31.5118220765682f},
  { -14.2863986758393f,    30.9182723263292f},
  { -20.9051574478893f,    30.0000000000000f},
  { 18.3321778008559f,    35.4690506268222f},
  { 11.2440778024743f,    36.3591526402786f},
  { 3.79135084476807f,    36.8305660202992f},
  { -3.79135084476807f,    36.8305660202992f},
  { -11.2440778024743f,    36.3591526402786f},
  { -18.3321778008559f,    35.4690506268222f},
  { 15.4504370863258f,    40.7647476003615f},
  { 7.86824453619775f,    41.5411547603032f},
  { 0.0f,    41.8103148957786f},
  { -7.86824453619775f,    41.5411547603032f},
  { -15.4504370863258f,    40.7647476003615f},
  { 12.2183198662487f,    45.7717617455715f},
  { 4.12849455138704f,    46.3533031992989f},
  { -4.12849455138704f,    46.3533031992989f},
  { -12.2183198662487f,    45.7717617455715f},
  { 8.59252164094535f,    50.3983682135674f},
  { 0.0f,    50.7156671204829f},
  { -8.59252164094535f,    50.3983682135674f},
  { 4.53131404587798f,    54.5808573006801f},
  { -4.53131404587798f,    54.5808573006801f},
  { 30.3343354039248f,    -4.24864627780931f},
  { 28.8127609872844f,    -8.86454598437525f},
  { 27.1333780408531f,    -13.8190092060518f},
  { 25.2736805860404f,    -19.0551045548580f},
  { 23.2076057298227f,    -24.4861741588079f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 18.3321778008559f,    -35.4690506268222f},
  { 15.4504370863258f,    -40.7647476003615f},
  { 12.2183198662487f,    -45.7717617455715f},
  { 8.59252164094535f,    -50.3983682135674f},
  { 4.53131404587798f,    -54.5808573006801f},
  { 25.5834595342983f,    -4.43921003719892f},
  { 23.7512118858359f,    -9.25345760302041f},
  { 21.7312825906606f,    -14.3996630265555f},
  { 19.4991545452123f,    -19.8018634182196f},
  { 17.0274949342831f,    -25.3520541808113f},
  { 14.2863986758393f,    -30.9182723263292f},
  { 11.2440778024743f,    -36.3591526402786f},
  { 7.86824453619775f,    -41.5411547603032f},
  { 4.12849455138704f,    -46.3533031992989f},
  { 0.0f,    -50.7156671204829f},
  { -4.53131404587798f,    -54.5808573006801f},
  { 20.4239537377296f,    -4.61161924706584f},
  { 18.2642763775679f,    -9.59406822686046f},
  { 15.8916879482649f,    -14.8866391823119f},
  { 13.2825255885390f,    -20.3925183693779f},
  { 10.4121356147178f,    -25.9833065914758f},
  { 7.25597292540464f,    -31.5118220765682f},
  { 3.79135084476807f,    -36.8305660202992f},
  { 0.0f,    -41.8103148957786f},
  { -4.12849455138704f,    -46.3533031992989f},
  { -8.59252164094535f,    -50.3983682135674f},
  { 14.8947040331857f,    -4.75497448828983f},
  { 12.4083961597877f,    -9.86176221445378f},
  { 9.69372389531481f,    -15.2401622826778f},
  { 6.73170262204313f,    -20.7733751937705f},
  { 3.50505049071442f,    -26.3172227713073f},
  { 0.0f,    -31.7174744114610f},
  { -3.79135084476808f,    -36.8305660202992f},
  { -7.86824453619775f,    -41.5411547603032f},
  { -12.2183198662487f,    -45.7717617455715f},
  { 9.06727399481518f,    -4.85832138221458f},
  { 6.27780483909150f,    -10.0335510226076f},
  { 3.25891024627191f,    -15.4266423269812f},
  { 0.0f,    -20.9051574478893f},
  { -3.50505049071442f,    -26.3172227713073f},
  { -7.25597292540464f,    -31.5118220765682f},
  { -11.2440778024743f,    -36.3591526402786f},
  { -15.4504370863258f,    -40.7647476003615f},
  { 3.04504133231241f,    -4.91258754288848f},
  { 9.98293961704937e-16f,    -10.0928404843176f},
  { -3.25891024627190f,    -15.4266423269812f},
  { -6.73170262204313f,    -20.7733751937705f},
  { -10.4121356147178f,    -25.9833065914758f},
  { -14.2863986758393f,    -30.9182723263292f},
  { -18.3321778008559f,    -35.4690506268222f},
  { -3.04504133231240f,    -4.91258754288848f},
  { -6.27780483909150f,    -10.0335510226076f},
  { -9.69372389531481f,    -15.2401622826778f},
  { -13.2825255885390f,    -20.3925183693779f},
  { -17.0274949342831f,    -25.3520541808113f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -9.06727399481518f,    -4.85832138221458f},
  { -12.4083961597877f,    -9.86176221445378f},
  { -15.8916879482649f,    -14.8866391823119f},
  { -19.4991545452123f,    -19.8018634182196f},
  { -23.2076057298227f,    -24.4861741588079f},
  { -14.8947040331857f,    -4.75497448828983f},
  { -18.2642763775679f,    -9.59406822686046f},
  { -21.7312825906606f,    -14.3996630265555f},
  { -25.2736805860404f,    -19.0551045548580f},
  { -20.4239537377296f,    -4.61161924706584f},
  { -23.7512118858360f,    -9.25345760302040f},
  { -27.1333780408531f,    -13.8190092060518f},
  { -25.5834595342983f,    -4.43921003719892f},
  { -28.8127609872844f,    -8.86454598437524f},
  { -30.3343354039248f,    -4.24864627780932f},
  { -152.750249936230f,    0.0f},
  { -157.607229071045f,    0.0f},
  { -162.827962149486f,    0.0f},
  { -168.359276863229f,    0.0f},
  { -174.118951252713f,    0.0f},
  { 180.0f,    0.0f},
  { 174.118951252713f,    0.0f},
  { 168.359276863229f,    0.0f},
  { 162.827962149486f,    0.0f},
  { 157.607229071045f,    0.0f},
  { 152.750249936230f,    0.0f},
  { -149.665664596075f,    4.24864627780931f},
  { -154.416540465702f,    4.43921003719893f},
  { -159.576046262270f,    4.61161924706584f},
  { -165.105295966814f,    4.75497448828983f},
  { -170.932726005185f,    4.85832138221458f},
  { -176.954958667688f,    4.91258754288848f},
  { 176.954958667688f,    4.91258754288848f},
  { 170.932726005185f,    4.85832138221458f},
  { 165.105295966814f,    4.75497448828983f},
  { 159.576046262270f,    4.61161924706584f},
  { 154.416540465702f,    4.43921003719893f},
  { 149.665664596075f,    4.24864627780931f},
  { -151.187239012716f,    8.86454598437525f},
  { -156.248788114164f,    9.25345760302041f},
  { -161.735723622432f,    9.59406822686046f},
  { -167.591603840212f,    9.86176221445378f},
  { -173.722195160908f,    10.0335510226076f},
  { 180.0f,    10.0928404843176f},
  { 173.722195160908f,    10.0335510226076f},
  { 167.591603840212f,    9.86176221445378f},
  { 161.735723622432f,    9.59406822686046f},
  { 156.248788114164f,    9.25345760302041f},
  { 151.187239012716f,    8.86454598437525f},
  { -152.866621959147f,    13.8190092060518f},
  { -158.268717409339f,    14.3996630265555f},
  { -164.108312051735f,    14.8866391823119f},
  { -170.306276104685f,    15.2401622826778f},
  { -176.741089753728f,    15.4266423269812f},
  { 176.741089753728f,    15.4266423269812f},
  { 170.306276104685f,    15.2401622826778f},
  { 164.108312051735f,    14.8866391823119f},
  { 158.268717409339f,    14.3996630265555f},
  { 152.866621959147f,    13.8190092060518f},
  { -154.726319413960f,    19.0551045548580f},
  { -160.500845454788f,    19.8018634182196f},
  { -166.717474411461f,    20.3925183693779f},
  { -173.268297377957f,    20.7733751937705f},
  { 180.0f,    20.9051574478893f},
  { 173.268297377957f,    20.7733751937705f},
  { 166.717474411461f,    20.3925183693779f},
  { 160.500845454788f,    19.8018634182196f},
  { 154.726319413960f,    19.0551045548580f},
  { -156.792394270177f,    24.4861741588079f},
  { -162.972505065717f,    25.3520541808113f},
  { -169.587864385282f,    25.9833065914758f},
  { -176.494949509286f,    26.3172227713073f},
  { 176.494949509286f,    26.3172227713073f},
  { 169.587864385282f,    25.9833065914758f},
  { 162.972505065717f,    25.3520541808113f},
  { 156.792394270177f,    24.4861741588079f},
  { -159.094842552111f,    30.0000000000000f},
  { -165.713601324161f,    30.9182723263291f},
  { -172.744027074595f,    31.5118220765682f},
  { 180.0f,    31.7174744114610f},
  { 172.744027074595f,    31.5118220765682f},
  { 165.713601324161f,    30.9182723263291f},
  { 159.094842552111f,    30.0000000000000f},
  { -161.667822199144f,    35.4690506268222f},
  { -168.755922197526f,    36.3591526402786f},
  { -176.208649155232f,    36.8305660202992f},
  { 176.208649155232f,    36.8305660202992f},
  { 168.755922197526f,    36.3591526402786f},
  { 161.667822199144f,    35.4690506268222f},
  { -164.549562913674f,    40.7647476003615f},
  { -172.131755463802f,    41.5411547603032f},
  { 180.0f,    41.8103148957786f},
  { 172.131755463802f,    41.5411547603032f},
  { 164.549562913674f,    40.7647476003615f},
  { -167.781680133751f,    45.7717617455715f},
  { -175.871505448613f,    46.3533031992989f},
  { 175.871505448613f,    46.3533031992989f},
  { 167.781680133751f,    45.7717617455715f},
  { -171.407478359055f,    50.3983682135674f},
  { 180.0f,    50.7156671204829f},
  { 171.407478359055f,    50.3983682135674f},
  { -175.468685954122f,    54.5808573006801f},
  { 175.468685954122f,    54.5808573006801f},
  { -149.665664596075f,    -4.24864627780931f},
  { -151.187239012716f,    -8.86454598437525f},
  { -152.866621959147f,    -13.8190092060518f},
  { -154.726319413960f,    -19.0551045548580f},
  { -156.792394270177f,    -24.4861741588079f},
  { -159.094842552111f,    -30.0000000000000f},
  { -161.667822199144f,    -35.4690506268222f},
  { -164.549562913674f,    -40.7647476003615f},
  { -167.781680133751f,    -45.7717617455715f},
  { -171.407478359055f,    -50.3983682135674f},
  { -175.468685954122f,    -54.5808573006801f},
  { -154.416540465702f,    -4.43921003719893f},
  { -156.248788114164f,    -9.25345760302041f},
  { -158.268717409339f,    -14.3996630265555f},
  { -160.500845454788f,    -19.8018634182196f},
  { -162.972505065717f,    -25.3520541808113f},
  { -165.713601324161f,    -30.9182723263291f},
  { -168.755922197526f,    -36.3591526402786f},
  { -172.131755463802f,    -41.5411547603032f},
  { -175.871505448613f,    -46.3533031992989f},
  { 180.0f,    -50.7156671204829f},
  { 175.468685954122f,    -54.5808573006801f},
  { -159.576046262270f,    -4.61161924706584f},
  { -161.735723622432f,    -9.59406822686046f},
  { -164.108312051735f,    -14.8866391823119f},
  { -166.717474411461f,    -20.3925183693779f},
  { -169.587864385282f,    -25.9833065914758f},
  { -172.744027074595f,    -31.5118220765682f},
  { -176.208649155232f,    -36.8305660202992f},
  { 180.0f,    -41.8103148957786f},
  { 175.871505448613f,    -46.3533031992989f},
  { 171.407478359055f,    -50.3983682135674f},
  { -165.105295966814f,    -4.75497448828983f},
  { -167.591603840212f,    -9.86176221445378f},
  { -170.306276104685f,    -15.2401622826778f},
  { -173.268297377957f,    -20.7733751937705f},
  { -176.494949509286f,    -26.3172227713073f},
  { 180.0f,    -31.7174744114610f},
  { 176.208649155232f,    -36.8305660202992f},
  { 172.131755463802f,    -41.5411547603032f},
  { 167.781680133751f,    -45.7717617455715f},
  { -170.932726005185f,    -4.85832138221458f},
  { -173.722195160908f,    -10.0335510226076f},
  { -176.741089753728f,    -15.4266423269812f},
  { 180.0f,    -20.9051574478893f},
  { 176.494949509286f,    -26.3172227713073f},
  { 172.744027074595f,    -31.5118220765682f},
  { 168.755922197526f,    -36.3591526402786f},
  { 164.549562913674f,    -40.7647476003615f},
  { -176.954958667688f,    -4.91258754288848f},
  { -180.0f,    -10.0928404843176f},
  { 176.741089753728f,    -15.4266423269812f},
  { 173.268297377957f,    -20.7733751937705f},
  { 169.587864385282f,    -25.9833065914758f},
  { 165.713601324161f,    -30.9182723263291f},
  { 161.667822199144f,    -35.4690506268222f},
  { 176.954958667688f,    -4.91258754288848f},
  { 173.722195160908f,    -10.0335510226076f},
  { 170.306276104685f,    -15.2401622826778f},
  { 166.717474411461f,    -20.3925183693779f},
  { 162.972505065717f,    -25.3520541808113f},
  { 159.094842552111f,    -30.0000000000000f},
  { 170.932726005185f,    -4.85832138221458f},
  { 167.591603840212f,    -9.86176221445378f},
  { 164.108312051735f,    -14.8866391823119f},
  { 160.500845454788f,    -19.8018634182196f},
  { 156.792394270177f,    -24.4861741588079f},
  { 165.105295966814f,    -4.75497448828983f},
  { 161.735723622432f,    -9.59406822686046f},
  { 158.268717409339f,    -14.3996630265555f},
  { 154.726319413960f,    -19.0551045548580f},
  { 159.576046262270f,    -4.61161924706584f},
  { 156.248788114164f,    -9.25345760302040f},
  { 152.866621959147f,    -13.8190092060518f},
  { 154.416540465702f,    -4.43921003719892f},
  { 151.187239012716f,    -8.86454598437524f},
  { 149.665664596075f,    -4.24864627780932f},
  { 90.0f,    27.2497500637699f},
  { 90.0f,    22.3927709289549f},
  { 90.0f,    17.1720378505138f},
  { 90.0f,    11.6407231367706f},
  { 90.0f,    5.88104874728658f},
  { 90.0f,    0.0f},
  { 90.0f,    -5.88104874728658f},
  { 90.0f,    -11.6407231367706f},
  { 90.0f,    -17.1720378505138f},
  { 90.0f,    -22.3927709289549f},
  { 90.0f,    -27.2497500637699f},
  { 85.0805005522338f,    30.2422442495968f},
  { 85.0805005522338f,    25.5011946243577f},
  { 85.0805005522338f,    20.3548979307814f},
  { 85.0805005522338f,    14.8422605657152f},
  { 85.0805005522338f,    9.03442358812835f},
  { 85.0805005522338f,    3.03384490759980f},
  { 85.0805005522338f,    -3.03384490759980f},
  { 85.0805005522338f,    -9.03442358812835f},
  { 85.0805005522338f,    -14.8422605657153f},
  { 85.0805005522338f,    -20.3548979307814f},
  { 85.0805005522338f,    -25.5011946243577f},
  { 85.0805005522338f,    -30.2422442495968f},
  { 79.9071595156824f,    28.4369994004269f},
  { 79.9071595156824f,    23.4235277049200f},
  { 79.9071595156824f,    18.0f},
  { 79.9071595156824f,    12.2221928049541f},
  { 79.9071595156824f,    6.18141426337592f},
  { 79.9071595156824f,    0.0f},
  { 79.9071595156824f,    -6.18141426337592f},
  { 79.9071595156824f,    -12.2221928049541f},
  { 79.9071595156824f,    -18.0000000000000f},
  { 79.9071595156824f,    -23.4235277049200f},
  { 79.9071595156824f,    -28.4369994004269f},
  { 74.5495629136742f,    26.2866766550738f},
  { 74.5495629136742f,    21.0156123886821f},
  { 74.5495629136742f,    15.3449217472209f},
  { 74.5495629136742f,    9.34970354257458f},
  { 74.5495629136742f,    3.14137763310105f},
  { 74.5495629136742f,    -3.14137763310105f},
  { 74.5495629136742f,    -9.34970354257458f},
  { 74.5495629136742f,    -15.3449217472209f},
  { 74.5495629136742f,    -21.0156123886821f},
  { 74.5495629136742f,    -26.2866766550738f},
  { 69.0948425521107f,    23.8001826021751f},
  { 69.0948425521107f,    18.3038160575204f},
  { 69.0948425521107f,    12.4362806936207f},
  { 69.0948425521107f,    6.29224838690085f},
  { 69.0948425521107f,    0.0f},
  { 69.0948425521107f,    -6.29224838690085f},
  { 69.0948425521107f,    -12.4362806936207f},
  { 69.0948425521107f,    -18.3038160575204f},
  { 69.0948425521107f,    -23.8001826021751f},
  { 63.6401221905472f,    21.0156123886821f},
  { 63.6401221905472f,    15.3449217472209f},
  { 63.6401221905472f,    9.34970354257458f},
  { 63.6401221905472f,    3.14137763310105f},
  { 63.6401221905472f,    -3.14137763310105f},
  { 63.6401221905472f,    -9.34970354257458f},
  { 63.6401221905472f,    -15.3449217472209f},
  { 63.6401221905472f,    -21.0156123886821f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    12.2221928049541f},
  { 58.2825255885390f,    6.18141426337591f},
  { 58.2825255885390f,    0.0f},
  { 58.2825255885390f,    -6.18141426337592f},
  { 58.2825255885390f,    -12.2221928049541f},
  { 58.2825255885390f,    -18.0f},
  { 53.1091845519876f,    14.8422605657152f},
  { 53.1091845519876f,    9.03442358812835f},
  { 53.1091845519876f,    3.03384490759980f},
  { 53.1091845519876f,    -3.03384490759980f},
  { 53.1091845519876f,    -9.03442358812835f},
  { 53.1091845519876f,    -14.8422605657152f},
  { 48.1896851042214f,    11.6407231367706f},
  { 48.1896851042214f,    5.88104874728657f},
  { 48.1896851042214f,    0.0f},
  { 48.1896851042214f,    -5.88104874728657f},
  { 48.1896851042214f,    -11.6407231367706f},
  { 43.5723495188822f,    8.48907956344660f},
  { 43.5723495188822f,    2.84823403241321f},
  { 43.5723495188822f,    -2.84823403241321f},
  { 43.5723495188822f,    -8.48907956344660f},
  { 39.2843328795171f,    5.46504310153393f},
  { 39.2843328795171f,    0.0f},
  { 39.2843328795171f,    -5.46504310153393f},
  { 35.3344704180614f,    2.62431957300546f},
  { 35.3344704180614f,    -2.62431957300546f},
  { 94.9194994477662f,    30.2422442495968f},
  { 100.092840484318f,    28.4369994004269f},
  { 105.450437086326f,    26.2866766550738f},
  { 110.905157447889f,    23.8001826021751f},
  { 116.359877809453f,    21.0156123886821f},
  { 121.717474411461f,    18.0f},
  { 126.890815448012f,    14.8422605657152f},
  { 131.810314895779f,    11.6407231367706f},
  { 136.427650481118f,    8.48907956344661f},
  { 140.715667120483f,    5.46504310153393f},
  { 144.665529581939f,    2.62431957300546f},
  { 94.9194994477662f,    25.5011946243577f},
  { 100.092840484318f,    23.4235277049200f},
  { 105.450437086326f,    21.0156123886821f},
  { 110.905157447889f,    18.3038160575204f},
  { 116.359877809453f,    15.3449217472209f},
  { 121.717474411461f,    12.2221928049541f},
  { 126.890815448012f,    9.03442358812835f},
  { 131.810314895779f,    5.88104874728657f},
  { 136.427650481118f,    2.84823403241321f},
  { 140.715667120483f,    0.0f},
  { 144.665529581939f,    -2.62431957300546f},
  { 94.9194994477662f,    20.3548979307814f},
  { 100.092840484318f,    18.0000000000000f},
  { 105.450437086326f,    15.3449217472209f},
  { 110.905157447889f,    12.4362806936207f},
  { 116.359877809453f,    9.34970354257458f},
  { 121.717474411461f,    6.18141426337591f},
  { 126.890815448012f,    3.03384490759979f},
  { 131.810314895779f,    0.0f},
  { 136.427650481118f,    -2.84823403241321f},
  { 140.715667120483f,    -5.46504310153393f},
  { 94.9194994477662f,    14.8422605657152f},
  { 100.092840484318f,    12.2221928049541f},
  { 105.450437086326f,    9.34970354257458f},
  { 110.905157447889f,    6.29224838690085f},
  { 116.359877809453f,    3.14137763310105f},
  { 121.717474411461f,    0.0f},
  { 126.890815448012f,    -3.03384490759980f},
  { 131.810314895779f,    -5.88104874728658f},
  { 136.427650481118f,    -8.48907956344661f},
  { 94.9194994477662f,    9.03442358812835f},
  { 100.092840484318f,    6.18141426337591f},
  { 105.450437086326f,    3.14137763310105f},
  { 110.905157447889f,    0.0f},
  { 116.359877809453f,    -3.14137763310105f},
  { 121.717474411461f,    -6.18141426337591f},
  { 126.890815448012f,    -9.03442358812835f},
  { 131.810314895779f,    -11.6407231367706f},
  { 94.9194994477662f,    3.03384490759980f},
  { 100.092840484318f,    9.82845448111024e-16f},
  { 105.450437086326f,    -3.14137763310105f},
  { 110.905157447889f,    -6.29224838690085f},
  { 116.359877809453f,    -9.34970354257458f},
  { 121.717474411461f,    -12.2221928049541f},
  { 126.890815448012f,    -14.8422605657152f},
  { 94.9194994477662f,    -3.03384490759980f},
  { 100.092840484318f,    -6.18141426337591f},
  { 105.450437086326f,    -9.34970354257458f},
  { 110.905157447889f,    -12.4362806936207f},
  { 116.359877809453f,    -15.3449217472209f},
  { 121.717474411461f,    -18.0f},
  { 94.9194994477662f,    -9.03442358812835f},
  { 100.092840484318f,    -12.2221928049541f},
  { 105.450437086326f,    -15.3449217472209f},
  { 110.905157447889f,    -18.3038160575204f},
  { 116.359877809453f,    -21.0156123886821f},
  { 94.9194994477662f,    -14.8422605657153f},
  { 100.092840484318f,    -18.0f},
  { 105.450437086326f,    -21.0156123886821f},
  { 110.905157447889f,    -23.8001826021751f},
  { 94.9194994477662f,    -20.3548979307814f},
  { 100.092840484318f,    -23.4235277049200f},
  { 105.450437086326f,    -26.2866766550738f},
  { 94.9194994477662f,    -25.5011946243577f},
  { 100.092840484318f,    -28.4369994004269f},
  { 94.9194994477662f,    -30.2422442495968f},
  { -85.0805005522338f,    30.2422442495968f},
  { -79.9071595156824f,    28.4369994004269f},
  { -74.5495629136742f,    26.2866766550738f},
  { -69.0948425521107f,    23.8001826021751f},
  { -63.6401221905472f,    21.0156123886821f},
  { -58.2825255885390f,    18.0f},
  { -53.1091845519876f,    14.8422605657152f},
  { -48.1896851042214f,    11.6407231367706f},
  { -43.5723495188822f,    8.48907956344660f},
  { -39.2843328795171f,    5.46504310153393f},
  { -35.3344704180614f,    2.62431957300546f},
  { -90.0f,    27.2497500637699f},
  { -85.0805005522338f,    25.5011946243577f},
  { -79.9071595156824f,    23.4235277049200f},
  { -74.5495629136742f,    21.0156123886821f},
  { -69.0948425521107f,    18.3038160575204f},
  { -63.6401221905472f,    15.3449217472209f},
  { -58.2825255885390f,    12.2221928049541f},
  { -53.1091845519876f,    9.03442358812835f},
  { -48.1896851042214f,    5.88104874728657f},
  { -43.5723495188822f,    2.84823403241321f},
  { -39.2843328795171f,    0.0f},
  { -35.3344704180614f,    -2.62431957300546f},
  { -90.0f,    22.3927709289549f},
  { -85.0805005522338f,    20.3548979307814f},
  { -79.9071595156824f,    18.0000000000000f},
  { -74.5495629136742f,    15.3449217472209f},
  { -69.0948425521107f,    12.4362806936207f},
  { -63.6401221905472f,    9.34970354257458f},
  { -58.2825255885390f,    6.18141426337591f},
  { -53.1091845519876f,    3.03384490759979f},
  { -48.1896851042214f,    0.0f},
  { -43.5723495188822f,    -2.84823403241321f},
  { -39.2843328795171f,    -5.46504310153393f},
  { -90.0f,    17.1720378505138f},
  { -85.0805005522338f,    14.8422605657152f},
  { -79.9071595156824f,    12.2221928049541f},
  { -74.5495629136742f,    9.34970354257458f},
  { -69.0948425521107f,    6.29224838690085f},
  { -63.6401221905472f,    3.14137763310105f},
  { -58.2825255885390f,    0.0f},
  { -53.1091845519876f,    -3.03384490759980f},
  { -48.1896851042214f,    -5.88104874728658f},
  { -43.5723495188822f,    -8.48907956344661f},
  { -90.0f,    11.6407231367706f},
  { -85.0805005522338f,    9.03442358812835f},
  { -79.9071595156824f,    6.18141426337591f},
  { -74.5495629136742f,    3.14137763310105f},
  { -69.0948425521107f,    0.0f},
  { -63.6401221905472f,    -3.14137763310105f},
  { -58.2825255885390f,    -6.18141426337591f},
  { -53.1091845519876f,    -9.03442358812835f},
  { -48.1896851042214f,    -11.6407231367706f},
  { -90.0f,    5.88104874728658f},
  { -85.0805005522338f,    3.03384490759980f},
  { -79.9071595156824f,    9.82845448111024e-16f},
  { -74.5495629136742f,    -3.14137763310105f},
  { -69.0948425521107f,    -6.29224838690085f},
  { -63.6401221905472f,    -9.34970354257458f},
  { -58.2825255885390f,    -12.2221928049541f},
  { -53.1091845519876f,    -14.8422605657152f},
  { -90.0f,    0.0f},
  { -85.0805005522338f,    -3.03384490759980f},
  { -79.9071595156824f,    -6.18141426337591f},
  { -74.5495629136742f,    -9.34970354257458f},
  { -69.0948425521107f,    -12.4362806936207f},
  { -63.6401221905472f,    -15.3449217472209f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -5.88104874728658f},
  { -85.0805005522338f,    -9.03442358812836f},
  { -79.9071595156824f,    -12.2221928049541f},
  { -74.5495629136742f,    -15.3449217472209f},
  { -69.0948425521107f,    -18.3038160575204f},
  { -63.6401221905472f,    -21.0156123886821f},
  { -90.0f,    -11.6407231367706f},
  { -85.0805005522338f,    -14.8422605657153f},
  { -79.9071595156824f,    -18.0f},
  { -74.5495629136742f,    -21.0156123886821f},
  { -69.0948425521107f,    -23.8001826021751f},
  { -90.0f,    -17.1720378505138f},
  { -85.0805005522338f,    -20.3548979307814f},
  { -79.9071595156824f,    -23.4235277049200f},
  { -74.5495629136742f,    -26.2866766550738f},
  { -90.0f,    -22.3927709289549f},
  { -85.0805005522338f,    -25.5011946243577f},
  { -79.9071595156824f,    -28.4369994004269f},
  { -90.0f,    -27.2497500637699f},
  { -85.0805005522338f,    -30.2422442495968f},
  { -94.9194994477662f,    30.2422442495968f},
  { -94.9194994477662f,    25.5011946243577f},
  { -94.9194994477662f,    20.3548979307814f},
  { -94.9194994477662f,    14.8422605657152f},
  { -94.9194994477662f,    9.03442358812835f},
  { -94.9194994477662f,    3.03384490759980f},
  { -94.9194994477662f,    -3.03384490759980f},
  { -94.9194994477662f,    -9.03442358812835f},
  { -94.9194994477662f,    -14.8422605657153f},
  { -94.9194994477662f,    -20.3548979307814f},
  { -94.9194994477662f,    -25.5011946243577f},
  { -94.9194994477662f,    -30.2422442495968f},
  { -100.092840484318f,    28.4369994004269f},
  { -100.092840484318f,    23.4235277049200f},
  { -100.092840484318f,    18.0f},
  { -100.092840484318f,    12.2221928049541f},
  { -100.092840484318f,    6.18141426337592f},
  { -100.092840484318f,    0.0f},
  { -100.092840484318f,    -6.18141426337592f},
  { -100.092840484318f,    -12.2221928049541f},
  { -100.092840484318f,    -18.0000000000000f},
  { -100.092840484318f,    -23.4235277049200f},
  { -100.092840484318f,    -28.4369994004269f},
  { -105.450437086326f,    26.2866766550738f},
  { -105.450437086326f,    21.0156123886821f},
  { -105.450437086326f,    15.3449217472209f},
  { -105.450437086326f,    9.34970354257458f},
  { -105.450437086326f,    3.14137763310105f},
  { -105.450437086326f,    -3.14137763310105f},
  { -105.450437086326f,    -9.34970354257458f},
  { -105.450437086326f,    -15.3449217472209f},
  { -105.450437086326f,    -21.0156123886821f},
  { -105.450437086326f,    -26.2866766550738f},
  { -110.905157447889f,    23.8001826021751f},
  { -110.905157447889f,    18.3038160575204f},
  { -110.905157447889f,    12.4362806936207f},
  { -110.905157447889f,    6.29224838690085f},
  { -110.905157447889f,    0.0f},
  { -110.905157447889f,    -6.29224838690085f},
  { -110.905157447889f,    -12.4362806936207f},
  { -110.905157447889f,    -18.3038160575204f},
  { -110.905157447889f,    -23.8001826021751f},
  { -116.359877809453f,    21.0156123886821f},
  { -116.359877809453f,    15.3449217472209f},
  { -116.359877809453f,    9.34970354257458f},
  { -116.359877809453f,    3.14137763310105f},
  { -116.359877809453f,    -3.14137763310105f},
  { -116.359877809453f,    -9.34970354257458f},
  { -116.359877809453f,    -15.3449217472209f},
  { -116.359877809453f,    -21.0156123886821f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    12.2221928049541f},
  { -121.717474411461f,    6.18141426337591f},
  { -121.717474411461f,    0.0f},
  { -121.717474411461f,    -6.18141426337592f},
  { -121.717474411461f,    -12.2221928049541f},
  { -121.717474411461f,    -18.0f},
  { -126.890815448012f,    14.8422605657152f},
  { -126.890815448012f,    9.03442358812835f},
  { -126.890815448012f,    3.03384490759980f},
  { -126.890815448012f,    -3.03384490759980f},
  { -126.890815448012f,    -9.03442358812835f},
  { -126.890815448012f,    -14.8422605657152f},
  { -131.810314895779f,    11.6407231367706f},
  { -131.810314895779f,    5.88104874728658f},
  { -131.810314895779f,    0.0f},
  { -131.810314895779f,    -5.88104874728657f},
  { -131.810314895779f,    -11.6407231367706f},
  { -136.427650481118f,    8.48907956344661f},
  { -136.427650481118f,    2.84823403241321f},
  { -136.427650481118f,    -2.84823403241321f},
  { -136.427650481118f,    -8.48907956344661f},
  { -140.715667120483f,    5.46504310153393f},
  { -140.715667120483f,    0.0f},
  { -140.715667120483f,    -5.46504310153393f},
  { -144.665529581939f,    2.62431957300546f},
  { -144.665529581939f,    -2.62431957300546f},
  { 81.3356121863394f,    33.7636228467164f},
  { 75.6532686891708f,    31.8445438275894f},
  { 69.8387746668677f,    29.5214139777455f},
  { 64.0089693077334f,    26.8136666500115f},
  { 58.2825255885390f,    23.7778071950459f},
  { 52.7660043154438f,    20.5035260967303f},
  { 47.5432608746571f,    17.1013346715150f},
  { 42.6704663121986f,    13.6853671173645f},
  { 38.1766660785064f,    10.3570763724901f},
  { 34.0681325143608f,    7.19449987970156f},
  { 76.9119732396282f,    37.4462884164009f},
  { 70.6909217483267f,    35.3584095083404f},
  { 64.4321489417198f,    32.8030205324522f},
  { 58.2825255885390f,    29.8185857366241f},
  { 52.3742314373761f,    26.4879839228922f},
  { 46.8095496995643f,    22.9282785675942f},
  { 41.6540851162230f,    19.2717588609047f},
  { 36.9382074899021f,    15.6447696005703f},
  { 32.6637843665159f,    12.1512282836016f},
  { 71.6762730547777f,    41.1884885194893f},
  { 64.9225623039876f,    38.8577003880126f},
  { 58.2825255885390f,    36.0f},
  { 51.9228047015969f,    32.6839097448066f},
  { 45.9698897815951f,    29.0258428248200f},
  { 40.5007050124200f,    25.1712621719368f},
  { 35.5457299459382f,    21.2697401431116f},
  { 31.0995633015582f,    17.4525550414161f},
  { 65.4975223656942f,    44.8430231404270f},
  { 58.2825255885390f,    42.1814142633759f},
  { 51.3970478198948f,    38.9522209765832f},
  { 45.0000000000000f,    35.2643896827547f},
  { 39.1817809345817f,    31.2700419814429f},
  { 33.9705954101743f,    27.1374399585075f},
  { 29.3493047034929f,    23.0236551514198f},
  { 58.2825255885390f,    48.2221928049541f},
  { 50.7770706423091f,    45.1435105228856f},
  { 43.8677469697587f,    41.4939010991043f},
  { 37.6606701725719f,    37.4262981266073f},
  { 32.1772657124426f,    33.1214693878281f},
  { 27.3816514158873f,    28.7575899095502f},
  { 50.0352463212626f,    51.1183373464232f},
  { 42.5297913750327f,    47.5637193063793f},
  { 35.8897546595841f,    43.4870777455218f},
  { 30.1213952152451f,    39.0760490721854f},
  { 25.1590482744009f,    34.5232572531018f},
  { 40.9264284253102f,    53.3421784101822f},
  { 33.8063837786152f,    49.3082490896532f},
  { 27.7474379908482f,    44.8539773066727f},
  { 22.6366917551365f,    40.1797369172332f},
  { 31.3271562819128f,    54.7710088310124f},
  { 24.9857717894179f,    50.3244736173956f},
  { 19.7616067396473f,    45.5931971809751f},
  { 21.7495436012359f,    55.3853099512686f},
  { 16.4722106927604f,    50.6511725865265f},
  { 12.6990470336401f,    55.2704201111625f},
  { 81.3356121863394f,    -33.7636228467164f},
  { 76.9119732396282f,    -37.4462884164009f},
  { 71.6762730547777f,    -41.1884885194893f},
  { 65.4975223656942f,    -44.8430231404270f},
  { 58.2825255885390f,    -48.2221928049541f},
  { 50.0352463212626f,    -51.1183373464232f},
  { 40.9264284253102f,    -53.3421784101822f},
  { 31.3271562819128f,    -54.7710088310124f},
  { 21.7495436012359f,    -55.3853099512686f},
  { 12.6990470336401f,    -55.2704201111625f},
  { 75.6532686891708f,    -31.8445438275894f},
  { 70.6909217483267f,    -35.3584095083404f},
  { 64.9225623039876f,    -38.8577003880126f},
  { 58.2825255885390f,    -42.1814142633759f},
  { 50.7770706423091f,    -45.1435105228856f},
  { 42.5297913750327f,    -47.5637193063793f},
  { 33.8063837786152f,    -49.3082490896532f},
  { 24.9857717894179f,    -50.3244736173956f},
  { 16.4722106927604f,    -50.6511725865265f},
  { 69.8387746668677f,    -29.5214139777455f},
  { 64.4321489417198f,    -32.8030205324522f},
  { 58.2825255885390f,    -36.0f},
  { 51.3970478198948f,    -38.9522209765832f},
  { 43.8677469697587f,    -41.4939010991043f},
  { 35.8897546595841f,    -43.4870777455218f},
  { 27.7474379908481f,    -44.8539773066727f},
  { 19.7616067396473f,    -45.5931971809751f},
  { 64.0089693077334f,    -26.8136666500115f},
  { 58.2825255885390f,    -29.8185857366241f},
  { 51.9228047015969f,    -32.6839097448066f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 37.6606701725719f,    -37.4262981266073f},
  { 30.1213952152451f,    -39.0760490721854f},
  { 22.6366917551365f,    -40.1797369172332f},
  { 58.2825255885390f,    -23.7778071950459f},
  { 52.3742314373762f,    -26.4879839228922f},
  { 45.9698897815950f,    -29.0258428248200f},
  { 39.1817809345817f,    -31.2700419814429f},
  { 32.1772657124426f,    -33.1214693878281f},
  { 25.1590482744009f,    -34.5232572531018f},
  { 52.7660043154438f,    -20.5035260967303f},
  { 46.8095496995643f,    -22.9282785675942f},
  { 40.5007050124200f,    -25.1712621719368f},
  { 33.9705954101743f,    -27.1374399585075f},
  { 27.3816514158873f,    -28.7575899095502f},
  { 47.5432608746571f,    -17.1013346715150f},
  { 41.6540851162230f,    -19.2717588609047f},
  { 35.5457299459382f,    -21.2697401431116f},
  { 29.3493047034929f,    -23.0236551514198f},
  { 42.6704663121986f,    -13.6853671173645f},
  { 36.9382074899021f,    -15.6447696005703f},
  { 31.0995633015582f,    -17.4525550414161f},
  { 38.1766660785064f,    -10.3570763724901f},
  { 32.6637843665159f,    -12.1512282836016f},
  { 34.0681325143608f,    -7.19449987970155f},
  { -12.6990470336401f,    55.2704201111625f},
  { -16.4722106927604f,    50.6511725865265f},
  { -19.7616067396473f,    45.5931971809751f},
  { -22.6366917551365f,    40.1797369172332f},
  { -25.1590482744010f,    34.5232572531018f},
  { -27.3816514158873f,    28.7575899095502f},
  { -29.3493047034929f,    23.0236551514198f},
  { -31.0995633015582f,    17.4525550414161f},
  { -32.6637843665159f,    12.1512282836016f},
  { -34.0681325143608f,    7.19449987970155f},
  { -21.7495436012359f,    55.3853099512686f},
  { -24.9857717894179f,    50.3244736173956f},
  { -27.7474379908482f,    44.8539773066727f},
  { -30.1213952152451f,    39.0760490721854f},
  { -32.1772657124426f,    33.1214693878282f},
  { -33.9705954101743f,    27.1374399585075f},
  { -35.5457299459382f,    21.2697401431116f},
  { -36.9382074899021f,    15.6447696005703f},
  { -38.1766660785064f,    10.3570763724901f},
  { -31.3271562819128f,    54.7710088310124f},
  { -33.8063837786152f,    49.3082490896532f},
  { -35.8897546595841f,    43.4870777455218f},
  { -37.6606701725719f,    37.4262981266073f},
  { -39.1817809345817f,    31.2700419814429f},
  { -40.5007050124200f,    25.1712621719368f},
  { -41.6540851162230f,    19.2717588609047f},
  { -42.6704663121986f,    13.6853671173645f},
  { -40.9264284253102f,    53.3421784101822f},
  { -42.5297913750327f,    47.5637193063793f},
  { -43.8677469697587f,    41.4939010991043f},
  { -45.0000000000000f,    35.2643896827547f},
  { -45.9698897815951f,    29.0258428248200f},
  { -46.8095496995643f,    22.9282785675942f},
  { -47.5432608746571f,    17.1013346715150f},
  { -50.0352463212626f,    51.1183373464232f},
  { -50.7770706423091f,    45.1435105228856f},
  { -51.3970478198948f,    38.9522209765831f},
  { -51.9228047015969f,    32.6839097448066f},
  { -52.3742314373761f,    26.4879839228922f},
  { -52.7660043154438f,    20.5035260967303f},
  { -58.2825255885390f,    48.2221928049541f},
  { -58.2825255885390f,    42.1814142633759f},
  { -58.2825255885390f,    36.0f},
  { -58.2825255885390f,    29.8185857366241f},
  { -58.2825255885390f,    23.7778071950459f},
  { -65.4975223656942f,    44.8430231404270f},
  { -64.9225623039876f,    38.8577003880126f},
  { -64.4321489417198f,    32.8030205324522f},
  { -64.0089693077334f,    26.8136666500115f},
  { -71.6762730547777f,    41.1884885194893f},
  { -70.6909217483267f,    35.3584095083404f},
  { -69.8387746668677f,    29.5214139777455f},
  { -76.9119732396282f,    37.4462884164009f},
  { -75.6532686891708f,    31.8445438275894f},
  { -81.3356121863394f,    33.7636228467164f},
  { -12.6990470336401f,    -55.2704201111625f},
  { -21.7495436012359f,    -55.3853099512686f},
  { -31.3271562819128f,    -54.7710088310124f},
  { -40.9264284253102f,    -53.3421784101822f},
  { -50.0352463212626f,    -51.1183373464232f},
  { -58.2825255885390f,    -48.2221928049541f},
  { -65.4975223656942f,    -44.8430231404270f},
  { -71.6762730547777f,    -41.1884885194893f},
  { -76.9119732396282f,    -37.4462884164009f},
  { -81.3356121863394f,    -33.7636228467164f},
  { -16.4722106927604f,    -50.6511725865265f},
  { -24.9857717894179f,    -50.3244736173956f},
  { -33.8063837786152f,    -49.3082490896532f},
  { -42.5297913750327f,    -47.5637193063793f},
  { -50.7770706423091f,    -45.1435105228856f},
  { -58.2825255885390f,    -42.1814142633759f},
  { -64.9225623039876f,    -38.8577003880126f},
  { -70.6909217483267f,    -35.3584095083404f},
  { -75.6532686891708f,    -31.8445438275894f},
  { -19.7616067396473f,    -45.5931971809751f},
  { -27.7474379908482f,    -44.8539773066727f},
  { -35.8897546595841f,    -43.4870777455218f},
  { -43.8677469697587f,    -41.4939010991043f},
  { -51.3970478198948f,    -38.9522209765831f},
  { -58.2825255885390f,    -36.0f},
  { -64.4321489417198f,    -32.8030205324522f},
  { -69.8387746668677f,    -29.5214139777455f},
  { -22.6366917551365f,    -40.1797369172332f},
  { -30.1213952152451f,    -39.0760490721854f},
  { -37.6606701725719f,    -37.4262981266073f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -51.9228047015969f,    -32.6839097448066f},
  { -58.2825255885390f,    -29.8185857366241f},
  { -64.0089693077334f,    -26.8136666500115f},
  { -25.1590482744009f,    -34.5232572531018f},
  { -32.1772657124426f,    -33.1214693878281f},
  { -39.1817809345817f,    -31.2700419814429f},
  { -45.9698897815950f,    -29.0258428248200f},
  { -52.3742314373761f,    -26.4879839228922f},
  { -58.2825255885390f,    -23.7778071950459f},
  { -27.3816514158873f,    -28.7575899095502f},
  { -33.9705954101743f,    -27.1374399585075f},
  { -40.5007050124200f,    -25.1712621719368f},
  { -46.8095496995643f,    -22.9282785675942f},
  { -52.7660043154438f,    -20.5035260967303f},
  { -29.3493047034929f,    -23.0236551514198f},
  { -35.5457299459382f,    -21.2697401431116f},
  { -41.6540851162230f,    -19.2717588609047f},
  { -47.5432608746571f,    -17.1013346715150f},
  { -31.0995633015582f,    -17.4525550414161f},
  { -36.9382074899021f,    -15.6447696005703f},
  { -42.6704663121986f,    -13.6853671173645f},
  { -32.6637843665159f,    -12.1512282836016f},
  { -38.1766660785064f,    -10.3570763724901f},
  { -34.0681325143608f,    -7.19449987970155f},
  { 167.300952966360f,    55.2704201111625f},
  { 163.527789307240f,    50.6511725865265f},
  { 160.238393260353f,    45.5931971809751f},
  { 157.363308244864f,    40.1797369172332f},
  { 154.840951725599f,    34.5232572531018f},
  { 152.618348584113f,    28.7575899095502f},
  { 150.650695296507f,    23.0236551514198f},
  { 148.900436698442f,    17.4525550414161f},
  { 147.336215633484f,    12.1512282836016f},
  { 145.931867485639f,    7.19449987970155f},
  { 158.250456398764f,    55.3853099512686f},
  { 155.014228210582f,    50.3244736173956f},
  { 152.252562009152f,    44.8539773066727f},
  { 149.878604784755f,    39.0760490721854f},
  { 147.822734287557f,    33.1214693878282f},
  { 146.029404589826f,    27.1374399585075f},
  { 144.454270054062f,    21.2697401431116f},
  { 143.061792510098f,    15.6447696005703f},
  { 141.823333921494f,    10.3570763724901f},
  { 148.672843718087f,    54.7710088310124f},
  { 146.193616221385f,    49.3082490896532f},
  { 144.110245340416f,    43.4870777455218f},
  { 142.339329827428f,    37.4262981266073f},
  { 140.818219065418f,    31.2700419814429f},
  { 139.499294987580f,    25.1712621719368f},
  { 138.345914883777f,    19.2717588609047f},
  { 137.329533687801f,    13.6853671173645f},
  { 139.073571574690f,    53.3421784101822f},
  { 137.470208624967f,    47.5637193063793f},
  { 136.132253030241f,    41.4939010991043f},
  { 135.0f,    35.2643896827547f},
  { 134.030110218405f,    29.0258428248200f},
  { 133.190450300436f,    22.9282785675942f},
  { 132.456739125343f,    17.1013346715150f},
  { 129.964753678737f,    51.1183373464232f},
  { 129.222929357691f,    45.1435105228856f},
  { 128.602952180105f,    38.9522209765831f},
  { 128.077195298403f,    32.6839097448066f},
  { 127.625768562624f,    26.4879839228922f},
  { 127.233995684556f,    20.5035260967303f},
  { 121.717474411461f,    48.2221928049541f},
  { 121.717474411461f,    42.1814142633759f},
  { 121.717474411461f,    36.0f},
  { 121.717474411461f,    29.8185857366241f},
  { 121.717474411461f,    23.7778071950459f},
  { 114.502477634306f,    44.8430231404270f},
  { 115.077437696012f,    38.8577003880126f},
  { 115.567851058280f,    32.8030205324522f},
  { 115.991030692267f,    26.8136666500115f},
  { 108.323726945222f,    41.1884885194893f},
  { 109.309078251673f,    35.3584095083404f},
  { 110.161225333132f,    29.5214139777455f},
  { 103.088026760372f,    37.4462884164009f},
  { 104.346731310829f,    31.8445438275894f},
  { 98.6643878136606f,    33.7636228467164f},
  { 167.300952966360f,    -55.2704201111625f},
  { 158.250456398764f,    -55.3853099512686f},
  { 148.672843718087f,    -54.7710088310124f},
  { 139.073571574690f,    -53.3421784101822f},
  { 129.964753678737f,    -51.1183373464232f},
  { 121.717474411461f,    -48.2221928049541f},
  { 114.502477634306f,    -44.8430231404270f},
  { 108.323726945222f,    -41.1884885194893f},
  { 103.088026760372f,    -37.4462884164009f},
  { 98.6643878136606f,    -33.7636228467164f},
  { 163.527789307240f,    -50.6511725865265f},
  { 155.014228210582f,    -50.3244736173956f},
  { 146.193616221385f,    -49.3082490896532f},
  { 137.470208624967f,    -47.5637193063793f},
  { 129.222929357691f,    -45.1435105228856f},
  { 121.717474411461f,    -42.1814142633759f},
  { 115.077437696012f,    -38.8577003880126f},
  { 109.309078251673f,    -35.3584095083403f},
  { 104.346731310829f,    -31.8445438275894f},
  { 160.238393260353f,    -45.5931971809751f},
  { 152.252562009152f,    -44.8539773066727f},
  { 144.110245340416f,    -43.4870777455218f},
  { 136.132253030241f,    -41.4939010991043f},
  { 128.602952180105f,    -38.9522209765831f},
  { 121.717474411461f,    -36.0f},
  { 115.567851058280f,    -32.8030205324522f},
  { 110.161225333132f,    -29.5214139777455f},
  { 157.363308244864f,    -40.1797369172332f},
  { 149.878604784755f,    -39.0760490721854f},
  { 142.339329827428f,    -37.4262981266073f},
  { 135.0f,    -35.2643896827547f},
  { 128.077195298403f,    -32.6839097448066f},
  { 121.717474411461f,    -29.8185857366241f},
  { 115.991030692267f,    -26.8136666500115f},
  { 154.840951725599f,    -34.5232572531018f},
  { 147.822734287557f,    -33.1214693878281f},
  { 140.818219065418f,    -31.2700419814429f},
  { 134.030110218405f,    -29.0258428248200f},
  { 127.625768562624f,    -26.4879839228922f},
  { 121.717474411461f,    -23.7778071950459f},
  { 152.618348584113f,    -28.7575899095502f},
  { 146.029404589826f,    -27.1374399585075f},
  { 139.499294987580f,    -25.1712621719368f},
  { 133.190450300436f,    -22.9282785675942f},
  { 127.233995684556f,    -20.5035260967303f},
  { 150.650695296507f,    -23.0236551514198f},
  { 144.454270054062f,    -21.2697401431116f},
  { 138.345914883777f,    -19.2717588609047f},
  { 132.456739125343f,    -17.1013346715150f},
  { 148.900436698442f,    -17.4525550414161f},
  { 143.061792510098f,    -15.6447696005703f},
  { 137.329533687801f,    -13.6853671173645f},
  { 147.336215633484f,    -12.1512282836016f},
  { 141.823333921494f,    -10.3570763724901f},
  { 145.931867485639f,    -7.19449987970155f},
  { -167.300952966360f,    55.2704201111625f},
  { -158.250456398764f,    55.3853099512686f},
  { -148.672843718087f,    54.7710088310124f},
  { -139.073571574690f,    53.3421784101822f},
  { -129.964753678737f,    51.1183373464232f},
  { -121.717474411461f,    48.2221928049541f},
  { -114.502477634306f,    44.8430231404270f},
  { -108.323726945222f,    41.1884885194893f},
  { -103.088026760372f,    37.4462884164009f},
  { -98.6643878136606f,    33.7636228467164f},
  { -163.527789307240f,    50.6511725865265f},
  { -155.014228210582f,    50.3244736173956f},
  { -146.193616221385f,    49.3082490896532f},
  { -137.470208624967f,    47.5637193063793f},
  { -129.222929357691f,    45.1435105228856f},
  { -121.717474411461f,    42.1814142633759f},
  { -115.077437696012f,    38.8577003880126f},
  { -109.309078251673f,    35.3584095083403f},
  { -104.346731310829f,    31.8445438275894f},
  { -160.238393260353f,    45.5931971809751f},
  { -152.252562009152f,    44.8539773066727f},
  { -144.110245340416f,    43.4870777455218f},
  { -136.132253030241f,    41.4939010991043f},
  { -128.602952180105f,    38.9522209765831f},
  { -121.717474411461f,    36.0f},
  { -115.567851058280f,    32.8030205324522f},
  { -110.161225333132f,    29.5214139777455f},
  { -157.363308244864f,    40.1797369172332f},
  { -149.878604784755f,    39.0760490721854f},
  { -142.339329827428f,    37.4262981266073f},
  { -135.0f,    35.2643896827547f},
  { -128.077195298403f,    32.6839097448066f},
  { -121.717474411461f,    29.8185857366241f},
  { -115.991030692267f,    26.8136666500115f},
  { -154.840951725599f,    34.5232572531018f},
  { -147.822734287557f,    33.1214693878281f},
  { -140.818219065418f,    31.2700419814429f},
  { -134.030110218405f,    29.0258428248200f},
  { -127.625768562624f,    26.4879839228922f},
  { -121.717474411461f,    23.7778071950459f},
  { -152.618348584113f,    28.7575899095502f},
  { -146.029404589826f,    27.1374399585075f},
  { -139.499294987580f,    25.1712621719368f},
  { -133.190450300436f,    22.9282785675942f},
  { -127.233995684556f,    20.5035260967303f},
  { -150.650695296507f,    23.0236551514198f},
  { -144.454270054062f,    21.2697401431116f},
  { -138.345914883777f,    19.2717588609047f},
  { -132.456739125343f,    17.1013346715150f},
  { -148.900436698442f,    17.4525550414161f},
  { -143.061792510098f,    15.6447696005703f},
  { -137.329533687801f,    13.6853671173645f},
  { -147.336215633484f,    12.1512282836016f},
  { -141.823333921494f,    10.3570763724901f},
  { -145.931867485639f,    7.19449987970155f},
  { -167.300952966360f,    -55.2704201111625f},
  { -163.527789307240f,    -50.6511725865265f},
  { -160.238393260353f,    -45.5931971809751f},
  { -157.363308244864f,    -40.1797369172332f},
  { -154.840951725599f,    -34.5232572531018f},
  { -152.618348584113f,    -28.7575899095502f},
  { -150.650695296507f,    -23.0236551514198f},
  { -148.900436698442f,    -17.4525550414161f},
  { -147.336215633484f,    -12.1512282836016f},
  { -145.931867485639f,    -7.19449987970155f},
  { -158.250456398764f,    -55.3853099512686f},
  { -155.014228210582f,    -50.3244736173956f},
  { -152.252562009152f,    -44.8539773066727f},
  { -149.878604784755f,    -39.0760490721854f},
  { -147.822734287557f,    -33.1214693878282f},
  { -146.029404589826f,    -27.1374399585075f},
  { -144.454270054062f,    -21.2697401431116f},
  { -143.061792510098f,    -15.6447696005703f},
  { -141.823333921494f,    -10.3570763724901f},
  { -148.672843718087f,    -54.7710088310124f},
  { -146.193616221385f,    -49.3082490896532f},
  { -144.110245340416f,    -43.4870777455218f},
  { -142.339329827428f,    -37.4262981266073f},
  { -140.818219065418f,    -31.2700419814429f},
  { -139.499294987580f,    -25.1712621719368f},
  { -138.345914883777f,    -19.2717588609047f},
  { -137.329533687801f,    -13.6853671173645f},
  { -139.073571574690f,    -53.3421784101822f},
  { -137.470208624967f,    -47.5637193063793f},
  { -136.132253030241f,    -41.4939010991043f},
  { -135.0f,    -35.2643896827547f},
  { -134.030110218405f,    -29.0258428248200f},
  { -133.190450300436f,    -22.9282785675942f},
  { -132.456739125343f,    -17.1013346715150f},
  { -129.964753678737f,    -51.1183373464232f},
  { -129.222929357691f,    -45.1435105228856f},
  { -128.602952180105f,    -38.9522209765831f},
  { -128.077195298403f,    -32.6839097448066f},
  { -127.625768562624f,    -26.4879839228922f},
  { -127.233995684556f,    -20.5035260967303f},
  { -121.717474411461f,    -48.2221928049541f},
  { -121.717474411461f,    -42.1814142633759f},
  { -121.717474411461f,    -36.0f},
  { -121.717474411461f,    -29.8185857366241f},
  { -121.717474411461f,    -23.7778071950459f},
  { -114.502477634306f,    -44.8430231404270f},
  { -115.077437696012f,    -38.8577003880126f},
  { -115.567851058280f,    -32.8030205324522f},
  { -115.991030692267f,    -26.8136666500115f},
  { -108.323726945222f,    -41.1884885194893f},
  { -109.309078251673f,    -35.3584095083404f},
  { -110.161225333132f,    -29.5214139777455f},
  { -103.088026760372f,    -37.4462884164009f},
  { -104.346731310829f,    -31.8445438275894f},
  { -98.6643878136606f,    -33.7636228467164f}};

/* 3LD geosphere 'ico' [13 0] (1692 points) */
const float __geosphere_ico_13_0_dirs_deg[1692][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    62.3925997209885f},
  { 0.0f,    66.8353860746534f},
  { 0.0f,    71.5932006647639f},
  { 0.0f,    76.6286429612929f},
  { 0.0f,    81.8830323534313f},
  { 0.0f,    87.2781463566519f},
  { 180.0f,    87.2781463566519f},
  { 180.0f,    81.8830323534313f},
  { 180.0f,    76.6286429612929f},
  { 180.0f,    71.5932006647639f},
  { 180.0f,    66.8353860746534f},
  { 180.0f,    62.3925997209885f},
  { 7.67922857366964f,    59.3286053642054f},
  { 9.19099647608508f,    63.6106663265394f},
  { 11.4340697442537f,    68.2096620005233f},
  { 15.0920752881032f,    73.0708132363913f},
  { 22.0237505161462f,    78.0665504624893f},
  { 38.9734473368657f,    82.8183370989074f},
  { 90.0f,    85.4686859541220f},
  { 141.026552663134f,    82.8183370989074f},
  { 157.976249483854f,    78.0665504624893f},
  { 164.907924711897f,    73.0708132363913f},
  { 168.565930255746f,    68.2096620005233f},
  { 170.809003523915f,    63.6106663265394f},
  { 172.320771426330f,    59.3286053642054f},
  { 16.3932501484725f,    59.9229355250510f},
  { 19.7767131841238f,    64.2159562975888f},
  { 24.8108847601265f,    68.7182036256999f},
  { 32.9114931857064f,    73.2573914137460f},
  { 47.1679114977868f,    77.4351099417722f},
  { 72.8279621494862f,    80.2919692516987f},
  { 107.172037850514f,    80.2919692516987f},
  { 132.832088502213f,    77.4351099417722f},
  { 147.088506814294f,    73.2573914137460f},
  { 155.189115239873f,    68.7182036256999f},
  { 160.223286815876f,    64.2159562975888f},
  { 163.606749851527f,    59.9229355250510f},
  { 25.8924090881804f,    59.9058043457859f},
  { 31.2478160930689f,    63.9937346963090f},
  { 38.9734473368657f,    68.0814487319598f},
  { 50.5099309427673f,    71.8439998608869f},
  { 67.6072290710451f,    74.6921232020624f},
  { 90.0000000000000f,    75.7979510216779f},
  { 112.392770928955f,    74.6921232020624f},
  { 129.490069057233f,    71.8439998608869f},
  { 141.026552663134f,    68.0814487319598f},
  { 148.752183906931f,    63.9937346963090f},
  { 154.107590911820f,    59.9058043457859f},
  { 35.7209019991337f,    59.1541202718966f},
  { 42.7561895905164f,    62.8150947166279f},
  { 52.3123893000598f,    66.2217370325843f},
  { 65.1310748815380f,    68.9778919653611f},
  { 81.2167693990410f,    70.5667382332046f},
  { 98.7832306009590f,    70.5667382332046f},
  { 114.868925118462f,    68.9778919653611f},
  { 127.687610699940f,    66.2217370325843f},
  { 137.243810409484f,    62.8150947166278f},
  { 144.279098000866f,    59.1541202718966f},
  { 45.3210844354825f,    57.6230740536504f},
  { 53.4378607612508f,    60.6941105431031f},
  { 63.6909565444517f,    63.3000410294001f},
  { 76.1141128518623f,    65.0887998071003f},
  { 90.0f,    65.7317808244396f},
  { 103.885887148138f,    65.0887998071003f},
  { 116.309043455548f,    63.3000410294001f},
  { 126.562139238749f,    60.6941105431031f},
  { 134.678915564518f,    57.6230740536504f},
  { 54.2069014710865f,    55.3639291306766f},
  { 62.7502499362301f,    57.7781854400785f},
  { 72.8279621494862f,    59.6100403971132f},
  { 84.1189512527134f,    60.6081893164628f},
  { 95.8810487472866f,    60.6081893164628f},
  { 107.172037850514f,    59.6100403971132f},
  { 117.249750063770f,    57.7781854400785f},
  { 125.793098528914f,    55.3639291306766f},
  { 62.0878885994974f,    52.5072028198446f},
  { 70.5487009486867f,    54.2869960799927f},
  { 79.9858774849308f,    55.4575554335454f},
  { 90.0f,    55.8672503778391f},
  { 100.014122515069f,    55.4575554335454f},
  { 109.451299051313f,    54.2869960799927f},
  { 117.912111400503f,    52.5072028198446f},
  { 68.8798871556760f,    49.2250531726891f},
  { 76.9513489599765f,    50.4498800376610f},
  { 85.5824320004692f,    51.1006230243263f},
  { 94.4175679995308f,    51.1006230243263f},
  { 103.048651040024f,    50.4498800376610f},
  { 111.120112844324f,    49.2250531726891f},
  { 74.6407012762525f,    45.6942050250012f},
  { 82.1798710590254f,    46.4678741774347f},
  { 90.0f,    46.7351241499154f},
  { 97.8201289409746f,    46.4678741774347f},
  { 105.359298723748f,    45.6942050250012f},
  { 79.4960587445786f,    42.0709854335440f},
  { 86.4634243465556f,    42.4985838991928f},
  { 93.5365756534444f,    42.4985838991928f},
  { 100.503941255421f,    42.0709854335440f},
  { 83.5885791392606f,    38.4794203215505f},
  { 90.0f,    38.6546432459521f},
  { 96.4114208607394f,    38.4794203215505f},
  { 87.0517100501096f,    35.0088964967567f},
  { 92.9482899498904f,    35.0088964967567f},
  { -7.67922857366964f,    59.3286053642054f},
  { -16.3932501484725f,    59.9229355250510f},
  { -25.8924090881804f,    59.9058043457859f},
  { -35.7209019991337f,    59.1541202718966f},
  { -45.3210844354825f,    57.6230740536504f},
  { -54.2069014710865f,    55.3639291306766f},
  { -62.0878885994974f,    52.5072028198446f},
  { -68.8798871556760f,    49.2250531726891f},
  { -74.6407012762525f,    45.6942050250012f},
  { -79.4960587445786f,    42.0709854335440f},
  { -83.5885791392606f,    38.4794203215505f},
  { -87.0517100501096f,    35.0088964967567f},
  { -9.19099647608508f,    63.6106663265394f},
  { -19.7767131841238f,    64.2159562975888f},
  { -31.2478160930689f,    63.9937346963090f},
  { -42.7561895905164f,    62.8150947166279f},
  { -53.4378607612508f,    60.6941105431031f},
  { -62.7502499362301f,    57.7781854400785f},
  { -70.5487009486867f,    54.2869960799927f},
  { -76.9513489599765f,    50.4498800376610f},
  { -82.1798710590254f,    46.4678741774347f},
  { -86.4634243465556f,    42.4985838991928f},
  { -90.0f,    38.6546432459521f},
  { -92.9482899498904f,    35.0088964967567f},
  { -11.4340697442537f,    68.2096620005233f},
  { -24.8108847601265f,    68.7182036256999f},
  { -38.9734473368657f,    68.0814487319598f},
  { -52.3123893000598f,    66.2217370325843f},
  { -63.6909565444517f,    63.3000410294001f},
  { -72.8279621494862f,    59.6100403971132f},
  { -79.9858774849308f,    55.4575554335454f},
  { -85.5824320004692f,    51.1006230243263f},
  { -90.0f,    46.7351241499154f},
  { -93.5365756534444f,    42.4985838991928f},
  { -96.4114208607394f,    38.4794203215505f},
  { -15.0920752881032f,    73.0708132363913f},
  { -32.9114931857064f,    73.2573914137460f},
  { -50.5099309427673f,    71.8439998608869f},
  { -65.1310748815380f,    68.9778919653611f},
  { -76.1141128518623f,    65.0887998071003f},
  { -84.1189512527134f,    60.6081893164628f},
  { -90.0f,    55.8672503778391f},
  { -94.4175679995308f,    51.1006230243263f},
  { -97.8201289409746f,    46.4678741774347f},
  { -100.503941255421f,    42.0709854335440f},
  { -22.0237505161462f,    78.0665504624893f},
  { -47.1679114977868f,    77.4351099417722f},
  { -67.6072290710451f,    74.6921232020624f},
  { -81.2167693990410f,    70.5667382332046f},
  { -90.0f,    65.7317808244396f},
  { -95.8810487472866f,    60.6081893164627f},
  { -100.014122515069f,    55.4575554335454f},
  { -103.048651040024f,    50.4498800376610f},
  { -105.359298723748f,    45.6942050250012f},
  { -38.9734473368657f,    82.8183370989074f},
  { -72.8279621494862f,    80.2919692516987f},
  { -90.0f,    75.7979510216779f},
  { -98.7832306009590f,    70.5667382332046f},
  { -103.885887148138f,    65.0887998071003f},
  { -107.172037850514f,    59.6100403971132f},
  { -109.451299051313f,    54.2869960799927f},
  { -111.120112844324f,    49.2250531726891f},
  { -90.0000000000000f,    85.4686859541220f},
  { -107.172037850514f,    80.2919692516987f},
  { -112.392770928955f,    74.6921232020624f},
  { -114.868925118462f,    68.9778919653611f},
  { -116.309043455548f,    63.3000410294001f},
  { -117.249750063770f,    57.7781854400785f},
  { -117.912111400503f,    52.5072028198446f},
  { -141.026552663134f,    82.8183370989074f},
  { -132.832088502213f,    77.4351099417722f},
  { -129.490069057233f,    71.8439998608869f},
  { -127.687610699940f,    66.2217370325843f},
  { -126.562139238749f,    60.6941105431031f},
  { -125.793098528914f,    55.3639291306766f},
  { -157.976249483854f,    78.0665504624893f},
  { -147.088506814294f,    73.2573914137460f},
  { -141.026552663134f,    68.0814487319598f},
  { -137.243810409484f,    62.8150947166279f},
  { -134.678915564518f,    57.6230740536504f},
  { -164.907924711897f,    73.0708132363913f},
  { -155.189115239873f,    68.7182036256999f},
  { -148.752183906931f,    63.9937346963090f},
  { -144.279098000866f,    59.1541202718966f},
  { -168.565930255746f,    68.2096620005233f},
  { -160.223286815876f,    64.2159562975888f},
  { -154.107590911820f,    59.9058043457859f},
  { -170.809003523915f,    63.6106663265394f},
  { -163.606749851527f,    59.9229355250511f},
  { -172.320771426330f,    59.3286053642054f},
  { 7.67922857366964f,    -59.3286053642054f},
  { 16.3932501484725f,    -59.9229355250510f},
  { 25.8924090881804f,    -59.9058043457859f},
  { 35.7209019991337f,    -59.1541202718966f},
  { 45.3210844354825f,    -57.6230740536504f},
  { 54.2069014710865f,    -55.3639291306766f},
  { 62.0878885994974f,    -52.5072028198446f},
  { 68.8798871556760f,    -49.2250531726891f},
  { 74.6407012762525f,    -45.6942050250012f},
  { 79.4960587445786f,    -42.0709854335440f},
  { 83.5885791392606f,    -38.4794203215505f},
  { 87.0517100501096f,    -35.0088964967567f},
  { 0.0f,    -62.3925997209885f},
  { 9.19099647608508f,    -63.6106663265394f},
  { 19.7767131841238f,    -64.2159562975888f},
  { 31.2478160930689f,    -63.9937346963090f},
  { 42.7561895905164f,    -62.8150947166279f},
  { 53.4378607612508f,    -60.6941105431031f},
  { 62.7502499362301f,    -57.7781854400785f},
  { 70.5487009486867f,    -54.2869960799927f},
  { 76.9513489599765f,    -50.4498800376610f},
  { 82.1798710590254f,    -46.4678741774347f},
  { 86.4634243465556f,    -42.4985838991928f},
  { 90.0f,    -38.6546432459521f},
  { 92.9482899498904f,    -35.0088964967567f},
  { 0.0f,    -66.8353860746534f},
  { 11.4340697442537f,    -68.2096620005233f},
  { 24.8108847601265f,    -68.7182036256999f},
  { 38.9734473368657f,    -68.0814487319598f},
  { 52.3123893000598f,    -66.2217370325843f},
  { 63.6909565444517f,    -63.3000410294001f},
  { 72.8279621494862f,    -59.6100403971132f},
  { 79.9858774849308f,    -55.4575554335454f},
  { 85.5824320004692f,    -51.1006230243263f},
  { 90.0f,    -46.7351241499154f},
  { 93.5365756534444f,    -42.4985838991928f},
  { 96.4114208607394f,    -38.4794203215505f},
  { 0.0f,    -71.5932006647639f},
  { 15.0920752881032f,    -73.0708132363913f},
  { 32.9114931857064f,    -73.2573914137460f},
  { 50.5099309427673f,    -71.8439998608869f},
  { 65.1310748815380f,    -68.9778919653611f},
  { 76.1141128518623f,    -65.0887998071003f},
  { 84.1189512527134f,    -60.6081893164628f},
  { 90.0f,    -55.8672503778391f},
  { 94.4175679995308f,    -51.1006230243263f},
  { 97.8201289409746f,    -46.4678741774347f},
  { 100.503941255421f,    -42.0709854335440f},
  { 0.0f,    -76.6286429612929f},
  { 22.0237505161462f,    -78.0665504624893f},
  { 47.1679114977868f,    -77.4351099417722f},
  { 67.6072290710451f,    -74.6921232020624f},
  { 81.2167693990410f,    -70.5667382332046f},
  { 90.0f,    -65.7317808244396f},
  { 95.8810487472866f,    -60.6081893164627f},
  { 100.014122515069f,    -55.4575554335454f},
  { 103.048651040024f,    -50.4498800376610f},
  { 105.359298723748f,    -45.6942050250012f},
  { 0.0f,    -81.8830323534313f},
  { 38.9734473368657f,    -82.8183370989074f},
  { 72.8279621494862f,    -80.2919692516987f},
  { 90.0f,    -75.7979510216779f},
  { 98.7832306009590f,    -70.5667382332046f},
  { 103.885887148138f,    -65.0887998071003f},
  { 107.172037850514f,    -59.6100403971132f},
  { 109.451299051313f,    -54.2869960799927f},
  { 111.120112844324f,    -49.2250531726891f},
  { 0.0f,    -87.2781463566519f},
  { 90.0000000000000f,    -85.4686859541220f},
  { 107.172037850514f,    -80.2919692516987f},
  { 112.392770928955f,    -74.6921232020624f},
  { 114.868925118462f,    -68.9778919653611f},
  { 116.309043455548f,    -63.3000410294001f},
  { 117.249750063770f,    -57.7781854400785f},
  { 117.912111400503f,    -52.5072028198446f},
  { 180.0f,    -87.2781463566519f},
  { 141.026552663134f,    -82.8183370989074f},
  { 132.832088502213f,    -77.4351099417722f},
  { 129.490069057233f,    -71.8439998608869f},
  { 127.687610699940f,    -66.2217370325843f},
  { 126.562139238749f,    -60.6941105431031f},
  { 125.793098528914f,    -55.3639291306766f},
  { 180.0f,    -81.8830323534313f},
  { 157.976249483854f,    -78.0665504624893f},
  { 147.088506814294f,    -73.2573914137460f},
  { 141.026552663134f,    -68.0814487319598f},
  { 137.243810409484f,    -62.8150947166279f},
  { 134.678915564518f,    -57.6230740536504f},
  { 180.0f,    -76.6286429612929f},
  { 164.907924711897f,    -73.0708132363913f},
  { 155.189115239873f,    -68.7182036256999f},
  { 148.752183906931f,    -63.9937346963090f},
  { 144.279098000866f,    -59.1541202718966f},
  { 180.0f,    -71.5932006647639f},
  { 168.565930255746f,    -68.2096620005233f},
  { 160.223286815876f,    -64.2159562975888f},
  { 154.107590911820f,    -59.9058043457859f},
  { 180.0f,    -66.8353860746534f},
  { 170.809003523915f,    -63.6106663265394f},
  { 163.606749851527f,    -59.9229355250511f},
  { 180.0f,    -62.3925997209885f},
  { 172.320771426330f,    -59.3286053642054f},
  { -7.67922857366964f,    -59.3286053642054f},
  { -9.19099647608508f,    -63.6106663265394f},
  { -11.4340697442537f,    -68.2096620005233f},
  { -15.0920752881032f,    -73.0708132363913f},
  { -22.0237505161462f,    -78.0665504624893f},
  { -38.9734473368657f,    -82.8183370989074f},
  { -90.0f,    -85.4686859541220f},
  { -141.026552663134f,    -82.8183370989074f},
  { -157.976249483854f,    -78.0665504624893f},
  { -164.907924711897f,    -73.0708132363913f},
  { -168.565930255746f,    -68.2096620005233f},
  { -170.809003523915f,    -63.6106663265394f},
  { -172.320771426330f,    -59.3286053642054f},
  { -16.3932501484725f,    -59.9229355250510f},
  { -19.7767131841238f,    -64.2159562975888f},
  { -24.8108847601265f,    -68.7182036256999f},
  { -32.9114931857064f,    -73.2573914137460f},
  { -47.1679114977868f,    -77.4351099417722f},
  { -72.8279621494862f,    -80.2919692516987f},
  { -107.172037850514f,    -80.2919692516987f},
  { -132.832088502213f,    -77.4351099417722f},
  { -147.088506814294f,    -73.2573914137460f},
  { -155.189115239873f,    -68.7182036256999f},
  { -160.223286815876f,    -64.2159562975888f},
  { -163.606749851527f,    -59.9229355250510f},
  { -25.8924090881804f,    -59.9058043457859f},
  { -31.2478160930689f,    -63.9937346963090f},
  { -38.9734473368657f,    -68.0814487319598f},
  { -50.5099309427673f,    -71.8439998608869f},
  { -67.6072290710451f,    -74.6921232020624f},
  { -90.0000000000000f,    -75.7979510216779f},
  { -112.392770928955f,    -74.6921232020624f},
  { -129.490069057233f,    -71.8439998608869f},
  { -141.026552663134f,    -68.0814487319598f},
  { -148.752183906931f,    -63.9937346963090f},
  { -154.107590911820f,    -59.9058043457859f},
  { -35.7209019991337f,    -59.1541202718966f},
  { -42.7561895905164f,    -62.8150947166279f},
  { -52.3123893000598f,    -66.2217370325843f},
  { -65.1310748815380f,    -68.9778919653611f},
  { -81.2167693990410f,    -70.5667382332046f},
  { -98.7832306009590f,    -70.5667382332046f},
  { -114.868925118462f,    -68.9778919653611f},
  { -127.687610699940f,    -66.2217370325843f},
  { -137.243810409484f,    -62.8150947166278f},
  { -144.279098000866f,    -59.1541202718966f},
  { -45.3210844354825f,    -57.6230740536504f},
  { -53.4378607612508f,    -60.6941105431031f},
  { -63.6909565444517f,    -63.3000410294001f},
  { -76.1141128518623f,    -65.0887998071003f},
  { -90.0f,    -65.7317808244396f},
  { -103.885887148138f,    -65.0887998071003f},
  { -116.309043455548f,    -63.3000410294001f},
  { -126.562139238749f,    -60.6941105431031f},
  { -134.678915564518f,    -57.6230740536504f},
  { -54.2069014710865f,    -55.3639291306766f},
  { -62.7502499362301f,    -57.7781854400785f},
  { -72.8279621494862f,    -59.6100403971132f},
  { -84.1189512527134f,    -60.6081893164628f},
  { -95.8810487472866f,    -60.6081893164628f},
  { -107.172037850514f,    -59.6100403971132f},
  { -117.249750063770f,    -57.7781854400785f},
  { -125.793098528914f,    -55.3639291306766f},
  { -62.0878885994974f,    -52.5072028198446f},
  { -70.5487009486867f,    -54.2869960799927f},
  { -79.9858774849308f,    -55.4575554335454f},
  { -90.0f,    -55.8672503778391f},
  { -100.014122515069f,    -55.4575554335454f},
  { -109.451299051313f,    -54.2869960799927f},
  { -117.912111400503f,    -52.5072028198446f},
  { -68.8798871556760f,    -49.2250531726891f},
  { -76.9513489599765f,    -50.4498800376610f},
  { -85.5824320004692f,    -51.1006230243263f},
  { -94.4175679995308f,    -51.1006230243263f},
  { -103.048651040024f,    -50.4498800376610f},
  { -111.120112844324f,    -49.2250531726891f},
  { -74.6407012762525f,    -45.6942050250012f},
  { -82.1798710590254f,    -46.4678741774347f},
  { -90.0f,    -46.7351241499154f},
  { -97.8201289409746f,    -46.4678741774347f},
  { -105.359298723748f,    -45.6942050250012f},
  { -79.4960587445786f,    -42.0709854335440f},
  { -86.4634243465556f,    -42.4985838991928f},
  { -93.5365756534444f,    -42.4985838991928f},
  { -100.503941255421f,    -42.0709854335440f},
  { -83.5885791392606f,    -38.4794203215505f},
  { -90.0f,    -38.6546432459521f},
  { -96.4114208607394f,    -38.4794203215505f},
  { -87.0517100501096f,    -35.0088964967567f},
  { -92.9482899498904f,    -35.0088964967567f},
  { 27.6074002790115f,    0.0f},
  { 23.1646139253466f,    0.0f},
  { 18.4067993352361f,    0.0f},
  { 13.3713570387071f,    0.0f},
  { 8.11696764656871f,    0.0f},
  { 2.72185364334815f,    0.0f},
  { -2.72185364334815f,    0.0f},
  { -8.11696764656871f,    0.0f},
  { -13.3713570387071f,    0.0f},
  { -18.4067993352361f,    0.0f},
  { -23.1646139253466f,    0.0f},
  { -27.6074002790115f,    0.0f},
  { 30.4454176709642f,    3.90859206904998f},
  { 26.0957110621738f,    4.07103490692145f},
  { 21.3973172505811f,    4.22015361469322f},
  { 16.3770747556587f,    4.34818434409423f},
  { 11.0850283259250f,    4.44711536751566f},
  { 5.59486793001947f,    4.50981674511779f},
  { 0.0f,    4.53131404587798f},
  { -5.59486793001947f,    4.50981674511779f},
  { -11.0850283259250f,    4.44711536751566f},
  { -16.3770747556587f,    4.34818434409423f},
  { -21.3973172505811f,    4.22015361469322f},
  { -26.0957110621738f,    4.07103490692145f},
  { -30.4454176709642f,    3.90859206904998f},
  { 29.0566018665596f,    8.13134581637292f},
  { 24.4457275424006f,    8.46342216991022f},
  { 19.4717808726325f,    8.76052292495804f},
  { 14.1734548258462f,    9.00509325454033f},
  { 8.61634520322905f,    9.17999902746237f},
  { 2.89150666537607f,    9.27136259634132f},
  { -2.89150666537607f,    9.27136259634132f},
  { -8.61634520322905f,    9.17999902746237f},
  { -14.1734548258462f,    9.00509325454033f},
  { -19.4717808726325f,    8.76052292495804f},
  { -24.4457275424006f,    8.46342216991022f},
  { -29.0566018665596f,    8.13134581637292f},
  { 27.5360583904933f,    12.6481171283761f},
  { 22.6407123054497f,    13.1471022071415f},
  { 17.3707431006318f,    13.5788550623864f},
  { 11.7800146807428f,    13.9146750776789f},
  { 5.95291492000182f,    14.1285386836260f},
  { 4.10040015440350e-15f,    14.2020489783221f},
  { -5.95291492000182f,    14.1285386836260f},
  { -11.7800146807428f,    13.9146750776789f},
  { -17.3707431006318f,    13.5788550623864f},
  { -22.6407123054497f,    13.1471022071415f},
  { -27.5360583904933f,    12.6481171283761f},
  { 25.8665919900316f,    17.4186427615353f},
  { 20.6618767156354f,    18.0686071475266f},
  { 15.0755567325921f,    18.6068834781394f},
  { 9.18063632147644f,    18.9936942234650f},
  { 3.08369267728157f,    19.1963816375260f},
  { -3.08369267728157f,    19.1963816375260f},
  { -9.18063632147644f,    18.9936942234650f},
  { -15.0755567325921f,    18.6068834781394f},
  { -20.6618767156354f,    18.0686071475266f},
  { -25.8665919900316f,    17.4186427615353f},
  { 24.0284943196596f,    22.3809070640302f},
  { 18.4883139102092f,    23.1506132809357f},
  { 12.5664833805232f,    23.7518587925884f},
  { 6.35972088694213f,    24.1359682142136f},
  { 0.0f,    24.2682191755604f},
  { -6.35972088694213f,    24.1359682142136f},
  { -12.5664833805232f,    23.7518587925884f},
  { -18.4883139102092f,    23.1506132809357f},
  { -24.0284943196596f,    22.3809070640302f},
  { 21.9992764540863f,    27.4528018619471f},
  { 16.0970645866426f,    28.2957953815474f},
  { 9.82324349971454f,    28.9032250934550f},
  { 3.30321485537249f,    29.2220877594473f},
  { -3.30321485537249f,    29.2220877594473f},
  { -9.82324349971454f,    28.9032250934550f},
  { -16.0970645866426f,    28.2957953815474f},
  { -21.9992764540863f,    27.4528018619471f},
  { 19.7534683531809f,    32.5376748964660f},
  { 13.4634052440863f,    33.3950976385748f},
  { 6.82592270704169f,    33.9437031295283f},
  { 0.0f,    34.1327496221609f},
  { -6.82592270704169f,    33.9437031295283f},
  { -13.4634052440863f,    33.3950976385748f},
  { -19.7534683531809f,    32.5376748964660f},
  { 17.2625662187059f,    37.5330980040246f},
  { 10.5614849257909f,    38.3388616453546f},
  { 3.55634403816706f,    38.7621659989582f},
  { -3.55634403816706f,    38.7621659989582f},
  { -10.5614849257909f,    38.3388616453546f},
  { -17.2625662187059f,    37.5330980040246f},
  { 14.4952570253960f,    42.3409020031123f},
  { 7.36547512352164f,    43.0280070882578f},
  { 0.0f,    43.2648758500846f},
  { -7.36547512352164f,    43.0280070882578f},
  { -14.4952570253960f,    42.3409020031123f},
  { 11.4181151643826f,    46.8759833695976f},
  { 3.85142474386815f,    47.3824692147210f},
  { -3.85142474386815f,    47.3824692147210f},
  { -11.4181151643826f,    46.8759833695976f},
  { 7.99704436061702f,    51.0719496923957f},
  { 0.0f,    51.3453567540479f},
  { -7.99704436061702f,    51.0719496923957f},
  { 4.19980790267961f,    54.8829757396876f},
  { -4.19980790267961f,    54.8829757396876f},
  { 30.4454176709642f,    -3.90859206904998f},
  { 29.0566018665596f,    -8.13134581637292f},
  { 27.5360583904933f,    -12.6481171283761f},
  { 25.8665919900316f,    -17.4186427615353f},
  { 24.0284943196596f,    -22.3809070640302f},
  { 21.9992764540863f,    -27.4528018619471f},
  { 19.7534683531809f,    -32.5376748964660f},
  { 17.2625662187059f,    -37.5330980040246f},
  { 14.4952570253960f,    -42.3409020031123f},
  { 11.4181151643826f,    -46.8759833695976f},
  { 7.99704436061702f,    -51.0719496923957f},
  { 4.19980790267961f,    -54.8829757396876f},
  { 26.0957110621738f,    -4.07103490692145f},
  { 24.4457275424006f,    -8.46342216991022f},
  { 22.6407123054497f,    -13.1471022071415f},
  { 20.6618767156354f,    -18.0686071475266f},
  { 18.4883139102092f,    -23.1506132809357f},
  { 16.0970645866426f,    -28.2957953815474f},
  { 13.4634052440863f,    -33.3950976385748f},
  { 10.5614849257909f,    -38.3388616453546f},
  { 7.36547512352164f,    -43.0280070882578f},
  { 3.85142474386815f,    -47.3824692147210f},
  { 5.52449375491284e-15f,    -51.3453567540479f},
  { -4.19980790267961f,    -54.8829757396876f},
  { 21.3973172505811f,    -4.22015361469322f},
  { 19.4717808726325f,    -8.76052292495804f},
  { 17.3707431006318f,    -13.5788550623864f},
  { 15.0755567325921f,    -18.6068834781394f},
  { 12.5664833805232f,    -23.7518587925884f},
  { 9.82324349971454f,    -28.9032250934550f},
  { 6.82592270704169f,    -33.9437031295283f},
  { 3.55634403816706f,    -38.7621659989582f},
  { 0.0f,    -43.2648758500846f},
  { -3.85142474386816f,    -47.3824692147210f},
  { -7.99704436061703f,    -51.0719496923957f},
  { 16.3770747556587f,    -4.34818434409423f},
  { 14.1734548258462f,    -9.00509325454033f},
  { 11.7800146807428f,    -13.9146750776789f},
  { 9.18063632147644f,    -18.9936942234650f},
  { 6.35972088694213f,    -24.1359682142136f},
  { 3.30321485537249f,    -29.2220877594473f},
  { 0.0f,    -34.1327496221609f},
  { -3.55634403816706f,    -38.7621659989582f},
  { -7.36547512352164f,    -43.0280070882578f},
  { -11.4181151643826f,    -46.8759833695976f},
  { 11.0850283259250f,    -4.44711536751566f},
  { 8.61634520322905f,    -9.17999902746237f},
  { 5.95291492000182f,    -14.1285386836260f},
  { 3.08369267728157f,    -19.1963816375260f},
  { 2.19142545963059e-15f,    -24.2682191755604f},
  { -3.30321485537249f,    -29.2220877594473f},
  { -6.82592270704168f,    -33.9437031295283f},
  { -10.5614849257909f,    -38.3388616453546f},
  { -14.4952570253960f,    -42.3409020031123f},
  { 5.59486793001947f,    -4.50981674511779f},
  { 2.89150666537608f,    -9.27136259634132f},
  { 1.02510003860087e-15f,    -14.2020489783221f},
  { -3.08369267728157f,    -19.1963816375260f},
  { -6.35972088694213f,    -24.1359682142136f},
  { -9.82324349971453f,    -28.9032250934550f},
  { -13.4634052440863f,    -33.3950976385748f},
  { -17.2625662187059f,    -37.5330980040246f},
  { -9.63037537703222e-16f,    -4.53131404587798f},
  { -2.89150666537608f,    -9.27136259634132f},
  { -5.95291492000182f,    -14.1285386836260f},
  { -9.18063632147644f,    -18.9936942234650f},
  { -12.5664833805232f,    -23.7518587925884f},
  { -16.0970645866426f,    -28.2957953815474f},
  { -19.7534683531809f,    -32.5376748964660f},
  { -5.59486793001947f,    -4.50981674511779f},
  { -8.61634520322905f,    -9.17999902746237f},
  { -11.7800146807428f,    -13.9146750776789f},
  { -15.0755567325921f,    -18.6068834781394f},
  { -18.4883139102092f,    -23.1506132809357f},
  { -21.9992764540863f,    -27.4528018619471f},
  { -11.0850283259250f,    -4.44711536751566f},
  { -14.1734548258462f,    -9.00509325454033f},
  { -17.3707431006318f,    -13.5788550623864f},
  { -20.6618767156354f,    -18.0686071475266f},
  { -24.0284943196596f,    -22.3809070640302f},
  { -16.3770747556587f,    -4.34818434409423f},
  { -19.4717808726325f,    -8.76052292495803f},
  { -22.6407123054497f,    -13.1471022071415f},
  { -25.8665919900316f,    -17.4186427615353f},
  { -21.3973172505811f,    -4.22015361469322f},
  { -24.4457275424006f,    -8.46342216991022f},
  { -27.5360583904933f,    -12.6481171283761f},
  { -26.0957110621738f,    -4.07103490692145f},
  { -29.0566018665596f,    -8.13134581637292f},
  { -30.4454176709642f,    -3.90859206904998f},
  { -152.392599720989f,    0.0f},
  { -156.835386074653f,    0.0f},
  { -161.593200664764f,    0.0f},
  { -166.628642961293f,    0.0f},
  { -171.883032353431f,    0.0f},
  { -177.278146356652f,    0.0f},
  { 177.278146356652f,    0.0f},
  { 171.883032353431f,    0.0f},
  { 166.628642961293f,    0.0f},
  { 161.593200664764f,    0.0f},
  { 156.835386074653f,    0.0f},
  { 152.392599720989f,    0.0f},
  { -149.554582329036f,    3.90859206904998f},
  { -153.904288937826f,    4.07103490692145f},
  { -158.602682749419f,    4.22015361469322f},
  { -163.622925244341f,    4.34818434409423f},
  { -168.914971674075f,    4.44711536751566f},
  { -174.405132069981f,    4.50981674511779f},
  { 180.0f,    4.53131404587798f},
  { 174.405132069981f,    4.50981674511779f},
  { 168.914971674075f,    4.44711536751566f},
  { 163.622925244341f,    4.34818434409423f},
  { 158.602682749419f,    4.22015361469322f},
  { 153.904288937826f,    4.07103490692145f},
  { 149.554582329036f,    3.90859206904998f},
  { -150.943398133440f,    8.13134581637292f},
  { -155.554272457599f,    8.46342216991022f},
  { -160.528219127368f,    8.76052292495803f},
  { -165.826545174154f,    9.00509325454033f},
  { -171.383654796771f,    9.17999902746237f},
  { -177.108493334624f,    9.27136259634132f},
  { 177.108493334624f,    9.27136259634132f},
  { 171.383654796771f,    9.17999902746237f},
  { 165.826545174154f,    9.00509325454033f},
  { 160.528219127368f,    8.76052292495803f},
  { 155.554272457599f,    8.46342216991022f},
  { 150.943398133440f,    8.13134581637292f},
  { -152.463941609507f,    12.6481171283761f},
  { -157.359287694550f,    13.1471022071415f},
  { -162.629256899368f,    13.5788550623864f},
  { -168.219985319257f,    13.9146750776789f},
  { -174.047085079998f,    14.1285386836260f},
  { -180.0f,    14.2020489783221f},
  { 174.047085079998f,    14.1285386836260f},
  { 168.219985319257f,    13.9146750776789f},
  { 162.629256899368f,    13.5788550623864f},
  { 157.359287694550f,    13.1471022071415f},
  { 152.463941609507f,    12.6481171283761f},
  { -154.133408009968f,    17.4186427615353f},
  { -159.338123284365f,    18.0686071475266f},
  { -164.924443267408f,    18.6068834781394f},
  { -170.819363678524f,    18.9936942234650f},
  { -176.916307322718f,    19.1963816375260f},
  { 176.916307322718f,    19.1963816375260f},
  { 170.819363678524f,    18.9936942234650f},
  { 164.924443267408f,    18.6068834781394f},
  { 159.338123284365f,    18.0686071475266f},
  { 154.133408009968f,    17.4186427615353f},
  { -155.971505680340f,    22.3809070640302f},
  { -161.511686089791f,    23.1506132809357f},
  { -167.433516619477f,    23.7518587925884f},
  { -173.640279113058f,    24.1359682142136f},
  { 180.0f,    24.2682191755604f},
  { 173.640279113058f,    24.1359682142136f},
  { 167.433516619477f,    23.7518587925884f},
  { 161.511686089791f,    23.1506132809357f},
  { 155.971505680340f,    22.3809070640302f},
  { -158.000723545914f,    27.4528018619471f},
  { -163.902935413357f,    28.2957953815474f},
  { -170.176756500285f,    28.9032250934550f},
  { -176.696785144628f,    29.2220877594473f},
  { 176.696785144628f,    29.2220877594473f},
  { 170.176756500285f,    28.9032250934550f},
  { 163.902935413357f,    28.2957953815474f},
  { 158.000723545914f,    27.4528018619471f},
  { -160.246531646819f,    32.5376748964660f},
  { -166.536594755914f,    33.3950976385748f},
  { -173.174077292958f,    33.9437031295283f},
  { 180.0f,    34.1327496221609f},
  { 173.174077292958f,    33.9437031295283f},
  { 166.536594755914f,    33.3950976385748f},
  { 160.246531646819f,    32.5376748964660f},
  { -162.737433781294f,    37.5330980040246f},
  { -169.438515074209f,    38.3388616453546f},
  { -176.443655961833f,    38.7621659989582f},
  { 176.443655961833f,    38.7621659989582f},
  { 169.438515074209f,    38.3388616453546f},
  { 162.737433781294f,    37.5330980040246f},
  { -165.504742974604f,    42.3409020031123f},
  { -172.634524876478f,    43.0280070882578f},
  { 180.0f,    43.2648758500846f},
  { 172.634524876478f,    43.0280070882578f},
  { 165.504742974604f,    42.3409020031123f},
  { -168.581884835617f,    46.8759833695976f},
  { -176.148575256132f,    47.3824692147210f},
  { 176.148575256132f,    47.3824692147210f},
  { 168.581884835617f,    46.8759833695976f},
  { -172.002955639383f,    51.0719496923957f},
  { 180.0f,    51.3453567540479f},
  { 172.002955639383f,    51.0719496923957f},
  { -175.800192097320f,    54.8829757396876f},
  { 175.800192097320f,    54.8829757396876f},
  { -149.554582329036f,    -3.90859206904998f},
  { -150.943398133440f,    -8.13134581637292f},
  { -152.463941609507f,    -12.6481171283761f},
  { -154.133408009968f,    -17.4186427615353f},
  { -155.971505680340f,    -22.3809070640302f},
  { -158.000723545914f,    -27.4528018619471f},
  { -160.246531646819f,    -32.5376748964660f},
  { -162.737433781294f,    -37.5330980040246f},
  { -165.504742974604f,    -42.3409020031123f},
  { -168.581884835617f,    -46.8759833695976f},
  { -172.002955639383f,    -51.0719496923957f},
  { -175.800192097320f,    -54.8829757396876f},
  { -153.904288937826f,    -4.07103490692145f},
  { -155.554272457599f,    -8.46342216991022f},
  { -157.359287694550f,    -13.1471022071415f},
  { -159.338123284365f,    -18.0686071475266f},
  { -161.511686089791f,    -23.1506132809357f},
  { -163.902935413357f,    -28.2957953815474f},
  { -166.536594755914f,    -33.3950976385748f},
  { -169.438515074209f,    -38.3388616453546f},
  { -172.634524876478f,    -43.0280070882578f},
  { -176.148575256132f,    -47.3824692147210f},
  { -180.0f,    -51.3453567540479f},
  { 175.800192097320f,    -54.8829757396876f},
  { -158.602682749419f,    -4.22015361469322f},
  { -160.528219127368f,    -8.76052292495803f},
  { -162.629256899368f,    -13.5788550623864f},
  { -164.924443267408f,    -18.6068834781394f},
  { -167.433516619477f,    -23.7518587925884f},
  { -170.176756500285f,    -28.9032250934550f},
  { -173.174077292958f,    -33.9437031295283f},
  { -176.443655961833f,    -38.7621659989582f},
  { 180.0f,    -43.2648758500846f},
  { 176.148575256132f,    -47.3824692147210f},
  { 172.002955639383f,    -51.0719496923957f},
  { -163.622925244341f,    -4.34818434409423f},
  { -165.826545174154f,    -9.00509325454033f},
  { -168.219985319257f,    -13.9146750776789f},
  { -170.819363678524f,    -18.9936942234650f},
  { -173.640279113058f,    -24.1359682142136f},
  { -176.696785144628f,    -29.2220877594473f},
  { 180.0f,    -34.1327496221609f},
  { 176.443655961833f,    -38.7621659989582f},
  { 172.634524876478f,    -43.0280070882578f},
  { 168.581884835617f,    -46.8759833695976f},
  { -168.914971674075f,    -4.44711536751566f},
  { -171.383654796771f,    -9.17999902746237f},
  { -174.047085079998f,    -14.1285386836260f},
  { -176.916307322718f,    -19.1963816375260f},
  { -180.0f,    -24.2682191755604f},
  { 176.696785144628f,    -29.2220877594473f},
  { 173.174077292958f,    -33.9437031295283f},
  { 169.438515074209f,    -38.3388616453546f},
  { 165.504742974604f,    -42.3409020031123f},
  { -174.405132069981f,    -4.50981674511779f},
  { -177.108493334624f,    -9.27136259634132f},
  { -180.0f,    -14.2020489783221f},
  { 176.916307322718f,    -19.1963816375260f},
  { 173.640279113058f,    -24.1359682142136f},
  { 170.176756500285f,    -28.9032250934550f},
  { 166.536594755914f,    -33.3950976385748f},
  { 162.737433781294f,    -37.5330980040246f},
  { 180.0f,    -4.53131404587798f},
  { 177.108493334624f,    -9.27136259634132f},
  { 174.047085079998f,    -14.1285386836260f},
  { 170.819363678524f,    -18.9936942234650f},
  { 167.433516619477f,    -23.7518587925884f},
  { 163.902935413357f,    -28.2957953815474f},
  { 160.246531646819f,    -32.5376748964660f},
  { 174.405132069981f,    -4.50981674511779f},
  { 171.383654796771f,    -9.17999902746237f},
  { 168.219985319257f,    -13.9146750776789f},
  { 164.924443267408f,    -18.6068834781394f},
  { 161.511686089791f,    -23.1506132809357f},
  { 158.000723545914f,    -27.4528018619471f},
  { 168.914971674075f,    -4.44711536751566f},
  { 165.826545174154f,    -9.00509325454033f},
  { 162.629256899368f,    -13.5788550623864f},
  { 159.338123284365f,    -18.0686071475266f},
  { 155.971505680340f,    -22.3809070640302f},
  { 163.622925244341f,    -4.34818434409423f},
  { 160.528219127368f,    -8.76052292495803f},
  { 157.359287694550f,    -13.1471022071415f},
  { 154.133408009968f,    -17.4186427615353f},
  { 158.602682749419f,    -4.22015361469322f},
  { 155.554272457599f,    -8.46342216991022f},
  { 152.463941609507f,    -12.6481171283761f},
  { 153.904288937826f,    -4.07103490692145f},
  { 150.943398133440f,    -8.13134581637292f},
  { 149.554582329036f,    -3.90859206904998f},
  { 90.0f,    27.6074002790115f},
  { 90.0f,    23.1646139253466f},
  { 90.0f,    18.4067993352361f},
  { 90.0f,    13.3713570387071f},
  { 90.0f,    8.11696764656871f},
  { 90.0f,    2.72185364334815f},
  { 90.0f,    -2.72185364334815f},
  { 90.0f,    -8.11696764656871f},
  { 90.0f,    -13.3713570387071f},
  { 90.0f,    -18.4067993352361f},
  { 90.0f,    -23.1646139253466f},
  { 90.0f,    -27.6074002790115f},
  { 85.4686859541220f,    30.3671202772999f},
  { 85.4686859541220f,    26.0249224015083f},
  { 85.4686859541220f,    21.3364578171157f},
  { 85.4686859541220f,    16.3286159371973f},
  { 85.4686859541220f,    11.0512341613279f},
  { 85.4686859541220f,    5.57749051605622f},
  { 85.4686859541220f,    0.0f},
  { 85.4686859541220f,    -5.57749051605622f},
  { 85.4686859541220f,    -11.0512341613279f},
  { 85.4686859541220f,    -16.3286159371973f},
  { 85.4686859541220f,    -21.3364578171157f},
  { 85.4686859541220f,    -26.0249224015083f},
  { 85.4686859541220f,    -30.3671202772999f},
  { 80.7170241039594f,    28.7370537573102f},
  { 80.7170241039594f,    24.1624080353593f},
  { 80.7170241039594f,    19.2356163910667f},
  { 80.7170241039594f,    13.9951758196867f},
  { 80.7170241039594f,    8.50516395475853f},
  { 80.7170241039594f,    2.85370160571421f},
  { 80.7170241039594f,    -2.85370160571421f},
  { 80.7170241039594f,    -8.50516395475853f},
  { 80.7170241039594f,    -13.9951758196867f},
  { 80.7170241039594f,    -19.2356163910667f},
  { 80.7170241039594f,    -24.1624080353593f},
  { 80.7170241039594f,    -28.7370537573102f},
  { 75.7979510216779f,    26.8135138908583f},
  { 75.7979510216779f,    22.0157460533216f},
  { 75.7979510216779f,    16.8704233090811f},
  { 75.7979510216779f,    11.4295958417735f},
  { 75.7979510216779f,    5.77222118150410f},
  { 75.7979510216779f,    3.97507788892970e-15f},
  { 75.7979510216779f,    -5.77222118150410f},
  { 75.7979510216779f,    -11.4295958417735f},
  { 75.7979510216779f,    -16.8704233090811f},
  { 75.7979510216779f,    -22.0157460533216f},
  { 75.7979510216779f,    -26.8135138908583f},
  { 70.7778230000097f,    24.5993720079874f},
  { 70.7778230000097f,    19.6000088800500f},
  { 70.7778230000097f,    14.2703533897831f},
  { 70.7778230000097f,    8.67682044375011f},
  { 70.7778230000097f,    2.91207837763861f},
  { 70.7778230000097f,    -2.91207837763861f},
  { 70.7778230000097f,    -8.67682044375011f},
  { 70.7778230000097f,    -14.2703533897831f},
  { 70.7778230000097f,    -19.6000088800500f},
  { 70.7778230000097f,    -24.5993720079874f},
  { 65.7317808244396f,    22.1181924436406f},
  { 65.7317808244396f,    16.9523149239568f},
  { 65.7317808244396f,    11.4868833486947f},
  { 65.7317808244396f,    5.80173857670207f},
  { 65.7317808244396f,    0.0f},
  { 65.7317808244396f,    -5.80173857670207f},
  { 65.7317808244396f,    -11.4868833486947f},
  { 65.7317808244396f,    -16.9523149239568f},
  { 65.7317808244396f,    -22.1181924436406f},
  { 60.7373028213072f,    19.4152452833601f},
  { 60.7373028213072f,    14.1307548028182f},
  { 60.7373028213072f,    8.58970601608376f},
  { 60.7373028213072f,    2.88244673260874f},
  { 60.7373028213072f,    -2.88244673260873f},
  { 60.7373028213072f,    -8.58970601608376f},
  { 60.7373028213072f,    -14.1307548028182f},
  { 60.7373028213072f,    -19.4152452833601f},
  { 55.8672503778391f,    16.5543446861012f},
  { 55.8672503778391f,    11.2087294646755f},
  { 55.8672503778391f,    5.65850224279542f},
  { 55.8672503778391f,    0.0f},
  { 55.8672503778391f,    -5.65850224279542f},
  { 55.8672503778391f,    -11.2087294646755f},
  { 55.8672503778391f,    -16.5543446861012f},
  { 51.1839057293312f,    13.6107234706627f},
  { 51.1839057293312f,    8.26577083364685f},
  { 51.1839057293312f,    2.77236622585358f},
  { 51.1839057293312f,    -2.77236622585358f},
  { 51.1839057293312f,    -8.26577083364685f},
  { 51.1839057293312f,    -13.6107234706627f},
  { 46.7351241499154f,    10.6616720566457f},
  { 46.7351241499154f,    5.37738474595216f},
  { 46.7351241499154f,    0.0f},
  { 46.7351241499154f,    -5.37738474595216f},
  { 46.7351241499154f,    -10.6616720566457f},
  { 42.5529887259861f,    7.77749539255846f},
  { 42.5529887259861f,    2.60673989305160f},
  { 42.5529887259861f,    -2.60673989305160f},
  { 42.5529887259861f,    -7.77749539255846f},
  { 38.6546432459521f,    5.01499655085836f},
  { 38.6546432459521f,    0.0f},
  { 38.6546432459521f,    -5.01499655085836f},
  { 35.0445648098716f,    2.41448414154904f},
  { 35.0445648098716f,    -2.41448414154904f},
  { 94.5313140458780f,    30.3671202772999f},
  { 99.2829758960407f,    28.7370537573102f},
  { 104.202048978322f,    26.8135138908583f},
  { 109.222176999990f,    24.5993720079874f},
  { 114.268219175560f,    22.1181924436406f},
  { 119.262697178693f,    19.4152452833601f},
  { 124.132749622161f,    16.5543446861012f},
  { 128.816094270669f,    13.6107234706627f},
  { 133.264875850085f,    10.6616720566457f},
  { 137.447011274014f,    7.77749539255846f},
  { 141.345356754048f,    5.01499655085836f},
  { 144.955435190128f,    2.41448414154904f},
  { 94.5313140458780f,    26.0249224015083f},
  { 99.2829758960407f,    24.1624080353593f},
  { 104.202048978322f,    22.0157460533216f},
  { 109.222176999990f,    19.6000088800500f},
  { 114.268219175560f,    16.9523149239568f},
  { 119.262697178693f,    14.1307548028182f},
  { 124.132749622161f,    11.2087294646755f},
  { 128.816094270669f,    8.26577083364685f},
  { 133.264875850085f,    5.37738474595217f},
  { 137.447011274014f,    2.60673989305159f},
  { 141.345356754048f,    3.45073499193325e-15f},
  { 144.955435190128f,    -2.41448414154904f},
  { 94.5313140458780f,    21.3364578171157f},
  { 99.2829758960407f,    19.2356163910667f},
  { 104.202048978322f,    16.8704233090811f},
  { 109.222176999990f,    14.2703533897831f},
  { 114.268219175560f,    11.4868833486947f},
  { 119.262697178693f,    8.58970601608376f},
  { 124.132749622161f,    5.65850224279542f},
  { 128.816094270669f,    2.77236622585358f},
  { 133.264875850085f,    0.0f},
  { 137.447011274014f,    -2.60673989305160f},
  { 141.345356754048f,    -5.01499655085837f},
  { 94.5313140458780f,    16.3286159371973f},
  { 99.2829758960407f,    13.9951758196867f},
  { 104.202048978322f,    11.4295958417735f},
  { 109.222176999990f,    8.67682044375011f},
  { 114.268219175560f,    5.80173857670207f},
  { 119.262697178693f,    2.88244673260874f},
  { 124.132749622161f,    0.0f},
  { 128.816094270669f,    -2.77236622585358f},
  { 133.264875850085f,    -5.37738474595217f},
  { 137.447011274014f,    -7.77749539255846f},
  { 94.5313140458780f,    11.0512341613279f},
  { 99.2829758960407f,    8.50516395475853f},
  { 104.202048978322f,    5.77222118150410f},
  { 109.222176999990f,    2.91207837763861f},
  { 114.268219175560f,    1.99777224926825e-15f},
  { 119.262697178693f,    -2.88244673260873f},
  { 124.132749622161f,    -5.65850224279542f},
  { 128.816094270669f,    -8.26577083364685f},
  { 133.264875850085f,    -10.6616720566457f},
  { 94.5313140458780f,    5.57749051605622f},
  { 99.2829758960407f,    2.85370160571422f},
  { 104.202048978322f,    9.93769472232424e-16f},
  { 109.222176999990f,    -2.91207837763861f},
  { 114.268219175560f,    -5.80173857670207f},
  { 119.262697178693f,    -8.58970601608376f},
  { 124.132749622161f,    -11.2087294646755f},
  { 128.816094270669f,    -13.6107234706627f},
  { 94.5313140458780f,    -9.60027375456523e-16f},
  { 99.2829758960407f,    -2.85370160571422f},
  { 104.202048978322f,    -5.77222118150410f},
  { 109.222176999990f,    -8.67682044375011f},
  { 114.268219175560f,    -11.4868833486947f},
  { 119.262697178693f,    -14.1307548028182f},
  { 124.132749622161f,    -16.5543446861012f},
  { 94.5313140458780f,    -5.57749051605622f},
  { 99.2829758960407f,    -8.50516395475853f},
  { 104.202048978322f,    -11.4295958417735f},
  { 109.222176999990f,    -14.2703533897831f},
  { 114.268219175560f,    -16.9523149239568f},
  { 119.262697178693f,    -19.4152452833601f},
  { 94.5313140458780f,    -11.0512341613279f},
  { 99.2829758960407f,    -13.9951758196867f},
  { 104.202048978322f,    -16.8704233090811f},
  { 109.222176999990f,    -19.6000088800500f},
  { 114.268219175560f,    -22.1181924436406f},
  { 94.5313140458780f,    -16.3286159371973f},
  { 99.2829758960407f,    -19.2356163910667f},
  { 104.202048978322f,    -22.0157460533216f},
  { 109.222176999990f,    -24.5993720079874f},
  { 94.5313140458780f,    -21.3364578171157f},
  { 99.2829758960407f,    -24.1624080353593f},
  { 104.202048978322f,    -26.8135138908583f},
  { 94.5313140458780f,    -26.0249224015083f},
  { 99.2829758960407f,    -28.7370537573102f},
  { 94.5313140458780f,    -30.3671202772999f},
  { -85.4686859541220f,    30.3671202772999f},
  { -80.7170241039594f,    28.7370537573102f},
  { -75.7979510216779f,    26.8135138908583f},
  { -70.7778230000097f,    24.5993720079874f},
  { -65.7317808244396f,    22.1181924436406f},
  { -60.7373028213072f,    19.4152452833601f},
  { -55.8672503778391f,    16.5543446861012f},
  { -51.1839057293312f,    13.6107234706627f},
  { -46.7351241499154f,    10.6616720566457f},
  { -42.5529887259861f,    7.77749539255846f},
  { -38.6546432459521f,    5.01499655085836f},
  { -35.0445648098716f,    2.41448414154904f},
  { -90.0f,    27.6074002790115f},
  { -85.4686859541220f,    26.0249224015083f},
  { -80.7170241039594f,    24.1624080353593f},
  { -75.7979510216779f,    22.0157460533216f},
  { -70.7778230000097f,    19.6000088800500f},
  { -65.7317808244396f,    16.9523149239568f},
  { -60.7373028213072f,    14.1307548028182f},
  { -55.8672503778391f,    11.2087294646755f},
  { -51.1839057293312f,    8.26577083364685f},
  { -46.7351241499154f,    5.37738474595217f},
  { -42.5529887259861f,    2.60673989305159f},
  { -38.6546432459521f,    3.45073499193325e-15f},
  { -35.0445648098716f,    -2.41448414154904f},
  { -90.0f,    23.1646139253466f},
  { -85.4686859541220f,    21.3364578171157f},
  { -80.7170241039594f,    19.2356163910667f},
  { -75.7979510216779f,    16.8704233090811f},
  { -70.7778230000097f,    14.2703533897831f},
  { -65.7317808244396f,    11.4868833486947f},
  { -60.7373028213072f,    8.58970601608376f},
  { -55.8672503778391f,    5.65850224279542f},
  { -51.1839057293312f,    2.77236622585358f},
  { -46.7351241499154f,    0.0f},
  { -42.5529887259861f,    -2.60673989305160f},
  { -38.6546432459521f,    -5.01499655085837f},
  { -90.0f,    18.4067993352361f},
  { -85.4686859541220f,    16.3286159371973f},
  { -80.7170241039594f,    13.9951758196867f},
  { -75.7979510216779f,    11.4295958417735f},
  { -70.7778230000098f,    8.67682044375011f},
  { -65.7317808244396f,    5.80173857670207f},
  { -60.7373028213072f,    2.88244673260874f},
  { -55.8672503778391f,    0.0f},
  { -51.1839057293312f,    -2.77236622585358f},
  { -46.7351241499154f,    -5.37738474595217f},
  { -42.5529887259861f,    -7.77749539255846f},
  { -90.0f,    13.3713570387071f},
  { -85.4686859541220f,    11.0512341613279f},
  { -80.7170241039594f,    8.50516395475853f},
  { -75.7979510216779f,    5.77222118150410f},
  { -70.7778230000097f,    2.91207837763861f},
  { -65.7317808244396f,    1.99777224926825e-15f},
  { -60.7373028213072f,    -2.88244673260873f},
  { -55.8672503778391f,    -5.65850224279542f},
  { -51.1839057293312f,    -8.26577083364685f},
  { -46.7351241499154f,    -10.6616720566457f},
  { -90.0f,    8.11696764656871f},
  { -85.4686859541220f,    5.57749051605622f},
  { -80.7170241039594f,    2.85370160571422f},
  { -75.7979510216779f,    9.93769472232424e-16f},
  { -70.7778230000097f,    -2.91207837763861f},
  { -65.7317808244396f,    -5.80173857670207f},
  { -60.7373028213072f,    -8.58970601608376f},
  { -55.8672503778391f,    -11.2087294646755f},
  { -51.1839057293312f,    -13.6107234706627f},
  { -90.0f,    2.72185364334815f},
  { -85.4686859541220f,    -9.60027375456523e-16f},
  { -80.7170241039594f,    -2.85370160571422f},
  { -75.7979510216779f,    -5.77222118150410f},
  { -70.7778230000097f,    -8.67682044375011f},
  { -65.7317808244396f,    -11.4868833486947f},
  { -60.7373028213072f,    -14.1307548028182f},
  { -55.8672503778391f,    -16.5543446861012f},
  { -90.0f,    -2.72185364334815f},
  { -85.4686859541220f,    -5.57749051605622f},
  { -80.7170241039594f,    -8.50516395475853f},
  { -75.7979510216779f,    -11.4295958417735f},
  { -70.7778230000097f,    -14.2703533897831f},
  { -65.7317808244396f,    -16.9523149239568f},
  { -60.7373028213072f,    -19.4152452833601f},
  { -90.0f,    -8.11696764656871f},
  { -85.4686859541220f,    -11.0512341613279f},
  { -80.7170241039594f,    -13.9951758196867f},
  { -75.7979510216779f,    -16.8704233090811f},
  { -70.7778230000098f,    -19.6000088800500f},
  { -65.7317808244396f,    -22.1181924436406f},
  { -90.0f,    -13.3713570387071f},
  { -85.4686859541220f,    -16.3286159371973f},
  { -80.7170241039594f,    -19.2356163910667f},
  { -75.7979510216779f,    -22.0157460533216f},
  { -70.7778230000098f,    -24.5993720079874f},
  { -90.0f,    -18.4067993352361f},
  { -85.4686859541220f,    -21.3364578171157f},
  { -80.7170241039594f,    -24.1624080353593f},
  { -75.7979510216779f,    -26.8135138908583f},
  { -90.0f,    -23.1646139253466f},
  { -85.4686859541220f,    -26.0249224015083f},
  { -80.7170241039594f,    -28.7370537573102f},
  { -90.0f,    -27.6074002790115f},
  { -85.4686859541220f,    -30.3671202772999f},
  { -94.5313140458780f,    30.3671202772999f},
  { -94.5313140458780f,    26.0249224015083f},
  { -94.5313140458780f,    21.3364578171157f},
  { -94.5313140458780f,    16.3286159371973f},
  { -94.5313140458780f,    11.0512341613279f},
  { -94.5313140458780f,    5.57749051605622f},
  { -94.5313140458780f,    0.0f},
  { -94.5313140458780f,    -5.57749051605622f},
  { -94.5313140458780f,    -11.0512341613279f},
  { -94.5313140458780f,    -16.3286159371973f},
  { -94.5313140458780f,    -21.3364578171157f},
  { -94.5313140458780f,    -26.0249224015083f},
  { -94.5313140458780f,    -30.3671202772999f},
  { -99.2829758960407f,    28.7370537573102f},
  { -99.2829758960407f,    24.1624080353593f},
  { -99.2829758960407f,    19.2356163910667f},
  { -99.2829758960407f,    13.9951758196867f},
  { -99.2829758960407f,    8.50516395475853f},
  { -99.2829758960407f,    2.85370160571421f},
  { -99.2829758960407f,    -2.85370160571421f},
  { -99.2829758960407f,    -8.50516395475853f},
  { -99.2829758960407f,    -13.9951758196867f},
  { -99.2829758960407f,    -19.2356163910667f},
  { -99.2829758960407f,    -24.1624080353593f},
  { -99.2829758960407f,    -28.7370537573102f},
  { -104.202048978322f,    26.8135138908583f},
  { -104.202048978322f,    22.0157460533216f},
  { -104.202048978322f,    16.8704233090811f},
  { -104.202048978322f,    11.4295958417735f},
  { -104.202048978322f,    5.77222118150410f},
  { -104.202048978322f,    3.97507788892970e-15f},
  { -104.202048978322f,    -5.77222118150410f},
  { -104.202048978322f,    -11.4295958417735f},
  { -104.202048978322f,    -16.8704233090811f},
  { -104.202048978322f,    -22.0157460533216f},
  { -104.202048978322f,    -26.8135138908583f},
  { -109.222176999990f,    24.5993720079874f},
  { -109.222176999990f,    19.6000088800500f},
  { -109.222176999990f,    14.2703533897831f},
  { -109.222176999990f,    8.67682044375011f},
  { -109.222176999990f,    2.91207837763861f},
  { -109.222176999990f,    -2.91207837763861f},
  { -109.222176999990f,    -8.67682044375011f},
  { -109.222176999990f,    -14.2703533897831f},
  { -109.222176999990f,    -19.6000088800500f},
  { -109.222176999990f,    -24.5993720079874f},
  { -114.268219175560f,    22.1181924436406f},
  { -114.268219175560f,    16.9523149239568f},
  { -114.268219175560f,    11.4868833486947f},
  { -114.268219175560f,    5.80173857670207f},
  { -114.268219175560f,    0.0f},
  { -114.268219175560f,    -5.80173857670207f},
  { -114.268219175560f,    -11.4868833486947f},
  { -114.268219175560f,    -16.9523149239568f},
  { -114.268219175560f,    -22.1181924436406f},
  { -119.262697178693f,    19.4152452833601f},
  { -119.262697178693f,    14.1307548028182f},
  { -119.262697178693f,    8.58970601608376f},
  { -119.262697178693f,    2.88244673260874f},
  { -119.262697178693f,    -2.88244673260873f},
  { -119.262697178693f,    -8.58970601608376f},
  { -119.262697178693f,    -14.1307548028182f},
  { -119.262697178693f,    -19.4152452833601f},
  { -124.132749622161f,    16.5543446861012f},
  { -124.132749622161f,    11.2087294646755f},
  { -124.132749622161f,    5.65850224279542f},
  { -124.132749622161f,    0.0f},
  { -124.132749622161f,    -5.65850224279542f},
  { -124.132749622161f,    -11.2087294646755f},
  { -124.132749622161f,    -16.5543446861012f},
  { -128.816094270669f,    13.6107234706627f},
  { -128.816094270669f,    8.26577083364685f},
  { -128.816094270669f,    2.77236622585358f},
  { -128.816094270669f,    -2.77236622585358f},
  { -128.816094270669f,    -8.26577083364685f},
  { -128.816094270669f,    -13.6107234706627f},
  { -133.264875850085f,    10.6616720566457f},
  { -133.264875850085f,    5.37738474595216f},
  { -133.264875850085f,    0.0f},
  { -133.264875850085f,    -5.37738474595216f},
  { -133.264875850085f,    -10.6616720566457f},
  { -137.447011274014f,    7.77749539255846f},
  { -137.447011274014f,    2.60673989305160f},
  { -137.447011274014f,    -2.60673989305160f},
  { -137.447011274014f,    -7.77749539255846f},
  { -141.345356754048f,    5.01499655085836f},
  { -141.345356754048f,    0.0f},
  { -141.345356754048f,    -5.01499655085836f},
  { -144.955435190128f,    2.41448414154904f},
  { -144.955435190128f,    -2.41448414154904f},
  { 82.0714620672413f,    33.6165047722674f},
  { 76.8839456727258f,    31.8970076530459f},
  { 71.5650511770780f,    29.8371409870851f},
  { 66.2033358867346f,    27.4459913473483f},
  { 60.8916879482650f,    24.7589480673360f},
  { 55.7179416832170f,    21.8369553839562f},
  { 50.7567131910110f,    18.7602981099717f},
  { 46.0642057222903f,    15.6181902414563f},
  { 41.6765797484498f,    12.4971076405263f},
  { 37.6113554237721f,    9.47109376781112f},
  { 33.8707002168329f,    6.59609223407253f},
  { 78.1077643231523f,    37.0120512651164f},
  { 72.4559804143852f,    35.1618531958101f},
  { 66.7381636782945f,    32.9199372930422f},
  { 61.0667791149539f,    30.3070063208102f},
  { 55.5489714704350f,    27.3758768211568f},
  { 50.2751680301179f,    24.2073506421853f},
  { 45.3119315754831f,    20.8997946068491f},
  { 40.7000005891269f,    17.5553136605234f},
  { 36.4566226416715f,    14.2666249148246f},
  { 32.5803608147442f,    11.1079714409386f},
  { 73.4720159159454f,    40.4791269870344f},
  { 67.3497995833542f,    38.4424112745958f},
  { 61.2670373251352f,    35.9630277751952f},
  { 55.3561859774543f,    33.0807088681832f},
  { 49.7282755595884f,    29.8716190222766f},
  { 44.4628625903442f,    26.4390750029195f},
  { 39.6057152432606f,    22.8984738947884f},
  { 35.1726881738394f,    19.3610274237451f},
  { 31.1568325566855f,    15.9208749961433f},
  { 68.0558945017039f,    43.9101112683018f},
  { 61.4983027419833f,    41.6186650159532f},
  { 55.1341740234119f,    38.8428070417901f},
  { 49.1018892670626f,    35.6494412835039f},
  { 43.4974890922038f,    32.1421016882594f},
  { 38.3720427649533f,    28.4456554391894f},
  { 33.7381178311653f,    24.6877109680317f},
  { 29.5804340103015f,    20.9821608982710f},
  { 61.7683715890198f,    47.1655463617031f},
  { 54.8757528580636f,    44.5468124755062f},
  { 48.3775070406968f,    41.4235128973441f},
  { 42.3908376402741f,    37.8954593557137f},
  { 36.9719365628432f,    34.0926768215121f},
  { 32.1269097703806f,    30.1554764761311f},
  { 27.8279621494862f,    26.2153012260126f},
  { 54.5711796517762f,    50.0840260637785f},
  { 47.5304777963547f,    47.0762912260020f},
  { 41.1104877380252f,    43.5782038962524f},
  { 35.3713475459593f,    39.7215413185100f},
  { 30.3073880426967f,    35.6550166885512f},
  { 25.8720714079013f,    31.5236664757459f},
  { 46.5272576755544f,    52.5033623066590f},
  { 39.6137022171015f,    49.0757348734261f},
  { 33.5270241110253f,    45.2133553579987f},
  { 28.2408519647374f,    41.0697333442294f},
  { 23.6806178175289f,    36.7986876565218f},
  { 37.8432080152807f,    54.2915904436854f},
  { 31.3835645530307f,    50.4611561555181f},
  { 25.8799832093179f,    46.2882241477093f},
  { 21.2160429905407f,    41.9308212597237f},
  { 28.8695199818914f,    55.3790614040513f},
  { 23.1670893038819f,    51.2149727211204f},
  { 18.4349488229220f,    46.8204952158267f},
  { 20.0324335759375f,    55.7766002623725f},
  { 15.2881409375360f,    51.3862242437938f},
  { 11.7216151928660f,    55.5683540408088f},
  { 82.0714620672413f,    -33.6165047722674f},
  { 78.1077643231523f,    -37.0120512651164f},
  { 73.4720159159454f,    -40.4791269870344f},
  { 68.0558945017039f,    -43.9101112683018f},
  { 61.7683715890198f,    -47.1655463617031f},
  { 54.5711796517762f,    -50.0840260637785f},
  { 46.5272576755543f,    -52.5033623066590f},
  { 37.8432080152807f,    -54.2915904436854f},
  { 28.8695199818914f,    -55.3790614040513f},
  { 20.0324335759375f,    -55.7766002623725f},
  { 11.7216151928660f,    -55.5683540408088f},
  { 76.8839456727258f,    -31.8970076530459f},
  { 72.4559804143852f,    -35.1618531958101f},
  { 67.3497995833542f,    -38.4424112745958f},
  { 61.4983027419833f,    -41.6186650159532f},
  { 54.8757528580636f,    -44.5468124755062f},
  { 47.5304777963547f,    -47.0762912260020f},
  { 39.6137022171015f,    -49.0757348734261f},
  { 31.3835645530307f,    -50.4611561555181f},
  { 23.1670893038819f,    -51.2149727211204f},
  { 15.2881409375360f,    -51.3862242437938f},
  { 71.5650511770780f,    -29.8371409870851f},
  { 66.7381636782945f,    -32.9199372930422f},
  { 61.2670373251352f,    -35.9630277751952f},
  { 55.1341740234119f,    -38.8428070417901f},
  { 48.3775070406968f,    -41.4235128973441f},
  { 41.1104877380252f,    -43.5782038962524f},
  { 33.5270241110253f,    -45.2133553579987f},
  { 25.8799832093179f,    -46.2882241477093f},
  { 18.4349488229220f,    -46.8204952158267f},
  { 66.2033358867346f,    -27.4459913473483f},
  { 61.0667791149539f,    -30.3070063208102f},
  { 55.3561859774543f,    -33.0807088681832f},
  { 49.1018892670626f,    -35.6494412835039f},
  { 42.3908376402741f,    -37.8954593557137f},
  { 35.3713475459593f,    -39.7215413185100f},
  { 28.2408519647374f,    -41.0697333442295f},
  { 21.2160429905407f,    -41.9308212597237f},
  { 60.8916879482649f,    -24.7589480673360f},
  { 55.5489714704350f,    -27.3758768211569f},
  { 49.7282755595884f,    -29.8716190222766f},
  { 43.4974890922038f,    -32.1421016882595f},
  { 36.9719365628432f,    -34.0926768215121f},
  { 30.3073880426967f,    -35.6550166885512f},
  { 23.6806178175289f,    -36.7986876565218f},
  { 55.7179416832170f,    -21.8369553839562f},
  { 50.2751680301179f,    -24.2073506421853f},
  { 44.4628625903442f,    -26.4390750029195f},
  { 38.3720427649533f,    -28.4456554391894f},
  { 32.1269097703806f,    -30.1554764761311f},
  { 25.8720714079013f,    -31.5236664757459f},
  { 50.7567131910110f,    -18.7602981099717f},
  { 45.3119315754831f,    -20.8997946068491f},
  { 39.6057152432605f,    -22.8984738947884f},
  { 33.7381178311653f,    -24.6877109680317f},
  { 27.8279621494862f,    -26.2153012260126f},
  { 46.0642057222903f,    -15.6181902414563f},
  { 40.7000005891269f,    -17.5553136605234f},
  { 35.1726881738394f,    -19.3610274237451f},
  { 29.5804340103015f,    -20.9821608982710f},
  { 41.6765797484498f,    -12.4971076405263f},
  { 36.4566226416715f,    -14.2666249148246f},
  { 31.1568325566855f,    -15.9208749961433f},
  { 37.6113554237721f,    -9.47109376781113f},
  { 32.5803608147442f,    -11.1079714409386f},
  { 33.8707002168329f,    -6.59609223407253f},
  { -11.7216151928660f,    55.5683540408088f},
  { -15.2881409375360f,    51.3862242437938f},
  { -18.4349488229220f,    46.8204952158267f},
  { -21.2160429905407f,    41.9308212597237f},
  { -23.6806178175289f,    36.7986876565218f},
  { -25.8720714079013f,    31.5236664757459f},
  { -27.8279621494862f,    26.2153012260126f},
  { -29.5804340103015f,    20.9821608982710f},
  { -31.1568325566855f,    15.9208749961434f},
  { -32.5803608147442f,    11.1079714409386f},
  { -33.8707002168329f,    6.59609223407253f},
  { -20.0324335759375f,    55.7766002623725f},
  { -23.1670893038819f,    51.2149727211204f},
  { -25.8799832093179f,    46.2882241477093f},
  { -28.2408519647374f,    41.0697333442295f},
  { -30.3073880426967f,    35.6550166885512f},
  { -32.1269097703806f,    30.1554764761311f},
  { -33.7381178311653f,    24.6877109680317f},
  { -35.1726881738393f,    19.3610274237451f},
  { -36.4566226416715f,    14.2666249148246f},
  { -37.6113554237721f,    9.47109376781112f},
  { -28.8695199818914f,    55.3790614040513f},
  { -31.3835645530307f,    50.4611561555181f},
  { -33.5270241110253f,    45.2133553579987f},
  { -35.3713475459593f,    39.7215413185100f},
  { -36.9719365628432f,    34.0926768215121f},
  { -38.3720427649533f,    28.4456554391894f},
  { -39.6057152432606f,    22.8984738947884f},
  { -40.7000005891269f,    17.5553136605234f},
  { -41.6765797484498f,    12.4971076405263f},
  { -37.8432080152807f,    54.2915904436854f},
  { -39.6137022171015f,    49.0757348734261f},
  { -41.1104877380252f,    43.5782038962524f},
  { -42.3908376402741f,    37.8954593557137f},
  { -43.4974890922038f,    32.1421016882595f},
  { -44.4628625903441f,    26.4390750029195f},
  { -45.3119315754831f,    20.8997946068491f},
  { -46.0642057222903f,    15.6181902414563f},
  { -46.5272576755543f,    52.5033623066590f},
  { -47.5304777963547f,    47.0762912260020f},
  { -48.3775070406968f,    41.4235128973441f},
  { -49.1018892670626f,    35.6494412835039f},
  { -49.7282755595884f,    29.8716190222766f},
  { -50.2751680301179f,    24.2073506421853f},
  { -50.7567131910110f,    18.7602981099717f},
  { -54.5711796517762f,    50.0840260637785f},
  { -54.8757528580636f,    44.5468124755062f},
  { -55.1341740234119f,    38.8428070417901f},
  { -55.3561859774543f,    33.0807088681832f},
  { -55.5489714704350f,    27.3758768211569f},
  { -55.7179416832170f,    21.8369553839562f},
  { -61.7683715890198f,    47.1655463617031f},
  { -61.4983027419833f,    41.6186650159532f},
  { -61.2670373251352f,    35.9630277751952f},
  { -61.0667791149539f,    30.3070063208102f},
  { -60.8916879482649f,    24.7589480673360f},
  { -68.0558945017039f,    43.9101112683018f},
  { -67.3497995833542f,    38.4424112745958f},
  { -66.7381636782945f,    32.9199372930422f},
  { -66.2033358867346f,    27.4459913473483f},
  { -73.4720159159454f,    40.4791269870344f},
  { -72.4559804143852f,    35.1618531958101f},
  { -71.5650511770780f,    29.8371409870851f},
  { -78.1077643231523f,    37.0120512651164f},
  { -76.8839456727258f,    31.8970076530459f},
  { -82.0714620672413f,    33.6165047722674f},
  { -11.7216151928660f,    -55.5683540408088f},
  { -20.0324335759375f,    -55.7766002623725f},
  { -28.8695199818914f,    -55.3790614040513f},
  { -37.8432080152807f,    -54.2915904436854f},
  { -46.5272576755543f,    -52.5033623066590f},
  { -54.5711796517762f,    -50.0840260637785f},
  { -61.7683715890198f,    -47.1655463617031f},
  { -68.0558945017039f,    -43.9101112683018f},
  { -73.4720159159454f,    -40.4791269870344f},
  { -78.1077643231523f,    -37.0120512651164f},
  { -82.0714620672413f,    -33.6165047722674f},
  { -15.2881409375360f,    -51.3862242437938f},
  { -23.1670893038819f,    -51.2149727211204f},
  { -31.3835645530307f,    -50.4611561555181f},
  { -39.6137022171015f,    -49.0757348734261f},
  { -47.5304777963547f,    -47.0762912260020f},
  { -54.8757528580636f,    -44.5468124755062f},
  { -61.4983027419833f,    -41.6186650159532f},
  { -67.3497995833542f,    -38.4424112745958f},
  { -72.4559804143852f,    -35.1618531958101f},
  { -76.8839456727258f,    -31.8970076530459f},
  { -18.4349488229220f,    -46.8204952158267f},
  { -25.8799832093179f,    -46.2882241477093f},
  { -33.5270241110253f,    -45.2133553579987f},
  { -41.1104877380252f,    -43.5782038962524f},
  { -48.3775070406968f,    -41.4235128973441f},
  { -55.1341740234119f,    -38.8428070417901f},
  { -61.2670373251352f,    -35.9630277751952f},
  { -66.7381636782945f,    -32.9199372930422f},
  { -71.5650511770780f,    -29.8371409870851f},
  { -21.2160429905407f,    -41.9308212597237f},
  { -28.2408519647374f,    -41.0697333442295f},
  { -35.3713475459593f,    -39.7215413185100f},
  { -42.3908376402741f,    -37.8954593557137f},
  { -49.1018892670626f,    -35.6494412835039f},
  { -55.3561859774543f,    -33.0807088681832f},
  { -61.0667791149539f,    -30.3070063208102f},
  { -66.2033358867346f,    -27.4459913473483f},
  { -23.6806178175289f,    -36.7986876565218f},
  { -30.3073880426967f,    -35.6550166885512f},
  { -36.9719365628432f,    -34.0926768215121f},
  { -43.4974890922038f,    -32.1421016882595f},
  { -49.7282755595884f,    -29.8716190222766f},
  { -55.5489714704350f,    -27.3758768211569f},
  { -60.8916879482649f,    -24.7589480673360f},
  { -25.8720714079013f,    -31.5236664757459f},
  { -32.1269097703806f,    -30.1554764761311f},
  { -38.3720427649533f,    -28.4456554391894f},
  { -44.4628625903442f,    -26.4390750029195f},
  { -50.2751680301179f,    -24.2073506421853f},
  { -55.7179416832170f,    -21.8369553839562f},
  { -27.8279621494862f,    -26.2153012260126f},
  { -33.7381178311653f,    -24.6877109680317f},
  { -39.6057152432606f,    -22.8984738947884f},
  { -45.3119315754831f,    -20.8997946068491f},
  { -50.7567131910110f,    -18.7602981099717f},
  { -29.5804340103015f,    -20.9821608982710f},
  { -35.1726881738394f,    -19.3610274237451f},
  { -40.7000005891269f,    -17.5553136605234f},
  { -46.0642057222903f,    -15.6181902414563f},
  { -31.1568325566855f,    -15.9208749961433f},
  { -36.4566226416715f,    -14.2666249148246f},
  { -41.6765797484498f,    -12.4971076405263f},
  { -32.5803608147442f,    -11.1079714409386f},
  { -37.6113554237721f,    -9.47109376781113f},
  { -33.8707002168329f,    -6.59609223407253f},
  { 168.278384807134f,    55.5683540408088f},
  { 164.711859062464f,    51.3862242437938f},
  { 161.565051177078f,    46.8204952158267f},
  { 158.783957009459f,    41.9308212597237f},
  { 156.319382182471f,    36.7986876565218f},
  { 154.127928592099f,    31.5236664757459f},
  { 152.172037850514f,    26.2153012260126f},
  { 150.419565989699f,    20.9821608982710f},
  { 148.843167443314f,    15.9208749961434f},
  { 147.419639185256f,    11.1079714409386f},
  { 146.129299783167f,    6.59609223407253f},
  { 159.967566424063f,    55.7766002623725f},
  { 156.832910696118f,    51.2149727211204f},
  { 154.120016790682f,    46.2882241477093f},
  { 151.759148035263f,    41.0697333442294f},
  { 149.692611957303f,    35.6550166885512f},
  { 147.873090229619f,    30.1554764761311f},
  { 146.261882168835f,    24.6877109680317f},
  { 144.827311826161f,    19.3610274237451f},
  { 143.543377358329f,    14.2666249148246f},
  { 142.388644576228f,    9.47109376781112f},
  { 151.130480018109f,    55.3790614040513f},
  { 148.616435446969f,    50.4611561555181f},
  { 146.472975888975f,    45.2133553579987f},
  { 144.628652454041f,    39.7215413185100f},
  { 143.028063437157f,    34.0926768215121f},
  { 141.627957235047f,    28.4456554391894f},
  { 140.394284756739f,    22.8984738947884f},
  { 139.299999410873f,    17.5553136605234f},
  { 138.323420251550f,    12.4971076405263f},
  { 142.156791984719f,    54.2915904436854f},
  { 140.386297782898f,    49.0757348734261f},
  { 138.889512261975f,    43.5782038962524f},
  { 137.609162359726f,    37.8954593557137f},
  { 136.502510907796f,    32.1421016882594f},
  { 135.537137409656f,    26.4390750029195f},
  { 134.688068424517f,    20.8997946068491f},
  { 133.935794277710f,    15.6181902414563f},
  { 133.472742324446f,    52.5033623066590f},
  { 132.469522203645f,    47.0762912260020f},
  { 131.622492959303f,    41.4235128973441f},
  { 130.898110732937f,    35.6494412835039f},
  { 130.271724440412f,    29.8716190222766f},
  { 129.724831969882f,    24.2073506421853f},
  { 129.243286808989f,    18.7602981099717f},
  { 125.428820348224f,    50.0840260637785f},
  { 125.124247141936f,    44.5468124755062f},
  { 124.865825976588f,    38.8428070417901f},
  { 124.643814022546f,    33.0807088681832f},
  { 124.451028529565f,    27.3758768211569f},
  { 124.282058316783f,    21.8369553839562f},
  { 118.231628410980f,    47.1655463617031f},
  { 118.501697258017f,    41.6186650159532f},
  { 118.732962674865f,    35.9630277751952f},
  { 118.933220885046f,    30.3070063208102f},
  { 119.108312051735f,    24.7589480673360f},
  { 111.944105498296f,    43.9101112683018f},
  { 112.650200416646f,    38.4424112745958f},
  { 113.261836321706f,    32.9199372930422f},
  { 113.796664113265f,    27.4459913473483f},
  { 106.527984084055f,    40.4791269870344f},
  { 107.544019585615f,    35.1618531958101f},
  { 108.434948822922f,    29.8371409870851f},
  { 101.892235676848f,    37.0120512651164f},
  { 103.116054327274f,    31.8970076530459f},
  { 97.9285379327587f,    33.6165047722674f},
  { 168.278384807134f,    -55.5683540408088f},
  { 159.967566424063f,    -55.7766002623725f},
  { 151.130480018109f,    -55.3790614040513f},
  { 142.156791984719f,    -54.2915904436854f},
  { 133.472742324446f,    -52.5033623066590f},
  { 125.428820348224f,    -50.0840260637785f},
  { 118.231628410980f,    -47.1655463617031f},
  { 111.944105498296f,    -43.9101112683018f},
  { 106.527984084055f,    -40.4791269870344f},
  { 101.892235676848f,    -37.0120512651164f},
  { 97.9285379327587f,    -33.6165047722674f},
  { 164.711859062464f,    -51.3862242437938f},
  { 156.832910696118f,    -51.2149727211204f},
  { 148.616435446969f,    -50.4611561555181f},
  { 140.386297782898f,    -49.0757348734261f},
  { 132.469522203645f,    -47.0762912260020f},
  { 125.124247141936f,    -44.5468124755062f},
  { 118.501697258017f,    -41.6186650159532f},
  { 112.650200416646f,    -38.4424112745958f},
  { 107.544019585615f,    -35.1618531958101f},
  { 103.116054327274f,    -31.8970076530459f},
  { 161.565051177078f,    -46.8204952158267f},
  { 154.120016790682f,    -46.2882241477093f},
  { 146.472975888975f,    -45.2133553579987f},
  { 138.889512261975f,    -43.5782038962524f},
  { 131.622492959303f,    -41.4235128973441f},
  { 124.865825976588f,    -38.8428070417901f},
  { 118.732962674865f,    -35.9630277751952f},
  { 113.261836321706f,    -32.9199372930422f},
  { 108.434948822922f,    -29.8371409870851f},
  { 158.783957009459f,    -41.9308212597237f},
  { 151.759148035263f,    -41.0697333442294f},
  { 144.628652454041f,    -39.7215413185100f},
  { 137.609162359726f,    -37.8954593557137f},
  { 130.898110732937f,    -35.6494412835039f},
  { 124.643814022546f,    -33.0807088681832f},
  { 118.933220885046f,    -30.3070063208102f},
  { 113.796664113265f,    -27.4459913473483f},
  { 156.319382182471f,    -36.7986876565218f},
  { 149.692611957303f,    -35.6550166885512f},
  { 143.028063437157f,    -34.0926768215121f},
  { 136.502510907796f,    -32.1421016882594f},
  { 130.271724440412f,    -29.8716190222766f},
  { 124.451028529565f,    -27.3758768211569f},
  { 119.108312051735f,    -24.7589480673360f},
  { 154.127928592099f,    -31.5236664757459f},
  { 147.873090229619f,    -30.1554764761311f},
  { 141.627957235047f,    -28.4456554391894f},
  { 135.537137409656f,    -26.4390750029195f},
  { 129.724831969882f,    -24.2073506421853f},
  { 124.282058316783f,    -21.8369553839562f},
  { 152.172037850514f,    -26.2153012260126f},
  { 146.261882168835f,    -24.6877109680317f},
  { 140.394284756739f,    -22.8984738947884f},
  { 134.688068424517f,    -20.8997946068491f},
  { 129.243286808989f,    -18.7602981099717f},
  { 150.419565989699f,    -20.9821608982710f},
  { 144.827311826161f,    -19.3610274237451f},
  { 139.299999410873f,    -17.5553136605234f},
  { 133.935794277710f,    -15.6181902414563f},
  { 148.843167443314f,    -15.9208749961433f},
  { 143.543377358329f,    -14.2666249148246f},
  { 138.323420251550f,    -12.4971076405263f},
  { 147.419639185256f,    -11.1079714409386f},
  { 142.388644576228f,    -9.47109376781113f},
  { 146.129299783167f,    -6.59609223407253f},
  { -168.278384807134f,    55.5683540408088f},
  { -159.967566424063f,    55.7766002623725f},
  { -151.130480018109f,    55.3790614040513f},
  { -142.156791984719f,    54.2915904436854f},
  { -133.472742324446f,    52.5033623066590f},
  { -125.428820348224f,    50.0840260637785f},
  { -118.231628410980f,    47.1655463617031f},
  { -111.944105498296f,    43.9101112683018f},
  { -106.527984084055f,    40.4791269870344f},
  { -101.892235676848f,    37.0120512651164f},
  { -97.9285379327587f,    33.6165047722674f},
  { -164.711859062464f,    51.3862242437938f},
  { -156.832910696118f,    51.2149727211204f},
  { -148.616435446969f,    50.4611561555181f},
  { -140.386297782898f,    49.0757348734261f},
  { -132.469522203645f,    47.0762912260020f},
  { -125.124247141936f,    44.5468124755062f},
  { -118.501697258017f,    41.6186650159532f},
  { -112.650200416646f,    38.4424112745958f},
  { -107.544019585615f,    35.1618531958101f},
  { -103.116054327274f,    31.8970076530459f},
  { -161.565051177078f,    46.8204952158267f},
  { -154.120016790682f,    46.2882241477093f},
  { -146.472975888975f,    45.2133553579987f},
  { -138.889512261975f,    43.5782038962524f},
  { -131.622492959303f,    41.4235128973441f},
  { -124.865825976588f,    38.8428070417901f},
  { -118.732962674865f,    35.9630277751952f},
  { -113.261836321706f,    32.9199372930422f},
  { -108.434948822922f,    29.8371409870851f},
  { -158.783957009459f,    41.9308212597237f},
  { -151.759148035263f,    41.0697333442294f},
  { -144.628652454041f,    39.7215413185100f},
  { -137.609162359726f,    37.8954593557137f},
  { -130.898110732937f,    35.6494412835039f},
  { -124.643814022546f,    33.0807088681832f},
  { -118.933220885046f,    30.3070063208102f},
  { -113.796664113265f,    27.4459913473483f},
  { -156.319382182471f,    36.7986876565218f},
  { -149.692611957303f,    35.6550166885512f},
  { -143.028063437157f,    34.0926768215121f},
  { -136.502510907796f,    32.1421016882594f},
  { -130.271724440412f,    29.8716190222766f},
  { -124.451028529565f,    27.3758768211569f},
  { -119.108312051735f,    24.7589480673360f},
  { -154.127928592099f,    31.5236664757459f},
  { -147.873090229619f,    30.1554764761311f},
  { -141.627957235047f,    28.4456554391894f},
  { -135.537137409656f,    26.4390750029195f},
  { -129.724831969882f,    24.2073506421853f},
  { -124.282058316783f,    21.8369553839562f},
  { -152.172037850514f,    26.2153012260126f},
  { -146.261882168835f,    24.6877109680317f},
  { -140.394284756739f,    22.8984738947884f},
  { -134.688068424517f,    20.8997946068491f},
  { -129.243286808989f,    18.7602981099717f},
  { -150.419565989699f,    20.9821608982710f},
  { -144.827311826161f,    19.3610274237451f},
  { -139.299999410873f,    17.5553136605234f},
  { -133.935794277710f,    15.6181902414563f},
  { -148.843167443314f,    15.9208749961433f},
  { -143.543377358329f,    14.2666249148246f},
  { -138.323420251550f,    12.4971076405263f},
  { -147.419639185256f,    11.1079714409386f},
  { -142.388644576228f,    9.47109376781113f},
  { -146.129299783167f,    6.59609223407253f},
  { -168.278384807134f,    -55.5683540408088f},
  { -164.711859062464f,    -51.3862242437938f},
  { -161.565051177078f,    -46.8204952158267f},
  { -158.783957009459f,    -41.9308212597237f},
  { -156.319382182471f,    -36.7986876565218f},
  { -154.127928592099f,    -31.5236664757459f},
  { -152.172037850514f,    -26.2153012260126f},
  { -150.419565989699f,    -20.9821608982710f},
  { -148.843167443314f,    -15.9208749961434f},
  { -147.419639185256f,    -11.1079714409386f},
  { -146.129299783167f,    -6.59609223407253f},
  { -159.967566424063f,    -55.7766002623725f},
  { -156.832910696118f,    -51.2149727211204f},
  { -154.120016790682f,    -46.2882241477093f},
  { -151.759148035263f,    -41.0697333442294f},
  { -149.692611957303f,    -35.6550166885512f},
  { -147.873090229619f,    -30.1554764761311f},
  { -146.261882168835f,    -24.6877109680317f},
  { -144.827311826161f,    -19.3610274237451f},
  { -143.543377358329f,    -14.2666249148246f},
  { -142.388644576228f,    -9.47109376781112f},
  { -151.130480018109f,    -55.3790614040513f},
  { -148.616435446969f,    -50.4611561555181f},
  { -146.472975888975f,    -45.2133553579987f},
  { -144.628652454041f,    -39.7215413185100f},
  { -143.028063437157f,    -34.0926768215121f},
  { -141.627957235047f,    -28.4456554391894f},
  { -140.394284756739f,    -22.8984738947884f},
  { -139.299999410873f,    -17.5553136605234f},
  { -138.323420251550f,    -12.4971076405263f},
  { -142.156791984719f,    -54.2915904436854f},
  { -140.386297782898f,    -49.0757348734261f},
  { -138.889512261975f,    -43.5782038962524f},
  { -137.609162359726f,    -37.8954593557137f},
  { -136.502510907796f,    -32.1421016882594f},
  { -135.537137409656f,    -26.4390750029195f},
  { -134.688068424517f,    -20.8997946068491f},
  { -133.935794277710f,    -15.6181902414563f},
  { -133.472742324446f,    -52.5033623066590f},
  { -132.469522203645f,    -47.0762912260020f},
  { -131.622492959303f,    -41.4235128973441f},
  { -130.898110732937f,    -35.6494412835039f},
  { -130.271724440412f,    -29.8716190222766f},
  { -129.724831969882f,    -24.2073506421853f},
  { -129.243286808989f,    -18.7602981099717f},
  { -125.428820348224f,    -50.0840260637785f},
  { -125.124247141936f,    -44.5468124755062f},
  { -124.865825976588f,    -38.8428070417901f},
  { -124.643814022546f,    -33.0807088681832f},
  { -124.451028529565f,    -27.3758768211569f},
  { -124.282058316783f,    -21.8369553839562f},
  { -118.231628410980f,    -47.1655463617031f},
  { -118.501697258017f,    -41.6186650159532f},
  { -118.732962674865f,    -35.9630277751952f},
  { -118.933220885046f,    -30.3070063208102f},
  { -119.108312051735f,    -24.7589480673360f},
  { -111.944105498296f,    -43.9101112683018f},
  { -112.650200416646f,    -38.4424112745958f},
  { -113.261836321706f,    -32.9199372930422f},
  { -113.796664113265f,    -27.4459913473483f},
  { -106.527984084055f,    -40.4791269870344f},
  { -107.544019585615f,    -35.1618531958101f},
  { -108.434948822922f,    -29.8371409870851f},
  { -101.892235676848f,    -37.0120512651164f},
  { -103.116054327274f,    -31.8970076530459f},
  { -97.9285379327587f,    -33.6165047722674f}};

/* 3LD geosphere 'ico' [14 0] (1962 points) */
const float __geosphere_ico_14_0_dirs_deg[1962][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    62.0878885994974f},
  { 0.0f,    66.1808024884231f},
  { 0.0f,    70.5487009486867f},
  { 0.0f,    75.1646413480039f},
  { 0.0f,    79.9858774849308f},
  { 0.0f,    84.9544063480986f},
  { 0.0f,    90.0f},
  { 180.0f,    84.9544063480986f},
  { 180.0f,    79.9858774849308f},
  { 180.0f,    75.1646413480039f},
  { 180.0f,    70.5487009486867f},
  { 180.0f,    66.1808024884231f},
  { 180.0f,    62.0878885994974f},
  { 7.09478492022928f,    59.2669202265768f},
  { 8.36785885565049f,    63.2249334105983f},
  { 10.1918500362761f,    67.4615581142125f},
  { 13.0152040462228f,    71.9408793552494f},
  { 17.9319334355159f,    76.5853803432626f},
  { 28.3399673707642f,    81.2065723886572f},
  { 58.2825255885390f,    85.0661994333044f},
  { 121.717474411461f,    85.0661994333044f},
  { 151.660032629236f,    81.2065723886572f},
  { 162.068066564484f,    76.5853803432626f},
  { 166.984795953777f,    71.9408793552494f},
  { 169.808149963724f,    67.4615581142125f},
  { 171.632141144350f,    63.2249334105983f},
  { 172.905215079771f,    59.2669202265768f},
  { 15.0920752881032f,    59.8719245592249f},
  { 17.9319334355159f,    63.8594901030400f},
  { 22.0237505161462f,    68.0530702426802f},
  { 28.3399673707642f,    72.3432296064903f},
  { 38.9734473368657f,    76.4911891305542f},
  { 58.2825255885390f,    79.9275332607765f},
  { 90.0000000000000f,    81.4074783590547f},
  { 121.717474411461f,    79.9275332607765f},
  { 141.026552663134f,    76.4911891305542f},
  { 151.660032629236f,    72.3432296064903f},
  { 157.976249483854f,    68.0530702426802f},
  { 162.068066564484f,    63.8594901030400f},
  { 164.907924711897f,    59.8719245592249f},
  { 23.8110044533781f,    59.9680684957604f},
  { 28.3399673707642f,    63.8188491077792f},
  { 34.7391081893039f,    67.7275573789671f},
  { 44.1517511327961f,    71.4761517051299f},
  { 58.2825255885390f,    74.6575963529091f},
  { 78.3592768632294f,    76.5969610856024f},
  { 101.640723136771f,    76.5969610856024f},
  { 121.717474411461f,    74.6575963529091f},
  { 135.848248867204f,    71.4761517051299f},
  { 145.260891810696f,    67.7275573789671f},
  { 151.660032629236f,    63.8188491077792f},
  { 156.188995546622f,    59.9680684957604f},
  { 32.9114931857064f,    59.4482400957790f},
  { 38.9734473368657f,    62.9823782611085f},
  { 47.1679114977869f,    66.3788444575833f},
  { 58.2825255885390f,    69.3424036470909f},
  { 72.8279621494862f,    71.4441996352265f},
  { 90.0f,    72.2181794238810f},
  { 107.172037850514f,    71.4441996352265f},
  { 121.717474411461f,    69.3424036470909f},
  { 132.832088502213f,    66.3788444575833f},
  { 141.026552663134f,    62.9823782611085f},
  { 147.088506814294f,    59.4482400957790f},
  { 41.9526190816497f,    58.2581213197305f},
  { 49.1320813134511f,    61.3267760588334f},
  { 58.2825255885390f,    64.0724667392235f},
  { 69.6541923856616f,    66.1988092480856f},
  { 82.9535936191428f,    67.3781928139341f},
  { 97.0464063808572f,    67.3781928139341f},
  { 110.345807614338f,    66.1988092480856f},
  { 121.717474411461f,    64.0724667392235f},
  { 130.867918686549f,    61.3267760588334f},
  { 138.047380918350f,    58.2581213197305f},
  { 50.5099309427673f,    56.4143320128269f},
  { 58.2825255885390f,    58.9338005666956f},
  { 67.6072290710451f,    61.0023561147386f},
  { 78.3592768632294f,    62.3804108615600f},
  { 90.0f,    62.8666219591469f},
  { 101.640723136771f,    62.3804108615600f},
  { 112.392770928955f,    61.0023561147386f},
  { 121.717474411461f,    58.9338005666956f},
  { 129.490069057233f,    56.4143320128269f},
  { 58.2825255885390f,    54.0f},
  { 66.1808024884231f,    55.9579589628024f},
  { 75.1646413480039f,    57.4073382612483f},
  { 84.9544063480986f,    58.1829563353826f},
  { 95.0455936519014f,    58.1829563353826f},
  { 104.835358651996f,    57.4073382612483f},
  { 113.819197511577f,    55.9579589628024f},
  { 121.717474411461f,    54.0f},
  { 65.1310748815380f,    51.1421142488325f},
  { 72.8279621494862f,    52.5810709845051f},
  { 81.2167693990410f,    53.5115270803982f},
  { 90.0f,    53.8340928444730f},
  { 98.7832306009590f,    53.5115270803982f},
  { 107.172037850514f,    52.5810709845051f},
  { 114.868925118462f,    51.1421142488325f},
  { 71.0499775117081f,    47.9838604695474f},
  { 78.3592768632294f,    48.9773719471623f},
  { 86.0716405780585f,    49.4995799724251f},
  { 93.9283594219415f,    49.4995799724251f},
  { 101.640723136771f,    48.9773719471623f},
  { 108.950022488292f,    47.9838604695474f},
  { 76.1141128518623f,    44.6623463365959f},
  { 82.9535936191428f,    45.2948319044883f},
  { 90.0f,    45.5120052212781f},
  { 97.0464063808572f,    45.2948319044883f},
  { 103.885887148138f,    44.6623463365959f},
  { 80.4324561201698f,    41.2951365783062f},
  { 86.7842228465557f,    41.6485706123396f},
  { 93.2157771534443f,    41.6485706123396f},
  { 99.5675438798302f,    41.2951365783062f},
  { 84.1189512527134f,    37.9745512724670f},
  { 90.0f,    38.1213002554066f},
  { 95.8810487472866f,    37.9745512724670f},
  { 87.2781463566518f,    34.7673390037789f},
  { 92.7218536433482f,    34.7673390037789f},
  { -7.09478492022928f,    59.2669202265768f},
  { -15.0920752881032f,    59.8719245592249f},
  { -23.8110044533781f,    59.9680684957604f},
  { -32.9114931857064f,    59.4482400957790f},
  { -41.9526190816497f,    58.2581213197305f},
  { -50.5099309427673f,    56.4143320128269f},
  { -58.2825255885390f,    54.0f},
  { -65.1310748815380f,    51.1421142488325f},
  { -71.0499775117081f,    47.9838604695474f},
  { -76.1141128518623f,    44.6623463365959f},
  { -80.4324561201698f,    41.2951365783062f},
  { -84.1189512527134f,    37.9745512724670f},
  { -87.2781463566518f,    34.7673390037789f},
  { -8.36785885565049f,    63.2249334105983f},
  { -17.9319334355159f,    63.8594901030400f},
  { -28.3399673707642f,    63.8188491077792f},
  { -38.9734473368657f,    62.9823782611085f},
  { -49.1320813134511f,    61.3267760588334f},
  { -58.2825255885390f,    58.9338005666956f},
  { -66.1808024884231f,    55.9579589628024f},
  { -72.8279621494862f,    52.5810709845051f},
  { -78.3592768632294f,    48.9773719471623f},
  { -82.9535936191428f,    45.2948319044883f},
  { -86.7842228465557f,    41.6485706123396f},
  { -90.0f,    38.1213002554066f},
  { -92.7218536433482f,    34.7673390037789f},
  { -10.1918500362761f,    67.4615581142125f},
  { -22.0237505161462f,    68.0530702426802f},
  { -34.7391081893039f,    67.7275573789671f},
  { -47.1679114977869f,    66.3788444575833f},
  { -58.2825255885390f,    64.0724667392235f},
  { -67.6072290710451f,    61.0023561147386f},
  { -75.1646413480039f,    57.4073382612483f},
  { -81.2167693990410f,    53.5115270803982f},
  { -86.0716405780585f,    49.4995799724251f},
  { -90.0000000000000f,    45.5120052212781f},
  { -93.2157771534443f,    41.6485706123396f},
  { -95.8810487472866f,    37.9745512724671f},
  { -13.0152040462228f,    71.9408793552494f},
  { -28.3399673707642f,    72.3432296064903f},
  { -44.1517511327961f,    71.4761517051299f},
  { -58.2825255885390f,    69.3424036470909f},
  { -69.6541923856616f,    66.1988092480856f},
  { -78.3592768632294f,    62.3804108615600f},
  { -84.9544063480986f,    58.1829563353825f},
  { -90.0000000000000f,    53.8340928444730f},
  { -93.9283594219415f,    49.4995799724251f},
  { -97.0464063808572f,    45.2948319044883f},
  { -99.5675438798302f,    41.2951365783062f},
  { -17.9319334355159f,    76.5853803432626f},
  { -38.9734473368657f,    76.4911891305542f},
  { -58.2825255885390f,    74.6575963529091f},
  { -72.8279621494862f,    71.4441996352265f},
  { -82.9535936191428f,    67.3781928139341f},
  { -90.0f,    62.8666219591469f},
  { -95.0455936519014f,    58.1829563353825f},
  { -98.7832306009590f,    53.5115270803982f},
  { -101.640723136771f,    48.9773719471623f},
  { -103.885887148138f,    44.6623463365959f},
  { -28.3399673707642f,    81.2065723886572f},
  { -58.2825255885390f,    79.9275332607765f},
  { -78.3592768632294f,    76.5969610856024f},
  { -90.0f,    72.2181794238810f},
  { -97.0464063808572f,    67.3781928139341f},
  { -101.640723136771f,    62.3804108615600f},
  { -104.835358651996f,    57.4073382612483f},
  { -107.172037850514f,    52.5810709845051f},
  { -108.950022488292f,    47.9838604695474f},
  { -58.2825255885390f,    85.0661994333044f},
  { -90.0000000000000f,    81.4074783590547f},
  { -101.640723136771f,    76.5969610856024f},
  { -107.172037850514f,    71.4441996352265f},
  { -110.345807614338f,    66.1988092480856f},
  { -112.392770928955f,    61.0023561147386f},
  { -113.819197511577f,    55.9579589628024f},
  { -114.868925118462f,    51.1421142488325f},
  { -121.717474411461f,    85.0661994333044f},
  { -121.717474411461f,    79.9275332607765f},
  { -121.717474411461f,    74.6575963529091f},
  { -121.717474411461f,    69.3424036470909f},
  { -121.717474411461f,    64.0724667392235f},
  { -121.717474411461f,    58.9338005666956f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    81.2065723886572f},
  { -141.026552663134f,    76.4911891305542f},
  { -135.848248867204f,    71.4761517051299f},
  { -132.832088502213f,    66.3788444575833f},
  { -130.867918686549f,    61.3267760588334f},
  { -129.490069057233f,    56.4143320128269f},
  { -162.068066564484f,    76.5853803432626f},
  { -151.660032629236f,    72.3432296064903f},
  { -145.260891810696f,    67.7275573789671f},
  { -141.026552663134f,    62.9823782611085f},
  { -138.047380918350f,    58.2581213197305f},
  { -166.984795953777f,    71.9408793552494f},
  { -157.976249483854f,    68.0530702426802f},
  { -151.660032629236f,    63.8188491077792f},
  { -147.088506814294f,    59.4482400957790f},
  { -169.808149963724f,    67.4615581142125f},
  { -162.068066564484f,    63.8594901030400f},
  { -156.188995546622f,    59.9680684957604f},
  { -171.632141144350f,    63.2249334105982f},
  { -164.907924711897f,    59.8719245592249f},
  { -172.905215079771f,    59.2669202265768f},
  { 7.09478492022928f,    -59.2669202265768f},
  { 15.0920752881032f,    -59.8719245592249f},
  { 23.8110044533781f,    -59.9680684957604f},
  { 32.9114931857064f,    -59.4482400957790f},
  { 41.9526190816497f,    -58.2581213197305f},
  { 50.5099309427673f,    -56.4143320128269f},
  { 58.2825255885390f,    -54.0f},
  { 65.1310748815380f,    -51.1421142488325f},
  { 71.0499775117081f,    -47.9838604695474f},
  { 76.1141128518623f,    -44.6623463365959f},
  { 80.4324561201698f,    -41.2951365783062f},
  { 84.1189512527134f,    -37.9745512724670f},
  { 87.2781463566518f,    -34.7673390037789f},
  { 0.0f,    -62.0878885994974f},
  { 8.36785885565049f,    -63.2249334105983f},
  { 17.9319334355159f,    -63.8594901030400f},
  { 28.3399673707642f,    -63.8188491077792f},
  { 38.9734473368657f,    -62.9823782611085f},
  { 49.1320813134511f,    -61.3267760588334f},
  { 58.2825255885390f,    -58.9338005666956f},
  { 66.1808024884231f,    -55.9579589628024f},
  { 72.8279621494862f,    -52.5810709845051f},
  { 78.3592768632294f,    -48.9773719471623f},
  { 82.9535936191428f,    -45.2948319044883f},
  { 86.7842228465557f,    -41.6485706123396f},
  { 90.0f,    -38.1213002554066f},
  { 92.7218536433482f,    -34.7673390037789f},
  { 0.0f,    -66.1808024884231f},
  { 10.1918500362761f,    -67.4615581142125f},
  { 22.0237505161462f,    -68.0530702426802f},
  { 34.7391081893039f,    -67.7275573789671f},
  { 47.1679114977869f,    -66.3788444575833f},
  { 58.2825255885390f,    -64.0724667392235f},
  { 67.6072290710451f,    -61.0023561147386f},
  { 75.1646413480039f,    -57.4073382612483f},
  { 81.2167693990410f,    -53.5115270803982f},
  { 86.0716405780585f,    -49.4995799724251f},
  { 90.0000000000000f,    -45.5120052212781f},
  { 93.2157771534443f,    -41.6485706123396f},
  { 95.8810487472866f,    -37.9745512724671f},
  { 0.0f,    -70.5487009486867f},
  { 13.0152040462228f,    -71.9408793552494f},
  { 28.3399673707642f,    -72.3432296064903f},
  { 44.1517511327961f,    -71.4761517051299f},
  { 58.2825255885390f,    -69.3424036470909f},
  { 69.6541923856616f,    -66.1988092480856f},
  { 78.3592768632294f,    -62.3804108615600f},
  { 84.9544063480986f,    -58.1829563353825f},
  { 90.0000000000000f,    -53.8340928444730f},
  { 93.9283594219415f,    -49.4995799724251f},
  { 97.0464063808572f,    -45.2948319044883f},
  { 99.5675438798302f,    -41.2951365783062f},
  { 0.0f,    -75.1646413480039f},
  { 17.9319334355159f,    -76.5853803432626f},
  { 38.9734473368657f,    -76.4911891305542f},
  { 58.2825255885390f,    -74.6575963529091f},
  { 72.8279621494862f,    -71.4441996352265f},
  { 82.9535936191428f,    -67.3781928139341f},
  { 90.0f,    -62.8666219591469f},
  { 95.0455936519014f,    -58.1829563353825f},
  { 98.7832306009590f,    -53.5115270803982f},
  { 101.640723136771f,    -48.9773719471623f},
  { 103.885887148138f,    -44.6623463365959f},
  { 0.0f,    -79.9858774849308f},
  { 28.3399673707642f,    -81.2065723886572f},
  { 58.2825255885390f,    -79.9275332607765f},
  { 78.3592768632294f,    -76.5969610856024f},
  { 90.0f,    -72.2181794238810f},
  { 97.0464063808572f,    -67.3781928139341f},
  { 101.640723136771f,    -62.3804108615600f},
  { 104.835358651996f,    -57.4073382612483f},
  { 107.172037850514f,    -52.5810709845051f},
  { 108.950022488292f,    -47.9838604695474f},
  { 0.0f,    -84.9544063480986f},
  { 58.2825255885390f,    -85.0661994333044f},
  { 90.0000000000000f,    -81.4074783590547f},
  { 101.640723136771f,    -76.5969610856024f},
  { 107.172037850514f,    -71.4441996352265f},
  { 110.345807614338f,    -66.1988092480856f},
  { 112.392770928955f,    -61.0023561147386f},
  { 113.819197511577f,    -55.9579589628024f},
  { 114.868925118462f,    -51.1421142488325f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -85.0661994333044f},
  { 121.717474411461f,    -79.9275332607765f},
  { 121.717474411461f,    -74.6575963529091f},
  { 121.717474411461f,    -69.3424036470909f},
  { 121.717474411461f,    -64.0724667392235f},
  { 121.717474411461f,    -58.9338005666956f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -84.9544063480986f},
  { 151.660032629236f,    -81.2065723886572f},
  { 141.026552663134f,    -76.4911891305542f},
  { 135.848248867204f,    -71.4761517051299f},
  { 132.832088502213f,    -66.3788444575833f},
  { 130.867918686549f,    -61.3267760588334f},
  { 129.490069057233f,    -56.4143320128269f},
  { 180.0f,    -79.9858774849308f},
  { 162.068066564484f,    -76.5853803432626f},
  { 151.660032629236f,    -72.3432296064903f},
  { 145.260891810696f,    -67.7275573789671f},
  { 141.026552663134f,    -62.9823782611085f},
  { 138.047380918350f,    -58.2581213197305f},
  { 180.0f,    -75.1646413480039f},
  { 166.984795953777f,    -71.9408793552494f},
  { 157.976249483854f,    -68.0530702426802f},
  { 151.660032629236f,    -63.8188491077792f},
  { 147.088506814294f,    -59.4482400957790f},
  { 180.0f,    -70.5487009486867f},
  { 169.808149963724f,    -67.4615581142125f},
  { 162.068066564484f,    -63.8594901030400f},
  { 156.188995546622f,    -59.9680684957604f},
  { 180.0f,    -66.1808024884231f},
  { 171.632141144350f,    -63.2249334105982f},
  { 164.907924711897f,    -59.8719245592249f},
  { 180.0f,    -62.0878885994974f},
  { 172.905215079771f,    -59.2669202265768f},
  { -7.09478492022928f,    -59.2669202265768f},
  { -8.36785885565049f,    -63.2249334105983f},
  { -10.1918500362761f,    -67.4615581142125f},
  { -13.0152040462228f,    -71.9408793552494f},
  { -17.9319334355159f,    -76.5853803432626f},
  { -28.3399673707642f,    -81.2065723886572f},
  { -58.2825255885390f,    -85.0661994333044f},
  { -121.717474411461f,    -85.0661994333044f},
  { -151.660032629236f,    -81.2065723886572f},
  { -162.068066564484f,    -76.5853803432626f},
  { -166.984795953777f,    -71.9408793552494f},
  { -169.808149963724f,    -67.4615581142125f},
  { -171.632141144350f,    -63.2249334105983f},
  { -172.905215079771f,    -59.2669202265768f},
  { -15.0920752881032f,    -59.8719245592249f},
  { -17.9319334355159f,    -63.8594901030400f},
  { -22.0237505161462f,    -68.0530702426802f},
  { -28.3399673707642f,    -72.3432296064903f},
  { -38.9734473368657f,    -76.4911891305542f},
  { -58.2825255885390f,    -79.9275332607765f},
  { -90.0000000000000f,    -81.4074783590547f},
  { -121.717474411461f,    -79.9275332607765f},
  { -141.026552663134f,    -76.4911891305542f},
  { -151.660032629236f,    -72.3432296064903f},
  { -157.976249483854f,    -68.0530702426802f},
  { -162.068066564484f,    -63.8594901030400f},
  { -164.907924711897f,    -59.8719245592249f},
  { -23.8110044533781f,    -59.9680684957604f},
  { -28.3399673707642f,    -63.8188491077792f},
  { -34.7391081893039f,    -67.7275573789671f},
  { -44.1517511327961f,    -71.4761517051299f},
  { -58.2825255885390f,    -74.6575963529091f},
  { -78.3592768632294f,    -76.5969610856024f},
  { -101.640723136771f,    -76.5969610856024f},
  { -121.717474411461f,    -74.6575963529091f},
  { -135.848248867204f,    -71.4761517051299f},
  { -145.260891810696f,    -67.7275573789671f},
  { -151.660032629236f,    -63.8188491077792f},
  { -156.188995546622f,    -59.9680684957604f},
  { -32.9114931857064f,    -59.4482400957790f},
  { -38.9734473368657f,    -62.9823782611085f},
  { -47.1679114977869f,    -66.3788444575833f},
  { -58.2825255885390f,    -69.3424036470909f},
  { -72.8279621494862f,    -71.4441996352265f},
  { -90.0f,    -72.2181794238810f},
  { -107.172037850514f,    -71.4441996352265f},
  { -121.717474411461f,    -69.3424036470909f},
  { -132.832088502213f,    -66.3788444575833f},
  { -141.026552663134f,    -62.9823782611085f},
  { -147.088506814294f,    -59.4482400957790f},
  { -41.9526190816497f,    -58.2581213197305f},
  { -49.1320813134511f,    -61.3267760588334f},
  { -58.2825255885390f,    -64.0724667392235f},
  { -69.6541923856616f,    -66.1988092480856f},
  { -82.9535936191428f,    -67.3781928139341f},
  { -97.0464063808572f,    -67.3781928139341f},
  { -110.345807614338f,    -66.1988092480856f},
  { -121.717474411461f,    -64.0724667392235f},
  { -130.867918686549f,    -61.3267760588334f},
  { -138.047380918350f,    -58.2581213197305f},
  { -50.5099309427673f,    -56.4143320128269f},
  { -58.2825255885390f,    -58.9338005666956f},
  { -67.6072290710451f,    -61.0023561147386f},
  { -78.3592768632294f,    -62.3804108615600f},
  { -90.0f,    -62.8666219591469f},
  { -101.640723136771f,    -62.3804108615600f},
  { -112.392770928955f,    -61.0023561147386f},
  { -121.717474411461f,    -58.9338005666956f},
  { -129.490069057233f,    -56.4143320128269f},
  { -58.2825255885390f,    -54.0f},
  { -66.1808024884231f,    -55.9579589628024f},
  { -75.1646413480039f,    -57.4073382612483f},
  { -84.9544063480986f,    -58.1829563353826f},
  { -95.0455936519014f,    -58.1829563353826f},
  { -104.835358651996f,    -57.4073382612483f},
  { -113.819197511577f,    -55.9579589628024f},
  { -121.717474411461f,    -54.0f},
  { -65.1310748815380f,    -51.1421142488325f},
  { -72.8279621494862f,    -52.5810709845051f},
  { -81.2167693990410f,    -53.5115270803982f},
  { -90.0f,    -53.8340928444730f},
  { -98.7832306009590f,    -53.5115270803982f},
  { -107.172037850514f,    -52.5810709845051f},
  { -114.868925118462f,    -51.1421142488325f},
  { -71.0499775117081f,    -47.9838604695474f},
  { -78.3592768632294f,    -48.9773719471623f},
  { -86.0716405780585f,    -49.4995799724251f},
  { -93.9283594219415f,    -49.4995799724251f},
  { -101.640723136771f,    -48.9773719471623f},
  { -108.950022488292f,    -47.9838604695474f},
  { -76.1141128518623f,    -44.6623463365959f},
  { -82.9535936191428f,    -45.2948319044883f},
  { -90.0f,    -45.5120052212781f},
  { -97.0464063808572f,    -45.2948319044883f},
  { -103.885887148138f,    -44.6623463365959f},
  { -80.4324561201698f,    -41.2951365783062f},
  { -86.7842228465557f,    -41.6485706123396f},
  { -93.2157771534443f,    -41.6485706123396f},
  { -99.5675438798302f,    -41.2951365783062f},
  { -84.1189512527134f,    -37.9745512724670f},
  { -90.0f,    -38.1213002554066f},
  { -95.8810487472866f,    -37.9745512724670f},
  { -87.2781463566518f,    -34.7673390037789f},
  { -92.7218536433482f,    -34.7673390037789f},
  { 27.9121114005026f,    0.0f},
  { 23.8191975115769f,    0.0f},
  { 19.4512990513133f,    0.0f},
  { 14.8353586519961f,    0.0f},
  { 10.0141225150692f,    0.0f},
  { 5.04559365190141f,    0.0f},
  { 0.0f,    0.0f},
  { -5.04559365190141f,    0.0f},
  { -10.0141225150692f,    0.0f},
  { -14.8353586519961f,    0.0f},
  { -19.4512990513133f,    0.0f},
  { -23.8191975115769f,    0.0f},
  { -27.9121114005026f,    0.0f},
  { 30.5399862756552f,    3.61886081217281f},
  { 26.5292165556322f,    3.75893835920351f},
  { 22.2176315024152f,    3.88898509435710f},
  { 17.6242442962652f,    4.00333665233492f},
  { 12.7849100780905f,    4.09604375815233f},
  { 7.75317609251711f,    4.16155052992443f},
  { 2.59849973931977f,    4.19550490880869f},
  { -2.59849973931977f,    4.19550490880869f},
  { -7.75317609251711f,    4.16155052992443f},
  { -12.7849100780905f,    4.09604375815233f},
  { -17.6242442962652f,    4.00333665233492f},
  { -22.2176315024152f,    3.88898509435710f},
  { -26.5292165556322f,    3.75893835920351f},
  { -30.5399862756552f,    3.61886081217281f},
  { 29.2626971786928f,    7.50942039010128f},
  { 25.0293624731769f,    7.79603305581719f},
  { 20.4829414460949f,    8.05663962251492f},
  { 15.6506952965071f,    8.27844578473018f},
  { 10.5793931769549f,    8.44858701261369f},
  { 5.33516983949597f,    8.55584929178107f},
  { 3.95476420138187e-15f,    8.59252164094535f},
  { -5.33516983949598f,    8.55584929178107f},
  { -10.5793931769549f,    8.44858701261369f},
  { -15.6506952965071f,    8.27844578473018f},
  { -20.4829414460949f,    8.05663962251492f},
  { -25.0293624731769f,    7.79603305581719f},
  { -29.2626971786928f,    7.50942039010128f},
  { 27.8738037888168f,    11.6571750608704f},
  { 23.3993965486495f,    12.0897537624483f},
  { 18.6014200841868f,    12.4727880075744f},
  { 13.5175861242013f,    12.7852248345909f},
  { 8.20784935162439f,    13.0070533216399f},
  { 2.75270027273345f,    13.1223880193257f},
  { -2.75270027273344f,    13.1223880193257f},
  { -8.20784935162439f,    13.0070533216399f},
  { -13.5175861242013f,    12.7852248345909f},
  { -18.6014200841868f,    12.4727880075744f},
  { -23.3993965486495f,    12.0897537624483f},
  { -27.8738037888168f,    11.6571750608704f},
  { 26.3598778094528f,    16.0329525274240f},
  { 21.6246339271434f,    16.6015495990202f},
  { 16.5582178401620f,    17.0882173737518f},
  { 11.2114335525745f,    17.4632892048279f},
  { 5.65989372918869f,    17.7005548371051f},
  { 0.0f,    17.7818205761190f},
  { -5.65989372918869f,    17.7005548371051f},
  { -11.2114335525745f,    17.4632892048279f},
  { -16.5582178401620f,    17.0882173737518f},
  { -21.6246339271434f,    16.6015495990202f},
  { -26.3598778094528f,    16.0329525274240f},
  { 24.7056676962513f,    20.5914375932754f},
  { 19.6887853149864f,    21.2749936141146f},
  { 14.3374810358614f,    21.8349088916767f},
  { 8.71873446428270f,    22.2337710047006f},
  { 2.92633961108472f,    22.4415960116381f},
  { -2.92633961108472f,    22.4415960116381f},
  { -8.71873446428270f,    22.2337710047006f},
  { -14.3374810358613f,    21.8349088916767f},
  { -19.6887853149864f,    21.2749936141146f},
  { -24.7056676962513f,    20.5914375932754f},
  { 22.8939075042604f,    25.2716662118425f},
  { 17.5739541017579f,    26.0375260507155f},
  { 11.9226312702234f,    26.6294653632014f},
  { 6.02655266313435f,    27.0047092696301f},
  { 0.0f,    27.1333780408531f},
  { -6.02655266313435f,    27.0047092696301f},
  { -11.9226312702234f,    26.6294653632014f},
  { -17.5739541017579f,    26.0375260507155f},
  { -22.8939075042604f,    25.2716662118425f},
  { 20.9051574478893f,    30.0000000000000f},
  { 15.2607497076377f,    30.8051235981542f},
  { 9.29684389959181f,    31.3796780746849f},
  { 3.12333687177029f,    31.6793967072377f},
  { -3.12333687177030f,    31.6793967072377f},
  { -9.29684389959181f,    31.3796780746849f},
  { -15.2607497076377f,    30.8051235981542f},
  { -20.9051574478893f,    30.0000000000000f},
  { 18.7177168090702f,    34.6954547665617f},
  { 12.7285834429777f,    35.4893277794011f},
  { 6.44379382542064f,    35.9930793767210f},
  { 2.39145775228081e-15f,    36.1659071555270f},
  { -6.44379382542064f,    35.9930793767210f},
  { -12.7285834429777f,    35.4893277794011f},
  { -18.7177168090702f,    34.6954547665617f},
  { 16.3076732152769f,    39.2764805249330f},
  { 9.95623567583585f,    40.0052008848602f},
  { 3.34873931107063f,    40.3855428879772f},
  { -3.34873931107063f,    40.3855428879772f},
  { -9.95623567583585f,    40.0052008848602f},
  { -16.3076732152769f,    39.2764805249330f},
  { 13.6491845224847f,    43.6677528316015f},
  { 6.92280470159686f,    44.2784197953269f},
  { 0.0f,    44.4879947787219f},
  { -6.92280470159686f,    44.2784197953269f},
  { -13.6491845224847f,    43.6677528316015f},
  { 10.7151302774653f,    47.8055407787321f},
  { 3.60915914452229f,    48.2500856534970f},
  { -3.60915914452229f,    48.2500856534970f},
  { -10.7151302774653f,    47.8055407787321f},
  { 7.47831395519902f,    51.6407658043736f},
  { 0.0f,    51.8786997445934f},
  { -7.47831395519902f,    51.6407658043736f},
  { 3.91343002460889f,    55.1396530212479f},
  { -3.91343002460889f,    55.1396530212479f},
  { 30.5399862756552f,    -3.61886081217281f},
  { 29.2626971786928f,    -7.50942039010128f},
  { 27.8738037888168f,    -11.6571750608704f},
  { 26.3598778094528f,    -16.0329525274240f},
  { 24.7056676962513f,    -20.5914375932754f},
  { 22.8939075042604f,    -25.2716662118425f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 18.7177168090702f,    -34.6954547665617f},
  { 16.3076732152769f,    -39.2764805249330f},
  { 13.6491845224847f,    -43.6677528316015f},
  { 10.7151302774653f,    -47.8055407787321f},
  { 7.47831395519902f,    -51.6407658043736f},
  { 3.91343002460889f,    -55.1396530212479f},
  { 26.5292165556322f,    -3.75893835920351f},
  { 25.0293624731769f,    -7.79603305581719f},
  { 23.3993965486495f,    -12.0897537624483f},
  { 21.6246339271434f,    -16.6015495990202f},
  { 19.6887853149864f,    -21.2749936141146f},
  { 17.5739541017580f,    -26.0375260507155f},
  { 15.2607497076377f,    -30.8051235981542f},
  { 12.7285834429777f,    -35.4893277794011f},
  { 9.95623567583585f,    -40.0052008848602f},
  { 6.92280470159686f,    -44.2784197953269f},
  { 3.60915914452229f,    -48.2500856534970f},
  { 0.0f,    -51.8786997445934f},
  { -3.91343002460890f,    -55.1396530212479f},
  { 22.2176315024152f,    -3.88898509435710f},
  { 20.4829414460949f,    -8.05663962251492f},
  { 18.6014200841868f,    -12.4727880075744f},
  { 16.5582178401620f,    -17.0882173737518f},
  { 14.3374810358613f,    -21.8349088916767f},
  { 11.9226312702234f,    -26.6294653632014f},
  { 9.29684389959181f,    -31.3796780746849f},
  { 6.44379382542064f,    -35.9930793767210f},
  { 3.34873931107063f,    -40.3855428879772f},
  { -5.14182516329846e-15f,    -44.4879947787219f},
  { -3.60915914452229f,    -48.2500856534970f},
  { -7.47831395519902f,    -51.6407658043736f},
  { 17.6242442962652f,    -4.00333665233492f},
  { 15.6506952965071f,    -8.27844578473018f},
  { 13.5175861242013f,    -12.7852248345909f},
  { 11.2114335525745f,    -17.4632892048279f},
  { 8.71873446428270f,    -22.2337710047006f},
  { 6.02655266313435f,    -27.0047092696301f},
  { 3.12333687177029f,    -31.6793967072377f},
  { -4.78291550456161e-15f,    -36.1659071555270f},
  { -3.34873931107063f,    -40.3855428879772f},
  { -6.92280470159686f,    -44.2784197953269f},
  { -10.7151302774653f,    -47.8055407787321f},
  { 12.7849100780905f,    -4.09604375815233f},
  { 10.5793931769549f,    -8.44858701261369f},
  { 8.20784935162439f,    -13.0070533216399f},
  { 5.65989372918869f,    -17.7005548371051f},
  { 2.92633961108472f,    -22.4415960116381f},
  { 0.0f,    -27.1333780408531f},
  { -3.12333687177030f,    -31.6793967072377f},
  { -6.44379382542064f,    -35.9930793767210f},
  { -9.95623567583585f,    -40.0052008848602f},
  { -13.6491845224847f,    -43.6677528316015f},
  { 7.75317609251711f,    -4.16155052992442f},
  { 5.33516983949598f,    -8.55584929178106f},
  { 2.75270027273345f,    -13.1223880193257f},
  { 2.09849892565876e-15f,    -17.7818205761190f},
  { -2.92633961108472f,    -22.4415960116381f},
  { -6.02655266313435f,    -27.0047092696301f},
  { -9.29684389959181f,    -31.3796780746849f},
  { -12.7285834429777f,    -35.4893277794011f},
  { -16.3076732152769f,    -39.2764805249330f},
  { 2.59849973931977f,    -4.19550490880869f},
  { -1.97738210069094e-15f,    -8.59252164094535f},
  { -2.75270027273345f,    -13.1223880193257f},
  { -5.65989372918869f,    -17.7005548371051f},
  { -8.71873446428270f,    -22.2337710047006f},
  { -11.9226312702234f,    -26.6294653632014f},
  { -15.2607497076377f,    -30.8051235981542f},
  { -18.7177168090702f,    -34.6954547665617f},
  { -2.59849973931977f,    -4.19550490880869f},
  { -5.33516983949598f,    -8.55584929178107f},
  { -8.20784935162439f,    -13.0070533216399f},
  { -11.2114335525745f,    -17.4632892048279f},
  { -14.3374810358613f,    -21.8349088916767f},
  { -17.5739541017580f,    -26.0375260507155f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -7.75317609251711f,    -4.16155052992442f},
  { -10.5793931769549f,    -8.44858701261369f},
  { -13.5175861242013f,    -12.7852248345909f},
  { -16.5582178401620f,    -17.0882173737518f},
  { -19.6887853149864f,    -21.2749936141146f},
  { -22.8939075042604f,    -25.2716662118425f},
  { -12.7849100780905f,    -4.09604375815233f},
  { -15.6506952965071f,    -8.27844578473018f},
  { -18.6014200841868f,    -12.4727880075744f},
  { -21.6246339271434f,    -16.6015495990202f},
  { -24.7056676962513f,    -20.5914375932754f},
  { -17.6242442962652f,    -4.00333665233492f},
  { -20.4829414460949f,    -8.05663962251491f},
  { -23.3993965486495f,    -12.0897537624483f},
  { -26.3598778094528f,    -16.0329525274240f},
  { -22.2176315024152f,    -3.88898509435710f},
  { -25.0293624731769f,    -7.79603305581719f},
  { -27.8738037888168f,    -11.6571750608704f},
  { -26.5292165556322f,    -3.75893835920350f},
  { -29.2626971786928f,    -7.50942039010127f},
  { -30.5399862756552f,    -3.61886081217281f},
  { -152.087888599497f,    0.0f},
  { -156.180802488423f,    0.0f},
  { -160.548700948687f,    0.0f},
  { -165.164641348004f,    0.0f},
  { -169.985877484931f,    0.0f},
  { -174.954406348099f,    0.0f},
  { 180.0f,    0.0f},
  { 174.954406348099f,    0.0f},
  { 169.985877484931f,    0.0f},
  { 165.164641348004f,    0.0f},
  { 160.548700948687f,    0.0f},
  { 156.180802488423f,    0.0f},
  { 152.087888599497f,    0.0f},
  { -149.460013724345f,    3.61886081217281f},
  { -153.470783444368f,    3.75893835920351f},
  { -157.782368497585f,    3.88898509435710f},
  { -162.375755703735f,    4.00333665233492f},
  { -167.215089921910f,    4.09604375815233f},
  { -172.246823907483f,    4.16155052992443f},
  { -177.401500260680f,    4.19550490880869f},
  { 177.401500260680f,    4.19550490880869f},
  { 172.246823907483f,    4.16155052992443f},
  { 167.215089921910f,    4.09604375815233f},
  { 162.375755703735f,    4.00333665233492f},
  { 157.782368497585f,    3.88898509435710f},
  { 153.470783444368f,    3.75893835920351f},
  { 149.460013724345f,    3.61886081217281f},
  { -150.737302821307f,    7.50942039010128f},
  { -154.970637526823f,    7.79603305581719f},
  { -159.517058553905f,    8.05663962251492f},
  { -164.349304703493f,    8.27844578473018f},
  { -169.420606823045f,    8.44858701261369f},
  { -174.664830160504f,    8.55584929178107f},
  { -180.0f,    8.59252164094535f},
  { 174.664830160504f,    8.55584929178107f},
  { 169.420606823045f,    8.44858701261369f},
  { 164.349304703493f,    8.27844578473018f},
  { 159.517058553905f,    8.05663962251492f},
  { 154.970637526823f,    7.79603305581719f},
  { 150.737302821307f,    7.50942039010128f},
  { -152.126196211183f,    11.6571750608704f},
  { -156.600603451350f,    12.0897537624483f},
  { -161.398579915813f,    12.4727880075744f},
  { -166.482413875799f,    12.7852248345909f},
  { -171.792150648376f,    13.0070533216399f},
  { -177.247299727267f,    13.1223880193257f},
  { 177.247299727267f,    13.1223880193257f},
  { 171.792150648376f,    13.0070533216399f},
  { 166.482413875799f,    12.7852248345909f},
  { 161.398579915813f,    12.4727880075744f},
  { 156.600603451350f,    12.0897537624483f},
  { 152.126196211183f,    11.6571750608704f},
  { -153.640122190547f,    16.0329525274240f},
  { -158.375366072857f,    16.6015495990202f},
  { -163.441782159838f,    17.0882173737518f},
  { -168.788566447426f,    17.4632892048279f},
  { -174.340106270811f,    17.7005548371051f},
  { 180.0f,    17.7818205761190f},
  { 174.340106270811f,    17.7005548371051f},
  { 168.788566447426f,    17.4632892048279f},
  { 163.441782159838f,    17.0882173737518f},
  { 158.375366072857f,    16.6015495990202f},
  { 153.640122190547f,    16.0329525274240f},
  { -155.294332303749f,    20.5914375932754f},
  { -160.311214685014f,    21.2749936141146f},
  { -165.662518964139f,    21.8349088916767f},
  { -171.281265535717f,    22.2337710047006f},
  { -177.073660388915f,    22.4415960116381f},
  { 177.073660388915f,    22.4415960116381f},
  { 171.281265535717f,    22.2337710047006f},
  { 165.662518964139f,    21.8349088916767f},
  { 160.311214685014f,    21.2749936141146f},
  { 155.294332303749f,    20.5914375932754f},
  { -157.106092495740f,    25.2716662118425f},
  { -162.426045898242f,    26.0375260507155f},
  { -168.077368729777f,    26.6294653632014f},
  { -173.973447336866f,    27.0047092696301f},
  { 180.0f,    27.1333780408531f},
  { 173.973447336866f,    27.0047092696301f},
  { 168.077368729777f,    26.6294653632014f},
  { 162.426045898242f,    26.0375260507155f},
  { 157.106092495740f,    25.2716662118425f},
  { -159.094842552111f,    30.0000000000000f},
  { -164.739250292362f,    30.8051235981542f},
  { -170.703156100408f,    31.3796780746849f},
  { -176.876663128230f,    31.6793967072377f},
  { 176.876663128230f,    31.6793967072377f},
  { 170.703156100408f,    31.3796780746849f},
  { 164.739250292362f,    30.8051235981542f},
  { 159.094842552111f,    30.0000000000000f},
  { -161.282283190930f,    34.6954547665617f},
  { -167.271416557022f,    35.4893277794011f},
  { -173.556206174579f,    35.9930793767210f},
  { -180.0f,    36.1659071555270f},
  { 173.556206174579f,    35.9930793767210f},
  { 167.271416557022f,    35.4893277794011f},
  { 161.282283190930f,    34.6954547665617f},
  { -163.692326784723f,    39.2764805249330f},
  { -170.043764324164f,    40.0052008848602f},
  { -176.651260688929f,    40.3855428879772f},
  { 176.651260688929f,    40.3855428879772f},
  { 170.043764324164f,    40.0052008848602f},
  { 163.692326784723f,    39.2764805249330f},
  { -166.350815477515f,    43.6677528316015f},
  { -173.077195298403f,    44.2784197953269f},
  { 180.0f,    44.4879947787219f},
  { 173.077195298403f,    44.2784197953269f},
  { 166.350815477515f,    43.6677528316015f},
  { -169.284869722535f,    47.8055407787321f},
  { -176.390840855478f,    48.2500856534970f},
  { 176.390840855478f,    48.2500856534970f},
  { 169.284869722535f,    47.8055407787321f},
  { -172.521686044801f,    51.6407658043736f},
  { 180.0f,    51.8786997445934f},
  { 172.521686044801f,    51.6407658043736f},
  { -176.086569975391f,    55.1396530212479f},
  { 176.086569975391f,    55.1396530212479f},
  { -149.460013724345f,    -3.61886081217281f},
  { -150.737302821307f,    -7.50942039010128f},
  { -152.126196211183f,    -11.6571750608704f},
  { -153.640122190547f,    -16.0329525274240f},
  { -155.294332303749f,    -20.5914375932754f},
  { -157.106092495740f,    -25.2716662118425f},
  { -159.094842552111f,    -30.0000000000000f},
  { -161.282283190930f,    -34.6954547665617f},
  { -163.692326784723f,    -39.2764805249330f},
  { -166.350815477515f,    -43.6677528316015f},
  { -169.284869722535f,    -47.8055407787321f},
  { -172.521686044801f,    -51.6407658043736f},
  { -176.086569975391f,    -55.1396530212479f},
  { -153.470783444368f,    -3.75893835920351f},
  { -154.970637526823f,    -7.79603305581719f},
  { -156.600603451350f,    -12.0897537624483f},
  { -158.375366072857f,    -16.6015495990202f},
  { -160.311214685014f,    -21.2749936141146f},
  { -162.426045898242f,    -26.0375260507155f},
  { -164.739250292362f,    -30.8051235981542f},
  { -167.271416557022f,    -35.4893277794011f},
  { -170.043764324164f,    -40.0052008848602f},
  { -173.077195298403f,    -44.2784197953269f},
  { -176.390840855478f,    -48.2500856534970f},
  { 180.0f,    -51.8786997445934f},
  { 176.086569975391f,    -55.1396530212479f},
  { -157.782368497585f,    -3.88898509435710f},
  { -159.517058553905f,    -8.05663962251492f},
  { -161.398579915813f,    -12.4727880075744f},
  { -163.441782159838f,    -17.0882173737518f},
  { -165.662518964139f,    -21.8349088916767f},
  { -168.077368729777f,    -26.6294653632014f},
  { -170.703156100408f,    -31.3796780746849f},
  { -173.556206174579f,    -35.9930793767210f},
  { -176.651260688929f,    -40.3855428879772f},
  { 180.0f,    -44.4879947787219f},
  { 176.390840855478f,    -48.2500856534970f},
  { 172.521686044801f,    -51.6407658043736f},
  { -162.375755703735f,    -4.00333665233492f},
  { -164.349304703493f,    -8.27844578473018f},
  { -166.482413875799f,    -12.7852248345909f},
  { -168.788566447426f,    -17.4632892048279f},
  { -171.281265535717f,    -22.2337710047006f},
  { -173.973447336866f,    -27.0047092696301f},
  { -176.876663128230f,    -31.6793967072377f},
  { 180.0f,    -36.1659071555270f},
  { 176.651260688929f,    -40.3855428879772f},
  { 173.077195298403f,    -44.2784197953269f},
  { 169.284869722535f,    -47.8055407787321f},
  { -167.215089921910f,    -4.09604375815233f},
  { -169.420606823045f,    -8.44858701261369f},
  { -171.792150648376f,    -13.0070533216399f},
  { -174.340106270811f,    -17.7005548371051f},
  { -177.073660388915f,    -22.4415960116381f},
  { 180.0f,    -27.1333780408531f},
  { 176.876663128230f,    -31.6793967072377f},
  { 173.556206174579f,    -35.9930793767210f},
  { 170.043764324164f,    -40.0052008848602f},
  { 166.350815477515f,    -43.6677528316015f},
  { -172.246823907483f,    -4.16155052992442f},
  { -174.664830160504f,    -8.55584929178106f},
  { -177.247299727267f,    -13.1223880193257f},
  { -180.0f,    -17.7818205761190f},
  { 177.073660388915f,    -22.4415960116381f},
  { 173.973447336866f,    -27.0047092696301f},
  { 170.703156100408f,    -31.3796780746849f},
  { 167.271416557022f,    -35.4893277794011f},
  { 163.692326784723f,    -39.2764805249330f},
  { -177.401500260680f,    -4.19550490880869f},
  { 180.0f,    -8.59252164094535f},
  { 177.247299727267f,    -13.1223880193257f},
  { 174.340106270811f,    -17.7005548371051f},
  { 171.281265535717f,    -22.2337710047006f},
  { 168.077368729777f,    -26.6294653632014f},
  { 164.739250292362f,    -30.8051235981542f},
  { 161.282283190930f,    -34.6954547665617f},
  { 177.401500260680f,    -4.19550490880869f},
  { 174.664830160504f,    -8.55584929178107f},
  { 171.792150648376f,    -13.0070533216399f},
  { 168.788566447426f,    -17.4632892048279f},
  { 165.662518964139f,    -21.8349088916767f},
  { 162.426045898242f,    -26.0375260507155f},
  { 159.094842552111f,    -30.0000000000000f},
  { 172.246823907483f,    -4.16155052992442f},
  { 169.420606823045f,    -8.44858701261369f},
  { 166.482413875799f,    -12.7852248345909f},
  { 163.441782159838f,    -17.0882173737518f},
  { 160.311214685014f,    -21.2749936141146f},
  { 157.106092495740f,    -25.2716662118425f},
  { 167.215089921910f,    -4.09604375815233f},
  { 164.349304703493f,    -8.27844578473018f},
  { 161.398579915813f,    -12.4727880075744f},
  { 158.375366072857f,    -16.6015495990202f},
  { 155.294332303749f,    -20.5914375932754f},
  { 162.375755703735f,    -4.00333665233492f},
  { 159.517058553905f,    -8.05663962251492f},
  { 156.600603451350f,    -12.0897537624483f},
  { 153.640122190547f,    -16.0329525274240f},
  { 157.782368497585f,    -3.88898509435710f},
  { 154.970637526823f,    -7.79603305581719f},
  { 152.126196211183f,    -11.6571750608704f},
  { 153.470783444368f,    -3.75893835920350f},
  { 150.737302821307f,    -7.50942039010127f},
  { 149.460013724345f,    -3.61886081217281f},
  { 90.0f,    27.9121114005026f},
  { 90.0f,    23.8191975115769f},
  { 90.0f,    19.4512990513133f},
  { 90.0f,    14.8353586519961f},
  { 90.0f,    10.0141225150692f},
  { 90.0f,    5.04559365190141f},
  { 90.0f,    0.0f},
  { 90.0f,    -5.04559365190141f},
  { 90.0f,    -10.0141225150692f},
  { 90.0f,    -14.8353586519961f},
  { 90.0f,    -19.4512990513133f},
  { 90.0f,    -23.8191975115769f},
  { 90.0f,    -27.9121114005026f},
  { 85.8001920973204f,    30.4726052669007f},
  { 85.8001920973204f,    26.4676993864943f},
  { 85.8001920973204f,    22.1637536457461f},
  { 85.8001920973204f,    17.5798366791228f},
  { 85.8001920973204f,    12.7517029748288f},
  { 85.8001920973204f,    7.73260888272149f},
  { 85.8001920973204f,    2.59153157275679f},
  { 85.8001920973204f,    -2.59153157275679f},
  { 85.8001920973204f,    -7.73260888272149f},
  { 85.8001920973204f,    -12.7517029748288f},
  { 85.8001920973204f,    -17.5798366791228f},
  { 85.8001920973204f,    -22.1637536457461f},
  { 85.8001920973204f,    -26.4676993864943f},
  { 85.8001920973204f,    -30.4726052669007f},
  { 81.4074783590547f,    28.9877234630627f},
  { 81.4074783590547f,    24.7823366506442f},
  { 81.4074783590547f,    20.2718429567436f},
  { 81.4074783590547f,    15.4835026881960f},
  { 81.4074783590547f,    10.4632857609175f},
  { 81.4074783590547f,    5.27562709814274f},
  { 81.4074783590547f,    3.91037549404927e-15f},
  { 81.4074783590547f,    -5.27562709814274f},
  { 81.4074783590547f,    -10.4632857609175f},
  { 81.4074783590547f,    -15.4835026881960f},
  { 81.4074783590547f,    -20.2718429567436f},
  { 81.4074783590547f,    -24.7823366506442f},
  { 81.4074783590547f,    -28.9877234630627f},
  { 76.8629783273088f,    27.2505574101825f},
  { 76.8629783273088f,    22.8506257028115f},
  { 76.8629783273088f,    18.1469005785316f},
  { 76.8629783273088f,    13.1763196305665f},
  { 76.8629783273088f,    7.99585892685531f},
  { 76.8629783273088f,    2.68076671575566f},
  { 76.8629783273088f,    -2.68076671575566f},
  { 76.8629783273088f,    -7.99585892685532f},
  { 76.8629783273088f,    -13.1763196305665f},
  { 76.8629783273088f,    -18.1469005785316f},
  { 76.8629783273088f,    -22.8506257028115f},
  { 76.8629783273088f,    -27.2505574101825f},
  { 72.2181794238810f,    25.2606763729821f},
  { 72.2181794238810f,    20.6808566887685f},
  { 72.2181794238810f,    15.8076162308333f},
  { 72.2181794238810f,    10.6884601294477f},
  { 72.2181794238810f,    5.39113258470827f},
  { 72.2181794238810f,    0.0f},
  { 72.2181794238810f,    -5.39113258470827f},
  { 72.2181794238810f,    -10.6884601294477f},
  { 72.2181794238810f,    -15.8076162308333f},
  { 72.2181794238810f,    -20.6808566887685f},
  { 72.2181794238810f,    -25.2606763729821f},
  { 67.5320130849268f,    23.0325838541662f},
  { 67.5320130849268f,    18.2974610284673f},
  { 67.5320130849268f,    13.2892725737601f},
  { 67.5320130849268f,    8.06598231735982f},
  { 67.5320130849268f,    2.70455387726614f},
  { 67.5320130849268f,    -2.70455387726614f},
  { 67.5320130849268f,    -8.06598231735982f},
  { 67.5320130849268f,    -13.2892725737601f},
  { 67.5320130849268f,    -18.2974610284673f},
  { 67.5320130849268f,    -23.0325838541662f},
  { 62.8666219591469f,    20.5970703451119f},
  { 62.8666219591469f,    15.7411644579173f},
  { 62.8666219591469f,    10.6422621988362f},
  { 62.8666219591469f,    5.36742458645144f},
  { 62.8666219591469f,    0.0f},
  { 62.8666219591469f,    -5.36742458645144f},
  { 62.8666219591469f,    -10.6422621988362f},
  { 62.8666219591469f,    -15.7411644579173f},
  { 62.8666219591469f,    -20.5970703451119f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    13.0661994333044f},
  { 58.2825255885390f,    7.92753326077648f},
  { 58.2825255885390f,    2.65759635290911f},
  { 58.2825255885390f,    -2.65759635290911f},
  { 58.2825255885390f,    -7.92753326077648f},
  { 58.2825255885390f,    -13.0661994333044f},
  { 58.2825255885390f,    -18.0f},
  { 53.8340928444730f,    15.2983539399373f},
  { 53.8340928444730f,    10.3348262206107f},
  { 53.8340928444730f,    5.20978847514459f},
  { 53.8340928444730f,    1.93065158137489e-15f},
  { 53.8340928444730f,    -5.20978847514459f},
  { 53.8340928444730f,    -10.3348262206107f},
  { 53.8340928444730f,    -15.2983539399373f},
  { 49.5661258277807f,    12.5542339264727f},
  { 49.5661258277807f,    7.61037380956407f},
  { 49.5661258277807f,    2.55013072981304f},
  { 49.5661258277807f,    -2.55013072981304f},
  { 49.5661258277807f,    -7.61037380956407f},
  { 49.5661258277807f,    -12.5542339264727f},
  { 45.5120052212781f,    9.82822484687696f},
  { 45.5120052212781f,    4.95052824638692f},
  { 45.5120052212781f,    0.0f},
  { 45.5120052212781f,    -4.95052824638692f},
  { 45.5120052212781f,    -9.82822484687696f},
  { 41.6934119446187f,    7.17365637790974f},
  { 41.6934119446187f,    2.40238371993281f},
  { 41.6934119446187f,    -2.40238371993281f},
  { 41.6934119446187f,    -7.17365637790974f},
  { 38.1213002554066f,    4.63284787319209f},
  { 38.1213002554066f,    0.0f},
  { 38.1213002554066f,    -4.63284787319209f},
  { 34.7976417346129f,    2.23565947565241f},
  { 34.7976417346129f,    -2.23565947565241f},
  { 94.1998079026796f,    30.4726052669007f},
  { 98.5925216409454f,    28.9877234630627f},
  { 103.137021672691f,    27.2505574101825f},
  { 107.781820576119f,    25.2606763729821f},
  { 112.467986915073f,    23.0325838541662f},
  { 117.133378040853f,    20.5970703451119f},
  { 121.717474411461f,    18.0f},
  { 126.165907155527f,    15.2983539399373f},
  { 130.433874172219f,    12.5542339264727f},
  { 134.487994778722f,    9.82822484687696f},
  { 138.306588055381f,    7.17365637790974f},
  { 141.878699744593f,    4.63284787319209f},
  { 145.202358265387f,    2.23565947565241f},
  { 94.1998079026796f,    26.4676993864943f},
  { 98.5925216409454f,    24.7823366506442f},
  { 103.137021672691f,    22.8506257028115f},
  { 107.781820576119f,    20.6808566887685f},
  { 112.467986915073f,    18.2974610284673f},
  { 117.133378040853f,    15.7411644579173f},
  { 121.717474411461f,    13.0661994333044f},
  { 126.165907155527f,    10.3348262206107f},
  { 130.433874172219f,    7.61037380956407f},
  { 134.487994778722f,    4.95052824638692f},
  { 138.306588055381f,    2.40238371993281f},
  { 141.878699744593f,    0.0f},
  { 145.202358265387f,    -2.23565947565241f},
  { 94.1998079026796f,    22.1637536457461f},
  { 98.5925216409454f,    20.2718429567436f},
  { 103.137021672691f,    18.1469005785316f},
  { 107.781820576119f,    15.8076162308333f},
  { 112.467986915073f,    13.2892725737601f},
  { 117.133378040853f,    10.6422621988362f},
  { 121.717474411461f,    7.92753326077648f},
  { 126.165907155527f,    5.20978847514459f},
  { 130.433874172219f,    2.55013072981304f},
  { 134.487994778722f,    -3.66816416523558e-15f},
  { 138.306588055381f,    -2.40238371993281f},
  { 141.878699744593f,    -4.63284787319209f},
  { 94.1998079026796f,    17.5798366791228f},
  { 98.5925216409454f,    15.4835026881960f},
  { 103.137021672691f,    13.1763196305665f},
  { 107.781820576119f,    10.6884601294477f},
  { 112.467986915073f,    8.06598231735982f},
  { 117.133378040853f,    5.36742458645144f},
  { 121.717474411461f,    2.65759635290911f},
  { 126.165907155527f,    -3.86130316274977e-15f},
  { 130.433874172219f,    -2.55013072981304f},
  { 134.487994778722f,    -4.95052824638691f},
  { 138.306588055381f,    -7.17365637790974f},
  { 94.1998079026796f,    12.7517029748288f},
  { 98.5925216409454f,    10.4632857609175f},
  { 103.137021672691f,    7.99585892685532f},
  { 107.781820576119f,    5.39113258470827f},
  { 112.467986915073f,    2.70455387726614f},
  { 117.133378040853f,    0.0f},
  { 121.717474411461f,    -2.65759635290911f},
  { 126.165907155527f,    -5.20978847514458f},
  { 130.433874172219f,    -7.61037380956407f},
  { 134.487994778722f,    -9.82822484687696f},
  { 94.1998079026796f,    7.73260888272149f},
  { 98.5925216409454f,    5.27562709814274f},
  { 103.137021672691f,    2.68076671575566f},
  { 107.781820576119f,    1.99824594963184e-15f},
  { 112.467986915073f,    -2.70455387726614f},
  { 117.133378040853f,    -5.36742458645144f},
  { 121.717474411461f,    -7.92753326077648f},
  { 126.165907155527f,    -10.3348262206107f},
  { 130.433874172219f,    -12.5542339264727f},
  { 94.1998079026796f,    2.59153157275679f},
  { 98.5925216409454f,    -1.95518774702464e-15f},
  { 103.137021672691f,    -2.68076671575566f},
  { 107.781820576119f,    -5.39113258470827f},
  { 112.467986915073f,    -8.06598231735982f},
  { 117.133378040853f,    -10.6422621988362f},
  { 121.717474411461f,    -13.0661994333044f},
  { 126.165907155527f,    -15.2983539399373f},
  { 94.1998079026796f,    -2.59153157275679f},
  { 98.5925216409454f,    -5.27562709814274f},
  { 103.137021672691f,    -7.99585892685532f},
  { 107.781820576119f,    -10.6884601294477f},
  { 112.467986915073f,    -13.2892725737601f},
  { 117.133378040853f,    -15.7411644579173f},
  { 121.717474411461f,    -18.0f},
  { 94.1998079026796f,    -7.73260888272149f},
  { 98.5925216409454f,    -10.4632857609175f},
  { 103.137021672691f,    -13.1763196305665f},
  { 107.781820576119f,    -15.8076162308333f},
  { 112.467986915073f,    -18.2974610284673f},
  { 117.133378040853f,    -20.5970703451119f},
  { 94.1998079026796f,    -12.7517029748288f},
  { 98.5925216409454f,    -15.4835026881960f},
  { 103.137021672691f,    -18.1469005785316f},
  { 107.781820576119f,    -20.6808566887685f},
  { 112.467986915073f,    -23.0325838541662f},
  { 94.1998079026796f,    -17.5798366791228f},
  { 98.5925216409454f,    -20.2718429567436f},
  { 103.137021672691f,    -22.8506257028115f},
  { 107.781820576119f,    -25.2606763729821f},
  { 94.1998079026796f,    -22.1637536457461f},
  { 98.5925216409454f,    -24.7823366506442f},
  { 103.137021672691f,    -27.2505574101825f},
  { 94.1998079026796f,    -26.4676993864943f},
  { 98.5925216409454f,    -28.9877234630627f},
  { 94.1998079026796f,    -30.4726052669007f},
  { -85.8001920973204f,    30.4726052669007f},
  { -81.4074783590547f,    28.9877234630627f},
  { -76.8629783273088f,    27.2505574101825f},
  { -72.2181794238810f,    25.2606763729821f},
  { -67.5320130849268f,    23.0325838541662f},
  { -62.8666219591469f,    20.5970703451119f},
  { -58.2825255885390f,    18.0f},
  { -53.8340928444730f,    15.2983539399373f},
  { -49.5661258277807f,    12.5542339264727f},
  { -45.5120052212781f,    9.82822484687696f},
  { -41.6934119446187f,    7.17365637790974f},
  { -38.1213002554066f,    4.63284787319209f},
  { -34.7976417346129f,    2.23565947565241f},
  { -90.0f,    27.9121114005026f},
  { -85.8001920973204f,    26.4676993864943f},
  { -81.4074783590547f,    24.7823366506442f},
  { -76.8629783273088f,    22.8506257028115f},
  { -72.2181794238810f,    20.6808566887685f},
  { -67.5320130849268f,    18.2974610284673f},
  { -62.8666219591469f,    15.7411644579173f},
  { -58.2825255885390f,    13.0661994333044f},
  { -53.8340928444730f,    10.3348262206107f},
  { -49.5661258277807f,    7.61037380956407f},
  { -45.5120052212781f,    4.95052824638692f},
  { -41.6934119446187f,    2.40238371993281f},
  { -38.1213002554066f,    0.0f},
  { -34.7976417346129f,    -2.23565947565241f},
  { -90.0f,    23.8191975115769f},
  { -85.8001920973204f,    22.1637536457461f},
  { -81.4074783590547f,    20.2718429567436f},
  { -76.8629783273088f,    18.1469005785316f},
  { -72.2181794238810f,    15.8076162308333f},
  { -67.5320130849268f,    13.2892725737601f},
  { -62.8666219591469f,    10.6422621988362f},
  { -58.2825255885390f,    7.92753326077648f},
  { -53.8340928444730f,    5.20978847514459f},
  { -49.5661258277807f,    2.55013072981304f},
  { -45.5120052212781f,    -3.66816416523558e-15f},
  { -41.6934119446187f,    -2.40238371993281f},
  { -38.1213002554066f,    -4.63284787319209f},
  { -90.0f,    19.4512990513133f},
  { -85.8001920973204f,    17.5798366791228f},
  { -81.4074783590547f,    15.4835026881960f},
  { -76.8629783273088f,    13.1763196305665f},
  { -72.2181794238810f,    10.6884601294477f},
  { -67.5320130849268f,    8.06598231735982f},
  { -62.8666219591469f,    5.36742458645144f},
  { -58.2825255885390f,    2.65759635290911f},
  { -53.8340928444730f,    -3.86130316274977e-15f},
  { -49.5661258277807f,    -2.55013072981304f},
  { -45.5120052212781f,    -4.95052824638691f},
  { -41.6934119446187f,    -7.17365637790974f},
  { -90.0f,    14.8353586519961f},
  { -85.8001920973204f,    12.7517029748288f},
  { -81.4074783590547f,    10.4632857609175f},
  { -76.8629783273088f,    7.99585892685532f},
  { -72.2181794238810f,    5.39113258470827f},
  { -67.5320130849268f,    2.70455387726614f},
  { -62.8666219591469f,    0.0f},
  { -58.2825255885390f,    -2.65759635290911f},
  { -53.8340928444730f,    -5.20978847514458f},
  { -49.5661258277807f,    -7.61037380956407f},
  { -45.5120052212781f,    -9.82822484687696f},
  { -90.0f,    10.0141225150692f},
  { -85.8001920973204f,    7.73260888272149f},
  { -81.4074783590547f,    5.27562709814274f},
  { -76.8629783273088f,    2.68076671575566f},
  { -72.2181794238810f,    1.99824594963184e-15f},
  { -67.5320130849268f,    -2.70455387726614f},
  { -62.8666219591469f,    -5.36742458645144f},
  { -58.2825255885390f,    -7.92753326077648f},
  { -53.8340928444730f,    -10.3348262206107f},
  { -49.5661258277807f,    -12.5542339264727f},
  { -90.0f,    5.04559365190141f},
  { -85.8001920973204f,    2.59153157275679f},
  { -81.4074783590547f,    -1.95518774702464e-15f},
  { -76.8629783273088f,    -2.68076671575566f},
  { -72.2181794238810f,    -5.39113258470827f},
  { -67.5320130849268f,    -8.06598231735982f},
  { -62.8666219591469f,    -10.6422621988362f},
  { -58.2825255885390f,    -13.0661994333044f},
  { -53.8340928444730f,    -15.2983539399373f},
  { -90.0f,    0.0f},
  { -85.8001920973204f,    -2.59153157275679f},
  { -81.4074783590547f,    -5.27562709814274f},
  { -76.8629783273088f,    -7.99585892685532f},
  { -72.2181794238810f,    -10.6884601294477f},
  { -67.5320130849268f,    -13.2892725737601f},
  { -62.8666219591469f,    -15.7411644579173f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -5.04559365190141f},
  { -85.8001920973204f,    -7.73260888272149f},
  { -81.4074783590547f,    -10.4632857609175f},
  { -76.8629783273088f,    -13.1763196305665f},
  { -72.2181794238810f,    -15.8076162308333f},
  { -67.5320130849268f,    -18.2974610284673f},
  { -62.8666219591469f,    -20.5970703451119f},
  { -90.0f,    -10.0141225150692f},
  { -85.8001920973204f,    -12.7517029748288f},
  { -81.4074783590547f,    -15.4835026881960f},
  { -76.8629783273088f,    -18.1469005785316f},
  { -72.2181794238810f,    -20.6808566887685f},
  { -67.5320130849268f,    -23.0325838541662f},
  { -90.0f,    -14.8353586519961f},
  { -85.8001920973204f,    -17.5798366791228f},
  { -81.4074783590547f,    -20.2718429567436f},
  { -76.8629783273088f,    -22.8506257028115f},
  { -72.2181794238810f,    -25.2606763729821f},
  { -90.0f,    -19.4512990513133f},
  { -85.8001920973204f,    -22.1637536457461f},
  { -81.4074783590547f,    -24.7823366506442f},
  { -76.8629783273088f,    -27.2505574101825f},
  { -90.0f,    -23.8191975115769f},
  { -85.8001920973204f,    -26.4676993864943f},
  { -81.4074783590547f,    -28.9877234630627f},
  { -90.0f,    -27.9121114005026f},
  { -85.8001920973204f,    -30.4726052669007f},
  { -94.1998079026796f,    30.4726052669007f},
  { -94.1998079026796f,    26.4676993864943f},
  { -94.1998079026796f,    22.1637536457461f},
  { -94.1998079026796f,    17.5798366791228f},
  { -94.1998079026796f,    12.7517029748288f},
  { -94.1998079026796f,    7.73260888272149f},
  { -94.1998079026796f,    2.59153157275679f},
  { -94.1998079026796f,    -2.59153157275679f},
  { -94.1998079026796f,    -7.73260888272149f},
  { -94.1998079026796f,    -12.7517029748288f},
  { -94.1998079026796f,    -17.5798366791228f},
  { -94.1998079026796f,    -22.1637536457461f},
  { -94.1998079026796f,    -26.4676993864943f},
  { -94.1998079026796f,    -30.4726052669007f},
  { -98.5925216409454f,    28.9877234630627f},
  { -98.5925216409454f,    24.7823366506442f},
  { -98.5925216409454f,    20.2718429567436f},
  { -98.5925216409454f,    15.4835026881960f},
  { -98.5925216409454f,    10.4632857609175f},
  { -98.5925216409454f,    5.27562709814274f},
  { -98.5925216409454f,    3.91037549404927e-15f},
  { -98.5925216409454f,    -5.27562709814274f},
  { -98.5925216409454f,    -10.4632857609175f},
  { -98.5925216409454f,    -15.4835026881960f},
  { -98.5925216409454f,    -20.2718429567436f},
  { -98.5925216409454f,    -24.7823366506442f},
  { -98.5925216409454f,    -28.9877234630627f},
  { -103.137021672691f,    27.2505574101825f},
  { -103.137021672691f,    22.8506257028115f},
  { -103.137021672691f,    18.1469005785316f},
  { -103.137021672691f,    13.1763196305665f},
  { -103.137021672691f,    7.99585892685531f},
  { -103.137021672691f,    2.68076671575566f},
  { -103.137021672691f,    -2.68076671575566f},
  { -103.137021672691f,    -7.99585892685532f},
  { -103.137021672691f,    -13.1763196305665f},
  { -103.137021672691f,    -18.1469005785316f},
  { -103.137021672691f,    -22.8506257028115f},
  { -103.137021672691f,    -27.2505574101825f},
  { -107.781820576119f,    25.2606763729821f},
  { -107.781820576119f,    20.6808566887685f},
  { -107.781820576119f,    15.8076162308333f},
  { -107.781820576119f,    10.6884601294477f},
  { -107.781820576119f,    5.39113258470827f},
  { -107.781820576119f,    0.0f},
  { -107.781820576119f,    -5.39113258470827f},
  { -107.781820576119f,    -10.6884601294477f},
  { -107.781820576119f,    -15.8076162308333f},
  { -107.781820576119f,    -20.6808566887685f},
  { -107.781820576119f,    -25.2606763729821f},
  { -112.467986915073f,    23.0325838541662f},
  { -112.467986915073f,    18.2974610284673f},
  { -112.467986915073f,    13.2892725737601f},
  { -112.467986915073f,    8.06598231735982f},
  { -112.467986915073f,    2.70455387726614f},
  { -112.467986915073f,    -2.70455387726614f},
  { -112.467986915073f,    -8.06598231735982f},
  { -112.467986915073f,    -13.2892725737601f},
  { -112.467986915073f,    -18.2974610284673f},
  { -112.467986915073f,    -23.0325838541662f},
  { -117.133378040853f,    20.5970703451119f},
  { -117.133378040853f,    15.7411644579173f},
  { -117.133378040853f,    10.6422621988362f},
  { -117.133378040853f,    5.36742458645144f},
  { -117.133378040853f,    0.0f},
  { -117.133378040853f,    -5.36742458645144f},
  { -117.133378040853f,    -10.6422621988362f},
  { -117.133378040853f,    -15.7411644579173f},
  { -117.133378040853f,    -20.5970703451119f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    13.0661994333044f},
  { -121.717474411461f,    7.92753326077648f},
  { -121.717474411461f,    2.65759635290911f},
  { -121.717474411461f,    -2.65759635290911f},
  { -121.717474411461f,    -7.92753326077648f},
  { -121.717474411461f,    -13.0661994333044f},
  { -121.717474411461f,    -18.0f},
  { -126.165907155527f,    15.2983539399373f},
  { -126.165907155527f,    10.3348262206107f},
  { -126.165907155527f,    5.20978847514459f},
  { -126.165907155527f,    1.93065158137489e-15f},
  { -126.165907155527f,    -5.20978847514459f},
  { -126.165907155527f,    -10.3348262206107f},
  { -126.165907155527f,    -15.2983539399373f},
  { -130.433874172219f,    12.5542339264727f},
  { -130.433874172219f,    7.61037380956407f},
  { -130.433874172219f,    2.55013072981304f},
  { -130.433874172219f,    -2.55013072981304f},
  { -130.433874172219f,    -7.61037380956407f},
  { -130.433874172219f,    -12.5542339264727f},
  { -134.487994778722f,    9.82822484687696f},
  { -134.487994778722f,    4.95052824638692f},
  { -134.487994778722f,    0.0f},
  { -134.487994778722f,    -4.95052824638692f},
  { -134.487994778722f,    -9.82822484687696f},
  { -138.306588055381f,    7.17365637790974f},
  { -138.306588055381f,    2.40238371993281f},
  { -138.306588055381f,    -2.40238371993281f},
  { -138.306588055381f,    -7.17365637790974f},
  { -141.878699744593f,    4.63284787319209f},
  { -141.878699744593f,    0.0f},
  { -141.878699744593f,    -4.63284787319209f},
  { -145.202358265387f,    2.23565947565241f},
  { -145.202358265387f,    -2.23565947565241f},
  { 82.6927335271427f,    33.4884987864554f},
  { 77.9230649877361f,    31.9320345172748f},
  { 73.0270942134697f,    30.0855362676247f},
  { 68.0730689456761f,    27.9522875976322f},
  { 63.1342382541714f,    25.5544940369357f},
  { 58.2825255885390f,    22.9338005666956f},
  { 53.5824807116749f,    20.1483462191348f},
  { 49.0867190496941f,    17.2666558898369f},
  { 44.8335287387352f,    14.3597683383896f},
  { 40.8466553841700f,    11.4935660517816f},
  { 37.1367620427039f,    8.72301827225606f},
  { 33.7038507373336f,    6.08918401662294f},
  { 79.1057669885777f,    36.6365158565735f},
  { 73.9332208850461f,    34.9768443320308f},
  { 68.6806178175289f,    32.9853991131827f},
  { 63.4349488229220f,    30.6726725373555f},
  { 58.2825255885390f,    28.0724667392235f},
  { 53.3007747995101f,    25.2406273115173f},
  { 48.5520446456536f,    22.2495231164141f},
  { 44.0804766102169f,    19.1794139209880f},
  { 39.9119385723822f,    16.1089297860205f},
  { 36.0562259208489f,    13.1070197584612f},
  { 32.5104534680331f,    10.2279132248474f},
  { 74.9546561542605f,    39.8605646102295f},
  { 69.3675539144640f,    38.0549736525038f},
  { 63.7752769377408f,    35.8746974937919f},
  { 58.2825255885390f,    33.3424036470909f},
  { 52.9832491424158f,    30.5080014196374f},
  { 47.9520724581596f,    27.4443942526968f},
  { 43.2401471431053f,    24.2387427969541f},
  { 38.8753897469057f,    20.9814763421170f},
  { 34.8658259765881f,    17.7559305016477f},
  { 31.2043868376226f,    14.6309156267791f},
  { 70.1502822665428f,    43.0802347503797f},
  { 64.1635743358256f,    41.0756272443228f},
  { 58.2825255885390f,    38.6575963529091f},
  { 52.6226318593503f,    35.8669489234169f},
  { 47.2742221363758f,    32.7737057972756f},
  { 42.2968675884159f,    29.4688797542767f},
  { 37.7202134140217f,    26.0526065796734f},
  { 33.5491126061725f,    22.6217951428160f},
  { 29.7704751700052f,    19.2602522246902f},
  { 64.6107072735303f,    46.1904624830411f},
  { 58.2825255885390f,    43.9275332607765f},
  { 52.2095478183208f,    41.2246619591039f},
  { 46.5025109077962f,    38.1462352379972f},
  { 41.2311043609539f,    34.7840430571184f},
  { 36.4259321088004f,    31.2450805567779f},
  { 32.0864948242697f,    27.6377926894235f},
  { 28.1910708643724f,    24.0600964664452f},
  { 58.2825255885390f,    49.0661994333044f},
  { 51.7317026220431f,    46.4882636786595f},
  { 45.6163420286281f,    43.4656373529475f},
  { 40.0182492109711f,    40.0886354813392f},
  { 34.9674044813552f,    36.4682426764923f},
  { 30.4545634390528f,    32.7215661485233f},
  { 26.4456546236829f,    28.9584386201125f},
  { 51.1726569250027f,    51.5740177485737f},
  { 44.5887483926325f,    48.6401573061787f},
  { 38.6269447803609f,    45.2865532254442f},
  { 33.3135536076770f,    41.6261313598461f},
  { 28.6252826074369f,    37.7816432786130f},
  { 24.5104108657590f,    33.8716956871667f},
  { 43.3837914212946f,    53.5910969687983f},
  { 37.0167508575842f,    50.2904398912844f},
  { 31.4257275700258f,    46.6266750339363f},
  { 26.5650511770780f,    42.7257778636242f},
  { 22.3578135700600f,    38.7116521932282f},
  { 35.1348647148694f,    55.0286031459780f},
  { 29.2556126306040f,    51.3896882964333f},
  { 24.2336676872848f,    47.4692538215910f},
  { 19.9562913769288f,    43.3932909455523f},
  { 26.7426486500324f,    55.8518607409545f},
  { 21.5833003880491f,    51.9411964822148f},
  { 17.2700954404738f,    47.8417144313409f},
  { 18.5576936186569f,    56.0877288234295f},
  { 14.2595776043354f,    51.9967277986567f},
  { 10.8822053021771f,    55.8145355246517f},
  { 82.6927335271427f,    -33.4884987864554f},
  { 79.1057669885777f,    -36.6365158565735f},
  { 74.9546561542605f,    -39.8605646102295f},
  { 70.1502822665428f,    -43.0802347503797f},
  { 64.6107072735303f,    -46.1904624830411f},
  { 58.2825255885390f,    -49.0661994333044f},
  { 51.1726569250027f,    -51.5740177485737f},
  { 43.3837914212946f,    -53.5910969687983f},
  { 35.1348647148694f,    -55.0286031459779f},
  { 26.7426486500324f,    -55.8518607409545f},
  { 18.5576936186569f,    -56.0877288234295f},
  { 10.8822053021771f,    -55.8145355246517f},
  { 77.9230649877361f,    -31.9320345172748f},
  { 73.9332208850461f,    -34.9768443320308f},
  { 69.3675539144640f,    -38.0549736525038f},
  { 64.1635743358256f,    -41.0756272443228f},
  { 58.2825255885390f,    -43.9275332607765f},
  { 51.7317026220431f,    -46.4882636786595f},
  { 44.5887483926325f,    -48.6401573061787f},
  { 37.0167508575842f,    -50.2904398912844f},
  { 29.2556126306040f,    -51.3896882964333f},
  { 21.5833003880491f,    -51.9411964822148f},
  { 14.2595776043354f,    -51.9967277986567f},
  { 73.0270942134697f,    -30.0855362676247f},
  { 68.6806178175289f,    -32.9853991131827f},
  { 63.7752769377408f,    -35.8746974937919f},
  { 58.2825255885390f,    -38.6575963529091f},
  { 52.2095478183208f,    -41.2246619591039f},
  { 45.6163420286281f,    -43.4656373529475f},
  { 38.6269447803609f,    -45.2865532254442f},
  { 31.4257275700258f,    -46.6266750339363f},
  { 24.2336676872848f,    -47.4692538215910f},
  { 17.2700954404738f,    -47.8417144313409f},
  { 68.0730689456761f,    -27.9522875976322f},
  { 63.4349488229220f,    -30.6726725373555f},
  { 58.2825255885390f,    -33.3424036470909f},
  { 52.6226318593503f,    -35.8669489234169f},
  { 46.5025109077962f,    -38.1462352379972f},
  { 40.0182492109711f,    -40.0886354813392f},
  { 33.3135536076770f,    -41.6261313598461f},
  { 26.5650511770780f,    -42.7257778636242f},
  { 19.9562913769288f,    -43.3932909455523f},
  { 63.1342382541714f,    -25.5544940369357f},
  { 58.2825255885390f,    -28.0724667392235f},
  { 52.9832491424158f,    -30.5080014196374f},
  { 47.2742221363758f,    -32.7737057972756f},
  { 41.2311043609539f,    -34.7840430571184f},
  { 34.9674044813552f,    -36.4682426764923f},
  { 28.6252826074369f,    -37.7816432786130f},
  { 22.3578135700600f,    -38.7116521932282f},
  { 58.2825255885390f,    -22.9338005666956f},
  { 53.3007747995101f,    -25.2406273115173f},
  { 47.9520724581596f,    -27.4443942526968f},
  { 42.2968675884159f,    -29.4688797542767f},
  { 36.4259321088004f,    -31.2450805567779f},
  { 30.4545634390528f,    -32.7215661485233f},
  { 24.5104108657590f,    -33.8716956871667f},
  { 53.5824807116749f,    -20.1483462191348f},
  { 48.5520446456536f,    -22.2495231164141f},
  { 43.2401471431053f,    -24.2387427969541f},
  { 37.7202134140217f,    -26.0526065796734f},
  { 32.0864948242697f,    -27.6377926894235f},
  { 26.4456546236829f,    -28.9584386201125f},
  { 49.0867190496941f,    -17.2666558898369f},
  { 44.0804766102169f,    -19.1794139209880f},
  { 38.8753897469057f,    -20.9814763421170f},
  { 33.5491126061725f,    -22.6217951428160f},
  { 28.1910708643724f,    -24.0600964664452f},
  { 44.8335287387352f,    -14.3597683383897f},
  { 39.9119385723822f,    -16.1089297860205f},
  { 34.8658259765881f,    -17.7559305016477f},
  { 29.7704751700052f,    -19.2602522246902f},
  { 40.8466553841700f,    -11.4935660517816f},
  { 36.0562259208489f,    -13.1070197584612f},
  { 31.2043868376226f,    -14.6309156267791f},
  { 37.1367620427039f,    -8.72301827225605f},
  { 32.5104534680331f,    -10.2279132248474f},
  { 33.7038507373336f,    -6.08918401662294f},
  { -10.8822053021772f,    55.8145355246517f},
  { -14.2595776043354f,    51.9967277986567f},
  { -17.2700954404738f,    47.8417144313409f},
  { -19.9562913769288f,    43.3932909455523f},
  { -22.3578135700600f,    38.7116521932282f},
  { -24.5104108657590f,    33.8716956871667f},
  { -26.4456546236829f,    28.9584386201125f},
  { -28.1910708643724f,    24.0600964664452f},
  { -29.7704751700052f,    19.2602522246902f},
  { -31.2043868376226f,    14.6309156267791f},
  { -32.5104534680331f,    10.2279132248474f},
  { -33.7038507373336f,    6.08918401662294f},
  { -18.5576936186569f,    56.0877288234295f},
  { -21.5833003880491f,    51.9411964822148f},
  { -24.2336676872848f,    47.4692538215910f},
  { -26.5650511770780f,    42.7257778636242f},
  { -28.6252826074369f,    37.7816432786130f},
  { -30.4545634390528f,    32.7215661485233f},
  { -32.0864948242697f,    27.6377926894235f},
  { -33.5491126061725f,    22.6217951428160f},
  { -34.8658259765881f,    17.7559305016477f},
  { -36.0562259208489f,    13.1070197584612f},
  { -37.1367620427039f,    8.72301827225606f},
  { -26.7426486500324f,    55.8518607409545f},
  { -29.2556126306040f,    51.3896882964333f},
  { -31.4257275700258f,    46.6266750339363f},
  { -33.3135536076770f,    41.6261313598461f},
  { -34.9674044813552f,    36.4682426764923f},
  { -36.4259321088004f,    31.2450805567779f},
  { -37.7202134140217f,    26.0526065796734f},
  { -38.8753897469057f,    20.9814763421170f},
  { -39.9119385723822f,    16.1089297860205f},
  { -40.8466553841700f,    11.4935660517816f},
  { -35.1348647148694f,    55.0286031459780f},
  { -37.0167508575842f,    50.2904398912844f},
  { -38.6269447803609f,    45.2865532254442f},
  { -40.0182492109711f,    40.0886354813392f},
  { -41.2311043609539f,    34.7840430571184f},
  { -42.2968675884159f,    29.4688797542767f},
  { -43.2401471431053f,    24.2387427969541f},
  { -44.0804766102169f,    19.1794139209880f},
  { -44.8335287387352f,    14.3597683383897f},
  { -43.3837914212946f,    53.5910969687983f},
  { -44.5887483926325f,    48.6401573061787f},
  { -45.6163420286281f,    43.4656373529475f},
  { -46.5025109077962f,    38.1462352379973f},
  { -47.2742221363758f,    32.7737057972756f},
  { -47.9520724581596f,    27.4443942526968f},
  { -48.5520446456536f,    22.2495231164141f},
  { -49.0867190496941f,    17.2666558898369f},
  { -51.1726569250027f,    51.5740177485737f},
  { -51.7317026220431f,    46.4882636786595f},
  { -52.2095478183208f,    41.2246619591039f},
  { -52.6226318593503f,    35.8669489234169f},
  { -52.9832491424158f,    30.5080014196374f},
  { -53.3007747995101f,    25.2406273115173f},
  { -53.5824807116749f,    20.1483462191348f},
  { -58.2825255885390f,    49.0661994333044f},
  { -58.2825255885390f,    43.9275332607765f},
  { -58.2825255885390f,    38.6575963529091f},
  { -58.2825255885390f,    33.3424036470909f},
  { -58.2825255885390f,    28.0724667392235f},
  { -58.2825255885390f,    22.9338005666956f},
  { -64.6107072735303f,    46.1904624830411f},
  { -64.1635743358256f,    41.0756272443228f},
  { -63.7752769377408f,    35.8746974937919f},
  { -63.4349488229220f,    30.6726725373555f},
  { -63.1342382541714f,    25.5544940369357f},
  { -70.1502822665428f,    43.0802347503797f},
  { -69.3675539144640f,    38.0549736525038f},
  { -68.6806178175289f,    32.9853991131827f},
  { -68.0730689456761f,    27.9522875976322f},
  { -74.9546561542605f,    39.8605646102295f},
  { -73.9332208850461f,    34.9768443320308f},
  { -73.0270942134698f,    30.0855362676247f},
  { -79.1057669885777f,    36.6365158565735f},
  { -77.9230649877361f,    31.9320345172748f},
  { -82.6927335271427f,    33.4884987864554f},
  { -10.8822053021772f,    -55.8145355246517f},
  { -18.5576936186569f,    -56.0877288234295f},
  { -26.7426486500324f,    -55.8518607409545f},
  { -35.1348647148694f,    -55.0286031459780f},
  { -43.3837914212946f,    -53.5910969687983f},
  { -51.1726569250027f,    -51.5740177485737f},
  { -58.2825255885390f,    -49.0661994333044f},
  { -64.6107072735303f,    -46.1904624830411f},
  { -70.1502822665428f,    -43.0802347503797f},
  { -74.9546561542605f,    -39.8605646102295f},
  { -79.1057669885777f,    -36.6365158565735f},
  { -82.6927335271427f,    -33.4884987864554f},
  { -14.2595776043354f,    -51.9967277986567f},
  { -21.5833003880491f,    -51.9411964822148f},
  { -29.2556126306040f,    -51.3896882964333f},
  { -37.0167508575842f,    -50.2904398912844f},
  { -44.5887483926325f,    -48.6401573061787f},
  { -51.7317026220431f,    -46.4882636786595f},
  { -58.2825255885390f,    -43.9275332607765f},
  { -64.1635743358256f,    -41.0756272443228f},
  { -69.3675539144640f,    -38.0549736525038f},
  { -73.9332208850461f,    -34.9768443320308f},
  { -77.9230649877361f,    -31.9320345172748f},
  { -17.2700954404738f,    -47.8417144313409f},
  { -24.2336676872848f,    -47.4692538215910f},
  { -31.4257275700258f,    -46.6266750339363f},
  { -38.6269447803609f,    -45.2865532254442f},
  { -45.6163420286281f,    -43.4656373529475f},
  { -52.2095478183208f,    -41.2246619591039f},
  { -58.2825255885390f,    -38.6575963529091f},
  { -63.7752769377408f,    -35.8746974937919f},
  { -68.6806178175289f,    -32.9853991131827f},
  { -73.0270942134697f,    -30.0855362676247f},
  { -19.9562913769288f,    -43.3932909455523f},
  { -26.5650511770780f,    -42.7257778636242f},
  { -33.3135536076770f,    -41.6261313598461f},
  { -40.0182492109711f,    -40.0886354813392f},
  { -46.5025109077962f,    -38.1462352379972f},
  { -52.6226318593503f,    -35.8669489234169f},
  { -58.2825255885390f,    -33.3424036470909f},
  { -63.4349488229220f,    -30.6726725373555f},
  { -68.0730689456761f,    -27.9522875976322f},
  { -22.3578135700600f,    -38.7116521932282f},
  { -28.6252826074369f,    -37.7816432786130f},
  { -34.9674044813552f,    -36.4682426764923f},
  { -41.2311043609539f,    -34.7840430571184f},
  { -47.2742221363758f,    -32.7737057972756f},
  { -52.9832491424158f,    -30.5080014196374f},
  { -58.2825255885390f,    -28.0724667392235f},
  { -63.1342382541714f,    -25.5544940369357f},
  { -24.5104108657590f,    -33.8716956871667f},
  { -30.4545634390528f,    -32.7215661485233f},
  { -36.4259321088004f,    -31.2450805567779f},
  { -42.2968675884159f,    -29.4688797542767f},
  { -47.9520724581596f,    -27.4443942526968f},
  { -53.3007747995101f,    -25.2406273115173f},
  { -58.2825255885390f,    -22.9338005666956f},
  { -26.4456546236829f,    -28.9584386201125f},
  { -32.0864948242697f,    -27.6377926894235f},
  { -37.7202134140217f,    -26.0526065796734f},
  { -43.2401471431053f,    -24.2387427969541f},
  { -48.5520446456536f,    -22.2495231164141f},
  { -53.5824807116749f,    -20.1483462191348f},
  { -28.1910708643724f,    -24.0600964664452f},
  { -33.5491126061725f,    -22.6217951428160f},
  { -38.8753897469057f,    -20.9814763421170f},
  { -44.0804766102169f,    -19.1794139209880f},
  { -49.0867190496941f,    -17.2666558898369f},
  { -29.7704751700052f,    -19.2602522246902f},
  { -34.8658259765881f,    -17.7559305016477f},
  { -39.9119385723822f,    -16.1089297860205f},
  { -44.8335287387352f,    -14.3597683383897f},
  { -31.2043868376226f,    -14.6309156267791f},
  { -36.0562259208489f,    -13.1070197584612f},
  { -40.8466553841700f,    -11.4935660517816f},
  { -32.5104534680331f,    -10.2279132248474f},
  { -37.1367620427039f,    -8.72301827225605f},
  { -33.7038507373336f,    -6.08918401662294f},
  { 169.117794697823f,    55.8145355246517f},
  { 165.740422395665f,    51.9967277986567f},
  { 162.729904559526f,    47.8417144313409f},
  { 160.043708623071f,    43.3932909455523f},
  { 157.642186429940f,    38.7116521932282f},
  { 155.489589134241f,    33.8716956871667f},
  { 153.554345376317f,    28.9584386201125f},
  { 151.808929135628f,    24.0600964664452f},
  { 150.229524829995f,    19.2602522246902f},
  { 148.795613162377f,    14.6309156267791f},
  { 147.489546531967f,    10.2279132248474f},
  { 146.296149262666f,    6.08918401662294f},
  { 161.442306381343f,    56.0877288234295f},
  { 158.416699611951f,    51.9411964822148f},
  { 155.766332312715f,    47.4692538215910f},
  { 153.434948822922f,    42.7257778636242f},
  { 151.374717392563f,    37.7816432786130f},
  { 149.545436560947f,    32.7215661485233f},
  { 147.913505175730f,    27.6377926894235f},
  { 146.450887393828f,    22.6217951428160f},
  { 145.134174023412f,    17.7559305016477f},
  { 143.943774079151f,    13.1070197584612f},
  { 142.863237957296f,    8.72301827225606f},
  { 153.257351349968f,    55.8518607409545f},
  { 150.744387369396f,    51.3896882964333f},
  { 148.574272429974f,    46.6266750339363f},
  { 146.686446392323f,    41.6261313598461f},
  { 145.032595518645f,    36.4682426764923f},
  { 143.574067891200f,    31.2450805567779f},
  { 142.279786585978f,    26.0526065796734f},
  { 141.124610253094f,    20.9814763421170f},
  { 140.088061427618f,    16.1089297860205f},
  { 139.153344615830f,    11.4935660517816f},
  { 144.865135285131f,    55.0286031459780f},
  { 142.983249142416f,    50.2904398912844f},
  { 141.373055219639f,    45.2865532254442f},
  { 139.981750789029f,    40.0886354813392f},
  { 138.768895639046f,    34.7840430571184f},
  { 137.703132411584f,    29.4688797542767f},
  { 136.759852856895f,    24.2387427969541f},
  { 135.919523389783f,    19.1794139209880f},
  { 135.166471261265f,    14.3597683383897f},
  { 136.616208578705f,    53.5910969687983f},
  { 135.411251607368f,    48.6401573061787f},
  { 134.383657971372f,    43.4656373529475f},
  { 133.497489092204f,    38.1462352379973f},
  { 132.725777863624f,    32.7737057972756f},
  { 132.047927541840f,    27.4443942526968f},
  { 131.447955354346f,    22.2495231164141f},
  { 130.913280950306f,    17.2666558898369f},
  { 128.827343074997f,    51.5740177485737f},
  { 128.268297377957f,    46.4882636786595f},
  { 127.790452181679f,    41.2246619591039f},
  { 127.377368140650f,    35.8669489234169f},
  { 127.016750857584f,    30.5080014196374f},
  { 126.699225200490f,    25.2406273115173f},
  { 126.417519288325f,    20.1483462191348f},
  { 121.717474411461f,    49.0661994333044f},
  { 121.717474411461f,    43.9275332607765f},
  { 121.717474411461f,    38.6575963529091f},
  { 121.717474411461f,    33.3424036470909f},
  { 121.717474411461f,    28.0724667392235f},
  { 121.717474411461f,    22.9338005666956f},
  { 115.389292726470f,    46.1904624830411f},
  { 115.836425664174f,    41.0756272443228f},
  { 116.224723062259f,    35.8746974937919f},
  { 116.565051177078f,    30.6726725373555f},
  { 116.865761745829f,    25.5544940369357f},
  { 109.849717733457f,    43.0802347503797f},
  { 110.632446085536f,    38.0549736525038f},
  { 111.319382182471f,    32.9853991131827f},
  { 111.926931054324f,    27.9522875976322f},
  { 105.045343845740f,    39.8605646102295f},
  { 106.066779114954f,    34.9768443320308f},
  { 106.972905786530f,    30.0855362676247f},
  { 100.894233011422f,    36.6365158565735f},
  { 102.076935012264f,    31.9320345172748f},
  { 97.3072664728573f,    33.4884987864554f},
  { 169.117794697823f,    -55.8145355246517f},
  { 161.442306381343f,    -56.0877288234295f},
  { 153.257351349968f,    -55.8518607409545f},
  { 144.865135285131f,    -55.0286031459780f},
  { 136.616208578705f,    -53.5910969687983f},
  { 128.827343074997f,    -51.5740177485737f},
  { 121.717474411461f,    -49.0661994333044f},
  { 115.389292726470f,    -46.1904624830411f},
  { 109.849717733457f,    -43.0802347503797f},
  { 105.045343845740f,    -39.8605646102295f},
  { 100.894233011422f,    -36.6365158565735f},
  { 97.3072664728573f,    -33.4884987864554f},
  { 165.740422395665f,    -51.9967277986567f},
  { 158.416699611951f,    -51.9411964822148f},
  { 150.744387369396f,    -51.3896882964333f},
  { 142.983249142416f,    -50.2904398912844f},
  { 135.411251607368f,    -48.6401573061787f},
  { 128.268297377957f,    -46.4882636786595f},
  { 121.717474411461f,    -43.9275332607765f},
  { 115.836425664174f,    -41.0756272443228f},
  { 110.632446085536f,    -38.0549736525038f},
  { 106.066779114954f,    -34.9768443320308f},
  { 102.076935012264f,    -31.9320345172748f},
  { 162.729904559526f,    -47.8417144313409f},
  { 155.766332312715f,    -47.4692538215910f},
  { 148.574272429974f,    -46.6266750339363f},
  { 141.373055219639f,    -45.2865532254442f},
  { 134.383657971372f,    -43.4656373529475f},
  { 127.790452181679f,    -41.2246619591039f},
  { 121.717474411461f,    -38.6575963529091f},
  { 116.224723062259f,    -35.8746974937919f},
  { 111.319382182471f,    -32.9853991131827f},
  { 106.972905786530f,    -30.0855362676247f},
  { 160.043708623071f,    -43.3932909455523f},
  { 153.434948822922f,    -42.7257778636242f},
  { 146.686446392323f,    -41.6261313598461f},
  { 139.981750789029f,    -40.0886354813392f},
  { 133.497489092204f,    -38.1462352379972f},
  { 127.377368140650f,    -35.8669489234169f},
  { 121.717474411461f,    -33.3424036470909f},
  { 116.565051177078f,    -30.6726725373555f},
  { 111.926931054324f,    -27.9522875976322f},
  { 157.642186429940f,    -38.7116521932282f},
  { 151.374717392563f,    -37.7816432786130f},
  { 145.032595518645f,    -36.4682426764923f},
  { 138.768895639046f,    -34.7840430571184f},
  { 132.725777863624f,    -32.7737057972756f},
  { 127.016750857584f,    -30.5080014196374f},
  { 121.717474411461f,    -28.0724667392235f},
  { 116.865761745829f,    -25.5544940369357f},
  { 155.489589134241f,    -33.8716956871667f},
  { 149.545436560947f,    -32.7215661485233f},
  { 143.574067891200f,    -31.2450805567779f},
  { 137.703132411584f,    -29.4688797542767f},
  { 132.047927541840f,    -27.4443942526968f},
  { 126.699225200490f,    -25.2406273115173f},
  { 121.717474411461f,    -22.9338005666956f},
  { 153.554345376317f,    -28.9584386201125f},
  { 147.913505175730f,    -27.6377926894235f},
  { 142.279786585978f,    -26.0526065796734f},
  { 136.759852856895f,    -24.2387427969541f},
  { 131.447955354346f,    -22.2495231164141f},
  { 126.417519288325f,    -20.1483462191348f},
  { 151.808929135628f,    -24.0600964664452f},
  { 146.450887393828f,    -22.6217951428160f},
  { 141.124610253094f,    -20.9814763421170f},
  { 135.919523389783f,    -19.1794139209880f},
  { 130.913280950306f,    -17.2666558898369f},
  { 150.229524829995f,    -19.2602522246902f},
  { 145.134174023412f,    -17.7559305016477f},
  { 140.088061427618f,    -16.1089297860205f},
  { 135.166471261265f,    -14.3597683383897f},
  { 148.795613162377f,    -14.6309156267791f},
  { 143.943774079151f,    -13.1070197584612f},
  { 139.153344615830f,    -11.4935660517816f},
  { 147.489546531967f,    -10.2279132248474f},
  { 142.863237957296f,    -8.72301827225605f},
  { 146.296149262666f,    -6.08918401662294f},
  { -169.117794697823f,    55.8145355246517f},
  { -161.442306381343f,    56.0877288234295f},
  { -153.257351349968f,    55.8518607409545f},
  { -144.865135285131f,    55.0286031459780f},
  { -136.616208578705f,    53.5910969687983f},
  { -128.827343074997f,    51.5740177485737f},
  { -121.717474411461f,    49.0661994333044f},
  { -115.389292726470f,    46.1904624830411f},
  { -109.849717733457f,    43.0802347503797f},
  { -105.045343845740f,    39.8605646102295f},
  { -100.894233011422f,    36.6365158565735f},
  { -97.3072664728573f,    33.4884987864554f},
  { -165.740422395665f,    51.9967277986567f},
  { -158.416699611951f,    51.9411964822148f},
  { -150.744387369396f,    51.3896882964333f},
  { -142.983249142416f,    50.2904398912844f},
  { -135.411251607368f,    48.6401573061787f},
  { -128.268297377957f,    46.4882636786595f},
  { -121.717474411461f,    43.9275332607765f},
  { -115.836425664174f,    41.0756272443228f},
  { -110.632446085536f,    38.0549736525038f},
  { -106.066779114954f,    34.9768443320308f},
  { -102.076935012264f,    31.9320345172748f},
  { -162.729904559526f,    47.8417144313409f},
  { -155.766332312715f,    47.4692538215910f},
  { -148.574272429974f,    46.6266750339363f},
  { -141.373055219639f,    45.2865532254442f},
  { -134.383657971372f,    43.4656373529475f},
  { -127.790452181679f,    41.2246619591039f},
  { -121.717474411461f,    38.6575963529091f},
  { -116.224723062259f,    35.8746974937919f},
  { -111.319382182471f,    32.9853991131827f},
  { -106.972905786530f,    30.0855362676247f},
  { -160.043708623071f,    43.3932909455523f},
  { -153.434948822922f,    42.7257778636242f},
  { -146.686446392323f,    41.6261313598461f},
  { -139.981750789029f,    40.0886354813392f},
  { -133.497489092204f,    38.1462352379972f},
  { -127.377368140650f,    35.8669489234169f},
  { -121.717474411461f,    33.3424036470909f},
  { -116.565051177078f,    30.6726725373555f},
  { -111.926931054324f,    27.9522875976322f},
  { -157.642186429940f,    38.7116521932282f},
  { -151.374717392563f,    37.7816432786130f},
  { -145.032595518645f,    36.4682426764923f},
  { -138.768895639046f,    34.7840430571184f},
  { -132.725777863624f,    32.7737057972756f},
  { -127.016750857584f,    30.5080014196374f},
  { -121.717474411461f,    28.0724667392235f},
  { -116.865761745829f,    25.5544940369357f},
  { -155.489589134241f,    33.8716956871667f},
  { -149.545436560947f,    32.7215661485233f},
  { -143.574067891200f,    31.2450805567779f},
  { -137.703132411584f,    29.4688797542767f},
  { -132.047927541840f,    27.4443942526968f},
  { -126.699225200490f,    25.2406273115173f},
  { -121.717474411461f,    22.9338005666956f},
  { -153.554345376317f,    28.9584386201125f},
  { -147.913505175730f,    27.6377926894235f},
  { -142.279786585978f,    26.0526065796734f},
  { -136.759852856895f,    24.2387427969541f},
  { -131.447955354346f,    22.2495231164141f},
  { -126.417519288325f,    20.1483462191348f},
  { -151.808929135628f,    24.0600964664452f},
  { -146.450887393828f,    22.6217951428160f},
  { -141.124610253094f,    20.9814763421170f},
  { -135.919523389783f,    19.1794139209880f},
  { -130.913280950306f,    17.2666558898369f},
  { -150.229524829995f,    19.2602522246902f},
  { -145.134174023412f,    17.7559305016477f},
  { -140.088061427618f,    16.1089297860205f},
  { -135.166471261265f,    14.3597683383897f},
  { -148.795613162377f,    14.6309156267791f},
  { -143.943774079151f,    13.1070197584612f},
  { -139.153344615830f,    11.4935660517816f},
  { -147.489546531967f,    10.2279132248474f},
  { -142.863237957296f,    8.72301827225605f},
  { -146.296149262666f,    6.08918401662294f},
  { -169.117794697823f,    -55.8145355246517f},
  { -165.740422395665f,    -51.9967277986567f},
  { -162.729904559526f,    -47.8417144313409f},
  { -160.043708623071f,    -43.3932909455523f},
  { -157.642186429940f,    -38.7116521932282f},
  { -155.489589134241f,    -33.8716956871667f},
  { -153.554345376317f,    -28.9584386201125f},
  { -151.808929135628f,    -24.0600964664452f},
  { -150.229524829995f,    -19.2602522246902f},
  { -148.795613162377f,    -14.6309156267791f},
  { -147.489546531967f,    -10.2279132248474f},
  { -146.296149262666f,    -6.08918401662294f},
  { -161.442306381343f,    -56.0877288234295f},
  { -158.416699611951f,    -51.9411964822148f},
  { -155.766332312715f,    -47.4692538215910f},
  { -153.434948822922f,    -42.7257778636242f},
  { -151.374717392563f,    -37.7816432786130f},
  { -149.545436560947f,    -32.7215661485233f},
  { -147.913505175730f,    -27.6377926894235f},
  { -146.450887393828f,    -22.6217951428160f},
  { -145.134174023412f,    -17.7559305016477f},
  { -143.943774079151f,    -13.1070197584612f},
  { -142.863237957296f,    -8.72301827225606f},
  { -153.257351349968f,    -55.8518607409545f},
  { -150.744387369396f,    -51.3896882964333f},
  { -148.574272429974f,    -46.6266750339363f},
  { -146.686446392323f,    -41.6261313598461f},
  { -145.032595518645f,    -36.4682426764923f},
  { -143.574067891200f,    -31.2450805567779f},
  { -142.279786585978f,    -26.0526065796734f},
  { -141.124610253094f,    -20.9814763421170f},
  { -140.088061427618f,    -16.1089297860205f},
  { -139.153344615830f,    -11.4935660517816f},
  { -144.865135285131f,    -55.0286031459780f},
  { -142.983249142416f,    -50.2904398912844f},
  { -141.373055219639f,    -45.2865532254442f},
  { -139.981750789029f,    -40.0886354813392f},
  { -138.768895639046f,    -34.7840430571184f},
  { -137.703132411584f,    -29.4688797542767f},
  { -136.759852856895f,    -24.2387427969541f},
  { -135.919523389783f,    -19.1794139209880f},
  { -135.166471261265f,    -14.3597683383897f},
  { -136.616208578705f,    -53.5910969687983f},
  { -135.411251607368f,    -48.6401573061787f},
  { -134.383657971372f,    -43.4656373529475f},
  { -133.497489092204f,    -38.1462352379973f},
  { -132.725777863624f,    -32.7737057972756f},
  { -132.047927541840f,    -27.4443942526968f},
  { -131.447955354346f,    -22.2495231164141f},
  { -130.913280950306f,    -17.2666558898369f},
  { -128.827343074997f,    -51.5740177485737f},
  { -128.268297377957f,    -46.4882636786595f},
  { -127.790452181679f,    -41.2246619591039f},
  { -127.377368140650f,    -35.8669489234169f},
  { -127.016750857584f,    -30.5080014196374f},
  { -126.699225200490f,    -25.2406273115173f},
  { -126.417519288325f,    -20.1483462191348f},
  { -121.717474411461f,    -49.0661994333044f},
  { -121.717474411461f,    -43.9275332607765f},
  { -121.717474411461f,    -38.6575963529091f},
  { -121.717474411461f,    -33.3424036470909f},
  { -121.717474411461f,    -28.0724667392235f},
  { -121.717474411461f,    -22.9338005666956f},
  { -115.389292726470f,    -46.1904624830411f},
  { -115.836425664174f,    -41.0756272443228f},
  { -116.224723062259f,    -35.8746974937919f},
  { -116.565051177078f,    -30.6726725373555f},
  { -116.865761745829f,    -25.5544940369357f},
  { -109.849717733457f,    -43.0802347503797f},
  { -110.632446085536f,    -38.0549736525038f},
  { -111.319382182471f,    -32.9853991131827f},
  { -111.926931054324f,    -27.9522875976322f},
  { -105.045343845740f,    -39.8605646102295f},
  { -106.066779114954f,    -34.9768443320308f},
  { -106.972905786530f,    -30.0855362676247f},
  { -100.894233011422f,    -36.6365158565735f},
  { -102.076935012264f,    -31.9320345172748f},
  { -97.3072664728573f,    -33.4884987864554f}};

/* 3LD geosphere 'ico' [15 0] (2252 points) */
const float __geosphere_ico_15_0_dirs_deg[2252][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    61.8251865351426f},
  { 0.0f,    65.6187815578030f},
  { 0.0f,    69.6541923856616f},
  { 0.0f,    73.9115997418271f},
  { 0.0f,    78.3592768632294f},
  { 0.0f,    82.9535936191428f},
  { 0.0f,    87.6406185690461f},
  { 180.0f,    87.6406185690461f},
  { 180.0f,    82.9535936191428f},
  { 180.0f,    78.3592768632294f},
  { 180.0f,    73.9115997418271f},
  { 180.0f,    69.6541923856616f},
  { 180.0f,    65.6187815578030f},
  { 180.0f,    61.8251865351426f},
  { 6.59264467355895f,    59.2116443441111f},
  { 7.67922857366964f,    62.8903831093188f},
  { 9.19099647608508f,    66.8151622059356f},
  { 11.4340697442537f,    70.9614896019555f},
  { 15.0920752881032f,    75.2790827296451f},
  { 22.0237505161462f,    79.6613630728832f},
  { 38.9734473368657f,    83.7926432688605f},
  { 90.0f,    86.0865699753911f},
  { 141.026552663134f,    83.7926432688605f},
  { 157.976249483854f,    79.6613630728832f},
  { 164.907924711897f,    75.2790827296451f},
  { 168.565930255746f,    70.9614896019555f},
  { 170.809003523915f,    66.8151622059356f},
  { 172.320771426330f,    62.8903831093188f},
  { 173.407355326441f,    59.2116443441111f},
  { 13.9784373388714f,    59.8179223197217f},
  { 16.3932501484725f,    63.5367057973473f},
  { 19.7767131841238f,    67.4513372081180f},
  { 24.8108847601265f,    71.4897072290348f},
  { 32.9114931857064f,    75.5029442615280f},
  { 47.1679114977869f,    79.1551986976493f},
  { 72.8279621494862f,    81.6349995128180f},
  { 107.172037850514f,    81.6349995128180f},
  { 132.832088502213f,    79.1551986976493f},
  { 147.088506814294f,    75.5029442615280f},
  { 155.189115239873f,    71.4897072290348f},
  { 160.223286815876f,    67.4513372081180f},
  { 163.606749851527f,    63.5367057973473f},
  { 166.021562661129f,    59.8179223197217f},
  { 22.0237505161462f,    59.9952713133382f},
  { 25.8924090881804f,    63.6240862949495f},
  { 31.2478160930689f,    67.3424609596128f},
  { 38.9734473368657f,    71.0021166134344f},
  { 50.5099309427673f,    74.3263225812526f},
  { 67.6072290710451f,    76.8187518549304f},
  { 90.0f,    77.7816801337513f},
  { 112.392770928955f,    76.8187518549304f},
  { 129.490069057233f,    74.3263225812526f},
  { 141.026552663134f,    71.0021166134344f},
  { 148.752183906931f,    67.3424609596128f},
  { 154.107590911820f,    63.6240862949495f},
  { 157.976249483854f,    59.9952713133382f},
  { 30.4715337744227f,    59.6513563396351f},
  { 35.7209019991337f,    63.0444971189200f},
  { 42.7561895905164f,    66.3777711696528f},
  { 52.3123893000598f,    69.4343887930543f},
  { 65.1310748815380f,    71.8790718535498f},
  { 81.2167693990410f,    73.2780371655037f},
  { 98.7832306009590f,    73.2780371655037f},
  { 114.868925118462f,    71.8790718535498f},
  { 127.687610699940f,    69.4343887930543f},
  { 137.243810409484f,    66.3777711696528f},
  { 144.279098000866f,    63.0444971189200f},
  { 149.528466225577f,    59.6513563396351f},
  { 38.9734473368657f,    58.7297899739513f},
  { 45.3210844354825f,    61.7562141472517f},
  { 53.4378607612508f,    64.5666406655262f},
  { 63.6909565444517f,    66.9209866913203f},
  { 76.1141128518623f,    68.5221100088469f},
  { 90.0f,    69.0948425521107f},
  { 103.885887148138f,    68.5221100088469f},
  { 116.309043455548f,    66.9209866913203f},
  { 126.562139238749f,    64.5666406655263f},
  { 134.678915564518f,    61.7562141472517f},
  { 141.026552663134f,    58.7297899739513f},
  { 47.1679114977868f,    57.2262493118836f},
  { 54.2069014710865f,    59.7976502527467f},
  { 62.7502499362301f,    62.0283401594414f},
  { 72.8279621494862f,    63.7030502822517f},
  { 84.1189512527134f,    64.6093910405445f},
  { 95.8810487472866f,    64.6093910405445f},
  { 107.172037850514f,    63.7030502822517f},
  { 117.249750063770f,    62.0283401594414f},
  { 125.793098528914f,    59.7976502527467f},
  { 132.832088502213f,    57.2262493118836f},
  { 54.7655000391642f,    55.1902992552913f},
  { 62.0878885994974f,    57.2729027490462f},
  { 70.5487009486867f,    58.9407544608438f},
  { 79.9858774849308f,    60.0288091226086f},
  { 90.0f,    60.4080049704701f},
  { 100.014122515069f,    60.0288091226086f},
  { 109.451299051313f,    58.9407544608438f},
  { 117.912111400503f,    57.2729027490462f},
  { 125.234499960836f,    55.1902992552913f},
  { 61.5963754664474f,    52.7128866391432f},
  { 68.8798871556760f,    54.3213589461635f},
  { 76.9513489599765f,    55.4903085181429f},
  { 85.5824320004692f,    56.1078668060580f},
  { 94.4175679995308f,    56.1078668060580f},
  { 103.048651040024f,    55.4903085181429f},
  { 111.120112844324f,    54.3213589461635f},
  { 118.403624533553f,    52.7128866391432f},
  { 67.6072290710451f,    49.9068566415137f},
  { 74.6407012762525f,    51.0887556818155f},
  { 82.1798710590254f,    51.8433375502915f},
  { 90.0f,    52.1031151288303f},
  { 97.8201289409746f,    51.8433375502915f},
  { 105.359298723748f,    51.0887556818155f},
  { 112.392770928955f,    49.9068566415137f},
  { 72.8279621494862f,    46.8885380691238f},
  { 79.4960587445786f,    47.7081148747826f},
  { 86.4634243465556f,    48.1354091502336f},
  { 93.5365756534444f,    48.1354091502336f},
  { 100.503941255421f,    47.7081148747826f},
  { 107.172037850514f,    46.8885380691238f},
  { 77.3338164400891f,    43.7646105401545f},
  { 83.5885791392606f,    44.2903590151516f},
  { 90.0f,    44.4700523736346f},
  { 96.4114208607394f,    44.2903590151516f},
  { 102.666183559911f,    43.7646105401545f},
  { 81.2167693990410f,    40.6247965576687f},
  { 87.0517100501096f,    40.9215270870819f},
  { 92.9482899498904f,    40.9215270870819f},
  { 98.7832306009590f,    40.6247965576687f},
  { 84.5685226684728f,    37.5391973811345f},
  { 90.0f,    37.6638374535070f},
  { 95.4314773315273f,    37.5391973811345f},
  { 87.4723026341838f,    34.5587491208397f},
  { 92.5276973658163f,    34.5587491208397f},
  { -6.59264467355895f,    59.2116443441111f},
  { -13.9784373388714f,    59.8179223197217f},
  { -22.0237505161462f,    59.9952713133382f},
  { -30.4715337744227f,    59.6513563396351f},
  { -38.9734473368657f,    58.7297899739513f},
  { -47.1679114977868f,    57.2262493118836f},
  { -54.7655000391642f,    55.1902992552913f},
  { -61.5963754664474f,    52.7128866391432f},
  { -67.6072290710451f,    49.9068566415137f},
  { -72.8279621494862f,    46.8885380691238f},
  { -77.3338164400891f,    43.7646105401545f},
  { -81.2167693990410f,    40.6247965576687f},
  { -84.5685226684728f,    37.5391973811345f},
  { -87.4723026341838f,    34.5587491208397f},
  { -7.67922857366964f,    62.8903831093188f},
  { -16.3932501484725f,    63.5367057973473f},
  { -25.8924090881804f,    63.6240862949495f},
  { -35.7209019991337f,    63.0444971189200f},
  { -45.3210844354825f,    61.7562141472517f},
  { -54.2069014710865f,    59.7976502527467f},
  { -62.0878885994974f,    57.2729027490462f},
  { -68.8798871556760f,    54.3213589461635f},
  { -74.6407012762525f,    51.0887556818155f},
  { -79.4960587445786f,    47.7081148747826f},
  { -83.5885791392606f,    44.2903590151516f},
  { -87.0517100501096f,    40.9215270870819f},
  { -90.0f,    37.6638374535070f},
  { -92.5276973658163f,    34.5587491208397f},
  { -9.19099647608508f,    66.8151622059356f},
  { -19.7767131841238f,    67.4513372081180f},
  { -31.2478160930689f,    67.3424609596128f},
  { -42.7561895905164f,    66.3777711696528f},
  { -53.4378607612508f,    64.5666406655262f},
  { -62.7502499362301f,    62.0283401594414f},
  { -70.5487009486867f,    58.9407544608438f},
  { -76.9513489599765f,    55.4903085181429f},
  { -82.1798710590254f,    51.8433375502915f},
  { -86.4634243465556f,    48.1354091502336f},
  { -90.0f,    44.4700523736346f},
  { -92.9482899498904f,    40.9215270870819f},
  { -95.4314773315273f,    37.5391973811345f},
  { -11.4340697442537f,    70.9614896019555f},
  { -24.8108847601265f,    71.4897072290348f},
  { -38.9734473368657f,    71.0021166134344f},
  { -52.3123893000598f,    69.4343887930543f},
  { -63.6909565444517f,    66.9209866913203f},
  { -72.8279621494862f,    63.7030502822517f},
  { -79.9858774849308f,    60.0288091226086f},
  { -85.5824320004692f,    56.1078668060580f},
  { -90.0f,    52.1031151288303f},
  { -93.5365756534444f,    48.1354091502336f},
  { -96.4114208607394f,    44.2903590151516f},
  { -98.7832306009590f,    40.6247965576687f},
  { -15.0920752881032f,    75.2790827296451f},
  { -32.9114931857064f,    75.5029442615280f},
  { -50.5099309427673f,    74.3263225812526f},
  { -65.1310748815380f,    71.8790718535498f},
  { -76.1141128518623f,    68.5221100088469f},
  { -84.1189512527134f,    64.6093910405445f},
  { -90.0000000000000f,    60.4080049704701f},
  { -94.4175679995308f,    56.1078668060580f},
  { -97.8201289409746f,    51.8433375502915f},
  { -100.503941255421f,    47.7081148747826f},
  { -102.666183559911f,    43.7646105401545f},
  { -22.0237505161462f,    79.6613630728832f},
  { -47.1679114977868f,    79.1551986976493f},
  { -67.6072290710451f,    76.8187518549304f},
  { -81.2167693990410f,    73.2780371655037f},
  { -90.0f,    69.0948425521107f},
  { -95.8810487472866f,    64.6093910405445f},
  { -100.014122515069f,    60.0288091226086f},
  { -103.048651040024f,    55.4903085181429f},
  { -105.359298723748f,    51.0887556818155f},
  { -107.172037850514f,    46.8885380691238f},
  { -38.9734473368657f,    83.7926432688605f},
  { -72.8279621494862f,    81.6349995128180f},
  { -90.0000000000000f,    77.7816801337513f},
  { -98.7832306009590f,    73.2780371655037f},
  { -103.885887148138f,    68.5221100088469f},
  { -107.172037850514f,    63.7030502822517f},
  { -109.451299051313f,    58.9407544608438f},
  { -111.120112844324f,    54.3213589461635f},
  { -112.392770928955f,    49.9068566415137f},
  { -90.0f,    86.0865699753911f},
  { -107.172037850514f,    81.6349995128180f},
  { -112.392770928955f,    76.8187518549304f},
  { -114.868925118462f,    71.8790718535498f},
  { -116.309043455548f,    66.9209866913203f},
  { -117.249750063770f,    62.0283401594414f},
  { -117.912111400503f,    57.2729027490462f},
  { -118.403624533553f,    52.7128866391432f},
  { -141.026552663134f,    83.7926432688605f},
  { -132.832088502213f,    79.1551986976493f},
  { -129.490069057233f,    74.3263225812526f},
  { -127.687610699940f,    69.4343887930543f},
  { -126.562139238749f,    64.5666406655263f},
  { -125.793098528914f,    59.7976502527467f},
  { -125.234499960836f,    55.1902992552913f},
  { -157.976249483854f,    79.6613630728832f},
  { -147.088506814294f,    75.5029442615280f},
  { -141.026552663134f,    71.0021166134344f},
  { -137.243810409484f,    66.3777711696528f},
  { -134.678915564518f,    61.7562141472517f},
  { -132.832088502213f,    57.2262493118837f},
  { -164.907924711897f,    75.2790827296451f},
  { -155.189115239873f,    71.4897072290348f},
  { -148.752183906931f,    67.3424609596128f},
  { -144.279098000866f,    63.0444971189200f},
  { -141.026552663134f,    58.7297899739513f},
  { -168.565930255746f,    70.9614896019555f},
  { -160.223286815876f,    67.4513372081180f},
  { -154.107590911820f,    63.6240862949495f},
  { -149.528466225577f,    59.6513563396351f},
  { -170.809003523915f,    66.8151622059356f},
  { -163.606749851527f,    63.5367057973473f},
  { -157.976249483854f,    59.9952713133382f},
  { -172.320771426330f,    62.8903831093188f},
  { -166.021562661129f,    59.8179223197217f},
  { -173.407355326441f,    59.2116443441111f},
  { 6.59264467355895f,    -59.2116443441111f},
  { 13.9784373388714f,    -59.8179223197217f},
  { 22.0237505161462f,    -59.9952713133382f},
  { 30.4715337744227f,    -59.6513563396351f},
  { 38.9734473368657f,    -58.7297899739513f},
  { 47.1679114977868f,    -57.2262493118836f},
  { 54.7655000391642f,    -55.1902992552913f},
  { 61.5963754664474f,    -52.7128866391432f},
  { 67.6072290710451f,    -49.9068566415137f},
  { 72.8279621494862f,    -46.8885380691238f},
  { 77.3338164400891f,    -43.7646105401545f},
  { 81.2167693990410f,    -40.6247965576687f},
  { 84.5685226684728f,    -37.5391973811345f},
  { 87.4723026341838f,    -34.5587491208397f},
  { 0.0f,    -61.8251865351426f},
  { 7.67922857366964f,    -62.8903831093188f},
  { 16.3932501484725f,    -63.5367057973473f},
  { 25.8924090881804f,    -63.6240862949495f},
  { 35.7209019991337f,    -63.0444971189200f},
  { 45.3210844354825f,    -61.7562141472517f},
  { 54.2069014710865f,    -59.7976502527467f},
  { 62.0878885994974f,    -57.2729027490462f},
  { 68.8798871556760f,    -54.3213589461635f},
  { 74.6407012762525f,    -51.0887556818155f},
  { 79.4960587445786f,    -47.7081148747826f},
  { 83.5885791392606f,    -44.2903590151516f},
  { 87.0517100501096f,    -40.9215270870819f},
  { 90.0f,    -37.6638374535070f},
  { 92.5276973658163f,    -34.5587491208397f},
  { 0.0f,    -65.6187815578030f},
  { 9.19099647608508f,    -66.8151622059356f},
  { 19.7767131841238f,    -67.4513372081180f},
  { 31.2478160930689f,    -67.3424609596128f},
  { 42.7561895905164f,    -66.3777711696528f},
  { 53.4378607612508f,    -64.5666406655262f},
  { 62.7502499362301f,    -62.0283401594414f},
  { 70.5487009486867f,    -58.9407544608438f},
  { 76.9513489599765f,    -55.4903085181429f},
  { 82.1798710590254f,    -51.8433375502915f},
  { 86.4634243465556f,    -48.1354091502336f},
  { 90.0f,    -44.4700523736346f},
  { 92.9482899498904f,    -40.9215270870819f},
  { 95.4314773315273f,    -37.5391973811345f},
  { 0.0f,    -69.6541923856616f},
  { 11.4340697442537f,    -70.9614896019555f},
  { 24.8108847601265f,    -71.4897072290348f},
  { 38.9734473368657f,    -71.0021166134344f},
  { 52.3123893000598f,    -69.4343887930543f},
  { 63.6909565444517f,    -66.9209866913203f},
  { 72.8279621494862f,    -63.7030502822517f},
  { 79.9858774849308f,    -60.0288091226086f},
  { 85.5824320004692f,    -56.1078668060580f},
  { 90.0f,    -52.1031151288303f},
  { 93.5365756534444f,    -48.1354091502336f},
  { 96.4114208607394f,    -44.2903590151516f},
  { 98.7832306009590f,    -40.6247965576687f},
  { 0.0f,    -73.9115997418271f},
  { 15.0920752881032f,    -75.2790827296451f},
  { 32.9114931857064f,    -75.5029442615280f},
  { 50.5099309427673f,    -74.3263225812526f},
  { 65.1310748815380f,    -71.8790718535498f},
  { 76.1141128518623f,    -68.5221100088469f},
  { 84.1189512527134f,    -64.6093910405445f},
  { 90.0000000000000f,    -60.4080049704701f},
  { 94.4175679995308f,    -56.1078668060580f},
  { 97.8201289409746f,    -51.8433375502915f},
  { 100.503941255421f,    -47.7081148747826f},
  { 102.666183559911f,    -43.7646105401545f},
  { 0.0f,    -78.3592768632294f},
  { 22.0237505161462f,    -79.6613630728832f},
  { 47.1679114977868f,    -79.1551986976493f},
  { 67.6072290710451f,    -76.8187518549304f},
  { 81.2167693990410f,    -73.2780371655037f},
  { 90.0f,    -69.0948425521107f},
  { 95.8810487472866f,    -64.6093910405445f},
  { 100.014122515069f,    -60.0288091226086f},
  { 103.048651040024f,    -55.4903085181429f},
  { 105.359298723748f,    -51.0887556818155f},
  { 107.172037850514f,    -46.8885380691238f},
  { 0.0f,    -82.9535936191428f},
  { 38.9734473368657f,    -83.7926432688605f},
  { 72.8279621494862f,    -81.6349995128180f},
  { 90.0000000000000f,    -77.7816801337513f},
  { 98.7832306009590f,    -73.2780371655037f},
  { 103.885887148138f,    -68.5221100088469f},
  { 107.172037850514f,    -63.7030502822517f},
  { 109.451299051313f,    -58.9407544608438f},
  { 111.120112844324f,    -54.3213589461635f},
  { 112.392770928955f,    -49.9068566415137f},
  { 0.0f,    -87.6406185690461f},
  { 90.0f,    -86.0865699753911f},
  { 107.172037850514f,    -81.6349995128180f},
  { 112.392770928955f,    -76.8187518549304f},
  { 114.868925118462f,    -71.8790718535498f},
  { 116.309043455548f,    -66.9209866913203f},
  { 117.249750063770f,    -62.0283401594414f},
  { 117.912111400503f,    -57.2729027490462f},
  { 118.403624533553f,    -52.7128866391432f},
  { 180.0f,    -87.6406185690461f},
  { 141.026552663134f,    -83.7926432688605f},
  { 132.832088502213f,    -79.1551986976493f},
  { 129.490069057233f,    -74.3263225812526f},
  { 127.687610699940f,    -69.4343887930543f},
  { 126.562139238749f,    -64.5666406655263f},
  { 125.793098528914f,    -59.7976502527467f},
  { 125.234499960836f,    -55.1902992552913f},
  { 180.0f,    -82.9535936191428f},
  { 157.976249483854f,    -79.6613630728832f},
  { 147.088506814294f,    -75.5029442615280f},
  { 141.026552663134f,    -71.0021166134344f},
  { 137.243810409484f,    -66.3777711696528f},
  { 134.678915564518f,    -61.7562141472517f},
  { 132.832088502213f,    -57.2262493118837f},
  { 180.0f,    -78.3592768632294f},
  { 164.907924711897f,    -75.2790827296451f},
  { 155.189115239873f,    -71.4897072290348f},
  { 148.752183906931f,    -67.3424609596128f},
  { 144.279098000866f,    -63.0444971189200f},
  { 141.026552663134f,    -58.7297899739513f},
  { 180.0f,    -73.9115997418271f},
  { 168.565930255746f,    -70.9614896019555f},
  { 160.223286815876f,    -67.4513372081180f},
  { 154.107590911820f,    -63.6240862949495f},
  { 149.528466225577f,    -59.6513563396351f},
  { 180.0f,    -69.6541923856616f},
  { 170.809003523915f,    -66.8151622059356f},
  { 163.606749851527f,    -63.5367057973473f},
  { 157.976249483854f,    -59.9952713133382f},
  { 180.0f,    -65.6187815578030f},
  { 172.320771426330f,    -62.8903831093188f},
  { 166.021562661129f,    -59.8179223197217f},
  { 180.0f,    -61.8251865351426f},
  { 173.407355326441f,    -59.2116443441111f},
  { -6.59264467355895f,    -59.2116443441111f},
  { -7.67922857366964f,    -62.8903831093188f},
  { -9.19099647608508f,    -66.8151622059356f},
  { -11.4340697442537f,    -70.9614896019555f},
  { -15.0920752881032f,    -75.2790827296451f},
  { -22.0237505161462f,    -79.6613630728832f},
  { -38.9734473368657f,    -83.7926432688605f},
  { -90.0f,    -86.0865699753911f},
  { -141.026552663134f,    -83.7926432688605f},
  { -157.976249483854f,    -79.6613630728832f},
  { -164.907924711897f,    -75.2790827296451f},
  { -168.565930255746f,    -70.9614896019555f},
  { -170.809003523915f,    -66.8151622059356f},
  { -172.320771426330f,    -62.8903831093188f},
  { -173.407355326441f,    -59.2116443441111f},
  { -13.9784373388714f,    -59.8179223197217f},
  { -16.3932501484725f,    -63.5367057973473f},
  { -19.7767131841238f,    -67.4513372081180f},
  { -24.8108847601265f,    -71.4897072290348f},
  { -32.9114931857064f,    -75.5029442615280f},
  { -47.1679114977869f,    -79.1551986976493f},
  { -72.8279621494862f,    -81.6349995128180f},
  { -107.172037850514f,    -81.6349995128180f},
  { -132.832088502213f,    -79.1551986976493f},
  { -147.088506814294f,    -75.5029442615280f},
  { -155.189115239873f,    -71.4897072290348f},
  { -160.223286815876f,    -67.4513372081180f},
  { -163.606749851527f,    -63.5367057973473f},
  { -166.021562661129f,    -59.8179223197217f},
  { -22.0237505161462f,    -59.9952713133382f},
  { -25.8924090881804f,    -63.6240862949495f},
  { -31.2478160930689f,    -67.3424609596128f},
  { -38.9734473368657f,    -71.0021166134344f},
  { -50.5099309427673f,    -74.3263225812526f},
  { -67.6072290710451f,    -76.8187518549304f},
  { -90.0f,    -77.7816801337513f},
  { -112.392770928955f,    -76.8187518549304f},
  { -129.490069057233f,    -74.3263225812526f},
  { -141.026552663134f,    -71.0021166134344f},
  { -148.752183906931f,    -67.3424609596128f},
  { -154.107590911820f,    -63.6240862949495f},
  { -157.976249483854f,    -59.9952713133382f},
  { -30.4715337744227f,    -59.6513563396351f},
  { -35.7209019991337f,    -63.0444971189200f},
  { -42.7561895905164f,    -66.3777711696528f},
  { -52.3123893000598f,    -69.4343887930543f},
  { -65.1310748815380f,    -71.8790718535498f},
  { -81.2167693990410f,    -73.2780371655037f},
  { -98.7832306009590f,    -73.2780371655037f},
  { -114.868925118462f,    -71.8790718535498f},
  { -127.687610699940f,    -69.4343887930543f},
  { -137.243810409484f,    -66.3777711696528f},
  { -144.279098000866f,    -63.0444971189200f},
  { -149.528466225577f,    -59.6513563396351f},
  { -38.9734473368657f,    -58.7297899739513f},
  { -45.3210844354825f,    -61.7562141472517f},
  { -53.4378607612508f,    -64.5666406655262f},
  { -63.6909565444517f,    -66.9209866913203f},
  { -76.1141128518623f,    -68.5221100088469f},
  { -90.0f,    -69.0948425521107f},
  { -103.885887148138f,    -68.5221100088469f},
  { -116.309043455548f,    -66.9209866913203f},
  { -126.562139238749f,    -64.5666406655263f},
  { -134.678915564518f,    -61.7562141472517f},
  { -141.026552663134f,    -58.7297899739513f},
  { -47.1679114977868f,    -57.2262493118836f},
  { -54.2069014710865f,    -59.7976502527467f},
  { -62.7502499362301f,    -62.0283401594414f},
  { -72.8279621494862f,    -63.7030502822517f},
  { -84.1189512527134f,    -64.6093910405445f},
  { -95.8810487472866f,    -64.6093910405445f},
  { -107.172037850514f,    -63.7030502822517f},
  { -117.249750063770f,    -62.0283401594414f},
  { -125.793098528914f,    -59.7976502527467f},
  { -132.832088502213f,    -57.2262493118836f},
  { -54.7655000391642f,    -55.1902992552913f},
  { -62.0878885994974f,    -57.2729027490462f},
  { -70.5487009486867f,    -58.9407544608438f},
  { -79.9858774849308f,    -60.0288091226086f},
  { -90.0f,    -60.4080049704701f},
  { -100.014122515069f,    -60.0288091226086f},
  { -109.451299051313f,    -58.9407544608438f},
  { -117.912111400503f,    -57.2729027490462f},
  { -125.234499960836f,    -55.1902992552913f},
  { -61.5963754664474f,    -52.7128866391432f},
  { -68.8798871556760f,    -54.3213589461635f},
  { -76.9513489599765f,    -55.4903085181429f},
  { -85.5824320004692f,    -56.1078668060580f},
  { -94.4175679995308f,    -56.1078668060580f},
  { -103.048651040024f,    -55.4903085181429f},
  { -111.120112844324f,    -54.3213589461635f},
  { -118.403624533553f,    -52.7128866391432f},
  { -67.6072290710451f,    -49.9068566415137f},
  { -74.6407012762525f,    -51.0887556818155f},
  { -82.1798710590254f,    -51.8433375502915f},
  { -90.0f,    -52.1031151288303f},
  { -97.8201289409746f,    -51.8433375502915f},
  { -105.359298723748f,    -51.0887556818155f},
  { -112.392770928955f,    -49.9068566415137f},
  { -72.8279621494862f,    -46.8885380691238f},
  { -79.4960587445786f,    -47.7081148747826f},
  { -86.4634243465556f,    -48.1354091502336f},
  { -93.5365756534444f,    -48.1354091502336f},
  { -100.503941255421f,    -47.7081148747826f},
  { -107.172037850514f,    -46.8885380691238f},
  { -77.3338164400891f,    -43.7646105401545f},
  { -83.5885791392606f,    -44.2903590151516f},
  { -90.0f,    -44.4700523736346f},
  { -96.4114208607394f,    -44.2903590151516f},
  { -102.666183559911f,    -43.7646105401545f},
  { -81.2167693990410f,    -40.6247965576687f},
  { -87.0517100501096f,    -40.9215270870819f},
  { -92.9482899498904f,    -40.9215270870819f},
  { -98.7832306009590f,    -40.6247965576687f},
  { -84.5685226684728f,    -37.5391973811345f},
  { -90.0f,    -37.6638374535070f},
  { -95.4314773315273f,    -37.5391973811345f},
  { -87.4723026341838f,    -34.5587491208397f},
  { -92.5276973658163f,    -34.5587491208397f},
  { 28.1748134648574f,    0.0f},
  { 24.3812184421970f,    0.0f},
  { 20.3458076143384f,    0.0f},
  { 16.0884002581729f,    0.0f},
  { 11.6407231367706f,    0.0f},
  { 7.04640638085718f,    0.0f},
  { 2.35938143095389f,    0.0f},
  { -2.35938143095389f,    0.0f},
  { -7.04640638085718f,    0.0f},
  { -11.6407231367706f,    0.0f},
  { -16.0884002581729f,    0.0f},
  { -20.3458076143384f,    0.0f},
  { -24.3812184421970f,    0.0f},
  { -28.1748134648574f,    0.0f},
  { 30.6214684405991f,    3.36906549364106f},
  { 26.9007978394868f,    3.49107314929588f},
  { 22.9180931626075f,    3.60536313394335f},
  { 18.6871295176278f,    3.70771430504662f},
  { 14.2341522984338f,    3.79364000883147f},
  { 9.59878281614505f,    3.85880705779658f},
  { 4.83330434995212f,    3.89955707931121f},
  { 0.0f,    3.91343002460889f},
  { -4.83330434995212f,    3.89955707931121f},
  { -9.59878281614505f,    3.85880705779658f},
  { -14.2341522984338f,    3.79364000883147f},
  { -18.6871295176278f,    3.70771430504662f},
  { -22.9180931626075f,    3.60536313394335f},
  { -26.9007978394868f,    3.49107314929588f},
  { -30.6214684405991f,    3.36906549364106f},
  { 29.4391773098121f,    6.97536279726341f},
  { 25.5266102849083f,    7.22510190546460f},
  { 21.3415446955180f,    7.45505888069346f},
  { 16.9036467955594f,    7.65572115352650f},
  { 12.2469889345236f,    7.81732554448432f},
  { 7.42042021204537f,    7.93092643056003f},
  { 2.48583513376763f,    7.98961616063538f},
  { -2.48583513376763f,    7.98961616063538f},
  { -7.42042021204537f,    7.93092643056003f},
  { -12.2469889345236f,    7.81732554448432f},
  { -16.9036467955594f,    7.65572115352650f},
  { -21.3415446955180f,    7.45505888069346f},
  { -25.5266102849083f,    7.22510190546460f},
  { -29.4391773098121f,    6.97536279726341f},
  { 28.1611303732939f,    10.8082208669889f},
  { 24.0417364554965f,    11.1863123685096f},
  { 19.6405393991971f,    11.5270445736850f},
  { 14.9845972116948f,    11.8147143864651f},
  { 10.1174139719544f,    12.0339031441344f},
  { 5.09845037281999f,    12.1714217257972f},
  { 0.0f,    12.2183198662487f},
  { -5.09845037281999f,    12.1714217257972f},
  { -10.1174139719544f,    12.0339031441344f},
  { -14.9845972116948f,    11.8147143864651f},
  { -19.6405393991971f,    11.5270445736850f},
  { -24.0417364554965f,    11.1863123685096f},
  { -28.1611303732939f,    10.8082208669889f},
  { 26.7766438797297f,    14.8460503635477f},
  { 22.4344985154204f,    15.3463091813622f},
  { 17.8031162035964f,    15.7851180659433f},
  { 12.9187441632093f,    16.1401147844812f},
  { 7.83610236269860f,    16.3905407735141f},
  { 2.62660128956667f,    16.5202084036662f},
  { -2.62660128956667f,    16.5202084036662f},
  { -7.83610236269860f,    16.3905407735141f},
  { -12.9187441632093f,    16.1401147844812f},
  { -17.8031162035964f,    15.7851180659433f},
  { -22.4344985154204f,    15.3463091813622f},
  { -26.7766438797297f,    14.8460503635477f},
  { 25.2736805860404f,    19.0551045548580f},
  { 20.6919903044741f,    19.6631651805948f},
  { 15.8164486453377f,    20.1786171531282f},
  { 10.6946017354743f,    20.5726887851795f},
  { 5.39428475673946f,    20.8205287470870f},
  { 0.0f,    20.9051574478893f},
  { -5.39428475673946f,    20.8205287470870f},
  { -10.6946017354743f,    20.5726887851795f},
  { -15.8164486453377f,    20.1786171531282f},
  { -20.6919903044741f,    19.6631651805948f},
  { -25.2736805860404f,    19.0551045548580f},
  { 23.6387115659933f,    23.3894469911546f},
  { 18.8000501570182f,    24.0821132106899f},
  { 13.6669873597994f,    24.6441246904973f},
  { 8.30077479951012f,    25.0415008212998f},
  { 2.78425352641492f,    25.2475623023264f},
  { -2.78425352641492f,    25.2475623023264f},
  { -8.30077479951012f,    25.0415008212998f},
  { -13.6669873597994f,    24.6441246904973f},
  { -18.8000501570182f,    24.0821132106899f},
  { -23.6387115659933f,    23.3894469911546f},
  { 21.8565934797386f,    27.7925137582971f},
  { 16.7432969906195f,    28.5381630591174f},
  { 11.3407167219705f,    29.1093331110031f},
  { 5.72644371919445f,    29.4690659822952f},
  { 0.0f,    29.5919950295299f},
  { -5.72644371919445f,    29.4690659822952f},
  { -11.3407167219705f,    29.1093331110031f},
  { -16.7432969906195f,    28.5381630591174f},
  { -21.8565934797386f,    27.7925137582971f},
  { 19.9104828235857f,    32.2003339721564f},
  { 14.5052658102205f,    32.9604933911101f},
  { 8.82356690720060f,    33.4986052140006f},
  { 2.96202127327168f,    33.7778636221748f},
  { -2.96202127327168f,    33.7778636221748f},
  { -8.82356690720060f,    33.4986052140006f},
  { -14.5052658102205f,    32.9604933911101f},
  { -19.9104828235857f,    32.2003339721564f},
  { 17.7818205761190f,    36.5460212963266f},
  { 12.0686915699763f,    37.2779247130735f},
  { 6.10202804800794f,    37.7391982305410f},
  { 0.0f,    37.8968848711697f},
  { -6.10202804800794f,    37.7391982305410f},
  { -12.0686915699763f,    37.2779247130735f},
  { -17.7818205761190f,    36.5460212963266f},
  { 15.4504370863258f,    40.7647476003615f},
  { 9.41600421572124f,    41.4244125764096f},
  { 3.16401262951146f,    41.7668858628286f},
  { -3.16401262951146f,    41.7668858628286f},
  { -9.41600421572124f,    41.4244125764096f},
  { -15.4504370863258f,    40.7647476003615f},
  { 12.8948484058316f,    44.7982500043257f},
  { 6.53010960224569f,    45.3435031381894f},
  { 0.0f,    45.5299476263654f},
  { -6.53010960224569f,    45.3435031381894f},
  { -12.8948484058316f,    44.7982500043257f},
  { 10.0928404843176f,    48.5980887666628f},
  { 3.39553463604395f,    48.9910654281789f},
  { -3.39553463604395f,    48.9910654281789f},
  { -10.0928404843176f,    48.5980887666628f},
  { 7.02246245033840f,    52.1272816881218f},
  { 0.0f,    52.3361625464930f},
  { -7.02246245033840f,    52.1272816881218f},
  { 3.66356443787723f,    55.3603981546409f},
  { -3.66356443787723f,    55.3603981546409f},
  { 30.6214684405991f,    -3.36906549364106f},
  { 29.4391773098121f,    -6.97536279726341f},
  { 28.1611303732939f,    -10.8082208669889f},
  { 26.7766438797297f,    -14.8460503635477f},
  { 25.2736805860404f,    -19.0551045548580f},
  { 23.6387115659933f,    -23.3894469911546f},
  { 21.8565934797386f,    -27.7925137582971f},
  { 19.9104828235857f,    -32.2003339721564f},
  { 17.7818205761190f,    -36.5460212963266f},
  { 15.4504370863258f,    -40.7647476003615f},
  { 12.8948484058316f,    -44.7982500043257f},
  { 10.0928404843176f,    -48.5980887666628f},
  { 7.02246245033840f,    -52.1272816881218f},
  { 3.66356443787723f,    -55.3603981546409f},
  { 26.9007978394868f,    -3.49107314929588f},
  { 25.5266102849083f,    -7.22510190546461f},
  { 24.0417364554965f,    -11.1863123685096f},
  { 22.4344985154204f,    -15.3463091813622f},
  { 20.6919903044741f,    -19.6631651805948f},
  { 18.8000501570182f,    -24.0821132106899f},
  { 16.7432969906195f,    -28.5381630591174f},
  { 14.5052658102205f,    -32.9604933911101f},
  { 12.0686915699763f,    -37.2779247130735f},
  { 9.41600421572124f,    -41.4244125764096f},
  { 6.53010960224569f,    -45.3435031381894f},
  { 3.39553463604395f,    -48.9910654281789f},
  { 5.58919916958504e-15f,    -52.3361625464930f},
  { -3.66356443787723f,    -55.3603981546409f},
  { 22.9180931626075f,    -3.60536313394335f},
  { 21.3415446955180f,    -7.45505888069346f},
  { 19.6405393991971f,    -11.5270445736850f},
  { 17.8031162035964f,    -15.7851180659433f},
  { 15.8164486453377f,    -20.1786171531282f},
  { 13.6669873597994f,    -24.6441246904973f},
  { 11.3407167219705f,    -29.1093331110031f},
  { 8.82356690720060f,    -33.4986052140006f},
  { 6.10202804800794f,    -37.7391982305410f},
  { 3.16401262951146f,    -41.7668858628286f},
  { 0.0f,    -45.5299476263654f},
  { -3.39553463604395f,    -48.9910654281789f},
  { -7.02246245033840f,    -52.1272816881218f},
  { 18.6871295176278f,    -3.70771430504662f},
  { 16.9036467955594f,    -7.65572115352650f},
  { 14.9845972116948f,    -11.8147143864651f},
  { 12.9187441632093f,    -16.1401147844812f},
  { 10.6946017354743f,    -20.5726887851795f},
  { 8.30077479951012f,    -25.0415008212998f},
  { 5.72644371919445f,    -29.4690659822952f},
  { 2.96202127327168f,    -33.7778636221748f},
  { 0.0f,    -37.8968848711697f},
  { -3.16401262951146f,    -41.7668858628286f},
  { -6.53010960224569f,    -45.3435031381894f},
  { -10.0928404843176f,    -48.5980887666628f},
  { 14.2341522984338f,    -3.79364000883147f},
  { 12.2469889345236f,    -7.81732554448432f},
  { 10.1174139719544f,    -12.0339031441344f},
  { 7.83610236269860f,    -16.3905407735141f},
  { 5.39428475673946f,    -20.8205287470870f},
  { 2.78425352641492f,    -25.2475623023264f},
  { -2.27500452427325e-15f,    -29.5919950295299f},
  { -2.96202127327169f,    -33.7778636221748f},
  { -6.10202804800794f,    -37.7391982305410f},
  { -9.41600421572124f,    -41.4244125764096f},
  { -12.8948484058316f,    -44.7982500043257f},
  { 9.59878281614505f,    -3.85880705779658f},
  { 7.42042021204537f,    -7.93092643056003f},
  { 5.09845037281999f,    -12.1714217257972f},
  { 2.62660128956667f,    -16.5202084036662f},
  { 2.14223704952744e-15f,    -20.9051574478893f},
  { -2.78425352641492f,    -25.2475623023264f},
  { -5.72644371919445f,    -29.4690659822952f},
  { -8.82356690720060f,    -33.4986052140006f},
  { -12.0686915699763f,    -37.2779247130735f},
  { -15.4504370863258f,    -40.7647476003615f},
  { 4.83330434995212f,    -3.89955707931121f},
  { 2.48583513376763f,    -7.98961616063538f},
  { -1.01205574733119e-15f,    -12.2183198662487f},
  { -2.62660128956667f,    -16.5202084036662f},
  { -5.39428475673946f,    -20.8205287470870f},
  { -8.30077479951012f,    -25.0415008212998f},
  { -11.3407167219705f,    -29.1093331110031f},
  { -14.5052658102205f,    -32.9604933911101f},
  { -17.7818205761190f,    -36.5460212963266f},
  { 0.0f,    -3.91343002460889f},
  { -2.48583513376763f,    -7.98961616063538f},
  { -5.09845037281999f,    -12.1714217257972f},
  { -7.83610236269860f,    -16.3905407735141f},
  { -10.6946017354743f,    -20.5726887851795f},
  { -13.6669873597994f,    -24.6441246904973f},
  { -16.7432969906195f,    -28.5381630591174f},
  { -19.9104828235857f,    -32.2003339721564f},
  { -4.83330434995212f,    -3.89955707931121f},
  { -7.42042021204537f,    -7.93092643056004f},
  { -10.1174139719544f,    -12.0339031441344f},
  { -12.9187441632093f,    -16.1401147844812f},
  { -15.8164486453377f,    -20.1786171531282f},
  { -18.8000501570182f,    -24.0821132106899f},
  { -21.8565934797386f,    -27.7925137582971f},
  { -9.59878281614505f,    -3.85880705779658f},
  { -12.2469889345236f,    -7.81732554448432f},
  { -14.9845972116948f,    -11.8147143864651f},
  { -17.8031162035964f,    -15.7851180659433f},
  { -20.6919903044741f,    -19.6631651805948f},
  { -23.6387115659933f,    -23.3894469911546f},
  { -14.2341522984338f,    -3.79364000883147f},
  { -16.9036467955594f,    -7.65572115352650f},
  { -19.6405393991971f,    -11.5270445736850f},
  { -22.4344985154204f,    -15.3463091813622f},
  { -25.2736805860404f,    -19.0551045548580f},
  { -18.6871295176278f,    -3.70771430504662f},
  { -21.3415446955180f,    -7.45505888069346f},
  { -24.0417364554965f,    -11.1863123685096f},
  { -26.7766438797297f,    -14.8460503635477f},
  { -22.9180931626075f,    -3.60536313394334f},
  { -25.5266102849083f,    -7.22510190546460f},
  { -28.1611303732939f,    -10.8082208669889f},
  { -26.9007978394868f,    -3.49107314929588f},
  { -29.4391773098121f,    -6.97536279726341f},
  { -30.6214684405991f,    -3.36906549364106f},
  { -151.825186535143f,    0.0f},
  { -155.618781557803f,    0.0f},
  { -159.654192385662f,    0.0f},
  { -163.911599741827f,    0.0f},
  { -168.359276863229f,    0.0f},
  { -172.953593619143f,    0.0f},
  { -177.640618569046f,    0.0f},
  { 177.640618569046f,    0.0f},
  { 172.953593619143f,    0.0f},
  { 168.359276863229f,    0.0f},
  { 163.911599741827f,    0.0f},
  { 159.654192385662f,    0.0f},
  { 155.618781557803f,    0.0f},
  { 151.825186535143f,    0.0f},
  { -149.378531559401f,    3.36906549364106f},
  { -153.099202160513f,    3.49107314929588f},
  { -157.081906837392f,    3.60536313394335f},
  { -161.312870482372f,    3.70771430504662f},
  { -165.765847701566f,    3.79364000883147f},
  { -170.401217183855f,    3.85880705779658f},
  { -175.166695650048f,    3.89955707931121f},
  { 180.0f,    3.91343002460889f},
  { 175.166695650048f,    3.89955707931121f},
  { 170.401217183855f,    3.85880705779658f},
  { 165.765847701566f,    3.79364000883147f},
  { 161.312870482372f,    3.70771430504662f},
  { 157.081906837392f,    3.60536313394335f},
  { 153.099202160513f,    3.49107314929588f},
  { 149.378531559401f,    3.36906549364106f},
  { -150.560822690188f,    6.97536279726341f},
  { -154.473389715092f,    7.22510190546460f},
  { -158.658455304482f,    7.45505888069346f},
  { -163.096353204441f,    7.65572115352650f},
  { -167.753011065476f,    7.81732554448432f},
  { -172.579579787955f,    7.93092643056003f},
  { -177.514164866232f,    7.98961616063538f},
  { 177.514164866232f,    7.98961616063538f},
  { 172.579579787955f,    7.93092643056003f},
  { 167.753011065476f,    7.81732554448432f},
  { 163.096353204441f,    7.65572115352650f},
  { 158.658455304482f,    7.45505888069346f},
  { 154.473389715092f,    7.22510190546460f},
  { 150.560822690188f,    6.97536279726341f},
  { -151.838869626706f,    10.8082208669889f},
  { -155.958263544504f,    11.1863123685096f},
  { -160.359460600803f,    11.5270445736850f},
  { -165.015402788305f,    11.8147143864651f},
  { -169.882586028046f,    12.0339031441344f},
  { -174.901549627180f,    12.1714217257972f},
  { 180.0f,    12.2183198662487f},
  { 174.901549627180f,    12.1714217257972f},
  { 169.882586028046f,    12.0339031441344f},
  { 165.015402788305f,    11.8147143864651f},
  { 160.359460600803f,    11.5270445736850f},
  { 155.958263544504f,    11.1863123685096f},
  { 151.838869626706f,    10.8082208669889f},
  { -153.223356120270f,    14.8460503635477f},
  { -157.565501484580f,    15.3463091813622f},
  { -162.196883796404f,    15.7851180659433f},
  { -167.081255836791f,    16.1401147844812f},
  { -172.163897637301f,    16.3905407735141f},
  { -177.373398710433f,    16.5202084036662f},
  { 177.373398710433f,    16.5202084036662f},
  { 172.163897637301f,    16.3905407735141f},
  { 167.081255836791f,    16.1401147844812f},
  { 162.196883796404f,    15.7851180659433f},
  { 157.565501484580f,    15.3463091813622f},
  { 153.223356120270f,    14.8460503635477f},
  { -154.726319413960f,    19.0551045548580f},
  { -159.308009695526f,    19.6631651805948f},
  { -164.183551354662f,    20.1786171531282f},
  { -169.305398264526f,    20.5726887851795f},
  { -174.605715243261f,    20.8205287470870f},
  { 180.0f,    20.9051574478893f},
  { 174.605715243261f,    20.8205287470870f},
  { 169.305398264526f,    20.5726887851795f},
  { 164.183551354662f,    20.1786171531282f},
  { 159.308009695526f,    19.6631651805948f},
  { 154.726319413960f,    19.0551045548580f},
  { -156.361288434007f,    23.3894469911546f},
  { -161.199949842982f,    24.0821132106899f},
  { -166.333012640201f,    24.6441246904973f},
  { -171.699225200490f,    25.0415008212998f},
  { -177.215746473585f,    25.2475623023264f},
  { 177.215746473585f,    25.2475623023264f},
  { 171.699225200490f,    25.0415008212998f},
  { 166.333012640201f,    24.6441246904973f},
  { 161.199949842982f,    24.0821132106899f},
  { 156.361288434007f,    23.3894469911546f},
  { -158.143406520261f,    27.7925137582971f},
  { -163.256703009381f,    28.5381630591174f},
  { -168.659283278030f,    29.1093331110031f},
  { -174.273556280806f,    29.4690659822952f},
  { 180.0f,    29.5919950295299f},
  { 174.273556280806f,    29.4690659822952f},
  { 168.659283278030f,    29.1093331110031f},
  { 163.256703009381f,    28.5381630591174f},
  { 158.143406520261f,    27.7925137582971f},
  { -160.089517176414f,    32.2003339721564f},
  { -165.494734189780f,    32.9604933911101f},
  { -171.176433092799f,    33.4986052140006f},
  { -177.037978726728f,    33.7778636221748f},
  { 177.037978726728f,    33.7778636221748f},
  { 171.176433092799f,    33.4986052140006f},
  { 165.494734189780f,    32.9604933911101f},
  { 160.089517176414f,    32.2003339721564f},
  { -162.218179423881f,    36.5460212963266f},
  { -167.931308430024f,    37.2779247130735f},
  { -173.897971951992f,    37.7391982305410f},
  { 180.0f,    37.8968848711697f},
  { 173.897971951992f,    37.7391982305410f},
  { 167.931308430024f,    37.2779247130735f},
  { 162.218179423881f,    36.5460212963266f},
  { -164.549562913674f,    40.7647476003615f},
  { -170.583995784279f,    41.4244125764096f},
  { -176.835987370489f,    41.7668858628286f},
  { 176.835987370489f,    41.7668858628286f},
  { 170.583995784279f,    41.4244125764096f},
  { 164.549562913674f,    40.7647476003615f},
  { -167.105151594168f,    44.7982500043257f},
  { -173.469890397754f,    45.3435031381894f},
  { 180.0f,    45.5299476263654f},
  { 173.469890397754f,    45.3435031381894f},
  { 167.105151594168f,    44.7982500043257f},
  { -169.907159515682f,    48.5980887666628f},
  { -176.604465363956f,    48.9910654281789f},
  { 176.604465363956f,    48.9910654281789f},
  { 169.907159515682f,    48.5980887666628f},
  { -172.977537549662f,    52.1272816881218f},
  { 180.0f,    52.3361625464930f},
  { 172.977537549662f,    52.1272816881218f},
  { -176.336435562123f,    55.3603981546409f},
  { 176.336435562123f,    55.3603981546409f},
  { -149.378531559401f,    -3.36906549364106f},
  { -150.560822690188f,    -6.97536279726341f},
  { -151.838869626706f,    -10.8082208669889f},
  { -153.223356120270f,    -14.8460503635477f},
  { -154.726319413960f,    -19.0551045548580f},
  { -156.361288434007f,    -23.3894469911546f},
  { -158.143406520261f,    -27.7925137582971f},
  { -160.089517176414f,    -32.2003339721564f},
  { -162.218179423881f,    -36.5460212963266f},
  { -164.549562913674f,    -40.7647476003615f},
  { -167.105151594168f,    -44.7982500043257f},
  { -169.907159515682f,    -48.5980887666628f},
  { -172.977537549662f,    -52.1272816881218f},
  { -176.336435562123f,    -55.3603981546409f},
  { -153.099202160513f,    -3.49107314929588f},
  { -154.473389715092f,    -7.22510190546461f},
  { -155.958263544504f,    -11.1863123685096f},
  { -157.565501484580f,    -15.3463091813622f},
  { -159.308009695526f,    -19.6631651805948f},
  { -161.199949842982f,    -24.0821132106899f},
  { -163.256703009381f,    -28.5381630591174f},
  { -165.494734189780f,    -32.9604933911101f},
  { -167.931308430024f,    -37.2779247130735f},
  { -170.583995784279f,    -41.4244125764096f},
  { -173.469890397754f,    -45.3435031381894f},
  { -176.604465363956f,    -48.9910654281789f},
  { -180.0f,    -52.3361625464930f},
  { 176.336435562123f,    -55.3603981546409f},
  { -157.081906837392f,    -3.60536313394335f},
  { -158.658455304482f,    -7.45505888069346f},
  { -160.359460600803f,    -11.5270445736850f},
  { -162.196883796404f,    -15.7851180659433f},
  { -164.183551354662f,    -20.1786171531282f},
  { -166.333012640201f,    -24.6441246904973f},
  { -168.659283278030f,    -29.1093331110031f},
  { -171.176433092799f,    -33.4986052140006f},
  { -173.897971951992f,    -37.7391982305410f},
  { -176.835987370489f,    -41.7668858628286f},
  { 180.0f,    -45.5299476263654f},
  { 176.604465363956f,    -48.9910654281789f},
  { 172.977537549662f,    -52.1272816881218f},
  { -161.312870482372f,    -3.70771430504662f},
  { -163.096353204441f,    -7.65572115352650f},
  { -165.015402788305f,    -11.8147143864651f},
  { -167.081255836791f,    -16.1401147844812f},
  { -169.305398264526f,    -20.5726887851795f},
  { -171.699225200490f,    -25.0415008212998f},
  { -174.273556280806f,    -29.4690659822952f},
  { -177.037978726728f,    -33.7778636221748f},
  { 180.0f,    -37.8968848711697f},
  { 176.835987370489f,    -41.7668858628286f},
  { 173.469890397754f,    -45.3435031381894f},
  { 169.907159515682f,    -48.5980887666628f},
  { -165.765847701566f,    -3.79364000883147f},
  { -167.753011065476f,    -7.81732554448432f},
  { -169.882586028046f,    -12.0339031441344f},
  { -172.163897637301f,    -16.3905407735141f},
  { -174.605715243261f,    -20.8205287470870f},
  { -177.215746473585f,    -25.2475623023264f},
  { 180.0f,    -29.5919950295299f},
  { 177.037978726728f,    -33.7778636221748f},
  { 173.897971951992f,    -37.7391982305410f},
  { 170.583995784279f,    -41.4244125764096f},
  { 167.105151594168f,    -44.7982500043257f},
  { -170.401217183855f,    -3.85880705779658f},
  { -172.579579787955f,    -7.93092643056003f},
  { -174.901549627180f,    -12.1714217257972f},
  { -177.373398710433f,    -16.5202084036662f},
  { -180.0f,    -20.9051574478893f},
  { 177.215746473585f,    -25.2475623023264f},
  { 174.273556280806f,    -29.4690659822952f},
  { 171.176433092799f,    -33.4986052140006f},
  { 167.931308430024f,    -37.2779247130735f},
  { 164.549562913674f,    -40.7647476003615f},
  { -175.166695650048f,    -3.89955707931121f},
  { -177.514164866232f,    -7.98961616063538f},
  { 180.0f,    -12.2183198662487f},
  { 177.373398710433f,    -16.5202084036662f},
  { 174.605715243261f,    -20.8205287470870f},
  { 171.699225200490f,    -25.0415008212998f},
  { 168.659283278030f,    -29.1093331110031f},
  { 165.494734189780f,    -32.9604933911101f},
  { 162.218179423881f,    -36.5460212963266f},
  { 180.0f,    -3.91343002460889f},
  { 177.514164866232f,    -7.98961616063538f},
  { 174.901549627180f,    -12.1714217257972f},
  { 172.163897637301f,    -16.3905407735141f},
  { 169.305398264526f,    -20.5726887851795f},
  { 166.333012640201f,    -24.6441246904973f},
  { 163.256703009381f,    -28.5381630591174f},
  { 160.089517176414f,    -32.2003339721564f},
  { 175.166695650048f,    -3.89955707931121f},
  { 172.579579787955f,    -7.93092643056004f},
  { 169.882586028046f,    -12.0339031441344f},
  { 167.081255836791f,    -16.1401147844812f},
  { 164.183551354662f,    -20.1786171531282f},
  { 161.199949842982f,    -24.0821132106899f},
  { 158.143406520261f,    -27.7925137582971f},
  { 170.401217183855f,    -3.85880705779658f},
  { 167.753011065476f,    -7.81732554448432f},
  { 165.015402788305f,    -11.8147143864651f},
  { 162.196883796404f,    -15.7851180659433f},
  { 159.308009695526f,    -19.6631651805948f},
  { 156.361288434007f,    -23.3894469911546f},
  { 165.765847701566f,    -3.79364000883147f},
  { 163.096353204441f,    -7.65572115352650f},
  { 160.359460600803f,    -11.5270445736850f},
  { 157.565501484580f,    -15.3463091813622f},
  { 154.726319413960f,    -19.0551045548580f},
  { 161.312870482372f,    -3.70771430504662f},
  { 158.658455304482f,    -7.45505888069346f},
  { 155.958263544504f,    -11.1863123685096f},
  { 153.223356120270f,    -14.8460503635477f},
  { 157.081906837392f,    -3.60536313394334f},
  { 154.473389715092f,    -7.22510190546460f},
  { 151.838869626706f,    -10.8082208669889f},
  { 153.099202160513f,    -3.49107314929588f},
  { 150.560822690188f,    -6.97536279726341f},
  { 149.378531559401f,    -3.36906549364106f},
  { 90.0f,    28.1748134648574f},
  { 90.0f,    24.3812184421970f},
  { 90.0f,    20.3458076143384f},
  { 90.0f,    16.0884002581729f},
  { 90.0f,    11.6407231367706f},
  { 90.0f,    7.04640638085718f},
  { 90.0f,    2.35938143095389f},
  { 90.0f,    -2.35938143095389f},
  { 90.0f,    -7.04640638085718f},
  { 90.0f,    -11.6407231367706f},
  { 90.0f,    -16.0884002581729f},
  { 90.0f,    -20.3458076143384f},
  { 90.0f,    -24.3812184421970f},
  { 90.0f,    -28.1748134648574f},
  { 86.0865699753911f,    30.5628733513954f},
  { 86.0865699753911f,    26.8468676531246f},
  { 86.0865699753911f,    22.8701586038975f},
  { 86.0865699753911f,    18.6465721493746f},
  { 86.0865699753911f,    14.2023070245089f},
  { 86.0865699753911f,    9.57681642812012f},
  { 86.0865699753911f,    4.82208777969570f},
  { 86.0865699753911f,    0.0f},
  { 86.0865699753911f,    -4.82208777969570f},
  { 86.0865699753911f,    -9.57681642812013f},
  { 86.0865699753911f,    -14.2023070245089f},
  { 86.0865699753911f,    -18.6465721493746f},
  { 86.0865699753911f,    -22.8701586038975f},
  { 86.0865699753911f,    -26.8468676531246f},
  { 86.0865699753911f,    -30.5628733513954f},
  { 82.0029556393830f,    29.2001208977139f},
  { 82.0029556393830f,    25.3095486329268f},
  { 82.0029556393830f,    21.1524312620308f},
  { 82.0029556393830f,    16.7485097268094f},
  { 82.0029556393830f,    12.1314342637763f},
  { 82.0029556393830f,    7.34905112912407f},
  { 82.0029556393830f,    2.46169088485867f},
  { 82.0029556393830f,    -2.46169088485866f},
  { 82.0029556393830f,    -7.34905112912407f},
  { 82.0029556393830f,    -12.1314342637763f},
  { 82.0029556393830f,    -16.7485097268094f},
  { 82.0029556393830f,    -21.1524312620308f},
  { 82.0029556393830f,    -25.3095486329268f},
  { 82.0029556393830f,    -29.2001208977139f},
  { 77.7816801337513f,    27.6183932347186f},
  { 77.7816801337513f,    23.5570490281124f},
  { 77.7816801337513f,    19.2286421245509f},
  { 77.7816801337513f,    14.6599491964913f},
  { 77.7816801337513f,    9.89281564338174f},
  { 77.7816801337513f,    4.98354995961543f},
  { 77.7816801337513f,    0.0f},
  { 77.7816801337513f,    -4.98354995961543f},
  { 77.7816801337513f,    -9.89281564338174f},
  { 77.7816801337513f,    -14.6599491964913f},
  { 77.7816801337513f,    -19.2286421245509f},
  { 77.7816801337513f,    -23.5570490281124f},
  { 77.7816801337513f,    -27.6183932347186f},
  { 73.4633656902618f,    25.8154906887824f},
  { 73.4633656902618f,    21.5935345276613f},
  { 73.4633656902618f,    17.1105890228290f},
  { 73.4633656902618f,    12.4012696205441f},
  { 73.4633656902618f,    7.51576871353677f},
  { 73.4633656902618f,    2.51810250878842f},
  { 73.4633656902618f,    -2.51810250878842f},
  { 73.4633656902618f,    -7.51576871353677f},
  { 73.4633656902618f,    -12.4012696205441f},
  { 73.4633656902618f,    -17.1105890228290f},
  { 73.4633656902618f,    -21.5935345276613f},
  { 73.4633656902618f,    -25.8154906887824f},
  { 69.0948425521107f,    23.8001826021751f},
  { 69.0948425521107f,    19.4351390051484f},
  { 69.0948425521107f,    14.8226211436785f},
  { 69.0948425521107f,    10.0053101578424f},
  { 69.0948425521107f,    5.04108531994386f},
  { 69.0948425521107f,    0.0f},
  { 69.0948425521107f,    -5.04108531994386f},
  { 69.0948425521107f,    -10.0053101578424f},
  { 69.0948425521107f,    -14.8226211436785f},
  { 69.0948425521107f,    -19.4351390051484f},
  { 69.0948425521107f,    -23.8001826021751f},
  { 64.7263194139596f,    21.5935345276613f},
  { 64.7263194139596f,    17.1105890228290f},
  { 64.7263194139596f,    12.4012696205441f},
  { 64.7263194139596f,    7.51576871353677f},
  { 64.7263194139596f,    2.51810250878842f},
  { 64.7263194139596f,    -2.51810250878842f},
  { 64.7263194139596f,    -7.51576871353677f},
  { 64.7263194139596f,    -12.4012696205441f},
  { 64.7263194139596f,    -17.1105890228290f},
  { 64.7263194139596f,    -21.5935345276613f},
  { 60.4080049704701f,    19.2286421245509f},
  { 60.4080049704701f,    14.6599491964913f},
  { 60.4080049704701f,    9.89281564338174f},
  { 60.4080049704701f,    4.98354995961543f},
  { 60.4080049704701f,    0.0f},
  { 60.4080049704701f,    -4.98354995961543f},
  { 60.4080049704701f,    -9.89281564338174f},
  { 60.4080049704701f,    -14.6599491964913f},
  { 60.4080049704701f,    -19.2286421245509f},
  { 56.1867294648384f,    16.7485097268094f},
  { 56.1867294648384f,    12.1314342637763f},
  { 56.1867294648384f,    7.34905112912407f},
  { 56.1867294648384f,    2.46169088485866f},
  { 56.1867294648384f,    -2.46169088485867f},
  { 56.1867294648384f,    -7.34905112912407f},
  { 56.1867294648384f,    -12.1314342637763f},
  { 56.1867294648384f,    -16.7485097268094f},
  { 52.1031151288303f,    14.2023070245089f},
  { 52.1031151288303f,    9.57681642812012f},
  { 52.1031151288303f,    4.82208777969570f},
  { 52.1031151288303f,    0.0f},
  { 52.1031151288303f,    -4.82208777969570f},
  { 52.1031151288303f,    -9.57681642812012f},
  { 52.1031151288303f,    -14.2023070245089f},
  { 48.1896851042214f,    11.6407231367706f},
  { 48.1896851042214f,    7.04640638085718f},
  { 48.1896851042214f,    2.35938143095389f},
  { 48.1896851042214f,    -2.35938143095389f},
  { 48.1896851042214f,    -7.04640638085718f},
  { 48.1896851042214f,    -11.6407231367706f},
  { 44.4700523736346f,    9.11139042918330f},
  { 44.4700523736346f,    4.58467988787900f},
  { 44.4700523736346f,    0.0f},
  { 44.4700523736346f,    -4.58467988787900f},
  { 44.4700523736346f,    -9.11139042918330f},
  { 40.9590912848605f,    6.65525720600019f},
  { 40.9590912848605f,    2.22732770546869f},
  { 40.9590912848605f,    -2.22732770546869f},
  { 40.9590912848605f,    -6.65525720600019f},
  { 37.6638374535070f,    4.30441784763801f},
  { 37.6638374535070f,    0.0f},
  { 37.6638374535070f,    -4.30441784763801f},
  { 34.5848092349368f,    2.08145509534415f},
  { 34.5848092349368f,    -2.08145509534415f},
  { 93.9134300246089f,    30.5628733513954f},
  { 97.9970443606170f,    29.2001208977139f},
  { 102.218319866249f,    27.6183932347186f},
  { 106.536634309738f,    25.8154906887824f},
  { 110.905157447889f,    23.8001826021751f},
  { 115.273680586040f,    21.5935345276613f},
  { 119.591995029530f,    19.2286421245509f},
  { 123.813270535162f,    16.7485097268094f},
  { 127.896884871170f,    14.2023070245089f},
  { 131.810314895779f,    11.6407231367706f},
  { 135.529947626365f,    9.11139042918330f},
  { 139.040908715140f,    6.65525720600019f},
  { 142.336162546493f,    4.30441784763801f},
  { 145.415190765063f,    2.08145509534415f},
  { 93.9134300246089f,    26.8468676531246f},
  { 97.9970443606170f,    25.3095486329268f},
  { 102.218319866249f,    23.5570490281124f},
  { 106.536634309738f,    21.5935345276613f},
  { 110.905157447889f,    19.4351390051484f},
  { 115.273680586040f,    17.1105890228290f},
  { 119.591995029530f,    14.6599491964913f},
  { 123.813270535162f,    12.1314342637763f},
  { 127.896884871170f,    9.57681642812012f},
  { 131.810314895779f,    7.04640638085718f},
  { 135.529947626365f,    4.58467988787900f},
  { 139.040908715140f,    2.22732770546869f},
  { 142.336162546493f,    3.41515458187918e-15f},
  { 145.415190765063f,    -2.08145509534415f},
  { 93.9134300246089f,    22.8701586038975f},
  { 97.9970443606170f,    21.1524312620308f},
  { 102.218319866249f,    19.2286421245509f},
  { 106.536634309738f,    17.1105890228290f},
  { 110.905157447889f,    14.8226211436785f},
  { 115.273680586040f,    12.4012696205441f},
  { 119.591995029530f,    9.89281564338174f},
  { 123.813270535162f,    7.34905112912407f},
  { 127.896884871170f,    4.82208777969570f},
  { 131.810314895779f,    2.35938143095389f},
  { 135.529947626365f,    0.0f},
  { 139.040908715140f,    -2.22732770546869f},
  { 142.336162546493f,    -4.30441784763801f},
  { 93.9134300246089f,    18.6465721493746f},
  { 97.9970443606170f,    16.7485097268094f},
  { 102.218319866249f,    14.6599491964913f},
  { 106.536634309738f,    12.4012696205441f},
  { 110.905157447889f,    10.0053101578424f},
  { 115.273680586040f,    7.51576871353677f},
  { 119.591995029530f,    4.98354995961544f},
  { 123.813270535162f,    2.46169088485867f},
  { 127.896884871170f,    0.0f},
  { 131.810314895779f,    -2.35938143095389f},
  { 135.529947626365f,    -4.58467988787899f},
  { 139.040908715140f,    -6.65525720600019f},
  { 93.9134300246089f,    14.2023070245089f},
  { 97.9970443606170f,    12.1314342637763f},
  { 102.218319866249f,    9.89281564338174f},
  { 106.536634309738f,    7.51576871353677f},
  { 110.905157447889f,    5.04108531994386f},
  { 115.273680586040f,    2.51810250878842f},
  { 119.591995029530f,    -1.97826187754271e-15f},
  { 123.813270535162f,    -2.46169088485867f},
  { 127.896884871170f,    -4.82208777969570f},
  { 131.810314895779f,    -7.04640638085718f},
  { 135.529947626365f,    -9.11139042918331f},
  { 93.9134300246089f,    9.57681642812013f},
  { 97.9970443606170f,    7.34905112912407f},
  { 102.218319866249f,    4.98354995961544f},
  { 106.536634309738f,    2.51810250878842f},
  { 110.905157447889f,    2.00121863801437e-15f},
  { 115.273680586040f,    -2.51810250878842f},
  { 119.591995029530f,    -4.98354995961543f},
  { 123.813270535162f,    -7.34905112912407f},
  { 127.896884871170f,    -9.57681642812012f},
  { 131.810314895779f,    -11.6407231367706f},
  { 93.9134300246089f,    4.82208777969570f},
  { 97.9970443606170f,    2.46169088485867f},
  { 102.218319866249f,    -9.89130938771355e-16f},
  { 106.536634309738f,    -2.51810250878842f},
  { 110.905157447889f,    -5.04108531994386f},
  { 115.273680586040f,    -7.51576871353677f},
  { 119.591995029530f,    -9.89281564338174f},
  { 123.813270535162f,    -12.1314342637763f},
  { 127.896884871170f,    -14.2023070245090f},
  { 93.9134300246089f,    0.0f},
  { 97.9970443606170f,    -2.46169088485867f},
  { 102.218319866249f,    -4.98354995961543f},
  { 106.536634309738f,    -7.51576871353677f},
  { 110.905157447889f,    -10.0053101578424f},
  { 115.273680586040f,    -12.4012696205441f},
  { 119.591995029530f,    -14.6599491964913f},
  { 123.813270535162f,    -16.7485097268094f},
  { 93.9134300246089f,    -4.82208777969570f},
  { 97.9970443606170f,    -7.34905112912407f},
  { 102.218319866249f,    -9.89281564338174f},
  { 106.536634309738f,    -12.4012696205441f},
  { 110.905157447889f,    -14.8226211436785f},
  { 115.273680586040f,    -17.1105890228290f},
  { 119.591995029530f,    -19.2286421245509f},
  { 93.9134300246089f,    -9.57681642812013f},
  { 97.9970443606170f,    -12.1314342637763f},
  { 102.218319866249f,    -14.6599491964913f},
  { 106.536634309738f,    -17.1105890228290f},
  { 110.905157447889f,    -19.4351390051484f},
  { 115.273680586040f,    -21.5935345276613f},
  { 93.9134300246089f,    -14.2023070245089f},
  { 97.9970443606170f,    -16.7485097268094f},
  { 102.218319866249f,    -19.2286421245509f},
  { 106.536634309738f,    -21.5935345276613f},
  { 110.905157447889f,    -23.8001826021751f},
  { 93.9134300246089f,    -18.6465721493746f},
  { 97.9970443606170f,    -21.1524312620308f},
  { 102.218319866249f,    -23.5570490281124f},
  { 106.536634309738f,    -25.8154906887824f},
  { 93.9134300246089f,    -22.8701586038975f},
  { 97.9970443606170f,    -25.3095486329268f},
  { 102.218319866249f,    -27.6183932347186f},
  { 93.9134300246089f,    -26.8468676531246f},
  { 97.9970443606170f,    -29.2001208977139f},
  { 93.9134300246089f,    -30.5628733513954f},
  { -86.0865699753911f,    30.5628733513954f},
  { -82.0029556393830f,    29.2001208977139f},
  { -77.7816801337513f,    27.6183932347186f},
  { -73.4633656902618f,    25.8154906887824f},
  { -69.0948425521107f,    23.8001826021751f},
  { -64.7263194139596f,    21.5935345276613f},
  { -60.4080049704701f,    19.2286421245509f},
  { -56.1867294648384f,    16.7485097268094f},
  { -52.1031151288303f,    14.2023070245089f},
  { -48.1896851042214f,    11.6407231367706f},
  { -44.4700523736346f,    9.11139042918330f},
  { -40.9590912848605f,    6.65525720600019f},
  { -37.6638374535070f,    4.30441784763801f},
  { -34.5848092349368f,    2.08145509534415f},
  { -90.0f,    28.1748134648574f},
  { -86.0865699753911f,    26.8468676531246f},
  { -82.0029556393830f,    25.3095486329268f},
  { -77.7816801337513f,    23.5570490281124f},
  { -73.4633656902618f,    21.5935345276613f},
  { -69.0948425521107f,    19.4351390051484f},
  { -64.7263194139596f,    17.1105890228290f},
  { -60.4080049704701f,    14.6599491964913f},
  { -56.1867294648384f,    12.1314342637763f},
  { -52.1031151288303f,    9.57681642812012f},
  { -48.1896851042214f,    7.04640638085718f},
  { -44.4700523736346f,    4.58467988787900f},
  { -40.9590912848605f,    2.22732770546869f},
  { -37.6638374535070f,    3.41515458187918e-15f},
  { -34.5848092349368f,    -2.08145509534415f},
  { -90.0f,    24.3812184421970f},
  { -86.0865699753911f,    22.8701586038975f},
  { -82.0029556393830f,    21.1524312620308f},
  { -77.7816801337513f,    19.2286421245509f},
  { -73.4633656902618f,    17.1105890228290f},
  { -69.0948425521107f,    14.8226211436785f},
  { -64.7263194139596f,    12.4012696205441f},
  { -60.4080049704701f,    9.89281564338174f},
  { -56.1867294648384f,    7.34905112912407f},
  { -52.1031151288303f,    4.82208777969570f},
  { -48.1896851042214f,    2.35938143095389f},
  { -44.4700523736346f,    0.0f},
  { -40.9590912848605f,    -2.22732770546869f},
  { -37.6638374535070f,    -4.30441784763801f},
  { -90.0f,    20.3458076143384f},
  { -86.0865699753911f,    18.6465721493746f},
  { -82.0029556393830f,    16.7485097268094f},
  { -77.7816801337513f,    14.6599491964913f},
  { -73.4633656902618f,    12.4012696205441f},
  { -69.0948425521107f,    10.0053101578424f},
  { -64.7263194139596f,    7.51576871353677f},
  { -60.4080049704701f,    4.98354995961544f},
  { -56.1867294648384f,    2.46169088485867f},
  { -52.1031151288303f,    0.0f},
  { -48.1896851042214f,    -2.35938143095389f},
  { -44.4700523736346f,    -4.58467988787899f},
  { -40.9590912848605f,    -6.65525720600019f},
  { -90.0f,    16.0884002581729f},
  { -86.0865699753911f,    14.2023070245089f},
  { -82.0029556393830f,    12.1314342637763f},
  { -77.7816801337513f,    9.89281564338174f},
  { -73.4633656902618f,    7.51576871353677f},
  { -69.0948425521107f,    5.04108531994386f},
  { -64.7263194139596f,    2.51810250878842f},
  { -60.4080049704701f,    -1.97826187754271e-15f},
  { -56.1867294648384f,    -2.46169088485867f},
  { -52.1031151288303f,    -4.82208777969570f},
  { -48.1896851042214f,    -7.04640638085718f},
  { -44.4700523736346f,    -9.11139042918331f},
  { -90.0f,    11.6407231367706f},
  { -86.0865699753911f,    9.57681642812013f},
  { -82.0029556393830f,    7.34905112912407f},
  { -77.7816801337513f,    4.98354995961544f},
  { -73.4633656902618f,    2.51810250878842f},
  { -69.0948425521107f,    2.00121863801437e-15f},
  { -64.7263194139596f,    -2.51810250878842f},
  { -60.4080049704701f,    -4.98354995961543f},
  { -56.1867294648384f,    -7.34905112912407f},
  { -52.1031151288303f,    -9.57681642812012f},
  { -48.1896851042214f,    -11.6407231367706f},
  { -90.0f,    7.04640638085718f},
  { -86.0865699753911f,    4.82208777969570f},
  { -82.0029556393830f,    2.46169088485867f},
  { -77.7816801337513f,    -9.89130938771355e-16f},
  { -73.4633656902618f,    -2.51810250878842f},
  { -69.0948425521107f,    -5.04108531994386f},
  { -64.7263194139596f,    -7.51576871353677f},
  { -60.4080049704701f,    -9.89281564338174f},
  { -56.1867294648384f,    -12.1314342637763f},
  { -52.1031151288303f,    -14.2023070245090f},
  { -90.0f,    2.35938143095389f},
  { -86.0865699753911f,    0.0f},
  { -82.0029556393830f,    -2.46169088485867f},
  { -77.7816801337513f,    -4.98354995961543f},
  { -73.4633656902618f,    -7.51576871353677f},
  { -69.0948425521107f,    -10.0053101578424f},
  { -64.7263194139596f,    -12.4012696205441f},
  { -60.4080049704701f,    -14.6599491964913f},
  { -56.1867294648384f,    -16.7485097268094f},
  { -90.0f,    -2.35938143095389f},
  { -86.0865699753911f,    -4.82208777969570f},
  { -82.0029556393830f,    -7.34905112912407f},
  { -77.7816801337513f,    -9.89281564338174f},
  { -73.4633656902618f,    -12.4012696205441f},
  { -69.0948425521107f,    -14.8226211436785f},
  { -64.7263194139596f,    -17.1105890228290f},
  { -60.4080049704701f,    -19.2286421245509f},
  { -90.0f,    -7.04640638085718f},
  { -86.0865699753911f,    -9.57681642812013f},
  { -82.0029556393830f,    -12.1314342637763f},
  { -77.7816801337513f,    -14.6599491964913f},
  { -73.4633656902618f,    -17.1105890228290f},
  { -69.0948425521107f,    -19.4351390051484f},
  { -64.7263194139596f,    -21.5935345276613f},
  { -90.0f,    -11.6407231367706f},
  { -86.0865699753911f,    -14.2023070245089f},
  { -82.0029556393830f,    -16.7485097268094f},
  { -77.7816801337513f,    -19.2286421245509f},
  { -73.4633656902618f,    -21.5935345276613f},
  { -69.0948425521107f,    -23.8001826021751f},
  { -90.0f,    -16.0884002581729f},
  { -86.0865699753911f,    -18.6465721493746f},
  { -82.0029556393830f,    -21.1524312620308f},
  { -77.7816801337513f,    -23.5570490281124f},
  { -73.4633656902618f,    -25.8154906887824f},
  { -90.0f,    -20.3458076143384f},
  { -86.0865699753911f,    -22.8701586038975f},
  { -82.0029556393830f,    -25.3095486329268f},
  { -77.7816801337513f,    -27.6183932347186f},
  { -90.0f,    -24.3812184421970f},
  { -86.0865699753911f,    -26.8468676531246f},
  { -82.0029556393830f,    -29.2001208977139f},
  { -90.0f,    -28.1748134648574f},
  { -86.0865699753911f,    -30.5628733513954f},
  { -93.9134300246089f,    30.5628733513954f},
  { -93.9134300246089f,    26.8468676531246f},
  { -93.9134300246089f,    22.8701586038975f},
  { -93.9134300246089f,    18.6465721493746f},
  { -93.9134300246089f,    14.2023070245089f},
  { -93.9134300246089f,    9.57681642812012f},
  { -93.9134300246089f,    4.82208777969570f},
  { -93.9134300246089f,    0.0f},
  { -93.9134300246089f,    -4.82208777969570f},
  { -93.9134300246089f,    -9.57681642812013f},
  { -93.9134300246089f,    -14.2023070245089f},
  { -93.9134300246089f,    -18.6465721493746f},
  { -93.9134300246089f,    -22.8701586038975f},
  { -93.9134300246089f,    -26.8468676531246f},
  { -93.9134300246089f,    -30.5628733513954f},
  { -97.9970443606170f,    29.2001208977139f},
  { -97.9970443606170f,    25.3095486329268f},
  { -97.9970443606170f,    21.1524312620308f},
  { -97.9970443606170f,    16.7485097268094f},
  { -97.9970443606170f,    12.1314342637763f},
  { -97.9970443606170f,    7.34905112912407f},
  { -97.9970443606170f,    2.46169088485867f},
  { -97.9970443606170f,    -2.46169088485866f},
  { -97.9970443606170f,    -7.34905112912407f},
  { -97.9970443606170f,    -12.1314342637763f},
  { -97.9970443606170f,    -16.7485097268094f},
  { -97.9970443606170f,    -21.1524312620308f},
  { -97.9970443606170f,    -25.3095486329268f},
  { -97.9970443606170f,    -29.2001208977139f},
  { -102.218319866249f,    27.6183932347186f},
  { -102.218319866249f,    23.5570490281124f},
  { -102.218319866249f,    19.2286421245509f},
  { -102.218319866249f,    14.6599491964913f},
  { -102.218319866249f,    9.89281564338174f},
  { -102.218319866249f,    4.98354995961543f},
  { -102.218319866249f,    0.0f},
  { -102.218319866249f,    -4.98354995961543f},
  { -102.218319866249f,    -9.89281564338174f},
  { -102.218319866249f,    -14.6599491964913f},
  { -102.218319866249f,    -19.2286421245509f},
  { -102.218319866249f,    -23.5570490281124f},
  { -102.218319866249f,    -27.6183932347186f},
  { -106.536634309738f,    25.8154906887824f},
  { -106.536634309738f,    21.5935345276613f},
  { -106.536634309738f,    17.1105890228290f},
  { -106.536634309738f,    12.4012696205441f},
  { -106.536634309738f,    7.51576871353677f},
  { -106.536634309738f,    2.51810250878842f},
  { -106.536634309738f,    -2.51810250878842f},
  { -106.536634309738f,    -7.51576871353677f},
  { -106.536634309738f,    -12.4012696205441f},
  { -106.536634309738f,    -17.1105890228290f},
  { -106.536634309738f,    -21.5935345276613f},
  { -106.536634309738f,    -25.8154906887824f},
  { -110.905157447889f,    23.8001826021751f},
  { -110.905157447889f,    19.4351390051484f},
  { -110.905157447889f,    14.8226211436785f},
  { -110.905157447889f,    10.0053101578424f},
  { -110.905157447889f,    5.04108531994386f},
  { -110.905157447889f,    0.0f},
  { -110.905157447889f,    -5.04108531994386f},
  { -110.905157447889f,    -10.0053101578424f},
  { -110.905157447889f,    -14.8226211436785f},
  { -110.905157447889f,    -19.4351390051484f},
  { -110.905157447889f,    -23.8001826021751f},
  { -115.273680586040f,    21.5935345276613f},
  { -115.273680586040f,    17.1105890228290f},
  { -115.273680586040f,    12.4012696205441f},
  { -115.273680586040f,    7.51576871353677f},
  { -115.273680586040f,    2.51810250878842f},
  { -115.273680586040f,    -2.51810250878842f},
  { -115.273680586040f,    -7.51576871353677f},
  { -115.273680586040f,    -12.4012696205441f},
  { -115.273680586040f,    -17.1105890228290f},
  { -115.273680586040f,    -21.5935345276613f},
  { -119.591995029530f,    19.2286421245509f},
  { -119.591995029530f,    14.6599491964913f},
  { -119.591995029530f,    9.89281564338174f},
  { -119.591995029530f,    4.98354995961543f},
  { -119.591995029530f,    0.0f},
  { -119.591995029530f,    -4.98354995961543f},
  { -119.591995029530f,    -9.89281564338174f},
  { -119.591995029530f,    -14.6599491964913f},
  { -119.591995029530f,    -19.2286421245509f},
  { -123.813270535162f,    16.7485097268094f},
  { -123.813270535162f,    12.1314342637763f},
  { -123.813270535162f,    7.34905112912407f},
  { -123.813270535162f,    2.46169088485866f},
  { -123.813270535162f,    -2.46169088485867f},
  { -123.813270535162f,    -7.34905112912407f},
  { -123.813270535162f,    -12.1314342637763f},
  { -123.813270535162f,    -16.7485097268094f},
  { -127.896884871170f,    14.2023070245089f},
  { -127.896884871170f,    9.57681642812012f},
  { -127.896884871170f,    4.82208777969570f},
  { -127.896884871170f,    0.0f},
  { -127.896884871170f,    -4.82208777969570f},
  { -127.896884871170f,    -9.57681642812012f},
  { -127.896884871170f,    -14.2023070245089f},
  { -131.810314895779f,    11.6407231367706f},
  { -131.810314895779f,    7.04640638085718f},
  { -131.810314895779f,    2.35938143095389f},
  { -131.810314895779f,    -2.35938143095389f},
  { -131.810314895779f,    -7.04640638085718f},
  { -131.810314895779f,    -11.6407231367706f},
  { -135.529947626365f,    9.11139042918330f},
  { -135.529947626365f,    4.58467988787900f},
  { -135.529947626365f,    0.0f},
  { -135.529947626365f,    -4.58467988787900f},
  { -135.529947626365f,    -9.11139042918330f},
  { -139.040908715140f,    6.65525720600019f},
  { -139.040908715140f,    2.22732770546869f},
  { -139.040908715140f,    -2.22732770546869f},
  { -139.040908715140f,    -6.65525720600019f},
  { -142.336162546493f,    4.30441784763801f},
  { -142.336162546493f,    0.0f},
  { -142.336162546493f,    -4.30441784763801f},
  { -145.415190765063f,    2.08145509534415f},
  { -145.415190765063f,    -2.08145509534415f},
  { 83.2241374186358f,    33.3762411909610f},
  { 78.8115722734176f,    31.9552588002486f},
  { 74.2797234415508f,    30.2845539162313f},
  { 69.6820165485128f,    28.3641579940766f},
  { 65.0767512746904f,    26.2079258130924f},
  { 60.5227999362094f,    23.8444836331026f},
  { 56.0752009382906f,    21.3159868409545f},
  { 51.7814426078490f,    18.6746301777835f},
  { 47.6790114502229f,    15.9775208518168f},
  { 43.7944094237249f,    13.2810244556367f},
  { 40.1434881637101f,    10.6357872625078f},
  { 36.7327233897000f,    8.08330568762523f},
  { 33.5609911178468f,    5.65435411978892f},
  { 79.9508244823842f,    36.3090746817282f},
  { 75.1861723840984f,    34.8051847162798f},
  { 70.3356309148077f,    33.0173980941400f},
  { 65.4672787464387f,    30.9502839364211f},
  { 60.6506952965071f,    28.6254919805188f},
  { 55.9511420987458f,    26.0817679706224f},
  { 51.4246101431585f,    23.3721653820369f},
  { 47.1145960682886f,    20.5588156366481f},
  { 43.0509090546002f,    17.7063797357477f},
  { 39.2502669310586f,    14.8756643542430f},
  { 35.7181136274822f,    12.1186878603085f},
  { 32.4510238289644f,    9.47587408300922f},
  { 76.1979576809410f,    39.3183133212560f},
  { 71.0674356666295f,    37.6983190307739f},
  { 65.9051574478893f,    35.7584993532105f},
  { 60.7940677526006f,    33.5113110478349f},
  { 55.8123169635717f,    30.9895202379748f},
  { 51.0265526631344f,    28.2445194108839f},
  { 46.4876134810656f,    25.3414348215180f},
  { 42.2290808062821f,    22.3520200659455f},
  { 38.2682973779569f,    19.3469890055802f},
  { 34.6089810768371f,    16.3894688982939f},
  { 31.2445048389264f,    13.5306916059832f},
  { 71.8920659165349f,    42.3426786955557f},
  { 66.3994932351077f,    40.5649232343349f},
  { 60.9559054807148f,    38.4329531156089f},
  { 55.6559242989723f,    35.9713652379727f},
  { 50.5797341367620f,    33.2274263316385f},
  { 45.7872281770683f,    30.2668945240291f},
  { 41.3162572276061f,    27.1667567482826f},
  { 37.1843345585789f,    24.0065729514478f},
  { 33.3924317939481f,    20.8604039083950f},
  { 29.9294869771194f,    17.7908732767440f},
  { 66.9618670980072f,    45.3018275871428f},
  { 61.1400181243506f,    43.3183341586149f},
  { 55.4784071051414f,    40.9529798978727f},
  { 50.0746762369146f,    38.2469452829479f},
  { 45.0000000000000f,    35.2643896827547f},
  { 40.2969906210295f,    32.0853535661644f},
  { 35.9825296400333f,    28.7964908583898f},
  { 32.0533416234180f,    25.4817604601001f},
  { 28.4922372576800f,    22.2149507419033f},
  { 61.3513396175938f,    48.0984455182148f},
  { 55.2751795539159f,    45.8600299163516f},
  { 49.4992949875800f,    43.2255973783311f},
  { 44.1090707626928f,    40.2561142171484f},
  { 39.1522843343359f,    37.0326759995490f},
  { 34.6438140225457f,    33.6469345870906f},
  { 30.5739237032190f,    30.1909667191287f},
  { 26.9169702938414f,    26.7485898826142f},
  { 55.0402264518196f,    50.6246018090077f},
  { 48.8379484773704f,    48.0892359713763f},
  { 43.0930352611326f,    45.1640025965927f},
  { 37.8585718508094f,    41.9297634938229f},
  { 33.1451248925788f,    38.4812261402498f},
  { 28.9332208850461f,    34.9163921844233f},
  { 25.1855644625686f,    31.3271738136226f},
  { 48.0700392255717f,    52.7731586216040f},
  { 41.9243499008255f,    49.9164435802129f},
  { 36.3863520431954f,    46.7006131020347f},
  { 31.4584365263532f,    43.2221370583012f},
  { 27.1065240586251f,    39.5827886580625f},
  { 23.2772786024330f,    35.8801454421164f},
  { 40.5670532448711f,    54.4535116240989f},
  { 34.6983566904268f,    51.2781748097357f},
  { 29.5495629136742f,    47.7981724013467f},
  { 25.0647323337058f,    44.1173475260784f},
  { 21.1685029410260f,    40.3364105668209f},
  { 32.7470884470526f,    55.6083365989711f},
  { 27.3767115889741f,    52.1484754914004f},
  { 22.7737003323099f,    48.4550964117505f},
  { 18.8326032413430f,    44.6297351128758f},
  { 24.8888123884354f,    56.2254959548673f},
  { 20.1936716283567f,    52.5425946285438f},
  { 16.2399555770788f,    48.7030734152615f},
  { 17.2789906790511f,    56.3394641229789f},
  { 13.3583240910863f,    52.5111392197807f},
  { 10.1538057187409f,    56.0210853216559f},
  { 83.2241374186358f,    -33.3762411909610f},
  { 79.9508244823842f,    -36.3090746817282f},
  { 76.1979576809410f,    -39.3183133212560f},
  { 71.8920659165349f,    -42.3426786955557f},
  { 66.9618670980072f,    -45.3018275871428f},
  { 61.3513396175938f,    -48.0984455182149f},
  { 55.0402264518196f,    -50.6246018090077f},
  { 48.0700392255717f,    -52.7731586216040f},
  { 40.5670532448711f,    -54.4535116240989f},
  { 32.7470884470526f,    -55.6083365989711f},
  { 24.8888123884354f,    -56.2254959548673f},
  { 17.2789906790511f,    -56.3394641229789f},
  { 10.1538057187409f,    -56.0210853216559f},
  { 78.8115722734176f,    -31.9552588002486f},
  { 75.1861723840984f,    -34.8051847162798f},
  { 71.0674356666295f,    -37.6983190307739f},
  { 66.3994932351077f,    -40.5649232343349f},
  { 61.1400181243506f,    -43.3183341586149f},
  { 55.2751795539159f,    -45.8600299163516f},
  { 48.8379484773704f,    -48.0892359713763f},
  { 41.9243499008255f,    -49.9164435802129f},
  { 34.6983566904268f,    -51.2781748097357f},
  { 27.3767115889741f,    -52.1484754914004f},
  { 20.1936716283567f,    -52.5425946285438f},
  { 13.3583240910863f,    -52.5111392197807f},
  { 74.2797234415508f,    -30.2845539162313f},
  { 70.3356309148077f,    -33.0173980941400f},
  { 65.9051574478893f,    -35.7584993532105f},
  { 60.9559054807148f,    -38.4329531156089f},
  { 55.4784071051414f,    -40.9529798978727f},
  { 49.4992949875800f,    -43.2255973783311f},
  { 43.0930352611326f,    -45.1640025965927f},
  { 36.3863520431954f,    -46.7006131020347f},
  { 29.5495629136742f,    -47.7981724013467f},
  { 22.7737003323099f,    -48.4550964117505f},
  { 16.2399555770788f,    -48.7030734152615f},
  { 69.6820165485128f,    -28.3641579940766f},
  { 65.4672787464387f,    -30.9502839364211f},
  { 60.7940677526006f,    -33.5113110478349f},
  { 55.6559242989723f,    -35.9713652379727f},
  { 50.0746762369146f,    -38.2469452829479f},
  { 44.1090707626928f,    -40.2561142171484f},
  { 37.8585718508094f,    -41.9297634938229f},
  { 31.4584365263532f,    -43.2221370583012f},
  { 25.0647323337058f,    -44.1173475260784f},
  { 18.8326032413430f,    -44.6297351128758f},
  { 65.0767512746904f,    -26.2079258130924f},
  { 60.6506952965071f,    -28.6254919805188f},
  { 55.8123169635717f,    -30.9895202379748f},
  { 50.5797341367620f,    -33.2274263316385f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 39.1522843343359f,    -37.0326759995490f},
  { 33.1451248925788f,    -38.4812261402498f},
  { 27.1065240586251f,    -39.5827886580625f},
  { 21.1685029410260f,    -40.3364105668209f},
  { 60.5227999362094f,    -23.8444836331026f},
  { 55.9511420987458f,    -26.0817679706224f},
  { 51.0265526631344f,    -28.2445194108839f},
  { 45.7872281770683f,    -30.2668945240291f},
  { 40.2969906210295f,    -32.0853535661644f},
  { 34.6438140225457f,    -33.6469345870906f},
  { 28.9332208850461f,    -34.9163921844233f},
  { 23.2772786024330f,    -35.8801454421164f},
  { 56.0752009382906f,    -21.3159868409545f},
  { 51.4246101431585f,    -23.3721653820369f},
  { 46.4876134810656f,    -25.3414348215180f},
  { 41.3162572276061f,    -27.1667567482826f},
  { 35.9825296400333f,    -28.7964908583898f},
  { 30.5739237032190f,    -30.1909667191287f},
  { 25.1855644625686f,    -31.3271738136226f},
  { 51.7814426078490f,    -18.6746301777835f},
  { 47.1145960682886f,    -20.5588156366481f},
  { 42.2290808062821f,    -22.3520200659455f},
  { 37.1843345585789f,    -24.0065729514478f},
  { 32.0533416234180f,    -25.4817604601001f},
  { 26.9169702938414f,    -26.7485898826143f},
  { 47.6790114502229f,    -15.9775208518168f},
  { 43.0509090546002f,    -17.7063797357477f},
  { 38.2682973779569f,    -19.3469890055802f},
  { 33.3924317939481f,    -20.8604039083950f},
  { 28.4922372576800f,    -22.2149507419033f},
  { 43.7944094237249f,    -13.2810244556367f},
  { 39.2502669310586f,    -14.8756643542430f},
  { 34.6089810768371f,    -16.3894688982939f},
  { 29.9294869771194f,    -17.7908732767440f},
  { 40.1434881637101f,    -10.6357872625078f},
  { 35.7181136274822f,    -12.1186878603085f},
  { 31.2445048389264f,    -13.5306916059832f},
  { 36.7327233897000f,    -8.08330568762523f},
  { 32.4510238289644f,    -9.47587408300921f},
  { 33.5609911178468f,    -5.65435411978892f},
  { -10.1538057187409f,    56.0210853216559f},
  { -13.3583240910863f,    52.5111392197807f},
  { -16.2399555770788f,    48.7030734152615f},
  { -18.8326032413430f,    44.6297351128758f},
  { -21.1685029410260f,    40.3364105668209f},
  { -23.2772786024330f,    35.8801454421164f},
  { -25.1855644625686f,    31.3271738136226f},
  { -26.9169702938414f,    26.7485898826143f},
  { -28.4922372576800f,    22.2149507419033f},
  { -29.9294869771194f,    17.7908732767440f},
  { -31.2445048389264f,    13.5306916059832f},
  { -32.4510238289644f,    9.47587408300921f},
  { -33.5609911178468f,    5.65435411978892f},
  { -17.2789906790511f,    56.3394641229789f},
  { -20.1936716283567f,    52.5425946285438f},
  { -22.7737003323099f,    48.4550964117505f},
  { -25.0647323337058f,    44.1173475260784f},
  { -27.1065240586251f,    39.5827886580625f},
  { -28.9332208850461f,    34.9163921844233f},
  { -30.5739237032190f,    30.1909667191287f},
  { -32.0533416234180f,    25.4817604601001f},
  { -33.3924317939481f,    20.8604039083950f},
  { -34.6089810768371f,    16.3894688982939f},
  { -35.7181136274822f,    12.1186878603085f},
  { -36.7327233897000f,    8.08330568762523f},
  { -24.8888123884354f,    56.2254959548673f},
  { -27.3767115889741f,    52.1484754914004f},
  { -29.5495629136742f,    47.7981724013467f},
  { -31.4584365263532f,    43.2221370583012f},
  { -33.1451248925788f,    38.4812261402498f},
  { -34.6438140225457f,    33.6469345870906f},
  { -35.9825296400333f,    28.7964908583898f},
  { -37.1843345585789f,    24.0065729514478f},
  { -38.2682973779569f,    19.3469890055802f},
  { -39.2502669310586f,    14.8756643542430f},
  { -40.1434881637101f,    10.6357872625078f},
  { -32.7470884470526f,    55.6083365989711f},
  { -34.6983566904268f,    51.2781748097357f},
  { -36.3863520431954f,    46.7006131020347f},
  { -37.8585718508094f,    41.9297634938229f},
  { -39.1522843343359f,    37.0326759995490f},
  { -40.2969906210295f,    32.0853535661644f},
  { -41.3162572276061f,    27.1667567482826f},
  { -42.2290808062821f,    22.3520200659455f},
  { -43.0509090546002f,    17.7063797357477f},
  { -43.7944094237249f,    13.2810244556367f},
  { -40.5670532448711f,    54.4535116240989f},
  { -41.9243499008255f,    49.9164435802129f},
  { -43.0930352611326f,    45.1640025965927f},
  { -44.1090707626928f,    40.2561142171484f},
  { -45.0000000000000f,    35.2643896827547f},
  { -45.7872281770683f,    30.2668945240291f},
  { -46.4876134810656f,    25.3414348215180f},
  { -47.1145960682886f,    20.5588156366481f},
  { -47.6790114502229f,    15.9775208518168f},
  { -48.0700392255717f,    52.7731586216040f},
  { -48.8379484773704f,    48.0892359713763f},
  { -49.4992949875800f,    43.2255973783311f},
  { -50.0746762369146f,    38.2469452829479f},
  { -50.5797341367620f,    33.2274263316385f},
  { -51.0265526631344f,    28.2445194108839f},
  { -51.4246101431585f,    23.3721653820369f},
  { -51.7814426078490f,    18.6746301777835f},
  { -55.0402264518196f,    50.6246018090077f},
  { -55.2751795539159f,    45.8600299163516f},
  { -55.4784071051414f,    40.9529798978727f},
  { -55.6559242989723f,    35.9713652379727f},
  { -55.8123169635717f,    30.9895202379748f},
  { -55.9511420987458f,    26.0817679706224f},
  { -56.0752009382906f,    21.3159868409545f},
  { -61.3513396175938f,    48.0984455182149f},
  { -61.1400181243506f,    43.3183341586149f},
  { -60.9559054807148f,    38.4329531156089f},
  { -60.7940677526006f,    33.5113110478349f},
  { -60.6506952965071f,    28.6254919805188f},
  { -60.5227999362094f,    23.8444836331026f},
  { -66.9618670980072f,    45.3018275871428f},
  { -66.3994932351077f,    40.5649232343349f},
  { -65.9051574478893f,    35.7584993532105f},
  { -65.4672787464387f,    30.9502839364211f},
  { -65.0767512746904f,    26.2079258130924f},
  { -71.8920659165349f,    42.3426786955557f},
  { -71.0674356666295f,    37.6983190307739f},
  { -70.3356309148077f,    33.0173980941400f},
  { -69.6820165485128f,    28.3641579940766f},
  { -76.1979576809410f,    39.3183133212560f},
  { -75.1861723840984f,    34.8051847162798f},
  { -74.2797234415508f,    30.2845539162313f},
  { -79.9508244823842f,    36.3090746817282f},
  { -78.8115722734176f,    31.9552588002486f},
  { -83.2241374186358f,    33.3762411909610f},
  { -10.1538057187409f,    -56.0210853216559f},
  { -17.2789906790511f,    -56.3394641229789f},
  { -24.8888123884354f,    -56.2254959548673f},
  { -32.7470884470526f,    -55.6083365989711f},
  { -40.5670532448711f,    -54.4535116240989f},
  { -48.0700392255717f,    -52.7731586216040f},
  { -55.0402264518196f,    -50.6246018090077f},
  { -61.3513396175938f,    -48.0984455182149f},
  { -66.9618670980072f,    -45.3018275871428f},
  { -71.8920659165349f,    -42.3426786955557f},
  { -76.1979576809410f,    -39.3183133212560f},
  { -79.9508244823842f,    -36.3090746817282f},
  { -83.2241374186358f,    -33.3762411909610f},
  { -13.3583240910863f,    -52.5111392197807f},
  { -20.1936716283567f,    -52.5425946285438f},
  { -27.3767115889741f,    -52.1484754914004f},
  { -34.6983566904268f,    -51.2781748097357f},
  { -41.9243499008255f,    -49.9164435802129f},
  { -48.8379484773704f,    -48.0892359713763f},
  { -55.2751795539159f,    -45.8600299163516f},
  { -61.1400181243506f,    -43.3183341586148f},
  { -66.3994932351077f,    -40.5649232343349f},
  { -71.0674356666295f,    -37.6983190307739f},
  { -75.1861723840984f,    -34.8051847162798f},
  { -78.8115722734176f,    -31.9552588002486f},
  { -16.2399555770788f,    -48.7030734152615f},
  { -22.7737003323099f,    -48.4550964117505f},
  { -29.5495629136742f,    -47.7981724013467f},
  { -36.3863520431954f,    -46.7006131020347f},
  { -43.0930352611326f,    -45.1640025965927f},
  { -49.4992949875800f,    -43.2255973783311f},
  { -55.4784071051414f,    -40.9529798978727f},
  { -60.9559054807148f,    -38.4329531156089f},
  { -65.9051574478893f,    -35.7584993532105f},
  { -70.3356309148077f,    -33.0173980941400f},
  { -74.2797234415508f,    -30.2845539162313f},
  { -18.8326032413430f,    -44.6297351128758f},
  { -25.0647323337058f,    -44.1173475260784f},
  { -31.4584365263532f,    -43.2221370583012f},
  { -37.8585718508094f,    -41.9297634938229f},
  { -44.1090707626928f,    -40.2561142171484f},
  { -50.0746762369146f,    -38.2469452829479f},
  { -55.6559242989723f,    -35.9713652379727f},
  { -60.7940677526006f,    -33.5113110478349f},
  { -65.4672787464387f,    -30.9502839364211f},
  { -69.6820165485128f,    -28.3641579940766f},
  { -21.1685029410260f,    -40.3364105668209f},
  { -27.1065240586251f,    -39.5827886580625f},
  { -33.1451248925788f,    -38.4812261402498f},
  { -39.1522843343359f,    -37.0326759995489f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -50.5797341367620f,    -33.2274263316385f},
  { -55.8123169635717f,    -30.9895202379748f},
  { -60.6506952965071f,    -28.6254919805188f},
  { -65.0767512746904f,    -26.2079258130924f},
  { -23.2772786024330f,    -35.8801454421164f},
  { -28.9332208850461f,    -34.9163921844233f},
  { -34.6438140225457f,    -33.6469345870906f},
  { -40.2969906210295f,    -32.0853535661644f},
  { -45.7872281770683f,    -30.2668945240291f},
  { -51.0265526631344f,    -28.2445194108839f},
  { -55.9511420987458f,    -26.0817679706224f},
  { -60.5227999362094f,    -23.8444836331026f},
  { -25.1855644625686f,    -31.3271738136226f},
  { -30.5739237032190f,    -30.1909667191287f},
  { -35.9825296400333f,    -28.7964908583898f},
  { -41.3162572276060f,    -27.1667567482826f},
  { -46.4876134810656f,    -25.3414348215180f},
  { -51.4246101431585f,    -23.3721653820369f},
  { -56.0752009382906f,    -21.3159868409545f},
  { -26.9169702938414f,    -26.7485898826143f},
  { -32.0533416234180f,    -25.4817604601001f},
  { -37.1843345585789f,    -24.0065729514478f},
  { -42.2290808062821f,    -22.3520200659455f},
  { -47.1145960682886f,    -20.5588156366481f},
  { -51.7814426078490f,    -18.6746301777835f},
  { -28.4922372576800f,    -22.2149507419033f},
  { -33.3924317939481f,    -20.8604039083950f},
  { -38.2682973779569f,    -19.3469890055802f},
  { -43.0509090546002f,    -17.7063797357477f},
  { -47.6790114502229f,    -15.9775208518168f},
  { -29.9294869771194f,    -17.7908732767439f},
  { -34.6089810768371f,    -16.3894688982939f},
  { -39.2502669310586f,    -14.8756643542430f},
  { -43.7944094237249f,    -13.2810244556367f},
  { -31.2445048389264f,    -13.5306916059832f},
  { -35.7181136274822f,    -12.1186878603085f},
  { -40.1434881637101f,    -10.6357872625078f},
  { -32.4510238289644f,    -9.47587408300921f},
  { -36.7327233897000f,    -8.08330568762523f},
  { -33.5609911178468f,    -5.65435411978892f},
  { 169.846194281259f,    56.0210853216559f},
  { 166.641675908914f,    52.5111392197807f},
  { 163.760044422921f,    48.7030734152615f},
  { 161.167396758657f,    44.6297351128758f},
  { 158.831497058974f,    40.3364105668209f},
  { 156.722721397567f,    35.8801454421164f},
  { 154.814435537431f,    31.3271738136226f},
  { 153.083029706159f,    26.7485898826143f},
  { 151.507762742320f,    22.2149507419033f},
  { 150.070513022881f,    17.7908732767440f},
  { 148.755495161074f,    13.5306916059832f},
  { 147.548976171036f,    9.47587408300921f},
  { 146.439008882153f,    5.65435411978892f},
  { 162.721009320949f,    56.3394641229789f},
  { 159.806328371643f,    52.5425946285438f},
  { 157.226299667690f,    48.4550964117505f},
  { 154.935267666294f,    44.1173475260784f},
  { 152.893475941375f,    39.5827886580625f},
  { 151.066779114954f,    34.9163921844233f},
  { 149.426076296781f,    30.1909667191287f},
  { 147.946658376582f,    25.4817604601001f},
  { 146.607568206052f,    20.8604039083950f},
  { 145.391018923163f,    16.3894688982939f},
  { 144.281886372518f,    12.1186878603085f},
  { 143.267276610300f,    8.08330568762523f},
  { 155.111187611565f,    56.2254959548673f},
  { 152.623288411026f,    52.1484754914004f},
  { 150.450437086326f,    47.7981724013467f},
  { 148.541563473647f,    43.2221370583012f},
  { 146.854875107421f,    38.4812261402498f},
  { 145.356185977454f,    33.6469345870906f},
  { 144.017470359967f,    28.7964908583898f},
  { 142.815665441421f,    24.0065729514478f},
  { 141.731702622043f,    19.3469890055802f},
  { 140.749733068941f,    14.8756643542430f},
  { 139.856511836290f,    10.6357872625078f},
  { 147.252911552947f,    55.6083365989711f},
  { 145.301643309573f,    51.2781748097357f},
  { 143.613647956805f,    46.7006131020347f},
  { 142.141428149191f,    41.9297634938229f},
  { 140.847715665664f,    37.0326759995490f},
  { 139.703009378971f,    32.0853535661644f},
  { 138.683742772394f,    27.1667567482826f},
  { 137.770919193718f,    22.3520200659455f},
  { 136.949090945400f,    17.7063797357477f},
  { 136.205590576275f,    13.2810244556367f},
  { 139.432946755129f,    54.4535116240989f},
  { 138.075650099174f,    49.9164435802129f},
  { 136.906964738867f,    45.1640025965927f},
  { 135.890929237307f,    40.2561142171484f},
  { 135.0f,    35.2643896827547f},
  { 134.212771822932f,    30.2668945240291f},
  { 133.512386518934f,    25.3414348215180f},
  { 132.885403931711f,    20.5588156366481f},
  { 132.320988549777f,    15.9775208518168f},
  { 131.929960774428f,    52.7731586216040f},
  { 131.162051522630f,    48.0892359713763f},
  { 130.500705012420f,    43.2255973783311f},
  { 129.925323763085f,    38.2469452829479f},
  { 129.420265863238f,    33.2274263316385f},
  { 128.973447336866f,    28.2445194108839f},
  { 128.575389856841f,    23.3721653820369f},
  { 128.218557392151f,    18.6746301777835f},
  { 124.959773548180f,    50.6246018090077f},
  { 124.724820446084f,    45.8600299163516f},
  { 124.521592894859f,    40.9529798978727f},
  { 124.344075701028f,    35.9713652379727f},
  { 124.187683036428f,    30.9895202379748f},
  { 124.048857901254f,    26.0817679706224f},
  { 123.924799061709f,    21.3159868409545f},
  { 118.648660382406f,    48.0984455182149f},
  { 118.859981875649f,    43.3183341586149f},
  { 119.044094519285f,    38.4329531156089f},
  { 119.205932247399f,    33.5113110478349f},
  { 119.349304703493f,    28.6254919805188f},
  { 119.477200063791f,    23.8444836331026f},
  { 113.038132901993f,    45.3018275871428f},
  { 113.600506764892f,    40.5649232343349f},
  { 114.094842552111f,    35.7584993532105f},
  { 114.532721253561f,    30.9502839364211f},
  { 114.923248725310f,    26.2079258130924f},
  { 108.107934083465f,    42.3426786955557f},
  { 108.932564333371f,    37.6983190307739f},
  { 109.664369085192f,    33.0173980941400f},
  { 110.317983451487f,    28.3641579940766f},
  { 103.802042319059f,    39.3183133212560f},
  { 104.813827615902f,    34.8051847162798f},
  { 105.720276558449f,    30.2845539162313f},
  { 100.049175517616f,    36.3090746817282f},
  { 101.188427726582f,    31.9552588002486f},
  { 96.7758625813643f,    33.3762411909610f},
  { 169.846194281259f,    -56.0210853216559f},
  { 162.721009320949f,    -56.3394641229789f},
  { 155.111187611565f,    -56.2254959548673f},
  { 147.252911552947f,    -55.6083365989711f},
  { 139.432946755129f,    -54.4535116240989f},
  { 131.929960774428f,    -52.7731586216040f},
  { 124.959773548180f,    -50.6246018090077f},
  { 118.648660382406f,    -48.0984455182149f},
  { 113.038132901993f,    -45.3018275871428f},
  { 108.107934083465f,    -42.3426786955557f},
  { 103.802042319059f,    -39.3183133212560f},
  { 100.049175517616f,    -36.3090746817282f},
  { 96.7758625813643f,    -33.3762411909610f},
  { 166.641675908914f,    -52.5111392197807f},
  { 159.806328371643f,    -52.5425946285438f},
  { 152.623288411026f,    -52.1484754914004f},
  { 145.301643309573f,    -51.2781748097357f},
  { 138.075650099174f,    -49.9164435802129f},
  { 131.162051522630f,    -48.0892359713763f},
  { 124.724820446084f,    -45.8600299163516f},
  { 118.859981875649f,    -43.3183341586148f},
  { 113.600506764892f,    -40.5649232343349f},
  { 108.932564333371f,    -37.6983190307739f},
  { 104.813827615902f,    -34.8051847162798f},
  { 101.188427726582f,    -31.9552588002486f},
  { 163.760044422921f,    -48.7030734152615f},
  { 157.226299667690f,    -48.4550964117505f},
  { 150.450437086326f,    -47.7981724013467f},
  { 143.613647956805f,    -46.7006131020347f},
  { 136.906964738867f,    -45.1640025965927f},
  { 130.500705012420f,    -43.2255973783311f},
  { 124.521592894859f,    -40.9529798978727f},
  { 119.044094519285f,    -38.4329531156089f},
  { 114.094842552111f,    -35.7584993532105f},
  { 109.664369085192f,    -33.0173980941400f},
  { 105.720276558449f,    -30.2845539162313f},
  { 161.167396758657f,    -44.6297351128758f},
  { 154.935267666294f,    -44.1173475260784f},
  { 148.541563473647f,    -43.2221370583012f},
  { 142.141428149191f,    -41.9297634938229f},
  { 135.890929237307f,    -40.2561142171484f},
  { 129.925323763085f,    -38.2469452829479f},
  { 124.344075701028f,    -35.9713652379727f},
  { 119.205932247399f,    -33.5113110478349f},
  { 114.532721253561f,    -30.9502839364211f},
  { 110.317983451487f,    -28.3641579940766f},
  { 158.831497058974f,    -40.3364105668209f},
  { 152.893475941375f,    -39.5827886580625f},
  { 146.854875107421f,    -38.4812261402498f},
  { 140.847715665664f,    -37.0326759995489f},
  { 135.0f,    -35.2643896827547f},
  { 129.420265863238f,    -33.2274263316385f},
  { 124.187683036428f,    -30.9895202379748f},
  { 119.349304703493f,    -28.6254919805188f},
  { 114.923248725310f,    -26.2079258130924f},
  { 156.722721397567f,    -35.8801454421164f},
  { 151.066779114954f,    -34.9163921844233f},
  { 145.356185977454f,    -33.6469345870906f},
  { 139.703009378971f,    -32.0853535661644f},
  { 134.212771822932f,    -30.2668945240291f},
  { 128.973447336866f,    -28.2445194108839f},
  { 124.048857901254f,    -26.0817679706224f},
  { 119.477200063791f,    -23.8444836331026f},
  { 154.814435537431f,    -31.3271738136226f},
  { 149.426076296781f,    -30.1909667191287f},
  { 144.017470359967f,    -28.7964908583898f},
  { 138.683742772394f,    -27.1667567482826f},
  { 133.512386518934f,    -25.3414348215180f},
  { 128.575389856841f,    -23.3721653820369f},
  { 123.924799061709f,    -21.3159868409545f},
  { 153.083029706159f,    -26.7485898826143f},
  { 147.946658376582f,    -25.4817604601001f},
  { 142.815665441421f,    -24.0065729514478f},
  { 137.770919193718f,    -22.3520200659455f},
  { 132.885403931711f,    -20.5588156366481f},
  { 128.218557392151f,    -18.6746301777835f},
  { 151.507762742320f,    -22.2149507419033f},
  { 146.607568206052f,    -20.8604039083950f},
  { 141.731702622043f,    -19.3469890055802f},
  { 136.949090945400f,    -17.7063797357477f},
  { 132.320988549777f,    -15.9775208518168f},
  { 150.070513022881f,    -17.7908732767439f},
  { 145.391018923163f,    -16.3894688982939f},
  { 140.749733068941f,    -14.8756643542430f},
  { 136.205590576275f,    -13.2810244556367f},
  { 148.755495161074f,    -13.5306916059832f},
  { 144.281886372518f,    -12.1186878603085f},
  { 139.856511836290f,    -10.6357872625078f},
  { 147.548976171036f,    -9.47587408300922f},
  { 143.267276610300f,    -8.08330568762523f},
  { 146.439008882153f,    -5.65435411978892f},
  { -169.846194281259f,    56.0210853216559f},
  { -162.721009320949f,    56.3394641229789f},
  { -155.111187611565f,    56.2254959548673f},
  { -147.252911552947f,    55.6083365989711f},
  { -139.432946755129f,    54.4535116240989f},
  { -131.929960774428f,    52.7731586216040f},
  { -124.959773548180f,    50.6246018090077f},
  { -118.648660382406f,    48.0984455182149f},
  { -113.038132901993f,    45.3018275871428f},
  { -108.107934083465f,    42.3426786955557f},
  { -103.802042319059f,    39.3183133212560f},
  { -100.049175517616f,    36.3090746817282f},
  { -96.7758625813643f,    33.3762411909610f},
  { -166.641675908914f,    52.5111392197807f},
  { -159.806328371643f,    52.5425946285438f},
  { -152.623288411026f,    52.1484754914004f},
  { -145.301643309573f,    51.2781748097357f},
  { -138.075650099174f,    49.9164435802129f},
  { -131.162051522630f,    48.0892359713763f},
  { -124.724820446084f,    45.8600299163516f},
  { -118.859981875649f,    43.3183341586148f},
  { -113.600506764892f,    40.5649232343349f},
  { -108.932564333371f,    37.6983190307739f},
  { -104.813827615902f,    34.8051847162798f},
  { -101.188427726582f,    31.9552588002486f},
  { -163.760044422921f,    48.7030734152615f},
  { -157.226299667690f,    48.4550964117505f},
  { -150.450437086326f,    47.7981724013467f},
  { -143.613647956805f,    46.7006131020347f},
  { -136.906964738867f,    45.1640025965927f},
  { -130.500705012420f,    43.2255973783311f},
  { -124.521592894859f,    40.9529798978727f},
  { -119.044094519285f,    38.4329531156089f},
  { -114.094842552111f,    35.7584993532105f},
  { -109.664369085192f,    33.0173980941400f},
  { -105.720276558449f,    30.2845539162313f},
  { -161.167396758657f,    44.6297351128758f},
  { -154.935267666294f,    44.1173475260784f},
  { -148.541563473647f,    43.2221370583012f},
  { -142.141428149191f,    41.9297634938229f},
  { -135.890929237307f,    40.2561142171484f},
  { -129.925323763085f,    38.2469452829479f},
  { -124.344075701028f,    35.9713652379727f},
  { -119.205932247399f,    33.5113110478349f},
  { -114.532721253561f,    30.9502839364211f},
  { -110.317983451487f,    28.3641579940766f},
  { -158.831497058974f,    40.3364105668209f},
  { -152.893475941375f,    39.5827886580625f},
  { -146.854875107421f,    38.4812261402498f},
  { -140.847715665664f,    37.0326759995489f},
  { -135.0f,    35.2643896827547f},
  { -129.420265863238f,    33.2274263316385f},
  { -124.187683036428f,    30.9895202379748f},
  { -119.349304703493f,    28.6254919805188f},
  { -114.923248725310f,    26.2079258130924f},
  { -156.722721397567f,    35.8801454421164f},
  { -151.066779114954f,    34.9163921844233f},
  { -145.356185977454f,    33.6469345870906f},
  { -139.703009378971f,    32.0853535661644f},
  { -134.212771822932f,    30.2668945240291f},
  { -128.973447336866f,    28.2445194108839f},
  { -124.048857901254f,    26.0817679706224f},
  { -119.477200063791f,    23.8444836331026f},
  { -154.814435537431f,    31.3271738136226f},
  { -149.426076296781f,    30.1909667191287f},
  { -144.017470359967f,    28.7964908583898f},
  { -138.683742772394f,    27.1667567482826f},
  { -133.512386518934f,    25.3414348215180f},
  { -128.575389856841f,    23.3721653820369f},
  { -123.924799061709f,    21.3159868409545f},
  { -153.083029706159f,    26.7485898826143f},
  { -147.946658376582f,    25.4817604601001f},
  { -142.815665441421f,    24.0065729514478f},
  { -137.770919193718f,    22.3520200659455f},
  { -132.885403931711f,    20.5588156366481f},
  { -128.218557392151f,    18.6746301777835f},
  { -151.507762742320f,    22.2149507419033f},
  { -146.607568206052f,    20.8604039083950f},
  { -141.731702622043f,    19.3469890055802f},
  { -136.949090945400f,    17.7063797357477f},
  { -132.320988549777f,    15.9775208518168f},
  { -150.070513022881f,    17.7908732767439f},
  { -145.391018923163f,    16.3894688982939f},
  { -140.749733068941f,    14.8756643542430f},
  { -136.205590576275f,    13.2810244556367f},
  { -148.755495161074f,    13.5306916059832f},
  { -144.281886372518f,    12.1186878603085f},
  { -139.856511836290f,    10.6357872625078f},
  { -147.548976171036f,    9.47587408300922f},
  { -143.267276610300f,    8.08330568762523f},
  { -146.439008882153f,    5.65435411978892f},
  { -169.846194281259f,    -56.0210853216559f},
  { -166.641675908914f,    -52.5111392197807f},
  { -163.760044422921f,    -48.7030734152615f},
  { -161.167396758657f,    -44.6297351128758f},
  { -158.831497058974f,    -40.3364105668209f},
  { -156.722721397567f,    -35.8801454421164f},
  { -154.814435537431f,    -31.3271738136226f},
  { -153.083029706159f,    -26.7485898826143f},
  { -151.507762742320f,    -22.2149507419033f},
  { -150.070513022881f,    -17.7908732767440f},
  { -148.755495161074f,    -13.5306916059832f},
  { -147.548976171036f,    -9.47587408300921f},
  { -146.439008882153f,    -5.65435411978892f},
  { -162.721009320949f,    -56.3394641229789f},
  { -159.806328371643f,    -52.5425946285438f},
  { -157.226299667690f,    -48.4550964117505f},
  { -154.935267666294f,    -44.1173475260784f},
  { -152.893475941375f,    -39.5827886580625f},
  { -151.066779114954f,    -34.9163921844233f},
  { -149.426076296781f,    -30.1909667191287f},
  { -147.946658376582f,    -25.4817604601001f},
  { -146.607568206052f,    -20.8604039083950f},
  { -145.391018923163f,    -16.3894688982939f},
  { -144.281886372518f,    -12.1186878603085f},
  { -143.267276610300f,    -8.08330568762523f},
  { -155.111187611565f,    -56.2254959548673f},
  { -152.623288411026f,    -52.1484754914004f},
  { -150.450437086326f,    -47.7981724013467f},
  { -148.541563473647f,    -43.2221370583012f},
  { -146.854875107421f,    -38.4812261402498f},
  { -145.356185977454f,    -33.6469345870906f},
  { -144.017470359967f,    -28.7964908583898f},
  { -142.815665441421f,    -24.0065729514478f},
  { -141.731702622043f,    -19.3469890055802f},
  { -140.749733068941f,    -14.8756643542430f},
  { -139.856511836290f,    -10.6357872625078f},
  { -147.252911552947f,    -55.6083365989711f},
  { -145.301643309573f,    -51.2781748097357f},
  { -143.613647956805f,    -46.7006131020347f},
  { -142.141428149191f,    -41.9297634938229f},
  { -140.847715665664f,    -37.0326759995490f},
  { -139.703009378971f,    -32.0853535661644f},
  { -138.683742772394f,    -27.1667567482826f},
  { -137.770919193718f,    -22.3520200659455f},
  { -136.949090945400f,    -17.7063797357477f},
  { -136.205590576275f,    -13.2810244556367f},
  { -139.432946755129f,    -54.4535116240989f},
  { -138.075650099174f,    -49.9164435802129f},
  { -136.906964738867f,    -45.1640025965927f},
  { -135.890929237307f,    -40.2561142171484f},
  { -135.0f,    -35.2643896827547f},
  { -134.212771822932f,    -30.2668945240291f},
  { -133.512386518934f,    -25.3414348215180f},
  { -132.885403931711f,    -20.5588156366481f},
  { -132.320988549777f,    -15.9775208518168f},
  { -131.929960774428f,    -52.7731586216040f},
  { -131.162051522630f,    -48.0892359713763f},
  { -130.500705012420f,    -43.2255973783311f},
  { -129.925323763085f,    -38.2469452829479f},
  { -129.420265863238f,    -33.2274263316385f},
  { -128.973447336866f,    -28.2445194108839f},
  { -128.575389856841f,    -23.3721653820369f},
  { -128.218557392151f,    -18.6746301777835f},
  { -124.959773548180f,    -50.6246018090077f},
  { -124.724820446084f,    -45.8600299163516f},
  { -124.521592894859f,    -40.9529798978727f},
  { -124.344075701028f,    -35.9713652379727f},
  { -124.187683036428f,    -30.9895202379748f},
  { -124.048857901254f,    -26.0817679706224f},
  { -123.924799061709f,    -21.3159868409545f},
  { -118.648660382406f,    -48.0984455182149f},
  { -118.859981875649f,    -43.3183341586149f},
  { -119.044094519285f,    -38.4329531156089f},
  { -119.205932247399f,    -33.5113110478349f},
  { -119.349304703493f,    -28.6254919805188f},
  { -119.477200063791f,    -23.8444836331026f},
  { -113.038132901993f,    -45.3018275871428f},
  { -113.600506764892f,    -40.5649232343349f},
  { -114.094842552111f,    -35.7584993532105f},
  { -114.532721253561f,    -30.9502839364211f},
  { -114.923248725310f,    -26.2079258130924f},
  { -108.107934083465f,    -42.3426786955557f},
  { -108.932564333371f,    -37.6983190307739f},
  { -109.664369085192f,    -33.0173980941400f},
  { -110.317983451487f,    -28.3641579940766f},
  { -103.802042319059f,    -39.3183133212560f},
  { -104.813827615902f,    -34.8051847162798f},
  { -105.720276558449f,    -30.2845539162313f},
  { -100.049175517616f,    -36.3090746817282f},
  { -101.188427726582f,    -31.9552588002486f},
  { -96.7758625813643f,    -33.3762411909610f}};

/* 3LD geosphere 'ico' [16 0] (2562 points) */
const float __geosphere_ico_16_0_dirs_deg[2562][2] =
{ { 31.7174744114610f,    0.0f},
  { 148.282525588539f,    0.0f},
  { -148.282525588539f,    0.0f},
  { -31.7174744114610f,    0.0f},
  { 0.0f,    58.2825255885390f},
  { 0.0f,    -58.2825255885390f},
  { 180.0f,    -58.2825255885390f},
  { 180.0f,    58.2825255885390f},
  { 90.0f,    31.7174744114610f},
  { -90.0f,    31.7174744114610f},
  { -90.0f,    -31.7174744114610f},
  { 90.0f,    -31.7174744114610f},
  { 0.0f,    61.5963754664474f},
  { 0.0f,    65.1310748815380f},
  { 0.0f,    68.8798871556760f},
  { 0.0f,    72.8279621494862f},
  { 0.0f,    76.9513489599765f},
  { 0.0f,    81.2167693990410f},
  { 0.0f,    85.5824320004692f},
  { 0.0f,    90.0f},
  { 180.0f,    85.5824320004692f},
  { 180.0f,    81.2167693990410f},
  { 180.0f,    76.9513489599765f},
  { 180.0f,    72.8279621494862f},
  { 180.0f,    68.8798871556760f},
  { 180.0f,    65.1310748815380f},
  { 180.0f,    61.5963754664474f},
  { 6.15662931139232f,    59.1619275753025f},
  { 7.09478492022928f,    62.5976437065956f},
  { 8.36785885565049f,    66.2517584756324f},
  { 10.1918500362761f,    70.1070308125077f},
  { 13.0152040462228f,    74.1292066099994f},
  { 17.9319334355159f,    78.2521457203827f},
  { 28.3399673707642f,    82.3182215759721f},
  { 58.2825255885390f,    85.6954596895373f},
  { 121.717474411461f,    85.6954596895373f},
  { 151.660032629236f,    82.3182215759721f},
  { 162.068066564484f,    78.2521457203827f},
  { 166.984795953777f,    74.1292066099994f},
  { 169.808149963724f,    70.1070308125077f},
  { 171.632141144350f,    66.2517584756324f},
  { 172.905215079771f,    62.5976437065956f},
  { 173.843370688608f,    59.1619275753025f},
  { 13.0152040462228f,    59.7634611728920f},
  { 15.0920752881032f,    63.2447902091638f},
  { 17.9319334355159f,    66.9090449293358f},
  { 22.0237505161462f,    70.7071432582827f},
  { 28.3399673707642f,    74.5425010655038f},
  { 38.9734473368657f,    78.2112745279495f},
  { 58.2825255885390f,    81.2276448816065f},
  { 90.0f,    82.5216860448010f},
  { 121.717474411461f,    81.2276448816065f},
  { 141.026552663134f,    78.2112745279495f},
  { 151.660032629236f,    74.5425010655038f},
  { 157.976249483854f,    70.7071432582827f},
  { 162.068066564484f,    66.9090449293358f},
  { 164.907924711897f,    63.2447902091638f},
  { 166.984795953777f,    59.7634611728920f},
  { 20.4752527058863f,    59.9993016074299f},
  { 23.8110044533780f,    63.4231532306817f},
  { 28.3399673707642f,    66.9524350458202f},
  { 34.7391081893039f,    70.4845192768597f},
  { 44.1517511327961f,    73.8307483550080f},
  { 58.2825255885390f,    76.6439357010638f},
  { 78.3592768632294f,    78.3485977036152f},
  { 101.640723136771f,    78.3485977036152f},
  { 121.717474411461f,    76.6439357010638f},
  { 135.848248867204f,    73.8307483550080f},
  { 145.260891810696f,    70.4845192768597f},
  { 151.660032629236f,    66.9524350458202f},
  { 156.188995546622f,    63.4231532306817f},
  { 159.524747294114f,    59.9993016074299f},
  { 28.3399673707642f,    59.7900946927798f},
  { 32.9114931857064f,    63.0380044634447f},
  { 38.9734473368657f,    66.2771675744593f},
  { 47.1679114977868f,    69.3489926147040f},
  { 58.2825255885390f,    72.0f},
  { 72.8279621494862f,    73.8654271672930f},
  { 90.0f,    74.5495629136742f},
  { 107.172037850514f,    73.8654271672930f},
  { 121.717474411461f,    72.0f},
  { 132.832088502213f,    69.3489926147040f},
  { 141.026552663134f,    66.2771675744593f},
  { 147.088506814294f,    63.0380044634447f},
  { 151.660032629236f,    59.7900946927798f},
  { 36.3333661325874f,    59.0812124975607f},
  { 41.9526190816497f,    62.0400275829325f},
  { 49.1320813134511f,    64.8616135923531f},
  { 58.2825255885390f,    67.3560642989362f},
  { 69.6541923856616f,    69.2698711279441f},
  { 82.9535936191428f,    70.3250877451075f},
  { 97.0464063808572f,    70.3250877451075f},
  { 110.345807614338f,    69.2698711279441f},
  { 121.717474411461f,    67.3560642989362f},
  { 130.867918686549f,    64.8616135923531f},
  { 138.047380918350f,    62.0400275829325f},
  { 143.666633867413f,    59.0812124975607f},
  { 44.1517511327961f,    57.8562154800787f},
  { 50.5099309427673f,    60.4399963615406f},
  { 58.2825255885390f,    62.7723551183935f},
  { 67.6072290710451f,    64.6675682905954f},
  { 78.3592768632294f,    65.9208196976405f},
  { 90.0f,    66.3612884340067f},
  { 101.640723136771f,    65.9208196976405f},
  { 112.392770928955f,    64.6675682905954f},
  { 121.717474411461f,    62.7723551183935f},
  { 129.490069057233f,    60.4399963615406f},
  { 135.848248867204f,    57.8562154800787f},
  { 51.5288148377489f,    56.1414441683991f},
  { 58.2825255885390f,    58.3045403104627f},
  { 66.1808024884231f,    60.1361529891940f},
  { 75.1646413480039f,    61.4806421734066f},
  { 84.9544063480986f,    62.1962922822525f},
  { 95.0455936519014f,    62.1962922822525f},
  { 104.835358651996f,    61.4806421734066f},
  { 113.819197511577f,    60.1361529891940f},
  { 121.717474411461f,    58.3045403104628f},
  { 128.471185162251f,    56.1414441683991f},
  { 58.2825255885390f,    54.0f},
  { 65.1310748815380f,    55.7373027782207f},
  { 72.8279621494862f,    57.1023923174897f},
  { 81.2167693990410f,    57.9794735312795f},
  { 90.0f,    58.2825255885390f},
  { 98.7832306009590f,    57.9794735312795f},
  { 107.172037850514f,    57.1023923174897f},
  { 114.868925118462f,    55.7373027782207f},
  { 121.717474411461f,    54.0f},
  { 64.3267273510545f,    51.5187284413421f},
  { 71.0499775117081f,    52.8566942795796f},
  { 78.3592768632294f,    53.8155050474511f},
  { 86.0716405780585f,    54.3172601576489f},
  { 93.9283594219415f,    54.3172601576489f},
  { 101.640723136771f,    53.8155050474511f},
  { 108.950022488292f,    52.8566942795796f},
  { 115.673272648946f,    51.5187284413421f},
  { 69.6541923856616f,    48.7938239499718f},
  { 76.1141128518623f,    49.7784476076377f},
  { 82.9535936191428f,    50.4009770857117f},
  { 90.0f,    50.6141704736363f},
  { 97.0464063808572f,    50.4009770857117f},
  { 103.885887148138f,    49.7784476076377f},
  { 110.345807614338f,    48.7938239499718f},
  { 74.3086441830498f,    45.9190620385707f},
  { 80.4324561201698f,    46.6051356747135f},
  { 86.7842228465557f,    46.9605776701645f},
  { 93.2157771534443f,    46.9605776701645f},
  { 99.5675438798302f,    46.6051356747135f},
  { 105.691355816950f,    45.9190620385707f},
  { 78.3592768632294f,    42.9780676586279f},
  { 84.1189512527134f,    43.4213789054837f},
  { 90.0f,    43.5723495188822f},
  { 95.8810487472866f,    43.4213789054837f},
  { 101.640723136771f,    42.9780676586279f},
  { 81.8830323534313f,    40.0403293417631f},
  { 87.2781463566519f,    40.2927991911889f},
  { 92.7218536433482f,    40.2927991911889f},
  { 98.1169676465687f,    40.0403293417631f},
  { 84.9544063480986f,    37.1600422047896f},
  { 90.0f,    37.2671844918060f},
  { 95.0455936519014f,    37.1600422047896f},
  { 87.6406185690461f,    34.3768199416111f},
  { 92.3593814309539f,    34.3768199416111f},
  { -6.15662931139232f,    59.1619275753025f},
  { -13.0152040462228f,    59.7634611728920f},
  { -20.4752527058863f,    59.9993016074299f},
  { -28.3399673707642f,    59.7900946927798f},
  { -36.3333661325874f,    59.0812124975607f},
  { -44.1517511327961f,    57.8562154800787f},
  { -51.5288148377489f,    56.1414441683991f},
  { -58.2825255885390f,    54.0f},
  { -64.3267273510545f,    51.5187284413421f},
  { -69.6541923856616f,    48.7938239499718f},
  { -74.3086441830498f,    45.9190620385707f},
  { -78.3592768632294f,    42.9780676586279f},
  { -81.8830323534313f,    40.0403293417631f},
  { -84.9544063480986f,    37.1600422047896f},
  { -87.6406185690461f,    34.3768199416111f},
  { -7.09478492022928f,    62.5976437065956f},
  { -15.0920752881032f,    63.2447902091638f},
  { -23.8110044533780f,    63.4231532306817f},
  { -32.9114931857064f,    63.0380044634447f},
  { -41.9526190816497f,    62.0400275829325f},
  { -50.5099309427673f,    60.4399963615406f},
  { -58.2825255885390f,    58.3045403104627f},
  { -65.1310748815380f,    55.7373027782207f},
  { -71.0499775117081f,    52.8566942795796f},
  { -76.1141128518623f,    49.7784476076377f},
  { -80.4324561201698f,    46.6051356747135f},
  { -84.1189512527134f,    43.4213789054837f},
  { -87.2781463566518f,    40.2927991911889f},
  { -90.0f,    37.2671844918060f},
  { -92.3593814309539f,    34.3768199416111f},
  { -8.36785885565049f,    66.2517584756324f},
  { -17.9319334355159f,    66.9090449293358f},
  { -28.3399673707642f,    66.9524350458202f},
  { -38.9734473368657f,    66.2771675744593f},
  { -49.1320813134511f,    64.8616135923531f},
  { -58.2825255885390f,    62.7723551183935f},
  { -66.1808024884231f,    60.1361529891940f},
  { -72.8279621494862f,    57.1023923174897f},
  { -78.3592768632294f,    53.8155050474511f},
  { -82.9535936191428f,    50.4009770857117f},
  { -86.7842228465557f,    46.9605776701645f},
  { -90.0f,    43.5723495188822f},
  { -92.7218536433482f,    40.2927991911889f},
  { -95.0455936519014f,    37.1600422047896f},
  { -10.1918500362761f,    70.1070308125077f},
  { -22.0237505161462f,    70.7071432582827f},
  { -34.7391081893039f,    70.4845192768597f},
  { -47.1679114977868f,    69.3489926147040f},
  { -58.2825255885390f,    67.3560642989362f},
  { -67.6072290710451f,    64.6675682905954f},
  { -75.1646413480039f,    61.4806421734066f},
  { -81.2167693990410f,    57.9794735312795f},
  { -86.0716405780585f,    54.3172601576489f},
  { -90.0f,    50.6141704736363f},
  { -93.2157771534443f,    46.9605776701645f},
  { -95.8810487472866f,    43.4213789054837f},
  { -98.1169676465687f,    40.0403293417631f},
  { -13.0152040462228f,    74.1292066099994f},
  { -28.3399673707642f,    74.5425010655038f},
  { -44.1517511327961f,    73.8307483550080f},
  { -58.2825255885390f,    72.0f},
  { -69.6541923856616f,    69.2698711279441f},
  { -78.3592768632294f,    65.9208196976405f},
  { -84.9544063480986f,    62.1962922822525f},
  { -90.0f,    58.2825255885390f},
  { -93.9283594219415f,    54.3172601576489f},
  { -97.0464063808572f,    50.4009770857117f},
  { -99.5675438798302f,    46.6051356747135f},
  { -101.640723136771f,    42.9780676586279f},
  { -17.9319334355159f,    78.2521457203827f},
  { -38.9734473368657f,    78.2112745279495f},
  { -58.2825255885390f,    76.6439357010638f},
  { -72.8279621494862f,    73.8654271672930f},
  { -82.9535936191428f,    70.3250877451075f},
  { -90.0f,    66.3612884340067f},
  { -95.0455936519014f,    62.1962922822525f},
  { -98.7832306009590f,    57.9794735312795f},
  { -101.640723136771f,    53.8155050474511f},
  { -103.885887148138f,    49.7784476076377f},
  { -105.691355816950f,    45.9190620385707f},
  { -28.3399673707642f,    82.3182215759721f},
  { -58.2825255885390f,    81.2276448816065f},
  { -78.3592768632294f,    78.3485977036152f},
  { -90.0000000000000f,    74.5495629136742f},
  { -97.0464063808572f,    70.3250877451075f},
  { -101.640723136771f,    65.9208196976405f},
  { -104.835358651996f,    61.4806421734066f},
  { -107.172037850514f,    57.1023923174897f},
  { -108.950022488292f,    52.8566942795796f},
  { -110.345807614338f,    48.7938239499718f},
  { -58.2825255885390f,    85.6954596895373f},
  { -90.0f,    82.5216860448010f},
  { -101.640723136771f,    78.3485977036152f},
  { -107.172037850514f,    73.8654271672930f},
  { -110.345807614338f,    69.2698711279441f},
  { -112.392770928955f,    64.6675682905954f},
  { -113.819197511577f,    60.1361529891940f},
  { -114.868925118462f,    55.7373027782207f},
  { -115.673272648946f,    51.5187284413421f},
  { -121.717474411461f,    85.6954596895373f},
  { -121.717474411461f,    81.2276448816065f},
  { -121.717474411461f,    76.6439357010638f},
  { -121.717474411461f,    72.0f},
  { -121.717474411461f,    67.3560642989362f},
  { -121.717474411461f,    62.7723551183935f},
  { -121.717474411461f,    58.3045403104627f},
  { -121.717474411461f,    54.0f},
  { -151.660032629236f,    82.3182215759721f},
  { -141.026552663134f,    78.2112745279495f},
  { -135.848248867204f,    73.8307483550080f},
  { -132.832088502213f,    69.3489926147040f},
  { -130.867918686549f,    64.8616135923531f},
  { -129.490069057233f,    60.4399963615406f},
  { -128.471185162251f,    56.1414441683991f},
  { -162.068066564484f,    78.2521457203827f},
  { -151.660032629236f,    74.5425010655038f},
  { -145.260891810696f,    70.4845192768597f},
  { -141.026552663134f,    66.2771675744593f},
  { -138.047380918350f,    62.0400275829325f},
  { -135.848248867204f,    57.8562154800787f},
  { -166.984795953777f,    74.1292066099994f},
  { -157.976249483854f,    70.7071432582827f},
  { -151.660032629236f,    66.9524350458202f},
  { -147.088506814294f,    63.0380044634447f},
  { -143.666633867413f,    59.0812124975607f},
  { -169.808149963724f,    70.1070308125077f},
  { -162.068066564484f,    66.9090449293358f},
  { -156.188995546622f,    63.4231532306817f},
  { -151.660032629236f,    59.7900946927798f},
  { -171.632141144350f,    66.2517584756324f},
  { -164.907924711897f,    63.2447902091638f},
  { -159.524747294114f,    59.9993016074299f},
  { -172.905215079771f,    62.5976437065956f},
  { -166.984795953777f,    59.7634611728920f},
  { -173.843370688608f,    59.1619275753025f},
  { 6.15662931139232f,    -59.1619275753025f},
  { 13.0152040462228f,    -59.7634611728920f},
  { 20.4752527058863f,    -59.9993016074299f},
  { 28.3399673707642f,    -59.7900946927798f},
  { 36.3333661325874f,    -59.0812124975607f},
  { 44.1517511327961f,    -57.8562154800787f},
  { 51.5288148377489f,    -56.1414441683991f},
  { 58.2825255885390f,    -54.0f},
  { 64.3267273510545f,    -51.5187284413421f},
  { 69.6541923856616f,    -48.7938239499718f},
  { 74.3086441830498f,    -45.9190620385707f},
  { 78.3592768632294f,    -42.9780676586279f},
  { 81.8830323534313f,    -40.0403293417631f},
  { 84.9544063480986f,    -37.1600422047896f},
  { 87.6406185690461f,    -34.3768199416111f},
  { 0.0f,    -61.5963754664474f},
  { 7.09478492022928f,    -62.5976437065956f},
  { 15.0920752881032f,    -63.2447902091638f},
  { 23.8110044533780f,    -63.4231532306817f},
  { 32.9114931857064f,    -63.0380044634447f},
  { 41.9526190816497f,    -62.0400275829325f},
  { 50.5099309427673f,    -60.4399963615406f},
  { 58.2825255885390f,    -58.3045403104627f},
  { 65.1310748815380f,    -55.7373027782207f},
  { 71.0499775117081f,    -52.8566942795796f},
  { 76.1141128518623f,    -49.7784476076377f},
  { 80.4324561201698f,    -46.6051356747135f},
  { 84.1189512527134f,    -43.4213789054837f},
  { 87.2781463566518f,    -40.2927991911889f},
  { 90.0f,    -37.2671844918060f},
  { 92.3593814309539f,    -34.3768199416111f},
  { 0.0f,    -65.1310748815380f},
  { 8.36785885565049f,    -66.2517584756324f},
  { 17.9319334355159f,    -66.9090449293358f},
  { 28.3399673707642f,    -66.9524350458202f},
  { 38.9734473368657f,    -66.2771675744593f},
  { 49.1320813134511f,    -64.8616135923531f},
  { 58.2825255885390f,    -62.7723551183935f},
  { 66.1808024884231f,    -60.1361529891940f},
  { 72.8279621494862f,    -57.1023923174897f},
  { 78.3592768632294f,    -53.8155050474511f},
  { 82.9535936191428f,    -50.4009770857117f},
  { 86.7842228465557f,    -46.9605776701645f},
  { 90.0f,    -43.5723495188822f},
  { 92.7218536433482f,    -40.2927991911889f},
  { 95.0455936519014f,    -37.1600422047896f},
  { 0.0f,    -68.8798871556760f},
  { 10.1918500362761f,    -70.1070308125077f},
  { 22.0237505161462f,    -70.7071432582827f},
  { 34.7391081893039f,    -70.4845192768597f},
  { 47.1679114977868f,    -69.3489926147040f},
  { 58.2825255885390f,    -67.3560642989362f},
  { 67.6072290710451f,    -64.6675682905954f},
  { 75.1646413480039f,    -61.4806421734066f},
  { 81.2167693990410f,    -57.9794735312795f},
  { 86.0716405780585f,    -54.3172601576489f},
  { 90.0f,    -50.6141704736363f},
  { 93.2157771534443f,    -46.9605776701645f},
  { 95.8810487472866f,    -43.4213789054837f},
  { 98.1169676465687f,    -40.0403293417631f},
  { 0.0f,    -72.8279621494862f},
  { 13.0152040462228f,    -74.1292066099994f},
  { 28.3399673707642f,    -74.5425010655038f},
  { 44.1517511327961f,    -73.8307483550080f},
  { 58.2825255885390f,    -72.0f},
  { 69.6541923856616f,    -69.2698711279441f},
  { 78.3592768632294f,    -65.9208196976405f},
  { 84.9544063480986f,    -62.1962922822525f},
  { 90.0f,    -58.2825255885390f},
  { 93.9283594219415f,    -54.3172601576489f},
  { 97.0464063808572f,    -50.4009770857117f},
  { 99.5675438798302f,    -46.6051356747135f},
  { 101.640723136771f,    -42.9780676586279f},
  { 0.0f,    -76.9513489599765f},
  { 17.9319334355159f,    -78.2521457203827f},
  { 38.9734473368657f,    -78.2112745279495f},
  { 58.2825255885390f,    -76.6439357010638f},
  { 72.8279621494862f,    -73.8654271672930f},
  { 82.9535936191428f,    -70.3250877451075f},
  { 90.0f,    -66.3612884340067f},
  { 95.0455936519014f,    -62.1962922822525f},
  { 98.7832306009590f,    -57.9794735312795f},
  { 101.640723136771f,    -53.8155050474511f},
  { 103.885887148138f,    -49.7784476076377f},
  { 105.691355816950f,    -45.9190620385707f},
  { 0.0f,    -81.2167693990410f},
  { 28.3399673707642f,    -82.3182215759721f},
  { 58.2825255885390f,    -81.2276448816065f},
  { 78.3592768632294f,    -78.3485977036152f},
  { 90.0000000000000f,    -74.5495629136742f},
  { 97.0464063808572f,    -70.3250877451075f},
  { 101.640723136771f,    -65.9208196976405f},
  { 104.835358651996f,    -61.4806421734066f},
  { 107.172037850514f,    -57.1023923174897f},
  { 108.950022488292f,    -52.8566942795796f},
  { 110.345807614338f,    -48.7938239499718f},
  { 0.0f,    -85.5824320004692f},
  { 58.2825255885390f,    -85.6954596895373f},
  { 90.0f,    -82.5216860448010f},
  { 101.640723136771f,    -78.3485977036152f},
  { 107.172037850514f,    -73.8654271672930f},
  { 110.345807614338f,    -69.2698711279441f},
  { 112.392770928955f,    -64.6675682905954f},
  { 113.819197511577f,    -60.1361529891940f},
  { 114.868925118462f,    -55.7373027782207f},
  { 115.673272648946f,    -51.5187284413421f},
  { 0.0f,    -90.0f},
  { 121.717474411461f,    -85.6954596895373f},
  { 121.717474411461f,    -81.2276448816065f},
  { 121.717474411461f,    -76.6439357010638f},
  { 121.717474411461f,    -72.0f},
  { 121.717474411461f,    -67.3560642989362f},
  { 121.717474411461f,    -62.7723551183935f},
  { 121.717474411461f,    -58.3045403104627f},
  { 121.717474411461f,    -54.0f},
  { 180.0f,    -85.5824320004692f},
  { 151.660032629236f,    -82.3182215759721f},
  { 141.026552663134f,    -78.2112745279495f},
  { 135.848248867204f,    -73.8307483550080f},
  { 132.832088502213f,    -69.3489926147040f},
  { 130.867918686549f,    -64.8616135923531f},
  { 129.490069057233f,    -60.4399963615406f},
  { 128.471185162251f,    -56.1414441683991f},
  { 180.0f,    -81.2167693990410f},
  { 162.068066564484f,    -78.2521457203827f},
  { 151.660032629236f,    -74.5425010655038f},
  { 145.260891810696f,    -70.4845192768597f},
  { 141.026552663134f,    -66.2771675744593f},
  { 138.047380918350f,    -62.0400275829325f},
  { 135.848248867204f,    -57.8562154800787f},
  { 180.0f,    -76.9513489599765f},
  { 166.984795953777f,    -74.1292066099994f},
  { 157.976249483854f,    -70.7071432582827f},
  { 151.660032629236f,    -66.9524350458202f},
  { 147.088506814294f,    -63.0380044634447f},
  { 143.666633867413f,    -59.0812124975607f},
  { 180.0f,    -72.8279621494862f},
  { 169.808149963724f,    -70.1070308125077f},
  { 162.068066564484f,    -66.9090449293358f},
  { 156.188995546622f,    -63.4231532306817f},
  { 151.660032629236f,    -59.7900946927798f},
  { 180.0f,    -68.8798871556760f},
  { 171.632141144350f,    -66.2517584756324f},
  { 164.907924711897f,    -63.2447902091638f},
  { 159.524747294114f,    -59.9993016074299f},
  { 180.0f,    -65.1310748815380f},
  { 172.905215079771f,    -62.5976437065956f},
  { 166.984795953777f,    -59.7634611728920f},
  { 180.0f,    -61.5963754664474f},
  { 173.843370688608f,    -59.1619275753025f},
  { -6.15662931139232f,    -59.1619275753025f},
  { -7.09478492022928f,    -62.5976437065956f},
  { -8.36785885565049f,    -66.2517584756324f},
  { -10.1918500362761f,    -70.1070308125077f},
  { -13.0152040462228f,    -74.1292066099994f},
  { -17.9319334355159f,    -78.2521457203827f},
  { -28.3399673707642f,    -82.3182215759721f},
  { -58.2825255885390f,    -85.6954596895373f},
  { -121.717474411461f,    -85.6954596895373f},
  { -151.660032629236f,    -82.3182215759721f},
  { -162.068066564484f,    -78.2521457203827f},
  { -166.984795953777f,    -74.1292066099994f},
  { -169.808149963724f,    -70.1070308125077f},
  { -171.632141144350f,    -66.2517584756324f},
  { -172.905215079771f,    -62.5976437065956f},
  { -173.843370688608f,    -59.1619275753025f},
  { -13.0152040462228f,    -59.7634611728920f},
  { -15.0920752881032f,    -63.2447902091638f},
  { -17.9319334355159f,    -66.9090449293358f},
  { -22.0237505161462f,    -70.7071432582827f},
  { -28.3399673707642f,    -74.5425010655038f},
  { -38.9734473368657f,    -78.2112745279495f},
  { -58.2825255885390f,    -81.2276448816065f},
  { -90.0f,    -82.5216860448010f},
  { -121.717474411461f,    -81.2276448816065f},
  { -141.026552663134f,    -78.2112745279495f},
  { -151.660032629236f,    -74.5425010655038f},
  { -157.976249483854f,    -70.7071432582827f},
  { -162.068066564484f,    -66.9090449293358f},
  { -164.907924711897f,    -63.2447902091638f},
  { -166.984795953777f,    -59.7634611728920f},
  { -20.4752527058863f,    -59.9993016074299f},
  { -23.8110044533780f,    -63.4231532306817f},
  { -28.3399673707642f,    -66.9524350458202f},
  { -34.7391081893039f,    -70.4845192768597f},
  { -44.1517511327961f,    -73.8307483550080f},
  { -58.2825255885390f,    -76.6439357010638f},
  { -78.3592768632294f,    -78.3485977036152f},
  { -101.640723136771f,    -78.3485977036152f},
  { -121.717474411461f,    -76.6439357010638f},
  { -135.848248867204f,    -73.8307483550080f},
  { -145.260891810696f,    -70.4845192768597f},
  { -151.660032629236f,    -66.9524350458202f},
  { -156.188995546622f,    -63.4231532306817f},
  { -159.524747294114f,    -59.9993016074299f},
  { -28.3399673707642f,    -59.7900946927798f},
  { -32.9114931857064f,    -63.0380044634447f},
  { -38.9734473368657f,    -66.2771675744593f},
  { -47.1679114977868f,    -69.3489926147040f},
  { -58.2825255885390f,    -72.0f},
  { -72.8279621494862f,    -73.8654271672930f},
  { -90.0f,    -74.5495629136742f},
  { -107.172037850514f,    -73.8654271672930f},
  { -121.717474411461f,    -72.0f},
  { -132.832088502213f,    -69.3489926147040f},
  { -141.026552663134f,    -66.2771675744593f},
  { -147.088506814294f,    -63.0380044634447f},
  { -151.660032629236f,    -59.7900946927798f},
  { -36.3333661325874f,    -59.0812124975607f},
  { -41.9526190816497f,    -62.0400275829325f},
  { -49.1320813134511f,    -64.8616135923531f},
  { -58.2825255885390f,    -67.3560642989362f},
  { -69.6541923856616f,    -69.2698711279441f},
  { -82.9535936191428f,    -70.3250877451075f},
  { -97.0464063808572f,    -70.3250877451075f},
  { -110.345807614338f,    -69.2698711279441f},
  { -121.717474411461f,    -67.3560642989362f},
  { -130.867918686549f,    -64.8616135923531f},
  { -138.047380918350f,    -62.0400275829325f},
  { -143.666633867413f,    -59.0812124975607f},
  { -44.1517511327961f,    -57.8562154800787f},
  { -50.5099309427673f,    -60.4399963615406f},
  { -58.2825255885390f,    -62.7723551183935f},
  { -67.6072290710451f,    -64.6675682905954f},
  { -78.3592768632294f,    -65.9208196976405f},
  { -90.0f,    -66.3612884340067f},
  { -101.640723136771f,    -65.9208196976405f},
  { -112.392770928955f,    -64.6675682905954f},
  { -121.717474411461f,    -62.7723551183935f},
  { -129.490069057233f,    -60.4399963615406f},
  { -135.848248867204f,    -57.8562154800787f},
  { -51.5288148377489f,    -56.1414441683991f},
  { -58.2825255885390f,    -58.3045403104627f},
  { -66.1808024884231f,    -60.1361529891940f},
  { -75.1646413480039f,    -61.4806421734066f},
  { -84.9544063480986f,    -62.1962922822525f},
  { -95.0455936519014f,    -62.1962922822525f},
  { -104.835358651996f,    -61.4806421734066f},
  { -113.819197511577f,    -60.1361529891940f},
  { -121.717474411461f,    -58.3045403104628f},
  { -128.471185162251f,    -56.1414441683991f},
  { -58.2825255885390f,    -54.0f},
  { -65.1310748815380f,    -55.7373027782207f},
  { -72.8279621494862f,    -57.1023923174897f},
  { -81.2167693990410f,    -57.9794735312795f},
  { -90.0f,    -58.2825255885390f},
  { -98.7832306009590f,    -57.9794735312795f},
  { -107.172037850514f,    -57.1023923174897f},
  { -114.868925118462f,    -55.7373027782207f},
  { -121.717474411461f,    -54.0f},
  { -64.3267273510545f,    -51.5187284413421f},
  { -71.0499775117081f,    -52.8566942795796f},
  { -78.3592768632294f,    -53.8155050474511f},
  { -86.0716405780585f,    -54.3172601576489f},
  { -93.9283594219415f,    -54.3172601576489f},
  { -101.640723136771f,    -53.8155050474511f},
  { -108.950022488292f,    -52.8566942795796f},
  { -115.673272648946f,    -51.5187284413421f},
  { -69.6541923856616f,    -48.7938239499718f},
  { -76.1141128518623f,    -49.7784476076377f},
  { -82.9535936191428f,    -50.4009770857117f},
  { -90.0f,    -50.6141704736363f},
  { -97.0464063808572f,    -50.4009770857117f},
  { -103.885887148138f,    -49.7784476076377f},
  { -110.345807614338f,    -48.7938239499718f},
  { -74.3086441830498f,    -45.9190620385707f},
  { -80.4324561201698f,    -46.6051356747135f},
  { -86.7842228465557f,    -46.9605776701645f},
  { -93.2157771534443f,    -46.9605776701645f},
  { -99.5675438798302f,    -46.6051356747135f},
  { -105.691355816950f,    -45.9190620385707f},
  { -78.3592768632294f,    -42.9780676586279f},
  { -84.1189512527134f,    -43.4213789054837f},
  { -90.0f,    -43.5723495188822f},
  { -95.8810487472866f,    -43.4213789054837f},
  { -101.640723136771f,    -42.9780676586279f},
  { -81.8830323534313f,    -40.0403293417631f},
  { -87.2781463566519f,    -40.2927991911889f},
  { -92.7218536433482f,    -40.2927991911889f},
  { -98.1169676465687f,    -40.0403293417631f},
  { -84.9544063480986f,    -37.1600422047896f},
  { -90.0f,    -37.2671844918060f},
  { -95.0455936519014f,    -37.1600422047896f},
  { -87.6406185690461f,    -34.3768199416111f},
  { -92.3593814309539f,    -34.3768199416111f},
  { 28.4036245335526f,    0.0f},
  { 24.8689251184620f,    0.0f},
  { 21.1201128443240f,    0.0f},
  { 17.1720378505138f,    0.0f},
  { 13.0486510400235f,    0.0f},
  { 8.78323060095898f,    0.0f},
  { 4.41756799953082f,    0.0f},
  { 0.0f,    0.0f},
  { -4.41756799953082f,    0.0f},
  { -8.78323060095898f,    0.0f},
  { -13.0486510400235f,    0.0f},
  { -17.1720378505138f,    0.0f},
  { -21.1201128443240f,    0.0f},
  { -24.8689251184620f,    0.0f},
  { -28.4036245335526f,    0.0f},
  { 30.6924044714658f,    3.15149052708986f},
  { 27.2228132322093f,    3.25869596066204f},
  { 23.5230102505398f,    3.35985064813239f},
  { 19.6031715380625f,    3.45174929336715f},
  { 15.4828763647624f,    3.53095737084913f},
  { 11.1919366693299f,    3.59407691500755f},
  { 6.77020803584041f,    3.63808703344424f},
  { 2.26611575815915f,    3.66070715562197f},
  { -2.26611575815915f,    3.66070715562197f},
  { -6.77020803584041f,    3.63808703344424f},
  { -11.1919366693299f,    3.59407691500755f},
  { -15.4828763647624f,    3.53095737084913f},
  { -19.6031715380625f,    3.45174929336715f},
  { -23.5230102505398f,    3.35985064813239f},
  { -27.2228132322093f,    3.25869596066204f},
  { -30.6924044714658f,    3.15149052708986f},
  { 29.5919950295299f,    6.51186878797759f},
  { 25.9552684514728f,    6.73132488281448f},
  { 22.0793821319241f,    6.93543325054564f},
  { 17.9788178368046f,    7.11699209660929f},
  { 13.6788812485618f,    7.26850261623438f},
  { 9.21623776735576f,    7.38284764810955f},
  { 4.63812012275413f,    7.45410055983883f},
  { 0.0f,    7.47831395519902f},
  { -4.63812012275412f,    7.45410055983883f},
  { -9.21623776735576f,    7.38284764810955f},
  { -13.6788812485618f,    7.26850261623438f},
  { -17.9788178368046f,    7.11699209660929f},
  { -22.0793821319241f,    6.93543325054564f},
  { -25.9552684514728f,    6.73132488281448f},
  { -29.5919950295299f,    6.51186878797759f},
  { 28.4085320985182f,    10.0731345348674f},
  { 24.5924580625592f,    10.4061002836447f},
  { 20.5290466848786f,    10.7103228714417f},
  { 16.2381321258644f,    10.9739000012513f},
  { 11.7518940701965f,    11.1848733373988f},
  { 7.11491076737230f,    11.3324763209808f},
  { 2.38252886597663f,    11.4085037402399f},
  { -2.38252886597663f,    11.4085037402399f},
  { -7.11491076737230f,    11.3324763209808f},
  { -11.7518940701965f,    11.1848733373988f},
  { -16.2381321258644f,    10.9739000012513f},
  { -20.5290466848786f,    10.7103228714417f},
  { -24.5924580625592f,    10.4061002836447f},
  { -28.4085320985182f,    10.0731345348674f},
  { 27.1333780408531f,    13.8190092060518f},
  { 23.1249527705157f,    14.2617068098709f},
  { 18.8622597253010f,    14.6573954710189f},
  { 14.3717350878187f,    14.9890130887144f},
  { 9.69372389531482f,    15.2401622826778f},
  { 4.88179605763996f,    15.3970529773534f},
  { 0.0f,    15.4504370863258f},
  { -4.88179605763996f,    15.3970529773534f},
  { -9.69372389531481f,    15.2401622826778f},
  { -14.3717350878187f,    14.9890130887144f},
  { -18.8622597253010f,    14.6573954710189f},
  { -23.1249527705157f,    14.2617068098709f},
  { -27.1333780408531f,    13.8190092060518f},
  { 25.7568701135217f,    17.7239254437427f},
  { 21.5423716012440f,    18.2664663350416f},
  { 17.0685514198043f,    18.7383427004503f},
  { 12.3699166060263f,    19.1173021445873f},
  { 7.49638621109516f,    19.3831156058266f},
  { 2.51154216406159f,    19.5202541496330f},
  { -2.51154216406159f,    19.5202541496330f},
  { -7.49638621109516f,    19.3831156058266f},
  { -12.3699166060263f,    19.1173021445873f},
  { -17.0685514198043f,    18.7383427004503f},
  { -21.5423716012440f,    18.2664663350416f},
  { -25.7568701135217f,    17.7239254437427f},
  { 24.2682191755604f,    21.7527747654215f},
  { 19.8333485550349f,    22.3785130500013f},
  { 15.1367980035503f,    22.9044641727044f},
  { 10.2228355626197f,    23.3037526705896f},
  { 5.15242323438301f,    23.5536156829348f},
  { 0.0f,    23.6387115659933f},
  { -5.15242323438301f,    23.5536156829348f},
  { -10.2228355626197f,    23.3037526705896f},
  { -15.1367980035503f,    22.9044641727044f},
  { -19.8333485550349f,    22.3785130500013f},
  { -24.2682191755604f,    21.7527747654215f},
  { 22.6554160459546f,    25.8616933530005f},
  { 17.9855349675095f,    26.5472401625776f},
  { 13.0553607197916f,    27.0989277977966f},
  { 7.92081029819563f,    27.4865212747043f},
  { 2.65531685494693f,    27.6866946479527f},
  { -2.65531685494693f,    27.6866946479527f},
  { -7.92081029819563f,    27.4865212747043f},
  { -13.0553607197916f,    27.0989277977966f},
  { -17.9855349675095f,    26.5472401625776f},
  { -22.6554160459546f,    25.8616933530005f},
  { 20.9051574478893f,    30.0000000000000f},
  { 15.9856580001231f,    30.7160300252232f},
  { 10.8123169635717f,    31.2603465080791f},
  { 5.45472036156347f,    31.6012967532613f},
  { 0.0f,    31.7174744114610f},
  { -5.45472036156347f,    31.6012967532613f},
  { -10.8123169635717f,    31.2603465080791f},
  { -15.9856580001231f,    30.7160300252232f},
  { -20.9051574478893f,    30.0000000000000f},
  { 19.0028098100425f,    34.1131421237471f},
  { 13.8196629981948f,    34.8259516725867f},
  { 8.39581320776640f,    35.3271460882019f},
  { 2.81653794657769f,    35.5861276946272f},
  { -2.81653794657769f,    35.5861276946272f},
  { -8.39581320776640f,    35.3271460882019f},
  { -13.8196629981948f,    34.8259516725867f},
  { -19.0028098100425f,    34.1131421237471f},
  { 16.9324379151258f,    38.1462352379972f},
  { 11.4729758889747f,    38.8198268192794f},
  { 5.79457193955913f,    39.2419529905605f},
  { 0.0f,    39.3858295263638f},
  { -5.79457193955913f,    39.2419529905605f},
  { -11.4729758889747f,    38.8198268192794f},
  { -16.9324379151258f,    38.1462352379972f},
  { 14.6769368549255f,    42.0476073652717f},
  { 8.93093030789172f,    42.6459541515662f},
  { 2.99858302466767f,    42.9552319464629f},
  { -2.99858302466767f,    42.9552319464629f},
  { -8.93093030789172f,    42.6459541515662f},
  { -14.6769368549255f,    42.0476073652717f},
  { 12.2183198662487f,    45.7717617455715f},
  { 6.17941045970870f,    46.2608964712010f},
  { 0.0f,    46.4276504811178f},
  { -6.17941045970870f,    46.2608964712010f},
  { -12.2183198662487f,    45.7717617455715f},
  { 9.53823018668633f,    49.2813489417896f},
  { 3.20576070133417f,    49.6310212976971f},
  { -3.20576070133417f,    49.6310212976971f},
  { -9.53823018668633f,    49.2813489417896f},
  { 6.61875877204995f,    52.5480139114809f},
  { 0.0f,    52.7328155081940f},
  { -6.61875877204995f,    52.5480139114809f},
  { 3.44365473521952f,    55.5522456067903f},
  { -3.44365473521952f,    55.5522456067903f},
  { 30.6924044714658f,    -3.15149052708986f},
  { 29.5919950295299f,    -6.51186878797759f},
  { 28.4085320985182f,    -10.0731345348674f},
  { 27.1333780408531f,    -13.8190092060518f},
  { 25.7568701135217f,    -17.7239254437427f},
  { 24.2682191755604f,    -21.7527747654215f},
  { 22.6554160459546f,    -25.8616933530005f},
  { 20.9051574478893f,    -30.0000000000000f},
  { 19.0028098100425f,    -34.1131421237471f},
  { 16.9324379151258f,    -38.1462352379972f},
  { 14.6769368549255f,    -42.0476073652717f},
  { 12.2183198662487f,    -45.7717617455715f},
  { 9.53823018668633f,    -49.2813489417896f},
  { 6.61875877204995f,    -52.5480139114809f},
  { 3.44365473521952f,    -55.5522456067903f},
  { 27.2228132322093f,    -3.25869596066204f},
  { 25.9552684514728f,    -6.73132488281448f},
  { 24.5924580625592f,    -10.4061002836447f},
  { 23.1249527705157f,    -14.2617068098710f},
  { 21.5423716012440f,    -18.2664663350416f},
  { 19.8333485550349f,    -22.3785130500013f},
  { 17.9855349675095f,    -26.5472401625776f},
  { 15.9856580001231f,    -30.7160300252232f},
  { 13.8196629981948f,    -34.8259516725867f},
  { 11.4729758889747f,    -38.8198268192794f},
  { 8.93093030789172f,    -42.6459541515662f},
  { 6.17941045970870f,    -46.2608964712010f},
  { 3.20576070133417f,    -49.6310212976971f},
  { 0.0f,    -52.7328155081940f},
  { -3.44365473521952f,    -55.5522456067903f},
  { 23.5230102505398f,    -3.35985064813239f},
  { 22.0793821319241f,    -6.93543325054564f},
  { 20.5290466848786f,    -10.7103228714417f},
  { 18.8622597253010f,    -14.6573954710189f},
  { 17.0685514198043f,    -18.7383427004503f},
  { 15.1367980035503f,    -22.9044641727044f},
  { 13.0553607197916f,    -27.0989277977966f},
  { 10.8123169635717f,    -31.2603465080791f},
  { 8.39581320776640f,    -35.3271460882019f},
  { 5.79457193955914f,    -39.2419529905605f},
  { 2.99858302466767f,    -42.9552319464629f},
  { 5.24012991261292e-15f,    -46.4276504811178f},
  { -3.20576070133417f,    -49.6310212976971f},
  { -6.61875877204995f,    -52.5480139114809f},
  { 19.6031715380625f,    -3.45174929336715f},
  { 17.9788178368046f,    -7.11699209660929f},
  { 16.2381321258644f,    -10.9739000012513f},
  { 14.3717350878187f,    -14.9890130887144f},
  { 12.3699166060263f,    -19.1173021445873f},
  { 10.2228355626197f,    -23.3037526705896f},
  { 7.92081029819563f,    -27.4865212747043f},
  { 5.45472036156347f,    -31.6012967532613f},
  { 2.81653794657768f,    -35.5861276946272f},
  { 0.0f,    -39.3858295263638f},
  { -2.99858302466767f,    -42.9552319464629f},
  { -6.17941045970870f,    -46.2608964712010f},
  { -9.53823018668633f,    -49.2813489417896f},
  { 15.4828763647624f,    -3.53095737084913f},
  { 13.6788812485618f,    -7.26850261623438f},
  { 11.7518940701965f,    -11.1848733373988f},
  { 9.69372389531481f,    -15.2401622826778f},
  { 7.49638621109516f,    -19.3831156058266f},
  { 5.15242323438301f,    -23.5536156829348f},
  { 2.65531685494692f,    -27.6866946479527f},
  { 0.0f,    -31.7174744114610f},
  { -2.81653794657769f,    -35.5861276946272f},
  { -5.79457193955914f,    -39.2419529905605f},
  { -8.93093030789173f,    -42.6459541515662f},
  { -12.2183198662487f,    -45.7717617455715f},
  { 11.1919366693299f,    -3.59407691500755f},
  { 9.21623776735576f,    -7.38284764810955f},
  { 7.11491076737230f,    -11.3324763209808f},
  { 4.88179605763996f,    -15.3970529773534f},
  { 2.51154216406159f,    -19.5202541496330f},
  { 0.0f,    -23.6387115659933f},
  { -2.65531685494693f,    -27.6866946479527f},
  { -5.45472036156347f,    -31.6012967532613f},
  { -8.39581320776640f,    -35.3271460882019f},
  { -11.4729758889747f,    -38.8198268192794f},
  { -14.6769368549255f,    -42.0476073652718f},
  { 6.77020803584041f,    -3.63808703344424f},
  { 4.63812012275412f,    -7.45410055983883f},
  { 2.38252886597663f,    -11.4085037402399f},
  { -2.06684972190655e-15f,    -15.4504370863258f},
  { -2.51154216406159f,    -19.5202541496330f},
  { -5.15242323438302f,    -23.5536156829348f},
  { -7.92081029819563f,    -27.4865212747043f},
  { -10.8123169635717f,    -31.2603465080791f},
  { -13.8196629981948f,    -34.8259516725867f},
  { -16.9324379151258f,    -38.1462352379972f},
  { 2.26611575815915f,    -3.66070715562197f},
  { 0.0f,    -7.47831395519902f},
  { -2.38252886597663f,    -11.4085037402399f},
  { -4.88179605763996f,    -15.3970529773534f},
  { -7.49638621109516f,    -19.3831156058266f},
  { -10.2228355626197f,    -23.3037526705896f},
  { -13.0553607197916f,    -27.0989277977966f},
  { -15.9856580001231f,    -30.7160300252232f},
  { -19.0028098100425f,    -34.1131421237471f},
  { -2.26611575815915f,    -3.66070715562197f},
  { -4.63812012275413f,    -7.45410055983883f},
  { -7.11491076737230f,    -11.3324763209808f},
  { -9.69372389531481f,    -15.2401622826778f},
  { -12.3699166060263f,    -19.1173021445873f},
  { -15.1367980035503f,    -22.9044641727044f},
  { -17.9855349675095f,    -26.5472401625776f},
  { -20.9051574478893f,    -30.0000000000000f},
  { -6.77020803584041f,    -3.63808703344424f},
  { -9.21623776735576f,    -7.38284764810955f},
  { -11.7518940701965f,    -11.1848733373988f},
  { -14.3717350878187f,    -14.9890130887144f},
  { -17.0685514198043f,    -18.7383427004503f},
  { -19.8333485550349f,    -22.3785130500013f},
  { -22.6554160459546f,    -25.8616933530005f},
  { -11.1919366693299f,    -3.59407691500755f},
  { -13.6788812485618f,    -7.26850261623438f},
  { -16.2381321258644f,    -10.9739000012513f},
  { -18.8622597253010f,    -14.6573954710189f},
  { -21.5423716012440f,    -18.2664663350416f},
  { -24.2682191755604f,    -21.7527747654215f},
  { -15.4828763647624f,    -3.53095737084913f},
  { -17.9788178368046f,    -7.11699209660929f},
  { -20.5290466848786f,    -10.7103228714417f},
  { -23.1249527705157f,    -14.2617068098710f},
  { -25.7568701135217f,    -17.7239254437427f},
  { -19.6031715380625f,    -3.45174929336715f},
  { -22.0793821319241f,    -6.93543325054564f},
  { -24.5924580625592f,    -10.4061002836447f},
  { -27.1333780408531f,    -13.8190092060518f},
  { -23.5230102505398f,    -3.35985064813239f},
  { -25.9552684514728f,    -6.73132488281448f},
  { -28.4085320985182f,    -10.0731345348674f},
  { -27.2228132322093f,    -3.25869596066204f},
  { -29.5919950295299f,    -6.51186878797759f},
  { -30.6924044714658f,    -3.15149052708986f},
  { -151.596375466447f,    0.0f},
  { -155.131074881538f,    0.0f},
  { -158.879887155676f,    0.0f},
  { -162.827962149486f,    0.0f},
  { -166.951348959977f,    0.0f},
  { -171.216769399041f,    0.0f},
  { -175.582432000469f,    0.0f},
  { 180.0f,    0.0f},
  { 175.582432000469f,    0.0f},
  { 171.216769399041f,    0.0f},
  { 166.951348959977f,    0.0f},
  { 162.827962149486f,    0.0f},
  { 158.879887155676f,    0.0f},
  { 155.131074881538f,    0.0f},
  { 151.596375466447f,    0.0f},
  { -149.307595528534f,    3.15149052708986f},
  { -152.777186767791f,    3.25869596066204f},
  { -156.476989749460f,    3.35985064813239f},
  { -160.396828461937f,    3.45174929336715f},
  { -164.517123635238f,    3.53095737084913f},
  { -168.808063330670f,    3.59407691500755f},
  { -173.229791964160f,    3.63808703344424f},
  { -177.733884241841f,    3.66070715562197f},
  { 177.733884241841f,    3.66070715562197f},
  { 173.229791964160f,    3.63808703344424f},
  { 168.808063330670f,    3.59407691500755f},
  { 164.517123635238f,    3.53095737084913f},
  { 160.396828461937f,    3.45174929336715f},
  { 156.476989749460f,    3.35985064813239f},
  { 152.777186767791f,    3.25869596066204f},
  { 149.307595528534f,    3.15149052708986f},
  { -150.408004970470f,    6.51186878797759f},
  { -154.044731548527f,    6.73132488281448f},
  { -157.920617868076f,    6.93543325054564f},
  { -162.021182163195f,    7.11699209660929f},
  { -166.321118751438f,    7.26850261623438f},
  { -170.783762232644f,    7.38284764810955f},
  { -175.361879877246f,    7.45410055983883f},
  { 180.0f,    7.47831395519902f},
  { 175.361879877246f,    7.45410055983883f},
  { 170.783762232644f,    7.38284764810955f},
  { 166.321118751438f,    7.26850261623438f},
  { 162.021182163195f,    7.11699209660929f},
  { 157.920617868076f,    6.93543325054564f},
  { 154.044731548527f,    6.73132488281448f},
  { 150.408004970470f,    6.51186878797759f},
  { -151.591467901482f,    10.0731345348674f},
  { -155.407541937441f,    10.4061002836447f},
  { -159.470953315121f,    10.7103228714417f},
  { -163.761867874136f,    10.9739000012513f},
  { -168.248105929804f,    11.1848733373988f},
  { -172.885089232628f,    11.3324763209808f},
  { -177.617471134023f,    11.4085037402399f},
  { 177.617471134023f,    11.4085037402399f},
  { 172.885089232628f,    11.3324763209808f},
  { 168.248105929804f,    11.1848733373988f},
  { 163.761867874136f,    10.9739000012513f},
  { 159.470953315121f,    10.7103228714417f},
  { 155.407541937441f,    10.4061002836447f},
  { 151.591467901482f,    10.0731345348674f},
  { -152.866621959147f,    13.8190092060518f},
  { -156.875047229484f,    14.2617068098709f},
  { -161.137740274699f,    14.6573954710189f},
  { -165.628264912181f,    14.9890130887144f},
  { -170.306276104685f,    15.2401622826778f},
  { -175.118203942360f,    15.3970529773534f},
  { 180.0f,    15.4504370863258f},
  { 175.118203942360f,    15.3970529773534f},
  { 170.306276104685f,    15.2401622826778f},
  { 165.628264912181f,    14.9890130887144f},
  { 161.137740274699f,    14.6573954710189f},
  { 156.875047229484f,    14.2617068098709f},
  { 152.866621959147f,    13.8190092060518f},
  { -154.243129886478f,    17.7239254437427f},
  { -158.457628398756f,    18.2664663350416f},
  { -162.931448580196f,    18.7383427004503f},
  { -167.630083393974f,    19.1173021445873f},
  { -172.503613788905f,    19.3831156058266f},
  { -177.488457835938f,    19.5202541496330f},
  { 177.488457835938f,    19.5202541496330f},
  { 172.503613788905f,    19.3831156058266f},
  { 167.630083393974f,    19.1173021445873f},
  { 162.931448580196f,    18.7383427004503f},
  { 158.457628398756f,    18.2664663350416f},
  { 154.243129886478f,    17.7239254437427f},
  { -155.731780824440f,    21.7527747654215f},
  { -160.166651444965f,    22.3785130500013f},
  { -164.863201996450f,    22.9044641727044f},
  { -169.777164437380f,    23.3037526705896f},
  { -174.847576765617f,    23.5536156829348f},
  { 180.0f,    23.6387115659933f},
  { 174.847576765617f,    23.5536156829348f},
  { 169.777164437380f,    23.3037526705896f},
  { 164.863201996450f,    22.9044641727044f},
  { 160.166651444965f,    22.3785130500013f},
  { 155.731780824440f,    21.7527747654215f},
  { -157.344583954045f,    25.8616933530005f},
  { -162.014465032491f,    26.5472401625776f},
  { -166.944639280208f,    27.0989277977966f},
  { -172.079189701804f,    27.4865212747043f},
  { -177.344683145053f,    27.6866946479527f},
  { 177.344683145053f,    27.6866946479527f},
  { 172.079189701804f,    27.4865212747043f},
  { 166.944639280208f,    27.0989277977966f},
  { 162.014465032491f,    26.5472401625776f},
  { 157.344583954045f,    25.8616933530005f},
  { -159.094842552111f,    30.0000000000000f},
  { -164.014341999877f,    30.7160300252232f},
  { -169.187683036428f,    31.2603465080791f},
  { -174.545279638437f,    31.6012967532613f},
  { 180.0f,    31.7174744114610f},
  { 174.545279638437f,    31.6012967532613f},
  { 169.187683036428f,    31.2603465080791f},
  { 164.014341999877f,    30.7160300252232f},
  { 159.094842552111f,    30.0000000000000f},
  { -160.997190189958f,    34.1131421237471f},
  { -166.180337001805f,    34.8259516725867f},
  { -171.604186792234f,    35.3271460882019f},
  { -177.183462053422f,    35.5861276946272f},
  { 177.183462053422f,    35.5861276946272f},
  { 171.604186792234f,    35.3271460882019f},
  { 166.180337001805f,    34.8259516725867f},
  { 160.997190189958f,    34.1131421237471f},
  { -163.067562084874f,    38.1462352379972f},
  { -168.527024111025f,    38.8198268192794f},
  { -174.205428060441f,    39.2419529905605f},
  { 180.0f,    39.3858295263638f},
  { 174.205428060441f,    39.2419529905605f},
  { 168.527024111025f,    38.8198268192794f},
  { 163.067562084874f,    38.1462352379972f},
  { -165.323063145074f,    42.0476073652717f},
  { -171.069069692108f,    42.6459541515662f},
  { -177.001416975332f,    42.9552319464629f},
  { 177.001416975332f,    42.9552319464629f},
  { 171.069069692108f,    42.6459541515662f},
  { 165.323063145074f,    42.0476073652717f},
  { -167.781680133751f,    45.7717617455715f},
  { -173.820589540291f,    46.2608964712010f},
  { 180.0f,    46.4276504811178f},
  { 173.820589540291f,    46.2608964712010f},
  { 167.781680133751f,    45.7717617455715f},
  { -170.461769813314f,    49.2813489417896f},
  { -176.794239298666f,    49.6310212976971f},
  { 176.794239298666f,    49.6310212976971f},
  { 170.461769813314f,    49.2813489417896f},
  { -173.381241227950f,    52.5480139114809f},
  { 180.0f,    52.7328155081940f},
  { 173.381241227950f,    52.5480139114809f},
  { -176.556345264781f,    55.5522456067903f},
  { 176.556345264781f,    55.5522456067903f},
  { -149.307595528534f,    -3.15149052708986f},
  { -150.408004970470f,    -6.51186878797759f},
  { -151.591467901482f,    -10.0731345348674f},
  { -152.866621959147f,    -13.8190092060518f},
  { -154.243129886478f,    -17.7239254437427f},
  { -155.731780824440f,    -21.7527747654215f},
  { -157.344583954045f,    -25.8616933530005f},
  { -159.094842552111f,    -30.0000000000000f},
  { -160.997190189958f,    -34.1131421237471f},
  { -163.067562084874f,    -38.1462352379972f},
  { -165.323063145074f,    -42.0476073652717f},
  { -167.781680133751f,    -45.7717617455715f},
  { -170.461769813314f,    -49.2813489417896f},
  { -173.381241227950f,    -52.5480139114809f},
  { -176.556345264781f,    -55.5522456067903f},
  { -152.777186767791f,    -3.25869596066204f},
  { -154.044731548527f,    -6.73132488281448f},
  { -155.407541937441f,    -10.4061002836447f},
  { -156.875047229484f,    -14.2617068098709f},
  { -158.457628398756f,    -18.2664663350416f},
  { -160.166651444965f,    -22.3785130500013f},
  { -162.014465032491f,    -26.5472401625776f},
  { -164.014341999877f,    -30.7160300252232f},
  { -166.180337001805f,    -34.8259516725867f},
  { -168.527024111025f,    -38.8198268192794f},
  { -171.069069692108f,    -42.6459541515662f},
  { -173.820589540291f,    -46.2608964712010f},
  { -176.794239298666f,    -49.6310212976971f},
  { 180.0f,    -52.7328155081940f},
  { 176.556345264781f,    -55.5522456067903f},
  { -156.476989749460f,    -3.35985064813239f},
  { -157.920617868076f,    -6.93543325054564f},
  { -159.470953315121f,    -10.7103228714417f},
  { -161.137740274699f,    -14.6573954710189f},
  { -162.931448580196f,    -18.7383427004503f},
  { -164.863201996450f,    -22.9044641727044f},
  { -166.944639280208f,    -27.0989277977966f},
  { -169.187683036428f,    -31.2603465080791f},
  { -171.604186792234f,    -35.3271460882019f},
  { -174.205428060441f,    -39.2419529905605f},
  { -177.001416975332f,    -42.9552319464629f},
  { -180.0f,    -46.4276504811178f},
  { 176.794239298666f,    -49.6310212976971f},
  { 173.381241227950f,    -52.5480139114809f},
  { -160.396828461937f,    -3.45174929336715f},
  { -162.021182163195f,    -7.11699209660929f},
  { -163.761867874136f,    -10.9739000012513f},
  { -165.628264912181f,    -14.9890130887144f},
  { -167.630083393974f,    -19.1173021445873f},
  { -169.777164437380f,    -23.3037526705896f},
  { -172.079189701804f,    -27.4865212747043f},
  { -174.545279638437f,    -31.6012967532613f},
  { -177.183462053422f,    -35.5861276946272f},
  { 180.0f,    -39.3858295263638f},
  { 177.001416975332f,    -42.9552319464629f},
  { 173.820589540291f,    -46.2608964712010f},
  { 170.461769813314f,    -49.2813489417896f},
  { -164.517123635238f,    -3.53095737084913f},
  { -166.321118751438f,    -7.26850261623438f},
  { -168.248105929804f,    -11.1848733373988f},
  { -170.306276104685f,    -15.2401622826778f},
  { -172.503613788905f,    -19.3831156058266f},
  { -174.847576765617f,    -23.5536156829348f},
  { -177.344683145053f,    -27.6866946479527f},
  { 180.0f,    -31.7174744114610f},
  { 177.183462053422f,    -35.5861276946272f},
  { 174.205428060441f,    -39.2419529905605f},
  { 171.069069692108f,    -42.6459541515662f},
  { 167.781680133751f,    -45.7717617455715f},
  { -168.808063330670f,    -3.59407691500755f},
  { -170.783762232644f,    -7.38284764810955f},
  { -172.885089232628f,    -11.3324763209808f},
  { -175.118203942360f,    -15.3970529773534f},
  { -177.488457835938f,    -19.5202541496330f},
  { 180.0f,    -23.6387115659933f},
  { 177.344683145053f,    -27.6866946479527f},
  { 174.545279638437f,    -31.6012967532613f},
  { 171.604186792234f,    -35.3271460882019f},
  { 168.527024111025f,    -38.8198268192794f},
  { 165.323063145074f,    -42.0476073652718f},
  { -173.229791964160f,    -3.63808703344424f},
  { -175.361879877246f,    -7.45410055983883f},
  { -177.617471134023f,    -11.4085037402399f},
  { 180.0f,    -15.4504370863258f},
  { 177.488457835938f,    -19.5202541496330f},
  { 174.847576765617f,    -23.5536156829348f},
  { 172.079189701804f,    -27.4865212747043f},
  { 169.187683036428f,    -31.2603465080791f},
  { 166.180337001805f,    -34.8259516725867f},
  { 163.067562084874f,    -38.1462352379972f},
  { -177.733884241841f,    -3.66070715562197f},
  { 180.0f,    -7.47831395519902f},
  { 177.617471134023f,    -11.4085037402399f},
  { 175.118203942360f,    -15.3970529773534f},
  { 172.503613788905f,    -19.3831156058266f},
  { 169.777164437380f,    -23.3037526705896f},
  { 166.944639280208f,    -27.0989277977966f},
  { 164.014341999877f,    -30.7160300252232f},
  { 160.997190189958f,    -34.1131421237471f},
  { 177.733884241841f,    -3.66070715562197f},
  { 175.361879877246f,    -7.45410055983883f},
  { 172.885089232628f,    -11.3324763209808f},
  { 170.306276104685f,    -15.2401622826778f},
  { 167.630083393974f,    -19.1173021445873f},
  { 164.863201996450f,    -22.9044641727044f},
  { 162.014465032491f,    -26.5472401625776f},
  { 159.094842552111f,    -30.0000000000000f},
  { 173.229791964160f,    -3.63808703344424f},
  { 170.783762232644f,    -7.38284764810955f},
  { 168.248105929804f,    -11.1848733373988f},
  { 165.628264912181f,    -14.9890130887144f},
  { 162.931448580196f,    -18.7383427004503f},
  { 160.166651444965f,    -22.3785130500013f},
  { 157.344583954045f,    -25.8616933530005f},
  { 168.808063330670f,    -3.59407691500755f},
  { 166.321118751438f,    -7.26850261623438f},
  { 163.761867874136f,    -10.9739000012513f},
  { 161.137740274699f,    -14.6573954710189f},
  { 158.457628398756f,    -18.2664663350416f},
  { 155.731780824440f,    -21.7527747654215f},
  { 164.517123635238f,    -3.53095737084913f},
  { 162.021182163195f,    -7.11699209660929f},
  { 159.470953315121f,    -10.7103228714417f},
  { 156.875047229484f,    -14.2617068098709f},
  { 154.243129886478f,    -17.7239254437427f},
  { 160.396828461937f,    -3.45174929336714f},
  { 157.920617868076f,    -6.93543325054564f},
  { 155.407541937441f,    -10.4061002836447f},
  { 152.866621959147f,    -13.8190092060518f},
  { 156.476989749460f,    -3.35985064813239f},
  { 154.044731548527f,    -6.73132488281448f},
  { 151.591467901482f,    -10.0731345348674f},
  { 152.777186767791f,    -3.25869596066204f},
  { 150.408004970470f,    -6.51186878797759f},
  { 149.307595528534f,    -3.15149052708986f},
  { 90.0f,    28.4036245335526f},
  { 90.0f,    24.8689251184620f},
  { 90.0f,    21.1201128443240f},
  { 90.0f,    17.1720378505138f},
  { 90.0f,    13.0486510400235f},
  { 90.0f,    8.78323060095898f},
  { 90.0f,    4.41756799953082f},
  { 90.0f,    0.0f},
  { 90.0f,    -4.41756799953082f},
  { 90.0f,    -8.78323060095898f},
  { 90.0f,    -13.0486510400235f},
  { 90.0f,    -17.1720378505138f},
  { 90.0f,    -21.1201128443240f},
  { 90.0f,    -24.8689251184620f},
  { 90.0f,    -28.4036245335526f},
  { 86.3364355621228f,    30.6409845343737f},
  { 86.3364355621228f,    27.1751641552756f},
  { 86.3364355621228f,    23.4801484036882f},
  { 86.3364355621228f,    19.5661569361721f},
  { 86.3364355621228f,    15.4527499606845f},
  { 86.3364355621228f,    11.1696410784414f},
  { 86.3364355621228f,    6.75650083717288f},
  { 86.3364355621228f,    2.26148965929813f},
  { 86.3364355621228f,    -2.26148965929813f},
  { 86.3364355621228f,    -6.75650083717288f},
  { 86.3364355621228f,    -11.1696410784414f},
  { 86.3364355621228f,    -15.4527499606845f},
  { 86.3364355621228f,    -19.5661569361721f},
  { 86.3364355621228f,    -23.4801484036882f},
  { 86.3364355621228f,    -27.1751641552756f},
  { 86.3364355621228f,    -30.6409845343737f},
  { 82.5216860448010f,    29.3822901816024f},
  { 82.5216860448010f,    25.7631734014353f},
  { 82.5216860448010f,    21.9094235462107f},
  { 82.5216860448010f,    17.8356202570103f},
  { 82.5216860448010f,    13.5668491634509f},
  { 82.5216860448010f,    9.13917468233299f},
  { 82.5216860448010f,    4.59883911411213f},
  { 82.5216860448010f,    0.0f},
  { 82.5216860448010f,    -4.59883911411213f},
  { 82.5216860448010f,    -9.13917468233299f},
  { 82.5216860448010f,    -13.5668491634508f},
  { 82.5216860448010f,    -17.8356202570103f},
  { 82.5216860448010f,    -21.9094235462107f},
  { 82.5216860448010f,    -25.7631734014353f},
  { 82.5216860448010f,    -29.3822901816024f},
  { 78.5818848356174f,    27.9318860053080f},
  { 78.5818848356174f,    24.1618860804539f},
  { 78.5818848356174f,    20.1557367341806f},
  { 78.5818848356174f,    15.9332226373311f},
  { 78.5818848356174f,    11.5255919612906f},
  { 78.5818848356174f,    6.97549803830799f},
  { 78.5818848356174f,    2.33542809914013f},
  { 78.5818848356174f,    -2.33542809914013f},
  { 78.5818848356174f,    -6.97549803830799f},
  { 78.5818848356174f,    -11.5255919612906f},
  { 78.5818848356174f,    -15.9332226373311f},
  { 78.5818848356174f,    -20.1557367341806f},
  { 78.5818848356174f,    -24.1618860804539f},
  { 78.5818848356174f,    -27.9318860053080f},
  { 74.5495629136742f,    26.2866766550738f},
  { 74.5495629136742f,    22.3729427285926f},
  { 74.5495629136742f,    18.2264204988720f},
  { 74.5495629136742f,    13.8727732231513f},
  { 74.5495629136742f,    9.34970354257458f},
  { 74.5495629136742f,    4.70618130300619f},
  { 74.5495629136742f,    0.0f},
  { 74.5495629136742f,    -4.70618130300619f},
  { 74.5495629136742f,    -9.34970354257458f},
  { 74.5495629136742f,    -13.8727732231513f},
  { 74.5495629136742f,    -18.2264204988720f},
  { 74.5495629136742f,    -22.3729427285926f},
  { 74.5495629136742f,    -26.2866766550738f},
  { 70.4623978178261f,    24.4517589639450f},
  { 70.4623978178261f,    20.4069778874103f},
  { 70.4623978178261f,    16.1383704444056f},
  { 70.4623978178261f,    11.6778155497790f},
  { 70.4623978178261f,    7.06925915799702f},
  { 70.4623978178261f,    2.36710264078988f},
  { 70.4623978178261f,    -2.36710264078988f},
  { 70.4623978178261f,    -7.06925915799702f},
  { 70.4623978178261f,    -11.6778155497790f},
  { 70.4623978178261f,    -16.1383704444056f},
  { 70.4623978178261f,    -20.4069778874103f},
  { 70.4623978178261f,    -24.4517589639450f},
  { 66.3612884340067f,    22.4415960116381f},
  { 66.3612884340067f,    18.2843746791691f},
  { 66.3612884340067f,    13.9181885789507f},
  { 66.3612884340067f,    9.38098264244825f},
  { 66.3612884340067f,    4.72213775783758f},
  { 66.3612884340067f,    0.0f},
  { 66.3612884340067f,    -4.72213775783758f},
  { 66.3612884340067f,    -9.38098264244824f},
  { 66.3612884340067f,    -13.9181885789507f},
  { 66.3612884340067f,    -18.2843746791691f},
  { 66.3612884340067f,    -22.4415960116381f},
  { 62.2879732820537f,    20.2801884851370f},
  { 62.2879732820537f,    16.0348118271108f},
  { 62.2879732820537f,    11.6009546747238f},
  { 62.2879732820537f,    7.02190914735696f},
  { 62.2879732820537f,    2.35110536299009f},
  { 62.2879732820537f,    -2.35110536299009f},
  { 62.2879732820537f,    -7.02190914735696f},
  { 62.2879732820537f,    -11.6009546747238f},
  { 62.2879732820537f,    -16.0348118271108f},
  { 62.2879732820537f,    -20.2801884851370f},
  { 58.2825255885390f,    18.0f},
  { 58.2825255885390f,    13.6954596895373f},
  { 58.2825255885390f,    9.22764488160650f},
  { 58.2825255885390f,    4.64393570106378f},
  { 58.2825255885390f,    0.0f},
  { 58.2825255885390f,    -4.64393570106378f},
  { 58.2825255885390f,    -9.22764488160650f},
  { 58.2825255885390f,    -13.6954596895373f},
  { 58.2825255885390f,    -18.0f},
  { 54.3810910452200f,    15.6396680917009f},
  { 54.3810910452200f,    11.3080227948550f},
  { 54.3810910452200f,    6.84159869379449f},
  { 54.3810910452200f,    2.29021346643181f},
  { 54.3810910452200f,    -2.29021346643181f},
  { 54.3810910452200f,    -6.84159869379449f},
  { 54.3810910452200f,    -11.3080227948550f},
  { 54.3810910452200f,    -15.6396680917009f},
  { 50.6141704736363f,    13.2408470050412f},
  { 50.6141704736363f,    8.91515156310237f},
  { 50.6141704736363f,    4.48472037239199f},
  { 50.6141704736363f,    0.0f},
  { 50.6141704736363f,    -4.48472037239199f},
  { 50.6141704736363f,    -8.91515156310237f},
  { 50.6141704736363f,    -13.2408470050412f},
  { 47.0056153489970f,    10.8447588267182f},
  { 47.0056153489970f,    6.55691452862584f},
  { 47.0056153489970f,    2.19415657112221f},
  { 47.0056153489970f,    -2.19415657112221f},
  { 47.0056153489970f,    -6.55691452862584f},
  { 47.0056153489970f,    -10.8447588267182f},
  { 43.5723495188822f,    8.48907956344660f},
  { 43.5723495188822f,    4.26796226896432f},
  { 43.5723495188822f,    0.0f},
  { 43.5723495188822f,    -4.26796226896432f},
  { 43.5723495188822f,    -8.48907956344660f},
  { 40.3247043644708f,    6.20564401643037f},
  { 40.3247043644708f,    2.07576608018301f},
  { 40.3247043644708f,    -2.07576608018301f},
  { 40.3247043644708f,    -6.20564401643037f},
  { 37.2671844918060f,    4.01919298814923f},
  { 37.2671844918060f,    0.0f},
  { 37.2671844918060f,    -4.01919298814923f},
  { 34.3994681299533f,    1.94712105456852f},
  { 34.3994681299533f,    -1.94712105456852f},
  { 93.6635644378772f,    30.6409845343737f},
  { 97.4783139551990f,    29.3822901816024f},
  { 101.418115164383f,    27.9318860053080f},
  { 105.450437086326f,    26.2866766550738f},
  { 109.537602182174f,    24.4517589639450f},
  { 113.638711565993f,    22.4415960116381f},
  { 117.712026717946f,    20.2801884851370f},
  { 121.717474411461f,    18.0f},
  { 125.618908954780f,    15.6396680917009f},
  { 129.385829526364f,    13.2408470050412f},
  { 132.994384651003f,    10.8447588267182f},
  { 136.427650481118f,    8.48907956344661f},
  { 139.675295635529f,    6.20564401643037f},
  { 142.732815508194f,    4.01919298814923f},
  { 145.600531870047f,    1.94712105456852f},
  { 93.6635644378772f,    27.1751641552756f},
  { 97.4783139551990f,    25.7631734014353f},
  { 101.418115164383f,    24.1618860804539f},
  { 105.450437086326f,    22.3729427285926f},
  { 109.537602182174f,    20.4069778874103f},
  { 113.638711565993f,    18.2843746791691f},
  { 117.712026717946f,    16.0348118271108f},
  { 121.717474411461f,    13.6954596895373f},
  { 125.618908954780f,    11.3080227948550f},
  { 129.385829526364f,    8.91515156310236f},
  { 132.994384651003f,    6.55691452862585f},
  { 136.427650481118f,    4.26796226896432f},
  { 139.675295635529f,    2.07576608018301f},
  { 142.732815508194f,    0.0f},
  { 145.600531870047f,    -1.94712105456852f},
  { 93.6635644378772f,    23.4801484036882f},
  { 97.4783139551990f,    21.9094235462107f},
  { 101.418115164383f,    20.1557367341806f},
  { 105.450437086326f,    18.2264204988720f},
  { 109.537602182174f,    16.1383704444056f},
  { 113.638711565993f,    13.9181885789507f},
  { 117.712026717946f,    11.6009546747238f},
  { 121.717474411461f,    9.22764488160650f},
  { 125.618908954780f,    6.84159869379449f},
  { 129.385829526364f,    4.48472037239199f},
  { 132.994384651003f,    2.19415657112221f},
  { 136.427650481118f,    3.61186426073483e-15f},
  { 139.675295635529f,    -2.07576608018301f},
  { 142.732815508194f,    -4.01919298814923f},
  { 93.6635644378772f,    19.5661569361721f},
  { 97.4783139551990f,    17.8356202570103f},
  { 101.418115164383f,    15.9332226373311f},
  { 105.450437086326f,    13.8727732231513f},
  { 109.537602182174f,    11.6778155497790f},
  { 113.638711565993f,    9.38098264244824f},
  { 117.712026717946f,    7.02190914735696f},
  { 121.717474411461f,    4.64393570106378f},
  { 125.618908954780f,    2.29021346643180f},
  { 129.385829526364f,    0.0f},
  { 132.994384651003f,    -2.19415657112221f},
  { 136.427650481118f,    -4.26796226896433f},
  { 139.675295635529f,    -6.20564401643037f},
  { 93.6635644378772f,    15.4527499606845f},
  { 97.4783139551990f,    13.5668491634509f},
  { 101.418115164383f,    11.5255919612906f},
  { 105.450437086326f,    9.34970354257458f},
  { 109.537602182174f,    7.06925915799702f},
  { 113.638711565993f,    4.72213775783758f},
  { 117.712026717946f,    2.35110536299009f},
  { 121.717474411461f,    0.0f},
  { 125.618908954780f,    -2.29021346643181f},
  { 129.385829526364f,    -4.48472037239199f},
  { 132.994384651003f,    -6.55691452862585f},
  { 136.427650481118f,    -8.48907956344661f},
  { 93.6635644378772f,    11.1696410784414f},
  { 97.4783139551990f,    9.13917468233299f},
  { 101.418115164383f,    6.97549803830799f},
  { 105.450437086326f,    4.70618130300619f},
  { 109.537602182174f,    2.36710264078988f},
  { 113.638711565993f,    0.0f},
  { 117.712026717946f,    -2.35110536299009f},
  { 121.717474411461f,    -4.64393570106378f},
  { 125.618908954780f,    -6.84159869379449f},
  { 129.385829526364f,    -8.91515156310236f},
  { 132.994384651003f,    -10.8447588267182f},
  { 93.6635644378772f,    6.75650083717287f},
  { 97.4783139551990f,    4.59883911411213f},
  { 101.418115164383f,    2.33542809914013f},
  { 105.450437086326f,    -1.99215638435455e-15f},
  { 109.537602182174f,    -2.36710264078988f},
  { 113.638711565993f,    -4.72213775783758f},
  { 117.712026717946f,    -7.02190914735696f},
  { 121.717474411461f,    -9.22764488160650f},
  { 125.618908954780f,    -11.3080227948550f},
  { 129.385829526364f,    -13.2408470050412f},
  { 93.6635644378772f,    2.26148965929813f},
  { 97.4783139551990f,    0.0f},
  { 101.418115164383f,    -2.33542809914013f},
  { 105.450437086326f,    -4.70618130300619f},
  { 109.537602182174f,    -7.06925915799702f},
  { 113.638711565993f,    -9.38098264244825f},
  { 117.712026717946f,    -11.6009546747238f},
  { 121.717474411461f,    -13.6954596895372f},
  { 125.618908954780f,    -15.6396680917009f},
  { 93.6635644378772f,    -2.26148965929813f},
  { 97.4783139551990f,    -4.59883911411213f},
  { 101.418115164383f,    -6.97549803830799f},
  { 105.450437086326f,    -9.34970354257458f},
  { 109.537602182174f,    -11.6778155497790f},
  { 113.638711565993f,    -13.9181885789507f},
  { 117.712026717946f,    -16.0348118271108f},
  { 121.717474411461f,    -18.0f},
  { 93.6635644378772f,    -6.75650083717288f},
  { 97.4783139551990f,    -9.13917468233299f},
  { 101.418115164383f,    -11.5255919612906f},
  { 105.450437086326f,    -13.8727732231513f},
  { 109.537602182174f,    -16.1383704444056f},
  { 113.638711565993f,    -18.2843746791691f},
  { 117.712026717946f,    -20.2801884851370f},
  { 93.6635644378772f,    -11.1696410784414f},
  { 97.4783139551990f,    -13.5668491634508f},
  { 101.418115164383f,    -15.9332226373311f},
  { 105.450437086326f,    -18.2264204988720f},
  { 109.537602182174f,    -20.4069778874103f},
  { 113.638711565993f,    -22.4415960116381f},
  { 93.6635644378772f,    -15.4527499606845f},
  { 97.4783139551990f,    -17.8356202570103f},
  { 101.418115164383f,    -20.1557367341806f},
  { 105.450437086326f,    -22.3729427285926f},
  { 109.537602182174f,    -24.4517589639450f},
  { 93.6635644378772f,    -19.5661569361721f},
  { 97.4783139551990f,    -21.9094235462107f},
  { 101.418115164383f,    -24.1618860804539f},
  { 105.450437086326f,    -26.2866766550738f},
  { 93.6635644378772f,    -23.4801484036882f},
  { 97.4783139551990f,    -25.7631734014353f},
  { 101.418115164383f,    -27.9318860053080f},
  { 93.6635644378772f,    -27.1751641552756f},
  { 97.4783139551990f,    -29.3822901816024f},
  { 93.6635644378772f,    -30.6409845343737f},
  { -86.3364355621228f,    30.6409845343737f},
  { -82.5216860448010f,    29.3822901816024f},
  { -78.5818848356174f,    27.9318860053080f},
  { -74.5495629136742f,    26.2866766550738f},
  { -70.4623978178261f,    24.4517589639450f},
  { -66.3612884340067f,    22.4415960116381f},
  { -62.2879732820537f,    20.2801884851370f},
  { -58.2825255885390f,    18.0f},
  { -54.3810910452200f,    15.6396680917009f},
  { -50.6141704736363f,    13.2408470050412f},
  { -47.0056153489970f,    10.8447588267182f},
  { -43.5723495188822f,    8.48907956344660f},
  { -40.3247043644708f,    6.20564401643037f},
  { -37.2671844918060f,    4.01919298814923f},
  { -34.3994681299533f,    1.94712105456852f},
  { -90.0f,    28.4036245335526f},
  { -86.3364355621228f,    27.1751641552756f},
  { -82.5216860448010f,    25.7631734014353f},
  { -78.5818848356174f,    24.1618860804539f},
  { -74.5495629136742f,    22.3729427285926f},
  { -70.4623978178261f,    20.4069778874103f},
  { -66.3612884340067f,    18.2843746791691f},
  { -62.2879732820537f,    16.0348118271108f},
  { -58.2825255885390f,    13.6954596895373f},
  { -54.3810910452200f,    11.3080227948550f},
  { -50.6141704736363f,    8.91515156310236f},
  { -47.0056153489970f,    6.55691452862584f},
  { -43.5723495188822f,    4.26796226896432f},
  { -40.3247043644708f,    2.07576608018301f},
  { -37.2671844918060f,    0.0f},
  { -34.3994681299533f,    -1.94712105456852f},
  { -90.0f,    24.8689251184620f},
  { -86.3364355621228f,    23.4801484036882f},
  { -82.5216860448010f,    21.9094235462107f},
  { -78.5818848356174f,    20.1557367341806f},
  { -74.5495629136742f,    18.2264204988720f},
  { -70.4623978178261f,    16.1383704444056f},
  { -66.3612884340067f,    13.9181885789507f},
  { -62.2879732820537f,    11.6009546747238f},
  { -58.2825255885390f,    9.22764488160650f},
  { -54.3810910452200f,    6.84159869379449f},
  { -50.6141704736363f,    4.48472037239199f},
  { -47.0056153489970f,    2.19415657112221f},
  { -43.5723495188822f,    3.61186426073483e-15f},
  { -40.3247043644708f,    -2.07576608018301f},
  { -37.2671844918060f,    -4.01919298814923f},
  { -90.0f,    21.1201128443240f},
  { -86.3364355621228f,    19.5661569361721f},
  { -82.5216860448010f,    17.8356202570103f},
  { -78.5818848356174f,    15.9332226373311f},
  { -74.5495629136742f,    13.8727732231513f},
  { -70.4623978178261f,    11.6778155497790f},
  { -66.3612884340067f,    9.38098264244824f},
  { -62.2879732820537f,    7.02190914735696f},
  { -58.2825255885390f,    4.64393570106378f},
  { -54.3810910452200f,    2.29021346643180f},
  { -50.6141704736363f,    0.0f},
  { -47.0056153489970f,    -2.19415657112221f},
  { -43.5723495188822f,    -4.26796226896433f},
  { -40.3247043644708f,    -6.20564401643037f},
  { -90.0f,    17.1720378505138f},
  { -86.3364355621228f,    15.4527499606845f},
  { -82.5216860448010f,    13.5668491634509f},
  { -78.5818848356174f,    11.5255919612906f},
  { -74.5495629136742f,    9.34970354257458f},
  { -70.4623978178261f,    7.06925915799702f},
  { -66.3612884340067f,    4.72213775783758f},
  { -62.2879732820537f,    2.35110536299009f},
  { -58.2825255885390f,    0.0f},
  { -54.3810910452200f,    -2.29021346643181f},
  { -50.6141704736363f,    -4.48472037239199f},
  { -47.0056153489970f,    -6.55691452862585f},
  { -43.5723495188822f,    -8.48907956344661f},
  { -90.0f,    13.0486510400235f},
  { -86.3364355621228f,    11.1696410784414f},
  { -82.5216860448010f,    9.13917468233299f},
  { -78.5818848356174f,    6.97549803830799f},
  { -74.5495629136742f,    4.70618130300619f},
  { -70.4623978178260f,    2.36710264078988f},
  { -66.3612884340067f,    0.0f},
  { -62.2879732820537f,    -2.35110536299009f},
  { -58.2825255885390f,    -4.64393570106378f},
  { -54.3810910452200f,    -6.84159869379449f},
  { -50.6141704736363f,    -8.91515156310236f},
  { -47.0056153489970f,    -10.8447588267182f},
  { -90.0f,    8.78323060095898f},
  { -86.3364355621228f,    6.75650083717287f},
  { -82.5216860448010f,    4.59883911411213f},
  { -78.5818848356174f,    2.33542809914013f},
  { -74.5495629136742f,    -1.99215638435455e-15f},
  { -70.4623978178261f,    -2.36710264078988f},
  { -66.3612884340067f,    -4.72213775783758f},
  { -62.2879732820537f,    -7.02190914735696f},
  { -58.2825255885390f,    -9.22764488160650f},
  { -54.3810910452200f,    -11.3080227948550f},
  { -50.6141704736363f,    -13.2408470050412f},
  { -90.0f,    4.41756799953082f},
  { -86.3364355621228f,    2.26148965929813f},
  { -82.5216860448010f,    0.0f},
  { -78.5818848356174f,    -2.33542809914013f},
  { -74.5495629136742f,    -4.70618130300619f},
  { -70.4623978178261f,    -7.06925915799702f},
  { -66.3612884340067f,    -9.38098264244825f},
  { -62.2879732820537f,    -11.6009546747238f},
  { -58.2825255885390f,    -13.6954596895372f},
  { -54.3810910452200f,    -15.6396680917009f},
  { -90.0f,    0.0f},
  { -86.3364355621228f,    -2.26148965929813f},
  { -82.5216860448010f,    -4.59883911411213f},
  { -78.5818848356174f,    -6.97549803830799f},
  { -74.5495629136742f,    -9.34970354257458f},
  { -70.4623978178261f,    -11.6778155497790f},
  { -66.3612884340067f,    -13.9181885789507f},
  { -62.2879732820537f,    -16.0348118271108f},
  { -58.2825255885390f,    -18.0f},
  { -90.0f,    -4.41756799953082f},
  { -86.3364355621228f,    -6.75650083717288f},
  { -82.5216860448010f,    -9.13917468233299f},
  { -78.5818848356174f,    -11.5255919612906f},
  { -74.5495629136742f,    -13.8727732231513f},
  { -70.4623978178261f,    -16.1383704444056f},
  { -66.3612884340067f,    -18.2843746791691f},
  { -62.2879732820537f,    -20.2801884851370f},
  { -90.0f,    -8.78323060095898f},
  { -86.3364355621228f,    -11.1696410784414f},
  { -82.5216860448010f,    -13.5668491634508f},
  { -78.5818848356174f,    -15.9332226373311f},
  { -74.5495629136742f,    -18.2264204988720f},
  { -70.4623978178261f,    -20.4069778874103f},
  { -66.3612884340067f,    -22.4415960116381f},
  { -90.0f,    -13.0486510400235f},
  { -86.3364355621228f,    -15.4527499606845f},
  { -82.5216860448010f,    -17.8356202570103f},
  { -78.5818848356174f,    -20.1557367341806f},
  { -74.5495629136742f,    -22.3729427285926f},
  { -70.4623978178261f,    -24.4517589639450f},
  { -90.0f,    -17.1720378505138f},
  { -86.3364355621228f,    -19.5661569361721f},
  { -82.5216860448010f,    -21.9094235462107f},
  { -78.5818848356174f,    -24.1618860804539f},
  { -74.5495629136742f,    -26.2866766550738f},
  { -90.0f,    -21.1201128443240f},
  { -86.3364355621228f,    -23.4801484036882f},
  { -82.5216860448010f,    -25.7631734014353f},
  { -78.5818848356174f,    -27.9318860053080f},
  { -90.0f,    -24.8689251184620f},
  { -86.3364355621228f,    -27.1751641552756f},
  { -82.5216860448010f,    -29.3822901816024f},
  { -90.0f,    -28.4036245335526f},
  { -86.3364355621228f,    -30.6409845343737f},
  { -93.6635644378772f,    30.6409845343737f},
  { -93.6635644378772f,    27.1751641552756f},
  { -93.6635644378772f,    23.4801484036882f},
  { -93.6635644378772f,    19.5661569361721f},
  { -93.6635644378772f,    15.4527499606845f},
  { -93.6635644378772f,    11.1696410784414f},
  { -93.6635644378772f,    6.75650083717288f},
  { -93.6635644378772f,    2.26148965929813f},
  { -93.6635644378772f,    -2.26148965929813f},
  { -93.6635644378772f,    -6.75650083717288f},
  { -93.6635644378772f,    -11.1696410784414f},
  { -93.6635644378772f,    -15.4527499606845f},
  { -93.6635644378772f,    -19.5661569361721f},
  { -93.6635644378772f,    -23.4801484036882f},
  { -93.6635644378772f,    -27.1751641552756f},
  { -93.6635644378772f,    -30.6409845343737f},
  { -97.4783139551990f,    29.3822901816024f},
  { -97.4783139551990f,    25.7631734014353f},
  { -97.4783139551990f,    21.9094235462107f},
  { -97.4783139551990f,    17.8356202570103f},
  { -97.4783139551990f,    13.5668491634509f},
  { -97.4783139551990f,    9.13917468233299f},
  { -97.4783139551990f,    4.59883911411213f},
  { -97.4783139551990f,    0.0f},
  { -97.4783139551990f,    -4.59883911411213f},
  { -97.4783139551990f,    -9.13917468233299f},
  { -97.4783139551990f,    -13.5668491634508f},
  { -97.4783139551990f,    -17.8356202570103f},
  { -97.4783139551990f,    -21.9094235462107f},
  { -97.4783139551990f,    -25.7631734014353f},
  { -97.4783139551990f,    -29.3822901816024f},
  { -101.418115164383f,    27.9318860053080f},
  { -101.418115164383f,    24.1618860804539f},
  { -101.418115164383f,    20.1557367341806f},
  { -101.418115164383f,    15.9332226373311f},
  { -101.418115164383f,    11.5255919612906f},
  { -101.418115164383f,    6.97549803830799f},
  { -101.418115164383f,    2.33542809914013f},
  { -101.418115164383f,    -2.33542809914013f},
  { -101.418115164383f,    -6.97549803830799f},
  { -101.418115164383f,    -11.5255919612906f},
  { -101.418115164383f,    -15.9332226373311f},
  { -101.418115164383f,    -20.1557367341806f},
  { -101.418115164383f,    -24.1618860804539f},
  { -101.418115164383f,    -27.9318860053080f},
  { -105.450437086326f,    26.2866766550738f},
  { -105.450437086326f,    22.3729427285926f},
  { -105.450437086326f,    18.2264204988720f},
  { -105.450437086326f,    13.8727732231513f},
  { -105.450437086326f,    9.34970354257458f},
  { -105.450437086326f,    4.70618130300619f},
  { -105.450437086326f,    0.0f},
  { -105.450437086326f,    -4.70618130300619f},
  { -105.450437086326f,    -9.34970354257458f},
  { -105.450437086326f,    -13.8727732231513f},
  { -105.450437086326f,    -18.2264204988720f},
  { -105.450437086326f,    -22.3729427285926f},
  { -105.450437086326f,    -26.2866766550738f},
  { -109.537602182174f,    24.4517589639450f},
  { -109.537602182174f,    20.4069778874103f},
  { -109.537602182174f,    16.1383704444056f},
  { -109.537602182174f,    11.6778155497790f},
  { -109.537602182174f,    7.06925915799702f},
  { -109.537602182174f,    2.36710264078988f},
  { -109.537602182174f,    -2.36710264078988f},
  { -109.537602182174f,    -7.06925915799702f},
  { -109.537602182174f,    -11.6778155497790f},
  { -109.537602182174f,    -16.1383704444056f},
  { -109.537602182174f,    -20.4069778874103f},
  { -109.537602182174f,    -24.4517589639450f},
  { -113.638711565993f,    22.4415960116381f},
  { -113.638711565993f,    18.2843746791691f},
  { -113.638711565993f,    13.9181885789507f},
  { -113.638711565993f,    9.38098264244825f},
  { -113.638711565993f,    4.72213775783758f},
  { -113.638711565993f,    0.0f},
  { -113.638711565993f,    -4.72213775783758f},
  { -113.638711565993f,    -9.38098264244824f},
  { -113.638711565993f,    -13.9181885789507f},
  { -113.638711565993f,    -18.2843746791691f},
  { -113.638711565993f,    -22.4415960116381f},
  { -117.712026717946f,    20.2801884851370f},
  { -117.712026717946f,    16.0348118271108f},
  { -117.712026717946f,    11.6009546747238f},
  { -117.712026717946f,    7.02190914735696f},
  { -117.712026717946f,    2.35110536299009f},
  { -117.712026717946f,    -2.35110536299009f},
  { -117.712026717946f,    -7.02190914735696f},
  { -117.712026717946f,    -11.6009546747238f},
  { -117.712026717946f,    -16.0348118271108f},
  { -117.712026717946f,    -20.2801884851370f},
  { -121.717474411461f,    18.0f},
  { -121.717474411461f,    13.6954596895373f},
  { -121.717474411461f,    9.22764488160650f},
  { -121.717474411461f,    4.64393570106378f},
  { -121.717474411461f,    0.0f},
  { -121.717474411461f,    -4.64393570106378f},
  { -121.717474411461f,    -9.22764488160650f},
  { -121.717474411461f,    -13.6954596895373f},
  { -121.717474411461f,    -18.0f},
  { -125.618908954780f,    15.6396680917009f},
  { -125.618908954780f,    11.3080227948550f},
  { -125.618908954780f,    6.84159869379449f},
  { -125.618908954780f,    2.29021346643181f},
  { -125.618908954780f,    -2.29021346643181f},
  { -125.618908954780f,    -6.84159869379449f},
  { -125.618908954780f,    -11.3080227948550f},
  { -125.618908954780f,    -15.6396680917009f},
  { -129.385829526364f,    13.2408470050412f},
  { -129.385829526364f,    8.91515156310237f},
  { -129.385829526364f,    4.48472037239199f},
  { -129.385829526364f,    0.0f},
  { -129.385829526364f,    -4.48472037239199f},
  { -129.385829526364f,    -8.91515156310237f},
  { -129.385829526364f,    -13.2408470050412f},
  { -132.994384651003f,    10.8447588267182f},
  { -132.994384651003f,    6.55691452862584f},
  { -132.994384651003f,    2.19415657112221f},
  { -132.994384651003f,    -2.19415657112221f},
  { -132.994384651003f,    -6.55691452862585f},
  { -132.994384651003f,    -10.8447588267182f},
  { -136.427650481118f,    8.48907956344661f},
  { -136.427650481118f,    4.26796226896432f},
  { -136.427650481118f,    0.0f},
  { -136.427650481118f,    -4.26796226896432f},
  { -136.427650481118f,    -8.48907956344661f},
  { -139.675295635529f,    6.20564401643037f},
  { -139.675295635529f,    2.07576608018301f},
  { -139.675295635529f,    -2.07576608018301f},
  { -139.675295635529f,    -6.20564401643037f},
  { -142.732815508194f,    4.01919298814923f},
  { -142.732815508194f,    0.0f},
  { -142.732815508194f,    -4.01919298814923f},
  { -145.600531870047f,    1.94712105456852f},
  { -145.600531870047f,    -1.94712105456852f},
  { 83.6837831453820f,    33.2770772600221f},
  { 79.5796514274116f,    31.9703418211532f},
  { 75.3639465831322f,    30.4465738857480f},
  { 71.0791005425548f,    28.7040129778536f},
  { 66.7719770717799f,    26.7511554968076f},
  { 62.4909134890006f,    24.6077737779805f},
  { 58.2825255885390f,    22.3045403104628f},
  { 54.1887887080980f,    19.8810951781929f},
  { 50.2448283218477f,    17.3827856796100f},
  { 46.4776612097371f,    14.8566700706480f},
  { 42.9059021380434f,    12.3475590924140f},
  { 39.5402693503879f,    9.89479703232183f},
  { 36.3846272804150f,    7.53020899059060f},
  { 33.4372958758630f,    5.27729717955544f},
  { 80.6752965174939f,    36.0213780600392f},
  { 76.2613434253436f,    34.6470309014331f},
  { 71.7603695603461f,    33.0274403302419f},
  { 67.2262996676901f,    31.1637336664207f},
  { 62.7154723436679f,    29.0697052196415f},
  { 58.2825255885390f,    26.7723551183935f},
  { 53.9765838418650f,    24.3105938788995f},
  { 49.8383783329354f,    21.7321521847258f},
  { 45.8986531028256f,    19.0892233146767f},
  { 42.1778722582693f,    16.4337077246310f},
  { 38.6869780240083f,    13.8129800206087f},
  { 35.4288203482238f,    11.2668552009329f},
  { 32.3998806257594f,    8.82602735039754f},
  { 77.2546118211169f,    38.8402116496635f},
  { 72.5166778869728f,    37.3720700573987f},
  { 67.7315327880415f,    35.6285964660136f},
  { 62.9652905045299f,    33.6160922031027f},
  { 58.2825255885390f,    31.3560642989362f},
  { 53.7412249276072f,    28.8848052197659f},
  { 49.3889802274837f,    26.2507589510491f},
  { 45.2609467589194f,    23.5100597759421f},
  { 41.3795820583307f,    20.7211331459400f},
  { 37.7557817087012f,    17.9394582422071f},
  { 34.3908543036368f,    15.2134343179303f},
  { 31.2788043067482f,    12.5818702829736f},
  { 73.3608329604165f,    41.6860404488482f},
  { 68.2966481036082f,    40.0910997872291f},
  { 63.2448725293831f,    38.1909250447259f},
  { 58.2825255885390f,    36.0f},
  { 53.4787131470862f,    33.5502168403183f},
  { 48.8895122619748f,    30.8889289128016f},
  { 44.5553246366432f,    28.0746536530332f},
  { 40.5007050124200f,    25.1712621719368f},
  { 36.7360677626856f,    22.2418637966772f},
  { 33.2604381086868f,    19.3435614209637f},
  { 30.0644934600711f,    16.5238653501529f},
  { 68.9328300288820f,    44.4965112658956f},
  { 63.5598584970145f,    42.7360072735219f},
  { 58.2825255885390f,    40.6439357010638f},
  { 53.1840752157190f,    38.2463751067230f},
  { 48.3312033461314f,    35.5879507715174f},
  { 43.7705797377297f,    32.7278967671309f},
  { 39.5288736468407f,    29.7340780099319f},
  { 35.6153335175461f,    26.6762163700866f},
  { 32.0256543412126f,    23.6196557310574f},
  { 28.7460525381119f,    20.6206896955149f},
  { 63.9174046243709f,    47.1952759481286f},
  { 58.2825255885390f,    45.2276448816065f},
  { 52.8510482570118f,    42.9103372322102f},
  { 47.7031324115841f,    40.2840762101672f},
  { 42.8929642205859f,    37.4073679693909f},
  { 38.4491770335041f,    34.3505184589462f},
  { 34.3787851159820f,    31.1884471801445f},
  { 30.6726725373555f,    27.9936984541351f},
  { 27.3110807103421f,    24.8308196085475f},
  { 58.2825255885390f,    49.6954596895373f},
  { 52.4716343959111f,    47.4815872025892f},
  { 46.9915364853117f,    44.9134373033411f},
  { 41.9054508328803f,    42.0478882595460f},
  { 37.2435136550372f,    38.9563745751136f},
  { 33.0088450024986f,    35.7174705636480f},
  { 29.1854176298183f,    32.4094431897058f},
  { 25.7454766760442f,    29.1039569959989f},
  { 52.0354578629482f,    51.9064810941727f},
  { 46.1788380339297f,    49.4169517893457f},
  { 40.7867324071109f,    46.5865463730390f},
  { 35.8897546595841f,    43.4870777455218f},
  { 31.4845506248442f,    40.1994280057052f},
  { 27.5451941633379f,    36.8059220640418f},
  { 24.0332409763117f,    33.3837304845798f},
  { 45.2422521238312f,    53.7442816063043f},
  { 39.5098510924851f,    50.9672098019515f},
  { 34.3606597988619f,    47.8824607276022f},
  { 29.7808283305222f,    44.5723576414450f},
  { 25.7302743698864f,    41.1217160940118f},
  { 22.1562933196709f,    37.6113523273991f},
  { 38.0402996860279f,    55.1436992467066f},
  { 32.6224749368397f,    52.0905755468793f},
  { 27.8676337936429f,    48.7803925292739f},
  { 23.7154719527922f,    45.2992117296112f},
  { 20.0943348892778f,    41.7296289222711f},
  { 30.6331305614914f,    56.0699480487420f},
  { 25.7089698893421f,    52.7767392956372f},
  { 21.4717476597310f,    49.2880666149180f},
  { 17.8248042352999f,    45.6866114324158f},
  { 23.2618363217055f,    56.5250600061002f},
  { 18.9659334090314f,    53.0473569347008f},
  { 15.3230019912138f,    49.4382883828622f},
  { 16.1607266865462f,    56.5462580056053f},
  { 12.5624953312201f,    52.9500102616328f},
  { 9.51595614845805f,    56.1966706603438f},
  { 83.6837831453820f,    -33.2770772600221f},
  { 80.6752965174939f,    -36.0213780600392f},
  { 77.2546118211169f,    -38.8402116496635f},
  { 73.3608329604165f,    -41.6860404488482f},
  { 68.9328300288820f,    -44.4965112658956f},
  { 63.9174046243709f,    -47.1952759481287f},
  { 58.2825255885390f,    -49.6954596895373f},
  { 52.0354578629482f,    -51.9064810941727f},
  { 45.2422521238312f,    -53.7442816063043f},
  { 38.0402996860279f,    -55.1436992467066f},
  { 30.6331305614914f,    -56.0699480487420f},
  { 23.2618363217055f,    -56.5250600061002f},
  { 16.1607266865462f,    -56.5462580056053f},
  { 9.51595614845805f,    -56.1966706603438f},
  { 79.5796514274116f,    -31.9703418211533f},
  { 76.2613434253436f,    -34.6470309014331f},
  { 72.5166778869728f,    -37.3720700573987f},
  { 68.2966481036082f,    -40.0910997872291f},
  { 63.5598584970145f,    -42.7360072735219f},
  { 58.2825255885390f,    -45.2276448816065f},
  { 52.4716343959111f,    -47.4815872025892f},
  { 46.1788380339297f,    -49.4169517893457f},
  { 39.5098510924851f,    -50.9672098019515f},
  { 32.6224749368397f,    -52.0905755468793f},
  { 25.7089698893421f,    -52.7767392956372f},
  { 18.9659334090314f,    -53.0473569347008f},
  { 12.5624953312201f,    -52.9500102616328f},
  { 75.3639465831322f,    -30.4465738857480f},
  { 71.7603695603460f,    -33.0274403302419f},
  { 67.7315327880415f,    -35.6285964660136f},
  { 63.2448725293831f,    -38.1909250447259f},
  { 58.2825255885390f,    -40.6439357010638f},
  { 52.8510482570118f,    -42.9103372322102f},
  { 46.9915364853117f,    -44.9134373033411f},
  { 40.7867324071109f,    -46.5865463730390f},
  { 34.3606597988619f,    -47.8824607276022f},
  { 27.8676337936429f,    -48.7803925292739f},
  { 21.4717476597310f,    -49.2880666149180f},
  { 15.3230019912138f,    -49.4382883828622f},
  { 71.0791005425548f,    -28.7040129778536f},
  { 67.2262996676901f,    -31.1637336664207f},
  { 62.9652905045299f,    -33.6160922031027f},
  { 58.2825255885390f,    -36.0f},
  { 53.1840752157190f,    -38.2463751067230f},
  { 47.7031324115841f,    -40.2840762101672f},
  { 41.9054508328803f,    -42.0478882595460f},
  { 35.8897546595841f,    -43.4870777455218f},
  { 29.7808283305222f,    -44.5723576414450f},
  { 23.7154719527922f,    -45.2992117296112f},
  { 17.8248042352999f,    -45.6866114324158f},
  { 66.7719770717799f,    -26.7511554968076f},
  { 62.7154723436679f,    -29.0697052196415f},
  { 58.2825255885390f,    -31.3560642989362f},
  { 53.4787131470862f,    -33.5502168403183f},
  { 48.3312033461314f,    -35.5879507715174f},
  { 42.8929642205859f,    -37.4073679693909f},
  { 37.2435136550372f,    -38.9563745751136f},
  { 31.4845506248442f,    -40.1994280057052f},
  { 25.7302743698864f,    -41.1217160940118f},
  { 20.0943348892778f,    -41.7296289222711f},
  { 62.4909134890006f,    -24.6077737779805f},
  { 58.2825255885390f,    -26.7723551183935f},
  { 53.7412249276072f,    -28.8848052197659f},
  { 48.8895122619748f,    -30.8889289128016f},
  { 43.7705797377297f,    -32.7278967671309f},
  { 38.4491770335041f,    -34.3505184589462f},
  { 33.0088450024986f,    -35.7174705636480f},
  { 27.5451941633379f,    -36.8059220640418f},
  { 22.1562933196709f,    -37.6113523273991f},
  { 58.2825255885390f,    -22.3045403104628f},
  { 53.9765838418650f,    -24.3105938788995f},
  { 49.3889802274837f,    -26.2507589510491f},
  { 44.5553246366432f,    -28.0746536530332f},
  { 39.5288736468407f,    -29.7340780099319f},
  { 34.3787851159820f,    -31.1884471801445f},
  { 29.1854176298183f,    -32.4094431897058f},
  { 24.0332409763117f,    -33.3837304845798f},
  { 54.1887887080980f,    -19.8810951781929f},
  { 49.8383783329354f,    -21.7321521847258f},
  { 45.2609467589194f,    -23.5100597759421f},
  { 40.5007050124200f,    -25.1712621719368f},
  { 35.6153335175461f,    -26.6762163700866f},
  { 30.6726725373555f,    -27.9936984541351f},
  { 25.7454766760442f,    -29.1039569959989f},
  { 50.2448283218477f,    -17.3827856796100f},
  { 45.8986531028256f,    -19.0892233146767f},
  { 41.3795820583307f,    -20.7211331459400f},
  { 36.7360677626856f,    -22.2418637966772f},
  { 32.0256543412126f,    -23.6196557310574f},
  { 27.3110807103421f,    -24.8308196085475f},
  { 46.4776612097371f,    -14.8566700706480f},
  { 42.1778722582693f,    -16.4337077246310f},
  { 37.7557817087012f,    -17.9394582422071f},
  { 33.2604381086868f,    -19.3435614209637f},
  { 28.7460525381119f,    -20.6206896955149f},
  { 42.9059021380434f,    -12.3475590924140f},
  { 38.6869780240083f,    -13.8129800206087f},
  { 34.3908543036368f,    -15.2134343179303f},
  { 30.0644934600711f,    -16.5238653501529f},
  { 39.5402693503879f,    -9.89479703232183f},
  { 35.4288203482238f,    -11.2668552009329f},
  { 31.2788043067482f,    -12.5818702829736f},
  { 36.3846272804150f,    -7.53020899059060f},
  { 32.3998806257594f,    -8.82602735039754f},
  { 33.4372958758630f,    -5.27729717955544f},
  { -9.51595614845805f,    56.1966706603438f},
  { -12.5624953312201f,    52.9500102616329f},
  { -15.3230019912138f,    49.4382883828622f},
  { -17.8248042352999f,    45.6866114324158f},
  { -20.0943348892778f,    41.7296289222711f},
  { -22.1562933196709f,    37.6113523273991f},
  { -24.0332409763117f,    33.3837304845798f},
  { -25.7454766760442f,    29.1039569959989f},
  { -27.3110807103421f,    24.8308196085475f},
  { -28.7460525381118f,    20.6206896955149f},
  { -30.0644934600711f,    16.5238653501529f},
  { -31.2788043067482f,    12.5818702829736f},
  { -32.3998806257594f,    8.82602735039754f},
  { -33.4372958758630f,    5.27729717955544f},
  { -16.1607266865462f,    56.5462580056053f},
  { -18.9659334090314f,    53.0473569347008f},
  { -21.4717476597310f,    49.2880666149180f},
  { -23.7154719527922f,    45.2992117296112f},
  { -25.7302743698864f,    41.1217160940118f},
  { -27.5451941633379f,    36.8059220640418f},
  { -29.1854176298183f,    32.4094431897058f},
  { -30.6726725373555f,    27.9936984541352f},
  { -32.0256543412126f,    23.6196557310574f},
  { -33.2604381086868f,    19.3435614209637f},
  { -34.3908543036368f,    15.2134343179303f},
  { -35.4288203482238f,    11.2668552009329f},
  { -36.3846272804150f,    7.53020899059061f},
  { -23.2618363217055f,    56.5250600061002f},
  { -25.7089698893421f,    52.7767392956372f},
  { -27.8676337936429f,    48.7803925292739f},
  { -29.7808283305223f,    44.5723576414450f},
  { -31.4845506248442f,    40.1994280057052f},
  { -33.0088450024986f,    35.7174705636480f},
  { -34.3787851159820f,    31.1884471801445f},
  { -35.6153335175461f,    26.6762163700866f},
  { -36.7360677626856f,    22.2418637966772f},
  { -37.7557817087012f,    17.9394582422070f},
  { -38.6869780240083f,    13.8129800206087f},
  { -39.5402693503879f,    9.89479703232183f},
  { -30.6331305614914f,    56.0699480487420f},
  { -32.6224749368397f,    52.0905755468793f},
  { -34.3606597988619f,    47.8824607276022f},
  { -35.8897546595841f,    43.4870777455218f},
  { -37.2435136550372f,    38.9563745751136f},
  { -38.4491770335041f,    34.3505184589462f},
  { -39.5288736468407f,    29.7340780099319f},
  { -40.5007050124200f,    25.1712621719368f},
  { -41.3795820583307f,    20.7211331459400f},
  { -42.1778722582693f,    16.4337077246310f},
  { -42.9059021380434f,    12.3475590924140f},
  { -38.0402996860279f,    55.1436992467066f},
  { -39.5098510924851f,    50.9672098019515f},
  { -40.7867324071109f,    46.5865463730390f},
  { -41.9054508328803f,    42.0478882595460f},
  { -42.8929642205859f,    37.4073679693909f},
  { -43.7705797377297f,    32.7278967671309f},
  { -44.5553246366432f,    28.0746536530332f},
  { -45.2609467589194f,    23.5100597759421f},
  { -45.8986531028256f,    19.0892233146767f},
  { -46.4776612097371f,    14.8566700706480f},
  { -45.2422521238312f,    53.7442816063043f},
  { -46.1788380339297f,    49.4169517893457f},
  { -46.9915364853117f,    44.9134373033411f},
  { -47.7031324115841f,    40.2840762101672f},
  { -48.3312033461314f,    35.5879507715174f},
  { -48.8895122619748f,    30.8889289128016f},
  { -49.3889802274837f,    26.2507589510491f},
  { -49.8383783329354f,    21.7321521847258f},
  { -50.2448283218477f,    17.3827856796100f},
  { -52.0354578629482f,    51.9064810941727f},
  { -52.4716343959111f,    47.4815872025892f},
  { -52.8510482570118f,    42.9103372322102f},
  { -53.1840752157190f,    38.2463751067230f},
  { -53.4787131470862f,    33.5502168403183f},
  { -53.7412249276072f,    28.8848052197659f},
  { -53.9765838418650f,    24.3105938788995f},
  { -54.1887887080980f,    19.8810951781929f},
  { -58.2825255885390f,    49.6954596895373f},
  { -58.2825255885390f,    45.2276448816065f},
  { -58.2825255885390f,    40.6439357010638f},
  { -58.2825255885390f,    36.0f},
  { -58.2825255885390f,    31.3560642989362f},
  { -58.2825255885390f,    26.7723551183935f},
  { -58.2825255885390f,    22.3045403104628f},
  { -63.9174046243709f,    47.1952759481287f},
  { -63.5598584970145f,    42.7360072735219f},
  { -63.2448725293831f,    38.1909250447259f},
  { -62.9652905045299f,    33.6160922031027f},
  { -62.7154723436679f,    29.0697052196415f},
  { -62.4909134890006f,    24.6077737779805f},
  { -68.9328300288820f,    44.4965112658956f},
  { -68.2966481036082f,    40.0910997872291f},
  { -67.7315327880415f,    35.6285964660136f},
  { -67.2262996676901f,    31.1637336664207f},
  { -66.7719770717799f,    26.7511554968076f},
  { -73.3608329604165f,    41.6860404488482f},
  { -72.5166778869728f,    37.3720700573987f},
  { -71.7603695603461f,    33.0274403302419f},
  { -71.0791005425548f,    28.7040129778536f},
  { -77.2546118211169f,    38.8402116496635f},
  { -76.2613434253436f,    34.6470309014331f},
  { -75.3639465831322f,    30.4465738857480f},
  { -80.6752965174939f,    36.0213780600392f},
  { -79.5796514274116f,    31.9703418211533f},
  { -83.6837831453820f,    33.2770772600221f},
  { -9.51595614845805f,    -56.1966706603438f},
  { -16.1607266865462f,    -56.5462580056053f},
  { -23.2618363217055f,    -56.5250600061002f},
  { -30.6331305614914f,    -56.0699480487420f},
  { -38.0402996860279f,    -55.1436992467066f},
  { -45.2422521238312f,    -53.7442816063043f},
  { -52.0354578629482f,    -51.9064810941727f},
  { -58.2825255885390f,    -49.6954596895373f},
  { -63.9174046243709f,    -47.1952759481287f},
  { -68.9328300288820f,    -44.4965112658956f},
  { -73.3608329604165f,    -41.6860404488482f},
  { -77.2546118211169f,    -38.8402116496635f},
  { -80.6752965174939f,    -36.0213780600392f},
  { -83.6837831453820f,    -33.2770772600221f},
  { -12.5624953312201f,    -52.9500102616329f},
  { -18.9659334090314f,    -53.0473569347008f},
  { -25.7089698893421f,    -52.7767392956372f},
  { -32.6224749368397f,    -52.0905755468793f},
  { -39.5098510924851f,    -50.9672098019515f},
  { -46.1788380339297f,    -49.4169517893457f},
  { -52.4716343959111f,    -47.4815872025892f},
  { -58.2825255885390f,    -45.2276448816065f},
  { -63.5598584970145f,    -42.7360072735219f},
  { -68.2966481036082f,    -40.0910997872291f},
  { -72.5166778869728f,    -37.3720700573987f},
  { -76.2613434253436f,    -34.6470309014331f},
  { -79.5796514274116f,    -31.9703418211533f},
  { -15.3230019912138f,    -49.4382883828622f},
  { -21.4717476597310f,    -49.2880666149180f},
  { -27.8676337936429f,    -48.7803925292739f},
  { -34.3606597988619f,    -47.8824607276022f},
  { -40.7867324071109f,    -46.5865463730390f},
  { -46.9915364853117f,    -44.9134373033411f},
  { -52.8510482570118f,    -42.9103372322102f},
  { -58.2825255885390f,    -40.6439357010638f},
  { -63.2448725293831f,    -38.1909250447259f},
  { -67.7315327880415f,    -35.6285964660136f},
  { -71.7603695603460f,    -33.0274403302419f},
  { -75.3639465831322f,    -30.4465738857480f},
  { -17.8248042352999f,    -45.6866114324158f},
  { -23.7154719527922f,    -45.2992117296112f},
  { -29.7808283305223f,    -44.5723576414450f},
  { -35.8897546595841f,    -43.4870777455218f},
  { -41.9054508328803f,    -42.0478882595460f},
  { -47.7031324115841f,    -40.2840762101672f},
  { -53.1840752157190f,    -38.2463751067230f},
  { -58.2825255885390f,    -36.0f},
  { -62.9652905045299f,    -33.6160922031027f},
  { -67.2262996676901f,    -31.1637336664207f},
  { -71.0791005425548f,    -28.7040129778536f},
  { -20.0943348892778f,    -41.7296289222711f},
  { -25.7302743698865f,    -41.1217160940118f},
  { -31.4845506248442f,    -40.1994280057052f},
  { -37.2435136550372f,    -38.9563745751136f},
  { -42.8929642205859f,    -37.4073679693909f},
  { -48.3312033461314f,    -35.5879507715174f},
  { -53.4787131470862f,    -33.5502168403183f},
  { -58.2825255885390f,    -31.3560642989362f},
  { -62.7154723436679f,    -29.0697052196415f},
  { -66.7719770717799f,    -26.7511554968076f},
  { -22.1562933196709f,    -37.6113523273991f},
  { -27.5451941633379f,    -36.8059220640418f},
  { -33.0088450024986f,    -35.7174705636480f},
  { -38.4491770335041f,    -34.3505184589462f},
  { -43.7705797377297f,    -32.7278967671309f},
  { -48.8895122619748f,    -30.8889289128016f},
  { -53.7412249276072f,    -28.8848052197659f},
  { -58.2825255885390f,    -26.7723551183935f},
  { -62.4909134890006f,    -24.6077737779805f},
  { -24.0332409763117f,    -33.3837304845798f},
  { -29.1854176298183f,    -32.4094431897058f},
  { -34.3787851159820f,    -31.1884471801445f},
  { -39.5288736468407f,    -29.7340780099319f},
  { -44.5553246366432f,    -28.0746536530332f},
  { -49.3889802274837f,    -26.2507589510491f},
  { -53.9765838418650f,    -24.3105938788995f},
  { -58.2825255885390f,    -22.3045403104628f},
  { -25.7454766760442f,    -29.1039569959989f},
  { -30.6726725373555f,    -27.9936984541352f},
  { -35.6153335175461f,    -26.6762163700866f},
  { -40.5007050124200f,    -25.1712621719368f},
  { -45.2609467589194f,    -23.5100597759421f},
  { -49.8383783329354f,    -21.7321521847258f},
  { -54.1887887080980f,    -19.8810951781929f},
  { -27.3110807103421f,    -24.8308196085475f},
  { -32.0256543412126f,    -23.6196557310574f},
  { -36.7360677626856f,    -22.2418637966772f},
  { -41.3795820583307f,    -20.7211331459400f},
  { -45.8986531028256f,    -19.0892233146767f},
  { -50.2448283218477f,    -17.3827856796100f},
  { -28.7460525381119f,    -20.6206896955149f},
  { -33.2604381086868f,    -19.3435614209637f},
  { -37.7557817087012f,    -17.9394582422071f},
  { -42.1778722582693f,    -16.4337077246310f},
  { -46.4776612097371f,    -14.8566700706480f},
  { -30.0644934600711f,    -16.5238653501529f},
  { -34.3908543036368f,    -15.2134343179303f},
  { -38.6869780240083f,    -13.8129800206087f},
  { -42.9059021380434f,    -12.3475590924140f},
  { -31.2788043067482f,    -12.5818702829736f},
  { -35.4288203482238f,    -11.2668552009329f},
  { -39.5402693503879f,    -9.89479703232183f},
  { -32.3998806257594f,    -8.82602735039754f},
  { -36.3846272804150f,    -7.53020899059060f},
  { -33.4372958758630f,    -5.27729717955544f},
  { 170.484043851542f,    56.1966706603438f},
  { 167.437504668780f,    52.9500102616329f},
  { 164.676998008786f,    49.4382883828622f},
  { 162.175195764700f,    45.6866114324158f},
  { 159.905665110722f,    41.7296289222711f},
  { 157.843706680329f,    37.6113523273991f},
  { 155.966759023688f,    33.3837304845798f},
  { 154.254523323956f,    29.1039569959989f},
  { 152.688919289658f,    24.8308196085475f},
  { 151.253947461888f,    20.6206896955149f},
  { 149.935506539929f,    16.5238653501529f},
  { 148.721195693252f,    12.5818702829736f},
  { 147.600119374241f,    8.82602735039754f},
  { 146.562704124137f,    5.27729717955544f},
  { 163.839273313454f,    56.5462580056053f},
  { 161.034066590969f,    53.0473569347008f},
  { 158.528252340269f,    49.2880666149180f},
  { 156.284528047208f,    45.2992117296112f},
  { 154.269725630114f,    41.1217160940117f},
  { 152.454805836662f,    36.8059220640418f},
  { 150.814582370182f,    32.4094431897058f},
  { 149.327327462644f,    27.9936984541352f},
  { 147.974345658787f,    23.6196557310574f},
  { 146.739561891313f,    19.3435614209637f},
  { 145.609145696363f,    15.2134343179302f},
  { 144.571179651776f,    11.2668552009329f},
  { 143.615372719585f,    7.53020899059061f},
  { 156.738163678295f,    56.5250600061002f},
  { 154.291030110658f,    52.7767392956372f},
  { 152.132366206357f,    48.7803925292739f},
  { 150.219171669478f,    44.5723576414450f},
  { 148.515449375156f,    40.1994280057052f},
  { 146.991154997501f,    35.7174705636480f},
  { 145.621214884018f,    31.1884471801445f},
  { 144.384666482454f,    26.6762163700866f},
  { 143.263932237314f,    22.2418637966772f},
  { 142.244218291299f,    17.9394582422070f},
  { 141.313021975992f,    13.8129800206087f},
  { 140.459730649612f,    9.89479703232183f},
  { 149.366869438509f,    56.0699480487420f},
  { 147.377525063160f,    52.0905755468793f},
  { 145.639340201138f,    47.8824607276022f},
  { 144.110245340416f,    43.4870777455218f},
  { 142.756486344963f,    38.9563745751136f},
  { 141.550822966496f,    34.3505184589462f},
  { 140.471126353159f,    29.7340780099319f},
  { 139.499294987580f,    25.1712621719368f},
  { 138.620417941669f,    20.7211331459400f},
  { 137.822127741731f,    16.4337077246310f},
  { 137.094097861957f,    12.3475590924140f},
  { 141.959700313972f,    55.1436992467066f},
  { 140.490148907515f,    50.9672098019515f},
  { 139.213267592889f,    46.5865463730390f},
  { 138.094549167120f,    42.0478882595460f},
  { 137.107035779414f,    37.4073679693909f},
  { 136.229420262270f,    32.7278967671309f},
  { 135.444675363357f,    28.0746536530332f},
  { 134.739053241081f,    23.5100597759421f},
  { 134.101346897174f,    19.0892233146767f},
  { 133.522338790263f,    14.8566700706480f},
  { 134.757747876169f,    53.7442816063043f},
  { 133.821161966070f,    49.4169517893457f},
  { 133.008463514688f,    44.9134373033411f},
  { 132.296867588416f,    40.2840762101672f},
  { 131.668796653869f,    35.5879507715174f},
  { 131.110487738025f,    30.8889289128016f},
  { 130.611019772516f,    26.2507589510491f},
  { 130.161621667065f,    21.7321521847258f},
  { 129.755171678152f,    17.3827856796100f},
  { 127.964542137052f,    51.9064810941727f},
  { 127.528365604089f,    47.4815872025892f},
  { 127.148951742988f,    42.9103372322102f},
  { 126.815924784281f,    38.2463751067230f},
  { 126.521286852914f,    33.5502168403183f},
  { 126.258775072393f,    28.8848052197659f},
  { 126.023416158135f,    24.3105938788995f},
  { 125.811211291902f,    19.8810951781929f},
  { 121.717474411461f,    49.6954596895373f},
  { 121.717474411461f,    45.2276448816065f},
  { 121.717474411461f,    40.6439357010638f},
  { 121.717474411461f,    36.0f},
  { 121.717474411461f,    31.3560642989362f},
  { 121.717474411461f,    26.7723551183935f},
  { 121.717474411461f,    22.3045403104628f},
  { 116.082595375629f,    47.1952759481287f},
  { 116.440141502986f,    42.7360072735219f},
  { 116.755127470617f,    38.1909250447259f},
  { 117.034709495470f,    33.6160922031027f},
  { 117.284527656332f,    29.0697052196415f},
  { 117.509086510999f,    24.6077737779805f},
  { 111.067169971118f,    44.4965112658956f},
  { 111.703351896392f,    40.0910997872291f},
  { 112.268467211959f,    35.6285964660136f},
  { 112.773700332310f,    31.1637336664207f},
  { 113.228022928220f,    26.7511554968076f},
  { 106.639167039584f,    41.6860404488482f},
  { 107.483322113027f,    37.3720700573987f},
  { 108.239630439654f,    33.0274403302419f},
  { 108.920899457445f,    28.7040129778536f},
  { 102.745388178883f,    38.8402116496635f},
  { 103.738656574656f,    34.6470309014331f},
  { 104.636053416868f,    30.4465738857480f},
  { 99.3247034825061f,    36.0213780600392f},
  { 100.420348572588f,    31.9703418211533f},
  { 96.3162168546181f,    33.2770772600221f},
  { 170.484043851542f,    -56.1966706603438f},
  { 163.839273313454f,    -56.5462580056053f},
  { 156.738163678295f,    -56.5250600061002f},
  { 149.366869438509f,    -56.0699480487420f},
  { 141.959700313972f,    -55.1436992467066f},
  { 134.757747876169f,    -53.7442816063043f},
  { 127.964542137052f,    -51.9064810941727f},
  { 121.717474411461f,    -49.6954596895373f},
  { 116.082595375629f,    -47.1952759481287f},
  { 111.067169971118f,    -44.4965112658956f},
  { 106.639167039584f,    -41.6860404488482f},
  { 102.745388178883f,    -38.8402116496635f},
  { 99.3247034825061f,    -36.0213780600392f},
  { 96.3162168546181f,    -33.2770772600221f},
  { 167.437504668780f,    -52.9500102616329f},
  { 161.034066590969f,    -53.0473569347008f},
  { 154.291030110658f,    -52.7767392956372f},
  { 147.377525063160f,    -52.0905755468793f},
  { 140.490148907515f,    -50.9672098019515f},
  { 133.821161966070f,    -49.4169517893457f},
  { 127.528365604089f,    -47.4815872025892f},
  { 121.717474411461f,    -45.2276448816065f},
  { 116.440141502986f,    -42.7360072735219f},
  { 111.703351896392f,    -40.0910997872291f},
  { 107.483322113027f,    -37.3720700573987f},
  { 103.738656574656f,    -34.6470309014331f},
  { 100.420348572588f,    -31.9703418211533f},
  { 164.676998008786f,    -49.4382883828622f},
  { 158.528252340269f,    -49.2880666149180f},
  { 152.132366206357f,    -48.7803925292739f},
  { 145.639340201138f,    -47.8824607276022f},
  { 139.213267592889f,    -46.5865463730390f},
  { 133.008463514688f,    -44.9134373033411f},
  { 127.148951742988f,    -42.9103372322102f},
  { 121.717474411461f,    -40.6439357010638f},
  { 116.755127470617f,    -38.1909250447259f},
  { 112.268467211959f,    -35.6285964660136f},
  { 108.239630439654f,    -33.0274403302419f},
  { 104.636053416868f,    -30.4465738857480f},
  { 162.175195764700f,    -45.6866114324158f},
  { 156.284528047208f,    -45.2992117296112f},
  { 150.219171669478f,    -44.5723576414450f},
  { 144.110245340416f,    -43.4870777455218f},
  { 138.094549167120f,    -42.0478882595460f},
  { 132.296867588416f,    -40.2840762101672f},
  { 126.815924784281f,    -38.2463751067230f},
  { 121.717474411461f,    -36.0f},
  { 117.034709495470f,    -33.6160922031027f},
  { 112.773700332310f,    -31.1637336664207f},
  { 108.920899457445f,    -28.7040129778536f},
  { 159.905665110722f,    -41.7296289222711f},
  { 154.269725630114f,    -41.1217160940118f},
  { 148.515449375156f,    -40.1994280057052f},
  { 142.756486344963f,    -38.9563745751136f},
  { 137.107035779414f,    -37.4073679693909f},
  { 131.668796653869f,    -35.5879507715174f},
  { 126.521286852914f,    -33.5502168403183f},
  { 121.717474411461f,    -31.3560642989362f},
  { 117.284527656332f,    -29.0697052196415f},
  { 113.228022928220f,    -26.7511554968076f},
  { 157.843706680329f,    -37.6113523273991f},
  { 152.454805836662f,    -36.8059220640418f},
  { 146.991154997501f,    -35.7174705636480f},
  { 141.550822966496f,    -34.3505184589462f},
  { 136.229420262270f,    -32.7278967671309f},
  { 131.110487738025f,    -30.8889289128016f},
  { 126.258775072393f,    -28.8848052197659f},
  { 121.717474411461f,    -26.7723551183935f},
  { 117.509086510999f,    -24.6077737779805f},
  { 155.966759023688f,    -33.3837304845798f},
  { 150.814582370182f,    -32.4094431897058f},
  { 145.621214884018f,    -31.1884471801445f},
  { 140.471126353159f,    -29.7340780099319f},
  { 135.444675363357f,    -28.0746536530332f},
  { 130.611019772516f,    -26.2507589510491f},
  { 126.023416158135f,    -24.3105938788995f},
  { 121.717474411461f,    -22.3045403104627f},
  { 154.254523323956f,    -29.1039569959989f},
  { 149.327327462644f,    -27.9936984541352f},
  { 144.384666482454f,    -26.6762163700866f},
  { 139.499294987580f,    -25.1712621719368f},
  { 134.739053241081f,    -23.5100597759421f},
  { 130.161621667065f,    -21.7321521847258f},
  { 125.811211291902f,    -19.8810951781929f},
  { 152.688919289658f,    -24.8308196085475f},
  { 147.974345658787f,    -23.6196557310574f},
  { 143.263932237314f,    -22.2418637966772f},
  { 138.620417941669f,    -20.7211331459400f},
  { 134.101346897174f,    -19.0892233146767f},
  { 129.755171678152f,    -17.3827856796100f},
  { 151.253947461888f,    -20.6206896955149f},
  { 146.739561891313f,    -19.3435614209637f},
  { 142.244218291299f,    -17.9394582422071f},
  { 137.822127741731f,    -16.4337077246310f},
  { 133.522338790263f,    -14.8566700706480f},
  { 149.935506539929f,    -16.5238653501529f},
  { 145.609145696363f,    -15.2134343179303f},
  { 141.313021975992f,    -13.8129800206087f},
  { 137.094097861957f,    -12.3475590924140f},
  { 148.721195693252f,    -12.5818702829736f},
  { 144.571179651776f,    -11.2668552009329f},
  { 140.459730649612f,    -9.89479703232183f},
  { 147.600119374241f,    -8.82602735039754f},
  { 143.615372719585f,    -7.53020899059060f},
  { 146.562704124137f,    -5.27729717955544f},
  { -170.484043851542f,    56.1966706603438f},
  { -163.839273313454f,    56.5462580056053f},
  { -156.738163678295f,    56.5250600061002f},
  { -149.366869438509f,    56.0699480487420f},
  { -141.959700313972f,    55.1436992467066f},
  { -134.757747876169f,    53.7442816063043f},
  { -127.964542137052f,    51.9064810941727f},
  { -121.717474411461f,    49.6954596895373f},
  { -116.082595375629f,    47.1952759481287f},
  { -111.067169971118f,    44.4965112658956f},
  { -106.639167039584f,    41.6860404488482f},
  { -102.745388178883f,    38.8402116496635f},
  { -99.3247034825061f,    36.0213780600392f},
  { -96.3162168546181f,    33.2770772600221f},
  { -167.437504668780f,    52.9500102616329f},
  { -161.034066590969f,    53.0473569347008f},
  { -154.291030110658f,    52.7767392956372f},
  { -147.377525063160f,    52.0905755468793f},
  { -140.490148907515f,    50.9672098019515f},
  { -133.821161966070f,    49.4169517893457f},
  { -127.528365604089f,    47.4815872025892f},
  { -121.717474411461f,    45.2276448816065f},
  { -116.440141502986f,    42.7360072735219f},
  { -111.703351896392f,    40.0910997872291f},
  { -107.483322113027f,    37.3720700573987f},
  { -103.738656574656f,    34.6470309014331f},
  { -100.420348572588f,    31.9703418211533f},
  { -164.676998008786f,    49.4382883828622f},
  { -158.528252340269f,    49.2880666149180f},
  { -152.132366206357f,    48.7803925292739f},
  { -145.639340201138f,    47.8824607276022f},
  { -139.213267592889f,    46.5865463730390f},
  { -133.008463514688f,    44.9134373033411f},
  { -127.148951742988f,    42.9103372322102f},
  { -121.717474411461f,    40.6439357010638f},
  { -116.755127470617f,    38.1909250447259f},
  { -112.268467211959f,    35.6285964660136f},
  { -108.239630439654f,    33.0274403302419f},
  { -104.636053416868f,    30.4465738857480f},
  { -162.175195764700f,    45.6866114324158f},
  { -156.284528047208f,    45.2992117296112f},
  { -150.219171669478f,    44.5723576414450f},
  { -144.110245340416f,    43.4870777455218f},
  { -138.094549167120f,    42.0478882595460f},
  { -132.296867588416f,    40.2840762101672f},
  { -126.815924784281f,    38.2463751067230f},
  { -121.717474411461f,    36.0f},
  { -117.034709495470f,    33.6160922031027f},
  { -112.773700332310f,    31.1637336664207f},
  { -108.920899457445f,    28.7040129778536f},
  { -159.905665110722f,    41.7296289222711f},
  { -154.269725630114f,    41.1217160940118f},
  { -148.515449375156f,    40.1994280057052f},
  { -142.756486344963f,    38.9563745751136f},
  { -137.107035779414f,    37.4073679693909f},
  { -131.668796653869f,    35.5879507715174f},
  { -126.521286852914f,    33.5502168403183f},
  { -121.717474411461f,    31.3560642989362f},
  { -117.284527656332f,    29.0697052196415f},
  { -113.228022928220f,    26.7511554968076f},
  { -157.843706680329f,    37.6113523273991f},
  { -152.454805836662f,    36.8059220640418f},
  { -146.991154997501f,    35.7174705636480f},
  { -141.550822966496f,    34.3505184589462f},
  { -136.229420262270f,    32.7278967671309f},
  { -131.110487738025f,    30.8889289128016f},
  { -126.258775072393f,    28.8848052197659f},
  { -121.717474411461f,    26.7723551183935f},
  { -117.509086510999f,    24.6077737779805f},
  { -155.966759023688f,    33.3837304845798f},
  { -150.814582370182f,    32.4094431897058f},
  { -145.621214884018f,    31.1884471801445f},
  { -140.471126353159f,    29.7340780099319f},
  { -135.444675363357f,    28.0746536530332f},
  { -130.611019772516f,    26.2507589510491f},
  { -126.023416158135f,    24.3105938788995f},
  { -121.717474411461f,    22.3045403104627f},
  { -154.254523323956f,    29.1039569959989f},
  { -149.327327462644f,    27.9936984541352f},
  { -144.384666482454f,    26.6762163700866f},
  { -139.499294987580f,    25.1712621719368f},
  { -134.739053241081f,    23.5100597759421f},
  { -130.161621667065f,    21.7321521847258f},
  { -125.811211291902f,    19.8810951781929f},
  { -152.688919289658f,    24.8308196085475f},
  { -147.974345658787f,    23.6196557310574f},
  { -143.263932237314f,    22.2418637966772f},
  { -138.620417941669f,    20.7211331459400f},
  { -134.101346897174f,    19.0892233146767f},
  { -129.755171678152f,    17.3827856796100f},
  { -151.253947461888f,    20.6206896955149f},
  { -146.739561891313f,    19.3435614209637f},
  { -142.244218291299f,    17.9394582422071f},
  { -137.822127741731f,    16.4337077246310f},
  { -133.522338790263f,    14.8566700706480f},
  { -149.935506539929f,    16.5238653501529f},
  { -145.609145696363f,    15.2134343179303f},
  { -141.313021975992f,    13.8129800206087f},
  { -137.094097861957f,    12.3475590924140f},
  { -148.721195693252f,    12.5818702829736f},
  { -144.571179651776f,    11.2668552009329f},
  { -140.459730649612f,    9.89479703232183f},
  { -147.600119374241f,    8.82602735039754f},
  { -143.615372719585f,    7.53020899059060f},
  { -146.562704124137f,    5.27729717955544f},
  { -170.484043851542f,    -56.1966706603438f},
  { -167.437504668780f,    -52.9500102616329f},
  { -164.676998008786f,    -49.4382883828622f},
  { -162.175195764700f,    -45.6866114324158f},
  { -159.905665110722f,    -41.7296289222711f},
  { -157.843706680329f,    -37.6113523273991f},
  { -155.966759023688f,    -33.3837304845798f},
  { -154.254523323956f,    -29.1039569959989f},
  { -152.688919289658f,    -24.8308196085475f},
  { -151.253947461888f,    -20.6206896955149f},
  { -149.935506539929f,    -16.5238653501529f},
  { -148.721195693252f,    -12.5818702829736f},
  { -147.600119374241f,    -8.82602735039754f},
  { -146.562704124137f,    -5.27729717955544f},
  { -163.839273313454f,    -56.5462580056053f},
  { -161.034066590969f,    -53.0473569347008f},
  { -158.528252340269f,    -49.2880666149180f},
  { -156.284528047208f,    -45.2992117296112f},
  { -154.269725630114f,    -41.1217160940117f},
  { -152.454805836662f,    -36.8059220640418f},
  { -150.814582370182f,    -32.4094431897058f},
  { -149.327327462644f,    -27.9936984541352f},
  { -147.974345658787f,    -23.6196557310574f},
  { -146.739561891313f,    -19.3435614209637f},
  { -145.609145696363f,    -15.2134343179302f},
  { -144.571179651776f,    -11.2668552009329f},
  { -143.615372719585f,    -7.53020899059061f},
  { -156.738163678295f,    -56.5250600061002f},
  { -154.291030110658f,    -52.7767392956372f},
  { -152.132366206357f,    -48.7803925292739f},
  { -150.219171669478f,    -44.5723576414450f},
  { -148.515449375156f,    -40.1994280057052f},
  { -146.991154997501f,    -35.7174705636480f},
  { -145.621214884018f,    -31.1884471801445f},
  { -144.384666482454f,    -26.6762163700866f},
  { -143.263932237314f,    -22.2418637966772f},
  { -142.244218291299f,    -17.9394582422070f},
  { -141.313021975992f,    -13.8129800206087f},
  { -140.459730649612f,    -9.89479703232183f},
  { -149.366869438509f,    -56.0699480487420f},
  { -147.377525063160f,    -52.0905755468793f},
  { -145.639340201138f,    -47.8824607276022f},
  { -144.110245340416f,    -43.4870777455218f},
  { -142.756486344963f,    -38.9563745751136f},
  { -141.550822966496f,    -34.3505184589462f},
  { -140.471126353159f,    -29.7340780099319f},
  { -139.499294987580f,    -25.1712621719368f},
  { -138.620417941669f,    -20.7211331459400f},
  { -137.822127741731f,    -16.4337077246310f},
  { -137.094097861957f,    -12.3475590924140f},
  { -141.959700313972f,    -55.1436992467066f},
  { -140.490148907515f,    -50.9672098019515f},
  { -139.213267592889f,    -46.5865463730390f},
  { -138.094549167120f,    -42.0478882595460f},
  { -137.107035779414f,    -37.4073679693909f},
  { -136.229420262270f,    -32.7278967671309f},
  { -135.444675363357f,    -28.0746536530332f},
  { -134.739053241081f,    -23.5100597759421f},
  { -134.101346897174f,    -19.0892233146767f},
  { -133.522338790263f,    -14.8566700706480f},
  { -134.757747876169f,    -53.7442816063043f},
  { -133.821161966070f,    -49.4169517893457f},
  { -133.008463514688f,    -44.9134373033411f},
  { -132.296867588416f,    -40.2840762101672f},
  { -131.668796653869f,    -35.5879507715174f},
  { -131.110487738025f,    -30.8889289128016f},
  { -130.611019772516f,    -26.2507589510491f},
  { -130.161621667065f,    -21.7321521847258f},
  { -129.755171678152f,    -17.3827856796100f},
  { -127.964542137052f,    -51.9064810941727f},
  { -127.528365604089f,    -47.4815872025892f},
  { -127.148951742988f,    -42.9103372322102f},
  { -126.815924784281f,    -38.2463751067230f},
  { -126.521286852914f,    -33.5502168403183f},
  { -126.258775072393f,    -28.8848052197659f},
  { -126.023416158135f,    -24.3105938788995f},
  { -125.811211291902f,    -19.8810951781929f},
  { -121.717474411461f,    -49.6954596895373f},
  { -121.717474411461f,    -45.2276448816065f},
  { -121.717474411461f,    -40.6439357010638f},
  { -121.717474411461f,    -36.0f},
  { -121.717474411461f,    -31.3560642989362f},
  { -121.717474411461f,    -26.7723551183935f},
  { -121.717474411461f,    -22.3045403104628f},
  { -116.082595375629f,    -47.1952759481287f},
  { -116.440141502986f,    -42.7360072735219f},
  { -116.755127470617f,    -38.1909250447259f},
  { -117.034709495470f,    -33.6160922031027f},
  { -117.284527656332f,    -29.0697052196415f},
  { -117.509086510999f,    -24.6077737779805f},
  { -111.067169971118f,    -44.4965112658956f},
  { -111.703351896392f,    -40.0910997872291f},
  { -112.268467211959f,    -35.6285964660136f},
  { -112.773700332310f,    -31.1637336664207f},
  { -113.228022928220f,    -26.7511554968076f},
  { -106.639167039584f,    -41.6860404488482f},
  { -107.483322113027f,    -37.3720700573987f},
  { -108.239630439654f,    -33.0274403302419f},
  { -108.920899457445f,    -28.7040129778536f},
  { -102.745388178883f,    -38.8402116496635f},
  { -103.738656574656f,    -34.6470309014331f},
  { -104.636053416868f,    -30.4465738857480f},
  { -99.3247034825061f,    -36.0213780600392f},
  { -100.420348572588f,    -31.9703418211533f},
  { -96.3162168546181f,    -33.2770772600221f}};

/* 3LD geosphere 'oct' [0 0] (6 points) */
const float __geosphere_oct_0_0_dirs_deg[6][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f}};
  
/* 3LD geosphere 'oct' [1 0] (14 points) */
const float __geosphere_oct_1_0_dirs_deg[14][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 45.0000000000000f,    35.2643896827547f},
  { 135.0f,    35.2643896827547f},
  { -135.0f,    35.2643896827547f},
  { -45.0000000000000f,    35.2643896827547f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 135.0f,    -35.2643896827547f},
  { -135.0f,    -35.2643896827547f},
  { -45.0000000000000f,    -35.2643896827547f}};

/* 3LD geosphere 'oct' [2 0] (18 points) */
const float __geosphere_oct_2_0_dirs_deg[18][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    45.0f},
  { 45.0f,    0.0f},
  { 90.0f,    45.0f},
  { 135.0f,    0.0f},
  { 180.0f,    45.0f},
  { -135.0f,    0.0f},
  { -90.0f,    45.0f},
  { -45.0f,    0.0f},
  { 0.0f,    -45.0f},
  { 90.0f,    -45.0f},
  { 180.0f,    -45.0f},
  { -90.0f,    -45.0f}};

/* 3LD geosphere 'oct' [3 0] (38 points) */
const float __geosphere_oct_3_0_dirs_deg[38][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    26.5650511770780f},
  { 0.0f,    63.4349488229220f},
  { 26.5650511770780f,    0.0f},
  { 45.0000000000000f,    35.2643896827547f},
  { 90.0f,    63.4349488229220f},
  { 63.4349488229220f,    0.0f},
  { 90.0f,    26.5650511770780f},
  { 116.565051177078f,    0.0f},
  { 135.0f,    35.2643896827547f},
  { 180.0f,    63.4349488229220f},
  { 153.434948822922f,    0.0f},
  { 180.0f,    26.5650511770780f},
  { -153.434948822922f,    0.0f},
  { -135.0f,    35.2643896827547f},
  { -90.0f,    63.4349488229220f},
  { -116.565051177078f,    0.0f},
  { -90.0f,    26.5650511770780f},
  { -63.4349488229220f,    0.0f},
  { -45.0000000000000f,    35.2643896827547f},
  { -26.5650511770780f,    0.0f},
  { 0.0f,    -26.5650511770780f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 90.0f,    -26.5650511770780f},
  { 0.0f,    -63.4349488229220f},
  { 90.0f,    -63.4349488229220f},
  { 135.0f,    -35.2643896827547f},
  { 180.0f,    -26.5650511770780f},
  { 180.0f,    -63.4349488229220f},
  { -135.0f,    -35.2643896827547f},
  { -90.0f,    -26.5650511770780f},
  { -90.0f,    -63.4349488229220f},
  { -45.0000000000000f,    -35.2643896827547f}};

/* 3LD geosphere 'oct' [4 0] (66 points) */
const float __geosphere_oct_4_0_dirs_deg[66][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    18.4349488229220f},
  { 0.0f,    45.0f},
  { 0.0f,    71.5650511770780f},
  { 18.4349488229220f,    0.0f},
  { 26.5650511770780f,    24.0948425521107f},
  { 45.0f,    54.7356103172453f},
  { 90.0f,    71.5650511770780f},
  { 45.0f,    0.0f},
  { 63.4349488229220f,    24.0948425521107f},
  { 90.0f,    45.0f},
  { 71.5650511770780f,    0.0f},
  { 90.0f,    18.4349488229220f},
  { 108.434948822922f,    0.0f},
  { 116.565051177078f,    24.0948425521107f},
  { 135.0f,    54.7356103172453f},
  { 180.0f,    71.5650511770780f},
  { 135.0f,    0.0f},
  { 153.434948822922f,    24.0948425521107f},
  { 180.0f,    45.0f},
  { 161.565051177078f,    0.0f},
  { 180.0f,    18.4349488229220f},
  { -161.565051177078f,    0.0f},
  { -153.434948822922f,    24.0948425521107f},
  { -135.0f,    54.7356103172453f},
  { -90.0f,    71.5650511770780f},
  { -135.0f,    0.0f},
  { -116.565051177078f,    24.0948425521107f},
  { -90.0f,    45.0f},
  { -108.434948822922f,    0.0f},
  { -90.0f,    18.4349488229220f},
  { -71.5650511770780f,    0.0f},
  { -63.4349488229220f,    24.0948425521107f},
  { -45.0f,    54.7356103172453f},
  { -45.0f,    0.0f},
  { -26.5650511770780f,    24.0948425521107f},
  { -18.4349488229220f,    0.0f},
  { 0.0f,    -18.4349488229220f},
  { 26.5650511770780f,    -24.0948425521107f},
  { 63.4349488229220f,    -24.0948425521107f},
  { 90.0f,    -18.4349488229220f},
  { 0.0f,    -45.0f},
  { 45.0f,    -54.7356103172453f},
  { 90.0f,    -45.0f},
  { 0.0f,    -71.5650511770780f},
  { 90.0f,    -71.5650511770780f},
  { 116.565051177078f,    -24.0948425521107f},
  { 153.434948822922f,    -24.0948425521107f},
  { 180.0f,    -18.4349488229220f},
  { 135.0f,    -54.7356103172453f},
  { 180.0f,    -45.0f},
  { 180.0f,    -71.5650511770780f},
  { -153.434948822922f,    -24.0948425521107f},
  { -116.565051177078f,    -24.0948425521107f},
  { -90.0f,    -18.4349488229220f},
  { -135.0f,    -54.7356103172453f},
  { -90.0f,    -45.0f},
  { -90.0f,    -71.5650511770780f},
  { -63.4349488229220f,    -24.0948425521107f},
  { -26.5650511770780f,    -24.0948425521107f},
  { -45.0f,    -54.7356103172453f}};

/* 3LD geosphere 'oct' [5 0] (102 points) */
const float __geosphere_oct_5_0_dirs_deg[102][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    14.0362434679265f},
  { 0.0f,    33.6900675259798f},
  { 0.0f,    56.3099324740202f},
  { 0.0f,    75.9637565320735f},
  { 14.0362434679265f,    0.0f},
  { 18.4349488229220f,    17.5484006137923f},
  { 26.5650511770780f,    41.8103148957786f},
  { 45.0f,    64.7605981793211f},
  { 90.0f,    75.9637565320735f},
  { 33.6900675259798f,    0.0f},
  { 45.0f,    19.4712206344907f},
  { 63.4349488229220f,    41.8103148957786f},
  { 90.0f,    56.3099324740202f},
  { 56.3099324740202f,    0.0f},
  { 71.5650511770780f,    17.5484006137923f},
  { 90.0f,    33.6900675259798f},
  { 75.9637565320735f,    0.0f},
  { 90.0f,    14.0362434679265f},
  { 104.036243467927f,    0.0f},
  { 108.434948822922f,    17.5484006137923f},
  { 116.565051177078f,    41.8103148957786f},
  { 135.0f,    64.7605981793211f},
  { 180.0f,    75.9637565320735f},
  { 123.690067525980f,    0.0f},
  { 135.0f,    19.4712206344907f},
  { 153.434948822922f,    41.8103148957786f},
  { 180.0f,    56.3099324740202f},
  { 146.309932474020f,    0.0f},
  { 161.565051177078f,    17.5484006137923f},
  { 180.0f,    33.6900675259798f},
  { 165.963756532074f,    0.0f},
  { 180.0f,    14.0362434679265f},
  { -165.963756532074f,    0.0f},
  { -161.565051177078f,    17.5484006137923f},
  { -153.434948822922f,    41.8103148957786f},
  { -135.0f,    64.7605981793211f},
  { -90.0f,    75.9637565320735f},
  { -146.309932474020f,    0.0f},
  { -135.0f,    19.4712206344907f},
  { -116.565051177078f,    41.8103148957786f},
  { -90.0f,    56.3099324740202f},
  { -123.690067525980f,    0.0f},
  { -108.434948822922f,    17.5484006137923f},
  { -90.0f,    33.6900675259798f},
  { -104.036243467927f,    0.0f},
  { -90.0f,    14.0362434679265f},
  { -75.9637565320735f,    0.0f},
  { -71.5650511770780f,    17.5484006137923f},
  { -63.4349488229220f,    41.8103148957786f},
  { -45.0000000000000f,    64.7605981793211f},
  { -56.3099324740202f,    0.0f},
  { -45.0f,    19.4712206344907f},
  { -26.5650511770780f,    41.8103148957786f},
  { -33.6900675259798f,    0.0f},
  { -18.4349488229220f,    17.5484006137923f},
  { -14.0362434679265f,    0.0f},
  { 0.0f,    -14.0362434679265f},
  { 18.4349488229220f,    -17.5484006137923f},
  { 45.0f,    -19.4712206344907f},
  { 71.5650511770780f,    -17.5484006137923f},
  { 90.0f,    -14.0362434679265f},
  { 0.0f,    -33.6900675259798f},
  { 26.5650511770780f,    -41.8103148957786f},
  { 63.4349488229220f,    -41.8103148957786f},
  { 90.0f,    -33.6900675259798f},
  { 0.0f,    -56.3099324740202f},
  { 45.0f,    -64.7605981793211f},
  { 90.0f,    -56.3099324740202f},
  { 0.0f,    -75.9637565320735f},
  { 90.0f,    -75.9637565320735f},
  { 108.434948822922f,    -17.5484006137923f},
  { 135.0f,    -19.4712206344907f},
  { 161.565051177078f,    -17.5484006137923f},
  { 180.0f,    -14.0362434679265f},
  { 116.565051177078f,    -41.8103148957786f},
  { 153.434948822922f,    -41.8103148957786f},
  { 180.0f,    -33.6900675259798f},
  { 135.0f,    -64.7605981793211f},
  { 180.0f,    -56.3099324740202f},
  { 180.0f,    -75.9637565320735f},
  { -161.565051177078f,    -17.5484006137923f},
  { -135.0f,    -19.4712206344907f},
  { -108.434948822922f,    -17.5484006137923f},
  { -90.0f,    -14.0362434679265f},
  { -153.434948822922f,    -41.8103148957786f},
  { -116.565051177078f,    -41.8103148957786f},
  { -90.0f,    -33.6900675259798f},
  { -135.0f,    -64.7605981793211f},
  { -90.0f,    -56.3099324740202f},
  { -90.0f,    -75.9637565320735f},
  { -71.5650511770780f,    -17.5484006137923f},
  { -45.0f,    -19.4712206344907f},
  { -18.4349488229220f,    -17.5484006137923f},
  { -63.4349488229220f,    -41.8103148957786f},
  { -26.5650511770780f,    -41.8103148957786f},
  { -45.0f,    -64.7605981793211f}};

/* 3LD geosphere 'oct' [6 0] (146 points) */
const float __geosphere_oct_6_0_dirs_deg[146][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    11.3099324740202f},
  { 0.0f,    26.5650511770780f},
  { 0.0f,    45.0f},
  { 0.0f,    63.4349488229220f},
  { 0.0f,    78.6900675259798f},
  { 11.3099324740202f,    0.0f},
  { 14.0362434679265f,    13.6330222253664f},
  { 18.4349488229220f,    32.3115332374239f},
  { 26.5650511770780f,    53.3007747995101f},
  { 45.0000000000000f,    70.5287793655093f},
  { 90.0f,    78.6900675259798f},
  { 26.5650511770780f,    0.0f},
  { 33.6900675259798f,    15.5013595669370f},
  { 45.0000000000000f,    35.2643896827547f},
  { 63.4349488229220f,    53.3007747995101f},
  { 90.0f,    63.4349488229220f},
  { 45.0f,    0.0f},
  { 56.3099324740202f,    15.5013595669370f},
  { 71.5650511770780f,    32.3115332374238f},
  { 90.0f,    45.0f},
  { 63.4349488229220f,    0.0f},
  { 75.9637565320735f,    13.6330222253664f},
  { 90.0f,    26.5650511770780f},
  { 78.6900675259798f,    0.0f},
  { 90.0f,    11.3099324740202f},
  { 101.309932474020f,    0.0f},
  { 104.036243467927f,    13.6330222253664f},
  { 108.434948822922f,    32.3115332374239f},
  { 116.565051177078f,    53.3007747995101f},
  { 135.0f,    70.5287793655093f},
  { 180.0f,    78.6900675259798f},
  { 116.565051177078f,    0.0f},
  { 123.690067525980f,    15.5013595669370f},
  { 135.0f,    35.2643896827547f},
  { 153.434948822922f,    53.3007747995101f},
  { 180.0f,    63.4349488229220f},
  { 135.0f,    0.0f},
  { 146.309932474020f,    15.5013595669370f},
  { 161.565051177078f,    32.3115332374238f},
  { 180.0f,    45.0f},
  { 153.434948822922f,    0.0f},
  { 165.963756532074f,    13.6330222253664f},
  { 180.0f,    26.5650511770780f},
  { 168.690067525980f,    0.0f},
  { 180.0f,    11.3099324740202f},
  { -168.690067525980f,    0.0f},
  { -165.963756532074f,    13.6330222253664f},
  { -161.565051177078f,    32.3115332374239f},
  { -153.434948822922f,    53.3007747995101f},
  { -135.0f,    70.5287793655093f},
  { -90.0f,    78.6900675259798f},
  { -153.434948822922f,    0.0f},
  { -146.309932474020f,    15.5013595669370f},
  { -135.0f,    35.2643896827547f},
  { -116.565051177078f,    53.3007747995101f},
  { -90.0f,    63.4349488229220f},
  { -135.0f,    0.0f},
  { -123.690067525980f,    15.5013595669370f},
  { -108.434948822922f,    32.3115332374238f},
  { -90.0f,    45.0f},
  { -116.565051177078f,    0.0f},
  { -104.036243467927f,    13.6330222253664f},
  { -90.0f,    26.5650511770780f},
  { -101.309932474020f,    0.0f},
  { -90.0f,    11.3099324740202f},
  { -78.6900675259798f,    0.0f},
  { -75.9637565320735f,    13.6330222253664f},
  { -71.5650511770780f,    32.3115332374239f},
  { -63.4349488229220f,    53.3007747995101f},
  { -45.0000000000000f,    70.5287793655093f},
  { -63.4349488229220f,    0.0f},
  { -56.3099324740202f,    15.5013595669370f},
  { -45.0000000000000f,    35.2643896827547f},
  { -26.5650511770780f,    53.3007747995101f},
  { -45.0f,    0.0f},
  { -33.6900675259798f,    15.5013595669370f},
  { -18.4349488229220f,    32.3115332374238f},
  { -26.5650511770780f,    0.0f},
  { -14.0362434679265f,    13.6330222253664f},
  { -11.3099324740202f,    0.0f},
  { 0.0f,    -11.3099324740202f},
  { 14.0362434679265f,    -13.6330222253664f},
  { 33.6900675259798f,    -15.5013595669370f},
  { 56.3099324740202f,    -15.5013595669370f},
  { 75.9637565320735f,    -13.6330222253664f},
  { 90.0f,    -11.3099324740202f},
  { 0.0f,    -26.5650511770780f},
  { 18.4349488229220f,    -32.3115332374238f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 71.5650511770780f,    -32.3115332374238f},
  { 90.0f,    -26.5650511770780f},
  { 0.0f,    -45.0f},
  { 26.5650511770780f,    -53.3007747995101f},
  { 63.4349488229220f,    -53.3007747995101f},
  { 90.0f,    -45.0f},
  { 0.0f,    -63.4349488229220f},
  { 45.0000000000000f,    -70.5287793655093f},
  { 90.0f,    -63.4349488229220f},
  { 0.0f,    -78.6900675259798f},
  { 90.0f,    -78.6900675259798f},
  { 104.036243467927f,    -13.6330222253664f},
  { 123.690067525980f,    -15.5013595669370f},
  { 146.309932474020f,    -15.5013595669370f},
  { 165.963756532074f,    -13.6330222253664f},
  { 180.0f,    -11.3099324740202f},
  { 108.434948822922f,    -32.3115332374238f},
  { 135.0f,    -35.2643896827547f},
  { 161.565051177078f,    -32.3115332374238f},
  { 180.0f,    -26.5650511770780f},
  { 116.565051177078f,    -53.3007747995101f},
  { 153.434948822922f,    -53.3007747995101f},
  { 180.0f,    -45.0f},
  { 135.0f,    -70.5287793655093f},
  { 180.0f,    -63.4349488229220f},
  { 180.0f,    -78.6900675259798f},
  { -165.963756532074f,    -13.6330222253664f},
  { -146.309932474020f,    -15.5013595669370f},
  { -123.690067525980f,    -15.5013595669370f},
  { -104.036243467927f,    -13.6330222253664f},
  { -90.0f,    -11.3099324740202f},
  { -161.565051177078f,    -32.3115332374238f},
  { -135.0f,    -35.2643896827547f},
  { -108.434948822922f,    -32.3115332374238f},
  { -90.0f,    -26.5650511770780f},
  { -153.434948822922f,    -53.3007747995101f},
  { -116.565051177078f,    -53.3007747995101f},
  { -90.0f,    -45.0f},
  { -135.0f,    -70.5287793655093f},
  { -90.0f,    -63.4349488229220f},
  { -90.0f,    -78.6900675259798f},
  { -75.9637565320735f,    -13.6330222253664f},
  { -56.3099324740202f,    -15.5013595669370f},
  { -33.6900675259798f,    -15.5013595669370f},
  { -14.0362434679265f,    -13.6330222253664f},
  { -71.5650511770780f,    -32.3115332374238f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -18.4349488229220f,    -32.3115332374238f},
  { -63.4349488229220f,    -53.3007747995101f},
  { -26.5650511770780f,    -53.3007747995101f},
  { -45.0000000000000f,    -70.5287793655093f}};

/* 3LD geosphere 'oct' [7 0] (198 points) */
const float __geosphere_oct_7_0_dirs_deg[198][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    9.46232220802562f},
  { 0.0f,    21.8014094863518f},
  { 0.0f,    36.8698976458440f},
  { 0.0f,    53.1301023541560f},
  { 0.0f,    68.1985905136482f},
  { 0.0f,    80.5376777919744f},
  { 9.46232220802562f,    0.0f},
  { 11.3099324740202f,    11.0958032831364f},
  { 14.0362434679265f,    25.8766900608275f},
  { 18.4349488229220f,    43.4915193450920f},
  { 26.5650511770780f,    60.7940677526006f},
  { 45.0000000000000f,    74.2068309517361f},
  { 90.0f,    80.5376777919744f},
  { 21.8014094863518f,    0.0f},
  { 26.5650511770780f,    12.6043826483792f},
  { 33.6900675259798f,    29.0171406246015f},
  { 45.0000000000000f,    46.6861433417170f},
  { 63.4349488229220f,    60.7940677526006f},
  { 90.0f,    68.1985905136482f},
  { 36.8698976458440f,    0.0f},
  { 45.0f,    13.2626760083048f},
  { 56.3099324740202f,    29.0171406246015f},
  { 71.5650511770780f,    43.4915193450920f},
  { 90.0f,    53.1301023541560f},
  { 53.1301023541560f,    0.0f},
  { 63.4349488229220f,    12.6043826483792f},
  { 75.9637565320735f,    25.8766900608275f},
  { 90.0f,    36.8698976458440f},
  { 68.1985905136482f,    0.0f},
  { 78.6900675259798f,    11.0958032831364f},
  { 90.0f,    21.8014094863518f},
  { 80.5376777919744f,    0.0f},
  { 90.0f,    9.46232220802562f},
  { 99.4623222080256f,    0.0f},
  { 101.309932474020f,    11.0958032831364f},
  { 104.036243467927f,    25.8766900608275f},
  { 108.434948822922f,    43.4915193450920f},
  { 116.565051177078f,    60.7940677526006f},
  { 135.000000000000f,    74.2068309517361f},
  { 180.0f,    80.5376777919744f},
  { 111.801409486352f,    0.0f},
  { 116.565051177078f,    12.6043826483792f},
  { 123.690067525980f,    29.0171406246015f},
  { 135.0f,    46.6861433417170f},
  { 153.434948822922f,    60.7940677526006f},
  { 180.0f,    68.1985905136482f},
  { 126.869897645844f,    0.0f},
  { 135.0f,    13.2626760083048f},
  { 146.309932474020f,    29.0171406246015f},
  { 161.565051177078f,    43.4915193450920f},
  { 180.0f,    53.1301023541560f},
  { 143.130102354156f,    0.0f},
  { 153.434948822922f,    12.6043826483792f},
  { 165.963756532074f,    25.8766900608275f},
  { 180.0f,    36.8698976458440f},
  { 158.198590513648f,    0.0f},
  { 168.690067525980f,    11.0958032831364f},
  { 180.0f,    21.8014094863518f},
  { 170.537677791974f,    0.0f},
  { 180.0f,    9.46232220802562f},
  { -170.537677791974f,    0.0f},
  { -168.690067525980f,    11.0958032831364f},
  { -165.963756532074f,    25.8766900608275f},
  { -161.565051177078f,    43.4915193450920f},
  { -153.434948822922f,    60.7940677526006f},
  { -135.0f,    74.2068309517361f},
  { -90.0f,    80.5376777919744f},
  { -158.198590513648f,    0.0f},
  { -153.434948822922f,    12.6043826483792f},
  { -146.309932474020f,    29.0171406246015f},
  { -135.0f,    46.6861433417170f},
  { -116.565051177078f,    60.7940677526006f},
  { -90.0f,    68.1985905136482f},
  { -143.130102354156f,    0.0f},
  { -135.0f,    13.2626760083048f},
  { -123.690067525980f,    29.0171406246015f},
  { -108.434948822922f,    43.4915193450920f},
  { -90.0f,    53.1301023541560f},
  { -126.869897645844f,    0.0f},
  { -116.565051177078f,    12.6043826483792f},
  { -104.036243467927f,    25.8766900608275f},
  { -90.0f,    36.8698976458440f},
  { -111.801409486352f,    0.0f},
  { -101.309932474020f,    11.0958032831364f},
  { -90.0f,    21.8014094863518f},
  { -99.4623222080256f,    0.0f},
  { -90.0f,    9.46232220802562f},
  { -80.5376777919744f,    0.0f},
  { -78.6900675259798f,    11.0958032831364f},
  { -75.9637565320735f,    25.8766900608275f},
  { -71.5650511770780f,    43.4915193450920f},
  { -63.4349488229220f,    60.7940677526006f},
  { -45.0000000000000f,    74.2068309517361f},
  { -68.1985905136482f,    0.0f},
  { -63.4349488229220f,    12.6043826483792f},
  { -56.3099324740202f,    29.0171406246015f},
  { -45.0000000000000f,    46.6861433417170f},
  { -26.5650511770780f,    60.7940677526006f},
  { -53.1301023541560f,    0.0f},
  { -45.0f,    13.2626760083048f},
  { -33.6900675259798f,    29.0171406246015f},
  { -18.4349488229220f,    43.4915193450920f},
  { -36.8698976458440f,    0.0f},
  { -26.5650511770780f,    12.6043826483792f},
  { -14.0362434679265f,    25.8766900608275f},
  { -21.8014094863518f,    0.0f},
  { -11.3099324740202f,    11.0958032831364f},
  { -9.46232220802562f,    0.0f},
  { 0.0f,    -9.46232220802562f},
  { 11.3099324740202f,    -11.0958032831364f},
  { 26.5650511770780f,    -12.6043826483792f},
  { 45.0f,    -13.2626760083048f},
  { 63.4349488229220f,    -12.6043826483792f},
  { 78.6900675259798f,    -11.0958032831364f},
  { 90.0f,    -9.46232220802562f},
  { 0.0f,    -21.8014094863518f},
  { 14.0362434679265f,    -25.8766900608275f},
  { 33.6900675259798f,    -29.0171406246015f},
  { 56.3099324740202f,    -29.0171406246015f},
  { 75.9637565320735f,    -25.8766900608275f},
  { 90.0f,    -21.8014094863518f},
  { 0.0f,    -36.8698976458440f},
  { 18.4349488229220f,    -43.4915193450920f},
  { 45.0f,    -46.6861433417170f},
  { 71.5650511770780f,    -43.4915193450920f},
  { 90.0f,    -36.8698976458440f},
  { 0.0f,    -53.1301023541560f},
  { 26.5650511770780f,    -60.7940677526006f},
  { 63.4349488229220f,    -60.7940677526006f},
  { 90.0f,    -53.1301023541560f},
  { 0.0f,    -68.1985905136482f},
  { 45.0f,    -74.2068309517360f},
  { 90.0f,    -68.1985905136482f},
  { 0.0f,    -80.5376777919744f},
  { 90.0f,    -80.5376777919744f},
  { 101.309932474020f,    -11.0958032831364f},
  { 116.565051177078f,    -12.6043826483792f},
  { 135.0f,    -13.2626760083048f},
  { 153.434948822922f,    -12.6043826483792f},
  { 168.690067525980f,    -11.0958032831364f},
  { 180.0f,    -9.46232220802562f},
  { 104.036243467927f,    -25.8766900608275f},
  { 123.690067525980f,    -29.0171406246015f},
  { 146.309932474020f,    -29.0171406246015f},
  { 165.963756532074f,    -25.8766900608275f},
  { 180.0f,    -21.8014094863518f},
  { 108.434948822922f,    -43.4915193450920f},
  { 135.0f,    -46.6861433417170f},
  { 161.565051177078f,    -43.4915193450920f},
  { 180.0f,    -36.8698976458440f},
  { 116.565051177078f,    -60.7940677526006f},
  { 153.434948822922f,    -60.7940677526006f},
  { 180.0f,    -53.1301023541560f},
  { 135.0f,    -74.2068309517360f},
  { 180.0f,    -68.1985905136482f},
  { 180.0f,    -80.5376777919744f},
  { -168.690067525980f,    -11.0958032831364f},
  { -153.434948822922f,    -12.6043826483792f},
  { -135.0f,    -13.2626760083048f},
  { -116.565051177078f,    -12.6043826483792f},
  { -101.309932474020f,    -11.0958032831364f},
  { -90.0f,    -9.46232220802562f},
  { -165.963756532074f,    -25.8766900608275f},
  { -146.309932474020f,    -29.0171406246015f},
  { -123.690067525980f,    -29.0171406246015f},
  { -104.036243467927f,    -25.8766900608275f},
  { -90.0f,    -21.8014094863518f},
  { -161.565051177078f,    -43.4915193450920f},
  { -135.0f,    -46.6861433417170f},
  { -108.434948822922f,    -43.4915193450920f},
  { -90.0f,    -36.8698976458440f},
  { -153.434948822922f,    -60.7940677526006f},
  { -116.565051177078f,    -60.7940677526006f},
  { -90.0f,    -53.1301023541560f},
  { -135.0f,    -74.2068309517360f},
  { -90.0f,    -68.1985905136482f},
  { -90.0f,    -80.5376777919744f},
  { -78.6900675259798f,    -11.0958032831364f},
  { -63.4349488229220f,    -12.6043826483792f},
  { -45.0f,    -13.2626760083048f},
  { -26.5650511770780f,    -12.6043826483792f},
  { -11.3099324740202f,    -11.0958032831364f},
  { -75.9637565320735f,    -25.8766900608275f},
  { -56.3099324740202f,    -29.0171406246015f},
  { -33.6900675259798f,    -29.0171406246015f},
  { -14.0362434679265f,    -25.8766900608275f},
  { -71.5650511770780f,    -43.4915193450920f},
  { -45.0f,    -46.6861433417170f},
  { -18.4349488229220f,    -43.4915193450920f},
  { -63.4349488229220f,    -60.7940677526006f},
  { -26.5650511770780f,    -60.7940677526006f},
  { -45.0f,    -74.2068309517360f}};

/* 3LD geosphere 'oct' [8 0] (258 points) */
const float __geosphere_oct_8_0_dirs_deg[258][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    8.13010235415598f},
  { 0.0f,    18.4349488229220f},
  { 0.0f,    30.9637565320735f},
  { 0.0f,    45.0f},
  { 0.0f,    59.0362434679265f},
  { 0.0f,    71.5650511770780f},
  { 0.0f,    81.8698976458440f},
  { 8.13010235415598f,    0.0f},
  { 9.46232220802562f,    9.33585903200394f},
  { 11.3099324740202f,    21.4167140330336f},
  { 14.0362434679265f,    36.0398934303039f},
  { 18.4349488229220f,    51.6711818985441f},
  { 26.5650511770780f,    65.9051574478893f},
  { 45.0000000000000f,    76.7373239916952f},
  { 90.0f,    81.8698976458440f},
  { 18.4349488229220f,    0.0f},
  { 21.8014094863518f,    10.5197348906586f},
  { 26.5650511770780f,    24.0948425521107f},
  { 33.6900675259798f,    39.7621591518051f},
  { 45.0f,    54.7356103172453f},
  { 63.4349488229220f,    65.9051574478893f},
  { 90.0f,    71.5650511770780f},
  { 30.9637565320735f,    0.0f},
  { 36.8698976458440f,    11.3099324740202f},
  { 45.0000000000000f,    25.2394018206789f},
  { 56.3099324740202f,    39.7621591518051f},
  { 71.5650511770780f,    51.6711818985441f},
  { 90.0f,    59.0362434679265f},
  { 45.0f,    0.0f},
  { 53.1301023541560f,    11.3099324740202f},
  { 63.4349488229220f,    24.0948425521107f},
  { 75.9637565320735f,    36.0398934303039f},
  { 90.0f,    45.0f},
  { 59.0362434679265f,    0.0f},
  { 68.1985905136482f,    10.5197348906586f},
  { 78.6900675259798f,    21.4167140330336f},
  { 90.0f,    30.9637565320735f},
  { 71.5650511770780f,    0.0f},
  { 80.5376777919744f,    9.33585903200394f},
  { 90.0f,    18.4349488229220f},
  { 81.8698976458440f,    0.0f},
  { 90.0f,    8.13010235415598f},
  { 98.1301023541560f,    0.0f},
  { 99.4623222080256f,    9.33585903200394f},
  { 101.309932474020f,    21.4167140330336f},
  { 104.036243467927f,    36.0398934303039f},
  { 108.434948822922f,    51.6711818985441f},
  { 116.565051177078f,    65.9051574478893f},
  { 135.0f,    76.7373239916952f},
  { 180.0f,    81.8698976458440f},
  { 108.434948822922f,    0.0f},
  { 111.801409486352f,    10.5197348906586f},
  { 116.565051177078f,    24.0948425521107f},
  { 123.690067525980f,    39.7621591518051f},
  { 135.0f,    54.7356103172453f},
  { 153.434948822922f,    65.9051574478893f},
  { 180.0f,    71.5650511770780f},
  { 120.963756532074f,    0.0f},
  { 126.869897645844f,    11.3099324740202f},
  { 135.0f,    25.2394018206789f},
  { 146.309932474020f,    39.7621591518051f},
  { 161.565051177078f,    51.6711818985441f},
  { 180.0f,    59.0362434679265f},
  { 135.0f,    0.0f},
  { 143.130102354156f,    11.3099324740202f},
  { 153.434948822922f,    24.0948425521107f},
  { 165.963756532074f,    36.0398934303039f},
  { 180.0f,    45.0f},
  { 149.036243467926f,    0.0f},
  { 158.198590513648f,    10.5197348906586f},
  { 168.690067525980f,    21.4167140330336f},
  { 180.0f,    30.9637565320735f},
  { 161.565051177078f,    0.0f},
  { 170.537677791974f,    9.33585903200394f},
  { 180.0f,    18.4349488229220f},
  { 171.869897645844f,    0.0f},
  { 180.0f,    8.13010235415598f},
  { -171.869897645844f,    0.0f},
  { -170.537677791974f,    9.33585903200394f},
  { -168.690067525980f,    21.4167140330336f},
  { -165.963756532074f,    36.0398934303039f},
  { -161.565051177078f,    51.6711818985441f},
  { -153.434948822922f,    65.9051574478893f},
  { -135.0f,    76.7373239916952f},
  { -90.0f,    81.8698976458440f},
  { -161.565051177078f,    0.0f},
  { -158.198590513648f,    10.5197348906586f},
  { -153.434948822922f,    24.0948425521107f},
  { -146.309932474020f,    39.7621591518051f},
  { -135.0f,    54.7356103172453f},
  { -116.565051177078f,    65.9051574478893f},
  { -90.0f,    71.5650511770780f},
  { -149.036243467926f,    0.0f},
  { -143.130102354156f,    11.3099324740202f},
  { -135.0f,    25.2394018206789f},
  { -123.690067525980f,    39.7621591518051f},
  { -108.434948822922f,    51.6711818985441f},
  { -90.0f,    59.0362434679265f},
  { -135.0f,    0.0f},
  { -126.869897645844f,    11.3099324740202f},
  { -116.565051177078f,    24.0948425521107f},
  { -104.036243467927f,    36.0398934303039f},
  { -90.0f,    45.0f},
  { -120.963756532074f,    0.0f},
  { -111.801409486352f,    10.5197348906586f},
  { -101.309932474020f,    21.4167140330336f},
  { -90.0f,    30.9637565320735f},
  { -108.434948822922f,    0.0f},
  { -99.4623222080256f,    9.33585903200394f},
  { -90.0f,    18.4349488229220f},
  { -98.1301023541560f,    0.0f},
  { -90.0f,    8.13010235415598f},
  { -81.8698976458440f,    0.0f},
  { -80.5376777919744f,    9.33585903200394f},
  { -78.6900675259798f,    21.4167140330336f},
  { -75.9637565320735f,    36.0398934303039f},
  { -71.5650511770780f,    51.6711818985441f},
  { -63.4349488229220f,    65.9051574478893f},
  { -45.0000000000000f,    76.7373239916952f},
  { -71.5650511770780f,    0.0f},
  { -68.1985905136482f,    10.5197348906586f},
  { -63.4349488229220f,    24.0948425521107f},
  { -56.3099324740202f,    39.7621591518051f},
  { -45.0f,    54.7356103172453f},
  { -26.5650511770780f,    65.9051574478893f},
  { -59.0362434679265f,    0.0f},
  { -53.1301023541560f,    11.3099324740202f},
  { -45.0000000000000f,    25.2394018206789f},
  { -33.6900675259798f,    39.7621591518051f},
  { -18.4349488229220f,    51.6711818985441f},
  { -45.0f,    0.0f},
  { -36.8698976458440f,    11.3099324740202f},
  { -26.5650511770780f,    24.0948425521107f},
  { -14.0362434679265f,    36.0398934303039f},
  { -30.9637565320735f,    0.0f},
  { -21.8014094863518f,    10.5197348906586f},
  { -11.3099324740202f,    21.4167140330336f},
  { -18.4349488229220f,    0.0f},
  { -9.46232220802562f,    9.33585903200394f},
  { -8.13010235415598f,    0.0f},
  { 0.0f,    -8.13010235415598f},
  { 9.46232220802562f,    -9.33585903200394f},
  { 21.8014094863518f,    -10.5197348906586f},
  { 36.8698976458440f,    -11.3099324740202f},
  { 53.1301023541560f,    -11.3099324740202f},
  { 68.1985905136482f,    -10.5197348906586f},
  { 80.5376777919744f,    -9.33585903200394f},
  { 90.0f,    -8.13010235415598f},
  { 0.0f,    -18.4349488229220f},
  { 11.3099324740202f,    -21.4167140330336f},
  { 26.5650511770780f,    -24.0948425521107f},
  { 45.0000000000000f,    -25.2394018206789f},
  { 63.4349488229220f,    -24.0948425521107f},
  { 78.6900675259798f,    -21.4167140330336f},
  { 90.0f,    -18.4349488229220f},
  { 0.0f,    -30.9637565320735f},
  { 14.0362434679265f,    -36.0398934303039f},
  { 33.6900675259798f,    -39.7621591518051f},
  { 56.3099324740202f,    -39.7621591518051f},
  { 75.9637565320735f,    -36.0398934303039f},
  { 90.0f,    -30.9637565320735f},
  { 0.0f,    -45.0f},
  { 18.4349488229220f,    -51.6711818985441f},
  { 45.0f,    -54.7356103172453f},
  { 71.5650511770780f,    -51.6711818985441f},
  { 90.0f,    -45.0f},
  { 0.0f,    -59.0362434679265f},
  { 26.5650511770780f,    -65.9051574478893f},
  { 63.4349488229220f,    -65.9051574478893f},
  { 90.0f,    -59.0362434679265f},
  { 0.0f,    -71.5650511770780f},
  { 45.0000000000000f,    -76.7373239916952f},
  { 90.0f,    -71.5650511770780f},
  { 0.0f,    -81.8698976458440f},
  { 90.0f,    -81.8698976458440f},
  { 99.4623222080256f,    -9.33585903200394f},
  { 111.801409486352f,    -10.5197348906586f},
  { 126.869897645844f,    -11.3099324740202f},
  { 143.130102354156f,    -11.3099324740202f},
  { 158.198590513648f,    -10.5197348906586f},
  { 170.537677791974f,    -9.33585903200394f},
  { 180.0f,    -8.13010235415598f},
  { 101.309932474020f,    -21.4167140330336f},
  { 116.565051177078f,    -24.0948425521107f},
  { 135.0f,    -25.2394018206789f},
  { 153.434948822922f,    -24.0948425521107f},
  { 168.690067525980f,    -21.4167140330336f},
  { 180.0f,    -18.4349488229220f},
  { 104.036243467927f,    -36.0398934303039f},
  { 123.690067525980f,    -39.7621591518051f},
  { 146.309932474020f,    -39.7621591518051f},
  { 165.963756532074f,    -36.0398934303039f},
  { 180.0f,    -30.9637565320735f},
  { 108.434948822922f,    -51.6711818985441f},
  { 135.0f,    -54.7356103172453f},
  { 161.565051177078f,    -51.6711818985441f},
  { 180.0f,    -45.0f},
  { 116.565051177078f,    -65.9051574478893f},
  { 153.434948822922f,    -65.9051574478893f},
  { 180.0f,    -59.0362434679265f},
  { 135.0f,    -76.7373239916952f},
  { 180.0f,    -71.5650511770780f},
  { 180.0f,    -81.8698976458440f},
  { -170.537677791974f,    -9.33585903200394f},
  { -158.198590513648f,    -10.5197348906586f},
  { -143.130102354156f,    -11.3099324740202f},
  { -126.869897645844f,    -11.3099324740202f},
  { -111.801409486352f,    -10.5197348906586f},
  { -99.4623222080256f,    -9.33585903200394f},
  { -90.0f,    -8.13010235415598f},
  { -168.690067525980f,    -21.4167140330336f},
  { -153.434948822922f,    -24.0948425521107f},
  { -135.0f,    -25.2394018206789f},
  { -116.565051177078f,    -24.0948425521107f},
  { -101.309932474020f,    -21.4167140330336f},
  { -90.0f,    -18.4349488229220f},
  { -165.963756532074f,    -36.0398934303039f},
  { -146.309932474020f,    -39.7621591518051f},
  { -123.690067525980f,    -39.7621591518051f},
  { -104.036243467927f,    -36.0398934303039f},
  { -90.0f,    -30.9637565320735f},
  { -161.565051177078f,    -51.6711818985441f},
  { -135.0f,    -54.7356103172453f},
  { -108.434948822922f,    -51.6711818985441f},
  { -90.0f,    -45.0f},
  { -153.434948822922f,    -65.9051574478893f},
  { -116.565051177078f,    -65.9051574478893f},
  { -90.0f,    -59.0362434679265f},
  { -135.0f,    -76.7373239916952f},
  { -90.0f,    -71.5650511770780f},
  { -90.0f,    -81.8698976458440f},
  { -80.5376777919744f,    -9.33585903200394f},
  { -68.1985905136482f,    -10.5197348906586f},
  { -53.1301023541560f,    -11.3099324740202f},
  { -36.8698976458440f,    -11.3099324740202f},
  { -21.8014094863518f,    -10.5197348906586f},
  { -9.46232220802562f,    -9.33585903200394f},
  { -78.6900675259798f,    -21.4167140330336f},
  { -63.4349488229220f,    -24.0948425521107f},
  { -45.0000000000000f,    -25.2394018206789f},
  { -26.5650511770780f,    -24.0948425521107f},
  { -11.3099324740202f,    -21.4167140330336f},
  { -75.9637565320735f,    -36.0398934303039f},
  { -56.3099324740202f,    -39.7621591518051f},
  { -33.6900675259798f,    -39.7621591518051f},
  { -14.0362434679265f,    -36.0398934303039f},
  { -71.5650511770780f,    -51.6711818985441f},
  { -45.0f,    -54.7356103172453f},
  { -18.4349488229220f,    -51.6711818985441f},
  { -63.4349488229220f,    -65.9051574478893f},
  { -26.5650511770780f,    -65.9051574478893f},
  { -45.0000000000000f,    -76.7373239916952f}};

/* 3LD geosphere 'oct' [9 0] (326 points) */
const float __geosphere_oct_9_0_dirs_deg[326][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    7.12501634890180f},
  { 0.0f,    15.9453959009229f},
  { 0.0f,    26.5650511770780f},
  { 0.0f,    38.6598082540901f},
  { 0.0f,    51.3401917459099f},
  { 0.0f,    63.4349488229220f},
  { 0.0f,    74.0546040990772f},
  { 0.0f,    82.8749836510982f},
  { 7.12501634890180f,    0.0f},
  { 8.13010235415598f,    8.04946697552840f},
  { 9.46232220802562f,    18.2007602645333f},
  { 11.3099324740202f,    30.4703594659797f},
  { 14.0362434679265f,    44.1317492120310f},
  { 18.4349488229220f,    57.6884667625762f},
  { 26.5650511770780f,    69.5606824267418f},
  { 45.0000000000000f,    78.5782463410377f},
  { 90.0f,    82.8749836510982f},
  { 15.9453959009229f,    0.0f},
  { 18.4349488229220f,    8.98487693168568f},
  { 21.8014094863518f,    20.3745271887352f},
  { 26.5650511770780f,    33.8545148126205f},
  { 33.6900675259798f,    47.9688862258027f},
  { 45.0f,    60.5037915034336f},
  { 63.4349488229220f,    69.5606824267418f},
  { 90.0f,    74.0546040990772f},
  { 26.5650511770780f,    0.0f},
  { 30.9637565320735f,    9.73147547315763f},
  { 36.8698976458440f,    21.8014094863518f},
  { 45.0000000000000f,    35.2643896827547f},
  { 56.3099324740202f,    47.9688862258027f},
  { 71.5650511770780f,    57.6884667625762f},
  { 90.0000000000000f,    63.4349488229220f},
  { 38.6598082540901f,    0.0f},
  { 45.0f,    10.0249878620757f},
  { 53.1301023541560f,    21.8014094863518f},
  { 63.4349488229220f,    33.8545148126205f},
  { 75.9637565320735f,    44.1317492120310f},
  { 90.0f,    51.3401917459099f},
  { 51.3401917459099f,    0.0f},
  { 59.0362434679265f,    9.73147547315763f},
  { 68.1985905136482f,    20.3745271887352f},
  { 78.6900675259798f,    30.4703594659797f},
  { 90.0f,    38.6598082540901f},
  { 63.4349488229220f,    0.0f},
  { 71.5650511770780f,    8.98487693168568f},
  { 80.5376777919744f,    18.2007602645333f},
  { 90.0f,    26.5650511770780f},
  { 74.0546040990772f,    0.0f},
  { 81.8698976458440f,    8.04946697552840f},
  { 90.0f,    15.9453959009229f},
  { 82.8749836510982f,    0.0f},
  { 90.0f,    7.12501634890180f},
  { 97.1250163489018f,    0.0f},
  { 98.1301023541560f,    8.04946697552840f},
  { 99.4623222080256f,    18.2007602645333f},
  { 101.309932474020f,    30.4703594659797f},
  { 104.036243467927f,    44.1317492120310f},
  { 108.434948822922f,    57.6884667625762f},
  { 116.565051177078f,    69.5606824267418f},
  { 135.0f,    78.5782463410377f},
  { 180.0f,    82.8749836510982f},
  { 105.945395900923f,    0.0f},
  { 108.434948822922f,    8.98487693168568f},
  { 111.801409486352f,    20.3745271887352f},
  { 116.565051177078f,    33.8545148126205f},
  { 123.690067525980f,    47.9688862258027f},
  { 135.0f,    60.5037915034336f},
  { 153.434948822922f,    69.5606824267418f},
  { 180.0f,    74.0546040990772f},
  { 116.565051177078f,    0.0f},
  { 120.963756532074f,    9.73147547315763f},
  { 126.869897645844f,    21.8014094863518f},
  { 135.0f,    35.2643896827547f},
  { 146.309932474020f,    47.9688862258027f},
  { 161.565051177078f,    57.6884667625762f},
  { 180.0f,    63.4349488229220f},
  { 128.659808254090f,    0.0f},
  { 135.0f,    10.0249878620757f},
  { 143.130102354156f,    21.8014094863518f},
  { 153.434948822922f,    33.8545148126205f},
  { 165.963756532074f,    44.1317492120310f},
  { 180.0f,    51.3401917459099f},
  { 141.340191745910f,    0.0f},
  { 149.036243467926f,    9.73147547315763f},
  { 158.198590513648f,    20.3745271887352f},
  { 168.690067525980f,    30.4703594659797f},
  { 180.0f,    38.6598082540901f},
  { 153.434948822922f,    0.0f},
  { 161.565051177078f,    8.98487693168568f},
  { 170.537677791974f,    18.2007602645333f},
  { 180.0f,    26.5650511770780f},
  { 164.054604099077f,    0.0f},
  { 171.869897645844f,    8.04946697552840f},
  { 180.0f,    15.9453959009229f},
  { 172.874983651098f,    0.0f},
  { 180.0f,    7.12501634890180f},
  { -172.874983651098f,    0.0f},
  { -171.869897645844f,    8.04946697552840f},
  { -170.537677791974f,    18.2007602645333f},
  { -168.690067525980f,    30.4703594659797f},
  { -165.963756532074f,    44.1317492120310f},
  { -161.565051177078f,    57.6884667625762f},
  { -153.434948822922f,    69.5606824267418f},
  { -135.000000000000f,    78.5782463410377f},
  { -90.0f,    82.8749836510982f},
  { -164.054604099077f,    0.0f},
  { -161.565051177078f,    8.98487693168568f},
  { -158.198590513648f,    20.3745271887352f},
  { -153.434948822922f,    33.8545148126205f},
  { -146.309932474020f,    47.9688862258027f},
  { -135.0f,    60.5037915034336f},
  { -116.565051177078f,    69.5606824267418f},
  { -90.0f,    74.0546040990772f},
  { -153.434948822922f,    0.0f},
  { -149.036243467926f,    9.73147547315763f},
  { -143.130102354156f,    21.8014094863518f},
  { -135.0f,    35.2643896827547f},
  { -123.690067525980f,    47.9688862258027f},
  { -108.434948822922f,    57.6884667625762f},
  { -90.0000000000000f,    63.4349488229220f},
  { -141.340191745910f,    0.0f},
  { -135.0f,    10.0249878620757f},
  { -126.869897645844f,    21.8014094863518f},
  { -116.565051177078f,    33.8545148126205f},
  { -104.036243467927f,    44.1317492120310f},
  { -90.0f,    51.3401917459099f},
  { -128.659808254090f,    0.0f},
  { -120.963756532074f,    9.73147547315763f},
  { -111.801409486352f,    20.3745271887352f},
  { -101.309932474020f,    30.4703594659797f},
  { -90.0f,    38.6598082540901f},
  { -116.565051177078f,    0.0f},
  { -108.434948822922f,    8.98487693168568f},
  { -99.4623222080256f,    18.2007602645333f},
  { -90.0000000000000f,    26.5650511770780f},
  { -105.945395900923f,    0.0f},
  { -98.1301023541560f,    8.04946697552840f},
  { -90.0f,    15.9453959009229f},
  { -97.1250163489018f,    0.0f},
  { -90.0f,    7.12501634890180f},
  { -82.8749836510982f,    0.0f},
  { -81.8698976458440f,    8.04946697552840f},
  { -80.5376777919744f,    18.2007602645333f},
  { -78.6900675259798f,    30.4703594659797f},
  { -75.9637565320735f,    44.1317492120310f},
  { -71.5650511770780f,    57.6884667625762f},
  { -63.4349488229220f,    69.5606824267418f},
  { -45.0000000000000f,    78.5782463410377f},
  { -74.0546040990772f,    0.0f},
  { -71.5650511770780f,    8.98487693168568f},
  { -68.1985905136482f,    20.3745271887352f},
  { -63.4349488229220f,    33.8545148126205f},
  { -56.3099324740202f,    47.9688862258027f},
  { -45.0f,    60.5037915034336f},
  { -26.5650511770780f,    69.5606824267418f},
  { -63.4349488229220f,    0.0f},
  { -59.0362434679265f,    9.73147547315763f},
  { -53.1301023541560f,    21.8014094863518f},
  { -45.0000000000000f,    35.2643896827547f},
  { -33.6900675259798f,    47.9688862258027f},
  { -18.4349488229220f,    57.6884667625762f},
  { -51.3401917459099f,    0.0f},
  { -45.0000000000000f,    10.0249878620757f},
  { -36.8698976458440f,    21.8014094863518f},
  { -26.5650511770780f,    33.8545148126205f},
  { -14.0362434679265f,    44.1317492120310f},
  { -38.6598082540901f,    0.0f},
  { -30.9637565320735f,    9.73147547315763f},
  { -21.8014094863518f,    20.3745271887352f},
  { -11.3099324740202f,    30.4703594659797f},
  { -26.5650511770780f,    0.0f},
  { -18.4349488229220f,    8.98487693168568f},
  { -9.46232220802562f,    18.2007602645333f},
  { -15.9453959009229f,    0.0f},
  { -8.13010235415598f,    8.04946697552840f},
  { -7.12501634890180f,    0.0f},
  { 0.0f,    -7.12501634890180f},
  { 8.13010235415598f,    -8.04946697552840f},
  { 18.4349488229220f,    -8.98487693168568f},
  { 30.9637565320735f,    -9.73147547315763f},
  { 45.0f,    -10.0249878620757f},
  { 59.0362434679265f,    -9.73147547315763f},
  { 71.5650511770780f,    -8.98487693168568f},
  { 81.8698976458440f,    -8.04946697552840f},
  { 90.0f,    -7.12501634890180f},
  { 0.0f,    -15.9453959009229f},
  { 9.46232220802562f,    -18.2007602645333f},
  { 21.8014094863518f,    -20.3745271887352f},
  { 36.8698976458440f,    -21.8014094863518f},
  { 53.1301023541560f,    -21.8014094863518f},
  { 68.1985905136482f,    -20.3745271887352f},
  { 80.5376777919744f,    -18.2007602645333f},
  { 90.0f,    -15.9453959009229f},
  { 0.0f,    -26.5650511770780f},
  { 11.3099324740202f,    -30.4703594659797f},
  { 26.5650511770780f,    -33.8545148126205f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 63.4349488229220f,    -33.8545148126205f},
  { 78.6900675259798f,    -30.4703594659797f},
  { 90.0f,    -26.5650511770780f},
  { 0.0f,    -38.6598082540901f},
  { 14.0362434679265f,    -44.1317492120310f},
  { 33.6900675259798f,    -47.9688862258027f},
  { 56.3099324740202f,    -47.9688862258027f},
  { 75.9637565320735f,    -44.1317492120310f},
  { 90.0f,    -38.6598082540901f},
  { 0.0f,    -51.3401917459099f},
  { 18.4349488229220f,    -57.6884667625762f},
  { 45.0f,    -60.5037915034336f},
  { 71.5650511770780f,    -57.6884667625762f},
  { 90.0f,    -51.3401917459099f},
  { 0.0f,    -63.4349488229220f},
  { 26.5650511770780f,    -69.5606824267418f},
  { 63.4349488229220f,    -69.5606824267418f},
  { 90.0f,    -63.4349488229220f},
  { 0.0f,    -74.0546040990772f},
  { 45.0f,    -78.5782463410377f},
  { 90.0f,    -74.0546040990772f},
  { 0.0f,    -82.8749836510982f},
  { 90.0f,    -82.8749836510982f},
  { 98.1301023541560f,    -8.04946697552840f},
  { 108.434948822922f,    -8.98487693168568f},
  { 120.963756532074f,    -9.73147547315763f},
  { 135.0f,    -10.0249878620757f},
  { 149.036243467926f,    -9.73147547315763f},
  { 161.565051177078f,    -8.98487693168568f},
  { 171.869897645844f,    -8.04946697552840f},
  { 180.0f,    -7.12501634890180f},
  { 99.4623222080256f,    -18.2007602645333f},
  { 111.801409486352f,    -20.3745271887352f},
  { 126.869897645844f,    -21.8014094863518f},
  { 143.130102354156f,    -21.8014094863518f},
  { 158.198590513648f,    -20.3745271887352f},
  { 170.537677791974f,    -18.2007602645333f},
  { 180.0f,    -15.9453959009229f},
  { 101.309932474020f,    -30.4703594659797f},
  { 116.565051177078f,    -33.8545148126205f},
  { 135.0f,    -35.2643896827547f},
  { 153.434948822922f,    -33.8545148126205f},
  { 168.690067525980f,    -30.4703594659797f},
  { 180.0f,    -26.5650511770780f},
  { 104.036243467927f,    -44.1317492120310f},
  { 123.690067525980f,    -47.9688862258027f},
  { 146.309932474020f,    -47.9688862258027f},
  { 165.963756532074f,    -44.1317492120310f},
  { 180.0f,    -38.6598082540901f},
  { 108.434948822922f,    -57.6884667625762f},
  { 135.0f,    -60.5037915034336f},
  { 161.565051177078f,    -57.6884667625762f},
  { 180.0f,    -51.3401917459099f},
  { 116.565051177078f,    -69.5606824267418f},
  { 153.434948822922f,    -69.5606824267418f},
  { 180.0f,    -63.4349488229220f},
  { 135.0f,    -78.5782463410377f},
  { 180.0f,    -74.0546040990772f},
  { 180.0f,    -82.8749836510982f},
  { -171.869897645844f,    -8.04946697552840f},
  { -161.565051177078f,    -8.98487693168568f},
  { -149.036243467926f,    -9.73147547315763f},
  { -135.0f,    -10.0249878620757f},
  { -120.963756532074f,    -9.73147547315763f},
  { -108.434948822922f,    -8.98487693168568f},
  { -98.1301023541560f,    -8.04946697552840f},
  { -90.0f,    -7.12501634890180f},
  { -170.537677791974f,    -18.2007602645333f},
  { -158.198590513648f,    -20.3745271887352f},
  { -143.130102354156f,    -21.8014094863518f},
  { -126.869897645844f,    -21.8014094863518f},
  { -111.801409486352f,    -20.3745271887352f},
  { -99.4623222080256f,    -18.2007602645333f},
  { -90.0f,    -15.9453959009229f},
  { -168.690067525980f,    -30.4703594659797f},
  { -153.434948822922f,    -33.8545148126205f},
  { -135.0f,    -35.2643896827547f},
  { -116.565051177078f,    -33.8545148126205f},
  { -101.309932474020f,    -30.4703594659797f},
  { -90.0000000000000f,    -26.5650511770780f},
  { -165.963756532074f,    -44.1317492120310f},
  { -146.309932474020f,    -47.9688862258027f},
  { -123.690067525980f,    -47.9688862258027f},
  { -104.036243467927f,    -44.1317492120310f},
  { -90.0f,    -38.6598082540901f},
  { -161.565051177078f,    -57.6884667625762f},
  { -135.0f,    -60.5037915034336f},
  { -108.434948822922f,    -57.6884667625762f},
  { -90.0f,    -51.3401917459099f},
  { -153.434948822922f,    -69.5606824267418f},
  { -116.565051177078f,    -69.5606824267418f},
  { -90.0000000000000f,    -63.4349488229220f},
  { -135.0f,    -78.5782463410377f},
  { -90.0f,    -74.0546040990772f},
  { -90.0f,    -82.8749836510982f},
  { -81.8698976458440f,    -8.04946697552840f},
  { -71.5650511770780f,    -8.98487693168568f},
  { -59.0362434679265f,    -9.73147547315763f},
  { -45.0f,    -10.0249878620757f},
  { -30.9637565320735f,    -9.73147547315763f},
  { -18.4349488229220f,    -8.98487693168568f},
  { -8.13010235415598f,    -8.04946697552840f},
  { -80.5376777919744f,    -18.2007602645333f},
  { -68.1985905136482f,    -20.3745271887352f},
  { -53.1301023541560f,    -21.8014094863518f},
  { -36.8698976458440f,    -21.8014094863518f},
  { -21.8014094863518f,    -20.3745271887352f},
  { -9.46232220802561f,    -18.2007602645333f},
  { -78.6900675259798f,    -30.4703594659797f},
  { -63.4349488229220f,    -33.8545148126205f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -26.5650511770780f,    -33.8545148126205f},
  { -11.3099324740202f,    -30.4703594659797f},
  { -75.9637565320735f,    -44.1317492120310f},
  { -56.3099324740202f,    -47.9688862258027f},
  { -33.6900675259798f,    -47.9688862258027f},
  { -14.0362434679265f,    -44.1317492120310f},
  { -71.5650511770780f,    -57.6884667625762f},
  { -45.0f,    -60.5037915034336f},
  { -18.4349488229220f,    -57.6884667625762f},
  { -63.4349488229220f,    -69.5606824267418f},
  { -26.5650511770780f,    -69.5606824267418f},
  { -45.0f,    -78.5782463410377f}};

/* 3LD geosphere 'oct' [10 0] (402 points) */
const float __geosphere_oct_10_0_dirs_deg[402][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    6.34019174590991f},
  { 0.0f,    14.0362434679265f},
  { 0.0f,    23.1985905136482f},
  { 0.0f,    33.6900675259798f},
  { 0.0f,    45.0f},
  { 0.0f,    56.3099324740202f},
  { 0.0f,    66.8014094863518f},
  { 0.0f,    75.9637565320735f},
  { 0.0f,    83.6598082540901f},
  { 6.34019174590991f,    0.0f},
  { 7.12501634890180f,    7.07055511696583f},
  { 8.13010235415598f,    15.7931690482640f},
  { 9.46232220802562f,    26.2523751177207f},
  { 11.3099324740202f,    38.1129265009866f},
  { 14.0362434679265f,    50.4902877138061f},
  { 18.4349488229220f,    62.2086943552208f},
  { 26.5650511770780f,    72.2845276563321f},
  { 45.0000000000000f,    79.9750121379243f},
  { 89.9999999999999f,    83.6598082540901f},
  { 14.0362434679265f,    0.0f},
  { 15.9453959009229f,    7.82123550649965f},
  { 18.4349488229220f,    17.5484006137923f},
  { 21.8014094863518f,    29.1215680703514f},
  { 26.5650511770780f,    41.8103148957786f},
  { 33.6900675259798f,    54.2042400852929f},
  { 45.0f,    64.7605981793211f},
  { 63.4349488229220f,    72.2845276563321f},
  { 90.0f,    75.9637565320735f},
  { 23.1985905136482f,    0.0f},
  { 26.5650511770780f,    8.47871314708616f},
  { 30.9637565320735f,    18.9318231808652f},
  { 36.8698976458440f,    30.9637565320735f},
  { 45.0000000000000f,    43.3138566582830f},
  { 56.3099324740202f,    54.2042400852929f},
  { 71.5650511770780f,    62.2086943552208f},
  { 90.0f,    66.8014094863518f},
  { 33.6900675259798f,    0.0f},
  { 38.6598082540901f,    8.87639508097334f},
  { 45.0f,    19.4712206344907f},
  { 53.1301023541560f,    30.9637565320735f},
  { 63.4349488229220f,    41.8103148957786f},
  { 75.9637565320735f,    50.4902877138061f},
  { 90.0f,    56.3099324740202f},
  { 45.0f,    0.0f},
  { 51.3401917459099f,    8.87639508097334f},
  { 59.0362434679265f,    18.9318231808652f},
  { 68.1985905136482f,    29.1215680703514f},
  { 78.6900675259798f,    38.1129265009866f},
  { 90.0f,    45.0f},
  { 56.3099324740202f,    0.0f},
  { 63.4349488229220f,    8.47871314708616f},
  { 71.5650511770780f,    17.5484006137923f},
  { 80.5376777919744f,    26.2523751177207f},
  { 90.0f,    33.6900675259798f},
  { 66.8014094863518f,    0.0f},
  { 74.0546040990771f,    7.82123550649965f},
  { 81.8698976458440f,    15.7931690482640f},
  { 90.0f,    23.1985905136482f},
  { 75.9637565320735f,    0.0f},
  { 82.8749836510982f,    7.07055511696583f},
  { 90.0f,    14.0362434679265f},
  { 83.6598082540901f,    0.0f},
  { 90.0f,    6.34019174590991f},
  { 96.3401917459099f,    0.0f},
  { 97.1250163489018f,    7.07055511696583f},
  { 98.1301023541560f,    15.7931690482640f},
  { 99.4623222080256f,    26.2523751177207f},
  { 101.309932474020f,    38.1129265009866f},
  { 104.036243467927f,    50.4902877138061f},
  { 108.434948822922f,    62.2086943552208f},
  { 116.565051177078f,    72.2845276563321f},
  { 135.0f,    79.9750121379243f},
  { 180.000000000000f,    83.6598082540901f},
  { 104.036243467927f,    0.0f},
  { 105.945395900923f,    7.82123550649965f},
  { 108.434948822922f,    17.5484006137923f},
  { 111.801409486352f,    29.1215680703514f},
  { 116.565051177078f,    41.8103148957786f},
  { 123.690067525980f,    54.2042400852929f},
  { 135.0f,    64.7605981793211f},
  { 153.434948822922f,    72.2845276563321f},
  { 180.0f,    75.9637565320735f},
  { 113.198590513648f,    0.0f},
  { 116.565051177078f,    8.47871314708616f},
  { 120.963756532074f,    18.9318231808652f},
  { 126.869897645844f,    30.9637565320735f},
  { 135.0f,    43.3138566582830f},
  { 146.309932474020f,    54.2042400852929f},
  { 161.565051177078f,    62.2086943552208f},
  { 180.0f,    66.8014094863518f},
  { 123.690067525980f,    0.0f},
  { 128.659808254090f,    8.87639508097334f},
  { 135.0f,    19.4712206344907f},
  { 143.130102354156f,    30.9637565320735f},
  { 153.434948822922f,    41.8103148957786f},
  { 165.963756532074f,    50.4902877138061f},
  { 180.0f,    56.3099324740202f},
  { 135.0f,    0.0f},
  { 141.340191745910f,    8.87639508097334f},
  { 149.036243467926f,    18.9318231808652f},
  { 158.198590513648f,    29.1215680703514f},
  { 168.690067525980f,    38.1129265009866f},
  { 180.0f,    45.0f},
  { 146.309932474020f,    0.0f},
  { 153.434948822922f,    8.47871314708616f},
  { 161.565051177078f,    17.5484006137923f},
  { 170.537677791974f,    26.2523751177207f},
  { 180.0f,    33.6900675259798f},
  { 156.801409486352f,    0.0f},
  { 164.054604099077f,    7.82123550649965f},
  { 171.869897645844f,    15.7931690482640f},
  { 180.0f,    23.1985905136482f},
  { 165.963756532074f,    0.0f},
  { 172.874983651098f,    7.07055511696583f},
  { 180.0f,    14.0362434679265f},
  { 173.659808254090f,    0.0f},
  { 180.0f,    6.34019174590991f},
  { -173.659808254090f,    0.0f},
  { -172.874983651098f,    7.07055511696583f},
  { -171.869897645844f,    15.7931690482640f},
  { -170.537677791974f,    26.2523751177207f},
  { -168.690067525980f,    38.1129265009866f},
  { -165.963756532074f,    50.4902877138061f},
  { -161.565051177078f,    62.2086943552208f},
  { -153.434948822922f,    72.2845276563321f},
  { -135.0f,    79.9750121379243f},
  { -90.0000000000001f,    83.6598082540901f},
  { -165.963756532074f,    0.0f},
  { -164.054604099077f,    7.82123550649965f},
  { -161.565051177078f,    17.5484006137923f},
  { -158.198590513648f,    29.1215680703514f},
  { -153.434948822922f,    41.8103148957786f},
  { -146.309932474020f,    54.2042400852929f},
  { -135.0f,    64.7605981793211f},
  { -116.565051177078f,    72.2845276563321f},
  { -90.0f,    75.9637565320735f},
  { -156.801409486352f,    0.0f},
  { -153.434948822922f,    8.47871314708616f},
  { -149.036243467926f,    18.9318231808652f},
  { -143.130102354156f,    30.9637565320735f},
  { -135.0f,    43.3138566582830f},
  { -123.690067525980f,    54.2042400852929f},
  { -108.434948822922f,    62.2086943552208f},
  { -90.0f,    66.8014094863518f},
  { -146.309932474020f,    0.0f},
  { -141.340191745910f,    8.87639508097334f},
  { -135.0f,    19.4712206344907f},
  { -126.869897645844f,    30.9637565320735f},
  { -116.565051177078f,    41.8103148957786f},
  { -104.036243467927f,    50.4902877138061f},
  { -90.0f,    56.3099324740202f},
  { -135.0f,    0.0f},
  { -128.659808254090f,    8.87639508097334f},
  { -120.963756532074f,    18.9318231808652f},
  { -111.801409486352f,    29.1215680703514f},
  { -101.309932474020f,    38.1129265009866f},
  { -90.0f,    45.0f},
  { -123.690067525980f,    0.0f},
  { -116.565051177078f,    8.47871314708616f},
  { -108.434948822922f,    17.5484006137923f},
  { -99.4623222080256f,    26.2523751177207f},
  { -90.0f,    33.6900675259798f},
  { -113.198590513648f,    0.0f},
  { -105.945395900923f,    7.82123550649965f},
  { -98.1301023541560f,    15.7931690482640f},
  { -90.0f,    23.1985905136482f},
  { -104.036243467927f,    0.0f},
  { -97.1250163489018f,    7.07055511696582f},
  { -90.0f,    14.0362434679265f},
  { -96.3401917459099f,    0.0f},
  { -90.0f,    6.34019174590991f},
  { -83.6598082540901f,    0.0f},
  { -82.8749836510982f,    7.07055511696583f},
  { -81.8698976458440f,    15.7931690482640f},
  { -80.5376777919744f,    26.2523751177207f},
  { -78.6900675259798f,    38.1129265009866f},
  { -75.9637565320735f,    50.4902877138061f},
  { -71.5650511770780f,    62.2086943552208f},
  { -63.4349488229220f,    72.2845276563321f},
  { -45.0000000000000f,    79.9750121379243f},
  { -75.9637565320735f,    0.0f},
  { -74.0546040990772f,    7.82123550649965f},
  { -71.5650511770780f,    17.5484006137923f},
  { -68.1985905136482f,    29.1215680703514f},
  { -63.4349488229220f,    41.8103148957786f},
  { -56.3099324740202f,    54.2042400852929f},
  { -45.0000000000000f,    64.7605981793211f},
  { -26.5650511770780f,    72.2845276563321f},
  { -66.8014094863518f,    0.0f},
  { -63.4349488229220f,    8.47871314708616f},
  { -59.0362434679265f,    18.9318231808652f},
  { -53.1301023541560f,    30.9637565320735f},
  { -45.0000000000000f,    43.3138566582830f},
  { -33.6900675259798f,    54.2042400852929f},
  { -18.4349488229220f,    62.2086943552208f},
  { -56.3099324740202f,    0.0f},
  { -51.3401917459099f,    8.87639508097334f},
  { -45.0f,    19.4712206344907f},
  { -36.8698976458440f,    30.9637565320735f},
  { -26.5650511770780f,    41.8103148957786f},
  { -14.0362434679265f,    50.4902877138061f},
  { -45.0f,    0.0f},
  { -38.6598082540901f,    8.87639508097334f},
  { -30.9637565320735f,    18.9318231808652f},
  { -21.8014094863518f,    29.1215680703514f},
  { -11.3099324740202f,    38.1129265009866f},
  { -33.6900675259798f,    0.0f},
  { -26.5650511770780f,    8.47871314708616f},
  { -18.4349488229220f,    17.5484006137923f},
  { -9.46232220802562f,    26.2523751177207f},
  { -23.1985905136482f,    0.0f},
  { -15.9453959009229f,    7.82123550649965f},
  { -8.13010235415598f,    15.7931690482640f},
  { -14.0362434679265f,    0.0f},
  { -7.12501634890180f,    7.07055511696582f},
  { -6.34019174590991f,    0.0f},
  { 0.0f,    -6.34019174590991f},
  { 7.12501634890180f,    -7.07055511696583f},
  { 15.9453959009229f,    -7.82123550649965f},
  { 26.5650511770780f,    -8.47871314708616f},
  { 38.6598082540901f,    -8.87639508097334f},
  { 51.3401917459099f,    -8.87639508097334f},
  { 63.4349488229220f,    -8.47871314708616f},
  { 74.0546040990771f,    -7.82123550649965f},
  { 82.8749836510982f,    -7.07055511696583f},
  { 90.0f,    -6.34019174590991f},
  { 0.0f,    -14.0362434679265f},
  { 8.13010235415598f,    -15.7931690482640f},
  { 18.4349488229220f,    -17.5484006137923f},
  { 30.9637565320735f,    -18.9318231808652f},
  { 45.0f,    -19.4712206344907f},
  { 59.0362434679265f,    -18.9318231808652f},
  { 71.5650511770780f,    -17.5484006137923f},
  { 81.8698976458440f,    -15.7931690482640f},
  { 90.0f,    -14.0362434679265f},
  { 0.0f,    -23.1985905136482f},
  { 9.46232220802562f,    -26.2523751177207f},
  { 21.8014094863518f,    -29.1215680703514f},
  { 36.8698976458440f,    -30.9637565320735f},
  { 53.1301023541560f,    -30.9637565320735f},
  { 68.1985905136482f,    -29.1215680703514f},
  { 80.5376777919744f,    -26.2523751177207f},
  { 90.0f,    -23.1985905136482f},
  { 0.0f,    -33.6900675259798f},
  { 11.3099324740202f,    -38.1129265009866f},
  { 26.5650511770780f,    -41.8103148957786f},
  { 45.0000000000000f,    -43.3138566582831f},
  { 63.4349488229220f,    -41.8103148957786f},
  { 78.6900675259798f,    -38.1129265009866f},
  { 90.0f,    -33.6900675259798f},
  { 0.0f,    -45.0f},
  { 14.0362434679265f,    -50.4902877138061f},
  { 33.6900675259798f,    -54.2042400852929f},
  { 56.3099324740202f,    -54.2042400852929f},
  { 75.9637565320735f,    -50.4902877138061f},
  { 90.0f,    -45.0f},
  { 0.0f,    -56.3099324740202f},
  { 18.4349488229220f,    -62.2086943552208f},
  { 45.0f,    -64.7605981793211f},
  { 71.5650511770780f,    -62.2086943552208f},
  { 90.0f,    -56.3099324740202f},
  { 0.0f,    -66.8014094863518f},
  { 26.5650511770780f,    -72.2845276563321f},
  { 63.4349488229220f,    -72.2845276563321f},
  { 90.0f,    -66.8014094863518f},
  { 0.0f,    -75.9637565320735f},
  { 45.0000000000000f,    -79.9750121379243f},
  { 90.0f,    -75.9637565320735f},
  { 0.0f,    -83.6598082540901f},
  { 90.0f,    -83.6598082540901f},
  { 97.1250163489018f,    -7.07055511696583f},
  { 105.945395900923f,    -7.82123550649965f},
  { 116.565051177078f,    -8.47871314708616f},
  { 128.659808254090f,    -8.87639508097334f},
  { 141.340191745910f,    -8.87639508097334f},
  { 153.434948822922f,    -8.47871314708616f},
  { 164.054604099077f,    -7.82123550649965f},
  { 172.874983651098f,    -7.07055511696583f},
  { 180.0f,    -6.34019174590991f},
  { 98.1301023541560f,    -15.7931690482640f},
  { 108.434948822922f,    -17.5484006137923f},
  { 120.963756532074f,    -18.9318231808652f},
  { 135.0f,    -19.4712206344907f},
  { 149.036243467926f,    -18.9318231808652f},
  { 161.565051177078f,    -17.5484006137923f},
  { 171.869897645844f,    -15.7931690482640f},
  { 180.0f,    -14.0362434679265f},
  { 99.4623222080256f,    -26.2523751177207f},
  { 111.801409486352f,    -29.1215680703514f},
  { 126.869897645844f,    -30.9637565320735f},
  { 143.130102354156f,    -30.9637565320735f},
  { 158.198590513648f,    -29.1215680703514f},
  { 170.537677791974f,    -26.2523751177207f},
  { 180.0f,    -23.1985905136482f},
  { 101.309932474020f,    -38.1129265009866f},
  { 116.565051177078f,    -41.8103148957786f},
  { 135.0f,    -43.3138566582831f},
  { 153.434948822922f,    -41.8103148957786f},
  { 168.690067525980f,    -38.1129265009866f},
  { 180.0f,    -33.6900675259798f},
  { 104.036243467927f,    -50.4902877138061f},
  { 123.690067525980f,    -54.2042400852929f},
  { 146.309932474020f,    -54.2042400852929f},
  { 165.963756532074f,    -50.4902877138061f},
  { 180.0f,    -45.0f},
  { 108.434948822922f,    -62.2086943552208f},
  { 135.0f,    -64.7605981793211f},
  { 161.565051177078f,    -62.2086943552208f},
  { 180.0f,    -56.3099324740202f},
  { 116.565051177078f,    -72.2845276563321f},
  { 153.434948822922f,    -72.2845276563321f},
  { 180.0f,    -66.8014094863518f},
  { 135.0f,    -79.9750121379243f},
  { 180.0f,    -75.9637565320735f},
  { 180.0f,    -83.6598082540901f},
  { -172.874983651098f,    -7.07055511696583f},
  { -164.054604099077f,    -7.82123550649965f},
  { -153.434948822922f,    -8.47871314708616f},
  { -141.340191745910f,    -8.87639508097334f},
  { -128.659808254090f,    -8.87639508097334f},
  { -116.565051177078f,    -8.47871314708616f},
  { -105.945395900923f,    -7.82123550649965f},
  { -97.1250163489018f,    -7.07055511696583f},
  { -90.0000000000000f,    -6.34019174590991f},
  { -171.869897645844f,    -15.7931690482640f},
  { -161.565051177078f,    -17.5484006137923f},
  { -149.036243467926f,    -18.9318231808652f},
  { -135.0f,    -19.4712206344907f},
  { -120.963756532074f,    -18.9318231808652f},
  { -108.434948822922f,    -17.5484006137923f},
  { -98.1301023541560f,    -15.7931690482640f},
  { -90.0f,    -14.0362434679265f},
  { -170.537677791974f,    -26.2523751177207f},
  { -158.198590513648f,    -29.1215680703514f},
  { -143.130102354156f,    -30.9637565320735f},
  { -126.869897645844f,    -30.9637565320735f},
  { -111.801409486352f,    -29.1215680703514f},
  { -99.4623222080256f,    -26.2523751177207f},
  { -90.0f,    -23.1985905136482f},
  { -168.690067525980f,    -38.1129265009866f},
  { -153.434948822922f,    -41.8103148957786f},
  { -135.0f,    -43.3138566582831f},
  { -116.565051177078f,    -41.8103148957786f},
  { -101.309932474020f,    -38.1129265009866f},
  { -90.0f,    -33.6900675259798f},
  { -165.963756532074f,    -50.4902877138061f},
  { -146.309932474020f,    -54.2042400852929f},
  { -123.690067525980f,    -54.2042400852929f},
  { -104.036243467927f,    -50.4902877138061f},
  { -90.0f,    -45.0f},
  { -161.565051177078f,    -62.2086943552208f},
  { -135.0f,    -64.7605981793211f},
  { -108.434948822922f,    -62.2086943552208f},
  { -90.0f,    -56.3099324740202f},
  { -153.434948822922f,    -72.2845276563321f},
  { -116.565051177078f,    -72.2845276563321f},
  { -90.0f,    -66.8014094863518f},
  { -135.0f,    -79.9750121379243f},
  { -90.0f,    -75.9637565320735f},
  { -90.0f,    -83.6598082540901f},
  { -82.8749836510982f,    -7.07055511696583f},
  { -74.0546040990771f,    -7.82123550649965f},
  { -63.4349488229220f,    -8.47871314708616f},
  { -51.3401917459099f,    -8.87639508097334f},
  { -38.6598082540901f,    -8.87639508097334f},
  { -26.5650511770780f,    -8.47871314708616f},
  { -15.9453959009229f,    -7.82123550649965f},
  { -7.12501634890180f,    -7.07055511696583f},
  { -81.8698976458440f,    -15.7931690482640f},
  { -71.5650511770780f,    -17.5484006137923f},
  { -59.0362434679265f,    -18.9318231808652f},
  { -45.0f,    -19.4712206344907f},
  { -30.9637565320735f,    -18.9318231808652f},
  { -18.4349488229220f,    -17.5484006137923f},
  { -8.13010235415598f,    -15.7931690482640f},
  { -80.5376777919744f,    -26.2523751177207f},
  { -68.1985905136482f,    -29.1215680703514f},
  { -53.1301023541560f,    -30.9637565320735f},
  { -36.8698976458440f,    -30.9637565320735f},
  { -21.8014094863518f,    -29.1215680703514f},
  { -9.46232220802563f,    -26.2523751177207f},
  { -78.6900675259798f,    -38.1129265009866f},
  { -63.4349488229220f,    -41.8103148957786f},
  { -45.0000000000000f,    -43.3138566582831f},
  { -26.5650511770780f,    -41.8103148957786f},
  { -11.3099324740202f,    -38.1129265009866f},
  { -75.9637565320735f,    -50.4902877138061f},
  { -56.3099324740202f,    -54.2042400852929f},
  { -33.6900675259798f,    -54.2042400852929f},
  { -14.0362434679265f,    -50.4902877138061f},
  { -71.5650511770780f,    -62.2086943552208f},
  { -45.0f,    -64.7605981793211f},
  { -18.4349488229220f,    -62.2086943552208f},
  { -63.4349488229220f,    -72.2845276563321f},
  { -26.5650511770780f,    -72.2845276563321f},
  { -45.0000000000000f,    -79.9750121379243f}};

/* 3LD geosphere 'oct' [11 0] (486 points) */
const float __geosphere_oct_11_0_dirs_deg[486][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    5.71059313749964f},
  { 0.0f,    12.5288077091515f},
  { 0.0f,    20.5560452195835f},
  { 0.0f,    29.7448812969422f},
  { 0.0f,    39.8055710922652f},
  { 0.0f,    50.1944289077348f},
  { 0.0f,    60.2551187030578f},
  { 0.0f,    69.4439547804165f},
  { 0.0f,    77.4711922908485f},
  { 0.0f,    84.2894068625003f},
  { 5.71059313749964f,    0.0f},
  { 6.34019174590991f,    6.30172637399719f},
  { 7.12501634890180f,    13.9320915492581f},
  { 8.13010235415598f,    22.9897677736088f},
  { 9.46232220802562f,    33.3287577974902f},
  { 11.3099324740202f,    44.4382406711498f},
  { 14.0362434679265f,    55.5037627099091f},
  { 18.4349488229220f,    65.6887386511478f},
  { 26.5650511770780f,    74.3838705949755f},
  { 45.0000000000000f,    81.0698581640985f},
  { 89.9999999999999f,    84.2894068625003f},
  { 12.5288077091515f,    0.0f},
  { 14.0362434679265f,    6.91437177215212f},
  { 15.9453959009229f,    15.3613953897309f},
  { 18.4349488229220f,    25.3769335251523f},
  { 21.8014094863518f,    36.6042764427057f},
  { 26.5650511770780f,    48.1896851042214f},
  { 33.6900675259798f,    58.9972808661260f},
  { 45.0000000000000f,    67.9982863255502f},
  { 63.4349488229220f,    74.3838705949755f},
  { 90.0f,    77.4711922908485f},
  { 20.5560452195835f,    0.0f},
  { 23.1985905136482f,    7.48050889170827f},
  { 26.5650511770780f,    16.6015495990202f},
  { 30.9637565320735f,    27.2256279985640f},
  { 36.8698976458440f,    38.6598082540901f},
  { 45.0000000000000f,    49.6844577892274f},
  { 56.3099324740202f,    58.9972808661260f},
  { 71.5650511770780f,    65.6887386511478f},
  { 90.0f,    69.4439547804165f},
  { 29.7448812969422f,    0.0f},
  { 33.6900675259798f,    7.89514210504579f},
  { 38.6598082540901f,    17.3460652926699f},
  { 45.0000000000000f,    27.9383527296024f},
  { 53.1301023541560f,    38.6598082540901f},
  { 63.4349488229220f,    48.1896851042214f},
  { 75.9637565320736f,    55.5037627099091f},
  { 90.0f,    60.2551187030578f},
  { 39.8055710922652f,    0.0f},
  { 45.0f,    8.04946697552840f},
  { 51.3401917459099f,    17.3460652926699f},
  { 59.0362434679265f,    27.2256279985640f},
  { 68.1985905136482f,    36.6042764427057f},
  { 78.6900675259798f,    44.4382406711498f},
  { 90.0f,    50.1944289077348f},
  { 50.1944289077348f,    0.0f},
  { 56.3099324740202f,    7.89514210504579f},
  { 63.4349488229220f,    16.6015495990202f},
  { 71.5650511770780f,    25.3769335251523f},
  { 80.5376777919744f,    33.3287577974902f},
  { 90.0f,    39.8055710922652f},
  { 60.2551187030578f,    0.0f},
  { 66.8014094863518f,    7.48050889170827f},
  { 74.0546040990772f,    15.3613953897309f},
  { 81.8698976458440f,    22.9897677736088f},
  { 90.0f,    29.7448812969422f},
  { 69.4439547804165f,    0.0f},
  { 75.9637565320735f,    6.91437177215212f},
  { 82.8749836510982f,    13.9320915492581f},
  { 90.0f,    20.5560452195835f},
  { 77.4711922908485f,    0.0f},
  { 83.6598082540901f,    6.30172637399719f},
  { 90.0f,    12.5288077091515f},
  { 84.2894068625003f,    0.0f},
  { 90.0f,    5.71059313749965f},
  { 95.7105931374996f,    0.0f},
  { 96.3401917459099f,    6.30172637399719f},
  { 97.1250163489018f,    13.9320915492581f},
  { 98.1301023541560f,    22.9897677736088f},
  { 99.4623222080256f,    33.3287577974902f},
  { 101.309932474020f,    44.4382406711498f},
  { 104.036243467927f,    55.5037627099091f},
  { 108.434948822922f,    65.6887386511478f},
  { 116.565051177078f,    74.3838705949755f},
  { 135.0f,    81.0698581640985f},
  { 180.000000000000f,    84.2894068625003f},
  { 102.528807709152f,    0.0f},
  { 104.036243467927f,    6.91437177215212f},
  { 105.945395900923f,    15.3613953897309f},
  { 108.434948822922f,    25.3769335251523f},
  { 111.801409486352f,    36.6042764427057f},
  { 116.565051177078f,    48.1896851042214f},
  { 123.690067525980f,    58.9972808661260f},
  { 135.0f,    67.9982863255502f},
  { 153.434948822922f,    74.3838705949755f},
  { 180.0f,    77.4711922908485f},
  { 110.556045219583f,    0.0f},
  { 113.198590513648f,    7.48050889170827f},
  { 116.565051177078f,    16.6015495990202f},
  { 120.963756532074f,    27.2256279985640f},
  { 126.869897645844f,    38.6598082540901f},
  { 135.0f,    49.6844577892274f},
  { 146.309932474020f,    58.9972808661260f},
  { 161.565051177078f,    65.6887386511478f},
  { 180.0f,    69.4439547804165f},
  { 119.744881296942f,    0.0f},
  { 123.690067525980f,    7.89514210504579f},
  { 128.659808254090f,    17.3460652926699f},
  { 135.0f,    27.9383527296024f},
  { 143.130102354156f,    38.6598082540901f},
  { 153.434948822922f,    48.1896851042214f},
  { 165.963756532074f,    55.5037627099091f},
  { 180.0f,    60.2551187030578f},
  { 129.805571092265f,    0.0f},
  { 135.0f,    8.04946697552840f},
  { 141.340191745910f,    17.3460652926699f},
  { 149.036243467926f,    27.2256279985640f},
  { 158.198590513648f,    36.6042764427057f},
  { 168.690067525980f,    44.4382406711498f},
  { 180.0f,    50.1944289077348f},
  { 140.194428907735f,    0.0f},
  { 146.309932474020f,    7.89514210504579f},
  { 153.434948822922f,    16.6015495990202f},
  { 161.565051177078f,    25.3769335251523f},
  { 170.537677791974f,    33.3287577974902f},
  { 180.0f,    39.8055710922652f},
  { 150.255118703058f,    0.0f},
  { 156.801409486352f,    7.48050889170827f},
  { 164.054604099077f,    15.3613953897309f},
  { 171.869897645844f,    22.9897677736088f},
  { 180.0f,    29.7448812969422f},
  { 159.443954780417f,    0.0f},
  { 165.963756532074f,    6.91437177215212f},
  { 172.874983651098f,    13.9320915492581f},
  { 180.0f,    20.5560452195835f},
  { 167.471192290848f,    0.0f},
  { 173.659808254090f,    6.30172637399719f},
  { 180.0f,    12.5288077091515f},
  { 174.289406862500f,    0.0f},
  { 180.0f,    5.71059313749965f},
  { -174.289406862500f,    0.0f},
  { -173.659808254090f,    6.30172637399719f},
  { -172.874983651098f,    13.9320915492581f},
  { -171.869897645844f,    22.9897677736088f},
  { -170.537677791974f,    33.3287577974902f},
  { -168.690067525980f,    44.4382406711498f},
  { -165.963756532074f,    55.5037627099091f},
  { -161.565051177078f,    65.6887386511478f},
  { -153.434948822922f,    74.3838705949755f},
  { -135.0f,    81.0698581640985f},
  { -90.0000000000001f,    84.2894068625003f},
  { -167.471192290848f,    0.0f},
  { -165.963756532074f,    6.91437177215212f},
  { -164.054604099077f,    15.3613953897309f},
  { -161.565051177078f,    25.3769335251523f},
  { -158.198590513648f,    36.6042764427057f},
  { -153.434948822922f,    48.1896851042214f},
  { -146.309932474020f,    58.9972808661260f},
  { -135.0f,    67.9982863255502f},
  { -116.565051177078f,    74.3838705949755f},
  { -90.0f,    77.4711922908485f},
  { -159.443954780417f,    0.0f},
  { -156.801409486352f,    7.48050889170827f},
  { -153.434948822922f,    16.6015495990202f},
  { -149.036243467926f,    27.2256279985640f},
  { -143.130102354156f,    38.6598082540901f},
  { -135.0f,    49.6844577892274f},
  { -123.690067525980f,    58.9972808661260f},
  { -108.434948822922f,    65.6887386511478f},
  { -90.0f,    69.4439547804165f},
  { -150.255118703058f,    0.0f},
  { -146.309932474020f,    7.89514210504579f},
  { -141.340191745910f,    17.3460652926699f},
  { -135.0f,    27.9383527296024f},
  { -126.869897645844f,    38.6598082540901f},
  { -116.565051177078f,    48.1896851042214f},
  { -104.036243467926f,    55.5037627099091f},
  { -90.0f,    60.2551187030578f},
  { -140.194428907735f,    0.0f},
  { -135.0f,    8.04946697552840f},
  { -128.659808254090f,    17.3460652926699f},
  { -120.963756532074f,    27.2256279985640f},
  { -111.801409486352f,    36.6042764427057f},
  { -101.309932474020f,    44.4382406711498f},
  { -90.0f,    50.1944289077348f},
  { -129.805571092265f,    0.0f},
  { -123.690067525980f,    7.89514210504579f},
  { -116.565051177078f,    16.6015495990202f},
  { -108.434948822922f,    25.3769335251523f},
  { -99.4623222080256f,    33.3287577974902f},
  { -90.0f,    39.8055710922652f},
  { -119.744881296942f,    0.0f},
  { -113.198590513648f,    7.48050889170827f},
  { -105.945395900923f,    15.3613953897309f},
  { -98.1301023541560f,    22.9897677736088f},
  { -90.0f,    29.7448812969422f},
  { -110.556045219583f,    0.0f},
  { -104.036243467927f,    6.91437177215212f},
  { -97.1250163489018f,    13.9320915492581f},
  { -90.0f,    20.5560452195835f},
  { -102.528807709152f,    0.0f},
  { -96.3401917459099f,    6.30172637399719f},
  { -90.0f,    12.5288077091515f},
  { -95.7105931374997f,    0.0f},
  { -90.0f,    5.71059313749965f},
  { -84.2894068625004f,    0.0f},
  { -83.6598082540901f,    6.30172637399719f},
  { -82.8749836510982f,    13.9320915492581f},
  { -81.8698976458440f,    22.9897677736088f},
  { -80.5376777919744f,    33.3287577974902f},
  { -78.6900675259798f,    44.4382406711498f},
  { -75.9637565320735f,    55.5037627099091f},
  { -71.5650511770780f,    65.6887386511478f},
  { -63.4349488229220f,    74.3838705949755f},
  { -45.0000000000000f,    81.0698581640985f},
  { -77.4711922908485f,    0.0f},
  { -75.9637565320735f,    6.91437177215212f},
  { -74.0546040990772f,    15.3613953897309f},
  { -71.5650511770780f,    25.3769335251523f},
  { -68.1985905136482f,    36.6042764427057f},
  { -63.4349488229220f,    48.1896851042214f},
  { -56.3099324740202f,    58.9972808661260f},
  { -45.0000000000000f,    67.9982863255502f},
  { -26.5650511770780f,    74.3838705949755f},
  { -69.4439547804165f,    0.0f},
  { -66.8014094863518f,    7.48050889170827f},
  { -63.4349488229220f,    16.6015495990202f},
  { -59.0362434679265f,    27.2256279985640f},
  { -53.1301023541560f,    38.6598082540901f},
  { -45.0000000000000f,    49.6844577892274f},
  { -33.6900675259798f,    58.9972808661260f},
  { -18.4349488229220f,    65.6887386511478f},
  { -60.2551187030578f,    0.0f},
  { -56.3099324740202f,    7.89514210504579f},
  { -51.3401917459099f,    17.3460652926699f},
  { -45.0000000000000f,    27.9383527296024f},
  { -36.8698976458440f,    38.6598082540901f},
  { -26.5650511770780f,    48.1896851042214f},
  { -14.0362434679265f,    55.5037627099091f},
  { -50.1944289077348f,    0.0f},
  { -45.0f,    8.04946697552840f},
  { -38.6598082540901f,    17.3460652926699f},
  { -30.9637565320735f,    27.2256279985640f},
  { -21.8014094863518f,    36.6042764427057f},
  { -11.3099324740202f,    44.4382406711498f},
  { -39.8055710922652f,    0.0f},
  { -33.6900675259798f,    7.89514210504579f},
  { -26.5650511770780f,    16.6015495990202f},
  { -18.4349488229220f,    25.3769335251523f},
  { -9.46232220802562f,    33.3287577974902f},
  { -29.7448812969422f,    0.0f},
  { -23.1985905136482f,    7.48050889170827f},
  { -15.9453959009229f,    15.3613953897309f},
  { -8.13010235415598f,    22.9897677736088f},
  { -20.5560452195835f,    0.0f},
  { -14.0362434679265f,    6.91437177215212f},
  { -7.12501634890180f,    13.9320915492581f},
  { -12.5288077091515f,    0.0f},
  { -6.34019174590991f,    6.30172637399719f},
  { -5.71059313749965f,    0.0f},
  { 0.0f,    -5.71059313749964f},
  { 6.34019174590991f,    -6.30172637399719f},
  { 14.0362434679265f,    -6.91437177215212f},
  { 23.1985905136482f,    -7.48050889170827f},
  { 33.6900675259798f,    -7.89514210504579f},
  { 45.0000000000000f,    -8.04946697552840f},
  { 56.3099324740202f,    -7.89514210504579f},
  { 66.8014094863518f,    -7.48050889170827f},
  { 75.9637565320735f,    -6.91437177215212f},
  { 83.6598082540901f,    -6.30172637399719f},
  { 90.0f,    -5.71059313749965f},
  { 0.0f,    -12.5288077091515f},
  { 7.12501634890180f,    -13.9320915492581f},
  { 15.9453959009229f,    -15.3613953897309f},
  { 26.5650511770780f,    -16.6015495990202f},
  { 38.6598082540901f,    -17.3460652926699f},
  { 51.3401917459099f,    -17.3460652926699f},
  { 63.4349488229220f,    -16.6015495990202f},
  { 74.0546040990771f,    -15.3613953897309f},
  { 82.8749836510982f,    -13.9320915492581f},
  { 90.0f,    -12.5288077091515f},
  { 0.0f,    -20.5560452195835f},
  { 8.13010235415598f,    -22.9897677736088f},
  { 18.4349488229220f,    -25.3769335251523f},
  { 30.9637565320735f,    -27.2256279985640f},
  { 45.0000000000000f,    -27.9383527296024f},
  { 59.0362434679265f,    -27.2256279985640f},
  { 71.5650511770780f,    -25.3769335251523f},
  { 81.8698976458440f,    -22.9897677736088f},
  { 90.0f,    -20.5560452195835f},
  { 0.0f,    -29.7448812969422f},
  { 9.46232220802562f,    -33.3287577974902f},
  { 21.8014094863518f,    -36.6042764427057f},
  { 36.8698976458440f,    -38.6598082540901f},
  { 53.1301023541560f,    -38.6598082540901f},
  { 68.1985905136482f,    -36.6042764427057f},
  { 80.5376777919744f,    -33.3287577974902f},
  { 90.0f,    -29.7448812969422f},
  { 0.0f,    -39.8055710922652f},
  { 11.3099324740202f,    -44.4382406711498f},
  { 26.5650511770780f,    -48.1896851042214f},
  { 45.0f,    -49.6844577892274f},
  { 63.4349488229220f,    -48.1896851042214f},
  { 78.6900675259798f,    -44.4382406711498f},
  { 90.0f,    -39.8055710922652f},
  { 0.0f,    -50.1944289077348f},
  { 14.0362434679265f,    -55.5037627099091f},
  { 33.6900675259798f,    -58.9972808661260f},
  { 56.3099324740202f,    -58.9972808661260f},
  { 75.9637565320735f,    -55.5037627099091f},
  { 90.0f,    -50.1944289077348f},
  { 0.0f,    -60.2551187030578f},
  { 18.4349488229220f,    -65.6887386511478f},
  { 45.0000000000000f,    -67.9982863255502f},
  { 71.5650511770780f,    -65.6887386511478f},
  { 90.0f,    -60.2551187030578f},
  { 0.0f,    -69.4439547804165f},
  { 26.5650511770780f,    -74.3838705949755f},
  { 63.4349488229220f,    -74.3838705949755f},
  { 90.0f,    -69.4439547804165f},
  { 0.0f,    -77.4711922908485f},
  { 45.0f,    -81.0698581640985f},
  { 90.0f,    -77.4711922908485f},
  { 0.0f,    -84.2894068625003f},
  { 90.0f,    -84.2894068625003f},
  { 96.3401917459099f,    -6.30172637399719f},
  { 104.036243467927f,    -6.91437177215212f},
  { 113.198590513648f,    -7.48050889170827f},
  { 123.690067525980f,    -7.89514210504579f},
  { 135.0f,    -8.04946697552840f},
  { 146.309932474020f,    -7.89514210504579f},
  { 156.801409486352f,    -7.48050889170827f},
  { 165.963756532074f,    -6.91437177215212f},
  { 173.659808254090f,    -6.30172637399719f},
  { 180.0f,    -5.71059313749965f},
  { 97.1250163489018f,    -13.9320915492581f},
  { 105.945395900923f,    -15.3613953897309f},
  { 116.565051177078f,    -16.6015495990202f},
  { 128.659808254090f,    -17.3460652926699f},
  { 141.340191745910f,    -17.3460652926699f},
  { 153.434948822922f,    -16.6015495990202f},
  { 164.054604099077f,    -15.3613953897309f},
  { 172.874983651098f,    -13.9320915492581f},
  { 180.0f,    -12.5288077091515f},
  { 98.1301023541560f,    -22.9897677736088f},
  { 108.434948822922f,    -25.3769335251523f},
  { 120.963756532074f,    -27.2256279985640f},
  { 135.0f,    -27.9383527296024f},
  { 149.036243467926f,    -27.2256279985640f},
  { 161.565051177078f,    -25.3769335251523f},
  { 171.869897645844f,    -22.9897677736088f},
  { 180.0f,    -20.5560452195835f},
  { 99.4623222080256f,    -33.3287577974902f},
  { 111.801409486352f,    -36.6042764427057f},
  { 126.869897645844f,    -38.6598082540901f},
  { 143.130102354156f,    -38.6598082540901f},
  { 158.198590513648f,    -36.6042764427057f},
  { 170.537677791974f,    -33.3287577974902f},
  { 180.0f,    -29.7448812969422f},
  { 101.309932474020f,    -44.4382406711498f},
  { 116.565051177078f,    -48.1896851042214f},
  { 135.0f,    -49.6844577892274f},
  { 153.434948822922f,    -48.1896851042214f},
  { 168.690067525980f,    -44.4382406711498f},
  { 180.0f,    -39.8055710922652f},
  { 104.036243467927f,    -55.5037627099091f},
  { 123.690067525980f,    -58.9972808661260f},
  { 146.309932474020f,    -58.9972808661260f},
  { 165.963756532074f,    -55.5037627099091f},
  { 180.0f,    -50.1944289077348f},
  { 108.434948822922f,    -65.6887386511478f},
  { 135.0f,    -67.9982863255502f},
  { 161.565051177078f,    -65.6887386511478f},
  { 180.0f,    -60.2551187030578f},
  { 116.565051177078f,    -74.3838705949755f},
  { 153.434948822922f,    -74.3838705949755f},
  { 180.0f,    -69.4439547804165f},
  { 135.0f,    -81.0698581640985f},
  { 180.0f,    -77.4711922908485f},
  { 180.0f,    -84.2894068625003f},
  { -173.659808254090f,    -6.30172637399719f},
  { -165.963756532074f,    -6.91437177215212f},
  { -156.801409486352f,    -7.48050889170827f},
  { -146.309932474020f,    -7.89514210504579f},
  { -135.0f,    -8.04946697552840f},
  { -123.690067525980f,    -7.89514210504579f},
  { -113.198590513648f,    -7.48050889170827f},
  { -104.036243467927f,    -6.91437177215212f},
  { -96.3401917459099f,    -6.30172637399719f},
  { -90.0000000000000f,    -5.71059313749965f},
  { -172.874983651098f,    -13.9320915492581f},
  { -164.054604099077f,    -15.3613953897309f},
  { -153.434948822922f,    -16.6015495990202f},
  { -141.340191745910f,    -17.3460652926699f},
  { -128.659808254090f,    -17.3460652926699f},
  { -116.565051177078f,    -16.6015495990202f},
  { -105.945395900923f,    -15.3613953897309f},
  { -97.1250163489018f,    -13.9320915492581f},
  { -90.0f,    -12.5288077091515f},
  { -171.869897645844f,    -22.9897677736088f},
  { -161.565051177078f,    -25.3769335251523f},
  { -149.036243467926f,    -27.2256279985640f},
  { -135.0f,    -27.9383527296024f},
  { -120.963756532074f,    -27.2256279985640f},
  { -108.434948822922f,    -25.3769335251523f},
  { -98.1301023541560f,    -22.9897677736088f},
  { -90.0f,    -20.5560452195835f},
  { -170.537677791974f,    -33.3287577974902f},
  { -158.198590513648f,    -36.6042764427057f},
  { -143.130102354156f,    -38.6598082540901f},
  { -126.869897645844f,    -38.6598082540901f},
  { -111.801409486352f,    -36.6042764427057f},
  { -99.4623222080256f,    -33.3287577974902f},
  { -90.0f,    -29.7448812969422f},
  { -168.690067525980f,    -44.4382406711498f},
  { -153.434948822922f,    -48.1896851042214f},
  { -135.0f,    -49.6844577892274f},
  { -116.565051177078f,    -48.1896851042214f},
  { -101.309932474020f,    -44.4382406711498f},
  { -90.0f,    -39.8055710922652f},
  { -165.963756532074f,    -55.5037627099091f},
  { -146.309932474020f,    -58.9972808661260f},
  { -123.690067525980f,    -58.9972808661260f},
  { -104.036243467927f,    -55.5037627099091f},
  { -90.0f,    -50.1944289077348f},
  { -161.565051177078f,    -65.6887386511478f},
  { -135.0f,    -67.9982863255502f},
  { -108.434948822922f,    -65.6887386511478f},
  { -90.0f,    -60.2551187030578f},
  { -153.434948822922f,    -74.3838705949755f},
  { -116.565051177078f,    -74.3838705949755f},
  { -90.0f,    -69.4439547804165f},
  { -135.0f,    -81.0698581640985f},
  { -90.0f,    -77.4711922908485f},
  { -90.0f,    -84.2894068625003f},
  { -83.6598082540901f,    -6.30172637399719f},
  { -75.9637565320735f,    -6.91437177215212f},
  { -66.8014094863518f,    -7.48050889170827f},
  { -56.3099324740202f,    -7.89514210504579f},
  { -45.0000000000000f,    -8.04946697552840f},
  { -33.6900675259798f,    -7.89514210504579f},
  { -23.1985905136482f,    -7.48050889170827f},
  { -14.0362434679265f,    -6.91437177215212f},
  { -6.34019174590991f,    -6.30172637399719f},
  { -82.8749836510982f,    -13.9320915492581f},
  { -74.0546040990772f,    -15.3613953897309f},
  { -63.4349488229220f,    -16.6015495990202f},
  { -51.3401917459099f,    -17.3460652926699f},
  { -38.6598082540901f,    -17.3460652926699f},
  { -26.5650511770780f,    -16.6015495990202f},
  { -15.9453959009229f,    -15.3613953897309f},
  { -7.12501634890180f,    -13.9320915492581f},
  { -81.8698976458440f,    -22.9897677736088f},
  { -71.5650511770780f,    -25.3769335251523f},
  { -59.0362434679265f,    -27.2256279985640f},
  { -45.0000000000000f,    -27.9383527296024f},
  { -30.9637565320735f,    -27.2256279985640f},
  { -18.4349488229220f,    -25.3769335251523f},
  { -8.13010235415598f,    -22.9897677736088f},
  { -80.5376777919744f,    -33.3287577974902f},
  { -68.1985905136482f,    -36.6042764427057f},
  { -53.1301023541560f,    -38.6598082540901f},
  { -36.8698976458440f,    -38.6598082540901f},
  { -21.8014094863518f,    -36.6042764427057f},
  { -9.46232220802561f,    -33.3287577974902f},
  { -78.6900675259798f,    -44.4382406711498f},
  { -63.4349488229220f,    -48.1896851042214f},
  { -45.0f,    -49.6844577892274f},
  { -26.5650511770780f,    -48.1896851042214f},
  { -11.3099324740202f,    -44.4382406711498f},
  { -75.9637565320735f,    -55.5037627099091f},
  { -56.3099324740202f,    -58.9972808661260f},
  { -33.6900675259798f,    -58.9972808661260f},
  { -14.0362434679265f,    -55.5037627099091f},
  { -71.5650511770780f,    -65.6887386511478f},
  { -45.0000000000000f,    -67.9982863255502f},
  { -18.4349488229220f,    -65.6887386511478f},
  { -63.4349488229220f,    -74.3838705949755f},
  { -26.5650511770780f,    -74.3838705949755f},
  { -45.0f,    -81.0698581640985f}};

/* 3LD geosphere 'oct' [12 0] (578 points) */
const float __geosphere_oct_12_0_dirs_deg[578][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    5.19442890773481f},
  { 0.0f,    11.3099324740202f},
  { 0.0f,    18.4349488229220f},
  { 0.0f,    26.5650511770780f},
  { 0.0f,    35.5376777919744f},
  { 0.0f,    45.0f},
  { 0.0f,    54.4623222080256f},
  { 0.0f,    63.4349488229220f},
  { 0.0f,    71.5650511770780f},
  { 0.0f,    78.6900675259798f},
  { 0.0f,    84.8055710922652f},
  { 5.19442890773481f,    0.0f},
  { 5.71059313749964f,    5.68243848351684f},
  { 6.34019174590991f,    12.4545763684750f},
  { 7.12501634890180f,    20.4104455216585f},
  { 8.13010235415598f,    29.4962084965664f},
  { 9.46232220802562f,    39.4200315639073f},
  { 11.3099324740202f,    49.6408995433388f},
  { 14.0362434679265f,    59.5012730133354f},
  { 18.4349488229220f,    68.4318705643521f},
  { 26.5650511770780f,    76.0472636044633f},
  { 45.0000000000000f,    81.9505330244716f},
  { 89.9999999999999f,    84.8055710922652f},
  { 11.3099324740202f,    0.0f},
  { 12.5288077091515f,    6.19039945857186f},
  { 14.0362434679265f,    13.6330222253664f},
  { 15.9453959009229f,    22.3956623300392f},
  { 18.4349488229220f,    32.3115332374239f},
  { 21.8014094863518f,    42.8759886666357f},
  { 26.5650511770780f,    53.3007747995101f},
  { 33.6900675259798f,    62.7479663251258f},
  { 45.0000000000000f,    70.5287793655093f},
  { 63.4349488229220f,    76.0472636044633f},
  { 90.0f,    78.6900675259798f},
  { 18.4349488229220f,    0.0f},
  { 20.5560452195835f,    6.67559216607672f},
  { 23.1985905136482f,    14.7143535884335f},
  { 26.5650511770780f,    24.0948425521107f},
  { 30.9637565320735f,    34.4499019879535f},
  { 36.8698976458440f,    45.0f},
  { 45.0f,    54.7356103172453f},
  { 56.3099324740202f,    62.7479663251258f},
  { 71.5650511770780f,    68.4318705643521f},
  { 90.0f,    71.5650511770780f},
  { 26.5650511770780f,    0.0f},
  { 29.7448812969422f,    7.07055511696583f},
  { 33.6900675259798f,    15.5013595669370f},
  { 38.6598082540901f,    25.1040902502214f},
  { 45.0000000000000f,    35.2643896827547f},
  { 53.1301023541560f,    45.0000000000000f},
  { 63.4349488229220f,    53.3007747995101f},
  { 75.9637565320735f,    59.5012730133354f},
  { 90.0f,    63.4349488229220f},
  { 35.5376777919744f,    0.0f},
  { 39.8055710922652f,    7.29627530101671f},
  { 45.0000000000000f,    15.7931690482640f},
  { 51.3401917459099f,    25.1040902502214f},
  { 59.0362434679265f,    34.4499019879535f},
  { 68.1985905136482f,    42.8759886666357f},
  { 78.6900675259798f,    49.6408995433388f},
  { 90.0f,    54.4623222080256f},
  { 45.0f,    0.0f},
  { 50.1944289077348f,    7.29627530101671f},
  { 56.3099324740202f,    15.5013595669370f},
  { 63.4349488229220f,    24.0948425521107f},
  { 71.5650511770780f,    32.3115332374238f},
  { 80.5376777919744f,    39.4200315639073f},
  { 90.0f,    45.0f},
  { 54.4623222080256f,    0.0f},
  { 60.2551187030578f,    7.07055511696583f},
  { 66.8014094863518f,    14.7143535884335f},
  { 74.0546040990772f,    22.3956623300392f},
  { 81.8698976458440f,    29.4962084965664f},
  { 90.0f,    35.5376777919744f},
  { 63.4349488229220f,    0.0f},
  { 69.4439547804165f,    6.67559216607672f},
  { 75.9637565320735f,    13.6330222253664f},
  { 82.8749836510982f,    20.4104455216585f},
  { 90.0f,    26.5650511770780f},
  { 71.5650511770780f,    0.0f},
  { 77.4711922908485f,    6.19039945857186f},
  { 83.6598082540901f,    12.4545763684750f},
  { 90.0f,    18.4349488229220f},
  { 78.6900675259798f,    0.0f},
  { 84.2894068625004f,    5.68243848351684f},
  { 90.0f,    11.3099324740202f},
  { 84.8055710922652f,    0.0f},
  { 90.0f,    5.19442890773481f},
  { 95.1944289077348f,    0.0f},
  { 95.7105931374996f,    5.68243848351684f},
  { 96.3401917459099f,    12.4545763684750f},
  { 97.1250163489018f,    20.4104455216585f},
  { 98.1301023541560f,    29.4962084965664f},
  { 99.4623222080256f,    39.4200315639073f},
  { 101.309932474020f,    49.6408995433388f},
  { 104.036243467927f,    59.5012730133354f},
  { 108.434948822922f,    68.4318705643521f},
  { 116.565051177078f,    76.0472636044633f},
  { 135.0f,    81.9505330244716f},
  { 180.000000000000f,    84.8055710922652f},
  { 101.309932474020f,    0.0f},
  { 102.528807709152f,    6.19039945857186f},
  { 104.036243467927f,    13.6330222253664f},
  { 105.945395900923f,    22.3956623300392f},
  { 108.434948822922f,    32.3115332374239f},
  { 111.801409486352f,    42.8759886666357f},
  { 116.565051177078f,    53.3007747995101f},
  { 123.690067525980f,    62.7479663251258f},
  { 135.0f,    70.5287793655093f},
  { 153.434948822922f,    76.0472636044633f},
  { 180.0f,    78.6900675259798f},
  { 108.434948822922f,    0.0f},
  { 110.556045219583f,    6.67559216607672f},
  { 113.198590513648f,    14.7143535884335f},
  { 116.565051177078f,    24.0948425521107f},
  { 120.963756532074f,    34.4499019879535f},
  { 126.869897645844f,    45.0f},
  { 135.0f,    54.7356103172453f},
  { 146.309932474020f,    62.7479663251258f},
  { 161.565051177078f,    68.4318705643521f},
  { 180.0f,    71.5650511770780f},
  { 116.565051177078f,    0.0f},
  { 119.744881296942f,    7.07055511696583f},
  { 123.690067525980f,    15.5013595669370f},
  { 128.659808254090f,    25.1040902502214f},
  { 135.0f,    35.2643896827547f},
  { 143.130102354156f,    45.0000000000000f},
  { 153.434948822922f,    53.3007747995101f},
  { 165.963756532074f,    59.5012730133354f},
  { 180.0f,    63.4349488229220f},
  { 125.537677791974f,    0.0f},
  { 129.805571092265f,    7.29627530101671f},
  { 135.0f,    15.7931690482640f},
  { 141.340191745910f,    25.1040902502214f},
  { 149.036243467926f,    34.4499019879535f},
  { 158.198590513648f,    42.8759886666357f},
  { 168.690067525980f,    49.6408995433388f},
  { 180.0f,    54.4623222080256f},
  { 135.0f,    0.0f},
  { 140.194428907735f,    7.29627530101671f},
  { 146.309932474020f,    15.5013595669370f},
  { 153.434948822922f,    24.0948425521107f},
  { 161.565051177078f,    32.3115332374238f},
  { 170.537677791974f,    39.4200315639073f},
  { 180.0f,    45.0f},
  { 144.462322208026f,    0.0f},
  { 150.255118703058f,    7.07055511696583f},
  { 156.801409486352f,    14.7143535884335f},
  { 164.054604099077f,    22.3956623300392f},
  { 171.869897645844f,    29.4962084965664f},
  { 180.0f,    35.5376777919744f},
  { 153.434948822922f,    0.0f},
  { 159.443954780417f,    6.67559216607672f},
  { 165.963756532074f,    13.6330222253664f},
  { 172.874983651098f,    20.4104455216585f},
  { 180.0f,    26.5650511770780f},
  { 161.565051177078f,    0.0f},
  { 167.471192290848f,    6.19039945857186f},
  { 173.659808254090f,    12.4545763684750f},
  { 180.0f,    18.4349488229220f},
  { 168.690067525980f,    0.0f},
  { 174.289406862500f,    5.68243848351684f},
  { 180.0f,    11.3099324740202f},
  { 174.805571092265f,    0.0f},
  { 180.0f,    5.19442890773481f},
  { -174.805571092265f,    0.0f},
  { -174.289406862500f,    5.68243848351684f},
  { -173.659808254090f,    12.4545763684750f},
  { -172.874983651098f,    20.4104455216585f},
  { -171.869897645844f,    29.4962084965664f},
  { -170.537677791974f,    39.4200315639073f},
  { -168.690067525980f,    49.6408995433388f},
  { -165.963756532074f,    59.5012730133354f},
  { -161.565051177078f,    68.4318705643521f},
  { -153.434948822922f,    76.0472636044633f},
  { -135.000000000000f,    81.9505330244716f},
  { -90.0000000000001f,    84.8055710922652f},
  { -168.690067525980f,    0.0f},
  { -167.471192290848f,    6.19039945857186f},
  { -165.963756532074f,    13.6330222253664f},
  { -164.054604099077f,    22.3956623300392f},
  { -161.565051177078f,    32.3115332374239f},
  { -158.198590513648f,    42.8759886666357f},
  { -153.434948822922f,    53.3007747995101f},
  { -146.309932474020f,    62.7479663251258f},
  { -135.0f,    70.5287793655093f},
  { -116.565051177078f,    76.0472636044633f},
  { -90.0f,    78.6900675259798f},
  { -161.565051177078f,    0.0f},
  { -159.443954780417f,    6.67559216607672f},
  { -156.801409486352f,    14.7143535884335f},
  { -153.434948822922f,    24.0948425521107f},
  { -149.036243467926f,    34.4499019879535f},
  { -143.130102354156f,    45.0f},
  { -135.0f,    54.7356103172453f},
  { -123.690067525980f,    62.7479663251258f},
  { -108.434948822922f,    68.4318705643521f},
  { -90.0f,    71.5650511770780f},
  { -153.434948822922f,    0.0f},
  { -150.255118703058f,    7.07055511696583f},
  { -146.309932474020f,    15.5013595669370f},
  { -141.340191745910f,    25.1040902502214f},
  { -135.0f,    35.2643896827547f},
  { -126.869897645844f,    45.0f},
  { -116.565051177078f,    53.3007747995101f},
  { -104.036243467927f,    59.5012730133354f},
  { -90.0f,    63.4349488229220f},
  { -144.462322208026f,    0.0f},
  { -140.194428907735f,    7.29627530101671f},
  { -135.0f,    15.7931690482640f},
  { -128.659808254090f,    25.1040902502214f},
  { -120.963756532074f,    34.4499019879535f},
  { -111.801409486352f,    42.8759886666357f},
  { -101.309932474020f,    49.6408995433388f},
  { -90.0f,    54.4623222080256f},
  { -135.0f,    0.0f},
  { -129.805571092265f,    7.29627530101671f},
  { -123.690067525980f,    15.5013595669370f},
  { -116.565051177078f,    24.0948425521107f},
  { -108.434948822922f,    32.3115332374238f},
  { -99.4623222080256f,    39.4200315639073f},
  { -90.0f,    45.0f},
  { -125.537677791974f,    0.0f},
  { -119.744881296942f,    7.07055511696583f},
  { -113.198590513648f,    14.7143535884335f},
  { -105.945395900923f,    22.3956623300392f},
  { -98.1301023541560f,    29.4962084965664f},
  { -90.0f,    35.5376777919744f},
  { -116.565051177078f,    0.0f},
  { -110.556045219583f,    6.67559216607672f},
  { -104.036243467927f,    13.6330222253664f},
  { -97.1250163489018f,    20.4104455216585f},
  { -90.0f,    26.5650511770780f},
  { -108.434948822922f,    0.0f},
  { -102.528807709152f,    6.19039945857186f},
  { -96.3401917459099f,    12.4545763684750f},
  { -90.0f,    18.4349488229220f},
  { -101.309932474020f,    0.0f},
  { -95.7105931374996f,    5.68243848351684f},
  { -90.0f,    11.3099324740202f},
  { -95.1944289077348f,    0.0f},
  { -90.0f,    5.19442890773481f},
  { -84.8055710922652f,    0.0f},
  { -84.2894068625004f,    5.68243848351684f},
  { -83.6598082540901f,    12.4545763684750f},
  { -82.8749836510982f,    20.4104455216585f},
  { -81.8698976458440f,    29.4962084965664f},
  { -80.5376777919744f,    39.4200315639073f},
  { -78.6900675259798f,    49.6408995433388f},
  { -75.9637565320735f,    59.5012730133354f},
  { -71.5650511770780f,    68.4318705643521f},
  { -63.4349488229220f,    76.0472636044633f},
  { -45.0000000000000f,    81.9505330244716f},
  { -78.6900675259798f,    0.0f},
  { -77.4711922908485f,    6.19039945857186f},
  { -75.9637565320735f,    13.6330222253664f},
  { -74.0546040990772f,    22.3956623300392f},
  { -71.5650511770780f,    32.3115332374239f},
  { -68.1985905136482f,    42.8759886666357f},
  { -63.4349488229220f,    53.3007747995101f},
  { -56.3099324740202f,    62.7479663251258f},
  { -45.0000000000000f,    70.5287793655093f},
  { -26.5650511770780f,    76.0472636044633f},
  { -71.5650511770780f,    0.0f},
  { -69.4439547804165f,    6.67559216607672f},
  { -66.8014094863518f,    14.7143535884335f},
  { -63.4349488229220f,    24.0948425521107f},
  { -59.0362434679265f,    34.4499019879535f},
  { -53.1301023541560f,    45.0f},
  { -45.0f,    54.7356103172453f},
  { -33.6900675259798f,    62.7479663251258f},
  { -18.4349488229220f,    68.4318705643521f},
  { -63.4349488229220f,    0.0f},
  { -60.2551187030578f,    7.07055511696583f},
  { -56.3099324740202f,    15.5013595669370f},
  { -51.3401917459099f,    25.1040902502214f},
  { -45.0000000000000f,    35.2643896827547f},
  { -36.8698976458440f,    45.0000000000000f},
  { -26.5650511770780f,    53.3007747995101f},
  { -14.0362434679265f,    59.5012730133354f},
  { -54.4623222080256f,    0.0f},
  { -50.1944289077348f,    7.29627530101671f},
  { -45.0f,    15.7931690482640f},
  { -38.6598082540901f,    25.1040902502214f},
  { -30.9637565320735f,    34.4499019879535f},
  { -21.8014094863518f,    42.8759886666357f},
  { -11.3099324740202f,    49.6408995433388f},
  { -45.0f,    0.0f},
  { -39.8055710922652f,    7.29627530101671f},
  { -33.6900675259798f,    15.5013595669370f},
  { -26.5650511770780f,    24.0948425521107f},
  { -18.4349488229220f,    32.3115332374238f},
  { -9.46232220802562f,    39.4200315639073f},
  { -35.5376777919744f,    0.0f},
  { -29.7448812969422f,    7.07055511696583f},
  { -23.1985905136482f,    14.7143535884335f},
  { -15.9453959009228f,    22.3956623300392f},
  { -8.13010235415598f,    29.4962084965664f},
  { -26.5650511770780f,    0.0f},
  { -20.5560452195835f,    6.67559216607672f},
  { -14.0362434679265f,    13.6330222253664f},
  { -7.12501634890180f,    20.4104455216585f},
  { -18.4349488229220f,    0.0f},
  { -12.5288077091515f,    6.19039945857186f},
  { -6.34019174590991f,    12.4545763684750f},
  { -11.3099324740202f,    0.0f},
  { -5.71059313749964f,    5.68243848351684f},
  { -5.19442890773481f,    0.0f},
  { 0.0f,    -5.19442890773481f},
  { 5.71059313749964f,    -5.68243848351684f},
  { 12.5288077091515f,    -6.19039945857186f},
  { 20.5560452195835f,    -6.67559216607672f},
  { 29.7448812969422f,    -7.07055511696583f},
  { 39.8055710922652f,    -7.29627530101671f},
  { 50.1944289077348f,    -7.29627530101671f},
  { 60.2551187030578f,    -7.07055511696583f},
  { 69.4439547804165f,    -6.67559216607672f},
  { 77.4711922908485f,    -6.19039945857186f},
  { 84.2894068625003f,    -5.68243848351684f},
  { 90.0f,    -5.19442890773481f},
  { 0.0f,    -11.3099324740202f},
  { 6.34019174590991f,    -12.4545763684750f},
  { 14.0362434679265f,    -13.6330222253664f},
  { 23.1985905136482f,    -14.7143535884335f},
  { 33.6900675259798f,    -15.5013595669370f},
  { 45.0f,    -15.7931690482640f},
  { 56.3099324740202f,    -15.5013595669370f},
  { 66.8014094863518f,    -14.7143535884335f},
  { 75.9637565320735f,    -13.6330222253664f},
  { 83.6598082540901f,    -12.4545763684750f},
  { 90.0f,    -11.3099324740202f},
  { 0.0f,    -18.4349488229220f},
  { 7.12501634890180f,    -20.4104455216585f},
  { 15.9453959009229f,    -22.3956623300392f},
  { 26.5650511770780f,    -24.0948425521107f},
  { 38.6598082540901f,    -25.1040902502214f},
  { 51.3401917459099f,    -25.1040902502214f},
  { 63.4349488229220f,    -24.0948425521107f},
  { 74.0546040990772f,    -22.3956623300392f},
  { 82.8749836510982f,    -20.4104455216585f},
  { 90.0f,    -18.4349488229220f},
  { 0.0f,    -26.5650511770780f},
  { 8.13010235415598f,    -29.4962084965664f},
  { 18.4349488229220f,    -32.3115332374238f},
  { 30.9637565320735f,    -34.4499019879535f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 59.0362434679265f,    -34.4499019879535f},
  { 71.5650511770780f,    -32.3115332374238f},
  { 81.8698976458440f,    -29.4962084965664f},
  { 90.0f,    -26.5650511770780f},
  { 0.0f,    -35.5376777919744f},
  { 9.46232220802562f,    -39.4200315639073f},
  { 21.8014094863518f,    -42.8759886666357f},
  { 36.8698976458440f,    -45.0000000000000f},
  { 53.1301023541560f,    -45.0000000000000f},
  { 68.1985905136482f,    -42.8759886666357f},
  { 80.5376777919744f,    -39.4200315639073f},
  { 90.0f,    -35.5376777919744f},
  { 0.0f,    -45.0f},
  { 11.3099324740202f,    -49.6408995433388f},
  { 26.5650511770780f,    -53.3007747995101f},
  { 45.0f,    -54.7356103172453f},
  { 63.4349488229220f,    -53.3007747995101f},
  { 78.6900675259798f,    -49.6408995433388f},
  { 90.0f,    -45.0f},
  { 0.0f,    -54.4623222080256f},
  { 14.0362434679265f,    -59.5012730133354f},
  { 33.6900675259798f,    -62.7479663251258f},
  { 56.3099324740202f,    -62.7479663251258f},
  { 75.9637565320735f,    -59.5012730133354f},
  { 90.0f,    -54.4623222080256f},
  { 0.0f,    -63.4349488229220f},
  { 18.4349488229220f,    -68.4318705643521f},
  { 45.0000000000000f,    -70.5287793655093f},
  { 71.5650511770780f,    -68.4318705643521f},
  { 90.0f,    -63.4349488229220f},
  { 0.0f,    -71.5650511770780f},
  { 26.5650511770780f,    -76.0472636044633f},
  { 63.4349488229220f,    -76.0472636044633f},
  { 90.0f,    -71.5650511770780f},
  { 0.0f,    -78.6900675259798f},
  { 45.0f,    -81.9505330244716f},
  { 90.0f,    -78.6900675259798f},
  { 0.0f,    -84.8055710922652f},
  { 90.0f,    -84.8055710922652f},
  { 95.7105931374996f,    -5.68243848351684f},
  { 102.528807709152f,    -6.19039945857186f},
  { 110.556045219583f,    -6.67559216607672f},
  { 119.744881296942f,    -7.07055511696583f},
  { 129.805571092265f,    -7.29627530101671f},
  { 140.194428907735f,    -7.29627530101671f},
  { 150.255118703058f,    -7.07055511696583f},
  { 159.443954780417f,    -6.67559216607672f},
  { 167.471192290848f,    -6.19039945857186f},
  { 174.289406862500f,    -5.68243848351684f},
  { 180.0f,    -5.19442890773481f},
  { 96.3401917459099f,    -12.4545763684750f},
  { 104.036243467927f,    -13.6330222253664f},
  { 113.198590513648f,    -14.7143535884335f},
  { 123.690067525980f,    -15.5013595669370f},
  { 135.0f,    -15.7931690482640f},
  { 146.309932474020f,    -15.5013595669370f},
  { 156.801409486352f,    -14.7143535884335f},
  { 165.963756532074f,    -13.6330222253664f},
  { 173.659808254090f,    -12.4545763684750f},
  { 180.0f,    -11.3099324740202f},
  { 97.1250163489018f,    -20.4104455216585f},
  { 105.945395900923f,    -22.3956623300392f},
  { 116.565051177078f,    -24.0948425521107f},
  { 128.659808254090f,    -25.1040902502214f},
  { 141.340191745910f,    -25.1040902502214f},
  { 153.434948822922f,    -24.0948425521107f},
  { 164.054604099077f,    -22.3956623300392f},
  { 172.874983651098f,    -20.4104455216585f},
  { 180.0f,    -18.4349488229220f},
  { 98.1301023541560f,    -29.4962084965664f},
  { 108.434948822922f,    -32.3115332374238f},
  { 120.963756532074f,    -34.4499019879535f},
  { 135.0f,    -35.2643896827547f},
  { 149.036243467926f,    -34.4499019879535f},
  { 161.565051177078f,    -32.3115332374238f},
  { 171.869897645844f,    -29.4962084965664f},
  { 180.0f,    -26.5650511770780f},
  { 99.4623222080256f,    -39.4200315639073f},
  { 111.801409486352f,    -42.8759886666357f},
  { 126.869897645844f,    -45.0f},
  { 143.130102354156f,    -45.0000000000000f},
  { 158.198590513648f,    -42.8759886666357f},
  { 170.537677791974f,    -39.4200315639073f},
  { 180.0f,    -35.5376777919744f},
  { 101.309932474020f,    -49.6408995433388f},
  { 116.565051177078f,    -53.3007747995101f},
  { 135.0f,    -54.7356103172453f},
  { 153.434948822922f,    -53.3007747995101f},
  { 168.690067525980f,    -49.6408995433388f},
  { 180.0f,    -45.0f},
  { 104.036243467927f,    -59.5012730133354f},
  { 123.690067525980f,    -62.7479663251258f},
  { 146.309932474020f,    -62.7479663251258f},
  { 165.963756532074f,    -59.5012730133354f},
  { 180.0f,    -54.4623222080256f},
  { 108.434948822922f,    -68.4318705643521f},
  { 135.0f,    -70.5287793655093f},
  { 161.565051177078f,    -68.4318705643521f},
  { 180.0f,    -63.4349488229220f},
  { 116.565051177078f,    -76.0472636044633f},
  { 153.434948822922f,    -76.0472636044633f},
  { 180.0f,    -71.5650511770780f},
  { 135.0f,    -81.9505330244716f},
  { 180.0f,    -78.6900675259798f},
  { 180.0f,    -84.8055710922652f},
  { -174.289406862500f,    -5.68243848351684f},
  { -167.471192290848f,    -6.19039945857186f},
  { -159.443954780417f,    -6.67559216607672f},
  { -150.255118703058f,    -7.07055511696583f},
  { -140.194428907735f,    -7.29627530101671f},
  { -129.805571092265f,    -7.29627530101671f},
  { -119.744881296942f,    -7.07055511696583f},
  { -110.556045219583f,    -6.67559216607672f},
  { -102.528807709152f,    -6.19039945857186f},
  { -95.7105931374997f,    -5.68243848351684f},
  { -90.0000000000000f,    -5.19442890773481f},
  { -173.659808254090f,    -12.4545763684750f},
  { -165.963756532074f,    -13.6330222253664f},
  { -156.801409486352f,    -14.7143535884335f},
  { -146.309932474020f,    -15.5013595669370f},
  { -135.0f,    -15.7931690482640f},
  { -123.690067525980f,    -15.5013595669370f},
  { -113.198590513648f,    -14.7143535884335f},
  { -104.036243467927f,    -13.6330222253664f},
  { -96.3401917459099f,    -12.4545763684750f},
  { -90.0f,    -11.3099324740202f},
  { -172.874983651098f,    -20.4104455216585f},
  { -164.054604099077f,    -22.3956623300392f},
  { -153.434948822922f,    -24.0948425521107f},
  { -141.340191745910f,    -25.1040902502214f},
  { -128.659808254090f,    -25.1040902502214f},
  { -116.565051177078f,    -24.0948425521107f},
  { -105.945395900923f,    -22.3956623300392f},
  { -97.1250163489018f,    -20.4104455216585f},
  { -90.0f,    -18.4349488229220f},
  { -171.869897645844f,    -29.4962084965664f},
  { -161.565051177078f,    -32.3115332374238f},
  { -149.036243467926f,    -34.4499019879535f},
  { -135.0f,    -35.2643896827547f},
  { -120.963756532074f,    -34.4499019879535f},
  { -108.434948822922f,    -32.3115332374238f},
  { -98.1301023541560f,    -29.4962084965664f},
  { -90.0f,    -26.5650511770780f},
  { -170.537677791974f,    -39.4200315639073f},
  { -158.198590513648f,    -42.8759886666357f},
  { -143.130102354156f,    -45.0000000000000f},
  { -126.869897645844f,    -45.0f},
  { -111.801409486352f,    -42.8759886666357f},
  { -99.4623222080256f,    -39.4200315639073f},
  { -90.0f,    -35.5376777919744f},
  { -168.690067525980f,    -49.6408995433388f},
  { -153.434948822922f,    -53.3007747995101f},
  { -135.0f,    -54.7356103172453f},
  { -116.565051177078f,    -53.3007747995101f},
  { -101.309932474020f,    -49.6408995433388f},
  { -90.0f,    -45.0f},
  { -165.963756532074f,    -59.5012730133354f},
  { -146.309932474020f,    -62.7479663251258f},
  { -123.690067525980f,    -62.7479663251258f},
  { -104.036243467927f,    -59.5012730133354f},
  { -90.0f,    -54.4623222080256f},
  { -161.565051177078f,    -68.4318705643521f},
  { -135.0f,    -70.5287793655093f},
  { -108.434948822922f,    -68.4318705643521f},
  { -90.0f,    -63.4349488229220f},
  { -153.434948822922f,    -76.0472636044633f},
  { -116.565051177078f,    -76.0472636044633f},
  { -90.0f,    -71.5650511770780f},
  { -135.0f,    -81.9505330244716f},
  { -90.0f,    -78.6900675259798f},
  { -90.0f,    -84.8055710922652f},
  { -84.2894068625004f,    -5.68243848351684f},
  { -77.4711922908485f,    -6.19039945857186f},
  { -69.4439547804165f,    -6.67559216607672f},
  { -60.2551187030578f,    -7.07055511696583f},
  { -50.1944289077348f,    -7.29627530101671f},
  { -39.8055710922652f,    -7.29627530101671f},
  { -29.7448812969422f,    -7.07055511696583f},
  { -20.5560452195835f,    -6.67559216607672f},
  { -12.5288077091515f,    -6.19039945857186f},
  { -5.71059313749965f,    -5.68243848351684f},
  { -83.6598082540901f,    -12.4545763684750f},
  { -75.9637565320735f,    -13.6330222253664f},
  { -66.8014094863518f,    -14.7143535884335f},
  { -56.3099324740202f,    -15.5013595669370f},
  { -45.0f,    -15.7931690482640f},
  { -33.6900675259798f,    -15.5013595669370f},
  { -23.1985905136482f,    -14.7143535884335f},
  { -14.0362434679265f,    -13.6330222253664f},
  { -6.34019174590991f,    -12.4545763684750f},
  { -82.8749836510982f,    -20.4104455216585f},
  { -74.0546040990772f,    -22.3956623300392f},
  { -63.4349488229220f,    -24.0948425521107f},
  { -51.3401917459099f,    -25.1040902502214f},
  { -38.6598082540901f,    -25.1040902502214f},
  { -26.5650511770780f,    -24.0948425521107f},
  { -15.9453959009228f,    -22.3956623300392f},
  { -7.12501634890180f,    -20.4104455216585f},
  { -81.8698976458440f,    -29.4962084965664f},
  { -71.5650511770780f,    -32.3115332374238f},
  { -59.0362434679265f,    -34.4499019879535f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -30.9637565320735f,    -34.4499019879535f},
  { -18.4349488229220f,    -32.3115332374238f},
  { -8.13010235415598f,    -29.4962084965664f},
  { -80.5376777919744f,    -39.4200315639073f},
  { -68.1985905136482f,    -42.8759886666357f},
  { -53.1301023541560f,    -45.0000000000000f},
  { -36.8698976458440f,    -45.0000000000000f},
  { -21.8014094863518f,    -42.8759886666357f},
  { -9.46232220802562f,    -39.4200315639073f},
  { -78.6900675259798f,    -49.6408995433388f},
  { -63.4349488229220f,    -53.3007747995101f},
  { -45.0f,    -54.7356103172453f},
  { -26.5650511770780f,    -53.3007747995101f},
  { -11.3099324740202f,    -49.6408995433388f},
  { -75.9637565320735f,    -59.5012730133354f},
  { -56.3099324740202f,    -62.7479663251258f},
  { -33.6900675259798f,    -62.7479663251258f},
  { -14.0362434679265f,    -59.5012730133354f},
  { -71.5650511770780f,    -68.4318705643521f},
  { -45.0000000000000f,    -70.5287793655093f},
  { -18.4349488229220f,    -68.4318705643521f},
  { -63.4349488229220f,    -76.0472636044633f},
  { -26.5650511770780f,    -76.0472636044633f},
  { -45.0f,    -81.9505330244716f}};

/* 3LD geosphere 'oct' [13 0] (678 points) */
const float __geosphere_oct_13_0_dirs_deg[678][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    4.76364169072618f},
  { 0.0f,    10.3048464687660f},
  { 0.0f,    16.6992442339936f},
  { 0.0f,    23.9624889745782f},
  { 0.0f,    32.0053832080835f},
  { 0.0f,    40.6012946450045f},
  { 0.0f,    49.3987053549955f},
  { 0.0f,    57.9946167919165f},
  { 0.0f,    66.0375110254218f},
  { 0.0f,    73.3007557660064f},
  { 0.0f,    79.6951535312340f},
  { 0.0f,    85.2363583092738f},
  { 4.76364169072618f,    0.0f},
  { 5.19442890773481f,    5.17321244611695f},
  { 5.71059313749964f,    11.2552397326621f},
  { 6.34019174590991f,    18.3297537440740f},
  { 7.12501634890180f,    26.3877999612430f},
  { 8.13010235415598f,    35.2643896827547f},
  { 9.46232220802562f,    44.6075508793597f},
  { 11.3099324740202f,    53.9292313492037f},
  { 14.0362434679265f,    62.7339555492672f},
  { 18.4349488229220f,    70.6403513388797f},
  { 26.5650511770780f,    77.3956173516208f},
  { 45.0000000000000f,    82.6739630468477f},
  { 89.9999999999999f,    85.2363583092738f},
  { 10.3048464687660f,    0.0f},
  { 11.3099324740202f,    5.60040918481662f},
  { 12.5288077091515f,    12.2395582977766f},
  { 14.0362434679265f,    19.9915607271414f},
  { 15.9453959009229f,    28.7862533091452f},
  { 18.4349488229220f,    38.3288181014559f},
  { 21.8014094863518f,    48.0911521181967f},
  { 26.5650511770780f,    57.4264443008031f},
  { 33.6900675259798f,    65.7392004151320f},
  { 45.0f,    72.5536476627867f},
  { 63.4349488229220f,    77.3956173516208f},
  { 90.0000000000001f,    79.6951535312340f},
  { 16.6992442339936f,    0.0f},
  { 18.4349488229220f,    6.01728486403203f},
  { 20.5560452195835f,    13.1747121595676f},
  { 23.1985905136482f,    21.5004376144442f},
  { 26.5650511770780f,    30.8069811171052f},
  { 30.9637565320735f,    40.6128551760767f},
  { 36.8698976458440f,    50.1944289077348f},
  { 45.0000000000000f,    58.7803015526316f},
  { 56.3099324740202f,    65.7392004151319f},
  { 71.5650511770780f,    70.6403513388797f},
  { 90.0f,    73.3007557660064f},
  { 23.9624889745782f,    0.0f},
  { 26.5650511770780f,    6.37937020844280f},
  { 29.7448812969422f,    13.9320915492581f},
  { 33.6900675259798f,    22.5885387879849f},
  { 38.6598082540901f,    31.9928170001817f},
  { 45.0f,    41.4729343089405f},
  { 53.1301023541560f,    50.1944289077348f},
  { 63.4349488229220f,    57.4264443008031f},
  { 75.9637565320735f,    62.7339555492672f},
  { 90.0f,    66.0375110254218f},
  { 32.0053832080835f,    0.0f},
  { 35.5376777919744f,    6.63073776807866f},
  { 39.8055710922652f,    14.3633026740704f},
  { 45.0000000000000f,    22.9897677736089f},
  { 51.3401917459099f,    31.9928170001817f},
  { 59.0362434679265f,    40.6128551760767f},
  { 68.1985905136482f,    48.0911521181967f},
  { 78.6900675259798f,    53.9292313492037f},
  { 90.0f,    57.9946167919165f},
  { 40.6012946450045f,    0.0f},
  { 45.0000000000000f,    6.72136933936946f},
  { 50.1944289077348f,    14.3633026740704f},
  { 56.3099324740202f,    22.5885387879849f},
  { 63.4349488229220f,    30.8069811171052f},
  { 71.5650511770780f,    38.3288181014559f},
  { 80.5376777919744f,    44.6075508793597f},
  { 90.0f,    49.3987053549955f},
  { 49.3987053549955f,    0.0f},
  { 54.4623222080256f,    6.63073776807866f},
  { 60.2551187030578f,    13.9320915492581f},
  { 66.8014094863518f,    21.5004376144442f},
  { 74.0546040990771f,    28.7862533091452f},
  { 81.8698976458440f,    35.2643896827547f},
  { 90.0f,    40.6012946450045f},
  { 57.9946167919165f,    0.0f},
  { 63.4349488229220f,    6.37937020844280f},
  { 69.4439547804165f,    13.1747121595676f},
  { 75.9637565320735f,    19.9915607271414f},
  { 82.8749836510982f,    26.3877999612430f},
  { 90.0f,    32.0053832080835f},
  { 66.0375110254218f,    0.0f},
  { 71.5650511770780f,    6.01728486403203f},
  { 77.4711922908485f,    12.2395582977766f},
  { 83.6598082540901f,    18.3297537440740f},
  { 90.0f,    23.9624889745782f},
  { 73.3007557660064f,    0.0f},
  { 78.6900675259798f,    5.60040918481662f},
  { 84.2894068625004f,    11.2552397326621f},
  { 90.0f,    16.6992442339936f},
  { 79.6951535312340f,    0.0f},
  { 84.8055710922652f,    5.17321244611695f},
  { 90.0f,    10.3048464687660f},
  { 85.2363583092738f,    0.0f},
  { 90.0f,    4.76364169072617f},
  { 94.7636416907262f,    0.0f},
  { 95.1944289077348f,    5.17321244611695f},
  { 95.7105931374996f,    11.2552397326621f},
  { 96.3401917459099f,    18.3297537440740f},
  { 97.1250163489018f,    26.3877999612430f},
  { 98.1301023541560f,    35.2643896827547f},
  { 99.4623222080256f,    44.6075508793597f},
  { 101.309932474020f,    53.9292313492037f},
  { 104.036243467927f,    62.7339555492672f},
  { 108.434948822922f,    70.6403513388797f},
  { 116.565051177078f,    77.3956173516208f},
  { 135.000000000000f,    82.6739630468477f},
  { 180.000000000000f,    85.2363583092738f},
  { 100.304846468766f,    0.0f},
  { 101.309932474020f,    5.60040918481662f},
  { 102.528807709152f,    12.2395582977766f},
  { 104.036243467927f,    19.9915607271414f},
  { 105.945395900923f,    28.7862533091452f},
  { 108.434948822922f,    38.3288181014559f},
  { 111.801409486352f,    48.0911521181967f},
  { 116.565051177078f,    57.4264443008031f},
  { 123.690067525980f,    65.7392004151319f},
  { 135.0f,    72.5536476627867f},
  { 153.434948822922f,    77.3956173516208f},
  { -180.000000000000f,    79.6951535312340f},
  { 106.699244233994f,    0.0f},
  { 108.434948822922f,    6.01728486403203f},
  { 110.556045219583f,    13.1747121595676f},
  { 113.198590513648f,    21.5004376144442f},
  { 116.565051177078f,    30.8069811171052f},
  { 120.963756532074f,    40.6128551760767f},
  { 126.869897645844f,    50.1944289077348f},
  { 135.0f,    58.7803015526316f},
  { 146.309932474020f,    65.7392004151319f},
  { 161.565051177078f,    70.6403513388797f},
  { 180.0f,    73.3007557660064f},
  { 113.962488974578f,    0.0f},
  { 116.565051177078f,    6.37937020844280f},
  { 119.744881296942f,    13.9320915492581f},
  { 123.690067525980f,    22.5885387879849f},
  { 128.659808254090f,    31.9928170001817f},
  { 135.0f,    41.4729343089405f},
  { 143.130102354156f,    50.1944289077348f},
  { 153.434948822922f,    57.4264443008031f},
  { 165.963756532074f,    62.7339555492672f},
  { 180.0f,    66.0375110254218f},
  { 122.005383208084f,    0.0f},
  { 125.537677791974f,    6.63073776807866f},
  { 129.805571092265f,    14.3633026740704f},
  { 135.0f,    22.9897677736089f},
  { 141.340191745910f,    31.9928170001817f},
  { 149.036243467926f,    40.6128551760767f},
  { 158.198590513648f,    48.0911521181967f},
  { 168.690067525980f,    53.9292313492037f},
  { 180.0f,    57.9946167919165f},
  { 130.601294645005f,    0.0f},
  { 135.0f,    6.72136933936946f},
  { 140.194428907735f,    14.3633026740704f},
  { 146.309932474020f,    22.5885387879849f},
  { 153.434948822922f,    30.8069811171052f},
  { 161.565051177078f,    38.3288181014559f},
  { 170.537677791974f,    44.6075508793597f},
  { 180.0f,    49.3987053549955f},
  { 139.398705354996f,    0.0f},
  { 144.462322208026f,    6.63073776807866f},
  { 150.255118703058f,    13.9320915492581f},
  { 156.801409486352f,    21.5004376144442f},
  { 164.054604099077f,    28.7862533091452f},
  { 171.869897645844f,    35.2643896827547f},
  { 180.0f,    40.6012946450045f},
  { 147.994616791917f,    0.0f},
  { 153.434948822922f,    6.37937020844280f},
  { 159.443954780417f,    13.1747121595676f},
  { 165.963756532074f,    19.9915607271414f},
  { 172.874983651098f,    26.3877999612430f},
  { 180.0f,    32.0053832080835f},
  { 156.037511025422f,    0.0f},
  { 161.565051177078f,    6.01728486403203f},
  { 167.471192290848f,    12.2395582977766f},
  { 173.659808254090f,    18.3297537440740f},
  { 180.0f,    23.9624889745782f},
  { 163.300755766006f,    0.0f},
  { 168.690067525980f,    5.60040918481661f},
  { 174.289406862500f,    11.2552397326621f},
  { 180.0f,    16.6992442339936f},
  { 169.695153531234f,    0.0f},
  { 174.805571092265f,    5.17321244611695f},
  { 180.0f,    10.3048464687660f},
  { 175.236358309274f,    0.0f},
  { 180.0f,    4.76364169072617f},
  { -175.236358309274f,    0.0f},
  { -174.805571092265f,    5.17321244611695f},
  { -174.289406862500f,    11.2552397326621f},
  { -173.659808254090f,    18.3297537440740f},
  { -172.874983651098f,    26.3877999612430f},
  { -171.869897645844f,    35.2643896827547f},
  { -170.537677791974f,    44.6075508793597f},
  { -168.690067525980f,    53.9292313492036f},
  { -165.963756532074f,    62.7339555492672f},
  { -161.565051177078f,    70.6403513388797f},
  { -153.434948822922f,    77.3956173516208f},
  { -135.0f,    82.6739630468477f},
  { -90.0000000000001f,    85.2363583092738f},
  { -169.695153531234f,    0.0f},
  { -168.690067525980f,    5.60040918481662f},
  { -167.471192290848f,    12.2395582977766f},
  { -165.963756532074f,    19.9915607271414f},
  { -164.054604099077f,    28.7862533091452f},
  { -161.565051177078f,    38.3288181014559f},
  { -158.198590513648f,    48.0911521181967f},
  { -153.434948822922f,    57.4264443008031f},
  { -146.309932474020f,    65.7392004151320f},
  { -135.0f,    72.5536476627867f},
  { -116.565051177078f,    77.3956173516208f},
  { -90.0000000000000f,    79.6951535312340f},
  { -163.300755766006f,    0.0f},
  { -161.565051177078f,    6.01728486403203f},
  { -159.443954780417f,    13.1747121595676f},
  { -156.801409486352f,    21.5004376144442f},
  { -153.434948822922f,    30.8069811171052f},
  { -149.036243467926f,    40.6128551760767f},
  { -143.130102354156f,    50.1944289077348f},
  { -135.0f,    58.7803015526316f},
  { -123.690067525980f,    65.7392004151319f},
  { -108.434948822922f,    70.6403513388797f},
  { -90.0f,    73.3007557660064f},
  { -156.037511025422f,    0.0f},
  { -153.434948822922f,    6.37937020844280f},
  { -150.255118703058f,    13.9320915492581f},
  { -146.309932474020f,    22.5885387879849f},
  { -141.340191745910f,    31.9928170001817f},
  { -135.0f,    41.4729343089405f},
  { -126.869897645844f,    50.1944289077348f},
  { -116.565051177078f,    57.4264443008031f},
  { -104.036243467927f,    62.7339555492672f},
  { -90.0f,    66.0375110254218f},
  { -147.994616791916f,    0.0f},
  { -144.462322208026f,    6.63073776807866f},
  { -140.194428907735f,    14.3633026740704f},
  { -135.0f,    22.9897677736089f},
  { -128.659808254090f,    31.9928170001817f},
  { -120.963756532074f,    40.6128551760767f},
  { -111.801409486352f,    48.0911521181967f},
  { -101.309932474020f,    53.9292313492037f},
  { -90.0f,    57.9946167919165f},
  { -139.398705354996f,    0.0f},
  { -135.0f,    6.72136933936946f},
  { -129.805571092265f,    14.3633026740704f},
  { -123.690067525980f,    22.5885387879849f},
  { -116.565051177078f,    30.8069811171052f},
  { -108.434948822922f,    38.3288181014559f},
  { -99.4623222080256f,    44.6075508793597f},
  { -90.0f,    49.3987053549955f},
  { -130.601294645005f,    0.0f},
  { -125.537677791974f,    6.63073776807866f},
  { -119.744881296942f,    13.9320915492581f},
  { -113.198590513648f,    21.5004376144442f},
  { -105.945395900923f,    28.7862533091452f},
  { -98.1301023541560f,    35.2643896827547f},
  { -90.0000000000000f,    40.6012946450045f},
  { -122.005383208084f,    0.0f},
  { -116.565051177078f,    6.37937020844280f},
  { -110.556045219583f,    13.1747121595676f},
  { -104.036243467927f,    19.9915607271414f},
  { -97.1250163489018f,    26.3877999612430f},
  { -90.0f,    32.0053832080835f},
  { -113.962488974578f,    0.0f},
  { -108.434948822922f,    6.01728486403203f},
  { -102.528807709152f,    12.2395582977766f},
  { -96.3401917459099f,    18.3297537440740f},
  { -90.0f,    23.9624889745782f},
  { -106.699244233994f,    0.0f},
  { -101.309932474020f,    5.60040918481662f},
  { -95.7105931374996f,    11.2552397326621f},
  { -90.0f,    16.6992442339936f},
  { -100.304846468766f,    0.0f},
  { -95.1944289077348f,    5.17321244611695f},
  { -90.0f,    10.3048464687660f},
  { -94.7636416907262f,    0.0f},
  { -90.0f,    4.76364169072617f},
  { -85.2363583092738f,    0.0f},
  { -84.8055710922652f,    5.17321244611695f},
  { -84.2894068625004f,    11.2552397326621f},
  { -83.6598082540901f,    18.3297537440740f},
  { -82.8749836510982f,    26.3877999612430f},
  { -81.8698976458440f,    35.2643896827547f},
  { -80.5376777919744f,    44.6075508793597f},
  { -78.6900675259798f,    53.9292313492037f},
  { -75.9637565320735f,    62.7339555492672f},
  { -71.5650511770780f,    70.6403513388797f},
  { -63.4349488229220f,    77.3956173516208f},
  { -45.0000000000000f,    82.6739630468477f},
  { -79.6951535312340f,    0.0f},
  { -78.6900675259798f,    5.60040918481662f},
  { -77.4711922908485f,    12.2395582977766f},
  { -75.9637565320735f,    19.9915607271414f},
  { -74.0546040990771f,    28.7862533091452f},
  { -71.5650511770780f,    38.3288181014559f},
  { -68.1985905136482f,    48.0911521181967f},
  { -63.4349488229220f,    57.4264443008031f},
  { -56.3099324740202f,    65.7392004151319f},
  { -45.0f,    72.5536476627867f},
  { -26.5650511770780f,    77.3956173516208f},
  { -73.3007557660064f,    0.0f},
  { -71.5650511770780f,    6.01728486403203f},
  { -69.4439547804165f,    13.1747121595676f},
  { -66.8014094863518f,    21.5004376144442f},
  { -63.4349488229220f,    30.8069811171052f},
  { -59.0362434679265f,    40.6128551760767f},
  { -53.1301023541560f,    50.1944289077348f},
  { -45.0000000000000f,    58.7803015526316f},
  { -33.6900675259798f,    65.7392004151319f},
  { -18.4349488229220f,    70.6403513388797f},
  { -66.0375110254218f,    0.0f},
  { -63.4349488229220f,    6.37937020844280f},
  { -60.2551187030578f,    13.9320915492581f},
  { -56.3099324740202f,    22.5885387879849f},
  { -51.3401917459099f,    31.9928170001817f},
  { -45.0000000000000f,    41.4729343089405f},
  { -36.8698976458440f,    50.1944289077348f},
  { -26.5650511770780f,    57.4264443008031f},
  { -14.0362434679265f,    62.7339555492672f},
  { -57.9946167919165f,    0.0f},
  { -54.4623222080256f,    6.63073776807866f},
  { -50.1944289077348f,    14.3633026740704f},
  { -45.0000000000000f,    22.9897677736089f},
  { -38.6598082540901f,    31.9928170001817f},
  { -30.9637565320735f,    40.6128551760767f},
  { -21.8014094863518f,    48.0911521181967f},
  { -11.3099324740202f,    53.9292313492037f},
  { -49.3987053549955f,    0.0f},
  { -45.0f,    6.72136933936946f},
  { -39.8055710922652f,    14.3633026740704f},
  { -33.6900675259798f,    22.5885387879849f},
  { -26.5650511770780f,    30.8069811171052f},
  { -18.4349488229220f,    38.3288181014559f},
  { -9.46232220802561f,    44.6075508793597f},
  { -40.6012946450045f,    0.0f},
  { -35.5376777919744f,    6.63073776807866f},
  { -29.7448812969422f,    13.9320915492581f},
  { -23.1985905136482f,    21.5004376144442f},
  { -15.9453959009229f,    28.7862533091452f},
  { -8.13010235415598f,    35.2643896827547f},
  { -32.0053832080835f,    0.0f},
  { -26.5650511770780f,    6.37937020844280f},
  { -20.5560452195835f,    13.1747121595676f},
  { -14.0362434679265f,    19.9915607271414f},
  { -7.12501634890180f,    26.3877999612430f},
  { -23.9624889745782f,    0.0f},
  { -18.4349488229220f,    6.01728486403203f},
  { -12.5288077091515f,    12.2395582977766f},
  { -6.34019174590991f,    18.3297537440740f},
  { -16.6992442339936f,    0.0f},
  { -11.3099324740202f,    5.60040918481661f},
  { -5.71059313749964f,    11.2552397326621f},
  { -10.3048464687660f,    0.0f},
  { -5.19442890773481f,    5.17321244611695f},
  { -4.76364169072617f,    0.0f},
  { 0.0f,    -4.76364169072618f},
  { 5.19442890773481f,    -5.17321244611695f},
  { 11.3099324740202f,    -5.60040918481662f},
  { 18.4349488229220f,    -6.01728486403203f},
  { 26.5650511770780f,    -6.37937020844280f},
  { 35.5376777919744f,    -6.63073776807866f},
  { 45.0000000000000f,    -6.72136933936946f},
  { 54.4623222080256f,    -6.63073776807866f},
  { 63.4349488229220f,    -6.37937020844280f},
  { 71.5650511770780f,    -6.01728486403203f},
  { 78.6900675259798f,    -5.60040918481662f},
  { 84.8055710922652f,    -5.17321244611695f},
  { 90.0f,    -4.76364169072618f},
  { 0.0f,    -10.3048464687660f},
  { 5.71059313749964f,    -11.2552397326621f},
  { 12.5288077091515f,    -12.2395582977766f},
  { 20.5560452195835f,    -13.1747121595676f},
  { 29.7448812969422f,    -13.9320915492581f},
  { 39.8055710922652f,    -14.3633026740704f},
  { 50.1944289077348f,    -14.3633026740704f},
  { 60.2551187030578f,    -13.9320915492581f},
  { 69.4439547804165f,    -13.1747121595676f},
  { 77.4711922908485f,    -12.2395582977766f},
  { 84.2894068625004f,    -11.2552397326621f},
  { 90.0000000000000f,    -10.3048464687660f},
  { 0.0f,    -16.6992442339936f},
  { 6.34019174590991f,    -18.3297537440740f},
  { 14.0362434679265f,    -19.9915607271414f},
  { 23.1985905136482f,    -21.5004376144442f},
  { 33.6900675259798f,    -22.5885387879849f},
  { 45.0000000000000f,    -22.9897677736089f},
  { 56.3099324740202f,    -22.5885387879849f},
  { 66.8014094863518f,    -21.5004376144442f},
  { 75.9637565320735f,    -19.9915607271414f},
  { 83.6598082540901f,    -18.3297537440740f},
  { 90.0f,    -16.6992442339936f},
  { 0.0f,    -23.9624889745782f},
  { 7.12501634890180f,    -26.3877999612430f},
  { 15.9453959009229f,    -28.7862533091452f},
  { 26.5650511770780f,    -30.8069811171052f},
  { 38.6598082540901f,    -31.9928170001817f},
  { 51.3401917459099f,    -31.9928170001817f},
  { 63.4349488229220f,    -30.8069811171052f},
  { 74.0546040990771f,    -28.7862533091452f},
  { 82.8749836510982f,    -26.3877999612430f},
  { 90.0f,    -23.9624889745782f},
  { 0.0f,    -32.0053832080835f},
  { 8.13010235415598f,    -35.2643896827547f},
  { 18.4349488229220f,    -38.3288181014559f},
  { 30.9637565320735f,    -40.6128551760767f},
  { 45.0f,    -41.4729343089405f},
  { 59.0362434679265f,    -40.6128551760767f},
  { 71.5650511770780f,    -38.3288181014559f},
  { 81.8698976458440f,    -35.2643896827547f},
  { 90.0f,    -32.0053832080835f},
  { 0.0f,    -40.6012946450045f},
  { 9.46232220802562f,    -44.6075508793597f},
  { 21.8014094863518f,    -48.0911521181967f},
  { 36.8698976458440f,    -50.1944289077348f},
  { 53.1301023541560f,    -50.1944289077348f},
  { 68.1985905136482f,    -48.0911521181967f},
  { 80.5376777919744f,    -44.6075508793597f},
  { 90.0f,    -40.6012946450045f},
  { 0.0f,    -49.3987053549955f},
  { 11.3099324740202f,    -53.9292313492037f},
  { 26.5650511770780f,    -57.4264443008031f},
  { 45.0000000000000f,    -58.7803015526316f},
  { 63.4349488229220f,    -57.4264443008031f},
  { 78.6900675259798f,    -53.9292313492037f},
  { 90.0f,    -49.3987053549955f},
  { 0.0f,    -57.9946167919165f},
  { 14.0362434679265f,    -62.7339555492672f},
  { 33.6900675259798f,    -65.7392004151320f},
  { 56.3099324740202f,    -65.7392004151320f},
  { 75.9637565320735f,    -62.7339555492672f},
  { 90.0f,    -57.9946167919165f},
  { 0.0f,    -66.0375110254218f},
  { 18.4349488229220f,    -70.6403513388797f},
  { 45.0f,    -72.5536476627867f},
  { 71.5650511770780f,    -70.6403513388797f},
  { 90.0f,    -66.0375110254218f},
  { 0.0f,    -73.3007557660064f},
  { 26.5650511770780f,    -77.3956173516208f},
  { 63.4349488229220f,    -77.3956173516208f},
  { 90.0f,    -73.3007557660064f},
  { 0.0f,    -79.6951535312340f},
  { 45.0f,    -82.6739630468477f},
  { 90.0f,    -79.6951535312340f},
  { 0.0f,    -85.2363583092738f},
  { 90.0f,    -85.2363583092738f},
  { 95.1944289077348f,    -5.17321244611695f},
  { 101.309932474020f,    -5.60040918481662f},
  { 108.434948822922f,    -6.01728486403203f},
  { 116.565051177078f,    -6.37937020844280f},
  { 125.537677791974f,    -6.63073776807866f},
  { 135.0f,    -6.72136933936946f},
  { 144.462322208026f,    -6.63073776807866f},
  { 153.434948822922f,    -6.37937020844280f},
  { 161.565051177078f,    -6.01728486403203f},
  { 168.690067525980f,    -5.60040918481662f},
  { 174.805571092265f,    -5.17321244611695f},
  { 180.0f,    -4.76364169072618f},
  { 95.7105931374997f,    -11.2552397326621f},
  { 102.528807709152f,    -12.2395582977766f},
  { 110.556045219583f,    -13.1747121595676f},
  { 119.744881296942f,    -13.9320915492581f},
  { 129.805571092265f,    -14.3633026740704f},
  { 140.194428907735f,    -14.3633026740704f},
  { 150.255118703058f,    -13.9320915492581f},
  { 159.443954780417f,    -13.1747121595676f},
  { 167.471192290848f,    -12.2395582977766f},
  { 174.289406862500f,    -11.2552397326621f},
  { -180.0f,    -10.3048464687660f},
  { 96.3401917459099f,    -18.3297537440740f},
  { 104.036243467927f,    -19.9915607271414f},
  { 113.198590513648f,    -21.5004376144442f},
  { 123.690067525980f,    -22.5885387879849f},
  { 135.0f,    -22.9897677736089f},
  { 146.309932474020f,    -22.5885387879849f},
  { 156.801409486352f,    -21.5004376144442f},
  { 165.963756532074f,    -19.9915607271414f},
  { 173.659808254090f,    -18.3297537440740f},
  { 180.0f,    -16.6992442339936f},
  { 97.1250163489018f,    -26.3877999612430f},
  { 105.945395900923f,    -28.7862533091452f},
  { 116.565051177078f,    -30.8069811171052f},
  { 128.659808254090f,    -31.9928170001817f},
  { 141.340191745910f,    -31.9928170001817f},
  { 153.434948822922f,    -30.8069811171052f},
  { 164.054604099077f,    -28.7862533091452f},
  { 172.874983651098f,    -26.3877999612430f},
  { 180.0f,    -23.9624889745782f},
  { 98.1301023541560f,    -35.2643896827547f},
  { 108.434948822922f,    -38.3288181014559f},
  { 120.963756532074f,    -40.6128551760767f},
  { 135.0f,    -41.4729343089405f},
  { 149.036243467926f,    -40.6128551760767f},
  { 161.565051177078f,    -38.3288181014559f},
  { 171.869897645844f,    -35.2643896827547f},
  { 180.0f,    -32.0053832080835f},
  { 99.4623222080256f,    -44.6075508793597f},
  { 111.801409486352f,    -48.0911521181967f},
  { 126.869897645844f,    -50.1944289077348f},
  { 143.130102354156f,    -50.1944289077348f},
  { 158.198590513648f,    -48.0911521181967f},
  { 170.537677791974f,    -44.6075508793597f},
  { 180.0f,    -40.6012946450045f},
  { 101.309932474020f,    -53.9292313492037f},
  { 116.565051177078f,    -57.4264443008031f},
  { 135.0f,    -58.7803015526316f},
  { 153.434948822922f,    -57.4264443008031f},
  { 168.690067525980f,    -53.9292313492036f},
  { 180.0f,    -49.3987053549955f},
  { 104.036243467927f,    -62.7339555492672f},
  { 123.690067525980f,    -65.7392004151319f},
  { 146.309932474020f,    -65.7392004151320f},
  { 165.963756532074f,    -62.7339555492672f},
  { 180.0f,    -57.9946167919165f},
  { 108.434948822922f,    -70.6403513388797f},
  { 135.0f,    -72.5536476627867f},
  { 161.565051177078f,    -70.6403513388797f},
  { 180.0f,    -66.0375110254218f},
  { 116.565051177078f,    -77.3956173516208f},
  { 153.434948822922f,    -77.3956173516208f},
  { 180.0f,    -73.3007557660064f},
  { 135.0f,    -82.6739630468477f},
  { 180.0f,    -79.6951535312340f},
  { 180.0f,    -85.2363583092738f},
  { -174.805571092265f,    -5.17321244611695f},
  { -168.690067525980f,    -5.60040918481662f},
  { -161.565051177078f,    -6.01728486403203f},
  { -153.434948822922f,    -6.37937020844280f},
  { -144.462322208026f,    -6.63073776807866f},
  { -135.0f,    -6.72136933936946f},
  { -125.537677791974f,    -6.63073776807866f},
  { -116.565051177078f,    -6.37937020844280f},
  { -108.434948822922f,    -6.01728486403203f},
  { -101.309932474020f,    -5.60040918481662f},
  { -95.1944289077348f,    -5.17321244611695f},
  { -90.0000000000000f,    -4.76364169072618f},
  { -174.289406862500f,    -11.2552397326621f},
  { -167.471192290848f,    -12.2395582977766f},
  { -159.443954780417f,    -13.1747121595676f},
  { -150.255118703058f,    -13.9320915492581f},
  { -140.194428907735f,    -14.3633026740704f},
  { -129.805571092265f,    -14.3633026740704f},
  { -119.744881296942f,    -13.9320915492581f},
  { -110.556045219583f,    -13.1747121595676f},
  { -102.528807709152f,    -12.2395582977766f},
  { -95.7105931374996f,    -11.2552397326621f},
  { -90.0f,    -10.3048464687660f},
  { -173.659808254090f,    -18.3297537440740f},
  { -165.963756532074f,    -19.9915607271414f},
  { -156.801409486352f,    -21.5004376144442f},
  { -146.309932474020f,    -22.5885387879849f},
  { -135.0f,    -22.9897677736089f},
  { -123.690067525980f,    -22.5885387879849f},
  { -113.198590513648f,    -21.5004376144442f},
  { -104.036243467927f,    -19.9915607271414f},
  { -96.3401917459099f,    -18.3297537440740f},
  { -90.0f,    -16.6992442339936f},
  { -172.874983651098f,    -26.3877999612430f},
  { -164.054604099077f,    -28.7862533091452f},
  { -153.434948822922f,    -30.8069811171052f},
  { -141.340191745910f,    -31.9928170001817f},
  { -128.659808254090f,    -31.9928170001817f},
  { -116.565051177078f,    -30.8069811171052f},
  { -105.945395900923f,    -28.7862533091452f},
  { -97.1250163489018f,    -26.3877999612430f},
  { -90.0f,    -23.9624889745782f},
  { -171.869897645844f,    -35.2643896827547f},
  { -161.565051177078f,    -38.3288181014559f},
  { -149.036243467926f,    -40.6128551760767f},
  { -135.0f,    -41.4729343089405f},
  { -120.963756532074f,    -40.6128551760767f},
  { -108.434948822922f,    -38.3288181014559f},
  { -98.1301023541560f,    -35.2643896827547f},
  { -90.0f,    -32.0053832080835f},
  { -170.537677791974f,    -44.6075508793597f},
  { -158.198590513648f,    -48.0911521181967f},
  { -143.130102354156f,    -50.1944289077348f},
  { -126.869897645844f,    -50.1944289077348f},
  { -111.801409486352f,    -48.0911521181967f},
  { -99.4623222080256f,    -44.6075508793597f},
  { -90.0f,    -40.6012946450045f},
  { -168.690067525980f,    -53.9292313492036f},
  { -153.434948822922f,    -57.4264443008031f},
  { -135.0f,    -58.7803015526316f},
  { -116.565051177078f,    -57.4264443008031f},
  { -101.309932474020f,    -53.9292313492037f},
  { -90.0000000000000f,    -49.3987053549955f},
  { -165.963756532074f,    -62.7339555492672f},
  { -146.309932474020f,    -65.7392004151320f},
  { -123.690067525980f,    -65.7392004151319f},
  { -104.036243467927f,    -62.7339555492672f},
  { -90.0f,    -57.9946167919165f},
  { -161.565051177078f,    -70.6403513388797f},
  { -135.0f,    -72.5536476627867f},
  { -108.434948822922f,    -70.6403513388797f},
  { -90.0f,    -66.0375110254218f},
  { -153.434948822922f,    -77.3956173516208f},
  { -116.565051177078f,    -77.3956173516208f},
  { -90.0f,    -73.3007557660064f},
  { -135.0f,    -82.6739630468477f},
  { -90.0f,    -79.6951535312340f},
  { -90.0f,    -85.2363583092738f},
  { -84.8055710922652f,    -5.17321244611695f},
  { -78.6900675259798f,    -5.60040918481662f},
  { -71.5650511770780f,    -6.01728486403203f},
  { -63.4349488229220f,    -6.37937020844280f},
  { -54.4623222080256f,    -6.63073776807866f},
  { -45.0000000000000f,    -6.72136933936946f},
  { -35.5376777919744f,    -6.63073776807866f},
  { -26.5650511770780f,    -6.37937020844280f},
  { -18.4349488229220f,    -6.01728486403203f},
  { -11.3099324740202f,    -5.60040918481662f},
  { -5.19442890773480f,    -5.17321244611695f},
  { -84.2894068625003f,    -11.2552397326621f},
  { -77.4711922908485f,    -12.2395582977766f},
  { -69.4439547804165f,    -13.1747121595676f},
  { -60.2551187030578f,    -13.9320915492581f},
  { -50.1944289077348f,    -14.3633026740704f},
  { -39.8055710922652f,    -14.3633026740704f},
  { -29.7448812969422f,    -13.9320915492581f},
  { -20.5560452195835f,    -13.1747121595676f},
  { -12.5288077091515f,    -12.2395582977766f},
  { -5.71059313749964f,    -11.2552397326621f},
  { -83.6598082540901f,    -18.3297537440740f},
  { -75.9637565320735f,    -19.9915607271414f},
  { -66.8014094863518f,    -21.5004376144442f},
  { -56.3099324740202f,    -22.5885387879849f},
  { -45.0000000000000f,    -22.9897677736089f},
  { -33.6900675259798f,    -22.5885387879849f},
  { -23.1985905136482f,    -21.5004376144442f},
  { -14.0362434679265f,    -19.9915607271414f},
  { -6.34019174590991f,    -18.3297537440740f},
  { -82.8749836510982f,    -26.3877999612430f},
  { -74.0546040990771f,    -28.7862533091452f},
  { -63.4349488229220f,    -30.8069811171052f},
  { -51.3401917459099f,    -31.9928170001817f},
  { -38.6598082540901f,    -31.9928170001817f},
  { -26.5650511770780f,    -30.8069811171052f},
  { -15.9453959009229f,    -28.7862533091452f},
  { -7.12501634890179f,    -26.3877999612430f},
  { -81.8698976458440f,    -35.2643896827547f},
  { -71.5650511770780f,    -38.3288181014559f},
  { -59.0362434679265f,    -40.6128551760767f},
  { -45.0f,    -41.4729343089405f},
  { -30.9637565320735f,    -40.6128551760767f},
  { -18.4349488229220f,    -38.3288181014559f},
  { -8.13010235415597f,    -35.2643896827547f},
  { -80.5376777919744f,    -44.6075508793597f},
  { -68.1985905136482f,    -48.0911521181967f},
  { -53.1301023541560f,    -50.1944289077348f},
  { -36.8698976458440f,    -50.1944289077348f},
  { -21.8014094863518f,    -48.0911521181967f},
  { -9.46232220802561f,    -44.6075508793597f},
  { -78.6900675259798f,    -53.9292313492037f},
  { -63.4349488229220f,    -57.4264443008031f},
  { -45.0000000000000f,    -58.7803015526316f},
  { -26.5650511770780f,    -57.4264443008031f},
  { -11.3099324740202f,    -53.9292313492037f},
  { -75.9637565320735f,    -62.7339555492672f},
  { -56.3099324740202f,    -65.7392004151320f},
  { -33.6900675259798f,    -65.7392004151320f},
  { -14.0362434679265f,    -62.7339555492672f},
  { -71.5650511770780f,    -70.6403513388797f},
  { -45.0f,    -72.5536476627867f},
  { -18.4349488229220f,    -70.6403513388797f},
  { -63.4349488229220f,    -77.3956173516208f},
  { -26.5650511770780f,    -77.3956173516208f},
  { -45.0f,    -82.6739630468477f}};

/* 3LD geosphere 'oct' [14 0] (786 points) */
const float __geosphere_oct_14_0_dirs_deg[786][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    4.39870535499553f},
  { 0.0f,    9.46232220802562f},
  { 0.0f,    15.2551187030578f},
  { 0.0f,    21.8014094863518f},
  { 0.0f,    29.0546040990771f},
  { 0.0f,    36.8698976458440f},
  { 0.0f,    45.0f},
  { 0.0f,    53.1301023541560f},
  { 0.0f,    60.9453959009229f},
  { 0.0f,    68.1985905136482f},
  { 0.0f,    74.7448812969422f},
  { 0.0f,    80.5376777919744f},
  { 0.0f,    85.6012946450045f},
  { 4.39870535499553f,    0.0f},
  { 4.76364169072618f,    4.74726225485260f},
  { 5.19442890773481f,    10.2634278919935f},
  { 5.71059313749964f,    16.6209512676335f},
  { 6.34019174590991f,    23.8322990618998f},
  { 7.12501634890180f,    31.8061000193785f},
  { 8.13010235415598f,    40.3155422107726f},
  { 9.46232220802562f,    49.0104664376223f},
  { 11.3099324740202f,    57.4874826838300f},
  { 14.0362434679265f,    65.3864023470214f},
  { 18.4349488229220f,    72.4515993862077f},
  { 26.5650511770780f,    78.5095400962685f},
  { 45.0000000000000f,    83.2786306606305f},
  { 90.0f,    85.6012946450045f},
  { 9.46232220802562f,    0.0f},
  { 10.3048464687660f,    5.11108969528872f},
  { 11.3099324740202f,    11.0958032831364f},
  { 12.5288077091515f,    18.0246645268103f},
  { 14.0362434679265f,    25.8766900608275f},
  { 15.9453959009229f,    34.4813893719816f},
  { 18.4349488229220f,    43.4915193450920f},
  { 21.8014094863518f,    52.4285827724619f},
  { 26.5650511770780f,    60.7940677526006f},
  { 33.6900675259798f,    68.1681310319003f},
  { 45.0000000000000f,    74.2068309517361f},
  { 63.4349488229220f,    78.5095400962685f},
  { 90.0f,    80.5376777919744f},
  { 15.2551187030578f,    0.0f},
  { 16.6992442339936f,    5.47124945343343f},
  { 18.4349488229220f,    11.9046881128481f},
  { 20.5560452195835f,    19.3473699609355f},
  { 23.1985905136482f,    27.7096107852625f},
  { 26.5650511770780f,    36.6992252004899f},
  { 30.9637565320735f,    45.8186225388889f},
  { 36.8698976458440f,    54.4623222080256f},
  { 45.0000000000000f,    62.0616472703976f},
  { 56.3099324740202f,    68.1681310319003f},
  { 71.5650511770780f,    72.4515993862077f},
  { 90.0f,    74.7448812969422f},
  { 21.8014094863518f,    0.0f},
  { 23.9624889745782f,    5.79763632953063f},
  { 26.5650511770780f,    12.6043826483792f},
  { 29.7448812969422f,    20.4104455216585f},
  { 33.6900675259798f,    29.0171406246015f},
  { 38.6598082540901f,    37.9851672993556f},
  { 45.0000000000000f,    46.6861433417170f},
  { 53.1301023541560f,    54.4623222080256f},
  { 63.4349488229220f,    60.7940677526006f},
  { 75.9637565320735f,    65.3864023470214f},
  { 90.0f,    68.1985905136482f},
  { 29.0546040990771f,    0.0f},
  { 32.0053832080835f,    6.05074601591175f},
  { 35.5376777919744f,    13.0884760858333f},
  { 39.8055710922652f,    21.0123137896886f},
  { 45.0000000000000f,    29.4962084965664f},
  { 51.3401917459099f,    37.9851672993556f},
  { 59.0362434679265f,    45.8186225388889f},
  { 68.1985905136482f,    52.4285827724619f},
  { 78.6900675259798f,    57.4874826838300f},
  { 90.0f,    60.9453959009229f},
  { 36.8698976458440f,    0.0f},
  { 40.6012946450045f,    6.19039945857186f},
  { 45.0f,    13.2626760083048f},
  { 50.1944289077348f,    21.0123137896886f},
  { 56.3099324740202f,    29.0171406246015f},
  { 63.4349488229220f,    36.6992252004899f},
  { 71.5650511770780f,    43.4915193450920f},
  { 80.5376777919744f,    49.0104664376223f},
  { 90.0f,    53.1301023541560f},
  { 45.0f,    0.0f},
  { 49.3987053549955f,    6.19039945857186f},
  { 54.4623222080256f,    13.0884760858333f},
  { 60.2551187030578f,    20.4104455216585f},
  { 66.8014094863518f,    27.7096107852625f},
  { 74.0546040990772f,    34.4813893719816f},
  { 81.8698976458440f,    40.3155422107726f},
  { 90.0f,    45.0f},
  { 53.1301023541560f,    0.0f},
  { 57.9946167919165f,    6.05074601591175f},
  { 63.4349488229220f,    12.6043826483792f},
  { 69.4439547804165f,    19.3473699609355f},
  { 75.9637565320735f,    25.8766900608275f},
  { 82.8749836510982f,    31.8061000193785f},
  { 90.0f,    36.8698976458440f},
  { 60.9453959009229f,    0.0f},
  { 66.0375110254218f,    5.79763632953063f},
  { 71.5650511770780f,    11.9046881128481f},
  { 77.4711922908485f,    18.0246645268103f},
  { 83.6598082540901f,    23.8322990618998f},
  { 90.0f,    29.0546040990771f},
  { 68.1985905136482f,    0.0f},
  { 73.3007557660064f,    5.47124945343343f},
  { 78.6900675259798f,    11.0958032831364f},
  { 84.2894068625004f,    16.6209512676335f},
  { 90.0f,    21.8014094863518f},
  { 74.7448812969422f,    0.0f},
  { 79.6951535312340f,    5.11108969528872f},
  { 84.8055710922652f,    10.2634278919936f},
  { 90.0f,    15.2551187030578f},
  { 80.5376777919744f,    0.0f},
  { 85.2363583092738f,    4.74726225485260f},
  { 90.0f,    9.46232220802562f},
  { 85.6012946450045f,    0.0f},
  { 90.0f,    4.39870535499553f},
  { 94.3987053549955f,    0.0f},
  { 94.7636416907262f,    4.74726225485260f},
  { 95.1944289077348f,    10.2634278919935f},
  { 95.7105931374996f,    16.6209512676335f},
  { 96.3401917459099f,    23.8322990618998f},
  { 97.1250163489018f,    31.8061000193785f},
  { 98.1301023541560f,    40.3155422107726f},
  { 99.4623222080256f,    49.0104664376223f},
  { 101.309932474020f,    57.4874826838300f},
  { 104.036243467927f,    65.3864023470214f},
  { 108.434948822922f,    72.4515993862077f},
  { 116.565051177078f,    78.5095400962685f},
  { 135.000000000000f,    83.2786306606305f},
  { 180.0f,    85.6012946450045f},
  { 99.4623222080256f,    0.0f},
  { 100.304846468766f,    5.11108969528872f},
  { 101.309932474020f,    11.0958032831364f},
  { 102.528807709152f,    18.0246645268103f},
  { 104.036243467927f,    25.8766900608275f},
  { 105.945395900923f,    34.4813893719816f},
  { 108.434948822922f,    43.4915193450920f},
  { 111.801409486352f,    52.4285827724619f},
  { 116.565051177078f,    60.7940677526006f},
  { 123.690067525980f,    68.1681310319003f},
  { 135.000000000000f,    74.2068309517361f},
  { 153.434948822922f,    78.5095400962685f},
  { 180.0f,    80.5376777919744f},
  { 105.255118703058f,    0.0f},
  { 106.699244233994f,    5.47124945343343f},
  { 108.434948822922f,    11.9046881128481f},
  { 110.556045219583f,    19.3473699609355f},
  { 113.198590513648f,    27.7096107852625f},
  { 116.565051177078f,    36.6992252004899f},
  { 120.963756532074f,    45.8186225388889f},
  { 126.869897645844f,    54.4623222080256f},
  { 135.0f,    62.0616472703976f},
  { 146.309932474020f,    68.1681310319003f},
  { 161.565051177078f,    72.4515993862077f},
  { 180.0f,    74.7448812969422f},
  { 111.801409486352f,    0.0f},
  { 113.962488974578f,    5.79763632953063f},
  { 116.565051177078f,    12.6043826483792f},
  { 119.744881296942f,    20.4104455216585f},
  { 123.690067525980f,    29.0171406246015f},
  { 128.659808254090f,    37.9851672993556f},
  { 135.0f,    46.6861433417170f},
  { 143.130102354156f,    54.4623222080256f},
  { 153.434948822922f,    60.7940677526006f},
  { 165.963756532074f,    65.3864023470214f},
  { 180.0f,    68.1985905136482f},
  { 119.054604099077f,    0.0f},
  { 122.005383208084f,    6.05074601591175f},
  { 125.537677791974f,    13.0884760858333f},
  { 129.805571092265f,    21.0123137896886f},
  { 135.0f,    29.4962084965664f},
  { 141.340191745910f,    37.9851672993556f},
  { 149.036243467926f,    45.8186225388889f},
  { 158.198590513648f,    52.4285827724619f},
  { 168.690067525980f,    57.4874826838300f},
  { 180.0f,    60.9453959009229f},
  { 126.869897645844f,    0.0f},
  { 130.601294645005f,    6.19039945857186f},
  { 135.0f,    13.2626760083048f},
  { 140.194428907735f,    21.0123137896886f},
  { 146.309932474020f,    29.0171406246015f},
  { 153.434948822922f,    36.6992252004899f},
  { 161.565051177078f,    43.4915193450920f},
  { 170.537677791974f,    49.0104664376223f},
  { 180.0f,    53.1301023541560f},
  { 135.0f,    0.0f},
  { 139.398705354996f,    6.19039945857186f},
  { 144.462322208026f,    13.0884760858333f},
  { 150.255118703058f,    20.4104455216585f},
  { 156.801409486352f,    27.7096107852625f},
  { 164.054604099077f,    34.4813893719816f},
  { 171.869897645844f,    40.3155422107726f},
  { 180.0f,    45.0f},
  { 143.130102354156f,    0.0f},
  { 147.994616791916f,    6.05074601591175f},
  { 153.434948822922f,    12.6043826483792f},
  { 159.443954780417f,    19.3473699609355f},
  { 165.963756532074f,    25.8766900608275f},
  { 172.874983651098f,    31.8061000193785f},
  { 180.0f,    36.8698976458440f},
  { 150.945395900923f,    0.0f},
  { 156.037511025422f,    5.79763632953063f},
  { 161.565051177078f,    11.9046881128481f},
  { 167.471192290848f,    18.0246645268103f},
  { 173.659808254090f,    23.8322990618998f},
  { 180.0f,    29.0546040990771f},
  { 158.198590513648f,    0.0f},
  { 163.300755766006f,    5.47124945343343f},
  { 168.690067525980f,    11.0958032831364f},
  { 174.289406862500f,    16.6209512676335f},
  { 180.0f,    21.8014094863518f},
  { 164.744881296942f,    0.0f},
  { 169.695153531234f,    5.11108969528872f},
  { 174.805571092265f,    10.2634278919936f},
  { 180.0f,    15.2551187030578f},
  { 170.537677791974f,    0.0f},
  { 175.236358309274f,    4.74726225485260f},
  { 180.0f,    9.46232220802562f},
  { 175.601294645005f,    0.0f},
  { 180.0f,    4.39870535499553f},
  { -175.601294645005f,    0.0f},
  { -175.236358309274f,    4.74726225485260f},
  { -174.805571092265f,    10.2634278919935f},
  { -174.289406862500f,    16.6209512676335f},
  { -173.659808254090f,    23.8322990618998f},
  { -172.874983651098f,    31.8061000193785f},
  { -171.869897645844f,    40.3155422107726f},
  { -170.537677791974f,    49.0104664376223f},
  { -168.690067525980f,    57.4874826838300f},
  { -165.963756532074f,    65.3864023470214f},
  { -161.565051177078f,    72.4515993862077f},
  { -153.434948822922f,    78.5095400962685f},
  { -135.000000000000f,    83.2786306606305f},
  { -90.0f,    85.6012946450045f},
  { -170.537677791974f,    0.0f},
  { -169.695153531234f,    5.11108969528872f},
  { -168.690067525980f,    11.0958032831364f},
  { -167.471192290848f,    18.0246645268103f},
  { -165.963756532074f,    25.8766900608275f},
  { -164.054604099077f,    34.4813893719816f},
  { -161.565051177078f,    43.4915193450920f},
  { -158.198590513648f,    52.4285827724619f},
  { -153.434948822922f,    60.7940677526006f},
  { -146.309932474020f,    68.1681310319003f},
  { -135.0f,    74.2068309517361f},
  { -116.565051177078f,    78.5095400962685f},
  { -90.0f,    80.5376777919744f},
  { -164.744881296942f,    0.0f},
  { -163.300755766006f,    5.47124945343343f},
  { -161.565051177078f,    11.9046881128481f},
  { -159.443954780417f,    19.3473699609355f},
  { -156.801409486352f,    27.7096107852625f},
  { -153.434948822922f,    36.6992252004899f},
  { -149.036243467926f,    45.8186225388889f},
  { -143.130102354156f,    54.4623222080256f},
  { -135.0f,    62.0616472703976f},
  { -123.690067525980f,    68.1681310319003f},
  { -108.434948822922f,    72.4515993862077f},
  { -90.0f,    74.7448812969422f},
  { -158.198590513648f,    0.0f},
  { -156.037511025422f,    5.79763632953063f},
  { -153.434948822922f,    12.6043826483792f},
  { -150.255118703058f,    20.4104455216585f},
  { -146.309932474020f,    29.0171406246015f},
  { -141.340191745910f,    37.9851672993556f},
  { -135.0f,    46.6861433417170f},
  { -126.869897645844f,    54.4623222080256f},
  { -116.565051177078f,    60.7940677526006f},
  { -104.036243467927f,    65.3864023470214f},
  { -90.0f,    68.1985905136482f},
  { -150.945395900923f,    0.0f},
  { -147.994616791916f,    6.05074601591175f},
  { -144.462322208026f,    13.0884760858333f},
  { -140.194428907735f,    21.0123137896886f},
  { -135.0f,    29.4962084965664f},
  { -128.659808254090f,    37.9851672993556f},
  { -120.963756532074f,    45.8186225388889f},
  { -111.801409486352f,    52.4285827724619f},
  { -101.309932474020f,    57.4874826838300f},
  { -90.0f,    60.9453959009229f},
  { -143.130102354156f,    0.0f},
  { -139.398705354996f,    6.19039945857186f},
  { -135.0f,    13.2626760083048f},
  { -129.805571092265f,    21.0123137896886f},
  { -123.690067525980f,    29.0171406246015f},
  { -116.565051177078f,    36.6992252004899f},
  { -108.434948822922f,    43.4915193450920f},
  { -99.4623222080256f,    49.0104664376223f},
  { -90.0f,    53.1301023541560f},
  { -135.0f,    0.0f},
  { -130.601294645005f,    6.19039945857186f},
  { -125.537677791974f,    13.0884760858333f},
  { -119.744881296942f,    20.4104455216585f},
  { -113.198590513648f,    27.7096107852625f},
  { -105.945395900923f,    34.4813893719816f},
  { -98.1301023541560f,    40.3155422107726f},
  { -90.0f,    45.0f},
  { -126.869897645844f,    0.0f},
  { -122.005383208084f,    6.05074601591175f},
  { -116.565051177078f,    12.6043826483792f},
  { -110.556045219583f,    19.3473699609355f},
  { -104.036243467927f,    25.8766900608275f},
  { -97.1250163489018f,    31.8061000193785f},
  { -90.0f,    36.8698976458440f},
  { -119.054604099077f,    0.0f},
  { -113.962488974578f,    5.79763632953063f},
  { -108.434948822922f,    11.9046881128481f},
  { -102.528807709152f,    18.0246645268103f},
  { -96.3401917459099f,    23.8322990618998f},
  { -90.0f,    29.0546040990771f},
  { -111.801409486352f,    0.0f},
  { -106.699244233994f,    5.47124945343343f},
  { -101.309932474020f,    11.0958032831364f},
  { -95.7105931374996f,    16.6209512676335f},
  { -90.0f,    21.8014094863518f},
  { -105.255118703058f,    0.0f},
  { -100.304846468766f,    5.11108969528872f},
  { -95.1944289077348f,    10.2634278919936f},
  { -90.0f,    15.2551187030578f},
  { -99.4623222080256f,    0.0f},
  { -94.7636416907262f,    4.74726225485260f},
  { -90.0f,    9.46232220802562f},
  { -94.3987053549955f,    0.0f},
  { -90.0f,    4.39870535499553f},
  { -85.6012946450045f,    0.0f},
  { -85.2363583092738f,    4.74726225485260f},
  { -84.8055710922652f,    10.2634278919935f},
  { -84.2894068625004f,    16.6209512676335f},
  { -83.6598082540901f,    23.8322990618998f},
  { -82.8749836510982f,    31.8061000193785f},
  { -81.8698976458440f,    40.3155422107726f},
  { -80.5376777919744f,    49.0104664376223f},
  { -78.6900675259798f,    57.4874826838300f},
  { -75.9637565320735f,    65.3864023470214f},
  { -71.5650511770780f,    72.4515993862077f},
  { -63.4349488229220f,    78.5095400962685f},
  { -45.0000000000000f,    83.2786306606305f},
  { -80.5376777919744f,    0.0f},
  { -79.6951535312340f,    5.11108969528872f},
  { -78.6900675259798f,    11.0958032831364f},
  { -77.4711922908485f,    18.0246645268103f},
  { -75.9637565320735f,    25.8766900608275f},
  { -74.0546040990772f,    34.4813893719816f},
  { -71.5650511770780f,    43.4915193450920f},
  { -68.1985905136482f,    52.4285827724619f},
  { -63.4349488229220f,    60.7940677526006f},
  { -56.3099324740202f,    68.1681310319003f},
  { -45.0000000000000f,    74.2068309517361f},
  { -26.5650511770780f,    78.5095400962685f},
  { -74.7448812969422f,    0.0f},
  { -73.3007557660064f,    5.47124945343343f},
  { -71.5650511770780f,    11.9046881128481f},
  { -69.4439547804165f,    19.3473699609355f},
  { -66.8014094863518f,    27.7096107852625f},
  { -63.4349488229220f,    36.6992252004899f},
  { -59.0362434679265f,    45.8186225388889f},
  { -53.1301023541560f,    54.4623222080256f},
  { -45.0000000000000f,    62.0616472703976f},
  { -33.6900675259798f,    68.1681310319003f},
  { -18.4349488229220f,    72.4515993862077f},
  { -68.1985905136482f,    0.0f},
  { -66.0375110254218f,    5.79763632953063f},
  { -63.4349488229220f,    12.6043826483792f},
  { -60.2551187030578f,    20.4104455216585f},
  { -56.3099324740202f,    29.0171406246015f},
  { -51.3401917459099f,    37.9851672993556f},
  { -45.0000000000000f,    46.6861433417170f},
  { -36.8698976458440f,    54.4623222080256f},
  { -26.5650511770780f,    60.7940677526006f},
  { -14.0362434679265f,    65.3864023470214f},
  { -60.9453959009229f,    0.0f},
  { -57.9946167919165f,    6.05074601591175f},
  { -54.4623222080256f,    13.0884760858333f},
  { -50.1944289077348f,    21.0123137896886f},
  { -45.0000000000000f,    29.4962084965664f},
  { -38.6598082540901f,    37.9851672993556f},
  { -30.9637565320735f,    45.8186225388889f},
  { -21.8014094863518f,    52.4285827724619f},
  { -11.3099324740202f,    57.4874826838300f},
  { -53.1301023541560f,    0.0f},
  { -49.3987053549955f,    6.19039945857186f},
  { -45.0f,    13.2626760083048f},
  { -39.8055710922652f,    21.0123137896886f},
  { -33.6900675259798f,    29.0171406246015f},
  { -26.5650511770780f,    36.6992252004899f},
  { -18.4349488229220f,    43.4915193450920f},
  { -9.46232220802561f,    49.0104664376223f},
  { -45.0f,    0.0f},
  { -40.6012946450045f,    6.19039945857186f},
  { -35.5376777919744f,    13.0884760858333f},
  { -29.7448812969422f,    20.4104455216585f},
  { -23.1985905136482f,    27.7096107852625f},
  { -15.9453959009229f,    34.4813893719816f},
  { -8.13010235415598f,    40.3155422107726f},
  { -36.8698976458440f,    0.0f},
  { -32.0053832080835f,    6.05074601591175f},
  { -26.5650511770780f,    12.6043826483792f},
  { -20.5560452195835f,    19.3473699609355f},
  { -14.0362434679265f,    25.8766900608275f},
  { -7.12501634890180f,    31.8061000193785f},
  { -29.0546040990771f,    0.0f},
  { -23.9624889745782f,    5.79763632953063f},
  { -18.4349488229220f,    11.9046881128481f},
  { -12.5288077091515f,    18.0246645268103f},
  { -6.34019174590991f,    23.8322990618998f},
  { -21.8014094863518f,    0.0f},
  { -16.6992442339936f,    5.47124945343343f},
  { -11.3099324740202f,    11.0958032831364f},
  { -5.71059313749964f,    16.6209512676335f},
  { -15.2551187030578f,    0.0f},
  { -10.3048464687660f,    5.11108969528872f},
  { -5.19442890773481f,    10.2634278919936f},
  { -9.46232220802562f,    0.0f},
  { -4.76364169072618f,    4.74726225485260f},
  { -4.39870535499553f,    0.0f},
  { 0.0f,    -4.39870535499553f},
  { 4.76364169072618f,    -4.74726225485260f},
  { 10.3048464687660f,    -5.11108969528872f},
  { 16.6992442339936f,    -5.47124945343343f},
  { 23.9624889745782f,    -5.79763632953063f},
  { 32.0053832080835f,    -6.05074601591175f},
  { 40.6012946450045f,    -6.19039945857186f},
  { 49.3987053549955f,    -6.19039945857186f},
  { 57.9946167919165f,    -6.05074601591175f},
  { 66.0375110254218f,    -5.79763632953063f},
  { 73.3007557660064f,    -5.47124945343343f},
  { 79.6951535312340f,    -5.11108969528871f},
  { 85.2363583092738f,    -4.74726225485260f},
  { 90.0f,    -4.39870535499553f},
  { 0.0f,    -9.46232220802562f},
  { 5.19442890773481f,    -10.2634278919935f},
  { 11.3099324740202f,    -11.0958032831364f},
  { 18.4349488229220f,    -11.9046881128481f},
  { 26.5650511770780f,    -12.6043826483792f},
  { 35.5376777919744f,    -13.0884760858333f},
  { 45.0f,    -13.2626760083048f},
  { 54.4623222080256f,    -13.0884760858333f},
  { 63.4349488229220f,    -12.6043826483792f},
  { 71.5650511770780f,    -11.9046881128481f},
  { 78.6900675259798f,    -11.0958032831364f},
  { 84.8055710922652f,    -10.2634278919936f},
  { 90.0f,    -9.46232220802562f},
  { 0.0f,    -15.2551187030578f},
  { 5.71059313749964f,    -16.6209512676335f},
  { 12.5288077091515f,    -18.0246645268103f},
  { 20.5560452195835f,    -19.3473699609355f},
  { 29.7448812969422f,    -20.4104455216585f},
  { 39.8055710922652f,    -21.0123137896886f},
  { 50.1944289077348f,    -21.0123137896886f},
  { 60.2551187030578f,    -20.4104455216585f},
  { 69.4439547804165f,    -19.3473699609355f},
  { 77.4711922908485f,    -18.0246645268103f},
  { 84.2894068625004f,    -16.6209512676335f},
  { 90.0f,    -15.2551187030578f},
  { 0.0f,    -21.8014094863518f},
  { 6.34019174590991f,    -23.8322990618998f},
  { 14.0362434679265f,    -25.8766900608275f},
  { 23.1985905136482f,    -27.7096107852625f},
  { 33.6900675259798f,    -29.0171406246015f},
  { 45.0000000000000f,    -29.4962084965664f},
  { 56.3099324740202f,    -29.0171406246015f},
  { 66.8014094863518f,    -27.7096107852625f},
  { 75.9637565320735f,    -25.8766900608275f},
  { 83.6598082540901f,    -23.8322990618998f},
  { 90.0f,    -21.8014094863518f},
  { 0.0f,    -29.0546040990771f},
  { 7.12501634890180f,    -31.8061000193785f},
  { 15.9453959009229f,    -34.4813893719816f},
  { 26.5650511770780f,    -36.6992252004899f},
  { 38.6598082540901f,    -37.9851672993556f},
  { 51.3401917459099f,    -37.9851672993556f},
  { 63.4349488229220f,    -36.6992252004899f},
  { 74.0546040990772f,    -34.4813893719816f},
  { 82.8749836510982f,    -31.8061000193785f},
  { 90.0f,    -29.0546040990771f},
  { 0.0f,    -36.8698976458440f},
  { 8.13010235415598f,    -40.3155422107726f},
  { 18.4349488229220f,    -43.4915193450920f},
  { 30.9637565320735f,    -45.8186225388889f},
  { 45.0f,    -46.6861433417170f},
  { 59.0362434679265f,    -45.8186225388889f},
  { 71.5650511770780f,    -43.4915193450920f},
  { 81.8698976458440f,    -40.3155422107726f},
  { 90.0f,    -36.8698976458440f},
  { 0.0f,    -45.0f},
  { 9.46232220802562f,    -49.0104664376223f},
  { 21.8014094863518f,    -52.4285827724619f},
  { 36.8698976458440f,    -54.4623222080256f},
  { 53.1301023541560f,    -54.4623222080256f},
  { 68.1985905136482f,    -52.4285827724619f},
  { 80.5376777919744f,    -49.0104664376223f},
  { 90.0f,    -45.0f},
  { 0.0f,    -53.1301023541560f},
  { 11.3099324740202f,    -57.4874826838300f},
  { 26.5650511770780f,    -60.7940677526006f},
  { 45.0000000000000f,    -62.0616472703976f},
  { 63.4349488229220f,    -60.7940677526006f},
  { 78.6900675259798f,    -57.4874826838300f},
  { 90.0f,    -53.1301023541560f},
  { 0.0f,    -60.9453959009229f},
  { 14.0362434679265f,    -65.3864023470214f},
  { 33.6900675259798f,    -68.1681310319003f},
  { 56.3099324740202f,    -68.1681310319003f},
  { 75.9637565320735f,    -65.3864023470214f},
  { 90.0f,    -60.9453959009229f},
  { 0.0f,    -68.1985905136482f},
  { 18.4349488229220f,    -72.4515993862077f},
  { 45.0f,    -74.2068309517360f},
  { 71.5650511770780f,    -72.4515993862077f},
  { 90.0f,    -68.1985905136482f},
  { 0.0f,    -74.7448812969422f},
  { 26.5650511770780f,    -78.5095400962685f},
  { 63.4349488229220f,    -78.5095400962685f},
  { 90.0f,    -74.7448812969422f},
  { 0.0f,    -80.5376777919744f},
  { 45.0000000000000f,    -83.2786306606305f},
  { 90.0f,    -80.5376777919744f},
  { 0.0f,    -85.6012946450045f},
  { 90.0f,    -85.6012946450045f},
  { 94.7636416907262f,    -4.74726225485260f},
  { 100.304846468766f,    -5.11108969528872f},
  { 106.699244233994f,    -5.47124945343343f},
  { 113.962488974578f,    -5.79763632953063f},
  { 122.005383208084f,    -6.05074601591175f},
  { 130.601294645005f,    -6.19039945857186f},
  { 139.398705354996f,    -6.19039945857186f},
  { 147.994616791916f,    -6.05074601591175f},
  { 156.037511025422f,    -5.79763632953063f},
  { 163.300755766006f,    -5.47124945343343f},
  { 169.695153531234f,    -5.11108969528872f},
  { 175.236358309274f,    -4.74726225485260f},
  { 180.0f,    -4.39870535499553f},
  { 95.1944289077348f,    -10.2634278919935f},
  { 101.309932474020f,    -11.0958032831364f},
  { 108.434948822922f,    -11.9046881128481f},
  { 116.565051177078f,    -12.6043826483792f},
  { 125.537677791974f,    -13.0884760858333f},
  { 135.0f,    -13.2626760083048f},
  { 144.462322208026f,    -13.0884760858333f},
  { 153.434948822922f,    -12.6043826483792f},
  { 161.565051177078f,    -11.9046881128481f},
  { 168.690067525980f,    -11.0958032831364f},
  { 174.805571092265f,    -10.2634278919936f},
  { 180.0f,    -9.46232220802562f},
  { 95.7105931374996f,    -16.6209512676335f},
  { 102.528807709152f,    -18.0246645268103f},
  { 110.556045219583f,    -19.3473699609355f},
  { 119.744881296942f,    -20.4104455216585f},
  { 129.805571092265f,    -21.0123137896886f},
  { 140.194428907735f,    -21.0123137896886f},
  { 150.255118703058f,    -20.4104455216585f},
  { 159.443954780417f,    -19.3473699609355f},
  { 167.471192290848f,    -18.0246645268103f},
  { 174.289406862500f,    -16.6209512676335f},
  { 180.0f,    -15.2551187030578f},
  { 96.3401917459099f,    -23.8322990618998f},
  { 104.036243467927f,    -25.8766900608275f},
  { 113.198590513648f,    -27.7096107852625f},
  { 123.690067525980f,    -29.0171406246015f},
  { 135.0f,    -29.4962084965664f},
  { 146.309932474020f,    -29.0171406246015f},
  { 156.801409486352f,    -27.7096107852625f},
  { 165.963756532074f,    -25.8766900608275f},
  { 173.659808254090f,    -23.8322990618998f},
  { 180.0f,    -21.8014094863518f},
  { 97.1250163489018f,    -31.8061000193785f},
  { 105.945395900923f,    -34.4813893719816f},
  { 116.565051177078f,    -36.6992252004899f},
  { 128.659808254090f,    -37.9851672993556f},
  { 141.340191745910f,    -37.9851672993556f},
  { 153.434948822922f,    -36.6992252004899f},
  { 164.054604099077f,    -34.4813893719816f},
  { 172.874983651098f,    -31.8061000193785f},
  { 180.0f,    -29.0546040990771f},
  { 98.1301023541560f,    -40.3155422107726f},
  { 108.434948822922f,    -43.4915193450920f},
  { 120.963756532074f,    -45.8186225388889f},
  { 135.0f,    -46.6861433417170f},
  { 149.036243467926f,    -45.8186225388889f},
  { 161.565051177078f,    -43.4915193450920f},
  { 171.869897645844f,    -40.3155422107726f},
  { 180.0f,    -36.8698976458440f},
  { 99.4623222080256f,    -49.0104664376223f},
  { 111.801409486352f,    -52.4285827724619f},
  { 126.869897645844f,    -54.4623222080256f},
  { 143.130102354156f,    -54.4623222080256f},
  { 158.198590513648f,    -52.4285827724619f},
  { 170.537677791974f,    -49.0104664376223f},
  { 180.0f,    -45.0f},
  { 101.309932474020f,    -57.4874826838300f},
  { 116.565051177078f,    -60.7940677526006f},
  { 135.0f,    -62.0616472703976f},
  { 153.434948822922f,    -60.7940677526006f},
  { 168.690067525980f,    -57.4874826838300f},
  { 180.0f,    -53.1301023541560f},
  { 104.036243467927f,    -65.3864023470214f},
  { 123.690067525980f,    -68.1681310319003f},
  { 146.309932474020f,    -68.1681310319003f},
  { 165.963756532074f,    -65.3864023470214f},
  { 180.0f,    -60.9453959009229f},
  { 108.434948822922f,    -72.4515993862077f},
  { 135.0f,    -74.2068309517360f},
  { 161.565051177078f,    -72.4515993862077f},
  { 180.0f,    -68.1985905136482f},
  { 116.565051177078f,    -78.5095400962685f},
  { 153.434948822922f,    -78.5095400962685f},
  { 180.0f,    -74.7448812969422f},
  { 135.0f,    -83.2786306606305f},
  { 180.0f,    -80.5376777919744f},
  { 180.0f,    -85.6012946450045f},
  { -175.236358309274f,    -4.74726225485260f},
  { -169.695153531234f,    -5.11108969528872f},
  { -163.300755766006f,    -5.47124945343343f},
  { -156.037511025422f,    -5.79763632953063f},
  { -147.994616791916f,    -6.05074601591175f},
  { -139.398705354996f,    -6.19039945857186f},
  { -130.601294645005f,    -6.19039945857186f},
  { -122.005383208084f,    -6.05074601591174f},
  { -113.962488974578f,    -5.79763632953063f},
  { -106.699244233994f,    -5.47124945343343f},
  { -100.304846468766f,    -5.11108969528872f},
  { -94.7636416907262f,    -4.74726225485260f},
  { -90.0f,    -4.39870535499553f},
  { -174.805571092265f,    -10.2634278919935f},
  { -168.690067525980f,    -11.0958032831364f},
  { -161.565051177078f,    -11.9046881128481f},
  { -153.434948822922f,    -12.6043826483792f},
  { -144.462322208026f,    -13.0884760858333f},
  { -135.0f,    -13.2626760083048f},
  { -125.537677791974f,    -13.0884760858333f},
  { -116.565051177078f,    -12.6043826483792f},
  { -108.434948822922f,    -11.9046881128481f},
  { -101.309932474020f,    -11.0958032831364f},
  { -95.1944289077348f,    -10.2634278919936f},
  { -90.0f,    -9.46232220802562f},
  { -174.289406862500f,    -16.6209512676335f},
  { -167.471192290848f,    -18.0246645268103f},
  { -159.443954780417f,    -19.3473699609355f},
  { -150.255118703058f,    -20.4104455216585f},
  { -140.194428907735f,    -21.0123137896886f},
  { -129.805571092265f,    -21.0123137896886f},
  { -119.744881296942f,    -20.4104455216585f},
  { -110.556045219583f,    -19.3473699609355f},
  { -102.528807709152f,    -18.0246645268103f},
  { -95.7105931374996f,    -16.6209512676335f},
  { -90.0f,    -15.2551187030578f},
  { -173.659808254090f,    -23.8322990618998f},
  { -165.963756532074f,    -25.8766900608275f},
  { -156.801409486352f,    -27.7096107852625f},
  { -146.309932474020f,    -29.0171406246015f},
  { -135.0f,    -29.4962084965664f},
  { -123.690067525980f,    -29.0171406246015f},
  { -113.198590513648f,    -27.7096107852625f},
  { -104.036243467927f,    -25.8766900608275f},
  { -96.3401917459099f,    -23.8322990618998f},
  { -90.0f,    -21.8014094863518f},
  { -172.874983651098f,    -31.8061000193785f},
  { -164.054604099077f,    -34.4813893719816f},
  { -153.434948822922f,    -36.6992252004899f},
  { -141.340191745910f,    -37.9851672993556f},
  { -128.659808254090f,    -37.9851672993556f},
  { -116.565051177078f,    -36.6992252004899f},
  { -105.945395900923f,    -34.4813893719816f},
  { -97.1250163489018f,    -31.8061000193785f},
  { -90.0f,    -29.0546040990771f},
  { -171.869897645844f,    -40.3155422107726f},
  { -161.565051177078f,    -43.4915193450920f},
  { -149.036243467926f,    -45.8186225388889f},
  { -135.0f,    -46.6861433417170f},
  { -120.963756532074f,    -45.8186225388889f},
  { -108.434948822922f,    -43.4915193450920f},
  { -98.1301023541560f,    -40.3155422107726f},
  { -90.0f,    -36.8698976458440f},
  { -170.537677791974f,    -49.0104664376223f},
  { -158.198590513648f,    -52.4285827724619f},
  { -143.130102354156f,    -54.4623222080256f},
  { -126.869897645844f,    -54.4623222080256f},
  { -111.801409486352f,    -52.4285827724619f},
  { -99.4623222080256f,    -49.0104664376223f},
  { -90.0f,    -45.0f},
  { -168.690067525980f,    -57.4874826838300f},
  { -153.434948822922f,    -60.7940677526006f},
  { -135.0f,    -62.0616472703976f},
  { -116.565051177078f,    -60.7940677526006f},
  { -101.309932474020f,    -57.4874826838300f},
  { -90.0f,    -53.1301023541560f},
  { -165.963756532074f,    -65.3864023470214f},
  { -146.309932474020f,    -68.1681310319003f},
  { -123.690067525980f,    -68.1681310319003f},
  { -104.036243467927f,    -65.3864023470214f},
  { -90.0f,    -60.9453959009229f},
  { -161.565051177078f,    -72.4515993862077f},
  { -135.0f,    -74.2068309517360f},
  { -108.434948822922f,    -72.4515993862077f},
  { -90.0f,    -68.1985905136482f},
  { -153.434948822922f,    -78.5095400962685f},
  { -116.565051177078f,    -78.5095400962685f},
  { -90.0f,    -74.7448812969422f},
  { -135.0f,    -83.2786306606305f},
  { -90.0f,    -80.5376777919744f},
  { -90.0f,    -85.6012946450045f},
  { -85.2363583092738f,    -4.74726225485260f},
  { -79.6951535312340f,    -5.11108969528871f},
  { -73.3007557660064f,    -5.47124945343343f},
  { -66.0375110254218f,    -5.79763632953063f},
  { -57.9946167919165f,    -6.05074601591175f},
  { -49.3987053549955f,    -6.19039945857186f},
  { -40.6012946450045f,    -6.19039945857186f},
  { -32.0053832080835f,    -6.05074601591175f},
  { -23.9624889745782f,    -5.79763632953063f},
  { -16.6992442339936f,    -5.47124945343343f},
  { -10.3048464687660f,    -5.11108969528871f},
  { -4.76364169072618f,    -4.74726225485260f},
  { -84.8055710922652f,    -10.2634278919935f},
  { -78.6900675259798f,    -11.0958032831364f},
  { -71.5650511770780f,    -11.9046881128481f},
  { -63.4349488229220f,    -12.6043826483792f},
  { -54.4623222080256f,    -13.0884760858333f},
  { -45.0f,    -13.2626760083048f},
  { -35.5376777919744f,    -13.0884760858333f},
  { -26.5650511770780f,    -12.6043826483792f},
  { -18.4349488229220f,    -11.9046881128481f},
  { -11.3099324740202f,    -11.0958032831364f},
  { -5.19442890773481f,    -10.2634278919936f},
  { -84.2894068625004f,    -16.6209512676335f},
  { -77.4711922908485f,    -18.0246645268103f},
  { -69.4439547804165f,    -19.3473699609355f},
  { -60.2551187030578f,    -20.4104455216585f},
  { -50.1944289077348f,    -21.0123137896886f},
  { -39.8055710922652f,    -21.0123137896886f},
  { -29.7448812969422f,    -20.4104455216585f},
  { -20.5560452195835f,    -19.3473699609355f},
  { -12.5288077091515f,    -18.0246645268103f},
  { -5.71059313749964f,    -16.6209512676335f},
  { -83.6598082540901f,    -23.8322990618998f},
  { -75.9637565320735f,    -25.8766900608275f},
  { -66.8014094863518f,    -27.7096107852625f},
  { -56.3099324740202f,    -29.0171406246015f},
  { -45.0000000000000f,    -29.4962084965664f},
  { -33.6900675259798f,    -29.0171406246015f},
  { -23.1985905136482f,    -27.7096107852625f},
  { -14.0362434679265f,    -25.8766900608275f},
  { -6.34019174590991f,    -23.8322990618998f},
  { -82.8749836510982f,    -31.8061000193785f},
  { -74.0546040990772f,    -34.4813893719816f},
  { -63.4349488229220f,    -36.6992252004899f},
  { -51.3401917459099f,    -37.9851672993556f},
  { -38.6598082540901f,    -37.9851672993556f},
  { -26.5650511770780f,    -36.6992252004899f},
  { -15.9453959009228f,    -34.4813893719816f},
  { -7.12501634890180f,    -31.8061000193785f},
  { -81.8698976458440f,    -40.3155422107726f},
  { -71.5650511770780f,    -43.4915193450920f},
  { -59.0362434679265f,    -45.8186225388889f},
  { -45.0f,    -46.6861433417170f},
  { -30.9637565320735f,    -45.8186225388889f},
  { -18.4349488229220f,    -43.4915193450920f},
  { -8.13010235415598f,    -40.3155422107726f},
  { -80.5376777919744f,    -49.0104664376223f},
  { -68.1985905136482f,    -52.4285827724619f},
  { -53.1301023541560f,    -54.4623222080256f},
  { -36.8698976458440f,    -54.4623222080256f},
  { -21.8014094863518f,    -52.4285827724619f},
  { -9.46232220802562f,    -49.0104664376223f},
  { -78.6900675259798f,    -57.4874826838300f},
  { -63.4349488229220f,    -60.7940677526006f},
  { -45.0000000000000f,    -62.0616472703976f},
  { -26.5650511770780f,    -60.7940677526006f},
  { -11.3099324740202f,    -57.4874826838300f},
  { -75.9637565320735f,    -65.3864023470214f},
  { -56.3099324740202f,    -68.1681310319003f},
  { -33.6900675259798f,    -68.1681310319003f},
  { -14.0362434679265f,    -65.3864023470214f},
  { -71.5650511770780f,    -72.4515993862077f},
  { -45.0f,    -74.2068309517360f},
  { -18.4349488229220f,    -72.4515993862077f},
  { -63.4349488229220f,    -78.5095400962685f},
  { -26.5650511770780f,    -78.5095400962685f},
  { -45.0000000000000f,    -83.2786306606305f}};

/* 3LD geosphere 'oct' [15 0] (902 points) */
const float __geosphere_oct_15_0_dirs_deg[902][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    4.08561677997488f},
  { 0.0f,    8.74616226255521f},
  { 0.0f,    14.0362434679265f},
  { 0.0f,    19.9831065219000f},
  { 0.0f,    26.5650511770780f},
  { 0.0f,    33.6900675259798f},
  { 0.0f,    41.1859251657097f},
  { 0.0f,    48.8140748342904f},
  { 0.0f,    56.3099324740202f},
  { 0.0f,    63.4349488229220f},
  { 0.0f,    70.0168934781000f},
  { 0.0f,    75.9637565320735f},
  { 0.0f,    81.2538377374448f},
  { 0.0f,    85.9143832200251f},
  { 4.08561677997488f,    0.0f},
  { 4.39870535499553f,    4.38579953077501f},
  { 4.76364169072618f,    9.43022513800569f},
  { 5.19442890773481f,    15.1953712458899f},
  { 5.71059313749964f,    21.7032913642261f},
  { 6.34019174590991f,    28.9056194412282f},
  { 7.12501634890180f,    36.6569329544820f},
  { 8.13010235415598f,    44.7106224551696f},
  { 9.46232220802562f,    52.7526267976188f},
  { 11.3099324740202f,    60.4659427495364f},
  { 14.0362434679265f,    67.5931287950626f},
  { 18.4349488229220f,    73.9611263864812f},
  { 26.5650511770780f,    79.4446195339388f},
  { 45.0000000000000f,    83.7914553738141f},
  { 90.0f,    85.9143832200251f},
  { 8.74616226255521f,    0.0f},
  { 9.46232220802562f,    4.69911946436983f},
  { 10.3048464687660f,    10.1421061565740f},
  { 11.3099324740202f,    16.3925226947094f},
  { 12.5288077091515f,    23.4541373159961f},
  { 14.0362434679265f,    31.2300697755710f},
  { 15.9453959009229f,    39.4940976758250f},
  { 18.4349488229220f,    47.9019378821367f},
  { 21.8014094863518f,    56.0537049722461f},
  { 26.5650511770780f,    63.5770349075918f},
  { 33.6900675259798f,    70.1729713465918f},
  { 45.0000000000000f,    75.5799318935272f},
  { 63.4349488229220f,    79.4446195339389f},
  { 90.0f,    81.2538377374448f},
  { 14.0362434679265f,    0.0f},
  { 15.2551187030578f,    5.01234637696902f},
  { 16.6992442339936f,    10.8445000673424f},
  { 18.4349488229220f,    17.5484006137923f},
  { 20.5560452195835f,    25.0873294286139f},
  { 23.1985905136482f,    33.2861867456403f},
  { 26.5650511770780f,    41.8103148957786f},
  { 30.9637565320735f,    50.2059345082351f},
  { 36.8698976458440f,    57.9946167919165f},
  { 45.0000000000000f,    64.7605981793211f},
  { 56.3099324740202f,    70.1729713465918f},
  { 71.5650511770780f,    73.9611263864812f},
  { 90.0000000000000f,    75.9637565320735f},
  { 19.9831065219000f,    0.0f},
  { 21.8014094863518f,    5.30457143909761f},
  { 23.9624889745782f,    11.4789227189784f},
  { 26.5650511770780f,    18.5419779639972f},
  { 29.7448812969422f,    26.3877999612430f},
  { 33.6900675259798f,    34.7364812812580f},
  { 38.6598082540901f,    43.1384376196711f},
  { 45.0f,    51.0575587310186f},
  { 53.1301023541560f,    57.9946167919165f},
  { 63.4349488229220f,    63.5770349075918f},
  { 75.9637565320735f,    67.5931287950626f},
  { 90.0000000000000f,    70.0168934781000f},
  { 26.5650511770780f,    0.0f},
  { 29.0546040990771f,    5.54765627378992f},
  { 32.0053832080835f,    11.9694631246073f},
  { 35.5376777919744f,    19.2258557454440f},
  { 39.8055710922652f,    27.1191427733811f},
  { 45.0000000000000f,    35.2643896827547f},
  { 51.3401917459099f,    43.1384376196711f},
  { 59.0362434679265f,    50.2059345082351f},
  { 68.1985905136482f,    56.0537049722461f},
  { 78.6900675259798f,    60.4659427495364f},
  { 90.0f,    63.4349488229220f},
  { 33.6900675259798f,    0.0f},
  { 36.8698976458440f,    5.71059313749964f},
  { 40.6012946450045f,    12.2395582977766f},
  { 45.0f,    19.4712206344907f},
  { 50.1944289077348f,    27.1191427733811f},
  { 56.3099324740202f,    34.7364812812580f},
  { 63.4349488229220f,    41.8103148957786f},
  { 71.5650511770780f,    47.9019378821367f},
  { 80.5376777919744f,    52.7526267976188f},
  { 90.0f,    56.3099324740202f},
  { 41.1859251657097f,    0.0f},
  { 45.0f,    5.76818118618822f},
  { 49.3987053549955f,    12.2395582977766f},
  { 54.4623222080256f,    19.2258557454440f},
  { 60.2551187030578f,    26.3877999612430f},
  { 66.8014094863518f,    33.2861867456403f},
  { 74.0546040990772f,    39.4940976758250f},
  { 81.8698976458440f,    44.7106224551696f},
  { 90.0f,    48.8140748342904f},
  { 48.8140748342904f,    0.0f},
  { 53.1301023541560f,    5.71059313749964f},
  { 57.9946167919165f,    11.9694631246073f},
  { 63.4349488229220f,    18.5419779639972f},
  { 69.4439547804165f,    25.0873294286139f},
  { 75.9637565320735f,    31.2300697755710f},
  { 82.8749836510982f,    36.6569329544821f},
  { 90.0f,    41.1859251657097f},
  { 56.3099324740202f,    0.0f},
  { 60.9453959009229f,    5.54765627378992f},
  { 66.0375110254218f,    11.4789227189784f},
  { 71.5650511770780f,    17.5484006137923f},
  { 77.4711922908485f,    23.4541373159961f},
  { 83.6598082540901f,    28.9056194412282f},
  { 90.0000000000000f,    33.6900675259798f},
  { 63.4349488229220f,    0.0f},
  { 68.1985905136482f,    5.30457143909761f},
  { 73.3007557660064f,    10.8445000673424f},
  { 78.6900675259798f,    16.3925226947094f},
  { 84.2894068625004f,    21.7032913642261f},
  { 90.0f,    26.5650511770780f},
  { 70.0168934781000f,    0.0f},
  { 74.7448812969422f,    5.01234637696902f},
  { 79.6951535312340f,    10.1421061565740f},
  { 84.8055710922652f,    15.1953712458899f},
  { 90.0f,    19.9831065219000f},
  { 75.9637565320735f,    0.0f},
  { 80.5376777919744f,    4.69911946436983f},
  { 85.2363583092738f,    9.43022513800569f},
  { 90.0f,    14.0362434679265f},
  { 81.2538377374448f,    0.0f},
  { 85.6012946450045f,    4.38579953077501f},
  { 90.0f,    8.74616226255521f},
  { 85.9143832200251f,    0.0f},
  { 90.0f,    4.08561677997488f},
  { 94.0856167799749f,    0.0f},
  { 94.3987053549955f,    4.38579953077501f},
  { 94.7636416907262f,    9.43022513800569f},
  { 95.1944289077348f,    15.1953712458899f},
  { 95.7105931374996f,    21.7032913642261f},
  { 96.3401917459099f,    28.9056194412282f},
  { 97.1250163489018f,    36.6569329544820f},
  { 98.1301023541560f,    44.7106224551696f},
  { 99.4623222080256f,    52.7526267976188f},
  { 101.309932474020f,    60.4659427495364f},
  { 104.036243467927f,    67.5931287950626f},
  { 108.434948822922f,    73.9611263864812f},
  { 116.565051177078f,    79.4446195339388f},
  { 135.000000000000f,    83.7914553738141f},
  { 180.0f,    85.9143832200251f},
  { 98.7461622625552f,    0.0f},
  { 99.4623222080256f,    4.69911946436983f},
  { 100.304846468766f,    10.1421061565740f},
  { 101.309932474020f,    16.3925226947094f},
  { 102.528807709152f,    23.4541373159961f},
  { 104.036243467927f,    31.2300697755710f},
  { 105.945395900923f,    39.4940976758250f},
  { 108.434948822922f,    47.9019378821367f},
  { 111.801409486352f,    56.0537049722461f},
  { 116.565051177078f,    63.5770349075918f},
  { 123.690067525980f,    70.1729713465918f},
  { 135.0f,    75.5799318935272f},
  { 153.434948822922f,    79.4446195339389f},
  { 180.0f,    81.2538377374448f},
  { 104.036243467927f,    0.0f},
  { 105.255118703058f,    5.01234637696902f},
  { 106.699244233994f,    10.8445000673424f},
  { 108.434948822922f,    17.5484006137923f},
  { 110.556045219583f,    25.0873294286139f},
  { 113.198590513648f,    33.2861867456403f},
  { 116.565051177078f,    41.8103148957786f},
  { 120.963756532074f,    50.2059345082351f},
  { 126.869897645844f,    57.9946167919165f},
  { 135.0f,    64.7605981793211f},
  { 146.309932474020f,    70.1729713465918f},
  { 161.565051177078f,    73.9611263864812f},
  { -180.000000000000f,    75.9637565320735f},
  { 109.983106521900f,    0.0f},
  { 111.801409486352f,    5.30457143909761f},
  { 113.962488974578f,    11.4789227189784f},
  { 116.565051177078f,    18.5419779639972f},
  { 119.744881296942f,    26.3877999612430f},
  { 123.690067525980f,    34.7364812812580f},
  { 128.659808254090f,    43.1384376196711f},
  { 135.0f,    51.0575587310186f},
  { 143.130102354156f,    57.9946167919165f},
  { 153.434948822922f,    63.5770349075918f},
  { 165.963756532074f,    67.5931287950626f},
  { 180.000000000000f,    70.0168934781000f},
  { 116.565051177078f,    0.0f},
  { 119.054604099077f,    5.54765627378992f},
  { 122.005383208084f,    11.9694631246073f},
  { 125.537677791974f,    19.2258557454440f},
  { 129.805571092265f,    27.1191427733811f},
  { 135.0f,    35.2643896827547f},
  { 141.340191745910f,    43.1384376196711f},
  { 149.036243467926f,    50.2059345082351f},
  { 158.198590513648f,    56.0537049722461f},
  { 168.690067525980f,    60.4659427495364f},
  { 180.0f,    63.4349488229220f},
  { 123.690067525980f,    0.0f},
  { 126.869897645844f,    5.71059313749964f},
  { 130.601294645005f,    12.2395582977766f},
  { 135.0f,    19.4712206344907f},
  { 140.194428907735f,    27.1191427733811f},
  { 146.309932474020f,    34.7364812812580f},
  { 153.434948822922f,    41.8103148957786f},
  { 161.565051177078f,    47.9019378821367f},
  { 170.537677791974f,    52.7526267976188f},
  { 180.0f,    56.3099324740202f},
  { 131.185925165710f,    0.0f},
  { 135.0f,    5.76818118618822f},
  { 139.398705354996f,    12.2395582977766f},
  { 144.462322208026f,    19.2258557454440f},
  { 150.255118703058f,    26.3877999612430f},
  { 156.801409486352f,    33.2861867456403f},
  { 164.054604099077f,    39.4940976758250f},
  { 171.869897645844f,    44.7106224551696f},
  { 180.0f,    48.8140748342904f},
  { 138.814074834290f,    0.0f},
  { 143.130102354156f,    5.71059313749964f},
  { 147.994616791916f,    11.9694631246073f},
  { 153.434948822922f,    18.5419779639972f},
  { 159.443954780417f,    25.0873294286139f},
  { 165.963756532074f,    31.2300697755710f},
  { 172.874983651098f,    36.6569329544821f},
  { 180.0f,    41.1859251657097f},
  { 146.309932474020f,    0.0f},
  { 150.945395900923f,    5.54765627378992f},
  { 156.037511025422f,    11.4789227189784f},
  { 161.565051177078f,    17.5484006137923f},
  { 167.471192290848f,    23.4541373159961f},
  { 173.659808254090f,    28.9056194412282f},
  { -180.0f,    33.6900675259798f},
  { 153.434948822922f,    0.0f},
  { 158.198590513648f,    5.30457143909761f},
  { 163.300755766006f,    10.8445000673424f},
  { 168.690067525980f,    16.3925226947094f},
  { 174.289406862500f,    21.7032913642261f},
  { 180.0f,    26.5650511770780f},
  { 160.016893478100f,    0.0f},
  { 164.744881296942f,    5.01234637696902f},
  { 169.695153531234f,    10.1421061565740f},
  { 174.805571092265f,    15.1953712458899f},
  { 180.0f,    19.9831065219000f},
  { 165.963756532074f,    0.0f},
  { 170.537677791974f,    4.69911946436983f},
  { 175.236358309274f,    9.43022513800569f},
  { 180.0f,    14.0362434679265f},
  { 171.253837737445f,    0.0f},
  { 175.601294645005f,    4.38579953077501f},
  { 180.0f,    8.74616226255521f},
  { 175.914383220025f,    0.0f},
  { 180.0f,    4.08561677997488f},
  { -175.914383220025f,    0.0f},
  { -175.601294645005f,    4.38579953077501f},
  { -175.236358309274f,    9.43022513800569f},
  { -174.805571092265f,    15.1953712458899f},
  { -174.289406862500f,    21.7032913642261f},
  { -173.659808254090f,    28.9056194412282f},
  { -172.874983651098f,    36.6569329544820f},
  { -171.869897645844f,    44.7106224551696f},
  { -170.537677791974f,    52.7526267976188f},
  { -168.690067525980f,    60.4659427495364f},
  { -165.963756532074f,    67.5931287950626f},
  { -161.565051177078f,    73.9611263864812f},
  { -153.434948822922f,    79.4446195339388f},
  { -135.000000000000f,    83.7914553738141f},
  { -90.0f,    85.9143832200251f},
  { -171.253837737445f,    0.0f},
  { -170.537677791974f,    4.69911946436983f},
  { -169.695153531234f,    10.1421061565740f},
  { -168.690067525980f,    16.3925226947094f},
  { -167.471192290848f,    23.4541373159961f},
  { -165.963756532074f,    31.2300697755710f},
  { -164.054604099077f,    39.4940976758250f},
  { -161.565051177078f,    47.9019378821367f},
  { -158.198590513648f,    56.0537049722461f},
  { -153.434948822922f,    63.5770349075918f},
  { -146.309932474020f,    70.1729713465918f},
  { -135.000000000000f,    75.5799318935272f},
  { -116.565051177078f,    79.4446195339389f},
  { -90.0f,    81.2538377374448f},
  { -165.963756532074f,    0.0f},
  { -164.744881296942f,    5.01234637696902f},
  { -163.300755766006f,    10.8445000673424f},
  { -161.565051177078f,    17.5484006137923f},
  { -159.443954780417f,    25.0873294286139f},
  { -156.801409486352f,    33.2861867456403f},
  { -153.434948822922f,    41.8103148957786f},
  { -149.036243467926f,    50.2059345082351f},
  { -143.130102354156f,    57.9946167919165f},
  { -135.0f,    64.7605981793211f},
  { -123.690067525980f,    70.1729713465918f},
  { -108.434948822922f,    73.9611263864812f},
  { -90.0000000000000f,    75.9637565320735f},
  { -160.016893478100f,    0.0f},
  { -158.198590513648f,    5.30457143909761f},
  { -156.037511025422f,    11.4789227189784f},
  { -153.434948822922f,    18.5419779639972f},
  { -150.255118703058f,    26.3877999612430f},
  { -146.309932474020f,    34.7364812812580f},
  { -141.340191745910f,    43.1384376196711f},
  { -135.0f,    51.0575587310186f},
  { -126.869897645844f,    57.9946167919165f},
  { -116.565051177078f,    63.5770349075918f},
  { -104.036243467927f,    67.5931287950626f},
  { -90.0000000000000f,    70.0168934781000f},
  { -153.434948822922f,    0.0f},
  { -150.945395900923f,    5.54765627378992f},
  { -147.994616791917f,    11.9694631246073f},
  { -144.462322208026f,    19.2258557454440f},
  { -140.194428907735f,    27.1191427733811f},
  { -135.0f,    35.2643896827547f},
  { -128.659808254090f,    43.1384376196711f},
  { -120.963756532074f,    50.2059345082351f},
  { -111.801409486352f,    56.0537049722461f},
  { -101.309932474020f,    60.4659427495364f},
  { -90.0f,    63.4349488229220f},
  { -146.309932474020f,    0.0f},
  { -143.130102354156f,    5.71059313749964f},
  { -139.398705354996f,    12.2395582977766f},
  { -135.0f,    19.4712206344907f},
  { -129.805571092265f,    27.1191427733811f},
  { -123.690067525980f,    34.7364812812580f},
  { -116.565051177078f,    41.8103148957786f},
  { -108.434948822922f,    47.9019378821367f},
  { -99.4623222080256f,    52.7526267976188f},
  { -90.0f,    56.3099324740202f},
  { -138.814074834290f,    0.0f},
  { -135.0f,    5.76818118618822f},
  { -130.601294645005f,    12.2395582977766f},
  { -125.537677791974f,    19.2258557454440f},
  { -119.744881296942f,    26.3877999612430f},
  { -113.198590513648f,    33.2861867456403f},
  { -105.945395900923f,    39.4940976758250f},
  { -98.1301023541560f,    44.7106224551696f},
  { -90.0f,    48.8140748342904f},
  { -131.185925165710f,    0.0f},
  { -126.869897645844f,    5.71059313749964f},
  { -122.005383208084f,    11.9694631246073f},
  { -116.565051177078f,    18.5419779639972f},
  { -110.556045219583f,    25.0873294286139f},
  { -104.036243467927f,    31.2300697755710f},
  { -97.1250163489018f,    36.6569329544821f},
  { -90.0f,    41.1859251657097f},
  { -123.690067525980f,    0.0f},
  { -119.054604099077f,    5.54765627378992f},
  { -113.962488974578f,    11.4789227189784f},
  { -108.434948822922f,    17.5484006137923f},
  { -102.528807709152f,    23.4541373159961f},
  { -96.3401917459099f,    28.9056194412282f},
  { -90.0f,    33.6900675259798f},
  { -116.565051177078f,    0.0f},
  { -111.801409486352f,    5.30457143909761f},
  { -106.699244233994f,    10.8445000673424f},
  { -101.309932474020f,    16.3925226947094f},
  { -95.7105931374996f,    21.7032913642261f},
  { -90.0f,    26.5650511770780f},
  { -109.983106521900f,    0.0f},
  { -105.255118703058f,    5.01234637696902f},
  { -100.304846468766f,    10.1421061565740f},
  { -95.1944289077348f,    15.1953712458899f},
  { -90.0f,    19.9831065219000f},
  { -104.036243467927f,    0.0f},
  { -99.4623222080256f,    4.69911946436983f},
  { -94.7636416907262f,    9.43022513800569f},
  { -90.0f,    14.0362434679265f},
  { -98.7461622625552f,    0.0f},
  { -94.3987053549955f,    4.38579953077501f},
  { -90.0f,    8.74616226255521f},
  { -94.0856167799749f,    0.0f},
  { -90.0f,    4.08561677997488f},
  { -85.9143832200251f,    0.0f},
  { -85.6012946450045f,    4.38579953077501f},
  { -85.2363583092738f,    9.43022513800569f},
  { -84.8055710922652f,    15.1953712458899f},
  { -84.2894068625004f,    21.7032913642261f},
  { -83.6598082540901f,    28.9056194412282f},
  { -82.8749836510982f,    36.6569329544820f},
  { -81.8698976458440f,    44.7106224551696f},
  { -80.5376777919744f,    52.7526267976188f},
  { -78.6900675259798f,    60.4659427495364f},
  { -75.9637565320735f,    67.5931287950626f},
  { -71.5650511770780f,    73.9611263864812f},
  { -63.4349488229220f,    79.4446195339388f},
  { -45.0000000000001f,    83.7914553738141f},
  { -81.2538377374448f,    0.0f},
  { -80.5376777919744f,    4.69911946436983f},
  { -79.6951535312340f,    10.1421061565740f},
  { -78.6900675259798f,    16.3925226947094f},
  { -77.4711922908485f,    23.4541373159961f},
  { -75.9637565320735f,    31.2300697755710f},
  { -74.0546040990772f,    39.4940976758250f},
  { -71.5650511770780f,    47.9019378821367f},
  { -68.1985905136482f,    56.0537049722461f},
  { -63.4349488229220f,    63.5770349075918f},
  { -56.3099324740202f,    70.1729713465918f},
  { -45.0000000000000f,    75.5799318935272f},
  { -26.5650511770780f,    79.4446195339389f},
  { -75.9637565320735f,    0.0f},
  { -74.7448812969422f,    5.01234637696902f},
  { -73.3007557660064f,    10.8445000673424f},
  { -71.5650511770780f,    17.5484006137923f},
  { -69.4439547804165f,    25.0873294286139f},
  { -66.8014094863518f,    33.2861867456403f},
  { -63.4349488229220f,    41.8103148957786f},
  { -59.0362434679265f,    50.2059345082351f},
  { -53.1301023541560f,    57.9946167919165f},
  { -45.0f,    64.7605981793211f},
  { -33.6900675259798f,    70.1729713465918f},
  { -18.4349488229220f,    73.9611263864812f},
  { -70.0168934781000f,    0.0f},
  { -68.1985905136482f,    5.30457143909761f},
  { -66.0375110254218f,    11.4789227189784f},
  { -63.4349488229220f,    18.5419779639972f},
  { -60.2551187030578f,    26.3877999612430f},
  { -56.3099324740202f,    34.7364812812580f},
  { -51.3401917459099f,    43.1384376196711f},
  { -45.0f,    51.0575587310186f},
  { -36.8698976458440f,    57.9946167919165f},
  { -26.5650511770780f,    63.5770349075918f},
  { -14.0362434679265f,    67.5931287950626f},
  { -63.4349488229220f,    0.0f},
  { -60.9453959009229f,    5.54765627378992f},
  { -57.9946167919165f,    11.9694631246073f},
  { -54.4623222080256f,    19.2258557454440f},
  { -50.1944289077348f,    27.1191427733811f},
  { -45.0000000000000f,    35.2643896827547f},
  { -38.6598082540901f,    43.1384376196711f},
  { -30.9637565320735f,    50.2059345082351f},
  { -21.8014094863518f,    56.0537049722461f},
  { -11.3099324740202f,    60.4659427495364f},
  { -56.3099324740202f,    0.0f},
  { -53.1301023541560f,    5.71059313749964f},
  { -49.3987053549955f,    12.2395582977766f},
  { -45.0f,    19.4712206344907f},
  { -39.8055710922652f,    27.1191427733811f},
  { -33.6900675259798f,    34.7364812812580f},
  { -26.5650511770780f,    41.8103148957786f},
  { -18.4349488229220f,    47.9019378821367f},
  { -9.46232220802562f,    52.7526267976188f},
  { -48.8140748342904f,    0.0f},
  { -45.0f,    5.76818118618822f},
  { -40.6012946450045f,    12.2395582977766f},
  { -35.5376777919744f,    19.2258557454440f},
  { -29.7448812969422f,    26.3877999612430f},
  { -23.1985905136482f,    33.2861867456403f},
  { -15.9453959009228f,    39.4940976758250f},
  { -8.13010235415598f,    44.7106224551696f},
  { -41.1859251657097f,    0.0f},
  { -36.8698976458440f,    5.71059313749964f},
  { -32.0053832080835f,    11.9694631246073f},
  { -26.5650511770780f,    18.5419779639972f},
  { -20.5560452195835f,    25.0873294286139f},
  { -14.0362434679265f,    31.2300697755710f},
  { -7.12501634890180f,    36.6569329544821f},
  { -33.6900675259798f,    0.0f},
  { -29.0546040990772f,    5.54765627378992f},
  { -23.9624889745782f,    11.4789227189784f},
  { -18.4349488229220f,    17.5484006137923f},
  { -12.5288077091515f,    23.4541373159961f},
  { -6.34019174590991f,    28.9056194412282f},
  { -26.5650511770780f,    0.0f},
  { -21.8014094863518f,    5.30457143909761f},
  { -16.6992442339936f,    10.8445000673424f},
  { -11.3099324740202f,    16.3925226947094f},
  { -5.71059313749964f,    21.7032913642261f},
  { -19.9831065219000f,    0.0f},
  { -15.2551187030578f,    5.01234637696902f},
  { -10.3048464687660f,    10.1421061565740f},
  { -5.19442890773481f,    15.1953712458899f},
  { -14.0362434679265f,    0.0f},
  { -9.46232220802562f,    4.69911946436983f},
  { -4.76364169072618f,    9.43022513800569f},
  { -8.74616226255521f,    0.0f},
  { -4.39870535499553f,    4.38579953077501f},
  { -4.08561677997488f,    0.0f},
  { 0.0f,    -4.08561677997488f},
  { 4.39870535499553f,    -4.38579953077501f},
  { 9.46232220802562f,    -4.69911946436983f},
  { 15.2551187030578f,    -5.01234637696902f},
  { 21.8014094863518f,    -5.30457143909761f},
  { 29.0546040990772f,    -5.54765627378992f},
  { 36.8698976458440f,    -5.71059313749964f},
  { 45.0f,    -5.76818118618822f},
  { 53.1301023541560f,    -5.71059313749964f},
  { 60.9453959009229f,    -5.54765627378992f},
  { 68.1985905136482f,    -5.30457143909761f},
  { 74.7448812969422f,    -5.01234637696902f},
  { 80.5376777919744f,    -4.69911946436983f},
  { 85.6012946450045f,    -4.38579953077501f},
  { 90.0f,    -4.08561677997488f},
  { 0.0f,    -8.74616226255521f},
  { 4.76364169072618f,    -9.43022513800569f},
  { 10.3048464687660f,    -10.1421061565740f},
  { 16.6992442339936f,    -10.8445000673424f},
  { 23.9624889745782f,    -11.4789227189784f},
  { 32.0053832080835f,    -11.9694631246073f},
  { 40.6012946450045f,    -12.2395582977766f},
  { 49.3987053549955f,    -12.2395582977766f},
  { 57.9946167919165f,    -11.9694631246073f},
  { 66.0375110254218f,    -11.4789227189784f},
  { 73.3007557660064f,    -10.8445000673424f},
  { 79.6951535312340f,    -10.1421061565740f},
  { 85.2363583092738f,    -9.43022513800569f},
  { 90.0f,    -8.74616226255521f},
  { 0.0f,    -14.0362434679265f},
  { 5.19442890773481f,    -15.1953712458899f},
  { 11.3099324740202f,    -16.3925226947094f},
  { 18.4349488229220f,    -17.5484006137923f},
  { 26.5650511770780f,    -18.5419779639972f},
  { 35.5376777919744f,    -19.2258557454440f},
  { 45.0000000000000f,    -19.4712206344907f},
  { 54.4623222080256f,    -19.2258557454440f},
  { 63.4349488229220f,    -18.5419779639972f},
  { 71.5650511770780f,    -17.5484006137923f},
  { 78.6900675259798f,    -16.3925226947094f},
  { 84.8055710922652f,    -15.1953712458899f},
  { 90.0000000000000f,    -14.0362434679265f},
  { 0.0f,    -19.9831065219000f},
  { 5.71059313749964f,    -21.7032913642261f},
  { 12.5288077091515f,    -23.4541373159961f},
  { 20.5560452195835f,    -25.0873294286139f},
  { 29.7448812969422f,    -26.3877999612430f},
  { 39.8055710922652f,    -27.1191427733811f},
  { 50.1944289077348f,    -27.1191427733811f},
  { 60.2551187030578f,    -26.3877999612430f},
  { 69.4439547804165f,    -25.0873294286139f},
  { 77.4711922908485f,    -23.4541373159961f},
  { 84.2894068625004f,    -21.7032913642261f},
  { 90.0f,    -19.9831065219000f},
  { 0.0f,    -26.5650511770780f},
  { 6.34019174590991f,    -28.9056194412281f},
  { 14.0362434679265f,    -31.2300697755710f},
  { 23.1985905136482f,    -33.2861867456403f},
  { 33.6900675259798f,    -34.7364812812580f},
  { 45.0000000000000f,    -35.2643896827547f},
  { 56.3099324740202f,    -34.7364812812580f},
  { 66.8014094863518f,    -33.2861867456403f},
  { 75.9637565320735f,    -31.2300697755710f},
  { 83.6598082540901f,    -28.9056194412281f},
  { 90.0f,    -26.5650511770780f},
  { 0.0f,    -33.6900675259798f},
  { 7.12501634890180f,    -36.6569329544821f},
  { 15.9453959009229f,    -39.4940976758250f},
  { 26.5650511770780f,    -41.8103148957786f},
  { 38.6598082540901f,    -43.1384376196711f},
  { 51.3401917459099f,    -43.1384376196711f},
  { 63.4349488229220f,    -41.8103148957786f},
  { 74.0546040990772f,    -39.4940976758250f},
  { 82.8749836510982f,    -36.6569329544821f},
  { 90.0f,    -33.6900675259798f},
  { 0.0f,    -41.1859251657097f},
  { 8.13010235415598f,    -44.7106224551696f},
  { 18.4349488229220f,    -47.9019378821367f},
  { 30.9637565320735f,    -50.2059345082351f},
  { 45.0f,    -51.0575587310186f},
  { 59.0362434679265f,    -50.2059345082351f},
  { 71.5650511770780f,    -47.9019378821367f},
  { 81.8698976458440f,    -44.7106224551696f},
  { 90.0f,    -41.1859251657097f},
  { 0.0f,    -48.8140748342904f},
  { 9.46232220802562f,    -52.7526267976188f},
  { 21.8014094863518f,    -56.0537049722461f},
  { 36.8698976458440f,    -57.9946167919165f},
  { 53.1301023541560f,    -57.9946167919165f},
  { 68.1985905136482f,    -56.0537049722461f},
  { 80.5376777919744f,    -52.7526267976188f},
  { 90.0f,    -48.8140748342904f},
  { 0.0f,    -56.3099324740202f},
  { 11.3099324740202f,    -60.4659427495364f},
  { 26.5650511770780f,    -63.5770349075918f},
  { 45.0f,    -64.7605981793211f},
  { 63.4349488229220f,    -63.5770349075918f},
  { 78.6900675259798f,    -60.4659427495364f},
  { 90.0000000000000f,    -56.3099324740202f},
  { 0.0f,    -63.4349488229220f},
  { 14.0362434679265f,    -67.5931287950626f},
  { 33.6900675259798f,    -70.1729713465918f},
  { 56.3099324740202f,    -70.1729713465918f},
  { 75.9637565320735f,    -67.5931287950626f},
  { 90.0f,    -63.4349488229220f},
  { 0.0f,    -70.0168934781000f},
  { 18.4349488229220f,    -73.9611263864812f},
  { 45.0f,    -75.5799318935272f},
  { 71.5650511770780f,    -73.9611263864812f},
  { 90.0f,    -70.0168934781000f},
  { 0.0f,    -75.9637565320735f},
  { 26.5650511770780f,    -79.4446195339389f},
  { 63.4349488229220f,    -79.4446195339389f},
  { 90.0f,    -75.9637565320735f},
  { 0.0f,    -81.2538377374448f},
  { 45.0000000000000f,    -83.7914553738142f},
  { 90.0f,    -81.2538377374448f},
  { 0.0f,    -85.9143832200251f},
  { 90.0f,    -85.9143832200251f},
  { 94.3987053549955f,    -4.38579953077501f},
  { 99.4623222080256f,    -4.69911946436983f},
  { 105.255118703058f,    -5.01234637696902f},
  { 111.801409486352f,    -5.30457143909761f},
  { 119.054604099077f,    -5.54765627378992f},
  { 126.869897645844f,    -5.71059313749964f},
  { 135.0f,    -5.76818118618822f},
  { 143.130102354156f,    -5.71059313749964f},
  { 150.945395900923f,    -5.54765627378992f},
  { 158.198590513648f,    -5.30457143909761f},
  { 164.744881296942f,    -5.01234637696902f},
  { 170.537677791974f,    -4.69911946436983f},
  { 175.601294645005f,    -4.38579953077501f},
  { 180.0f,    -4.08561677997488f},
  { 94.7636416907262f,    -9.43022513800569f},
  { 100.304846468766f,    -10.1421061565740f},
  { 106.699244233994f,    -10.8445000673424f},
  { 113.962488974578f,    -11.4789227189784f},
  { 122.005383208084f,    -11.9694631246073f},
  { 130.601294645005f,    -12.2395582977766f},
  { 139.398705354996f,    -12.2395582977766f},
  { 147.994616791916f,    -11.9694631246073f},
  { 156.037511025422f,    -11.4789227189784f},
  { 163.300755766006f,    -10.8445000673424f},
  { 169.695153531234f,    -10.1421061565740f},
  { 175.236358309274f,    -9.43022513800569f},
  { 180.0f,    -8.74616226255521f},
  { 95.1944289077348f,    -15.1953712458899f},
  { 101.309932474020f,    -16.3925226947094f},
  { 108.434948822922f,    -17.5484006137923f},
  { 116.565051177078f,    -18.5419779639972f},
  { 125.537677791974f,    -19.2258557454440f},
  { 135.0f,    -19.4712206344907f},
  { 144.462322208026f,    -19.2258557454440f},
  { 153.434948822922f,    -18.5419779639972f},
  { 161.565051177078f,    -17.5484006137923f},
  { 168.690067525980f,    -16.3925226947094f},
  { 174.805571092265f,    -15.1953712458899f},
  { -180.0f,    -14.0362434679265f},
  { 95.7105931374996f,    -21.7032913642261f},
  { 102.528807709152f,    -23.4541373159961f},
  { 110.556045219583f,    -25.0873294286139f},
  { 119.744881296942f,    -26.3877999612430f},
  { 129.805571092265f,    -27.1191427733811f},
  { 140.194428907735f,    -27.1191427733811f},
  { 150.255118703058f,    -26.3877999612430f},
  { 159.443954780417f,    -25.0873294286139f},
  { 167.471192290848f,    -23.4541373159961f},
  { 174.289406862500f,    -21.7032913642261f},
  { 180.0f,    -19.9831065219000f},
  { 96.3401917459099f,    -28.9056194412281f},
  { 104.036243467927f,    -31.2300697755710f},
  { 113.198590513648f,    -33.2861867456403f},
  { 123.690067525980f,    -34.7364812812580f},
  { 135.0f,    -35.2643896827547f},
  { 146.309932474020f,    -34.7364812812580f},
  { 156.801409486352f,    -33.2861867456403f},
  { 165.963756532074f,    -31.2300697755710f},
  { 173.659808254090f,    -28.9056194412281f},
  { 180.0f,    -26.5650511770780f},
  { 97.1250163489018f,    -36.6569329544821f},
  { 105.945395900923f,    -39.4940976758250f},
  { 116.565051177078f,    -41.8103148957786f},
  { 128.659808254090f,    -43.1384376196711f},
  { 141.340191745910f,    -43.1384376196711f},
  { 153.434948822922f,    -41.8103148957786f},
  { 164.054604099077f,    -39.4940976758250f},
  { 172.874983651098f,    -36.6569329544821f},
  { 180.0f,    -33.6900675259798f},
  { 98.1301023541560f,    -44.7106224551696f},
  { 108.434948822922f,    -47.9019378821367f},
  { 120.963756532074f,    -50.2059345082351f},
  { 135.0f,    -51.0575587310186f},
  { 149.036243467926f,    -50.2059345082351f},
  { 161.565051177078f,    -47.9019378821367f},
  { 171.869897645844f,    -44.7106224551696f},
  { 180.0f,    -41.1859251657097f},
  { 99.4623222080256f,    -52.7526267976188f},
  { 111.801409486352f,    -56.0537049722461f},
  { 126.869897645844f,    -57.9946167919165f},
  { 143.130102354156f,    -57.9946167919165f},
  { 158.198590513648f,    -56.0537049722461f},
  { 170.537677791974f,    -52.7526267976188f},
  { 180.0f,    -48.8140748342904f},
  { 101.309932474020f,    -60.4659427495364f},
  { 116.565051177078f,    -63.5770349075918f},
  { 135.0f,    -64.7605981793211f},
  { 153.434948822922f,    -63.5770349075918f},
  { 168.690067525980f,    -60.4659427495364f},
  { -180.0f,    -56.3099324740202f},
  { 104.036243467927f,    -67.5931287950626f},
  { 123.690067525980f,    -70.1729713465918f},
  { 146.309932474020f,    -70.1729713465918f},
  { 165.963756532074f,    -67.5931287950626f},
  { 180.0f,    -63.4349488229220f},
  { 108.434948822922f,    -73.9611263864812f},
  { 135.0f,    -75.5799318935272f},
  { 161.565051177078f,    -73.9611263864812f},
  { 180.0f,    -70.0168934781000f},
  { 116.565051177078f,    -79.4446195339389f},
  { 153.434948822922f,    -79.4446195339389f},
  { 180.0f,    -75.9637565320735f},
  { 135.0f,    -83.7914553738142f},
  { 180.0f,    -81.2538377374448f},
  { 180.0f,    -85.9143832200251f},
  { -175.601294645005f,    -4.38579953077501f},
  { -170.537677791974f,    -4.69911946436983f},
  { -164.744881296942f,    -5.01234637696902f},
  { -158.198590513648f,    -5.30457143909761f},
  { -150.945395900923f,    -5.54765627378992f},
  { -143.130102354156f,    -5.71059313749964f},
  { -135.0f,    -5.76818118618822f},
  { -126.869897645844f,    -5.71059313749964f},
  { -119.054604099077f,    -5.54765627378992f},
  { -111.801409486352f,    -5.30457143909761f},
  { -105.255118703058f,    -5.01234637696902f},
  { -99.4623222080256f,    -4.69911946436983f},
  { -94.3987053549956f,    -4.38579953077501f},
  { -90.0f,    -4.08561677997488f},
  { -175.236358309274f,    -9.43022513800569f},
  { -169.695153531234f,    -10.1421061565740f},
  { -163.300755766006f,    -10.8445000673424f},
  { -156.037511025422f,    -11.4789227189784f},
  { -147.994616791916f,    -11.9694631246073f},
  { -139.398705354996f,    -12.2395582977766f},
  { -130.601294645005f,    -12.2395582977766f},
  { -122.005383208084f,    -11.9694631246073f},
  { -113.962488974578f,    -11.4789227189784f},
  { -106.699244233994f,    -10.8445000673424f},
  { -100.304846468766f,    -10.1421061565740f},
  { -94.7636416907262f,    -9.43022513800569f},
  { -90.0f,    -8.74616226255521f},
  { -174.805571092265f,    -15.1953712458899f},
  { -168.690067525980f,    -16.3925226947094f},
  { -161.565051177078f,    -17.5484006137923f},
  { -153.434948822922f,    -18.5419779639972f},
  { -144.462322208026f,    -19.2258557454440f},
  { -135.0f,    -19.4712206344907f},
  { -125.537677791974f,    -19.2258557454440f},
  { -116.565051177078f,    -18.5419779639972f},
  { -108.434948822922f,    -17.5484006137923f},
  { -101.309932474020f,    -16.3925226947094f},
  { -95.1944289077348f,    -15.1953712458899f},
  { -90.0f,    -14.0362434679265f},
  { -174.289406862500f,    -21.7032913642261f},
  { -167.471192290848f,    -23.4541373159961f},
  { -159.443954780417f,    -25.0873294286139f},
  { -150.255118703058f,    -26.3877999612430f},
  { -140.194428907735f,    -27.1191427733811f},
  { -129.805571092265f,    -27.1191427733811f},
  { -119.744881296942f,    -26.3877999612430f},
  { -110.556045219583f,    -25.0873294286139f},
  { -102.528807709152f,    -23.4541373159961f},
  { -95.7105931374996f,    -21.7032913642261f},
  { -90.0000000000000f,    -19.9831065219000f},
  { -173.659808254090f,    -28.9056194412281f},
  { -165.963756532074f,    -31.2300697755710f},
  { -156.801409486352f,    -33.2861867456403f},
  { -146.309932474020f,    -34.7364812812580f},
  { -135.0f,    -35.2643896827547f},
  { -123.690067525980f,    -34.7364812812580f},
  { -113.198590513648f,    -33.2861867456403f},
  { -104.036243467927f,    -31.2300697755710f},
  { -96.3401917459099f,    -28.9056194412281f},
  { -90.0f,    -26.5650511770780f},
  { -172.874983651098f,    -36.6569329544821f},
  { -164.054604099077f,    -39.4940976758250f},
  { -153.434948822922f,    -41.8103148957786f},
  { -141.340191745910f,    -43.1384376196711f},
  { -128.659808254090f,    -43.1384376196711f},
  { -116.565051177078f,    -41.8103148957786f},
  { -105.945395900923f,    -39.4940976758250f},
  { -97.1250163489018f,    -36.6569329544821f},
  { -90.0f,    -33.6900675259798f},
  { -171.869897645844f,    -44.7106224551696f},
  { -161.565051177078f,    -47.9019378821367f},
  { -149.036243467926f,    -50.2059345082351f},
  { -135.0f,    -51.0575587310186f},
  { -120.963756532074f,    -50.2059345082351f},
  { -108.434948822922f,    -47.9019378821367f},
  { -98.1301023541560f,    -44.7106224551696f},
  { -90.0f,    -41.1859251657097f},
  { -170.537677791974f,    -52.7526267976188f},
  { -158.198590513648f,    -56.0537049722461f},
  { -143.130102354156f,    -57.9946167919165f},
  { -126.869897645844f,    -57.9946167919165f},
  { -111.801409486352f,    -56.0537049722461f},
  { -99.4623222080256f,    -52.7526267976188f},
  { -90.0f,    -48.8140748342904f},
  { -168.690067525980f,    -60.4659427495364f},
  { -153.434948822922f,    -63.5770349075918f},
  { -135.0f,    -64.7605981793211f},
  { -116.565051177078f,    -63.5770349075918f},
  { -101.309932474020f,    -60.4659427495364f},
  { -90.0f,    -56.3099324740202f},
  { -165.963756532074f,    -67.5931287950626f},
  { -146.309932474020f,    -70.1729713465918f},
  { -123.690067525980f,    -70.1729713465918f},
  { -104.036243467927f,    -67.5931287950626f},
  { -90.0f,    -63.4349488229220f},
  { -161.565051177078f,    -73.9611263864812f},
  { -135.0f,    -75.5799318935272f},
  { -108.434948822922f,    -73.9611263864812f},
  { -90.0f,    -70.0168934781000f},
  { -153.434948822922f,    -79.4446195339389f},
  { -116.565051177078f,    -79.4446195339389f},
  { -90.0f,    -75.9637565320735f},
  { -135.0f,    -83.7914553738142f},
  { -90.0f,    -81.2538377374448f},
  { -90.0f,    -85.9143832200251f},
  { -85.6012946450045f,    -4.38579953077501f},
  { -80.5376777919744f,    -4.69911946436983f},
  { -74.7448812969422f,    -5.01234637696902f},
  { -68.1985905136482f,    -5.30457143909761f},
  { -60.9453959009229f,    -5.54765627378992f},
  { -53.1301023541560f,    -5.71059313749964f},
  { -45.0f,    -5.76818118618822f},
  { -36.8698976458440f,    -5.71059313749964f},
  { -29.0546040990772f,    -5.54765627378992f},
  { -21.8014094863518f,    -5.30457143909761f},
  { -15.2551187030578f,    -5.01234637696902f},
  { -9.46232220802563f,    -4.69911946436983f},
  { -4.39870535499554f,    -4.38579953077501f},
  { -85.2363583092738f,    -9.43022513800569f},
  { -79.6951535312340f,    -10.1421061565740f},
  { -73.3007557660064f,    -10.8445000673424f},
  { -66.0375110254218f,    -11.4789227189784f},
  { -57.9946167919165f,    -11.9694631246073f},
  { -49.3987053549955f,    -12.2395582977766f},
  { -40.6012946450045f,    -12.2395582977766f},
  { -32.0053832080835f,    -11.9694631246073f},
  { -23.9624889745782f,    -11.4789227189784f},
  { -16.6992442339936f,    -10.8445000673424f},
  { -10.3048464687660f,    -10.1421061565740f},
  { -4.76364169072618f,    -9.43022513800569f},
  { -84.8055710922652f,    -15.1953712458899f},
  { -78.6900675259798f,    -16.3925226947094f},
  { -71.5650511770780f,    -17.5484006137923f},
  { -63.4349488229220f,    -18.5419779639972f},
  { -54.4623222080256f,    -19.2258557454440f},
  { -45.0000000000000f,    -19.4712206344907f},
  { -35.5376777919744f,    -19.2258557454440f},
  { -26.5650511770780f,    -18.5419779639972f},
  { -18.4349488229220f,    -17.5484006137923f},
  { -11.3099324740202f,    -16.3925226947094f},
  { -5.19442890773480f,    -15.1953712458899f},
  { -84.2894068625004f,    -21.7032913642261f},
  { -77.4711922908485f,    -23.4541373159961f},
  { -69.4439547804165f,    -25.0873294286139f},
  { -60.2551187030578f,    -26.3877999612430f},
  { -50.1944289077348f,    -27.1191427733811f},
  { -39.8055710922652f,    -27.1191427733811f},
  { -29.7448812969422f,    -26.3877999612430f},
  { -20.5560452195835f,    -25.0873294286139f},
  { -12.5288077091515f,    -23.4541373159961f},
  { -5.71059313749964f,    -21.7032913642261f},
  { -83.6598082540901f,    -28.9056194412281f},
  { -75.9637565320735f,    -31.2300697755710f},
  { -66.8014094863518f,    -33.2861867456403f},
  { -56.3099324740202f,    -34.7364812812580f},
  { -45.0000000000000f,    -35.2643896827547f},
  { -33.6900675259798f,    -34.7364812812580f},
  { -23.1985905136482f,    -33.2861867456403f},
  { -14.0362434679265f,    -31.2300697755710f},
  { -6.34019174590991f,    -28.9056194412281f},
  { -82.8749836510982f,    -36.6569329544821f},
  { -74.0546040990772f,    -39.4940976758250f},
  { -63.4349488229220f,    -41.8103148957786f},
  { -51.3401917459099f,    -43.1384376196711f},
  { -38.6598082540901f,    -43.1384376196711f},
  { -26.5650511770780f,    -41.8103148957786f},
  { -15.9453959009228f,    -39.4940976758250f},
  { -7.12501634890180f,    -36.6569329544821f},
  { -81.8698976458440f,    -44.7106224551696f},
  { -71.5650511770780f,    -47.9019378821367f},
  { -59.0362434679265f,    -50.2059345082351f},
  { -45.0f,    -51.0575587310186f},
  { -30.9637565320735f,    -50.2059345082351f},
  { -18.4349488229220f,    -47.9019378821367f},
  { -8.13010235415598f,    -44.7106224551696f},
  { -80.5376777919744f,    -52.7526267976188f},
  { -68.1985905136482f,    -56.0537049722461f},
  { -53.1301023541560f,    -57.9946167919165f},
  { -36.8698976458440f,    -57.9946167919165f},
  { -21.8014094863518f,    -56.0537049722461f},
  { -9.46232220802563f,    -52.7526267976188f},
  { -78.6900675259798f,    -60.4659427495364f},
  { -63.4349488229220f,    -63.5770349075918f},
  { -45.0000000000000f,    -64.7605981793211f},
  { -26.5650511770780f,    -63.5770349075918f},
  { -11.3099324740202f,    -60.4659427495364f},
  { -75.9637565320735f,    -67.5931287950626f},
  { -56.3099324740202f,    -70.1729713465918f},
  { -33.6900675259798f,    -70.1729713465918f},
  { -14.0362434679265f,    -67.5931287950626f},
  { -71.5650511770780f,    -73.9611263864812f},
  { -45.0f,    -75.5799318935272f},
  { -18.4349488229220f,    -73.9611263864812f},
  { -63.4349488229220f,    -79.4446195339389f},
  { -26.5650511770780f,    -79.4446195339389f},
  { -45.0000000000000f,    -83.7914553738142f}};

/* 3LD geosphere 'oct' [16 0] (1026 points) */
const float __geosphere_oct_16_0_dirs_deg[1026][2] =
{ { 0.0f,    0.0f},
  { 180.0f,    0.0f},
  { 90.0f,    0.0f},
  { -90.0f,    0.0f},
  { 0.0f,    90.0f},
  { 0.0f,    -90.0f},
  { 0.0f,    3.81407483429035f},
  { 0.0f,    8.13010235415598f},
  { 0.0f,    12.9946167919165f},
  { 0.0f,    18.4349488229220f},
  { 0.0f,    24.4439547804165f},
  { 0.0f,    30.9637565320735f},
  { 0.0f,    37.8749836510982f},
  { 0.0f,    45.0f},
  { 0.0f,    52.1250163489018f},
  { 0.0f,    59.0362434679265f},
  { 0.0f,    65.5560452195835f},
  { 0.0f,    71.5650511770780f},
  { 0.0f,    77.0053832080835f},
  { 0.0f,    81.8698976458440f},
  { 0.0f,    86.1859251657097f},
  { 3.81407483429035f,    0.0f},
  { 4.08561677997488f,    4.07526902813210f},
  { 4.39870535499553f,    8.72079692816341f},
  { 4.76364169072618f,    13.9896661378727f},
  { 5.19442890773481f,    19.9074988774736f},
  { 5.71059313749964f,    26.4511990993406f},
  { 6.34019174590991f,    33.5280232217311f},
  { 7.12501634890180f,    40.9659247499738f},
  { 8.13010235415598f,    48.5270656910595f},
  { 9.46232220802562f,    55.9467140446854f},
  { 11.3099324740202f,    62.9828767821498f},
  { 14.0362434679265f,    69.4526060024815f},
  { 18.4349488229220f,    75.2368673763685f},
  { 26.5650511770780f,    80.2403277394902f},
  { 45.0000000000000f,    84.2318188138118f},
  { 90.0f,    86.1859251657097f},
  { 8.13010235415598f,    0.0f},
  { 8.74616226255521f,    4.34775315133540f},
  { 9.46232220802562f,    9.33585903200394f},
  { 10.3048464687660f,    15.0202566665138f},
  { 11.3099324740202f,    21.4167140330336f},
  { 12.5288077091515f,    28.4721344245730f},
  { 14.0362434679265f,    36.0398934303039f},
  { 15.9453959009229f,    43.8762651909286f},
  { 18.4349488229220f,    51.6711818985441f},
  { 21.8014094863518f,    59.1057379206062f},
  { 26.5650511770780f,    65.9051574478893f},
  { 33.6900675259798f,    71.8520260682270f},
  { 45.0000000000000f,    76.7373239916952f},
  { 63.4349488229220f,    80.2403277394902f},
  { 90.0f,    81.8698976458440f},
  { 12.9946167919165f,    0.0f},
  { 14.0362434679265f,    4.62203692583713f},
  { 15.2551187030578f,    9.94912629715978f},
  { 16.6992442339936f,    16.0318928587692f},
  { 18.4349488229220f,    22.8620891442349f},
  { 20.5560452195835f,    30.3364156173112f},
  { 23.1985905136482f,    38.2324043541196f},
  { 26.5650511770780f,    46.2194273454749f},
  { 30.9637565320735f,    53.9128529342688f},
  { 36.8698976458440f,    60.9453959009229f},
  { 45.0000000000000f,    67.0102322263912f},
  { 56.3099324740202f,    71.8520260682270f},
  { 71.5650511770780f,    75.2368673763685f},
  { 90.0f,    77.0053832080835f},
  { 18.4349488229220f,    0.0f},
  { 19.9831065219000f,    4.88325044901346f},
  { 21.8014094863518f,    10.5197348906586f},
  { 23.9624889745782f,    16.9409375987462f},
  { 26.5650511770780f,    24.0948425521107f},
  { 29.7448812969422f,    31.8061000193785f},
  { 33.6900675259798f,    39.7621591518051f},
  { 38.6598082540901f,    47.5498444571087f},
  { 45.0f,    54.7356103172453f},
  { 53.1301023541560f,    60.9453959009229f},
  { 63.4349488229220f,    65.9051574478893f},
  { 75.9637565320735f,    69.4526060024815f},
  { 90.0f,    71.5650511770780f},
  { 24.4439547804165f,    0.0f},
  { 26.5650511770780f,    5.11108969528872f},
  { 29.0546040990771f,    10.9932014985969f},
  { 32.0053832080835f,    17.6406309136330f},
  { 35.5376777919744f,    24.9379827032418f},
  { 39.8055710922652f,    32.6267377065311f},
  { 45.0000000000000f,    40.3155422107726f},
  { 51.3401917459099f,    47.5498444571087f},
  { 59.0362434679265f,    53.9128529342688f},
  { 68.1985905136482f,    59.1057379206062f},
  { 78.6900675259798f,    62.9828767821498f},
  { 90.0f,    65.5560452195835f},
  { 30.9637565320735f,    0.0f},
  { 33.6900675259798f,    5.28198245234341f},
  { 36.8698976458440f,    11.3099324740202f},
  { 40.6012946450045f,    18.0246645268103f},
  { 45.0000000000000f,    25.2394018206789f},
  { 50.1944289077348f,    32.6267377065311f},
  { 56.3099324740202f,    39.7621591518051f},
  { 63.4349488229220f,    46.2194273454749f},
  { 71.5650511770780f,    51.6711818985441f},
  { 80.5376777919744f,    55.9467140446854f},
  { 90.0f,    59.0362434679265f},
  { 37.8749836510982f,    0.0f},
  { 41.1859251657097f,    5.37411792901766f},
  { 45.0f,    11.4217536589623f},
  { 49.3987053549955f,    18.0246645268103f},
  { 54.4623222080256f,    24.9379827032418f},
  { 60.2551187030578f,    31.8061000193785f},
  { 66.8014094863518f,    38.2324043541196f},
  { 74.0546040990772f,    43.8762651909286f},
  { 81.8698976458440f,    48.5270656910595f},
  { 90.0f,    52.1250163489018f},
  { 45.0f,    0.0f},
  { 48.8140748342904f,    5.37411792901766f},
  { 53.1301023541560f,    11.3099324740202f},
  { 57.9946167919165f,    17.6406309136330f},
  { 63.4349488229220f,    24.0948425521107f},
  { 69.4439547804165f,    30.3364156173112f},
  { 75.9637565320735f,    36.0398934303039f},
  { 82.8749836510982f,    40.9659247499738f},
  { 90.0f,    45.0f},
  { 52.1250163489018f,    0.0f},
  { 56.3099324740202f,    5.28198245234341f},
  { 60.9453959009229f,    10.9932014985969f},
  { 66.0375110254218f,    16.9409375987462f},
  { 71.5650511770780f,    22.8620891442349f},
  { 77.4711922908485f,    28.4721344245730f},
  { 83.6598082540901f,    33.5280232217311f},
  { 90.0f,    37.8749836510982f},
  { 59.0362434679265f,    0.0f},
  { 63.4349488229220f,    5.11108969528872f},
  { 68.1985905136482f,    10.5197348906586f},
  { 73.3007557660064f,    16.0318928587692f},
  { 78.6900675259798f,    21.4167140330336f},
  { 84.2894068625004f,    26.4511990993406f},
  { 90.0f,    30.9637565320735f},
  { 65.5560452195835f,    0.0f},
  { 70.0168934781000f,    4.88325044901346f},
  { 74.7448812969422f,    9.94912629715978f},
  { 79.6951535312340f,    15.0202566665138f},
  { 84.8055710922652f,    19.9074988774736f},
  { 90.0f,    24.4439547804165f},
  { 71.5650511770780f,    0.0f},
  { 75.9637565320735f,    4.62203692583713f},
  { 80.5376777919744f,    9.33585903200394f},
  { 85.2363583092738f,    13.9896661378727f},
  { 90.0f,    18.4349488229220f},
  { 77.0053832080835f,    0.0f},
  { 81.2538377374448f,    4.34775315133540f},
  { 85.6012946450045f,    8.72079692816341f},
  { 90.0f,    12.9946167919165f},
  { 81.8698976458440f,    0.0f},
  { 85.9143832200251f,    4.07526902813210f},
  { 90.0f,    8.13010235415598f},
  { 86.1859251657097f,    0.0f},
  { 90.0f,    3.81407483429035f},
  { 93.8140748342904f,    0.0f},
  { 94.0856167799749f,    4.07526902813210f},
  { 94.3987053549955f,    8.72079692816341f},
  { 94.7636416907262f,    13.9896661378727f},
  { 95.1944289077348f,    19.9074988774736f},
  { 95.7105931374996f,    26.4511990993406f},
  { 96.3401917459099f,    33.5280232217311f},
  { 97.1250163489018f,    40.9659247499738f},
  { 98.1301023541560f,    48.5270656910595f},
  { 99.4623222080256f,    55.9467140446854f},
  { 101.309932474020f,    62.9828767821498f},
  { 104.036243467927f,    69.4526060024815f},
  { 108.434948822922f,    75.2368673763685f},
  { 116.565051177078f,    80.2403277394902f},
  { 135.0f,    84.2318188138118f},
  { 180.0f,    86.1859251657097f},
  { 98.1301023541560f,    0.0f},
  { 98.7461622625552f,    4.34775315133540f},
  { 99.4623222080256f,    9.33585903200394f},
  { 100.304846468766f,    15.0202566665138f},
  { 101.309932474020f,    21.4167140330336f},
  { 102.528807709152f,    28.4721344245730f},
  { 104.036243467927f,    36.0398934303039f},
  { 105.945395900923f,    43.8762651909286f},
  { 108.434948822922f,    51.6711818985441f},
  { 111.801409486352f,    59.1057379206062f},
  { 116.565051177078f,    65.9051574478893f},
  { 123.690067525980f,    71.8520260682270f},
  { 135.0f,    76.7373239916952f},
  { 153.434948822922f,    80.2403277394902f},
  { 180.0f,    81.8698976458440f},
  { 102.994616791917f,    0.0f},
  { 104.036243467927f,    4.62203692583713f},
  { 105.255118703058f,    9.94912629715978f},
  { 106.699244233994f,    16.0318928587692f},
  { 108.434948822922f,    22.8620891442349f},
  { 110.556045219583f,    30.3364156173112f},
  { 113.198590513648f,    38.2324043541196f},
  { 116.565051177078f,    46.2194273454749f},
  { 120.963756532074f,    53.9128529342687f},
  { 126.869897645844f,    60.9453959009229f},
  { 135.0f,    67.0102322263912f},
  { 146.309932474020f,    71.8520260682270f},
  { 161.565051177078f,    75.2368673763685f},
  { 180.0f,    77.0053832080835f},
  { 108.434948822922f,    0.0f},
  { 109.983106521900f,    4.88325044901346f},
  { 111.801409486352f,    10.5197348906586f},
  { 113.962488974578f,    16.9409375987462f},
  { 116.565051177078f,    24.0948425521107f},
  { 119.744881296942f,    31.8061000193785f},
  { 123.690067525980f,    39.7621591518051f},
  { 128.659808254090f,    47.5498444571087f},
  { 135.0f,    54.7356103172453f},
  { 143.130102354156f,    60.9453959009229f},
  { 153.434948822922f,    65.9051574478893f},
  { 165.963756532074f,    69.4526060024815f},
  { 180.0f,    71.5650511770780f},
  { 114.443954780417f,    0.0f},
  { 116.565051177078f,    5.11108969528872f},
  { 119.054604099077f,    10.9932014985969f},
  { 122.005383208084f,    17.6406309136330f},
  { 125.537677791974f,    24.9379827032418f},
  { 129.805571092265f,    32.6267377065311f},
  { 135.0f,    40.3155422107726f},
  { 141.340191745910f,    47.5498444571087f},
  { 149.036243467926f,    53.9128529342688f},
  { 158.198590513648f,    59.1057379206062f},
  { 168.690067525980f,    62.9828767821498f},
  { 180.0f,    65.5560452195835f},
  { 120.963756532074f,    0.0f},
  { 123.690067525980f,    5.28198245234341f},
  { 126.869897645844f,    11.3099324740202f},
  { 130.601294645005f,    18.0246645268103f},
  { 135.0f,    25.2394018206789f},
  { 140.194428907735f,    32.6267377065311f},
  { 146.309932474020f,    39.7621591518051f},
  { 153.434948822922f,    46.2194273454749f},
  { 161.565051177078f,    51.6711818985441f},
  { 170.537677791974f,    55.9467140446854f},
  { 180.0f,    59.0362434679265f},
  { 127.874983651098f,    0.0f},
  { 131.185925165710f,    5.37411792901766f},
  { 135.0f,    11.4217536589623f},
  { 139.398705354996f,    18.0246645268103f},
  { 144.462322208026f,    24.9379827032418f},
  { 150.255118703058f,    31.8061000193785f},
  { 156.801409486352f,    38.2324043541196f},
  { 164.054604099077f,    43.8762651909286f},
  { 171.869897645844f,    48.5270656910595f},
  { 180.0f,    52.1250163489018f},
  { 135.0f,    0.0f},
  { 138.814074834290f,    5.37411792901766f},
  { 143.130102354156f,    11.3099324740202f},
  { 147.994616791916f,    17.6406309136330f},
  { 153.434948822922f,    24.0948425521107f},
  { 159.443954780417f,    30.3364156173112f},
  { 165.963756532074f,    36.0398934303039f},
  { 172.874983651098f,    40.9659247499738f},
  { 180.0f,    45.0f},
  { 142.125016348902f,    0.0f},
  { 146.309932474020f,    5.28198245234341f},
  { 150.945395900923f,    10.9932014985969f},
  { 156.037511025422f,    16.9409375987462f},
  { 161.565051177078f,    22.8620891442349f},
  { 167.471192290848f,    28.4721344245730f},
  { 173.659808254090f,    33.5280232217311f},
  { 180.0f,    37.8749836510982f},
  { 149.036243467926f,    0.0f},
  { 153.434948822922f,    5.11108969528872f},
  { 158.198590513648f,    10.5197348906586f},
  { 163.300755766006f,    16.0318928587692f},
  { 168.690067525980f,    21.4167140330336f},
  { 174.289406862500f,    26.4511990993406f},
  { 180.0f,    30.9637565320735f},
  { 155.556045219583f,    0.0f},
  { 160.016893478100f,    4.88325044901346f},
  { 164.744881296942f,    9.94912629715978f},
  { 169.695153531234f,    15.0202566665138f},
  { 174.805571092265f,    19.9074988774736f},
  { 180.0f,    24.4439547804165f},
  { 161.565051177078f,    0.0f},
  { 165.963756532074f,    4.62203692583712f},
  { 170.537677791974f,    9.33585903200394f},
  { 175.236358309274f,    13.9896661378727f},
  { 180.0f,    18.4349488229220f},
  { 167.005383208084f,    0.0f},
  { 171.253837737445f,    4.34775315133540f},
  { 175.601294645005f,    8.72079692816341f},
  { 180.0f,    12.9946167919165f},
  { 171.869897645844f,    0.0f},
  { 175.914383220025f,    4.07526902813210f},
  { 180.0f,    8.13010235415598f},
  { 176.185925165710f,    0.0f},
  { 180.0f,    3.81407483429035f},
  { -176.185925165710f,    0.0f},
  { -175.914383220025f,    4.07526902813210f},
  { -175.601294645005f,    8.72079692816341f},
  { -175.236358309274f,    13.9896661378727f},
  { -174.805571092265f,    19.9074988774736f},
  { -174.289406862500f,    26.4511990993406f},
  { -173.659808254090f,    33.5280232217311f},
  { -172.874983651098f,    40.9659247499738f},
  { -171.869897645844f,    48.5270656910595f},
  { -170.537677791974f,    55.9467140446854f},
  { -168.690067525980f,    62.9828767821498f},
  { -165.963756532074f,    69.4526060024815f},
  { -161.565051177078f,    75.2368673763685f},
  { -153.434948822922f,    80.2403277394902f},
  { -135.0f,    84.2318188138118f},
  { -90.0f,    86.1859251657097f},
  { -171.869897645844f,    0.0f},
  { -171.253837737445f,    4.34775315133540f},
  { -170.537677791974f,    9.33585903200394f},
  { -169.695153531234f,    15.0202566665138f},
  { -168.690067525980f,    21.4167140330336f},
  { -167.471192290848f,    28.4721344245730f},
  { -165.963756532074f,    36.0398934303039f},
  { -164.054604099077f,    43.8762651909286f},
  { -161.565051177078f,    51.6711818985441f},
  { -158.198590513648f,    59.1057379206062f},
  { -153.434948822922f,    65.9051574478893f},
  { -146.309932474020f,    71.8520260682270f},
  { -135.0f,    76.7373239916952f},
  { -116.565051177078f,    80.2403277394902f},
  { -90.0f,    81.8698976458440f},
  { -167.005383208084f,    0.0f},
  { -165.963756532074f,    4.62203692583712f},
  { -164.744881296942f,    9.94912629715978f},
  { -163.300755766006f,    16.0318928587692f},
  { -161.565051177078f,    22.8620891442349f},
  { -159.443954780417f,    30.3364156173112f},
  { -156.801409486352f,    38.2324043541196f},
  { -153.434948822922f,    46.2194273454749f},
  { -149.036243467926f,    53.9128529342688f},
  { -143.130102354156f,    60.9453959009229f},
  { -135.0f,    67.0102322263912f},
  { -123.690067525980f,    71.8520260682270f},
  { -108.434948822922f,    75.2368673763685f},
  { -90.0f,    77.0053832080835f},
  { -161.565051177078f,    0.0f},
  { -160.016893478100f,    4.88325044901346f},
  { -158.198590513648f,    10.5197348906586f},
  { -156.037511025422f,    16.9409375987462f},
  { -153.434948822922f,    24.0948425521107f},
  { -150.255118703058f,    31.8061000193785f},
  { -146.309932474020f,    39.7621591518051f},
  { -141.340191745910f,    47.5498444571087f},
  { -135.0f,    54.7356103172453f},
  { -126.869897645844f,    60.9453959009229f},
  { -116.565051177078f,    65.9051574478893f},
  { -104.036243467927f,    69.4526060024815f},
  { -90.0f,    71.5650511770780f},
  { -155.556045219583f,    0.0f},
  { -153.434948822922f,    5.11108969528872f},
  { -150.945395900923f,    10.9932014985969f},
  { -147.994616791916f,    17.6406309136330f},
  { -144.462322208026f,    24.9379827032418f},
  { -140.194428907735f,    32.6267377065311f},
  { -135.0f,    40.3155422107726f},
  { -128.659808254090f,    47.5498444571087f},
  { -120.963756532074f,    53.9128529342687f},
  { -111.801409486352f,    59.1057379206062f},
  { -101.309932474020f,    62.9828767821498f},
  { -90.0f,    65.5560452195835f},
  { -149.036243467926f,    0.0f},
  { -146.309932474020f,    5.28198245234341f},
  { -143.130102354156f,    11.3099324740202f},
  { -139.398705354996f,    18.0246645268103f},
  { -135.0f,    25.2394018206789f},
  { -129.805571092265f,    32.6267377065311f},
  { -123.690067525980f,    39.7621591518051f},
  { -116.565051177078f,    46.2194273454749f},
  { -108.434948822922f,    51.6711818985441f},
  { -99.4623222080256f,    55.9467140446854f},
  { -90.0f,    59.0362434679265f},
  { -142.125016348902f,    0.0f},
  { -138.814074834290f,    5.37411792901766f},
  { -135.0f,    11.4217536589623f},
  { -130.601294645005f,    18.0246645268103f},
  { -125.537677791974f,    24.9379827032418f},
  { -119.744881296942f,    31.8061000193785f},
  { -113.198590513648f,    38.2324043541196f},
  { -105.945395900923f,    43.8762651909286f},
  { -98.1301023541560f,    48.5270656910595f},
  { -90.0f,    52.1250163489018f},
  { -135.0f,    0.0f},
  { -131.185925165710f,    5.37411792901766f},
  { -126.869897645844f,    11.3099324740202f},
  { -122.005383208084f,    17.6406309136330f},
  { -116.565051177078f,    24.0948425521107f},
  { -110.556045219583f,    30.3364156173112f},
  { -104.036243467927f,    36.0398934303039f},
  { -97.1250163489018f,    40.9659247499738f},
  { -90.0f,    45.0f},
  { -127.874983651098f,    0.0f},
  { -123.690067525980f,    5.28198245234341f},
  { -119.054604099077f,    10.9932014985969f},
  { -113.962488974578f,    16.9409375987462f},
  { -108.434948822922f,    22.8620891442349f},
  { -102.528807709152f,    28.4721344245730f},
  { -96.3401917459099f,    33.5280232217311f},
  { -90.0f,    37.8749836510982f},
  { -120.963756532074f,    0.0f},
  { -116.565051177078f,    5.11108969528872f},
  { -111.801409486352f,    10.5197348906586f},
  { -106.699244233994f,    16.0318928587692f},
  { -101.309932474020f,    21.4167140330336f},
  { -95.7105931374996f,    26.4511990993406f},
  { -90.0f,    30.9637565320735f},
  { -114.443954780417f,    0.0f},
  { -109.983106521900f,    4.88325044901346f},
  { -105.255118703058f,    9.94912629715978f},
  { -100.304846468766f,    15.0202566665138f},
  { -95.1944289077348f,    19.9074988774736f},
  { -90.0f,    24.4439547804165f},
  { -108.434948822922f,    0.0f},
  { -104.036243467927f,    4.62203692583713f},
  { -99.4623222080256f,    9.33585903200394f},
  { -94.7636416907262f,    13.9896661378727f},
  { -90.0f,    18.4349488229220f},
  { -102.994616791917f,    0.0f},
  { -98.7461622625552f,    4.34775315133540f},
  { -94.3987053549955f,    8.72079692816341f},
  { -90.0f,    12.9946167919165f},
  { -98.1301023541560f,    0.0f},
  { -94.0856167799749f,    4.07526902813210f},
  { -90.0f,    8.13010235415598f},
  { -93.8140748342904f,    0.0f},
  { -90.0f,    3.81407483429035f},
  { -86.1859251657097f,    0.0f},
  { -85.9143832200251f,    4.07526902813210f},
  { -85.6012946450045f,    8.72079692816341f},
  { -85.2363583092738f,    13.9896661378727f},
  { -84.8055710922652f,    19.9074988774736f},
  { -84.2894068625004f,    26.4511990993406f},
  { -83.6598082540901f,    33.5280232217311f},
  { -82.8749836510982f,    40.9659247499738f},
  { -81.8698976458440f,    48.5270656910595f},
  { -80.5376777919744f,    55.9467140446854f},
  { -78.6900675259798f,    62.9828767821498f},
  { -75.9637565320735f,    69.4526060024815f},
  { -71.5650511770780f,    75.2368673763685f},
  { -63.4349488229220f,    80.2403277394902f},
  { -45.0000000000000f,    84.2318188138118f},
  { -81.8698976458440f,    0.0f},
  { -81.2538377374448f,    4.34775315133540f},
  { -80.5376777919744f,    9.33585903200394f},
  { -79.6951535312340f,    15.0202566665138f},
  { -78.6900675259798f,    21.4167140330336f},
  { -77.4711922908485f,    28.4721344245730f},
  { -75.9637565320735f,    36.0398934303039f},
  { -74.0546040990772f,    43.8762651909286f},
  { -71.5650511770780f,    51.6711818985441f},
  { -68.1985905136482f,    59.1057379206062f},
  { -63.4349488229220f,    65.9051574478893f},
  { -56.3099324740202f,    71.8520260682270f},
  { -45.0000000000000f,    76.7373239916952f},
  { -26.5650511770780f,    80.2403277394902f},
  { -77.0053832080835f,    0.0f},
  { -75.9637565320735f,    4.62203692583713f},
  { -74.7448812969422f,    9.94912629715978f},
  { -73.3007557660064f,    16.0318928587692f},
  { -71.5650511770780f,    22.8620891442349f},
  { -69.4439547804165f,    30.3364156173112f},
  { -66.8014094863518f,    38.2324043541196f},
  { -63.4349488229220f,    46.2194273454749f},
  { -59.0362434679265f,    53.9128529342688f},
  { -53.1301023541560f,    60.9453959009229f},
  { -45.0000000000000f,    67.0102322263912f},
  { -33.6900675259798f,    71.8520260682270f},
  { -18.4349488229220f,    75.2368673763685f},
  { -71.5650511770780f,    0.0f},
  { -70.0168934781000f,    4.88325044901346f},
  { -68.1985905136482f,    10.5197348906586f},
  { -66.0375110254218f,    16.9409375987462f},
  { -63.4349488229220f,    24.0948425521107f},
  { -60.2551187030578f,    31.8061000193785f},
  { -56.3099324740202f,    39.7621591518051f},
  { -51.3401917459099f,    47.5498444571087f},
  { -45.0f,    54.7356103172453f},
  { -36.8698976458440f,    60.9453959009229f},
  { -26.5650511770780f,    65.9051574478893f},
  { -14.0362434679265f,    69.4526060024815f},
  { -65.5560452195835f,    0.0f},
  { -63.4349488229220f,    5.11108969528872f},
  { -60.9453959009229f,    10.9932014985969f},
  { -57.9946167919165f,    17.6406309136330f},
  { -54.4623222080256f,    24.9379827032418f},
  { -50.1944289077348f,    32.6267377065311f},
  { -45.0000000000000f,    40.3155422107726f},
  { -38.6598082540901f,    47.5498444571087f},
  { -30.9637565320735f,    53.9128529342688f},
  { -21.8014094863518f,    59.1057379206062f},
  { -11.3099324740202f,    62.9828767821498f},
  { -59.0362434679265f,    0.0f},
  { -56.3099324740202f,    5.28198245234341f},
  { -53.1301023541560f,    11.3099324740202f},
  { -49.3987053549955f,    18.0246645268103f},
  { -45.0000000000000f,    25.2394018206789f},
  { -39.8055710922652f,    32.6267377065311f},
  { -33.6900675259798f,    39.7621591518051f},
  { -26.5650511770780f,    46.2194273454749f},
  { -18.4349488229220f,    51.6711818985441f},
  { -9.46232220802562f,    55.9467140446854f},
  { -52.1250163489018f,    0.0f},
  { -48.8140748342904f,    5.37411792901766f},
  { -45.0f,    11.4217536589623f},
  { -40.6012946450045f,    18.0246645268103f},
  { -35.5376777919744f,    24.9379827032418f},
  { -29.7448812969422f,    31.8061000193785f},
  { -23.1985905136482f,    38.2324043541196f},
  { -15.9453959009229f,    43.8762651909286f},
  { -8.13010235415598f,    48.5270656910595f},
  { -45.0f,    0.0f},
  { -41.1859251657097f,    5.37411792901766f},
  { -36.8698976458440f,    11.3099324740202f},
  { -32.0053832080835f,    17.6406309136330f},
  { -26.5650511770780f,    24.0948425521107f},
  { -20.5560452195835f,    30.3364156173112f},
  { -14.0362434679265f,    36.0398934303039f},
  { -7.12501634890180f,    40.9659247499738f},
  { -37.8749836510982f,    0.0f},
  { -33.6900675259798f,    5.28198245234341f},
  { -29.0546040990771f,    10.9932014985969f},
  { -23.9624889745782f,    16.9409375987462f},
  { -18.4349488229220f,    22.8620891442349f},
  { -12.5288077091515f,    28.4721344245730f},
  { -6.34019174590991f,    33.5280232217311f},
  { -30.9637565320735f,    0.0f},
  { -26.5650511770780f,    5.11108969528872f},
  { -21.8014094863518f,    10.5197348906586f},
  { -16.6992442339936f,    16.0318928587692f},
  { -11.3099324740202f,    21.4167140330336f},
  { -5.71059313749964f,    26.4511990993406f},
  { -24.4439547804165f,    0.0f},
  { -19.9831065219000f,    4.88325044901346f},
  { -15.2551187030578f,    9.94912629715978f},
  { -10.3048464687660f,    15.0202566665138f},
  { -5.19442890773481f,    19.9074988774736f},
  { -18.4349488229220f,    0.0f},
  { -14.0362434679265f,    4.62203692583713f},
  { -9.46232220802562f,    9.33585903200394f},
  { -4.76364169072618f,    13.9896661378727f},
  { -12.9946167919165f,    0.0f},
  { -8.74616226255521f,    4.34775315133540f},
  { -4.39870535499553f,    8.72079692816341f},
  { -8.13010235415598f,    0.0f},
  { -4.08561677997488f,    4.07526902813210f},
  { -3.81407483429035f,    0.0f},
  { 0.0f,    -3.81407483429035f},
  { 4.08561677997488f,    -4.07526902813210f},
  { 8.74616226255521f,    -4.34775315133540f},
  { 14.0362434679265f,    -4.62203692583713f},
  { 19.9831065219000f,    -4.88325044901346f},
  { 26.5650511770780f,    -5.11108969528872f},
  { 33.6900675259798f,    -5.28198245234341f},
  { 41.1859251657097f,    -5.37411792901766f},
  { 48.8140748342904f,    -5.37411792901766f},
  { 56.3099324740202f,    -5.28198245234341f},
  { 63.4349488229220f,    -5.11108969528872f},
  { 70.0168934781000f,    -4.88325044901346f},
  { 75.9637565320735f,    -4.62203692583713f},
  { 81.2538377374448f,    -4.34775315133540f},
  { 85.9143832200251f,    -4.07526902813210f},
  { 90.0f,    -3.81407483429035f},
  { 0.0f,    -8.13010235415598f},
  { 4.39870535499553f,    -8.72079692816341f},
  { 9.46232220802562f,    -9.33585903200394f},
  { 15.2551187030578f,    -9.94912629715978f},
  { 21.8014094863518f,    -10.5197348906586f},
  { 29.0546040990771f,    -10.9932014985969f},
  { 36.8698976458440f,    -11.3099324740202f},
  { 45.0f,    -11.4217536589623f},
  { 53.1301023541560f,    -11.3099324740202f},
  { 60.9453959009229f,    -10.9932014985969f},
  { 68.1985905136482f,    -10.5197348906586f},
  { 74.7448812969422f,    -9.94912629715978f},
  { 80.5376777919744f,    -9.33585903200394f},
  { 85.6012946450045f,    -8.72079692816341f},
  { 90.0f,    -8.13010235415598f},
  { 0.0f,    -12.9946167919165f},
  { 4.76364169072618f,    -13.9896661378727f},
  { 10.3048464687660f,    -15.0202566665138f},
  { 16.6992442339936f,    -16.0318928587692f},
  { 23.9624889745782f,    -16.9409375987462f},
  { 32.0053832080835f,    -17.6406309136330f},
  { 40.6012946450045f,    -18.0246645268103f},
  { 49.3987053549955f,    -18.0246645268103f},
  { 57.9946167919165f,    -17.6406309136330f},
  { 66.0375110254218f,    -16.9409375987462f},
  { 73.3007557660064f,    -16.0318928587692f},
  { 79.6951535312340f,    -15.0202566665138f},
  { 85.2363583092738f,    -13.9896661378727f},
  { 90.0f,    -12.9946167919165f},
  { 0.0f,    -18.4349488229220f},
  { 5.19442890773481f,    -19.9074988774736f},
  { 11.3099324740202f,    -21.4167140330336f},
  { 18.4349488229220f,    -22.8620891442349f},
  { 26.5650511770780f,    -24.0948425521107f},
  { 35.5376777919744f,    -24.9379827032418f},
  { 45.0000000000000f,    -25.2394018206789f},
  { 54.4623222080256f,    -24.9379827032418f},
  { 63.4349488229220f,    -24.0948425521107f},
  { 71.5650511770780f,    -22.8620891442349f},
  { 78.6900675259798f,    -21.4167140330336f},
  { 84.8055710922652f,    -19.9074988774736f},
  { 90.0f,    -18.4349488229220f},
  { 0.0f,    -24.4439547804165f},
  { 5.71059313749964f,    -26.4511990993406f},
  { 12.5288077091515f,    -28.4721344245730f},
  { 20.5560452195835f,    -30.3364156173112f},
  { 29.7448812969422f,    -31.8061000193785f},
  { 39.8055710922652f,    -32.6267377065311f},
  { 50.1944289077348f,    -32.6267377065311f},
  { 60.2551187030578f,    -31.8061000193785f},
  { 69.4439547804165f,    -30.3364156173112f},
  { 77.4711922908485f,    -28.4721344245730f},
  { 84.2894068625004f,    -26.4511990993406f},
  { 90.0f,    -24.4439547804165f},
  { 0.0f,    -30.9637565320735f},
  { 6.34019174590991f,    -33.5280232217311f},
  { 14.0362434679265f,    -36.0398934303039f},
  { 23.1985905136482f,    -38.2324043541196f},
  { 33.6900675259798f,    -39.7621591518051f},
  { 45.0000000000000f,    -40.3155422107726f},
  { 56.3099324740202f,    -39.7621591518051f},
  { 66.8014094863518f,    -38.2324043541196f},
  { 75.9637565320735f,    -36.0398934303039f},
  { 83.6598082540901f,    -33.5280232217311f},
  { 90.0f,    -30.9637565320735f},
  { 0.0f,    -37.8749836510982f},
  { 7.12501634890180f,    -40.9659247499738f},
  { 15.9453959009229f,    -43.8762651909286f},
  { 26.5650511770780f,    -46.2194273454749f},
  { 38.6598082540901f,    -47.5498444571087f},
  { 51.3401917459099f,    -47.5498444571087f},
  { 63.4349488229220f,    -46.2194273454749f},
  { 74.0546040990772f,    -43.8762651909286f},
  { 82.8749836510982f,    -40.9659247499738f},
  { 90.0f,    -37.8749836510982f},
  { 0.0f,    -45.0f},
  { 8.13010235415598f,    -48.5270656910595f},
  { 18.4349488229220f,    -51.6711818985441f},
  { 30.9637565320735f,    -53.9128529342688f},
  { 45.0f,    -54.7356103172453f},
  { 59.0362434679265f,    -53.9128529342688f},
  { 71.5650511770780f,    -51.6711818985441f},
  { 81.8698976458440f,    -48.5270656910595f},
  { 90.0f,    -45.0f},
  { 0.0f,    -52.1250163489018f},
  { 9.46232220802562f,    -55.9467140446854f},
  { 21.8014094863518f,    -59.1057379206062f},
  { 36.8698976458440f,    -60.9453959009229f},
  { 53.1301023541560f,    -60.9453959009229f},
  { 68.1985905136482f,    -59.1057379206062f},
  { 80.5376777919744f,    -55.9467140446854f},
  { 90.0f,    -52.1250163489018f},
  { 0.0f,    -59.0362434679265f},
  { 11.3099324740202f,    -62.9828767821498f},
  { 26.5650511770780f,    -65.9051574478893f},
  { 45.0000000000000f,    -67.0102322263912f},
  { 63.4349488229220f,    -65.9051574478893f},
  { 78.6900675259798f,    -62.9828767821498f},
  { 90.0f,    -59.0362434679265f},
  { 0.0f,    -65.5560452195835f},
  { 14.0362434679265f,    -69.4526060024815f},
  { 33.6900675259798f,    -71.8520260682270f},
  { 56.3099324740202f,    -71.8520260682270f},
  { 75.9637565320735f,    -69.4526060024815f},
  { 90.0f,    -65.5560452195835f},
  { 0.0f,    -71.5650511770780f},
  { 18.4349488229220f,    -75.2368673763685f},
  { 45.0000000000000f,    -76.7373239916952f},
  { 71.5650511770780f,    -75.2368673763685f},
  { 90.0f,    -71.5650511770780f},
  { 0.0f,    -77.0053832080835f},
  { 26.5650511770780f,    -80.2403277394902f},
  { 63.4349488229220f,    -80.2403277394902f},
  { 90.0f,    -77.0053832080835f},
  { 0.0f,    -81.8698976458440f},
  { 45.0000000000000f,    -84.2318188138118f},
  { 90.0f,    -81.8698976458440f},
  { 0.0f,    -86.1859251657097f},
  { 90.0f,    -86.1859251657097f},
  { 94.0856167799749f,    -4.07526902813210f},
  { 98.7461622625552f,    -4.34775315133540f},
  { 104.036243467927f,    -4.62203692583713f},
  { 109.983106521900f,    -4.88325044901346f},
  { 116.565051177078f,    -5.11108969528872f},
  { 123.690067525980f,    -5.28198245234341f},
  { 131.185925165710f,    -5.37411792901766f},
  { 138.814074834290f,    -5.37411792901766f},
  { 146.309932474020f,    -5.28198245234341f},
  { 153.434948822922f,    -5.11108969528872f},
  { 160.016893478100f,    -4.88325044901346f},
  { 165.963756532074f,    -4.62203692583712f},
  { 171.253837737445f,    -4.34775315133540f},
  { 175.914383220025f,    -4.07526902813210f},
  { 180.0f,    -3.81407483429035f},
  { 94.3987053549955f,    -8.72079692816341f},
  { 99.4623222080256f,    -9.33585903200394f},
  { 105.255118703058f,    -9.94912629715978f},
  { 111.801409486352f,    -10.5197348906586f},
  { 119.054604099077f,    -10.9932014985969f},
  { 126.869897645844f,    -11.3099324740202f},
  { 135.0f,    -11.4217536589623f},
  { 143.130102354156f,    -11.3099324740202f},
  { 150.945395900923f,    -10.9932014985969f},
  { 158.198590513648f,    -10.5197348906586f},
  { 164.744881296942f,    -9.94912629715978f},
  { 170.537677791974f,    -9.33585903200394f},
  { 175.601294645005f,    -8.72079692816341f},
  { 180.0f,    -8.13010235415598f},
  { 94.7636416907262f,    -13.9896661378727f},
  { 100.304846468766f,    -15.0202566665138f},
  { 106.699244233994f,    -16.0318928587692f},
  { 113.962488974578f,    -16.9409375987462f},
  { 122.005383208084f,    -17.6406309136330f},
  { 130.601294645005f,    -18.0246645268103f},
  { 139.398705354996f,    -18.0246645268103f},
  { 147.994616791916f,    -17.6406309136330f},
  { 156.037511025422f,    -16.9409375987462f},
  { 163.300755766006f,    -16.0318928587692f},
  { 169.695153531234f,    -15.0202566665138f},
  { 175.236358309274f,    -13.9896661378727f},
  { 180.0f,    -12.9946167919165f},
  { 95.1944289077348f,    -19.9074988774736f},
  { 101.309932474020f,    -21.4167140330336f},
  { 108.434948822922f,    -22.8620891442349f},
  { 116.565051177078f,    -24.0948425521107f},
  { 125.537677791974f,    -24.9379827032418f},
  { 135.0f,    -25.2394018206789f},
  { 144.462322208026f,    -24.9379827032418f},
  { 153.434948822922f,    -24.0948425521107f},
  { 161.565051177078f,    -22.8620891442349f},
  { 168.690067525980f,    -21.4167140330336f},
  { 174.805571092265f,    -19.9074988774736f},
  { 180.0f,    -18.4349488229220f},
  { 95.7105931374996f,    -26.4511990993406f},
  { 102.528807709152f,    -28.4721344245730f},
  { 110.556045219583f,    -30.3364156173112f},
  { 119.744881296942f,    -31.8061000193785f},
  { 129.805571092265f,    -32.6267377065311f},
  { 140.194428907735f,    -32.6267377065311f},
  { 150.255118703058f,    -31.8061000193785f},
  { 159.443954780417f,    -30.3364156173112f},
  { 167.471192290848f,    -28.4721344245730f},
  { 174.289406862500f,    -26.4511990993406f},
  { 180.0f,    -24.4439547804165f},
  { 96.3401917459099f,    -33.5280232217311f},
  { 104.036243467927f,    -36.0398934303039f},
  { 113.198590513648f,    -38.2324043541196f},
  { 123.690067525980f,    -39.7621591518051f},
  { 135.0f,    -40.3155422107726f},
  { 146.309932474020f,    -39.7621591518051f},
  { 156.801409486352f,    -38.2324043541196f},
  { 165.963756532074f,    -36.0398934303039f},
  { 173.659808254090f,    -33.5280232217311f},
  { 180.0f,    -30.9637565320735f},
  { 97.1250163489018f,    -40.9659247499738f},
  { 105.945395900923f,    -43.8762651909286f},
  { 116.565051177078f,    -46.2194273454749f},
  { 128.659808254090f,    -47.5498444571087f},
  { 141.340191745910f,    -47.5498444571087f},
  { 153.434948822922f,    -46.2194273454749f},
  { 164.054604099077f,    -43.8762651909286f},
  { 172.874983651098f,    -40.9659247499738f},
  { 180.0f,    -37.8749836510982f},
  { 98.1301023541560f,    -48.5270656910595f},
  { 108.434948822922f,    -51.6711818985441f},
  { 120.963756532074f,    -53.9128529342687f},
  { 135.0f,    -54.7356103172453f},
  { 149.036243467926f,    -53.9128529342688f},
  { 161.565051177078f,    -51.6711818985441f},
  { 171.869897645844f,    -48.5270656910595f},
  { 180.0f,    -45.0f},
  { 99.4623222080256f,    -55.9467140446854f},
  { 111.801409486352f,    -59.1057379206062f},
  { 126.869897645844f,    -60.9453959009229f},
  { 143.130102354156f,    -60.9453959009229f},
  { 158.198590513648f,    -59.1057379206062f},
  { 170.537677791974f,    -55.9467140446854f},
  { 180.0f,    -52.1250163489018f},
  { 101.309932474020f,    -62.9828767821498f},
  { 116.565051177078f,    -65.9051574478893f},
  { 135.0f,    -67.0102322263912f},
  { 153.434948822922f,    -65.9051574478893f},
  { 168.690067525980f,    -62.9828767821498f},
  { 180.0f,    -59.0362434679265f},
  { 104.036243467927f,    -69.4526060024815f},
  { 123.690067525980f,    -71.8520260682270f},
  { 146.309932474020f,    -71.8520260682270f},
  { 165.963756532074f,    -69.4526060024815f},
  { 180.0f,    -65.5560452195835f},
  { 108.434948822922f,    -75.2368673763685f},
  { 135.0f,    -76.7373239916952f},
  { 161.565051177078f,    -75.2368673763685f},
  { 180.0f,    -71.5650511770780f},
  { 116.565051177078f,    -80.2403277394902f},
  { 153.434948822922f,    -80.2403277394902f},
  { 180.0f,    -77.0053832080835f},
  { 135.0f,    -84.2318188138118f},
  { 180.0f,    -81.8698976458440f},
  { 180.0f,    -86.1859251657097f},
  { -175.914383220025f,    -4.07526902813210f},
  { -171.253837737445f,    -4.34775315133540f},
  { -165.963756532074f,    -4.62203692583712f},
  { -160.016893478100f,    -4.88325044901346f},
  { -153.434948822922f,    -5.11108969528872f},
  { -146.309932474020f,    -5.28198245234341f},
  { -138.814074834290f,    -5.37411792901766f},
  { -131.185925165710f,    -5.37411792901766f},
  { -123.690067525980f,    -5.28198245234341f},
  { -116.565051177078f,    -5.11108969528872f},
  { -109.983106521900f,    -4.88325044901346f},
  { -104.036243467927f,    -4.62203692583713f},
  { -98.7461622625552f,    -4.34775315133540f},
  { -94.0856167799749f,    -4.07526902813210f},
  { -90.0f,    -3.81407483429035f},
  { -175.601294645005f,    -8.72079692816341f},
  { -170.537677791974f,    -9.33585903200394f},
  { -164.744881296942f,    -9.94912629715978f},
  { -158.198590513648f,    -10.5197348906586f},
  { -150.945395900923f,    -10.9932014985969f},
  { -143.130102354156f,    -11.3099324740202f},
  { -135.0f,    -11.4217536589623f},
  { -126.869897645844f,    -11.3099324740202f},
  { -119.054604099077f,    -10.9932014985969f},
  { -111.801409486352f,    -10.5197348906586f},
  { -105.255118703058f,    -9.94912629715978f},
  { -99.4623222080256f,    -9.33585903200394f},
  { -94.3987053549955f,    -8.72079692816341f},
  { -90.0f,    -8.13010235415598f},
  { -175.236358309274f,    -13.9896661378727f},
  { -169.695153531234f,    -15.0202566665138f},
  { -163.300755766006f,    -16.0318928587692f},
  { -156.037511025422f,    -16.9409375987462f},
  { -147.994616791916f,    -17.6406309136330f},
  { -139.398705354996f,    -18.0246645268103f},
  { -130.601294645005f,    -18.0246645268103f},
  { -122.005383208084f,    -17.6406309136330f},
  { -113.962488974578f,    -16.9409375987462f},
  { -106.699244233994f,    -16.0318928587692f},
  { -100.304846468766f,    -15.0202566665138f},
  { -94.7636416907262f,    -13.9896661378727f},
  { -90.0f,    -12.9946167919165f},
  { -174.805571092265f,    -19.9074988774736f},
  { -168.690067525980f,    -21.4167140330336f},
  { -161.565051177078f,    -22.8620891442349f},
  { -153.434948822922f,    -24.0948425521107f},
  { -144.462322208026f,    -24.9379827032418f},
  { -135.0f,    -25.2394018206789f},
  { -125.537677791974f,    -24.9379827032418f},
  { -116.565051177078f,    -24.0948425521107f},
  { -108.434948822922f,    -22.8620891442349f},
  { -101.309932474020f,    -21.4167140330336f},
  { -95.1944289077348f,    -19.9074988774736f},
  { -90.0f,    -18.4349488229220f},
  { -174.289406862500f,    -26.4511990993406f},
  { -167.471192290848f,    -28.4721344245730f},
  { -159.443954780417f,    -30.3364156173112f},
  { -150.255118703058f,    -31.8061000193785f},
  { -140.194428907735f,    -32.6267377065311f},
  { -129.805571092265f,    -32.6267377065311f},
  { -119.744881296942f,    -31.8061000193785f},
  { -110.556045219583f,    -30.3364156173112f},
  { -102.528807709152f,    -28.4721344245730f},
  { -95.7105931374996f,    -26.4511990993406f},
  { -90.0f,    -24.4439547804165f},
  { -173.659808254090f,    -33.5280232217311f},
  { -165.963756532074f,    -36.0398934303039f},
  { -156.801409486352f,    -38.2324043541196f},
  { -146.309932474020f,    -39.7621591518051f},
  { -135.0f,    -40.3155422107726f},
  { -123.690067525980f,    -39.7621591518051f},
  { -113.198590513648f,    -38.2324043541196f},
  { -104.036243467927f,    -36.0398934303039f},
  { -96.3401917459099f,    -33.5280232217311f},
  { -90.0f,    -30.9637565320735f},
  { -172.874983651098f,    -40.9659247499738f},
  { -164.054604099077f,    -43.8762651909286f},
  { -153.434948822922f,    -46.2194273454749f},
  { -141.340191745910f,    -47.5498444571087f},
  { -128.659808254090f,    -47.5498444571087f},
  { -116.565051177078f,    -46.2194273454749f},
  { -105.945395900923f,    -43.8762651909286f},
  { -97.1250163489018f,    -40.9659247499738f},
  { -90.0f,    -37.8749836510982f},
  { -171.869897645844f,    -48.5270656910595f},
  { -161.565051177078f,    -51.6711818985441f},
  { -149.036243467926f,    -53.9128529342688f},
  { -135.0f,    -54.7356103172453f},
  { -120.963756532074f,    -53.9128529342687f},
  { -108.434948822922f,    -51.6711818985441f},
  { -98.1301023541560f,    -48.5270656910595f},
  { -90.0f,    -45.0f},
  { -170.537677791974f,    -55.9467140446854f},
  { -158.198590513648f,    -59.1057379206062f},
  { -143.130102354156f,    -60.9453959009229f},
  { -126.869897645844f,    -60.9453959009229f},
  { -111.801409486352f,    -59.1057379206062f},
  { -99.4623222080256f,    -55.9467140446854f},
  { -90.0f,    -52.1250163489018f},
  { -168.690067525980f,    -62.9828767821498f},
  { -153.434948822922f,    -65.9051574478893f},
  { -135.0f,    -67.0102322263912f},
  { -116.565051177078f,    -65.9051574478893f},
  { -101.309932474020f,    -62.9828767821498f},
  { -90.0f,    -59.0362434679265f},
  { -165.963756532074f,    -69.4526060024815f},
  { -146.309932474020f,    -71.8520260682270f},
  { -123.690067525980f,    -71.8520260682270f},
  { -104.036243467927f,    -69.4526060024815f},
  { -90.0f,    -65.5560452195835f},
  { -161.565051177078f,    -75.2368673763685f},
  { -135.0f,    -76.7373239916952f},
  { -108.434948822922f,    -75.2368673763685f},
  { -90.0f,    -71.5650511770780f},
  { -153.434948822922f,    -80.2403277394902f},
  { -116.565051177078f,    -80.2403277394902f},
  { -90.0f,    -77.0053832080835f},
  { -135.0f,    -84.2318188138118f},
  { -90.0f,    -81.8698976458440f},
  { -90.0f,    -86.1859251657097f},
  { -85.9143832200251f,    -4.07526902813210f},
  { -81.2538377374448f,    -4.34775315133540f},
  { -75.9637565320735f,    -4.62203692583713f},
  { -70.0168934781000f,    -4.88325044901346f},
  { -63.4349488229220f,    -5.11108969528872f},
  { -56.3099324740202f,    -5.28198245234341f},
  { -48.8140748342904f,    -5.37411792901766f},
  { -41.1859251657097f,    -5.37411792901766f},
  { -33.6900675259798f,    -5.28198245234341f},
  { -26.5650511770780f,    -5.11108969528872f},
  { -19.9831065219000f,    -4.88325044901346f},
  { -14.0362434679265f,    -4.62203692583713f},
  { -8.74616226255521f,    -4.34775315133540f},
  { -4.08561677997488f,    -4.07526902813210f},
  { -85.6012946450045f,    -8.72079692816341f},
  { -80.5376777919744f,    -9.33585903200394f},
  { -74.7448812969422f,    -9.94912629715978f},
  { -68.1985905136482f,    -10.5197348906586f},
  { -60.9453959009229f,    -10.9932014985969f},
  { -53.1301023541560f,    -11.3099324740202f},
  { -45.0f,    -11.4217536589623f},
  { -36.8698976458440f,    -11.3099324740202f},
  { -29.0546040990771f,    -10.9932014985969f},
  { -21.8014094863518f,    -10.5197348906586f},
  { -15.2551187030578f,    -9.94912629715978f},
  { -9.46232220802562f,    -9.33585903200394f},
  { -4.39870535499553f,    -8.72079692816341f},
  { -85.2363583092738f,    -13.9896661378727f},
  { -79.6951535312340f,    -15.0202566665138f},
  { -73.3007557660064f,    -16.0318928587692f},
  { -66.0375110254218f,    -16.9409375987462f},
  { -57.9946167919165f,    -17.6406309136330f},
  { -49.3987053549955f,    -18.0246645268103f},
  { -40.6012946450045f,    -18.0246645268103f},
  { -32.0053832080835f,    -17.6406309136330f},
  { -23.9624889745782f,    -16.9409375987462f},
  { -16.6992442339936f,    -16.0318928587692f},
  { -10.3048464687660f,    -15.0202566665138f},
  { -4.76364169072618f,    -13.9896661378727f},
  { -84.8055710922652f,    -19.9074988774736f},
  { -78.6900675259798f,    -21.4167140330336f},
  { -71.5650511770780f,    -22.8620891442349f},
  { -63.4349488229220f,    -24.0948425521107f},
  { -54.4623222080256f,    -24.9379827032418f},
  { -45.0000000000000f,    -25.2394018206789f},
  { -35.5376777919744f,    -24.9379827032418f},
  { -26.5650511770780f,    -24.0948425521107f},
  { -18.4349488229220f,    -22.8620891442349f},
  { -11.3099324740202f,    -21.4167140330336f},
  { -5.19442890773481f,    -19.9074988774736f},
  { -84.2894068625004f,    -26.4511990993406f},
  { -77.4711922908485f,    -28.4721344245730f},
  { -69.4439547804165f,    -30.3364156173112f},
  { -60.2551187030578f,    -31.8061000193785f},
  { -50.1944289077348f,    -32.6267377065311f},
  { -39.8055710922652f,    -32.6267377065311f},
  { -29.7448812969422f,    -31.8061000193785f},
  { -20.5560452195835f,    -30.3364156173112f},
  { -12.5288077091515f,    -28.4721344245730f},
  { -5.71059313749964f,    -26.4511990993406f},
  { -83.6598082540901f,    -33.5280232217311f},
  { -75.9637565320735f,    -36.0398934303039f},
  { -66.8014094863518f,    -38.2324043541196f},
  { -56.3099324740202f,    -39.7621591518051f},
  { -45.0000000000000f,    -40.3155422107726f},
  { -33.6900675259798f,    -39.7621591518051f},
  { -23.1985905136482f,    -38.2324043541196f},
  { -14.0362434679265f,    -36.0398934303039f},
  { -6.34019174590991f,    -33.5280232217311f},
  { -82.8749836510982f,    -40.9659247499738f},
  { -74.0546040990772f,    -43.8762651909286f},
  { -63.4349488229220f,    -46.2194273454749f},
  { -51.3401917459099f,    -47.5498444571087f},
  { -38.6598082540901f,    -47.5498444571087f},
  { -26.5650511770780f,    -46.2194273454749f},
  { -15.9453959009229f,    -43.8762651909286f},
  { -7.12501634890180f,    -40.9659247499738f},
  { -81.8698976458440f,    -48.5270656910595f},
  { -71.5650511770780f,    -51.6711818985441f},
  { -59.0362434679265f,    -53.9128529342688f},
  { -45.0f,    -54.7356103172453f},
  { -30.9637565320735f,    -53.9128529342688f},
  { -18.4349488229220f,    -51.6711818985441f},
  { -8.13010235415598f,    -48.5270656910595f},
  { -80.5376777919744f,    -55.9467140446854f},
  { -68.1985905136482f,    -59.1057379206062f},
  { -53.1301023541560f,    -60.9453959009229f},
  { -36.8698976458440f,    -60.9453959009229f},
  { -21.8014094863518f,    -59.1057379206062f},
  { -9.46232220802562f,    -55.9467140446854f},
  { -78.6900675259798f,    -62.9828767821498f},
  { -63.4349488229220f,    -65.9051574478893f},
  { -45.0000000000000f,    -67.0102322263912f},
  { -26.5650511770780f,    -65.9051574478893f},
  { -11.3099324740202f,    -62.9828767821498f},
  { -75.9637565320735f,    -69.4526060024815f},
  { -56.3099324740202f,    -71.8520260682270f},
  { -33.6900675259798f,    -71.8520260682270f},
  { -14.0362434679265f,    -69.4526060024815f},
  { -71.5650511770780f,    -75.2368673763685f},
  { -45.0000000000000f,    -76.7373239916952f},
  { -18.4349488229220f,    -75.2368673763685f},
  { -63.4349488229220f,    -80.2403277394902f},
  { -26.5650511770780f,    -80.2403277394902f},
  { -45.0000000000000f,    -84.2318188138118f}};

/* 3LD 'ico' geosphere arrangements HANDLES */
const float* __HANDLES_geosphere_ico_dirs_deg[17] =
{ &__geosphere_ico_0_0_dirs_deg[0][0],
  &__geosphere_ico_1_0_dirs_deg[0][0],
  &__geosphere_ico_2_0_dirs_deg[0][0],
  &__geosphere_ico_3_0_dirs_deg[0][0],
  &__geosphere_ico_4_0_dirs_deg[0][0],
  &__geosphere_ico_5_0_dirs_deg[0][0],
  &__geosphere_ico_6_0_dirs_deg[0][0],
  &__geosphere_ico_7_0_dirs_deg[0][0],
  &__geosphere_ico_8_0_dirs_deg[0][0],
  &__geosphere_ico_9_0_dirs_deg[0][0],
  &__geosphere_ico_10_0_dirs_deg[0][0],
  &__geosphere_ico_11_0_dirs_deg[0][0],
  &__geosphere_ico_12_0_dirs_deg[0][0],
  &__geosphere_ico_13_0_dirs_deg[0][0],
  &__geosphere_ico_14_0_dirs_deg[0][0],
  &__geosphere_ico_15_0_dirs_deg[0][0],
  &__geosphere_ico_16_0_dirs_deg[0][0]};

/* 3LD 'oct' geosphere arrangements HANDLES */
const float* __HANDLES_geosphere_oct_dirs_deg[17] =
{ &__geosphere_oct_0_0_dirs_deg[0][0],
  &__geosphere_oct_1_0_dirs_deg[0][0],
  &__geosphere_oct_2_0_dirs_deg[0][0],
  &__geosphere_oct_3_0_dirs_deg[0][0],
  &__geosphere_oct_4_0_dirs_deg[0][0],
  &__geosphere_oct_5_0_dirs_deg[0][0],
  &__geosphere_oct_6_0_dirs_deg[0][0],
  &__geosphere_oct_7_0_dirs_deg[0][0],
  &__geosphere_oct_8_0_dirs_deg[0][0],
  &__geosphere_oct_9_0_dirs_deg[0][0],
  &__geosphere_oct_10_0_dirs_deg[0][0],
  &__geosphere_oct_11_0_dirs_deg[0][0],
  &__geosphere_oct_12_0_dirs_deg[0][0],
  &__geosphere_oct_13_0_dirs_deg[0][0],
  &__geosphere_oct_14_0_dirs_deg[0][0],
  &__geosphere_oct_15_0_dirs_deg[0][0],
  &__geosphere_oct_16_0_dirs_deg[0][0]};

/* Number of points in each 'ico' geosphere arrangement */
const int __geosphere_ico_nPoints[17] =
{   12, 32, 42, 92, 162, 252, 362, 492, 642, 812, 1002, 1212, 1442, 1692, 1962, 2252, 2562   };

/* Number of points in each 'oct' geosphere arrangement */
const int __geosphere_oct_nPoints[17] =
{   6, 14, 18, 38, 66, 102, 146, 198, 258, 326, 402, 486, 578, 678, 786, 902, 1026   };






